/*****************************************************************************

NAME: GETDDRINFO

PURPOSE: Retrieve image information from the DDR and return to TAE-land.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
     None.

REFERENCES:
     None.
*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "typlim.h"
#include "util.h"


int main(void)
{
  struct PARBLK vblock;     /* TAE parameter block                          */
  
  struct VARIABLE *var;     /* struct for TAE parameters                    */

  struct DDR lowddr,        /*  input ddr                                   */
             highddr;

  int ii;                   /* loop counter                                 */
  int highnl, highns;       /* number of lines and samples in high res image*/
  int lowwind[4],           /* image window starting line, starting sample, */
      highwind[4];          /* number of lines, number of samples           */
  int nlevels;              /* number of transform levels to be applied     */

  long lowbands[3][MAXBND+1], /* bands specified for images                 */
       highbands[1][MAXBND+1];
  long first_element = 0;   /* index of first array element                 */
  long imglimit;            /* max images allowed                           */
  long nmrimg;              /* number of images input                       */
  long numbands[3];         /* number of bands in each input image          */
  long qflag[1][2];         /* flag denoting whether the user specified
                               windows and/or bands for the input images    */
  long same_window;         /* flag denoting whether the user requested
                               the same windows for all input images        */
  long std_out;             /* standard output LUN                          */
  long totbands;            /* total number of bands for low res            */
  long window[3][4];        /* window returned from c_getin()               */
  
  double scalex, scaley;    /* X and Y scale factor between low res and high
                               res                                          */
  double xlevels, ylevels;  /* number of transform levels difference for
                               X and Y projection directions                */
  
  char lowtae[3][CMLEN],    /* TAE version of image name                    */
       hightae[1][CMLEN];
  char lowhos[3][CMLEN],    /* host version of image name                   */
       highhos[1][CMLEN];
  char *strptr[1];          /* used to return string value to TAE           */
  char uselowwind[4] = "NO",/* strings denoting whether image windows       */
       usehighwind[4] = "NO"; /* have been requested:  "YES" or "NO"        */
  char validgeo[4] = "YES"; /* flag denoting whether the image DDR
                               geographic information is completely valid   */
  char windstr[CMLEN];      /* high res window string                       */
  

  /* Initialize TAE processing.
  -----------------------------*/
  c_inlas(&vblock, &std_out);

  /* Retrieve image names.
  ------------------------*/
  imglimit = 3;
  if (c_getin(&vblock, "LOW", &first_element, lowtae, lowhos, &nmrimg,
              window, lowbands, numbands, &totbands, &imglimit,
              &same_window, qflag) != E_SUCC)
      c_lasmsg(TRUE, "getddrinfo-getin",
               "Error parsing input image specification for LOW.");
  if (totbands != 3)
      c_lasmsg(TRUE, "getddrinfo-bands",
               "Number of low resolution bands not equal to three.");

  /* Check to see if the input images all have the same window size.
  ------------------------------------------------------------------*/
  if( same_window == 0)
      c_lasmsg(TRUE, "wavemerge-window", "Input images are not same size.");

  /* Copy image window into low resolution window storage.
   -------------------------------------------------------*/
  for (ii=0; ii<4; ii++)
      lowwind[ii] = (int)window[0][ii];

  imglimit = 1;
  if (c_getin(&vblock, "HIGH", &first_element, hightae, highhos, &nmrimg,
              window, highbands, numbands, &totbands, &imglimit,
              &same_window, qflag) != E_SUCC)
      c_lasmsg(TRUE, "getddrinfo-getin",
               "Error parsing input image specification for HIGH.");
  if (totbands != 1)
      c_lasmsg(TRUE, "getddrinfo-bands",
               "More than one high resolution band.");
  
  /* Copy image window into high resolution window storage. */
  for (ii=0; ii<4; ii++)
      highwind[ii] = (int)window[0][ii];  

  /* Get the image DDRs.
     -------------------*/
  if (c_getddr(lowhos[0], &lowddr) != E_SUCC)
      c_lasmsg(TRUE,"getddrinfo-ddr","Error retrieving DDR for %s.",
               lowhos[0]);
  if (c_getddr(highhos[0], &highddr) != E_SUCC)
      c_lasmsg(TRUE,"getddrinfo-ddr","Error retrieving DDR for %s.",
               highhos[0]);

  /* Retrieve the user input for the number of transform levels.
     -----------------------------------------------------------*/
  var = p_find(&vblock, "NLEVELS");
  nlevels = IVAL(*var, 0);

  /* Do the DDRs contain valid geographic information?
     -------------------------------------------------*/
  for (ii=0; ii<7; ii++)
      if (lowddr.valid[ii] != VALID || highddr.valid[ii] != VALID)
      {
          if (lowddr.valid[ii] != VALID)
              c_lasmsg(FALSE, "getddrinfo-valid",
                       "LOW DDR contains invalid projection information.");
          if (highddr.valid[ii] != VALID)
              c_lasmsg(FALSE, "getddrinfo-valid",
                       "HIGH DDR contains invalid projection information.");
          c_lasmsg(FALSE, "getddrinfo-valid",
                   "Input images are assumed to be in same projection "
                   "and coregistered at upper left corner.");
          
          strcpy(validgeo, "NO");
          break;
      }

  /* Error check values.
     -------------------*/
  if ((lowddr.pdist_x <= 0.) || (highddr.pdist_x <= 0.) ||
      (lowddr.pdist_y <= 0.) || (highddr.pdist_y <= 0.))
      c_lasmsg(TRUE, "getddrinfo-pixsiz", "Pixel has illegal size.");
      
  /* Compute scale values.
     ---------------------*/
  scalex = lowddr.pdist_x/highddr.pdist_x;
  scaley = lowddr.pdist_y/highddr.pdist_y;
  
  /* Determine whether a window has been requested for either the
     low or high resolution image.
     ------------------------------------------------------------*/
  if ((lowwind[2] != lowddr.nl) || (lowwind[3] != lowddr.ns))
      strcpy(uselowwind, "YES");
  if ((highwind[2] != highddr.nl) || (highwind[3] != highddr.ns))
      strcpy(usehighwind, "YES");      

  /* If the geographic data is invalid, it is assumed that the windows
     specified for the low and high resolution images are coregistered
     at the upper left corner.  The window sizes (allowing for resolution
     differences) may not be equal, so lets check the window sizes and
     make alterations if necessary.
     --------------------------------------------------------------------*/
  if (!strcmp(validgeo, "NO"))
  {
      if (lowwind[2]*scaley < highwind[2])
          highwind[2] = lowwind[2]*scaley;
      if (lowwind[3]*scalex < highwind[3])
          highwind[3] = lowwind[3]*scalex;
  }
  
  /* If the user didn't specify the number of transform levels (input 0),
     compute the optimal number of levels based on the resolution
     differences for the low and high resolution images.
     --------------------------------------------------------------------*/
  if (nlevels == 0)
  {
      xlevels = log(scalex)/log(2);
      ylevels = log(scaley)/log(2);

      /* The number of levels should be the maximum of the two dimensions.
         If the scaling is an exact power of 2, add one for good measure. */
      nlevels = (xlevels > ylevels) ? (int)xlevels + 1 : (int)ylevels + 1;
  }

  /* Return info.
     ------------*/
  sprintf(windstr, "(%d, %d, %d, %d: %d)", highwind[0], highwind[1],
          highwind[2], highwind[3], highbands[0][0]);
  strcat(hightae[0], windstr);
  strptr[0] = hightae[0];
  if (q_string(&vblock, "HIGH", 1, strptr, P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
  strptr[0] = validgeo;
  if (q_string(&vblock, "VALIDGEO", 1, strptr, P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
  strptr[0] = uselowwind;
  if (q_string(&vblock, "USELOWWIND", 1, strptr, P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
  if (q_real(&vblock, "SCALEY", 1, &scaley, P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
  if (q_real(&vblock, "SCALEX", 1, &scalex, P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
  highnl = (int)highddr.nl;
  if (q_intg(&vblock, "HIGHNL", 1, &highnl, P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
  highns = (int)highddr.ns;
  if (q_intg(&vblock, "HIGHNS", 1, &highns, P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
  if (q_intg(&vblock, "HIGHWIND", 4, highwind, P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
  if (q_intg(&vblock, "NLEVELS", 1, &nlevels, P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
  if (q_out(&vblock) != SUCCESS)
      c_lasmsg(TRUE, "getddrinfo-update", "Error updating variable block.");
 
  /* Return successfully.
  -----------------------*/
  z_exit(SUCCESS, "");
  return SUCCESS;  /* Only used to stop compiler complaints about no return
                      value. */
}
