/******************************************************************************
NAME: getpar.c

PURPOSE:  Retrieve input parameters from TAE.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

ALGORITHM DESCRIPTION:
     See inline comments.

RETURN VALUES:
     E_SUCC              successful completion
     E_FAIL              an error occurred during operation

REFERENCES:
     None.
******************************************************************************/

#include <string.h>
#include "las.h"
#include "util.h"
#include "wavemerge.h"


static float hp_factor;      /* high pass coefficient scaling factor        */


int getpar
(
    struct PARBLK *vblock,        /* I: Pointer to TAE parameter block      */
    char (*lowtae)[],             /* O: TAE file name of low res image      */
    char (*lowhos)[],             /* O: Host file name of low res image     */
    char (*hightae)[],            /* O: TAE file name of high res image     */
    char (*highhos)[],            /* O: Host file name of high res image    */
    char (*outtae)[],             /* O: TAE file name of output image       */
    char (*outhos)[],             /* O: Host file name of output image      */
    long highwind[][4],           /* O: Window specification for high res   */
    long lowwind[][4],            /* O: Window specification for low res    */
    long *lowband,                /* O: Band number for low res intensity   */
    long *highband,               /* O: Band number for high res            */
    int *nlevels,                 /* O: Number of transform levels to apply */
    char *minmax,                 /* O: Min max flag buffer                 */
    int *msgtim                   /* O: Time between display messages       */
)
{
  struct VARIABLE *var;        /* struct for TAE parameters                 */
  
  long highbands[1][MAXBND+1]; /* bands specified for high res image        */
  long highnumbands[1];        /* number of bands for high res image        */
  long hightotbands;           /* total number of bands for high res        */
  long imglimit = 1;           /* max images allowed                        */
  long lowbands[1][MAXBND+1];  /* bands specified for low res image         */
  long lownumbands[1];         /* number of bands for low res image         */
  long lowtotbands;            /* total number of bands for low res         */
  long nmrimg;                 /* number of images input                    */
  long qflag[1][2];            /* flag denoting whether the user specified
                                  windows and/or bands for the input images */
  long same_window;            /* flag denoting whether the user requested
                                  the same windows for all input images     */
  long first_element = 0;      /* index of first array element              */


  /* Retrieve low resolution image name
  -------------------------------------*/
  if (c_getin(vblock, "LOW", &first_element, lowtae, lowhos, &nmrimg, lowwind,
              lowbands, lownumbands, &lowtotbands, &imglimit,
              &same_window, qflag) != E_SUCC)
  {
      c_lasmsg(FALSE, "wavemerge-getin",
               "Error parsing input image specification for LOW.");
      return E_FAIL;
  }

  /* Retrieve high resolution image name
  --------------------------------------*/
  if (c_getin(vblock, "HIGH", &first_element, hightae, highhos, &nmrimg,
              highwind, highbands, highnumbands, &hightotbands, &imglimit,
              &same_window, qflag) != E_SUCC)
  {
      c_lasmsg(FALSE, "wavemerge-getin",
               "Error parsing input image specification for HIGH.");
      return E_FAIL;
  }

  /* Confirm image appropriateness.
     ------------------------------*/
  if (lowtotbands > 1)
  {
      c_lasmsg(FALSE, "wavemerge-bands",
               "Input image %s has more than one band.", lowhos);
      return E_FAIL;
  }
  else if (hightotbands > 1)
  {
      c_lasmsg(FALSE, "wavemerge-bands",
               "Input image %s has more than one band.", highhos);
      return E_FAIL;
  }
  if ((lowwind[0][NL] != highwind[0][NL]) ||
      (lowwind[0][NS] != highwind[0][NS]))
  {
      c_lasmsg(FALSE, "wavemerge-window", "Input images are not same size.");
      return E_FAIL;
  }

  /* Assign band numbers.
     --------------------*/
  *lowband = lowbands[0][0];
  *highband = highbands[0][0];

  /* Retrieve output image name
  -----------------------------*/
  if (c_getout(vblock, "OUT", outtae, outhos) != E_SUCC)
  {
      c_lasmsg(FALSE, "wavemerge-getout",
               "Error parsing output image specification for OUT."); 
      return E_FAIL;
  }

  /* Retrieve number of transform levels.
     ------------------------------------*/
  var = p_find(vblock, "NLEVELS");
  *nlevels = IVAL(*var, 0);
  c_lasmsg(FALSE, "wavemerge-info", "Using %d levels of wavelet transform.",
           *nlevels);

  /* Retrieve high-pass scaling factor.
     ----------------------------------*/
  var = p_find(vblock, "HPFACTOR");
  hp_factor = RVAL(*var, 0);  

  /* Retrieve globals
  -------------------*/
  var = p_find(vblock,"$msgtim");
  *msgtim = IVAL(*var, 0);
  var = p_find(vblock, "$minmax");
  strcpy(minmax, SVAL(*var, 0));

  return E_SUCC;
}


float get_hp_factor(void)
{
  return hp_factor;
}
