/*****************************************************************************

NAME: idwt2.c

PURPOSE: Apply the inverse wavelet transform to a two-dimensional array using
         the wavelet of choice.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     Taken from "Numerical Recipes in C".  See WAVEMERGE design document.

RETURN VALUE:
     pointer to generated inverse transform array

REFERENCES:
     Numerical Recipes in C.
*****************************************************************************/

#include <stdlib.h>
#include "wavelet.h"


float *idwt2                     /* Returns pointer to generated inverse
                                    transform array.  Calling routine must
                                    ensure that memory is freed.            */
(
    float aa[],                  /* I: Array to be inverse transformed      */
    int nrows,                   /* I: Number of rows in resultant array    */
    int ncols,                   /* I: Number of columns in resultant array */
    struct transform func        /* I: Function to be used for 1D transform */
)
{
  int lof;                /* length of the wavelet filter                   */
  int ntrows, ntcols;     /* number of rows and columns in transform array  */
  int offset, toffset;    /* indexing offsets for array and transform array */
  int irow, icol;         /* row and column loop counters                   */

  float *ita;             /* inverse transformed array                      */
  float *wksp;            /* temporary storage for 1D array                 */


  /* Get the length of the wavelet.
     ------------------------------*/
  lof = func.lof;
  
  /* Determine transform array size.  In general, the input array size
     is extended by lof - 1.

     For one-dimensional arrays, the inverse transform takes place in only
     one dimension, so the data extension only exists in the transformable
     dimension.
   -----------------------------------------------------------------------*/
  if (nrows == 1)
      ntrows = 1;
  else if (nrows < lof)
      return NULL;
  else
      ntrows = nrows + lof - 1;
  if (ncols == 1)
      ntcols = 1;
  else if (ncols < lof)
      return NULL;
  else
      ntcols = ncols + lof - 1;

  /* Allocate storage for the inverse transform array and work space.
  -------------------------------------------------------------------*/
  if ((ita = (float *)malloc(ntrows*ncols*sizeof(float))) == NULL)
      return NULL;
  if (ntcols > ntrows)
      wksp = (float *)malloc(ntcols*sizeof(float));
  else
      wksp = (float *)malloc(ntrows*sizeof(float));
  if (wksp == NULL)
  {
      free(ita);
      return NULL;
  }
  
  /* Convolve matrix with filters.
   -------------------------------*/

  /* If more than one column, convolve rows.
   -----------------------------------------*/
  if (ncols > 1)
  {
      /* Inverse transform all rows.
         ---------------------------*/
      for (irow=0, offset=0, toffset=0; irow<ntrows;
           irow++, offset+=ncols, toffset+=ntcols)
      {
          /* Copy row to workspace.
           ------------------------*/
          for (icol=0; icol<ntcols; icol++)
              wksp[icol] = aa[toffset+icol];
          
          /* Inverse transform row.
             ----------------------*/
          if ((*(func.dwt1))(wksp, ntcols, INVERSE) == WAVE_FAILURE)
          {
              free(wksp);
              free(ita);
              return NULL;
          }

          /* Copy transformed row into transform array.
             ------------------------------------------*/
          for (icol=0; icol<ncols; icol++)
              ita[offset+icol] = wksp[icol];
      }
  } /* end row transforms */
  else
  {
      /* Only one column, so copy original array to transform array.
         -----------------------------------------------------------*/
      for (irow=0; irow<ntrows; irow++)
          ita[irow] = aa[irow];
  }
  
  /* Convolve columns. */

  if (nrows > 1)
  {
      /* Inverse transform all columns.
         ------------------------------*/
      for (icol=0; icol<ncols; icol++)
      {
          /* Copy column to workspace.
           ---------------------------*/
          for (irow=0, offset=icol; irow<ntrows; irow++, offset+=ncols)
              wksp[irow] = ita[offset];
      
          /* Inverse transform column.
             -------------------------*/
          if ((*(func.dwt1))(wksp, ntrows, INVERSE) == WAVE_FAILURE)
          {
              free(wksp);
              free(ita);
              return NULL;
          }
      
          /* Copy transformed column into transform array.
             ---------------------------------------------*/
          for (irow=0, offset=icol; irow<nrows; irow++, offset+=ncols)
              ita[offset] = wksp[irow];
      }
  }

  /* Free temporary storage.
     -----------------------*/
  free(wksp);

  /* Return inverse transform array.
     -------------------------------*/
  return ita;
}
