/*****************************************************************************

NAME: inverse_transfrom.c

PURPOSE: For all transform levels, apply the ISIDWT to merged data and copy
         the data into the appropriate buffers at the next higher level.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     See WAVEMERGE design document.

RETURN VALUE:
     pointer to generated inverse transform array (level 0)

REFERENCES:
     WAVEMERGE design document.
*****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "wavemerge.h"
#include "wavelet.h"


struct array_buf *inverse_transform
                              /* Returns a pointer to the generated inverse
                                 transform array (level 0).  The calling
                                 routine is responsible for freeing the
                                 memory.                                    */
(
    struct array_buf *l_last, /* I: Ptr to "last" low res buf in linked list*/
    int nl,                   /* I: Number of lines in merged image block   */
    int ns,                   /* I: Number of samples in merged image block */
    int nlevels               /* I: Number of transform levels to apply     */
)
{
  int nlx, nsx;         /* number of lines and samples, extended by wavelet */
  int nlx2, nsx2;       /* nlx, nsx divided by 2                            */
  int irow;             /* row loop counter                                 */
  int index;            /* array index                                      */
  int offset;           /* array offset pointer                             */
  int level;            /* transform level                                  */
  int ii;               /* loop counter                                     */
  int ishift;           /* transform shift number                           */
  int numbufs;          /* num. of buffers associated with transform level  */
  int ibuf;             /* buffer loop counter                              */

  float *l_buf;         /* temporary pointer for low res shift buffers      */
  
  struct array_buf *l_top_buf, *i_top_buf;
                        /* low res and inverse transform buffer pointers    */
  struct array_buf *i_last;
                        /* ptr to "last" inv. transform buf in linked list  */
  

  /* Compute the number of buffers used for the last transform level (4^n).
     ----------------------------------------------------------------------*/
  numbufs = (int)pow(4, nlevels-1);

  /* Set l_top_buf to the first buffer of the last transform level.
     --------------------------------------------------------------*/
  for (ibuf=0; ibuf<numbufs; ibuf++)
  {
      if (ibuf == 0)
          l_top_buf = l_last;
      else if (l_top_buf->prev != NULL)
          l_top_buf = l_top_buf->prev;
      else /* Hit top of linked list prematurely. */
      {
          c_lasmsg(FALSE, "wavemerge-buffer",
                   "Reached end of buffer list prematurely.");
          return NULL;
      }
  }

  /* Starting from the last transform level, loop
     over all transform levels but the first.
     -------------------------------------------*/
  for (level=nlevels-1; level; level--) /* transform level loop */
  {
      /* The number of buffers at this level is 4^n, n = level number. */
      numbufs = (int)pow(4, level);

      /* Recompute the size of the transform array sizes for this level.
         ---------------------------------------------------------------*/
      nlx2 = nl;
      nsx2 = ns;
      for (ii=0; ii<level; ii++)
      {
          nlx = nlx2 + LOF - 1;
          nsx = nsx2 + LOF - 1;
          nlx2 = nlx/2;
          nsx2 = nsx/2;
      }

      /* Apply ISIDWT to I band for all buffers in this transform level.
       -----------------------------------------------------------------*/
      for (ibuf=0; ibuf<numbufs; ibuf++) /* buffer loop */
      {
          /* Allocate storage for the inverse transform.
             -------------------------------------------*/
          if (ibuf == 0)
          {
              i_last = create_array_buf(NULL);
              if (i_last == NULL)
              {
                  free_all_linked_buffers(l_last);
                  c_lasmsg(FALSE, "wavemerge-buffer",
                           "Error creating transform buffer.");
                  return NULL;
              }
              
          }
          else
          {
              l_top_buf = l_top_buf->next;
              i_last->next = create_array_buf(i_last);
              if (i_last->next == NULL)
              {
                  free_all_linked_buffers(l_last);
                  free_all_linked_buffers(i_last);
                  c_lasmsg(FALSE, "wavemerge-buffer",
                           "Error creating transform buffer.");
                  return NULL;
              }
              i_last = i_last->next;
          }

          /* Apply ISIDWT.
             -------------*/
          i_last->buf = isidwt(l_top_buf->shift_buf, nlx2, nsx2);
          if (i_last->buf == NULL)
          {
              free_all_linked_buffers(l_last);
              free_all_linked_buffers(i_last);
              c_lasmsg(FALSE, "wavemerge-isidwt",
                       "Error computing the ISIDWT.");
              return NULL;
          }
          
      } /* end buffer loop */

      /* Free low resolution storage for this level.
       ---------------------------------------------*/
      for (ibuf=0; ibuf<numbufs; ibuf++)
      {
          free_shift_buf(l_top_buf->shift_buf);
          l_top_buf = l_top_buf->prev;
          free(l_top_buf->next);
          l_top_buf->next = NULL;
      }
      l_last = l_top_buf;

      /* Copy low-pass data into proper buffers in previous
         transform level for all buffers in current level.
       ----------------------------------------------------*/
      i_top_buf = i_last;

      for (ibuf=0, ishift=DIAGONAL+1; ibuf<numbufs; ibuf++) /* buffer loop */
      {
          if (ibuf != 0)
              i_top_buf = i_top_buf->prev; /* previous buffer in linked list*/

          /* We are looping over four shift buffers in order DIAGONAL,
             DOWN, RIGHT, NONE.  If we have completed the no-shift
             inverse transform, prepare for the next diagonal-shift buffer.
             --------------------------------------------------------------*/
          if (ishift == NONE)
          {
              if (l_top_buf->prev != NULL)
                  l_top_buf = l_top_buf->prev;
              else
              {
                  /* Hit top of linked list prematurely. */
                  free_all_linked_buffers(i_last);
                  c_lasmsg(FALSE, "wavemerge-buffer",
                           "Reached end of buffer list prematurely.");
                  return NULL;
              }
              ishift = DIAGONAL + 1;
          }
              
          ishift--;

          /* Copy the data to the buffers of the previous level.
             ---------------------------------------------------*/
          l_buf = l_top_buf->shift_buf[ishift];

          for (irow=0, offset=0, index=0; irow<nlx2;
               irow++, offset+=nsx, index+=nsx2)
              memcpy(&l_buf[offset], &i_top_buf->buf[index],
                     nsx2*sizeof(float));
          
      } /* end buffer loop */

      /* Free inverse transform storage.
       ---------------------------------*/
      free_all_linked_buffers(i_last);
      i_last = NULL;
  
  } /* transform level loop */
  
  /* level 0 */

  /* Apply inverse transfrom to level 0 buffer.
     ------------------------------------------*/
  i_last = create_array_buf(NULL);
  if (i_last == NULL)
  {
      free_all_linked_buffers(l_last);
      c_lasmsg(FALSE, "wavemerge-buffer", "Error creating transform buffer.");
      return NULL;
  }
  i_last->buf = isidwt(l_top_buf->shift_buf, nl, ns);
  if (i_last->buf == NULL)
  {
      free_all_linked_buffers(l_last);
      free_all_linked_buffers(i_last);      
      c_lasmsg(FALSE, "wavemerge-isidwt", "Error computing the ISIDWT.");
      return NULL;
  }

  /* Free all remaining low resolution storage.
   --------------------------------------------*/
  free_all_linked_buffers(l_last);
  l_last = NULL;

  /* Return final inverse transform.
     -------------------------------*/
  return i_last;
}
