/*****************************************************************************

NAME: isidwt.c

PURPOSE: Apply the inverse shift-invariant wavelet transform to a two-
         dimensional array using the Daubechies 4 wavelet.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     See WAVEMERGE design document.

RETURN VALUE:
     pointer to generated inverse transform array

REFERENCES:
     WAVEMERGE design document.
*****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "wavelet.h"


float *isidwt             /* Returns a pointer to the generated inverse
                             transform array.  The calling routine is
                             responsible for freeing the memory.           */
(
    shift_buf ta,         /* I: group of shifted 2D arrays to be inverse
                                transformed                                 */
    int nrows,            /* I: Number of rows in resulting array           */
    int ncols             /* I: Number of columns in resulting array        */
)
{
  int index1, index2;       /* array and transform array indices            */
  int irow, icol;           /* row and column loop counters                 */

  float *s0, *sr, *sd, *sa; /* inverse transformed shift arrays             */
  float *aa;                /* inverse transformed array                    */
  
  struct transform daub = {daub4, 4};


  /* Allocate storage for inverse transform array.
     ---------------------------------------------*/
  if ((aa = (float *)malloc(nrows*ncols*sizeof(float))) == NULL)
      return NULL;

  /* Inverse transform the four shifted transforms.
   ------------------------------------------------*/
  if ((s0 = idwt2(ta[NONE], nrows, ncols, daub)) == NULL)
  {
      free(aa);
      return NULL;
  }
  if ((sr = idwt2(ta[RIGHT], nrows, ncols, daub)) == NULL)
  {
      free(aa);
      free(s0);
      return NULL;
  }
  if ((sd = idwt2(ta[DOWN], nrows, ncols, daub)) == NULL)
  {
      free(aa);
      free(s0);
      free(sr);
      return NULL;
  }
  if ((sa = idwt2(ta[DIAGONAL], nrows, ncols, daub)) == NULL)
  {
      free(aa);
      free(s0);
      free(sr);
      free(sd);
      return NULL;
  }

  /* Merge shift arrays.
     -------------------*/
  /* No shift. */
  memcpy(aa, s0, nrows*ncols*sizeof(float));
  
  /* Shift left. */
  for (irow=0; irow<nrows; irow++)
  {
      index2 = irow*ncols + 1;
      index1 = index2 - 1;
      for (icol=1; icol<ncols; icol++)
          aa[index1++] += sr[index2++];
      aa[index1] += sr[index2-ncols];   /* Wrap from left edge. */
  }

  /* Shift up. */
  for (irow=1; irow<nrows; irow++)
  {
      index2 = irow*ncols;
      index1 = index2 - ncols;
      for (icol=0; icol<ncols; icol++)
          aa[index1++] += sd[index2++];
  }
  index2 = 0;                           /* Wrap from top edge. */
  for (icol=0; icol<ncols; icol++)
      aa[index1++] += sd[index2++];

  /* Shift to left and up. */
  for (irow=1; irow<nrows; irow++)
  {
      index2 = irow*ncols + 1;
      index1 = index2 - ncols - 1;
      for (icol=1; icol<ncols; icol++)
          aa[index1++] += sa[index2++];
      aa[index1++] += sa[index2-ncols]; /* Wrap from left edge. */
 }
  index2 = 1;                           /* Wrap from top edge. */
  for (icol=1; icol<ncols; icol++)
      aa[index1++] += sa[index2++];
  aa[index1++] += sa[index2-ncols];     /* Wrap from left edge. */
  
  /* Average values from the four shift images.
   --------------------------------------------*/
  for (irow=0, index1=0; irow<nrows; irow++)
      for (icol=0; icol<ncols; icol++)
          aa[index1++] *= 0.25;

  /* Free temporary storage.
     -----------------------*/
  free(s0);
  free(sr);
  free(sd);
  free(sa);

  /* Return inverse transform array.
     -------------------------------*/
  return aa;
}
