/*****************************************************************************

NAME: max_merge_data.c

PURPOSE: Merge the high resolution data into the low resolution array for
         the wavelet transform high-pass data.  The merge is based on the
         maximum value for each high pass element.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     See WAVEMERGE design document.

     Note that we are only merging the high pass data.  This means that only
     the HL, LH, and HH quadrants of the transform arrays are merged.  The
     LL quadrant within the low resolution transform is left unchanged.

                                        ---------------
                                        |      |      |
                                        |  LL  |  HL  |  transform
                                        |      |      |    array
                                        ---------------
                                        |      |      |
                                        |  LH  |  HH  |
                                        |      |      |
                                        ---------------

REFERENCES:
     WAVEMERGE design document.
*****************************************************************************/

#include <math.h>
#include "wavemerge.h"


void max_merge_data
(
    shift_buf h_t,           /* I: High resolution wavelet transform array */
    shift_buf l_t,           /* I/O: On input: low res wavelet transform
                                     On output: merged wavelet transform   */
    int nrows,               /* I: Number of rows in array                 */
    int ncols                /* I: Number of columns in array              */
)
{
  int index;                /* buffer index                                */
  int irow, icol;           /* row and column loop counters                */
  int ishift;               /* shift counter                               */
  int mid_col = ncols/2;    /* midpoint column number                      */
  int mid_row = nrows/2;    /* midpoint row number                         */
  int offset;               /* array pointer offset                        */

  float *h_buf, *l_buf;     /* temporary buffer pointers                   */
  float hp_factor = get_hp_factor();
                            /* high pass coefficient scaling factor        */
  float hp_factor_m1 = 1. - hp_factor; /* var. to reduce loop computations */


  /* Loop over all four shift arrays.
     --------------------------------*/
  for (ishift=NONE; ishift<=DIAGONAL; ishift++)  /* shift loop */
  {
      /* Use temporary array pointers to simplify notation.
         --------------------------------------------------*/
      h_buf = h_t[ishift];
      l_buf = l_t[ishift];

      /* Loop over all rows and columns, merging the high res data
         into the low res array for the high pass transform components.
         --------------------------------------------------------------*/
      for (irow=0, offset = 0; irow<mid_row; irow++, offset += ncols)
      {
          for (icol=mid_col, index=offset+mid_col; icol<ncols;
               icol++, index++)
              if (fabs(l_buf[index]) < fabs(h_buf[index]))
                  l_buf[index] = hp_factor_m1*l_buf[index] +
                                 hp_factor*h_buf[index];
              else
                  l_buf[index] = hp_factor*l_buf[index] +
                                 hp_factor_m1*h_buf[index];
      }
  
      for (irow=mid_row; irow<nrows; irow++, offset += ncols)
      {
          for (icol=0, index=offset; icol<ncols; icol++, index++)
              if (fabs(l_buf[index]) < fabs(h_buf[index]))
                  l_buf[index] = hp_factor_m1*l_buf[index] +
                                 hp_factor*h_buf[index];
              else
                  l_buf[index] = hp_factor*l_buf[index] +
                                 hp_factor_m1*h_buf[index];
      }

  }  /* end shift loop */
}
