/*****************************************************************************

NAME: merge_image_block.c

PURPOSE: Merge the high resolution data and low resolution image blocks
         using multiple shift-invariant discrete wavelet transform levels.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     See WAVEMERGE design document.

RETURN VALUES:
     E_SUCC              successful completion
     E_FAIL              an error occurred during operation

REFERENCES:
     WAVEMERGE design document.
*****************************************************************************/

#include <string.h>
#include "las.h"
#include "wavemerge.h"


int merge_image_block
(
    float *lr_data_buf, /* I/O: On input: low res image block
                                On output: merged image block               */
    float *hr_data_buf, /* I: High resolution image block                   */
    int nl,             /* I: Number of lines in image block                */
    int ns,             /* I: Number of samples in image block              */
    int nlevels         /* I: Number of transform levels to apply           */
)
{
  int index, index1;    /* array index                                      */
  int iline;            /* line loop counter                                */
  
  struct array_buf *l_last, *i_last;
                        /* ptr to "last" low res and inverse transform
                           buffers in linked list                           */


  /* Apply the SIDWT to both the high resolution band and the low
     resolution intensity band for all transform levels.  The high
     frequency data from both is then merged and stored in the
     low resolution buffers.

     Note that transform_merge will reduce the values of nl and ns
     by LOF - 1.
    --------------------------------------------------------------*/
  if ((l_last = transform_merge(lr_data_buf, hr_data_buf, &nl, &ns, nlevels))
      == NULL)
  {
      c_lasmsg(FALSE, "wavemerge-transform",
               "Error encountered during transform-merge process.");
      return E_FAIL;
  }

  /* For all transform levels, apply the ISIDWT to merged data
     and copy the data into the appropriate buffers at the next
     higher level.
    -----------------------------------------------------------*/
  if ((i_last = inverse_transform(l_last, nl, ns, nlevels)) == NULL)
  {
      c_lasmsg(FALSE, "wavemerge-invtrans",
               "Error encountered during inverse transform process.");
      return E_FAIL;
  }

  /* Copy level 0 inverse transform back into low resolution buffer.
     The valid data in the inverse transform buffer is surrounded by
     a boundary region of size LOF - 1.

                +-----------------+ --- ^
                |  <---- ns --->  |     | LOF - 1
                |^ +-----------+  | --- v
                || |           |  |
                || |           |  |<-- inverse transform buffer
                || |           |  |
                |nl|           |<-|--- valid data 
                || |           |  |
                || |           |  |
                |v +-----------+  |
                |                 |
                +-----------------+               
  ------------------------------------------------------------------*/
  index=(LOF - 1)*(ns + 2*(LOF - 1)) + LOF - 1;
  index1 = 0;
  for (iline=0; iline<nl; iline++)
  {
      memcpy(&lr_data_buf[index], &i_last->buf[index1], ns*sizeof(float));
      index += ns + 2*(LOF - 1);
      index1 += ns;
  }
  
  /* Free inverse transform storage.
   ---------------------------------*/
  free_all_linked_buffers(i_last);
      
  return E_SUCC;
}
