/*****************************************************************************

NAME: mirror_bounds.c

PURPOSE: Given a two-dimensional array, mirror a specified number of rows
         and columns around the entire image.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     See WAVEMERGE design document.

RETURN VALUE:
     pointer to generated mirror boundary array

REFERENCES:
     WAVEMERGE design document.
*****************************************************************************/

#include <stdlib.h>
#include "wavelet.h"

float *mirror_bounds   /* Returns a pointer to the generated mirror boundary
                          array.  The calling routine is responsible for
                          freeing the array memory.                         */
(
    float *aa,         /* I: 2D array                                       */
    int nrows,         /* I: Number of rows in input array                  */
    int ncols,         /* I: Number of columns in input array               */
    int boundary       /* I: Number of boundary rows, columns to mirror     */
)
{
  int index;           /* array index counter                               */
  int irow, icol;      /* row and column loop counters                      */
  int nmrows, nmcols;  /* number of rows and columns in mirrored array      */
  int offset;          /* array offset pointer                              */

  float *ma;           /* mirror array                                      */


  /* Compute the size of the new array.
     ----------------------------------*/
  nmrows = nrows + 2*boundary;
  nmcols = ncols + 2*boundary;

  /* Allocate space for mirror array.
   ----------------------------------*/
  if ((ma = (float *)malloc(nmrows*nmcols*sizeof(float))) == NULL)
      return NULL;

  /* Fill the mirror array.
   ------------------------*/

  /* Mirror top boundary.
   ----------------------*/
  index = 0;
  for (irow=0, offset=ncols*(boundary-1); irow<boundary;
       irow++, offset-=ncols)
  {
      /* Fill vector mirror extension of left boundary.
       ------------------------------------------------*/
      for (icol=0; icol<boundary; icol++)
          ma[index++] = aa[offset+boundary-icol-1];

      /* Copy row data.
       ----------------*/
      for (icol=0; icol<ncols; icol++)
          ma[index++] = aa[offset+icol];
          
      /* Fill vector mirror extension of right boundary.
       -------------------------------------------------*/
      for (icol=0; icol<boundary; icol++)
          ma[index++] = aa[offset+ncols-icol-1];
  }

  /* Copy body of matrix.
   ----------------------*/
  for (irow=0, offset=0; irow<nrows; irow++, offset+=ncols)
  {
      /* Fill vector mirror extension of left boundary.
       ------------------------------------------------*/
      for (icol=0; icol<boundary; icol++)
          ma[index++] = aa[offset+boundary-icol-1];

      /* Copy row data.
       ----------------*/
      for (icol=0; icol<ncols; icol++)
          ma[index++] = aa[offset+icol];
          
      /* Fill vector mirror extension of right boundary.
       -------------------------------------------------*/
      for (icol=0; icol<boundary; icol++)
          ma[index++] = aa[offset+ncols-icol-1];
  }

  /* Mirror bottom boundary.
   -------------------------*/
  for (irow=0, offset=ncols*(nrows-1); irow<boundary; irow++, offset-=ncols)
  {
      /* Fill vector mirror extension of left boundary.
       ------------------------------------------------*/
      for (icol=0; icol<boundary; icol++)
          ma[index++] = aa[offset+boundary-icol-1];

      /* Copy row data.
       ----------------*/
      for (icol=0; icol<ncols; icol++)
          ma[index++] = aa[offset+icol];
          
      /* Fill vector mirror extension of right boundary.
       -------------------------------------------------*/
      for (icol=0; icol<boundary; icol++)
          ma[index++] = aa[offset+ncols-icol-1];
  }

  /* Return the mirrored-boundary array.
     -----------------------------------*/
  return ma;
  
}
