/*****************************************************************************

NAME: sidwt.c

PURPOSE: Apply the shift-invariant wavelet transform to a two-dimensional
         array using the Daubechies 4 wavelet.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     See WAVEMERGE design document.

RETURN VALUES:
     WAVE_SUCCESS              successful completion
     WAVE_FAILURE              an error occurred during operation

REFERENCES:
     WAVEMERGE design document.
*****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "wavelet.h"


int sidwt
(
    float aa[],             /* I: 2D array to be transformed                */
    int nrows,              /* I: Number of rows in array                   */
    int ncols,              /* I: Number of columns in array                */
    int level,              /* I: Transform level being applied             */
    shift_buf transform_buf /* O: Shift buffer                              */
)
{
  int index;                /* array index                                  */
  int ntot;                 /* total number of elements in the "smooth"
                               low-pass region of the input array           */
  int irow;                 /* row loop counter                             */
  int ishift;           /* shift loop counter (NONE, RIGHT, DOWN, DIAGONAL) */
  int boundary;             /* number of rows, columns comprising the mirror
                               boundary                                     */
  int ntrows, ntcols;       /* number of rows and columns in the "smooth"
                               low-pass region of the input array           */
  int nmrows, nmcols;       /* number of rows and columns in the mirrored
                               array                                        */
  int offset;               /* array offset pointer                         */

  float *ma;                /* mirror array                                 */
  float *shift_array,       /* shift arrays                                 */
        *shift_array2;
  float *temp_buf;          /* temporary pointer                            */

  struct transform daub = {daub4, 4}; /* transform type structure           */

  
  /* Initialize various parameters.
     ------------------------------*/
  ntrows = nrows;
  ntcols = ncols;

  /* For level 0 (first level) transforms, the entire input array is
     "smooth" data (hasn't been separated into low pass and high pass
     quadrants).  Also, the input array is assumed to already contain
     the "mirror" boundaries.  This is because the input array represents
     an sub-image block that contains boundary data to handle multi-level
     transform boundary data corruption.  See the WAVEMERGE design
     document for a better description.
     --------------------------------------------------------------------*/
  if (level != 0)
  {
      boundary = daub.lof - 1;
      ntrows /= 2;
      ntcols /= 2;
      nmrows = ntrows + 2*boundary;
      nmcols = ntcols + 2*boundary;
  }
  else
  {
      nmrows = ntrows;
      nmcols = ntcols;
  }

  ntot = ntrows*ntcols;

  /* Set up pointers to the four shift transforms.  Note that the shift
     transforms are stacked in memory.
  ---------------------------------------------------------------------*/
  transform_buf[NONE] = NULL;
  transform_buf[RIGHT] = NULL;
  transform_buf[DOWN] = NULL;
  transform_buf[DIAGONAL] = NULL;

  /* Allocate space for shift arrays.
   ---------------------------------*/
  if ((shift_array = (float *)malloc(ntrows*ntcols*sizeof(float))) == NULL)
      return WAVE_FAILURE;
  if ((shift_array2 = (float *)malloc(ntrows*ntcols*sizeof(float))) == NULL)
  {
      free(shift_array);
      return WAVE_FAILURE;
  }      
  
  /* Extract smooth data from input array.
     -------------------------------------*/
  for (irow=0, index=0, offset=0; irow<ntrows;
       irow++, offset+=ncols, index+=ntcols)
      memcpy(&shift_array[index], &aa[offset], ntcols*sizeof(float));

  /* Loop over four shifts (none, right, down, diagonal).
     
     ishift = NONE:     shift_array contains un-shifted data
                        shift_array2 unused
     ishift = RIGHT:    shift_array contains un-shifted data
                        shift_array2 contains right-shifted data
     ishift = DOWN:     shift_array contains un-shifted data
                        shift_array2 contains down-shifted data
     ishift = DIAGONAL: shift_array contains diagonal-shifted data
                        shift_array2 contains down-shifted data
     ---------------------------------------------------*/
  for (ishift=NONE; ishift<=DIAGONAL; ishift++)
  {
      if (ishift == RIGHT)
      {
          /* Apply right shift to unshifted data.
             ------------------------------------*/
          for (irow=0, index=0, offset=0; irow<ntrows;
               irow++, offset+=ntcols, index+=ntcols)
          {
              /* wrap from right edge */
              shift_array2[index] = shift_array[ntcols+offset-1];
              memcpy(&shift_array2[index+1], &shift_array[offset],
                     (ntcols-1)*sizeof(float));
          }
      }
      else if (ishift == DOWN)
      {
          /* Apply down shift to unshifted data.
             -----------------------------------*/
          /* wrap from bottom edge */
          memcpy(shift_array2, &shift_array[ntot-ntcols],
                 ntcols*sizeof(float));
          for (irow=1, index=ntcols, offset=0; irow<ntrows;
               irow++, offset+=ntcols, index+=ntcols)
              memcpy(&shift_array2[index], &shift_array[offset],
                     ntcols*sizeof(float));
      }
      else if (ishift == DIAGONAL)
      {
          /* Apply right shift to down-shifted data.
             ---------------------------------------*/
          for (irow=0, index=0, offset=0; irow<ntrows;
               irow++, offset+=ntcols, index += ntcols)
          {
              /* wrap from right edge */
              shift_array[index] = shift_array2[ntcols+offset-1];
              memcpy(&shift_array[index+1], &shift_array2[offset],
                     (ntcols-1)*sizeof(float));
          }  
      }
      
      /* If this is not a level 0 transform, then the data array boundaries
         must be mirrored.
      ---------------------------------------------------------------------*/
      if (ishift == NONE || ishift == DIAGONAL)
      {
          if (level != 0)
              ma = mirror_bounds(shift_array, ntrows, ntcols, boundary);
          else
              ma = shift_array;
      }
      else
      {
          if (level != 0)
              ma = mirror_bounds(shift_array2, ntrows, ntcols, boundary);
          else
              ma = shift_array2;
      }
      if (ma == NULL)
      {
          free_shift_buf(transform_buf);
          free(shift_array);
          free(shift_array2);
          return WAVE_FAILURE;
      }
      
      /* Apply DWT.
       ------------*/
      if ((temp_buf = dwt2(ma, nmrows, nmcols, daub)) == NULL)
      {
          free_shift_buf(transform_buf);
          free(shift_array);
          free(shift_array2);
          if (level != 0)
              free(ma);
          return WAVE_FAILURE;
      }

      /* Tie to appropriate shift buffer.
         --------------------------------*/
      transform_buf[ishift] = temp_buf;

      if (level != 0)
          free(ma);   /* Free the mirror array. */
      
  } /* end shift loop */

  /* Free shift arrays.
     ------------------*/
  free(shift_array);
  free(shift_array2);
  
  /* Return the transform array.
     ---------------------------*/
  return WAVE_SUCCESS;

}
