/*****************************************************************************

NAME: transform_merge.c

PURPOSE: Apply a given number of levels of shift-invariant discrete wavelet
         transforms (SIDWT) to a high resolution data set and a low resolution
         data set, and merge the high pass components.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     See WAVEMERGE design document.

RETURN VALUE:
     pointer to generated transform array linked list

REFERENCES:
     WAVEMERGE design document.
*****************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "wavemerge.h"
#include "wavelet.h"


struct array_buf *transform_merge
                        /* Returns a pointer to the generated transform
                           array linked list.  The calling routine is
                           responsible for freeing the memory.              */
(
    float *lr_data_buf, /* I: Low resolution image block                    */
    float *hr_data_buf, /* I: High resolution image block                   */
    int *nl,            /* I/O: On input: num of lines in low res image blk
                                On output: num of lines in merged image blk */
    int *ns,            /* I/O: On input: num of samps in low res image blk
                                On output: num of samps in merged image blk */
    int nlevels         /* I: Number of transform levels to apply           */
)
{
  int nlx, nsx;         /* number of lines and samples, extended by wavelet */
  int nlx2, nsx2;       /* nlx, nsx divided by 2                            */
  int level;            /* transform level                                  */
  int ishift;           /* transform shift number                           */
  int numbufs;          /* num. of buffers associated with transform level  */
  int ibuf;             /* buffer loop counter                              */
  
  float *h_buf, *l_buf; /* temporary ptrs to high and low res shift arrays  */

  struct array_buf *h_top_buf, *l_top_buf;
                        /* high res and low res buffer pointers             */
  struct array_buf *h_last, *l_last;
                        /* ptr to "last" high res and low res buffers in
                           linked list                                      */


  /*------------------------------------------------------------
    ------------------------------------------------------------
     Apply the SIDWT to both the high resolution band and the low
     resolution intensity band for all transform levels.  The high
     frequency data from both is then merged and stored in the
     low resolution buffers.
    ------------------------------------------------------------
    ------------------------------------------------------------*/

  /* Establish the first members of the linked lists.
   --------------------------------------------------*/
  level = 0;
  numbufs = 1;
  
  h_last = create_array_buf(NULL);
  if (h_last == NULL)
      return NULL;
  if (sidwt(hr_data_buf, *nl, *ns, level, h_last->shift_buf) != WAVE_SUCCESS)
  {
      free_all_linked_buffers(h_last);
      return NULL;
  }
  
  l_last = create_array_buf(NULL);
  if (l_last == NULL)
  {
      free_all_linked_buffers(h_last);
      return NULL;
  }
  if (sidwt(lr_data_buf, *nl, *ns, level, l_last->shift_buf) != WAVE_SUCCESS)
  {
      free_all_linked_buffers(h_last);
      free_all_linked_buffers(l_last);
      return NULL;
  }      

  /* Reduce the dimensions of the original array.
     --------------------------------------------*/
  *nl -= 2*(LOF - 1);
  *ns -= 2*(LOF - 1);

  /* Set size of transform array.
     ----------------------------*/
  nlx = *nl + LOF - 1;
  nsx = *ns + LOF - 1;

  /* Merge transform data for first level.
     -------------------------------------*/
  h_top_buf = h_last;
  l_top_buf = l_last;
  max_merge_data(h_top_buf->shift_buf, l_top_buf->shift_buf, nlx, nsx);

  /* Loop over all remaining transform levels.
     -----------------------------------------*/
  for (level=1; level<nlevels; level++)   /* transform level loop */
  {
      /* The number of buffers is a power of four, corresponding to the
         four shifts (no shift, right shift, down shift, and diagonal
         shift). */
      numbufs = (int)pow(4, level);

      /* Each shift array in the next transform will be 1/4 the size of
         the input array. */
      nlx2 = nlx/2;
      nsx2 = nsx/2;

      /* Apply SIDWT to all buffers in level.
       --------------------------------------*/
      for (ibuf=0, ishift=NONE; ibuf<numbufs; ibuf++) /* buffer loop */
      {
          /* The shift buffers are ordered NONE, RIGHT, DOWN, and DIAGONAL.
             So if we have finished a diagonal buffer, go to the next
             no-shift buffer.
             --------------------------------------------------------------*/
          if (ishift > DIAGONAL)
          {
              h_top_buf = h_top_buf->next;
              l_top_buf = l_top_buf->next;
              ishift = NONE;
          }

          /* The SIDWT returns a buffer consisting of the results from
             the four shifts stacked one behind the other.
                  +-----+ +-----+ +-----+ +-----+
                  | none| |right| | down| | diag|
                  +-----+ +-----+ +-----+ +-----+
          */

          /* Allocate space for next elements in linked lists.
             -------------------------------------------------*/
          h_last->next = create_array_buf(h_last);
          if (h_last->next == NULL)
          {
              free_all_linked_buffers(h_last);
              free_all_linked_buffers(l_last);
              return NULL;
          }
          h_last = h_last->next;
          
          l_last->next = create_array_buf(l_last);
          if (l_last->next == NULL)
          {
              free_all_linked_buffers(h_last);
              free_all_linked_buffers(l_last);
              return NULL;
          }
          l_last = l_last->next;

          /* Apply transform to appropriate shift buffer.
             --------------------------------------------*/
          h_buf = h_top_buf->shift_buf[ishift];
          l_buf = l_top_buf->shift_buf[ishift];
          if (sidwt(h_buf, nlx, nsx, level, h_last->shift_buf)
              != WAVE_SUCCESS)
          {
              free_all_linked_buffers(h_last);
              free_all_linked_buffers(l_last);
              return NULL;
          }
          if (sidwt(l_buf, nlx, nsx, level, l_last->shift_buf)
              != WAVE_SUCCESS)
          {
              free_all_linked_buffers(h_last);
              free_all_linked_buffers(l_last);
              return NULL;
          }

          ishift++;
          
      } /* end buffer loop */

      /* Free high resolution storage that is no longer needed.
       --------------------------------------------------------*/
      free_shift_buf(h_top_buf->shift_buf);
      for (ibuf=0; ibuf<numbufs/4-1; ibuf++)
      {   
          h_top_buf = h_top_buf->prev;
          free_shift_buf(h_top_buf->shift_buf);
      }

      /* Set the size of the transform arrays.
         -------------------------------------*/
      nlx = nlx2 + LOF - 1;
      nsx = nsx2 + LOF - 1;

      /* Merge the high resolution and low resolution data for all buffers
         at this level.
      --------------------------------------------------------------------*/
      h_top_buf = h_last;
      l_top_buf = l_last;
      max_merge_data(h_top_buf->shift_buf, l_top_buf->shift_buf, nlx,nsx);
      for (ibuf=1; ibuf<numbufs; ibuf++)
      {
          h_top_buf = h_top_buf->prev;
          l_top_buf = l_top_buf->prev;
          max_merge_data(h_top_buf->shift_buf, l_top_buf->shift_buf, nlx,nsx);
      }
      
  }   /* end transform level loop */
  
  /* Free all high resolution buffers.
   -----------------------------------*/
  free_all_linked_buffers(h_last);
  h_last = NULL;

  /* Return last member of low resolution list.
     ------------------------------------------*/
  return l_last;
}
