#ifndef _WAVELET_H_
#define _WAVELET_H_

/* Transform or inverse transform. */
#define FORWARD 1
#define INVERSE -1

/* Daubechies fourth-order coefficients. */
#define C0  0.4829629131445341
#define C1  0.8365163037378079
#define C2  0.2241438680420134
#define C3 -0.1294095225512604

/* Shift types. */
enum
{
    NONE,             /* no shift               */
    RIGHT,            /* shift right one column */
    DOWN,             /* shift down one row     */
    DIAGONAL          /* shift right and down   */
};

/* Wavelet function success/failure. */
#define WAVE_SUCCESS 0
#define WAVE_FAILURE 1

/* Structure definitions */

/* shift buffers: four pointers to arrays representing none, right, down, and
   diagonal shifts */
typedef float *shift_buf[4];

/* wavelet transform function (type) and size */
struct transform
{
    int (*dwt1)(float [], int, int);  /* wavelet function name      */
    int lof;                          /* length of wavelet (filter) */
};

/* Functional prototypes */

int daub4
(
    float aa[],            /* I/O: One-dimensional array of data to be
                                   transformed                              */
    int na,                /* I: Size of array
                                 (not counting mirror extension)            */
    int type               /* I: Flag denoting type of transform:
                                 forward or reverse                         */
);

float *dwt2                      /* Returns pointer to generated transform
                                    array.  Calling routine must ensure that
                                    memory is freed.                        */
(
    float aa[],                  /* I: Array to be transformed              */
    int nrows,                   /* I: Number of rows in array              */
    int ncols,                   /* I: Number of columns in array           */
    struct transform func        /* I: Function to be used for 1D transform */
);

void free_shift_buf
(
    shift_buf buf           /* I: shift buffer structure to be freed        */
);

float *idwt2                     /* Returns pointer to generated inverse
                                    transform array.  Calling routine must
                                    ensure that memory is freed.            */
(
    float aa[],                  /* I: Array to be inverse transformed      */
    int nrows,                   /* I: Number of rows in resultant array    */
    int ncols,                   /* I: Number of columns in resultant array */
    struct transform func        /* I: Function to be used for 1D transform */
);

float *isidwt             /* Returns a pointer to the generated inverse
                             transform array.  The calling routine is
                             responsible for freeing the memory.           */
(
    shift_buf ta,         /* I: group of shifted 2D arrays to be inverse
                                transformed                                 */
    int nrows,            /* I: Number of rows in resulting array           */
    int ncols             /* I: Number of columns in resulting array        */
);

float *mirror_bounds   /* Returns a pointer to the generated mirror boundary
                          array.  The calling routine is responsible for
                          freeing the array memory.                         */
(
    float *aa,         /* I: 2D array                                       */
    int nrows,         /* I: Number of rows in input array                  */
    int ncols,         /* I: Number of columns in input array               */
    int boundary       /* I: Number of boundary rows, columns to mirror     */
);

int sidwt
(
    float aa[],             /* I: 2D array to be transformed                */
    int nrows,              /* I: Number of rows in array                   */
    int ncols,              /* I: Number of columns in array                */
    int level,              /* I: Transform level being applied             */
    shift_buf transform_buf /* O: Shift buffer                              */
);

#endif /* #ifndef _WAVELET_H_ */
