/*****************************************************************************

NAME: WAVEMERGE

PURPOSE: Merge high resolution single-band data into a lower resolution RGB
         image.  Wavelets are used to retain image details.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
     See WAVEMERGE design document.

REFERENCES:
     See WAVEMERGE design document.
*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "typlim.h"
#include "util.h"
#include "wavemerge.h"

#define MIRROR 1                /* mirror value for get_buf()               */
#define BLOCK_IMAGE_SIZE 1000   /* number of lines/samples in image block   */


int main(void)
{
  struct PARBLK vblock;       /* TAE parameter block                        */
  
  struct FDESC *lowfdesc;     /* input image file descriptor                */
  struct FDESC *highfdesc;    /* input image file descriptor                */
  struct FDESC *outfdesc;     /* output image file descriptor               */

  int denom;                  /* denominator for computing boundary         */
  int icb, irb;               /* column and row image block counters        */
  int ii;                     /* counter                                    */
  int index, index1;          /* array indices                              */
  int iline, isamp;           /* line and sample counters                   */
  int msgtime;                /* time between LAS display messages          */
  int nrows_extra_line, ncols_extra_samp;
                              /* the number of rows and columns of image
                                 blocks that will contain an extra line and
                                 sample, respectively */
  int nlevels;                /* number of wavelet transform levels         */
  int num_block_rows, num_block_cols;
                              /* number of image block rows and columns
                                 required */
  int status;                 /* function return status                     */
  
  long boundary;              /* image block boundary size                  */
  long dtype;                 /* data type                                  */
  long fill = MIRROR;         /* buffer fill value for get_buf              */
  long highbands[1][MAXBND+1];/* high resolution band; in array to satisfy
                                 c_upddr                                    */
  long lowband, highband,     /* low res intensity band, high res band, and */
       outband = 1;           /* merged band                                */
  long nbands = 1;            /* number of bands (1 intensity band)         */
  long nextflg, nextflp;      /* flags telling get_buf and put_buf where to
                                 get next image block                       */
  long nlav, nsav;            /* number of valid lines, samples in image
                                 block buffer                               */
  long *nl_block, *ns_block;  /* number of lines/samples in blocks          */
  long nlbuf, nsbuf;          /* number of lines and samples in image
                                 block buffer                               */
  long num_in = 2;            /* number of input files                      */
  long num_out = 1;           /* number of output files                     */
  long odtype;                /* output data type                           */
  long slb_high, ssb_high,    /* starting line, sample for image block      */
       slb_low, ssb_low,      /* in various temporary forms                 */
       slb_out, ssb_out;
  long std_out;               /* standard output LUN                        */
  long highwind[1][4],        /* image window starting line, starting       */
       lowwind[1][4],         /* sample, number of lines, number of samples */
       outwind[4];            /* for high res and low res images            */
  long upddr_flag = NOCOMB;   /* flag denoting whether bands will be combined
                                 in the output image; set to no             */
  
  float *lr_data_buf, *hr_data_buf; /* low and high resolution image buffers*/

  double line_inc = 1,        /* line/sample increment values for c_upddr   */
         samp_inc = 1;
  double minpix[3], maxpix[3];/* Array of min/max pixel values for all bands*/

  char lowtae[1][CMLEN];      /* TAE version of low resolution image name   */
  char lowhos[1][CMLEN];      /* host version of low res. image name        */
  char hightae[1][CMLEN];     /* TAE version of high res. image name        */
  char highhos[1][CMLEN];     /* host version of high res. image name       */
  char outtae[1][CMLEN];      /* TAE version of output image name           */
  char outhos[1][CMLEN];      /* host version of output image name          */
  char inhos[2][CMLEN];       /* host versions of input image names         */
  char minmax[8];             /* min/max pixel calculation flag             */
  

  /* Initialize TAE processing.
  -----------------------------*/
  c_inlas(&vblock, &std_out);

  /* Get the user input.
   ---------------------*/
  if (getpar(&vblock, lowtae, lowhos, hightae, highhos, outtae, outhos,
             highwind, lowwind, &lowband, &highband, &nlevels, minmax,
             &msgtime) != E_SUCC)
        c_lasmsg(TRUE, "wavemerge-parm", "Error retrieving parameters.");

  /* Set the window and data types.
     ------------------------------*/
  outwind[SL] = 1;
  outwind[SS] = 1;
  outwind[NL] = lowwind[0][NL];
  outwind[NS] = lowwind[0][NS];
  dtype = EREAL;                /* Work in floating point. */
  odtype = EWORD;               /* LAS HIS-HIS2RGB requires I*2 input data. */

  /* Compute the size of the buffer boundary required based on the
     wavelet size (LOF) and the number of levels requested.
  ----------------------------------------------------------------*/
  if ((nlevels == 1) || (LOF == 2))
      boundary = 1;   /* first element is always corrupted by mirroring */
  else
  {
      denom = 2;
      boundary = (LOF - 2)/denom;
      for (ii=2; ii<nlevels; ii++)
      {
          boundary += (LOF + denom - 1)/(2*denom);
          denom *=2;
      }
      boundary *= pow(2, nlevels);
      boundary += 1;  /* first element is always corrupted by mirroring */
  }
  boundary += LOF;  /* boundary with wavelet length extension */
  boundary += 1;    /* for shift buffers */

  /* Compute the number of image blocks required.
   ----------------------------------------------*/
  num_block_rows = (int)((float)outwind[NL]/BLOCK_IMAGE_SIZE + 0.5);
  num_block_cols = (int)((float)outwind[NS]/BLOCK_IMAGE_SIZE + 0.5);

  if (num_block_rows == 0)
      num_block_rows = 1;
  if (num_block_cols == 0)
      num_block_cols = 1;

  if ((nl_block = (long *)malloc(num_block_rows*sizeof(long))) == NULL)
  {
      c_lasmsg(FALSE, "wavemerge-alloc", "Error allocating memory.");
      c_lasmsg(TRUE, "wavemerge-fatal", "Fatal error encountered.");
  }
  
  if ((ns_block = (long *)malloc(num_block_cols*sizeof(long))) == NULL)
  {
      free(nl_block);
      c_lasmsg(FALSE, "wavemerge-alloc", "Error allocating memory.");
      c_lasmsg(TRUE, "wavemerge-fatal", "Fatal error encountered.");
  }
  
  /* Compute the number of lines and samples in each block.
  ---------------------------------------------------------*/
  nrows_extra_line = outwind[NL] % num_block_rows;
  ncols_extra_samp = outwind[NS] % num_block_cols;
  nl_block[0] = outwind[NL]/num_block_rows;
  for (ii=1; ii<num_block_rows; ii++)
      nl_block[ii] = nl_block[0];
  ns_block[0] = outwind[NS]/num_block_cols;
  for (ii=1; ii<num_block_cols; ii++)
      ns_block[ii] = ns_block[0];
  for (ii=0; ii<nrows_extra_line; ii++)
      nl_block[ii]++;
  for (ii=0; ii<ncols_extra_samp; ii++)
      ns_block[ii]++;

  /* Allocate storage for image blocks.
   ------------------------------------*/
  if ((lr_data_buf = (float *)malloc((nl_block[0]+2*boundary)*
                                     (ns_block[0]+2*boundary)*
                                     sizeof(float))) == NULL)
  {
      free(nl_block);
      free(ns_block);
      c_lasmsg(FALSE, "wavemerge-alloc", "Error allocating memory.");
      c_lasmsg(TRUE, "wavemerge-fatal", "Fatal error encountered.");
  }
  if ((hr_data_buf = (float *)malloc((nl_block[0]+2*boundary)*
                                     (ns_block[0]+2*boundary)*
                                     sizeof(float))) == NULL)
  {
      free(nl_block);
      free(ns_block);
      free(lr_data_buf);
      c_lasmsg(FALSE, "wavemerge-alloc", "Error allocating memory.");
      c_lasmsg(TRUE, "wavemerge-fatal", "Fatal error encountered.");
  }

  /* Loop over all image blocks.
     ---------------------------*/
  nextflg = INIT;                       /* first call to get_buf()      */
  nextflp = CREAT;                      /* first call to put_buf()      */
  slb_high = highwind[0][SL];
  ssb_high = highwind[0][SS];
  slb_low = lowwind[0][SL];
  ssb_low = lowwind[0][SS];
  slb_out = outwind[SL];
  ssb_out = outwind[SS];
  for (icb=0; icb<num_block_cols; icb++) /* image block column loop */
  {
      /* Work-around for get_buf's inability to properly handle changing
         block dimensions.
      ------------------------------------------------------------------*/
      if (icb > 0)
      {
          ssb_high += ns_block[icb-1] - ns_block[icb];
          ssb_low += ns_block[icb-1] - ns_block[icb];
          ssb_out += ns_block[icb-1] - ns_block[icb];
      }
      
      for(irb=0; irb<num_block_rows; irb++) /* image block row loop */
      {
          /* Work-around for get_buf's inability to properly handle changing
             block dimensions.
          ------------------------------------------------------------------*/
          if (irb > 0)
          {
              slb_high += nl_block[irb-1] - nl_block[irb];
              slb_low += nl_block[irb-1] - nl_block[irb];
              slb_out += nl_block[irb-1] - nl_block[irb];
          }
          
          /* Add boundary region to image block size.
             ----------------------------------------*/
          nlbuf = nl_block[irb] + 2*boundary;
          nsbuf = ns_block[icb] + 2*boundary;

          /* Read high resolution image block.
           -----------------------------------*/
          status = get_buf((void *)hr_data_buf, &nlbuf, &nsbuf, &nlav, &nsav,
                           &highwind[0][SL], &highwind[0][SS],
                           &highwind[0][NL], &highwind[0][NS], &boundary,
                           &boundary, &slb_high, &ssb_high, &nextflg,
                           &highband, &fill, &highfdesc, highhos[0], &dtype);
              
          /* If the number of lines and samples in the block aren't
                 what we expected, we have failed.
          ---------------------------------------------------------*/
          if (status == E_FAIL ||
              (status == E_SUCC && (nlav != nlbuf || nsav != nsbuf)))
          {
              free(nl_block);
              free(ns_block);
              free(lr_data_buf);
              free(hr_data_buf);
              
              c_lasmsg(FALSE, "wavemerge-read",
                       "Error reading input image %s.", highhos);
              c_clean(&vblock, "wavemerge", outhos, &num_out);
          }

          /* Read low resolution image block.
           ----------------------------------*/
          status = get_buf((void *)lr_data_buf, &nlbuf, &nsbuf, &nlav, &nsav,
                           &lowwind[0][SL], &lowwind[0][SS], &lowwind[0][NL],
                           &lowwind[0][NS], &boundary, &boundary, &slb_low,
                           &ssb_low, &nextflg, &lowband, &fill, &lowfdesc,
                           lowhos[0], &dtype);

          /* If the number of lines and samples in the block aren't
             what we expected, we have failed.
          ---------------------------------------------------------*/
          if (status == E_FAIL ||
              (status == E_SUCC && (nlav != nlbuf || nsav != nsbuf)))
          {
              free(nl_block);
              free(ns_block);
              free(lr_data_buf);
              free(hr_data_buf);
              
              c_lasmsg(FALSE, "wavemerge-read",
                       "Error reading input image %s.", lowhos);
              c_clean(&vblock, "wavemerge", outhos, &num_out);
          }

          /* Merge the high and low resolution image block.
           ------------------------------------------------*/
          if (merge_image_block(lr_data_buf, hr_data_buf,
                                (int)nlbuf, (int)nsbuf, nlevels) != E_SUCC)
          {
              /* Failed to merge image blocks. */
              free(nl_block);
              free(ns_block);
              free(lr_data_buf);
              free(hr_data_buf);
              
              c_lasmsg(FALSE, "wavemerge-merge",
                       "Error merging image blocks.");
              c_clean(&vblock, "wavemerge", outhos, &num_out);
          }              

          /* Remove boundary region from merged band.
           ------------------------------------------*/
          index = 0;
          index1 = boundary*nsbuf;
          for (iline=0; iline<nl_block[irb]; iline++)
          {
              index1 += boundary;
              for (isamp=0; isamp<ns_block[icb]; isamp++)
              {
                  lr_data_buf[index] = lr_data_buf[index1];
                  index++;
                  index1++;
              }
              index1 += boundary;
          }
          
          /* Write the merged image block.
           -------------------------------*/
          status = put_buf((void *)lr_data_buf, &nl_block[irb],
                           &ns_block[icb], &nl_block[irb], &ns_block[icb],
                           &slb_out, &ssb_out, &outwind[NL], &outwind[NS],
                           &nextflp, &outband, &nbands, &outfdesc, outhos[0],
                           &dtype, &odtype);
          if (status == E_FAIL)
          {
              /* Failed to write buffer to file. */
              free(nl_block);
              free(ns_block);
              free(lr_data_buf);
              free(hr_data_buf);
              
              c_lasmsg(FALSE, "wavemerge-write",
                       "Error writing output image %s.", outhos);
              c_clean(&vblock, "wavemerge", outhos, &num_out);
          }

          nextflg = NEXT;        /* move to next block before reading image */
          nextflp = NEXT;
      } /* image block row loop */
  } /* image block column loop */

  /* Free storage.
     -------------*/
  free(nl_block);
  free(ns_block);
  free(lr_data_buf);
  free(hr_data_buf);

  /* Close images.
   ---------------*/
  c_eclose(&lowfdesc);
  c_eclose(&highfdesc);
  if (c_eclose(&outfdesc) != E_SUCC)
  {
      /* Failed to close output file properly. */
      c_lasmsg(FALSE, "wavemerge-close", "Error closing output image %s.",
               outhos);
      c_clean(&vblock, "wavemerge", outhos, &num_out);
  }

  /* Update the DDR.
     ---------------*/
  highbands[0][0] = highband;
  if (c_upddr(&vblock, highhos, outhos[0], &num_out, highwind, highbands,
              &nbands, &line_inc, &samp_inc, &nbands, &upddr_flag) != E_SUCC)
  {
      c_lasmsg(FALSE, "wavemerge-upddr",
               "Error updating the DDR for %s.", outhos);
      c_clean(&vblock, "wavemerge", outhos, &num_out);
  }

/* Update min/max info in the DDR.
     -------------------------------*/
  if (c_svmnmx(outhos[0], (long *)&nbands, minpix, maxpix, minmax) != E_SUCC)
      c_lasmsg(FALSE,"wavemerge-minmax",
               "Error updating the min/max DDR for %s.", outhos);

  /* Write history information.
  -----------------------------*/
  strcpy(inhos[0], lowhos[0]);
  strcpy(inhos[1], highhos[0]);
  if (c_hstout(&vblock, outhos[0], inhos, &num_in, " ") != E_SUCC)
      c_lasmsg(FALSE,"wavemerge-hist","Error writing the history for %s.",
               outhos);

  /* Write completion message.
  ----------------------------*/
  c_complt("WAVEMERGE", outtae, &num_out);

  return SUCCESS;
}
