/****************************************************************************
NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the WINDOP function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      April 1989       converted to LAS 5.0

S.NELSON	March 1992	 added masked value

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   The global parameters are first received.
   Next the input and output images are gotten.
   Various validity checks are then made before returning.
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "las.h"	
#include "util.h"
#include "windop.h"

void getpar
(
  struct PARBLK *vblock,         /* I: Pointer to TAE block */
  char taein[][CMLEN],           /* O: TAE filenames of input images */
  char hosin[][CMLEN],           /* O: Host filenames of input images */
  char *taeout,                  /* O: TAE filenmae of output image */
  char *hosout,                  /* O: Host filename of output image */
  long *nmrimg,                  /* O: Number of input images */
  long (*bands)[MAXBND+1],       /* O: Band specification */
  long *nbands,                  /* O: Number of bands per input image */
  long (*window)[4],             /* O: Window specification */
  long *msgtim,                  /* O: Time between display messages */
  long *mnmx,                    /* O: Min/Max value */
  long *totbnd,                  /* O: Total number of bands */
  long *kernel,                  /* O: Kernel dimensions (nl,ns) */
  long *skippix,                 /* O: Number of pixels to skip */
  char *typeopt,                 /* O: Type of averaging */
  long *odtype,                  /* O: Output data types */
  float maskval[][2],            /* O: Mask values */
  long *masknum,                 /* O: Number of masked values */
  float *pixval                  /* O: Pixel value to replace mask values */
)
{
struct  VARIABLE  *val;	      	 /* pointer to TAE variable within v-block  */

long     element = 0;		 /* parameter element number		    */
long     fatal = -1;		 /* fatal error flag			    */
long     i;			 /* counter-no. of input images	 	    */
long     imglim;		 /* max number of images to be processed    */
long     nonfatal = 1;		 /* value of one			    */
long     qflag[MAXIMG][2];  	 /* flag indicating if subwindows specified */
long     same;		    	 /* flag indicating if images have same wind*/
long     status;	   	 /* function return status code		    */
long     xdir;			 /* number of pixels in x direction         */
long     ydir;			 /* number of pixels in y direction         */
long     xxdir;			 /* number of pixels in x direction         */
long     yydir;			 /* number of pixels in y direction         */

char	dtype[5];		 /* data type of previous input image       */
char	mask[10][50];            /* mask values                             */
char    minmax[3];		 /* min max flag buffer		            */
char    *ptr;                    /* temporary storage space for pointer     */
char    *ptr2;                   /* temporary storage space for pointer     */
char    tempstr[80];             /* temporary storage space for string      */

val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));
if (strcmp(minmax,"YES") == 0)
   *mnmx = TRUE;
else
   *mnmx = FALSE;

/* Build the string descriptor for TAE variable "IN".
   Call getin to parse the input image specification.
   Getin returns the TAE image name(s), host image name(s),
   bands to be processed, window specification, and number
   of images parsed.
----------------------------------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","windop-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","windop-window",
		&nonfatal);
   c_errmsg("Fatal error encountered","windop-fatal",&fatal);
   }
/* Retrieve the TAE output image name from the parameter
   block and store it in "taeout". 
   Create a host file name for the output image and store 
   it in "hosout".
----------------------------------------------------------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","windop-fatal",&fatal);
/*
   Retrieve the kernel dimensions
---------------------------------------------------------------------------*/
val = p_find(vblock,"kerndim");
xdir = IVAL(*val,0);
ydir = IVAL(*val,1);
kernel[0] = xdir;
kernel[1] = ydir;
if (xdir > window[0][2])
   {
   c_errmsg("KERNDIM larger than input image", "windop-kernel",&fatal);
   c_errmsg("Fatal error encountered","windop-fatal",&fatal);
   }

if (ydir > window[0][3])
   {
   c_errmsg("KERNDIM larger than input image", "windop-kernel",&fatal);
   c_errmsg("Fatal error encountered","windop-fatal",&fatal);
   }

/*
   Retrieve the number of pixels to skip
---------------------------------------------------------------------------*/
val = p_find(vblock,"skippix");
xxdir = IVAL(*val,0);
yydir = IVAL(*val,1);
skippix[0] = xxdir;
skippix[1] = yydir;
/*
if (xxdir < 0)
   c_errmsg("Incorrect number of lines to skip",
		"windop-lines",&fatal);
if (yydir < 0)
   c_errmsg("Incorrect number of samples to skip",
		"windop-samples",&fatal);
*/

if (xxdir < xdir)
   c_errmsg("KERNDIM values are greater than the SKIPPIX values",
		"windop-values",&fatal);
if (yydir < ydir)
   c_errmsg("KERNDIM values are greater than the SKIPPIX values",
		"windop-values",&fatal);
/*
  Retrieve the type of averaging to perform
--------------------------------------------------------------------------*/
val = p_find(vblock,"typeopt");
strcpy(typeopt,SVAL(*val,0));
/*
  Retrieve the output data type
--------------------------------------------------------------------------*/
val = p_find(vblock,"odtype");
strcpy(dtype,SVAL(*val,0));
if (strcmp(dtype,"SAME") == 0)
	*odtype = 0;
else if (strcmp(dtype,"BYTE") == 0)
        *odtype = EBYTE;
else if (strcmp(dtype,"I*2") == 0)
        *odtype = EWORD;
else if (strcmp(dtype,"I*4") == 0)
        *odtype = ELONG;
else if (strcmp(dtype,"R*4") == 0)
        *odtype = EREAL;
/*
  Retrieve the mask values
--------------------------------------------------------------------------*/
val = p_find(vblock,"MASKVAL");
*masknum=val->v_count;
for (i = 0; i < *masknum; i++)
   {
   strcpy(mask[i],SVAL(*val,i));

   /* Locate spaces and delete
     ------------------------*/
   ptr = mask[i];
   ptr2 = mask[i];
   while (*ptr2 != '\0')
      {
      if (!isspace(*ptr2))
         *(ptr++) = *(ptr2++);
      else
         ptr2++;
      }
   *ptr = '\0';

   /* Locate the dash and divide mask values accordingly
    ---------------------------------------------------*/
   ptr = (char *)strchr(mask[i],'-');
   if (ptr == mask[i])
      {
      sscanf(mask[i],"-%s",tempstr);
      ptr = (char *)strchr(tempstr,'-');
      }
   if (ptr == NULL)
      {
      sscanf(mask[i],"%f",&maskval[i][0]);
      maskval[i][1] = maskval[i][0];
      }
   else
      {
      sscanf(mask[i],"%f-%f",&maskval[i][0],&maskval[i][1]);
      }

   /* Test to see that range mask values are in ascending order
     ---------------------------------------------------------*/
   if (maskval[i][0] > maskval[i][1])
      {
      c_errmsg("Error in mask value range encountered","windop-mask",
								&nonfatal);
      c_errmsg("Fatal error encountered","windop-fatal",&fatal);
      }
   }
   
   /*
Retrieve the pixel value
--------------------------------------------------------------------------*/
val = p_find(vblock,"PIXVAL");
*pixval = RVAL(*val,0);
}
