/*******************************************************************************
NAME        	    	           APPLY_XCOR	

PURPOSE     APPLY_XCOR applies systematic corrections and resamples the data
            from the input MSS image.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Jensen         5/92
D. Eichacker	  5/94		Added opt[].

ALGORITHM:
   Systematic corrections are applied to the image data using the
method described in the user's guide.  The data is then resampled 
along-line using the specified algorithm and the result written to the 
output image.

ddr file is updated for the output image.

ALGORITHM REFERENCES 		
1.  This function was derived from the LAS CORRECT function, apply_cor.c 
    routine.
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "xcorrect.h"
#include "protos.h"


void apply_xcor
(
    struct PARBLK *vblock,  /* TAE parameter block			  */
    double mnlinlen[],      /* avg raw line length                        */
    long nomlen,            /* nominal line length     		   */
    long msgtim, 	     /* elapsed time between display messages      */
    char minmax[],          /* min max flag				   */
    long bands[][MAXBND+1], /* band specification			   */
    long btot,	             /* total number of output image bands         */
    char method[],          /* resampling method		           */
    char hosin[][CMLEN],	/* pointer to input host file descriptor	*/
    char hosout[][CMLEN],   /* pointer to output host file descriptor	*/
    long bckgnd,            /* background fill value 		  	   */
    float alpha,		     /* tension parameter for parametric cubic
			        convolution resampling 	        */
    long maxlim,            /* maximum interpolation value     	   */
    long minlim,            /* minimum interpolation value		   */
    char *tabhos    	/* pointer to table host file descriptor	*/
)
{
double   inpix;              /* pixel location in input image   	   */
double   linadj;	     /* term used in calculating "inpix"	   */
double   rsamp;              /* resampled pixel value			   */

float    table [TBLSIZ];     /* weight table used for resampling           */

static	long	opt[3]={0,0,0};	/* optional parameter array for EOPENS	   */
long	  access;	     /* image I/O access type                      */
long      band;	             /* loop index counting the bands              */
long   	  bufsiz;            /* size of "thebuf"			   */
long      konst;	     /* constant term used in resampling	   */
long      count;	     /* loop index			           */
long      det = 0;           /* current detector number and initialize to 0*/
long      dtype;	     /* data type				   */
long      flag = ALLCAT;     /* process all bands at once 		   */
long      i;		     /* loop index			           */
long      line;	             /* loop index counting image lines            */
long      lowlim;            /* lower limit			  	   */
long	  nl;		     /* number of image lines                      */
long	  ns;		     /* number of image samples                    */
long      nonfatal = 1;      /* nonfatal error flag		           */
long      fatal = -1;        /* fatal error flag		           */
long      odtype;	     /* output image data type		           */
long	  one = 1;	     /* value of one                               */
long      outsamp;           /* pixel location in output image		   */
long      pixloc;            /* pixel location of beginning of kernel	   */   
long      samp;	             /* loop index counting samples per line       */
long	  status;	     /* function return status code                */
long      tabsiz;	     /* size of resampling weight table		   */
long      tblloc;            /* location in weight table		   */ 
long      zero = 0;          /* value of zero				   */

double	line_inc = 1.0      /* dummy values to pass by refrence	   */,
	samp_inc = 1.0;

long	window[1][4],
	nbands[1][2],
	cflag = NOCOMB;
 
float   sdelay[6];

double minval [MAXBND];       /* array of minimum pixel values		   */
double maxval [MAXBND];       /* array of maximum pixel values		   */

unsigned char *thebuf;	      /* image input buffer			   */


/* Initialize sdelay for each of six detectors                                */

sdelay[0]= 0.2;
sdelay[1]= 0.12;
sdelay[2]= 0.04;
sdelay[3]= -0.04;
sdelay[4]= -0.12;
sdelay[5]= -0.2;
  
/* If the user wants the min max pixel values computed for each band,         */
/* initialize the array of min max values				      */
if (strcmp(minmax,"YES") == 0)
	{
	dtype = EBYTE;
	c_inmnmx(&dtype,&btot,minval,maxval);
	}

/* Make the resampling weight table */
make_table(table,&tabsiz,method,alpha,tabhos);

/* Open the input image for sequential read access */
access = IREAD;
nl = zero;
ns = zero;
odtype = zero;
status = c_eopens(&fdesc[0],hosin[0],bands[0],&one,&one,&nl,&ns,
     		    offset1,&access,&odtype,&zero,opt);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","xcorrect-fatal",&fatal);

if (odtype != EBYTE)
   c_errmsg("Input image is not BYTE","xcorrect-notbyte",
		&fatal);

if (nl != MAXLINE)
   c_errmsg("Input image is not a full-sized MSS scene","xcorrect-sizerr",
		&fatal);

/* Open the output image for sequential write access */
access = IWRITE;
status = c_eopens(&fdesc[1],hosout[0],&btot,&one,&one,&nl,&ns,offset2,
         &access,&odtype,&zero,opt);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","xcorrect-fatal",&fatal);

/* Allocate the I/O buffer  */
if ((thebuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   status = c_egclse(&gdesc);
   c_clean(vblock,"xcorrect",hosout,&one);
   }
    

/* Group the input image and the output image   */
fdesc[2] = 0;
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   status = c_egclse(&gdesc);
   c_clean(vblock,"xcorrect",hosout,&one);
   }

/* Calculate the constants in the correction formula and resampling */
konst = 1 - (tabsiz / 2);
lowlim = (tabsiz / 2) - 1;

/* Initialize and start the display message utility */
c_inittm("XCORRECT","Processed","line","bands"," ",&nl,&btot,&msgtim,&flag);
curent = 1;
c_startm();

/* Process all the image bands */
for (line = 0; line < MAXLINE; line++)
    {
    /* Read in the input image line */
    if (c_estep(&gdesc) != E_SUCC)
        {
        status = c_egclse(&gdesc);
	c_clean(vblock,"xcorrect",hosout,&one);
	}

    linadj = mnlinlen[line] / (double)nomlen;

    outsamp = 0;
    for (samp = 0; samp < ns; samp++)
        {
        outsamp++;

        for (band = 0; band < btot; band++)
	    {
            inpix = linadj * outsamp + sdelay[det];

            /* Adjust inpix for the zero relative array */
            inpix = inpix - 1;

            /* Resampling */
            if ((inpix < lowlim) || (inpix > mnlinlen[line]))
		{
                *(thebuf + *(offset2 + band) + samp) = (unsigned char) bckgnd;
		}
 	     else 
		{
		pixloc = (long) inpix +  konst;
                tblloc = tabsiz * (long)(FACT3 * (FACT2 + inpix -
  			 (long) inpix));

     	        for (count = 0, rsamp = 0; count < tabsiz; count++)
    		   rsamp += *(table + tblloc + count) *
    		   	     *(thebuf + *(offset1 + band) + (pixloc + count));

 		if (rsamp < minlim) 
  		   *(thebuf + *(offset2 + band) + samp ) = (unsigned char) minlim;
	        else if (rsamp > maxlim)
 		   *(thebuf + *(offset2 + band) + samp ) = (unsigned char) maxlim;
 		else
	          *(thebuf + *(offset2 + band) + samp ) = (unsigned char) rsamp;
  	      	}
	    }
        }
     
    if (strcmp(minmax,"YES") == 0)
	{
	for (i = 0; i < btot; i++)
	     status = c_pxmnmx(thebuf + (*(offset2 + i)),&ns,&minval[i],
     		       &maxval[i], &odtype);
        }

    if (det == 5)
        det = 0;
    else
        det++;

    curent ++;
    }

/* Final c_estep */
   if (c_estep(&gdesc) != E_SUCC)
      {
        status = c_egclse(&gdesc);
	c_clean(vblock,"xcorrect",hosout,&one);
      }

/* Disable the display message utility */
c_stoptm();

/* Close the images in the group */
status = c_egclse(&gdesc);
if (status != E_SUCC) 
   {
   c_clean(vblock,"xcorrect",hosout,&one);
   }

/*
  Update the DDR information based on the input file
*/
window[0][0] = one;
window[0][1] = one;
window[0][2] = nl;
window[0][3] = ns;
nbands[0][0] = btot;
status = c_upddr(vblock,hosin,hosout[0],&one,window,bands,nbands[0],
	&line_inc,&samp_inc,&btot,&cflag);
if (status != E_SUCC)
   {
   c_errmsg("Nonfatal error encountered", "xcorrect-warn",&nonfatal);
   }

/* update the ddr with minmax, if appropriate */
status = c_svmnmx(hosout[0],&btot,minval,maxval,minmax);
if (status != E_SUCC)
   {
   c_errmsg("Nonfatal error encountered", "xcorrect-warn",&nonfatal);
   }
/*
  Free the buffer
*/
free(thebuf);

return;
}
