/*******************************************************************************
NAME        	    	            GETPAR	

PURPOSE    GETPAR reads and verifies the parameters.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Jensen         5/92

ALGORITHM:
   The values of the TAE input parameters and the global variables
are retrieved.  The existence of the associated file containing the
ancillary information needed to apply systematic corrections is checked.

ALGORITHM REFERENCES:
1.  This function was derived from the LAS CORRECT function, getpar.c
    routine.
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "xcorrect.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,       /* pointer to TAE parameter block            */
    char taein[][CMLEN],        /* pointer to input TAE file descriptor      */
    char hosin[][CMLEN],	/* pointer to input host file descriptor     */
    char taeout[][CMLEN],	/* pointer to output TAE file descriptor     */
    char hosout[][CMLEN],   /* pointer to output host file descriptor    */
    long bands[][MAXBND+1],	/* band specification 			     */
    long *btot,		        /* total number of output image bands	     */
    long *msgtim,		/* elapsed time between display messages     */
    char *minmax,		/* min max flag buffer                       */
    char *method,		/* resampling method		             */
    long *bckgnd,                /* background fill value	             */
    float *alpha,               /* cubic convolution alpha parameter         */
    long *maxlim,               /* maximum interpolation value		     */
    long *minlim,                /* minimum interpolation value		     */
    char *tabhos, 	        /* pointer to user-defined table file desc.  */
    char *xcorhos               /* pointer to corrections file descriptor    */
)
   
{

struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

long    element = 0;		/* element number			     */
long    fatal = -1;	        /* fatal error flag		   	     */
long	imglim;		        /* maximum number of images to be processed  */
long    mode;			/* mode for c_lsstat call		     */
long    nbands;  		/* number of bands -- needed just for c_getin*/
long    nonfatal = 1;           /* nonfatal error flag			     */
long	nmrimg;		        /* number of input images		     */
long	qflag[IMGMAX][2];	/* flag indicating if subwindows specified   */
long	same;			/* flag indicating if images have same window*/
long	status;			/* function return status code               */
long	window[IMGMAX][4];      /* window  specification	             */


/* Get the global variables */
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

/* Get the output image name */
status = c_getout(vblock,"OUT",taeout,hosout);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","xcorrect-fatal",&fatal);

/* Parse the input image specification */
/* Note:  XCORRECT is designed to process one entire MSS image at a time ONLY */
imglim = IMGMAX;
status = c_getin(vblock,"IN",&element,taein,hosin,&nmrimg,window,bands,
	         &nbands,btot,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","xcorrect-fatal",&fatal);

/* Get the file name for the image's associated correction file */
c_lsmknm(hosin[0],"cctx_rll",xcorhos);
mode = 0;
status = c_lsstat(xcorhos,&mode);
if (status != E_SUCC)
   c_errmsg("Input image's correction file does not exist",
	    "xcorrect-nocorr",&fatal);

/* Get the minimum and maximum interpolation limits */
/* If they were null, set the minmax range to the range for byte data type */
val = p_find(vblock,"INTLIM");
if ((*val).v_count == 0)
	{
	*minlim = 0;
	*maxlim = 255;
	}
else
	{
	*minlim = IVAL(*val,0);
	*maxlim = IVAL(*val,1);
	}

/* Get the background fill value */
val = p_find(vblock, "BACKGRND");
*bckgnd = IVAL(*val,0);

/* Get the subcommand */
val = p_find(vblock,"_SUBCMD");
strcpy(method,SVAL(*val,0));

/* If method is "CUBIC", get alpha parameter*/
if (strcmp (method, "CUBIC") == 0)
	{
	val = p_find(vblock, "PCCALPHA");
	*alpha = RVAL(*val,0);
	}

/* If method is "TABLE", get the weight table file name */
if (strcmp (method, "TABLE") == 0)
	{
	mode = IREAD;
	status = c_getnam(vblock,"INRWT",&element,tabhos,"rwt",&mode);
	if (status != E_SUCC)
	   {
	   c_errmsg("User-defined weight table file does not exist",
		    "xcorrect-notable",&nonfatal);
           c_errmsg("Fatal error encountered","xcorrect-fatal",&fatal);
	   }
	}
	
return;
}
