/*******************************************************************************
NAME        	    	           MAKE_TABLE	

PURPOSE     MAKE_TABLE creates the weight table used in resampling.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Jensen         5/92

ALGORITHM:
If the CUBIC subcommand is specified, the appropriate weight table
for cubic convolution resampling is built.  
If the -TABLE subcommand is specified, the weight table is read
from INFILE, the disk file containing the table of user-entered
resampling weights.
If the BILINEAR subcommand is specified, the appropriate weight table
for bilinear resampling is built.  
 
ALGORITHM REFERENCES:
1.  This function was derived from the LAS CORRECT function, make_table.c
    routine.
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "xcorrect.h"
#include "protos.h"


void make_table
(
    float table[],		/* resampling weight table	        */
    long *tabsiz,		        /* size of resampling table	        */
    char method[],		/* method of resampling	                */
    float alpha,			/* tension parameter for parametric cubic
				   convolution resampling 	        */
    char *tabhos  		/* pointer to table file descriptor     */
)

{
struct RWTAB	rwtable;

float  delta;			/* weight table positional value	*/
float  *index; 		        /* pointer to weight table              */
float  tmpdel;			/* temporary values of delta		*/

long    i, j;			/* loop index  			        */
long    status;			/* Status variable			*/
long    fatal = -1;             /* fatal error flag		        */

/* Set "index" to the starting address of the table */
index = table;

/* For the -CUBIC subcommand, build the appropriate weight table for    */
/* cubic convolution resampling						*/

if (strcmp (method,"CUBIC") == 0)
 	{
	for (i = 0; i < MATSIZ; i++)
         	{
		delta = i * FACT1;

 		tmpdel = 1 + delta;
	 	*index++ = (-alpha) * (4 - 8 * tmpdel + 5 *
 			   SQUARE(tmpdel) - CUBE(tmpdel));

	        *index++ = 1.0 - (alpha + 3.0) * SQUARE(delta) +
			   (alpha + 2.0) * CUBE(delta);

  		tmpdel = 1 - delta;
	        *index++ = 1.0 - (alpha + 3.0) * SQUARE(tmpdel) + 
 			   (alpha + 2.0) * CUBE(tmpdel);

		tmpdel = 2 - delta;
	 	*index++ = (-alpha) * (4 - 8 * tmpdel + 5 * 
			   SQUARE(tmpdel) - CUBE(tmpdel));
		}
    	*tabsiz = 4;
        return;
        }

/* For the -TABLE subcommand, read in the user-defined weight table */

if (strcmp (method,"TABLE") == 0)
	{
	/* Get the resampling weight table */
	status = c_getrwt(tabhos,&rwtable);
	if (status != E_SUCC)
   	   c_errmsg("Fatal error encountered","xcorrect-getrwt",&fatal);

	/*
	  Get the size of the horizontal table size
	*/
	*tabsiz = rwtable.dim[0];

	/*
	  Copy the horizontal table to the weight table
	*/
	for (i = 0; i < MATSIZ; i++)
	    {
	    for (j = 0; j < *tabsiz; j++)
		*index++ = rwtable.table1[i][j];
	    }

	return;
	}

/* For the -BILINEAR subcommand, build the appropriate weight table for    */
/* bilinear resampling         					           */

if (strcmp (method,"BILINEAR") == 0)
    	{
        for (i = 0; i < MATSIZ; i++)
  		{
		delta = i * FACT1;
		*index++ = 1 - delta;
		*index++ = delta;
		}
	*tabsiz = 2;
        return;
 	}
}
