/******************************************************************************
NAME        	    	            XCORRECT	

PURPOSE    XCORRECT performs along-line systematic corrections on a full-sized,
	   non-geometrically corrected Landsat MSS CCT-X scene.  The image data
	   is corrected for the following systematic errors:
                1)  Line length adjustment
	        2)  Detector-to-detector sampling delay
           
  	   After the systematic corrections are applied, the data is 
           resampled along-line using the specified algorithm.
            
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Jensen         5/92		PRF# 3391 New development for MSS CCT-X scenes

ALGORITHM REFERENCES 		
1.  This function was derived from the LAS CORRECT function.

*******************************************************************************/

#include <desc.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "xcorrect.h"
#include "protos.h"


/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC * fdesc[IMGMAX+2];  /* image file descriptors used for image I/O */
struct GDESC * gdesc;		     /* group descriptor used for image I/O       */
long offset1[IMGMAX*MAXBND+1];  /* offsets into "thebuf" for input bands     */
long offset2[MAXBND+1];         /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

int main
(void)

{

struct PARBLK vblock;        /* TAE parameter block		           */

float    alpha;              /* cubic convolution alpha parameter          */
 
double   mnlinlen [MAXLINE];   /* mean actual line length                  */

long	bands[IMGMAX][MAXBND+1];/* band specification 			   */  
long    bckgnd;              /* background fill value			   */
long    btot;	  	     /* total number of output image bands	   */
long    maxlim;              /* maximum interpolation value     	   */
long    minlim;              /* minimum interpolation value		   */
long	msgtim;		     /* elapsed time between display messages      */
long    nomlen;              /* nominal line length			   */
long    nonfatal = 1;        /* nonfatal error flag			   */
long	one = 1;	     /* value of one			           */
long	status;		     /* function return status code                */
long	std_out;	     /* LUN for standard output			   */

char    xcorhos[CMLEN];       /* buffer storage for "xcorhos"                */
char	hosin[IMGMAX][CMLEN];/* buffer storage for "hosin"                 */
char	hosout[1][CMLEN];       /* buffer storage for "hosout"                */
char    method [9];	     /* resampling method		           */
char	minmax[4];	     /* min max flag                               */
char    tabhos[CMLEN];       /* buffer storage for "tabhos"                */
char    taein[IMGMAX][CMLEN];/* buffer storage for "taein"		   */
char	taeout[1][CMLEN];       /* buffer storage for "taeout"                */

/* Initialization  */
c_inlas(&vblock,&std_out);

/* Get user-specified parameters and global variable values from TAE */
getpar(&vblock,taein,hosin,taeout,hosout,bands,&btot,&msgtim,minmax,
       method,&bckgnd,&alpha,&maxlim,&minlim,tabhos,xcorhos);

/* Get the ancillary information stored in the associated correction file */
get_info(mnlinlen,&nomlen,xcorhos);

/* Process the image data */
apply_xcor (&vblock,mnlinlen,nomlen,msgtim,minmax,bands,btot,method,hosin,
           hosout,bckgnd,alpha,maxlim,minlim,tabhos);

/* Create the history file for the output image      */
status = c_hstout(&vblock,hosout[0],hosin,&one," ");
if (status != E_SUCC) 
   c_errmsg("Nonfatal error encountered--Image data saved","xcorrect-warn"
		,&nonfatal);

/* Write a completion message to the user            */
c_complt("XCORRECT",taeout,&one);

return EXIT_SUCCESS;

}		
