/***************************************************************************
  Purpose: This module provides the routines used to manage the cursor 
           displayed in each window.

  Note: it is assumed that only one display is managed.  This limitation
        can be removed by allocating the wait cursor for each window in the
        loop.
****************************************************************************/
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include "manage_cursor.h"


#define MAX_MANAGED 20
/* maximum number of windows to manage windows for */

static int wait_cursor_shown = 0;
/* flag to indicate the wait cursor is shown */

static int num_managed = 0;
/* number of windows the cursor is being managed for */

struct window_cursors
{
    Display *display;
    Window window;     
    Cursor cursor;  /* current cursor */
};
/* data to track a single window's cursor */

static struct window_cursors managed_list[MAX_MANAGED];
/* list of windows for which the cursors are managed */

void set_window_cursor
(
    Display *display,   /* I: display of window */
    Window window,      /* I: window to set the cursor */
    Cursor cursor       /* I: cursor to set in the window */
)
{
    int index;
    int new_window = 0;
    
    /* free any cursor that was previously set for this window */
    remove_window_cursor(display,window);

    /* look for the window in the list */
    for (index = 0; index < num_managed; index++)
    {
        if ((display == managed_list[index].display) && 
            (window == managed_list[index].window))
        {
            /* found the current window, so stop searching */
            break;
        }
    }

    /* if the window wasn't found, add it to the list */
    if (index >= num_managed)
    {
        new_window = 1;
        
        if (num_managed >= MAX_MANAGED)
        {
            /* list is full, issue warning an return.  The busy cursor will
               not be displayed in all windows.  If this warning is received,
               the list size probably needs to be increased.*/
            fprintf(stderr,"Warning: too many windows in managed cursor list. "
                    "The busy cursor may not display properly in some "
                    "windows.\n");
            return;
        }
     
        managed_list[index].display = display;
        managed_list[index].window = window;
        num_managed++;
    }
   
    /* set the cursor for the window */
    managed_list[index].cursor = cursor;

    /* if the wait cursor isn't already being displayed, set the new cursor
       in the window */
    if (!wait_cursor_shown)
    {
        XDefineCursor(display,window,cursor);
    }
    else if (new_window)
    {
        Cursor wait_cursor;
        
        /* new window and already waiting, so show wait cursor in the new 
           window too */
        wait_cursor = XCreateFontCursor(display,XC_watch);
        XDefineCursor(display,window,wait_cursor);
        XFreeCursor(display,wait_cursor);
    }
}

void remove_window_cursor
(
    Display *display,   /* I: display of window */
    Window window       /* I: window to remove custom cursor from */
)
{
    int index;
    int found = -1;
    
    /* look for the window in the list */
    for (index = 0; index < num_managed; index++)
    {
        if ((display == managed_list[index].display) && 
            (window == managed_list[index].window))
        {
            /* found the current window, so free the current cursor, undefine
               the cursor for the window, and remove the window from the list */
            found = index;
            break;
        }
    }
    if (found >= 0)
    {
        /* the window requested was found, make sure this is the only window
           using this cursor */
        for (index = 0; index < num_managed; index++)
        {
            if (index == found)
                continue;
            if (managed_list[found].cursor == managed_list[index].cursor)
                break;
        }
        if (index >= num_managed)
        {
            /* No other windows use this cursor, so free it */
            XFreeCursor(display, managed_list[found].cursor);
        }
    
        /* remove the found entry from the list */
        managed_list[found] = managed_list[num_managed-1];
        num_managed--;
    }
    
    /* no matter what, undefine the cursor for the window so it returns
       to the default cursor */
    XUndefineCursor(display,window);
}

void show_wait_cursor(void)
{
    int index;
    Cursor cursor;
    Display *display;

    /* set the waiting flag */
    wait_cursor_shown++;
    if (wait_cursor_shown > 1)
    {
        /* already showing the wait cursor, so just return */
        return;
    }

    /* if any windows are managed, update them with the wait cursor */
    if (num_managed > 0)
    {
        /* assume all displays are the same as the first one */
        display = managed_list[0].display;
        
        /* get the wait cursor */
        cursor = XCreateFontCursor(display,XC_watch);
        
        /* set the cursor for each window in the list */
        for (index = 0; index < num_managed; index++)
        {
            XDefineCursor(managed_list[index].display, 
                          managed_list[index].window, cursor);
        }
        
        /* free the wait cursor */
        XFreeCursor(display,cursor);

        /* force an update of the display */
        XFlush(display);
    }
}

void unshow_wait_cursor(void)
{
    int index;
    Display *display;

    /* set the waiting flag */
    wait_cursor_shown--;
    if (wait_cursor_shown)
    {
        /* wait cursor was recursively shown, so don't undisplay it yet */
        return;
    }

    /* if any windows are managed, show their normal cursor */
    if (num_managed > 0)
    {
        /* assume all displays are the same as the first one */
        display = managed_list[0].display;
        
        /* set the cursor for each window in the list */
        for (index = 0; index < num_managed; index++)
        {
            XDefineCursor(managed_list[index].display, 
                          managed_list[index].window, 
                          managed_list[index].cursor);
        }
        
        /* force an update of the display */
        XFlush(display);
    }
}
