/*
 *  This is the main source file for the X Image Display (xid) application.
 *
 *  By D. Akkerman/L. Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *  
 *  Modified by C. Olsen, USGS/EROS Data Center, 4/99.
 *    - Added shells for GOF name and label popups.
 *
 *  Modified by A. Shimitz, USGS/EROS Data Center, 7/00.
 *    - Added functionality to specify an image to load when XID initially
 *      is started.
 *
 *  Modified by A. Shimitz, USGS/EROS Data Center, 8/00.
 *    _ Added init_map_histo() routine to support the functionality of the
 *      Mapped button on the histogram panel. (CR #3892)
 */
#define _xid_h          /* Assign global structures.                         */

#include <X11/cursorfont.h>
#include <X11/IntrinsicP.h>
#include <X11/CoreP.h>
#include "pixman.h" /* c_pxfill definition */
#include "xid.h"
#include "manage_cursor.h"
#include "xid.xbm"
#include "xid_graphics.h"
#include "xid_func.h"
#include "xid_create.h"
#include "main.h"
#include "xid_widgets/Context.h"
#include <signal.h>
 
/* local function prototypes */
void xid_initialize
(void);

int parseargs
(
   int    argc,               /* Number of command line arguments. (I)       */
   char   *argv[],            /* Command line arguements. (I)                */
   char   *geometry_menu,     /* Geometry specification for main menu. (O)   */
   char   *geometry_win       /* Geometry specification for main window. (O) */
);
/* end prototypes */

/* The following variables have been declared global for image I/O routines. */
struct FDESC *fdesc[2]; /* Image file descriptors used for image I/O.        */
struct GDESC *gdesc;    /* Group descriptor used for image I/O.              */
long ioffset[3];        /* Offsets into thebuf for output bands.             */

/* Resource converter variables for the XID_StringToPixel resource converter
routine.  */

XtConvertArgRec color_ConvertArgs[] = {
    {XtWidgetBaseOffset, (XtPointer)XtOffset(Widget, core.screen),
        sizeof(Screen *)},
    {XtWidgetBaseOffset, (XtPointer)XtOffset(Widget, core.colormap),
        sizeof(Colormap)}
};

/* Declarations for defining XID specific resources.  */
typedef struct {
   Boolean     colors_msg;
}app_resourceRec, *app_resourceRecPtr;

app_resourceRec       app_resources;     /* Application resources.           */

static  XtResource resources[] = {
    {"colors_message","Colors_Message",XtRBoolean, sizeof(Boolean),
     XtOffset(app_resourceRecPtr, colors_msg), XtRImmediate, (caddr_t)True},
};
char *wk_colors[] = {"black","white","red","green","blue","yellow","magenta",
                     "cyan"};            /* Working color strings.           */

XtAppContext context;                    /* Xt application context.          */

/*****************************************************************************
 *                                                                           *
 *                      X Image Display main application.                    *
 *                                                                           *
 *****************************************************************************/

int main
(
   int argc,
   char **argv
)
{
   Arg               args[256];         /* X resource arguments.             */
   Cursor            pointer;           /* pointer cursor - need to use this
                                           so show_wait_cursor and
                                           unshow_wait_cursor can be used    */
   Status            status;            /* Return status variable.           */
   XVisualInfo       visual_template;   /* Visual template to match against
                                           server.                           */
   unsigned long     pixelval[256];     /* Array for pixel values.           */
   unsigned long     visual_mask=0;     /* Specifies which elements to match
                                           using the visual template.        */
   XColor            col_cell;          /* Working colorcell.                */
   XrmOptionDescRec  options;           /* Command line options table.       */
   char              errmsg[80];        /* Working error message string.     */
   char              geometry_menu[10]; /* Main menu geometry specification. */
   char              geometry_win[10];  /* Main window geometry specification*/
   int               number_visuals;    /* Number of server visuals that
                                           match request.                    */
   int               exitflg;           /* Flag to determine if they want
                                           command line args help displayed  */
   long              argcnt;            /* Resource argument count.          */
   long              i;                 /* Working loop indices.             */
   long              offset;            /* Used in scaling the 24-bit LUT.   */
   unsigned long     plane_masks[1];    /* Working plane mask.               */
   unsigned long     wk_pixels[1];      /* Return pixel value array.         */
   unsigned long     red_mask;          /* Mask values to reference 24-bit
                                           pixels.                           */
   unsigned long     green_mask;
   unsigned long     blue_mask;
   float             slope;             /* Used in scaling the 24-bit LUT.   */

   static String     fallback_resources[] = {
      #include "XId.ad.h"
      NULL
   };                                   /* Fallback resource specifications. */
   XImage *ximage;    /* XImage pointer for use when adjusting color shifts for 
                         byte swapping */
   long *data = NULL; /* temp ptr for the ximage for the byte swapping check */

   /* Initialize globals and widget IDs */
   xid_initialize();

   /* Parse command line arguments. Don't open xid if they have specified
      the help option only. If they specified the help option along with other
      valid options, xid will still open and the help message will be printed
      to the display. */
   exitflg = parseargs(argc,argv,geometry_menu,geometry_win);
   if(exitflg)
      return 0;

   /* Initialize the X Toolkit and assign standard globals.  */
   XtToolkitInitialize();
   context = XtCreateApplicationContext();

   /* Set fallback resources.  */
   XtAppSetFallbackResources(context,fallback_resources);

   /* Open the display.  */
   xid.display = XtOpenDisplay(context,NULL,BX_APP_NAME,BX_APP_CLASS,
                               &options,0,&argc,argv);
   if (xid.display == NULL)
   {
      XtWarning("Unable to open display");
      exit(1);
   }
   xid.screen = DefaultScreen(xid.display);

   /* Assign the visual for the application; always look for a 24-bit visual
   first.  */
   visual_template.screen = xid.screen;
   visual_template.colormap_size = 256;
   visual_template.class = DirectColor;
   visual_template.depth = 24;

   visual_mask = VisualScreenMask;        /* Limit to present screen.        */
   visual_mask |= VisualDepthMask;        /* Limit to standard depth.        */
   visual_mask |= VisualClassMask;        /* Limit to class assigned.        */
   visual_mask |= VisualColormapSizeMask; /* Limit to colormap size assigned */

   /* Get the visual information structures that match the specified template*/
   number_visuals = 0;
   xid.visual_info = XGetVisualInfo(xid.display,(long)visual_mask,
                                    &visual_template,&number_visuals);
   if ((number_visuals == 0) || (xid.visual_info == (XVisualInfo *)NULL))
   {
      visual_template.class = PseudoColor;
      visual_template.depth = 8;
      number_visuals = 0;
      xid.visual_info = XGetVisualInfo(xid.display,(long)visual_mask,
                                       &visual_template,&number_visuals);
      if ((number_visuals == 0) || (xid.visual_info == (XVisualInfo *)NULL))
      {
         XtWarning("error retrieving visual information");
         exit(1);
      }
   }

   /* Create the application colormap.  */
   xid.colormap = XCreateColormap(xid.display, RootWindow(xid.display,
                                  xid.screen), xid.visual_info[0].visual,
                                  AllocNone);
   if (xid.colormap == DefaultColormap(xid.display,xid.screen))
   {
      XtWarning("error creating colormap");
      exit(1);
   }

   /* Retrieve the color database information from the X standard color
   database.  */
   load_clist();

   /* Allocate the working colorcells.  */
   if (xid.visual_info[0].visual->class == PseudoColor)
   {
      /* Allocate the colorcells.  */
      status = XAllocColorCells(xid.display,xid.colormap,True,plane_masks,
                                0,pixelval,256);
      if (status == 0)
      {
         XtWarning("unable to allocate colorcells");
         exit(1);
      }

      /* To minimize color flashing copy the first NUM_WMCOLORS out of the
      default color map.  */
      for (i = 0; i < NUM_WMCOLORS; i++)
         xid.color_table[WM_IDX + i].pixel = i;
      XQueryColors(xid.display,DefaultColormap(xid.display,xid.screen),
                   &xid.color_table[WM_IDX],NUM_WMCOLORS);
      XStoreColors(xid.display,xid.colormap, &xid.color_table[WM_IDX],
                   NUM_WMCOLORS);

      /* Assign the drawing/working colors; these are static.  */
      for (i = 0; i < NUM_CVALS; i++)
      {
         status = XParseColor(xid.display,xid.colormap,wk_colors[i],
                              &xid.color_table[RGBBW_IDX + i]);
         if (status == 0)
         {
            sprintf(errmsg,"unable to parse %s color",wk_colors[i]);
            XtWarning(errmsg);
            exit(1);
         }
         xid.color_table[RGBBW_IDX + i].pixel = pixelval[i + NUM_WMCOLORS];
         XStoreColor(xid.display,xid.colormap,&xid.color_table[RGBBW_IDX + i]);
      }
      
      /* Assign the working colorcell.  */
      xid.color_table[WORK_IDX].pixel =  
                                  xid.color_table[RGBBW_IDX + BLACK_COL].pixel;
      XQueryColor(xid.display,xid.colormap,&xid.color_table[WORK_IDX]);
      xid.color_table[WORK_IDX].pixel = pixelval[NUM_WMCOLORS + NUM_CVALS];
      xid.color_workclear = xid.color_table[WORK_IDX];
      XStoreColor(xid.display,xid.colormap,&xid.color_table[WORK_IDX]);

      /* Assign the color store colorcells.  */
      for (i = 0; i < NUM_CSTORE; i++)
         xid.color_table[CSTORE_IDX + i].pixel = 
                          pixelval[i + NUM_WMCOLORS + NUM_CVALS + NUM_WKCOLOR];

      /* Assign the remaining colorcells for image data.  */
      for (i = 0; i < NUM_IMGCOLORS; i++)
         xid.color_table[IMG_IDX + i].pixel = pixelval[i + NUM_WMCOLORS + 
                                        NUM_CVALS + NUM_WKCOLOR  + NUM_CSTORE]; 
   }
   else         /* DirectColor */
   {

      /* Assign shift values according to rgb masks returned from
      XGetVisualInfo */
      if (xid.visual_info->red_mask == 0xff)
         xid.red_shift = 0;
      else if (xid.visual_info->red_mask == 0xff0000)
         xid.red_shift = 16;
      else
         xid.red_shift = 8;

      if (xid.visual_info->green_mask == 0xff00)
         xid.green_shift = 8;
      else if (xid.visual_info->green_mask == 0xff0000)
         xid.green_shift = 16;
      else
         xid.green_shift = 0;

      if (xid.visual_info->blue_mask == 0xff0000)
         xid.blue_shift = 16;
      else if (xid.visual_info->blue_mask == 0xff)
         xid.blue_shift = 0;
      else
         xid.blue_shift = 8;

      /* Create an X Image to allow the byte order of the X server to be 
         detected (allocate data since XDestroyImage free's the image data */
      data = (long*)malloc(sizeof(long));
      if (data == NULL)
      {
          XtWarning("unable to allocate memory");
          exit(1);
      }
      /* store a value that can be used to detect byte swapping */
      *data = 1;
      ximage = XCreateImage(xid.display,xid.visual_info[0].visual,
              xid.visual_info[0].depth,ZPixmap,0,(char *)data,1,1,8,0);

      /* if the byte order of the display is different than the byte order
         in memory where the program is running, adjust the color shift 
         to accommodate the byte swapping */
      if (((ximage->byte_order == LSBFirst) && (*(char *)data == 0)) ||
          ((ximage->byte_order == MSBFirst) && (*(char *)data == 1)))
      {
          xid.red_shift = 24 - xid.red_shift;
          xid.blue_shift = 24 - xid.blue_shift;
          xid.green_shift = 24 - xid.green_shift;
      }

      /* destroy the temporary image */
      XDestroyImage(ximage);

      /* Allocate color planes.  */
      status = XAllocColorPlanes(xid.display,xid.colormap,True,wk_pixels,
                                1,8,8,8,&red_mask,&green_mask,&blue_mask);
      if (status == 0)
      {
         XtWarning("unable to allocate color planes");
         exit(1);
      }

      /* Assign the pixel values for the color cells.  */
      for (i = 0; i < 256; i++)
         xid.color_table[i].pixel = (i) + (i << 8) + (i << 16);

      /* Assign the working colors.  */
      for (i = 0; i < NUM_COLORS; i++)
      {
         status = XParseColor(xid.display,xid.colormap,wk_colors[i],
                              &xid.color_table[RGBBW_IDX + i]);
         if (status == 0)
         {
            sprintf(errmsg,"unable to parse %s color",wk_colors[i]);
            XtWarning(errmsg);
            xid.color_table[RGBBW_IDX + i].flags = DoRed | DoGreen | DoBlue;
            xid.color_table[RGBBW_IDX + i].red = 
               xid.color_table[RGBBW_IDX + i].green = 
               xid.color_table[RGBBW_IDX + i].blue = 0;
         }
      }

      /* Assign the working color cell.  */
      status = XParseColor(xid.display,xid.colormap,"black",&col_cell);
      if (status == 0 )
      {
         strcpy(errmsg,"unable to parse black color");
         XtWarning(errmsg);
         col_cell.red = col_cell.green = col_cell.blue = 0;
         col_cell.flags = DoRed | DoGreen | DoBlue;
      }

      xid.color_table[WORK_IDX].red   = xid.color_workclear.red = col_cell.red;
      xid.color_table[WORK_IDX].green = xid.color_workclear.green
                                      = col_cell.green;
      xid.color_table[WORK_IDX].blue  = xid.color_workclear.blue
                                      = col_cell.blue;
      xid.color_table[WORK_IDX].flags = xid.color_workclear.flags
                                      = col_cell.flags;

      XStoreColor(xid.display,xid.colormap,&xid.color_table[WORK_IDX]);

      /* Assign a gray scale ramp to the image data colors.  */
      slope = (256.0 / (float)NUM_IMGCOLORS);
      offset = (long)(0 - (slope * (float)(NUM_COLORS + NUM_CSTORE +
                  NUM_WMCOLORS + NUM_WKCOLOR)));
      for (i = IMG_IDX; i < (IMG_IDX + NUM_IMGCOLORS); i++)
      {
         xid.color_table[i].red = xid.color_table[i].green = 
            xid.color_table[i].blue = (unsigned short)((long)(((float)i *
               slope) + (float)offset) << 8);
         xid.color_table[i].flags = DoRed | DoGreen | DoBlue;
      }
      XStoreColors(xid.display,xid.colormap,xid.color_table,256);
   }

   /* Set the colormap and color table pointers.  */
   xid.colormap_ptr = &xid.colormap;
   xid.colortable_ptr = &xid.color_table[0];

   /* Set the String -> Pixel resource converter.  */
   XtSetTypeConverter(XtRString,XtRPixel,XID_StringToPixel,color_ConvertArgs,
                      XtNumber(color_ConvertArgs),XtCacheByDisplay,NULL);

   /* Register the converter.  */
   XmRegisterConverters();

   /* Load the font used to display general information.  */
   if ((xid.font_info = XLoadQueryFont(xid.display,INFO_FONT)) == NULL)
      XtWarning("Unable to load general font");

   /* The applicationShell is created as an unrealized parent for multiple
   topLevelShells.  The topLevelShells are created as popup children of the
   applicationShell. This is a recommendation of Paul Asente & Ralph Swick in
   _X_Window_System_Toolkit_ p. 677.  */
   argcnt = 0;
   AppShell = XtAppCreateShell(BX_APP_NAME, BX_APP_CLASS,
                               applicationShellWidgetClass, xid.display, args,
                               argcnt);

   /* Get the application resources.  */
   XtGetApplicationResources(AppShell,&app_resources,resources,
                             XtNumber(resources), NULL,0);
   if (app_resources.colors_msg)
      xid.flags |= XID_COLMSG;
   else
      xid.flags &= ~XID_COLMSG;

   /* Create the main application shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNallowShellResize, False); argcnt++;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "XID"); argcnt++;
   if (geometry_menu[0] != 0)
   {
      XtSetArg(args[argcnt], XmNgeometry, geometry_menu); argcnt++;
   }
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_MainMenu = XtCreatePopupShell("shell_mainmenu",
                                       topLevelShellWidgetClass, AppShell,
                                       args, argcnt);
   XtAddCallback(Shell_MainMenu, XmNdestroyCallback, main_destroy, 0);

   /* Create the layout panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNallowShellResize, False); argcnt++;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Layout"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Layout"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 169); argcnt++;
   XtSetArg(args[argcnt], XmNy, 647); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Layout = XtCreatePopupShell("shell_layout",
                                     topLevelShellWidgetClass, AppShell, args,
                                     argcnt);
   XtAddCallback(Shell_Layout, XmNdestroyCallback, layout_destroy, 0);

   /* Create the load panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNallowShellResize, True); argcnt++;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Load"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Load"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 836); argcnt++;
   XtSetArg(args[argcnt], XmNy, 330); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Load = XtCreatePopupShell("shell_load",
                                   topLevelShellWidgetClass, AppShell, args,
                                   argcnt);
   XtAddCallback(Shell_Load, XmNdestroyCallback, load_destroy, 0);

   /* Create the histogram panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNallowShellResize, False); argcnt++;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Histogram"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Histogram"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 395); argcnt++;
   XtSetArg(args[argcnt], XmNy, 97); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNkeyboardFocusPolicy,XmPOINTER); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Histo = XtCreatePopupShell("shell_histo",
                                    topLevelShellWidgetClass, AppShell, args,
                                    argcnt);
   XtAddCallback(Shell_Histo, XmNdestroyCallback, histo_destroy, 0);

   /* Create the cursor panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Cursor"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Cursor"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 83); argcnt++;
   XtSetArg(args[argcnt], XmNy, 70); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Cursor = XtCreatePopupShell("shell_cursor",
                                     topLevelShellWidgetClass, AppShell, args,
                                     argcnt);
   XtAddCallback(Shell_Cursor, XmNdestroyCallback, cursor_destroy, 0);

   /* Create the full resolution panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Full Res"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Full Res (Panel)"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 517); argcnt++;
   XtSetArg(args[argcnt], XmNy, 161); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_FullRes = XtCreatePopupShell("shell_fullres",
                                      topLevelShellWidgetClass, AppShell, args,
                                      argcnt);
   XtAddCallback(Shell_FullRes, XmNdestroyCallback, fullres_destroy, 0);

   /* Create the full resolution window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Full Res"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Full Res(Window)"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 914); argcnt++;
   XtSetArg(args[argcnt], XmNy, 672); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_FullResWin = XtCreatePopupShell("shell_fullreswin",
                                         topLevelShellWidgetClass, AppShell,
                                         args, argcnt);
   XtAddCallback(Shell_FullResWin, XmNdestroyCallback, fullres_destroy, 0);

   /* Create the adjust panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNallowShellResize, False); argcnt++;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Adjust"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Map-Adjust"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 381); argcnt++;
   XtSetArg(args[argcnt], XmNy, 118); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Adjust = XtCreatePopupShell("shell_adjust",
                                     topLevelShellWidgetClass, AppShell, args,
                                     argcnt);
   XtAddCallback(Shell_Adjust, XmNdestroyCallback, map_destroy, 0);

   /* Create the piece-wise linear panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - PWL"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Map-PWL"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 573); argcnt++;
   XtSetArg(args[argcnt], XmNy, 134); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_PWL = XtCreatePopupShell("shell_pwl",
                                  topLevelShellWidgetClass, AppShell, args,
                                  argcnt);
   XtAddCallback(Shell_PWL, XmNdestroyCallback, map_destroy, 0);

   /* Create the pseudocolor panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - PSD"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Map-PSD"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 380); argcnt++;
   XtSetArg(args[argcnt], XmNy, 77); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Pseudo = XtCreatePopupShell("shell_pseudo",
                                     topLevelShellWidgetClass, AppShell, args,
                                     argcnt);
   XtAddCallback(Shell_Pseudo, XmNdestroyCallback, map_destroy, 0);

   /* Create the profile panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Profile"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Profile"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 76); argcnt++;
   XtSetArg(args[argcnt], XmNy, 142); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Profile = XtCreatePopupShell("shell_profile",topLevelShellWidgetClass,
                                      AppShell, args, argcnt);
   XtAddCallback(Shell_Profile, XmNdestroyCallback, profile_destroy, 0);

   /* Create the init panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Init"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Init"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 544); argcnt++;
   XtSetArg(args[argcnt], XmNy, 238); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Init = XtCreatePopupShell("shell_init", topLevelShellWidgetClass,
                                   AppShell, args, argcnt);
   XtAddCallback(Shell_Init, XmNdestroyCallback, init_destroy, 0);

   /* Create the zoom panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Zoom"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Zoom"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 59); argcnt++;
   XtSetArg(args[argcnt], XmNy, 162); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Zoom = XtCreatePopupShell("shell_zoom", topLevelShellWidgetClass,
                                   AppShell, args, argcnt);
   XtAddCallback(Shell_Zoom, XmNdestroyCallback, zoom_destroy, 0);

   /* Create the mask panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Mask"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Mask"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 610); argcnt++;
   XtSetArg(args[argcnt], XmNy, 86); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Mask = XtCreatePopupShell("shell_mask", topLevelShellWidgetClass,
                                   AppShell, args, argcnt);
   XtAddCallback(Shell_Mask, XmNdestroyCallback, mask_destroy, 0);

   /* Create the fill panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Fill"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Fill"); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Fill = XtCreatePopupShell("shell_fill", topLevelShellWidgetClass,
                                   AppShell, args, argcnt);
   XtAddCallback(Shell_Fill, XmNdestroyCallback, fill_destroy, 0);

   /* Create the main image window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Main Window"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Main Window"); argcnt++;
   if (geometry_win[0] != 0)
      XtSetArg(args[argcnt], XmNgeometry, geometry_win); argcnt++;
   XtSetArg(args[argcnt], XmNkeyboardFocusPolicy, XmPOINTER); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_MainWin = XtCreatePopupShell("shell_mainwin",topLevelShellWidgetClass,
                                      AppShell, args, argcnt);
   XtAddCallback(Shell_MainWin, XmNdestroyCallback, main_image_destroy, 0);

   /* Create the help panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Help"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Help"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 129); argcnt++;
   XtSetArg(args[argcnt], XmNy, 203); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Help = XtCreatePopupShell("shell_help", topLevelShellWidgetClass,
                                   AppShell, args, argcnt);
   XtAddCallback(Shell_Help, XmNdestroyCallback, help_destroy, 0);

   /* Create the show panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Show"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Show"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 759); argcnt++;
   XtSetArg(args[argcnt], XmNy, 30); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Show = XtCreatePopupShell("shell_show", topLevelShellWidgetClass,
                                   AppShell, args, argcnt);

   /* Create the flicker panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Flicker"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Flicker"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 759); argcnt++;
   XtSetArg(args[argcnt], XmNy, 30); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Flicker = XtCreatePopupShell("shell_flicker",topLevelShellWidgetClass,
                                      AppShell, args, argcnt);

   /* Create the zoom window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Zoom"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Zoom (Window)"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 796); argcnt++;
   XtSetArg(args[argcnt], XmNy, 462); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_ZoomWin = XtCreatePopupShell("shell_zoomwin",topLevelShellWidgetClass,
                                      AppShell, args, argcnt);
   XtAddCallback(Shell_ZoomWin, XmNdestroyCallback, zoom_destroy, 0);

   /* Create the graphics window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Graphics"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Graphics"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 796); argcnt++;
   XtSetArg(args[argcnt], XmNy, 462); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Graphics = XtCreatePopupShell("shell_graphics",
                                       topLevelShellWidgetClass, AppShell,
                                       args, argcnt);
   XtAddCallback(Shell_Graphics, XmNdestroyCallback, graphics_destroy, 0);

   /* Create the save mapping window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle,"X Image Display - Save Mapping"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 398); argcnt++;
   XtSetArg(args[argcnt], XmNy, 267); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_SaveMap = XtCreatePopupShell("shell_savemap",topLevelShellWidgetClass,
                                      AppShell, args, argcnt);
   XtAddCallback(Shell_SaveMap, XmNdestroyCallback, map_destroy, 0);

   /* Create the load mapping window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Load Mapping"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 398); argcnt++;
   XtSetArg(args[argcnt], XmNy, 267); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_LoadMap = XtCreatePopupShell("shell_loadmap",topLevelShellWidgetClass,
                                      AppShell, args, argcnt);
   XtAddCallback(Shell_LoadMap, XmNdestroyCallback, map_destroy, 0);

   /* Create the graphics set attributes window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Set Attributes");
   argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Setatt"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 796); argcnt++;
   XtSetArg(args[argcnt], XmNy, 462); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Setatt = XtCreatePopupShell("shell_setatt", topLevelShellWidgetClass,
                                     AppShell, args, argcnt);
   XtAddCallback(Shell_Setatt, XmNdestroyCallback, graphics_destroy, 0);

   /* Create the graphics define attributes window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Define Attributes");
   argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Defatt"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 796); argcnt++;
   XtSetArg(args[argcnt], XmNy, 462); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Defatt = XtCreatePopupShell("shell_defatt", topLevelShellWidgetClass,
                                     AppShell, args, argcnt);
   XtAddCallback(Shell_Defatt, XmNdestroyCallback, graphics_destroy, 0);

   /* Create the GOF name window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - GOF Name"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "GOFname"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 398); argcnt++;
   XtSetArg(args[argcnt], XmNy, 267); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Gofname = XtCreatePopupShell("shell_gofname",topLevelShellWidgetClass,
                                      AppShell, args, argcnt);
   XtAddCallback(Shell_Gofname, XmNdestroyCallback, graphics_destroy, 0);

   /* Create the GOF label and text window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Graphics"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Label"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 398); argcnt++;
   XtSetArg(args[argcnt], XmNy, 267); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Goflabel= XtCreatePopupShell("shell_goflabel",
                                      topLevelShellWidgetClass, AppShell, args,
                                      argcnt);
   XtAddCallback(Shell_Goflabel, XmNdestroyCallback, graphics_destroy, 0);

   /* Create the mouse legend window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Mouse Legend"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Legend"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 796); argcnt++;
   XtSetArg(args[argcnt], XmNy, 462); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Mouse = XtCreatePopupShell("shell_mouse", topLevelShellWidgetClass,
                                    AppShell, args, argcnt);
   XtAddCallback(Shell_Mouse, XmNdestroyCallback, graphics_destroy, 0);

   /* Create the configuration window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt],XmNtitle,"X Image Display - Configuration"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Config"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 796); argcnt++;
   XtSetArg(args[argcnt], XmNy, 462); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Config = XtCreatePopupShell("shell_config", topLevelShellWidgetClass,
                                     AppShell, args, argcnt);
   XtAddCallback(Shell_Config, XmNdestroyCallback, config_destroy, 0);

   /* Create the preferences window shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Preferences"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Pref"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 796); argcnt++;
   XtSetArg(args[argcnt], XmNy, 462); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Pref = XtCreatePopupShell("shell_pref", topLevelShellWidgetClass,
                                   AppShell, args, argcnt);

   /* Create the power spectrum panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Power Spectrum");
   argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Power"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 76); argcnt++;
   XtSetArg(args[argcnt], XmNy, 142); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Power = XtCreatePopupShell("shell_power", topLevelShellWidgetClass,
                                    AppShell, args, argcnt);
   XtAddCallback(Shell_Power, XmNdestroyCallback, power_destroy, 0);

   /* Create the save panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Save"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Save"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 76); argcnt++;
   XtSetArg(args[argcnt], XmNy, 142); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Save = XtCreatePopupShell("shell_save", topLevelShellWidgetClass,
                                   AppShell, args, argcnt);
   XtAddCallback(Shell_Save, XmNdestroyCallback, save_destroy, 0);

   /* Create the verify panel shell.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "X Image Display - Verify File Name");
   argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "VerifyName"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 76); argcnt++;
   XtSetArg(args[argcnt], XmNy, 142); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
   XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
   Shell_Verify = XtCreatePopupShell("shell_verify", topLevelShellWidgetClass,
                                     AppShell, args, argcnt);
   XtAddCallback(Shell_Verify, XmNdestroyCallback, save_destroy, 0);

   /* Create and manage the main panel shell and main panel window.  */
   Main_shell = Createmain_shell(Shell_MainMenu);
   XtManageChild(Main_shell);

   Main_image_scrollwin = Createmain_image_scrollwin(Shell_MainWin);
   XtManageChild(Main_image_scrollwin);

   /* Create the context and configuration panels.  */
   Context_form = Createcontext_form(Shell_Mouse);
   Config_form = Createconfig_form(Shell_Config);

   /* Create the full resolution and zoom windows (but don't manage them) so
   that they can be used in creating the initial context. */
   Fullres_panel_bulletinboard = Createfullres_panel_bulletinboard
                                                               (Shell_FullRes);
   Fullres_scrollwin = Createfullres_scrollwin(Shell_FullResWin);

   Zoom_bulletinboard = Createzoom_bulletinboard(Shell_Zoom);
   Zoom_scrollwin = Createzoom_scrollwin(Shell_ZoomWin);
   Createzoom_alpha(Shell_Zoom);

   /* Activate the main panel and main window shells.  */
   XtPopup(Shell_MainMenu,XtGrabNone);
   XtPopup(Shell_MainWin,XtGrabNone);

   /* set the cursor for the main menu and main image window */
   pointer = XCreateFontCursor(xid.display,XC_top_left_arrow);
   set_window_cursor(xid.display,XtWindow(Shell_MainMenu),pointer);
   set_window_cursor(xid.display,XtWindow(Shell_MainWin),pointer);

   /* Create the processing message dialog.  */
   Createload_processing_scale(Shell_MainMenu);

   /* Create and assign the pixmap to be used for the application icon.  */
   xid.icon_pixmap=XCreateBitmapFromData(xid.display,XtWindow(Shell_MainMenu),
                                         xid_bits,xid_width,xid_height);
   if (xid.icon_pixmap == NULL)
      XtWarning("Unable to create application icon");
   else
   {
      argcnt = 0;
      XtSetArg(args[argcnt], XmNiconPixmap, xid.icon_pixmap); argcnt++;
      XtSetValues(Shell_MainMenu,args,argcnt);
      XtSetValues(Shell_Layout,args,argcnt);
      XtSetValues(Shell_Load,args,argcnt);
      XtSetValues(Shell_Histo,args,argcnt);
      XtSetValues(Shell_Cursor,args,argcnt);
      /*
      XtSetValues(Shell_FullRes,args,argcnt);
      XtSetValues(Shell_FullResWin,args,argcnt);
      */
      XtSetValues(Shell_Adjust,args,argcnt);
      XtSetValues(Shell_PWL,args,argcnt);
      XtSetValues(Shell_Pseudo,args,argcnt);
      XtSetValues(Shell_Profile,args,argcnt);
      XtSetValues(Shell_Power,args,argcnt);
      XtSetValues(Shell_Init,args,argcnt);
      /*
      XtSetValues(Shell_Zoom,args,argcnt);
      */
      XtSetValues(Shell_Mask,args,argcnt);
      XtSetValues(Shell_MainWin,args,argcnt);
      XtSetValues(Shell_Help,args,argcnt);
      XtSetValues(Shell_Show,args,argcnt);
      XtSetValues(Shell_ZoomWin,args,argcnt);
      XtSetValues(Shell_Graphics,args,argcnt);
      XtSetValues(Shell_SaveMap,args,argcnt);
      XtSetValues(Shell_LoadMap,args,argcnt);
      XtSetValues(Shell_Setatt,args,argcnt);
      XtSetValues(Shell_Defatt,args,argcnt);
      XtSetValues(Shell_Gofname,args,argcnt);
      XtSetValues(Shell_Goflabel,args,argcnt);
      XtSetValues(Shell_Config,args,argcnt);
      XtSetValues(Shell_Pref,args,argcnt);
      XtSetValues(Shell_Save,args,argcnt);
      XtSetValues(Shell_Fill,args,argcnt);
   }

   /* Do the initial memory plane allocation.  */
   status = layout_allocmem();
   if (status != XID_S)
   {
      xid_message("Error allocating default memory configuration.\n"
                  "Execute the LAYOUT function to adjust values.");
   }

   /* Retrieve the key codes used for single stepping of the cursor.  */
   xid_cursor.h_key    = XKeysymToKeycode(xid.display,XK_h);
   xid_cursor.j_key    = XKeysymToKeycode(xid.display,XK_j);
   xid_cursor.k_key    = XKeysymToKeycode(xid.display,XK_k);
   xid_cursor.l_key    = XKeysymToKeycode(xid.display,XK_l);
   xid_cursor.left_key = XKeysymToKeycode(xid.display,XK_Left);
   xid_cursor.down_key = XKeysymToKeycode(xid.display,XK_Down);
   xid_cursor.up_key   = XKeysymToKeycode(xid.display,XK_Up);
   xid_cursor.right_key= XKeysymToKeycode(xid.display,XK_Right);

   /* Set up the initial context.  */
   xid_contextadd(0);
   XeContextSetActive((XeContextWidget)context_mouse,context_names[CON_IMAGE]);

   /* If the user specified an image to open from command line, open it now */
   if((strcmp(xid_load.image_name, "") != 0) && (xid_load.type == NONLAS))
   {
      /* Open the image as a non-LAS format image */
      printf("Opening non-LAS image(s) %s\n", xid_load.image_name);
      fflush(stdout);
      xid_load.init_load = TRUE;

      /* Initialize projection information structure.  */
      xid_projinfo.proj_code = xid_projinfo.zone_code =
         xid_projinfo.datum_code = -999;
      strcpy(xid_projinfo.proj_units,"");
      xid_projinfo.pdist_x = xid_projinfo.pdist_y = xid_projinfo.line_inc = 
         xid_projinfo.sample_inc = 0.0;
      xid_projinfo.upleft[0] = xid_projinfo.upleft[1] = 0.0; 
      xid_projinfo.loleft[0] = xid_projinfo.loleft[1] = 0.0; 
      xid_projinfo.upright[0] = xid_projinfo.upright[1] = 0.0; 
      xid_projinfo.loright[0] = xid_projinfo.loright[1] = 0.0; 
      for (i=0; i < 15; i++)
         xid_projinfo.proj_coef[i] = 0.0;
      nonlas_dialog(NULL, NULL, NULL);
   }
   else if(strcmp(xid_load.image_name, "") != 0)
   {
      /* If the user specified to open a LAS image or If the user supplied a
         file name, but did not specify the type of image, we assume it is in
         LAS format. */
      printf("Opening LAS image(s) %s\n", xid_load.image_name);
      fflush(stdout);
      xid_load.init_load = TRUE;
      xid_load.type = LAS;
      load_localok(NULL, NULL, NULL);
   }
   else if((strcmp(xid_load.image_name, "") == 0) && ((xid_load.type == LAS) ||
           (xid_load.type == NONLAS)))
   {
      /* The user specified a image format option but did not supply an image
         name.  Just print a little message to let them know we ignored the
         image format option. */
      printf("Image name was not specified. Ignoring image format option.\n");
      fflush(stdout);
   }

   /* Solicit events.  */
   XtAppMainLoop(context);
   return 0;
}

void xid_initialize
(void)

/******************************************************************************
Purpose:
   This routine initializes the appropriate XID structure elements and widget
   IDs.  It also sets the appropriate signal handler for trapping the demise 
   of child processes.  This is necessary so that XID knows whether or not
   xmosaic is currently running.
                                                                
By L. Huewe, USGS/EROS Data Center, 2/93

Algorithm Description:
******************************************************************************/
{
   long  i,j;                    /* Looping indices.                         */
   long  num_bytes;              /* Number of bytes to allocate for graphics
                                    elements.                                */

   /* Set signal traps.  */
   signal(SIGCLD,(void (*) ())help_reset);
   xid.help_pid = -1;

   /* Initialize globals.  */
   xid.cursor_color = GREEN_COL;
   xid.cindex = 0;
   xid.flags = 0;
   xid.line_x = xid.line_y = -1;
   xid.line_gc = (GC)NULL;
   xid.num_vert = xid.num_horiz = 0;
   xid.imagemap = (Colormap)NULL;
   xid.num_composite = -1;
   xid_composite = NULL;
   xid_zoom.parms.work = NULL;
   xid_zoom.parms.horiz_table = NULL;
   xid_zoom.parms.input = NULL;
   xid_zoom.parms.output = NULL;
   xid_zoom.factor = 2;
   xid_adjust.lut_slope = 45.0;
   xid_adjust.lut_offset = 0.0; 
   xid_load.save = 0;
   xid_psd.flags = PSD_INIT;
   xid_psd.color_psd = (XColor *)NULL;
   xid_psd.range = NULL;
   xid_psd.bin = NULL;
   xid_cursor.pointer_pixmap = NULL;
   xid_cursor.info_pixmap = 0;
   xid_cursor.clear_pixmap = 0;
   xid_help.flags = 0; 
   xid_help.widget = (Widget)NULL;
   xid_load.dirmask = (XmString)NULL;
   xid_load.init_load = FALSE;
   xid_load.loading = FALSE;
   xid_load.savetiff_dirmask = (XmString)NULL;
   xid_load.type = -1;
   strcpy(xid_load.image_name, "");
   for (i=0; i < MASK_MAX; i++)
   {
      xid_mask.mask_pixmap[i] = 0;
      xid_mask.color[i] = -1;
   }
   xid_mask.current_color = -1;
   xid_layout.view_width = xid_mem.view_width = DEF_VWIDTH;
   xid_layout.mem_depth = DEF_NPLANE;
   xid_layout.mem_height = xid_mem.mem_height = DEF_HEIGHT;
   xid_layout.mem_width = xid_mem.mem_width = DEF_WIDTH;
   xid_layout.view_height = xid_mem.view_height = DEF_VHEIGHT;
   xid_fullres = NULL;
   xid.current_plane = 0;
   xid.x = xid.y = 0;
   xid.max_plane = DEF_NPLANE;
   xid_mem.imgpix = FALSE;
   xid_mem.image_pixmap = 0;
   xid_cursor.fp = NULL;
   xid_show.composite = NULL;
   xid.icon_pixmap = 0;
   xid.message = FALSE;
   xid.red_shift = 0;
   xid.green_shift = 0;
   xid.blue_shift = 0;
   xid_power.coef = NULL;
   xid_power.gc = (GC)NULL;
   xid_power.flags = POWER_X | POWER_ENT | POWER_CONT | POWER_AUTO;
   xid_save.name = NULL;
   xid_graphics.flags = 0;
   xid_graphics.type = NO_TYPE;
   xid_graphics.current = 0;
   xid_graphics.gc = (GC)NULL;
   xid_graphics.rb_gc = (GC)NULL;
   for (i=0; i < USER_GRFONTS; i++)
      xid_graphics.font_info[i] = (XFontStruct *)NULL;
   strcpy(xid_graphics.lname_gof,"");
   for (i=0; i < USER_GRCOLORS; i++)
   {
      xid_graphics.pixmap[i] = 0;
      xid_graphics.full_pix[i] = 0;
      xid_graphics.zoom_pix[i] = 0;
   }
   xid_fill.num = xid_fill.num_poly = xid_fill.constant = 0;
   xid_fill.flags = FILL_USER | FILL_INTERNAL | FILL_MAIN | FILL_CONSTANT | 
                    FILL_UPDATE;
   xid_fill.max_breakpts = FILL_MAXBREAK;
   xid_fill.from = (long *)malloc(FILL_MAXBREAK * sizeof(long));
   if (xid_fill.from == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   xid_fill.to = (long *)malloc(FILL_MAXBREAK * sizeof(long));
   if (xid_fill.to == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   xid_fill.max_polys = FILL_MAXPOLY;
   xid_fill.polys = (long *)malloc(FILL_MAXPOLY * sizeof(long));
   if (xid_fill.polys == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   fill_breakpt_dialog=fill_const_dialog= fill_from_list = fill_to_list = NULL;

   /* Allocate space for graphics elements.  */
   num_bytes = MAXELEMENT * sizeof(struct GRAPH);
   xid_graphics.max_element = MAXELEMENT;
   xid_element = (struct GRAPH *)malloc(num_bytes);
   if (xid_element == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   for (i=0; i < MAXELEMENT; i++)
   {
      xid_element[i].type = NO_TYPE;
      xid_element[i].offset = xid_element[i].writeflg = xid_element[i].color =
         xid_element[i].font = 0;
      xid_element[i].x = xid_element[i].y = -1;
      xid_element[i].labelx = xid_element[i].labely = -1;
      xid_element[i].upleft[0] = xid_element[i].upleft[1] = 0.0;
      xid_element[i].loright[0] = xid_element[i].loright[1] = 0.0;
      for (j=0; j < MAXATT; j++)
         strcpy(xid_element[i].attributes[j],"");
      strcpy(xid_element[i].label,"");
      xid_element[i].graph_ptr = NULL;
   }
   for (i=0; i < MAXATT; i++)
      strcpy(xid_graphics.attr_values[i],"");

   xid_editorig.graph_ptr = NULL;
   xid_editundo.graph_ptr = NULL;

   /* Initialize widget IDs.  */
   cursor_bulletinboard = NULL;
   profile_form = NULL;
   power_form = NULL;
   histo_form = NULL;
   init_bulletinboard = NULL;
   layout_bulletinBoard = NULL;
   adjust_form = NULL;
   pwl_bulletinboard = NULL;
   psd_bulletinboard = NULL;
   savemap_form = NULL;
   loadmap_form = NULL;
   mask_bulletinboard = NULL;
   Show_form = NULL;
   pref_form = NULL;
   power_form = NULL;
   flicker_form = NULL;
   save_form = NULL;

   /* Set the appropriate flags.  */
   xid_cursor.flags = (CURSOR_XY | CURSOR_ORIG | CURSOR_AUTO);
   xid_histo.flags = (HISTO_DIST | HISTO_XY | HISTO_ORIG | HISTO_IMAGE);
   xid_zoom.flags = 0;

   /* Set the default resampling method in the zoom parameter structure.  */
   strcpy(xid_zoom.parms.method,"NN");
   xid_map.flags = MAP_ALL;

   /* Initialize the pseudocolor structure.  */
   xid_psd.flags = (PSD_INIT & PSD_SLIDER);
   xid_psd.text_list = (char *)NULL;
   xid_psd.range = xid_psd.bin = (long *)NULL;
   xid_psd.savecolors = (XColor *)NULL;

   /* Initialize mask structure values.  */
   xid_mask.rotate_value = 1;
   xid_mask.graphics_mask = 0;
   xid_mask.current_color = -1;
   xid_mask.gc = (GC)NULL;
}

int parseargs
(
   int    argc,               /* Number of command line arguments. (I)       */
   char   *argv[],            /* Command line arguements. (I)                */
   char   *geometry_menu,     /* Geometry specification for main menu. (O)   */
   char   *geometry_win       /* Geometry specification for main window. (O) */
)

/******************************************************************************
Purpose:
   This function parses the command line arguments.

By L. Hemmer, USGS/EROS Data Center, 2/93

Modified by A. Shimitz 11/00
 - Added statement for the -ws option (CR #3967)

Algorithm Description:
    Taken from TIEPTS function.
******************************************************************************/
{
   int    i;               /* Loop counter                                   */
   int    helpflg = FALSE; /* Used to determine if the user want help about
                              the command line options available             */
   int    exitflg = FALSE; /* Set to TRUE if the user wants to see the help
                              message and they did not specify any other
                              command line arguments                         */
   int    getfile_flg = FALSE;
                           /* Set to TRUE after the -las or -non option is
                              found to indicate the next param should be
                              the file name.                                 */
   void   print_help(void); /* Help message printing routine                 */

   /* Start with this. Change them if command line arguments were specified. */
   strcpy(geometry_menu,"");
   strcpy(geometry_win,"");

   /* Examine each of the command line arguments.  */
   for (i = 1; i < argc; i++) 
   {
      if(getfile_flg || (strncmp(&argv[i][0], "-", 1) != 0))
      {
         if(strncmp(&argv[i][0], "-", 1) != 0)
         {
            /* Get the file name specified */
            strcpy(xid_load.image_name, &argv[i][0]);
         }
         else
         {
            /* Print help file.  */
            print_help();
            exitflg = TRUE;
            break;
         }
         getfile_flg = FALSE;
      }
      if (strncmp(&argv[i][0],"-ul",3) == 0)
      {
         strcpy(geometry_menu,"+0+0");
         strcpy(geometry_win,"+0+90");
      }
      else if (strncmp(&argv[i][0],"-ur",3) == 0)
      {
         strcpy(geometry_menu,"-0+0");
         strcpy(geometry_win,"-0+90");
      }
      else if (strncmp(&argv[i][0],"-ll",3) == 0)
      {
         strcpy(geometry_menu,"+0+383");
         strcpy(geometry_win,"+0-0");
      }
      else if (strncmp(&argv[i][0],"-lr",3) == 0)
      {
         strcpy(geometry_menu,"-0+383");
         strcpy(geometry_win,"-0-0");
      }
      else if (strncmp(&argv[i][0],"-las",4) == 0)
      {
         xid_load.type = LAS;
         getfile_flg = TRUE;
      }
      else if (strncmp(&argv[i][0],"-non",4) == 0)
      {
         xid_load.type = NONLAS;
         getfile_flg = TRUE;
      }
      else if (strncmp(&argv[i][0],"-ws",3) == 0)
         xid.flags = xid.flags | XID_WINNOREINIT;
      else if (strncmp(&argv[i][0],"-h",2) == 0)
      {
         helpflg = TRUE;

         /* Print help file.  */
         print_help();
      }
      else if (strncmp(&argv[i][0], "-", 1) == 0)
      {
         /* Print help file.  */
         print_help();
         exitflg = TRUE;
         break;
      }
   }
   if(getfile_flg)
   {
      /* Print help file.  */
      print_help();
      exitflg = TRUE;
   }
   if(argc < 3 && helpflg)
      exitflg = TRUE;
   return exitflg;
}

void print_help
(void)

/*****************************************************************************
Purpose:
   This function will print the help message explaining the usage of
   the command line parameters to the terminal display.
By A. Shimitz, USGS/EROS Data Center, 7/00

Modified by A. Shimitz 11/00
 - Added ws option (CR #3967)

Algorithm Description:
******************************************************************************/
{
   printf("usage: xid [ -ul|-ur|-ll|-lr ] [ -las <file>|-non <file> ] [ -noresize] [ -h ]\n"
          "  where\n"
          "  -ul         Opens windows in the upper left corner of the"
                         " display.\n"
          "  -ur         Opens windows in the upper right corner of the"
                         " display.\n"
          "  -ll         Opens windows in the lower left corner of the"
                         " display.\n"
          "  -lr         Opens windows in the lower right corner of the"
                        " display.\n"
          "  -las <file> Opens LAS format image <file>.\n"
          "  -non <file> Opens non-LAS format image <file>.\n"
          "  <file>      Opens LAS format image <file>.\n"
          "  -ws         Supresses resize of main image window upon"
                         " loading new data in it.\n"
          "  -h          Displays help message for command line"
                         " parameters.\n\n");
   fflush(stdout);
}

void init_map_histo
(void)

/*****************************************************************************
Purpose:
   This function will reinitialize the flags used for the Mapped button on the
   histogram panel. It will then zero out the mapped histogram values. If the
   Mapped button was selected, copy the original histogram pixel values to the
   mapped histogram pixel values and refresh the panel.

By A. Shimitz, USGS/EROS Data Center, 7/00

Modified by A. Shimitz 11/00
 - Modified to support storage of each bands of z-dimensional histogram data
   separately.  (CR #4156)

Algorithm Description:
******************************************************************************/
{
   long    dtype = ELONG;      /* Data type of histogram buffer.             */
   long    nbins = 256;        /* Number of histogram bins.                  */
   long    i,j;                /* Looping indices                            */
   long    zero = 0;           /* Constant zero.                             */

   /* (Re)Initialize each memory plane's associated mapped histogram as well as
      the z dimension mapped histogram.  */
   for (i = 0; i < 3; i++)
   {
      c_pxfill((unsigned char *)xid_planes[i].histo_map,&dtype,&nbins,
               (unsigned char *)&zero);
      xid_histcalc(xid_planes[i].histo_map, &xid_planes[i].minval,
                   &xid_planes[i].maxval, &xid_planes[i].mean,
                   &xid_planes[i].stdev);
   }
   c_pxfill((unsigned char *)xid_histo.z_histo_map,&dtype,&nbins,
            (unsigned char *)&zero);

   /* If user is viewing Mapped histogram, copy the original histogram values
      to the mapped histogram and redisplay the panel. */
   if(XtIsRealized(Shell_Histo))
   {
      if(xid_histo.flags & HISTO_MAPPED)
      {
         /* Copy the current histogram information to the mapped histogram */
         for(i = 0; i < 3; i++)
         {
            for(j = 0; j < 256; j++)
            {
               if(xid_histo.flags & HISTO_XY)
                  xid_planes[i].histo_map[j] = xid_planes[i].histogram[j];
               else
                  xid_histo.z_histo_map[i][j] = xid_histo.z_histo[i][j];
            }
         }

         /* Recalculate the histogram statistics */
         for(i = 0; i < 3; i++)
         {
            if(xid_histo.flags & HISTO_XY)
            {
               xid_histcalc(xid_planes[i].histo_map, &xid_planes[i].minval,
                            &xid_planes[i].maxval, &xid_planes[i].mean,
                            &xid_planes[i].stdev);
            }
            else
            {
               xid_histcalc(xid_histo.z_histo_map[i],&xid_histo.minval,
                            &xid_histo.maxval,&xid_histo.mean,&xid_histo.stdev);
            }
         }

         /* Refresh the histogram panel */
         if(XtIsManaged(Histo_form))
         {
            XClearWindow(xid.display,XtWindow(histo_drawingarea));
            histo_draw();
         }
      }

      /* The user may have been viewing the mapped histogram data and then
         selected the Original button without pressing Update. So if this is
         a possibility (a mapping has been applied and the Original button is
         selected) we will just update the histogram using the original
         data. */
      else if ((xid_histo.flags & HISTO_ORIG) && xid.mapped)
      {
         xid.mapped_mod = TRUE;
         xid.mapped_z_mod = TRUE;
         if(XtIsManaged(Histo_form))
            histo_update(NULL, NULL, NULL);
      }
   }

   /* Unset any flags to indicate we no longer are storing any mapped
      histogram data. */
   xid.mapped = FALSE;
   xid.mapped_mod = FALSE;
   xid.mapped_z_mod = FALSE;
}
