/*
 *  This is the fullres function source file for the fullres panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include "xid.h"
#include "manage_cursor.h"
#include "xid_graphics.h"
#include "xid_func.h"
#include "../xid_widgets/ProcessM.h"
#include "../xid_widgets/Context.h"

/* local function prototypes */
void fullres_drag
(
   Widget w,
   XtPointer client,
   XEvent *call,
   Boolean *cont
);
/* end prototypes */

void fullres_create
(void)

/******************************************************************************
Purpose:
   This routine calls the routine which allocates memory for the full
   resolution structure, calls the routine which initializes the variables
   defining the default full resolution window size and adds the appropriate 
   event handlers.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   Widget  widget = 0;         /* Widget ID.                                 */
   long    create = FALSE;     /* Indicates whether or not full resolution
                                  pixmaps were created.                      */
   long    i;                  /* Looping index.                             */
   long    state;              /* Toggle button state.                       */
   char    fullres_opt[4];     /* Full resolution window size option.        */

   /* Allocate memory for full resolution structure and initialize appropriate
      variables.  */
   fullres_setup();

   /* Make sure the appropriate flags and variables are set for the default
      size full resolution window.  */
   state = XmToggleButtonGetState(fullres_256_button);
   if (state)
   {
      strcpy(fullres_opt,"256");
      widget = fullres_256_button;
   }
   else
   {
      state = XmToggleButtonGetState(fullres_512_button);
      if (state)
      {
         strcpy(fullres_opt,"512");
         widget = fullres_512_button;
      }
      else
      {
         state = XmToggleButtonGetState(fullres_full_button);
         if (state)
         {
            strcpy(fullres_opt,"FS");
            widget = fullres_full_button;
         }
         else
         {
            state = XmToggleButtonGetState(fullres_user_button);
            if (state)
            {
               strcpy(fullres_opt,"US");
               widget = fullres_user_button;
            }
         }
      }
   }

   /* Set appropriate values based on the full resolution size.  */
   fullres_size(widget,fullres_opt,NULL);

   /* Create full resolution window pixmaps if the the graphics window is
      active or if the main window graphics pixmaps exist.  */
   if (((xid_graphics.flags & GR_ACTIVE) || (xid_graphics.pixmap[0] != NULL))
       && (xid_fullres[0].height != 0) && (xid_fullres[0].width != 0))
   {
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);

      if (xid_graphics.full_pix[0] == NULL)
      {
         for (i=0; i < USER_GRCOLORS; i++)
            xid_graphics.full_pix[i] = XCreatePixmap(xid.display,XtWindow(
                                                    fullres_win_bulletinboard),
                                                    xid_fullres[0].width,
                                                    xid_fullres[0].height,1);
         create = TRUE;
      }

      if (create) 
      {
         for (i=0; i < USER_GRCOLORS; i++)
         {
            XFillRectangle(xid.display,xid_graphics.full_pix[i],
                           xid_graphics.rb_gc,0,0,xid_fullres[0].width,
                           xid_fullres[0].height);
         }
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
   }

   /* Add appropriate event handlers if the cursor window is active.  */
   if ((xid_cursor.flags & CURSOR_ACTIVE) && (xid_cursor.flags & CURSOR_MAN))
   {
      XtAddEventHandler(fullres_win_bulletinboard,ButtonReleaseMask|
                        KeyReleaseMask,FALSE,cursor_track,NULL);
   }
   else if ((xid_cursor.flags & CURSOR_ACTIVE) &&
            (xid_cursor.flags & CURSOR_AUTO))
   {
      XtAddEventHandler(fullres_win_bulletinboard,ButtonReleaseMask|
                        PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,
                        NULL);
   }

   /* Add an event handler to detect pointer motion and key release events in
      the full resolution image window.  */
   XtAddEventHandler(fullres_win_bulletinboard,PointerMotionMask|
                     KeyReleaseMask,FALSE,cursor_pointer,NULL);

   /* Add an event handler to detect exposure events in the full resolution
      image window.  */
   XtAddEventHandler(fullres_win_bulletinboard,ExposureMask,FALSE,
                     graphics_expose,NULL);
}

void fullres_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine closes the full resolution window and full resolution image
   window, sets the main menu option to sensitive, resets bit flags and
   removes the appropriate event handler.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   long    i;                  /* Looping index.                             */
   long    plane_idx;          /* Index into memory plane structure.         */
   int     xwin;               /* X window coordinate.                       */
   int     ywin;               /* Y window coordinate.                       */

   /* Erase any full resolution outlines.  */
   if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
   {
      /* Determine the memory plane index.  */
      plane_idx = xid_view.rgbplanes[RED] - 1;
      if (plane_idx != -1)
      {
         xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
                      (float)xid_fullres[0].window[SL],&xwin,&ywin);
         XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,
                    ywin-5,xid_fullres[0].width + 10,
                    xid_fullres[0].height + 10,TRUE);
      }
   }

   /* Free resources.  */
   if (xid_fullres[0].flags & FULLRES_USER)
   {
      XtRemoveEventHandler(main_image_bulletinboard,Button1MotionMask|
                           ButtonReleaseMask,FALSE,fullres_drag,NULL);
   }
   XFreeGC(xid.display,xid_fullres[0].gc);
   xid_fullres[0].gc = (GC)NULL;

   for (i=0; i < xid.num_fullres; i++)
   {
      if (xid_fullres[i].image_data != NULL)
      {
         free(xid_fullres[i].image_data);
         xid_fullres[i].image_data = NULL;
      }
      if (xid_fullres[i].image_pixmap != NULL)
      {
         XFreePixmap(xid.display,xid_fullres[i].image_pixmap);
         xid_fullres[i].image_pixmap = NULL;
      }
   }

   /* If flicker pixmaps were created and the flicker panel is still open
      free the pixmaps and the GC */
   if((xid_fullres[0].flicker_pixmaps[0] != NULL) &&
      (xid_flicker.flags & FLICKER_ACTIVE))
   {
      for(i = 0; i < xid_flicker.nplanes; i++)
         XFreePixmap(xid.display,xid_fullres[0].flicker_pixmaps[i]);
      if(xid_fullres[0].flicker_gc != NULL)
         XFreeGC(xid.display,xid_fullres[0].flicker_gc);
   }

   free(xid_fullres);
   xid_fullres = (struct XID_FULLRES *)NULL;
   xid.num_fullres = 0;

   /* Clear the full resolution window.  */
   XSetWindowBackground(xid.display,XtWindow(fullres_win_bulletinboard),
                        xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
   XClearWindow(xid.display,XtWindow(fullres_win_bulletinboard));

   if (xid_graphics.flags & GR_ACTIVE)
   {
      XtRemoveEventHandler(fullres_win_bulletinboard,LeaveWindowMask|
                           EnterWindowMask,FALSE,graphics_expose,NULL);
   }
   if ((xid_cursor.flags & CURSOR_ACTIVE) && (xid_cursor.flags & CURSOR_MAN))
   {
      XtRemoveEventHandler(fullres_win_bulletinboard,ButtonReleaseMask|
                           KeyReleaseMask,FALSE,cursor_track,NULL);
   }
   else if ((xid_cursor.flags & CURSOR_ACTIVE) &&
            (xid_cursor.flags & CURSOR_AUTO))
   {
      XtRemoveEventHandler(fullres_win_bulletinboard,ButtonReleaseMask|
                           PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,
                           NULL);
   }

   if (xid_zoom.flags & ZOOM_ACTIVE) 
   {
      /* If a zoomed window which was calculated from the full resolution
         window is currently displayed, clear the zoomed window (i.e., if the
         full resolution window is gone, the corresponding zoomed window no
         longer makes sense).  */
      if (xid_zoom.flags & ZOOM_FULLRES)
      {
         if (xid_zoom.flags & ZOOM_ACTIVE)
         {
            XSetWindowBackground(xid.display,XtWindow(zoom_win_bulletinboard),
                                 xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
            XClearWindow(xid.display,XtWindow(zoom_win_bulletinboard));
            if (xid_zoom.image_pixmap != NULL)
            {
               XFreePixmap(xid.display,xid_zoom.image_pixmap);
               xid_zoom.image_pixmap = 0;
            }
            if (xid_zoom.image_data != NULL)
            {
               free(xid_zoom.image_data);
               xid_zoom.image_data = NULL;
            }
         }
      }
   }

   /* No longer need to track the cursor in the full resolution window.  */
   XtRemoveEventHandler(fullres_win_bulletinboard,PointerMotionMask|
                        KeyReleaseMask,FALSE,cursor_pointer,NULL);

   /* Remove the cursor for the full res windows before they are destroyed */
   remove_window_cursor(xid.display,XtWindow(Fullres_scrollwin));
   remove_window_cursor(xid.display,XtWindow(Fullres_panel_bulletinboard));

   /* Close both the full resolution panel and image window.  */
   XtUnmanageChild(Fullres_panel_bulletinboard);
   XtUnmanageChild(Fullres_scrollwin);
   XtPopdown(Shell_FullRes);
   XtPopdown(Shell_FullResWin);

   /* Set the full resolution button on the main panel to sensitive.  */
   XtSetSensitive(main_fullres_button,True);

   /* Reset the active flag.  */
   for (i=0; i < xid.num_fullres; i++)
      xid_fullres[i].flags = xid_fullres[i].flags ^ FULLRES_ACTIVE;
}

void fullres_size
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine sets the appropriate variables and flags based on the full
   resolution size chosen by the user.  If the user chose the user selected
   size, an event handler is added which allows the user to select the size
   of the full resolution window with the cursor.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   long    i;                  /* Looping index.                             */
   long    plane_idx;          /* Index into memory plane structure.         */
   int     xwin;               /* X window coordinate.                       */
   int     ywin;               /* Y window coordinate.                       */
   char    height[5];          /* Height text string.                        */
   char    width[5];           /* Width text string.                         */

   /* Determine the memory plane index.  */
   plane_idx = xid_view.rgbplanes[RED] - 1;

   /* Based on the option selected, set the appropriate variables and flags.  */
   if (strcmp(client,"256") == 0)
   {
      /* Erase any previous full resolution outlines.  */
      if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
      {
         xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
                      (float)xid_fullres[0].window[SL],&xwin,&ywin);
         XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,
                    ywin-5,xid_fullres[0].width + 10,
                    xid_fullres[0].height + 10,TRUE);
      }

      /* Remove the event handler that allows the user to select the full
         resolution size with the cursor.  */
      if (xid_fullres[0].flags & FULLRES_USER)
      {
         XtRemoveEventHandler(main_image_bulletinboard,Button1MotionMask|
                              ButtonReleaseMask,FALSE,fullres_drag,NULL);
      }
      for (i=0; i < xid.num_fullres; i++)
         xid_fullres[i].width = xid_fullres[i].height = 256;
      XmTextSetString(fullres_h_text,"256");
      XmTextSetString(fullres_w_text,"256");
      for (i=0; i < xid.num_fullres; i++)
         xid_fullres[i].flags = xid_fullres[i].flags & ~FULLRES_USER;
   }
   else if (strcmp(client,"512") == 0)
   {
      /* Erase any previous full resolution outlines.  */
      if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
      {
         xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
                      (float)xid_fullres[0].window[SL],&xwin,&ywin);
         XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,
                    ywin-5,xid_fullres[0].width + 10,
                    xid_fullres[0].height + 10,TRUE);
      }

      /* Remove the event handler that allows the user to select the full
         resolution size with the cursor.  */
      if (xid_fullres[0].flags & FULLRES_USER)
      {
         XtRemoveEventHandler(main_image_bulletinboard,Button1MotionMask|
                              ButtonReleaseMask,FALSE,fullres_drag,NULL);
      }
      for (i=0; i < xid.num_fullres; i++)
         xid_fullres[i].width = xid_fullres[i].height = 512;
      XmTextSetString(fullres_h_text,"512");
      XmTextSetString(fullres_w_text,"512");
      for (i=0; i < xid.num_fullres; i++)
         xid_fullres[i].flags = xid_fullres[i].flags & ~FULLRES_USER;
   }
   else if (strcmp(client,"FS") == 0)
   {

      /* Erase any previous full resolution outlines.  */
      if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
      {
         xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
                      (float)xid_fullres[0].window[SL],&xwin,&ywin);
         XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,
                    ywin-5,xid_fullres[0].width + 10,
                    xid_fullres[0].height + 10,TRUE);
      }

      /* Remove the event handler that allows the user to select the full
         resolution size with the cursor.  */
      if (xid_fullres[0].flags & FULLRES_USER)
      {
         XtRemoveEventHandler(main_image_bulletinboard,Button1MotionMask|
                              ButtonReleaseMask,FALSE,fullres_drag,NULL);
      }

      /* Get the display size.  */
      for (i=0; i < xid.num_fullres; i++)
      {
         xid_fullres[i].width = DisplayWidth(xid.display,xid.screen);
         xid_fullres[i].height= DisplayHeight(xid.display,xid.screen);
      }
      sprintf(width,"%ld",xid_fullres[0].width);
      sprintf(height,"%ld",xid_fullres[0].height);
      XmTextSetString(fullres_h_text,height);
      XmTextSetString(fullres_w_text,width);
      for (i=0; i < xid.num_fullres; i++)
         xid_fullres[i].flags = xid_fullres[i].flags & ~FULLRES_USER;
   }
   else
   {
      xid_fullres[0].ulx = xid_fullres[0].uly = xid_fullres[0].prevx =
                                                     xid_fullres[0].prevy = -1;

      /* Add the event handler that allows the user to select the full
         resolution size with the cursor.  */
      XtAddEventHandler(main_image_bulletinboard,Button1MotionMask|
                        ButtonReleaseMask,FALSE,fullres_drag,NULL);
      for (i=0; i < xid.num_fullres; i++)
         xid_fullres[i].flags = xid_fullres[i].flags | FULLRES_USER;
   }

   /* Free and recreate the full resolution pixmaps if the graphics window is
      active.  */
   if (((xid_graphics.flags & GR_ACTIVE) || (xid_graphics.full_pix[0] != NULL))
       && !(xid_fullres[0].flags & FULLRES_USER))
   {
      if (xid_graphics.full_pix[0] != NULL)
      {
         for (i=0; i < USER_GRCOLORS; i++)
            XFreePixmap(xid.display,xid_graphics.full_pix[i]);
      }

      /* Create full resolution window pixmaps.  */
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);

      for (i=0; i < USER_GRCOLORS; i++)
      {
         xid_graphics.full_pix[i] = XCreatePixmap(xid.display,XtWindow(
                                                  fullres_win_bulletinboard),
                                                  xid_fullres[0].width,
                                                  xid_fullres[0].height,1);
         XFillRectangle(xid.display,xid_graphics.full_pix[i],
                        xid_graphics.rb_gc,0,0,xid_fullres[0].width,
                        xid_fullres[0].height);
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
   }
}

void fullres_update
(
   long *sl,                   /* Starting lines of window to load.          */
   long *ss                    /* Starting samples of window to load.        */
)

/******************************************************************************
Purpose:
   This routine determines the full resolution window to be loaded and calls
   the appropriate routine to load and display the window.

By L. Huewe, USGS/EROS Data Center, 11/91

Modified by A.Shimitz 1/01
  - Added a (- 1) to the offset calculations because the last pixel was being
    missed.  CR 3610

Modified by A.Shimitz 7/03
  - Fixed section that determines the area to do fullrez on if the user's
    selection falls partially outside the actual image. It doesn't move the
    selection inside the image correctly when working with a windowed image.
    It was comparing the selection area with the total lines and samples in
    the image, not just the size of the windowed portion of the image the
    user loaded. CR 8077

Algorithm Description:
******************************************************************************/
{
   long    band_ind;           /* Variable to indicate which band the current
                                  plane data was being viewed in (R, G, or B)*/
   long    height;             /* Height of window (main image window 
                                  coords).                                   */
   long    i, j, k;            /* Looping index.                             */
   long    offset;             /* Adjustment for fullres window.             */
   long    plane_idx;          /* Index into memory plane structure.         */
   long    status;             /* Status indicator.                          */
   long    width;              /* Width of window (main image window coords).*/
   int     xwin;               /* X window coordinate of upper left corner.  */
   int     ywin;               /* Y window coordinate of lower right corner. */
   char    outheight[5];       /* Height character string.                   */
   char    outwidth[5];        /* Width character string.                    */

   /* Output the user-selected height and width.  */
   if (xid_fullres[0].flags & FULLRES_USER)
   {
      sprintf(outheight,"%ld",xid_fullres[0].height);
      sprintf(outwidth,"%ld",xid_fullres[0].width);
      XmTextSetString(fullres_h_text,outheight);
      XmTextSetString(fullres_w_text,outwidth);
   }

   /* Determine the starting line and sample of the window to be extracted.  */
   plane_idx = xid_view.rgbplanes[RED] - 1;
   if (!(xid_fullres[0].flags & FULLRES_USER))
   {
      for (i = 0; i < xid.num_fullres; i++)
      {
         sl[i] = sl[i] - xid_fullres[0].height / 2;
         ss[i] = ss[i] - xid_fullres[0].width / 2;
      }
   }

   /* Reset starting line and sample if the window falls outside the displayed
      image.  */
   for (i = 0; i < xid.num_fullres; i++)
   {
      if ((sl[i] + xid_fullres[0].height) >
          (xid_planes[plane_idx].window[SL] + xid_planes[plane_idx].window[NL]))
      {
         offset = xid_fullres[0].height - ((xid_planes[plane_idx].window[SL] +
                                            xid_planes[plane_idx].window[NL]) -
                                           sl[i]);
         for (j = 0; j < xid.num_fullres; j++)
            sl[j] -= offset;
      }
      if (sl[i] < xid_planes[plane_idx].window[SL])
      {
         offset = xid_planes[plane_idx].window[SL] - sl[i];     
         for (k = 0; k < xid.num_fullres; k++)
            sl[k] += offset;
      }
      if ((ss[i] + xid_fullres[0].width) >
          (xid_planes[plane_idx].window[SS] + xid_planes[plane_idx].window[NS]))
      {
         offset = xid_fullres[0].width - ((xid_planes[plane_idx].window[SS] +
                                           xid_planes[plane_idx].window[NS]) -
                                          ss[i]);
         for (j = 0; j < xid.num_fullres; j++)
            ss[j] -= offset; 
      }
      if (ss[i] < xid_planes[plane_idx].window[SS])
      {
         offset = xid_planes[plane_idx].window[SS] - ss[i];
         for (k = 0; k < xid.num_fullres; k++)
            ss[k] += offset;
      }
   }

   /* Draw a new outline of the full resolution window in the main image
      window.  */
   if (xid_fullres[0].flags & FULLRES_SHOW)
   {
      XSetForeground(xid.display,xid_fullres[0].gc,xid.color_table[RGBBW_IDX + 
                     xid.cursor_color].pixel);
      XSetFunction(xid.display,xid_fullres[0].gc,GXcopy);
      xid_file2win(plane_idx,0,(float)ss[0],(float)sl[0],&xwin,&ywin);
      width = (long)((float)xid_fullres[0].width /
                     (float)xid_planes[plane_idx].skips);
      height = (long)((float)xid_fullres[0].height /
                      (float)xid_planes[plane_idx].skipl);
      XDrawRectangle(xid.display,XtWindow(main_image_bulletinboard),
                     xid_fullres[0].gc,xwin,ywin,width,height);
      XSetForeground(xid.display,xid_fullres[0].gc,xid_fullres[0].rb_fore);
      XSetFunction(xid.display,xid_fullres[0].gc,GXxor);
   }

   /* Initialize each band and the band indicators. */
   for(j=0;j<3;j++)
   {
      for(i=0;i<3;i++)
      {
         xid_load.bands[j][i] = 0;
         xid_load.rgb[j][i] = -1;
      }
   }

   /* Initialize the band indicator to RED */
   band_ind = RED;

   /* For each full resolution band to be loaded ... Here j represents the
      image number */
   for (j = 0, i=0; i < xid.num_fullres; i++)
   {
      /* Determine the memory plane index.  */
      plane_idx = xid_view.rgbplanes[i] - 1;

      /* Determine the proper image number */
      for(j = 0; j < i; j++)
      {
         if(strcmp(xid_planes[xid_view.rgbplanes[j] - 1].filename,
                   xid_planes[plane_idx].filename) == 0)
            break;
      }

      /* Free any previously allocated memory and allocate the amount needed
         for the new window to be loaded.  */
      if (xid_fullres[i].image_data != NULL)
      {
         free(xid_fullres[i].image_data);
         xid_fullres[i].image_data = NULL;
      }
      xid_fullres[i].image_data = (unsigned char *)malloc(xid_fullres[i].width* 
                                                        xid_fullres[i].height);
      if (xid_fullres[i].image_data == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }

      /* Set the window, bands and file name in the load structure.  */
      xid_load.windows[i][SL] = xid_fullres[i].window[SL] = sl[i]; 
      xid_load.windows[i][SS] = xid_fullres[i].window[SS] = ss[i]; 
      xid_load.windows[i][NL] = xid_fullres[i].window[NL] =
                                                         xid_fullres[i].height;
      xid_load.windows[i][NS] = xid_fullres[i].window[NS] = 
                                                          xid_fullres[i].width;
      xid_load.bands[j][i] = xid_planes[plane_idx].band;
      xid_load.dtype[j] = xid_planes[plane_idx].image_info->dtype;
      xid_load.rgb[j][i] = band_ind;
      xid_load.type = xid_planes[plane_idx].image_type;
      strcpy(xid_load.tnames[i],xid_planes[plane_idx].filename);
      if (xid_load.type == LAS)
      {
         status = c_cmhost(xid_planes[plane_idx].filename,xid_load.hnames[i]);
         if (status != E_SUCC)
         {
            xid_message("Error retrieving host file name");
            return;
         }
      }
      else
         strcpy(xid_load.hnames[i],xid_load.tnames[i]);
      band_ind++;
   }
   xid_load.totbnd = xid_load.nimgs = xid.num_fullres;

   /* Display the wait cursor (watch) in both the full resolution image
      window and main panel.  */
   show_wait_cursor();

   /* Set the title for the processing message and processing icon.  */
   XtVaSetValues(load_dialog_shell,XmNtitle,"Load - Processing",NULL);

   /* Load and display the full resolution window.  */
   status = loadimg(XtWindow(fullres_win_bulletinboard),FALSE,FALSE,FALSE);
   if (status != XID_S)
      xid_message("Error loading full resolution image");

   /* Redisplay the default arrow cursor.  */
   unshow_wait_cursor();

   /* Turn on the bell to let the user know the image load is complete.  */
   XBell(xid.display,0);

   /* If the zoom window is currently active and has been calculated from the
      full resolution window, clear the window and free its resources (the
      zoomed information is no longer valid because a new full resolution image
      has been loaded).  */
   if ((xid_zoom.flags & ZOOM_ACTIVE) && (xid_zoom.flags & ZOOM_FULLRES))
   {
      XSetWindowBackground(xid.display,XtWindow(zoom_win_bulletinboard),
                           xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
      XClearWindow(xid.display,XtWindow(zoom_win_bulletinboard));
      if (xid_zoom.image_pixmap != NULL)
      {
         XFreePixmap(xid.display,xid_zoom.image_pixmap);
         xid_zoom.image_pixmap = 0;
      }
      if (xid_zoom.image_data != NULL)
      {
         free(xid_zoom.image_data);
         xid_zoom.image_data = NULL;
      }
   }
}

void fullres_button
(
   Widget w,
   caddr_t client,
   caddr_t call
)

/******************************************************************************
Purpose:
   This routine detects that the user has selected the left mouse button
   in the main image window and calls the appropriate routine to update 
   the full resolution window accordingly.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   XButtonEvent  *event;       /* X Event structure.                         */
   long          i;            /* Looping index.                             */
   long          temp_numfullres; /* Used to save the original number
                                     planes to load since the value needs to
                                     be modified to (re)create all the fullres
                                     flicker pixmaps                         */
   long          plane_idx[3]; /* Index to the plane numbers of the currently
                                  viewed image.                              */
   long          sl[3];        /* Index of starting lines.                   */
   long          ss[3];        /* Index of starting samples.                 */
   long          temp_nplanes; /* Used to save the original number of plane
                                  combinations to flicker since the value
                                  needs to be modified to (re)create all the
                                  fullres flicker pixmaps                    */
   int           xwin;         /* X window coordinate.                       */
   int           ywin;         /* Y window coordinate.                       */
   int           status;       /* routine return status                      */

   event = (XButtonEvent *)call;

   /* Return if the full resolution window is not active or this event was not
      triggered from the main image window.  */
   if ((xid_fullres == NULL) || (w != main_image_bulletinboard))
      return;

   if (xid_fullres[0].flags & FULLRES_USER)
      return;

   /* If button 1 was selected in the main image window, update the full
      resolution window.  */
   if ((w == main_image_bulletinboard) && (event->button == 1) &&
       (xid.view != -1))
   {
      /* Determine the memory plane index.  */
      plane_idx[0] = xid_view.rgbplanes[RED] - 1;
      plane_idx[1] = xid_view.rgbplanes[GREEN] - 1;
      plane_idx[2] = xid_view.rgbplanes[BLUE] - 1;

      /* Erase any previous full resolution outlines.  */
      if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
      {
         xid_file2win(plane_idx[0],0,(float)xid_fullres[0].window[SS],
                      (float)xid_fullres[0].window[SL],&xwin,&ywin);
         XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,
                    ywin-5,xid_fullres[0].width + 10,
                    xid_fullres[0].height + 10,TRUE);
      }
      for (i = 0; i < xid.num_fullres; i++)
      { 
         ss[i] = (((event->x + 1) * xid_planes[(plane_idx[i])].skips) +
                 xid_planes[(plane_idx[i])].window[SS] - 1) -
                 (xid_planes[(plane_idx[i])].skips - 1);
         sl[i] = (((event->y + 1) * xid_planes[(plane_idx[i])].skipl) +
                 xid_planes[(plane_idx[i])].window[SL] - 1) - 
                 (xid_planes[(plane_idx[i])].skipl - 1);
      }

      /* Load the full resolution window.  */
      fullres_update(sl,ss);

      /* Create the full resolution pixmaps if they have not been previously
         created and there is graphics data to display.  */
      if ((xid_graphics.pixmap[0] != NULL) &&
          (xid_graphics.full_pix[0] == NULL))
      {
         /* Create full resolution window pixmaps.  */
         XSetBackground(xid.display,xid_graphics.rb_gc,0);
         XSetForeground(xid.display,xid_graphics.rb_gc,0);

         for (i=0; i < USER_GRCOLORS; i++)
         {
            xid_graphics.full_pix[i] = XCreatePixmap(xid.display,XtWindow(
                                                    fullres_win_bulletinboard),
                                                    xid_fullres[0].width,
                                                    xid_fullres[0].height,1);
            XFillRectangle(xid.display,xid_graphics.full_pix[i],
                           xid_graphics.rb_gc,0,0,xid_fullres[0].width,
                           xid_fullres[0].height);
         }
         XSetBackground(xid.display,xid_graphics.rb_gc,1);
         XSetForeground(xid.display,xid_graphics.rb_gc,1);
      }

      /* Update the graphics information.  */
      if (xid_graphics.pixmap[0] != NULL)
      {
         XSetBackground(xid.display,xid_graphics.rb_gc,0);
         XSetForeground(xid.display,xid_graphics.rb_gc,0);
         for (i=0; i < USER_GRCOLORS; i++)
            XFillRectangle(xid.display,xid_graphics.full_pix[i],
                           xid_graphics.rb_gc,0,0,xid_fullres[0].width,
                           xid_fullres[0].height);
         XSetBackground(xid.display,xid_graphics.rb_gc,1);
         XSetForeground(xid.display,xid_graphics.rb_gc,1);
         graphics_update(1,plane_idx[0]);
      }
   }

   /* If the flicker panel is open and flicker planes have already been
      selected, we need to (re)generate associated fullres flicker pixmaps */
   if((xid_flicker.flags & FLICKER_ACTIVE) && (xid_flicker.pixmaps[0] != NULL))
   {
      temp_nplanes = xid_flicker.nplanes;
      temp_numfullres = xid.num_fullres;
      for(i = 0; i < temp_nplanes; i++)
      {
         xid_flicker.nplanes = i;
         status = fullres_make_flicker_pixmaps();
         if(status != XID_S)
         {
            xid_message("Error encountered creating the full resolution"
                        " flicker pixmaps.");
            return;
         }
      }
      xid_flicker.nplanes = temp_nplanes;
      xid.num_fullres = temp_numfullres;
   }
}

void fullres_setup
(void)

/******************************************************************************
Purpose:
   This routine determines the number of full resolution bands to load, 
   allocates the full resolution structures and initializes the appropriate
   variables and flags.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   XGCValues       values;            /* Values array for the GC.            */
   unsigned long   valuemask = 0;     /* Specify no values.                  */
   long            i;                 /* Looping index.                      */

   /* Determine the number of full resolution bands to be loaded.  */
   if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) && 
       (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
      xid.num_fullres = 1;
   else
      xid.num_fullres = 3;

   /* Allocate memory for the full resolution structure.  */
   if (xid_fullres == NULL)
   {
      xid_fullres = (struct XID_FULLRES *) calloc(xid.num_fullres,
                    sizeof(struct XID_FULLRES));
      if (xid_fullres == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
   }

   /* Set appopriate variables and flags.  */
   for (i=0; i < xid.num_fullres; i++)
   {
      xid_fullres[i].flags = (FULLRES_ACTIVE | FULLRES_SHOW); 
      xid_fullres[i].image_data = NULL;
      xid_fullres[i].image_pixmap = 0;
      xid_fullres[i].gc = NULL;
      if (xid.visual_info[0].depth == 24)
         xid_fullres[i].rb_fore = 0xffffff;
      else
         xid_fullres[i].rb_fore = 0xff;
      xid_fullres[i].window[SL] = xid_fullres[i].window[SS] = -1;
   }

   /* Set initial state of "show outline" button.  */
   XmToggleButtonSetState(fullres_outline_toggle,True,False);

   /* Create the GC.  */
   if (xid_fullres[0].gc == NULL)
   {
      xid_fullres[0].gc = XCreateGC(xid.display,XtWindow(
                                    main_image_bulletinboard),valuemask,
                                    &values);
      XSetLineAttributes(xid.display,xid_fullres[0].gc,2,LineSolid,CapButt,
                         JoinMiter);
      XSetForeground(xid.display,xid_fullres[0].gc,xid_fullres[0].rb_fore);
      XSetFunction(xid.display,xid_fullres[0].gc,GXxor);
   }
}

void fullres_drag
(
   Widget w,
   XtPointer client,
   XEvent *call,
   Boolean *cont
)

/******************************************************************************
Purpose:
   Based on the cursor position, this routine draws a rubber-band rectangle
   outlining the currently selected full resolution area. 

By L. Huewe, USGS/EROS Data Center, 1/93

Algorithm Description:
******************************************************************************/
{
   XEvent  *event;             /* X event structure defining input event.    */
   long    height;             /* Rectangle height.                          */
   long    i;                  /* Looping index.                             */
   long    plane_idx[3];       /* Index into memory plane structure.         */
   long    sl[3];              /* Starting lines of window to load.          */
   long    ss[3];              /* Starting lines of window to load.          */
   long    width;              /* Rectangle width.                           */
   int     xwin;               /* X window coordinate.                       */
   int     ywin;               /* Y window coordinate.                       */
   char    context_name[ELEMENT_MAX_LENGTH];
                               /* Current active context name.               */

   event = (XEvent *)call;

   if (event->type == MotionNotify)
   {
      /* This statement is necessary until all DG systems are upgraded to dgux
         5.4.2.  Versions previous to this seem to not detect the event
         Button1MotionMask correctly.  The event handler gets fired for any
         motion event not just when button 1 is pressed.  */
      if (event->xmotion.state != Button1Mask)
         return;

      /* If the current context is not the "Image" context, do nothing.  */
      XeContextGetActive((XeContextWidget)context_mouse,context_name);
      if (strcmp(context_name,context_names[CON_IMAGE]) != 0)
         return;

      /* Erase the previous rubber-band rectangle.  */
      if ((xid_fullres[0].ulx != -1) && (xid_fullres[0].prevx != -1))
      {
         width = xid_fullres[0].prevx - xid_fullres[0].ulx;
         height = xid_fullres[0].prevy - xid_fullres[0].uly;
         if ((width > 0) && (height > 0))
            XDrawRectangle(xid.display,XtWindow(w),xid_fullres[0].gc,
                           xid_fullres[0].ulx,xid_fullres[0].uly,width,height);
      }
      if (xid_fullres[0].ulx == -1)
      {
         xid_fullres[0].ulx = event->xmotion.x;
         xid_fullres[0].uly = event->xmotion.y;
      }

      /* Draw the new rubber band rectangle.  */
      if (xid_fullres[0].ulx != -1)
      {
         width = event->xmotion.x - xid_fullres[0].ulx;
         height = event->xmotion.y - xid_fullres[0].uly;
         if ((width > 0) && (height > 0))
         {
            XDrawRectangle(xid.display,XtWindow(w),xid_fullres[0].gc,
                           xid_fullres[0].ulx,xid_fullres[0].uly,width,height);
            xid_fullres[0].prevx = event->xmotion.x;
            xid_fullres[0].prevy = event->xmotion.y;
         }
         else
            xid_fullres[0].prevx = xid_fullres[0].prevy = -1;
      }
   }
   else if (event->type == ButtonRelease)
   {
      /* If the current context is not the "Image" context, do nothing.  */
      XeContextGetActive((XeContextWidget)context_mouse,context_name);
      if (strcmp(context_name,context_names[CON_IMAGE]) != 0)
         return;

      /* Determine the memory plane index.  */
      plane_idx[0] = xid_view.rgbplanes[RED] - 1;
      plane_idx[1] = xid_view.rgbplanes[GREEN] - 1;
      plane_idx[2] = xid_view.rgbplanes[BLUE] - 1;

      /* Erase any previous full resolution outlines.  */
      if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
      {
         for (i = 0; i < xid.num_fullres; i++)
         {
            xid_file2win(plane_idx[i],0,(float)xid_fullres[0].window[SS],
                         (float)xid_fullres[0].window[SL],&xwin,&ywin);
            XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,
                       ywin-5,xid_fullres[0].width + 10,
                       xid_fullres[0].height + 10,TRUE);
         }
      }

      /* Erase the rubber-band rectangle.  */
      if ((xid_fullres[0].ulx != -1) && (xid_fullres[0].prevx != -1))
      {
         width = xid_fullres[0].prevx - xid_fullres[0].ulx;
         height = xid_fullres[0].prevy - xid_fullres[0].uly;
         if ((width > 0) && (height > 0))
            XDrawRectangle(xid.display,XtWindow(w),xid_fullres[0].gc,
                           xid_fullres[0].ulx,xid_fullres[0].uly,width,height);
      }
      if ((xid.view != -1) && (xid_fullres[0].ulx != -1))
      {
         if ((((xid_fullres[0].prevy - xid_fullres[0].uly + 1) * 
             xid_planes[(plane_idx[0])].skipl) < 0) || 
             (((xid_fullres[0].prevx - xid_fullres[0].ulx + 1) * 
             xid_planes[(plane_idx[0])].skips) < 0))
         {
            xid_message("Invalid width or height");
            xid_fullres[0].ulx = xid_fullres[0].uly = 
                              xid_fullres[0].prevx = xid_fullres[0].prevy = -1;
            return;
         }
         for ( i = 0; i < xid.num_fullres; i++)
         {
            ss[i] = (((xid_fullres[0].ulx + 1) * 
                     xid_planes[(plane_idx[i])].skips) +
                     xid_planes[(plane_idx[i])].window[SS] - 1) - 
                     (xid_planes[(plane_idx[i])].skips - 1);
            sl[i] = (((xid_fullres[0].uly + 1) * 
                     xid_planes[(plane_idx[i])].skipl) +
                     xid_planes[(plane_idx[i])].window[SL] - 1) - 
                     (xid_planes[(plane_idx[i])].skipl - 1);
            xid_fullres[0].height = (xid_fullres[0].prevy - 
                    xid_fullres[0].uly + 1) * xid_planes[(plane_idx[i])].skipl;
            xid_fullres[0].width = (xid_fullres[0].prevx - 
                    xid_fullres[0].ulx + 1) * xid_planes[(plane_idx[i])].skips;
            xid_fullres[i].height = xid_fullres[0].height; 
            xid_fullres[i].width = xid_fullres[0].width; 
         }

         /* Load the full resolution window.  */
         fullres_update(sl,ss);

         /* Free and recreate the full resolution pixmaps if the graphics
            window is active or graphics data is displayed.  */
         if ((xid_graphics.flags & GR_ACTIVE) || (xid_graphics.full_pix[0] != 
             NULL))
         {
            if (xid_graphics.full_pix[0] != NULL)
            {
               for (i=0; i < USER_GRCOLORS; i++)
                  XFreePixmap(xid.display,xid_graphics.full_pix[i]);
            }

            /* Create full resolution window pixmaps.  */
            XSetBackground(xid.display,xid_graphics.rb_gc,0);
            XSetForeground(xid.display,xid_graphics.rb_gc,0);

            for (i=0; i < USER_GRCOLORS; i++)
            {
               xid_graphics.full_pix[i] = XCreatePixmap(xid.display,XtWindow(
                                                    fullres_win_bulletinboard),
                                                    xid_fullres[0].width,
                                                    xid_fullres[0].height,1);
               XFillRectangle(xid.display,xid_graphics.full_pix[i],
                              xid_graphics.rb_gc,0,0,xid_fullres[0].width,
                              xid_fullres[0].height);
            }
            XSetBackground(xid.display,xid_graphics.rb_gc,1);
            XSetForeground(xid.display,xid_graphics.rb_gc,1);
         }

         /* Update the graphics information.  */
         if (xid_graphics.pixmap[0] != NULL)
         {
            XSetBackground(xid.display,xid_graphics.rb_gc,0);
            XSetForeground(xid.display,xid_graphics.rb_gc,0);
            for (i=0; i < USER_GRCOLORS; i++)
            {
               XFillRectangle(xid.display,xid_graphics.full_pix[i],
                              xid_graphics.rb_gc,0,0,xid_fullres[0].width,
                              xid_fullres[0].height);
            }
            XSetBackground(xid.display,xid_graphics.rb_gc,1);
            XSetForeground(xid.display,xid_graphics.rb_gc,1);
            graphics_update(1,plane_idx[0]);
         }
      }
      xid_fullres[0].ulx = xid_fullres[0].uly = xid_fullres[0].prevx =
                                                     xid_fullres[0].prevy = -1;
   }
}

void fullres_outline
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine sets the appropriate flag indicating whether or not the user
   wishes to see an outline in the main image window of the full resolution
   area.

By L. Huewe, USGS/EROS Data Center, 01/93

Algorithm Description:
******************************************************************************/
{
   XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call;

   if (acs->set)
      xid_fullres[0].flags |= FULLRES_SHOW;
   else
      xid_fullres[0].flags &= ~FULLRES_SHOW;
}

int fullres_make_flicker_pixmaps
(void)

/******************************************************************************
Purpose:
   This routine creates the fullres flicker pixmaps.

By A. Shimitz, USGS/EROS Data Center, 01/01
******************************************************************************/
{
   long           i,j;             /* Looping indices.                       */
   long           nplanes;         /* Number of planes (bands) to display.   */
   long           plane_idx;       /* Current index into memory plane struct */
   long           status;          /* Status indicator                       */

   /* Call loadimg to display the appropriate image data in the appropriate
      RGB order in the full res window.  */
   if((xid_fullres != NULL) && (xid_fullres[0].image_data != NULL) &&
      (xid.view != -1))
   {
      /* Reset the num_fullres since flickering could have changed this value */
      nplanes = 1;
      for(i = 1; i < 3; i++)
      {
         for(j = 0; j < i; j++)
         {
            if(xid_flicker.planes[xid_flicker.nplanes][j] !=
               xid_flicker.planes[xid_flicker.nplanes][i])
            {
               nplanes = 3;
               break;
            }
         }
         if(nplanes == 3)
            break;
      }
      if(nplanes == 1)
         xid.num_fullres = 1;
      else
         xid.num_fullres = 3;

      /* Initialize each band */
      for(j = 0; j < 3; j++)
      {
         for(i = 0; i < 3; i++)
            xid_load.bands[j][i] = 0;
      }
   
      /* For each full resolution band to be loaded ... Here j represents the
         image number */
      for (i = 0; i < nplanes; i++)
      {
         /* Determine the memory plane index.  */
         plane_idx = xid_flicker.planes[xid_flicker.nplanes][i] - 1;
   
         /* Determine the proper image number */
         for(j = 0; j < i; j++)
         {
            if(strcmp(xid_planes[xid_flicker.
                         planes[xid_flicker.nplanes][j] - 1].filename,
                      xid_planes[plane_idx].filename) == 0)
               break;
         }
   
         /* Free any previously allocated memory and allocate the amount needed
            for the new window to be loaded.  */
         if (xid_fullres[i].image_data != NULL)
         {
            free(xid_fullres[i].image_data);
            xid_fullres[i].image_data = NULL;
         }
         xid_fullres[i].image_data = (unsigned char *)malloc(xid_fullres[i].
                                     width * xid_fullres[i].height);
         if (xid_fullres[i].image_data == NULL)
         {
            xid_message("Insufficient memory available");
            return (XID_F);
         }
   
         /* Set the window, bands and file name in the load structure.  */
         xid_load.windows[i][SL] = xid_fullres[i].window[SL];
         xid_load.windows[i][SS] = xid_fullres[i].window[SS];
         xid_load.windows[i][NL] = xid_fullres[i].window[NL] =
                                                         xid_fullres[i].height;
         xid_load.windows[i][NS] = xid_fullres[i].window[NS] = 
                                                          xid_fullres[i].width;
         xid_load.bands[j][i] = xid_planes[plane_idx].band;
         xid_load.type = xid_planes[plane_idx].image_type;
         strcpy(xid_load.tnames[i],xid_planes[plane_idx].filename);
         if (xid_load.type == LAS)
         {
            status = c_cmhost(xid_planes[plane_idx].filename,
                              xid_load.hnames[i]);
            if (status != E_SUCC)
            {
               xid_message("Error retrieving host file name");
               return(XID_F);
            }
         }
         else
            strcpy(xid_load.hnames[i],xid_load.tnames[i]);
      }
      xid_load.totbnd = xid_load.nimgs = nplanes;
   
      /* Free the fullres flicker pixmap if necessary before
         creating a new one */
      if(xid_fullres[0].flicker_pixmaps[xid_flicker.nplanes] != NULL)
      {
         XFreePixmap(xid.display,
                     xid_fullres[0].flicker_pixmaps[xid_flicker.nplanes]);
         xid_fullres[0].flicker_pixmaps[xid_flicker.nplanes] = NULL;
      }

      /* Load the full resolution window.  */
      status = loadimg(XtWindow(fullres_win_bulletinboard),FALSE,TRUE,TRUE);
      if (status != XID_S)
      {
         xid_message("Error loading full resolution image");
      }
   }
   return (XID_S);
}
