/*
 *  This is the histo function source file for the histo panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 *  Modified by A. Shimitz 8/00
 *   - Added functionality to support the Mapped button on the histogram panel
 *     (CR #3892).  This functionality includes the histo_up_map_data() routine
 *     which parses through an XImage of the currently displayed image and 
 *     calculates a histogram and histogram statistics.  The histo_draw()
 *     routine was modified to support drawing this mapped histogram data too.
 */
#include <ctype.h>
#include "pixman.h"
#include "xid.h"
#include "xid_func.h"
#include "typlim.h"

/* local function prototypes */
void histo_z
(
   Widget w,
   XtPointer client,
   XEvent *call,
   Boolean *cont
);

int histo_up_map_data
(void);
/* end prototypes */

void histo_create
(
   Widget w,
   XtPointer client,
   XEvent *call,
   Boolean *cont
)
/******************************************************************************
Purpose:
   This routine is used to create the GC and set the appropriate default
   flags for the histogram window.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   XEvent           *event;            /* X Event structure.                 */
   Window           histo_win;         /* Histogram window ID.               */
   XGCValues        values;            /* Values array for the GC.           */
   unsigned long    valuemask = 0;     /* Specify no values.                 */

   event = (XEvent *)call;
   if (event->type == MapNotify)
   {
      /* Create the working GC for the histogram window.  */
      histo_win = XtWindow(histo_form);
      xid_histo.gc = XCreateGC(xid.display,histo_win,valuemask,&values);

      XSetForeground(xid.display,xid_histo.gc,
                     xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
      XSetBackground(xid.display,xid_histo.gc,
                     xid.color_table[RGBBW_IDX + WHITE_COL].pixel);
      XSetFont(xid.display,xid_histo.gc,xid.font_info->fid);

      /* Set the appropriate histogram flags.  */
      xid_histo.flags |= HISTO_ACTIVE;
   }
}

void histo_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)
/******************************************************************************
Purpose:
   This routine is used to redraw any image data destroyed by the horizontal
   and/or vertical cursor tracking line.  The appropriate event handlers are
   removed and the histogram window is closed.  The main panel button is
   then set to sensitive.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   long    plane_idx;          /* Memory plane index.                        */

   /* Remove any event handlers no longer needed.  */
   if (!(xid_histo.flags & HISTO_XY))
   {
      XtRemoveEventHandler(main_image_bulletinboard,PointerMotionMask,
                           FALSE,histo_z,NULL);
   }

   /* Determine which (if any) tracking line was displayed.  */
   if (xid_histo.flags & HISTO_XZ)
      xid.num_horiz--;
   else if (xid_histo.flags & HISTO_YZ)
      xid.num_vert--;

   /* Restore any image data written over by the vertical or horizontal cursor
      tracking line if it is no longer needed.  */
   plane_idx = xid_view.rgbplanes[RED] - 1;

   if ((xid.num_vert == 0) && (xid.line_x != -1))
   {
      XClearArea(xid.display,XtWindow(main_image_bulletinboard),xid.line_x,0,1,
                 xid_planes[plane_idx].dspwin[NL],True);
      xid.line_x = -1;
      xid.flags = xid.flags & (~XID_VERT);
   }
   if ((xid.num_horiz == 0) && (xid.line_y != -1))
   {
      XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,xid.line_y,
                 xid_planes[plane_idx].dspwin[NS],1,True);
      xid.line_y = -1;
      xid.flags = xid.flags & (~XID_HORIZ);
   }
   if ((xid.num_horiz == 0) && (xid.num_vert == 0) && (xid.line_gc != NULL))
   {
      XFreeGC(xid.display,xid.line_gc);
      xid.line_gc = (GC)NULL;
   }

   /* Free the histogram graphics context.  Set it to null so the histo_xyz
      function will not attempt to draw the histogram immediately before
      the histo_create routine has been executed. */
   XFreeGC(xid.display,xid_histo.gc);
   xid_histo.gc = (GC)NULL;

   /* Close the histogram window and set the histogram button on the main panel
      to sensitive.  The widget is no longer unrealized so the init_map_histo
      routine in xid.c c could reset the default selections on the panel
      before if it is not currently being viewed. */
   XtUnmanageChild(Histo_form);
   XtPopdown(Shell_Histo);
   XtSetSensitive(main_histo_button,True);

   /* Reset the active flag.  */
   xid_histo.flags = xid_histo.flags ^ HISTO_ACTIVE;
}

void histo_expose
(
   Widget w,
   XtPointer client,
   XEvent *call,
   Boolean *cont
)
/******************************************************************************
Purpose:
   This routine is used to redraw the histogram window after an expose event. 
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   /* Redraw the histogram window.  */
   if(xid_histo.gc != NULL)
      histo_draw();
   else
      xid_message("Error drawing histogram panel");
}

void histo_ilp
(
   Widget w,
   XtPointer client,
   XtPointer call
)
/******************************************************************************
Purpose:
   This routine sets the appropriate flag indicating the object the user
   wishes to histogram (i.e. line, polygon or image).

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   char *data = (char *)client;

   /* Set the flag indicating which object to histogram.  */
   xid_histo.flags = xid_histo.flags&~(HISTO_IMAGE | HISTO_LINE | HISTO_POLY);
   switch (*data)
   {
      case 'I':
         xid_histo.flags = xid_histo.flags | HISTO_IMAGE;
         break;
      case 'L':
         xid_histo.flags = xid_histo.flags | HISTO_LINE;
         break;
      case 'P':
         xid_histo.flags = xid_histo.flags | HISTO_POLY;
         break;
   }
}

void histo_mo
(
   Widget w,
   XtPointer client,
   XtPointer call
)
/******************************************************************************
Purpose:
   This routine sets the appropriate flag indicating whether the user wishes
   to histogram the original image data or the mapped image data.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   char *data = (char *)client;

   /* Set the flag indicating which type of data to histogram.  */
   xid_histo.flags = xid_histo.flags & ~(HISTO_MAPPED | HISTO_ORIG);
   if (*data == 'O')
      xid_histo.flags = xid_histo.flags | HISTO_ORIG;
   else
      xid_histo.flags = xid_histo.flags | HISTO_MAPPED;

   /* Updated the histogram.  */
   histo_update(w, client, call);
}

void histo_type
(
   Widget w,
   XtPointer client,
   XtPointer call
)
/******************************************************************************
Purpose:
   This routine sets the appropriate flag indicating whether the user has
   selected a distributional or cumulative histogram. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   char *data = (char *)client;

   /* Set the flag indicating the type of histogram to draw.  */
   xid_histo.flags = xid_histo.flags & ~(HISTO_DIST | HISTO_CUM);
   if (*data == 'D')
      xid_histo.flags = xid_histo.flags | HISTO_DIST;
   else
      xid_histo.flags = xid_histo.flags | HISTO_CUM;

   /* Updated the histogram.  */
   histo_update(w, client, call);
}

void histo_update
(
   Widget w,
   XtPointer client,
   XtPointer call
)
/******************************************************************************
Purpose:
   This routine parses the specified mask values and draws the histogram. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   long    i;                  /* Looping index.                             */
   long    nonwhite = FALSE;   /* Indicates whether or not nonwhite space 
                                  was found in mask value string.            */
   long    status;             /* Status indicator.                          */
   char    *maskvals;          /* Pointer to mask value string.              */

   /* Determine whether or not any mask values were specified.  */
   maskvals = XmTextGetString(histo_mask_text);
   for (i=0; i < strlen(maskvals); i++)
   {
      if (isspace((int)maskvals[i]) == 0)
         nonwhite = TRUE;
   }
   if (!nonwhite)
      xid_histo.rcount = xid_histo.bcount = 0;
   else
   {
      /* Parse the specified mask values into ranges and/or individual bins.  */
      if (xid_histo.range != NULL)
      {
         free(xid_histo.range);
         xid_histo.range = NULL;
      }
      if (xid_histo.bin != NULL)
      {
         free(xid_histo.bin);
         xid_histo.bin = NULL;
      }
      status = parse_rb(maskvals,&xid_histo.range,&xid_histo.rcount,
                        &xid_histo.bin,&xid_histo.bcount,MINUTINY,MAXUTINY);
      if (status != XID_S)
         xid_histo.rcount = xid_histo.bcount = 0;
   }

   /* Draw the updated histogram.  */
   XClearWindow(xid.display,XtWindow(histo_drawingarea));
   histo_draw();

   /* Free resources */
   XtFree(maskvals);
}

void histo_xyz
(
   Widget w,
   XtPointer client,
   XtPointer call
)
/******************************************************************************
Purpose:
   This routine sets the appropriate flag indicating which dimension the
   histogram is to be calculated in and adds the appropriate event handlers.  
   It then determines whether the cursor tracking line is on and/or needs
   to be turned on/off.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   XGCValues       values;           /* Values array for the GC.             */
   unsigned long   valuemask = 0;    /* Specify no values.                   */
   long            plane_idx;        /* Index to memory plane structure.     */

   /* Determine whether or not a tracking line is currently in use.  */
   if (xid_histo.flags & HISTO_XZ)
      xid.num_horiz--;
   else if (xid_histo.flags & HISTO_YZ)
      xid.num_vert--;

   /* Set the flag indicating the dimension to histogram in.  */
   xid_histo.flags = xid_histo.flags & ~(HISTO_XY | HISTO_XZ | HISTO_YZ);
   if (strcmp(client,"XY") == 0)
      xid_histo.flags = xid_histo.flags | HISTO_XY;
   else if (strcmp(client,"XZ") == 0)
      xid_histo.flags = xid_histo.flags | HISTO_XZ;
   else if (strcmp(client,"YZ") == 0)
      xid_histo.flags = xid_histo.flags | HISTO_YZ;

   plane_idx = xid_view.rgbplanes[RED] - 1;
   if (!(xid_histo.flags & HISTO_XY))
   {
      /* Add the appropriate event handler for a Z dimension histogram.  */
      XtAddEventHandler(main_image_bulletinboard,PointerMotionMask,
                        FALSE,histo_z,NULL);
      if (xid_histo.flags & HISTO_XZ)
         xid.num_horiz++;
      else if (xid_histo.flags & HISTO_YZ)
         xid.num_vert++;

      /* Create a GC for the cursor tracking line.  */
      if (xid.line_gc == NULL)
         xid.line_gc = XCreateGC(xid.display,XtWindow(
                                 main_image_bulletinboard),valuemask,&values);
      XSetForeground(xid.display,xid.line_gc,
                     xid.color_table[RGBBW_IDX + WHITE_COL].pixel);
   }
   else
   {
      /* Remove the Z dimension event handler.  */
      XtRemoveEventHandler(main_image_bulletinboard,PointerMotionMask,
                           FALSE,histo_z,NULL);
   }

   /* If a tracking line is no longer needed, restore the image data.  */
   if ((xid.num_vert == 0) && (xid.line_x != -1)) /* Y/Z (vertical) line */
   {
      XClearArea(xid.display,XtWindow(main_image_bulletinboard),xid.line_x,0,1,
                 xid_planes[plane_idx].dspwin[NL],True);
      xid.line_x = -1;
      xid.flags = xid.flags & (~XID_VERT);
   } 
   if ((xid.num_horiz == 0) && (xid.line_y != -1)) /* X/Z (horizontal) line */
   {
      XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,xid.line_y,
                 xid_planes[plane_idx].dspwin[NS],1,True);
      xid.line_y = -1;
      xid.flags = xid.flags & (~XID_HORIZ);
   }
   if ((xid.num_horiz == 0) && (xid.num_vert == 0) && (xid.line_gc != NULL))
   {
      XFreeGC(xid.display,xid.line_gc);
      xid.line_gc = (GC)NULL;
   }

   /* Redraw the updated histogram if this is not the initial load of the
      histogram panel.  */
   if(xid_histo.gc != NULL)
   {
      XClearWindow(xid.display,XtWindow(histo_drawingarea));
      histo_draw();
   }
}


#define SMALLX 100      /* Smallest acceptable x axis.                  */
#define SMALLY 25       /* Smallest acceptable y axis.                  */

#define DSPANNOT 25     /* X coordinate of the Y axis (arbitrary).      */


int histo_draw
(void)
/******************************************************************************
Purpose:
   This routine is used to draw the appropriate number of cumulative or
   distributional histograms (typically 1 for black and white images or 
   3 for color images).  Any values specified by the user to be masked out
   are not plotted on the histogram.  The histogram is annotated with
   the mean, standard deviation, min and max pixel values.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Modified by A. Shimitz 11/00
 - Now drawing separate R,G,B histogram graphs for the z-dimensional histograms
   as well as the X/Y histogram.  Needed to modify the procedure to determine
   the number of bands to histogram.  Also modified the assignment of the
   histptr to the z-dimensional histogram data to point at each band now
   instead of one cumulative data array.

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
******************************************************************************/
{
   Window  histo_win;        /* Histogram window ID.                         */
   long    c_width;          /* Calculated font character width.             */
   long    c_height;         /* Calculated font character height.            */
   long    *histptr;         /* Pointer to histogram array.                  */
   int     yoffset;          /* Location of x axis on screen.                */
   int     uscreen;          /* Useable part of screen.                      */
   int     pscreen;          /* Part of screen for a single component.       */
   int     height;           /* Height of the yaxis for single component.    */
   int     length;           /* Length of the xaxis.                         */
   int     border;           /* Area between axis and border of screen.      */
   int     yaxis;            /* A screen location on yaxis.                  */
   int     xaxis;            /* A screen location on the xaxis.              */
   int     xshift;           /* Shift x axis for annotation,
                                where xshift is the maximum number
                                of characters between the left
                                margin and the y axis.                       */
   int     freq;             /* Frequency.                                   */
   int     x1;               /* Screen x location.                           */
   int     x2;               /* Screen x location.                           */
   int     y1;               /* Screen y location.                           */
   int     y2;               /* Screen y location.                           */
   int     comp;             /* Color component.                             */
   int     i,j;              /* Variable for loop.                           */
   int     xoff;             /* X and Y offsets for the histogram axis.      */
   int     yoff;
   int     ymax;             /* Used to label the y axis of the histogram.   */
   int     slen;             /* Annotation length for annot.                 */
   int     totbnd;           /* Total number of bands to histogram.          */
   int     bnds[3];          /* Bands to histogram.                          */
   int     mskflag = FALSE;  /* Flag indicating whether mask values were
                                applied to the histogram data.               */
   int     zflag = FALSE;    /* Flag indicating histogram in z dimension.    */
   int     plane_idx[3];     /* Indices into memory plane structure of the
                                currently viewed image.                      */
   int     nplanes;          /* Number of planes currently displayed.        */
   float   maxp = 0.0;       /* Maximum histogram value.                     */
   float   lnmx = 255;       /* Maximum length for a histogram.              */
   char    annot[50];        /* Annotation for histogram.                    */
   char    stdeviat[20];     /* Annotation for standard deviation.           */
   char    meann[20];        /* Annotation for the mean.                     */
   char    minstr[20];       /* Annotation for the min band value.           */
   char    maxstr[20];       /* Annotation for the max band value.           */
   long    no_data;          /* Indicates whether there is valid data in
                                any of the memory planes                     */

   /* Print a warning and return if no image data is currently displayed.  */
   no_data = TRUE;
   for(i = 0; i < xid.max_plane; i++)
   {
      if(xid_planes[i].flags & USED)
         no_data = FALSE;
   }
   if ((xid.view == -1) || no_data)
   {
      xid_message("No image data currently being viewed");
      return(XID_S);
   }

   /* Update the appropriate histogram data and reset the flag, if neccessary*/
   if(((xid_histo.flags & HISTO_MAPPED) && xid.mapped && (xid.mapped_mod ||
      xid.mapped_z_mod)) || ((xid_histo.flags & HISTO_MAPPED) && !xid.mapped &&
      ((xid_histo.flags & HISTO_XZ) || (xid_histo.flags & HISTO_YZ))))
   {
      /* Viewing the mapped histogram reflecting the currently applied mapping.
         Histogram the appropriate row/column from the currently displayed
         image if X/Z or Y/Z is selected (whether a mapping is applied or not).
         */
      if(histo_up_map_data() != XID_S)
         return(XID_F);
   }
   else if ((xid_histo.flags & HISTO_ORIG) || ((xid_histo.flags & HISTO_MAPPED)
            && !xid.mapped && (xid_histo.flags & HISTO_XY)))
   {
      /* Viewing the original histogram or mapped X/Y histogram but no mapping
         has been applied.  Use the original histogram data. */

      /* Find the memory plane indices of the displayed image. */
      plane_idx[RED] = xid_view.rgbplanes[RED] - 1;
      plane_idx[GREEN] = xid_view.rgbplanes[GREEN] - 1;
      plane_idx[BLUE] = xid_view.rgbplanes[BLUE] - 1;

      /* Determine the number of planes displayed */
      if ((plane_idx[RED] == plane_idx[GREEN]) &&
          (plane_idx[GREEN] == plane_idx[BLUE]))
         nplanes = 1;
      else
         nplanes = 3;

      /* Recalculate the histogram statistics. */
      for(i = 0; i < nplanes; i++)
      {
         xid_histcalc(xid_planes[plane_idx[i]].histogram, 
                      &xid_planes[plane_idx[i]].minval,
                      &xid_planes[plane_idx[i]].maxval, 
                      &xid_planes[plane_idx[i]].mean,
                      &xid_planes[plane_idx[i]].stdev);
      }
   }

   /* Get the window ID of the histogram window.  */
   histo_win = XtWindow(histo_drawingarea);

   /* Set the zflag if doing an X/Z or Y/Z histogram */
   if ((xid_histo.flags & HISTO_XZ) || (xid_histo.flags & HISTO_YZ))
      zflag = TRUE;

   /* Calculate the number of bands to histogram.  */
   if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) && 
       (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
   {
      bnds[0] = xid_view.rgbplanes[RED];
      totbnd = 1;
   }
   else
   {
      for (i=0, totbnd=0; i < 3; i++)
      {
         if (xid_view.rgbplanes[i] != 0)
         {
            bnds[totbnd] = xid_view.rgbplanes[i];
            totbnd++;
         }
      }
   }

   /* Calculate the width and height of characters in the working font.  */
   c_width = xid.font_info->max_bounds.rbearing - 
                xid.font_info->min_bounds.lbearing;
   c_height = xid.font_info->max_bounds.ascent + 
                 xid.font_info->max_bounds.descent;

   /* Assign the working window location offsets.  */
   xshift = c_width * DSPANNOT;    
   xoff = (1 * c_width);
   yoff = 0;

   /* Set the size variables.  */
   xid_histo.xoffset = xoff += xshift;
   uscreen = HISTY - yoff;
   pscreen = uscreen / totbnd;
   height = .70 * pscreen;
   xid_histo.xlength = length = ((HISTX - xoff) > lnmx) ? lnmx : HISTX - xoff;
   border = (pscreen - height) / 2;

   if (totbnd == 1)
   {
      /* Throughout the routine we need to set the histptr to the mapped
         data when appropriate, using the xid_planes[].histo_map data for an
         X/Y histo and the xid_histo.z_histo_map data for an X/Z or Y/Z histo*/
      if (!zflag)
      {
         if((xid_histo.flags & HISTO_MAPPED) && xid.mapped)
            histptr = xid_planes[bnds[0]-1].histo_map;
         else
            histptr = xid_planes[bnds[0]-1].histogram;
      }
      else
      {
         if(xid_histo.flags & HISTO_MAPPED)
            histptr = xid_histo.z_histo_map[bnds[0] - 1];
         else
            histptr = xid_histo.z_histo[bnds[0] - 1];
      }

      /* Mask out the specified values.  */
      if (xid_histo.bcount != 0)
      {
         for (i=0; i < xid_histo.bcount; i++)
            *(histptr + xid_histo.bin[i]) = -(*(histptr + xid_histo.bin[i]));
      }
      if (xid_histo.rcount != 0)
      {
         for (i=0; i < xid_histo.rcount; i++)
         {
            for (j=xid_histo.range[i*2]; j < xid_histo.range[i*2+1] + 1; j++)
               *(histptr + j) = -(*(histptr + j));
         }
      }

      /* Calculate the cumulative histogram.  This is done regardless of the
         option chosen because the cumulative histogram is used in percentage
         calculations.  */
      for (freq=0; freq < 256; freq++)
      {
         if (*(histptr + freq) > 0)
         {
            xid_histo.cum_histo[0][freq] = *(histptr + freq);
            break;
         }
         else
            xid_histo.cum_histo[0][freq] = 0;
      }
      for (freq=freq+1; freq < 256; freq++)
      {
         if (*(histptr + freq) >= 0)
            xid_histo.cum_histo[0][freq] = *(histptr + freq) + 
                                              xid_histo.cum_histo[0][freq-1]; 
         else 
            xid_histo.cum_histo[0][freq] = xid_histo.cum_histo[0][freq-1];
      }

      /* Find maximum value.  */
      if (xid_histo.flags & HISTO_CUM)
         maxp = (float)xid_histo.cum_histo[0][255];
      if (xid_histo.flags & HISTO_DIST)
      {
         for (freq = 0; freq <= 255; histptr++, freq++)
            if (maxp < (float)*histptr) maxp = (float) *histptr;
      }
   }
   else
   {
      for (comp = 0; comp < totbnd; comp++) /* 1 or 3 */
      {
         if(!zflag)
         {
            if((xid_histo.flags & HISTO_MAPPED) && xid.mapped)
               histptr = xid_planes[bnds[comp]-1].histo_map;
            else
               histptr = xid_planes[bnds[comp]-1].histogram;
         }
         else
         {
            if(xid_histo.flags & HISTO_MAPPED)
               histptr = xid_histo.z_histo_map[bnds[comp] - 1];
            else
               histptr = xid_histo.z_histo[bnds[comp] - 1];
         }

         /* Mask out the specified values.  */
         if (xid_histo.bcount != 0)
         {
            for (i=0; i < xid_histo.bcount; i++)
               *(histptr + xid_histo.bin[i]) = -(*(histptr+xid_histo.bin[i]));
          }
          if (xid_histo.rcount != 0)
          {
             for (i=0; i < xid_histo.rcount; i++)
             {
                for (j=xid_histo.range[i*2]; j < xid_histo.range[i*2+1] + 1; 
                     j++)
                   *(histptr + j) = -(*(histptr + j));
             }
          }

         /* Calculate the cumulative histogram.  This is done regardless of the
            option chosen because the cumulative histogram is used in
            percentage calculations.  */
         for (freq=0; freq < 256; freq++)
         {
            if (*(histptr + freq) > 0)
            {
               xid_histo.cum_histo[comp][freq] = *(histptr + freq);
               break;
            }
            else
               xid_histo.cum_histo[comp][freq] = 0;
         }
         for (freq=freq+1; freq < 256; freq++)
         {
            if (*(histptr + freq) >= 0)
               xid_histo.cum_histo[comp][freq] = *(histptr + freq) + 
                                             xid_histo.cum_histo[comp][freq-1]; 
            else 
               xid_histo.cum_histo[comp][freq] = 
                                             xid_histo.cum_histo[comp][freq-1];
         }

         /* Find maximum value.  */
         if (xid_histo.flags & HISTO_CUM)
            maxp = (float)xid_histo.cum_histo[comp][255]; 
         if (xid_histo.flags & HISTO_DIST)
         {
            for (freq = 0; freq <= 255; histptr++, freq++)
            {
               if (maxp < (float)*histptr)
                  maxp = (float) *histptr;
            }
         }
      }
   }

   /* Check for a zero image.  */
   if (maxp == 0) maxp = 1;

   /* Draw the histograms.  */
   for (comp = 0; comp < totbnd; comp++)
   {
      /* Draw the X and Y axis.  */
      yoffset = yoff + ((comp + 1) * pscreen) - border;
      x1 = xoff;
      x2 = xoff + length + 1;
      XSetForeground(xid.display,xid_histo.gc,
                     xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
      XDrawLine(xid.display,histo_win,xid_histo.gc,x1,yoffset,x2,yoffset);

      /* Save the coordinates of the X axis for use in the calculation of
         percentages.  */
      xid_histo.xaxis_x[0] = x1;
      xid_histo.xaxis_x[1] = x2;

      /* Draw the component labels.  */
      x1 = xoff - xshift;
      y1 = yoffset - (height * .75) - border;
      if (totbnd == 1)
         strcpy(annot,"Black and White");
      else if (comp == 0)
         strcpy(annot,"Red Component");
      else if (comp == 1) 
         strcpy(annot,"Green Component");
      else if (comp == 2)
         strcpy(annot,"Blue Component");
      slen = strlen(annot);
      XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

      /* Draw the histogram hash marks.  Note that here the value .008
         represents 8% of the display screen.  */
      x2 = xoff - (int)(HISTX * .008);
      y2 = yoffset + (int)(HISTY * .008);
      XSetForeground(xid.display,xid_histo.gc,
                     xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
      for(i = 1; i <= 4; i++ )
      {
         yaxis = yoffset - (i * .25 * height) - 1;
         XDrawLine(xid.display,histo_win,xid_histo.gc,xoff,yaxis,x2,yaxis);
         xaxis = xoff + (i * .25 * length) + 1;
         XDrawLine(xid.display,histo_win,xid_histo.gc,xaxis,yoffset,xaxis,y2);
      }

      /* Label the X and Y axis.  */
      for (i = 1; i <= 4; i++)
      {
         x1 = xoff + length * (i * .25);
         y1 = yoffset + c_height + 4; 
         freq = (int) 255 * (i * .25);
         sprintf(annot,"%d",freq);
         slen = strlen(annot);
         x1 = x1 - ((slen * c_width) / 2);
         XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

         ymax = (int) maxp * (i * .25);
         sprintf(annot,"%d",ymax);
         slen = strlen(annot);
         x1 = xoff - (slen * c_width);
         x1 = x1 - c_width;
         y1 = yoffset - (height * (i * .25));
         y1 = y1 + (c_height / 2);
         XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);
      }

      /* Recalculate statistics if any mask values were applied.  */
      if ((xid_histo.bcount != 0) || (xid_histo.rcount != 0))
      {
         if (!zflag)
         {
            if((xid_histo.flags & HISTO_MAPPED) && xid.mapped)
               histptr = xid_planes[bnds[comp]-1].histo_map;
            else
               histptr = xid_planes[bnds[comp]-1].histogram;
         }
         else
         {
            if(xid_histo.flags & HISTO_MAPPED)
               histptr = xid_histo.z_histo_map[bnds[comp] - 1];
            else
               histptr = xid_histo.z_histo[bnds[comp] - 1];
         }
         mskflag = TRUE;
         xid_histcalc(histptr,&xid_histo.minval,&xid_histo.maxval,
                      &xid_histo.mean,&xid_histo.stdev);
      }

      /* Draw the histogram statistics.  */
      x1 = xoff - xshift;
      y1 = yoffset - (height * .75) - border;

      y1 += height * .18;        /* Output mean */
      strcpy(annot,"Mean    = ");
      if (!mskflag)
         sprintf(meann,"%f",xid_planes[bnds[comp]-1].mean);
      else
         sprintf(meann,"%f",xid_histo.mean);
      meann[strlen(meann) - 4] = '\0';
      strcat(annot,meann);
      slen = strlen(annot);
      XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

      y1 += height * .18;         /* Output stdev. */
      strcpy(annot,"St Dev  = ");
      if (!mskflag)
         sprintf(stdeviat,"%f",xid_planes[bnds[comp]-1].stdev);
      else
         sprintf(stdeviat,"%f",xid_histo.stdev);
      stdeviat[strlen(stdeviat) - 4] = '\0';
      strcat(annot,stdeviat);
      slen = strlen(annot);
      XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

      y1 += height * .18;         /* Output min */
      strcpy(annot,"Min Pix = ");
      if (!mskflag)
         sprintf(minstr,"%ld",xid_planes[bnds[comp]-1].minval);
      else
         sprintf(minstr,"%ld",xid_histo.minval);
      strcat(annot,minstr);
      slen = strlen(annot);
      XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

      y1 += height * .18;         /* Output max */
      strcpy(annot,"Max Pix = ");
      if (!mskflag)
         sprintf(maxstr,"%ld",xid_planes[bnds[comp]-1].maxval);
      else
         sprintf(maxstr,"%ld",xid_histo.maxval);
      strcat(annot,maxstr);
      slen = strlen(annot);
      XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

      /* Store appropriate X and Y coordinates for displaying annotation string
         output by histogram percentage calculation.  */
      xid_histo.annot_x[comp] = x1;
      xid_histo.annot_y1[comp] = (long)((double)y1 + (double)height * .18);
      xid_histo.annot_y2[comp] = (long)((double)xid_histo.annot_y1[comp] +
                                        (double)height * .18);

      /* Set a pointer to the appropriate histogram data.  */
      if (xid_histo.flags & HISTO_CUM)
         histptr = xid_histo.cum_histo[comp];
      else if (zflag)
      {
         if(xid_histo.flags & HISTO_MAPPED)
            histptr = xid_histo.z_histo_map[bnds[comp] - 1];
         else
            histptr = xid_histo.z_histo[bnds[comp] - 1];
      }
      else if((xid_histo.flags & HISTO_MAPPED) && xid.mapped)
         histptr = xid_planes[bnds[comp]-1].histo_map;
      else
         histptr = xid_planes[bnds[comp]-1].histogram;    

      /* Plot the histogram data.  */
      if(totbnd != 1)
      {
      XSetForeground(xid.display,xid_histo.gc,
                     xid.color_table[RGBBW_IDX + comp+2].pixel);
      }
      else
      {
         /* Set the foreground value to black. */
         XSetForeground(xid.display,xid_histo.gc,
                        xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
      }
      yoffset--;
      for (freq = 0; freq <= 255; histptr++,freq++)
      {
         if (*histptr > 0)
         {
            y2 = yoffset - (((float)*histptr / maxp) * height);
            x1 = xoff+(((float)freq/(float)255)*length)+ 1;
            x2 = x1;
            XDrawLine(xid.display,histo_win,xid_histo.gc,x1,yoffset,x2,y2);
         }
      }

      /* Reset masked out values.  */
      if (!zflag)
      {
         if((xid_histo.flags & HISTO_MAPPED) && xid.mapped)
            histptr = xid_planes[bnds[comp]-1].histo_map;
         else
            histptr = xid_planes[bnds[comp]-1].histogram;
      }
      else
      {
         if(xid_histo.flags & HISTO_MAPPED)
            histptr = xid_histo.z_histo_map[bnds[comp] - 1];
         else
            histptr = xid_histo.z_histo[bnds[comp] - 1];
      }

      if (xid_histo.bcount != 0)
      {
         for (i=0; i < xid_histo.bcount; i++)
            *(histptr + xid_histo.bin[i]) = -(*(histptr + xid_histo.bin[i]));
      }
      if (xid_histo.rcount != 0)
      {
         for (i=0; i < xid_histo.rcount; i++)
         {
            for (j=xid_histo.range[i*2]; j < xid_histo.range[i*2+1] + 1; j++)
               *(histptr + j) = -(*(histptr + j));
         }
      }
   }
   return (XID_S);
}

void histo_z
(
   Widget w,
   XtPointer client,
   XEvent *call,
   Boolean *cont
)
/******************************************************************************
Purpose:
   This routine calculates and subsequently draws the histogram in the XZ or
   YZ dimension.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   long    dtype;          /* Data type.                                     */
   long    i,j;            /* Looping indices.                               */
   long    ns;             /* Number of values to fill/histogram.            */
   long    num = 0;        /* Number of planes to histogram.                 */
   long    offset;         /* Offset into image data buffer.                 */
   long    *plane_nums;    /* Plane numbers to histogram.                    */
   long    zero = 0;       /* Constant zero.                                 */

   /* If user wants to view the histogram for the mapped image data, update the
      data then use it to redraw the histogram */
   if(xid_histo.flags & HISTO_MAPPED)
   {
      /* Redraw the histogram */
      if(xid.mapped)
         xid.mapped_z_mod = TRUE;
      XClearWindow(xid.display,XtWindow(histo_drawingarea));
      histo_draw();
   }

   /* Otherwise user wants to view the histogram with original histo data */
   else
   {
      /* Calculate the plane numbers to be histogrammed (i.e. histogram only
         those planes which contain image data).  */
      plane_nums = (long *)calloc(xid.max_plane,sizeof(long));
      for (i=0; i < xid.max_plane; i++)
      {
         if(xid_view.rgbplanes[i] != 0)
         {
            plane_nums[num] = xid_view.rgbplanes[i] - 1;
            ++num;
         }
      }
   
      /* Initialize number of samples, data type and histogram array.  */
      ns = 256;
      dtype = ELONG;
      for(i = 0; i < num; i++)
      {
         c_pxfill((unsigned char *)xid_histo.z_histo[i],&dtype,&ns,
                  (unsigned char *)&zero);
      }
   
      if (xid_histo.flags & HISTO_XZ)
      {
         /* Histogram a single line through all bands.  */
         dtype = EBYTE;
         for (i=0; i < num; i++)
         {
            offset = (xid.line_y * xid_planes[plane_nums[i]].dspwin[NS]); 
            c_pxhist(&xid_planes[plane_nums[i]].dspwin[NS],
                     (long *)(xid_planes[plane_nums[i]].image_data+offset),
                     &dtype,xid_histo.z_histo[i],&zero);
         }
      }
      else /* HISTO_YZ */
      {
         /* Histogram a single column through all bands.  */
         for (i=0; i < num; i++)
         {
            for (j=0; j < xid_planes[plane_nums[0]].dspwin[NL]; j++)
            {
               offset = (j * xid_planes[plane_nums[i]].dspwin[NS]) +
                        xid.line_x;
               xid_histo.z_histo[i][*(xid_planes[plane_nums[i]].image_data +
                                 offset)]++;
            }
         }
      }
   
      /* Calculate the histogram statistics.  */
      for(i = 0; i < num; i++)
      {
         xid_histcalc(xid_histo.z_histo[i],&xid_planes[i].minval,
                      &xid_planes[i].maxval,&xid_planes[i].mean,
                      &xid_planes[i].stdev);
      }
   
      /* Draw the histogram.  */
      XClearWindow(xid.display,XtWindow(histo_drawingarea));
      histo_draw();
      free(plane_nums);
   }
}

void histo_pointer
(
   Widget w,
   XtPointer client_data,
   XEvent *call_data,
   Boolean *cont
)
/******************************************************************************
Purpose:
   This routine is used to draw the cursor at the appropriate location in the
   histogram drawing area.  This routine handles both mouse movement and 
   keyboard movement of the cursor.  If the cursor location is within the
   bounds of the X-axis of the histogram, it also outputs the percentage of 
   image data prior to and including the cursor position.
   
By L. Huewe, USGS/EROS Data Center, 02/92

Algorithm Description:
******************************************************************************/
{
   XEvent  *event;             /* The input event.                           */
   float   percent;            /* Represents the percentage of data prior to
                                  and including the specified pixel value.   */
   long    deltax;             /* Amount to warp the cursor.                 */
   long    deltay;
   long    gray_val;           /* Gray level value at cursor location.       */
   long    height;             /* Height of annotation string.               */
   long    i;                  /* Looping index.                             */
   long    maxlen = 15;        /* Maximum length of annotation string.       */
   long    move;               /* Amount of cursor single step.              */
   long    totbnd;             /* Total number of histogram graphs displayed.*/
   long    width;              /* Width of annotation string.                */
   char    annot[50];          /* Annotation string.                         */
   unsigned int keycode;       /* Key code from keyboard.                    */
   unsigned int state;         /* Keyboard state (Shift key?)                */

   /* Get the event values from the widget.  */
   event = (XEvent *)call_data;

   /* Remove all other MotionNotify events from queue -- this makes for
      smoother scrolling by helping to limit the number of events generated
      while we're acting on the present one.  */
   while (XCheckTypedEvent(xid.display,MotionNotify,event))
      /* do nothing */;

   /* If a a valid cursor key was pressed on the keyboard then move the cursor
      to the new location.  */
   if (event->type == KeyRelease)
   {
      /* Check the validity of the cursor coordinates.  */
      if ((event->xkey.x < 0) || (event->xkey.y < 0) ||
          (event->xkey.x > HISTX) || (event->xkey.y > HISTY))
         return;

      /* Capture the key code and the keyboard state.  */
      keycode = event->xkey.keycode;
      state   = event->xkey.state;

      /* Ensure the keys validity.  */
      if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.j_key) || 
          (keycode == xid_cursor.h_key) || (keycode == xid_cursor.l_key) ||
          (keycode == xid_cursor.left_key) || (keycode == xid_cursor.up_key) ||
          (keycode == xid_cursor.right_key) ||
          (keycode == xid_cursor.down_key))
      {
         /* Reset the delta X and Y amounts.  */
         deltax = deltay = 0;

         /* Check the keyboard state to see if the Shift key was used.  */
         if (state == ShiftMask)
            move = 10;              /* Shift key so use delta of 10.         */
         else
            move = 1;

         /* Set the delta X/Y amount based on the key pressed.  */
         if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.up_key))
            deltay -= move;
         else if ((keycode == xid_cursor.j_key) || (keycode == 
            xid_cursor.down_key))
            deltay += move;
         else if ((keycode == xid_cursor.h_key) || (keycode == 
            xid_cursor.left_key))
            deltax -= move;
         else
            deltax += move;

         /* If a valid move can be made then move the cursor.  */
         if (((event->xkey.x + deltax) < 0) ||
             ((event->xkey.x + deltax) > HISTX) || ((event->xkey.y + deltay) 
             < 0) || ((event->xkey.y + deltay) > HISTY))
            /* do nothing */;
         else
            XWarpPointer(xid.display,None,None,None,None,None,None,
                deltax,deltay);
      }
   }

   /* Calculate the number of histogram graphs displayed.  */
   if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) && 
       (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
      totbnd = 1;
   else
   {
      for (i=0, totbnd=0; i < 3; i++)
      {
         if (xid_view.rgbplanes[i] != 0)
            totbnd++;
      }
   }

   /* Calculate the gray level value corresponding to the cursor position if
      the cursor is within the bounds of the X axis of the histogram.  */
   if ((event->xmotion.x > xid_histo.xaxis_x[0]) && 
       (event->xmotion.x <= xid_histo.xaxis_x[1])) 
   {
      gray_val = (long)(((float)(event->xmotion.x - 1 - xid_histo.xoffset) /
                  (float)xid_histo.xlength) * 255);
   }
   else
      return;

   /* Set the foreground value to black.  */
   XSetForeground(xid.display,xid_histo.gc,
                  xid.color_table[RGBBW_IDX + BLACK_COL].pixel);

   /* Calculate the width and height of characters in the working font.  */
   width = (xid.font_info->max_bounds.rbearing -
              xid.font_info->min_bounds.lbearing) * maxlen;
   height = xid.font_info->max_bounds.ascent +
               xid.font_info->max_bounds.descent;

   /* Output the percentage information.  */
   if (totbnd == 1)
   {
      /* Clear the old annotation string containing percentage information.  */
      XClearArea(xid.display,XtWindow(histo_drawingarea),xid_histo.annot_x[0],
                 xid_histo.annot_y1[0]-xid.font_info->max_bounds.ascent,width,
                 height,False);
      XClearArea(xid.display,XtWindow(histo_drawingarea),xid_histo.annot_x[0],
                 xid_histo.annot_y2[0]-xid.font_info->max_bounds.ascent,width,
                 height,False);

      /* Calculate the percentage of image data prior to and including the gray
         level calculated above (this is calculated from the cumulative
         histogram).  */
      percent = (float)(((double)xid_histo.cum_histo[0][gray_val] / 
                 (double)xid_histo.cum_histo[0][255]) * 100.0);

      /* Output the percentage information as well as the gray level value
         corresponding to the cursor position.  */
      sprintf(annot,"%3.2f%% of Data",percent);
      XDrawString(xid.display,XtWindow(histo_drawingarea),xid_histo.gc,
                  xid_histo.annot_x[0],xid_histo.annot_y1[0],annot,
                  strlen(annot));
      sprintf(annot,"  at Pixel %ld",gray_val);
      XDrawString(xid.display,XtWindow(histo_drawingarea),xid_histo.gc,
                  xid_histo.annot_x[0],xid_histo.annot_y2[0],annot,
                  strlen(annot));
   }
   else
   {
      for (i=0; i < totbnd; i++)
      {
         /* Clear the old annotation string containing percentage
            information.  */
         XClearArea(xid.display,XtWindow(histo_drawingarea),
                    xid_histo.annot_x[i],
                    xid_histo.annot_y1[i]-xid.font_info->max_bounds.ascent,
                    width,height,False);
         XClearArea(xid.display,XtWindow(histo_drawingarea),
                    xid_histo.annot_x[i],
                    xid_histo.annot_y2[i]-xid.font_info->max_bounds.ascent,
                    width,height,False);

         /* Calculate the percentage of image data prior to and including the
            gray level calculated above (this is calculated from the cumulative
            histogram).  */
         percent = ((float)xid_histo.cum_histo[i][gray_val] / 
                    (float)xid_histo.cum_histo[i][255]) * 100;

         /* Output the percentage information as well as the gray level value
            corresponding to the cursor position.  */
         sprintf(annot,"%3.2f%% of Data",percent);
         XDrawString(xid.display,XtWindow(histo_drawingarea),xid_histo.gc,
                     xid_histo.annot_x[i],xid_histo.annot_y1[i],annot,
                     strlen(annot));
         sprintf(annot,"  at Pixel %ld",gray_val);
         XDrawString(xid.display,XtWindow(histo_drawingarea),xid_histo.gc,
                     xid_histo.annot_x[i],xid_histo.annot_y2[i],annot,
                     strlen(annot));
      }
   }
}

int histo_up_map_data
(void)
/******************************************************************************
Purpose:
   This routine is used to update the mapped histogram data using the currently
   displayed image information. Using an XImage, it parses through the image,
   incrementing the appropriate histogram data values.  Once the histogram
   values have been gathered, the histogram statistics are calculated with
   xid_histcalc. If user has the X/Z or Y/Z button selected or if they are
   calculating the histogram for a black and white image, the histogram data
   is all collected into the same histogram buffer.
   
By A. Shimitz, USGS/EROS Data Center, 08/00

Algorithm Description:
******************************************************************************/
{
   GC              copy_gc;          /* GC used to copy pixmaps              */
   long            blue;             /* Blue pixel value                     */
   long            color_idx = IMG_IDX;
                                     /* Index into color table               */
   long            end;              /* Ending loop value for X/Z or Y/Z     */
   long            green;            /* Green pixel value                    */
   long            i, j;             /* Looping indices                      */
   long            line;             /* Current line for processing          */
   long            nbands;           /* Number of bands to process           */
   long            nl;               /* Number of lines in image             */
   long            ns;               /* Number of samples in image           */
   long            plane_idx[3];     /* Image plane index of current image   */
   long            red;              /* Red pixel value                      */
   long            samp;             /* Current sample for processing        */
   long            sl;               /* Starting line                        */
   long            ss;               /* Starting sample                      */
   long            start;            /* Starting loop value for X/Z or Y/Z   */
   long            valuemask = 0;    /* Specify no values                    */
   long            zero = 0;         /* Constant zero                        */
   Pixmap          img_pix;          /* Image pixmap                         */
   Pixmap          *pixptr = NULL;   /* Pixmap pointer                       */
   unsigned long   pixel;            /* Single pixel value                   */
   Window          image_win;        /* Image window                         */
   XColor          *color_ptr;       /* Colortable pointer                   */
   XGCValues       values;           /* Values array for the GC              */
   XImage          *image = NULL;    /* X image structure                    */

   image_win = XtWindow(main_image_bulletinboard);

   /* Get the memory plane indices of the currently displayed image and set
      the starting line and starting sample. */
   plane_idx[RED] = xid_view.rgbplanes[RED] - 1;
   plane_idx[GREEN] = xid_view.rgbplanes[GREEN] - 1;
   plane_idx[BLUE] = xid_view.rgbplanes[BLUE] - 1;
   sl = ss = 0;

   /* Determine the number of lines and samples in the output image.  Also,
      set a pointer to the image data. */
   nl = xid_planes[plane_idx[RED]].dspwin[NL];
   ns = xid_planes[plane_idx[RED]].dspwin[NS];
   pixptr = &xid_mem.image_pixmap;
   if(pixptr == NULL)
   {
      xid_message("Error defining the Pixmap while updating the histogram"
                  " data.");
      return (XID_F);
   }

   /* Create the graphics context for saving the image data */
   copy_gc = XCreateGC(xid.display, image_win, valuemask, &values);

   /* Determine the number of bands to save */
   if((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) &&
      (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
      nbands = 1;
   else
      nbands = 3;


   /* Determine which, if any, mapping function is being used (adjust, pwl, or
      pseudocolor) and set the color table pointer to the appropriate one. */
   if((xid_map.subpanel == Shell_Adjust) && (xid_adjust.color_adjust != NULL))
      color_ptr = xid_adjust.color_adjust;
   else if((xid_map.subpanel == Shell_PWL) && (xid_pwl.color_pwl != NULL))
      color_ptr = xid_pwl.color_pwl;
   else if((xid_map.subpanel == Shell_Pseudo) && (xid_psd.color_psd != NULL))
      color_ptr = xid_psd.color_psd;
   /* If no mapping panels are currently open or the color table for the
      current mapping is NULL (panel is open but no mapping has been applied
      yet), use the current color table */
   else
      color_ptr = xid.colortable_ptr;

   /* Zero out all old values before recalculating */
   if(!(xid_histo.flags & HISTO_XY))
   {
      for(i = 0; i < 3; i++)
      {
         for(j = 0; j < 256; j++)
            xid_histo.z_histo_map[i][j] = 0;
      }
   }
   else
   {
      for(i = 0; i < 3; i++)
      {
         for(j = 0; j < 256; j++)
            xid_planes[i].histo_map[j] = 0;
      }
   }

   if(!(xid_histo.flags & HISTO_XY))
   {
      if(xid_histo.flags & HISTO_XZ)
      {
         /* Create a pixmap to store the image data in */
         img_pix = XCreatePixmap(xid.display, image_win, ns, 1,
                                 xid.visual_info[0].depth);

         /* Capture the current image data into the pixmap */
         XCopyArea(xid.display,*pixptr,img_pix,copy_gc,ss,xid.line_y,ns,1,zero,
                   zero);

         /* Read the image data into an X image structure */
         image = XGetImage(xid.display,img_pix,ss,sl,ns,1,AllPlanes,ZPixmap);

         /* Determing the start and end of the loop index, depending on the
            location of the type of histogram */
         start = ss;
         end = ns;
      }
      else /* HISTO_YZ */
      {
         /* Create a pixmap to store the image data in */
         img_pix = XCreatePixmap(xid.display, image_win, 1, nl,
                                 xid.visual_info[0].depth);

         /* Capture the current image data into the pixmap */
         XCopyArea(xid.display,*pixptr,img_pix,copy_gc,xid.line_x,sl,1,nl,zero,
                   zero);

         /* Read the image data into an X image structure */
         image = XGetImage(xid.display,img_pix,ss,sl,1,nl,AllPlanes,ZPixmap);

         /* Determing the start and end of the loop index, depending on the
            location of the type of histogram */
         start = sl;
         end = nl;
      }

      /* Make sure the XImage got defined properly */
      if(image == NULL)
      {
         xid_message("Error reading image data while updating the histogram"
                     "data.");
         XFreePixmap(xid.display, img_pix);
         XFreeGC(xid.display, copy_gc);
         return (XID_F);
      }

      /* Now update the histogram. We only want to read the specified
         row/column from the image, indicated by the cursor position, when
         viewing a z dimensional histogram */
      for(i = start; i < end; i++)
      {
         /* Get the appropriate pixel value from the X image and scale it to a
            value between 0 and 255 */
         if(xid_histo.flags & HISTO_XZ)
            pixel = XGetPixel(image, i, 0);
         else /* HISTO_YZ */
            pixel = XGetPixel(image, 0, i);
         if(xid.visual_info[0].depth == 24)
         {
            red = (long)(pixel & 0xff);
            green = (long)((pixel & 0xff00) >> 8);
            blue = (long)((pixel & 0xff0000) >> 16);
         }
         else
            red = green = blue = (long)pixel;

         /* Adjust the index into the mapping color maps if in PseudoColor mode
            since this is how the values are stored (colors have been reduced
            to 200 colors) */
         if((xid.visual_info[0].visual->class == PseudoColor) &&
            (color_ptr != xid.colortable_ptr))
         {
            red -= color_idx;
            green -= color_idx;
            blue -= color_idx;
         }

         /* Use the pixel value as an index into the appropriate color table.
            Again, scale the resulting value to a value between 0 and 255 */
         if(nbands == 1)
         {
            xid_histo.z_histo_map[plane_idx[RED]][color_ptr[red].red >> 8]++;
            xid_histo.z_histo_map[plane_idx[RED]]
               [color_ptr[green].green >> 8]++;
            xid_histo.z_histo_map[plane_idx[RED]][color_ptr[blue].blue >> 8]++;

         }
         else
         {
            xid_histo.z_histo_map[plane_idx[RED]][color_ptr[red].red >> 8]++;
            xid_histo.z_histo_map[plane_idx[GREEN]]
               [color_ptr[green].green >> 8]++;
            xid_histo.z_histo_map[plane_idx[BLUE]][color_ptr[blue].blue >> 8]++;
         }
      }

      /* Calculate the histogram statistics */
      for(i = 0; i < nbands; i++)
      {
         xid_histcalc(xid_histo.z_histo_map[i],&xid_planes[i].minval,
                      &xid_planes[i].maxval,&xid_planes[i].mean,
                      &xid_planes[i].stdev);
      }
   }

   /* Otherwise create histogram using entire image */
   else
   {
      /* Create a pixmap to store the image data in */
      img_pix = XCreatePixmap(xid.display, image_win, ns, nl,
                              xid.visual_info[0].depth);
   
      /* Capture the current image data into the pixmap */
      XCopyArea(xid.display,*pixptr,img_pix,copy_gc,ss,sl,ns,nl,zero,zero);
   
      /* Read the image data into an X image structure */
      image = XGetImage(xid.display,img_pix,ss,sl,ns,nl,AllPlanes,ZPixmap);
      if(image == NULL)
      {
         xid_message("Error reading image data while updating the histogram"
                     "data.");
         XFreePixmap(xid.display, img_pix);
         XFreeGC(xid.display, copy_gc);
         return (XID_F);
      }

      /* Fill the histogram data buffers with tally of pixel colormap values
         from current image */
      for(line = sl; line < nl; line++)
      {
         for(samp = ss; samp < ns; samp++)
         {
            /* Get the appropriate pixel value from the X image and scale it to
               a value between 0 and 255 */
            pixel = XGetPixel(image, samp, line);
            if(xid.visual_info[0].depth == 24)
            {
               red = (long)(pixel & 0xff);
               green = (long)((pixel & 0xff00) >> 8);
               blue = (long)((pixel & 0xff0000) >> 16);
            }
            else
               red = green = blue = (long)pixel;
   
            /* Adjust the index into the mapping color maps if in PseudoColor
               mode since this is how the values are stored (colors have been
               reduced to 200 colors) */
            if((xid.visual_info[0].visual->class == PseudoColor) &&
               (color_ptr != xid.colortable_ptr))
            {
               red -= color_idx;
               green -= color_idx;
               blue -= color_idx;
            }

            /* Use the pixel value as an index into the current color table.
               Again, scale the resulting value to a value between 0 and 255 */
            xid_planes[plane_idx[RED]].histo_map[color_ptr[red].red >> 8]++;
            if(nbands == 1)
            {
               /* If calculating histogram for a black and white image, keep
                  the histogram data all in the first plane histo*/
               xid_planes[plane_idx[RED]].histo_map[color_ptr[green].green
                  >> 8]++;
               xid_planes[plane_idx[RED]].histo_map[color_ptr[blue].blue
                  >> 8]++;
            }
            else
            {
               xid_planes[plane_idx[GREEN]].histo_map[color_ptr[green].green
                  >> 8]++;
               xid_planes[plane_idx[BLUE]].histo_map[color_ptr[blue].blue
                  >> 8]++;
            }
         }
      }

      /* Recalculate the histogram statistics based on the new image data. */
      xid_histcalc(xid_planes[plane_idx[RED]].histo_map,
                   &xid_planes[plane_idx[RED]].minval,
                   &xid_planes[plane_idx[RED]].maxval,
                   &xid_planes[plane_idx[RED]].mean,
                   &xid_planes[plane_idx[RED]].stdev);
      if(nbands != 1)
      {
         xid_histcalc(xid_planes[plane_idx[GREEN]].histo_map,
                      &xid_planes[plane_idx[GREEN]].minval,
                      &xid_planes[plane_idx[GREEN]].maxval,
                      &xid_planes[plane_idx[GREEN]].mean,
                      &xid_planes[plane_idx[GREEN]].stdev);
         xid_histcalc(xid_planes[plane_idx[BLUE]].histo_map,
                      &xid_planes[plane_idx[BLUE]].minval,
                      &xid_planes[plane_idx[BLUE]].maxval,
                      &xid_planes[plane_idx[BLUE]].mean,
                      &xid_planes[plane_idx[BLUE]].stdev);
      }
   }

   /* Free buffers */
   XFreePixmap(xid.display, img_pix);
   XFreeGC(xid.display, copy_gc);
   XDestroyImage(image);

   /* Reset the modified flags */
   if(!(xid_histo.flags & HISTO_XY))
      xid.mapped_z_mod = FALSE;
   else
      xid.mapped_mod = FALSE;

   return(XID_S);
}
