/*
 *  This is the layout function source file for the layout panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */

#include "xid.h"
#include "xid_func.h"

void layout_allocate
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine retrieves the user specified values defining the memory
   layout and attempts to allocate the memory needed to accomodate the
   specifications.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   long    error = FALSE,      /* Has a memory allocation error occurred?    */
           i;                  /* Looping index.                             */
   char    *tmpstr;            /* Temporary storage for a text string.       */

   /* Retrieve the user specified memory plane width, height and depth and the
   viewport width and height.  */
   tmpstr = XmTextGetString(layout_mw_text);
   sscanf(tmpstr,"%ld",&xid_layout.mem_width);
   xid_mem.mem_width = xid_layout.mem_width;
   XtFree(tmpstr);

   tmpstr = XmTextGetString(layout_mh_text);
   sscanf(tmpstr,"%ld",&xid_layout.mem_height);
   xid_mem.mem_height = xid_layout.mem_height;
   XtFree(tmpstr);

   tmpstr = XmTextGetString(layout_md_text);
   sscanf(tmpstr,"%ld",&xid_layout.mem_depth);
   XtFree(tmpstr);

   tmpstr = XmTextGetString(layout_vw_text);
   sscanf(tmpstr,"%ld",&xid_layout.view_width);
   xid_mem.view_width = xid_layout.view_width;
   XtFree(tmpstr);

   tmpstr = XmTextGetString(layout_vh_text);
   sscanf(tmpstr,"%ld",&xid_layout.view_height);
   xid_mem.view_height = xid_layout.view_height;
   XtFree(tmpstr);

   /* If memory planes have previously been allocated, free them before
   attempting to allocate new ones.  */
   if (xid_planes != NULL)
   {
      for (i=0; i < xid.max_plane; i++)
      {
         free(xid_planes[i].image_data);
         free(xid_planes[i].image_info);
      }
      free(xid_planes);
   }

   /* Attempt to allocate the number of memory planes specified.  */
   error = layout_allocmem();

   /* If successful, close the layout panel.  */
   if (error == XID_S)
   {
      layout_destroy(w, client, call);
      xid.current_plane = 0;
      xid.max_plane = xid_layout.mem_depth;
   }
}

void layout_cancel
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine calls layout_destroy to close the layout panel. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   /* Close the layout panel.  */
   layout_destroy(w, client, call);
}

void layout_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine closes the layout window and sets the main menu option to
   sensitive. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   /* Close the layout window and set the layout button on the main panel to
   sensitive.  */
   XtUnmanageChild(Layout_bulletinboard);
   XtPopdown(Shell_Layout);
   XtSetSensitive(main_layout_button,True);
}

void layout_reset
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine resets memory layout parameters to their default values. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   /* Reset parameters to their default values.  */
   XmTextSetString(layout_vh_text,DEFC_VHEIGHT);
   XmTextSetString(layout_vw_text,DEFC_VWIDTH);
   XmTextSetString(layout_md_text,DEFC_NPLANE);
   XmTextSetString(layout_mh_text,DEFC_HEIGHT);
   XmTextSetString(layout_mw_text,DEFC_WIDTH);
}

int layout_allocmem
(void)

/******************************************************************************
Purpose:
   This routine dynamically allocates the amount of memory needed to 
   accomodate the user specified layout.  This includes memory for storing
   image data as well as image related information.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   int     error = FALSE,      /* Has a memory allocation error occurred.    */
           i,j;                /* Looping indices.                           */

   /* Allocate memory for the number of composite views to store.  */
   if (xid_composite != NULL)
   {
      free(xid_composite);
      xid_composite = NULL;
   }
   xid_composite = (struct XID_COMPOSITE *) calloc(NUM_COMPOSITE, sizeof
                                                   (struct XID_COMPOSITE));
   if (xid_composite == NULL)
   {
      xid_message("Insufficient memory available");
      return(XID_F);
   }
   for (i=0; i < NUM_COMPOSITE; i++)
      xid_composite[i].flags = 0;
   xid.view = -1;

   /* Allocate memory for the number of memory planes specified.  */
   xid_planes = (struct XID_MEMPLANE *) calloc(xid_layout.mem_depth, sizeof
                                               (struct XID_MEMPLANE));
   if (xid_planes == NULL)
   {
      xid_message("Insufficient memory available");
      free(xid_composite);
      error = TRUE;
   }
   else
   {

      /* Allocate memory for the actual image data.  */
      for (i=0; ((i < xid_layout.mem_depth) && (!error)); i++)
      {
         xid_planes[i].flags = 0;
         xid_planes[i].image_data = (unsigned char *) malloc(
                                    xid_layout.mem_width *
                                    xid_layout.mem_height);
         if (xid_planes[i].image_data == NULL)
         {
            xid_message("Insufficient memory available");
            error = TRUE;
            for (j=0; j < i; j++)
            {
               free(xid_planes[j].image_data);
               free(xid_planes[j].image_info);
            }
            break;
         }

         /* Allocate memory for the image information (i.e. projection
         information, number of lines, number of samples, etc.).  */
         xid_planes[i].image_info = (struct XID_IMGINFO *) calloc(1, sizeof
                                                         (struct XID_IMGINFO));
         if (xid_planes[i].image_info == NULL)
         {
            xid_message("Insufficient memory available");
            error = TRUE;
            free(xid_planes[i].image_data);
            for (j=0; j < i; j++)
            {
               free(xid_planes[j].image_data);
               free(xid_planes[j].image_info);
            }
            break;
         }
      }
   }

   /* If an error occurred, free any previously allocated memory.  */
   if (error)
   {
      free(xid_composite);
      free(xid_planes);
      return(XID_F);
   }
   else
      return(XID_S);
}
