/*
 *  This is the load function source file for the load panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 *  Modified by A. Shimitz 6/00
 *   - Added functionality to load_destroy to re-enable the save options on
 *     the file menu b/c of the new save "Thumbnail TIFF" option
 *
 *  Modified by A. Shimitz 7/00
 *   - Also updated it so if the Show panel is open it will deselect all the
 *     list items before loading the new image
 *
 *  Modified by A. Shimitz 8/00
 *   - Added a call to the routine init_map_histo() to support the
 *     functionality of the Mapped button on the histogram panel (CR #3892).
 *
 *  Modified by A. Shimitz 3/01
 *   - Replaced c_cmtae calls with simple strcpy calls since the c_cmtae
 *     routine is now obsolete.
 *
 */

#include <math.h>
#include <string.h>
#include <X11/cursorfont.h>
#include <X11/IntrinsicP.h>
#include <unistd.h>
#include "las.h" /* to get CMLEN definition */
#include "util.h" /* c_getin2 & c_up2low definitions */
#include "labserv.h" /* c_lsmknm definition */
#include "xid.h"
#include "manage_cursor.h"
#include "xid_func.h"
#include "xid_create.h"
#include "xid_graphics.h"
#include "main.h"
#include "../xid_widgets/ProcessM.h"

/* local function prototypes */
int load_makelas
(
   char *inspec,               /* Input specification. (I)                   */
   char *lasspec               /* LAS specification. (O)                     */
);
/* end of prototypes */

void load_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine closes the load image panel and sets the load options to 
   sensitive.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Modified by A. Shimitz 11/00
 - Added call to remove_window_cursor to remove the load window from the
   managed_list of windows to display the wait cursor in when show_wait_cursor
   is called.  (CR #3967)

Algorithm Description:
******************************************************************************/
{
   /* Set the load and save buttons to sensitive */
   XtSetSensitive(main_load_las,True);
   XtSetSensitive(main_load_nonlas,True);
   XtSetSensitive(main_save_las,True);
   XtSetSensitive(main_save_tiff,True);

   /* Close the file selector */
   remove_window_cursor(xid.display,XtWindow(Load_form));
   XtUnmanageChild(Load_form);
   XtDestroyWidget(Load_form);
   XtPopdown(Shell_Load);

   /* Set attributes back to the "Load" settings if the load selection box was
   being used to save a TIFF image */
   if(xid_load.save)
   {
      XtVaSetValues(Shell_Load, XmNtitle, "X Image Display - Load", NULL);
      XtVaSetValues(Shell_Load, XmNiconName, "Load", NULL);
      xid_load.save = FALSE;
   }
}

void load_localcancel
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine is called when the user selects cancel from the file
   selector and calls load_destroy to close the file selector window.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   /* Close the file selector.  */
   load_destroy(w, client, call);
}

void load_localok
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine retrieves the file name specified by the user, parses the
   name, changes the cursor shape to a watch, pops up the processing
   message dialog, loads the appropriate image to the display and updates
   any graphics information.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Modified by L. Huewe, USGS/EROS Data Center, 8/93
   - Modified to output an XID message when xid_getin fails.

Modified by C. Olsen, USGS/EROS Data Center, 4/99
   - Added XmString variable and modified function to display GOF name in 
     a label instead of text field on the GOF window. (SR #2428)

Modified by A. Shimitz 11/00
 - Replaced the hourglass cursor with the watch cursor, managed by the
   manage_cursor routines.  (CR #3967)

Algorithm Description:
******************************************************************************/
{
   Window   imgwin;            /* Window ID for main image window.           */
   int      access_mode,       /* File open access mode.                     */
            cflag;             /* Indicates whether or not a GOF was created.*/
   long     i,                 /* Looping index.                             */
            imglimit,          /* Input image limit.                         */
            lasflg,            /* Indicates whether a LAS name or host name 
                                  was specified.                             */
            length,            /* String length.                             */
            plane_idx,         /* Index into memory plane structure.         */
            pos,               /* Position of a specified character in a 
                                  string.                                    */
            qflag[MAXIMG][2],  /* Indicates whether user specified a window
                                  and band numbers for each input image.     */
            same_window,       /* Indicates if specified images have the
                                  same window size.                          */
            status;            /* Status indicator.                          */
   char     ext[16],           /* File extension.                            */
            *filename,         /* Input specification.                       */
            goftyp[11],        /* Type of GOF.                               */
            hname[CMLEN],      /* Host file name of displayed image (first
                                  band).                                     */
            hostnam[CMLEN],    /* Host name of the GOF.                      */
            lasspec[CMLEN],    /* User specification with LAS names.         */
            tmpname[3][CMLEN]; /* Temporary storage for input name.          */
   char     port_filename[CMLEN];
                               /* Portable GOF file name.                    */
   char     *left,             /* Location of the left paren.                */
            *right,            /* Location of the right paren.               */
            *semi;             /* Location of the semi colon.                */
   XmFileSelectionBoxCallbackStruct *acs =
     (XmFileSelectionBoxCallbackStruct*)call;

   /* If the graphics panel is displayed and a type is currently selected,
      make sure any graphics elements the user created get written to the
      GOF file, then close the GOF */
   if ((xid_graphics.flags & GR_ACTIVE) && (xid_graphics.type != NO_TYPE))
   {
      /* Set flag for GOF function to key off to indicate the loading routine
         is executing */
      xid_load.loading = TRUE;

      /* Write the current record and close the current GOF.  */
      if ((xid_graphics.type != NO_TYPE) && (xid_graphics.flags & GR_SELECT))
      {
         if (!(xid_element[xid_graphics.current].writeflg))
         {
            status = graphics_gofwrite(w, client, call);
            if (status != XID_S)
               return;
            xid_element[xid_graphics.current].writeflg = TRUE;
         }
         if (xid_graphics.fd != -1)
            c_gofcls(&xid_graphics.fd);
      }
   }

   /* Retrieve the file name selected and parse it.  This determines its LAS
   name (if applicable), host file name, window specified and bands specified*/
   /* Don't do this if loading an image from command line */
   if(!xid_load.init_load)
   {
      if (xid_load.type == LAS)
      {
         XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&filename);
         strcpy(xid_load.image_name,filename);
         if (xid_load.dirmask != NULL)
            XmStringFree(xid_load.dirmask);
         xid_load.dirmask = XmStringCopy(acs->mask);
         XtFree(filename);
      }
   }

   /* Make sure the file name is not too long */
   if(strlen(xid_load.image_name) > CMLEN)
   {
      xid_message("Specified file name exceeds allowable length (256).");
      return;
   }

   /* Determine whether the input name is a LAS name or a host name.
      - If the name contains a bracket ([) or a semicolon (;), it is assumed to
        be a LAS name.
      - If the name contains a slash (/), it is assumed to be a host name.  */
   lasflg = -1;
   if (strcspn(xid_load.image_name,"[") != strlen(xid_load.image_name))
      lasflg = TRUE;
   else if (strcspn(xid_load.image_name,"/") != strlen(xid_load.image_name))
      lasflg = FALSE;
   else if (strcspn(xid_load.image_name,";") != strlen(xid_load.image_name))
   {
      left = strchr(xid_load.image_name,'(');
      right = strchr(xid_load.image_name,')');
      semi = strchr(xid_load.image_name,';');
      if ((left == NULL) || (right == NULL))
         lasflg = TRUE;
      else if (semi > right)
         lasflg = TRUE;
   }
   pos = (long)strcspn(xid_load.image_name,"(");
   if (pos != strlen(xid_load.image_name))
   {
      strncpy(tmpname[0],xid_load.image_name,pos);
      tmpname[0][pos] = '\0';
   }
   else 
      strcpy(tmpname[0],xid_load.image_name);
   if (lasflg == -1)
   {
      if (access(tmpname[0],0) != 0)
         lasflg = TRUE;
      else
         lasflg = FALSE;
   }

   /* If loading a LAS image and the input specification was made with host
   names, convert the image names to LAS names.  */
   if ((xid_load.type == LAS) && (!lasflg))
   {
      if(load_makelas(xid_load.image_name,lasspec) != XID_S)
         return;
   }
   else
      strcpy(lasspec,xid_load.image_name);

   /* Parse the image name, window and band specifications.  */
   imglimit = xid.max_plane;
   if (xid_load.type == LAS)
      status = c_getin2(lasspec,xid_load.tnames,xid_load.hnames,
      &xid_load.nimgs,xid_load.windows,xid_load.bands,xid_load.nbands,
      &xid_load.totbnd,&imglimit,&same_window,qflag);
   else
      status = xid_getin(xid_load.image_name,xid_load.tnames,xid_load.hnames,
      &xid_load.nimgs,xid_load.windows,xid_load.bands,xid_load.nbands,
      &xid_load.totbnd,&imglimit,&same_window,qflag);
   if (status != E_SUCC)
   {
      xid_message("Error parsing image specification");
      xid_load.init_load = FALSE;
      return;
   }

   /* In order to display images from separate files as one image, they must
   all be the same size.  */
   if (!same_window)
   {
      xid_message("Windows of input images are not the same size");
      strcpy(xid_load.image_name, "");
      return;
   }

   /* Display the wait cursor (watch) in both the image window and main
   panel.  */
   show_wait_cursor();
   imgwin = XtWindow(main_image_bulletinboard);

   /* Set the title for the processing message and processing icon.  */
   XtVaSetValues(load_dialog_shell,XmNtitle,"Load - Processing",NULL);

   /* Close the file selection box.  */
   /* Don't do this if loading the image from command line */
   if(!xid_load.init_load)
      load_destroy(w, client, call);

   /* THIS IS TEMPORARY UNTIL THE PROJ INFO PANEL IS IMPLEMENTED.  */
   xid_projinfo.pdist_y = xid_projinfo.pdist_x = -999;
   xid_projinfo.line_inc = xid_projinfo.sample_inc = -999;
   *xid_projinfo.proj_units = '\0';

   /* Clear the graphics pixmap if the user has the save portable GOF
      toggle selected. This will load no GOFs onto the new image by default
      so the user can select which, if any GOF to load onto this new image */
   if ((xid_graphics.pixmap[0] != 0) && (xid_graphics.flags & GR_SAVEPORT))
   {
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);
      for (i=0; i < USER_GRCOLORS; i++)
      {
         XFillRectangle(xid.display,xid_graphics.pixmap[i],xid_graphics.rb_gc,
                        0,0,xid_mem.mem_width,xid_mem.mem_height);
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
      plane_idx = xid_view.rgbplanes[RED] - 1;

      XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,0,
                 xid_planes[plane_idx].window[NS],
                 xid_planes[plane_idx].window[NL],True);
   }

   /* Load the specified image.  */
   xid.view = -1;
   loadimg(imgwin,TRUE,FALSE,FALSE);

   /* Redisplay the default arrow cursor.  */
   unshow_wait_cursor();

   /* Turn on the bell to let the user know the image load is complete.  */
   XBell(xid.display,0);

   /* Update the graphics information. Load any traditional GOF of the
      current type that exists for the new image loaded - UNLESS the user
      has the save portable GOF toggle selected; then just save and close
      the current GOF, but don't load anything - let the user load which
      GOF they want to view (traditional or portable). */
   if ((xid_graphics.pixmap[0] != 0) && !(xid_graphics.flags & GR_SAVEPORT))
   {
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);
      for (i=0; i < USER_GRCOLORS; i++)
      {
         XFillRectangle(xid.display,xid_graphics.pixmap[i],xid_graphics.rb_gc,
                        0,0,xid_mem.mem_width,xid_mem.mem_height);
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
      plane_idx = xid_view.rgbplanes[RED] - 1;
      graphics_update(0,plane_idx);
   }

   /* If the graphics panel is displayed, update the GOF name.  */
   if ((xid_graphics.flags & GR_ACTIVE) && (xid_graphics.type != NO_TYPE))
   {
      switch (xid_graphics.type)
      {
         case POINT:
            strcpy(goftyp,"POINT");
            break;
         case LINE:
            strcpy(goftyp,"LINE");
            break;
         case RECT:
         case POLY:
            strcpy(goftyp,"POLY");
            break;
         case TEXT:
            strcpy(goftyp,"ANNOT");
            break;
      }

      /* Determine the default name for the GOF based on the first band of the
         displayed image.  */
      plane_idx = xid_view.rgbplanes[RED] - 1;
      if (xid_planes[plane_idx].image_type == LAS)
      {
         status = c_cmhost(xid_planes[plane_idx].filename,hname);
         if (status != E_SUCC)
         {
            xid_message("Error retrieving host file name");
            return;
         }
      }
      else
         strcpy(hname,xid_planes[plane_idx].filename);
      if ((xid_graphics.flags & GR_SELECT) || (xid_graphics.flags & GR_LOAD) ||
          (xid_graphics.flags & GR_EDIT))
      {
         if ((xid_graphics.flags & GR_SELECT) || (xid_graphics.flags &GR_EDIT))
            access_mode = APPEND;
         else
            access_mode = READ;
         status = c_gofopn(&xid_graphics.fd,hname,xid_graphics.lname_gof,
                 goftyp, &access_mode, &cflag);
         if (status != E_SUCC)
         {
            xid_message("Error opening GOF");
            xid_graphics.fd = -1;
         }
         if (xid_planes[plane_idx].image_type != LAS)
            strcpy(xid_graphics.lname_gof,hname);
      }

      /* Set up the GOF extension based on the type specified.  */
      else
      {
         sprintf(ext,"GOF%c%s",GOF_SEP,goftyp);
         length = (long)strlen(ext);
         c_up2low(ext,&length);

         /* Get the associated file name.  */
         c_lsmknm(hname,ext,hostnam);
         strcpy(xid_graphics.lname_gof,hostnam);
      }
      XtVaSetValues(graphics_gof_label,XmNlabelString,
                    (XmStringCreateLtoR(xid_graphics.lname_gof,
                                        XmSTRING_DEFAULT_CHARSET)),NULL);
      graphics_port_filename(xid_graphics.lname_gof, xid_graphics.type,
                             port_filename);
      XmTextSetString(graphics_port_gof_text, port_filename);
   }

   XmUpdateDisplay(Shell_MainMenu);

   /* Close the nonlas information panel.  */
   if (xid_load.type != LAS)
   {
      /* Don't do this if loading an image from command line */
      if(!xid_load.init_load)
         nonlas_destroy(w,client,call);
   }

   /* Set the init_load flag to false so everything works regular from now on*/
   if(xid_load.init_load)
      xid_load.init_load = FALSE;

   /* Done loading, unset the loading flag */
   xid_load.loading = FALSE;

   /* (Re)Initialize the mapped histogram information (flags & data) */
   init_map_histo();
}

int load_display
(
   long *rgbmem,        /* Memory planes to be loaded with image data (I).   */
   long *nplanes,       /* Number of planes loaded (I).                      */
   Window imgwin,       /* Window ID for main image window (I).              */
   Widget shell,        /* Widget ID of image window shell (I).              */
   Widget win,          /* Widget ID of image window (I).                    */
   long *nl,            /* Number of lines to load (I).                      */
   long *ns,            /* Number of samples to load (I).                    */
   long *maxval,        /* Maximum image data value (I).                     */
   unsigned char *rptr, /* Pointer to red image data (I).                    */
   unsigned char *gptr, /* Pointer to green image data (I).                  */
   unsigned char *bptr, /* Pointer to blue image data (I).                   */
   long cmflg,          /* Indicates whether or not color map should
                           be modified (I).                                  */
   Pixmap *pixptr,      /* Pointer to pixmap which is to contain the
                           loaded image data (I).                            */
   long loadflg,        /* Indicates whether this is being called from
                           the loadimg routine (I).                          */
   long viewflg,        /* If TRUE, resize viewport to original
                           specification.  If FALSE, base resizing on
                           the current viewport width and height.  (I)       */
   long flickerflg,     /* Indicates whether or not this load is part
                           of the flicker process (I).                       */
   long pixmapflg       /* Indicates if it should just create the pixmap or
                           create and display the pixmap                     */
)
/******************************************************************************
Purpose:
    This routine outputs the input image data to the current display.  The
    appropriate colormap is defined and installed based on the display type.
    The image data is actually loaded into a pixmap which is used to set
    the background of the main image window.  Load_display also clears any
    currently displayed zoomed or full resolution windows, verifies whether
    or not the profile and cursor Z options are valid and updates the plane
    list displayed by the show function.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Modified by A.Shimitz 11/00
  - Added code to initially hightlight the appropriate planes in the red,
    green, blue, and saved lists on the Show panel.  It also sets the
    xid_show.planes values.  (CR #4157)

Modified by A.Shimitz 12/00
  - Added additional condition needed to reinitialize the fullres window and
    related variables - don't want to do this when reloading the main image
    window as a result of pressing "Stop" on the flicker panel because the
    next thing that is done is the fullres window is reloaded, which relies on
    the values of those variables to still be there.  (CR #1976)

Algorithm Description:
******************************************************************************/
{
   Arg       args[4];         /* Variable used to set widget resources.     */
   Widget    widget = 0;      /* Widget ID.                                 */
   GC        mask_gc,         /* GC for displaying graphics masks.          */
             rgb_gc;          /* Working color GC.                          */
   XGCValues gc_val;          /* GC set value array.                        */
   XImage    *ximage = NULL;  /* X image structure.                         */
   Dimension view_height,     /* Viewport height.                           */
             view_width;      /* Viewport width.                            */
   XmString  xmstring;        /* Motif string                               */
   static unsigned char
             rr[256],         /* Reserved color values.                     */
             gr[256],
             br[256];
   float     scaler;          /* LUT scaler for actual number of colors.    */
   long      buf_size,        /* Number of bytes in a single band.          */
             count,           /* Number of active planes.                   */
             i,               /* Looping index.                             */
             j,               /* looping index                              */
             img_idx,         /* Index into colormap of first image color.  */
             num_imgcolors,   /* Number of image colors.                    */
             offset,          /* Offset used in scaling the 24-bit LUT.     */
             show_flg,        /* Indicates whether or not the show panel
                                 is active.                                 */
             state,           /* Toggle button state.                       */
             tform[256];      /* Pixel transform array.                     */
   long      tmp;             /* holds a temporary value                    */
   long      num_composite;   /* number of composite images                 */
   long      same_flg;        /* indicates if the current composite has the
                                 same planes as the current view            */
   long      found = FALSE;   /* indicates if a saved composite matched the
                                 current plane selections - if not then
                                 highlight "Current" in the save list       */
   unsigned long
             *data = NULL;    /* Pointer to output values for 24-bit data.  */
   unsigned char
             *blut = NULL,    /* Blue LUT for composite image.              */
             *composite,      /* Composite RGB 8-bit image or single band
                                 black white 8-bit image.                   */
             *comptr = NULL,  /* Pointer to composite buffer.               */
             *dataptr,        /* Pointer to image data stored in memory
                                 plane.                                     */
             *glut = NULL,    /* Green LUT for composite image.             */
             *rgbptr[3],      /* Pointers to RGB components.                */
             *rlut = NULL;    /* Red LUT for composite image.               */
   char      filename[256];   /* Filename for ASCII field on PWL widget     */
   char      fullres_opt[4],  /* Current full resolution size option.       */
             points[10];      /* Number of points string.                   */
   char      planestr[20];    /* Plane number string                        */
   char      *ptr;            /* used to set up the PWL filename            */
   char      cur_name[CMLEN]; /* current composite view name                */

   /* Calculate the image data buffer size.  */
   buf_size = *nl * *ns;

   /* Calculate the scaler needed to reduce the image to the appropriate number
   of colors.  */
   if ((*xid.colormap_ptr == xid.colormap) && (*maxval < NUM_IMGCOLORS) &&
       (*nplanes == 1))
   {
      scaler = 1.0;
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
   }
   else if (*xid.colormap_ptr == xid.colormap)
   {
      scaler = (256.0 / (float)NUM_IMGCOLORS);       /* (2^8 / # of colors) */
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
   }
   else 
   {
      scaler = 1.0;
      num_imgcolors = 256;
      img_idx = 0;
   }

   /* 8-bit monitor -- black and white image.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) && (*nplanes == 1))
   {
      if (!flickerflg || (xid_flicker.flags & FLICKER_STOP) ||
          (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard))))
      {
         comptr = composite = (unsigned char *)malloc(buf_size);
         if (composite == NULL)
         {
            xid_message("Insufficient memory available");
            return(XID_F);
         }
      }

      /* Create a linear ramp.  */
      if (cmflg)
      {
         for (i = 0; i < num_imgcolors; i++)
         {
            if (!flickerflg)
            {
               xid.colortable_ptr[img_idx + i].blue = xid.colortable_ptr[
                  img_idx + i].green = xid.colortable_ptr[img_idx + i].red = 
                  (((float)i * scaler) * 257);
            }
            else
            {
               xid.colortable_ptr[img_idx + i].blue = 
                  xid_flicker.colortable[xid_flicker.current_image]
                  [img_idx + i].blue;
               xid.colortable_ptr[img_idx + i].green = 
                  xid_flicker.colortable[xid_flicker.current_image]
                  [img_idx + i].green;
               xid.colortable_ptr[img_idx + i].red = 
                  xid_flicker.colortable[xid_flicker.current_image]
                  [img_idx + i].red;
            }
            xid.color_linear[img_idx + i].blue =
               xid.color_linear[img_idx + i].green =
               xid.color_linear[img_idx + i].red = (((float)i * scaler) * 257);
            xid.color_linear[img_idx + i].flags =
               xid.colortable_ptr[img_idx + i].flags = DoRed | DoGreen | 
               DoBlue;
         }
      }

      /* Scale the input data.  */
      if (!flickerflg || (xid_flicker.flags & FLICKER_STOP) ||
          (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard))))
      {
         dataptr = rptr;
         for (i=0; i < buf_size; i++,comptr++,dataptr++)
         {
            if(!pixmapflg)
            {
               *comptr = (unsigned char)xid.colortable_ptr[(unsigned char)
                         img_idx + (unsigned char)((float)(*dataptr /
                         scaler))].pixel; 
            }
            else
            {
               *comptr = (unsigned char)xid_flicker.colortable[xid_flicker.
                         nplanes][(unsigned char)img_idx + (unsigned char)
                         ((float)(*dataptr / scaler))].pixel; 
            }
         }
      }

      /* Store the new colormap.  */
      if (cmflg)
      {
         XStoreColors(xid.display,*xid.colormap_ptr,
                      &xid.colortable_ptr[img_idx],num_imgcolors);
      }
   }

   /* 8-bit monitor -- RGB image.  */ 
   else if (xid.visual_info[0].visual->class == PseudoColor)
   {

      /* Allocate buffer space for LUTs.  */
      rlut = (unsigned char *)malloc(256);
      if (rlut == NULL)
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }
      glut = (unsigned char *)malloc(256);
      if (glut == NULL)
      {
         xid_message("Insufficient memory available");
         free(rlut);
         return(XID_F);
      }
      blut = (unsigned char *)malloc(256);
      if (blut == NULL)
      {
         xid_message("Insufficient memory available");
         free(rlut);
         free(glut);
         return(XID_F);
      }

      /* Compress the 3 band RGB image into a single band of image data and 3
      LUTs.  */
      comptr = composite = (unsigned char *)malloc(buf_size);
      if (composite == NULL)
      {
         xid_message("Insufficient memory available");
         free(rlut);
         free(glut);
         free(blut);
         return(XID_F);
      }
      rgbptr[RED] = rptr; 
      rgbptr[GREEN] = gptr; 
      rgbptr[BLUE] = bptr; 
      if (cmflg)
      {
         xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
                     num_imgcolors,&composite,rlut,glut,blut,0,rr,gr,br);

         /* Save this LUT to be used as reserved colors for loading of full
         resolution windows.  */
         for (i=0; i < 256; i++)
         {
            rr[i] = rlut[i];
            gr[i] = glut[i];
            br[i] = blut[i];
         }
      }
      else
      {
         if(pixmapflg)
         {
            xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
                        num_imgcolors,&composite,rlut,glut,blut,num_imgcolors,
                        xid_flicker.rr[xid_flicker.nplanes],
                        xid_flicker.gr[xid_flicker.nplanes],
                        xid_flicker.br[xid_flicker.nplanes]);
         }
         else
         {
            xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
                        num_imgcolors,&composite,rlut,glut,blut,num_imgcolors,
                        rr,gr,br);
         }
      }

      /* Build the new colormap.  */
      if (cmflg)
      {
          for (i = 0; i < num_imgcolors; i++)
          {
             if (!flickerflg)
             {
                xid.color_linear[img_idx + i].red = 
                   xid.colortable_ptr[img_idx + i].red = (rlut[i] * 257);
                xid.color_linear[img_idx + i].green =
                   xid.colortable_ptr[img_idx + i].green = (glut[i] * 257);
                xid.color_linear[img_idx + i].blue =
                   xid.colortable_ptr[img_idx + i].blue = (blut[i] * 257);
             }
             else
             {
                xid.color_linear[img_idx + i].red = (rlut[i] * 257);
                xid.color_linear[img_idx + i].green = (glut[i] * 257);
                xid.color_linear[img_idx + i].blue = (blut[i] * 257);
                xid.colortable_ptr[img_idx + i].red =
                   xid_flicker.colortable[xid_flicker.current_image]
                   [img_idx + i].red;
                xid.colortable_ptr[img_idx + i].green =
                   xid_flicker.colortable[xid_flicker.current_image]
                   [img_idx + i].green;
                xid.colortable_ptr[img_idx + i].blue =
                   xid_flicker.colortable[xid_flicker.current_image]
                   [img_idx + i].blue;
             }
             xid.color_linear[img_idx + i].flags =
                xid.colortable_ptr[img_idx + i].flags = DoRed | DoGreen 
                | DoBlue;
          }
      }
      if (!flickerflg || (xid_flicker.flags & FLICKER_STOP) ||
          (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard))))
      {
         if(!pixmapflg)
         {
            for (i=0; i < buf_size; i++)
            {
               composite[i] = (unsigned char)xid.colortable_ptr[img_idx +
                              composite[i]].pixel;
            }
         }
         else
         {
            for (i=0; i < buf_size; i++)
            {
               composite[i] = (unsigned char)xid_flicker.colortable[
                              xid_flicker.nplanes][img_idx +
                              composite[i]].pixel;
            }
         }
      }

      /* Store the new colormap.  */
      if (cmflg)
      {
         XStoreColors(xid.display,*xid.colormap_ptr,
                      &xid.colortable_ptr[img_idx],num_imgcolors);
      }
   }
   else if ((xid.visual_info[0].visual->class == DirectColor)&&(*nplanes == 1))
   {

      /* Allocate buffer space.  */
      if (!flickerflg || (xid_flicker.flags & FLICKER_STOP) ||
          (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard))))
      {
         data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
         if (data == NULL)
         {
            xid_message("Insufficient memory available");
            return(XID_F);
         }

         /* Set a pointer to the image data.  */
         rgbptr[RED] = rptr; 

         /* Create the transform array which scales output values such that 56
         colormap entries can be reserved as working entries.  */
         for (i = 0; i < 256; i++)
            tform[i] = (long)(((float)i / scaler) + (float)img_idx);

         /* Scale the input data.  */
         for (i = 0; i < buf_size; i++, rgbptr[RED]++)
         {
            data[i] = (tform[(*rgbptr[RED])] << xid.red_shift) +
                      (tform[(*rgbptr[RED])] << xid.green_shift) +
                      (tform[(*rgbptr[RED])] << xid.blue_shift);
         }
      }

      /* Assign a gray scale ramp to the image data colors.  */
      if (*xid.colormap_ptr == xid.colormap)
      {
         offset = (long)(0 - (scaler * (float)(NUM_COLORS + NUM_CSTORE +
                  NUM_WMCOLORS + NUM_WKCOLOR)));
      }
      else
         offset = 0;
      if (cmflg)
      {
         for (i = img_idx; i < num_imgcolors + img_idx; i++)
         {
            if (!flickerflg)
            {
               xid.colortable_ptr[i].red = xid.colortable_ptr[i].green = 
                  xid.colortable_ptr[i].blue = (unsigned short)((float)
                  (((float)i * scaler) + (float)offset)) << 8;
            }
            else
            {
               xid.colortable_ptr[i].red =
                  xid_flicker.colortable[xid_flicker.current_image][i].red;
               xid.colortable_ptr[i].green =
                  xid_flicker.colortable[xid_flicker.current_image][i].green;
               xid.colortable_ptr[i].blue  =
                  xid_flicker.colortable[xid_flicker.current_image][i].blue;
            }
            xid.color_linear[i].red = xid.color_linear[i].green = 
               xid.color_linear[i].blue = (unsigned short)((float)(((float)i *
               scaler) + (float)offset)) << 8;
            xid.color_linear[i].flags = 
            xid.colortable_ptr[i].flags = DoRed | DoGreen | DoBlue;
         }
         XStoreColors(xid.display,*xid.colormap_ptr,xid.colortable_ptr,256);
      }
   }
   else if (xid.visual_info[0].visual->class == DirectColor)
   {
      /* Allocate buffer space.  */
      if (!flickerflg || (xid_flicker.flags & FLICKER_STOP) ||
          (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard))))
      {
         data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
         if (data == NULL)
         {
            xid_message("Insufficient memory available");
            return(XID_F);
         }

         /* Set pointers to the red, green and blue bands of image data.  */
         rgbptr[RED] = rptr;
         rgbptr[GREEN] = gptr; 
         rgbptr[BLUE] = bptr; 

         /* Create the transform array which scales output values such that 56
         colormap entries can be reserved as working entries.  */
         for (i = 0; i < 256; i++)
            tform[i] = (long)(((float)i / scaler) + (float)img_idx);

         /* Scale the input data.  */
         for (i = 0; i < buf_size; i++, rgbptr[RED]++, rgbptr[GREEN]++, 
              rgbptr[BLUE]++)
         {
            data[i] = (tform[(*rgbptr[RED])]   << xid.red_shift) +
               (tform[(*rgbptr[GREEN])] << xid.green_shift) +
               (tform[(*rgbptr[BLUE])]  << xid.blue_shift);
         }
      }

      /* Assign a gray scale ramp to the image data colors.  */
      if (*xid.colormap_ptr == xid.colormap)
      {
         offset = (long)(0 - (scaler * (float)(NUM_COLORS + NUM_CSTORE +
                  NUM_WMCOLORS + 1)));
      }
      else
         offset = 0;
      if (cmflg)
      {
         for (i = img_idx; i < (img_idx + num_imgcolors); i++)
         {
            if (!flickerflg)
            {
               xid.colortable_ptr[i].red = xid.colortable_ptr[i].green = 
                  xid.colortable_ptr[i].blue = (unsigned short)((float)((float)
                  ((float)i * scaler) + (float)offset)) << 8;
            }
            else
            {
               xid.colortable_ptr[i].red =
                  xid_flicker.colortable[xid_flicker.current_image][i].red;
               xid.colortable_ptr[i].green =
                  xid_flicker.colortable[xid_flicker.current_image][i].green;
               xid.colortable_ptr[i].blue  =
                  xid_flicker.colortable[xid_flicker.current_image][i].blue;
            }
            xid.color_linear[i].red = xid.color_linear[i].green = 
               xid.color_linear[i].blue = (unsigned short)((float)(((float)i *
               scaler) + (float)offset)) << 8;
            xid.color_linear[i].flags = xid.colortable_ptr[i].flags =
               DoRed | DoGreen | DoBlue;
         }
         XStoreColors(xid.display,*xid.colormap_ptr,xid.colortable_ptr,256);
      }
   }

   /* Create a graphics context for the image window.  */
   gc_val.function = GXcopy;
   gc_val.plane_mask = AllPlanes;
   rgb_gc = XCreateGC(xid.display,imgwin,GCFunction | GCPlaneMask,&gc_val);

   /* Allocate buffer space.  */
   if (!flickerflg || (xid_flicker.flags & FLICKER_STOP) ||
       (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard))))
   {
      if ((xid.visual_info[0].visual->class == PseudoColor) &&
          (xid.visual_info[0].depth == 24))
      {
         data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
         if (data == NULL)
         {
            xid_message("Insufficient memory available");
            return(XID_F);
         }
         for (i=0; i < buf_size; i++)
            data[i] = composite[i];
      }

      /* Create an X image.  */
      if (xid.visual_info[0].depth == 24)
      {
         ximage = XCreateImage(xid.display,xid.visual_info[0].visual,
            xid.visual_info[0].depth,ZPixmap,0,(char *)data,*ns,*nl,8,0);
      }
      else
      {
         ximage = XCreateImage(xid.display,xid.visual_info[0].visual,
            xid.visual_info[0].depth,ZPixmap,0,(char *)composite,*ns,
            *nl,8,0);
      }

      /* Create a pixmap for the image window.  */
      *pixptr = XCreatePixmap(xid.display,imgwin,*ns,*nl,
                              xid.visual_info[0].depth);
      if (loadflg)
         xid_mem.imgpix = TRUE;

      /* Draw the image into the image pixmap.  */
      XPutImage(xid.display,*pixptr,rgb_gc,ximage,0,0,0,0,*ns,*nl);
   }

   /* Set the current width and height and the minimum and maximum width and
   height of the image window bulletinboard.  */
   XtSetArg(args[0],XmNwidth,*ns);
   XtSetArg(args[1],XmNheight,*nl);
   XtSetArg(args[2],XmNmaxWidth,*ns);
   XtSetArg(args[3],XmNmaxHeight,*nl);
   XtSetValues(win,args,4);
   XFlush(xid.display);
   XtResizeWidget(win,(unsigned short)*ns,(unsigned short)*nl,0);

   /* The following size checks and calculations take into an account an
   offset which represents the width and height of the scrollbar region.  */
   if (viewflg)
   {
      view_width = (Dimension)xid_mem.view_width;
      view_height = (Dimension)xid_mem.view_height;
   }
   else
   {
      XtVaGetValues(shell,XmNwidth,&view_width,XmNheight,&view_height,NULL); 
      view_width -= xid_mem.view_xoffset;
      view_height -= xid_mem.view_yoffset;
   }

   /* If the viewport width is larger than full image width, set the current
   image window width to the full image width.  */
   if (view_width > *ns)
      XtSetArg(args[0],XmNwidth,*ns + xid_mem.view_xoffset);
   else
   {
      /* Set the current image window width based on the viewport width.  */
      XtSetArg(args[0],XmNwidth,view_width+xid_mem.view_xoffset);
   }

   /* If the viewport height is larger than full image height, set the
      current image window height to the full image height.  */
   if (view_height > *nl)
      XtSetArg(args[1],XmNheight,*nl + xid_mem.view_yoffset);
   else
   {
      /* Set the current image window height based on the viewport height */
      XtSetArg(args[1],XmNheight,view_height+xid_mem.view_yoffset);
   }

   /* Set the maximum image window width and height based on the full image
   size.  */
   XtSetArg(args[2],XmNmaxWidth,*ns + xid_mem.view_xoffset);
   XtSetArg(args[3],XmNmaxHeight,*nl + xid_mem.view_yoffset);
   XtSetValues(shell,args,4);
   XFlush(xid.display);

   /* For main image window only, copy any graphics plane masks currently
   defined.  */
   if (win == main_image_bulletinboard)
   {

      /* Create the mask graphics context.  */
      mask_gc = XCreateGC(xid.display, XtWindow(Shell_MainMenu), 0, NULL);
      for (i=0; i < MASK_MAX; i++)
      {
         if (xid_mask.mask_pixmap[i] != 0)
         {
            XSetBackground(xid.display,mask_gc,xid.color_table[RGBBW_IDX + 
               xid_mask.color[i]].pixel);
            XSetForeground(xid.display,mask_gc,xid.color_table[RGBBW_IDX + 
               xid_mask.color[i]].pixel);
            XSetClipMask(xid.display,mask_gc,xid_mask.mask_pixmap[i]);
            XCopyPlane(xid.display,xid_mask.mask_pixmap[i],*pixptr,
               mask_gc,0,0,xid_mem.mem_width,xid_mem.mem_height,0,0,1);
         }
      }

      /* Free the graphics context.  */
      XFreeGC(xid.display, mask_gc);
   }

   if(!pixmapflg)
   {
      /* Set the image pixmap to be the background of the image window.  */
      XSetWindowBackgroundPixmap(xid.display,imgwin,*pixptr);

      /* Clear the window to update the background.  */
      XClearArea(xid.display,imgwin,0,0,0,0,True);
   }

   /* Store the appropriate information in the composite view structure and
   reset flags and values necessary when loading a new image.  */
   if (loadflg)
   {

      /* Reset pseudo-color flags and free color table.  */
      xid_psd.flags = (xid_psd.flags  & ~PSD_NOINIT) | PSD_INIT;
      if (xid_psd.color_psd != NULL)
      {
         free(xid_psd.color_psd);
         xid_psd.color_psd = (XColor *)NULL;
         XmTextSetString(psd_def_text,"");
      }

      /* Reset slope and offset and free adjusted color table and linear color
      table.  */
      xid_adjust.lut_slope = 45.0;
      xid_adjust.lut_offset = 0.0;
      if (xid_adjust.color_adjust != NULL)
      {
         free(xid_adjust.color_adjust);
         xid_adjust.color_adjust = (XColor *)NULL;
      }
      if (xid_adjust.color_linear != NULL)
      {
         free(xid_adjust.color_linear);
         xid_adjust.color_linear = (XColor *)NULL;
      }

      /* Determine whether or not the profile and cursor Z
         options are valid.  */
      if ((xid_profile.flags & PROF_ACTIVE) ||
          (xid_cursor.flags & CURSOR_ACTIVE))
      {
         for (i=0, count=0; i < xid.max_plane; i++)
         {
            if (xid_planes[i].flags & USED)
               count++;
            if (count < 2)
            {
               if (xid_profile.flags & PROF_ACTIVE)
                  XtSetSensitive(profile_z_button,False);
               if (xid_cursor.flags & CURSOR_ACTIVE)
                  XtSetSensitive(cursor_z_button,False);
            }
            else
            {
               if (xid_profile.flags & PROF_ACTIVE)
                  XtSetSensitive(profile_z_button,True);
               if (xid_cursor.flags & CURSOR_ACTIVE)
                  XtSetSensitive(cursor_z_button,True);
            }
         }
      }

      /* Set the composite structure fields to reflect the currently displayed
      image.  */
      xid.line_x = xid.line_y = -1;
      xid.view = 0;
      if (*nplanes >= 3)
      {
         xid_view.rgbplanes[RED] = rgbmem[RED];
         xid_view.rgbplanes[GREEN] = rgbmem[GREEN];
         xid_view.rgbplanes[BLUE] = rgbmem[BLUE];
      }
      else if (*nplanes == 1)
      {
         xid_view.rgbplanes[RED] = rgbmem[RED];
         xid_view.rgbplanes[GREEN] = rgbmem[RED];
         xid_view.rgbplanes[BLUE] = rgbmem[RED];
      }
      else if (*nplanes == 2)
      {
         xid_view.rgbplanes[RED] = rgbmem[RED];
         xid_view.rgbplanes[GREEN] = rgbmem[GREEN];
         xid_view.rgbplanes[BLUE] = rgbmem[GREEN];
      }

      /* If the zoom window is currently active, clear the window and free its
      resources (the zoomed information is no longer valid because a new image
      has been loaded).  */
      if (xid_zoom.flags & ZOOM_ACTIVE)
      {
         XSetWindowBackground(xid.display,XtWindow(zoom_win_bulletinboard),
            xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
         XClearWindow(xid.display,XtWindow(zoom_win_bulletinboard));
         if (xid_zoom.image_pixmap != 0)
         {
            XFreePixmap(xid.display,xid_zoom.image_pixmap);
            xid_zoom.image_pixmap = 0;
         }
         if (xid_zoom.image_data != NULL)
         {
            free(xid_zoom.image_data);
            xid_zoom.image_data = NULL;
         }
      }

      /* If the PWL window is currently initialized, re-create the ASCII
         filename. */
      if(pwl_bulletinboard != NULL)
      {
         strcpy(filename, xid_load.image_name);
         ptr = strchr(filename, '(');
         if(ptr)
            *ptr = '\0';
         ptr = strchr(filename, '/');
         if(!ptr)
            ptr = filename;
         ptr = strrchr(ptr, '.');
         if(ptr)
            *ptr = '\0';

         /* Add the appropriate extension to the filename, if needed */
         if (XmToggleButtonGetState(pwl_red_button))
            strcat(filename, ".enr");
         else if(XmToggleButtonGetState(pwl_green_button))
            strcat(filename, ".eng");
         else if(XmToggleButtonGetState(pwl_blue_button))
            strcat(filename, ".enb");

         /* Assign the new filename to the text area of the widget */
         XmTextSetString(pwl_file_text,filename);
      }
         
      /* If the full resolution window is currently active, clear the window
         and free its resources (the full resolution information is no longer
         valid because a new image has been loaded). Don't do this if
         load_display is being called by the flicker_stop routine */
      if ((xid_fullres != NULL) && !(xid_flicker.flags & FLICKER_STOP) &&
          !(xid_flicker.flags & FLICKER_STEP))
      {
         XSetWindowBackground(xid.display,XtWindow(fullres_win_bulletinboard),
            xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
         XClearWindow(xid.display,XtWindow(fullres_win_bulletinboard));

         /* Before freeing the full resolution structure, save the current size
         information.  */
         state = XmToggleButtonGetState(fullres_256_button);
         if (state)
         {
            strcpy(fullres_opt,"256");
            widget = fullres_256_button;
         }
         else
         {
            state = XmToggleButtonGetState(fullres_512_button);
            if (state)
            {
               strcpy(fullres_opt,"512");
               widget = fullres_512_button;
            }
            else
            {
               state = XmToggleButtonGetState(fullres_full_button);
               if (state)
               {
                  strcpy(fullres_opt,"FS");
                  widget = fullres_full_button;
               }
               else
               {
                  state = XmToggleButtonGetState(fullres_user_button);
                  if (state)
                  {
                     strcpy(fullres_opt,"US");
                     widget = fullres_user_button;
                  }
               }
            }
         }
         for (i=0; i < xid.num_fullres; i++)
         {
            if (xid_fullres[i].image_pixmap != 0)
            {
               XFreePixmap(xid.display,xid_fullres[i].image_pixmap);
               xid_fullres[i].image_pixmap = 0;
            }
            if (xid_fullres[i].image_data != NULL)
            {
               free(xid_fullres[i].image_data);
               xid_fullres[i].image_data = NULL;
            }
         }
         free(xid_fullres);
         xid_fullres = (struct XID_FULLRES *)NULL;

         /* Set appropriate variables and flags and allocate structures for full
         resolution window.  */
         fullres_setup();
         fullres_size(widget,fullres_opt,NULL);
      }
      if (!flickerflg || (xid_flicker.flags & FLICKER_STOP) ||
          (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard))))
      {

         /* If the show panel is open, update the plane list.  */
         show_flg = XtIsRealized(Shell_Show);
         if (show_flg)
         {
            xid_image_makelist(show_redlist,show_greenlist,show_bluelist,
                               show_savelist,TRUE);
            XmListDeselectAllItems(show_redlist);
            XmListDeselectAllItems(show_greenlist);
            XmListDeselectAllItems(show_bluelist);
            XmListDeselectAllItems(show_savelist);

            /* Highlight the save list items that match the current plane
               selection */
            xid_get_composite(&tmp, &num_composite);
            for(i = 0; i < num_composite; i++)
            {
               same_flg = TRUE;
               strcpy(cur_name, xid_composite[i].name);
               xmstring = XmStringCreateLtoR(cur_name,
                                             XmSTRING_DEFAULT_CHARSET);
               for(j = 0; j < 3; j++)
               {
                  if(xid_composite[i].rgbplanes[j] != xid_view.rgbplanes[j])
                     same_flg = FALSE;
               }
               if(same_flg)
               {
                  XmListSelectItem(show_savelist, xmstring, True);
                  found = TRUE;
                  break;
               }
               XmStringFree(xmstring);
            }
            if(!found)
            {
               strcpy(cur_name, "Current");
               xmstring = XmStringCreateLtoR(cur_name,
                                             XmSTRING_DEFAULT_CHARSET);
               XmListSelectItem(show_savelist, xmstring, True);
               XmStringFree(xmstring);
            }

            /* Highlight the current plane selections and set the show panel
               plane values appropriately */
            for(i = 0; i < 3; i++)
            {
               sprintf(planestr,"Plane %ld",xid_view.rgbplanes[i]);
               xmstring = XmStringCreateLtoR(planestr,
                                             XmSTRING_DEFAULT_CHARSET);
               if(i == 0)
               {
                  XmListSelectItem(show_redlist,xmstring,True);
                  xid_show.planes[RED] = xid_view.rgbplanes[i];
               }
               else if(i == 1)
               {
                  XmListSelectItem(show_greenlist,xmstring,True);
                  xid_show.planes[GREEN] = xid_view.rgbplanes[i];
               }
               else
               {
                  XmListSelectItem(show_bluelist,xmstring,True);
                  xid_show.planes[BLUE] = xid_view.rgbplanes[i];
               }
               XmStringFree(xmstring);
            }
         }

         /* If the flicker panel is open, update the plane list.  */
         if (xid_flicker.flags & FLICKER_ACTIVE)
         {
            xid_image_makelist(flicker_redlist,flicker_greenlist,
                               flicker_bluelist,flicker_savelist,FALSE);
         }
      }

      /* If the profile window is active, clear the current profile and redraw
         the profile axis.  This needs to be done because the newly loaded
         image may not be the same size as the previous image.  */
      if (xid_profile.flags & PROF_ACTIVE)
      {

         /* Free the appropriate resources.  */
         XFreeGC(xid.display,xid_profile.gc);
         XFreePixmap(xid.display,xid_profile.info_pixmap);
         XFreePixmap(xid.display,xid_profile.clear_pixmap);
         free(xid_profile.points[RED]);
         free(xid_profile.points[GREEN]);
         free(xid_profile.points[BLUE]);

         /* Clear the window and create the new profile.  */
         XClearWindow(xid.display,XtWindow(profile_scrollwin_bulletinboard));
         profile_create();
         XCopyArea(xid.display,xid_profile.info_pixmap,
                   XtWindow(profile_scrollwin_bulletinboard),xid_profile.gc,0,
                   0,xid_profile.width,xid_profile.height,0,0);
      }

      /* If the power spectrum window is active, clear the current plot and
      redraw.  This needs to be done because the newly loaded image may not be
      the same size as the previous image.  */
      if (xid_power.flags & POWER_ACTIVE)
      {

         /* Free the appropriate resources.  */
         XFreeGC(xid.display,xid_power.gc);
         XFreePixmap(xid.display,xid_power.info_pixmap);
         XFreePixmap(xid.display,xid_power.clear_pixmap);

         free(xid_power.ent_points);
         free(xid_power.per_points);

         xid_power.maxdb = (float)(20.0 * log10((double)(255.0 * 255.0 * 
                                                         (*ns))) + 10);
         xid_power.per_npts = (long)(nearest_power2(*ns) / 2.0);
         sprintf(points,"%ld",xid_power.per_npts);
         XmTextFieldSetString(power_ppoints_text,points);

         /* If both the entropy plot and the periodogram are being displayed,
         force the maximum entropy start frequency, end frequency and number of
         points to be the same as for the periodogram.  */
         if (xid_power.flags & POWER_PER)
         {
            xid_power.ent_npts = xid_power.per_npts;
            sprintf(points,"%ld",xid_power.per_npts);
            XmTextFieldSetString(power_epoints_text,points);
         }
         else
         {
            xid_power.ent_npts = (long)(*ns / 2.0);
            sprintf(points,"%ld",xid_power.ent_npts);
            XmTextFieldSetString(power_epoints_text,points);
         }

         /* Clear the window and create the new plot.  */
         XClearWindow(xid.display,XtWindow(power_scrollwin_bulletinboard));
         power_create();
         XCopyArea(xid.display,xid_power.info_pixmap,
                   XtWindow(power_scrollwin_bulletinboard),xid_power.gc,0,0,
                   xid_power.width,xid_power.height,0,0);
      }
   }

   /* Free the appropriate resources.  */
   if ((xid.visual_info[0].visual->class == DirectColor) && 
       (!flickerflg || (xid_flicker.flags & FLICKER_STOP) ||
        (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard)))))
      free(data);
   else if (xid.visual_info[0].visual->class == PseudoColor)
   {
      if ((!flickerflg || (xid_flicker.flags & FLICKER_STOP)) ||
          (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard))) ||
          (flickerflg && *nplanes == 3))
         free(composite);
      if (*nplanes != 1)
      {
         free(rlut);
         free(glut);
         free(blut);
      }
   }

   /* The data in the image structure has already been freed above.  */
   if (!flickerflg || (xid_flicker.flags & FLICKER_STOP) ||
       (flickerflg && (imgwin == XtWindow(fullres_win_bulletinboard))))
   {
      ximage->data = NULL;
      XDestroyImage(ximage);
   }
   XFreeGC(xid.display,rgb_gc);
   return(XID_S);
}

void nonlas_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine closes and destroys the load nonLAS image panel and sets the 
   file selection box to sensitive.
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
******************************************************************************/
{
   /* Close the image information panel and set the nonLAS load button to
   sensitive.  */
   XtPopdown(load_nonlas_dialog);
   if(!xid_load.init_load)
      XtSetSensitive(load_fileselector,True);
   else
      xid_load.init_load = FALSE;
}

/* ARGSUSED */
void nonlas_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   Retrieve the image information specified by the user and verify that
   the appropriate information was specified.  Close the nonLAS dialog
   and call the routine to load the image to the display.
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
******************************************************************************/
{
   long  state,          /* Current toggle button state.             */
         startline,      /* Starting line number.                    */
         startsamp,      /* Starting sample number.                  */
         displines,      /* Number of lines to display.              */
         dispsamples;    /* Number of samples to display.            */
   char  *tmpstr,        /* Temporary storage for a text string.     */
         *fname;         /* Temporary storage for the image name.    */

   /* Retrieve the number of header bytes, number of bands, number of lines,
   and number of samples in the image.  */
   tmpstr = XmTextFieldGetString(nonlas_sizehdr_text);
   if (*tmpstr == '\0')
   {
      xid_message("User must specify number of header bytes");
      return;
   }
   sscanf(tmpstr,"%ld",&xid_nonlas.header_bytes);
   XtFree(tmpstr);

   tmpstr = XmTextFieldGetString(nonlas_bands_text);
   if (*tmpstr == '\0')
   {
      xid_message("User must specify number of bands");
      return;
   }
   sscanf(tmpstr,"%ld",&xid_nonlas.nbands);
   if (xid_nonlas.nbands == 0)
   {
      xid_message("Image may not contain zero bands");
      return;
   }
   XtFree(tmpstr);

   tmpstr = XmTextFieldGetString(nonlas_lines_text);
   if (*tmpstr == '\0')
   {
      xid_message("User must specify number of lines");
      return;
   }
   sscanf(tmpstr,"%ld",&xid_nonlas.lines);
   if (xid_nonlas.lines == 0)
   {
      xid_message("Image may not contain zero lines");
      return;
   }
   XtFree(tmpstr);

   tmpstr = XmTextFieldGetString(nonlas_samples_text);
   if (*tmpstr == '\0')
   {
      xid_message("User must specify number of samples");
      return;
   }
   sscanf(tmpstr,"%ld",&xid_nonlas.samples);
   if (xid_nonlas.samples == 0)
   {
      xid_message("Image may not contain zero samples");
      return;
   }
   XtFree(tmpstr);

   /* Retrieve the display information: starting line and sample, number of
   lines and samples, and bands to display if the user did not enter these
   values with the image name.  */
   if (strstr(xid_load.image_name, "(") == NULL)
   {
      tmpstr = XmTextFieldGetString(nonlas_displines_text);
      if (*tmpstr == '\0')
         displines = xid_nonlas.lines;
      else
         sscanf(tmpstr,"%ld",&displines);
      if (displines < 1)
      {
         xid_message("Number of lines to display must be greater than zero");
         return;
      }
      XtFree(tmpstr);

      tmpstr = XmTextFieldGetString(nonlas_dispsamples_text);
      if (*tmpstr == '\0')
         dispsamples = xid_nonlas.samples;
      else
         sscanf(tmpstr,"%ld",&dispsamples);
      if (dispsamples < 1)
      {
         xid_message("Number of samples to display must be greater than zero");
         return;
      }
      XtFree(tmpstr);

      tmpstr = XmTextFieldGetString(nonlas_startline_text);
      if (*tmpstr == '\0')
         startline = 1;
      else
         sscanf(tmpstr,"%ld",&startline);
      if (startline < 1)
      {
         xid_message("Starting line number must be greater than zero");
         return;
      }
      XtFree(tmpstr);

      tmpstr = XmTextFieldGetString(nonlas_startsamp_text);
      if (*tmpstr == '\0')
         startsamp = 1;
      else
         sscanf(tmpstr,"%ld",&startsamp);
      if (startsamp < 1)
      {
         xid_message("Starting sample number must be greater than zero");
         return;
      }
      XtFree(tmpstr);

      tmpstr = XmTextFieldGetString(nonlas_dispband_text);
      if (*tmpstr == '\0')
         sprintf(tmpstr,"1-%ld",xid_nonlas.nbands); 

      /* Convert the image name entered by the user to LAS format with the
      parameters specified in the nonlas dialog popup.  */
      fname = malloc(sizeof(char)*256);
      sprintf(fname,"%s(%ld %ld %ld %ld:%s)",xid_load.image_name,startline,
              startsamp,displines,dispsamples,tmpstr);
      strcpy(xid_load.image_name, fname);
      free(fname);
   }
   XtFree(tmpstr);

   /* Determine the data type of the image being loaded. Since the
      xid_load structure is just temporary, used to pass values on to
      load_localok and loadimg, consistently place the data type for the
      image being loaded into index 0. loadimg will make sure to retrieve
      it again from this same location when loading new imagery into memory
      and will then save the data type along with the rest of the information
      about this imagery in the appropriate index of the xid_planes. */
   state = XmToggleButtonGetState(nonlas_byte_button);
   if (state)
      xid_load.dtype[0] = BYTE;
   else
   {
      state = XmToggleButtonGetState(nonlas_i2_button);
      if (state)
         xid_load.dtype[0] = I2;
      else
      {
         state = XmToggleButtonGetState(nonlas_i4_button);
         if (state)
            xid_load.dtype[0] = I4;
         else
         {
            state = XmToggleButtonGetState(nonlas_real_button);
            if (state)
               xid_load.dtype[0] = REAL;
         }
      }
   }

   state = XmToggleButtonGetState(nonlas_bsq_button);
   if (state)
      xid_nonlas.format = BSQ;
   else
      xid_nonlas.format = BIL;

   /* Load the image.  */
   load_localok(w, client, call);
}

/* ARGSUSED */
void nonlas_next
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine is used to switch activity to the input widget.  This is
   used to allow the user to move from one text field to the next by
   simply pressing the "Enter" key (rather than having to click the
   left mouse button in the desired field).
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
******************************************************************************/
{
   /* Move to the next item.  */
   XmProcessTraversal(w,XmTRAVERSE_NEXT_TAB_GROUP);
}
 
/* ARGSUSED */
void nonlas_dialog
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine is used to pop up the panel requesting image information
   for a nonLAS image (information such as number of lines, number of
   samples, number of bands, etc.).
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
******************************************************************************/
{
   Position
          popx,       /* X position of pop-up panel.         */
          popy;       /* Y position of pop-up panel.         */
   char   *filename;  /* Input specification.                */
   static int
          first_time; /* Create window one time.             */
   /*SUPPRESS 594*/
   XmFileSelectionBoxCallbackStruct *acs =
      (XmFileSelectionBoxCallbackStruct*)call;

   /* Retrieve the file name specified and save the current directory mask.  */
   if(!xid_load.init_load)
   {
      XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&filename);
      strcpy(xid_load.image_name,filename);
      if (xid_load.dirmask != NULL)
         XmStringFree(xid_load.dirmask);
      xid_load.dirmask = XmStringCopy(acs->mask);
      XtFree(filename);
   }

   /* Create the load nonLAS images popup.  */
   if (first_time == 0)
   {
      Createload_nonlas(Shell_Load);
      first_time = 1;
   }

   /* Translate the X/Y position relative to the file selection box to an X/Y
   position relative to the screen.  */
   if(!xid_load.init_load)
      XtTranslateCoords(load_form,(Position)0,(Position)0,&popx,&popy);
   else
   {
      /* Translate the X/Y position relative to the main image window to an
         X/Y position relative to the screen */
      XtTranslateCoords(main_image_scrollwin, (Position)0, (Position)0, &popx,
                        &popy);
   }

   /* Position the pop-up panel.  */
   XtMoveWidget(load_nonlas_dialog,popx,popy);

   /* Create the load nonLAS images information panel.  */
   if (xid_load.type == NONLAS) 
   {
      XtPopup(load_nonlas_dialog,XtGrabNone);
      if(!xid_load.init_load)
         XtSetSensitive(load_fileselector,False);
      XmTextFieldSetString(nonlas_startline_text, "");
      XmTextFieldSetString(nonlas_startsamp_text, "");
      XmTextFieldSetString(nonlas_displines_text, "");
      XmTextFieldSetString(nonlas_dispsamples_text, "");
      XmTextFieldSetString(nonlas_dispband_text, "");
   }
}

int load_makelas
(
   char *inspec,               /* Input specification. (I)                   */
   char *lasspec               /* LAS specification. (O)                     */
)

/******************************************************************************
Purpose:
   This routine takes as input a character string containing host image names 
   and window and/or band specifications.  This is converted to a string
   containing LAS names as well as the window and/or band specifications.
    
By L. Huewe, USGS/EROS Data Center, 10/93

Algorithm Description:
******************************************************************************/
{
   long    paren;            /* Position of left paren.                      */
   long    pflag=FALSE;      /* Indicates whether or not the current image
                                name has been processed.                     */
   char    lasname[CMLEN];   /* LAS name specification.                      */
   char    *ptr;             /* Pointer to input string.                     */
   char    *tmp_ptr;         /* Pointer to temporary string.                 */
   char    tmpname[CMLEN];   /* Temporary string for holding a single
                                image name.                                  */
   char    hostname[CMLEN];  /* Temporary string for holding a single        */
   int     exp_path(char*,char*);
                             /* Function to expand a relative host file name
                                to a fully specified host file name          */

   /* Initialize strings.  */
   strcpy(lasspec,"");
   strcpy(tmpname,"");

   /* Examine each character in the input string.  */
   for (ptr=inspec,tmp_ptr=tmpname; *ptr != '\0'; ptr++)
   {

      /* Just copy the character to a temporary string if it is not a "+" or a
      "(" (i.e., it must be part of the image name).  */
      if ((*ptr != '+') && (*ptr != '('))
      {
         *tmp_ptr = *ptr;
         tmp_ptr++;
      }

      /* A left paren indicates a window and/or band specification.  Convert
      the host name contained in the temporary string to a LAS name and
      concatenate it to the output string.  Also, concatenate the window and/or
      band specification to the output string.  */
      else if (*ptr == '(')
      {
         *tmp_ptr = '\0';
       
         /* Check the format of the given host file name and convert it to an
            absolute file name if a relative file name was given. */
         if(exp_path(tmpname, hostname) != XID_S)
            return (XID_F);
         strcpy(lasname, hostname);
         strcat(lasspec,lasname);
         paren = (long)strcspn(ptr,")");
         strncat(lasspec,ptr,paren+1);
         ptr += paren;
         pflag = TRUE;
      }

      /* A "+" indicates another image name follows.  Convert the current image
      name to a LAS name if it has not already been done and concatenate the
      resulting LAS name and a "+" to the output string.  */
      else if (*ptr == '+')
      {
         if (!pflag)
         {
            *tmp_ptr = '\0';

            /* Check the format of the given host file name and convert it to an
               absolute file name if a relative file name was given. */
            if(exp_path(tmpname, hostname) != XID_S)
               return (XID_F);
            strcpy(lasname, hostname);
            strcat(lasspec,lasname);
         }
         strcat(lasspec,"+");
         strcpy(tmpname,"");
         tmp_ptr = tmpname;
         pflag = FALSE;
      }
   }

   /* If the current image name has not been processed, convert it to a LAS
   name and append it to the output string.  */
   if (!pflag)
   {
      *tmp_ptr = '\0';

      /* Check the format of the given host file name and convert it to an
         absolute file name if a relative file name was given. */
      if(exp_path(tmpname, hostname) != XID_S)
         return (XID_F);
      strcpy(lasname, hostname);
      strcat(lasspec,lasname);
   }
   return E_SUCC;
}

int exp_path
(
   char *user_name,             /* Given filename. (I)                       */
   char *hostname               /* Fully specified host filename. (O)        */
)

/******************************************************************************
Purpose:
   This routine takes as input a character string containing the user-specified
   host image name without the window and/or band specifications.  This is
   converted to a fully specified host filename if the user did not specify it
   as such. 
    
By L. Huewe, USGS/EROS Data Center, 10/93

Algorithm Description:
******************************************************************************/
{
   char *char_ptr;        /* Pointer used to parse the given file name       */

   char_ptr = user_name;

   /* If user provided a relative path, convert it to an absolute file name.
      We are assuming if the file name begins with a '/', they have specified
      an absolute path and we don't need to manipulate it. */
   if(*char_ptr != '/')
   {
      /* Get the current working directory and set the beginning of the
         hostname equal to it */
      getcwd(hostname, CMLEN);

      if(*char_ptr == '.' && *++char_ptr == '/')
      {
         /* Make sure adding on the path for the working directory doesn't make
            our file name length > CMLEN */
         if((strlen(hostname) + strlen(char_ptr)) > CMLEN)
         {
            xid_message("Expanded file name exceeds allowable length (256).");
            xid_load.init_load = FALSE;
            return (XID_F);
         }

         /* Now append the relative host name to the path to the current
            working directory for a fully specified host name */
         strcat(hostname, char_ptr);
      }
      else
      {
         /* Make sure adding on the path for the working directory doesn't make
            our file name length > CMLEN */
         if((strlen(hostname) + strlen(char_ptr)) > CMLEN)
         {
            xid_message("Expanded file name exceeds allowable length (256).");
            xid_load.init_load = FALSE;
            return (XID_F);
         }

         /* Simply prepend the current working directoy to the file name for
            a fully specified host name */
         strcat(hostname, "/");
         strcat(hostname, user_name);
      }
   }

   /* Otherwise, just return the file name given */
   else
      strcpy(hostname, user_name);
   return (XID_S);
}
