/*
 *  This is the map function source file for the map panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 *  Modified by A. Shimitz 6/00
 *   - Modified the pwl_apply function to treat a multi-band image, of
 *     which only one band is being viewed, as the function would
 *     treat a single-band image
 *  Modified by A. Shimitz 8/00
 *   - Made each mapping function set the xid.mapped and xid.mapped_mod flags
 *     to TRUE each time a mapping is applied. Also modified the cancel
 *     routines so they deallocate the mapping color table, set it to NULL, and
 *     refresh the image display. (CR #3892)
 *
 */
#include <math.h>
#include <ctype.h> /* isspace definition */
#include <fcntl.h> /* creat definition */
#include <X11/IntrinsicP.h>
#include "labserv.h" /* c_lsmknm & c_lsstat definitions */
#include "xid.h"
#include "typlim.h"
#include "xid_func.h"
#include "xid_create.h"
#include "main.h"

/* local function prototypes */
void savemap_dltfw
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void loadmap_dltfr
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void pwl_load
(
   char *filename             /* ASCII file name.                           */
);

void pwl_confirmation
(
   Widget w,
   XtPointer client,
   XtPointer call
);
/* end of prototypes */

/******************************************************************************
Purpose:
   This routine sets the appropriate flag indicating which component the
   user wishes to map (i.e. red, green, blue or all).

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void map_component
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   char *data = (char *)client;

   /* Set the flag indicating which image component to map.  */
   xid_map.flags = xid_map.flags & ~(MAP_RED | MAP_GREEN | 
                   MAP_BLUE | MAP_ALL);
   switch (*data)
   {
      case 'R':
         xid_map.flags = xid_map.flags | MAP_RED;
         break;
      case 'G':
         xid_map.flags = xid_map.flags | MAP_GREEN;
         break;
      case 'B':
         xid_map.flags = xid_map.flags | MAP_BLUE;
         break;
      case 'A':
         xid_map.flags = xid_map.flags | MAP_ALL;
         break;
   }
}

/******************************************************************************
Purpose:
   This routine closes the appropriate map window, sets the main menu mapping
   options to sensitive and resets the color of the working color cell.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void map_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   long    i,                  /* Looping index.                             */
           img_idx,            /* Index into color table of the first image
                                  color.                                     */
           num_imgcolors,      /* Number of image colors.                    */
           start;              /* Starting index into color table.           */

   /* Assign number of colors to store and the starting index of the image
   colors based on the type of visual.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) &&
       (*xid.colormap_ptr == xid.colormap))
   {
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
      start = 0;  
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) &&
            (*xid.colormap_ptr == xid.colormap))
   {
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = start = IMG_IDX;
   }
   else
   {
      num_imgcolors = 256;
      img_idx = start = 0;
   }

   /* If a map subpanel has been opened, close it too.  */
   if (xid_map.subpanel != NULL)
   {
      if (xid_map.subpanel == Shell_Pseudo)
         XtUnmanageChild(Psd_bulletinboard);
      else if (xid_map.subpanel == Shell_Adjust)
         XtUnmanageChild(Adjust_form);
      else if (xid_map.subpanel == Shell_PWL)
         XtUnmanageChild(Pwl_bulletinboard);
      else if (xid_map.subpanel == Shell_SaveMap)
      {
         XtUnmanageChild(Savemap_form);
         XtDestroyWidget(saveload_dltf_popup);
      }
      else if (xid_map.subpanel == Shell_LoadMap)
      {
         XtUnmanageChild(Loadmap_form);
         XtDestroyWidget(saveload_dltf_popup);
      }
      XtPopdown(xid_map.subpanel);

      /* Save the current mapping from the adjust function.  */
      if (xid_map.subpanel == Shell_Adjust)
      {
         if (xid_adjust.color_adjust != NULL)
         {
            for (i=0; i < num_imgcolors; i++)
               xid.colortable_ptr[img_idx + i] = 
                                            xid_adjust.color_adjust[start + i];
            free(xid_adjust.color_adjust);
            xid_adjust.color_adjust = (XColor *)NULL;
         }
         if (xid_adjust.color_linear != NULL)
         {
            free(xid_adjust.color_linear);
            xid_adjust.color_linear = (XColor *)NULL;
         } 
         xid_adjust.lut_slope = 45.0;
         xid_adjust.lut_offset = 0.0;

         /* Reset the slider values.  */
         XtVaSetValues(adjust_contrast_slider,XmNvalue,45,NULL);
         XtVaSetValues(adjust_bright_slider,XmNvalue,0,NULL);
      }

      /* Save the current mapping from the pseudo-color function and free the
      appropriate resources.  */
      if (xid_map.subpanel == Shell_Pseudo)
      {
         if (xid_psd.color_psd != NULL)
         {
            for (i=0; i < num_imgcolors; i++)
               xid.colortable_ptr[img_idx + i] = xid_psd.color_psd[start + i];
            free(xid_psd.color_psd);
            xid_psd.color_psd = (XColor *)NULL;
         }
         if (xid_psd.savecolors != NULL)
         {
            free(xid_psd.savecolors);
            xid_psd.savecolors = (XColor *)NULL;
         }
         if (xid_psd.range != NULL)
         {
            free(xid_psd.range);
            xid_psd.range = NULL;
         }
         if (xid_psd.bin != NULL)
         {
            free(xid_psd.bin);
            xid_psd.bin = NULL;
         }
         if (xid_psd.text_list != NULL)
            XtFree(xid_psd.text_list);
         xid_psd.flags = (xid_psd.flags  & ~PSD_NOINIT) | PSD_INIT;
      }

      /* Save the current mapping from the piece-wise linear mapping function*/
      if (xid_map.subpanel == Shell_PWL)
      {
         if (xid_pwl.color_pwl != NULL)
         {
            for (i=0; i < num_imgcolors; i++)
               xid.colortable_ptr[img_idx + i] = xid_pwl.color_pwl[start + i];
            free(xid_pwl.color_pwl);
            xid_pwl.color_pwl = (XColor *)NULL;
         }
      }
   }

   /* Set the mapping options to sensitive.  */
   XtSetSensitive(main_map_shell,True);

   /* Reset the color of the working colormap cell.  This colormap cell is used
   for such things as displaying the currently selected color in the
   pseudo-color function.  */
   xid.color_table[WORK_IDX].red   = xid.color_workclear.red;
   xid.color_table[WORK_IDX].green = xid.color_workclear.green;
   xid.color_table[WORK_IDX].blue  = xid.color_workclear.blue;
   xid.color_table[WORK_IDX].flags = xid.color_workclear.flags;
   XStoreColor(xid.display,xid.colormap,&xid.color_table[WORK_IDX]);

   /* Reset the subpanel field, indicating no map subpanel is currently
   active.  */
   xid_map.subpanel = (Widget)NULL;
}

/******************************************************************************
Purpose:
   This routine determines which mapping function the user has selected and
   displays the appropriate panel indicating the options for that function.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void map_func
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   static  long    band;          /* Band number pseudo-color mapping
                                     function was last run against.          */
   static  char    image[CMLEN];  /* Image name pseudo-color mapping function
                                     was last run against.                   */
   long            plane_idx;     /* Current memory plane index.             */

   /* Print a warning and return if no image data is currently displayed.  */
   if (xid.view == -1)
   {
      xid_message("No image data currently being viewed");
      return;
   }

   /* Determine the mapping function selected and display the appropriate
   subpanel.  */
   if (strcmp(client,"ADJ") == 0) 
   {
      if (adjust_form == NULL)
         Adjust_form = Createadjust_form(Shell_Adjust);
      XtManageChild(Adjust_form);
      XtPopup(Shell_Adjust,XtGrabNone);
      xid_map.subpanel = Shell_Adjust;
      xid_adjust.color_adjust = (XColor *)NULL;
      xid_adjust.color_linear = (XColor *)NULL;
   }
   else if (strcmp(client,"PWL") == 0)
   {
      if (pwl_bulletinboard == NULL)
         Pwl_bulletinboard = Createpwl_bulletinboard(Shell_PWL);
      XtManageChild(Pwl_bulletinboard);
      XtPopup(Shell_PWL,XtGrabNone);
      xid_map.subpanel = Shell_PWL;
      xid_pwl.color_pwl = (XColor *)NULL;
   }
   else if (strcmp(client,"PSD") == 0)
   {
      if (psd_bulletinboard == NULL)
      {
         Psd_bulletinboard = Createpsd_bulletinboard(Shell_Pseudo);
         strcpy(image,"");
         band = 0;
      }

      /* Determine whether or not a new image has been displayed since the last
      time PSD was run.  */
      plane_idx = xid_view.rgbplanes[RED] - 1;
      if ((strcmp(xid_planes[plane_idx].filename,image) != 0) || 
          (xid_planes[plane_idx].band != band))
      {
         XmTextSetString(psd_def_text,"");
         strcpy(image,xid_planes[plane_idx].filename);
         band = xid_planes[plane_idx].band;
      }

      /* Make sure all scale values are 0, the range text has been cleared, and
      no color names are selected in the color list.  */
      XmScaleSetValue(psd_red_slider,0);
      XmScaleSetValue(psd_green_slider,0);
      XmScaleSetValue(psd_blue_slider,0);
      XmTextSetString(psd_range_text,"");
      XmListDeselectAllItems(psd_scrolledlist_list);
      XtManageChild(Psd_bulletinboard);
      XtPopup(Shell_Pseudo,XtGrabNone);

      /* Make sure "color list" is displayed if appropriate.  */
      if (xid_psd.flags & PSD_LIST)
         psd_func(psd_clist_button,"CL",NULL);
      xid_map.subpanel = Shell_Pseudo;
      xid_psd.text_list = (char *)NULL;
      xid_psd.range = xid_psd.bin = (long *)NULL;
      xid_psd.savecolors = (XColor *)NULL;
   }
   else if (strcmp(client,"SM") == 0)
   {
      if (savemap_form == NULL)
         Savemap_form = Createsavemap_form(Shell_SaveMap);
      Createsaveload_popup(Shell_SaveMap);
      XmTextSetString(savemap_text,"");
      XmTextSetString(savemap_text2,"");
      XtManageChild(Savemap_form);
      XtPopup(Shell_SaveMap,XtGrabNone);
      xid_map.subpanel = Shell_SaveMap;
   }
   else if (strcmp(client,"LM") == 0)
   {
      if (loadmap_form == NULL)
         Loadmap_form = Createloadmap_form(Shell_LoadMap);
      Createsaveload_popup(Shell_LoadMap);
      XmTextSetString(loadmap_text,"");
      XtManageChild(Loadmap_form);
      XtPopup(Shell_LoadMap,XtGrabNone);
      xid_map.subpanel = Shell_LoadMap;
   }

   /* Make the mapping options insensitive.  */
   XtSetSensitive(main_map_shell,False);
}

/******************************************************************************
Purpose:
   This routine is called whenever the user selects the LINEAR button from
   any of the mapping functions.  The current colormap is set to a linear
   ramp.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void map_linear
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   XColor  *beg_ptr,          /* Pointer to the start of the appropriate 
                                 XColor structure.                           */
           *work_ptr;         /* Working pointer to appropriate XColor
                                 structure.                                  */
   int     i,                 /* Looping index.                              */
           img_idx,           /* Starting colormap index for image colors.   */
           ncolors,           /* Number of colors to store in colormap.      */
           nplanes,           /* Number of planes displayed.                 */
           num_imgcolors,     /* Number of image colors.                     */
           start;             /* Starting index into color table.            */

   /* Assign number of colors to store and the starting index of the image
   colors based on the type of visual.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) &&
       (*xid.colormap_ptr == xid.colormap))
   {
      start = 0;  
      ncolors = NUM_IMGCOLORS;
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) &&
            (*xid.colormap_ptr == xid.colormap))
   {
      start = IMG_IDX;
      ncolors = 256;
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
   }
   else
   {
      ncolors = 256;
      start = 0;
      num_imgcolors = 256;
      img_idx = 0;
   }

   /* If this was called from the pseudo-color panel, save the appropriate
   colormap entries for the "UNDO" function.  */
   if (xid_map.subpanel == Shell_Pseudo)
   {

      /* Allocate the array used to save the current color table.  */
      if (xid_psd.savecolors != NULL)
      {
         free(xid_psd.savecolors);
         xid_psd.savecolors = (XColor *)NULL;
      }
      xid_psd.savecolors = (XColor *) calloc(num_imgcolors,sizeof(XColor));

      if (xid_psd.savecolors == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }

      /* If a pseudo-color table has already been defined save the colors from
      it and erase the list of currently applied pseudo-colors.  */
      if (xid_psd.color_psd != NULL)
      {
         for (i=0; i < num_imgcolors; i++)
            xid_psd.savecolors[i] = xid_psd.color_psd[start + i];
         if (xid_psd.flags & PSD_NOINIT)
         {
            if (xid_psd.text_list != NULL)
               XtFree(xid_psd.text_list);
            xid_psd.text_list = XmTextGetString(psd_def_text);
            XmTextSetString(psd_def_text,"");
         }
      }
      else

      /* Save the colors from the current color table.  */
      {
         for (i=0; i < num_imgcolors; i++)
            xid_psd.savecolors[i] = xid.colortable_ptr[img_idx + i];
      }

      /* Set the range and bin values for the "UNDO" function.  Also reset the
      init flag so that the linear color table defined is not destroyed next
      time a pseudo-color is applied (in the function psd_apply).  */
      xid_psd.bcount = 0;
      xid_psd.rcount = 1;
      if (xid_psd.range == NULL)
      {
         xid_psd.range = (long *) calloc((xid_psd.rcount * 2),sizeof(long));
         if (xid_psd.range == NULL)
         {
            xid_message("Insufficient memory available");
            return;
         }
      }
      xid_psd.range[0] = 0;
      xid_psd.range[1] = 255;
      xid_psd.text_pos = -1;
      xid_psd.flags = (xid_psd.flags  & ~PSD_INIT) | PSD_NOINIT;

      /* If space has not been previously allocated, allocate space to store
      the pseudo-color table.  */
      if (xid_psd.color_psd == NULL)
      {
         xid_psd.color_psd = (XColor *) calloc(ncolors,sizeof(XColor));
         if (xid_psd.color_psd == NULL)
         {
            xid_message("Insufficient memory available");
            return;
         }
      }

      /* Set a pointer to the beginning of the pseudo-color table.  */
      beg_ptr = &xid_psd.color_psd[0];
   }

   /* If this function was called from the ADJUST panel, allocate the
   appropriate resources.  */
   else if (xid_map.subpanel == Shell_Adjust)
   {

      /* Reset the slope and offset.  */
      xid_adjust.lut_slope = 45.0;
      xid_adjust.lut_offset = 0.0;

      /* Allocate space for the adjusted color table.  */
      if (xid_adjust.color_adjust == NULL)
      {
         xid_adjust.color_adjust = (XColor *) calloc(ncolors,sizeof(XColor));
         if (xid_adjust.color_adjust == NULL)
         {
            xid_message("Insufficient memory available");
            return;
         }
      }

      /* Allocate space for the linear color table.  */
      if (xid_adjust.color_linear == NULL)
      {
         xid_adjust.color_linear = (XColor *) calloc(num_imgcolors,
                                                     sizeof(XColor));
         if (xid_adjust.color_linear == NULL)
         {
            xid_message("Insufficient memory available");
            return;
         }
      }
      beg_ptr = &xid_adjust.color_adjust[0];
      xid_adjust.adjust_ptr = &xid_adjust.color_linear[0];
   }

   /* If this function was called from the piece-wise linear mapping function,
   allocate the appropriate resources.  */
   else if (xid_map.subpanel == Shell_PWL)
   {
      if (xid_pwl.color_pwl == NULL)
      {
         xid_pwl.color_pwl = (XColor *) calloc(ncolors,sizeof(XColor));
         if (xid_pwl.color_pwl == NULL)
         {
            xid_message("Insufficient memory available");
            return;
         }
      }
      beg_ptr = &xid_pwl.color_pwl[0];
   }

   /* Update the current color table.  */
   else
   {
      if ((xid.visual_info[0].visual->class == PseudoColor) &&
          (*xid.colormap_ptr == xid.colormap))
         beg_ptr = &xid.colortable_ptr[IMG_IDX];
      else
         beg_ptr = &xid.colortable_ptr[0];
   }

   /* Determine how many planes are currently being displayed.  */
   if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) &&
       (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
      nplanes = 1;
   else
      nplanes = 3;

   /* Set the appropriate range value for the pseudo-color function.  */
   if ((nplanes == 1) && (xid_planes[xid_view.rgbplanes[RED]-1].maxval < 
       NUM_IMGCOLORS) && (*xid.colormap_ptr == xid.colormap))
   {
      if (xid_map.subpanel == Shell_Pseudo)
         xid_psd.range[1] = NUM_IMGCOLORS - 1;
   }

   /* Create a linear ramp based on the linear color table already stored when
   the image was displayed.  */
   if (xid.visual_info[0].visual->class == PseudoColor)
   {
      for (i = 0, work_ptr = beg_ptr + start; i < num_imgcolors;
           i++, work_ptr++)
      {
         work_ptr->pixel = xid.colortable_ptr[img_idx + i].pixel;
         work_ptr->flags = DoRed | DoGreen | DoBlue;
         work_ptr->red = xid.color_linear[img_idx + i].red;
         work_ptr->green = xid.color_linear[img_idx + i].green;
         work_ptr->blue = xid.color_linear[img_idx + i].blue;
      }

      /* Store the new colormap.  */
      XStoreColors(xid.display,*xid.colormap_ptr,beg_ptr,num_imgcolors);
   }
   else if (xid.visual_info[0].visual->class == DirectColor)
   {

      /* Create a linear ramp based on the linear color table already stored
      when the image was displayed.  */
      for (i = img_idx, work_ptr = beg_ptr + start; i < (img_idx + 
           num_imgcolors); i++,work_ptr++)
      {
         work_ptr->pixel = xid.colortable_ptr[i].pixel;
         work_ptr->red = xid.color_linear[i].red;
         work_ptr->green = xid.color_linear[i].green;
         work_ptr->blue = xid.color_linear[i].blue;
         work_ptr->flags = DoRed | DoGreen | DoBlue;
      }

      /* Store the new colormap.  */
      XStoreColors(xid.display,*xid.colormap_ptr,beg_ptr,256);
   }

   /* Set the linear color table for the ADJUST function.  */
   if (xid_map.subpanel == Shell_Adjust)
   {
      for (i=0; i < num_imgcolors; i++)
         xid_adjust.color_linear[i] = xid_adjust.color_adjust[start + i];
   }

   /* If the subpanel is not the ADJUST panel, set the appropriate arrays and
   pointers so that the next time ADJUST is run, the adjustment starts from a
   linear map.  */
   else 
   {

      /* Reset the slope and offset.  */
      xid_adjust.lut_slope = 45.0;
      xid_adjust.lut_offset = 0.0;

      /* Allocate space for the linear color table.  */
      if (xid_adjust.color_linear == NULL)
      {
         xid_adjust.color_linear = (XColor *) calloc(num_imgcolors,
                                                     sizeof(XColor));
         if (xid_adjust.color_linear == NULL)
         {
            xid_message("Insufficient memory available");
            return;
         }
         for (i=0, work_ptr = beg_ptr + start; i < num_imgcolors; i++, 
              work_ptr++)
            xid_adjust.color_linear[i] = *work_ptr; 
      }
      xid_adjust.adjust_ptr = &xid_adjust.color_linear[0];
   }

   /* Reinitialize the map histogram flags and data. */
   init_map_histo();
}

/******************************************************************************
Purpose:
   This routine is called whenever the user selects the RESTORE button from
   any of the mapping functions.  It restores the colormap that was installed
   when the function was started.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void map_restore
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   long    i,                  /* Looping index.                             */
           img_idx,            /* Index into colormap of first image colors. */
           num_imgcolors,      /* Number of image colors.                    */
           start;              /* Starting index into pseudo-color table.    */

   /* Set flag so the histogram knows a mapping was modified. */
   if(xid.mapped)
      xid.mapped_mod = TRUE;

   /* Assign number of image colors and starting colormap index.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) &&
       (*xid.colormap_ptr == xid.colormap))
   {
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
      start = 0;  
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) &&
            (*xid.colormap_ptr == xid.colormap))
   {
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = start = IMG_IDX;
   }
   else
   {
      num_imgcolors = 256;
      img_idx = start = 0;
   }

   /* If this was called from the pseudo-color panel, save the appropriate
   colormap entries for the "UNDO" function.  */
   if (xid_map.subpanel == Shell_Pseudo)
   {

      /* If no pseudo-color map has been defined, just return.  */
      if (xid_psd.color_psd == NULL)
         return;

      /* Allocate the array used to save the current color table.  */
      if (xid_psd.savecolors != NULL)
         free(xid_psd.savecolors);
      xid_psd.savecolors = (XColor *) calloc(num_imgcolors,sizeof(XColor));
      if (xid_psd.savecolors == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }

      /* Save the colors from the pseudo-color map and erase the list of
      currently applied pseudo-colors.  */
      for (i=0; i < num_imgcolors; i++)
         xid_psd.savecolors[i] = xid_psd.color_psd[start + i];
      if (xid_psd.flags & PSD_NOINIT)
      {
         if (xid_psd.text_list != NULL)
            XtFree(xid_psd.text_list);
         xid_psd.text_list = XmTextGetString(psd_def_text);
         XmTextSetString(psd_def_text,"");
      }

      /* Set the range and bin values for the "UNDO" function.  */
      xid_psd.flags = (xid_psd.flags  & ~PSD_NOINIT) | PSD_INIT;
      if (xid_psd.color_psd != NULL)
      {
         free(xid_psd.color_psd);
         xid_psd.color_psd = (XColor *)NULL;
      }
      xid_psd.bcount = 0;
      xid_psd.rcount = 1;
      if (xid_psd.range == NULL)
      {
         xid_psd.range = (long *) calloc((xid_psd.rcount * 2),sizeof(long));
         if (xid_psd.range == NULL)
         {
            xid_message("Insufficient memory available");
            return;
         }
      }
      xid_psd.range[0] = 0;
      if ((*xid.colormap_ptr == xid.colormap) &&
          (xid_planes[xid_view.rgbplanes[RED]-1].maxval < NUM_IMGCOLORS))
         xid_psd.range[1] = NUM_IMGCOLORS - 1;
      else
         xid_psd.range[1] = 255;
      xid_psd.text_pos = -1;
   }

   /* If this was called from the piece-wise linear function, set the color
   table to the restored values.  */
   if (xid_map.subpanel == Shell_PWL)
   {
      if (xid_pwl.color_pwl != NULL)
      {
         if ((*xid.colormap_ptr == xid.colormap) &&
             (xid.visual_info[0].visual->class == PseudoColor))
         {
            for (i=0; i < num_imgcolors; i++)
               xid_pwl.color_pwl[i] = xid.colortable_ptr[i + img_idx];
         }
         else
         {
            for (i=0; i < 256; i++)
               xid_pwl.color_pwl[i] = xid.colortable_ptr[i];
         }
      }
   }

   /* Store the original colormap.  */
   if ((*xid.colormap_ptr == xid.colormap) &&
       (xid.visual_info[0].visual->class == PseudoColor))
      XStoreColors(xid.display,xid.colormap,&xid.color_table[IMG_IDX],
                   NUM_IMGCOLORS);
   else
      XStoreColors(xid.display,*xid.colormap_ptr,xid.colortable_ptr,256);
}

/******************************************************************************
Purpose:
   This routine modifies the slope and offset of the current LUT to change the
   contrast and brightness of the displayed image.
    
By L. Huewe, USGS/EROS Data Center, 11/91
Modified by L. Huewe, USGS/EROS Data Center, 12/92
   - This routine was rewritten to use the slope and offset calculations used
     by the adjust function in GLIS.

Algorithm Description:
   This routine is roughly taken from the prototype function XLASDSP.
******************************************************************************/
void adjust_map
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   XColor  *color_ptr;         /* Pointer to the start of the appropriate 
                                  XColor structure.                          */
   int     bright,             /* Current brightness slider value.           */
           contrast;           /* Current contrast slider value.             */
   long    color_idx,          /* Index into color table.                    */
           i,                  /* Loop index.                                */
           init = FALSE,       /* Indicates whether or not the adjust color
                                  table needs to be initialized.             */
           ncolors,            /* Number of colors to store in colormap.     */
           start;              /* Starting index in the color table array.   */
   long    redval,             /* Working color table values.                */
           greenval,
           blueval;
   double  rslope;             /* Slope converted to radians and
                                  the tangent of the result.                 */

   /* Based on the visual type, determine which portion of the colormap needs
   to be modified and stored.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) &&
       (*xid.colormap_ptr == xid.colormap))
   {
      color_idx = 56;
      start = 0;
      ncolors = NUM_IMGCOLORS;
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) &&
            (*xid.colormap_ptr == xid.colormap))
   {
      color_idx = 0;
      start = 56;
      ncolors = 256;
   }
   else
   {
      color_idx = 0;
      ncolors = 256;
      start = 0;
   }

   /* Allocate space for the adjusted color table.  */
   if (xid_adjust.color_adjust == NULL)
   {
      xid_adjust.color_adjust = (XColor *) calloc(ncolors,sizeof(XColor));
      if (xid_adjust.color_adjust == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
      if (*xid.colormap_ptr == xid.colormap)
         xid_adjust.adjust_ptr = &xid.color_table[IMG_IDX];
      else
         xid_adjust.adjust_ptr = &xid.color_image[0];
      init = TRUE;
   }

   /* Get the current slope (contrast) and offset (brightness) values.  */
   XtVaGetValues(adjust_contrast_slider,XmNvalue,&contrast,NULL);
   xid_adjust.lut_slope = (float)contrast;

   rslope = DEGTORAD(xid_adjust.lut_slope);
   rslope = tan(rslope);

   XtVaGetValues(adjust_bright_slider,XmNvalue,&bright,NULL);
   xid_adjust.lut_offset = (float)bright / 100.0 * MAXCOLOR;

   /* Initialize the adjust color table.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) && (init))
   {
      /* Copy the current image colors.  */
      for (i = 0; i < (ncolors - start); i++)
         xid_adjust.color_adjust[i] = xid.colortable_ptr[color_idx + i];
   }
   else if (init)
   {
      /* Copy the reserved colors as well as the current image colors.  */
      for (i = 0; i < 256; i++)
         xid_adjust.color_adjust[i] = xid.colortable_ptr[i];
   }

   /* Set the slope and offset values if the "Linear" or "Restore" buttons were
   pressed.  */
   if (strcmp(client,"LN") == 0) 
   {
      /* Load a linear mapping.  */
      xid_adjust.lut_slope = 45.0;
      xid_adjust.lut_offset = 0.0;

      /* Reset the slider values.  */
      XtVaSetValues(adjust_contrast_slider,XmNvalue,45,NULL);
      XtVaSetValues(adjust_bright_slider,XmNvalue,0,NULL);

      /* Set the color map to linear.  */
      map_linear(w,client,call);

      return;
   }
   else if (strcmp(client,"RE") == 0) 
   {
      /* Restore the mapping loaded at the time the adjust function was
      started.  */
      if (xid_adjust.color_adjust != NULL)
      {
         if ((*xid.colormap_ptr == xid.colormap) &&
             (xid.visual_info[0].visual->class == PseudoColor))
         {
            for (i=0; i < ncolors; i++)
               xid_adjust.color_adjust[i] = xid.colortable_ptr[i + color_idx];
         }
         else
         {
            for (i=0; i < 256; i++)
               xid_adjust.color_adjust[i] = xid.colortable_ptr[i];
         }
      }
      XStoreColors(xid.display,*xid.colormap_ptr,&xid.colortable_ptr[color_idx],
                   ncolors);
      xid_adjust.lut_slope = 45.0;
      xid_adjust.lut_offset = 0.0;

      /* Reset the slider values.  */
      XtVaSetValues(adjust_contrast_slider,XmNvalue,45,NULL);
      XtVaSetValues(adjust_bright_slider,XmNvalue,0,NULL);

      /* Set flag so the histogram knows a mapping was modified. */
      if(xid.mapped)
         xid.mapped_mod = TRUE;

      return;
   }

   /* Calculate the new color table.  */
   for (i = 0, color_ptr = xid_adjust.adjust_ptr; i < (ncolors - start); 
        i++, color_ptr++)
   {

      /* Adjust the appropriate components and assign the new values.  */ 
      if ((xid_map.flags & MAP_RED) || (xid_map.flags & MAP_ALL))
      {
         redval = (long)(color_ptr->red * rslope + xid_adjust.lut_offset);
         redval = (redval < MINCOLOR)   ? MINCOLOR : redval;
         redval = (redval > MAXCOLOR)   ? MAXCOLOR : redval;
         xid_adjust.color_adjust[start + i].red   = (unsigned short)redval;
      }
      if ((xid_map.flags & MAP_GREEN) || (xid_map.flags & MAP_ALL))
      {
          greenval = (long)(color_ptr->green * rslope + xid_adjust.lut_offset);
          greenval = (greenval < MINCOLOR) ? MINCOLOR : greenval;
          greenval = (greenval > MAXCOLOR) ? MAXCOLOR : greenval;
          xid_adjust.color_adjust[start + i].green = (unsigned short)greenval;
      }
      if ((xid_map.flags & MAP_BLUE) || (xid_map.flags & MAP_ALL))
      {
         blueval = (long)(color_ptr->blue * rslope + xid_adjust.lut_offset);
         blueval  = (blueval < MINCOLOR)  ? MINCOLOR : blueval;
         blueval  = (blueval > MAXCOLOR)  ? MAXCOLOR : blueval;
         xid_adjust.color_adjust[start + i].blue  = (unsigned short)blueval;
      }

      xid_adjust.color_adjust[start + i].flags = DoRed | DoGreen | DoBlue;
   }

   /* Store and apply the new color table.  */
   XStoreColors(xid.display,*xid.colormap_ptr,xid_adjust.color_adjust,ncolors);

   /* Set flags to indicate a mapping has been applied. */
   xid.mapped = TRUE;
   xid.mapped_mod = TRUE;
   xid.mapped_z_mod = TRUE;
}

/******************************************************************************
Purpose:
   This routine closes the adjust window without saving the "adjusted"
   mapping.  The adjust color tables are deallocated and the display is
   refreshed to reflect the state of the image before adjust was opened too.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void adjust_cancel
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   /* Close the adjust window and make the mapping options sensitive.  */
   XtUnmanageChild(Adjust_form);
   XtPopdown(Shell_Adjust);
   XtSetSensitive(main_map_shell,True);
   xid_map.subpanel = (Widget)NULL;

   /* Deallocate the adjust color tables so the next time the panel is opened
      the current color table values will be used to initialize the adjust
      color table values. */
   if(xid_adjust.color_adjust != NULL)
   {
      free(xid_adjust.color_adjust);
      xid_adjust.color_adjust = (XColor *)NULL;
   }
   if(xid_adjust.color_linear != NULL)
   {
      free(xid_adjust.color_linear);
      xid_adjust.color_linear = (XColor *)NULL;
   }
   xid_adjust.lut_slope = 45.0;
   xid_adjust.lut_offset = 0.0;

   /* Reset the slider values */
   XtVaSetValues(adjust_contrast_slider,XmNvalue,45,NULL);
   XtVaSetValues(adjust_bright_slider,XmNvalue,0,NULL);

   /* Refresh the display to reflect the image as it was prior to opening
      the adjust mapping panel */
   if((*xid.colormap_ptr == xid.colormap) &&
      (xid.visual_info[0].visual->class == PseudoColor))
      XStoreColors(xid.display,xid.colormap,&xid.color_table[IMG_IDX],
                   NUM_IMGCOLORS);
   else
      XStoreColors(xid.display,*xid.colormap_ptr,xid.colortable_ptr,256);
}

/******************************************************************************
Purpose:
   This routine parses the currently selected color into its red, green and
   blue components and sets the appropriate colormap entries.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void psd_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   float   scaler;             /* Used to scale gray level values to the
                                  appropriate range (i.e. NUM_IMGCOLORS).    */
   long    high,               /* Upper range value to pseudocolor.          */
           i,j,                /* Looping indices.                           */
           idx,                /* Index into memory plane structure.         */
           low,                /* Lower range value to pseudocolor.          */
           maxval,             /* Maximum gray level value allowed.          */
           ncolors,            /* Number of colors to store in colormap.     */
           nonwhite = FALSE,   /* Indicates whether or not nonwhite space 
                                  was found in range/bin string.             */
           num_save,           /* Number of colors to save for undo function.*/
           prev_value,         /* Previous scaled gray level value.          */
           save_idx,           /* Index into save color array.               */
           start,              /* Starting index in the color table array.   */
           status,             /* Status indicator.                          */
           value;              /* Scaled gray level value.                   */
   char    *grayvals,          /* Pointer to mask value string.              */
           *psdlist = NULL;    /* Color name or RGB values and the gray
                                  levels the color has been applied to.      */

   /* Return if there is no image currently being viewed.  */
   if (xid.view == -1)
   {
      xid_message("No image data currently being viewed");
      return;
   }

   /* The dislayed image must be black and white (single band) in order to
   pseudo-color.  */
   if ((xid_view.rgbplanes[RED] != xid_view.rgbplanes[GREEN]) ||
       (xid_view.rgbplanes[GREEN] != xid_view.rgbplanes[BLUE]))
   {
      xid_message("Only black and white images may be pseudocolored");
      return;
   }

   /* Determine the maximum gray level value allowed to be colored.  */
   idx = xid_view.rgbplanes[RED] - 1;
   if ((xid_planes[idx].maxval < NUM_IMGCOLORS) &&
       (*xid.colormap_ptr == xid.colormap))
      maxval = NUM_IMGCOLORS - 1;
   else
      maxval = 255;

   /* Determine the number of colormap entries to store and the starting index
   for image colors.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) &&
       (*xid.colormap_ptr == xid.colormap))
   {
      ncolors = NUM_IMGCOLORS;
      start = 0;
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) && 
            (*xid.colormap_ptr == xid.colormap))
   {
      ncolors = 256;
      start = IMG_IDX;
   }
   else
   {
      ncolors = 256;
      start = 0;
   }

   /* Allocate space for the pseudo-color table if it has not previously been
   allocated.  */
   if (xid_psd.color_psd == NULL)
   {
      xid_psd.color_psd = (XColor *) calloc(ncolors,sizeof(XColor));
      if (xid_psd.color_psd == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
   }

   /* Determine whether or not any gray levels were specified to be colored.  */
   grayvals = XmTextGetString(psd_range_text);
   for (i=0; i < strlen(grayvals); i++)
   {
      if (isspace((int)grayvals[i]) == 0)
         nonwhite = TRUE;
   }
   if (!nonwhite)
      xid_psd.rcount = xid_psd.bcount = 0;
   else
   {

      /* Update the list of colors currently applied to the image.  */
      if (xid_psd.flags & PSD_LIST)
      {
         psdlist = (char *)malloc(strlen(grayvals) + strlen(xid_psd.color) + 5);
         if (psdlist == NULL)
         {
            xid_message("Insufficient memory available");
            return;
         }
         sprintf(psdlist,"%s : %s\n",xid_psd.color,grayvals);
      }
      else
      {
         psdlist = (char *)malloc(strlen(grayvals) + 12 + 5);
         if (psdlist == NULL)
         {
            xid_message("Insufficient memory available");
            return;
         }
         sprintf(psdlist,"%d %d %d : %s\n",
                 (xid.color_table[WORK_IDX].red >> 8),
                 (xid.color_table[WORK_IDX].green >> 8),
                 (xid.color_table[WORK_IDX].blue >> 8),grayvals);
      }
      xid_psd.text_length = (long)strlen(psdlist);

      /* Parse the specified gray level values into ranges and/or individual
      bins.  */
      if (xid_psd.range != NULL)
      {
         free(xid_psd.range);
         xid_psd.range = NULL;
      }
      if (xid_psd.bin != NULL)
      {
         free(xid_psd.bin);
         xid_psd.bin = NULL;
      }
      status = parse_rb(grayvals,&xid_psd.range,&xid_psd.rcount,
                        &xid_psd.bin,&xid_psd.bcount,MINUTINY,maxval);
      if (status != XID_S)
         xid_psd.rcount = xid_psd.bcount = 0;
   }
   XtFree(grayvals);

   if (xid_psd.flags & PSD_INIT)

      /* Copy the appropriate number of entries from the current color table */
   {
      if ((xid.visual_info[0].visual->class == PseudoColor) &&
          (*xid.colormap_ptr == xid.colormap))
      {

         /* Copy the current image colors.  */
         for (i = 0; i < NUM_IMGCOLORS; i++)
            xid_psd.color_psd[i] = xid.color_table[IMG_IDX + i];
      }
      else        /* DirectColor */
      {

         /* Copy the reserved colors as well as the current image colors.  */
         for (i = 0; i < 256; i++)
            xid_psd.color_psd[i] = xid.colortable_ptr[i];
      }
   }

   /* Calculate the new color table by setting the image values specified by
   the range and bin values to the specified color.  Also save the current
   values for the "UNDO" function.  */
   if ((*xid.colormap_ptr == xid.colormap) &&
       (xid_planes[xid_view.rgbplanes[RED] - 1].maxval < NUM_IMGCOLORS))
      scaler = 1.0;
   else if (*xid.colormap_ptr == xid.colormap)
      scaler = (256.0 / (float) NUM_IMGCOLORS);
   else
      scaler = 1.0;

   if (xid_psd.savecolors != NULL)
      free(xid_psd.savecolors);
   num_save = xid_psd.bcount;
   for (i=0; i < xid_psd.rcount; i++)
      num_save += (xid_psd.range[i*2+1] - xid_psd.range[i*2] + 1);
   if (num_save != 0)
   {
      xid_psd.savecolors = (XColor *) calloc(num_save,sizeof(XColor));
      if (xid_psd.savecolors == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
   }
   if ((xid.visual_info[0].visual->class == DirectColor) &&
       (*xid.colormap_ptr == xid.colormap))
      xid_psd.color_psd[WORK_IDX] = xid.color_table[WORK_IDX];

   for (i = 0, prev_value = -1, save_idx = 0; i < xid_psd.bcount; i++)
   {
      value = (long) ((float)xid_psd.bin[i] / scaler);
      if (value != prev_value)
      {
         xid_psd.savecolors[save_idx] = xid_psd.color_psd[start + value];
         xid_psd.color_psd[start + value].red = xid.color_table[WORK_IDX].red; 
         xid_psd.color_psd[start + value].green = xid.color_table[WORK_IDX].
                                                  green; 
         xid_psd.color_psd[start + value].blue = xid.color_table[WORK_IDX].
                                                 blue; 
         save_idx++;
      }
      prev_value = value;
   }
   for (i = 0; i < xid_psd.rcount; i++)
   {
      low = (long) ((float)xid_psd.range[i*2] / scaler);
      high = (long) ((float)xid_psd.range[i*2+1] / scaler);
      for (j=low; j <= high; j++, save_idx++)
      {
         xid_psd.savecolors[save_idx] = xid_psd.color_psd[start + j];
         xid_psd.color_psd[start + j].red = xid.color_table[WORK_IDX].red; 
         xid_psd.color_psd[start + j].green = xid.color_table[WORK_IDX].green; 
         xid_psd.color_psd[start + j].blue = xid.color_table[WORK_IDX].blue; 
      }
   }

   /* Store and apply the new color table.  */
   XStoreColors(xid.display,*xid.colormap_ptr,xid_psd.color_psd,ncolors);

   /* Add the color applied to the list of currently applied colors.  */
   if (num_save != 0)
   {
      xid_psd.text_pos = XmTextGetLastPosition(psd_def_text);
      XmTextInsert(psd_def_text,xid_psd.text_pos,psdlist);
      free(psdlist);
      if (xid_psd.text_list != NULL)
      {
         XtFree(xid_psd.text_list);
         xid_psd.text_list = NULL;
      }
   }
   xid_psd.flags = (xid_psd.flags  & ~PSD_INIT) | PSD_NOINIT;

   /* Set flags to indicate a mapping has been applied. */
   xid.mapped = TRUE;
   xid.mapped_mod = TRUE;
   xid.mapped_z_mod = TRUE;
}

/******************************************************************************
Purpose:
   This routine closes the pseudo-color window without saving the pseudo-
   color mapping.  The psd buffers are deallocated and the display is
   refreshed to reflect the previous state of the image before psd was opened.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void psd_cancel
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   /* Close the pseudo-color window and make the mapping options sensitive.  */
   XtUnmanageChild(Psd_bulletinboard);
   XtPopdown(Shell_Pseudo);
   XtSetSensitive(main_map_shell,True);

   /* Deallocate the psd color table so the next time the panel is opened
      the current color table values will be used to initialize the psd
      color table values. */
   if(xid_psd.color_psd != NULL)
   {
      free(xid_psd.color_psd);
      xid_psd.color_psd = (XColor *)NULL;
   }
   if(xid_psd.savecolors != NULL)
   {
      free(xid_psd.savecolors);
      xid_psd.savecolors = (XColor *)NULL;
   }
   if(xid_psd.range != NULL)
   {
      free(xid_psd.range);
      xid_psd.range = NULL;
   }
   if(xid_psd.bin != NULL)
   {
      free(xid_psd.bin);
      xid_psd.bin = NULL;
   }
   xid_psd.flags = (xid_psd.flags & ~PSD_NOINIT) | PSD_INIT;

   /* Clear the work cell in the color table.  */
   xid.color_table[WORK_IDX].red   = xid.color_workclear.red;
   xid.color_table[WORK_IDX].green = xid.color_workclear.green;
   xid.color_table[WORK_IDX].blue  = xid.color_workclear.blue;
   xid.color_table[WORK_IDX].flags = xid.color_workclear.flags;
   XStoreColor(xid.display,xid.colormap,&xid.color_table[WORK_IDX]);
   xid_map.subpanel = (Widget)NULL;

   /* Refresh the display to reflect the image as it was prior to opening
      the adjust mapping panel */
   if((*xid.colormap_ptr == xid.colormap) &&
      (xid.visual_info[0].visual->class == PseudoColor))
      XStoreColors(xid.display,xid.colormap,&xid.color_table[IMG_IDX],
                   NUM_IMGCOLORS);
   else
      XStoreColors(xid.display,*xid.colormap_ptr,xid.colortable_ptr,256);
}

/******************************************************************************
Purpose:
   This routine determines whether the user wishes to select colors from a
   standard X11 color list or to create his/her own colors using the red,
   green and blue color bars.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void psd_func
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   if (strcmp(client,"CL") == 0)
   {

      /* Unmap the color bars.  */
      XtUnmapWidget(psd_red_slider);
      XtUnmapWidget(psd_green_slider);
      XtUnmapWidget(psd_blue_slider);

      /* Display the color list.  */
      XtMapWidget(psd_scrolledlist);

      /* Set the appropriate flag indicating the current method of color
      selection.  */
      xid_psd.flags = (xid_psd.flags & ~PSD_SLIDER) | PSD_LIST;
   }
   else
   {

      /* Unmap the color list.  */
      XtUnmapWidget(psd_scrolledlist);

      /* Display the red, green and blue color bars */
      XtMapWidget(psd_red_slider);
      XtMapWidget(psd_green_slider);
      XtMapWidget(psd_blue_slider);

      /* Set the appropriate flag indicating the current method of color
      selection.  */
      xid_psd.flags = (xid_psd.flags & ~PSD_LIST) | PSD_SLIDER;
   }
}

/******************************************************************************
Purpose:
   This routine determines the color selected from the color list, parses
   the color and assigns that color to the work cell (this colors the box 
   next to the color list the color the user selected).
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void color_list_select
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   XColor          rgb_defs;           /* Components of the parsed color.    */
   Status          status = -1;        /* Status indicator.                  */
   unsigned long   red,                /* Red, green and blue values scaled  */
                   green,              /* to values meaningful to the user.  */
                   blue;
   XmListCallbackStruct *callback = (XmListCallbackStruct *)call;

   /* Determine the color specified by the user and parse it into its RGB
   components.  */
   if (w == psd_scrolledlist_list)
   { 
      XmStringGetLtoR(callback->item,XmSTRING_DEFAULT_CHARSET,&xid_psd.color);
      status = XParseColor(xid.display,xid.colormap,xid_psd.color,&rgb_defs);
   }

   if (status == 0)
      xid_message("Unable to locate specified color in color database");
   else
   {

      /* Assign the red, green and blue values to the work color cell.  */
      xid.color_table[WORK_IDX].red   = rgb_defs.red;
      xid.color_table[WORK_IDX].green = rgb_defs.green;
      xid.color_table[WORK_IDX].blue  = rgb_defs.blue;
      xid.color_table[WORK_IDX].flags = rgb_defs.flags;
      XStoreColor(xid.display,xid.colormap,&xid.color_table[WORK_IDX]);

      /* Scale the red, green and blue values so they are meaningful to the
      user.  */
      red   = (rgb_defs.red >> 8);
      green = (rgb_defs.green >> 8);
      blue  = (rgb_defs.blue >> 8);

      /* Set the red, green and blue scale values.  This allows the user to see
      the RGB values associated with the currently selected color if he/she
      changes the selection mode to color sliders.  */
      if (w == psd_scrolledlist_list)
      {
         XmScaleSetValue(psd_red_slider,(int)red);
         XmScaleSetValue(psd_green_slider,(int)green);
         XmScaleSetValue(psd_blue_slider,(int)blue);
      }
   }
}

/******************************************************************************
Purpose:
   This routine retrieves the red, green and blue values selected by the
   user and assigns them to the work color cell.  This allows the user
   to see the color before it is applied to the image.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void color_slider
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   int      red,                    /* Red, green and blue values scaled     */
            green,                  /* up for the colormap.                  */
            blue;

   /* Retrieve the red, green and blue values selected by the user.  */
   if ((w == psd_red_slider) || (w == psd_green_slider) ||
       (w == psd_blue_slider))
   {
      XmScaleGetValue(psd_red_slider,&red);
      XmScaleGetValue(psd_green_slider,&green);
      XmScaleGetValue(psd_blue_slider,&blue);
   }

   /* Assign these values to the work color cell.  */
   if (xid_psd.color != NULL)
      strcpy(xid_psd.color,"");
   xid.color_table[WORK_IDX].red   =  (red << 8);
   xid.color_table[WORK_IDX].green =  (green << 8);
   xid.color_table[WORK_IDX].blue  =  (blue << 8);
   XStoreColor(xid.display,xid.colormap,&xid.color_table[WORK_IDX]);
}

/******************************************************************************
Purpose:
   This routine undoes the last operation by restoring the colors saved
   by map_restore, map_linear or psd_apply.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void psd_undo
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   float   scaler;             /* Used to scale gray level values to the
                                  appropriate range (i.e. NUM_IMGCOLORS).    */
   long    high,               /* Upper range value to pseudocolor.          */
           i,j,                /* Looping indices.                           */
           low,                /* Lower range value to pseudocolor.          */
           ncolors,            /* Number of colors to store in colormap.     */
           prev_value,         /* Previous scaled gray level value.          */
           save_idx,           /* Index into save color array.               */
           start,              /* Starting index in the color table array.   */
           value;              /* Scaled gray level value.                   */

   /* Set flag so the histogram knows a mapping was modified. */
   if(xid.mapped)
      xid.mapped_mod = TRUE;

   /* If no colors are currently saved, return.  */
   if (xid_psd.savecolors == NULL)
      return;

   /* Determine the appropriate number of colors to store and the image data
   index into the color table.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) &&
       (*xid.colormap_ptr == xid.colormap))
   {
      ncolors = NUM_IMGCOLORS;
      start = 0;
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) &&
            (*xid.colormap_ptr == xid.colormap))
   {
      ncolors = 256;
      start = IMG_IDX;
   }
   else
   {
      ncolors = 256;
      start = 0;
   }

   /* Allocate space for the pseudo-color table if it has not already been
   done.  */
   if (xid_psd.color_psd == NULL)
   {
      xid_psd.color_psd = (XColor *) calloc(ncolors,sizeof(XColor));
      if (xid_psd.color_psd == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }

      /* Preserve the first 56 colormap entries.  */
      if ((ncolors == 256) && (start == IMG_IDX))
      {
         for (i = 0; i < start; i++)
            xid_psd.color_psd[i] = xid.color_table[i];
      }
   }

   /* Restore the saved color table values.  */
   if ((*xid.colormap_ptr == xid.colormap) &&
       (xid_planes[xid_view.rgbplanes[RED] - 1].maxval < NUM_IMGCOLORS))
      scaler = 1.0;
   else if (*xid.colormap_ptr == xid.colormap)
      scaler = (256.0 / (float) NUM_IMGCOLORS);
   else
      scaler = 1.0;

   for (i = 0, prev_value = -1, save_idx = 0; i < xid_psd.bcount; i++)
   {
      value = (long) ((float)xid_psd.bin[i] / scaler);
      if (value != prev_value)
      {
         xid_psd.color_psd[start + value] = xid_psd.savecolors[save_idx]; 
         save_idx++;
      }
      prev_value = value;
   }
   for (i = 0; i < xid_psd.rcount; i++)
   {
      low = (long) ((float)xid_psd.range[i*2] / scaler);
      high = (long) ((float)xid_psd.range[i*2+1] / scaler);
      for (j=low; j <= high; j++, save_idx++)
         xid_psd.color_psd[start + j] = xid_psd.savecolors[save_idx]; 
   }

   /* Store and apply the new color table.  */
   XStoreColors(xid.display,*xid.colormap_ptr,xid_psd.color_psd,ncolors);

   if (xid_psd.text_list != NULL)

      /* Restore the previous list of colors applied (this happens when undoing
      a RESTORE or LINEAR function).  */
   {
      XmTextSetString(psd_def_text,xid_psd.text_list);
      XtFree(xid_psd.text_list);
      xid_psd.text_list = NULL;
      xid_psd.flags = (xid_psd.flags  & ~PSD_INIT) | PSD_NOINIT;
   }
   else if (xid_psd.text_pos >= 0)

      /* Erase the specification from the list of currently applied pseudo-
      colors.  */
   {
      XmTextReplace(psd_def_text,xid_psd.text_pos,xid_psd.text_pos + 
                    xid_psd.text_length,"");
      xid_psd.text_pos = -1;
   }
}

/******************************************************************************
Purpose:
   This routine retrieves and parses/verifies the from and to break points
   specified by the user.  The mapping is then calculated and applied to
   displayed image.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void pwl_apply_func
(
   Widget from_text,
   Widget to_text
)
{
   float   scaler;             /* Used to scale gray level values to the
                                  appropriate range (i.e. NUM_IMGCOLORS).    */
   long    color_idx,          /* Index into color table.                    */
           i,                  /* Looping index.                             */
           idx,                /* Index into piece-wise linear table.        */
           img_idx,            /* Index into colormap of the first image 
                                  color.                                     */
           init = FALSE,       /* Indicates whether or not the pwl color
                                  table needs to be initialized.             */
           map[256],
           ncolors,            /* Number of colors to store in colormap.     */
           nplanes,            /* Number of planes displayed.                */
           offset,             /* Offset used in scaling 24-bit values.      */
           start,              /* Starting index in the color table array.   */
           status,             /* Status indicator.                          */
           to_num;             /* Number of TO values.                       */
   char    *grayvals;          /* Pointer to mask value string.              */

   /* Retrieve and parse the from and to break point values.  */
   grayvals = XmTextGetString(from_text);
   status = parse_fromto(grayvals,&xid_pwl.from,&xid_pwl.num);
   XtFree(grayvals);
   if (status != XID_S)
      return;

   /* Check to be sure that the FROM values are in ascending order.  */
   for (i=0; i < xid_pwl.num-1; i++)
   {
      if (xid_pwl.from[i+1] < xid_pwl.from[i])
      {
         xid_message("FROM values not in ascending order");
         return;
      }
   }
   grayvals = XmTextGetString(to_text);
   status = parse_fromto(grayvals,&xid_pwl.to,&to_num);
   XtFree(grayvals);
   if (status != XID_S)
      return;

   /* Verify that the number of FROM and TO values is the same.  */
   if (xid_pwl.num != to_num)
   {
      xid_message("The number of FROM and TO values must be the same");
      return;
   }

   /* Calculate the mapping.  */
   piece(xid_pwl.from,xid_pwl.to,&xid_pwl.num,map);

   /* Based on the visual type, determine which portion of the colormap needs
   to be modified and stored.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) &&
       (*xid.colormap_ptr == xid.colormap))
   {
      color_idx = img_idx = IMG_IDX;
      start = 0;
      ncolors = NUM_IMGCOLORS;
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) &&
            (*xid.colormap_ptr == xid.colormap))
   {
      color_idx = 0;
      start = img_idx = IMG_IDX;
      ncolors = 256;
   }
   else
   {
      color_idx = start = img_idx = 0;
      ncolors = 256;
   }

   /* Allocate buffer space for the piece-wise linear mapping.  */
   if (xid_pwl.color_pwl == NULL)
   {
      xid_pwl.color_pwl = (XColor *) calloc(ncolors,sizeof(XColor));
      if (xid_pwl.color_pwl == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
      init = TRUE;
   }

   /* Calculate the new color table.  */
   if ((*xid.colormap_ptr == xid.colormap) &&
       (xid.visual_info[0].visual->class == PseudoColor) && (init))
   {

      /* Copy the current image colors.  */
      for (i = 0; i < NUM_IMGCOLORS; i++)
         xid_pwl.color_pwl[i] = xid.color_table[color_idx + i];
   }
   else if (init)
   {

      /* Copy the reserved colors as well as the current image colors.  */
      for (i = 0; i < 256; i++)
         xid_pwl.color_pwl[i] = xid.colortable_ptr[i];
   }

   /* Determine how many planes are currently being displayed.  */
   if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) &&
       (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
      nplanes = 1;
   else
      nplanes = 3;

   /* Calculate the scaler needed to reduce the image to the appropriate number
   of colors.  */
   if ((*xid.colormap_ptr == xid.colormap) && (nplanes == 1) &&
       (xid_planes[xid_view.rgbplanes[RED]-1].maxval < NUM_IMGCOLORS))
      scaler = 1.0;
   else if (*xid.colormap_ptr == xid.colormap)
      scaler = (256.0 / (float)NUM_IMGCOLORS);        /* (2^8 / # of colors) */
   else
      scaler = 1.0;

   /* Apply the mapping to the color table.  */
   if (xid.visual_info[0].visual->class == DirectColor)
   {
      if (*xid.colormap_ptr == xid.colormap)
         offset = (long)(0 - (scaler * (float)(NUM_COLORS + NUM_CSTORE +
                                               NUM_WMCOLORS + 1)));
      else
         offset = 0;
      for (i = img_idx; i < 256; i++)
      {
         idx = (long)((float)i * scaler) + offset;
         if (xid_map.flags & MAP_RED)
            xid_pwl.color_pwl[start + (i - img_idx)].red = (unsigned short)
                                                           map[idx] << 8; 
         else if (xid_map.flags & MAP_GREEN)
            xid_pwl.color_pwl[start + (i - img_idx)].green = (unsigned short)
                                                             map[idx] << 8; 
         else if (xid_map.flags & MAP_BLUE)
            xid_pwl.color_pwl[start + (i - img_idx)].blue = (unsigned short)
                                                            map[idx] << 8; 
         xid_pwl.color_pwl[start + (i - img_idx)].flags = DoRed | DoGreen | 
                                                          DoBlue;
      }
   }
   else
   {
      for (i=img_idx; i < 256; i++)
      {
         if (xid_map.flags & MAP_RED) 
            xid_pwl.color_pwl[i - img_idx].red = (unsigned short)map[
                                            xid.color_linear[i].red >> 8] << 8;
         else if (xid_map.flags & MAP_GREEN) 
            xid_pwl.color_pwl[i - img_idx].green = (unsigned short)map[
                                        xid.color_linear[i].  green >> 8] << 8;
         else if (xid_map.flags & MAP_BLUE) 
            xid_pwl.color_pwl[i - img_idx].blue = (unsigned short)map[
                                           xid.color_linear[i].blue >> 8] << 8;
         xid_pwl.color_pwl[i - img_idx].flags = DoRed | DoGreen | DoBlue;
      }
   }

   /* Store the colormap.  */
   XStoreColors(xid.display,*xid.colormap_ptr,xid_pwl.color_pwl,ncolors);

   /* Set flags to indicate a mapping has been applied. */
   xid.mapped = TRUE;
   xid.mapped_mod = TRUE;
   xid.mapped_z_mod = TRUE;
}

/******************************************************************************
Purpose:
   This routine calls the apply function with the appropriate settings for
   RED, GREEN, BLUE, or ALL.

Algorithm Description:
******************************************************************************/
void pwl_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   char    *prfr,*prto;
   char    *pgfr,*pgto;
   char    *pbfr,*pbto;

   if (xid_map.flags & MAP_RED)
      pwl_apply_func(pwl_rfrom_text, pwl_rto_text);
   else if (xid_map.flags & MAP_GREEN) 
      pwl_apply_func(pwl_gfrom_text, pwl_gto_text);
   else if (xid_map.flags & MAP_BLUE)
      pwl_apply_func(pwl_bfrom_text, pwl_bto_text);
   else if (xid_map.flags & MAP_ALL)
   {
      if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) &&
          (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
      {
         prfr = XmTextGetString(pwl_rfrom_text);
         pgfr = XmTextGetString(pwl_gfrom_text);
         pbfr = XmTextGetString(pwl_bfrom_text);
         if (prfr[0] != '\0')
         {
            prto = XmTextGetString(pwl_rto_text);
            XmTextSetString(pwl_gfrom_text,prfr);
            XmTextSetString(pwl_bfrom_text,prfr);
            XmTextSetString(pwl_gto_text,prto);
            XmTextSetString(pwl_bto_text,prto);
         }
         else if (pgfr[0] != '\0')
         {
            pgto = XmTextGetString(pwl_gto_text);
            XmTextSetString(pwl_rfrom_text,pgfr);
            XmTextSetString(pwl_bfrom_text,pgfr);
            XmTextSetString(pwl_rto_text,pgto);
            XmTextSetString(pwl_bto_text,pgto);
         }
         else if (pbfr[0] != '\0')
         {
            pbto = XmTextGetString(pwl_bto_text);
            XmTextSetString(pwl_gfrom_text,pbfr);
            XmTextSetString(pwl_rfrom_text,pbfr);
            XmTextSetString(pwl_gto_text,pbto);
            XmTextSetString(pwl_rto_text,pbto);
         }
      }
      xid_map.flags = (xid_map.flags & ~(MAP_ALL)) | MAP_RED;
      pwl_apply_func(pwl_rfrom_text, pwl_rto_text);
      xid_map.flags = (xid_map.flags & ~(MAP_RED)) | MAP_GREEN;
      pwl_apply_func(pwl_gfrom_text, pwl_gto_text);
      xid_map.flags = (xid_map.flags & ~(MAP_GREEN)) | MAP_BLUE;
      pwl_apply_func(pwl_bfrom_text, pwl_bto_text);
      xid_map.flags = (xid_map.flags & ~(MAP_BLUE)) | MAP_ALL;
   } 
}

/******************************************************************************
Purpose:
   This routine closes the piece-wise linear mapping window without
   saving the mapping.  Also, the pwl is deallocated and the display is
   refreshed to reflect the state of the image before pwl was opened.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
void pwl_cancel
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   /* Close the piece-wise linear mapping window and make the mapping options
   sensitive.  */
   XtUnmanageChild(Pwl_bulletinboard);
   XtPopdown(Shell_PWL);
   XtSetSensitive(main_map_shell,True);
   xid_map.subpanel = (Widget)NULL;

   /* Deallocate the pwl color table so the next time the panel is opened
      the current color table values will be used to initialize the pwl
      color table values. */
   if(xid_pwl.color_pwl != NULL)
   {
      free(xid_pwl.color_pwl);
      xid_pwl.color_pwl = (XColor *)NULL;
   }

   /* Refresh the display to reflect the image as it was prior to opening
      the adjust mapping panel */
   if((*xid.colormap_ptr == xid.colormap) &&
      (xid.visual_info[0].visual->class == PseudoColor))
      XStoreColors(xid.display,xid.colormap,&xid.color_table[IMG_IDX],
                   NUM_IMGCOLORS);
   else
      XStoreColors(xid.display,*xid.colormap_ptr,xid.colortable_ptr,256);
}

/******************************************************************************
Purpose:
   This routine closes the save mapping or load mapping window.

By L. Huewe, USGS/EROS Data Center, 2/92

Algorithm Description:
******************************************************************************/
void saveload_cancel
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   /* Close the popup panel.  */
   if (w == saveloadpopup_cancel_button)
      XtPopdown(saveload_dltf_popup);

   /* Close the save mapping window and make the mapping options sensitive.  */
   if ((w == Savemap_form) || ((Savemap_form != 0) &&
       (XtIsManaged(Savemap_form))))
   {
      XtUnmanageChild(Savemap_form);
      XtPopdown(Shell_SaveMap);
      XtDestroyWidget(saveload_dltf_popup);
      XtSetSensitive(main_map_shell,True);
   }

   /* Close the load mapping window and make the mapping options sensitive.  */
   else if ((w == Loadmap_form) || ((Loadmap_form != 0) && 
            (XtIsManaged(Loadmap_form))))
   {
      XtUnmanageChild(Loadmap_form);
      XtPopdown(Shell_LoadMap);
      XtDestroyWidget(saveload_dltf_popup);
      XtSetSensitive(main_map_shell,True);
   }
   xid_map.subpanel = (Widget)NULL;
}

/******************************************************************************
Purpose:
   This routine retrieves and verfies the DLTF entry name and
   description specified by the user.  The DLTF name is also 
   determined based on the image name of the displayed image.  
   If necessary, the DLTF name is prompted for using a popup panel.  
   Finally, the appropriate routine is called to write the DLTF
   entry.

By L. Huewe, USGS/EROS Data Center, 3/92

Modified by L. Huewe, USGS/EROS Data Center, 8/93
   - Modified to use XtGrabNone instead of XtGrabExclusive for message pop up. 

Algorithm Description:
******************************************************************************/
void savemap_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   Arg        args[1];           /* Variable used to set widget resources.   */
   XmString   xmstr[1];          /* Motif format string.                     */
   Position   popx,              /* X position of pop-up panel.              */
              popy;              /* Y position of pop-up panel.              */
   long       exists=0,          /* Test for file existence.                 */
              plane_idx,         /* Index into memory plane structure.       */
              size,              /* Size of character string to allocate.    */
              status,            /* Status indicator.                        */
              write_access = 2;  /* Test for write access to a DLTF.         */
   char       *dltf_desc,        /* User specified DLTF description.         */
              *dltf_entry,       /* User specified DLTF entry name.          */
              hname[CMLEN];      /* Host file name of displayed image (first
                                    band).                                   */

   /* Print a warning and return if no image data is currently displayed.  */
   if (xid.view == -1)
   {
      xid_message("No image data currently being viewed");
      return;
   }

   /* Get the entry name and description specified by the user.  */
   dltf_entry = XmTextGetString(savemap_text);
   if (*dltf_entry == '\0')
   {
      xid_message("User must enter a DLTF entry name");
      return;
   }
   if (strlen(dltf_entry) > 12)
   {
      xid_message("DLTF entry name may only contain 12 characters");
      return;
   }
   xid_savemap.entry_dltf = (char *) malloc(strlen(dltf_entry)+1);
   if (xid_savemap.entry_dltf == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   strcpy(xid_savemap.entry_dltf,dltf_entry);
   dltf_desc = XmTextGetString(savemap_text2);
   if (*dltf_desc == '\0')
      size = 2;
   else 
   {
      if (strlen(dltf_desc) > 80)
      {
         xid_message("DLTF entry description may only contain 80 characters");
         return;
      }
      size = (long)strlen(dltf_desc);
   }
   xid_savemap.desc_dltf = (char *) malloc(size+1);
   if (xid_savemap.desc_dltf == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   if (size == 2)
      strcpy(xid_savemap.desc_dltf," ");
   else
      strcpy(xid_savemap.desc_dltf,dltf_desc);

   /* Determine the name of the DLTF based on the first band of the displayed
   image.  */
   plane_idx = xid_view.rgbplanes[RED] - 1;
   if (xid_planes[plane_idx].image_type == LAS)
   {
      status = c_cmhost(xid_planes[plane_idx].filename,hname);
      if (status != E_SUCC)
      {
         xid_message("Error retrieving host file name");
         return;
      }
   }
   else
      strcpy(hname,xid_planes[plane_idx].filename);
   c_lsmknm(hname,"dltf",xid_savemap.hname_dltf);

   /* Does the DLTF exist?  */
   if (c_lsstat(xid_savemap.hname_dltf,&exists) != E_SUCC)
   {

      /* Try to create the DLTF.  If it cannot be created, prompt the user for
      a DLTF name.  */
      if (creat(xid_savemap.hname_dltf,0775) == -1)
      {

         /* Set the appropriate error message for the popup panel.  */
         XtSetArg(args[0], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
                  "Unable to create DLTF for displayed image.\n\
                  User must enter DLTF name.", XmSTRING_DEFAULT_CHARSET)));
         XtSetValues(saveloadpopup_label,args,1);
         XmStringFree(xmstr[0]);

         /* Translate the X/Y position relative to the save mapping panel to an
         X/Y position relative to the screen.  */
         XtTranslateCoords(savemap_form,(Position)0,(Position)0,&popx,&popy);

         /* Position and display the pop-up panel.  */
         XtMoveWidget(saveload_dltf_popup,popx,popy);
         XtPopup(saveload_dltf_popup,XtGrabNone);
      }
      else

         /* Write the entry to the DLTF.  */
         savemap_dltfw(w, client, call);
   }

   /* Does the user have write access to the DLTF?  */
   else if (c_lsstat(xid_savemap.hname_dltf,&write_access) != E_SUCC)
   {

      /* Set the appropriate error message for the popup panel.  */
      XtSetArg(args[0], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
               "Unable to write to DLTF of displayed image.\n\
               User must enter DLTF name.", XmSTRING_DEFAULT_CHARSET)));
      XtSetValues(saveloadpopup_label,args,1);
      XmStringFree(xmstr[0]);

      /* Translate the X/Y position relative to the save mapping panel to an
      X/Y position relative to the screen.  */
      XtTranslateCoords(savemap_form,(Position)0,(Position)0,&popx,&popy);

      /* Position and display the pop-up panel.  */
      XtMoveWidget(saveload_dltf_popup,popx,popy);
      XtPopup(saveload_dltf_popup,XtGrabNone);
   }
   else

      /* Write the entry to the DLTF.  */
      savemap_dltfw(w, client, call);
   XtFree(dltf_entry);
}

/******************************************************************************
Purpose:
   This routine retrieves the DLTF name specified by the user and
   verifies that it exists and possibly creates it (if saving a
   DLTF).  After all verification of the DLTF has been completed,
   the appropriate DLTF read or write routine is called.

By L. Huewe, USGS/EROS Data Center, 3/92

Algorithm Description:
******************************************************************************/
void saveload_file
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   long    exists = 0,         /* Test for file existence.                   */
           lasflg = FALSE,     /* Indicates whether the file name specified is
                                  a LAS name or a host file name.            */
           status;             /* Status indicator.                          */
   char    desc[81],           /* DLTF entry description.                    */
           *dltf_name,         /* User specified DLTF name.                  */
           hname_dltf[CMLEN],  /* Host name of DLTF.                         */
           hname[CMLEN];       /* Host file name of user specified name.     */

   /* Get the DLTF name specified by the user.  */
   dltf_name = XmTextGetString(saveloadpopup_text);
   if (*dltf_name == '\0')
   {
      xid_message("User must enter a DLTF name");
      return;
   }

   /* Determine whether the input name is a LAS name or a host name.
   - If the name contains a slash (/) it is assumed to be a host file name, all
   other names are assumed to be LAS names.  */
   if (strcspn(dltf_name,"/") != strlen(dltf_name))
      lasflg = FALSE;
   else 
      lasflg = TRUE;

   /* Get the host file name of the specified LAS name.  */
   if (lasflg)
   {
      status = c_cmhost(dltf_name,hname);
      if (status != E_SUCC)
      {
         xid_message("Error retrieving host file name");
         return;
      }
   }
   else
      strcpy(hname,dltf_name);

   /* Generate the full DLTF name.  */
   c_lsmknm(hname,"dltf",hname_dltf);

   /* Does the DLTF exist?  */
   if (c_lsstat(hname_dltf,&exists) != E_SUCC)
   {

      /* Create the DLTF if saving a mapping.  */
      if (xid_map.subpanel == Shell_SaveMap)
      {
         if (creat(hname_dltf,0775) == -1)
         {
            xid_message("Error creating specified DLTF");
            return;
         }
      }

      /* Output an error message if retrieving a mapping.  */
      else
      {
         xid_message("Specified DLTF does not exist");
         return;
      }
   }
   XtFree(dltf_name);

   /* Pop down the panel.  */
   XtPopdown(saveload_dltf_popup);

   /* If saving a mapping, call the appropriate routine to write the DLTF
   entry.  */
   if (xid_map.subpanel == Shell_SaveMap)
   {
      strcpy(xid_savemap.hname_dltf,hname_dltf);
      savemap_dltfw(w, client, call);
   }

   /* If retrieving a mapping, read the first part of the record to verify its
   existence.  */
   else
   {
      strcpy(xid_loadmap.hname_dltf,hname_dltf);
      status = c_r1dltf(&xid_loadmap.fd, xid_loadmap.hname_dltf,
                        xid_loadmap.entry_dltf, desc,&xid_loadmap.asdltf,
                        &xid_loadmap.dlen, xid_loadmap.dtype);
      if (status != E_SUCC)
         xid_message("Error reading DLTF entry");

      /* Read and apply the LUT to the displayed image.  */
      else
         loadmap_dltfr(w, client, call);
   }
}

/******************************************************************************
Purpose:
   This routine reads the current display parameters (i.e., window, bands,
   mapping) and saves them to a DLTF.  All mappings are saved as 24-bit
   mappings (even from an 8-bit display).  This is to allow the DLTF entries to
   be independent of the type of display they were saved from.

By L. Huewe, USGS/EROS Data Center, 3/92

Algorithm Description:
******************************************************************************/
void savemap_dltfw
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   struct  ASDLTF  asdltf;     /* DLTF structure.                            */
   float   scaler;             /* LUT scaler for actual number of colors.    */
   long    blueval,            /* Blue gray level value.                     */
           bplane_idx,         /* Index into memory plane structure (blue).  */
           fd,                 /* File descriptor for DLTF.                  */
           gplane_idx,         /* Index into memory plane structure (green). */
           greenval,           /* Green gray level value.                    */
           i,j,                /* Looping variables.                         */
           img_idx,            /* Index into the colormap of the first image
                                  color.                                     */
           maxgray = 255,      /* Maximum gray level value constant.         */
           maxval,             /* Maximum gray level value defined by the
                                  current color table.                       */
           minblue = 255,      /* Minimum blue gray level value.             */
           mingreen = 255,     /* Minimum green gray level value.            */
           minred = 255,       /* Minimum red gray level value.              */
           nbands,             /* Number of bands currently displayed.       */
           redval,             /* Red gray level value.                      */
           rplane_idx,         /* Index into memory plane structure (red).   */
           status,             /* Status indicator.                          */
           table_idx,          /* Index into color table.                    */
           zero = 0;           /* Constant zero.                             */
   short   wlut[3][256];       /* Temporary LUT.                             */
   char    dtype[3];           /* Data type string.                          */
   unsigned char
           lut[3][256];        /* Temporary LUT.                             */

   /* Close the mapping panel.  */
   map_destroy(w, client, call);

   /* Determine the red, green and blue image plane indices and the number of
   bands displayed.  */
   rplane_idx = xid_view.rgbplanes[RED] - 1;
   gplane_idx = xid_view.rgbplanes[GREEN] - 1;
   bplane_idx = xid_view.rgbplanes[BLUE] - 1;
   if ((rplane_idx == gplane_idx) && (gplane_idx == bplane_idx))
      nbands = 1;
   else
      nbands = 3;

   /* Always store the LUT as byte.  */
   strcpy(dtype,"B");

   /* Always assign three LUTs (red, green and blue).  */
   asdltf.numbnd = 3;

   /* Assign the band numbers from the displayed image.  */
   asdltf.band[RED] = xid_planes[rplane_idx].band;
   asdltf.band[GREEN] = xid_planes[gplane_idx].band;
   asdltf.band[BLUE] = xid_planes[bplane_idx].band;

   /* Assign the window loaded (taken from the first band).  */
   for (i=0; i < 4; i++)
      asdltf.window[i] = xid_planes[rplane_idx].window[i];

   /* Assign the zoom and shift values of the image and graphics planes.  */
   for (i=0; i < 2; i++)
   {
      asdltf.izoom[i] = 1;
      asdltf.ishift[i] = 0;
   }

   asdltf.gzoom[0] = asdltf.gzoom[1] = 1;
   asdltf.gshift[0] = asdltf.gshift[1] = 0;

   /* Check for duplicate DLTF entries and open the DLTF for output.  */
   status = c_ckdltf(xid_savemap.hname_dltf,xid_savemap.entry_dltf,&fd);
   if (status != E_SUCC)
   {
      xid_message("Error opening DLTF");
      return;
   }

   /* Write the first DLTF record.  */
   status = c_w1dltf(&fd,xid_savemap.entry_dltf,xid_savemap.desc_dltf,dtype,
                     &asdltf);
   if (status != E_SUCC)
   {
      xid_message("Error writing DLTF record");
      return;
   }

   /* Process the LUT for each band specified; each becomes the second type of
   DLTF record.  Note also that the j variable is used to generate the LUT
   numbers of the second DLTF record.  These numbers are appended to the the
   DLTF entry name to form the key for each of the second record types.  */

   /* Calculate the scaler needed to scale values to the appropriate range.  */
   if ((*xid.colormap_ptr == xid.colormap) && (nbands == 1) && 
       (xid_planes[rplane_idx].maxval < NUM_IMGCOLORS))
   {
      scaler = 1.0;
      maxval = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
   }
   else if (*xid.colormap_ptr == xid.colormap)
   {
      scaler = (256.0 / (float)NUM_IMGCOLORS);
      maxval = 256;
      img_idx = IMG_IDX;
   }
   else
   {
      scaler = 1.0;
      maxval = 256;
      img_idx = 0;
   }

   /* Convert the 8-bit color LUT into a 24-bit LUT for storage in the DLTF.
   This only gives a rough approximation of the LUT as it would appear if saved
   from a 24-bit display.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) && (nbands == 3))
   {

      /* Initialize the LUT to -1 to distinguish between set/unset entries.  */
      for (i=0; i < 256; i++)
         wlut[RED][i] = wlut[GREEN][i] = wlut[BLUE][i] = -1; 
      for (i=img_idx; i < 256; i++)
      {

         /* Get the red, green and blue values from the linear color table,
         keeping track of the minimum value found for each color.  */
         redval = xid.color_linear[i].red >> 8;
         if (redval < minred)
            minred = redval;
         greenval = xid.color_linear[i].green >> 8;
         if (greenval < mingreen)
            mingreen = greenval;
         blueval = xid.color_linear[i].blue >> 8;
         if (blueval < minblue)
            minblue = blueval;

         /* Set the corresponding LUT value based on the values stored in the
         current color table.  */
         wlut[RED][redval] = xid.colortable_ptr[i].red >> 8;
         wlut[GREEN][greenval] = xid.colortable_ptr[i].green >> 8;
         wlut[BLUE][blueval] = xid.colortable_ptr[i].blue >> 8;
      }

      /* Set the values in the LUT up to the minimum to 0.  */
      for (i=0; i < minred; i++)
         wlut[RED][i] = 0;
      for (i=0; i < mingreen; i++)
         wlut[GREEN][i] = 0;
      for (i=0; i < minblue; i++)
         wlut[BLUE][i] = 0;

      /* Set each unset value in the LUT to the previous "set" value.  */ 
      for (i=minred+1; i < 256; i++)
      {
         if (wlut[RED][i] == -1)
            wlut[RED][i] = wlut[RED][minred];
         else
            minred = i;
      }
      for (i=mingreen+1; i < 256; i++)
      {
         if (wlut[GREEN][i] == -1)
            wlut[GREEN][i] = wlut[GREEN][mingreen];
         else
            mingreen = i;
      }
      for (i=minblue+1; i < 256; i++)
      {
         if (wlut[BLUE][i] == -1)
            wlut[BLUE][i] = wlut[BLUE][minblue];
         else
            minblue = i;
      }

      /* Assign the calculated 2-byte LUT into a byte LUT.  */
      for (i=0; i < 3; i++)
      {
         for (j=0; j < 256; j++)
            lut[i][j] = (unsigned char)wlut[i][j];
      }
   }
   else
   {

   /* Assign the LUT values based on the color table values.  */
      for (i=0; i < maxval; i++)
      {
         table_idx = (long)((float)i / scaler) + img_idx;
         lut[RED][i] = xid.colortable_ptr[table_idx].red >> 8;
         lut[GREEN][i] = xid.colortable_ptr[table_idx].green >> 8;
         lut[BLUE][i] = xid.colortable_ptr[table_idx].blue >> 8;
      }

      /* Set any unassigned values to 0.  */
      for (i=maxval; i < 256; i++)
         lut[RED][i] = lut[GREEN][i] = lut[BLUE][i] = 0;
   }

   /* Write the LUTs to the DLTF entry.  */
   for (i=0, j=1; i < 3; i++, j++)
   {
      status = c_w2dltf(&fd, dtype, xid_savemap.entry_dltf,
                        &xid_planes[rplane_idx].slope,
                        &xid_planes[rplane_idx].offset, &zero, &maxgray,
                        &j, lut[i]);
      if (status != E_SUCC)
      {
         xid_message("Error writing DLTF record");
         return;
      }
   }

   /* Close the DLTF.  */
   status = c_cldltf(&fd,xid_savemap.hname_dltf);
   if (status != E_SUCC)
   {
      xid_message("Error closing DLTF");
      return;
   }
}

/******************************************************************************
Purpose:
   This routine is used to automatically make the next widget on the panel (the
   next tab group) the active widget.

By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
******************************************************************************/
void savemap_next
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   /* Move to the next item.  */
   XmProcessTraversal(w,XmTRAVERSE_NEXT_TAB_GROUP);
}

/******************************************************************************
Purpose:
   This routine retrieves and verfies the DLTF entry name specified by the
   user.  The DLTF name is also determined based on the image name of the
   displayed image.  If necessary, the DLTF name is prompted for using a popup
   panel.  Finally, the first part of the DLTF entry record is read to
   determine whether or not the entry exists.

By L. Huewe, USGS/EROS Data Center, 3/92

Modified by L. Huewe, USGS/EROS Data Center, 8/93
   - Modified to use XtGrabNone instead of XtGrabExclusive for message pop up. 

Algorithm Description:
******************************************************************************/
void loadmap_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   Arg        args[1];         /* Variable used to set widget resources.     */
   XmString   xmstr[1];        /* Motif format string.                       */
   Position   popx,            /* X position of pop-up panel.                */
              popy;            /* Y position of pop-up panel.                */
   long       exists=0,        /* Test for file existence.                   */
              plane_idx,       /* Index into memory plane structure.         */
              status,          /* Status indicator.                          */
              read_access = 4; /* Test for read access to a DLTF.            */
   char       desc[81],        /* DLTF entry description.                    */
              *dltf_entry,     /* User specified DLTF entry name.            */
              hname[CMLEN];    /* Host file name of displayed image (first
                                  band).                                     */

   /* Print a warning and return if no image data is currently displayed.  */
   if (xid.view == -1)
   {
      xid_message("No image data currently being viewed");
      return;
   }

   /* Get the entry name specified by the user.  */
   dltf_entry = XmTextGetString(loadmap_text);
   if (*dltf_entry == '\0')
   {
      xid_message("User must enter a DLTF entry name");
      return;
   }
   if (strlen(dltf_entry) > 12)
   {
      xid_message("DLTF entry name may only contain 12 characters");
      return;
   }
   xid_loadmap.entry_dltf = (char *) malloc(strlen(dltf_entry)+1);
   if (xid_loadmap.entry_dltf == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   strcpy(xid_loadmap.entry_dltf,dltf_entry);

   /* Determine the name of the DLTF based on the image name of the first band
   of the displayed image.  */
   plane_idx = xid_view.rgbplanes[RED] - 1;
   if (xid_planes[plane_idx].image_type == LAS)
   {
      status = c_cmhost(xid_planes[plane_idx].filename,hname);
      if (status != E_SUCC)
      {
         xid_message("Error retrieving host file name");
         return;
      }
   }
   else
      strcpy(hname,xid_planes[plane_idx].filename);
   c_lsmknm(hname,"dltf",xid_loadmap.hname_dltf);

   /* Does the DLTF exist?  */
   if (c_lsstat(xid_loadmap.hname_dltf,&exists) != E_SUCC)
   {

      /* Set the appropriate error message for the popup panel.  */ 
      XtSetArg(args[0], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
               "DLTF for displayed image does not exist.\n\
               User must enter DLTF name.", XmSTRING_DEFAULT_CHARSET)));
      XtSetValues(saveloadpopup_label,args,1);
      XmStringFree(xmstr[0]);

      /* Translate the X/Y position relative to the load mapping panel to an
      X/Y position relative to the screen.  */
      XtTranslateCoords(loadmap_form,(Position)0,(Position)0,&popx,&popy);

      /* Position and display the pop-up panel.  */
      XtMoveWidget(saveload_dltf_popup,popx,popy);
      XtPopup(saveload_dltf_popup,XtGrabNone);
   }

   /* Does the user have read access to the DLTF?  */
   else if (c_lsstat(xid_loadmap.hname_dltf,&read_access) != E_SUCC)
   {

      /* Set the appropriate error message for the popup panel.  */
      XtSetArg(args[0], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
               "Unable to read DLTF of displayed image.\n\
               User must enter DLTF name.", XmSTRING_DEFAULT_CHARSET)));
      XtSetValues(saveloadpopup_label,args,1);
      XmStringFree(xmstr[0]);

      /* Translate the X/Y position relative to the load mapping panel to an
      X/Y position relative to the screen.  */
      XtTranslateCoords(loadmap_form,(Position)0,(Position)0,&popx,&popy);

      /* Position and display the pop-up panel.  */
      XtMoveWidget(saveload_dltf_popup,popx,popy);
      XtPopup(saveload_dltf_popup,XtGrabNone);
   }
   else
   {

      /* Read the first part of the DLTF record to determine whether or not the
      specified entry exists.  */
      status = c_r1dltf(&xid_loadmap.fd, xid_loadmap.hname_dltf,
                        xid_loadmap.entry_dltf, desc,&xid_loadmap.asdltf,
                        &xid_loadmap.dlen, xid_loadmap.dtype);
      if (status != E_SUCC)
      {

         /* Set the appropriate error message for the popup panel.  */
         XtSetArg(args[0], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
                  "DLTF entry does not exist in the current DLTF.\n\
                  User must enter DLTF name.", XmSTRING_DEFAULT_CHARSET)));
         XtSetValues(saveloadpopup_label,args,1);
         XmStringFree(xmstr[0]);

         /* Translate the X/Y position relative to the load mapping panel to an
         X/Y position relative to the screen.  */
         XtTranslateCoords(loadmap_form,(Position)0,(Position)0,&popx,&popy);

         /* Position and display the pop-up panel.  */
         XtMoveWidget(saveload_dltf_popup,popx,popy);
         XtPopup(saveload_dltf_popup,XtGrabNone);
      }
      else

         /* Read and apply the LUT to the displayed image.  */
         loadmap_dltfr(w, client, call);
   }
   XtFree(dltf_entry);

   /* Set flags to indicate a mapping has been applied. */
   xid.mapped = TRUE;
   xid.mapped_mod = TRUE;
   xid.mapped_z_mod = TRUE;
}

/******************************************************************************
Purpose:
   This routine reads the specified DLTF entry and applies the appropriate LUTs
   to the displayed image.

By L. Huewe, USGS/EROS Data Center, 3/92

Algorithm Description:
******************************************************************************/
void loadmap_dltfr
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   double  lut_offset,         /* Offset value for scaling LUT data.         */
           lut_slope,          /* Slope value for scaling LUT data.          */
           maxval,             /* Maximum LUT value.                         */
           minval,             /* Minimum LUT value.                         */
           offset,             /* Offset read from the DLTF entry.           */
           slope,              /* Slope read from the DLTF entry.            */
           temp;               /* Working target for scaled LUTs.            */
   float   scaler;             /* Used to scale gray level values to the
                                  appropriate range (i.e. NUM_IMGCOLORS).    */
   long    end,                /* LUT ending point after being scaled.       */
           endpt,              /* Ending point of the LUT.                   */
           i,j,k,              /* Looping variables.                         */
           idx,                /* Index into the DLTF mapping.               */
           img_idx,            /* Index into colormap of first image color.  */
           imgmax,             /* Maximum image value.                       */
           lut_index,          /* Index to I*2 LUT data.                     */
           luts[3],            /* LUT numbers to be read from the DLTF entry.*/
           luttype = 2,        /* LUT data type.                             */
           nbands,             /* Number of displayed bands.                 */
           nomatch,            /* Indicates that not all bands of the 
                                  displayed image were found in the DLTF.    */
           nplanes,            /* Number of image planes displayed.          */
           plane_idx[3],       /* Indices into memory plane structure.       */
           start,              /* LUT starting point after being scaled.     */
           startpt,            /* Starting point of the LUT.                 */
           status,             /* Status indicator.                          */
           target,             /* Target LUT entry to scale.                 */
           target2;            /* I*2 LUT target entry to scale.             */
   short   *shortptr;          /* Working pointer to SHORT LUT data.         */
   unsigned char
           *byteptr,           /* Working pointer to BYTE LUT data.          */
           *lutdata,           /* LUT data read from the DLTF entry.         */
           map[3][256];        /* LUT (scaled if necessary) from the DLTF
                                  entry.                                     */

   /* Close the mapping panel.  */
   map_destroy(w, client, call);

   /* If LUTs are not byte, they will be scaled on input.  */
   if (strcmp(xid_loadmap.dtype,"I2") == 0)
      xid_message("LUTs will be scaled on input");

   /* Get the red, green and blue memory plane indices.  */ 
   plane_idx[RED] = xid_view.rgbplanes[RED] - 1; 
   plane_idx[GREEN] = xid_view.rgbplanes[GREEN] - 1; 
   plane_idx[BLUE] = xid_view.rgbplanes[BLUE] - 1;
   if ((plane_idx[RED]) == (plane_idx[GREEN]) &&
       (plane_idx[GREEN]) == (plane_idx[BLUE]))
      nbands = 1;
   else
      nbands = 3;

   /* Determine which LUTs (bands) to read.  */ 
   /* If the DLTF entry contains only 1 or 2 LUTs (bands), use the first LUT
   for all displayed bands.  */
   if ((xid_loadmap.asdltf.numbnd == 1) || (xid_loadmap.asdltf.numbnd == 2))
   {
      luts[0] = 1;
      luts[1] = 1;
      luts[2] = 1;
   }

   /* If the DLTF entry contains 3 or more LUTs (bands) and the displayed image
   is black and white, use the first 3 LUTs.  This allows pseudo-color LUTs to
   be retrieved correctly.  */ 
   else if ((xid_loadmap.asdltf.numbnd >= 3) && (nbands == 1))
   {
      luts[0] = 1;
      luts[1] = 2;
      luts[2] = 3;
   }

   /* If none of the above conditions apply, try to match the displayed band
   numbers with those in the DLTF entry.  If a match is not found for ALL of
   the displayed bands, the first 3 LUTs are used.  */
   else
   {
      luts[0] = luts[1] = luts[2] = -1;
      for (i=0; i < 3; i++)
      {
         for (j=0; j < xid_loadmap.asdltf.numbnd; j++)
         {
            if (xid_loadmap.asdltf.band[j] == xid_planes[plane_idx[i]].band)
               luts[i] = j + 1; 
         }
      }
      nomatch = FALSE;
      for (i=0; i < 3; i++)
         if (luts[i] == -1)
            nomatch = TRUE;
      if (nomatch)
      {
         luts[0] = 1;
         luts[1] = 2;
         luts[2] = 3;
      }
   }

   /* Allocate buffer space to contain the LUT data.  */
   if ((lutdata = (unsigned char *)malloc(xid_loadmap.dlen)) == 0)
   {
      xid_message("Insufficent memory available");
      return;
   }

   /* Read the appropriate LUTs from the DLTF entry.  */
   for (i=0; i < 3; i++)
   {
      status = c_r2dltf(&xid_loadmap.fd, xid_loadmap.entry_dltf, &luts[i],
                        &startpt, &endpt, &xid_loadmap.dlen, lutdata, &slope,
                        &offset);
      if (status != E_SUCC)
      {
         xid_message("Error reading LUT data from DLTF");
         return;
      }

      /* If the LUTs are byte data, no conversion is necessary.  */
      if (strcmp(xid_loadmap.dtype,"B") == 0)
      {
         byteptr = lutdata;
         for (j=0; j < 256; j++, byteptr++)
            map[i][j] = *byteptr;
      }

      /* If the LUTs are integer data, scale the values to the byte range.  */
      else
      {

         /* Find the minimum and maximum LUT values.  */
         minval = MAXWORD;
         maxval = MINWORD;

         shortptr = (short *)lutdata;

         for (k = startpt; k <= endpt; k++, shortptr++)
         {
            if (*shortptr < minval)
               minval = (double)*shortptr;
            if (*shortptr > maxval)
               maxval = (double)*shortptr;
         }

         /* Calculate the slope and offset needed to scale the LUT data.  */
         status = xid_slpoff(&luttype,&minval,&maxval,&lut_slope,&lut_offset);
         if (status != E_SUCC)
         {
            xid_message("Error calculating LUT slope and offset");
            return;
         }

         /* Apply the calculated slope and offset to the starting and ending
         points of the LUT.  */
         start = (long)((startpt * lut_slope) + lut_offset);
         end = (long)((endpt * lut_slope) + lut_offset);

         /* Scale the LUT data.  */
         if ((start < 256) && (end > 0))
         {

         /* Make sure start is greater than 0 and that end is less than 256. */
            if (start < 0)
               start = 0;
            if (end > 255)
               end = 255;
            for (j=0; j < 256; j++)
            {
               target = j;
               shortptr = (short *)lutdata;

               /* Fill in values before starting point and after ending point*/
               if (j <= start)
                  target = start;
               if (j >= end)
                  target = end;

               /* Find the target entry in the I*2 LUT.  */
               temp = (double)((target - lut_offset) / lut_slope);
               if (temp >= 0)
                  temp += 0.5;
               else
                  temp -= 0.5;
               target2 = (long)temp;

               /* Calculate the index for the LUT data and increment the LUT
               pointer.  */
               lut_index = (long)(target2 - startpt);
               shortptr += lut_index;

               /* Scale the LUT data entry.  */ 
               map[i][j] = (unsigned char)(((*shortptr * lut_slope) + 
                                                            lut_offset) + 0.5);
            }
         }
         else
         {
            xid_message("Invalid LUT data");
            return;
         }
      }
   }

   /* Determine how many planes are currently being displayed.  */
   if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) &&
       (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
      nplanes = 1;
   else
      nplanes = 3;

   /* Calculate the scaler needed to reduce the image to the appropriate number
   of colors.  */
   if ((*xid.colormap_ptr == xid.colormap) && (nplanes == 1) &&
       (xid_planes[xid_view.rgbplanes[RED]-1].maxval < NUM_IMGCOLORS))
   {
      scaler = 1.0;
      imgmax = 200;
      img_idx = IMG_IDX;
   }
   else if (*xid.colormap_ptr == xid.colormap)
   {
      scaler = (256.0 / (float)NUM_IMGCOLORS);        /* (2^8 / # of colors) */
      imgmax = 256;
      img_idx = IMG_IDX;
   }
   else
   {
      scaler = 1.0;
      imgmax = 256;
      img_idx = 0;
   }

   /* Apply the LUT to the color table.  */
   if (xid.visual_info[0].visual->class == DirectColor)
   {
      for (i = 0; i < imgmax; i++) 
      {
         idx = (long)((float)i / scaler) + img_idx;
         xid.colortable_ptr[idx].red = map[RED][i] << 8; 
         xid.colortable_ptr[idx].green = map[GREEN][i] << 8; 
         xid.colortable_ptr[idx].blue = map[BLUE][i] << 8; 
         xid.colortable_ptr[idx].flags = DoRed | DoGreen | DoBlue;
      }
   }
   else
   {
      for (i = 0; i < imgmax; i++) 
      {
         idx = (long)((float)i / scaler) + img_idx;
         xid.colortable_ptr[idx].red = map[RED][xid.color_linear[idx].red >> 8]
                                       << 8;
         xid.colortable_ptr[idx].green = map[GREEN][xid.color_linear[idx].green 
                                                    >> 8] << 8;
         xid.colortable_ptr[idx].blue = map[BLUE][xid.color_linear[idx].blue >>
                                                  8] << 8;
         xid.colortable_ptr[idx].flags = DoRed | DoGreen | DoBlue;
      }
   }

   /* Store the colormap.  */
   XStoreColors(xid.display,*xid.colormap_ptr,xid.colortable_ptr,256);

   /* Free dynamically allocated memory.  */
   free(lutdata);
   return;
}

/******************************************************************************
Purpose:
   This routine retrieves the ASCII file name, adds extensions, and calls 
   the load function.

Algorithm Description:
******************************************************************************/
void pwl_loadfile
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   char    *fname,             /* User specified ASCII file name.            */
           *tmp,               /* Temporary file name.                       */
           *ext[] = {"enr", "eng", "enb"};
                               /* Array of file extensions.                  */
   char *ptr;
   int     i;                  /* Looping index.                             */
   int have_ext = 0;           /* flag that an extension is on the file name */
                                
   /* Print a warning and return if no image data is currently displayed.  */
   if (xid.view == -1)
   {
      xid_message("No image data currently being viewed");
      return;
   }

   /* Get the entry name specified by the user.  */
   fname = XmTextGetString(pwl_file_text);
   if (*fname == '\0')
   {
      xid_message("User must enter an ASCII file name");
      return;
   }

   /* Read and apply the LUT to the displayed image.  */
   else
   {
      ptr = strrchr(fname,'/');
      /* if no "/" appears, use the whole file name */
      if (!ptr)
          ptr = fname;
      if (strchr(ptr,'.'))
          have_ext = 1;

      if (have_ext)
         pwl_load(fname);
      else
      {
         /* load the PWL stretch for all three planes since the user has not
            specified a particular one to load */
         tmp = malloc(sizeof(char)*256);
         for (i = 0; i < 3; i++)
         {
            sprintf(tmp, "%s.%s", fname, ext[i]);
            pwl_load(tmp);
         }
         free(tmp);
      }
   }
   XtFree(fname);
}

/******************************************************************************
Purpose:
   This routine reads the specified ASCII file and displays the values in
   the TO and FROM text widgets.

Algorithm Description:
******************************************************************************/
void pwl_load
(
   char *filename             /* ASCII file name.                           */
)
{
   FILE              *fp;          /* Pointer to file.                       */
   char              *pto,         /* TO value from the string.              */
                     *pfr,         /* FROM value from the string.            */
                     *frto;        /* TO and FROM values from the file.      */
   XmTextPosition    pos;          /* Position value in a text widget.       */
   long              exists=0;     /* Test for file existence.               */
   Widget            from_text,    /* Text widget to display FROM values.    */
                     to_text;      /* Text widget to display TO values.      */

   /* Print a warning message if file cannot be opened.  */
   if ((fp = fopen(filename, "r")) == NULL)
   {
      xid_message("Cannot open specified file");

      /* Does the ASCII file exist?  */
      if (c_lsstat(filename,&exists) != E_SUCC)
         xid_message("Specified file does not exist");
   }
   else
  {

      /* Determine which text widgets to get values from.  */
      if (strstr(filename, ".enr") != NULL)
      {
         from_text = pwl_rfrom_text;
         to_text = pwl_rto_text;
      }
      else if (strstr(filename, ".eng") != NULL)
      {
         from_text = pwl_gfrom_text;
         to_text = pwl_gto_text;
      }
      else if (strstr(filename, ".enb") != NULL)
      {
         from_text = pwl_bfrom_text;
         to_text = pwl_bto_text;
      }
      else
      {
         xid_message ("Not a valid file name");
         return;
      }

      /* Read in numbers from the file and insert them into the appropriate
      text widgets.  */
      pto = malloc(4 * sizeof(char));
      pfr = malloc(4 * sizeof(char));
      frto = malloc(9 * sizeof(char));

      while (fscanf( fp, "%s", frto) != EOF)
      {
         sscanf(frto, "%s", pfr);
         pos = XmTextGetLastPosition(from_text);
         XmTextInsert(from_text, pos, pfr);
         pos = XmTextGetLastPosition(from_text);
         XmTextInsert(from_text, pos, "\n");
         fscanf( fp, "%s", frto); 
         sscanf(frto, "%s", pto);
         pos = XmTextGetLastPosition(to_text);
         XmTextInsert(to_text, pos, pto);
         pos = XmTextGetLastPosition(to_text);
         XmTextInsert(to_text, pos, "\n");
      }

      /* Close the file.  */
      fclose(fp);
      free(pto);
      free(pfr);
      free(frto);
   }
}

/******************************************************************************
Purpose:
   This routine saves the TO and FROM values from the text fields to the
   specified file.

Algorithm Description:
******************************************************************************/
void pwl_save_func
(
   char *filename            /* Name of file.                              */
)
{
   FILE    *fp;                /* Pointer to the file.                       */
   char    *pfr,               /* Pointer to FROM values.                    */
           *pto;               /* Pointer to TO values.                      */
   int     i = 0, j = 0,       /* Looping variables.                         */
           frlen,              /* Length of the FROM string of values.       */
           tolen;              /* Length of the TO string of values.         */
   Widget  from_text,          /* Widget holding FROM values.                */
           to_text;            /* Widget holding TO values.                  */
   
   /* Print out a warning message if ASCII file cannot be opened.  */
   if ((fp = fopen(filename, "w")) == NULL)
   {
      xid_message("Cannot open specified file");
      printf ("Could not open %s\n", filename);
   }

   /* Get data from FROM and TO text widgets and put into specified file,
   alternating with FROM and TO values.  */
   else
   {

      /* Determine which text widgets to get values from.  */
      if (strstr(filename, ".enr") != NULL)
      {
         from_text = pwl_rfrom_text;
         to_text = pwl_rto_text;
      }
      else if (strstr(filename, ".eng") != NULL)
      {
         from_text = pwl_gfrom_text;
         to_text = pwl_gto_text;
      }
      else if (strstr(filename, ".enb") != NULL)
      {
         from_text = pwl_bfrom_text;
         to_text = pwl_bto_text;
      }
      else
      {
         xid_message ("Not a valid file name");
         return;
      }

      /* Get values from text fields.  */
      pfr = XmTextGetString(from_text);    
      pto = XmTextGetString(to_text);

      /* Get the length of each string of values.  */
      frlen = strlen(pfr); 
      tolen = strlen(pto);

      /* Insert a blank line at the top of the file for the description.  */
      fprintf(fp, "\n"); 

      /* Put the values from the strings into the file.  */
      while (i < frlen || j < tolen)
      {
         while (!isdigit((int)pfr[i]) && i < frlen)
            i++;
         while (isdigit((int)pfr[i]))
         {
            fprintf( fp, "%c", pfr[i]);
            i++;
         }
         while (!isdigit((int)pto[j]) && j < tolen)
            j++;
         fprintf( fp, "\t");
         while (isdigit((int)pto[j]))
         {
            fprintf( fp, "%c", pto[j]);
            j++;
         }
         fprintf( fp, "\n");
      }
      XtFree(pfr);
      XtFree(pto);

      /* Close the ASCII file.  */
      fclose(fp);
   }
}

/******************************************************************************
Purpose:
   This routine gets the ASCII file name, attaches extensions, checks the 
   file(s) for existence, and calls the save function.

Algorithm Description:
******************************************************************************/
void pwl_save
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   char    *fname,             /* File name given by user.                   */
           *tmp,               /* Temporary file name.                       */
           *str,               /* Message string.                            */
           *msg,               /* String of message strings.                 */
           *ext[] = {"enr", "eng", "enb"};
   char *ptr;
                               /* Array of file extensions.                  */
   int     i;                  /* Looping index.                             */
   long    exists = 0;         /* Test for file existence.                   */
   int have_ext = 0;           /* flag for an extension already on the name */
   int     pwl_check_file(char *);
                               /* Function to check for overwrite.           */
   
   /* Get the ASCII file name from the text field.  */
   fname = XmTextGetString (pwl_file_text);
   if (*fname == '\0')
   {
      xid_message ("User must enter an ASCII file name");
      return;
   }

   /* determine if there is an extension already on the file */
   ptr = strrchr(fname,'/');
   /* if no "/" appears, use the whole file name */
   if (!ptr)
       ptr = fname;
   if (strchr(ptr,'.'))
       have_ext = 1;

   /* User entered a file name with an extension.  */
   if (have_ext && (c_lsstat(fname, &exists) == E_SUCC))
   {
      str = malloc(sizeof(char)*256);
      sprintf(str, "%s already exists.\n", fname);
      if (pwl_check_file(str)==1)
         pwl_save_func(fname);
      free (str);
   }
   else if (have_ext)
      pwl_save_func(fname);

   /* User entered a file name without an extension.  */
   else
   {
      tmp = malloc(sizeof(char)*256);
      str = malloc(sizeof(char)*256);
      msg = malloc(sizeof(char)*256);
      strcpy(msg, "");
      for (i = 0; i < 3; i++)
      {
         sprintf(tmp, "%s.%s", fname, ext[i]);
         if (c_lsstat(tmp, &exists) == E_SUCC)
         {
            sprintf(str, "%s already exists.\n", tmp);
            strcat(msg, str);
         }
      }
      if (strcmp(msg, "") == 0)
      {
         for (i = 0; i < 3; i++)
         {
            sprintf(tmp, "%s.%s", fname, ext[i]);
            pwl_save_func(tmp);
         }
      }
      else if (pwl_check_file(msg)==1)
      {
         for (i = 0; i < 3; i++)
         {
            sprintf(tmp, "%s.%s", fname, ext[i]);
            pwl_save_func(tmp);
         }
      }
      free (tmp);
      free (str);
      free (msg);
   }

   /* Free resources */
   XtFree(fname);
}

/******************************************************************************
Purpose:
   This routine pops up the save popup to check if the user wants to 
   overwrite an existing file.

Algorithm Description:
******************************************************************************/
int pwl_check_file
(
   char *fname                /* Files that already exist.                  */
)
{
   int                   answer = 0;     /* Test for loop.                   */
   char                  *question;      /* Question for user.               */
   extern XtAppContext   context;        /* Application context.             */
   XmString              text;           /* Text for question.               */
   
   /* Create the save popup.  */
   question = malloc(sizeof(char)*256);
   strcpy (question, "Do you want to overwrite?");
   strcat (fname, question);
   text = XmStringCreateLtoR (fname, XmSTRING_DEFAULT_CHARSET);
   XtVaSetValues (pwl_save_dialog, XmNmessageString, text, NULL);
   XtAddCallback (pwl_save_dialog, XmNokCallback, pwl_confirmation, &answer);
   XtAddCallback (pwl_save_dialog, XmNcancelCallback, pwl_confirmation,
                  &answer);
   XtManageChild (pwl_save_dialog);
   XtPopup (XtParent (pwl_save_dialog), XtGrabNone);
   XtUnmanageChild (XmMessageBoxGetChild (pwl_save_dialog,
                    XmDIALOG_HELP_BUTTON));

   /* Wait for user response.  */
   while (answer == 0)
      XtAppProcessEvent (context, XtIMAll);

   /* Pop down the save popup, sync with server, and update the display.  */
   XtPopdown (XtParent (pwl_save_dialog));
   XSync (XtDisplay (pwl_save_dialog), 0);
   XmUpdateDisplay (pwl_save_button);

   free (question);
   XmStringFree (text);

   return answer;
}

/******************************************************************************
Purpose:
   This routine is the callback for the save popup.

Algorithm Description:
******************************************************************************/
void pwl_confirmation
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   int *answer = (int *) client;   
   XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct *) call;

   if (cbs->reason == XmCR_OK)
      *answer = 1;
   else if (cbs->reason == XmCR_CANCEL)
      *answer = 2; 
} 

/******************************************************************************
Purpose:
   This routine automatically inserts an extension on the file name based on
   the radio button selected.

Algorithm Description:
******************************************************************************/
void pwl_file_ext
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   char    *fname;             /* File name from text widget.                */
   int     n;                  /* Number of characters in string before
                                  extension.                                 */
   char *data = (char *)client;
   char *ptr;

   XmTextSetString(pwl_file_text, xid_load.image_name);
   fname = XmTextGetString(pwl_file_text);

   /* strip off any window specification and extension */
   ptr = strchr(fname,'(');
   if (ptr)
       *ptr = '\0';
   ptr = strrchr(fname,'/');
   /* if no "/" appears, use the whole file name */
   if (!ptr)
       ptr = fname;
   ptr = strrchr(ptr,'.');
   if (ptr)
       *ptr = '\0';
   XmTextSetString(pwl_file_text, "");
   XmTextInsert(pwl_file_text, 0, fname);
   n = XmTextGetLastPosition(pwl_file_text);

   switch (*data)
   {
      case 'R':
         XmTextInsert(pwl_file_text, n, ".enr");
         break;
      case 'G':
         XmTextInsert(pwl_file_text, n, ".eng");
         break;
      case 'B':
         XmTextInsert(pwl_file_text, n, ".enb");
         break;
      case 'A':
         break;
   }

   XtFree(fname);
}

/******************************************************************************
Purpose:
   This routine clears the values in the TO and FROM text widgets.

Algorithm Description:
******************************************************************************/
void pwl_clear
(
   Widget w,
   XtPointer client,
   XtPointer call
)
{
   /* Chooses which colors to clear values for.  */
   if ((xid_map.flags & MAP_RED) || (xid_map.flags & MAP_ALL))
   {
      XmTextSetString(pwl_rfrom_text, "");
      XmTextSetString(pwl_rto_text, "");
   }
   if ((xid_map.flags & MAP_GREEN) || (xid_map.flags & MAP_ALL))
   {
      XmTextSetString(pwl_gfrom_text, "");
      XmTextSetString(pwl_gto_text, "");
   }
   if ((xid_map.flags & MAP_BLUE) || (xid_map.flags & MAP_ALL))
   {
      XmTextSetString(pwl_bfrom_text, "");
      XmTextSetString(pwl_bto_text, "");
   }
}
