#include <sys/file.h>
#include <fcntl.h> /* open definition */
#include "xid.h"
#include "typlim.h"
#include "xid_func.h"

/* GEN_IMAGEIO is a general structure used only by the subroutines in this
   source file for opening, closing and reading a "flat" image file.  */
struct  GEN_IMAGEIO {
   long        fd;     /* File descriptor.                                   */
   long        format; /* File format of image (BSQ or BIL).                 */
   long        dtype;  /* Data type of image.                                */
   long        nbands; /* Number of bands contained in image file.           */
   long        nl;     /* Number of lines to be processed.                   */
   long        ns;     /* Number of samples to be processed.                 */
   long        sl;     /* Starting line to be processed.                     */
   long        ss;     /* Starting sample to be processed.                   */
   long        tl;     /* Total number of lines in the image.                */
   long        ts;     /* Total number of samples in the image.              */
   long        header_bytes;
                       /* Number of header bytes.                            */
   long        current;
                       /* Current position in file.                          */
}gen_imageio;

long open_nonlas
(
   int *fd,            /* File descriptor of image file.  (O)                */
   char *image_name,   /* Name of image file to open.  (I)                   */
   long *nbands,       /* Number of bands contained in image file.  (I)      */
   long *sl,           /* Starting line to be processed.  (I)                */
   long *ss,           /* Starting sample to be processed.  (I)              */
   long *nl,           /* Number of lines to be processed.  (I)              */
   long *ns,           /* Number of samples to be processed.  (I)            */
   long *access,       /* Type of access.  (I)
                               0 - Read
                               1 - Write
                               2 - Update                                    */
   long *dtype         /* Data type of image.  (I)                           */
)

/******************************************************************************
Purpose:
   This routine opens a "flat" or nonLAS image file with the specified
   window and access type.  If number of lines, number of samples, data 
   type or number of bands is zero and read access is being requested, 
   return the actual image values as input by the user or read from the 
   image header file.
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
   This routine is patterned after the LAS routine c_eopenr.
******************************************************************************/
{
   /* Open the image for READ access.  */
   if (*access == READ)
   {
      *fd = open(image_name,O_RDONLY);
      if (*fd == -1)
         return(XID_F);

      /* Return the actual number of bands, lines or samples if the
         corresponding input parameter is zero.  */
      if (*nbands == 0)
         *nbands = xid_nonlas.nbands;
      if (*nl == 0)
         *nl = xid_nonlas.lines;
      if (*ns == 0)
         *ns = xid_nonlas.samples;

      /* Set the appropriate fields in the general image I/O structure.  */
      gen_imageio.tl = xid_nonlas.lines; 
      gen_imageio.ts = xid_nonlas.samples; 
      gen_imageio.nl = *nl;
      gen_imageio.ns = *ns;
      gen_imageio.sl = *sl;
      gen_imageio.ss = *ss;
      gen_imageio.dtype = *dtype;
      gen_imageio.nbands = *nbands; 
      gen_imageio.header_bytes = xid_nonlas.header_bytes;
      gen_imageio.current = 0;
      gen_imageio.format = xid_nonlas.format;
      gen_imageio.fd = *fd;

      /* Make sure we are at the beginning of the file.  */
      lseek(*fd,0,SEEK_SET);
   }
   return(XID_S);
}

long read_nonlas
(
   int *fd,              /* File descriptor of image file.  (I)              */
   long *band,           /* Band number to be read .  (I)                    */
   long *line,           /* Line number to be read (relative to starting
                            line specified in open call).  (I)               */
   unsigned char *buffer,/* User's buffer where data read is to be place.(I) */
   long *nlines          /* Number of lines to be read.  (I)                 */
)

/******************************************************************************
Purpose:
   This routine reads the specified number of lines from the input
   band starting at the input line into the input buffer.  The line 
   number is relative to the starting line the file was opened with. 
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
   This routine is patterned after the LAS routine c_eread.
******************************************************************************/
{
   long            i,        /* Looping index.                               */
                   offset,   /* Proper starting position in the file.        */
                   size = 0, /* Number of bytes/pixel.                       */
                   start,    /* Starting line number.                        */
                   status;   /* Status indicator.                            */
   unsigned char   *ptr;     /* Buffer pointer.                              */

   /* Calculate the true zero relative line number since the parameter "line"
      is relative to the starting line specified when the file was opened.  */
   start = gen_imageio.sl + (*line - 1) - 1; 

   /* Determine number of bytes/pixel.  */
   switch (gen_imageio.dtype)
   {
      case BYTE:
         size = 1;
         break;
      case I2:
         size = 2;
         break;
      case I4:
         size = 4;
         break;
      case REAL:
         size = 4;
         break;
   }

   /* Seek to the appropriate position in the file.  */
   if (gen_imageio.format == BSQ)
   {
      offset = (*band - 1) * (gen_imageio.ts * gen_imageio.tl * size) + 
               ((start * size * gen_imageio.ts) +
               (gen_imageio.ss - 1) * size) + gen_imageio.header_bytes; 
   }
   else
   {
      offset = (gen_imageio.nbands * gen_imageio.ts * size * start) +
               (*band - 1) * (gen_imageio.ts * size) + (gen_imageio.ss - 1) *
               size + gen_imageio.header_bytes;
   }

   /* Seeks are done from the current position so that if the file pointer is
      already at the proper position, a seek does not need to be done.  */
   if ((offset - gen_imageio.current) != 0)
   {
      if (lseek(*fd,(offset-gen_imageio.current),SEEK_CUR) == -1)
         return(XID_F);
   }

   /* If the number of samples in the image is equal to the number of samples
      being read and the image is in BSQ format, read a block of data at one
      time, otherwise read one line at a time.  */
   ptr = buffer;
   if ((*nlines > 1) && (gen_imageio.ns == gen_imageio.ts) &&
       (gen_imageio.format == BSQ))
   {
      status = read(*fd, ptr, gen_imageio.ns * size * gen_imageio.nl); 
      if (status != gen_imageio.ns * size * gen_imageio.nl)
         return(XID_EOF);
   }
   else if (gen_imageio.format == BSQ)
   {
      for (i=0; i < *nlines; i++)
      {
         /* Read a line of data.  */
         status = read(*fd, ptr, gen_imageio.ns * size);
         if (status != gen_imageio.ns * size)
            return(XID_EOF);
         ptr += gen_imageio.ns * size;
         start++;

         /* Calculate the offset and seek to the next line to be read.  */
         gen_imageio.current = (long)lseek(*fd, 0, SEEK_CUR);
         offset = (*band - 1) * (gen_imageio.ts * gen_imageio.tl * size) + 
                  ((start * size * gen_imageio.ts) + (gen_imageio.ss - 1) *
                  size) + gen_imageio.header_bytes; 
         if ((offset - gen_imageio.current) != 0)
         {
            if (lseek(*fd,(offset-gen_imageio.current),SEEK_CUR) == -1)
               return(XID_F);
         }
      }
   }
   else if (gen_imageio.format == BIL)
   {
      for (i=0; i < *nlines; i++)
      {
         /* Read a line of data.  */
         status = read(*fd, ptr, gen_imageio.ns * size);
         if (status != gen_imageio.ns * size)
            return(XID_EOF);
         ptr += gen_imageio.ns * size;
         start++;

         /* Calculate the offset and seek to the next line to be read.  */
         gen_imageio.current = (long)lseek(*fd, 0, SEEK_CUR);
         offset = (gen_imageio.nbands * gen_imageio.ts * size * start) + 
                  (*band - 1) * (gen_imageio.ts * size) +
                  (gen_imageio.ss - 1) * size + gen_imageio.header_bytes;
         if ((offset - gen_imageio.current) != 0)
         {
            if (lseek(*fd,(offset-gen_imageio.current),SEEK_CUR) == -1)
               return(XID_F);
         }
      }
   }

   /* Set the current file position.  */
   gen_imageio.current = (long)lseek(*fd, 0, SEEK_CUR);

   /* Return the number of bytes read.  */
   return(*nlines * gen_imageio.ns);
}

long close_nonlas
(
   int *fd            /* File descriptor of image file.  (I)                */
)

/******************************************************************************
Purpose:
   This routine closes the image specified image file.
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
   This routine is patterned after the LAS routine c_eclose.
******************************************************************************/
{
   long    status;         /* Status indicator.                              */

   /* Close the image file.  */
   status = close(*fd);
   if (status != 0)
      return(XID_F);
   return (XID_S);
}

int minmax_nonlas
(
   char name[][CMLEN],        /*  Image names. (I)                           */
   long dtype,                /*  Data type of image. (I)                    */
   long *nimg,                /*  Total number of images. (I)                */
   long window[][4],          /*  Starting line of each image. (I)           */
   long bands[][MAXBND + 1],  /*  Array of band numbers. (I)                 */
   long *nbands,              /*  Number of bands for each image.  (I)       */
   double *minval,            /*  Minimum value array. (O)                   */
   double *maxval             /*  Maximum value array. (O)                   */
)

/******************************************************************************
Purpose:
   This routine calculates the minimum and maximum pixel values for the
   images, bands, and windows specified.
    
By L. Huewe, USGS/EROS Data Center, 12/92

Modified by L. Huewe, USGS/EROS Data Center, 3/94
   - Modified to reset the buffer pointer to the beginning of the buffer
     after each new line of data is read.

Algorithm Description:
   This routine is patterned after the LAS routine c_minmax.

Bugs:
   This routine may cause problems in the main application if the image is
   opened in the application and then minmax_nonlas is called multiple times.
   minmax_nonlas saves the current image I/O information from the gen_imageio
   structure, but if minmax_nonlas is called multiple times, the information
   being saved is from the previous call to c_minmax rather than the
   information from the image open in the main application.
******************************************************************************/
{
   struct GEN_IMAGEIO  save_imageio;   /* Saves current settings.            */
   float               *realptr;       /* Pointer to a "REAL*4)" buffer.     */
   int                 fd;             /* File descriptor.                   */
   long                access = READ;  /* File access.                       */
   long                i,j,k,l;        /* Looping indices.                   */
   long                *longptr;       /* Pointer to a Integer*4 buffer.     */
   long                nlines = 1;     /* Read one line at a time.           */
   long                size;           /* Number of bytes/pixel.             */
   long                status;         /* Status indicator.                  */
   short               *shortptr;      /* Pointer to a Integer*2 buffer.     */
   unsigned char       *byteptr;       /* Pointer to a BYTE buffer.          */
   unsigned char       *linebuf;       /* Read buffer.                       */

   /* Save the current settings.  */
   save_imageio = gen_imageio;

   /* For each image specified ...  */
   for (i=0; i < *nimg; i++)
   {
      /* Open the image.  */
      status = open_nonlas(&fd,name[i],&nbands[i],&window[i][SL],
                           &window[i][SS],&window[i][NL],&window[i][NS],
                           &access,&dtype);
      if (status != XID_S)
      {
         gen_imageio = save_imageio;
         return(XID_F);
      }
      switch (dtype)
      {
         /* Initialize min and max values and set the appropriate pointer.  */
         case BYTE:
            size = 1;
            minval[i] = MAXUTINY;
            maxval[i] = MINUTINY;

            /* Allocate the input line buffer.  */
            if ((linebuf = (unsigned char *)malloc(gen_imageio.ns * size)) 
                == NULL)
            {
               gen_imageio = save_imageio;
               return(XID_F);
            }

            /* Read each line of each band, then examine each sample to
               determine the min and max values in the window specified.  */
            for (j=0; j < nbands[i]; j++)
            {
               for (k=1; k <= window[i][NL]; k++)
               {
                  status = read_nonlas(&fd,&bands[i][j],&k,linebuf,&nlines);
                  if (status != window[i][NS])
                  {
                     gen_imageio = save_imageio;
                     return(XID_F);
                  }
                  byteptr = linebuf;
                  for (l=0; l < window[i][NS]; l++)
                  {
                     if (*byteptr < minval[i])
                        minval[i] = *byteptr;
                     if (*byteptr > maxval[i])
                        maxval[i] = *byteptr;
                     byteptr++;
                  }
               }
            }
            break;

            /* Initialize min and max values and set the appropriate ptr.  */
         case I2:
            size = 2;
            minval[i] = MAXWORD;
            maxval[i] = MINWORD;

            /* Allocate the input line buffer.  */
            if ((linebuf = (unsigned char *)malloc(gen_imageio.ns * size)) 
                == NULL)
            {
               gen_imageio = save_imageio;
               return(XID_F);
            }

            /* Read each line of each band, then examine each sample to
               determine the min and max values in the window specified.  */
            for (j=0; j < nbands[i]; j++)
            {
               for (k=1; k <= window[i][NL]; k++)
               {
                  status = read_nonlas(&fd,&bands[i][j],&k,linebuf,&nlines);
                  if (status != window[i][NS])
                  {
                     gen_imageio = save_imageio;
                     return(XID_F);
                  }
                  shortptr = (short *)linebuf;
                  for (l=0; l < window[i][NS]; l++)
                  {
                     if (shortptr[l] < minval[i])
                        minval[i] = shortptr[l];
                     if (shortptr[l] > maxval[i])
                        maxval[i] = shortptr[l];
                  }
               }
            }
            break;
         case I4:
            /* Initialize min and max values and set the appropriate ptr.  */
            size = 4;
            minval[i] = MAXLONG;
            maxval[i] = MINLONG;

            /* Allocate the input line buffer.  */
            if ((linebuf = (unsigned char *)malloc(gen_imageio.ns * size)) 
                == NULL)
            {
               gen_imageio = save_imageio;
               return(XID_F);
            }

            /* Read each line of each band, then examine each sample to
               determine the min and max values in the window specified.  */
            for (j=0; j < nbands[i]; j++)
            {
               for (k=1; k <= window[i][NL]; k++)
               {
                  status = read_nonlas(&fd,&bands[i][j],&k,linebuf,&nlines);
                  if (status != window[i][NS])
                  {
                     gen_imageio = save_imageio;
                     return(XID_F);
                  }
                  longptr = (long *)linebuf;
                  for (l=0; l < window[i][NS]; l++)
                  {
                     if (*longptr < minval[i])
                        minval[i] = *longptr;
                     if (*longptr > maxval[i])
                        maxval[i] = *longptr;
                     longptr++;
                  }
               }
            }
            break;
         case REAL:

            /* Initialize min and max values and set the appropriate ptr.  */
            size = 4;
            minval[i] = MAXREAL;
            maxval[i] = MINREAL;

            /* Allocate the input line buffer.  */
            if ((linebuf = (unsigned char *)malloc(gen_imageio.ns * size)) 
                == NULL)
            {
               gen_imageio = save_imageio;
               return(XID_F);
            }

            /* Read each line of each band, then examine each sample to
               determine the min and max values in the window specified.  */
            for (j=0; j < nbands[i]; j++)
            {
               for (k=1; k <= window[i][NL]; k++)
               {
                  status = read_nonlas(&fd,&bands[i][j],&k,linebuf,&nlines);
                  if (status != window[i][NS])
                  {
                     gen_imageio = save_imageio;
                     return(XID_F);
                  }
                  realptr = (float *)linebuf;
                  for (l=0; l < window[i][NS]; l++)
                  {
                     if (*realptr < minval[i])
                        minval[i] = *realptr;
                     if (*realptr > maxval[i])
                        maxval[i] = *realptr;
                     realptr++;
                  }
               }
            }
            break;
      }
   }

   /* Reset the image I/O structure.  */
   gen_imageio = save_imageio;

   /* Set the current file position.  */
   lseek(fd, gen_imageio.current, SEEK_SET);
   return(XID_S);
}
