/*
 *  This is the profile function source file for the profile panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <X11/IntrinsicP.h>
#include "xid.h"
#include "xid_func.h"
#include "../xid_widgets/Context.h"

void profile_am
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine sets the appropriate flag indicating whether the user has
   selected automatic or manual mode.  It then adds the appropriate event
   handler based on that mode.

By L. Huewe, USGS/EROS Data Center, 12/91

Modified by L. Huewe, USGS/EROS Data Center, 3/94
    - Added the "Profile" context to handle the manual mode.

Algorithm Description:
******************************************************************************/
{
   char *data = (char *)client;

   /* Set the appropriate flags.  */
   xid_profile.flags = xid_profile.flags & ~(PROF_MAN | PROF_AUTO);
   if (*data == 'A')
      xid_profile.flags = xid_profile.flags | PROF_AUTO;
   else
      xid_profile.flags = xid_profile.flags | PROF_MAN;

   /* Add the appropriate event handler for the mode selected.  */
   if (xid_profile.flags & PROF_MAN)
   {
      XtRemoveEventHandler(main_image_bulletinboard,PointerMotionMask|
                           ButtonReleaseMask|KeyReleaseMask,FALSE,profile_draw,
                           NULL);

      /* Add and make the appropriate context active.  */
      xid_contextadd(CON_PROFILE);
      XeContextSetActive((XeContextWidget)context_mouse,
                         context_names[CON_PROFILE]);
   }
   else
   {
      /* Make the "image" context active.  */
      XeContextSetActive((XeContextWidget)context_mouse,
                         context_names[CON_IMAGE]);

      /* Delete the "manual profile" context.  */
      xid_contextdel(CON_PROFILE);

      XtAddEventHandler(main_image_bulletinboard,PointerMotionMask|
                        ButtonReleaseMask|KeyReleaseMask,FALSE,profile_draw,
                        NULL);
   }
}

void profile_cd
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine sets the appropriate flag indicating whether the user would
   like the profile displayed as a continuous line or as discrete points.

By L. Huewe, USGS/EROS Data Center, 4/92

Algorithm Description:
******************************************************************************/
{
   int * data = (int *)client;

   /* Set the appropriate flags.  */
   xid_profile.flags = xid_profile.flags & ~(PROF_CONT | PROF_DIS);
   if (*data == 'C')
      xid_profile.flags = xid_profile.flags | PROF_CONT;
   else
      xid_profile.flags = xid_profile.flags | PROF_DIS;
}

/* Profile constants.  */

#define NVALUES         256             /* Total # of (byte) values.         */
#define INDENT_LEFT     45              /* # of pixels to indent.            */
#define INDENT_TOP      45
#define MAX_WIDTH       358             /* Arbitrary viewport maximums.      */
#define MAX_HEIGHT      346
#define WIN_SIZE        MAX_WIDTH - 2 * INDENT_LEFT
                                        /* Size of profile window which
                                           does not require scroll bars.     */

long    profile_space;                  /* Number of pixels between profile
                                           points for a Z dimension profile. */
void profile_create
(void)

/******************************************************************************
Purpose:
   This routine is used to allocate the point array for the profile, draw
   and label the axis, create the GC, resize the profile viewport and add the
   appropriate event handler based on whether automatic or manual mode has
   been selected.
    
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   Arg        args[6];                /* Arguments used to reconfigure the
                                         profile widgets.                    */
   char       axis[10];               /* Working string array.               */
   int        cap_style = CapRound;
   int        line_style = LineSolid; /* GC attributes.                      */
   int        join_style = JoinRound;
   long       count;                  /* # of image planes containing data.  */
   long       depth;                  /* Window/pixmap plane depth.          */
   long       i;                      /* Looping index.                      */
   long       len;                    /* Working string length.              */
   long       npoints = 0;            /* # of points to plot.                */
   long       roi_y;                  /* Region of interest Y coordinate.    */
   long       rplane_idx;             /* Index to red plane number of the
                                         currently viewed image.             */
   long       pstore;                 /* Size of point storage.              */
   long       pwidth;                 /* Working string pixel width.         */
   long       view_height;
   long       view_width;             /* Working profile viewport dimensions.*/
   long       x_axis;                 /* X axis coodinate.                   */
   long       y_axis;                 /* Y axis coordinate.                  */
   Pixel      back;                   /* Background pixel value.             */
   unsigned int  line_width = 0;      /* Specifies optimal for server.       */
   Window     profile_win;            /* Window ID for profile window.       */
   XGCValues  values;                 /* Values array for the GC.            */

   /* Determine whether or not the Z option is valid.  */
   for (i=0, count=0; i < xid.max_plane; i++)
   {
      if (xid_planes[i].flags & USED)
         count++;
   }
   if (count < 2)
   {
      XtSetSensitive(profile_z_button,False);
      if (xid_profile.flags & PROF_Z)
      {
         xid_profile.flags = xid_profile.flags & ~PROF_Z;
         xid_profile.flags = xid_profile.flags | PROF_X;
         XmToggleButtonSetState(profile_x_button,True,True);
      }
   }
   else
      XtSetSensitive(profile_z_button,True);

   /* Find the memory plane index of the first band displayed.  */
   rplane_idx = xid_view.rgbplanes[RED] - 1;

   /* Allocate storage for the point arrays.  */
   if (xid_profile.flags & PROF_X)
      npoints = xid_planes[rplane_idx].dspwin[NS]; 
   else if (xid_profile.flags & PROF_Y)
      npoints = xid_planes[rplane_idx].dspwin[NL];
   else if (xid_profile.flags & PROF_Z)
   {
      npoints = count;

      /* Calculate the amount of space to leave between Z dimension profile
         points so that plots with a small number of bands are not so small. */
      profile_space = (long)((WIN_SIZE) / (float)(npoints - 1));
   }

   /* Allocate point arrays.  */
   pstore = (long)(npoints * sizeof(XPoint));

   xid_profile.points[RED]   = (XPoint *)malloc(pstore);
   xid_profile.points[GREEN] = (XPoint *)malloc(pstore);
   xid_profile.points[BLUE]  = (XPoint *)malloc(pstore);

   if ((xid_profile.points[RED] == NULL) ||
       (xid_profile.points[GREEN] == NULL) ||
       (xid_profile.points[BLUE] == NULL))
   {
      xid_message("Insufficient memory available"); 
      return;
   }

   /* Calculate profile width and height, x and y axis coordinates and region
      of interest values.  */
   if (xid_profile.flags & PROF_X)
   {
      xid_profile.width = (xid_planes[rplane_idx].dspwin[NS] + 3 *
                           INDENT_LEFT);
      xid_profile.height= (NVALUES + 2 * INDENT_TOP);
      x_axis = (INDENT_TOP + 1 + NVALUES);
      y_axis = (INDENT_LEFT - 2);
      roi_y = (x_axis - NVALUES);
   }
   else if (xid_profile.flags & PROF_Y)
   {
      xid_profile.width = (NVALUES + 2 * INDENT_LEFT);
      xid_profile.height= (xid_planes[rplane_idx].dspwin[NL] + 2 * INDENT_TOP);
      x_axis = (INDENT_TOP - 1);
      y_axis = (INDENT_LEFT + 19);
      roi_y = (x_axis + 1);
   }
   else
   {
      xid_profile.width = ((profile_space * (npoints - 1)) + 2 * INDENT_LEFT);
      xid_profile.height= (NVALUES + 2 * INDENT_TOP);
      x_axis = (INDENT_TOP + 1 + NVALUES);
      y_axis = (INDENT_LEFT + 1);
      roi_y = (x_axis - NVALUES);
   }

   /* Get the profile window ID and visual depth.  */
   profile_win = XtWindow(profile_form);
   depth = xid.visual_info[0].depth;

   /* Create the clear and working pixmaps for the profile.  */
   xid_profile.info_pixmap = XCreatePixmap(xid.display,profile_win,
                                           xid_profile.width,
                                           xid_profile.height,depth);
   xid_profile.clear_pixmap = XCreatePixmap(xid.display,profile_win,
                                            xid_profile.width,
                                            xid_profile.height,depth);
 
   /* Create the working GC for the profile window.  */
   values.function = GXclear;
   values.plane_mask = AllPlanes;
   xid_profile.gc = XCreateGC(xid.display,profile_win,GCFunction | GCPlaneMask,
                              &values);

   /* Clear the profile pixmaps.  */
   XSetFunction(xid.display,xid_profile.gc,GXcopy);

   XtVaGetValues(profile_form,XmNbackground,&back,NULL);
   XSetForeground(xid.display,xid_profile.gc,back);

   /*
   XSetForeground(xid.display,xid_profile.gc,
                  xid.color_table[RGBBW_IDX + WHITE_COL].pixel);
   */

   XSetBackground(xid.display,xid_profile.gc,
                  xid.color_table[RGBBW_IDX + BLACK_COL].pixel);

   XFillRectangle(xid.display,xid_profile.info_pixmap,xid_profile.gc,0,0,
                  xid_profile.width,xid_profile.height);
   XSetFont(xid.display,xid_profile.gc,xid.font_info->fid);
   XSetLineAttributes(xid.display,xid_profile.gc,line_width,line_style,
                      cap_style,join_style);
   XSetForeground(xid.display,xid_profile.gc,
                  xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
   XSetBackground(xid.display,xid_profile.gc,
                  xid.color_table[RGBBW_IDX + WHITE_COL].pixel);

   /* Draw the axis for the profile.  */
   if ((xid_profile.flags & PROF_X) || (xid_profile.flags & PROF_Z))
   {
      /* Horizontal profile or Z profile.  */
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                roi_y,y_axis,x_axis);                              /* Y axis */
      if (xid_profile.flags & PROF_X)
      {
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                   x_axis,(y_axis + npoints),x_axis);              /* X axis */
      }
      else
      {
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                   x_axis,(y_axis + profile_space * (npoints - 1)),x_axis);
                                                                   /* X axis */
      }

      /* Label the axis.  */
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                (x_axis - 1),(y_axis - 5),(x_axis - 1));
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                (x_axis - 64),(y_axis - 5),(x_axis - 64));
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                (x_axis - 128),(y_axis - 5),(x_axis - 128));
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                (x_axis - 192),(y_axis - 5),(x_axis - 192));
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                (x_axis - 256),(y_axis - 5),(x_axis - 256));

      strcpy(axis,"0");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis - pwidth - 5),(x_axis - 1) + xid.font_info->descent,
                  axis,len);

      strcpy(axis,"63");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis - pwidth - 5),
                  (x_axis - 1) + xid.font_info->descent - 64,axis,len);

      strcpy(axis,"127");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis - pwidth - 5),
                  (x_axis - 1) + xid.font_info->descent - 128,axis,len);

      strcpy(axis,"191");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis - pwidth - 5),
                  (x_axis - 1) + xid.font_info->descent - 192,axis,len);

      strcpy(axis,"255");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis - pwidth - 5),
                  (x_axis - 1) + xid.font_info->descent - 256,axis,len);
   }
   else if (xid_profile.flags & PROF_Y)                 /* Vertical profile. */
   {
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                x_axis,(y_axis + NVALUES),x_axis);                 /* X axis */
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,y_axis,
                x_axis,y_axis,(y_axis + xid_planes[rplane_idx].dspwin[NL]));
                                                                   /* Y axis */
      /* Label the axis.  */
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (y_axis + 1),x_axis,(y_axis + 1),(x_axis - 5));
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (y_axis + 64),x_axis,(y_axis + 64),(x_axis - 5));
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (y_axis + 128),x_axis,(y_axis + 128),(x_axis - 5));
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (y_axis + 192),x_axis,(y_axis + 192),(x_axis - 5));
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (y_axis + 256),x_axis,(y_axis + 256),(x_axis - 5));

      strcpy(axis,"0");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis + 1 - pwidth/2),
                  (x_axis - 5) - xid.font_info->descent,axis,len);

      strcpy(axis,"63");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis + 64 + 1 - pwidth/2),
                  (x_axis - 5) - xid.font_info->descent,axis,len);

      strcpy(axis,"127");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis + 128 + 1 - pwidth/2),
                  (x_axis - 5) - xid.font_info->descent,axis,len);

      strcpy(axis,"191");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis + 192 + 1 - pwidth/2),
                  (x_axis - 5) - xid.font_info->descent,axis,len);

      strcpy(axis,"255");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (y_axis + 256 + 1 - pwidth/2),
                  (x_axis - 5) - xid.font_info->descent,axis,len);
   }

   /* Copy the pixmap to the "clear template." */
   XCopyArea(xid.display,xid_profile.info_pixmap,xid_profile.clear_pixmap,
             xid_profile.gc,0,0,xid_profile.width,xid_profile.height,0,0);

   /* Assign width, height, maximum width and height for the profile scrolled
      window.  */
   XtSetArg(args[0],XmNwidth,xid_profile.width);
   XtSetArg(args[1],XmNheight,xid_profile.height);
   XtSetArg(args[2],XmNmaxWidth,xid_profile.width);
   XtSetArg(args[3],XmNmaxHeight,xid_profile.height);
   XtSetValues(profile_scrollwin_bulletinboard,args,4);
   XFlush(xid.display);

   /* Resize the profile viewport widget.  */
   if (xid_profile.flags & PROF_X)
   {
      view_width = (xid_planes[rplane_idx].dspwin[NS] + 2 * INDENT_LEFT);
      view_width = (view_width > MAX_WIDTH) ? MAX_WIDTH : view_width;
      view_height= (NVALUES + 2 * INDENT_TOP);
      view_height= (view_height > MAX_HEIGHT) ? MAX_HEIGHT : view_height;
   }
   else if (xid_profile.flags & PROF_Y)
   {
      view_height= (xid_planes[rplane_idx].dspwin[NL] + 2 * INDENT_TOP);
      view_height= (view_height > MAX_HEIGHT) ? MAX_HEIGHT : view_height;
      view_width = (NVALUES + 2 * INDENT_LEFT);
      view_width = (view_width > MAX_WIDTH) ? MAX_WIDTH : view_width;
   }
   else
   {
      view_width = ((profile_space * (npoints - 1)) + 2 * INDENT_LEFT);
      view_width = (view_width > MAX_WIDTH) ? MAX_WIDTH : view_width;
      view_height= (NVALUES + 2 * INDENT_TOP);
      view_height= (view_height > MAX_HEIGHT) ? MAX_HEIGHT : view_height;
   }
   XtResizeWidget(profile_scrollwin,(unsigned short)(view_width +
                  xid_profile.view_xoffset),(unsigned short)(view_height +
                  xid_profile.view_yoffset),1);

   /* Set the profile window flag to active.  */
   xid_profile.flags = xid_profile.flags | PROF_ACTIVE; 

   /* Add the appropriate event handler based on the mode (automatic or
      manual).  */
   if (xid_profile.flags & PROF_MAN)
   {
      XtRemoveEventHandler(main_image_bulletinboard,PointerMotionMask|
                           ButtonReleaseMask|KeyReleaseMask,FALSE,profile_draw,
                           NULL);
      XtAddEventHandler(main_image_bulletinboard,ButtonReleaseMask|
                        KeyReleaseMask,FALSE,profile_draw,NULL);
   }
   else
   {
      XtRemoveEventHandler(main_image_bulletinboard,ButtonReleaseMask|
                           KeyReleaseMask,FALSE,profile_draw,NULL);
      XtAddEventHandler(main_image_bulletinboard,PointerMotionMask|
                        ButtonReleaseMask|KeyReleaseMask,FALSE,profile_draw,
                        NULL);
   }
   return;
}

void profile_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine is used to restore any image data destroyed by the horizontal
   and/or vertical cursor tracking line.  The appropriate event handlers and
   callbacks are removed and resources are freed.  PROFILE_DESTROY then 
   closes the profile window and sets the main panel button to sensitive.
    
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   long    plane_idx;          /* Memory plane index.                        */

   /* Remove any event handlers and callbacks no longer needed.  */
   if (xid_profile.flags & PROF_AUTO)
   {
      XtRemoveEventHandler(main_image_bulletinboard,PointerMotionMask|
                           ButtonReleaseMask|KeyReleaseMask,FALSE,profile_draw,
                           NULL);
   }
   else
   {
      /* Delete the "manual profile" context.  */
      xid_contextdel(CON_PROFILE);
      xid_profile.flags = xid_profile.flags & ~(PROF_MAN | PROF_AUTO);
      xid_profile.flags = xid_profile.flags | PROF_AUTO;
      XmToggleButtonSetState(profile_auto_button,True,True);
   }

   /* Remove the scrollbar callbacks.  */
   XtRemoveCallback(main_hor_scrollbar,XmNdragCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_hor_scrollbar,XmNvalueChangedCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_hor_scrollbar,XmNdecrementCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_hor_scrollbar,XmNincrementCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_hor_scrollbar,XmNpageDecrementCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_hor_scrollbar,XmNpageIncrementCallback,profile_move,
                    (caddr_t)0);

   XtRemoveCallback(main_vert_scrollbar,XmNdragCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_vert_scrollbar,XmNvalueChangedCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_vert_scrollbar,XmNdecrementCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_vert_scrollbar,XmNincrementCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_vert_scrollbar,XmNpageDecrementCallback,profile_move,
                    (caddr_t)0);
   XtRemoveCallback(main_vert_scrollbar,XmNpageIncrementCallback,profile_move,
                    (caddr_t)0);

   /* Determine which (if any) tracking line was displayed.  */
   if (xid_profile.flags & PROF_X)
      xid.num_horiz--;
   else if (xid_profile.flags & PROF_Y)
      xid.num_vert--;

   /* Restore any image data written over by the vertical or horizontal cursor
      tracking line if it is no longer needed.  */
   plane_idx = xid_view.rgbplanes[RED] - 1;

   if ((xid.num_vert == 0) && (xid.line_x != -1))
   {
      XClearArea(xid.display,XtWindow(main_image_bulletinboard),xid.line_x,0,1,
                 xid_planes[plane_idx].dspwin[NL],True);
      xid.line_x = -1;
      xid.flags = xid.flags & (~XID_VERT);
   }
   if ((xid.num_horiz == 0) && (xid.line_y != -1))
   {
      XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,xid.line_y,
                 xid_planes[plane_idx].dspwin[NS],1,True);
      xid.line_y = -1;
      xid.flags = xid.flags & (~XID_HORIZ);
   }
   if ((xid.num_horiz == 0) && (xid.num_vert == 0) && (xid.line_gc != NULL))
   {
      XFreeGC(xid.display,xid.line_gc);
      xid.line_gc = (GC)NULL;
   }

   /* Free resources.  */
   XFreeGC(xid.display,xid_profile.gc);
   XFreePixmap(xid.display,xid_profile.info_pixmap);
   XFreePixmap(xid.display,xid_profile.clear_pixmap);

   free(xid_profile.points[RED]);
   free(xid_profile.points[GREEN]);
   free(xid_profile.points[BLUE]);

   /* Close the profile window and set the main panel button to sensitive.  */
   XtUnmanageChild(Profile_form);
   XtPopdown(Shell_Profile);
   XtSetSensitive(main_profile_button,True);

   /* Reset the active flag.  */
   xid_profile.flags  = xid_profile.flags ^ PROF_ACTIVE;
}

void profile_expose
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine is used to redraw the profile window after an expose event. 
    
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   /* Copy the pixmap to the profile window.  */
   XCopyArea(xid.display,xid_profile.info_pixmap,XtWindow(
             profile_scrollwin_bulletinboard),xid_profile.gc,0,0,
             xid_profile.width,xid_profile.height,0,0);
}

void profile_expose_event
(
   Widget w,
   XtPointer client,
   XEvent *call,
   Boolean *cont
)

/******************************************************************************
Purpose:
   This routine is used to redraw the profile window after an expose event. 
    
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   /* Copy the pixmap to the profile window.  */
   XCopyArea(xid.display,xid_profile.info_pixmap,XtWindow(
             profile_scrollwin_bulletinboard),xid_profile.gc,0,0,
             xid_profile.width,xid_profile.height,0,0);
}

void profile_dir
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine sets the appropriate flag indicating which dimension to
   profile in and adds the appropriate event handlers.  It then determines 
   whether the cursor tracking line is on and/or needs to be turned on/off.
   After the appropriate flags are set and resources freed, the profile
   window is cleared and the new profile is created.

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   XGCValues       values;          /* Values array for the GC.              */
   unsigned long   valuemask = 0;   /* Specify no values.                    */
   long            plane_idx;       /* Index to memory plane structure.      */

   /* Determine whether or not a tracking line is currently in use.  */
   if (xid_profile.flags & PROF_X)
      xid.num_horiz--;
   else if (xid_profile.flags & PROF_Y)
      xid.num_vert--;

   /* Set the flag indicating the dimension to profile in.  */
   if (strcmp(client,"X") == 0)
   {
      xid_profile.flags = xid_profile.flags & ~(PROF_X | PROF_Y | PROF_Z | 
                                                PROF_XY);
      xid_profile.flags = xid_profile.flags | PROF_X;
   }
   else if (strcmp(client,"Y") == 0)
   {
      xid_profile.flags = xid_profile.flags & ~(PROF_X | PROF_Y | PROF_Z | 
                                                PROF_XY);
      xid_profile.flags = xid_profile.flags | PROF_Y;
   }
   else if (strcmp(client,"Z") == 0)
   {
      xid_profile.flags = xid_profile.flags & ~(PROF_X | PROF_Y | PROF_Z | 
                                                PROF_XY);
      xid_profile.flags = xid_profile.flags | PROF_Z;
   }
   else
   {
      xid_profile.flags = xid_profile.flags & ~(PROF_X | PROF_Y | PROF_Z | 
                                                PROF_XY);
      xid_profile.flags = xid_profile.flags | PROF_XY;
   }

   /* Get the memory plane index of the first band displayed.  */
   plane_idx = xid_view.rgbplanes[RED] - 1;

   /* Set flags for the appropriate cursor tracking line.  */
   if (!(xid_profile.flags & PROF_Z))
   {
      if (xid_profile.flags & PROF_X)
         xid.num_horiz++;
      else if (xid_profile.flags & PROF_Y)
         xid.num_vert++;

      /* Create a GC for the cursor tracking line.  */
      if (xid.line_gc == NULL)
      {
         xid.line_gc = XCreateGC(xid.display,XtWindow(
                                 main_image_bulletinboard),valuemask,&values);
      }
      XSetForeground(xid.display,xid.line_gc,
                     xid.color_table[RGBBW_IDX + WHITE_COL].pixel);
   }

   /* If a tracking line is no longer needed, restore the image data.  */
   if ((xid.num_vert == 0) && (xid.line_y != -1))
   {
      XClearArea(xid.display,XtWindow(main_image_bulletinboard),xid.line_x,0,1,
                 xid_planes[plane_idx].dspwin[NL],True);
      xid.line_x = -1;
      xid.flags = xid.flags & (~XID_VERT);
   }
   if ((xid.num_horiz == 0) && (xid.line_x != -1))
   {
      XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,xid.line_y,
                 xid_planes[plane_idx].dspwin[NS],1,True);
      xid.line_y = -1;
      xid.flags = xid.flags & (~XID_HORIZ);
   }
   if ((xid.num_horiz == 0) && (xid.num_vert == 0) && (xid.line_gc != NULL))
   {
      XFreeGC(xid.display,xid.line_gc);
      xid.line_gc = (GC)NULL;
   }

   /* Free the appropriate resources.  */
   XFreeGC(xid.display,xid_profile.gc);
   XFreePixmap(xid.display,xid_profile.info_pixmap);
   XFreePixmap(xid.display,xid_profile.clear_pixmap);

   free(xid_profile.points[RED]);
   free(xid_profile.points[GREEN]);
   free(xid_profile.points[BLUE]);

   /* Clear the window and create the new profile.  */
   XClearWindow(xid.display,XtWindow(profile_scrollwin_bulletinboard));
   profile_create();
   XCopyArea(xid.display,xid_profile.info_pixmap,XtWindow(
             profile_scrollwin_bulletinboard),xid_profile.gc,0,0,
             xid_profile.width,xid_profile.height,0,0);
}

void profile_draw
(
   Widget  w,
   XtPointer client_data,
   XEvent *call_data,
   Boolean *cont
)

/******************************************************************************
Purpose:
   This routine is used to draw the current profile information based on the 
   cursor location.  
    
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
******************************************************************************/
{
   char          axis[10];       /* Working string array.                    */
   extern long   profile_space;  /* Number of pixels between profile
                                    points for a Z dimenstion profile.       */
   long          bplane_idx;     /* Index to blue plane number of the
                                    currently viewed image.                  */
   long          curx;           /* Current X coordinate.                    */
   long          cury;           /* Current Y coordinate.                    */
   long          gplane_idx;     /* Index to green plane number of the
                                    currently viewed image.                  */
   long          i;              /* Loop index.                              */
   long          len;            /* Working string length.                   */
   long          line;           /* Current line location in image.          */
   long          nbands;         /* Number of bands currently displayed.     */
   long          npoints;        /* Total number of points in profile lines. */
   long          nzplanes = 0;   /* Number of planes to profile in the Z
                                    dimension.                               */
   long          rplane_idx;     /* Index to red plane number of the
                                    currently viewed image.                  */
   long          roi_x;          /* Region of Interest X coordinate.         */
   long          roi_y;          /* Region of Interest Y coordinate.         */
   long          roi_w;          /* Region of Interest width.                */
   long          roi_h;          /* Region of Interest height.               */
   long          sample;         /* Current sample location in line.         */
   long          pwidth;         /* Working string pixel width.              */
   long          *zplanes = NULL;/* Plane numbers to profile in the Z
                                    dimension.                               */
   long          zpoint;         /* Index into image data buffer for profile
                                    in the Z dimension.                      */
   static char   lstr[20];       /* Line coordinate string.                  */
   static char   sstr[20];       /* Sample coordinate string.                */
   XEvent        *ev;            /* The input event.                         */

   /* Set a pointer to the event structure.  */
   ev = (XEvent *)call_data;

   /* Validate the cursor coordinates.  */
   rplane_idx = xid_view.rgbplanes[RED] - 1;
   gplane_idx = xid_view.rgbplanes[GREEN] - 1;
   bplane_idx = xid_view.rgbplanes[BLUE] - 1;

   if ((ev->xmotion.x < 0) || (ev->xmotion.x >
       xid_planes[rplane_idx].dspwin[NS] - 1) || (ev->xmotion.y < 0) ||
       (ev->xmotion.y > xid_planes[rplane_idx].dspwin[NL] - 1))
      return;

   /* Determine the number of bands displayed.  */
   if ((rplane_idx == gplane_idx) && (gplane_idx == bplane_idx))
      nbands = 1;
   else
      nbands = 3;

   /* Set the current line and sample based on the cursor location.  */
   line = ev->xmotion.y;
   sample = ev->xmotion.x;
   npoints = 0;

   /* Determine the plane numbers of the planes to profile in the Z dimension*/
   if (xid_profile.flags & PROF_Z)
   {
      zplanes = (long *) calloc(xid.max_plane,sizeof(long));
      if (zplanes == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
      for (i=0, nzplanes = 0; i < xid.max_plane; i++)
      {
         if (xid_planes[i].flags & USED)
         {
            zplanes[nzplanes] = i;
            nzplanes++; 
         }
      }
   }
   if (xid_profile.flags & PROF_X)
   {
      /* Set region of interest and clear the horizontal profile.  */
      roi_x = (INDENT_LEFT - 1);
      roi_y = (INDENT_TOP + 1);
      roi_w = (xid_planes[rplane_idx].dspwin[NS] + 2 * INDENT_LEFT);
      roi_h = (NVALUES + INDENT_TOP - 5);

      /* Clear the old data.  */
      XCopyArea(xid.display,xid_profile.clear_pixmap,xid_profile.info_pixmap,
                xid_profile.gc,roi_x,roi_y,roi_w,roi_h,roi_x,roi_y);

      /* Draw the vertical "marker" in the profile window -- this shows which
         point in the profile corresponds to the sample in the image window. */
      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (sample + roi_x),roi_y,(sample + roi_x),(roi_y + NVALUES));

      /* Label the marker.  */
      if ((sample + roi_x - 5) > roi_x)
      {
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   sample+roi_x,(roi_y + NVALUES - 1),((sample+roi_x) - 5),
                   (roi_y + NVALUES - 1));
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   sample+roi_x,(roi_y + NVALUES - 64),((sample+roi_x) - 5),
                   (roi_y + NVALUES - 64));
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   sample+roi_x,(roi_y + NVALUES - 128),((sample+roi_x) - 5),
                   (roi_y + NVALUES - 128));
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   sample+roi_x,(roi_y + NVALUES - 192),((sample+roi_x) - 5),
                   (roi_y + NVALUES - 192));
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   sample+roi_x,(roi_y + NVALUES - 256),((sample+roi_x) - 5),
                   (roi_y + NVALUES - 256));
      }

      strcpy(axis,"63");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      if ((sample + roi_x - pwidth - 5) > roi_x)
      {
         XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     (sample + roi_x - pwidth - 5),
                     (roi_y + NVALUES) + xid.font_info->descent - 64,axis,len);
      }

      strcpy(axis,"127");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      if ((sample + roi_x - pwidth - 5) > roi_x)
      {
         XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     (sample + roi_x - pwidth - 5),
                     (roi_y + NVALUES) + xid.font_info->descent - 128,axis,
                     len);
      }

      strcpy(axis,"191");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      if ((sample + roi_x - pwidth - 5) > roi_x)
      {
         XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
            (sample + roi_x - pwidth - 5),(roi_y + NVALUES) + 
            xid.font_info->descent - 192,axis,len);
      }

      strcpy(axis,"255");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      if ((sample + roi_x - pwidth - 5) > roi_x)
      {
         XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     (sample + roi_x - pwidth - 5),
                     (roi_y + NVALUES + 10) + xid.font_info->descent - 256,
                     axis,len);
      }

      /* Add line and sample labels.  */
      curx = (((ev->xmotion.x + 1) * xid_planes[rplane_idx].skips) +
                xid_planes[rplane_idx].window[SS] - 1) -
                (xid_planes[rplane_idx].skips - 1);
      cury = (((ev->xmotion.y + 1) * xid_planes[rplane_idx].skipl) +
                xid_planes[rplane_idx].window[SL] - 1) -
                (xid_planes[rplane_idx].skipl - 1);
      sprintf(lstr,"l:%ld",cury);
      sprintf(sstr,"s:%ld",curx);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (sample + INDENT_LEFT),(roi_y + roi_h - 20),lstr,
                  strlen(lstr));
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (sample + INDENT_LEFT),(roi_y + roi_h - 5),sstr,
                  strlen(sstr));
   }
   else if (xid_profile.flags & PROF_Y)
   {
      /* Set region of interest and clear the vertical profile.  */
      roi_x = 5;
      roi_y = INDENT_TOP;
      roi_w = (NVALUES + 2 * INDENT_LEFT);
      roi_h = (xid_planes[rplane_idx].dspwin[NL] + 25);

      /* Clear the old data.  */
      XCopyArea(xid.display,xid_profile.clear_pixmap,xid_profile.info_pixmap,
                xid_profile.gc,roi_x,roi_y,roi_w,roi_h,roi_x,roi_y);

      /* Draw the horizontal "marker" in the profile window -- this shows which
         point in the profile corresponds to the line in the image window.  */
      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (INDENT_LEFT + 19),(line + roi_y),(INDENT_LEFT + NVALUES + 19),
                (line + roi_y));

      /* Label the marker.  */
      if ((line + roi_y - 5) > roi_y)
      {
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   (INDENT_LEFT + 20),line + roi_y,(INDENT_LEFT + 20),
                   (line + roi_y - 5));
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   (INDENT_LEFT + 83),line + roi_y,(INDENT_LEFT + 83),
                   (line + roi_y - 5));
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   (INDENT_LEFT + 147),line + roi_y,(INDENT_LEFT + 147),
                   (line + roi_y - 5));
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   (INDENT_LEFT + 211),line + roi_y,(INDENT_LEFT + 211),
                   (line + roi_y - 5));
         XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                   (INDENT_LEFT + 275),line + roi_y,(INDENT_LEFT + 275),
                   (line + roi_y - 5));
      }

      strcpy(axis,"63");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      if (((line + roi_y - 5) - xid.font_info->descent - xid.font_info->ascent) 
          > roi_y)
      {
         XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     (INDENT_LEFT + 84  - pwidth/2),
                     (line + roi_y - 5) - xid.font_info->descent,axis,len);
      }

      strcpy(axis,"127");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      if (((line + roi_y - 5) - xid.font_info->descent - xid.font_info->ascent) 
          > roi_y)
      {
         XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     (INDENT_LEFT + 148 - pwidth/2),
                     (line + roi_y - 5) - xid.font_info->descent,axis,len);
      }

      strcpy(axis,"191");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      if (((line + roi_y - 5) - xid.font_info->descent - xid.font_info->ascent) 
          > roi_y)
      {
         XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     (INDENT_LEFT + 212 - pwidth/2),
                     (line + roi_y - 5) - xid.font_info->descent,axis,len);
      }

      strcpy(axis,"255");
      len = (long)strlen(axis);
      pwidth = XTextWidth(xid.font_info,axis,len);
      if (((line + roi_y - 5) - xid.font_info->descent - xid.font_info->ascent) 
          > roi_y)
      {
         XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     (INDENT_LEFT + 276 - pwidth/2),
                     (line + roi_y - 5) - xid.font_info->descent,axis,len);
      }

      /* Add line and sample labels.  */
      curx = (((ev->xmotion.x + 1) * xid_planes[rplane_idx].skips) +
                xid_planes[rplane_idx].window[SS] - 1) - 
                (xid_planes[rplane_idx].skips - 1);
      cury = (((ev->xmotion.y + 1) * xid_planes[rplane_idx].skipl) +
                xid_planes[rplane_idx].window[SL] - 1) -
                (xid_planes[rplane_idx].skipl - 1);
      sprintf(lstr,"l:%ld",cury);
      sprintf(sstr,"s:%ld",curx);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  roi_x,(line + INDENT_TOP + 10),lstr,strlen(lstr));
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  roi_x,(line + INDENT_TOP + 25),sstr,strlen(sstr));
   }
   else 
   {
      /* Set region of interest and clear the Z dimension profile.  */
      roi_x = (INDENT_LEFT + 1 + 1);
      roi_y = (INDENT_TOP + 1);
      roi_w = (profile_space * (nzplanes - 1) + 1); 
      roi_h = (NVALUES + INDENT_TOP - 5);

      /* Clear the old data.  */
      XCopyArea(xid.display,xid_profile.clear_pixmap,xid_profile.info_pixmap,
                xid_profile.gc,roi_x,roi_y,roi_w,roi_h,roi_x,roi_y);
   }

   /* Draw the profile information.  */
   if ((nbands == 1) && !(xid_profile.flags & PROF_Z))     
   {
      /* Single-band image.  */
      if (xid_profile.flags & PROF_X)                 /* Horizontal profile. */
      {
         for (i = roi_x; i < (roi_x + xid_planes[rplane_idx].dspwin[NS]); i++)
         {
            sample = (line * xid_planes[rplane_idx].dspwin[NS]) + (i - roi_x);
            xid_profile.points[RED][i - roi_x].x = (short)i;
            xid_profile.points[RED][i - roi_x].y = (short)((roi_y + NVALUES - 1)
                            - (*(xid_planes[rplane_idx].image_data + sample)));
         }
         npoints = xid_planes[rplane_idx].dspwin[NS];
      }
      else 
      {
         for (i = roi_y; i < (roi_y + xid_planes[rplane_idx].dspwin[NL]); i++)
         {
            line = ((i - roi_y) * xid_planes[rplane_idx].dspwin[NS]) + sample;
            xid_profile.points[RED][i - roi_y].x = (short)(roi_y + 20 +
                                (*(xid_planes[rplane_idx].image_data + line))); 
            xid_profile.points[RED][i - roi_y].y = (short)i;
         }
         npoints = xid_planes[rplane_idx].dspwin[NL];
      }

      /* Draw the profile line based on the points calculated above.  */
      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
      if (xid_profile.flags & PROF_CONT)
      {
         XDrawLines(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                    xid_profile.points[RED],npoints,CoordModeOrigin);
      }
      else
      {
         XDrawPoints(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     xid_profile.points[RED],npoints,CoordModeOrigin);
      }
   }
   else if (!(xid_profile.flags & PROF_Z))
   {
      /* 3-band image.  */
      if (xid_profile.flags & PROF_X)                 /* Horizontal profile. */
      {
         for (i = roi_x; i < (roi_x + xid_planes[rplane_idx].dspwin[NS]); i++)
         {
            sample = (line * xid_planes[rplane_idx].dspwin[NS]) + (i - roi_x);
            xid_profile.points[RED][i - roi_x].x = xid_profile.points[GREEN]
                     [i - roi_x].x = xid_profile.points[BLUE][i - roi_x].x =
                     (short)i;
            xid_profile.points[RED][i - roi_x].y = (short)((roi_y + NVALUES - 1)
                            - (*(xid_planes[rplane_idx].image_data + sample))); 
            xid_profile.points[GREEN][i - roi_x].y = (short)((roi_y + NVALUES -
                         1) - (*(xid_planes[gplane_idx].image_data + sample)));
            xid_profile.points[BLUE][i - roi_x].y = (short)((roi_y + NVALUES -
                         1) - (*(xid_planes[bplane_idx].image_data + sample)));
         }
         npoints = xid_planes[rplane_idx].dspwin[NS];
      }
      else                                              /* Vertical profile. */
      {
         for (i = roi_y; i < (roi_y + xid_planes[rplane_idx].dspwin[NL]); i++)
         {
            line = ((i - roi_y) * xid_planes[rplane_idx].dspwin[NS]) + sample;
            xid_profile.points[RED][i - roi_y].x = (short)(roi_y + 20 +
                                (*(xid_planes[rplane_idx].image_data + line)));
            xid_profile.points[GREEN][i - roi_y].x = (short)(roi_y + 20 +
                                (*(xid_planes[gplane_idx].image_data + line)));
            xid_profile.points[BLUE][i - roi_y].x = (short)(roi_y + 20 +
                                (*(xid_planes[bplane_idx].image_data + line)));
            xid_profile.points[RED][i - roi_y].y = xid_profile.points[GREEN]
                     [i - roi_y].y = xid_profile.points[BLUE][i - roi_y].y =
                     (short)i;
         }
         npoints = xid_planes[rplane_idx].dspwin[NL];
      }

      /* Draw the profile lines (in red, green and blue) based on the points
         calculated above.  */
      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + RED_COL].pixel);
      if (xid_profile.flags & PROF_CONT)
      {
         XDrawLines(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                    xid_profile.points[RED],npoints,CoordModeOrigin);
      }
      else
      {
         XDrawPoints(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     xid_profile.points[RED],npoints,CoordModeOrigin);
      }
      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + GREEN_COL].pixel);
      if (xid_profile.flags & PROF_CONT)
      {
         XDrawLines(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                    xid_profile.points[GREEN],npoints,CoordModeOrigin);
      }
      else
      {
         XDrawPoints(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     xid_profile.points[GREEN],npoints,CoordModeOrigin);
      }
      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + BLUE_COL].pixel);
      if (xid_profile.flags & PROF_CONT)
      {
         XDrawLines(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                    xid_profile.points[BLUE],npoints,CoordModeOrigin);
      }
      else
      {
         XDrawPoints(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                     xid_profile.points[BLUE],npoints,CoordModeOrigin);
      }
   }
   else if (xid_profile.flags & PROF_Z)
   {
      /* Calculate the points for a Z dimension profile.  */
      for (i = roi_x; i < (roi_x + nzplanes); i++)
      {
         zpoint = (line * xid_planes[rplane_idx].dspwin[NS]) + sample; 
         xid_profile.points[RED][i - roi_x].x = (short)i;
         xid_profile.points[RED][i - roi_x].y = (short)((roi_y + NVALUES - 1) -
                      (*(xid_planes[zplanes[i - roi_x]].image_data + zpoint)));
     }

      /* Draw the profile lines (in red, green, and blue) based on the points
         calculated above.  */
      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + RED_COL].pixel);
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (xid_profile.points[RED][0].x + 0 * (profile_space - 1)),
                xid_profile.points[RED][0].y,
                (xid_profile.points[RED][0].x + 0 * (profile_space - 1)),
                roi_y + NVALUES);

      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + GREEN_COL].pixel);
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (xid_profile.points[RED][1].x + 1 * (profile_space - 1)),
                xid_profile.points[RED][1].y,
                (xid_profile.points[RED][1].x + 1 * (profile_space - 1)),
                roi_y + NVALUES);

      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + BLUE_COL].pixel);
      XDrawLine(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                (xid_profile.points[RED][2].x + 2 * (profile_space - 1)),
                xid_profile.points[RED][2].y,
                (xid_profile.points[RED][2].x + 2 * (profile_space - 1)),
                roi_y + NVALUES);
      free(zplanes);

      /* Add line and sample labels.  */
      XSetForeground(xid.display,xid_profile.gc,
                     xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
      curx = (((ev->xmotion.x + 1) * xid_planes[rplane_idx].skips) +
                xid_planes[rplane_idx].window[SS] - 1) -
                (xid_planes[rplane_idx].skips - 1);
      cury = (((ev->xmotion.y + 1) * xid_planes[rplane_idx].skipl) +
                xid_planes[rplane_idx].window[SL] - 1) -
                (xid_planes[rplane_idx].skipl - 1);
      sprintf(lstr,"l:%ld",cury);
      sprintf(sstr,"s:%ld",curx);
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (NVALUES/2 + roi_x - 20),(roi_y + roi_h - 20),lstr,
                  strlen(lstr));
      XDrawString(xid.display,xid_profile.info_pixmap,xid_profile.gc,
                  (NVALUES/2 + roi_x - 20),(roi_y + roi_h - 5),sstr,
                  strlen(sstr));
   }

   /* Copy the working profile pixmap to the profile window.  */
   XCopyArea(xid.display,xid_profile.info_pixmap,XtWindow(
             profile_scrollwin_bulletinboard),xid_profile.gc,roi_x,roi_y,roi_w,
             roi_h,roi_x,roi_y);
}

void profile_move
(
   Widget  w,
   XtPointer client_data,
   XtPointer call_data
)

/******************************************************************************
Purpose:
   This routine is used to force the scrollbars on the profile window to move
   based on movement of the main image scrollbar.
    
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   Arg     args[1];            /* Argument used to retrieve scrollbar 
                                  maximum.                                   */
   float   relative_pos;       /* Relative scroll bar position.              */
   int     increment,          /* Amount of slider increment or decrement.   */
           max_image,          /* Maximum scroll bar value on image window.  */
           max_profile,        /* Maximum scroll bar value on profile window.*/
           page,               /* Amount of slider page increment or 
                                  decrement.                                 */
           pos,                /* Current slider position.                   */
           position,           /* Current slider position.                   */
           size_image,         /* Image slider size.                         */
           size_profile;       /* Profile slider size.                       */

   if (xid_profile.flags & PROF_X)
   {
      /* Get the current main image window and profile scrollbar positions.  */
      XmScrollBarGetValues(main_hor_scrollbar,&position,&size_image,&increment,
                           &page);
      XmScrollBarGetValues(profile_hor_scrollbar,&pos,&size_profile,&increment,
                           &page);
      XtSetArg(args[0],XmNmaximum,&max_profile);
      XtGetValues(profile_hor_scrollbar,args,1);
      XtSetArg(args[0],XmNmaximum,&max_image);
      XtGetValues(main_hor_scrollbar,args,1);

      /* Set the profile window scrollbar to its new position.  */
      relative_pos = position / (float)(max_image - size_image);
      pos = (int)(relative_pos * (max_profile - size_profile));
      XmScrollBarSetValues(profile_hor_scrollbar,pos,size_profile,increment,
                           page,True);
   }
   else if (xid_profile.flags & PROF_Y)
   {
      /* Get the current main image window and profile scrollbar positions.  */
      XmScrollBarGetValues(main_vert_scrollbar,&position,&size_image,&increment,
                           &page);
      XmScrollBarGetValues(profile_vert_scrollbar,&pos,&size_profile,&increment,
                           &page);
      XtSetArg(args[0],XmNmaximum,&max_profile);
      XtGetValues(profile_vert_scrollbar,args,1);
      XtSetArg(args[0],XmNmaximum,&max_image);
      XtGetValues(main_vert_scrollbar,args,1);

      /* Set the profile window scrollbar to its new position.  */
      relative_pos = position / (float)(max_image - size_image);
      pos = (int)(relative_pos * (max_profile - size_profile));
      XmScrollBarSetValues(profile_vert_scrollbar,pos,size_profile,increment,
                           page,True);
   }
}
