/*
 *  This is the save function source file for the save panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By L.Huewe, USGS/EROS Data Center, 5/93.
 *    - X Image Display, v1.0.
 *
 *  Modified by A. Shimitz 6/00
 *   - added routines save_tiff_apply, save_verify, and save_tiff to support
 *     the new save "Thumbnail TIFF" option
 *   - also modified the save_destroy routine to take care of removing the
 *     Verify form widget
 *
 */
#include <math.h>
#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <X11/Xmu/WinUtil.h>
#include "tiffio.h"
#include "las.h"
#include "util.h" /* c_getsys, c_splitname, & c_delfil definitions */
#include "labserv.h" /* c_lsstat definition */
#include "xid.h"
#include "manage_cursor.h"
#include "xid_graphics.h"
#include "xid_func.h"
#include "xid_create.h"
#include "notae.h"
#include "../xid_widgets/ProcessM.h"
#include "diskio.h"

/* local function prototypes */
void save_upddr
(void);

void save_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void save_copy_graphics
(
   Pixmap img_pix,  /* Image pixmap.  (I)                                    */
   long sl,         /* Starting line number.  (I)                            */
   long nl,         /* Number of lines in the pixmap.  (I)                   */
   long ns          /* Number of samples in the pixmap.  (I)                 */
);

int save_zoomddr
(
   struct DDR *in_ddr,    /* Band independent DDR record. (I/O)             */
   struct BDDR *in_bddr,  /* Band dependent DDR records. (I/O)              */
   long num_image,        /* Number of input images. (I)                    */
   long skipl[3],         /* Line subsampling factors. (I)                  */
   long skips[3]          /* Sample subsampling factors. (I)                */
);
/* end prototypes */

void save_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine frees the appropriate resources and closes the save LAS panel. 
   It also closes the verify TIFF panel.  We only need to do one of these since
   both panels will never exist concurrently.

By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
******************************************************************************/
{
   /* Clear the error flag.  */
   if (xid_save.flags & SAVE_ERROR)
   {
      xid_save.flags &= ~SAVE_ERROR;
      return;
   }

   /* Close the save LAS panel */
   if(XtIsRealized(Shell_Save))
   {
      XmTextSetString(save_image_text,"");
      if (xid_save.name != NULL)
         free(xid_save.name);
      XtUnmanageChild(save_form);
      XtPopdown(Shell_Save);
      XtUnrealizeWidget(Shell_Save);

      /* Set the load and save buttons on the main panel to sensitive again */
      XtSetSensitive(main_save_las,True);
      XtSetSensitive(main_save_tiff,True);
      XtSetSensitive(main_load_las, True);
      XtSetSensitive(main_load_nonlas, True);
      xid_save.flags &= ~SAVE_ACTIVE;
   }

   /* Close the verify panel */
   if(XtIsRealized(Shell_Verify))
   {
      XtUnmanageChild(Verify_form);
      XtDestroyWidget(Verify_form);
      XtPopdown(Shell_Verify);
      XtUnrealizeWidget(Shell_Verify);
   }
}

void save_lasimage
(
   Window target_win,
   int save_width,
   int save_height
)

/*******************************************************************************
Purpose:
   This routine saves the specified window as a LAS image.  The data is read
   directly from the display screen, therefore the LAS image will directly
   reflect what the user sees on the screen.  For example, if a window
   overlaps the specified target window, the overlapped portion will be 
   saved as well.

   Note:  This routine always generates a 3-band LAS image with the XID
          image colormap applied.

By L. Huewe, USGS/EROS Data Center, 9/93

Modified by L. Huewe, USGS/EROS Data Center, 3/94
   - Modified to retrieve the correct pixel value on 8-bit displays.

Algorithm Description:
*******************************************************************************/
{
   extern struct FDESC *fdesc[2];/* Image file descriptors used for image I/O*/
   extern struct GDESC *gdesc;   /* Group descriptor used for image I/O      */
   extern long ioffset[3];       /* Offsets into "thebuf" for output bands   */
   Arg         args[10];         /* Resource values                          */
   XImage      *main_image;      /* X Image structure                        */
   long        access = IWRITE;  /* File access                              */
   long        argcnt;           /* Argument count                           */
   long        blue;             /* Blue pixel value                         */
   long        bufsiz;           /* Size of image I/O buffer                 */
   long        dtype = EBYTE;    /* Data type of LAS image                   */
   long        green;            /* Green pixel value                        */
   long        i,j;              /* Looping indices                          */
   long        nl;               /* Number of lines                          */
   long        ns;               /* Number of samples                        */
   long        pixel;            /* Single pixel value                       */
   long        red;              /* Red pixel value                          */
   long        sl;               /* Starting line                            */
   long        ss;               /* Starting sample                          */
   long        status;           /* Status indicator                         */
   long        three = 3;        /* Constant three                           */
   long        zero = 0;         /* Constant zero                            */
   char        *thebuf;          /* Image I/O buffer                         */

   /* Capture the specified window from the display screen.  */
   main_image = XGetImage(xid.display,target_win,0,0,save_width,save_height,
                          AllPlanes,ZPixmap);

   /* Set up and display the processing message widget.  */
   argcnt = 0;
   XtSetArg(args[argcnt], XeNprocessMUpdatePercent, 5); argcnt++;
   XtSetValues(process_widget, args, argcnt);
   XtVaSetValues(load_dialog_shell,XmNtitle,"Save - Processing",NULL);

   /* Initialize the image size variables and the processing widget.  */
   sl = ss = 1;
   nl = save_height; 
   ns = save_width; 

   XeProcessMBegin(process_widget,1,nl+1,
                   "Saving image to disk, please wait ...  ",True,True,True);
   XeProcessMProcess(process_widget,1);

   /* Open the LAS image.  */
   status = c_eopens(&fdesc[0],xid_save.name,&three,&sl,&ss,
                     &nl,&ns,ioffset,&access,&dtype,&zero,&zero);
   if (status != E_SUCC)
   {
      xid_message("Error opening LAS image.");
      xid_save.flags |= SAVE_ERROR;
      return;
   }
   fdesc[1] = NULL;

   /* Allocate the I/O buffer.  */
   if ((thebuf = c_ealloc(fdesc,&bufsiz)) == NULL)
   {
      xid_message("Error allocating dynamic memory.");
      xid_save.flags |= SAVE_ERROR;
      return;
   }

   /* Group the images.  */
   if (c_egroup(fdesc,&gdesc,(unsigned char *)thebuf,&bufsiz) != E_SUCC)
   {
      xid_message("Error creating image group.");
      xid_save.flags |= SAVE_ERROR;
      return;
   }

   /* For each line of the image ...  */
   for (i=0;i < nl; i++)
   {

   /* Write out a line of image data.  */
   if (c_estep(&gdesc) != E_SUCC)
   {
      xid_message("Error writing LAS image.");
      xid_save.flags |= SAVE_ERROR;
      return;
   }

   /* Update the processing message widget.  This is necessary for the pause
      and cancel buttons to work even though the widget itself does not change
      based on the current line number.  */
   if (XeProcessMProcess(process_widget,i+1)  == XePROCESSM_CANCEL)
      break;

   /* For each sample of the current line.  */
   for (j=0;j < ns; j++)
   {

      /* Get the pixel value from the X image and scale it to a value between
      0 and 255.  */
      pixel = (long)XGetPixel(main_image,j,i);
      if (xid.visual_info[0].depth == 24)
      {
         red = pixel & 0xff;
         green = (pixel & 0xff00) >> 8;
         blue = (pixel & 0xff0000) >> 16;
      }
      else
         red = green = blue = pixel;

      /* Use the scaled value as an index into the current color table.  Again,
         scale the resulting value to a value between 0 and 255.  */
      thebuf[ioffset[0] + j] = (xid.colortable_ptr[red].red >> 8);
      thebuf[ioffset[1] + j] = (xid.colortable_ptr[green].green >> 8);
      thebuf[ioffset[2] + j] = (xid.colortable_ptr[blue].blue >> 8);
      }
   }

   /* Process the final line of image data.  */
   if (c_estep(&gdesc) != E_SUCC)
   {
      xid_message("Error writing LAS image.");
      xid_save.flags |= SAVE_ERROR;
      return;
   }

   /* Close all image files in the group.  */
   c_egclse(&gdesc);

   /* Free image I/O buffer and destroy the X image.  */
   free(thebuf);
   XDestroyImage(main_image);

   /* Close the processing message window.  */
   XeProcessMEnd(process_widget);
}

#define BLOCK_SIZE      500000.0
void save_all_lasimage
(
   Widget w,
   caddr_t client,
   caddr_t call
)

/******************************************************************************
Purpose:
   This routine saves the user-selected image window to a LAS image file.
   The entire displayed image is saved including those portions of the
   image not currently visible.  The current mapping is applied to the
   image and any graphics data is copied onto the image.  The number of 
   bands saved depends on the type of image displayed.  Three bands will
   be saved for color images and black and white images that have been
   pseudo-colored, that have graphics overlayed or have a graphics mask
   applied (i.e., black and white images that have any color applied).
   If the source image has valid DDR information, this information is
   retained in the resulting LAS image with the appropriate adjustments
   for subsampling, zooming, windowing, etc.

By L. Huewe, USGS/EROS Data Center, 5/93

Modified by L. Huewe, USGS/EROS Data Center, 3/94
   - Modified to retrieve the correct pixel value on 8-bit displays.
Modified by L. Huewe, USGS/EROS Data Center, 9/94
   - Modified to process the image in blocks in order to conserve memory
     usage.

Algorithm Description:
******************************************************************************/
{
   extern struct FDESC *fdesc[2];   /* Image file descriptors used for image
                                       I/O.                                  */
   extern struct GDESC *gdesc;      /* Group descriptor used for image I/O.  */
   extern long    ioffset[3];       /* Offsets into "thebuf" for output bands*/
   Window         image_win;        /* Image window.                         */
   XImage         *image;           /* X Image structure.                    */
   Pixmap         img_pix;          /* Image pixmap.                         */
   Pixmap         *pixptr;          /* Pixmap pointer.                       */
   GC             copy_gc;          /* GC used to copy pixmaps.              */
   XGCValues      values;           /* Values array for the GC.              */
   unsigned long  valuemask = 0;    /* Specify no values.                    */
   long           access = IWRITE;  /* File access.                          */
   long           block;            /* Current block number.                 */
   long           blue;             /* Blue pixel value.                     */
   long           bufsiz;           /* Size of image I/O buffer.             */
   long           dtype = EBYTE;    /* Data type of LAS image.               */
   long           green;            /* Green pixel value.                    */
   long           i,j;              /* Looping indices.                      */
   long           img_idx;          /* Index into color table of the first
                                       image color.                          */
   long           line;             /* Current line number.                  */
   long           nbands;           /* Number of bands in output image.      */
   long           nblocks;          /* Number of blocks to process.          */
   long           nl_block;         /* Number of lines per block.            */
   long           nl;               /* Number of lines.                      */
   long           ns;               /* Number of samples.                    */
   long           num_imgcolors;    /* Number of image colors.               */
   long           pixel;            /* Single pixel value.                   */
   long           plane_idx;        /* Image plane index of current image.   */
   long           red;              /* Red pixel value.                      */
   long           sl;               /* Starting line.                        */
   long           ss;               /* Starting sample.                      */
   long           status;           /* Status indicator.                     */
   long           zero = 0;         /* Constant zero.                        */
   char           *thebuf;          /* Image I/O buffer.                     */

   /* Make sure image data is available for saving.  */
   if ((xid_save.flags & SAVE_MAIN) && (xid.view == -1))
   {
      xid_message("No image data to save in main image window");
      return;
   }
   else if ((xid_save.flags & SAVE_FULL) && (xid_fullres == NULL))
   {
      xid_message("No image data to save in full resolution image window");
      return;
   }
   else if ((xid_save.flags & SAVE_ZOOM) && (xid_zoom.image_pixmap == 0))
   {
      xid_message("No image data to save in zoom image window");
      return;
   }

   /* Assign number of image colors and the starting index of the image colors
      based on the type of visual.  */
   if (*xid.colormap_ptr != xid.colormap)
   {
      num_imgcolors = 256;
      img_idx = 0;
   }
   else
   {
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
   }

   /* Get the memory plane index of the currently displayed image.  */
   plane_idx = xid_view.rgbplanes[RED] - 1;
   sl = ss = 1;

   /* Determine the number of lines and samples in the output image and the
      selected window ID.  Also, set a pointer to the appropriate image data */
   if (xid_save.flags & SAVE_MAIN)
   {
      image_win = XtWindow(main_image_bulletinboard);
      nl = xid_planes[plane_idx].dspwin[NL];
      ns = xid_planes[plane_idx].dspwin[NS];
      pixptr = &xid_mem.image_pixmap;
   }
   else if (xid_save.flags & SAVE_FULL)
   {
      image_win = XtWindow(fullres_win_bulletinboard);
      nl = xid_fullres[0].dspwin[NL];
      ns = xid_fullres[0].dspwin[NS];
      pixptr = &xid_fullres[0].image_pixmap;
   }
   else
   {
      image_win = XtWindow(zoom_win_bulletinboard);
      nl = xid_zoom.parms.factor * xid_zoom.parms.in_lines;
      ns = xid_zoom.parms.factor * xid_zoom.parms.in_samps;
      pixptr = &xid_zoom.image_pixmap;
   }

   /* Initialize and display the processing widget.  */
   XtVaSetValues(load_dialog_shell,XmNtitle,"Save - Processing",NULL);
   XeProcessMBegin(process_widget,0,nl+1,
                   "Saving image to disk, please wait ... ", True,True,True);
   XeProcessMProcess(process_widget,1);

   /* Create the graphics context for saving the image data.  */
   copy_gc = XCreateGC(xid.display,image_win,valuemask,&values);

   /* Determine the number of bands to save.  */
   if ((xid_view.rgbplanes[RED] != xid_view.rgbplanes[GREEN]) || (
      xid_view.rgbplanes[GREEN] != xid_view.rgbplanes[BLUE]))
      nbands = 3;
   else
      nbands = 1;

   /* Save 3 bands if any graphics masks have been applied.  */
   for (i=0; i < MASK_MAX; i++)
      if (xid_mask.mask_pixmap[i] != 0)
         nbands = 3;
   if (nbands == 1)
   {

      /* Save 3 bands if ....  */
      /* Any graphics data is displayed.  */
      if (xid_graphics.pixmap[0] != 0)
         nbands = 3;

      /* A full resolution outline is displayed and the main image window is
         being saved.  */
      else if ((xid_save.flags & SAVE_MAIN) && (xid_fullres != NULL) &&
               (xid_fullres[0].flags & FULLRES_SHOW))
         nbands = 3;

      /* A pseudo-colored image is displayed.  */
      else
      {
         for (i=0; i < num_imgcolors; i++)
         {
            if ((xid.colortable_ptr[img_idx + i].red != xid.colortable_ptr[
                img_idx + i].green) || (xid.colortable_ptr[img_idx + i].green
                != xid.colortable_ptr[img_idx + i].blue))
               nbands = 3;
         }
      }
   }

   /* Open the LAS image.  */
   status = c_eopens(&fdesc[0],xid_save.name,&nbands,&sl,&ss,
                     &nl,&ns,ioffset,&access,&dtype,&zero,&zero);
   if (status != E_SUCC)
   {
      xid_message("Error opening LAS image.");
      xid_save.flags |= SAVE_ERROR;
      return;
   }
   fdesc[1] = NULL;

   /* Allocate the I/O buffer.  */
   if ((thebuf = c_ealloc(fdesc,&bufsiz)) == NULL)
   {
      xid_message("Error allocating dynamic memory.");
      xid_save.flags |= SAVE_ERROR;
      return;
   }

   /* Group the images.  */
   if (c_egroup(fdesc,&gdesc,(unsigned char *)thebuf,&bufsiz) != E_SUCC)
   {
      xid_message("Error creating image group.");
      xid_save.flags |= SAVE_ERROR;
      return;
   }

   /* Calculate the number of lines per block and the number of blocks of image
      data to process.  */
   nl_block = (long)(BLOCK_SIZE / (float)ns);
   nblocks = (long)ceil((double)((float)nl / (float)nl_block));

   /* Create the pixmap for saving image data.  */
   img_pix = XCreatePixmap(xid.display, image_win, ns, nl_block, 
                           xid.visual_info[0].depth);

   /* Process the image in blocks to conserve memory usage.  */
   for (block = 0, line = 0; block < nblocks; block++)
   {
      if (block == (nblocks - 1))
         nl_block = nl - (nl_block * block);

      /* Copy the appropriate image data into the pixmap.  */
      XCopyArea(xid.display, *pixptr, img_pix, copy_gc, 0, line, ns, nl_block, 
                0, 0);

      /* Apply any graphics data to the image.  */
      save_copy_graphics(img_pix,line,nl_block,ns);

      /* Read a block of image data into an X image structure.  */
      image = XGetImage(xid.display, img_pix, 0, 0, ns, nl_block, AllPlanes, 
                        ZPixmap);

      /* For each line of the block.  */
      for (i=0;i < nl_block;i++,line++)
      {

         /* Write out a line of image data.  */
         if (c_estep(&gdesc) != E_SUCC)
         {
            xid_message("Error writing LAS image.");
            xid_save.flags |= SAVE_ERROR;
            return;
         }

         /* Update the processing message widget.  This is necessary for the
            pause and cancel buttons to work even though the widget itself does
            not change based on the current line number.  */
         if (XeProcessMProcess(process_widget,i) == XePROCESSM_CANCEL)
         {
            printf("User Cancelled\n");
            break;
         }

         /* For each sample of the current line.  */
         for (j=0;j < ns;j++)
         {

            /* Get the pixel value from the X image and scale it to a value
               between 0 and 255.  */
            pixel = (long)XGetPixel(image,j,i);
            if (xid.visual_info[0].depth == 24)
            {
               red = pixel & 0xff;
               green = (pixel & 0xff00) >> 8;
               blue = (pixel & 0xff0000) >> 16;
            }
            else
               red = green = blue = pixel;

            /* Use the scaled value as an index into the current color table.
               Again, scale the resulting value to a value between 0 and 255 */
            thebuf[ioffset[0] + j] = (char) (xid.colortable_ptr[red].red >> 8);
            if (nbands > 1)
            {
               thebuf[ioffset[1] + j] = (char) (xid.colortable_ptr[green].
                                                green >> 8);
               thebuf[ioffset[2] + j] = (char) (xid.colortable_ptr[blue].
                                                blue >> 8);
            }
         }
      }
      XDestroyImage(image);
   }

   /* Process the final line of image data.  */
   if (c_estep(&gdesc) != E_SUCC)
   {
      xid_message("Error writing LAS image.");
      xid_save.flags |= SAVE_ERROR;
      return;
   }

   /* Close all image files in the group.  */
   c_egclse(&gdesc);

   /* Free the image I/O buffer and destroy the X image.  */
   free(thebuf);
   XFreePixmap(xid.display,img_pix);
   XFreeGC(xid.display,copy_gc);
   
   /* Update the DDR information.  */
   save_upddr();

   /* Close the processing message window.  */
   XeProcessMEnd(process_widget);

   /* Close the save panel */
   save_destroy(w,client,call);

   /* Let the user know we are finished.  */
   XBell(xid.display,0);
}

void save_win
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   Sets the appropriate flags indicating which image window is to be
   saved.

By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
******************************************************************************/
{

   /* Set the flag indicating the window to save.  */
   xid_save.flags = xid_save.flags & ~(SAVE_MAIN | SAVE_FULL | SAVE_ZOOM);
   if (strcmp(client,"MW") == 0)
      xid_save.flags = xid_save.flags | SAVE_MAIN;
   else if (strcmp(client,"FW") == 0)
      xid_save.flags = xid_save.flags | SAVE_FULL;
   else if (strcmp(client,"ZW") == 0)
      xid_save.flags = xid_save.flags | SAVE_ZOOM;
}

void save_frame
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   Sets the appropriate flags indicating whether or not to save the window
   manager frame with the image data.

By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
******************************************************************************/
{

   /* Set the flag indicating whether to include the window manager frame.  */
   xid_save.flags = xid_save.flags & ~(SAVE_FRAME | SAVE_NOFRAME);
   if (strcmp(client,"F") == 0)
      xid_save.flags = xid_save.flags | SAVE_FRAME;
   else if (strcmp(client,"NF") == 0)
      xid_save.flags = xid_save.flags | SAVE_NOFRAME;
}

void save_type
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   Sets the appropriate flags indicating how much of the image to save.  
   Selecting the ALL option indicates the entire image loaded into memory 
   is to be saved (including any image data not currently visible).  Selecting
   the VIEW option indicates only the selected window is to be saved (this
   option works much like the X application xwd).

By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
******************************************************************************/
{

   /* Set the flag indicating how much image data to save.  */
   xid_save.flags = xid_save.flags & ~(SAVE_ALL | SAVE_VIEW);
   if (strcmp(client,"ALL") == 0)
   {
      xid_save.flags = xid_save.flags | SAVE_ALL;
      XtUnmapWidget(save_frame_option);
      XtMapWidget(save_win_option);
   }
   else if (strcmp(client,"VIEW") == 0)
   {
      xid_save.flags = xid_save.flags | SAVE_VIEW;
      XtUnmapWidget(save_win_option);
      XtMapWidget(save_frame_option);
   }
}

/* ARGSUSED */
void save_imagename
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine retrieves the image name specified by the user, determines
   whether the name entered is a LAS name or a host name and gets the
   host file name if a LAS name was specified.  It then calls the save_apply
   routine to actually save the image.
                                                                
By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
******************************************************************************/
{
   long  lasflg = FALSE;    /* Indicates whether the file name specified is
                               a LAS name or a host file name.               */
   long  status;            /* Status indicator.                             */
   char  *img_name;         /* User specified image name.                    */
   char  *hname;            /* Host file name.                               */
   char  out_name[CMLEN];   /* LAS image name with extension.                */
   char  *user_name;        /* User specified image name.                    */

   /* Get the image name specified by the user.  */
   img_name = XmTextGetString(save_image_text);
   if (*img_name == '\0')
   {
      xid_message("A null image name is not valid");
      return;
   }
   user_name = malloc(strlen(img_name) + 1);
   if (user_name == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   strcpy(user_name,img_name);
   XtFree(img_name);

   /* Determine whether the input name is a LAS name or a host name.  - If the
      name contains a slash (/) it is assumed to be a host file name, all other
      names are assumed to be LAS names.  */
   if (strcspn(user_name,"/") != strlen(user_name))
      lasflg = FALSE;
   else
      lasflg = TRUE;

   /* Get the host file name of the specified LAS name.  */
   if (lasflg)
   {
      status = c_cmhost(user_name,out_name);
      if (status != E_SUCC)
      {
         xid_message("Error retrieving host file name");
         return;
      }
      hname = (char *)malloc(strlen(out_name) + 5);
      if (hname == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
      c_lsmknm(out_name,"img",hname);
   }
   else
   {
      hname = (char *)malloc(strlen(user_name) + 1);
      if (hname == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
      strcpy(hname,user_name);
   }

   xid_save.name = (char *)malloc(strlen(hname) + 1);
   if (xid_save.name == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   strcpy(xid_save.name,hname);
   free(hname);
   free(user_name);

   /* Now call the routine to actually save the image */
   save_apply(w, client, call);
}

/* ARGSUSED */
void save_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine sets up and calls the appropriate "save" routine based
   on the user-specified options.  If the user is saving only the visible
   portion of the image, the user must interactively select the window to
   be saved (much like the X client xwd).
                                                                
By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
******************************************************************************/
{
   XEvent   event;      /* X event structure.                                */
   Cursor   cursor;     /* Cursor ID.                                        */
   Widget   widget;     /* Widget ID of the widget associated with
                           the selected window.                              */
   Window   root=RootWindow(xid.display,xid.screen);
                        /* Root window ID.                                   */
   Window   target_win=None;
                        /* Window ID of the user-selected window.            */
   XWindowAttributes 
            win_info;   /* Window information for the user-selected
                           window.                                           */
   int      buttons=0;  /* Mouse button.                                     */
   int      status;     /* Status indicator.                                 */

   /* The user must specify an image name.  */
   if ((xid_save.name == NULL) || (strcmp(xid_save.name,"") == 0))
   {
      xid_message("User must specify an image name");
      xid_save.flags |= SAVE_ERROR;
      return;
   }

   /* If saving only the visible portion of the image, allow the user to select
      the window to be saved. Technically, this does not have to be an image
      window.  This can be any window currently on the display including the
      root window.  The following code that allows the user to select the
      window to be saved was in large part borrowed from the X11R5 client xwd*/
   if (xid_save.flags & SAVE_VIEW)
   {

   /* Display the crosshair cursor.  */
      cursor = XCreateFontCursor(xid.display, XC_crosshair);

      /* Grab the pointer.  The user can do nothing else at this point until a
         button press event is received indicating they have selected a
         window. */
      status = XGrabPointer(xid.display,root,False,
                            ButtonPressMask|ButtonReleaseMask,GrabModeSync,
                            GrabModeAsync,root, cursor, CurrentTime);
      if (status != GrabSuccess)
      {
         xid_message("Error grabbing the mouse");
         xid_save.flags |= SAVE_ERROR;
         return;
      }

      /* Let the user select a window ...  */
      while ((target_win == None) || (buttons != 0)) 
      {

         /* Wait for a button press event indicating the user has selected a
            window.  */
         XAllowEvents(xid.display,SyncPointer,CurrentTime);
         XWindowEvent(xid.display,root,ButtonPressMask|ButtonReleaseMask,
                      &event);
         switch (event.type) 
         {
            case ButtonPress:
               if (target_win == None) 
               {
                  target_win = event.xbutton.subwindow;
                  if (target_win == None) 
                  target_win = root;
               }
               buttons++;
               break;
            case ButtonRelease:

               /* There may have been some buttons down before we started.  */
               if (buttons > 0)
                  buttons--;
               break;
         }
      }

      /* Release the pointer.  */
      XUngrabPointer(xid.display, CurrentTime);

      /* If the user does not want the window manager frame, get the
         appropriate window ID.  */
      if (xid_save.flags & SAVE_NOFRAME)
         target_win = XmuClientWindow (xid.display, target_win);

      /* Get the selected window's size and other attribute information.  */
      XGetWindowAttributes(xid.display,target_win,&win_info);

      /* Let the user know we are starting the save process.  */
      XBell(xid.display,0);

      /* If the window specified is one of the XID image windows and the
         window manager frame is not to be saved, don't save scrollbars with
         the image.  */
      widget = XtWindowToWidget(xid.display,target_win);
      if ((widget == Shell_MainWin) || (widget == Shell_FullResWin) || (widget
          == Shell_ZoomWin))
         save_lasimage(target_win,win_info.width-xid_mem.view_xoffset,
                       win_info.height-xid_mem.view_yoffset);
      else

      /* Save the entire window.  */
      save_lasimage(target_win,win_info.width,win_info.height);

      /* Close the save panel */
      save_destroy(w,client,call);

      /* Let the user know we are finished processing.  */
      XBell(xid.display,0);
   }
   else

   /* Save the entire image that has been loaded into memory.  */
   save_all_lasimage(w, client, call);
}

void save_copy_graphics
(
   Pixmap img_pix,  /* Image pixmap.  (I)                                    */
   long sl,         /* Starting line number.  (I)                            */
   long nl,         /* Number of lines in the pixmap.  (I)                   */
   long ns          /* Number of samples in the pixmap.  (I)                 */
)

/******************************************************************************
Purpose:
   This routine copies any currently displayed graphics data to the input
   image pixmap.
                                                                
By L. Huewe, USGS/EROS Data Center, 9/93
Modified by L. Huewe USGS/EROS Data Center, 9/94
   - Modified to allow copying of graphics data for just a portion of the
     image (added sl parameter).

Algorithm Description:
******************************************************************************/
{
   GC       graphics_gc;     /* Graphics context for copying graphics data.  */
   XColor   rect_color;      /* Rectangle color for full resolution
                                outline.                                     */
   Boolean  argok;           /* Status indicator.                            */
   Pixmap   tmp_pix = 0;
                             /* Temporary pixmap.                            */
   long     height;          /* Height of rectangle.                         */
   long     i;               /* Looping index.                               */
   long     plane_idx;       /* Index into memory plane structure of the
                                currently viewed image.                      */
   long     width;           /* Width of rectangle.                          */
   int      xwin;            /* X window coordinate.                         */
   int      ywin;            /* Y window coordinate.                         */

   /* If the main image window is being saved and a full resolution outline is
      displayed, save it along with the image data.  */
   if ((xid_save.flags & SAVE_MAIN) && (xid_fullres != NULL))
   {
      if ((xid_fullres[0].window[SS] > 0) && (xid_fullres[0].flags &
                                              FULLRES_SHOW))
      {

         /* Find the memory plane index of the first band of the displayed
            image.  */
         plane_idx = xid_view.rgbplanes[RED] - 1;

         /* Draw the full resolution outline.  */
         XParseColor(xid.display,xid.colormap,"green",&rect_color);
         XSetForeground(xid.display,xid_fullres[0].gc,xid.color_table[RGBBW_IDX
                        + GREEN_COL].pixel);
         XSetFunction(xid.display,xid_fullres[0].gc,GXcopy);
         xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
                      (float)xid_fullres[0].window[SL],&xwin,&ywin);
         width = (long)((float)xid_fullres[0].width /
                        (float)xid_planes[plane_idx].skips);
         height = (long)((float)xid_fullres[0].height /
                         (float)xid_planes[plane_idx].skipl);

         /* Make the necessary adjustments to the Y coordinate if we are
            processing a block of image data that does not start at the first
            line.  */
         if ((ywin + height - 1) >= sl)
         {
            if (ywin < sl)
            {
               ywin = -1;
               height = height - sl + 1;
            }
            else
               ywin -= sl;
            XDrawRectangle(xid.display, img_pix, xid_fullres[0].gc, xwin, ywin,
                           width, height);
            XSetForeground(xid.display, xid_fullres[0].gc, 
                           xid_fullres[0].rb_fore);
            XSetFunction(xid.display, xid_fullres[0].gc, GXxor);
         }
      }
   }

   /* Copy graphics information from the pixmap to the main image window.  */
   graphics_gc = XCreateGC(xid.display, XtWindow(Shell_MainMenu), 0, NULL);

   if ((xid_save.flags & SAVE_MAIN) && (xid_graphics.pixmap[0] != 0))
   {

      /* Create a temporary pixmap to contain the graphics for the current
         image window.  */
      if (sl != 0)
         tmp_pix = XCreatePixmap(xid.display,
                                 XtWindow(main_image_bulletinboard),ns,nl,1);
      for (i=0; i < USER_GRCOLORS; i++)
      {
         XSetForeground(xid.display, graphics_gc, (unsigned long)CONVERT(
                        Shell_Graphics, xid_graphics.colors[i],"Pixel", 0,
                        &argok));
         if (sl != 0)
         {

            /* Copy the appropriate window of graphics data to the temporary
               pixmap and then overlay it on the input pixmap.  */
            XCopyArea(xid.display, xid_graphics.pixmap[i], tmp_pix, 
                      xid_graphics.rb_gc, 0, sl, ns, nl, 0, 0);
            XSetClipMask(xid.display,graphics_gc,tmp_pix);
            XCopyPlane(xid.display,tmp_pix,img_pix,graphics_gc,0,0,ns,nl,
                       0,0,1);
         }
         else
         {

            /* Overlay the input pixmap with the appropriate graphics data.  */
            XSetClipMask(xid.display,graphics_gc,xid_graphics.pixmap[i]);
            XCopyPlane(xid.display,xid_graphics.pixmap[i],img_pix,graphics_gc,
                       0,0,ns,nl,0,0,1);
         }
      }
      if (sl != 0)
         XFreePixmap(xid.display, tmp_pix);

      /* Draw graphics handles if currently editing graphics.  */
      if (xid_graphics.flags & GR_EDIT)
      {
         if (xid_editgraph.current != -1)
            graphics_handles(xid_editgraph.current,0,img_pix,sl);
      }
   }

   /* Copy graphics information from the pixmap to the full res. window.  */
   else if ((xid_save.flags & SAVE_FULL) && (xid_graphics.full_pix[0] != 0))
   {

      /* Create a temporary pixmap to contain the graphics for the current
         image window.  */
      if (sl != 0)
         tmp_pix=XCreatePixmap(xid.display,XtWindow(fullres_win_bulletinboard),
                               ns, nl, 1);
      for (i=0; i < USER_GRCOLORS; i++)
      {
         XSetForeground(xid.display, graphics_gc, (unsigned long)CONVERT(
                        Shell_Graphics, xid_graphics.colors[i],"Pixel", 0,
                        &argok));
         if (sl != 0)
         {

            /* Copy the appropriate window of graphics data to the temporary
               pixmap and then overlay it on the input pixmap.  */
            XCopyArea(xid.display, xid_graphics.full_pix[i], tmp_pix, 
                      xid_graphics.rb_gc, 0, sl, ns, nl, 0, 0);
            XSetClipMask(xid.display,graphics_gc,tmp_pix);
            XCopyPlane(xid.display,tmp_pix,img_pix,graphics_gc,0,0,ns,nl,
                       0,0,1);
         }
         else
         {

            /* Overlay the input pixmap with the appropriate graphics data.  */
            XSetClipMask(xid.display,graphics_gc,xid_graphics.full_pix[i]);
            XCopyPlane(xid.display,xid_graphics.full_pix[i],img_pix,
                       graphics_gc,0,0,ns,nl,0,0,1);
         }
      }
      if (sl != 0)
         XFreePixmap(xid.display, tmp_pix);

      /* Draw graphics handles if currently editing graphics.  */
      if (xid_graphics.flags & GR_EDIT)
      {
         if (xid_editgraph.current != -1)
            graphics_handles(xid_editgraph.current,1,img_pix,sl);
      }
   }

   /* Copy graphics information from the pixmap to the zoom window.  */
   else if ((xid_save.flags & SAVE_ZOOM) && (xid_graphics.zoom_pix[0] != 0))
   {

      /* Create a temporary pixmap to contain the graphics for the current
         image window.  */
      if (sl != 0)
      {
         tmp_pix = XCreatePixmap(xid.display, XtWindow(zoom_win_bulletinboard),
                                 ns, nl, 1);
      }
      for (i=0; i < USER_GRCOLORS; i++)
      {
         XSetForeground(xid.display, graphics_gc, (unsigned long)CONVERT(
                        Shell_Graphics, xid_graphics.colors[i],"Pixel", 0,
                        &argok));
         if (sl != 0)
         {

            /* Copy the appropriate window of graphics data to the temporary
               pixmap and then overlay it on the input pixmap.  */
            XCopyArea(xid.display, xid_graphics.zoom_pix[i], tmp_pix, 
                      xid_graphics.rb_gc, 0, sl, ns, nl, 0, 0);
            XSetClipMask(xid.display,graphics_gc,tmp_pix);
            XCopyPlane(xid.display,tmp_pix,img_pix,graphics_gc,0,0,ns,nl,
                       0,0,1);
         }
         else
         {

            /* Overlay the input pixmap with the appropriate graphics data.  */
            XSetClipMask(xid.display,graphics_gc,xid_graphics.zoom_pix[i]);
            XCopyPlane(xid.display,xid_graphics.zoom_pix[i],img_pix,
                       graphics_gc,0,0,ns,nl,0,0,1);
         }
      }
      if (sl != 0)
         XFreePixmap(xid.display, tmp_pix);

      /* Draw graphics handles if currently editing graphics.  */
      if (xid_graphics.flags & GR_EDIT)
      {
         if (xid_editgraph.current != -1)
            graphics_handles(xid_editgraph.current,2,img_pix,sl);
      }
   }

   /* Free the graphics context.  */
   XFreeGC(xid.display, graphics_gc);
}

void save_upddr
(void)

/******************************************************************************
Purpose:
   This routine retrieves the projection information from the IMAGE_INFO
   structure for the source image(s) and attempts to update the output
   image's DDR.  This allows the output image to retain the projection
   information and to have updated corner coordinates based on the
   subsampling factor, zoom factor, window, etc.

By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
******************************************************************************/
{
   struct DDR    in_ddr[3];     /* DDRs for the displayed image(s).          */
   struct DDR    out_ddr;       /* DDR for output image.                     */
   struct BDDR   in_bddr[3];    /* Band dependent DDR records for displayed
                                   image(s).                                 */
   struct BDDR   out_bddr[3];   /* Band dependent DDR record for output image*/
   double        line_inc;      /* Line increment.                           */
   double        samp_inc;      /* Sample increment.                         */
   long          cflag = NOCOMB;/* Do not combine bands.                     */
   long          i,j;           /* Looping indices.                          */
   long          nbands[3];     /* Number of bands for each image.           */
   long          num_image;     /* Number of different images making up the
                                   displayed image.                          */
   long          rgbidx[3];     /* Indices to red, green and blue plane
                                   numbers of the currently viewed image.    */
   long          skipl[3];      /* Line subsampling factors.                 */
   long          skips[3];      /* Sample subsampling factors.               */
   long          status;        /* Status indicator.                         */
   long          windows[3][4]; /* Window specifications for displayed images*/

   /* Get the memory plane indices of the displayed image.  */
   rgbidx[RED] = xid_view.rgbplanes[RED] - 1;
   rgbidx[GREEN] = xid_view.rgbplanes[GREEN] - 1;
   rgbidx[BLUE] = xid_view.rgbplanes[BLUE] - 1;

   /* Determine the number of input images.  */
   if ((rgbidx[RED] == rgbidx[GREEN]) && (rgbidx[GREEN] == rgbidx[BLUE]))
      num_image = 1;
   else
   {
      if ((strcmp(xid_planes[rgbidx[RED]].filename,xid_planes[rgbidx[GREEN]].
          filename) == 0) && (strcmp(xid_planes[rgbidx[GREEN]].filename,
          xid_planes[rgbidx[BLUE]].filename) == 0))
         num_image = 1;
      else
         num_image = 3;
    }

   /* Get the output image's DDR and band dependent records.  */
   status = c_getddr(xid_save.name,&out_ddr);
   if (status != E_SUCC)
   {
      xid_message("Error udpating DDR");
      return;
   }
   for (i=1; i <= out_ddr.nbands; i++)
   {
      status = c_getbdr(xid_save.name,&out_bddr[i-1],&i);
      if (status != E_SUCC)
      {
         xid_message("Error updating DDR");
         return;
      }
   }

   /* Set the appropriate fields in the input DDRs based on the information
      stored in the IMAGE_INFO structure and in the memory plane structure.  */
   for (i=0; i < num_image; i++)
   {

      /* Set number of lines, samples and bands.  */
      in_ddr[i].nl = xid_planes[rgbidx[i]].image_info->nl;
      in_ddr[i].ns = xid_planes[rgbidx[i]].image_info->ns;
      if (num_image == 3) 
         in_ddr[i].nbands = 1;
      else if ((num_image == 1) && (out_ddr.nbands == 1))
         in_ddr[i].nbands = 1;
      else
         in_ddr[i].nbands = 3;

      /* Always working with byte data.  */
      in_ddr[i].dtype = 1;

      /* Set the master line and sample offsets.  */
      in_ddr[i].master_line = xid_planes[rgbidx[i]].image_info->master_line;
      in_ddr[i].master_sample=xid_planes[rgbidx[i]].image_info->master_sample;

      /* Set the valid flags.  */
      if (xid_planes[rgbidx[i]].proj_valid)
      {
         in_ddr[i].valid[DDPCV] = VALID;
         in_ddr[i].valid[DDZCV] = VALID;
         in_ddr[i].valid[DDDCV] = VALID;
         in_ddr[i].valid[DDPPV] = VALID;
      }
      else
      {
         in_ddr[i].valid[DDPCV] = INVAL;
         in_ddr[i].valid[DDZCV] = INVAL;
         in_ddr[i].valid[DDDCV] = INVAL;
         in_ddr[i].valid[DDPPV] = INVAL;
      }

      if (strcmp(xid_planes[rgbidx[i]].image_info->proj_units,"") == 0)
         in_ddr[i].valid[DDPUV] = INVAL;
      else
         in_ddr[i].valid[DDPUV] = VALID;

      if ((xid_planes[rgbidx[i]].image_info->pdist_x==0) || (xid_planes[rgbidx
          [i]].image_info->pdist_y == 0))
         in_ddr[i].valid[DDPDV] = INVAL;
      else
         in_ddr[i].valid[DDPDV] = VALID;

      if (xid_planes[rgbidx[i]].corner_valid)
         in_ddr[i].valid[DDCCV] = VALID;
      else
         in_ddr[i].valid[DDCCV] = INVAL;

      in_ddr[i].valid[DDINCV] = VALID;

      /* Set the projection information.  */
      in_ddr[i].proj_code = xid_planes[rgbidx[i]].image_info->proj_code;
      in_ddr[i].zone_code = xid_planes[rgbidx[i]].image_info->zone_code;
      in_ddr[i].datum_code = xid_planes[rgbidx[i]].image_info->datum_code;
      strcpy(in_ddr[i].proj_units,xid_planes[rgbidx[i]].image_info->proj_units);
      for (j=0; j < 15; j++)
         in_ddr[i].proj_coef[j]=xid_planes[rgbidx[i]].image_info->proj_coef[j];
      for (j=0; j < 2; j++)
      {
         in_ddr[i].upleft[j] = xid_planes[rgbidx[i]].image_info->upleft[j];
         in_ddr[i].loleft[j] = xid_planes[rgbidx[i]].image_info->loleft[j];
         in_ddr[i].upright[j] = xid_planes[rgbidx[i]].image_info->upright[j];
         in_ddr[i].loright[j] = xid_planes[rgbidx[i]].image_info->loright[j];
      }
      in_ddr[i].pdist_x = xid_planes[rgbidx[i]].image_info->pdist_x;
      in_ddr[i].pdist_y = xid_planes[rgbidx[i]].image_info->pdist_y;
      in_ddr[i].line_inc = xid_planes[rgbidx[i]].image_info->line_inc;
      in_ddr[i].sample_inc = xid_planes[rgbidx[i]].image_info->sample_inc;

      /* Set the system field.  */
      status = c_getsys(in_ddr[i].system);
      if (status != E_SUCC)
      {
         xid_message("Error updating DDR");
         return;
      }

      /* Set fields in the band dependent records.  */
      for (j=0; j < in_ddr[i].nbands; j++)
      {
         in_bddr[i+j].bandno = j+1; 
         in_bddr[i+j].valid = INVAL;
         in_bddr[i+j].minval = 0; 
         in_bddr[i+j].maxval = 0; 
         strcpy(in_bddr[i+j].source,"");
         strcpy(in_bddr[i+j].instrument,"");
         strcpy(in_bddr[i+j].direction,"");
         strcpy(in_bddr[i+j].date,"");
         strcpy(in_bddr[i+j].time,"");
      }
   }

   /* If we are saving the zoom window and the zoom window has been created
      from a main image window which has been subsampled, we must go through a
      2-step process to update the DDR in order to preserve the proper line and
      sample increments.  */
   if ((xid_save.flags & SAVE_ZOOM) && (xid_zoom.flags & ZOOM_MAIN) &&
       ((xid_planes[rgbidx[RED]].skipl != 1) || (xid_planes[rgbidx[RED]].skips 
       != 1)))
   {
      for (i=0; i < num_image; i++)
      {
         skipl[i] = xid_planes[rgbidx[i]].skipl;
         skips[i] = xid_planes[rgbidx[i]].skips;
      }
      save_zoomddr(in_ddr,in_bddr,num_image,skipl,skips);
   }

   /* Set the parameters to the routine which will update the DDR.  */
   if (xid_save.flags & SAVE_MAIN)
   {
      for (i=0; i < num_image; i++)
      {
         windows[i][SL] = xid_planes[rgbidx[i]].window[SL]; 
         windows[i][SS] = xid_planes[rgbidx[i]].window[SS];
         windows[i][NL] = xid_planes[rgbidx[i]].window[NL];
         windows[i][NS] = xid_planes[rgbidx[i]].window[NS];
         nbands[i] = in_ddr[i].nbands;
      }
      line_inc = (double)xid_planes[rgbidx[RED]].skipl;
      samp_inc = (double)xid_planes[rgbidx[RED]].skips;
   }
   else if (xid_save.flags & SAVE_FULL)
   {
      for (i=0; i < num_image; i++)
      {
         windows[i][SL] = xid_fullres[0].window[SL]; 
         windows[i][SS] = xid_fullres[0].window[SS];
         windows[i][NL] = xid_fullres[0].window[NL];
         windows[i][NS] = xid_fullres[0].window[NS];
         nbands[i] = in_ddr[i].nbands;
      }
      line_inc = 1.0;
      samp_inc = 1.0; 
   }
   else if (xid_save.flags & SAVE_ZOOM)
   {
      if (xid_zoom.flags & ZOOM_MAIN)
      {
         for (i=0; i < num_image; i++)
         {
            windows[i][SL] = xid_zoom.window[SL];
            windows[i][SS] = xid_zoom.window[SS];
            windows[i][NL] = xid_zoom.window[NL];
            windows[i][NS] = xid_zoom.window[NS];
            nbands[i] = in_ddr[i].nbands;
         }
      }
      else if (xid_zoom.flags & ZOOM_FULLRES)
      {
         for (i=0; i < num_image; i++)
         {
            windows[i][SL] = xid_fullres[0].window[SL]+xid_zoom.window[SL]-1;
            windows[i][SS] = xid_fullres[0].window[SS]+xid_zoom.window[SS]-1;
            windows[i][NL] = xid_zoom.window[NL];
            windows[i][NS] = xid_zoom.window[NS];
            nbands[i] = in_ddr[i].nbands;
         }
         line_inc = 1.0 / (float)xid_zoom.factor;
         samp_inc = 1.0 / (float)xid_zoom.factor; 
      }
   }

   /* Update the output image's DDR.  */
   status = notae_upddr(INVAL,0.5,in_ddr,&out_ddr,in_bddr,out_bddr,&num_image,
                        windows,nbands,&line_inc,&samp_inc,&out_ddr.nbands,
                        &cflag);
   if (status != E_SUCC)
   {
      xid_message("Error updating DDR");
      return;
   }

   /* Fix the zoom window corner coordinates.  The routine which updates the
      DDR does not calculate the corner coordinates of the zoom window
      properly.  I believe this is due to the fact that XID calculates the zoom
      image differently than other LAS routines.  Using an LAS support routine
      to update the DDR isn't accurate.  Corner coordinates are therefore
      calculated manually.  */
   if (xid_save.flags & SAVE_ZOOM)
   { 

      /* Determine the projection coordinates of the upper left corner.  */
      if (in_ddr[i].valid[DDCCV] == VALID) 
      {
         status = xid_win2proj(rgbidx[RED],2,0,0,&out_ddr.upleft[1],
         &out_ddr.upleft[0]);
         if (status != XID_S)
         {
            xid_message("Error updating DDR");
            out_ddr.valid[DDCCV] = INVAL;
         }
      }
      else
         out_ddr.valid[DDCCV] = INVAL;

      /* Calculate the other 3 corners given the image size and projection
         distance.  */
      out_ddr.upright[0] = out_ddr.upleft[0];
      out_ddr.upright[1] = out_ddr.upleft[1] + ((xid_zoom.window[NS] *
                           xid_zoom.factor - 1) * out_ddr.pdist_x);

      out_ddr.loleft[0] = out_ddr.upleft[0] - ((xid_zoom.window[NL] *
                          xid_zoom.factor - 1) * out_ddr.pdist_y);
      out_ddr.loleft[1] = out_ddr.upleft[1];

      out_ddr.loright[0] = out_ddr.upleft[0] - ((xid_zoom.window[NL] *
                           xid_zoom.factor - 1) * out_ddr.pdist_y);
      out_ddr.loright[1] = out_ddr.upleft[1] + ((xid_zoom.window[NS] *
                           xid_zoom.factor - 1) * out_ddr.pdist_x);
   }

   /* Write the band independent information to the output ddr.  */
   status = c_putddr(xid_save.name,&out_ddr);
   if (status != E_SUCC)
   {
      xid_message("Error updating DDR");  
      return;
   }

   for (i = 0; i < out_ddr.nbands; i++)
   {

      /* Write each of the band dependent records to the output ddr.  */
      status = c_putbdr(xid_save.name,&(out_bddr[i]));
      if (status != E_SUCC)
      {
         xid_message("Error updating DDR");
         return;
      }
   }
}

int save_zoomddr
(
   struct DDR *in_ddr,    /* Band independent DDR record. (I/O)             */
   struct BDDR *in_bddr,  /* Band dependent DDR records. (I/O)              */
   long num_image,        /* Number of input images. (I)                    */
   long skipl[3],         /* Line subsampling factors. (I)                  */
   long skips[3]          /* Sample subsampling factors. (I)                */
)

/******************************************************************************
Purpose:
   This routine updates the input DDR based on the input subsampling factor.

By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
******************************************************************************/
{
   struct  DDR    temp_ddr[3];     /* Updated DDR record.                    */
   struct  BDDR   temp_bddr[3];    /* Updated band dependent DDR record.     */
   double         line_inc;        /* Line increment.                        */
   double         samp_inc;        /* Sample increment.                      */
   long           cflag = NOCOMB;  /* Do not combine bands.                  */
   long           i,j;             /* Looping indices.                       */
   long           nbands[3];       /* Number of bands for each image.        */
   long           one = 1;         /* Constant one.                          */
   long           status;          /* Status indicator.                      */
   long           windows[3][4];   /* Window specifications for displayed
                                      images.                                */

   for (i=0; i < num_image; i++)
   {

      /* Assign the input DDR to a temporary variable.  */
      temp_ddr[i] = in_ddr[i];

      /* Keep the same window and bands.  */
      windows[0][SL] = 1;
      windows[0][SS] = 1;
      windows[0][NL] = in_ddr[i].nl;
      windows[0][NS] = in_ddr[i].ns;
      nbands[0] = in_ddr[i].nbands;
      for (j=0; j < nbands[0]; j++)
         temp_bddr[j] = in_bddr[i+j];

      /* Assign line and sample increments.  */
      line_inc = (double)skipl[i];
      samp_inc = (double)skips[i];

      /* Update the DDR.  */
      status = notae_upddr(INVAL,0.5,in_ddr,&temp_ddr[i],in_bddr,temp_bddr,
                           &one,windows,nbands,&line_inc,&samp_inc,
                           &in_ddr[i].nbands,&cflag);
      if (status != E_SUCC)
      {
         xid_message("Error updating DDR");
         return(XID_F);
      }
      else
      {
         in_ddr[i] = temp_ddr[i];
         return(XID_S);
      }
   }
   return(XID_S);
}

/* ARGSUSED */
void save_tiff_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine retrieves the file name specified by the user, parses the
   name, and pops up a window if they specified a file that already exists so
   so the user can verify that they want to overwrite the existing file.

By A.Shimitz, USGS/EROS Data Center, 6/00

Algorithm Description:
******************************************************************************/
{
   char    *initfilename = NULL;  /* Initial filename given                  */
   char    directory[CMLEN];      /* Directory portion of outfile            */
   char    extension[CMLEN];      /* Extension portion of outfile            */
   char    outfile[CMLEN];        /* Output TIFF filename                    */
   char    root[CMLEN];           /* Root portion of outfile                 */
   long    extflag;               /* Indicates whether the user has
                                     specified the extension of the outfile  */
   long    lasflg;                /* Indicates whether a LAS name or host
                                     name was specified                      */
   long    zero = 0;              /* zero                                    */
   XmFileSelectionBoxCallbackStruct
           *acs = (XmFileSelectionBoxCallbackStruct*)call;
                                  /* pointer to the file selection box
                                     structure                               */

   /* Retrieve the file name selected. */
   if(!(XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&initfilename)) ||
      (initfilename == NULL))
   {
      xid_message("Error retrieving file name.");
      return;
   }

   /* Since there are no restrictions on the length of initfilename, we need
      to check that it is not too long to be copied to outfile[CMLEN] */
   if(strlen(initfilename) > (NAMLEN - 1)) /* (CMLEN - 1) = 255 */
   {
      xid_message("Filename exceeds allowable length.");
      return;
   }
   strcpy(outfile, initfilename);
   XtFree(initfilename);
   initfilename = NULL;

   /* Check if the file was specified in LAS format.  If so, display an
      error message.  If the name contains a bracket or a semicolon ([,],or ;),
      it is assumed to be a LAS name. */
   lasflg = FALSE;
   if((strchr(outfile, '[') != NULL) || (strchr(outfile, ']') != NULL) ||
      (strchr(outfile, ';') != NULL))
      lasflg = TRUE;

   /* Have the user re-enter the outfile if they specified it in LAS format */
   if(lasflg)
   {
      xid_message("Invalid file name.");
      free(outfile);
      return;
   }   

   /* Make sure a filename has been specified */
   c_splitname(outfile, directory, root, extension);
   if(strcmp(root, "") == 0)
   {
      xid_message("Please specify a file name.");
      return;
   }

   /* Check if they have specified the extension of the outfile and append
      ".tif" if they have not. Note that we are not checking the validity of
      the file extension, that is left up to the user. */
   extflag = FALSE;
   if(strcmp(extension, "") != 0)
      extflag = TRUE;

   /* Append the default ".tif" extension, if one was not specified */
   if(!extflag)
      strcat(outfile, ".tif");

   /* Update the directory mask */
   if (xid_load.savetiff_dirmask != NULL)
      XmStringFree(xid_load.savetiff_dirmask);
   xid_load.savetiff_dirmask = XmStringCopy(acs->mask);

   /* If the user specified a file that already exists pop up a window to have
      the user verify that they want to overwrite the existing file with the
      new one.  */
   if(c_lsstat(outfile, &zero) == E_SUCC)
   {

      /* Open the verify form */
      strcpy(xid_load.save_image_name, outfile);
      Verify_form = Createverify_form(Shell_Verify);
      XtManageChild(Verify_form);
      XtPopup(Shell_Verify,XtGrabNone);
   }
   else
      save_tiff(outfile);
}

void save_verify
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine deletes the file with the user specified filename, upon
   verification, for use in the save_tiff routine.

By A.Shimitz, USGS/EROS Data Center, 6/00

Algorithm Description:
******************************************************************************/
{
   char    outfile[CMLEN];       /* Output TIFF file                        */

   /* Delete the old file with outfile filename and call save_tiff  */
   strcpy(outfile, xid_load.save_image_name);
   c_delfil(outfile);

   /* Close the Verify form and call the save_tiff routine to finish saving the
      image to file */
   XtUnmanageChild(Verify_form);
   XtDestroyWidget(Verify_form);
   XtPopdown(Shell_Verify);
   save_tiff(outfile);
}

void save_tiff
(
   char outfile[CMLEN]   /* Output TIFF file (O)                           */
)

/******************************************************************************
Purpose:
   This routine changes the cursor shape to a watch, pops up the
   processing message dialog, and saves the subsampled TIFF image to file.

By A.Shimitz, USGS/EROS Data Center, 6/00

Modified by A. Shimitz 11/00
 - Replaced the hourglass cursor with the watch cursor, managed by the
   manage_cursor routines.  (CR #3967)

Algorithm Description:
******************************************************************************/
{
   GC      copy_gc;               /* GC used to copy pixmaps                 */
   Pixmap  img_pix;               /* Image pixmap                            */
   Pixmap  *pixptr = NULL;        /* Pixmap pointer                          */
   TIFF    *tif = NULL;           /* TIFF image structure                    */
   Window  image_win;             /* Image window                            */
   XGCValues
           values;                /* Values array for the GC                 */
   XImage  *image = NULL;         /* X image structure                       */
   char    *tiffbuf = NULL;       /* Subsampled image line buf for TIFF      */
   double  lineinc;               /* Subsample line increment                */
   double  sampinc;               /* Subsample sample increment              */
   long    blue;                  /* Blue pixel value                        */
   long    green;                 /* Green pixel value                       */
   long    i;                     /* Looping index                           */
   long    img_idx;               /* Index into color table of the first
                                     image color                             */
   long    line;                  /* Current line number                     */
   long    nbands;                /* Number of bands in output image         */
   long    nl;                    /* Number of lines                         */
   long    ns;                    /* Number of samples                       */
   long    num_imgcolors;         /* Number of image colors                  */
   long    pixel;                 /* Single pixel value                      */
   long    plane_idx;             /* Image plane index of current image      */
   long    red;                   /* Red pixel value                         */
   long    samp;                  /* Current sample number                   */
   long    sl;                    /* Starting line                           */
   long    ss;                    /* Starting sample                         */
   long    subline;               /* Subsampled line number                  */
   long    subsamp;               /* Subsampled sample number                */
   long    tiffheight = 256;      /* Height of subsampled TIFF output image  */
   long    tiffwidth = 256;       /* Width of subsampled TIFF output image   */
   long    valuemask = 0;         /* Specify no values                       */

   /* Display the wait cursor (watch) in both the image window and main
      panel.  */
   image_win = XtWindow(main_image_bulletinboard);
   show_wait_cursor();

   /* Get the memory plane index of the currently displayed image */
   plane_idx = xid_view.rgbplanes[RED] - 1;
   sl = ss = 0;

   /* Determine the number of lines and samples in the output image.  Also, set
      a pointer to the image data */
   nl = xid_planes[plane_idx].dspwin[NL];
   ns = xid_planes[plane_idx].dspwin[NS];
   pixptr = &xid_mem.image_pixmap;
   if(pixptr == NULL)
   {
      xid_message("Error defining the Pixmap.");
      return;
   }

   /* Initialize and display the processing widget */
   XtVaSetValues(load_dialog_shell, XmNtitle, "Save - Processing", NULL);
   XeProcessMBegin(process_widget, 0, nl+1,
                   "Saving image to disk, please wait ...", True,True,True);
   XeProcessMProcess(process_widget, 1);

   /* Close the save (load) panel */
   load_destroy(NULL, NULL, NULL);

   /* Assign number of image colors and the starting index of the image colors
      based on the type of visual */
   if(*xid.colormap_ptr != xid.colormap)
   {
      num_imgcolors = 256;
      img_idx = 0;
   }  
   else
   {
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
   }

   /* Calculate trivial decimation factors based on size of band data versus
      size of desired TIFF */
   lineinc = (double)(nl - 1) / tiffheight;
   sampinc = (double)(ns - 1) / tiffwidth;

   /* Create the graphics context for saving the image data */
   copy_gc = XCreateGC(xid.display, image_win, valuemask, &values);

   /* Determine the number of bands to save */
   if((xid_view.rgbplanes[RED] != xid_view.rgbplanes[GREEN]) ||
      (xid_view.rgbplanes[GREEN] != xid_view.rgbplanes[BLUE]))
      nbands = 3;
   else
      nbands = 1;

   if(nbands == 1)
   {
      /* Save 3 bands if any graphics masks have been applied */
      for(i = 0; i < MASK_MAX; i++)
      {
         if(xid_mask.mask_pixmap[i] != 0)
         {
            nbands = 3;
            break;
         }
      }

      if(nbands == 1)
      {
         /* Or if any graphics data is displayed */
         if(xid_graphics.pixmap[0] != 0)
            nbands = 3;

         /* Or if a full resolution outline is displayed (b/c of the green
            outline box of the area being viewed in full res) */
         else if((xid_fullres != NULL) && (xid_fullres[0].flags & FULLRES_SHOW))
            nbands = 3;

         /* Or if a pseudo-color image is displayed */
         else
         {
            for(i = 0; i < num_imgcolors; i++)
            {
               if((xid.colortable_ptr[img_idx + i].red !=
                  xid.colortable_ptr[img_idx + i].green) ||
                  (xid.colortable_ptr[img_idx + i].green !=
                  xid.colortable_ptr[img_idx + i].blue))
               {
                  nbands = 3;
                  break;
               }
            }
         }
      }
   }

   /* Allocate the tiff data buffer */
   tiffbuf = malloc((size_t)tiffwidth * nbands);
   if(tiffbuf == NULL)
   {
      free(tiffbuf);
      xid_message("Error writing TIFF image.");
      xid_save.flags |= SAVE_ERROR;
      XFreeGC(xid.display, copy_gc);
      XeProcessMEnd(process_widget);

      /* Redisplay the default arrow cursor.  */
      unshow_wait_cursor();
      return;
   }

   /* Create a pixmap to store the image data in */
   img_pix = XCreatePixmap(xid.display, image_win, ns, nl,
                           xid.visual_info[0].depth);

   /* Capture the current image data into the pixmap */
   XCopyArea(xid.display,*pixptr,img_pix,copy_gc,ss,sl,ns,nl,0,0);

   /* Read the image data into an X image structure */
   image = XGetImage(xid.display,img_pix,ss,sl,ns,nl,AllPlanes,ZPixmap);
   if(image == NULL)
   {
      free(tiffbuf);
      xid_message("Error reading image data.");
      xid_save.flags |= SAVE_ERROR;
      XFreePixmap(xid.display, img_pix);
      XFreeGC(xid.display, copy_gc);
      XeProcessMEnd(process_widget);

      /* Redisplay the default arrow cursor.  */
      unshow_wait_cursor();
      return;
   }

   /* Open the TIFF image */
   tif = TIFFOpen(outfile, "w");
   if(tif == NULL)
   {
      xid_message("Error opening output image.");
      xid_save.flags |= SAVE_ERROR;
      XFreePixmap(xid.display, img_pix);
      XFreeGC(xid.display, copy_gc);
      XeProcessMEnd(process_widget);
      XDestroyImage(image);

      /* Redisplay the default arrow cursor.  */
      unshow_wait_cursor();
      return;
   }

   /* Set the TIFF image fields */
   TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
   TIFFSetField(tif, TIFFTAG_RESOLUTIONUNIT, RESUNIT_NONE);
   TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, (uint32) tiffwidth);
   TIFFSetField(tif, TIFFTAG_IMAGELENGTH, (uint32) tiffheight);
   TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
   TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, 1);
   TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8);
   if(nbands == 1)
      TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
   else
      TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
   TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, nbands);

   /* For tiffheight lines */
   for(line = sl; line < tiffheight; line++)
   {

      /* Define the subsampled line to read based on the current line and
         the line decimation factor */
      subline = (int)(line * lineinc);

      /* Update the processing message widget.  This is necessary for the
         pause and cancel buttons to work even though the widget itself does
         not change based on the current line number */
      if(XeProcessMProcess(process_widget, line) == XePROCESSM_CANCEL)
      {
         TIFFClose(tif);
         if(c_dkdel(outfile) != E_SUCC)
            xid_message("Error removing incomplete TIFF file.");
         free(tiffbuf);
         XeProcessMEnd(process_widget);
         XFreePixmap(xid.display, img_pix);
         XFreeGC(xid.display, copy_gc);
         XDestroyImage(image);

         /* Redisplay the default arrow cursor.  */
         unshow_wait_cursor();
         return;
      }

      /* For tiffwidth samples */
      for(samp = ss; samp < tiffwidth; samp++)
      {

         /* Define the subsampled sample to read based on the current sample
            and the sample decimation factor */
         subsamp = (int)(samp * sampinc);

         /* Get the appropriate pixel value from the X image and scale it to a
            value between 0 and 255 */
         pixel = (long)XGetPixel(image, subsamp, subline);
         if(xid.visual_info[0].depth == 24)
         {
            red = (pixel >> xid.red_shift) & 0xff;
            green = (pixel >> xid.green_shift) & 0xff;
            blue = (pixel >> xid.blue_shift) & 0xff;
         }
         else
            red = green = blue = pixel;

         /* Use the pixel values as indices into the current color table.
            Again, scale the resulting value to a value between 0 and 255 */
         tiffbuf[samp * nbands] = (char)(xid.colortable_ptr[red].red >> 8);
         if(nbands == 3)
         {
            tiffbuf[1 + (samp * nbands)] = (char)(xid.colortable_ptr[green].
                                                green >> 8);
            tiffbuf[2 + (samp * nbands)] = (char)(xid.colortable_ptr[blue].
                                                blue >> 8);
         }
      } /* end sample for loop*/

      /* Write the line to the TIFF image file */
      if(TIFFWriteScanline(tif, tiffbuf, line, 0) < 0)
      {
         TIFFClose(tif);
         if(c_dkdel(outfile) != E_SUCC)
            xid_message("Error removing incomplete TIFF file.");
         free(tiffbuf);
         xid_message("Error writing TIFF file.");
         XeProcessMEnd(process_widget);
         xid_save.flags |= SAVE_ERROR;
         XFreePixmap(xid.display, img_pix);
         XFreeGC(xid.display, copy_gc);
         XDestroyImage(image);

         /* Redisplay the default arrow cursor.  */
         unshow_wait_cursor();
         return;
      }
   } /* end line for loop*/

   /* Close files and free buffers */
   TIFFClose(tif);
   free(tiffbuf);
   XFreePixmap(xid.display, img_pix);
   XFreeGC(xid.display, copy_gc);
   XDestroyImage(image);
   tif = NULL;
   tiffbuf = NULL;
   image = NULL;
   pixptr = NULL;

   /* Close the processing message window */
   XeProcessMEnd(process_widget);

   /* Redisplay the default arrow cursor.  */
   unshow_wait_cursor();

   /* Let the user know we are finished */
   XBell(xid.display, 0);
}
