/*
 *  This is the show function source file for the show panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include "las.h" /* to get CMLEN definition for util.h */
#include "util.h" /* c_up2low definition */
#include "xid.h"
#include "xid_func.h"

void show_accept
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine is used to display the appropriate image data in the specified
   RGB order.

By L. Huewe, USGS/EROS Data Center, 12/91

Modified by A. Shimitz 7/00
 - Modified show_accept so that the last thing it does is calls
   histo_update if the histogram panel is open.

Modified by A.Shimitz 11/00
  - Changed functionality so the planes selected are not deselected when the
    user presses the "Accept" button. It now maintains the R,G,B plane numbers
    that were previously selected too.  (CR #4157)

Modified by A. Shimitz 11/00
  - Added conditional to determine when to set the viewflg being passed to
    load_display - base this on if the -ws command-line parameter was
    specified. (CR #3967)

Algorithm Description:
******************************************************************************/
{
   Arg     args[1];                /* Variable used to set widget resources. */
   long    composite_index;        /* Composite index.                       */
   long    i;                      /* Looping index.                         */
   long    nplanes;                /* Number of planes to be displayed.      */
   long    show_save = FALSE;      /* Indicates whether or not a saved
                                      composite image is being displayed.    */
   long    zero;                   /* Number of planes which are zero.       */
   int     reinit_flg;             /* Indicates if the load process should
                                      reinitialize the size of the window
                                      before loading the image               */

   /* Check whether or not a saved composite view is being displayed.  */
   if (xid_show.planes[RED] == -1)
   {
      if (strcmp(xid_show.composite,"Current") == 0)
         return;
      show_save = TRUE;

      /* Find the specified view.  */
      xid_find_composite(xid_show.composite,&composite_index);
      if (composite_index < 0)
      {
         xid_message("Unable to find specified composite image.");
         return;
      }

      /* Retrieve the plane numbers of the specified view.  */
      for (i=0; i < 3; i++)
         xid_show.planes[i] = xid_composite[composite_index].rgbplanes[i];
   }

   /* If a value was not specified for one of the display components, copy it
      from a specified component.  */
   for (i=0,zero=0; i < 3; i++)
   {
      if (xid_show.planes[i] == 0)
      {
         zero++;
         switch(i)
         {
            case 0:
               xid_show.planes[i] = (xid_show.planes[i+1] > 0) ? 
                                   xid_show.planes[i+1] : xid_show.planes[i+2]; 
               break;
            case 1:
               xid_show.planes[i] = (xid_show.planes[i-1] > 0) ? 
                                   xid_show.planes[i-1] : xid_show.planes[i+1]; 
               break;
            case 2:
               xid_show.planes[i] = (xid_show.planes[i-1] > 0) ? 
                                   xid_show.planes[i-1] : xid_show.planes[i-2]; 
               break;
         }
      }
   }

   /* If no values were specified, just return.  */
   if (zero == 3)
      return;

   /* Determine the number of planes to display.  */
   if ((xid_show.planes[RED] == xid_show.planes[GREEN]) && 
       (xid_show.planes[GREEN] == xid_show.planes[BLUE]))
      nplanes = 1;
   else
   {
      nplanes = 3;

      /* Report an error if the specified planes are not all the same size.  */
      if ((xid_planes[xid_show.planes[RED] - 1].dspwin[NL] != 
          xid_planes[xid_show.planes[GREEN] - 1].dspwin[NL]) ||
          (xid_planes[xid_show.planes[GREEN] - 1].dspwin[NL] !=
          xid_planes[xid_show.planes[BLUE] - 1].dspwin[NL]))
      {
         xid_show.planes[RED] = xid_show.planes[GREEN] = 
                                                     xid_show.planes[BLUE] = 0;
         xid_message("Image planes are different sizes");
         return;
      }
      if ((xid_planes[xid_show.planes[RED] - 1].dspwin[NS] != 
          xid_planes[xid_show.planes[GREEN] - 1].dspwin[NS]) ||
          (xid_planes[xid_show.planes[GREEN] - 1].dspwin[NS] !=
          xid_planes[xid_show.planes[BLUE] - 1].dspwin[NS]))
      {
         xid_show.planes[RED] = xid_show.planes[GREEN] = 
                                                     xid_show.planes[BLUE] = 0;
         xid_message("Image planes are different sizes");
         return;
      }
   }

   /* Free any previously allocated pixmaps.  */
   if (xid_mem.imgpix)
   {
      XFreePixmap(xid.display,xid_mem.image_pixmap);
      xid_mem.imgpix = FALSE;
      xid_mem.image_pixmap = 0;
   }

   /* Setup reinit flag to indicate if the window should be resized before
   loading the image */
   reinit_flg = !(xid.flags & XID_WINNOREINIT);
   /* Call load_display to display the appropriate image data in the
      appropriate RGB order.  */
   if (nplanes == 1)
   {
      load_display(xid_show.planes,&nplanes,XtWindow(main_image_bulletinboard),
                   Shell_MainWin,main_image_bulletinboard,
                   &xid_planes[xid_show.planes[RED] - 1].dspwin[NL],
                   &xid_planes[xid_show.planes[RED] - 1].dspwin[NS],
                   &xid_planes[xid_show.planes[RED] - 1].maxval,
                   xid_planes[xid_show.planes[RED] - 1].image_data,NULL,NULL,
                   TRUE,&xid_mem.image_pixmap,TRUE,reinit_flg,FALSE,FALSE);
   }
   else
   {
      load_display(xid_show.planes,&nplanes,XtWindow(main_image_bulletinboard),
                   Shell_MainWin,main_image_bulletinboard,
                   &xid_planes[xid_show.planes[RED] - 1].dspwin[NL],
                   &xid_planes[xid_show.planes[RED] - 1].dspwin[NS],
                   &xid_planes[xid_show.planes[RED] - 1].maxval,
                   xid_planes[xid_show.planes[RED] - 1].image_data,
                   xid_planes[xid_show.planes[GREEN] - 1].image_data,
                   xid_planes[xid_show.planes[BLUE] - 1].image_data,TRUE,
                   &xid_mem.image_pixmap,TRUE,reinit_flg,FALSE,FALSE);
   }

   /* Apply the appropriate color table.  */
   if (show_save)
   {
      XStoreColors(xid.display,*xid.colormap_ptr,
                   xid_composite[composite_index].color_table,256);
      for (i=0; i < 256; i++)
         xid.colortable_ptr[i] = xid_composite[composite_index].color_table[i];
   }

   /* Change the title to reflect the name of the image displayed in the main
      image window.  */
   XtSetArg(args[0],XmNtitle,xid_planes[xid_show.planes[RED] - 1].filename);
   XtSetValues(Shell_MainWin,args,1);

   /* Update the histogram panel if it is currently active */
   if(XtIsRealized(Shell_Histo))
   {
      if(XtIsManaged(Histo_form))
      {
         xid.mapped = FALSE;
         histo_update(w, client, call);
      }
   }
}

void show_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine closes the show panel. 

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   /* Close the show panel.  */
   XtUnmanageChild(Show_form);
   XtPopdown(Shell_Show);
   XtUnrealizeWidget(Shell_Show);

   /* Set the show button on the main panel to sensitive.  */
   XtSetSensitive(main_show_button,True);

   /* Free resources */
   if(xid_show.composite != NULL)
   {
      free(xid_show.composite);
      xid_show.composite = NULL;
   }
}

void show_select
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine is called each time the user selects an item from the
   red, green or blue list or from the save list.  The value selected
   is saved until the combination is accepted (using the "Accept" button).

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
******************************************************************************/
{
   long    plane_num;                  /* Plane number selected.             */
   long    tmp;                        /* holds a temporary value            */
   long    num_composite;              /* number of composite images         */
   long    i, j;                       /* looping counters                   */
   long    same_flg;                   /* indicates if the current composite
                                          has the same planes as the current
                                          selections on the Show panel       */
   long    found = FALSE;              /* indicates if one of the composite
                                          views matched the current Show
                                          panel plane selections             */
   long    plane_vals[3];              /* needed to set the plane values back
                                          after highlighting the saved list
                                          value                              */
   char    text[6],                    /* Dummy text variable.               */
           *tmpstr;                    /* Temporary string containing the
                                          value of the selected list element */
   char    cur_name[CMLEN];            /* current composite view name        */
   XmString xmstring = (XmString)NULL; /* Motif string                       */
   char *data = (char *)client;

   XmListCallbackStruct *acs=(XmListCallbackStruct*)call;

   /* Get the item the user selected.  */
   XmStringGetLtoR(acs->item,XmSTRING_DEFAULT_CHARSET,&tmpstr);

   /* Deselect all items in the save list and extract the plane number from the
      text string.  */
   if (*data != 'S')
   {
      sscanf(tmpstr,"%s%ld",text,&plane_num);
      XmListDeselectAllItems(show_savelist);

      XtSetSensitive(show_delete_button,False);
   }

   /* Enable the delete button.  */
   else
      XtSetSensitive(show_delete_button,True);

   /* Assign the plane number to the appropriate element in the RGB array.  */
   if (*data == 'R')
      xid_show.planes[RED] = plane_num; 
   else if (*data == 'G')
      xid_show.planes[GREEN] = plane_num; 
   else if (*data == 'B')
      xid_show.planes[BLUE] = plane_num; 

   /* If a saved composite name was selected, assign the plane numbers to a
      special value and retrieve the composite name.  */
   else if (*data == 'S')
   {
      xid_show.planes[RED] = xid_show.planes[GREEN] =
                                                    xid_show.planes[BLUE] = -1;
      /* Free the previous composite string */
      if(xid_show.composite != NULL)
         free(xid_show.composite);
      xid_show.composite = (char *)malloc(strlen(tmpstr) + 1);
      if (xid_show.composite == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
      strcpy(xid_show.composite,tmpstr);
   }

   /* Highlight only the item listed in the saved list that matches the
      current selection made by the user */
   if(*data != 'S')
   {
      /* Get the number of composites */
      xid_get_composite(&tmp, &num_composite);

      /* Compare each composite's planes to the current plane selections */
      for(i = 0; i < num_composite; i++)
      {
         same_flg = TRUE;

         /* Get the current composite's name and make it into a Motif string */
         strcpy(cur_name, xid_composite[i].name);
         xmstring = XmStringCreateLtoR(cur_name, XmSTRING_DEFAULT_CHARSET);

         /* Compare each of this composite's plane values to the current Show
            panel plane selections.  Also store these plane values for
            reassignment after the show_select callback is triggered by
            selecting the particular composite in the saved list. */
         for(j = 0; j < 3; j++)
         {
            plane_vals[j] = xid_show.planes[j];
            if(xid_composite[i].rgbplanes[j] != xid_show.planes[j])
               same_flg = FALSE;
         }

         /* If the plane values were the same, highlight the current composite
            and then retrieve the show plane values since they were modified
            by the callback triggered when highlighting the composite name */
         if(same_flg)
         {
            XmListSelectItem(show_savelist, xmstring, True);
            for(j = 0; j < 3; j++)
               xid_show.planes[j] = plane_vals[j];
            found = TRUE;
            break;
         }
         XmStringFree(xmstring);
      }

      /* If none of the composites matched what is currently selected then
         compare the selections to the current view ("Current")  */
      if(!found)
      {
         same_flg = TRUE;
         strcpy(cur_name, "Current");
         xmstring = XmStringCreateLtoR(cur_name, XmSTRING_DEFAULT_CHARSET);

         /* Compare each plane value for the current view to the Show panel
            current selections to see if they match.  Save the show plane
            values because of their modification by the callback triggered
            when highlighting the selection in the saved list. */
         for(i = 0; i < 3; i++)
         {
            plane_vals[i] = xid_show.planes[i];
            if(xid_view.rgbplanes[i] != xid_show.planes[i])
               same_flg = FALSE;
         }
         if(same_flg)
         {
            XmListSelectItem(show_savelist, xmstring, True);

            /* Retrieve the Show plane values */
            for(i = 0; i < 3; i++)
               xid_show.planes[i] = plane_vals[i];
         }
         XmStringFree(xmstring);
      }
   }

   /* Free appropriate resources.  */
   XtFree(tmpstr);
}

void show_save_dialog
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine pops up the dialog box prompting the user for a composite
   name.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
******************************************************************************/
{

   /* Pop up the dialog shell.  */
   XtManageChild(show_dialog_form);
}

void show_save
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine saves the currently displayed image view with the user
   specified name.  Information saved includes the image plane combination 
   as well as the colormap applied.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
******************************************************************************/
{
   char         *name;                  /* Save image name.                  */
   long         dup_index;              /* Index of duplicate entry.         */
   long         i;                      /* Looping index.                    */
   long         index;                  /* Free composite index.             */
   long         len;                    /* String length.                    */
   long         num_composite;          /* Number of composite entries used. */
   XmString     xmstring;               /* Motif string.                     */

   /* Make sure an image is displayed.  */
   if (xid.view == -1)
   {
      xid_message("No image data currently being viewed");
      return;
   }

   /* Get the next free composite view index.  */
   xid_get_composite(&index,&num_composite);

   /* Get the saved image name.  */
   name = XmTextFieldGetString(show_save_text);

   /* Make sure a name was specified.  */
   if (strcmp(name,"") == 0)
      return;

   /* Check for duplicate name.  */
   xid_find_composite(name,&dup_index);
   if (dup_index >= 0)
   {
      xid_message("Composite name already exists");
      return;
   }

   /* Copy the name into the composite structure.  */
   xid_composite[index].name = (char *)malloc(strlen(name) + 1);
   if (xid_composite[index].name == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }
   strcpy(xid_composite[index].name,name);

   /* Convert the name to lower case.  */
   len = (long)strlen(xid_composite[index].name);
   c_up2low(xid_composite[index].name,&len);

   /* Save the memory plane combination.  */
   for (i=0; i < 3; i++)
      xid_composite[index].rgbplanes[i] = xid_view.rgbplanes[i];

   /* Save the color table.  */
   for (i=0; i < 256; i++)
      xid_composite[index].color_table[i] = xid.colortable_ptr[i];

   /* Mark the entry as used.  */
   xid_composite[index].flags |= USED;

   /* Add this name to the show save list as well as the flicker save list if
      necessary.  */
   xmstring = XmStringCreateLtoR(xid_composite[index].name,
                                 XmSTRING_DEFAULT_CHARSET);
   XmListAddItem(show_savelist,xmstring,2);

   if (xid_flicker.flags & FLICKER_ACTIVE)
      XmListAddItem(flicker_savelist,xmstring,1);

   /* Clear the text field and pop down the dialog shell.  */
   XmTextFieldSetString(show_save_text,"");
   XtUnmanageChild(show_dialog_form);

   /* Free appropriate resources.  */
   XtFree(name);
   XmStringFree(xmstring);
}

void show_info
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine pops up a panel displaying to the user information about
   the selected image plane.  Information reported includes the image file
   name, the window loaded, the band number and the subsampling factor.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
******************************************************************************/
{
   XmString   xmstring;        /* Motif string.                              */
   long       plane_num;       /* Plane number to retrieve information for.  */
   char       *tmpstr,         /* Temporary string variable.                 */
              string[100],     /* Image window values.                       */
              text[10];        /* Temporary text string.                     */

   XmListCallbackStruct *acs=(XmListCallbackStruct*)call;

   /* Get the plane number for which the information is to be displayed.  */
   XmStringGetLtoR(acs->item,XmSTRING_DEFAULT_CHARSET,&tmpstr);
   sscanf(tmpstr,"%s%ld",text,&plane_num);
   free(tmpstr);

   /* Display the title (including the selected plane number).  */
   sprintf(string,"Image Information for Plane %ld\n",plane_num);
   xmstring = XmStringCreateLtoR(string,XmSTRING_DEFAULT_CHARSET);
   XtVaSetValues(info_title,XmNlabelString,xmstring,NULL);
   XmStringFree(xmstring);

   /* Display the image file name.  */
   XmTextFieldSetString(info_filename_text,xid_planes[plane_num-1].filename);

   /* Display the window that was loaded.  */
   sprintf(string,"(%ld, %ld, %ld, %ld)",xid_planes[plane_num-1].window[SL],
           xid_planes[plane_num-1].window[SS],
           xid_planes[plane_num-1].window[NL],
           xid_planes[plane_num-1].window[NS]);
   XmTextSetString(info_window_text,string);

   /* Display the band number.  */
   sprintf(string,"%ld",xid_planes[plane_num-1].band);
   XmTextFieldSetString(info_band_text,string);

   /* Display the subsampling factor.  */
   sprintf(string,"%ld",xid_planes[plane_num-1].skipl);
   XmTextFieldSetString(info_subsamp_text,string);

   /* Pop up the dialog shell.  */
   XtManageChild(info_form);
}

void info_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine pops down the image plane information panel.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
******************************************************************************/
{

   /* Pop down the dialog shell.  */
   XtUnmanageChild(info_form);
}

void show_planes
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine highlights or selects the plane numbers representing the
   selected composite view.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
******************************************************************************/
{
   char         *composite;             /* Composite view name selected.     */
   char         planestr[20];           /* Plane number string.              */
   long         composite_index;        /* Index into composite structure.   */
   long         i;                      /* Looping index.                    */
   XmString     xmstring;               /* Motif string.                     */

   /* Highlight the planes representing the selected composite view.  */
   XmListCallbackStruct *acs=(XmListCallbackStruct*)call;

   /* Get the name of the selected composite view.  */
   XmStringGetLtoR(acs->item,XmSTRING_DEFAULT_CHARSET,&composite);

   /* "Current" is a special case.  It is not a saved composite view, it is the
      currently displayed image.  */
   if (strcmp(composite,"Current") == 0)
   {
      for (i=0; i < 3; i++)
      {
         sprintf(planestr,"Plane %ld",xid_view.rgbplanes[i]);
         xmstring = XmStringCreateLtoR(planestr,XmSTRING_DEFAULT_CHARSET);
         if (i == 0)
            XmListSelectItem(show_redlist,xmstring,True);
         else if (i == 1)
            XmListSelectItem(show_greenlist,xmstring,True);
         else
            XmListSelectItem(show_bluelist,xmstring,True);
         XmStringFree(xmstring);
      }
   }

   /* Find the composite view index and plane numbers.  */
   else
   {
      xid_find_composite(composite,&composite_index);
      if (composite_index < 0)
      {
         xid_message("Unable to find specified composite image.");
         return;
      }
        
      for (i=0; i < 3; i++)
      {
         sprintf(planestr,"Plane %ld",
                 xid_composite[composite_index].rgbplanes[i]);
         xmstring = XmStringCreateLtoR(planestr,XmSTRING_DEFAULT_CHARSET);
         if (i == 0)
            XmListSelectItem(show_redlist,xmstring,True);
         else if (i == 1)
            XmListSelectItem(show_greenlist,xmstring,True);
         else
            XmListSelectItem(show_bluelist,xmstring,True);
         XmStringFree(xmstring);
      }
   }

   /* Free appropriate resources.  */
   XtFree(composite);
}

void show_save_cancel
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine pops down the save panel without saving a new composite
   view.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
******************************************************************************/
{
   /* Clear the text field and pop down the dialog shell.  */
   XmTextFieldSetString(show_save_text,"");
   XtUnmanageChild(show_dialog_form);

   return;
}

void show_delete
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine deletes the specified composite view.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
******************************************************************************/
{
   XmString     xmstring;             /* Motif string.                       */
   long         composite_index;      /* Composite index.                    */

   /* Find the composite view to be removed.  */
   xid_show.planes[RED] = xid_show.planes[GREEN] = xid_show.planes[BLUE] = 0;
   if (strcmp(xid_show.composite,"Current") == 0)
      return;
   xid_find_composite(xid_show.composite,&composite_index);
   if (composite_index < 0)
   {
      xid_message("Unable to find specified composite image.");
      return;
   }

   /* Remove the entry from the any currently displayed save list.  */
   xmstring = XmStringCreateLtoR(xid_composite[composite_index].name,
                                 XmSTRING_DEFAULT_CHARSET);
   XmListDeleteItem(show_savelist,xmstring);

   if (xid_flicker.flags & FLICKER_ACTIVE)
      XmListDeleteItem(flicker_savelist,xmstring);
   XmStringFree(xmstring);

   /* Reset the appropriate fields indicating this composite view has been
      removed.  */
   xid_composite[composite_index].rgbplanes[RED] =
      xid_composite[composite_index].rgbplanes[GREEN] =
      xid_composite[composite_index].rgbplanes[BLUE] = 0;
   xid_composite[composite_index].flags &= (~USED);
   free(xid_composite[composite_index].name);
}
