/*
 *  This is the utility source file for the X Image Display application.
 *  It contains procedure routines that are generic in nature and not
 *  necessarily tied to a specific function source file.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *
 *      Routine                         Description
 *      -------                         -----------
 *      COLOR                           Returns pixel value for color string.
 *      CONVERT                         Converts the specified string to the
 *                                       specified type such as a fontlist or
 *                                       pixel value.
 *      FONT_LIST                       Returns a font list pointer for
 *                                       a given font specification.
 *      XCreatePixmapFromData           XPM utility for using XPM in C.
 *      XPM_PIXMAP                      XPM utility for creating pixmaps
 *                                      from XPM specifications.
 *      STRING_TABLE                    Motif string table creation utility.
 *      FREE_STRING_TABLE               Motif string table deletion utility.
 *      load_clist                      Loads the X color database list.
 *      XID_StringToPixel               Resource converter to convert
 *                                       widget color string specifications to
 *                                       pixel values.
 *      CompareISOLatin1                ISOLatin character checker used with
 *                                       the XID_StringToPixel resource
 *                                       converter.
 *      getcolor                        Manages color specifications for the
 *                                       color store used by xid.
 *      parse_rb                        Parse range and bin values.
 *      parse_fromto                    Parse from and to mapping values.
 *      substr                          Determines whether string2 is a sub-
 *                                       string of string1 and returns its
 *                                       position within string1.
 *      piece                           Calculates a piecewise linear mapping
 *                                       given break point pairs.
 *      renumber                        Returns a mapping which changes the
 *                                       the value of the from pixels to the
 *                                       value of the to pixels.
 *      xid_noop                        A function which does nothing.
 */

#include <ctype.h> /* isspace definition */
#include "xid.h"
#include "xid_func.h"

/* local function prototypes */
Status getcolor
(
   char *str,                /* Input color specification. (I)               */
   XColor *color_cell        /* Color cell to be assigned. (O)               */
);
int substr
(
   char string1[],                      /* string for comparison    (I)      */
   char string2[],                      /* possible substring       (I)      */
   int *pos                             /* position of string2 in   (O)      */
                                        /* string1 (0 if not a substr)       */
);

static int CompareISOLatin1
(
   char *first,
   char *second
);
/* end prototypes */

/* Includes for the XID_StringToPixel resource converter.  */
#include <X11/cursorfont.h>
#include <X11/keysym.h>

/* Resource name variables for the XID_StringToPixel resource converter.  */
static String XtNwrongParameters = "wrongParameters";

/* Convenience routines; these originate from the ICS BX application.  */
#ifndef IGNORE_CONVERT
/*SUPRESS 592*/
caddr_t CONVERT
(
   Widget w,
   char *from_string,
   char *to_type,
   int to_size,
   Boolean *success
)
{
   XrmValue            fromVal, toVal;       /* resource holders             */
   Boolean             convResult;           /* return value                 */
   unsigned char       oByte;                /* one byte result              */
   unsigned short      tByte;                /* two byte result              */
   caddr_t             fByte;                /* four byte result             */
   caddr_t             aByte;                /* allocated result             */
    
   /* Zero it.  */
   fByte = aByte = NULL;
   *success = False;

   /* String to string.  */
   if( strcmp(XmRString, to_type) == 0 )
   {
      *success = True;
      /*SUPPRESS 80*/
      return( from_string );
   }

   /* Sometimes we do not know this at code output.  */
   if(to_size == 0) to_size = strlen(from_string);
        
   /* Motif String to StringTable converter does not cache correctly and so
      we need to special case this.  */ 
   if( strcmp(to_type, "XmStringTable") == 0)
   {
      int lcount, loop;
      char tmp, *ptr, *e_ptr;
      XmStringTable table;
        
      ptr = (char *)from_string;
      if ((ptr == NULL) || (*ptr == '\0'))
      {
         return( NULL );
      }
      else 
      {       
         lcount = 1;
         while (*ptr != '\0') 
         {
            if (*ptr++ == '\n')
               lcount++;
         }
         table = (XmStringTable) XtMalloc(sizeof(XmString) * lcount);
         ptr = (char *)from_string;
            
         for (loop = 0; loop < lcount; loop++) 
         {
            e_ptr = ptr;
            while((*e_ptr!='\0')&&(*e_ptr != '\n'))
               e_ptr++;         
            tmp = *e_ptr;
            *e_ptr = '\0';
            table[loop] = XmStringCreateSimple(ptr);
            *e_ptr = tmp;
            ptr = ++e_ptr;
         }
            
         fByte = (caddr_t)table;
      }
      *success = True;
      /*SUPPRESS 80*/
      return( fByte );
   }

   /* Set up the list.  */
   fromVal.size = strlen(from_string) + 1;
   fromVal.addr = from_string;

   switch( to_size )
   {
      case 1:
         toVal.size = sizeof(unsigned char);
         toVal.addr = (caddr_t)&oByte;
         break;
      case 2:
         toVal.size = sizeof(unsigned short);
         toVal.addr = (caddr_t)&tByte;
         break;
      default:
         toVal.size = sizeof(caddr_t);
         toVal.addr = (caddr_t)&fByte;
         break;
   }
    
   convResult = XtConvertAndStore(w, XmRString, &fromVal, to_type, &toVal);
    
   if( convResult )
   {
      switch( to_size )
      {
         case 1:
            fByte = (caddr_t)&oByte;
            break;
         case 2:
            fByte = (caddr_t)&tByte;
            break;
         default:
            break;
      }
   }
    
   /* Conversion will fail if we need more than 4 bytes. For strings it will
      fail always the first time.  */
   if( !convResult && toVal.size != to_size )
   {
      /* Need to allocate more space for this one.  */
      toVal.addr = (caddr_t)XtMalloc(toVal.size);
      fByte = aByte = toVal.addr;
      convResult = XtConvertAndStore(w, XmRString, &fromVal, to_type, &toVal);
   }
    
   /* Free anything useless we may have allocated.  */
   if( !convResult )
   {
      XtFree(aByte);
      aByte = NULL;
   }
    
   /* Return the result.  */
   *success = convResult;
   /*SUPPRESS 80*/
   return( fByte );
}

#endif
#ifndef IGNORE_COLOR
unsigned long COLOR
(
   Widget w,
   char *name
)
{
   XrmValue        fromVal, toVal;
   unsigned long   *pixel;

   fromVal.size = sizeof(char*);
   fromVal.addr = name;

   XtConvert(w, XmRString, &fromVal, XmRPixel, &toVal);
   pixel = (unsigned long*)toVal.addr;
   if( pixel == NULL )
   {
      fromVal.addr = XtDefaultBackground;
      XtConvert(w, XmRString, &fromVal, XmRPixel, &toVal);
      pixel = (unsigned long*)toVal.addr;
   }
   return(*pixel);
}
#endif
#ifndef IGNORE_FONT
XmFontList FONT_LIST
(
   Widget w,
   char *name
)
{
   XrmValue        fromVal, toVal;
   XmFontList      *fontListPtr;

   fromVal.size = sizeof(char*);
   fromVal.addr = name;

   XtConvert(w, XmRString, &fromVal, XmRFontList, &toVal);
   fontListPtr = (XmFontList*)toVal.addr;
   if(fontListPtr == NULL)
   {
      fromVal.addr = "fixed";
      XtConvert(w, XmRString, &fromVal, XmRFontList, &toVal);
      fontListPtr = (XmFontList*)toVal.addr;
   }
   return(*fontListPtr);
}

#endif
#ifndef IGNORE_XPM_PIXMAP
/* Copyright 1989 GROUPE BULL -- See licence conditions in file COPYRIGHT */
/*****************************************************************************
 ****  Create utility for XPM file format... [ X PixMap ]
 ****
 ****  Pixmap XCreatePixmapFromData(dpy, d, cmap, w, h, depth, n, c, col, pix)
 ****
 ****  original:  Daniel Dardailler - Bull RC (89/02/22) daniel@mirsa.inria.fr
 ****  contributors:
 ****             James Bash - AT&T Bell Labs            jmb@attunix.att.com
 *****************************************************************************
 ****  Version 1.2:  Improved storage mgmt & fatal error handling...
 ****  Version 1.1:  extended chars_per_pixel support... [ Read, Create ]
-------------------------------------------------------------------------------
WHO:    Richard Hess                    CORP:   Consilium
TITLE:  Project Manager                 VOICE:  [415] 691-6342
      [ User Interface Services ]       USNAIL: 640 Clyde Court
UUCP:   ...!uunet!cimshop!rhess                 Mountain View, CA 94043
-------------------------------------------------------------------------------
 */

#include <stdio.h>

/* Copyright 1989 GROUPE BULL -- See licence conditions in file COPYRIGHT */
/**************************************************************************
 ****  Constant used in xpm library functions :
 ****
 ****  int XWritePixmapFile(dpy, cmap, filename, pixmap, w, h)
 ****  int XReadPixmapFile(dpy, d, cmap, filename, w, h, depth, pixmap)
 ****  Pixmap XCreatePixmapFromData(dpy, d, cmap, w, h, depth, n, c, col, pix)
 ****
 ****  Daniel Dardailler - Bull RC (89/02/22) e-mail: daniel@mirsa.inria.fr
 **************************************************************************/

#define XPM_FORMAT 1

/* we keep the same codes as for Bitmap management */
#ifndef _XUTIL_H_
#ifdef VMS
#include "decw$include:Xutil.h"
#else
#include <X11/Xutil.h>
#endif
#endif

#define PixmapSuccess        BitmapSuccess 
#define PixmapOpenFailed     BitmapOpenFailed
#define PixmapFileInvalid    BitmapFileInvalid
#define PixmapNoMemory       BitmapNoMemory


typedef struct _Cixel
{
   char c1,c2 ;
} Cixel ;                    /* 2 chars for one pixel value */

typedef struct _CmapEntry
{
   Cixel cixel ;               
   int red,green,blue ;
} CmapEntry ;                    

#define MAXPRINTABLE 93             /* number of printable ascii chars 
                                       minus \ and " for string compat.      */

char * printable = " .XoO+@#$%&*=-;:?>,<1234567890qwertyuipasdfghjklzxcvbnmMNBVCZASDFGHJKLPIUYTREWQ!~^/()_`'][{}|" ;
           /* printable begin with a space, so in most case, due to my
              algorythm, when the number of different colors is less than
              MAXPRINTABLE, it will give a char follow by "nothing" (a space)
              in the readable xpm file */

/**[ XCreatePixmapFromData ]**************************************************
 *
 *  This function allows you to include in your C program (using #include)
 *  a pixmap file in XPM format that was written by XWritePixmapFile
 * (THIS VERSION SUPPORTS : '_chars_per_pixel 1', '_chars_per_pixel 2')
 *****************************************************************************/
Pixmap XCreatePixmapFromData
(
   Display *display,
   Drawable d,
   Colormap colormap,
   unsigned int width,
   unsigned int height,
   int depth,
   unsigned int ncolors,
   unsigned int chars_per_pixel,
   char **colors,          /* array of colormap entries  "cc","#RRGGBB"      */
   char **pixels           /* array of pixels lines    "cc..cc00cccc.."      */
)
{
   Pixmap       pixmap = NULL;
   GC           Gc = NULL;
   XGCValues    xgcv;
   XImage       *xim = NULL;

   CmapEntry    *cmap = NULL;
   int          *Tpixel = NULL;

   char         c1, c2;
   XColor       xcolor;
   Status       status;
   int          i,j,p;
  
   if (ncolors > (MAXPRINTABLE*MAXPRINTABLE))
      return (PixmapFileInvalid);

   if ((chars_per_pixel < 1) || (chars_per_pixel > 2))
      return (PixmapFileInvalid);

   /* now we construct cmap and Tpixel from colors array parameter */
   cmap = (CmapEntry *) malloc(ncolors*sizeof(CmapEntry)) ;
   Tpixel = (int *) malloc(ncolors*sizeof(int)) ;
   if ((cmap == NULL) || (Tpixel == NULL))
   {
      /* free up resources and exit */
      if(cmap)
         free (cmap);
      if(Tpixel)
         free (Tpixel);
      return (PixmapNoMemory);
   }

   if (!colors)
   {
      /* free up resources and exit */
      free (cmap);
      free (Tpixel);
      return (PixmapFileInvalid);
   }

   switch (chars_per_pixel)
   {
      case 1:
         for (i=0; i<2*ncolors ; i+=2)
         {
            if (sscanf(colors[i],"%c", &c1) != 1)
            {
               /* free up resources and exit */
               free (cmap);
               free (Tpixel);
               return (PixmapFileInvalid);
            }
            if (strchr(printable,c1))
            {
               cmap[i/2].cixel.c1 = c1 ;
               if (!XParseColor(display,colormap,colors[i+1],&xcolor))
               {
                  /* free up resources and exit */
                  free (cmap);
                  free (Tpixel);
                  return (PixmapFileInvalid);
               }
               status = getcolor(colors[i+1],&xcolor);
               if (status == 0)
               {
                  /* free up resources and exit */
                  free (cmap);
                  free (Tpixel);
                  return (PixmapFileInvalid);
               }
               Tpixel[i/2] = (int)xcolor.pixel ;
            }
            else
            {
               /* free up resources and exit */
               free (cmap);
               free (Tpixel);
               return (PixmapFileInvalid);
            }
         }
         break;
      case 2:
         for (i=0; i<2*ncolors ; i+=2)
         {
            if (sscanf(colors[i],"%c%c", &c1,&c2) != 2)
            {
               /* free up resources and exit */
               free (cmap);
               free (Tpixel);
               return (PixmapFileInvalid);
            }
            if ((strchr(printable,c1)) && (strchr(printable,c2)))
            {
               cmap[i/2].cixel.c1 = c1 ;
               cmap[i/2].cixel.c2 = c2 ;
               if (!XParseColor(display,colormap,colors[i+1],&xcolor))
               {
                  /* free up resources and exit */
                  free (cmap);
                  free (Tpixel);
                  return (PixmapFileInvalid);
               }
               status = getcolor(colors[i+1],&xcolor);
               if (status == 0)
               {
                  /* free up resources and exit */
                  free (cmap);
                  free (Tpixel);
                  return (PixmapFileInvalid);
               }
               Tpixel[i/2] = (int)xcolor.pixel ;
            }
            else
            {
               /* free up resources and exit */
               free (cmap);
               free (Tpixel);
               return (PixmapFileInvalid);
            }
         }
         break;
   }
  
   pixmap = XCreatePixmap(display,d,width,height,depth);
   Gc = XCreateGC(display,pixmap,0,&xgcv);
   xim = XGetImage(display,pixmap,0,0,width,height,AllPlanes,ZPixmap);
  
   if (!pixels)
   {
      /* free up resources and exit */
      free (cmap);
      free (Tpixel);
      XFreeGC(display, Gc);
      XDestroyImage(xim);
      XFreePixmap(display, pixmap);
      return (PixmapFileInvalid);
   }

   j = 0 ;
   while (j < height)
   {  
      if (strlen(pixels[j]) != (chars_per_pixel*width))
      {
         /* free up resources and exit */
         free (cmap);
         free (Tpixel);
         XFreeGC(display, Gc);
         XDestroyImage(xim);
         XFreePixmap(display, pixmap);
         return (PixmapFileInvalid);
      }
      switch (chars_per_pixel)
      {
         case 1:
            for (i=0; i< (width) ; i++)
            {
               c1 = pixels[j][i] ;
               for (p = 0 ; p < ncolors ; p++)
               {
                  if (cmap[p].cixel.c1 == c1)
                     break ;
               }
               if (p == ncolors)
               {
                  /* free up resources and exit */
                  free (cmap);
                  free (Tpixel);
                  XFreeGC(display, Gc);
                  XDestroyImage(xim);
                  XFreePixmap(display, pixmap);
                  return (PixmapFileInvalid);
               }
               XPutPixel(xim,i,j,Tpixel[p]);
            }
            break;
         case 2:
            for (i=0; i< (2*width) ; i+=2)
            {
               c1 = pixels[j][i] ;
               c2 = pixels[j][i+1] ;
               for (p = 0 ; p < ncolors ; p++)
               {
                  if ((cmap[p].cixel.c1 == c1) && (cmap[p].cixel.c2 == c2))
                     break ;
               }
               if (p == ncolors)
               {
                  /* free up resources and exit */
                  free (cmap);
                  free (Tpixel);
                  XFreeGC(display, Gc);
                  XDestroyImage(xim);
                  XFreePixmap(display, pixmap);
                  return (PixmapFileInvalid);
               }
               XPutPixel(xim,i/2,j,Tpixel[p]);
            }
            break;
      }
      j++;
   }
  
   XPutImage(display,pixmap,Gc,xim,0,0,0,0,width,height);

   /* free up resources and exit */
   free (cmap);
   free (Tpixel);
   XFreeGC(display, Gc);
   XDestroyImage(xim);
   return (pixmap);
}

/****<eof>********************************************************************/
Pixmap XPM_PIXMAP
(
   Widget w,
   unsigned int width,
   unsigned int height,
   unsigned int ncolors,
   unsigned int cpp,
   char **colors,
   char **pixels
)
{
   Colormap        colormap;
   int             depth;
   Arg             args[10];
   int             argcnt;
   Pixmap          pixmap;

   argcnt = 0;     
   XtSetArg(args[argcnt], XmNdepth, &depth); argcnt++;
   XtSetArg(args[argcnt], XmNcolormap, &colormap); argcnt++; 
   XtGetValues(w, args, argcnt);

   pixmap = XCreatePixmapFromData(XtDisplay(w),DefaultRootWindow(XtDisplay(w)),
                                  colormap,width,height,depth,ncolors,cpp,
                                  colors,pixels);

   switch(pixmap)
   {
      case BitmapOpenFailed:
      case BitmapFileInvalid:
      case BitmapNoMemory:
         XtWarning("Could not create pixmap.");
         return(XmUNSPECIFIED_PIXMAP);
      default:
         return(pixmap);
   }
}
#endif

#ifndef IGNORE_STRING_TABLE
#include <stdarg.h>

XmString* STRING_TABLE(int count, ...)
{
   va_list     ap;
   XmString    *array;
   int         i;

   va_start(ap,count);
   array = (XmString*)XtMalloc((count + 1) * sizeof(XmString*));
   for(i = 0;  i < count; i++ )
   {
      array[i] = XmStringCreateLtoR(va_arg(ap, char*),
                                    XmSTRING_DEFAULT_CHARSET);
   }
   array[count] = (XmString)0;
   va_end(ap);
   return(array);
}

void FREE_STRING_TABLE
(
   XmString *table
)
{
   int         i;

   for( i = 0; table[i] != 0 ; i++)
      XmStringFree(table[i]);
   XtFree((char *)table);
}
#endif

/*****************************************************************************/

void load_clist
(void)

/******************************************************************************
Purpose:
   This routine is used to read color names from the X11 color database
   (rgb.txt).

By L. Huewe, USGS/EROS Data Center, 12/91

Modified by L. Huewe, USGS/EROS Data Center, 8/93
   - Rewrote so that spaces OR tabs may separate fields in the rgb.txt file.
     The original version required tabs.

Algorithm Description:
******************************************************************************/
{
   char    color[81];              /* Color name.                            */
   char    *ptr;                   /* Looping index.                         */
   char    *status;                /* Status indicator.                      */
   char    string[81];             /* One line from RGB text file.           */
   FILE    *fp;                    /* Working text file pointer.             */
   long    blue;                   /* Blue value of the color.               */
   long    green;                  /* Green value of the color.              */
   long    i;                      /* Looping index.                         */
   long    red;                    /* Red value of the color.                */

   /* Open the color database for read.  */
   fp = fopen(RGB_DATABASE,"r");
   if (fp == NULL)
   {
      XtWarning("Error opening X color database");
      exit(1);
   }

   /* Read the file information one line at a time.  */
   status = fgets(string,80,fp);
   while (status != NULL)
   {
      sscanf(string,"%ld %ld %ld",&red,&green,&blue);
      for(i=0, ptr=string; i < strlen(string); i++,ptr++)
      {
         if (isalpha((int)*ptr))
         {
            strcpy(color,ptr);
            if (color[strlen(color)-1] == '\n')
               color[strlen(color)-1] = '\0';
            break;
         }
      }
      if (i == strlen(string))
      {
         XtWarning("Error reading the X RGB database,");
         XtWarning("file may be corrupted");
         exit(1);
      }
      xidutil.color_list[xidutil.color_list_num] =
                    (XmString *)XmStringCreate(color,XmSTRING_DEFAULT_CHARSET);
      xidutil.color_list_num++;
      status = fgets(string,80,fp);
   }

   /* Close the file and return.  */
   fclose(fp);
}

/*****************************************************************************
 *                                                                           *
 *  The XID_StringToPixel resource converter routine is taken from the       *
 *  CvtStringToPixel found in the Converters.c Xt source file.               *
 *                                                                           *
 *****************************************************************************/

/***********************************************************
Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

#define done(type, value) \
        {                                                       \
            if (toVal->addr != NULL) {                          \
                if (toVal->size < sizeof(type)) {               \
                    toVal->size = sizeof(type);                 \
                    return False;                               \
                }                                               \
                *(type*)(toVal->addr) = (value);                \
            }                                                   \
            else {                                              \
                static type static_val;                         \
                static_val = (value);                           \
                toVal->addr = (XtPointer)&static_val;           \
            }                                                   \
            toVal->size = sizeof(type);                         \
            return True;                                        \
        }

Boolean XID_StringToPixel
(
   Display *dpy,
   XrmValue *args,
   Cardinal *num_args,
   XrmValue *fromVal,
   XrmValue *toVal,
   XtPointer *converter_data
)
{
   static XColor   screenColor;
   Screen          *screen;
   Status          status;
   String          params[1];
   Cardinal        num_params=1;
   String          str = (String)fromVal->addr;

   if (*num_args != 2)
   {
      XtAppErrorMsg(XtDisplayToApplicationContext(dpy), XtNwrongParameters,
                    "cvtStringToPixel", "XtToolkitError",
                    "String to pixel conversion needs screen and colormap"
                    " arguments",(String *)NULL, (Cardinal *)NULL);
   }

   screen = *((Screen **) args[0].addr);

   if (CompareISOLatin1(str, XtDefaultBackground) == 0)
   {
      *converter_data = NULL;                 /* converter_data unused */
      done(Pixel, WhitePixelOfScreen(screen));/* could be black also   */
   }

   if (CompareISOLatin1(str, XtDefaultForeground) == 0)
   {
      *converter_data = NULL;                 /* converter_data unused */
      done(Pixel, BlackPixelOfScreen(screen));/* could be white also   */
   }

   if (*str == '#')    /* some color rgb definition */
   {
      status = getcolor((char *)str,&screenColor);

      if (status == 0)
      {
         params[0] = str;
         XtAppWarningMsg(XtDisplayToApplicationContext(dpy), "badFormat",
                         "cvtStringToPixel", "XtToolkitError",
                         "RGB color specification \"%s\" has invalid format",
                         params, &num_params);
         *converter_data = NULL;             /* converter_data unused */
         return(False);                      /* return bad status     */
      }
      else
      {
         *converter_data = NULL;             /* converter_data unused */
         done(Pixel, screenColor.pixel);
      }

   }
   else  /* some color name */
   {
      status = getcolor((char *)str,&screenColor);
      if (status == 0)
      {
         String msg, type;
         params[0] = str;
         /* Server returns a specific error code but Xlib discards it.  Ugh */
         if (*str == '#' || (getcolor((char *)str,&screenColor)))
         {
            type = "noColormap";
            msg = "Cannot allocate colormap entry for \"%s\"";
         }
         else
         {
            type = "badValue";
            msg = "Color name \"%s\" is not defined in server database";
         }

         XtAppWarningMsg(XtDisplayToApplicationContext(dpy), type, 
                         "cvtStringToPixel", "XtToolkitError", msg, params,
                         &num_params);
         *converter_data = NULL;               /* converter_data unused */
         return(False);                        /* return bad status     */
      }
      else
      {
         *converter_data = NULL;               /* converter_data unused */
         done(Pixel, screenColor.pixel);
      }
   }
}
/*****************************************************************************
 *                                                                           *
 *                                                                           *
 *****************************************************************************/

/*
 *  ISOLatin character checker taken from the Converters.c source file.
 *  This is used with the XID_StringToPixel resource converter routine.
 */

static int CompareISOLatin1
(
   char *first,
   char *second
)
{
   unsigned char *ap, *bp;

   for (ap = (unsigned char *) first, bp = (unsigned char *) second;
        *ap && *bp; ap++, bp++)
   {
      unsigned char a, b;

      if ((a = *ap) != (b = *bp))
      {
         /* try lowercasing and try again */

         if ((a >= XK_A) && (a <= XK_Z))
            a += (XK_a - XK_A);
         else if ((a >= XK_Agrave) && (a <= XK_Odiaeresis))
            a += (XK_agrave - XK_Agrave);
         else if ((a >= XK_Ooblique) && (a <= XK_Thorn))
            a += (XK_oslash - XK_Ooblique);

         if ((b >= XK_A) && (b <= XK_Z))
            b += (XK_a - XK_A);
         else if ((b >= XK_Agrave) && (b <= XK_Odiaeresis))
            b += (XK_agrave - XK_Agrave);
         else if ((b >= XK_Ooblique) && (b <= XK_Thorn))
            b += (XK_oslash - XK_Ooblique);

         if (a != b) break;
      }
   }
   return (((int) *bp) - ((int) *ap));
}

/*****************************************************************************
 *                                                                           *
 *                                                                           *
 *****************************************************************************/

Status getcolor
(
   char *str,                /* Input color specification. (I)               */
   XColor *color_cell        /* Color cell to be assigned. (O)               */
)

/*****************************************************************************
 *                                                                           *
 *      Purpose:  This function is designed to input an X color specification*
 *                string and return a pixel value that corresponds to a      *
 *                an entry in the color store (xid.color_table[CSTORE_IDX]). *
 *                The routine has been written for the X Image Display       *
 *                application and makes use of application-dependent globals.*
 *                                                                           *
 *      By D. Akkerman, USGS/EROS Data Center, 9/91.                         *
 *                                                                           *
 *      Algorithm(flow):  See in-line documentation.                         *
 *                                                                           *
 *****************************************************************************/
{
   Status       status;        /* Return status variable.                    */
   Status       zero = 0;      /* Failure.                                   */
   Status       nonzero = 1;   /* Success.                                   */
   static long  first = TRUE;  /* First time initialization flag.            */
   long         i;             /* Loop index.                                */
   long         retpix;        /* Working search flag.                       */

   /* Initialize the color store string table (once).  */
   if (first)
   {
      for (i = 0; i < NUM_CSTORE; i++)
         strcpy(xid.color_name[i],"");
      first = FALSE;
   }

   /* Search the work colors array for a matching specification.  */
   retpix = FALSE;
   for (i = 0; i < NUM_CVALS; i++)
   {
      if ((strcmp(wk_colors[i],str)) == 0)
      {
         retpix = TRUE;
         break;
      }
   }

   /* If a match was found then assign the output values and return.  */
   if (retpix)
   {
      color_cell->pixel = xid.color_table[RGBBW_IDX + i].pixel;
      color_cell->red   = xid.color_table[RGBBW_IDX + i].red;
      color_cell->green = xid.color_table[RGBBW_IDX + i].green;
      color_cell->blue  = xid.color_table[RGBBW_IDX + i].blue;
      color_cell->flags = xid.color_table[RGBBW_IDX + i].flags;
      return(nonzero);
   }

   /* Search the color store string table for a matching specification.  */
   retpix = FALSE;
   for (i = 0; i < xid.cindex; i++)
   {
      if ((strcmp(xid.color_name[i],str)) == 0)
      {
         retpix = TRUE;
         break;
      }
   }

   /* If a match was found then assign the output values and return.  */
   if (retpix)
   {
      color_cell->pixel = xid.color_table[CSTORE_IDX + i].pixel;
      color_cell->red   = xid.color_table[CSTORE_IDX + i].red;
      color_cell->green = xid.color_table[CSTORE_IDX + i].green;
      color_cell->blue  = xid.color_table[CSTORE_IDX + i].blue;
      color_cell->flags = xid.color_table[CSTORE_IDX + i].flags;
      return(nonzero);      /* For X compatability.         */
   }
   else    /* Else create a new entry.                             */
   {
      status = XParseColor(xid.display,xid.colormap,(char *)str,color_cell);
      if (status != 0)
      {
         xid.color_table[CSTORE_IDX + xid.cindex].red   = color_cell->red;
         xid.color_table[CSTORE_IDX + xid.cindex].green = color_cell->green;
         xid.color_table[CSTORE_IDX + xid.cindex].blue  = color_cell->blue;
         xid.color_table[CSTORE_IDX + xid.cindex].flags = color_cell->flags;
         color_cell->pixel = xid.color_table[CSTORE_IDX + xid.cindex].pixel;

         strcpy(xid.color_name[xid.cindex],str);

         XStoreColor(xid.display,xid.colormap,
                     &xid.color_table[CSTORE_IDX + xid.cindex]);

         xid.cindex++;
         if (xid.cindex >= NUM_CSTORE)
         {
            xid.cindex = 0;
            xid_message("Maximum number of colors exceeded, some colors may be"
                        " overridden");
         }
         return(nonzero);    /* For X compatability.         */
      }
      else
      {
         color_cell->red = color_cell->green = color_cell->blue = 0;
         color_cell->flags = 0;
         color_cell->pixel = 0;
         return(zero);       /* For X compatability.         */
      }
   }
}

#include "typlim.h"

#define DASH     -1            /* Indicates the last char was a dash.        */
#define COMMA    -2            /* Indicates the last char was a comma.       */
#define NOPUNCT   0            /* Indicates last char was not punc.          */

int parse_rb
(
   char *str,          /* String to be parsed. (I)                           */
   long **range,       /* Range values specified. (O)                        */
   long *rcount,       /* Number of range values specified. (O)              */
   long **bin,         /* Bin values specified. (O)                          */
   long *bcount,       /* Number of bin values specified. (O)                */
   long minval,        /* Minimum gray level value allowed. (I)              */
   long maxval         /* Maximum gray level value allowed. (I)              */
)

/******************************************************************************
Purpose:
   This routine parses the input string into range and bin values.  Range 
   values are separated by a dash and bin values are separated by spaces
   or commas.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   long    i;              /* Looping index.                                 */
   long    status=E_SUCC;  /* Status indicator.                              */
   long    blank;          /* Position of a blank in the string.             */
   long    dash;           /* Position of a dash in the string.              */
   long    comma;          /* Position of a comma in the string.             */
   long    previous=0;     /* Type of previous character.                    */
   long    store;          /* Temporary storage variable.                    */
   int     position;       /* Position of last value parsed.                 */
   long    match = TRUE;   /* Indicates range value was matched.             */
   char    *tptr;          /* Text pointer.                                  */
   char    last[4];        /* Last string read.                              */

   /* Determine the number of ranges and/or bin values specified and allocate
      buffer space accordingly.  */
   tptr = str;
   *rcount = *bcount = 0;
   while (*tptr != '\0')
   {
      if (*tptr == '-')
         (*rcount)++;
      else if ((*tptr == ',') || (*tptr == ' ') || (*tptr == '\n'))
         (*bcount)++;
      tptr++;
   }
   if ((*rcount == 0) && (*bcount == 0))
      *bcount = 1;

   if (*rcount != 0) 
      *range = (long *) calloc((*rcount * 2),sizeof(long));
   if (*bcount != 0) 
      *bin = (long *) calloc((*bcount+1),sizeof(long));

   /* Validate the input specification.  */
   if (!isdigit((int)str[0]))
   {
      xid_message("First character of the input specification must be numeric");
      status = XID_F;
   }
   else
   {
      for (i=0; i < strlen(str); i++)
      {
         if (isalpha((int)str[i]))
         {
            xid_message("Alphabetic characters are not valid");
            status = XID_F;
            break;
         }
         else if (ispunct((int)str[i]))
         {
            if ((str[i] != '-') && (str[i] != ',')) 
            {
               xid_message("Punctuation characters other than (-) or (,) are"
                           " not valid");
               status = XID_F;
               break;
            }
            if (previous != NOPUNCT)
            {
               xid_message("Two consecutive punctuation marks are not valid");
               status = XID_F;
               break;
            }
            if (str[i] == '-')
            {
               match = FALSE;
               previous = DASH;        
            }
            else
               previous = COMMA;
         }
         else if (!isspace((int)str[i]))
         {
            if (previous == DASH)
               match = TRUE;
            previous = NOPUNCT;
         } 
      }
   }

   /* Error was encountered so return.  */
   if (status != XID_S) return(status);

   /* Only one end of a range was specified (ie. " - 100" or "100 - ") */
   if (!match)
   {
      xid_message("Invalid range specification");
      return (XID_F);
   }
        
   /* Parse the string into bin and range values.  */
   dash = (long)strcspn (str,"-");
   comma = (long)strcspn (str,",");
   tptr = str;
   *bcount = *rcount = 0;

   /* While there is more string to parse.  */
   while (*tptr != '\0')
   {
      if ((dash == strlen(tptr)) && (comma == strlen(tptr)))
      {
         /* Read a bin value.  */
         sscanf (tptr,"%ld",(*bin + *bcount));
         if ((*(*bin + *bcount) > maxval) || (*(*bin + *bcount) < minval))
         {
            xid_message("Invalid bin value specified");
            return (XID_F);
         }
         (*bcount)++;
         blank = (long)strcspn (tptr," ");
         if (blank == strlen(tptr))
            return (XID_S);
         else
         {
            tptr += blank;
            while ((isdigit((int)*tptr) == 0) && (*tptr != '\0'))
               tptr++;
         }
      }
      else if (dash < comma)
      {
         /* Read range values.  */
         sscanf (tptr,"%ld - %ld",(*range + (*rcount * 2)),
                 (*range + (*rcount * 2) + 1));
         if ((*(*range + (*rcount * 2)) > maxval) ||
             (*(*range + (*rcount * 2)) < minval) ||
             (*(*range + (*rcount * 2) + 1) > maxval) || 
             (*(*range + (*rcount * 2) + 1) < minval))
         {
            xid_message("Invalid value specified in the range");
            return (XID_F);
         }
         sprintf (last,"%ld",*(*range + (*rcount * 2) + 1));
         tptr += dash;
         substr(tptr,last,&position);
         tptr += (position + strlen(last) - 1);
         while ((isdigit((int)*tptr) == 0) && (*tptr != '\0'))
            tptr++;

         /* Switch order if range[0] is not less than or equal to range[1].  */
         if (*(*range + (*rcount * 2)) > *(*range + (*rcount * 2) + 1))
         {
            store = *(*range + (*rcount * 2));
            *(*range + (*rcount * 2)) = *(*range + (*rcount * 2) + 1);
            *(*range + (*rcount * 2) + 1) = store;
         }
         (*rcount)++;
      }
      else if (comma < dash)
      {
         /* Read a bin value.  */
         sscanf (tptr,"%ld ,",(*bin + *bcount));
         if ((*(*bin + *bcount) > maxval) || (*(*bin + *bcount) < minval))
         {
            xid_message("Invalid bin value specified");
            return (XID_F);
         }
         sprintf (last,"%ld",*(*bin + *bcount));
         (*bcount)++;
         tptr += comma;
         while ((isdigit((int)*tptr) == 0) && (*tptr != '\0'))
            tptr++;
      }
      dash = (long)strcspn (tptr,"-");
      comma = (long)strcspn (tptr,",");
   }
   return (XID_S);
}

int parse_fromto
(
   char *str,          /* String to be parsed. (I)                           */
   long **breakpts,    /* Break point values specified. (O)                  */
   long *count         /* Number of breakpoints specified. (O)               */
)

/******************************************************************************
Purpose:
   This routine parses the input string into breakpoint values.  Breakpoint 
   values are separated by spaces or commas.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
******************************************************************************/
{
   char    last[4];        /* Last string read.                              */
   char    *tptr;          /* Text pointer.                                  */
   int     len;            /* Length of input string.                        */
   long    blank;          /* Position of a blank in the string.             */
   long    comma;          /* Position of a comma in the string.             */
   long    i;              /* Looping index.                                 */
   long    previous=0;     /* Type of previous character.                    */
   long    status=E_SUCC;  /* Status indicator.                              */

   /* Replacing the newline characters in the string with spaces.  */
   len = strlen(str);
   for (i = 0; i < len; i++)
   {
      if (str[i] == '\n')
         str[i] = ' ';
   }

   /* Determine the number of values specified and allocate buffer space
      accordingly.  */
   tptr = str;
   *count = 0;
   while (*tptr != '\0')
   {
      if ((*tptr == ',') || (*tptr == ' ') || (*tptr == '\n'))
         (*count)++;
      tptr++;
   }
   if (*count == 0)
      *count = 1;

   *breakpts = (long *) calloc((*count+1),sizeof(long));

   /* Validate the input specification.  */
   if (!isdigit((int)str[0]))
   {
      xid_message("First character of the input specification must be numeric");
      status = XID_F;
   }
   else
   {
      for (i=0; i < strlen(str); i++)
      {
         if (isalpha((int)str[i]))
         {
            xid_message("Alphabetic characters are not valid");
            status = XID_F;
            break;
         }
         else if (ispunct((int)str[i]))
         {
            if (str[i] != ',') 
            {
               xid_message("Punctuation characters other than (,) are not"
                           " valid");
               status = XID_F;
               break;
            }
            if (previous != NOPUNCT)
            {
               xid_message("Two consecutive punctuation marks are not valid");
               status = XID_F;
               break;
            }
            else
               previous = COMMA;
         }
         else if (!isspace((int)str[i]))
            previous = NOPUNCT;
      }
   }

   /* Error was encountered so return.  */
   if (status != XID_S) return(status);

   /* Parse the string into bin and range values.  */
   comma = (long)strcspn (str,",");
   tptr = str;
   *count = 0;

   /* While there is more string to parse.  */
   while (*tptr != '\0')
   {
      if (comma == strlen(tptr))
      {
         /* Read a break point value.  */
         sscanf (tptr,"%ld",(*breakpts + *count));
         if ((*(*breakpts + *count) > MAXUTINY) ||
             (*(*breakpts + *count) < MINUTINY))
         {
            xid_message("Invalid break point value specified");
            return (XID_F);
         }
         (*count)++;
         blank = (long)strcspn (tptr," ");
         if (blank == strlen(tptr))
            return (XID_S);
         else
         {
            tptr += blank;
            while ((isdigit((int)*tptr) == 0) && (*tptr != '\0'))
               tptr++;
         }
      }
      else if (comma < strlen(tptr))
      {
         /* Read a breakpoint value.  */
         sscanf (tptr,"%ld ,",(*breakpts + *count));
         if ((*(*breakpts + *count) > MAXUTINY) ||
             (*(*breakpts + *count) < MINUTINY))
         {
            xid_message("Invalid break point value specified");
            return (XID_F);
         }
         sprintf (last,"%ld",*(*breakpts + *count));
         (*count)++;
         tptr += comma;
         while ((isdigit((int)*tptr) == 0) && (*tptr != '\0'))
            tptr++;
      }
      comma = (long)strcspn (tptr,",");
   }
   return (XID_S);
}

int substr
(
   char string1[],                      /* string for comparison    (I)      */
   char string2[],                      /* possible substring       (I)      */
   int *pos                             /* position of string2 in   (O)      */
                                        /* string1 (0 if not a substr)       */
)

/************************************************************************
 *                                                                      *
 *    Purpose:  SUBSTR checks if string2 is a substring of string1 and  *
 *              returns the position (1-relative) of the substring      *
 *              in string1.  If string2 is not a substring of string1,  *
 *              position = zero.                                        *
 *                                                                      *
 *    By:  S. Contractor, Code 933.1/NASA/GSFC, 5/24/84                 *
 *    Modifications:                                                    *
 *      D. Maynard, NASA/GSFC RMS, Code 933.1, August 16, 1984          *
 *         Rewritten in C language to improve execution efficiency and  *
 *         bypass some undesirable quirks of the FORTRAN INDEX function.*
 *      D. Hollaren, USGS/EDC Sept 18, 1985                             *
 *         Rewritten to be a useful routine.                            *
 *      C. Greenhagen, USGS/EROS Data Center, 5/88                      *
 *         DMS v3_0 retrocode.                                          *
 *                                                                      *
 ***********************************************************************/
{
   int     len1;
   int     len2;
   char    *addr;
   char    *s1;

   /* Set pointer to start of string1, determine the length of both strings. */
   s1 = string1;
   len1 = strlen(string1);
   len2 = strlen(string2);

   for( ; ; )
   {
      /*  Find the address of the first occurrence in the search      */
      /*  string of the first character from string2.                 */

      addr = strchr (s1,*string2);

      /*  If the address returned is zero then the first character    */
      /*  from string2 is not in the search string.  Set the position */
      /*  variable to zero indicating that string2 is not a substring */
      /*  of string1.                                                 */

      if (addr == 0)
      {
         *pos = 0;
         break;
      }

      /*  If string2 is equal to the portion of string1 starting at   */
      /*  the address just found above and having the same length as  */
      /*  string2, then string2 is a substring of string1.  Calculate */
      /*  the position of the substring in string1.                   */

      /*  If string2 is not a substring at the address just found,    */
      /*  increment the pointer into string1 to start at the next     */
      /*  digit (address found above plus one).  This will in effect  */
      /*  shorten the search string so that this portion of string1   */
      /*  can be searched for the substring string2.                  */

      if (strncmp(addr,string2,len2) == 0)
      {
         *pos = len1 - strlen(addr) + 1;
         break;
      }
      else
         s1 = addr + 1;
   }
   return (E_SUCC);
}

int piece
(
   long from[],         /* The input FROM breakpoint values.              (I)*/
   long to[],           /* The input TO breakpoint values.                (I)*/
   long *count,         /* The number of breakpoint values.               (I)*/
   long map[]           /* Resulting piecewise linear mapping values.     (O)*/
)

/******************************************************************************
NAME:           PIECE

PURPOSE:        Create a piecewise linear mapping through a set of
                breakpoints.

ALGORITHM:
******************************************************************************/
{
   float   lowx;           /* Current low FROM value.                        */
   float   highx;          /* Current high FROM value.                       */
   float   lowy;           /* Current low TO value.                          */
   float   highy;          /* Current high TO value.                         */
   int     i;              /* Index looping variable.                        */
   int     index = 0;      /* Index place holder.                            */
   int     flag = FALSE;   /* Flag indicating not to increment the index.    */

   /* If the first breakpoint is not zero set the initial state of the variable
      as if it were zero.  The flag is set to skip the first increment as
      though the first breakpoint was zero.  Otherwise set the low and high
      values to the beginning breakpoints.  */
   if(from[0] != 0)
   {
      flag = TRUE;
      lowx = 0;
      lowy = (float)to[index];
      highx = (float)from[index] - 1 ;
      highy = (float)to[index];
   }
   else
   {
      lowx = (float)from[index];
      lowy = (float)to[index];
      highx = (float)from[index+1];
      highy = (float)to[index+1];
   }

   /* Calculate the new mapping values between each breakpoint.  If the last
      FROM breakpoint is not equal to MAXUTINY then the process will fill up
      the remain- ing mapping values.  */
   for(i=0; i <= MAXUTINY; i++)
   {
      /* If we have gone past a breakpoint increment the index and reset the
         high and low values to the next breakpoint.  */
      if (i > highx)
      {
         if (flag)
            flag = FALSE;
         else
            ++index;

         /* If this is the last breakpoint set the from high to MAXUTINY.  */
         if (index >= (*count)-1)
         {
            lowx  = (float)from[index];
            lowy  = (float)to[index];
            highx = MAXUTINY; 
            highy = (float)to[index];
         }
         else
         {
            lowx = (float)from[index];
            lowy  = (float)to[index];
            highx = (float)from[index+1];
            highy = (float)to[index+1];
         }
      }

      /* Calculate the new map value.  */
      if (highx == lowx)
         map[i] = (long)highy;
      else
         map[i] = (long)(lowy+((highy-lowy)*((i-lowx)/(highx-lowx))));
   }
   return(E_SUCC);
}

int renumber
(
   long from[],        /* The input FROM pixel values.                    (I)*/
   long to[],          /* The input TO pixel values.                      (I)*/
   long *count,        /* The number of input values.                     (I)*/
   long map[]          /* Resulting mapping values.                       (O)*/
)

/******************************************************************************
NAME:           RENUMBER

PURPOSE:        Returns a mapping which changes the the value of the "from" 
                pixels to the value of the "to" pixels.

ALGORITHM:
******************************************************************************/
{
   int     i;              /* Index looping variable.                        */

   /* Initialize the mapping to a linear mapping.  */
   for (i=0; i < 256; i++)
      map[i] = i;

   /* Renumber the appropriate values.  */
   for (i=0; i < *count; i++)
      map[from[i]] = to[i];

   return(E_SUCC);
}

void xid_noop
(void)

/******************************************************************************
NAME:           xid_noop

PURPOSE:        A noop function for context widget.     

ALGORITHM:
******************************************************************************/
{
   return;
}
