#include <ctype.h>
#include <string.h>
#include "las.h" /* to get CMLEN definition for cm.h (included in util.h) */
#include "util.h" /* c_imgprs definition */
#include "labserv.h" /* c_lsstat definition */
#include "xid.h"
#include "xid_func.h"

/* local function prototypes */
int xid_imgprs
(
   char *i_specs,
   struct PRS_LST *list,
   long *images
);

int xid_wndchk
(
   double inwind[],     /* Input window to be checked                        */
   long vlwind[]        /* Valid window                                      */
);

void xid_compress
(
    char *filled,
    char *small
);

int xid_image_info
(
   char *spec,               /* single image specification                */
   struct PRS_LST *i_record  /* record for name, window, & bands          */
);

int xid_make_window
(
   char *numerals,
   double window[]
);

int xid_make_bands
(
   char *numerals,
   long bands[]
);

int xid_make_proj
(
   char *numerals,
   long proj[]
);

int xid_bndchk
(
   long *inbnds,
   long *vlbnds,
   long *nbands
);
/* end prototypes */

int xid_getin
(
   char *fname,            /* Input image name                */
   char tnames[][CMLEN],   /* TAE names of images specified   */
   char hnames[][CMLEN],   /* host names of images specified  */
   long *images,           /* Number of images                */
   long (*window)[4],      /* Image windows (sl ss nl ns)     */
   long (*bands)[MAXBND+1],/* Image bands                     */
   long *nbands,           /* Number of bands per image       */
   long *totbnd,           /* total number of bands specified */
   long *imglimit,         /* Maximum images caller can handle*/
   long *same_window,      /* 1 if all windows the same       */
   long (*qflag)[2]        /* 1 if caller specified the window*/
                           /* and bands                       */
                           /* 0 if default window and bands   */
)

/*****************************************************************************
 *                                                                           *
 *      Purpose:  Break input specification into individual image            *
 *                specifications.  The input name may be a host file         *
 *                name or a LAS name containing window and/or band           *
 *                band specifications.                                       *
 *                                                                           *
 *      By D. Akkerman, USGS/EROS Data Center, 10/89.                        *
 *       - Adapted from the c_getin LAS routine for use with the LAS Pixar   *
 *         prototype system.                                                 *
 *      Modified by L. Huewe, USGS/EROS Data Center, 9/91                    *
 *          - Made more generic to allow the parsing of host file names      *
 *            containing window and/or band specifications.                  *
 *      Modified by L. Huewe, USGS/EROS Data Center, 8/93                    *
 *          - Modified to allow lat/long windows, projection coordinate      *
 *            windows and band ranges.                                       *
 *      Modified by L. Huewe, USGS/EROS Data Center, 10/93                   *
 *          - Modified to work with nonlas image names only.  LAS image names*
 *            are now parsed with c_getin2.  Also, modified to allow nonlas  *
 *            names to contain both upper and lower case.                    *
 *                                                                           *
 *      Algorithm References:                                                *
 *                                                                           *
 *      This algorithm is roughly based on the GETIN utility in LAS.         *
 *                                                                           *
 *****************************************************************************/
{
   static struct PRS_LST p_list[MAXIMG]; /* Image list returned from imgprs  */

   /* Declare all functions returning non-integer values.                  */

   char    err_text[ERRLEN+1];    /* error message text                      */
   char    tae[CMLEN+1];          /* current TAE name                        */
   long    image;                 /* image counter and index                 */
   long    mode = 0;              /* file access                             */
   long    nonfatal = 1;          /* Non-fatal key for c_errmsg              */
   long    status;                /* status code returned by CM              */

   /* Parse the input specification.  */
   if( E_FAIL == c_imgprs(fname,p_list,images))
   {
      c_errmsg("The input image parse was unsuccessful","xid_getin-parse",
               &nonfatal);
      return(E_FAIL);
   }

   /* Determine whether or not the user exceeded the maximum number of input
      images.  */
   if( *images > *imglimit )
   {
      sprintf(err_text, "%s %ld %s %ld %s.", "There were", *images,
              "parsed which is greater than the limit of",*imglimit, "images");
      c_errmsg(err_text,"xid_getin-imglmt",&nonfatal);
      return(E_FAIL);
   }

   for ( image = 0; image < *images; image++ )
   {
      /* Determine TAE name.  */
      strcpy(tae,p_list[image].name);
      strcpy(&tnames[image][0],tae);
   }

   for ( image = 0; image < *images; image++ )
   {
      strcpy(&hnames[image][0],&tnames[image][0]);

      /* Determine whether or not image exists.  */
      if (c_lsstat(&hnames[image][0],&mode) != E_SUCC)
      {
         sprintf(err_text,"Image file %s does not exist",&tnames[image][0]);
         c_errmsg(err_text,"xid_getin-exist",&nonfatal);
         return(E_FAIL);
      }
   }

   /* Check the windows and bands. */
   *totbnd = 0;
   *same_window = 1;
   for ( image = 0; image < *images; image++ )
   {
      status = xid_wndchk(p_list[image].window,window[image]);
      if (status != E_SUCC)
      {
         sprintf(err_text,"The window specified for %s was invalid",
                 &tnames[image][0]);
         c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
         return(E_FAIL);
      }

      if (image > 0)
      {
         if ((window[image][2] != window[image-1][2]) ||
             (window[image][3] != window[image-1][3]) )
            *same_window = 0;
      }

      status = xid_bndchk(p_list[image].bands,bands[image],&nbands[image]);
      if (status != E_SUCC)
      {
         sprintf(err_text,"The bands specified for %s were invalid",
                 &tnames[image][0]);
         c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
         return(E_FAIL);
      }

      *totbnd += nbands[image];
      qflag[image][0] = p_list[image].qflag[0];
      qflag[image][1] = p_list[image].qflag[1];
   }

   if (*totbnd > MAXBND)
   {
      c_errmsg("Too many bands specified in input image(s)",
               "xid_getin-invband", &nonfatal);
      return(E_FAIL);
   }
   return(E_SUCC);
}

int xid_imgprs
(
   char *i_specs,
   struct PRS_LST *list,
   long *images
)

/*****************************************************************************
 *                                                                           *
 *      Purpose:                                                             *
 *          Parse image names, bands and windows.                            *
 *                                                                           *
 *      Algorithm References:                                                *
 *          Taken from the LAS routine GETIN.                                * 
 *****************************************************************************/
{
   char    line[CMLEN+1];          /* holds compressed i_specs               */
   char    *spec_begin;            /* address of next image spec             */
   char    *pos;                   /* position of '+'                        */
   char    err_text[ERRLEN+1];     /* text of error message                  */
   long    nonfatal = 1;           /* Non-fatal key for c_errmsg             */

   /* Compress i_specs so that there is no white space except a single space
      between numbers */
   xid_compress( i_specs, line );

   /* For each image specification (seperated by '+'s), find the image name,
      window, and bands.*/
   spec_begin = line;
   *images = 0;
   do
   {
      if ( *images > MAXIMG )
      {
         sprintf(err_text," %s %s %d %s.", "Too many images were specified.",
                 "  Only",MAXIMG,"images are allowed");
         c_errmsg(err_text,"xid_getin-maximg",&nonfatal);
         return E_FAIL;
      }

      pos = strchr( spec_begin, '+' );
      if ( pos != (char *)0 )
         *pos = '\0';

      if (xid_image_info(spec_begin,&list[*images]) != E_SUCC)
         return E_FAIL;

      if ( pos != (char *)0 ) spec_begin = pos + 1;
         (*images)++;
   }
   while ( pos > (char *)0);
      return(E_SUCC);
}

void xid_compress
(
    char *filled,
    char *small
)
/*****************************************************************************
 *                                                                           *
 *      Purpose:                                                             *
 *          Remove extra white space from the input specification.           * 
 *                                                                           *
 *      Algorithm References:                                                *
 *          Taken from the LAS routine GETIN.                                * 
 *****************************************************************************/
{
   unsigned short  i;
   unsigned short  j;
   enum states
   {
      digit, non_digit
   }p_state;

   /* Copy all characters except white space.  If a digit was just read, copy a
      space and then set back to a non-digit state. */
   p_state = non_digit;
   i = j = 0;
   while ( filled[i] != '\0' )
   {
      /*
      if (( isspace(filled[i]) )  || (filled[i] == ','))
      {
         if ( p_state == digit )
         {
            small[j++] = ' ';
            i++;
         }
         else
            i++;
      }
      */
      if ( isspace((int)filled[i]) )
      {
         if ( p_state == digit )
         {
            small[j++] = ' ';
            i++;
         }
         else
            i++;
         p_state = non_digit;
      }
      else if ((filled[i] == ','))
      {
         small[j++] = ' ';
         i++;
         p_state = non_digit;
      }
      else if ( (j>0) && (filled[i]==':') && (small[j-1]==' ') )
      {
         small[j-1] = ':';
         i++;
         p_state = non_digit;
      }
      else if ( (j>0) && (filled[i]==')') && (small[j-1]==' ') )
      {
         small[j-1] = ')';
         i++;
         p_state = non_digit;
      }
      else
      {
         if (isdigit((int)filled[i]))
            p_state = digit;
         else
            p_state = non_digit;   
         small[j++] = filled[i++];
      }
   }
   small[j] = '\0';
   return;
}

int xid_image_info
(
   char *spec,               /* single image specification                */
   struct PRS_LST *i_record  /* record for name, window, & bands          */
)

/*****************************************************************************
 *                                                                           *
 *      Purpose:                                                             *
 *          Extract window and band specifications.                          *
 *                                                                           *
 *      Algorithm References:                                                *
 *          Taken from the LAS routine GETIN.                                * 
 *****************************************************************************/
{
   char             *left;               /* position of left parenthesis     */
   char             *colon;              /* position of colon                */
   char             *right;              /* position of right parenthesis    */
   char             *semi;               /* position of ';'                  */
   char             *ptr;                /* temporary pointer                */
   char             err_text[ERRLEN];
   unsigned short   wind;                /* window index and counter         */
   long             nonfatal = 1;        /* Non-fatal key for c_errmsg       */

   /* Find position of (, :, and ). */
   left = strchr( spec, '(' );
   if (left != NULL)
   {
      colon = strchr( left, ':' );
      semi = strchr( left, ';' );
   }
   else
   {
      colon = NULL;
      semi = NULL;
   }

   right = strchr( spec, ')' );
   if ( (left==0) && (colon==0) && (right==0) && (semi == 0))
   {
      /* no window, bands or projection specified */
      strcpy( i_record->name, spec);
      for ( wind=0; wind<4; wind++ )
         i_record->window[wind] = 0;
      i_record->bands[0] = 0;
      i_record->qflag[0] = i_record->qflag[1] = 0;
      i_record->proj[0] = 0;
   }
   else if (( left > (char *) 0 ) && (semi == (char *)0))
   {
      i_record->proj[0] = 0;
      *left = '\0';
      strcpy( i_record->name, spec );
      *left = '(';
      if ( (colon==0) && (left<right) )
      {
         /*      IMAGE(WINDOW)   */
         *right = '\0';
         if (xid_make_window(left+1,i_record->window) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "The window format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->bands[0] = 0;
         i_record->qflag[0] = 1;
      }
      else if ( (left<colon) && (colon+1==right) )
      {
         /*      IMAGE(WINDOW:)  */
         *colon = '\0';
         if (xid_make_window(left+1,i_record->window) != E_SUCC)
         {
            *colon = ':';
            sprintf(err_text, "The window format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
            return(E_FAIL);
         }

         *colon = ':';
         i_record->bands[0] = 0;
         i_record->qflag[0] = 1;
      }
      else if ( (left+1==colon) && (colon<right) )
      {
         /*      IMAGE(:BANDS)   */
         for ( wind=0; wind<4; wind++ )
            i_record->window[wind] = 0.0;
         *right = '\0';
         if (xid_make_bands(colon+1,i_record->bands) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text,"The band format error was generated by %s",spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->qflag[1] = 1;
      }
      else if ( (left<colon) && (colon<right) )
      {
         /*      IMAGE(WINDOW:BANDS)     */
         *colon = '\0';
         if (xid_make_window(left+1,i_record->window) != E_SUCC)
         {
            *colon = ':';
            sprintf(err_text, "The window format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
            return(E_FAIL);
         }

         *colon = ':';
         *right = '\0';
         if (xid_make_bands(colon+1,i_record->bands) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "The band format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->qflag[0] = i_record->qflag[1] = 1;
      }
      else
      {
         /* cannot parse window and bands */
         sprintf(err_text, "Error: Specification %s cannot be parsed",
                 spec);
         c_errmsg(err_text,"xid_getin-invspec",&nonfatal);
         return(E_FAIL);
      }
   }
   else if (( left > (char *) 0 ) && (colon == (char *)0))
   {
      i_record->bands[0] = 0;
      *left = '\0';
      strcpy( i_record->name, spec );
      *left = '(';
      if ( (left<semi) && (semi+1==right) )
      {
         /*      IMAGE(WINDOW;)  */
         *semi = '\0';
         if (xid_make_window(left+1,i_record->window) != E_SUCC)
         {
            *semi = ';';
            sprintf(err_text, "Error: Window format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
            return(E_FAIL);
         }

         *semi = ';';
         i_record->proj[0] = 0;
         i_record->qflag[0] = 1;
      }
      else if ( (left+1==semi) && (semi<right) )
      {
         /*      IMAGE(;PROJECTION       */
         for ( wind=0; wind<4; wind++ )
            i_record->window[wind] = 0;
         *right = '\0';
         if (xid_make_proj(semi+1,i_record->proj) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Projection format error was generated"
                    " by %s", spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->qflag[1] = 1;
      }
      else if ( (left<semi) && (semi<right) )
      {
         /*      IMAGE(WINDOW;PROJECTION */
         *semi = '\0';
         if (xid_make_window(left+1,i_record->window) != E_SUCC)
         {
            *semi = ';';
            sprintf(err_text, "Error: Window format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
            return(E_FAIL);
         }

         *semi = ';';
         *right = '\0';
         if (xid_make_proj(semi+1,i_record->proj) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Projection format error was generated"
                    " by %s", spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }
         *right = ')';
         i_record->qflag[0] = i_record->qflag[1] = 1;
      }
      else
      {
         /* cannot parse window and bands */
         sprintf(err_text, "Specification %s cannot be parsed", spec);
         c_errmsg(err_text,"xid_getin-invspec",&nonfatal);
         return(E_FAIL);
      }
   }
   else if (( left > (char *) 0 ) && (semi != (char *)0) &&
            (colon != (char *) 0))
   {
      i_record->proj[0] = 0;
      *left = '\0';
      strcpy( i_record->name, spec );
      *left = '(';
      if ( (left<colon) && (colon+1==semi) && (semi+1==right) )
      {
         /*      IMAGE(WINDOW:;) */
         *colon = '\0';
         *semi = '\0';
         if (xid_make_window(left+1,i_record->window) != E_SUCC)
         {
            *colon = ':';
            *semi = ';';
            sprintf(err_text, "Error: Window format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
            return(E_FAIL);
         }
         *colon = ':';
         *semi = ';';
         i_record->bands[0] = 0;
         i_record->qflag[0] = 1;
         i_record->proj[0] = 0;
      }
      else if ( (left+1==colon) && (colon+1==semi) && (semi<right))
      {
         /*      IMAGE(:;PROJECTION)     */
         for ( wind=0; wind<4; wind++ )
            i_record->window[wind] = 0;
         *right = '\0';
         if (xid_make_proj(semi+1,i_record->proj) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Projection format error was generated"
                    " by %s", spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->qflag[1] = 1;
         i_record->bands[0] = 0;
      }
      else if ( (left+1==colon) && (colon<semi) && (semi+1==right))
      {
         /*      IMAGE(:BANDS;)  */
         for ( wind=0; wind<4; wind++ )
            i_record->window[wind] = 0;
         *right = '\0';
         if (xid_make_bands(colon+1,i_record->bands) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Band format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->qflag[1] = 1;
         i_record->proj[0] = 0;
      }
      else if ( (left+1==colon) && (colon<semi) && (semi<right))
      {
         /*      IMAGE(:BANDS;PROJECTION)*/
         for ( wind=0; wind<4; wind++ )
            i_record->window[wind] = 0;
         *right = '\0';
         if (xid_make_bands(colon+1,i_record->bands) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Band format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }
         if (xid_make_proj(semi+1,i_record->proj) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Projection format error was generated"
                    " by %s", spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->qflag[1] = 1;
      }
      else if ( (left<colon) && (colon<semi) && (semi+1==right) )
      {
         /*      IMAGE(WINDOW:BANDS;)    */
         *colon = '\0';
         if (xid_make_window(left+1,i_record->window) != E_SUCC)
         {
            *colon = ':';
            sprintf(err_text, "Error: Window format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
            return(E_FAIL);
         }

         *colon = ':';
         *right = '\0';
         if (xid_make_bands(colon+1,i_record->bands) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Band format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->qflag[0] = i_record->qflag[1] = 1;
         i_record->proj[0] = 0;
      }
      else if ( (left<colon) && (colon+1==semi) && (semi<right) )
      {
         /*      IMAGE(WINDOW:;PROJECTION)*/
         *colon = '\0';
         if (xid_make_window(left+1,i_record->window) != E_SUCC)
         {
            *colon = ':';
            sprintf(err_text, "Error: Window format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
            return(E_FAIL);
         }

         *colon = ':';
         *right = '\0';
         if (xid_make_proj(semi+1,i_record->proj) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Band format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->qflag[0] = i_record->qflag[1] = 1;
         i_record->bands[0] = 0;
      }
      else if ( (left<colon) && (colon<semi) && (semi<right) )
      {
         /*      IMAGE(WINDOW:BANDS;PROJECTION)*/
         *colon = '\0';
         if (xid_make_window(left+1,i_record->window) != E_SUCC)
         {
            *colon = ':';
            sprintf(err_text, "Error: Window format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invwnd",&nonfatal);
            return(E_FAIL);
         }

         *colon = ':';
         *right = '\0';
         if (xid_make_bands(colon+1,i_record->bands) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Band format error was generated by %s",
                    spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }
         if (xid_make_proj(semi+1,i_record->proj) != E_SUCC)
         {
            *right = ')';
            sprintf(err_text, "Error: Projection format error was generated"
                    " by %s", spec);
            c_errmsg(err_text,"xid_getin-invbnds",&nonfatal);
            return(E_FAIL);
         }

         *right = ')';
         i_record->qflag[0] = i_record->qflag[1] = 1;
      }
      else
      {
         /* cannot parse window and bands */
         sprintf(err_text, "Error: Specification %s cannot be parsed", spec);
         c_errmsg(err_text,"xid_getin-invspec",&nonfatal);
         return(E_FAIL);
      }
   }
   else
   {
      /* cannot parse, no left paren */
      sprintf(err_text, "No left parenthesis in {%s}.", spec);
      c_errmsg(err_text,"xid_getin-lparen",&nonfatal);
      return(E_FAIL);
   }

   ptr = i_record->name + strlen(i_record->name) - 1;
   for (; (*ptr == ' ') && (ptr > i_record->name); ptr--)
      *ptr = '\0';
   return E_SUCC;
}

int xid_make_window
(
   char *numerals,
   double window[]
)

/*****************************************************************************
 *                                                                           *
 *      Purpose:                                                             *
 *          Extract window components.                                       *
 *                                                                           *
 *      Algorithm References:                                                *
 *          Taken from the LAS routine GETIN.                                * 
 *****************************************************************************/
{
   unsigned short  wind;           /* window counter and index               */
   char            *n_begin,       /* first numeral in window component      */
                   *n_end;         /* the space following the numeral        */
   long            nonfatal = 1;   /* Non-fatal key for c_errmsg             */

   /* Find each window component specification.  */
   wind = 0;
   n_begin = numerals;
   do
   {
      n_end = strchr( n_begin, ' ' );
      if ( n_end == 0 )
      {
         /* window[wind++] = atoi( n_begin ); */
         sscanf(n_begin,"%lf",&window[wind++]);
      }
      else
      {
         *n_end = '\0';
         /* window[wind++] = atoi( n_begin ); */
         sscanf(n_begin,"%lf",&window[wind++]);
         *n_end = ' ';
         n_begin = n_end + 1;
      }
   }

   while ((wind < 4) && (n_end != 0))
      /* do nothing */;
   if ( (wind==4) && (n_end!=0) )
   {
      c_errmsg("There are too many window components", "xid_getin-invwnd",
               &nonfatal);
      return E_FAIL;
   }
   else if ( (wind<4) && (n_end==0) )
   {
      c_errmsg("There are not enough window components", "xid_getin-invwnd",
               &nonfatal);
      return E_FAIL;
   }
   return E_SUCC;
}

int xid_make_bands
(
   char *numerals,
   long bands[]
)

/*****************************************************************************
 *                                                                           *
 *      Purpose:                                                             *
 *          Extract band numbers.                                            *
 *                                                                           *
 *      Algorithm References:                                                *
 *          Taken from the LAS routine GETIN.                                * 
 *****************************************************************************/
{
   unsigned short  band;           /* band counter and index                 */
   char            *n_begin,       /* first numeral in band component        */
                   *n_end,         /* the space following the numeral        */
                   *n_dash;        /* postion of the '-' for band range      */
   long            nonfatal = 1;   /* Non-fatal key for c_errmsg             */
   long            end,i,start;    /* loop variables for band range          */

   /* Find each band component specification.  */
   band = 0;
   n_begin = numerals;
   do
   {
      n_end = strchr( n_begin, ' ' );
      n_dash = strchr( n_begin, '-');
      if ( n_end == n_dash )
         bands[band++] = atoi( n_begin );
      else if (((n_end < n_dash) && (n_end !=0)) || n_dash==0 )
      {
         *n_end = '\0';
         bands[band++] = atoi( n_begin );
         *n_end = ' ';
         n_begin = n_end + 1;
      }
      else
      {
         start = atoi( n_begin );
         n_begin = n_dash + 1;
         end = atoi( n_begin );
         for (i = start; i <= end; i++)
            bands[band++] = i;
         n_begin = n_end + 1;
      }
   }

   while ((band <= MAXBND) && (n_end != 0))
      /* do nothing */;
   if (band > MAXBND)
   {
      c_errmsg("There are too many band components", "xid_getin-invbnds",
               &nonfatal);
      bands[band] = 0;
      return E_FAIL;
   }

   bands[band] = 0;
   return(E_SUCC);
}

int xid_make_proj
(
   char *numerals,
   long proj[]
)

{
   unsigned short  band;              /* band counter and index              */
   char            *n_begin,          /* first numeral in band component     */
                   *n_end,            /* the space following the numeral     */
                   *n_dash;           /* postion of the '-' for band range   */
   long            nonfatal = 1;      /* Non-fatal key for c_errmsg          */

   band = 0;
   n_begin = numerals;
   do
   {
      n_end = strchr( n_begin, ' ' );
      n_dash = strchr( n_begin, '-');
      if ( n_end == n_dash )
      {
         if ((strcmp(n_begin,"LS") == 0) || (strcmp(n_begin,"ls") == 0))
         {
            proj[0] = 0;
            band++;
         }
         else
         {
            if ((strcmp(n_begin,"DEG") == 0) || (strcmp(n_begin,"deg") == 0))
            {
               proj[0] = 4;
               band++;
            }
            else
            {
               if ((strcmp(n_begin,"DMS") == 0) ||
                   (strcmp(n_begin,"dms") == 0))
               {
                  proj[0] = 5;
                  band++;
               }
               else
               {
                  if ((strcmp(n_begin,"PRO") == 0) ||
                      (strcmp(n_begin,"pro") == 0))
                  {
                     proj[0] = 3;
                     band++;
                  }
                  else
                     proj[0] = 0;
               }
            }
         }
      }
   }
   while ((band == 0) && (n_end != 0))
      /* do nothing */;
   if ( band > 1 )
   {
      c_errmsg("Error: There are too many projection components",
               "getin-invbnds",&nonfatal);
      proj[band] = 0;
      return (E_FAIL);
   }
   proj[band] = 0;
   return(E_SUCC);
}

int xid_wndchk
(
   double inwind[],     /* Input window to be checked                        */
   long vlwind[]        /* Valid window                                      */
)

/******************************************************************************
NAME                            XID_WNDCHK

PURPOSE         Validate an image window

PROGRAM HISTORY
        PROGRAMMER        DATE          REASON
        ----------        ----          ------
        P. Mumford      April 85        Original development for NEWLAS.
        K. Zanter       Dec. 1987       1.  switched to raw C types 
                                        2.  changed include file specification
                                        3.  changed DESC argument to be a 
                                            character string
                                        4.  renamed source from wndchk to
                                            c_wndchk
        B. Ailts        12/87           Implemented new DDR structure and call
        D. Hollaren     04/88           changed include spec las
                                        Changed cerrmsg to c_errmsg and
                                        passed status code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
        D. VanderZee    07/90   Standardized error handling 
        L. Huewe        12/92           Adapted for non-LAS images in XID.

                                        
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   XID_WNDCHK works with XID global structures. 

PROJECT                         LAS

ALGORITHM 
        open the image in random mode and get the number of lines and samples
        if the window is 0 0 0 0 then return 1 1 #lines and #samples
        otherwise check that the starting line <= #lines
                                 starting sample <= #samples
                                 starting line + number lines -1 <= #lines
                                 starting sample + number samples -1 <= #samps

ALGORITHM REFERENCES
        Taken from LAS support routine c_wndchk.
******************************************************************************/
{
   long    sl=1;                   /* a starting line req. by imageio        */
   long    ss=1;                   /* a starting samp req. by imageio        */
   long    nl=0;                   /* number of lines in image               */
   long    nonfatal=1;             /* non-fatal error message code           */
   long    ns=0;                   /* number of samples in image             */
   char    err_text[ERRLEN+1];     /* error message text                     */

   /* Determine the maximum number of lines and samples.  If the window
      specification is all zeros, return the actual number of lines and
      samples.  */
   nl = xid_nonlas.lines;
   ns = xid_nonlas.samples;
   if ((inwind[0] == 0) && (inwind[1] == 0) && (inwind[2] == 0) &&
       (inwind[3] == 0))
   {
      vlwind[0] = sl;
      vlwind[1] = ss;
      vlwind[2] = nl;
      vlwind[3] = ns;
   }
   else
   {
      /* Validate starting line.  */
      if (inwind[0] > 0)
      {
         if (inwind[0] <= nl)
            vlwind[0] = (long)inwind[0];
         else
         {
            sprintf(err_text,"%s %f %s %ld %s", "Error: Starting line",
                    inwind[0], "is greater than the number of lines", nl,
                    "in the image");
            c_errmsg(err_text,"wndchk-invwnd",&nonfatal);
            return (E_FAIL);
         }
      }
      else
      {
         sprintf(err_text, "Error: Starting line %f %s", inwind[0],
                 "must be positive");
         c_errmsg(err_text,"wndchk-invwnd",&nonfatal);
         return (E_FAIL);
      }

      /* Validate starting sample.  */
      if (inwind[1] > 0)
      {
         if (inwind[1] <= ns)
            vlwind[1] = (long)inwind[1];
         else
         {
            sprintf(err_text, "%s %f %s %ld %s.", "Error: Starting sample",
                    inwind[1], "is greater than the number of samples" , ns,
                    "in the image");
            c_errmsg(err_text,"wndchk-invwnd",&nonfatal);
            return (E_FAIL);
         }
      }
      else
      {
         sprintf(err_text, "Error: Starting sample %f %s", inwind[1],
                 "must be positive");
         c_errmsg(err_text,"wndchk-invbnd",&nonfatal);
         return (E_FAIL);
      }

      /* Validate number of lines.  */
      if (inwind[2] > 0)
      {
         if ((inwind[0] + inwind[2] - 1) <= nl)
            vlwind[2] = (long)inwind[2];
         else
         {
            vlwind[2] = (long)(nl - inwind[0] + 1);
            sprintf(err_text, "%s %f %s %f %s %s %ld.",
                    "Error: Starting line", inwind[0],
                    "and the number of lines", inwind[2],
                    "put some of the window outside of the image.",
                    "Number of lines reset to", vlwind[2]);
            c_errmsg(err_text,"wndchk-invwnd",&nonfatal);
         }
      }
      else
      {
         sprintf(err_text, "Error: Number of lines %f %s", inwind[2],
                 "must be positive");
         c_errmsg(err_text,"wndchk-invwnd",&nonfatal);
         return (E_FAIL);
      }

      /* Validate number of samples.  */
      if (inwind[3] > 0)
      {
         if ((inwind[1] + inwind[3] - 1) <= ns)
            vlwind[3] = (long)inwind[3];
         else
         {
            vlwind[3] = (long)(ns - inwind[1] + 1);
            sprintf(err_text, "%s %f %s %f %s %s %ld.",
                    "Error: Starting sample", inwind[1],
                    "and the number of samples", inwind[3],
                    "put some of the window outside of the image.",
                    "Number of samples reset to", vlwind[3]);
            c_errmsg(err_text,"wndchk-invwnd",&nonfatal);
         }
      }
      else
      {
         sprintf(err_text, "Error: Number of samples %f %s.", inwind[3],
                 "must be positive" );
         c_errmsg(err_text,"wndchk-invwnd",&nonfatal);
         return (E_FAIL);
      }
   }
   return (E_SUCC);
}

int xid_bndchk
(
   long *inbnds,
   long *vlbnds,
   long *nbands
)

/******************************************************************************
NAME:                           XID_BNDCHK

FUNCTION:    Checks the band specification against the actual image.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         ??         ??
    2.0         9/86       K. Gacke     Added parameter "nbands" to return
                                        number of bands specified.
    3.0       Dec. 1987    K. Zanter    1.  switched to raw C types
                                        2.  changed include file specification
                                        3.  changed DESC argument to be a
                                            character string
                                        4.  renamed source from bndchk to
                                            c_bndchk
    4.0         12/87      B. Ailts     implemented new DDR structure and calls
    5.0         04/87      D. Hollaren  changed include spec to las
                                        change cerrmsg to c_errmsg and
                                        pass status code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
     5.0        07/90      D. VanderZee Standardized error handling 
     6.0        12/90      B. Ailts     Changed check for invalid band numbers
                                        Now will return a bad status when the
                                        first band is invalid
     7.0        3/91      B. Davis      put increment of band index of while
                                        loop at end of loop, not at beginning.
                                        the check was not working correctly for
                                        multi-image, multi-band input, but not
                                        erring out either, causing disastrous 
                                        delayed side effects if the input
                                        specification was incorrect.
    7.1         9/92        T. Mittan   Removed ; after last } to meet standard
                12/92       L. Huewe    Adapted for use with nonLAS images in
                                        XID.

PROJECT:        LAS

ALGORITHM:
        The input band specification is checked against the actual bands of 
        the image.

ALGORITHM REFERENCES:
        Taken from the LAS support routine c_bndchk.
******************************************************************************/
{
   long    band;                    /* counter and index for bands           */
   long    bands;                   /* number of bands in image              */
   long    nonfatal = 1;            /* Non-fatal error message code          */
   char    err_text[ERRLEN+1];      /* error message text                    */

   /* Determine maximum number of bands.  */
   bands = xid_nonlas.nbands;

   /* Validate number of bands.  */
   if (bands > MAXBND)
   {
      sprintf(err_text, "%s %ld %s %d %s", "Error: Image has too many bands."
              "  It has", bands, ".  The limit is", MAXBND, ".");
      c_errmsg(err_text,"bndchk-maxband",&nonfatal);
      return E_FAIL;
   }

   /* Make sure each band number specified is less than MAXBND, is a positive
      value and is less than the number of bands in the image.  */
   if (inbnds[0] == 0)
   {
      for (band=0; band < bands; band++) 
         vlbnds[band] = band+1;
      vlbnds[bands] = 0;
   }
   else
   {
      band = 0;
      while ((inbnds[band] > 0) && (band < MAXBND))
      {
         vlbnds[band] = inbnds[band];
         if (inbnds[band] < 0)
         {
            sprintf(err_text, "%s %ld %s %s", "Error: Band", band,
                    "is invalid.", " All band numbers must be positive.");
            c_errmsg(err_text,"bndchk-invband",&nonfatal);
            return E_FAIL;
         }
         else if (inbnds[band] > bands)
         {
            sprintf(err_text, "%s %ld %s%ld%s", "Error: Band", inbnds[band],
                    "is greater than the number of bands (", bands,
                    ") in the image" );
            c_errmsg(err_text,"bndchk-invband",&nonfatal);
            return E_FAIL;
         }
         else if ((band >= MAXBND) && (inbnds[band] != 0))
         {
            sprintf(err_text, "%s %d %s", "Error: Too many bands were"
                    " specified.  Only", MAXBND, "bands can be specified.");
            c_errmsg(err_text,"bndchk-invband",&nonfatal);
            return E_FAIL;
         }
         band++;
      }
      vlbnds[band] = 0;
   }
   *nbands = band;
   return E_SUCC;
}
