#include "xid.h"
#include "xid_func.h"

void xid_image_makelist
(
   Widget red_widget,          /* Red list widget.  (I)                      */
   Widget green_widget,        /* Green list widget.  (I)                    */
   Widget blue_widget,         /* Blue list widget.  (I)                     */
   Widget save_widget,         /* Save list widget. (I)                      */
   long current_flg            /* Indicates whether or not to make a special
                                  entry in the save list for the currently
                                  displayed image.  (I)                      */
)

/******************************************************************************
Purpose:
   This routine builds a list of plane numbers available for manipulation
   (i.e., display, flicker, etc.) and places this list into the widgets 
   specifed by red_widget, green_widget and blue_widget.  A list of previously
   saved widgets is also built and placed into the widget specified by 
   save_widget.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
******************************************************************************/
{
   XmString  *items;               /* Array of compound strings defining
                                      valid plane numbers.                   */
   int       plane_cnt;            /* Number of valid planes.                */
   long      current;              /* Current index into string array.       */
   long      dummy;                /* Dummy variable (place holder for
                                      function call).                        */
   long      i;                    /* Looping index.                         */
   long      idx;                  /* Index increment.                       */
   long      num_composite;        /* Number of saved composite views.       */
   char      tmpstr[50];           /* Temporary string variable.             */

   /* Determine the number of planes available for displaying.  */
   for (i=0,plane_cnt=0; i < xid.max_plane; i++)
   {
      if (xid_planes[i].flags & USED)
         plane_cnt++;
   }

   /* Report a warning and clear all lists if there are no planes available
      for display.  */
   if (plane_cnt == 0)
   {
      xid_message("No image data available for displaying");
      if (red_widget != NULL)
         XmListDeleteAllItems(red_widget);
      if (green_widget != NULL)
         XmListDeleteAllItems(green_widget);
      if (blue_widget != NULL)
         XmListDeleteAllItems(blue_widget);
      if (save_widget != NULL)
         XmListDeleteAllItems(save_widget);
      return;
   }

   /* Allocate space for the list of plane numbers.  */
   else if ((items = (XmString *) calloc(plane_cnt,sizeof(XmString))) == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }

   /* Assign the list of valid plane numbers to the appropriate widgets.  */
   for (i=0,current=0; i < xid.max_plane; i++)
   {
      if (xid_planes[i].flags & USED)
      {
         sprintf(tmpstr,"Plane %ld",i+1);
         items[current] = XmStringCreateLtoR(tmpstr,XmSTRING_DEFAULT_CHARSET);
         current++;
      }
   }

   if (red_widget != NULL)
      XtVaSetValues(red_widget,XmNitems,items,XmNitemCount,plane_cnt,NULL);
   if (green_widget != NULL)
      XtVaSetValues(green_widget,XmNitems,items,XmNitemCount,plane_cnt,NULL);
   if (blue_widget != NULL)
      XtVaSetValues(blue_widget,XmNitems,items,XmNitemCount,plane_cnt,NULL);

   /* Free appropriate resources.  */
   for (i=0; i < plane_cnt; i++)
      XmStringFree(items[i]);
   free(items);

   /* Assign entries to the save list.  */
   if (save_widget == NULL)
      return;

   /* Get the number of saved entries and allocate space for them.  */
   xid_get_composite(&dummy,&num_composite);
   if ((items = (XmString *) calloc(num_composite+1,sizeof(XmString))) == NULL)
   {
      xid_message("Insufficient memory available");
      return;
   }

   /* Make a special entry for the current image.  */
   if ((xid.view != -1) && current_flg)
   {
      items[0] = XmStringCreateLtoR("Current",XmSTRING_DEFAULT_CHARSET);
      idx = 1;
   }
   else
      idx = 0;

   /* Assign the list of valid names to the appropriate widget.  */
   for (i=0, current=0; i < NUM_COMPOSITE; i++)
   {
      if (xid_composite[i].flags & USED)
      {
         items[current+idx] = XmStringCreateLtoR(xid_composite[i].name,
                                                 XmSTRING_DEFAULT_CHARSET);
         current++;
      }
   }
   XtVaSetValues(save_widget,XmNitems,items,XmNitemCount,num_composite+idx,
                 NULL);

   /* Free appropriate resources.  */
   for (i=0; i < num_composite+1; i++)
      XmStringFree(items[i]);
   free(items);

   return;
}

long xid_make_ximage
(void)

/******************************************************************************
Purpose:
   This routine creates a pixmap for the currently selected flicker image.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
******************************************************************************/
{
   float          scaler;          /* LUT scaler for actual number of colors.*/
   long           buf_size;        /* Number of bytes in a single band.      */
   long           i,j;             /* Looping indices.                       */
   long           img_idx;         /* Index into colormap of first image
                                      color.                                 */
   long           maxval;          /* Maximum image data value.              */
   long           nl;              /* Number of lines.                       */
   long           nplanes;         /* Number of planes (bands) to display.   */
   long           ns;              /* Number of samples.                     */
   long           num_imgcolors;   /* Number of image colors.                */
   long           temp_numfullres; /* Used to save the xid.num_fullres value
                                      since it is modified when creating the
                                      fullres flicker pixmap                 */
   long           tform[256];      /* Pixel transform array.                 */
   static unsigned char
                  br[256];         /* Reserved color values.                 */
   static unsigned char
                  gr[256];
   static unsigned char
                  rr[256];
   unsigned char  *blut = NULL;    /* Blue LUT for composite image.          */
   unsigned char  *composite;      /* Composite RGB 8-bit image or single band
                                      black white 8-bit image.               */
   unsigned char  *comptr;         /* Pointer to composite buffer.           */
   unsigned long  *data = NULL;    /* Pointer to output values for 24-bit
                                      data.                                  */
   unsigned char  *dataptr;        /* Pointer to image data stored in memory
                                      plane.                                 */
   unsigned char  *glut = NULL;    /* Green LUT for composite image.         */
   unsigned char  *rgbptr[3];      /* Pointers to RGB components.            */
   unsigned char  *rlut = NULL;    /* Red LUT for composite image.           */
   XImage         *ximage;         /* X image structure.                     */
   int            status;          /* Routine return status                  */
   XColor *current_color_ptr=NULL; /* current color table pointer */
   int current_color_offset;   /* This indicates the offset in the real color
                                  that the current color table pointer starts
                                  at.  This is needed since the adjust table
                                  starts in different locations based on the 
                                  display mode. */

   /* Set the index into the flicker arrays.  */
   j = xid_flicker.nplanes;

   /* Calculate the image data buffer size.  */
   nl = xid_planes[xid_flicker.planes[j][0] - 1].dspwin[NL];
   ns = xid_planes[xid_flicker.planes[j][0] - 1].dspwin[NS];

   /* If the nl/ns are different for any of the planes, we can't
      flicker them */
   if((nl != xid_planes[xid_flicker.planes[j][1] - 1].dspwin[NL]) ||
      (nl != xid_planes[xid_flicker.planes[j][2] - 1].dspwin[NL]) ||
      (ns != xid_planes[xid_flicker.planes[j][1] - 1].dspwin[NS]) ||
      (ns != xid_planes[xid_flicker.planes[j][2] - 1].dspwin[NS]) )
   {
      xid_message("Image planes are not the same size");
      return(XID_F);
   }
   buf_size = nl * ns;

   /* Determine the number of planes to display.  */
   if (((xid_flicker.planes[j][RED]) == (xid_flicker.planes[j][GREEN])) &&
       ((xid_flicker.planes[j][GREEN]) == (xid_flicker.planes[j][BLUE])))
      nplanes = 1;
   else
      nplanes = 3;

   maxval = xid_planes[xid_flicker.planes[j][RED] - 1].maxval;

   /* Calculate the scaler needed to reduce the image to the appropriate number
      of colors.  */
   if ((*xid.colormap_ptr == xid.colormap) && (maxval < NUM_IMGCOLORS) &&
       (nplanes == 1))
   {
      scaler = 1.0;
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
   }
   else if (*xid.colormap_ptr == xid.colormap)
   {
      scaler = (256.0 / (float)NUM_IMGCOLORS);       /* (2^8 / # of colors) */
      num_imgcolors = NUM_IMGCOLORS;
      img_idx = IMG_IDX;
   }
   else 
   {
      scaler = 1.0;
      num_imgcolors = 256;
      img_idx = 0;
   }

   /* select the color table to use */
   if (xid_adjust.color_adjust != NULL)
   {
       /* the color is currently being adjusted, so use the current adjusted
          color table */
       current_color_ptr = xid_adjust.color_adjust;
       current_color_offset = 0;
   }
   else
   {
       /* not currently adjusting color, so use normal color table */
       current_color_ptr = xid.colortable_ptr;
       current_color_offset = img_idx;
   }
   /* Make sure the non-image portion of the flicker colortable has the
      appropriate values.  */
   for (i=0; i < img_idx; i++)
   {
      xid_flicker.colortable[j][i].pixel = current_color_ptr[i].pixel;
      xid_flicker.colortable[j][i].blue = current_color_ptr[i].blue; 
      xid_flicker.colortable[j][i].green = current_color_ptr[i].green; 
      xid_flicker.colortable[j][i].red = current_color_ptr[i].red; 
      xid_flicker.colortable[j][i].flags = DoRed | DoGreen | DoBlue;
   }

   /* 8-bit monitor -- black and white image.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) && (nplanes == 1))
   {
      comptr = composite = (unsigned char *)malloc(buf_size);
      if (composite == NULL)
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }

      /* if not a saved composite, create a linear ramp */
      if (!xid_flicker.cflags[j])
      {
         /* check if displaying the same band for all colors (i.e. greyscale
            image displayed) */
         if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) && 
               (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
         {
            /* currently displaying a single plane and flickering a single 
               plane, so use the current color table so any user made 
               adjustments to brightness and contrast can be kept. */
            for (i = 0; i < num_imgcolors; i++)
            {
               xid_flicker.colortable[j][i+img_idx].pixel = 
                    current_color_ptr[i+current_color_offset].pixel;
               xid_flicker.colortable[j][i+img_idx].red = 
                    current_color_ptr[i+current_color_offset].red;
               xid_flicker.colortable[j][i+img_idx].green = 
                    current_color_ptr[i+current_color_offset].green;
               xid_flicker.colortable[j][i+img_idx].blue = 
                    current_color_ptr[i+current_color_offset].blue;
               xid_flicker.colortable[j][i+img_idx].flags = 
                    DoRed | DoGreen | DoBlue;
            }
         }
         else
         {
            /* currently displayed color table does not match the planes
               selected, so use a linear color table since color tables just
               aren't compatible (there is no good way to recreate the 
               adjustment used to get the color table either) */
            for (i = 0; i < num_imgcolors; i++)
            {
               xid_flicker.colortable[j][img_idx + i].pixel =
                  xid.colortable_ptr[img_idx + i].pixel;
               xid_flicker.colortable[j][img_idx + i].blue =
                  xid_flicker.colortable[j][img_idx + i].green =
                  xid_flicker.colortable[j][img_idx + i].red =
                  (((float)i * scaler) * 257);
               xid_flicker.colortable[j][img_idx + i].flags = DoRed | DoGreen
                  | DoBlue;
            }
         }
      }

      /* Scale the input data.  */
      dataptr = xid_planes[xid_flicker.planes[j][0] - 1].image_data;
      for (i=0; i < buf_size; i++,comptr++,dataptr++)
      {
         *comptr = (unsigned char)xid_flicker.colortable[j][(unsigned char)
                      ((float)img_idx + ((float) (*dataptr / scaler)))].pixel; 
      }
   }

   /* 8-bit monitor -- RGB image.  */ 
   else if (xid.visual_info[0].visual->class == PseudoColor)
   {
      /* Allocate buffer space for LUTs.  */
      rlut = (unsigned char *)malloc(256);
      if (rlut == NULL)
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }
      glut = (unsigned char *)malloc(256);
      if (glut == NULL)
      {
         xid_message("Insufficient memory available");
         free(rlut);
         return(XID_F);
      }
      blut = (unsigned char *)malloc(256);
      if (blut == NULL)
      {
         xid_message("Insufficient memory available");
         free(rlut);
         free(glut);
         return(XID_F);
      }

      /* Compress the 3 band RGB image into a single band of image data and 3
         LUTs.  */
      comptr = composite = (unsigned char *)malloc(buf_size);
      if (composite == NULL)
      {
         xid_message("Insufficient memory available");
         free(rlut);
         free(glut);
         free(blut);
         return(XID_F);
      }
      rgbptr[RED] = xid_planes[xid_flicker.planes[j][RED] - 1].image_data; 
      rgbptr[GREEN] = xid_planes[xid_flicker.planes[j][GREEN] - 1].image_data;
      rgbptr[BLUE] = xid_planes[xid_flicker.planes[j][BLUE] - 1].image_data; 
      xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
                  num_imgcolors,&composite,rlut,glut,blut,0,rr,gr,br);
      /* Save these LUTs to be used as reserved colors for loading of full
         resolution windows.  */
      for (i=0; i < 256; i++)
      {
         xid_flicker.rr[xid_flicker.nplanes][i] = rlut[i];
         xid_flicker.gr[xid_flicker.nplanes][i] = glut[i];
         xid_flicker.br[xid_flicker.nplanes][i] = blut[i];
      }

      /* if not a saved composite, build a new colormap */
      if (!xid_flicker.cflags[j])
      {
         /* Build the new colormap.  The adjustments to the current color table 
            can not be applied without significant changes to how adjustments
            are managed.  Probably not important for Pseudocolor since most
            production people have a 24-bit direct color table.  Saved
            composites can be used to avoid this limitation. */
         for (i = 0; i < num_imgcolors; i++)
         {
            xid_flicker.colortable[j][img_idx + i].pixel =
               xid.colortable_ptr[img_idx + i].pixel;
            xid_flicker.colortable[j][img_idx + i].red = (rlut[i] * 257);
            xid_flicker.colortable[j][img_idx + i].green = (glut[i] * 257);
            xid_flicker.colortable[j][img_idx + i].blue = (blut[i] * 257);
            xid_flicker.colortable[j][img_idx + i].flags = DoRed | DoGreen 
               | DoBlue;
         }
      }
      for (i=0; i < buf_size; i++)
      {
         composite[i] = (unsigned char)xid_flicker.colortable[j][img_idx +
                                                           composite[i]].pixel;
      }
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) && 
            (nplanes == 1))
   {
      /* Allocate buffer space.  */
      data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
      if (data == NULL)
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }

      /* Set a pointer to the image data.  */
      rgbptr[RED] = xid_planes[xid_flicker.planes[j][RED] - 1].image_data; 

      /* Create the transform array which scales output values such that 56
         colormap entries can be reserved as working entries.  */
      for (i = 0; i < 256; i++)
         tform[i] = (long)(((float)i / scaler) + (float)img_idx);

      /* Scale the input data.  */
      for (i = 0; i < buf_size; i++, rgbptr[RED]++)
      {
         data[i] = (tform[(*rgbptr[RED])] << xid.red_shift) + 
                   (tform[(*rgbptr[RED])] << xid.green_shift) +
                   (tform[(*rgbptr[RED])] << xid.blue_shift);
      }

      if (!xid_flicker.cflags[j])
      {
         for (i = img_idx; i < num_imgcolors + img_idx; i++)
         {
            /* copy the current color table so any user made adjustments are 
               applied when flickering */
            xid_flicker.colortable[j][i].pixel = current_color_ptr[i].pixel;
            xid_flicker.colortable[j][i].red = current_color_ptr[i].red;
            xid_flicker.colortable[j][i].green = current_color_ptr[i].green;
            xid_flicker.colortable[j][i].blue = current_color_ptr[i].blue;
            xid_flicker.colortable[j][i].flags = DoRed | DoGreen | DoBlue;
         }
      }
   }
   else if (xid.visual_info[0].visual->class == DirectColor)
   {
      /* Allocate buffer space.  */
      data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
      if (data == NULL)
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }

      /* Set pointers to the red, green and blue bands of image data.  */
      rgbptr[RED] = xid_planes[xid_flicker.planes[j][RED] - 1].image_data;
      rgbptr[GREEN] = xid_planes[xid_flicker.planes[j][GREEN] - 1].image_data;
      rgbptr[BLUE] = xid_planes[xid_flicker.planes[j][BLUE] - 1].image_data;

      /* Create the transform array which scales output values such that 56
         colormap entries can be reserved as working entries.  */
      for (i = 0; i < 256; i++)
         tform[i] = (long)(((float)i / scaler) + (float)img_idx);

      /* Scale the input data.  */
      for (i = 0; i < buf_size; i++)
      {
         data[i] = (tform[(*rgbptr[RED])] << xid.red_shift) +
            (tform[(*rgbptr[GREEN])] << xid.green_shift) +
            (tform[(*rgbptr[BLUE])] << xid.blue_shift);
         rgbptr[RED]++;
         rgbptr[GREEN]++;
         rgbptr[BLUE]++;
      }

      if (!xid_flicker.cflags[j])
      {
         for (i = img_idx; i < (img_idx + num_imgcolors); i++)
         {
            /* copy the current color table so any user made adjustments are 
               applied when flickering */
            xid_flicker.colortable[j][i].pixel = current_color_ptr[i].pixel;
            xid_flicker.colortable[j][i].red = current_color_ptr[i].red;
            xid_flicker.colortable[j][i].green = current_color_ptr[i].green;
            xid_flicker.colortable[j][i].blue = current_color_ptr[i].blue;
            xid_flicker.colortable[j][i].flags = DoRed | DoGreen | DoBlue;
         }
      }
   }

   /* Allocate buffer space.  */
   if ((xid.visual_info[0].visual->class == PseudoColor) &&
       (xid.visual_info[0].depth == 24))
   {
      data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
      if (data == NULL)
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }
      for (i=0; i < buf_size; i++)
         data[i] = composite[i];
      free(composite);
   }

   /* Create an X image.  */
   if (xid.visual_info[0].depth == 24)
   {
      ximage = XCreateImage(xid.display,xid.visual_info[0].visual,
                            xid.visual_info[0].depth,ZPixmap,0,(char *)data,ns,
                            nl,8,0);
   }
   else
   {
      ximage = XCreateImage(xid.display,xid.visual_info[0].visual,
                          xid.visual_info[0].depth,ZPixmap,0,(char *)composite,
                          ns,nl,8,0);
   }

   /* Create a pixmap and write the image into it.  */
   xid_flicker.pixmaps[j] = XCreatePixmap(xid.display,XtWindow(
                                          main_image_bulletinboard),ns,nl,
                                          xid.visual_info[0].depth);
   XPutImage(xid.display,xid_flicker.pixmaps[j],xid_flicker.gc,ximage,0,0,0,0,
             ns,nl);

   /* Free the appropriate resources.  */
   if (xid.visual_info[0].visual->class == DirectColor)
      free(data);
   else
   {
      free(composite);
      if (nplanes != 1)
      {
         free(rlut);
         free(glut);
         free(blut);
      }
   }

   /* The data in the image structure has already been freed above.  */
   ximage->data = NULL;
   XDestroyImage(ximage);

   /* Now create the fullres flicker pixmap */
   temp_numfullres = xid.num_fullres;
   status = fullres_make_flicker_pixmaps();
   if(status != XID_S)
   {
      xid_message("Error encountered creating the full resolution flicker \
                  pixmaps.");
      return (XID_F);
   }
   xid.num_fullres = temp_numfullres;

   return(XID_S);
}

void xid_find_composite
(
   char *name,                /* Name of the image composite whose index is
                                 to be found.  (I)                          */
   long *cindex               /* Composite index of input name.  (O)        */
)

/******************************************************************************
Purpose:
   This routine finds the composite index of the entry with the specified
   composite name.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
******************************************************************************/
{
   long    i;                  /* Looping index.                             */

   /* Initialize the index.  */
   *cindex = -1;

   /* Search the composite array for a matching name.  If one is found, assign
      the appropriate index to the output parameter.  */
   for (i=0; i < NUM_COMPOSITE; i++)
   {
      if ((xid_composite[i].flags & USED) &&
          (strcmp(name,xid_composite[i].name) == 0))
      {
         *cindex = i;
         break;
      }
   }
}

void xid_get_composite
(
   long *cindex,               /* Composite index of first free entry.  (O)  */
   long *num_used              /* Number of currently used entries (not
                                  including index returned).  (O)            */
)

/******************************************************************************
Purpose:
   This routine finds the first free composite entry and returns it's index
   as well as the number of entries currently in use.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
******************************************************************************/
{
   long    i;                  /* Looping index.                             */

   /* Initialize values.  */
   *cindex = -1;
   *num_used = 0;

   /* Count the number of composite entries and assign the parameter cindex to
      the index of the first available entry.  */
   for (i=0; i < NUM_COMPOSITE; i++)
   {
      if ((*cindex == -1) && (!(xid_composite[i].flags & USED)))
         *cindex = i;
      else if (xid_composite[i].flags & USED)
         (*num_used)++;
   }

   /* Warn the user if composite entries are going to be overwritten.  */
   if (*cindex == -1)
   {
      *cindex = 0;
      xid_message("The maximum number of composite views have been saved.\n"
                  "Entries are now being overwritten.");
   }
}

void xid_clear_composite
(
   long plane_num             /* Plane number that has been initialized. (I)*/
)

/******************************************************************************
Purpose:
   This routine removes entries from the composite structure which contain
   plane numbers that have been initialized (cleared).

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
******************************************************************************/
{
   XmString        xmstring;       /* Motif format string.                   */
   long            i;              /* Looping index.                         */

   /* Loop through the composite structure; for each entry in use, compare the
      list of plane numbers with the plane number that was intialized.  If a
      match is found, remove the entry from the structure as well as from any
      displayed list (e.g., on the show panel or the flicker panel).  */
   for (i=0; i < NUM_COMPOSITE; i++)
   {
      if (xid_composite[i].flags & USED)
      {
         if ((xid_composite[i].rgbplanes[RED] == plane_num) ||
             (xid_composite[i].rgbplanes[GREEN] == plane_num) ||
             (xid_composite[i].rgbplanes[BLUE] == plane_num))
         {
            xmstring = XmStringCreateLtoR(xid_composite[i].name,
                                          XmSTRING_DEFAULT_CHARSET);
            if (XtIsRealized(Shell_Show))
               XmListDeleteItem(show_savelist,xmstring);
            if (xid_flicker.flags & FLICKER_ACTIVE)
               XmListDeleteItem(flicker_savelist,xmstring);
            XmStringFree(xmstring);

            xid_composite[i].rgbplanes[RED] = xid_composite[i].rgbplanes[GREEN]
               = xid_composite[i].rgbplanes[BLUE] = 0;
            xid_composite[i].flags &= (~USED);
            strcpy(xid_composite[i].name,"");
         }
      }
   }
}
