#include "xid.h" 
#include "xid_func.h"

#define BITS_USED 4     /* See "Notes" below.                           */

/* Macro used to calculate a color table index number.  */
#define RGB_TO_INDEX(r,g,b) \
        (*(shift_red + (r)) + *(shift_green + (g)) + *(shift_blue + (b)))

/* Macro used to expand a scaled value to an 8-bit value.  */
#define MAKE_8_BITS(v) (((v) << (8 - bits_used)) | (0xff >> bits_used))

/* Color table entry structure.  */
struct ENTRY_DEF
{
   unsigned long count;          /* Pixel population count.                  */
   unsigned long index;          /* Pixel value index.                       */
   unsigned char blue;
   unsigned char dn_1;           /* unknown                                  */
   unsigned char dn_2;
   unsigned char green;
   unsigned char red;            /* Color table entry RGB values.            */
   unsigned char reserved;       /* Boolean for a reserved color entry.      */
};

/* prototype */
int n_compare
(
   const void *entry1,
   const void *entry2
);

long              bits_used;         /* # of bits for each color.            */
long              table_length;      /* # of color table entries.            */
struct ENTRY_DEF  *table;            /* Working color table.                 */
unsigned long     shift_red[256];    /* Shift values to scale input data.    */
unsigned long     shift_green[256];
unsigned long     shift_blue[256];


int xid_rgb2lut
(
    long num_images,          /* Number of input images. (I)          */
    unsigned char **red,      /* Red pixel values. (I)                */
    unsigned char **green,    /* Green pixel values. (I)              */
    unsigned char **blue,     /* Blue pixel values. (I)               */
    long *npts,               /* Total number of input pixels. (I)    */
    long lutlen,              /* Number of LUT entries to use. (I)    */
    unsigned char **composite,/* Output composite image. (O)          */
    unsigned char *red_lut,   /* Output red LUT. (O)                  */
    unsigned char *green_lut, /* Output green LUT. (O)                */
    unsigned char *blue_lut,  /* Output blue LUT. (O)                 */
    long num_reserved,        /* Number of reserved colors in LUT. (I)*/
    unsigned char *reserved_red,   /* Input reserved red LUT values. (I)   */
    unsigned char *reserved_green, /* Input reserved green LUT values. (I) */
    unsigned char *reserved_blue   /* Input reserved blue LUT values. (I)  */
)

/*****************************************************************************
 *                                                                           *
 *     Purpose:  This color compression routine is designed to input three   *
 *               8-bit image bands and output a single image band (composite)*
 *               and three LUTs.  The routine is designed to work within a   *
 *               specified color limit.                                      *
 *                                                                           *
 *               This routine is basically a rewrite of the                  *
 *               NASA JPL rgbs_to_images_and_luts routine developed by       *
 *               CalTech.  The original copyright appears below:             *
 *                                                                           *
 *     Copyright (c) 1989, California Institiute of Technology.  U.S.        *
 *     Government sponsorship under NASA Contract NAS7-918 is acknowledged.  *
 *                                                                           *
 *     Modified by D. Akkerman, USGS/EROS Data Center, 8/90.                 *
 *     - adapted for use with the XLASDSP application.                       *
 *     Updated by D. Akkerman, USGS/EROS Data Center, 9/91.                  *
 *     - fixed a problem with clearing table space when the table is full.   *
 *     - rewritten for clarity, debugging, and portability.                  *
 *                                                                           *
 *     Algorithm(flow):  See in-line code.                                   *
 *                                                                           *
 *     Notes: The BITS_USED constant is used to define the number of bits    *
 *     per component value (R,G,B) that the data is to be scaled to.  This   *
 *     value is used as a power of 2 to determine the range of the component *
 *     values.  For example, if BITS_USED equals four, then 2^4 or 16 values *
 *     (0 to 15) are allowed for each R, G, or B component value.  Thus, red *
 *     component values are input with a range of 0 to 255 and are scaled    *
 *     (shifted) to a range of 0 to 15.  This value, in turn, also           *
 *     determines the length of the color compression table used:            *
 *     (1 << (3 * bits_used)).  Again if BITS_USED equals 4, then            *
 *     table_length = 4096 (or (1 << 12)).  Four is used as the default      *
 *     because it provides the best overall range of color compression,      *
 *     requires a moderate amount of memory, maintains a relatively small    *
 *     table size, and requires a modest amount of processing.  Lower values *
 *     are inadequate because too few colors are maintained, and conversely, *
 *     larger values are expensive in terms of processing resources and      *
 *     don't provide adequate color compression when the total number of     *
 *     colors gets large (simulated colors and total error gets very high).  *
 *                                                                           *
 *****************************************************************************/
{
   long          i,j,k;            /* Loop indices.                          */
   long          pixel;            /* Pixel loop counter.                    */
   long          source;           /* Working color table index (source
                                      entry).                                */
   long          next_free_entry;  /* Used to index temporary entries for
                                      table compression.                     */
   long          reserved_index;   /* Working reserved pixel index           */
   long          r_temp;           /* Working RGB pixel values; to shorten
                                      code.                                  */
   long          g_temp;
   long          b_temp;
   long          unclassed_index;  /* Unclassified index used in color
                                      compression.                           */
   long          best_dist;        /* Closest compressed color (least
                                      distance).                             */
   long          best_index;       /* Closest compressed color table entry.  */
   long          dist;             /* Working distance variable.             */
   long          square[513];      /* Squares of differences table.          */
   long          *square_off;      /* Pointer to differences table midpoint
                                      (zero).                                */
   long          red_diff;         /* Working pixel difference variables.    */
   long          green_diff;
   long          blue_diff;
   long          one = 1;          /* Nonfatal error specifier.              */
   struct ENTRY_DEF
                 *entry;           /* Working color table pointers.          */
   struct ENTRY_DEF
                 *table_ptr;
   unsigned long limit;            /* Largest internal R,G,B values.         */
   unsigned long index;            /* Color table index values.              */
   unsigned long index2;
   unsigned char *red_ptr;         /* Working reserved color pointers.       */
   unsigned char *green_ptr;
   unsigned char *blue_ptr;
                                   /* Working composite pixel (LUT entry)
                                      value.                                 */
   unsigned char *composite_ptr;
   unsigned char pix_red;          /* Working pixel component values.        */
   unsigned char pix_green;
   unsigned char pix_blue;

#ifdef DEBUG
   long          simulated = 0;    /* Number of pixels that were compressed. */
   float         total_error=0.0;  /* Calculated error based on the amount of
                                      difference between the target pixel and
                                      the closest matching and the number of
                                      target pixels so compressed.           */
#endif

   /* Initialize the color table.  */
   /* Assign the number of bits per component value to use.                */
   bits_used = BITS_USED;

   /* Create shift tables to scale component values.                       */
   for (i = 0; i < 256; i++)
   {
      shift_red[i]   = (i >> (8 - bits_used)) << (2 * bits_used);
      shift_green[i] = (i >> (8 - bits_used)) << bits_used;
      shift_blue[i]  = (i >> (8 - bits_used));
   }

   /* Assign the table length, allocate the table and initialize it.       */
   table_length = (1 << (3 * bits_used));

   table = (struct ENTRY_DEF *)malloc(table_length * sizeof(struct ENTRY_DEF));
   if (table == NULL)
   {
      c_errmsg("Error allocating dynamic memory","xid_rgb2lut-malloc",&one);
      return(E_FAIL);
   }

   (void)memset((char *)table,0,(table_length * sizeof(struct ENTRY_DEF)));

   limit = (1 << bits_used);
   for (pix_red = 0; pix_red < limit; pix_red++)
   {
      for (pix_green = 0; pix_green < limit; pix_green++)
      {
         for (pix_blue = 0; pix_blue < limit; pix_blue++)
         {
            index = (pix_red << (2 * bits_used)) | (pix_green << bits_used) |
                    pix_blue;
            entry = (table + index);
            entry->red   = pix_red;
            entry->green = pix_green;
            entry->blue  = pix_blue;
            entry->index = index;
         }
      }
   }

   /* Construct the color table from the input parameters.  */
   /* Mark the reserved colors (if any) in the color table.                */
   if (num_reserved != 0)
   {
      red_ptr   = reserved_red;
      green_ptr = reserved_green;
      blue_ptr  = reserved_blue;

      for (i = 0; i < num_reserved; i++)
      {
         index = RGB_TO_INDEX(*red_ptr,*green_ptr,*blue_ptr);
         (table + index)->reserved = 1;
         red_ptr++;
         green_ptr++;
         blue_ptr++;
      }
   }

   /* Populate the color table using the input data.                       */
   for (i = 0; i < num_images; i++)
   {
      red_ptr   = *(red + i);
      green_ptr = *(green + i);
      blue_ptr  = *(blue + i);

      for (pixel = 0; pixel < *(npts + i); pixel++)
      {
         index = RGB_TO_INDEX(*red_ptr,*green_ptr,*blue_ptr);
         (table + index)->count++;
         red_ptr++;
         green_ptr++;
         blue_ptr++;
      }
   }

   /* Clear space at the front of the table for reserved colors (if any) by
      moving existing entries to empty spaces (if they exist) at the end of
      the table. */
   if (num_reserved != 0)
   {
      j = (table_length - 1);
      for (i = 0; i < num_reserved; i++)
      {
         if ((table + i)->count != 0)
         {
            for (k = j; k > 0; k--)
            {
               if (((table + k)->count == 0) && ((table + k)->reserved == 0))
               {
                  (void)memcpy((char *)(table + k),(char *)(table + i),
                     sizeof(*table));
                  break;
               }
            }
            if (k == 0)
            {
#ifdef DEBUG
               printf("No free entries -- break used\n");
#endif
               break;        /* No free entries; no sense continuing.        */
            }
         }
      }
   }

   /* Compress the color table using reserved colors by preference.        */
   next_free_entry = num_reserved;
   for (source = num_reserved; source < table_length; source++)
   {
      if ((table + source)->reserved)
      {
         r_temp = (table + source)->red; /* Use temps to shorten statements. */
         g_temp = (table + source)->green;
         b_temp = (table + source)->blue;

         index2 = RGB_TO_INDEX(MAKE_8_BITS(r_temp),MAKE_8_BITS(g_temp),
                               MAKE_8_BITS(b_temp));

         for (reserved_index = 0; reserved_index < num_reserved;
              reserved_index++)
         {
            index = RGB_TO_INDEX(*(reserved_red + reserved_index),
                                 *(reserved_green + reserved_index),
                                 *(reserved_blue + reserved_index));

            if (index == index2)
            {
               (void)memcpy((char *)(table + reserved_index),
                            (char *)(table + source),sizeof(*table));
            }
         }
      }
      else if ((table + source)->count != 0)
      {
         (void)memcpy((char *)(table + next_free_entry),
                      (char *)(table + source),sizeof(*table));
         next_free_entry++;
      }
   }

   /* Sort the non-reserved colors to determine the most popular.  */
   qsort((char *)(table + num_reserved),(next_free_entry - num_reserved),
         sizeof(*table),n_compare);

   /* Now merge infrequent colors with the most popular.  */
   /* Create a table for the squares of differences and assign a pointer
      to zero squared (midpoint).                                          */
   for (i = -256; i < 256; i++)
      square[i + 256] = (i * i);

   square_off = &square[256];

   /* Perform the merge.                                                   */
   for (j = 0; j < lutlen; j++)
      (table + j)->dn_1 = (unsigned char)j; /* Number each table entry.    */

   for (unclassed_index = lutlen; unclassed_index < next_free_entry; 
        unclassed_index++)
   {
      /* Calculate the best distance value for the target entry.            */
      pix_red   = (table + unclassed_index)->red;
      pix_green = (table + unclassed_index)->green;
      pix_blue  = (table + unclassed_index)->blue;

      red_diff    = (pix_red   - (table + lutlen - 1)->red);
      green_diff  = (pix_green - (table + lutlen - 1)->green);
      blue_diff   = (pix_blue  - (table + lutlen - 1)->blue);

      best_dist  = (*(square_off + red_diff) + *(square_off + green_diff) +
                    *(square_off + blue_diff));

      best_index = lutlen - 1;

      table_ptr = (table + lutlen - 2);

      /* Search the most popular colors in the color table for the closest
         match.                                                             */

      for (j = (lutlen - 2); j >= 0; j--)
      {
         red_diff   = (pix_red   - table_ptr->red);
         green_diff = (pix_green - table_ptr->green);
         blue_diff  = (pix_blue  - table_ptr->blue);

         dist = (*(square_off + red_diff) + *(square_off + green_diff) +
                 *(square_off + blue_diff));

         if (dist < best_dist)
         {
            best_dist = dist;
            best_index = j;
         }

         table_ptr--;
      }

      /* Assign the index of the closest matching color to the compress
         color entry.                                                       */
      (table + unclassed_index)->dn_1 = (unsigned char)best_index;

#ifdef DEBUG
      simulated += (table + unclassed_index)->count;
      total_error += (table + unclassed_index)->count *
                      sqrt((double)best_dist);
#endif
   }

#ifdef DEBUG
   printf("simulated: %d\n",simulated);
   printf("total_error: %g\n",total_error);
#endif

   /* Now create the output LUTs.  */
   for (i = 0; i < lutlen; i++)
   {
      red_lut[i]   = MAKE_8_BITS((table + i)->red);
      green_lut[i] = MAKE_8_BITS((table + i)->green);
      blue_lut[i]  = MAKE_8_BITS((table + i)->blue);
   }

   /* Create the composite image.  */
   /* Associate the index value with an output pixel (LUT entry) value.    */
   for (i = 0; i < next_free_entry; i++)
   {
      index = (table + i)->index;
      pixel = (table + i)->dn_1;
      (table + index)->dn_2 = (unsigned char)pixel;
   }

   /* Create the output composite values.                                  */
   for (i = 0; i < num_images; i++)
   {
      red_ptr       = (*(red + i));
      green_ptr     = (*(green + i));
      blue_ptr      = (*(blue + i));
      composite_ptr = (*(composite + i));

      for (pixel = 0; pixel < *(npts + i); pixel++)
      {
         index = RGB_TO_INDEX(*red_ptr,*green_ptr,*blue_ptr);
         *composite_ptr = (table + index)->dn_2;
         red_ptr++;
         green_ptr++;
         blue_ptr++;
         composite_ptr++;
       }
   }

   /* Free the color table.  */
   free((char *)table);

   return(E_SUCC);
}

/*****************************************************************************
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
int n_compare
(
   const void *entry1,  /* Input color table entries. (I)        */
   const void *entry2
)

/*****************************************************************************
 *                                                                           *
 *     Purpose:  This routine is the compare function used by the qsort      *
 *               function.  It operates as follows:                          *
 *                                                                           *
 *               If arg1 is less than arg2 then return 1                     *
 *               If arg1 is equal to arg2 then return 0                      *
 *               If arg1 is greater than arg2 then return -1                 *
 *                                                                           *
 *               This arrangement allows the array to be sorted in           *
 *               descending order with the lowest address containing the     *
 *               highest element.                                            *
 *                                                                           *
 *     Notes:    See documentation for the standard qsort function.          *
 *                                                                           *
 *****************************************************************************/
{
   long    return_value;   /* Return compare value.                          */
   struct ENTRY_DEF *new_entry1;
   struct ENTRY_DEF *new_entry2;

   new_entry1 = (struct ENTRY_DEF *) entry1;
   new_entry2 = (struct ENTRY_DEF *) entry2;

   return_value = (new_entry1->count < new_entry2->count) ? 1 :
                  (new_entry1->count == new_entry2->count) ? 0 : -1;

   return(return_value);
} 
