#include "gof.h"

#define MAXELEMENT      512             /* Initial maximum number of graphics 
                                           elements.                         */
#define USER_GRCOLORS   8               /* Maximum number of user specified
                                           graphics colors.                  */
#define USER_GRFONTS    9               /* Maximum number of user specified
                                           fonts.                            */
#define HANDLE          5               /* Width and height of selection 
                                           handles.                          */

#define NO_TYPE         -1              /* No graphics type has been selected*/
#define POINT           0               /* Point graphics data.              */
#define LINE            1               /* Line graphics data.               */
#define RECT            2               /* Rectangle (polygon) graphics data.*/
#define POLY            3               /* Polygon graphics data.            */
#define TEXT            4               /* Annotation/text graphics data.    */

#define GR_ACTIVE       0x1             /* Active flag.                      */
#define GR_READPORT     0x2             /* Portable GOF flag for reading.    */
#define GR_SAVEPORT     0x4             /* Portable GOF flag for saving.     */
#define GR_CREATE       0x8             /* Newly created graphics panel.     */
#define GR_SELECT       0x10            /* Select mode for graphics.         */
#define GR_LOAD         0x20            /* Load mode for graphics.           */
#define GR_LABEL        0x40            /* Flag indicating whether or not a
                                           label is currently being placed.  */
#define GR_CLOSE        0x80            /* Flag indicating whether or not the
                                           user has finished selecting the
                                           current graphics element.         */
#define GR_CANCEL       0x100           /* Flag indicating whether or not 
                                           last item was cancelled.          */
#define GR_EDIT         0x200           /* Edit mode for graphics.           */

#define GR_ADD          0x1             /* Within Edit mode -- Add mode.     */
#define GR_MOVE         0x2             /* Within Edit mode -- Move mode.    */
#define GR_DELETE       0x4             /* Within Edit mode -- Delete mode.  */
#define GR_EDITLABEL    0x8             /* Within Edit mode -- Change label. */
#define GR_MODIFY       0x10            /* Within Edit mode -- Indicates the
                                           current element has had modifica-
                                           tions made.                       */
#define GR_REMOVE       0x20            /* Within Edit mode -- Indicates the
                                           current element is to be deleted
                                           from the graphics file.           */

#define GR_ORIG         1               /* Flag indicating a copy of the
                                           original state of the current 
                                           element is to be made.            */
#define GR_UNDO         2               /* Flag indicating a copy of the
                                           current state of the current 
                                           element is to be made.            */

#define XID_COL         0               /* Index into xid_atnames array for
                                           color.                            */
#define XID_FONT        1               /* Index into xid_atnames array for
                                           font.                             */
#define XID_LW          2               /* Index into xid_atnames array for
                                           line width.                       */
#define XID_LABX        3               /* Index into xid_atnames array for
                                           X label location.                 */
#define XID_LABY        4               /* Index into xid_atnames array for
                                           Y label location.                 */

struct XID_GRAPHICS
{
 Pixmap         pixmap[USER_GRCOLORS];  /* Graphics pixmap.                  */
 Pixmap         full_pix[USER_GRCOLORS];/* Full resolution window graphics
                                           pixmap.                           */
 Pixmap         zoom_pix[USER_GRCOLORS];/* Zoomed window graphics pixmap.    */
 GC             gc;                     /* Graphics GC.                      */
 GC             rb_gc;                  /* Rubber-band line GC.              */
 Widget         window;                 /* Widget the rubber-band line is
                                           drawn in.                         */
 XFontStruct    *font_info[USER_GRFONTS];
                                        /* Font information for graphics 
                                           fonts.                            */
 int            fd;                     /* Current file descriptor.          */
 struct PT_LIST *first;
 struct PT_LIST *cur;
 int            selectx;                /* Last selected point's X coordinate*/
 int            selecty;                /* Last selected point's Y coordinate*/
 int            prevx;                  /* Previous cursor position (X).     */
 int            prevy;                  /* Previous cursor position (Y).     */
 unsigned long  rb_fore;                /* Foreground pixel value for rubber-
                                           band line.                        */
 long           type;                   /* Current type of graphics data.    */
 long           current;                /* Index of current graphics element.*/
 long           max_element;            /* Maximum graphics element number in
                                           currently allocated space.        */
 long           temp_num;               /* Number of points in temp. array.  */
 long           color;                  /* Current color index.              */
 long           font;                   /* Current font index.               */
 long           line_width;             /* Current line width.               */
 long           flags;                  /* Bit flags
                                           - Active flag.
                                           - Portable GOF flag.
                                           - Newly created graphics panel.
                                           - Current mode (select or load).
                                           - Label placement toggle.
                                           - Cancel element flag.            */
 char           lname_gof[CMLEN];
                                        /* LAS file name of GOF.             */

 char           colors[USER_GRCOLORS][81];
                                        /* User defined graphics colors.     */
 char           fonts[USER_GRFONTS][81];
                                        /* User defined fonts.               */
 char           attr_types[MAXATT][ATTYP];
                                        /* Attribute types of current element*/
 char           attr_values[MAXATT][ATTVAL];
                                        /* Temporary array of attribute 
                                           values.                           */
};

struct XID_LABEL
{
 int            x,y;                    /* X/Y coordinate of upper left corner
                                           of annotation text.               */
 int            width,height;           /* Width and height of rectangle 
                                           surrounding text.                 */
};

struct GRAPH
{
 long           type;                   /* Type of graphics element.         */
 long           offset;                 /* Offset to the record in the GOF.  */
 long           writeflg;               /* Indicates whether or not the record
                                           has been written to the GOF.      */
 long           color;                  /* Color index of graphics element.  */
 long           font;                   /* Font index of graphics element.   */
 float          x;                      /* X location of point, first vertice
                                           of line or polygon, or upper left
                                           corner of an annotation string.   */
 float          y;                      /* Y location of point, first vertice
                                           of line or polygon, or upper left
                                           corner of an annotation string.   */
 float          labelx;                 /* X location of graphics label.     */
 float          labely;                 /* Y location of graphics label.     */
 float          upleft[2];              /* XY location of upper left corner of
                                           the bounding rectangle.           */
 float          loright[2];             /* XY location of lower right corner of
                                           the bounding rectangle.           */
 char           label[KEYL];            /* Graphics label.                   */
 char           attributes[MAXATT][ATTVAL];
                                        /* Attribute values.                 */
 long           *graph_ptr;             /* Pointer to the structure containing
                                           the element specific information. */
};

struct LINE_ELEMENT
{
 long           num_vert;               /* Number of vertices.               */
 long           line_width;             /* Line width.                       */
 float          *x;                     /* X locations of the vertices of
                                           the line.                         */
 float          *y;                     /* Y locations of the vertices of
                                           the line.                         */
};

struct POLY_ELEMENT
{
 long           num_vert;               /* Number of vertices.               */
 long           line_width;             /* Line width.                       */
 long           fill;                   /* Flag indicating whether or not
                                           the polygon is to be filled.      */
 float          *x;                     /* X locations of the vertices of
                                           the polygon.                      */
 float          *y;                     /* Y locations of the vertices of
                                           the polygon.                      */
};

struct ANNOT_ELEMENT
{
 long           size;                   /* Character size.                   */
 float          angle;                  /* Annotation angle.                 */
 char           *string;                /* Annotation string.                */
};

struct PT_LIST
{
 struct PT_LIST *next;
 struct PT_LIST *prev;
 float          x;
 float          y;
};

struct XID_EDITGRAPH
{
 float          xfil;                   /* File X coordinate of current 
                                           vertice.                          */
 float          yfil;                   /* File Y coordinate of current
                                           vertice.                          */
 float          fxprev;                 /* File X coordinate of previous 
                                           vertice.                          */
 float          fyprev;                 /* File Y coordinate of previous
                                           vertice.                          */
 float          fxnext;                 /* File X coordinate of next 
                                           vertice.                          */
 float          fynext;                 /* File Y coordinate of next
                                           vertice.                          */
 float          ulx;                    /* File X coordinate of upper left
                                           corner of original rectangle.     */
 float          uly;                    /* File Y coordinate of upper left
                                           corner of original rectangle.     */
 int            current;                /* Index of current graphics element.*/
 int            delete_idx;             /* Index of graphics element that has
                                           been deleted.                     */
 int            vert;                   /* Index of current vertice of current
                                           graphics element.                 */
 int            prevx;                  /* Window X coordinate of previous
                                           vertice (move command).           */
 int            prevy;                  /* Window Y coordinate of previous
                                           vertice (move command).           */
 int            nextx;                  /* Window X coordinate of next
                                           vertice (move command).           */
 int            nexty;                  /* Window Y coordinate of next
                                           vertice (move command).           */
 int            newx;                   /* Window X coordinate of new
                                           vertice (move command).           */
 int            newy;                   /* Window Y coordinate of new
                                           vertice (move command).           */
 int            new_ulx;                /* New upper left window X coordinate*/
 int            new_uly;                /* New upper left window Y coordinate*/
 int            width;                  /* Width of new rectangle.           */
 int            height;                 /* Height of new rectangle.          */
 int            color;                  /* Color index of edit handles.      */
 int            writeflg;               /* Indicates whether or not the record 
                                           being modified has been written to
                                           the GOF.                          */
 int            flags;                  /* Bit flags
                                           - Edit mode.
                                           - Modified flag.                  */
};

#ifndef _xid_h
extern struct XID_GRAPHICS      xid_graphics;
extern struct XID_LABEL         xid_label;
extern struct GRAPH             *xid_element;
extern struct XID_EDITGRAPH     xid_editgraph;
extern struct GRAPH             xid_editundo;
extern struct GRAPH             xid_editorig;
#else
struct XID_GRAPHICS             xid_graphics;
struct XID_LABEL                xid_label;
struct GRAPH                    *xid_element;
struct XID_EDITGRAPH            xid_editgraph;
struct GRAPH                    xid_editundo;
struct GRAPH                    xid_editorig;
#endif

/* prototypes */

void setatt_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void setatt_enter
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void setatt_popup
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void defatt_popup
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_create
(
   Widget w,
   XtPointer client,
   XEvent *call,
   Boolean *cont
);

void graphics_deletelinerect
(
   Widget w,               /* Widget ID.  (I)                            */
   long win,               /* Type of window. (I)
                                0 - Main image window.
                                1 - Full resolution window.
                                2 - Zoomed window.                       */
   long plane_idx,         /* Index into memory plane structure of the 
                              currently viewed image.  (I)               */
   float x1fil,            /* X file coordinate of first endpoint.  (I)  */
   float y1fil,            /* Y file coordinate of first endpoint.  (I)  */
   float x2fil,            /* X file coordinate of second endpoint.  (I) */
   float y2fil,            /* Y file coordinate of second endpoint.  (I) */
   long flag               /* Flag indicating whether or not the temporary
                              point structure is to be freed.  (I)       */
);

long graphics_draw
(
   long win,                   /* Window to draw into. (I)
                                      -1 - All windows
                                       0 - Main image window
                                       1 - Full resolution window
                                       2 - Zoom window                       */
   long plane_idx,             /* Index into memory plane structure of the 
                                  currently viewed image.  (I)               */
   long element                /* Graphics element to draw.  (I)             */
);

long graphics_drawlinerect
(
   Widget w,                   /* Widget ID.  (I)                            */
   long win,                   /* Type of window. (I)
                                    0 - Main image window.
                                    1 - Full resolution window.
                                    2 - Zoomed window.                       */
   int event_x,                /* X coordinate of mouse. (I)                 */
   int event_y,                /* Y coordinate of mouse. (I)                 */
   long plane_idx,             /* Index into memory plane structure of the 
                                  currently viewed image.  (I)               */
   float xfil,                 /* X file coordinate. (I)                     */
   float yfil                  /* Y file coordinate. (I)                     */
);

long graphics_drawstring
(
   Widget w,                   /* Widget ID.  (I)                            */
   long win,                   /* Type of window. (I)
                                0 - Main image window.
                                1 - Full resolution window.
                                2 - Zoomed window.                           */
   int event_x,                /* X coordinate of mouse. (I)                 */
   int event_y,                /* Y coordinate of mouse. (I)                 */
   long plane_idx              /* Index into memory plane structure of the 
                                  currently viewed image.  (I)               */
);

void graphics_expose
(
   Widget w,
   XtPointer client,
   XEvent *call,
   Boolean *cont
);

void graphics_gofname
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_labelcalc
(
   char *string                /* Character string for which calculations are
                                  to be done.                                */
);

long graphics_gofread
(void);

long graphics_gofwrite
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_mode
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_refresh
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_select
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_setlabel
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_setcolor
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_setfont
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_setwidth
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_type
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_update
(
   long win,                   /* Window to draw into. (I)
                                       0 - Main image window
                                       1 - Full resolution window
                                       2 - Zoom window                       */
   long plane_idx              /* Index into memory plane structure of the 
                                  currently viewed image. (I)                */
);

void setatt_dismiss
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void defatt_dismiss
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void defatt_ok
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_gofport
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_loadportgof
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_saveportgof
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
);

long graphics_drawpoint
(
   Widget w,                   /* Widget ID.  (I)                            */
   long win,                   /* Type of window. (I)
                                    0 - Main image window.
                                    1 - Full resolution window.
                                    2 - Zoomed window.                       */
   int event_x,                /* X coordinate of mouse. (I)                 */
   int event_y,                /* Y coordinate of mouse. (I)                 */
   long plane_idx,             /* Index into memory plane structure of the 
                                  currently viewed image.  (I)               */
   float xfil,                 /* X file coordinate. (I)                     */
   float yfil                  /* Y file coordinate. (I)                     */
);

void graphics_deletepoint
(
   long win,                   /* Type of window. (I)
                                    0 - Main image window.
                                    1 - Full resolution window.
                                    2 - Zoomed window.                       */
   long element,               /* Index of point to erase. (I)               */
   long plane_idx              /* Index into memory plane structure of the 
                                  currently viewed image.  (I)               */
);

void graphics_deletestring
(
   long plane_idx,             /* Index into memory plane structure of the 
                                  currently viewed image.  (I)               */
   long element                /* Graphics element index of text string to
                                  delete.  (I)                               */
);

void graphics_deletelinepoly
(
   long win,                   /* Type of window. (I)
                                    0 - Main image window.
                                    1 - Full resolution window.
                                    2 - Zoomed window.                       */
   long element,               /* Index of point to erase. (I)               */
   long plane_idx              /* Index into memory plane structure of the 
                                  currently viewed image.  (I)               */
);

void graphics_expose_area
(
   long win,                   /* Type of window. (I)
                                   -1 - All displayed windows.
                                    0 - Main image window.
                                    1 - Full resolution window.
                                    2 - Zoomed window.                       */
   long element,               /* Index of graphics element defining the
                                  area to expose. (I)                        */
   long plane_idx              /* Index into memory plane structure of the 
                                  currently viewed image.  (I)               */
);

void graphics_setbounds
(
   long element                /* Graphics element index. (I)                */
);

void goflabel_popup
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void gofname_popup
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void goflabel_cancel
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void gofname_cancel
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_editmode
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_editfind
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_vertfind
(
   Widget w,
   XtPointer client,
   XtPointer call
);

int graphics_closest
(
   int plane_idx,              /* Index into memory plane structure of the
                                  currently viewed image.                    */
   int type,                   /* Type of graphics elements to search. (I)   */
   int tol,                    /* Tolerance allowed around specified point. 
                                  (I)                                        */
   float point[],              /* XY pair for point chosen. (I)              */
   int nreq,                   /* Number of elements to return. (I)          */
   int elements[],             /* Indices to graphics records found. (O)     */
   int *nfnd                   /* Number of records found. (O)               */
);

void graphics_handles
(
   int element,    /* Index to graphics record. (I)                          */
   int win,        /* Image window to draw handles in. (I)
                            -1 = All existing windows
                             0 = Main image window only
                             1 = Full resolution window only
                             2 = Zoomed window only                          */
   Drawable drawable,   /* Specifies the window or pixmap to draw handles in.
                         This must be specified if WIN is not -1.  If WIN is
                         -1 and drawable is NULL, handles will be drawn in
                         the appropriate window on the display. (I)          */
   int sl          /* Starting line of the drawable relative to the full
                         image. (I)                                          */
);

void edit_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_sethandle
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_editselect
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_updategof
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_editwarn
(
   long flag,                  /* Flag indicating whether to create or
                                  destroy the warning panel.  (I)
                                       0 - Create the panel.
                                       1 - Destroy the panel.                */
   char *client_data           /* Client data to be passed to the okCallback
                                  and the cancelCallback.  (I)
                                       "E" - Continue with edit_destroy.
                                       "G" - Continue with graphics_destroy. 
                                       "N" - Do nothing in addition to 
                                             creating panel.                 */
);

long graphics_copy_element
(
   struct GRAPH *src,   /* Source graphics strucutre. (I)             */
   struct GRAPH *dst    /* Destination graphics structure. (I)        */
);

void graphics_undo
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_deleteelement
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_editadd
(
   Widget w,
   XtPointer client,
   XtPointer call
);

void graphics_rubberband
(
   Widget w,                           /* Widget ID.  (I)                    */
   int xcursor,                        /* X cursor position.  (I)            */
   int ycursor,                        /* Y cursor position.  (I)            */
   int plane_idx                       /* Index to the currently viewed 
                                          image.                             */
);

int graphics_updateatt
(
   int element_idx                   /* Graphics element index attribute 
                                        values are to be read from.        */
);

int graphics_realloc
(void);

void graphics_port_filename
(
   const char *filename,             /* I: Original GOF filename */
   int gof_type,                     /* I: Original GOF type */
   char *port_filename               /* O: Portable GOF filename */
);

void graphics_trad_filename
(
   const char *filename,             /* I: Portable GOF filename */
   int gtype,                        /* I: Original GOF type */
   char *port_filename               /* O: Traditional GOF filename */
);

void gof_inv_coef
(
   const double *coef,    /* I: Forward transformation coefficients */
   double *inv_coef       /* O: Inverse transformation coefficients */
);
