/****************************************************************************
NAME:		GETPAR

PURPOSE:	Get user entered parameters from the TAE parameter block

*****************************************************************************/

#include <stdio.h>
#include "xrectify.h"
#include "ltable.h"
#include "util.h"
#include "labserv.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblk,	/* TAE variable block				   */
    struct FRAME *frame,	/* Output framing parameters for GRID subcommand   */
    struct PROJ *outp,	/* Output projection parameters 		   */
    char *pname,		/* Output print file name			   */
    char *siat_name,
    long *msgtim		/* Time between processing messages		   */
)
{
struct VARIABLE *var;		/* pointer to a TAE variable in vblk	   */

double ul_corner[2];		/* Upper left corner coordinate		   */
double lr_corner[2];		/* Lower right corner coordinate	   */

long	lscoors[2];		/* Line,sample coordinates of ul coord	   */
long	nl, ns;			/* The number of lines & samples in output */
long	unit_code = 0;	/* Unit code of corner coordinates entered */
long	out_def;		/* Default flags for projection parameters */
long	mode;			/* Framing mode				   */
long	access;			/* File access indicatori		   */
long	status;			/* Status variable			   */
long	zero = 0;		/* Constant of zero			   */
long 	term, ln_pr, file;	/* Print option flags			   */

char	units[12];		/* Units of corner coordinates entered	   */
char	filename[CMLEN];	/* Temporary filename			   */
char	proj_name[CMLEN];	/* Projection parameter file name	   */
char	host_img[CMLEN];	/* Host input image name		   */
char	outkey[10];		/* Output projection key		   */

/* Get the input image
  -------------------*/
access = IREAD;
c_getnam(vblk,"IN",&zero,host_img,"img",&access);
c_lsmknm(host_img,"cctx_siat",siat_name);

/* Get the name of the output geometric mapping grid file
  ------------------------------------------------------*/
access = IWRITE;
status = c_getnam(vblk, "OUTGRID", &zero, frame->name, "grid", &access);
if (status != E_SUCC)
    return(E_FAIL);

/* Get the name of the file containing projection information
  ----------------------------------------------------------*/
access = IREAD;
status = c_getnam(vblk, "INPROJ", &zero, proj_name, "proj", &access);
if (status != E_SUCC)
    return(E_FAIL);

/* Get the output projection information
  -------------------------------------*/
var = p_find(vblk, "OPROJKEY");
out_def = FALSE;
strcpy(outkey, SVAL(*var, 0));
status = get_proj_prm(proj_name, outkey, out_def, outp);
if (status != E_SUCC)
    return(E_FAIL);

/* Get the pixel size of the output
  --------------------------------*/
var = p_find(vblk, "PIXSIZ");
frame->o_dist[0] = frame->o_dist[1] = RVAL(*var, 0);

/* Starting line/sample output coordinates
  ---------------------------------------*/
var = p_find(vblk, "LSCOORS");
lscoors[0] = IVAL(*var,0);
lscoors[1] = IVAL(*var,1);

/* Retreive the number of lines and samples in the output area
  -----------------------------------------------------------*/
var = p_find(vblk, "NL");
nl = IVAL(*var,0);
var = p_find(vblk, "NS");
ns = IVAL(*var,0);

/* Get the units that the coordinate are in
  ----------------------------------------*/
var = p_find(vblk, "COORUNIT");
strcpy(units, SVAL(*var,0));
if(strcmp(units,"DMS") == 0)
    unit_code = 5;
if(strcmp(units,"DEG") == 0)
    unit_code = 4;
if(strcmp(units,"RAD") == 0)
    unit_code = 0;
if(strcmp(units,"SEC") == 0)
    unit_code = 3;

/* Get the output projection coordinates
  -------------------------------------*/
var = p_find(vblk, "COORS");
sscanf(SVAL(*var,0), "%lf", &(ul_corner[0]));
sscanf(SVAL(*var,1), "%lf", &(ul_corner[1]));
if ((*var).v_count == 2) mode = OUTPUT_LS;
else if (strcmp(units, "PRO") == 0) mode = OUTPUT_XY;
else mode = INPUT;
if ((mode == INPUT) || (mode == OUTPUT_XY))
    {
    sscanf(SVAL(*var,2), "%lf", &(lr_corner[0]));
    sscanf(SVAL(*var,3), "%lf", &(lr_corner[1]));
    }

/* Get the number of lines and samples desired in each grid cell in the
   output space.  These can have values or can be null.
  ---------------------------------------------------*/
var = p_find(vblk, "NLGRID");
frame->ngline = ((*var).v_count > 0) ? IVAL(*var, 0) : 0;

var = p_find(vblk, "NSGRID");
frame->ngsamp = ((*var).v_count > 0) ? IVAL(*var, 0) : 0;

/* Get the tolerance value for grid reduction
  ------------------------------------------*/
var = p_find(vblk, "TOLVAL");
frame->tol = RVAL(*var, 0);

/* Get the message timer
  ---------------------*/
var = p_find(vblk, "$msgtim");
*msgtim = IVAL(*var, 0);

/* Get the print option.  Make a file name (if neccessary) and open the
   temporary file
  --------------*/
strcpy(host_img,"xrectify");
c_setprt(vblk, &term, &file, &ln_pr, host_img, filename);
access = IUPDATE;
if (file == 1)
    c_mknam(filename, pname, "prt", &access);
else 
    strcpy(pname, " ");

/* Open the temp print file
  ------------------------*/
frame->fptr = fopen(host_img, "w");

fprintf(frame->fptr,"Geometric Mapping Grid name:  %s\n",  frame->name);

/* Compute transformation mapping from output image coordinates to output
   space projection coordinates.
  -----------------------------*/
status =out_frame(frame,outp,mode,ul_corner,lr_corner,lscoors,nl,ns,unit_code);
if (status != E_SUCC)
    return(E_FAIL); 

return(E_SUCC);
}

/****************************************************************************
NAME:		GET_PROJ_PRM

PURPOSE:	Retrieves the projection parameters from the projection
		parameter file

*****************************************************************************/
int get_proj_prm
(
    char *proj_name,	/* Projection parameter file name */
    char *outkey,		/* Output projection parameter key */
    long out_def,		/* Boolean flags--default given by user? */
    struct PROJ *outp	/* Output projection parameters */
)
{
struct TAB_DEF tab;	/* Tabular structure of label table */
struct VECTOR *keyptr;	/* Pointer to key field */
struct VECTOR *codeptr;	/* Pointer to projection code field */
struct VECTOR *zoneptr;	/* Pointer to zone code field */
struct VECTOR *unitptr;	/* Pointer to unit of measure field */
struct VECTOR *sphptr;	/* Pointer to datum (spheroid) code field */
struct VECTOR *parmptr;	/* Pointer to projection coefficients field */
long status;		/* Function return status */
long i;			/* Loop counter */

/* Open the labeled table file & retrieve the label
  ------------------------------------------------*/
open_tab(&tab, proj_name, IREAD);
get_vector(&tab);

/* Determine pointers to fields in the file
  ----------------------------------------*/
keyptr = get_field_ptr(&tab, "PROJKEY");
if (keyptr == NULL)
    return(E_FAIL);
codeptr = get_field_ptr(&tab, "PROJTYPE");
if (codeptr == NULL)
    return(E_FAIL);
zoneptr = get_field_ptr(&tab, "PROJZONE");
if (zoneptr == NULL)
    return(E_FAIL);
unitptr = get_field_ptr(&tab, "PROJUNITS");
if (unitptr == NULL)
    return(E_FAIL);
sphptr = get_field_ptr(&tab, "PROJSPH");
if (sphptr == NULL)
    return(E_FAIL);
parmptr = get_field_ptr(&tab, "PROJPARMS");
if (parmptr == NULL)
    return(E_FAIL);

/* Loop until key is found or EOF
 -------------------------------*/
for (;;)
   {
   status = get_record(&tab);
   if (status == E_FAIL) return(E_FAIL);
   if (status == E_EOF) break;
   if((strcmp(keyptr->ptr.c, outkey) == 0) || out_def) break;
   }

outp->proj_code = *codeptr->ptr.i4;
outp->zone_code = *zoneptr->ptr.i4;
outp->units = *unitptr->ptr.i4;
outp->datum_code = *sphptr->ptr.i4;
for (i = 0; i < 15; i++)
    outp->proj_coef[i] = parmptr->ptr.r8[i];

/* Close the file & return
  -----------------------*/
close_tab(&tab);

return(E_SUCC);
}
