/****************************************************************************
NAME:		MAKE_GRID	

PURPOSE:	Creates a Geometric Mapping Grid file mapping from one map
		projection to another.  Grid reduction techniques may be
		applied and a summary of gridding errors is calcualted and
		reported to the user.

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "xrectify.h"
#include "util.h"
#include "protos.h"
#define PIE 3.141592653589793238

int make_grid
(
    struct FRAME *frame,		/* Output framing parameters		    */
    struct PROJ *outp,		/* Output projection parameters		    */
    long *msgtim	/* Message timing parameter		    */
)
{
extern long curent;		/* Current "line" for processing messages   */
struct GEOGRID grid;		/* Geometric mapping grid		    */
double proj_y, proj_x;		/* Projection coordinates in Y & X	    */
double outy, outx;		/* Projection coords in Y & X (after Xform) */
double all_zero[15];		/* Zero projection parameters		    */
double	lat, lon;		/* Latitude/Longitude  of the output grid   */
double  in_line,		/* Line of the input grid location	    */
	in_samp,		/* sample of the input grid location	    */
	temp;			/* Temporary variable for remainder test    */
long	one = 1,		/* Constant of one			    */
	three = 3,		/* Constant of three			    */
	nonfatal = 1,		/* Nonfatal error code			    */
	geo = 0,		/* Output projection system flag	    */
	degrees = 4,		/* Output unit code			    */
	null_zone = 62;		/* Null UTM or State Plane zone		    */
long	*x,*y,			/* x & y values for polynomial evaluation   */
	nel,			/* Number of input grid coordinates	    */
	reduce,			/* Flag for grid reduction		    */
	cnt, count,		/* Loop counters			    */
	status = 0;  	/* Return status			    */
long	prtprm[2];	/* Printing code		            */		

/* If the user entered number of lines and samples is null (zero), set the
   dimensions of the grid to the maximum.  This grid will later be reduced
   according to the tolerance value as entered by the user.  If the user
   selected the number of lines and samples, the resulting grid will be of
   those dimensions; it will not be refined.
  -----------------------------------------*/
if ((frame->ngline == 0) || (frame->ngsamp == 0))
    {
    reduce = TRUE;
    grid.nrows = grid.ncols = MAX_GRID_SIZE;

    temp=(float)(frame->window[2] - frame->window[0]) / (float)(grid.nrows - 1);
    frame->ngline = (long)temp;
    if (frame->ngline != temp) frame->ngline++;

    temp=(float)(frame->window[3] - frame->window[1]) / (float)(grid.ncols - 1);
    frame->ngsamp = (long)temp;
    if (frame->ngsamp != temp) frame->ngsamp++;
    }
else
    {
    /* Determine the number of rows in the grid
       ----------------------------------------*/
    reduce = FALSE;
    temp = ((float)frame->window[2] / (float)frame->ngline) + 1;
    grid.nrows = (long)temp;
    if (temp != grid.nrows) grid.nrows++;

/* Check the number of rows in the grid isn't over the maximum
  -----------------------------------------------------------*/
    if (grid.nrows > MAX_GRID_SIZE)
	{
	grid.nrows = MAX_GRID_SIZE;
	temp=(float)(frame->window[2]-frame->window[0]) / (float)(grid.nrows-1);
	frame->ngline = (long)temp;
	if (frame->ngline != temp) frame->ngline++;
	c_errmsg("Number of grid rows exceeded maximum","xrectify-nrows",
		&nonfatal);
	c_errmsg("Number of grid rows set to maximum","xrectify-nrows",
		&nonfatal);
	c_errmsg("Nonfatal error encountered","xrectify-warn",&nonfatal);
	}

    /* Determine the number of columns in the grid
       -------------------------------------------*/
    temp = ((float)frame->window[3] / (float)frame->ngsamp) + 1;
    grid.ncols = (long)temp;
    if (temp != grid.ncols) grid.ncols++;

    /* Check the number of columns in the grid isn't over the maximum
       --------------------------------------------------------------*/
    if (grid.ncols > MAX_GRID_SIZE)
	{
	grid.ncols = MAX_GRID_SIZE;

	temp=(float)(frame->window[3]-frame->window[1]) / (float)(grid.ncols-1);
	frame->ngsamp = (long)temp;
	if (frame->ngsamp != temp)
	  frame->ngsamp++;

	c_errmsg("Number or grid columns exceeded maximum","xrectify-ncols",
		&nonfatal);
	c_errmsg("Number of grid columns set to maximum","xrectify-ncols",
		&nonfatal);
	c_errmsg("Nonfatal error encountered","xrectify-warn",&nonfatal);
	}
    }

/* Place the number of output lines and samples into the grid record
  -----------------------------------------------------------------*/
grid.lines   = (frame->window[0] + frame->window[2]) - 1;
grid.samples = (frame->window[1] + frame->window[3]) - 1;

/* Calculate the amount of buffer space needed for the output grid
   intersections and the input grid coordinate pairs, then allocate memory.
  ------------------------------------------------------------------------*/
nel = grid.nrows * grid.ncols;
grid.out_lines = (long *)malloc(grid.nrows * sizeof(long));
grid.out_samps = (long *)malloc(grid.ncols * sizeof(long));
grid.in_lines  = (double *)malloc(nel * sizeof(double));
grid.in_samps  = (double *)malloc(nel * sizeof(double));

if ((grid.out_lines == NULL) || (grid.out_samps == NULL) ||
    (grid.in_lines == NULL)  || (grid.in_samps == NULL))
	return(NO_SPACE);

/* Calculate the output grid intersections and place in geometric mapping
   grid record.
  ------------*/
for (count = 0; count < grid.nrows; count++)
   grid.out_lines[count] = frame->window[0] + (count * frame->ngline);

for (count = 0; count < grid.ncols; count++)
   grid.out_samps[count] = frame->window[1] + (count * frame->ngsamp);

/* Compute input grid coordinates
  ------------------------------*/
for (count = 0; count < 15; count++)
    all_zero[count] = 0.0;

/* Start the processing message
  ----------------------------*/
curent = 1;
c_inittm("XRECTIFY","Generated","grid element","","",&nel,&one,msgtim,&three);
c_startm();

/* Calculate the grid locations
  ----------------------------*/
for (y = grid.out_lines, count = grid.nrows; count--; y++)
    {
    proj_y = frame->corners[0] - ((*y - 1) * frame->o_dist[0]);
    for (x = grid.out_samps, cnt = grid.ncols; cnt--; x++, curent++)
	{
	proj_x = frame->corners[1] + ((*x - 1) * frame->o_dist[1]);

/* Calculate the output space latitude/longitude
  ---------------------------------------------*/
        c_transinit (&outp->proj_code, &outp->units, &outp->zone_code,
	             &outp->datum_code, outp->proj_coef, &geo, &degrees,
		     &null_zone, &outp->datum_code, all_zero, prtprm, 
                     "\0"); 
   	status = c_trans(&outp->proj_code, &outp->units, 
			&geo, &degrees, 
			&proj_x, &proj_y, &outx, &outy);
	if (status != E_SUCC)
	    return(NO_RECTIFY);

/* Convert the latitude/longitude to radians
  -----------------------------------------*/
	lat = outy * PIE/180.0;
	lon = (outx< 0.0) ? (360.0+outx)*PIE/180.0 : outx*PIE/180.0;

/* Get the line/sample location for the specified latitude/longitude
  -----------------------------------------------------------------*/
	inv_mss_model(lat,lon,0.0,&in_line,&in_samp);

/* Place the line/sample location in the grid structure
  ----------------------------------------------------*/
	grid.in_lines[curent-1] = in_line;
	grid.in_samps[curent-1] = in_samp;
	}
    }

/* Stop the processing message
  ---------------------------*/
c_stoptm();

/* Pack projection related fields in the grid file
  -----------------------------------------------*/
grid.proj_valid = GEO_VALID;
grid.code = outp->proj_code;
grid.zone = outp->zone_code;
grid.datum = outp->datum_code;

for (count = 0; count < 8; count++)
   grid.corners[count] = frame->corners[count];

grid.pdist[0] = frame->o_dist[0];
grid.pdist[1] = frame->o_dist[1];

for (count = 0; count < 15; count++)
   grid.projprms[count] = outp->proj_coef[count];

switch (outp->units)
    {
    case 0:
	strcpy(grid.units,"radians");
	break;
    case 1:
	strcpy(grid.units,"feet");
	break;
    case 2:
    	strcpy(grid.units, "meters");
	break;
    case 3:
    	strcpy(grid.units, "seconds");
	break;
    case 4:
    	strcpy(grid.units, "degrees");
	break;
    case 5:
    	strcpy(grid.units, "dms");
	break;
    }

/* Pack modeling related fields in the grid file
  ---------------------------------------------*/
strcpy(grid.model, "SPACECRAFT");
grid.ncoeff = 0;
grid.coeffs = NULL;

/* Reduce grid, if necessary
  -------------------------*/
if (reduce)
    c_grdred(&grid, &(frame->tol));

/* Pack statistics related fields in the grid file 
  -----------------------------------------------*/
grid.fiterr_valid = GEO_NOTVALID;
grid.max_fit_err[0] = 0;
grid.max_fit_err[1] = 0;
grid.ave_fit_err[0] = 0;
grid.ave_fit_err[1] = 0;
grid.rms_fit_err = 0;
grid.griderr_valid = GEO_NOTVALID;
grid.max_grid_err[0] = 0;
grid.max_grid_err[1] = 0;
grid.ave_grid_err[0] = 0;
grid.ave_grid_err[1] = 0;
grid.rms_grid_err = 0;

fprintf(frame->fptr,
    "Geometric Mapping Grid file created with %d rows by %d columns\n\n",
     grid.nrows, grid.ncols);

/* Write the geometric mapping grid to disk
  ----------------------------------------*/
c_putgrd(frame->name, &grid);
if (status != E_SUCC)
    return(E_FAIL);

/* Close the temp print file
  -------------------------*/
fclose(frame->fptr);

/* Free the space allocated for the grid lines
  -------------------------------------------*/
free(grid.out_lines);
free(grid.out_samps);
free(grid.in_lines);
free(grid.in_samps);

return(E_SUCC);
}
