/***************************************************************************
NAME:		OUT_FRAME

PURPOSE:	Calculates a set of coefficients mapping from output space
		image coordinates to output space projection coordinates

*****************************************************************************/
#include <stdio.h>
#include "xrectify.h"
#include <math.h>
#include "protos.h"

int out_frame
(
    struct FRAME *frame,		/* Framing parameters */
    struct PROJ *outp,		/* Output projection parameters */
    long mode,			/* Type of framing */
    double *ul,			/* Upper left corner coordinates */
    double *lr,			/* Lower right corner coordinates */
    long *ls,			/* Line/sample coordinates of ul */
    long nl,			/* Number of lines in output space */
    long ns,			/* Number of samples in output space */
    long units 			/* Units code of coordinates entered */
)
{
double pxmax, pxmin;		/* Maximum & minimum X projection coordinates */
double pymax, pymin;		/* Maximum & minimum Y projection coordinates */
double temp_mod;		/* Results from DMOD operation... */

if (mode == INPUT)
   {

/* Determine output projection space minimums and maximums
  -------------------------------------------------------*/
   if(c_mapedg(&(ul[1]), &(lr[1]), &(ul[0]), &(lr[0]), &units, 
               &(outp->proj_code), &(outp->zone_code), outp->proj_coef, 
               &(outp->units), &pxmin, &pxmax, &pymin, &pymax, 
               &(outp->datum_code)) != E_SUCC) 
			return(E_FAIL);

    /* Adjust to even pixel increment
      ------------------------------*/
    temp_mod = DMOD(pymax, frame->o_dist[0]);
    if (temp_mod != 0.0) pymax += (frame->o_dist[0] - temp_mod);
    pymin -= DMOD(pymin, frame->o_dist[0]);
    pxmin -= DMOD(pxmin, frame->o_dist[1]);
    temp_mod = DMOD(pxmax, frame->o_dist[1]);
    if (temp_mod != 0.0) pxmax += (frame->o_dist[1] - temp_mod);
    }

/* If output frame was given in projection coordinates, for two points:
  --------------------------------------------------------------------*/
else if (mode == OUTPUT_XY)
    {
    pymax = ul[0] + ((ls[0] - 1) * frame->o_dist[0]);
    pxmin = ul[1] - ((ls[1] - 1) * frame->o_dist[1]);
    pymin = lr[0];
    pxmax = lr[1];
    }

/* If output frame was given in projection coordinates, 1 pt & nl, ns:
  -------------------------------------------------------------------*/
else if (mode == OUTPUT_LS)
    {
    pymax = ul[0] + ((ls[0] - 1) * frame->o_dist[0]);
    pxmin = ul[1] - ((ls[1] - 1) * frame->o_dist[1]);
    pymin = pymax - ((nl - 1) * frame->o_dist[0]);
    pxmax = pxmin + ((ns - 1) * frame->o_dist[1]);
    }

/* Report to the user the projection minimums and maximums of the frame
  --------------------------------------------------------------------*/
fprintf(frame->fptr, "   Min/max Y projection coordinates:  %f %f\n"
	, pymin, pymax);
fprintf(frame->fptr, "   Min/max X projection coordinates:  %f %f\n"
   	, pxmin, pxmax);

/* Report to the user the pixel size in line & sample directions
  -------------------------------------------------------------*/
fprintf(frame->fptr,"   Pixel size in line direction:    %f\n", 
		frame->o_dist[0]);
fprintf(frame->fptr,"   Pixel size in sample direction:  %f\n",
		frame->o_dist[1]);

/* Report to the user the number of lines and samples in the output space
  ----------------------------------------------------------------------*/
frame->window[0] = frame->window[1] = 1;
frame->window[2] = (long)((pymax - pymin) / frame->o_dist[0] + 1.5);
frame->window[3] = (long)((pxmax - pxmin) / frame->o_dist[1] + 1.5);
fprintf(frame->fptr,"   Number of lines in output space:    %d\n",
	frame->window[2]);
fprintf(frame->fptr,"   Number of samples in output space:  %d\n\n",
	frame->window[3]);

/* Pack the corner projection coordinates in the frame structure
  -------------------------------------------------------------*/
frame->corners[1] = frame->corners[3] = pxmin;
frame->corners[0] = frame->corners[4] = pymax;
frame->corners[6] = frame->corners[2] = pymin;
frame->corners[5] = frame->corners[7] = pxmax;

return(E_SUCC);
}
