/****************************************************************************
NAME:			      XUTM

PURPOSE:  Creates a Geometric Mapping Grid file mapping from systematicly
          corrected CCT-X data to the UTM map projection.  

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		REASON	
-------		----	------		------
  1.0		12/92	D. Steinwand
  2.0           3/93    D. Etrheim      Release to baseline.  This routine
                                does not conform to all LAS standards.  The
                                main effort was put into creating the standard
                                LAS interface, User Guide, and error
                                checking/trapping.  The supports can be looked
                                at later after the C version of GCTP is
                                finialized.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:                        LAS

ALGORITHM REFERENCES:           none
*****************************************************************************/
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "siat.h"
#include "cproj.h"
#include "util.h"
#include "header.h"
#include "proj.h"
#include "protos.h"

extern long curent;		/* Current "line" element for processing mess */
FILE *fp;                       /* File pointer for output print file */

int main
(void)
{
struct GEOGRID grid;		/* Geometric mapping grid */
double *iline_ptr;		/* Pointer to input space grid coordinate */
double *isamp_ptr;		/* Pointer to input space grid coordinate */
double proj_y, proj_x;		/* Projection coordinates in Y & X */
long msgtim;			/* Message timing parameter */
long *xptr,*yptr;		/* x & y values for polynomial evaluation */
long one = 1;			/* Constant of one */
long three = 3;			/* Constant of three */
long fail = -1;			/* Fatal error flag for c_errmsg */
long j, i;		        /* Loop counters */
long nel;			/* Number of input grid coordinates */
double xval,yval;
double x,y;
double a,b,c;
long nlines,nsamps;
long linc,sinc;
long zone;
double ulxp,ulyp,urxp,uryp,llxp,llyp,lrxp,lryp;
double ul_line, ur_line, lr_line, ll_line;
double ul_samp, ur_samp, lr_samp, ll_samp;
double lat,lon;
double A[12]; 			/* Matrix A (used for LSQ fit) */
double line,samp;
double v[4]; 			/* Work buffer for LSQ fit */
double outbx[4]; 		/* Vector containing X coordinates--output */
double outby[4]; 		/* Vector containing Y coordinates--output */
double t1,t2,alpha,beta;
double ap,bp;
double nsampsp;
char header_file[CMLEN];
char gridname[CMLEN];
struct SIAT siat;

struct PARBLK vblk;             /* TAE parameter block */
struct VARIABLE *var;           /* Pointer to a TAE variable in vblk */
long std_out;                   /* Standard output */
long access=IREAD;
long zero=0;
double scale_fact;
char lasname[2][CMLEN];
double r_major,r_minor;
long print[3];
char tmpfil[1][CMLEN];
char prtfile[CMLEN];            /* TAE name of output print file */
long numout=1;

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the landsat header file name
----------------------------------*/
if (c_getnam(&vblk,"INFILE",&zero,header_file,"cctx_siat",&access) != E_SUCC)
   c_errmsg("Fatal error encountered","xutm-fatal",&fail);

/* Get the output grid file name
--------------------------------*/
access = IWRITE;
if (c_getnam(&vblk,"OUTGRID",&zero,gridname,"grid",&access) != E_SUCC)
   c_errmsg("Fatal error encountered","xutm-fatal",&fail);

/* Get the print option
-----------------------*/
strcpy(tmpfil[0],"xutm");
if (c_setprt(&vblk,&print[0],&print[1],&print[3],tmpfil[0],prtfile) != E_SUCC)
   c_errmsg("Fatal error encountered","xutm-fatal",&fail);

/* Get time between processing message 
  -------------------------------------*/
var = p_find(&vblk, "$msgtim");
msgtim = IVAL(*var, 0);

/* Open the temporary print file
--------------------------------*/
fp = fopen(tmpfil[0],"w");
if (fp == NULL)
   {
   c_errmsg("Error opening temporary file","xutm-open",&one);
   c_errmsg("Fatal error encountered","xutm-fatal",&fail);
   }

/* Initialize the model
   --------------------*/
init_mss_model(header_file,0.0,0.0);

/* Read the Landsat Header file
  ----------------------------*/
if (c_read_siat(&siat,header_file) != E_SUCC)
    c_clean(&vblk,"xutm",tmpfil,&one);

/* Initalize projection routines
  -----------------------------*/
zone=calc_utm_zone(siat.mss_longitude[5]*R2D);
if (siat.mss_latitude[5] < 0) zone *= -1;
r_major = 6378206.4;
r_minor = 6356583.8;
scale_fact = .996;
utmforint(r_major,r_minor,scale_fact,zone);
utminvint(r_major,r_minor,scale_fact,zone);

/* Calculate the corner coordinates of the input image.
  ----------------------------------------------------*/
utmfor(siat.mss_longitude[5],siat.mss_latitude[5],&x,&y);
fprintf(fp,"\nCenter UTM Coordinates:  %f %f meters\n",y,x);

fclose(fp);

ulxp = x - 50000.0;
ulyp = y + 50000.0;
utminv(ulxp,ulyp,&lon,&lat);
inv_mss_model(lat,lon,0.0,&ul_line,&ul_samp);

urxp = x + 50000.0;
uryp = y + 50000.0;
utminv(urxp,uryp,&lon,&lat);
inv_mss_model(lat,lon,0.0,&ur_line,&ur_samp);

llxp = x - 50000.0;
llyp = y - 50000.0;
utminv(llxp,llyp,&lon,&lat);
inv_mss_model(lat,lon,0.0,&ll_line,&ll_samp);

lrxp = x + 50000.0;
lryp = y - 50000.0;
utminv(lrxp,lryp,&lon,&lat);
inv_mss_model(lat,lon,0.0,&lr_line,&lr_samp);

/* Calculate first order polynomial coefficients which map output line,sample
   coordinates to output projection coordinates
  --------------------------------------------*/
for(i=0;i<4;i++) A[i] = 1.0;
A[4]=ul_samp;
A[5]=ur_samp;
A[6]=lr_samp;
A[7]=ll_samp;
A[8]=ul_line;
A[9]=ur_line;
A[10]=lr_line;
A[11]=ll_line;
outbx[0]= ulxp;
outbx[1]= urxp;
outbx[2]= lrxp;
outbx[3]= llxp;
outby[0]= ulyp;
outby[1]= uryp;
outby[2]= lryp;
outby[3]= llyp;

QR(A, 4, 3, v);
QR_solve(A, 4, 3, v, outbx, 0);
QR_solve(A, 4, 3, v, outby, 0);

/* Calculate the UTM coordinates of the input image
  -----------------------------------------------*/
xval=1.0; yval=1.0;
ulxp=c_eval(&one,outbx,&xval,&yval);
ulyp=c_eval(&one,outby,&xval,&yval);
xval=3240.0; yval=1.0;
urxp=c_eval(&one,outbx,&xval,&yval);
uryp=c_eval(&one,outby,&xval,&yval);
xval=3240.0; yval=2340.0;
lrxp=c_eval(&one,outbx,&xval,&yval);
lryp=c_eval(&one,outby,&xval,&yval);
xval=1.0; yval=2340.0;
llxp=c_eval(&one,outbx,&xval,&yval);
llyp=c_eval(&one,outby,&xval,&yval);

/* Calculate output space relationships between projection coordinates
   and image coordinates.
  ----------------------*/
t1=urxp-llxp;
t2=uryp-llyp;
c=sqrt(t1*t1+t2*t2);
t1=urxp-lrxp;
t2=uryp-lryp;
bp=sqrt(t1*t1+t2*t2);
t1=lrxp-llxp;
t2=lryp-llyp;
ap=sqrt(t1*t1+t2*t2);
nsampsp=ap/57.0 + 1.0;
beta=acos(-(bp*bp-ap*ap-c*c)/(2.0*c*ap));
b=c*sin(beta);
nlines=(long) (b/57.0 + 1.0);
alpha=HALF_PI - beta;
a=c*sin(alpha);
nsamps=(long) (a/57.0 + 1.0);

for(i=0;i<4;i++) A[i] = 1.0;
A[4]=nsamps-nsampsp;
A[5]=nsamps;
A[6]=nsampsp;
A[7]=1.0;
A[8]=1.0;
A[9]=1.0;
A[10]=nlines;
A[11]=nlines;
outbx[0]= ulxp;
outbx[1]= urxp;
outbx[2]= lrxp;
outbx[3]= llxp;
outby[0]= ulyp;
outby[1]= uryp;
outby[2]= lryp;
outby[3]= llyp;

QR(A, 4, 3, v);
QR_solve(A, 4, 3, v, outbx, 0);
QR_solve(A, 4, 3, v, outby, 0);

/* Calculate DDR corners and put in grid structure
  -----------------------------------------------*/
xval=1.0; yval=1.0;
grid.corners[1]=c_eval(&one,outbx,&xval,&yval);
grid.corners[0]=c_eval(&one,outby,&xval,&yval);
xval=nsamps; yval=1.0;
grid.corners[5]=c_eval(&one,outbx,&xval,&yval);
grid.corners[4]=c_eval(&one,outby,&xval,&yval);
xval=nsamps; yval=nlines;
grid.corners[7]=c_eval(&one,outbx,&xval,&yval);
grid.corners[6]=c_eval(&one,outby,&xval,&yval);
xval=1.0; yval=nlines;
grid.corners[3]=c_eval(&one,outbx,&xval,&yval);
grid.corners[2]=c_eval(&one,outby,&xval,&yval);
 
/* Place the number of output lines and samples into the grid record
 ------------------------------------------------------------------*/
grid.nrows = grid.ncols = 30;
grid.lines = nlines;
grid.samples = nsamps;

/* Calculate the amount of buffer space needed for the output grid
   intersections and the input grid coordinate pairs, then allocate memory.
 ------------------------------------------------------------------------*/
nel = 900;
grid.out_lines = (long *)malloc(30 * sizeof(long));
grid.out_samps = (long *)malloc(30 * sizeof(long));
grid.in_lines = (double *)malloc(900 * sizeof(double));
grid.in_samps = (double *)malloc(900 * sizeof(double));
if((grid.out_lines == NULL) || (grid.out_samps == NULL) ||
    (grid.in_lines == NULL) || (grid.in_samps == NULL)) 
       {
       c_errmsg("Error allocating dynamic memory","xutm-alloc",&one);
       c_clean(&vblk,"xutm",tmpfil,&one);
       }

/* Calculate the output grid intersections and place in geometric mapping
   grid record.
 -------------*/
linc=nlines/29 + 1;
sinc=(nsamps)/29 + 1;
for (i = 0; i < 30; i++) grid.out_lines[i] = 1 + (i * linc);
for (i = 0; i < 30; i++) grid.out_samps[i] = 1 + (i * sinc);

/* Compute input grid coordinates
  ------------------------------*/
curent = 1;
c_inittm("XUTM","Generated","grid element","","",&nel,&one,&msgtim,&three);
c_startm();
iline_ptr = grid.in_lines;
isamp_ptr = grid.in_samps;

for (yptr = grid.out_lines, i= grid.nrows; i--; yptr++)
  for (xptr = grid.out_samps, j = grid.ncols; j--; xptr++, curent++)
    {
    xval=(double)(*xptr);
    yval=(double)(*yptr);
    proj_y = c_eval(&one,outby,&xval,&yval);
    proj_x = c_eval(&one,outbx,&xval,&yval);
    utminv(proj_x,proj_y,&lon,&lat);
    inv_mss_model(lat,lon,0.0,&line,&samp);
    *iline_ptr++ = line;
    *isamp_ptr++ = samp;
    }

c_stoptm();

/* Pack projection related fields in the grid file
  -----------------------------------------------*/
grid.proj_valid = GEO_VALID;
grid.code = 1;
grid.zone = zone;
grid.datum = 0;
grid.pdist[0] = grid.pdist[1] = 57.0;
for (i = 0; i < 15; i++) grid.projprms[i] = 0.0;
strcpy(grid.units, "meters");

/* Pack modeling related fields in the grid file
  ---------------------------------------------*/
strcpy(grid.model, "XUTM");
grid.ncoeff = 0;
grid.coeffs = NULL;

/* Pack statistics related fields in the grid file 
  -----------------------------------------------*/
grid.fiterr_valid = GEO_NOTVALID;
grid.max_fit_err[0] = grid.max_fit_err[1] = 0;
grid.ave_fit_err[0] = grid.ave_fit_err[1] = 0;
grid.rms_fit_err = 0;
grid.griderr_valid = GEO_NOTVALID;
grid.max_grid_err[0] = grid.max_grid_err[1] = 0;
grid.ave_grid_err[0] = grid.ave_grid_err[1] = 0;
grid.rms_grid_err = 0;

/* Write the geometric mapping grid to disk
  ----------------------------------------*/
if(c_putgrd(gridname, &grid) != E_SUCC)
   c_clean(&vblk,"xutm",tmpfil,&one);

strcpy(lasname[0], gridname);
if (print[1])
   {
   strcpy(lasname[1],prtfile);
   numout = 2;
   }
c_complt("XUTM",lasname,&numout);

return EXIT_SUCCESS;

}
