#include "zip.h"

void makelut
(
    short lut[],               /* I: look up table data */
    float from[50],
    float to[50],              /* I: mapping pairs */
    long npairs,               /* I/O: number of pairs in partial lut */
    long type                  /* I: indicates data type of lut */
)
/*
------------------------------------------------------------------------
NAME: MAKELUT

PROGRAM HISTORY:
      Version               date             Author
      -------               ----             ------
         3                 july-88          B. Allam
         2                 feb-87           R. Mckinney
         1                 june-84          Y. Sheu

FUNCTION:
        produces a piece-wise linear look up table by completing 
        a partial look up table provided by the user.  the gaps 
        in the partial look up table are filled by linear interpolation.
        (the partial look up table is specified by the user through
        the tae parameters from and to which contain the "from" and "to"
        coordinates respectively.) from and to each may specify up to 35 
        values. 
        
-----------------------------------------------------------------------
*/
{
long	partlut[102][2];    /* receives partial lut */
long	curr;          /* points to current position in partlut during interp */
long	lutmax, lutmin;     /* highest and lowest lut positions */
float	temp;               /* temporarary variable */
long	i, j;                  /* counter variables */
short   itemp;

/*
   set max and min positions according to data type
*/
if (type == 1)
   {
   lutmax = 255;
   lutmin = 0;
   }
else
   {
   lutmax =  32767;
   lutmin = -32768;      
   } /* if */

/*
   make sure that from & to values are not outside of range

*/
if (from[0]  >  (float)lutmax) 
  zip_err("!FIRST FROM VALUE LARGER THAN MAXIMUM PIXEL VALUE", "zip-from");
if (from[npairs-1]  <  (float)lutmin)
  zip_err("!LAST FROM VALUE SMALLER THAN MINIMUM PIXEL VALUE", "zip-from");


/*
   make sure that the from extends to beginning of range but not beyond
-----------------------------------------------------------------------
*/
if (from[0]  <  (float)lutmin) 
   {
   zip_err("!FIRST FROM VALUE SMALLER THAN MINIMUM PIXEL VALUE", "zip-from");
   }  /* if */

/*
   make sure that the from value lut extends to end of range but not beyond

*/
if (from[npairs-1]  >  (float)lutmax) 
   {
   zip_err("!LAST FROM VALUE LARGER THAN MAXIMUM PIXEL VALUE", "zip-from");
   }  /* if */

/*
   move mapping pairs into processing array...
   -------------------------------------------
*/
for (i=0; i < npairs; i++)
   {
   partlut[i][0] = (long)from[i];
   partlut[i][1] = (long)to[i];
   }  /* do */
/*
   
*/
if (partlut[0][0] < lutmin) 
   {
   for (i = npairs-1; i >= 0; i--)
      {
      for (j = 0; j <= 1; j++)
         {
         partlut[i+1][j] = partlut[i][j];
         } /* do */
      } /* do */
   partlut[0][0] = lutmin;
   partlut[0][1] = partlut[1][1];
   npairs = npairs + 1;
   }  /* if */

if (partlut[npairs-1][0] < lutmax) 
   {
   npairs = npairs + 1;
   partlut[npairs-1][0] = lutmax;
   partlut[npairs-1][1] = partlut[npairs-2][1];
   }  /* if */


/*
   construct lut by linear interpolation between entries in partial lut
*/
curr = 0;   /* keeps track of lower pair to be used in interpolation */
for (i = lutmin; i <= lutmax; i++)
   {
   if (partlut[curr+1][0]  <  i )    /* need to interp with next pair */
      {
      curr = curr + 1;
      } /* if */
   if (partlut[curr+1][0]  ==  partlut[curr][0])
     temp=(float)partlut[curr][1];
   else
     {
      temp = ( ( (float)(i - partlut[curr][0]) / 
             (float)(partlut[curr+1][0] - partlut[curr][0]) ) *
             (float)(partlut[curr+1][1] - partlut[curr][1]) + 
             (float)partlut[curr][1]);
      if ((temp+0.5) >= (int)temp+1) itemp = (int)temp+1;
      else itemp = (int)temp;
     }
   lut[i] = (short)(min( max(itemp, lutmin), lutmax));

   } /* do */


}
