/*******************************************************************************
NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the ZOOM function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
J.HAGEDORN      Nov. 1983       initial LAS programmer
B.KOVALICK      Mar. 1986       last LAS 4.0 programmer
R.MCKINNEY      May  1988       converted to LAS 5.0
R.MCKINNEY	Dec. 1988	Updated to close all files

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	       LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   The global parameters are first received.
   Next the input and output images are gotten.
   Various validity checks are then made before returning.
*******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC *fdesc1[MAXIMG+1];	
                                 /* image file descriptors used for image I/O */
extern struct FDESC *fdesc2[2];		/* output image file descriptor		     */
extern struct GDESC *gdesc1;	/* group descriptor used for image I/O       */
extern struct GDESC *gdesc2;	/* output group descriptor for image I/O     */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands  */
extern long offset2[MAXBND];    /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

void getpar
(
    struct PARBLK *vblock,      /* pointer to TAE block	 	      */
    char taein[][CMLEN],	/* TAE file names of input images	      */
    char hosin[][CMLEN],  	/* host file names of input images	      */
    char taeout[][CMLEN],		/* TAE file name of output image	      */
    char hosout[][CMLEN],		/* host file name of output image	      */
    long *nmrimg, 		 /* number of input images		      */
    long (*bands)[MAXBND+1],         /* band specification 		              */
    long *nbands,		 /* number of bands per input image           */
    long (*window)[4],	         /* window specification		      */
    long *msgtim,		 /* elapsed time between display messages     */
    char *minmax,		 /* min max flag buffer		              */
    char *subnam,		 /* subcommand specified		      */
    long *xfact,		 /* factor to enlarge or reduce the x-axis    */
    long *yfact,		 /* factor to enlarge or reduce the y-axis    */
    long *nol,			 /* number of image output lines              */
    long *nos,			 /* number of image output samples	      */
    long *odtype,		 /* data type of input image		      */
    long *totbnd		 /* total number of bands		      */
)
{
struct  VARIABLE  *val;	      	 /* pointer to TAE variable within v-block    */

long     access;		 /* type of access for file(write,read,update)*/
long     band;			 /* band index				      */
long     dtype;			 /* data type of previous input image         */
long     element = 0;		 /* parameter element number		      */
long     fatal = -1;		 /* fatal error flag			      */
long     i;			 /* counter-no. of input images	 	      */
long     imglim;		 /* max number of images to be processed      */
long     nl;			 /* temporary no. of lines in input image     */
long     nonfatal = 1;		 /* value of one			      */
long     ns;			 /* temporary no. of samples in input image   */
long     qflag[MAXIMG][2];  	 /* flag indicating if subwindows specified   */
long     same;		    	 /* flag indicating if images have same window*/
long     sl;			 /* starting line of input image              */
long     ss;			 /* starting sample of input image            */
long     status;	   	 /* function return status code		      */
long     zero = 0;		 /* 0					      */
long     opt[2];		 /* options for opening files		      */
 
float    xf;			 /* temporary varialbe for xfact              */
float    yf;			 /* temporary variable for yfact	      */
float    temp;			 /* temporary variable for output size	      */

/*
GETPAR retreives the required TAE parameters from the TAE parameter block.

The TAE local parameters used by the program are:
    1.  In--the input TAE image name
    2.  Out--the output TAE image name
    3.  Scalfact--the factor to enlarge or reduce the x and y axes

The TAE global parameters used by the program are:
    1.  Histry--a flag specifying if the history file should be updated or not
    2.  Minmax--a flag specifying if the minmax file should be updated or not
    3.  Msgtim--the amount of time between display messages

This subroutine then figures the center of the image and checks to verify that
the window lies within the image.  Finally, it calculates the size of the output
image.
*/

/* Retrieve the TAE global variables from the parameter block and store them
   in their respective program variables. 
-----------------------------------------------------------------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

/* Build the string descriptor for TAE variable "IN".
   Call getin to parse the input image specification.
   Getin returns the TAE image name(s), host image name(s),
   bands to be processed, window specification, and number
   of images parsed.
----------------------------------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","zoom-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","zoom-window",
		&nonfatal);
   c_errmsg("Fatal error encountered","zoom-fatal",&fatal);
   }

/* Retrieve the TAE output image name from the parameter
   block and store it in "taeout". 
   Create a host file name for the output image and store 
   it in "hosout".
-----------------------------------------------------------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","zoom-fatal",&fatal);

/* get XFACT and YFACT 
-------------------------------*/
val = p_find (vblock,"scalfact");
      *yfact = IVAL(*val,0);
      *xfact = IVAL(*val,1);

/* get the subcommand 
---------------------*/
val = p_find(vblock,"_subcmd");
strcpy(subnam,SVAL(*val,0));

access = IREAD;
opt[0] = opt[1] = 0;
nl = 0;
ns = 0;
sl = 1;
ss = 1;
dtype = 0;
	
for (band = 0,i = 0; i < *nmrimg; i++)
    {
    status = c_eopens(&fdesc1[i],hosin[i],bands[i],&sl,&ss,&nl,&ns,
       &offset1[band],&access,&dtype,&zero,opt);
    if (status != E_SUCC)
	 {
	 c_errmsg("Error opening input file",
               "zoom-open", &nonfatal);
	 c_errmsg("Fatal error encountered",
               "zoom-fatal", &fatal);
	 }
    band += nbands[i];
    status = c_eclose(&fdesc1[i]);
    if (status != E_SUCC)
	   c_errmsg("Nonfatal error encountered--Image data saved",
	       "zoom-warn", &nonfatal);
        
    if (i == 0)
       *odtype = dtype;
    else if (dtype != *odtype)
	  {
	  c_errmsg("Input images are not of same data type",
		      "zoom-input" ,&nonfatal);
  	  c_errmsg("Fatal error encountered",
		      "zoom-fatal" ,&fatal);
          }	


    }

if (strcmp(subnam,"EXPAND") == 0)
   {
   *nol = window[0][2] * *yfact;
   *nos = window[0][3] * *xfact;
   }
else
   {
   if (*yfact != 1)
      {
      yf = (float) *yfact;
      temp = (((float) window[0][2]) / yf) + .998;
      *nol = (long) temp;
      }
   else
      *nol = window[0][2];

   if (*xfact != 1)
      {
      xf = (float) *xfact;
      temp = (((float)window[0][3]) / xf) + .998;
      *nos = (long) temp;
      }
   else
      *nos = window[0][3];
   }

if ((*nol <= 0) || (*nos <= 0))
   {
   c_errmsg("Output image size is invalid","zoom-sizerr",&nonfatal);
   c_errmsg("Fatal error encountered","zoom-fatal",&fatal);
   }

return;
}

