/******************************************************************************
NAME:   		      ZMEXPAND 

PURPOSE     To reduce an image by the specified x and y factors. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   J. Hagedorn 	   3/82 	  CSC		Original Development
 
  4.0	   B. Kovalick 	   3/86		  SAR	        Inserted the timing 
							message. 	

  5.0	   R. McKinney     6/88		  EDC		Add UPDDR routine
							Take descriptors out of 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  5.0	   R. McKinney    12/88		 EDC		Updated to close all
							files.
  5.1	   B. Ailts	  09/89          EDC	        Now updates the DDR
							correctly

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   ZOOM must be run under TAE.

PROJECT			LAS

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   The data is converted upon input to the data type of the output image
   The respective lines of each band in each image are processed at
   the same time with the output image being created with a specified
   x and y factor. 
   If $MINMAX was set to "YES", save the minmax values.
   Update the output ddr.
   Close the input and output image files.

ALGORITHM REFERENCES		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void zmexpand
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*window)[4],		/* window specification of input images      */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long totbnd,			/* total number of bands to be processed     */
    long odtype,			/* data type of output image                 */
    long xfact,			/* expansion factor for the x-axis	     */
    long yfact,			/* expansion factor for the y-axis	     */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax,		/* min max flag                              */
    long nol,			/* number of image output lines              */
    long nos			/* number of image output samples            */
)

{

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
extern struct FDESC *fdesc1[MAXIMG+1];
                                 /* image file descriptors used for image I/O */
extern struct FDESC *fdesc2[2];         /* image file descriptor		     */
extern struct GDESC *gdesc1;	/* group descriptor used for image I/O       */
extern struct GDESC *gdesc2;	/* group descriptor used for image output    */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands  */
extern long offset2[MAXBND];    /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long    bufsiz2;		/* size of output image buffer		     */
long    fatal = -1;		/* fatal error flag			     */
long	image;			/* loop index counting input images          */
long    insamp;			/* input sample				     */
long    jj;			/* loop counter				     */
long    kk;			/* loop counter				     */
long	size = 0;   /* number of bytes per pixel 	 	     */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long    outpix;			/* output pixel				     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */
long    opt[2];			/* options for opening files		     */

unsigned char *thebuf1;		/* image input buffer                        */
unsigned char *thebuf2;		/* image output buffer			     */

/* Open input image(s)
----------------------*/
access = IREAD;
opt[0] = opt[1] = 0;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens(&fdesc1[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","zoom-fatal",&fatal);
   band += nbands[image];
   }
fdesc1[nmrimg] = 0;
/* Open output image
--------------------*/
access = IWRITE;	
status = c_eopens(&fdesc2[0],hosout[0],&totbnd,&one,&one,&nol,&nos,
         offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
   c_clean(vblock,"zoom",hosout,&one);
fdesc2[1] = 0;

/* Allocate I/O buffer
----------------------*/
if ((thebuf1 = (unsigned char *)c_ealloc(fdesc1,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","zoom-alloc",&nonfatal);
   if (c_eclose(&fdesc2[0]) != E_SUCC)
      c_errmsg("Nonfatal error encountered","zoom-alloc",&nonfatal);
   c_clean(vblock,"zoom",hosout,&one);
   }

if ((thebuf2 = (unsigned char *)c_ealloc(fdesc2,&bufsiz2)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","zoom-alloc",&nonfatal);
   if (c_eclose(&fdesc2[0]) != E_SUCC)
      c_errmsg("Nonfatal error encountered","zoom-alloc",&nonfatal);
   c_clean(vblock,"zoom",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc1,&gdesc1,thebuf1,&bufsiz) != E_SUCC)
   {
   if (c_eclose(&fdesc2[0]) != E_SUCC)
      c_errmsg("Nonfatal error encountered","zoom-alloc",&nonfatal);
   c_clean(vblock,"zoom",hosout,&one);
   }

if (c_egroup(fdesc2,&gdesc2,thebuf2,&bufsiz2) != E_SUCC)
   {
   if (c_eclose(&fdesc2[0]) != E_SUCC)
      c_errmsg("Nonfatal error encountered","zoom-alloc",&nonfatal);
   c_clean(vblock,"zoom",hosout,&one);
   }

/* Setup for data type
----------------------*/
switch(odtype)
   {
   case EBYTE:  size = 1; break;
   case EWORD:  size = 2; break;
   case ELONG:	size = 4; break;
   case EREAL:  size = 4; break;
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("ZOOM","Processed","line","bands"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&totbnd,minval,maxval);

/* Main processing loop
-----------------------*/
for (curent = 1;curent < nl+1;curent++)
   {
   if (c_estep(&gdesc1) != E_SUCC)
        {
        if (c_egclse(&gdesc2) != E_SUCC)
           c_errmsg("Nonfatal error encountered","zoom-warn",&one);
        c_clean(vblock,"zoom",hosout,&one);
        }
   for (jj = 0; jj < yfact; jj++)
      {
	if (c_estep(&gdesc2) != E_SUCC)
        {
        if (c_egclse(&gdesc2) != E_SUCC)
           c_errmsg("Nonfatal error encountered","zoom-warn",&one);
        c_clean(vblock,"zoom",hosout,&one);
        }
        for (band=0;band<totbnd;band++)
           {
	   outpix = 0;
	   for (insamp = 0; insamp < ns; insamp++)
	    {
	     for (kk = 0; kk < xfact; kk++)
	      {
		switch (odtype)
	         {
		 case 1:
         	 *(thebuf2 + (*(offset2+band) + outpix)) =
	         *(thebuf1 + (*(offset1+band) + insamp));
	         outpix ++;
		 break;
		 case 2:
         	 (*(short *)(thebuf2 + (*(offset2+band) + outpix) * size)) =
	         (*(short *)(thebuf1 + (*(offset1+band) + insamp) * size));
	         outpix ++;
		 break;
		 case 3:
         	 (*(long *)(thebuf2 + (*(offset2+band) + outpix) * size)) =
	         (*(long *)(thebuf1 + (*(offset1+band) + insamp) * size));
	         outpix ++;
		 break;
		 case 4:
         	 (*(float *)(thebuf2 + (*(offset2+band) + outpix) * size)) =
	         (*(float *)(thebuf1 + (*(offset1+band) + insamp) * size));
	         outpix ++;
		 break;
		 }
	      }
	    }
      if (strcmp(minmax,"YES") == 0)
         status = c_pxmnmx(thebuf2 + (*(offset2+band) * size),&outpix,
                         &minval[band],&maxval[band],&odtype); 
           }
      }
  }
/* Final write
--------------*/
if (c_estep(&gdesc2) != E_SUCC)
   {
   if (c_egclse(&gdesc2) != E_SUCC)
      c_errmsg("Nonfatal error encountered","zoom-warn",&one);
   c_clean(vblock,"zoom",hosout,&one);
   }

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double) one / yfact;
samp_inc = (double) one / xfact;
upddr_flag = NOCOMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
      c_clean(vblock,"zoom",hosout,&one);

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","zoom-warn",&one);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc1) != E_SUCC)
   c_errmsg("Nonfatal error encountered","zoom-warn",&one);

if (c_egclse(&gdesc2) != E_SUCC)
   c_errmsg("Nonfatal error encountered","zoom-warn",&one);

/* Free buffer
--------------*/
free(thebuf1);
free(thebuf2);

return;
}
