/****************************************************************************** 
NAME:	ZOOM 

PURPOSE:	 			
To zoom an image, i.e. to read an input image and
produce an output image of a different scale.

EXTERNAL ASSOCIATIONS:	
ZOOM is a module of the LAS system.

FILE REFERENCES:	none

EXTERNAL VARIABLE REFERENCES:      
Name		Type	I/O	Description
----		----	---	-----------
ALLCAT		long	 I	type of display message
BSIZE		long	 I	size of the storage buffers
CM_SUC          long	 I      success indicator for catalog manager
EBYTE		long	 I	unsigned byte data type 
ELONG		long	 I	integer*4 data type
EREAL		long	 I	real data type
EWORD		long	 I	integer*2 data type
E_SUCC          long	 I      success indicator for image I/O
IREAD		long	 I	image I/O read access
IWRITE	  	long	 I	image I/O write access    
MAXBND          long	 I      maximum number of bands that can be processed
MAXIMG		long	 I	maximum number of images that can be processed
CMLEN           long	 I      maximum length of a file name
P_ABORT	  	long	 I	status code to abort if TAE encounters error 

EXTERNAL VARIABLES:	commented in source code when declared

DATA STRUCTURES:			
struct DESC
	is a string descriptor definition.  Needed to call FORTRAN
	routines with character arguments.
struct PARBLK
	is a structure consisting of the TAE parameter block.
struct VARIABLE
	is a structure consisting of information about a variable within
	the TAE parameter block.

SUBPROGRAMS CALLED:
 
Name		Description
----		-----------
getpar	        gets the parameters from the user
zmexpand	expands the image from the user
zmreduce	reduces the image from the user

(TAE routines)
p_find		find a TAE variable within the parameter block
  
(Image I/O Routines)
c_egclse	close all files in a LAS image file group
c_egroup	group several LAS image files
c_eopens	open a LAS image file for sequential access
c_estep		sequentially steps through LAS image file groups

(LAS Utilities)
c_errmsg        build necessary string descriptors and call "errmsg"
		to write an informational or error message to the user
c_complt      	write a completion message to the terminal
c_getin		parse the input image specification
c_hstout	create a history record
c_inittm        output message to terminal after specified amount 
                of time 
c_inlas		initialize the TAE routines
c_stoptm        output final display message and terminate the
		the alarm interrupt
c_startm	starts the output message to the terminal
c_svmnmx	update min max file associated with the image
  
(Pixel Manipulation Routines)
c_pxmnmx        determine the minimum and maximum pixel
		values of an array	

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS:	
1.  [zoom-histry]  Error creating histry file--image data saved.
	The histry file could not be created.  The image data is probably
	valid, and therefore saved.

2.  [zoom-minmax]  Error creating minmax file-image data saved
	The minmax file could not be created.  The image data is probably
	valid, and therefore saved.

4.  [zoom-warn]  Nonfatal error encountered--Image data saved 
	A nonfatal error was encountered during processing.  The image
	data is probably valid, and therefore saved.

FATAL ERROR CONDITIONS:
1.  [zoom-badwin]  Specified center point results in an invalid window-fatal.
	The window calculated from the specified center point is invalid.
	Respecify center point.

2.  [zoom-band]  Too many bands specified in input image(s)-fatal
	The total number of bands specified in the input image(s) is greater 
	than MAXBND(the maximum number of bands that can be processed).

3.  [zoom-fatal]  Fatal error encountered--No output created
	A fatal error was encountered during processing.  The output image
	file is deleted, and processing is terminated.

4.  [zoom-input]  Input images are not of the same data type-fatal
	The input images are not of the same data type.  Use 'copy' to 
	change the data types so they are the same.

5.  [zoom-open]  Eror opening <> file.
	The <> file could not be opened and processing is terminated.
	check to see if the <> file exists.

6.  [zoom-sizerr] Output image size is invalid-fatal
	The calculated size of the output image is less than one.  Do not reduce
	the input image by such a large factor or increase the size of the 
	input image.

7.  [zoom-window]  Input images do not have the same window size-fatal
	The input images do not have the same window size.  Correct the input
	images window size.
   
PROGRAM HISTORY:
Version		Date	Author		Code/Conr	Request
-------		----	------		--------	-------
  1.0		1/86	K. Zanter+	   EDC     	CIPS conversion to UNIX
		3/86	B. Ailts
  5.0		6/88    R. McKinney	   EDC          Conversion to las 5.0
  5.1          12/88    R. McKinney        EDC          Updated to close all       							files.

  5.2		3/88	D. Hollaren	   EDC		Corrected an errmsg 
  5.3           9/89    B. Ailts	   EDC		Now updates the DDR
						        correctly
  5.4          11/89    B. Ailts	   EDC		Now correctly calculates							nl when reducing
  6.0           5/94    D. Bridges         EDC          Added options for 
							opening images
  
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        ZOOM must be run under TAE.

PROJECT:	LAS		

LANGUAGE:	C

ALGORITHM (FLOW):
The program begins by initializing TAE.
Then user-specified parameters and the LAS global variables are retrieved.
The user-specified parameters consist of:
    1.  In--the input TAE image name
    2.  Out--the output TAE image name
    3.  Sf1--the factor to enlarge or reduce the y-axis
    4.  Sf2--the factor to enlarge or reduce the x-axis

The appropriate processing subroutine is then called to expand or reduce
the input image.  The output data type will be the same as the input data type.
The input and output images are closed, the output image is cataloged, and the
min/max and history files are created/updated to represent the output image.
Finally, a completion message is written to the terminal.

ALGORITHM REFERENCES:	none

*******************************************************************************/


#include <stdio.h>  	/* standard 'C' program include files		     */
#include <stdlib.h>
#include "las.h"        /* include imageio, histio, labserv                  */
#include "util.h"
#include "protos.h"


/* The following variables must be declared global for image I/O routines    */

struct FDESC *fdesc1[MAXIMG+1];	 /* image file descriptors used for image I/O */
struct FDESC *fdesc2[2];		 /* output image file descriptors	     */
struct GDESC *gdesc1;		     /* group descriptor used for image I/O       */
struct GDESC *gdesc2;		     /* output group descriptor for image I/O     */
long offset1[MAXIMG*MAXBND];    /* offsets into "thebuf" for input bands     */
long offset2[MAXBND];           /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;	        /* current processing band		     */

int main
(void)
{
struct  PARBLK    vblock;           /* TAE parameter block                    */

static long bands[MAXIMG][MAXBND+1];/* band specification		      */
long     msgtim;		    /* elapsed time between display messages  */
long     nbands[MAXIMG];	    /* number of bands per input image        */
long     nol;			    /* number of image output lines           */
long     nos;			    /* number of image output samples         */
long     nmrimg;		    /* number of input images		      */
long     one;	 	            /* value of one			      */
long     odtype;		    /* data type of output image	      */
long	 std_out;		    /* standard output			      */
long	 totbnd;		    /* total number of bands		      */
long     window[MAXIMG][4];         /* window specification		      */
long     xfact;			    /* factor to enlarge or reduce the x-axis */
long     yfact;			    /* factor to enlarge or reduce the y-axis */

char    hosin[MAXIMG][CMLEN];       /* buffer storage for "hosin"	      */
char    hosout[1][CMLEN];	            /* buffer storage for "hosout"	      */
char    minmax[4];		    /* min max flag			      */
char    subnam[7];		    /* subcommand specified		      */
char    taein[MAXIMG][CMLEN];       /* buffer storage for "taein"	      */
char    taeout[1][CMLEN];	            /* buffer storage for "taeout"	      */

/*
The main program acts as a control routine.  It first initializes TAE 
and then calls 'getpar' to input the TAE parameters 
(including the TAE global variables).  The input and output files are opened 
and, depending upon the subcommand, the image is either expanded or reduced.
Finally, the history file is updated and the completion message is sent to
the terminal.
*/

/* initialize TAE 
-----------------*/
c_inlas (&vblock,&std_out);
one = 1;

/* get user specified parameters and global variable values from TAE and
checks window validity 
------------------------------------------------------------------------*/
getpar (&vblock,taein,hosin,taeout,hosout,&nmrimg,bands,nbands,window,
        &msgtim,minmax,subnam,&xfact,&yfact,&nol,&nos,&odtype,
	&totbnd);

/* determines the appropriate processing subroutine by subcommand
-----------------------------------------------------------------*/
if (strcmp(subnam,"EXPAND") == 0)
{
	zmexpand(&vblock,hosin,hosout,nmrimg,window,bands,nbands,
		totbnd,odtype,xfact,yfact,msgtim,minmax,nol,nos);
}
else
{
	zmreduce(&vblock,hosin,hosout,nmrimg,window,bands,nbands,
		totbnd,odtype,xfact,yfact,msgtim,minmax,nol,nos);
}

/* Update history file
----------------------*/
if (c_hstout(&vblock,hosout[0],hosin,&nmrimg," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","zoom-warn",&one);

/* Output completion message
----------------------------*/
c_complt("ZOOM",taeout,&one);

return EXIT_SUCCESS;

}

