/******************************************************************************
 
FUNCTION:	COPY

PURPOSE:	Part of the DITTO subsystem.  The copy function copies files
		from the current position of the input device to the output
		device.  If the output device is tape, an EOV (a double tape
		mark) is written after the copy has been completed and then
		back spaced one tape to insure an EOV exists at the end of
		tape.  If the output device is a disk file, a new file is
		created.  

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      12/85    K. Gacke   		initial development
  2.0       7/86    K. Gacke		LAS standards
  3.0       3/87    K. Gacke		fixed bug in reblocking routine of 
					variable length records to fix length
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1	    2/92    T. Mittan		Allowed 2 eofs to be written on tape
					close for vms.  This allows vms to 
					find the eov.  Corrected copy-reblock
					so that large block sizes can be 
					written.  Finally, added subroutine
					pits, which allows operator interv-
					vention if an I/O error occurs.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	The input parameters are retrieved from the TAE parameter block.  It
	is determined what subcommand is being executed.  The appropriate
	subroutine is called to perform the copy as specified by the subcommand
	(RECORD, FILE, REBLOCK, or MODREC).  The copy function copies files 
	from the current position of the DITTO input device to the DITTO 
	output device.  If the output device is tape, an EOV is written and 
	then backspaced one tape mark to insure an EOV exists at the end of 
	tape.  If the output device is a disk file, a new file is created.  

	The following transformations may be performed:
	    astoeb	convert ascii to ebcidic
	    ebtoas	convert ebcidic to ascii
	    lowtoup	convert lower case characters to upper case
	    uptolow	convert upper case characters to lower case
	    swap	byte swap the data
	    hion	convert to byte data with high bit on (PRIMOS ASCII)
	    hioff	convert to byte data with high bit off (nonPRIMOS ASCII)

	The algoritm for each subcommand is as follows:
	    RECORD
		Copy a specified number of records of a single file from the
		input device to the output device.  The record and block sizes
		of the output device should be equal to the record and block
		sizes of the input device.  

	    FILE
		Copy a specified number of files from the input device to the
		output device.  The record and block sizes of the output device
		should be equal to the record and block sizes of the input 
		device.

	    REBLOCK
		Copy and reblock a specified number of records of a single file
		from the input device to the output device.  It uses the record
		and block size as specified for the input/output DITTO device.

	    MODREC
		Copy a specified number of records of a single file from the
		input device to the output device allowing the user to 
		dynamically modify specified records.  The record and block
		sizes of the output device should be equal to the record and
		block sizes of the input device.

ALGORITHM REFERENCES
	This function allows three types of copies:
	1) FILE: 
		Copy a specified number of files from input device to output
	   	device.  The output file has the same record and block size
		as the input file. 

	2) RECORD:
		Copy a specified number of records of a single file from the
		input device to output device.  The output file has the same
		record and block size as the input file.

	3) REBLOCK:
		Copy and reblock a specified number of records of a single file
		from the input device to the output device.  It uses the record
		and block sizes as specified for the input/output DITTO devices.

	4) MODREC:
		Copy a specified number of records of a single file from the
		input device to the output device allowing the user to modify
		selected records.  It is required that this form of a copy to
		be run in interactive mode because the program dynamically
		queries the user to edit the record.

	Translations allowed by the copy function are:
		astoeb -- convert ascii to ebcidic
		ebtoas -- convert ebcidic to ascii
		low2up -- convert lower case characters to upper case
		up2low -- convert upper case characters to lower case
		swap   -- perform byte swapping
	        hion	convert to byte data with high bit on (PRIMOS ASCII)
	        hioff	convert to byte data with high bit off (nonPRIMOS ASCII)

	NOTE:  The translation is performed immediately after the record is
	read from the input device.  

*******************************************************************************/
#include <stdio.h> 			/* C standard I/O include file       */
#include <string.h>
#include <stdlib.h>
#include "las.h"	 		/* LAS include file 		     */
#include "util.h"
#include "ditto.h" 			/* ditto include file                */
#include "copy.h"  
#include "protos.h"
 
struct PARBLK   vblock;		/* TAE parameter block               */
struct DITTO_DEV ditto_device;	/* ditto I/O devices descriptors     */
struct DISK_FD idisk;		/* input disk file descriptor        */
struct DISK_FD odisk;		/* output disk file descriptor       */
struct TAPE_FD itape;		/* input tape descriptor             */
struct TAPE_FD otape;		/* output tape descriptor            */

int main
(void)
{
extern struct PARBLK vblock;		/* TAE parameter block		     */

struct VARIABLE *val;			/* pointer to TAE variable           */

long	std_out;			/* Fortran LUN for standard output   */
long    zero = 0;			/* value of zero                     */
long    outcnt = 0;			/* Number of output files	     */

char    out[1][CMLEN];			/* output files for completion msg   */
char    idevice[20];			/* input DITTO device (tape or disk) */
char    odevice[20];			/* output DITTO device (tape or disk)*/
char	*subcmd;			/* subcommand being executed         */
char	fname[CMLEN];			/* Host file name		     */
char	ext[CMLEN];			/* Extention location		     */
char	*adir;				/* address of directory within fname */
char	*aext;				/* address of directory within fname */

/*
  Initalize LAS
*/
c_inlas(&vblock,&std_out);

/*
  Set input and output devices to no values
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
  Retrieve the input device type
*/
val = p_find(&vblock,"idevice");
if (val->v_count == 0)
   {
   ditto_error("INPUT\0",DITTO_ALLOC,ERROR_WARN,NO_UPDATE_FD);
   c_clean(&vblock,"ditto copy",out,&zero);
   }

strcpy(idevice,SVAL(*val,0));	

/*
  Retrieve the output device
*/
val = p_find(&vblock,"odevice");
if (val->v_count == 0)
   {
   ditto_error("OUTPUT\0",DITTO_ALLOC,ERROR_WARN,NO_UPDATE_FD);
   c_clean(&vblock,"ditto copy",out,&zero);
   }

strcpy(odevice,SVAL(*val,0));

/*
  Retrieve the subcommand
*/
val = p_find(&vblock,"_SUBCMD");
subcmd = SVAL(*val,0);

if (strcmp(subcmd,"FILE") == 0)			/* FILE copy                 */
   file_copy(idevice,odevice);
else if (strcmp(subcmd,"RECORD") == 0)		/* RECORD copy               */
   record_copy(idevice,odevice);
else if (strcmp(subcmd,"REBLOCK") == 0)		/* REBLOCK copy              */
   reblock_copy(idevice,odevice);
else if (strcmp(subcmd,"MODREC") == 0)		/* MODREC copy               */
   modrec_copy(idevice,odevice);
else if (strcmp(subcmd,"PITS") == 0)		/* PITS copy               */
   pits_copy(idevice,odevice);

/*
  Output the name of the file that was created
*/
if (strcmp(odevice,"DISK") == 0)
    {
    strcpy(fname,odisk.fname);

    if (odisk.cur_file != NOVAL)
	{
	adir = strrchr(fname,HOST_DIR_END);
	if (adir == 0)
	    adir = fname;
	aext = strchr(adir,HOST_EXT);

	if (aext == 0)
	    sprintf(fname,"%s_%d",odisk.fname,odisk.cur_file-1);
	else
	    {
	    strcpy(ext,aext);
	    sprintf(aext,"_%d%s",odisk.cur_file-1,ext);
	    }
	}

    strcpy(out[0], fname);
    outcnt = 1;
    }

c_complt("DITTO COPY",out,&outcnt);

return EXIT_SUCCESS;

}
