/******************************************************************************
FUNCTION:	COPY_PROC

PURPOSE:	Copies a specified number of records from the current
		position of the input device to the output device.  The
		record and block sizes of both devices should be equal.
		Transformations are performed immediately after the input
		data has been read.

ALGORITHM 
  Allocate enough buffer space to read one input physical record as specified
  by the block size.
  If some sort of transformation is to be performed, "trans_flag" is set to
  true.
  Initialize the display message utility
  Copy the specified number of records from the input device to the output
  device.  For each record the following is done:
	read one physical record from input device
	if at end of file break out of loop
	if trans_flag is true call the transformation routine
	write one physical record to the output device
  Disable the display msg utility 
  Free the buffer space
  Update tape attributes
  If the output device is tape then
	write End of File marks

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h> 			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "copy.h"  
#include "protos.h"

#define currec   curent			/* curent is used for display msg.   */

int copy_proc
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev,	/* output DITTO device               */
    long nrecords,			/* number of records to be copied    */
    char *trans,			/* type of transformations           */
    long swap,				/* byte swapping flag                */
    long msgtim			/* # of seconds between display msg  */
)
{
extern long currec;			/* current record for display msg    */
extern struct TAPE_FD otape;		/* output tape descriptor            */

long dummy = -2;		/* dummy variable for display msg    */
long flag = GEN1;		/* flag indicating type display msg  */
long mode = HANDLE;		/* tape I/O package writes error msg */
long nbyte;			/* number of bytes read              */
long one = 1;			/* value of one                      */
long fatal = -1;		/* Fatal error code		     */

long   status;		/* function return status code       */
long   trans_flag = 0;	/* transformation flag               */
unsigned char *bufptr;	/* pointer to I/O buffer             */

/*
  Allocate enough buffer space to read one input physical record as specified
  by the block size.
*/
bufptr = (unsigned char *) malloc(indev->blksiz);
if (bufptr == 0)
   {
   ditto_error(" ",DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }

/*
  If some sort of transformation is to be performed, "trans_flag" is set to
  true.
*/
if ((swap > 0) || (strcmp(trans,"NONE") != 0))
   trans_flag = 1;

/*
   Initialize the display message utility
*/
currec = 0;
c_inittm("DITTO COPY ","Processed","record #"," "," ",&dummy,&dummy,&msgtim,
	&flag);
c_startm();

/*
  Copy the specified number of records from the input device to the output
  device.
*/
for (currec=0; currec<nrecords; currec++,indev->cur_rec++,outdev->cur_rec++)
   {
   /*
     Read one physical record from input device
   */
   nbyte = read_device(indev,outdev,bufptr);
   if (nbyte <= 0)				/* if at end of file break out*/
	break;

   /*
     if trans_flag is true call the transformation routine
   */
   if (trans_flag)
	transform(bufptr,&nbyte,&swap,trans);

   /*
     write one physical record to the output device
   */
   write_device(indev,outdev,bufptr,nbyte);
   }

/*
  Disable processing message
*/
c_stoptm();

/*
  Free any buffer space allocated
*/
free(bufptr);

/*
  Update attributes
*/
if (nbyte <= 0)
   indev->cur_file++;
outdev->cur_file++;

if ((nrecords != ALL) && (nbyte <= 0))
   ditto_error(" ",DITTO_EOF,ERROR_WARN,UPDATE_FD);

/*
  If the output device is tape write End of File marks
*/
#ifndef vms
if (outdev->device == TAPE)
   {
   status = c_tpweof(&otape.unit,&one,&mode);
   if (status != TAPE_SUCC)
	ditto_error("OUTPUT",DITTO_WEOF,ERROR_WARN,UPDATE_FD);
   }
#endif

return(nbyte);
}
