/******************************************************************************
FUNCTION:	FILE_COPY

PURPOSE:	Copies the specified number of files from the input device
		to the output device.  The record and block sizes of the
		output device should be equal to the input device.  The
		record types of both the input and output devices should
		also be the same.

ALGORITHM 
    Retrieve the input parameters
    Reopen the input tape device or get the disk file descriptor
    Reopen the output tape device or get the disk file descriptor

    Copy file(s) from the input device to the output device by calling routine
    "copy_proc" for each input file.  Note that all records are copied.
	Open the input and output devices
	Copy the input device records to the output device
	Close the input and output devices
	Update the input and output attributes

    Output the number of files copied
    Close the input tape device or disk file
    Close the output tape device or disk file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>  			/* C standard I/O include file       */
#include "las.h"    	/* LAS include file 		     */
#include "tapeio.h" 			/* tape I/O include file 	     */
#include "ditto.h"  			/* ditto include file                */
#include "copy.h"   
#include "protos.h"

void file_copy
(
    char *idevice,			/* input DITTO device (tape or disk) */
    char *odevice			/* output DITTO device (tape or disk)*/
)
{
extern struct PARBLK vblock;		/* TAE parameter block               */
extern struct DISK_FD idisk;		/* input disk descriptor             */
extern struct DISK_FD odisk;		/* output disk descriptor            */
extern struct TAPE_FD itape;		/* input tape descriptor             */
extern struct TAPE_FD otape;		/* output tape descriptor            */

struct COPY_DEV indev;			/* input copy device                 */
struct COPY_DEV outdev;			/* output copy device                */
struct VARIABLE *val;			/* pointer to TAE parameters         */

long  nonfatal = 1;			/* Non-fatal error code		     */
long  fatal = -1;			/* Fatal error code		     */
long  file;				/* current file being copied         */
long  msgtim;				/* # of seconds between display msg  */
long  nfiles;				/* # of files to be copied           */
long  status;				/* function return status code       */
long  swap;				/* byte swap flag                    */

char msg[ERRLEN];			/* buffer space for messages         */
char trans[10];				/* type of transformation            */
char which[7];				/**/

val = p_find(&vblock,"nfiles");		/* retrieve parameter "nfiles"       */
if (val->v_count > 0)
   nfiles = IVAL(*val,0);
else
   nfiles = ALL;

val = p_find(&vblock,"tranopt");	/* retrieve parameter "transopt"     */
strcpy(trans,SVAL(*val,0));

val = p_find(&vblock,"nbytes");		/* retrieve parameter "nbytes"       */
swap = IVAL(*val,0);

val = p_find(&vblock,"$msgtim");	/* retrieve parameter "$msgtim"      */
msgtim = IVAL(*val,0);

strcpy(which,"INPUT");

/*
  Reopen the input tape device or get the disk file descriptor
*/
if (strcmp(idevice,"TAPE") == 0)
   {
   status = reopen_tape(which,NOREW,TREAD,&itape);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }
else
   {
   status = get_disk_fd(which,&idisk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);

   if (idisk.cur_file == NOVAL)
      nfiles = 1;
   }

strcpy(which,"OUTPUT");

/*
  Reopen the output tape device or get the disk file descriptor
*/
if (strcmp(odevice,"TAPE") == 0)
   {
   status = reopen_tape(which,NOREW,TWRITE,&otape);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }
else
   {
   status = get_disk_fd(which,&odisk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);

   if (odisk.cur_file == NOVAL)
      odisk.cur_file = 1;

   put_disk_fd(which,&odisk);
   }

/*
  Copy file(s) from the input device to the output device by calling routine
  "copy_proc" for each input file.  Note that all records are copied.
*/
for (file = 0; file < nfiles; file++)
   {
   /*
     Open the input and output devices
   */
   status = file_open(idevice,odevice,&indev,&outdev,CHECK);

   /*
     Copy the input device file to the output device file
   */
   status = copy_proc(&indev,&outdev,ALL,trans,swap,msgtim);
#ifdef vms
   if (file < (nfiles - 1))
     {
     if (outdev.device == TAPE)
        {
        status = c_tpweof(&otape.unit,&one,&mode);
        if (status != TAPE_SUCC)
	     ditto_error("OUTPUT",DITTO_WEOF,ERROR_WARN,UPDATE_FD);
        }
     }
#endif
   /*
     Close the input and output devices
   */
   file_close(&indev,&outdev);

   /*
     Update the input and output attributes
   */
   ++indev.cur_file;
   ++outdev.cur_file;

   if (status == COPY_EOV)
	{
	file++;
	break;
	}
   }

/*
  Output the number of files copied
*/
sprintf(msg,"%d Files Copied.",file);
c_errmsg(msg," ",&nonfatal);

/*
  Close the input tape device or disk file
*/
strcpy(which,"INPUT");
if (strcmp(idevice,"TAPE") == 0)
   close_tape(which,&itape,NOREW,1);
else
   put_disk_fd(which,&idisk);

/*
  Close the output tape device or disk file
*/
strcpy(which,"OUTPUT");
#ifdef vms
if (strcmp(odevice,"TAPE") == 0)
   {
   close_tape(which,&otape,NOREW,2);
   }
else
   put_disk_fd(which,&odisk);
#else
if (strcmp(odevice,"TAPE") == 0)
   close_tape(which,&otape,NOREW,1);
else
   put_disk_fd(which,&odisk);
#endif
return;
}

/******************************************************************************
FUNCTION:	FILE_OPEN

PURPOSE:	Opens the input or output devices.  This function is
		called only if "copy-file" is being executed because
		the TAE parameter block cannot be updated for each
		file copied.  

ALGORITHM 
    If the input device is DISK
	open input disk file
	Initialize the input file descriptor
    Else the input device is TAPE
	Initialize the input file descriptor

    If the output device is DISK
        Update the output DISK attributes
	open output disk file
	Initialize the output file descriptor
    Else the output device is TAPE
        Update the output TAPE attributes
	Initialize the output file descriptor

    If flag is set to CHECK
	error check to insure the record types, record size, and block size
	of the input/output devices are compatiable.

ALGORITHM REFERENCES	none
*******************************************************************************/
int file_open
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice,		/* output DITTO device (tape or disk)*/
    struct COPY_DEV *indev,	/* input device descriptor           */
    struct COPY_DEV *outdev,	/* output device descriptor          */
    long flag				/* error check flag                  */
)
{
extern struct DISK_FD idisk;		/* DITTO input disk file             */
extern struct DISK_FD odisk;		/* DITTO output disk file            */
extern struct TAPE_FD itape;		/* DITTO input tape                  */
extern struct TAPE_FD otape;		/* DITTO output tape                 */

long status;				/* function return status code       */
long fatal = -1;			/* fatal error code		     */

if (strcmp(idevice,"DISK") == 0)	
   {
   /*
     Open the input disk file
   */
   status = dopen(&idisk,DREAD);
   if (status == E_FAIL)
      {
      ditto_error("OUTPUT",DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      return(COPY_EOV);
      }

   /*
     intialize input copy descriptor
   */
   indev->cc     = idisk.cc;
   indev->rectyp = idisk.rectyp;
   indev->recsiz = idisk.recsiz;
   indev->blksiz = idisk.blksiz;
   indev->device = DISK;
   indev->cur_rec = indev->cur_file = 0;
   }
else
   {
   /*
     initialize input copy descriptor
   */
   indev->cc     = itape.cc;
   indev->rectyp = itape.rectyp;
   indev->recsiz = itape.recsiz;
   indev->blksiz = itape.blksiz;
   indev->device = TAPE;
   indev->cur_rec = indev->cur_file = 0;
   }

if (strcmp(odevice,"DISK") == 0)
   {
   /*
     Update the output DISK attributes
   */
   if (odisk.cc == NOVAL)
      odisk.cc = indev->cc;

   if (odisk.recsiz == NOVAL)
      odisk.recsiz = indev->recsiz;

   if (odisk.blksiz == NOVAL)
      odisk.blksiz = indev->blksiz;

   if (odisk.rectyp == NOVAL)
      odisk.rectyp = indev->rectyp;

   /*
     Open the output disk file
   */
   status = dopen(&odisk,DCREATE);
   if (status == E_FAIL)
      {
      ditto_error("OUTPUT",DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
      }

   /*
     initialize output copy descriptor
   */
   outdev->cc     = odisk.cc;
   outdev->rectyp = odisk.rectyp;
   outdev->recsiz = odisk.recsiz;
   outdev->blksiz = odisk.blksiz;
   outdev->device = DISK;
   outdev->cur_rec = outdev->cur_file = 0;
   }
else
   {
   /*
     Update the output TAPE attributes
   */
   if (otape.cc == NOVAL)
      otape.cc = indev->cc;

   if ((otape.recsiz == NOVAL) || (otape.recsiz == MAXTAPEWT))
      otape.recsiz = indev->recsiz;

   if ((otape.blksiz == NOVAL) || (otape.blksiz == MAXTAPEWT))
      otape.blksiz = indev->blksiz;

   if (otape.rectyp == NOVAL)
      otape.rectyp = indev->rectyp;

   /*
     Initialize output copy descriptor
   */
   outdev->cc     = otape.cc;
   outdev->rectyp = otape.rectyp;
   outdev->recsiz = otape.recsiz;
   outdev->blksiz = otape.blksiz;
   outdev->device = TAPE;
   outdev->cur_rec = outdev->cur_file = 0;
   }

if ((indev->rectyp == NOVAL) && (outdev->rectyp != NOVAL))
   indev->rectyp = outdev->rectyp;
else if ((outdev->rectyp == NOVAL) && (indev->rectyp != NOVAL))
   outdev->rectyp = indev->rectyp;

/*
  If flag is set to CHECK, then error check to insure the record types,
  record size, and block size of the input/output devices are compatiable.
*/
if (flag == CHECK)
   {
   if (indev->recsiz != outdev->recsiz) 
      ditto_error(" ",DITTO_RECSIZ,ERROR_VERIFY,UPDATE_FD);

   if (indev->blksiz != outdev->blksiz) 
      ditto_error(" ",DITTO_BLKSIZ,ERROR_VERIFY,UPDATE_FD);

   if (indev->rectyp != outdev->rectyp)
      ditto_error(" ",DITTO_RECTYP,ERROR_VERIFY,UPDATE_FD);
   }

return(E_SUCC);
}

/******************************************************************************
FUNCTION:	FILE_CLOSE

PURPOSE:	Closes the input or output devices.  This function is
		called only if "copy-file" is being executed because
		the TAE parameter block cannot be updated for each file
		copied.  

ALGORITHM 
    If input device is disk file then 
	update current record attribute
	update current file attribute
	close the disk file
    Else if input device is tape then 
	update current record attribute
	update current file attribute

    If output device is disk file then 
	update current record attribute
	update current file attribute
	close the disk file
    Else if output device is tape then 
	update current record attribute
	update current file attribute

ALGORITHM REFERENCES	none
*******************************************************************************/
void file_close
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev	/* output DITTO device               */
)
{
extern struct DISK_FD idisk;		/* input disk file                   */
extern struct DISK_FD odisk;		/* output disk file                  */
extern struct TAPE_FD itape;		/* input tape                        */
extern struct TAPE_FD otape;		/* output tape                       */

long status;			/* function return status code       */

if (indev->device == DISK)		/* if input device is disk file then */
   {
   idisk.cur_rec = 0;			/*   update current record attribute */
   if (idisk.cur_file != NOVAL)		/*   update current file attribute   */
      idisk.cur_file++;	

   status = dclose(&idisk);		/*   close the disk file             */
   if (status != E_SUCC)
	ditto_error("INPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else					/* if input device is tape then      */
   {
   itape.cur_rec = 0;
   itape.cur_file++;
   }

if (outdev->device == DISK)		/* if output device is disk file then*/
   {
   odisk.cur_rec = 0;			/*   update current record attribute */
   odisk.cur_file++;			/*   update current file attribute   */
   status = dclose(&odisk);		/*   close the disk file             */
   if (status != E_SUCC)
	ditto_error("OUTPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else					/* if output device is tape then     */
   {					/*   update current record and file  */
   otape.cur_rec = 0;
   otape.cur_file++;
   }

return;
}
