/******************************************************************************
FUNCTION:	MODREC_COPY

PURPOSE: 	Copies the specified number of records of a single file
		from the input device to the output device allowing the
		user to modify selected records.  The record and block
		sizes of the output device should be equal to the input
		device.  The record types of both the input and output
		devices should also be the same.  Transformations are
		not allowed for this type of file copy.

ALGORITHM 
    Retrieve the input parameters
    Open the input and output DITTO devices.
    Allocate buffer space
    Set the initial processing message utililty

    Copy the specified number of input records to output device allowing
    the user to modify selected records.
	Get the record to modify
            Read one physical record from input device

            If trans_flag is true
		call the transformation routine

            If the record is to be modified 
		call the modify routine
		display a modify message to the user's terminal

            Write one physical record from input device

    Disable the display message utility
    Update attributes
    If the output device is tape
	 write End of File marks
    Close the input and output DITTO devices.
    Free the buffer space

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>  			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"	  		/* LAS include file 		     */
#include "util.h"
#include "tapeio.h" 		/* tape I/O include file 	     */
#include "ditto.h"  		/* ditto include file                */
#include "copy.h"   
#include "protos.h"

#define currec   curent			/* curent is used for display msg.   */

void modrec_copy
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice		/* output DITTO device (tape or disk)*/
)
{
extern struct PARBLK vblock;	/* TAPE parameter block              */
extern struct TAPE_FD otape;	/* DITTO tape descriptor             */
extern long currec;		/* current record for display msg    */

struct COPY_DEV indev;		/* input copy device descriptor      */
struct COPY_DEV outdev;		/* output copy device descriptor     */
struct VARIABLE *val;		/* pointer to TAE parameters         */

long dummy = -2;		/* dummy variable                    */
long fatal = -1;		/* Fatal error code		     */
long flag = GEN1;		/* type of display message           */
long mode = HANDLE;		/* tape I/O writes error message     */
long msgtim;			/* # seconds between display messages*/
long nbyte;			/* number of bytes read              */
long next_mod;			/* next record to be modified        */
long nrecords;			/* # input records to be processed   */
long one = 1;			/* value of one                      */
long status;			/* function return status code       */
long swap;			/* byte swapping flag                */

long  trans_flag = 0;	/* transformation flag               */
unsigned char *bufptr;	/* pointer to I/O buffer             */

char	   msg[ERRLEN];			/* buffer for output message         */
char	   trans[10];			/* type of transformation            */

/*
  Retrieve number of records parameter
*/
val = p_find(&vblock,"nrecs");
if (val->v_count > 0)
   nrecords = IVAL(*val,0);
else
   nrecords = ALL;

/*
  Retrieve transopt parameter
*/
val = p_find(&vblock,"tranopt");
strcpy(trans,SVAL(*val,0));

/*
  Retrieve number of bytes
*/
val = p_find(&vblock,"nbytes");
swap = IVAL(*val,0);
if ((swap > 0) || (strcmp(trans,"NONE") != 0))
   trans_flag = 1;

/*
  Retrieve message time parameter
*/
val = p_find(&vblock,"$msgtim");
msgtim = IVAL(*val,0);

/*
  Retrieve the runtype
*/
val = p_find(&vblock,"$RUNTYPE");
if (strcmp(SVAL(*val,0),"INTERACTIVE") != 0)
   {
   ditto_error(" ",DITTO_MODREC,ERROR_WARN,NO_UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }

/*
  Open the input and output DITTO devices.
*/
open_device(idevice,odevice,&indev,&outdev,CHECK);

/*
  Allocate buffer space
*/
bufptr = (unsigned char *) malloc(indev.blksiz);
if (bufptr == 0)
   {
   ditto_error(" ",DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }

/*
  Set the initial processing message
*/
c_inittm("DITTO COPY ","Processed","record #"," "," ",&dummy,&dummy,&msgtim,
	&flag);

/*
  Copy the specified number of input records to output device allowing the
  user to modify selected records.
*/
for (currec=0; currec<nrecords; )
   {
   /*
     Get the record to be modified
   */
   next_mod = get_modrec(currec,nrecords);

   for (; (currec<=next_mod) && (currec<nrecords); currec++)
      {
      /*
        Read one physical record from input device
      */
      nbyte = read_device(&indev,&outdev,bufptr);
      if (nbyte <= 0)
	 break;

      /*
        if trans_flag is true call the transformation routine
      */
      if (trans_flag)
	 transform(bufptr,&nbyte,&swap,trans);

      /*
        If the record was modified display a message to the user's terminal
      */
      if (currec == next_mod)
	 {
	 c_stoptm();
         modify_record(bufptr,nbyte);
	 sprintf(msg,"RECORD %d HAS BEEN MODIFIED",currec);
	 status = m_msg(msg,"ditto-modify");
	 status = t_write("\n",T_NULL);
	 }

      /*
        Write one physical record to output device
      */
      write_device(&indev,&outdev,bufptr,nbyte);
      indev.cur_rec++;
      outdev.cur_rec++;
      }

   if (nbyte <= 0)
      break;
   }

/*
  Disable the display message utility
*/
c_stoptm();

/*
  Update attributes
*/
if (nbyte <= 0)
   indev.cur_file++;
outdev.cur_file++;

if ((nrecords != ALL) && (currec < nrecords))
   ditto_error(" ",DITTO_EOF,ERROR_WARN,UPDATE_FD);

/*
  If the output device is tape write End of File marks
*/
#ifndef vms
if (outdev.device == TAPE)
   {
   status = c_tpweof(&otape.unit,&one,&mode);
   if (status != TAPE_SUCC)
	ditto_error("OUTPUT",DITTO_WEOF,ERROR_WARN,UPDATE_FD);
   }
#endif

/*
  Close the input and output DITTO devices.
*/
close_device(&indev,&outdev);
free(bufptr);				/* free the buffer space             */

return;
}
/******************************************************************************
FUNCTION:	GET_MODREC

PURPOSE:	Queries the user to input a record number of the record to
		be modified.  The record is zero relative.  The query is done
		through the use of the TAE terminal I/O package.  Error
		checking is done to insure the record is still to be processed.
		A value of -1 indicates the user does not want to make any
		more modifications within the file.

ALGORITHM 
    Clear current line
    Query user to input a valid record number
	If the record value is "-1"
	    break out of query loop

	Check that the input record is a valid record number
	    Display an error message and requery
        Else
	    Break from the query loop

    Enable display message utility
    Return record # to be modified 

ALGORITHM REFERENCES	none
*******************************************************************************/

int get_modrec
(
    long   currec,		/* current record being processed    */
    long   nrecords 		/* number of records to be processed */
)
{
long	next_mod;			/* next record number to be modified */
int 	term;				/* line terminator                   */

char	msg[ERRLEN];			/* message to be read/written        */

sprintf(msg,"%78s\r"," ");
t_write(msg,T_NULL);		/* clear current line                */

for (;;)				/* query user to input a valid record*/
   {					/*    number                         */
    t_write("INPUT PHYSICAL RECORD # TO BE MODIFIED: ",
		    T_PROMPT);
   t_read(msg,&term);
   next_mod = atoi(msg);

   /*
     Check to break out of query loop
    */
   if (next_mod < 0)
      {
      next_mod = nrecords;
      break;
      }

   /*
     Check that the input record is a valid record number
   */
   if ((next_mod < currec) || (next_mod > nrecords))
      {
      ditto_error(" ",DITTO_INVMOD,ERROR_WARN,UPDATE_FD);

      if (nrecords != ALL)
         sprintf(msg,"Valid record numbers are %d through %d",currec,nrecords);
      else
         sprintf(msg,"Valid record numbers are %d through # Records in File",
	   	 currec);

      m_msg(msg,"ditto-invmod");
      }
   else
      break;
   }

t_write("\n",T_NULL);		/* next line                         */

c_startm();				/* enable display message utility    */

return(next_mod);			/* return record # to be modified    */
}

/******************************************************************************
FUNCTION:	MODIFY_RECORD

PURPOSE:	Allows the user to modify the current record.  It queries
		the user to input the starting position, type of modify,
		and to input the string in hex or character format.  Function
		"get_modstr" is called to convert the string into a byte
		value buffer.  The original record is modified beginning
		at the starting position.  The user is asked if there are
		any more modifications to the current record.  If so
		"modify_rec" is called recursively to process the next modify.

ALGORITHM 
    Query the user for the input starting offset
    Query the user for the input format (hex or character)
	Input string of hex digits or characters

    Query the user for any more modifications
	recursively call this routine to process the next modify.

ALGORITHM REFERENCES	none
*******************************************************************************/
void modify_record
(
    unsigned char *bufptr,	/* buffer to be modified             */
    long nbyte		/* number of bytes in physical record*/
)
{
long	index;				/* loop index                        */
long	mod_length;			/* number of bytes to be modified    */
long	start = 0;			/* starting position within record   */
int 	term;				/* type of line terminator           */

unsigned char   *mod_string;		/* modify string in byte format      */

char	msg[ERRLEN];			/* message read/written to terminal  */
char	type[10];			/* type of string (char or hex)      */

/*
  Query the user for the input starting offset
*/
for (;;)
   {
   t_write("INPUT STARTING OFFSET WITHIN RECORD: ",T_PROMPT);
   t_read(msg,&term);
   start = atoi(msg);

   if (start >= 0)
      break;
   else
      m_msg("Starting offset required to be >= 0","ditto-start");
   }

/*
  Query the user for the input format (hex or character)
*/
for (;;)
   {
   t_write("ENTERING IN HEX OR CHARACTER FORMAT? (H/C) ",T_PROMPT);
   t_read(type,&term);

   if ((type[0] == 'H') || (type[0] == 'h'))
      {
      strcpy(type,"HEX");
      t_write("INPUT STRING OF HEX DIGITS: ",T_PROMPT);
      t_read(msg,&term);
      break;
      }

   if ((type[0] == 'C') || (type[0] == 'c'))
      {
      strcpy(type,"CHAR");
      t_write("INPUT STRING OF CHARACTERS: ",T_PROMPT);
      t_read(msg,&term);
      break;
      }
   }

/*
  Modify the string of characters
*/
mod_string = get_modstr(msg,type,&mod_length);
if (start+mod_length > nbyte)
   ditto_error(" ",DITTO_MODLEN,ERROR_WARN,UPDATE_FD);

for (index=start; (index<start+mod_length) && (index<nbyte); index++)
   *(bufptr+index) = *mod_string++;

/*
  Check to see if there are any more modifications
*/
t_write("ANY MORE MODIFICATIONS TO CURRENT RECORD? (Y/N) ",T_PROMPT);
t_read(msg,&term);

/*
  recursively call this routine to process the next modify.
*/
if ((msg[0] == 'Y') || (msg[0] == 'y'))
   modify_record(bufptr,nbyte);
else
   t_write("\n",T_NULL);

return;
}

/******************************************************************************
FUNCTION:	GET_MODSTR

PURPOSE:	Converts the character/hexidecimal user supplied string into
		a byte value buffer.

ALGORITHM 
    If string in characer format
	copy the character string into the byte buffer
    Else if string in hex format
	call "hex2byt" to convert the hex digits into a byte value.

    The resulting byte buf is returned 

ALGORITHM REFERENCES	none
*******************************************************************************/
unsigned char *get_modstr
(
    char *string,			/* user string in char/hex format    */
    char *type,				/* type of string (char or hex)      */
    long *mod_length			/* length of modification            */
)
{
long	index;				/* loop index                        */
long	length;				/* length of scan string	     */

unsigned char   strhex[3];		/* char hex to be converted to byte  */
unsigned char	*temp;			/* temporary pointer                 */
unsigned char   *mod_string;		/* modify string                     */


if (strcmp(type,"CHAR") == 0)		/* if string in characer format then */
   {					/*   copy the character string into  */
   *mod_length = (long) strlen(string);	/*   the byte buffer and return      */
   mod_string = (unsigned char *) malloc(*mod_length);
   strcpy((char *) mod_string,string);
   }
else					/* if string in hex format then call */
   {					/*   "hex2byt" to convert the hex    */
   length = (long) strlen(string);		/*   digits into a byte value.  The  */
   if (length % 2 != 0)			/*   resulting byte buf is returned  */
      {
      length++;
      temp = (unsigned char *) malloc(length);
      sprintf((char *)temp,"0%s",string);
      }
   else
      {
      temp = (unsigned char *) malloc(length);
      strcpy((char *)temp,string);
      }

   *mod_length = length / 2;
   mod_string = (unsigned char *) malloc(*mod_length);
   strhex[2] = '\0';

   for (index = 0; index < *mod_length; index++)
      {
      strhex[0] = *temp++;
      strhex[1] = *temp++;
      hex2byt((char *) strhex,&mod_string[index]);
      }
   }

return(mod_string);
}
