/******************************************************************************
FUNCTION:	OPEN_DEVICE

PURPOSE: 	Opens the input and output DITTO devices. It also retrieves
		the file attributes of the DITTO devices from the TAE
		parameter block.  If flag is set to CHECK, then error checking
		is done to insure the record types, record sizes, and block
		sizes are compatiable.

ALGORITHM 
    If the input device is DISK
	Retrieve input disk file attribute
	Open input disk file
	Intialize input copy descriptor
    Else if the input device is TAPE
	Reopen input tape
	Intialize input copy descriptor

    If the output device is DISK
	Retrieve output disk file attribute
        Update output disk attributes
	Open output disk file
	Intialize output copy descriptor
    Else if the output device is TAPE
	Reopen output tape
        Update output tape attributes
	Intialize output copy descriptor

  If flag is set to CHECK
	error check to insure the record types, record size, and block size
	of the input/output devices are compatiable.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h> 			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "copy.h"
#include "protos.h"

void open_device
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice,		/* output DITTO device (tape or disk)*/
    struct COPY_DEV *indev,	/* input device descriptor           */
    struct COPY_DEV *outdev,	/* output device descriptor          */
    long flag				/* error check flag                  */
)
{
extern struct DISK_FD idisk;		/* DITTO input disk file             */
extern struct DISK_FD odisk;		/* DITTO output disk file            */
extern struct TAPE_FD itape;		/* DITTO input tape                  */
extern struct TAPE_FD otape;		/* DITTO output tape                 */

long status;				/* function return status code       */
long fatal = -1;			/* Fatal error code		     */

if (strcmp(idevice,"DISK") == 0)	
   {
   status = get_disk_fd("INPUT",&idisk);/* retrieve input disk file attribute*/
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);

   status = dopen(&idisk,DREAD);	/* open input disk file              */
   if (status == E_FAIL)
      {
      ditto_error("INPUT\0",DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
      }

   indev->cc     = idisk.cc;		/* intialize input copy descriptor   */
   indev->rectyp = idisk.rectyp;
   indev->recsiz = idisk.recsiz;
   indev->blksiz = idisk.blksiz;
   indev->device = DISK;
   indev->cur_rec = indev->cur_file = 0;
   }
else
   {
   status = reopen_tape("INPUT\0",NOREW,TREAD,&itape); /* open input tape    */
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
	
   indev->cc     = itape.cc;		/* initialize input copy descriptor  */
   indev->rectyp = itape.rectyp;
   indev->recsiz = itape.recsiz;
   indev->blksiz = itape.blksiz;
   indev->device = TAPE;
   indev->cur_rec = indev->cur_file = 0;
   }

if (strcmp(odevice,"DISK") == 0)
   {
   /*
     Retrieve output attributs
   */
   status=get_disk_fd("OUTPUT\0",&odisk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);

   /*
     Update output disk attributes
   */
   if (odisk.cc == NOVAL)
      odisk.cc = indev->cc;

   if (odisk.recsiz == NOVAL)
      odisk.recsiz = indev->recsiz;

   if (odisk.blksiz == NOVAL)
      odisk.blksiz = indev->blksiz;

   if (odisk.rectyp == NOVAL)
      odisk.rectyp = indev->rectyp;

   /*
     Open output disk file
   */
   status = dopen(&odisk,DCREATE);
   if (status == E_FAIL)
      {
      ditto_error("OUTPUT\0",DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
      }

   /*
     Initialize output copy descriptor
   */
   outdev->cc     = odisk.cc;
   outdev->rectyp = odisk.rectyp;
   outdev->recsiz = odisk.recsiz;
   outdev->blksiz = odisk.blksiz;
   outdev->device = DISK;
   outdev->cur_rec = outdev->cur_file = 0;
   }
else
   {
   /*
     Reopen output tape
   */
   status = reopen_tape("OUTPUT",NOREW,TWRITE,&otape); /* open output tape    */
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);

   /*
     Update output tape attributes
   */
   if (otape.cc == NOVAL)
      otape.cc = indev->cc;

   if ((otape.recsiz == NOVAL) || (otape.recsiz == MAXTAPEWT))
      otape.recsiz = indev->recsiz;

   if ((otape.blksiz == NOVAL) || (otape.blksiz == MAXTAPEWT))
      otape.blksiz = indev->blksiz;

   if (otape.rectyp == NOVAL)
      otape.rectyp = indev->rectyp;

   /*
     initialize output copy descriptor
   */
   outdev->cc     = otape.cc;
   outdev->rectyp = otape.rectyp;
   outdev->recsiz = otape.recsiz;
   outdev->blksiz = otape.blksiz;
   outdev->device = TAPE;
   outdev->cur_rec = outdev->cur_file = 0;
   }

if ((indev->rectyp == NOVAL) && (outdev->rectyp != NOVAL))
   indev->rectyp = outdev->rectyp;
else if ((outdev->rectyp == NOVAL) && (indev->rectyp != NOVAL))
   outdev->rectyp = indev->rectyp;

/*
  If flag is set to CHECK, then error check to insure the record types,
  record size, and block size of the input/output devices are compatiable.
*/
if (flag == CHECK)
   {
   if (indev->recsiz != outdev->recsiz) 
      ditto_error(" ",DITTO_RECSIZ,ERROR_VERIFY,UPDATE_FD);

   if (indev->blksiz != outdev->blksiz) 
      ditto_error(" ",DITTO_BLKSIZ,ERROR_VERIFY,UPDATE_FD);

   if (indev->rectyp != outdev->rectyp)
      ditto_error(" ",DITTO_RECTYP,ERROR_VERIFY,UPDATE_FD);
   }

return;
}
