/*********************************************************************************
 FUNCTION: PITS_COPY

 PURPOSE:  Pits copy will copy tapes.  If an I/O error occurs control is passed
	   to an interactive session. 

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      ??/??    K. Gacke            initial development
  2.0       2/92    T. Mittan		Added to DITTO-COPY 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS


*********************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */

#ifdef vms
#include <unixio.h>
#include <file.h>
#else
#include <sys/file.h>
#endif
#ifdef sun
#include <fcntl.h>
#endif

#include <fcntl.h>
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "copy.h"
#include "protos.h"

static long  nonfatal = 1;		/* Non-fatal error code		     */
static long  fatal = -1;		/* Fatal error code		     */
static char *Prcnam = "DITTO COPY ";
static char *Verb = "Processed";
static char *Subj = "record #";
static char *Text1 = " ";
static char Text2[CMLEN];

#define currec   curent			/* curent is used for display msg.   */

void pits_copy
(
    char *idevice,			/* input DITTO device (tape or disk) */
    char *odevice			/* output DITTO device (tape or disk)*/
)
{
extern struct PARBLK vblock;		/* TAE parameter block               */
extern struct DISK_FD idisk;		/* input disk descriptor             */
extern struct DISK_FD odisk;		/* output disk descriptor            */
extern struct TAPE_FD itape;		/* input tape descriptor             */
extern struct TAPE_FD otape;		/* output tape descriptor            */

struct COPY_DEV indev;			/* input copy device                 */
struct COPY_DEV outdev;			/* output copy device                */
struct VARIABLE *val;			/* pointer to TAE parameters         */

long  file;				/* current file being copied         */
long  msgtim;				/* # of seconds between display msg  */
long  nfiles;				/* # of files to be copied           */
long  status;				/* function return status code       */

char msg[ERRLEN];			/* buffer space for messages         */
char which[7];				/**/

val = p_find(&vblock,"nfiles");		/* retrieve parameter "nfiles"       */
if (val->v_count > 0)
   nfiles = IVAL(*val,0);
else
   nfiles = ALL;
val = p_find(&vblock,"$msgtim");	/* retrieve parameter "$msgtim"      */
msgtim = IVAL(*val,0);
strcpy(which,"INPUT");
if (strcmp(idevice,"TAPE") == 0)	/* open input device */
   {
   status = reopen_tape(which,NOREW,TREAD,&itape);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }
else
   {
   status = get_disk_fd(which,&idisk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   if (idisk.cur_file == NOVAL)
      nfiles = 1;
   }
strcpy(which,"OUTPUT");
if (strcmp(odevice,"TAPE") == 0)	/* open output device */
   {
   status = reopen_tape(which,NOREW,TWRITE,&otape);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }
else
   {
   status = get_disk_fd(which,&odisk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);

   if (odisk.cur_file == NOVAL)
      odisk.cur_file = 1;
   put_disk_fd(which,&odisk);
   }
/*
  Copy file(s) from the input device to the output device by calling routine
  "copy_proc" for each input file.  Note that all records are copied.
*/
for (file = 1; file <= nfiles; file++)
   {
   status = pits_open(idevice,odevice,&indev,&outdev,CHECK);
   status = copy_pits(&indev,&outdev,&file,msgtim);
   pits_close(&indev,&outdev);
   ++indev.cur_file;
   ++outdev.cur_file;
   if (status == COPY_EOV)
	break;
   }
sprintf(msg,"%d Files Copied.",file-1);
c_errmsg(msg," ",&nonfatal);
strcpy(which,"INPUT");
if (strcmp(idevice,"TAPE") == 0)	/* close input device */
   close_tape(which,&itape,NOREW,1);
else
   put_disk_fd(which,&idisk);
strcpy(which,"OUTPUT");
if (strcmp(odevice,"TAPE") == 0)	/* close output device */
   close_tape(which,&otape,NOREW,1);
else
   put_disk_fd(which,&odisk);
return;
}

/******************************************************************************

PURPOSE:	Opens the input or output devices.  

ALGORITHM 
    If the input device is DISK
	open input disk file
	Initialize the input file descriptor
    Else the input device is TAPE
	Initialize the input file descriptor

    If the output device is DISK
        Update the output DISK attributes
	open output disk file
	Initialize the output file descriptor
    Else the output device is TAPE
        Update the output TAPE attributes
	Initialize the output file descriptor

    If flag is set to CHECK
	error check to insure the record types, record size, and block size
	of the input/output devices are compatiable.

ALGORITHM REFERENCES	none
*******************************************************************************/
int pits_open
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice,		/* output DITTO device (tape or disk)*/
    struct COPY_DEV *indev,	/* input device descriptor           */
    struct COPY_DEV *outdev,	/* output device descriptor          */
    long flag				/* error check flag                  */
)
{
extern struct DISK_FD idisk;		/* DITTO input disk file             */
extern struct DISK_FD odisk;		/* DITTO output disk file            */
extern struct TAPE_FD itape;		/* DITTO input tape                  */
extern struct TAPE_FD otape;		/* DITTO output tape                 */

long status;				/* function return status code       */

if (strcmp(idevice,"DISK") == 0)	/* open input disk file */
   {
   status = dopen(&idisk,DREAD);
   if (status == E_FAIL)
      {
      ditto_error("OUTPUT",DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      return(COPY_EOV);
      }
   indev->cc     = idisk.cc;		/* initialize input copy descriptor */
   indev->rectyp = idisk.rectyp;
   indev->recsiz = idisk.recsiz;
   indev->blksiz = idisk.blksiz;
   indev->device = DISK;
   indev->cur_rec = indev->cur_file = 0;
   }
else
   {
   indev->cc     = itape.cc;		/* initialize input copy descriptor */
   indev->rectyp = itape.rectyp;
   indev->recsiz = itape.recsiz;
   indev->blksiz = itape.blksiz;
   indev->device = TAPE;
   indev->cur_rec = indev->cur_file = 0;
   }
if (strcmp(odevice,"DISK") == 0)
   {
   if (odisk.cc == NOVAL)		/* init output DISK attributes */
      odisk.cc = indev->cc;
   if (odisk.recsiz == NOVAL)
      odisk.recsiz = indev->recsiz;
   if (odisk.blksiz == NOVAL)
      odisk.blksiz = indev->blksiz;
   if (odisk.rectyp == NOVAL)
      odisk.rectyp = indev->rectyp;
   status = dopen(&odisk,DCREATE);	/* open output disk device */
   if (status == E_FAIL)
      {
      ditto_error("OUTPUT",DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
      }
   outdev->cc     = odisk.cc;		/* init output copy descriptor */
   outdev->rectyp = odisk.rectyp;
   outdev->recsiz = odisk.recsiz;
   outdev->blksiz = odisk.blksiz;
   outdev->device = DISK;
   outdev->cur_rec = outdev->cur_file = 0;
   }
else
   {
   if (otape.cc == NOVAL)		/* update output TAPE attributes */
      otape.cc = indev->cc;
   if ((otape.recsiz == NOVAL) || (otape.recsiz == MAXTAPEWT))
      otape.recsiz = indev->recsiz;
   if ((otape.blksiz == NOVAL) || (otape.blksiz == MAXTAPEWT))
      otape.blksiz = indev->blksiz;
   if (otape.rectyp == NOVAL)
      otape.rectyp = indev->rectyp;
   outdev->cc     = otape.cc;		/* init output copy descriptor */
   outdev->rectyp = otape.rectyp;
   outdev->recsiz = otape.recsiz;
   outdev->blksiz = otape.blksiz;
   outdev->device = TAPE;
   outdev->cur_rec = outdev->cur_file = 0;
   }
if ((indev->rectyp == NOVAL) && (outdev->rectyp != NOVAL))
   indev->rectyp = outdev->rectyp;
else if ((outdev->rectyp == NOVAL) && (indev->rectyp != NOVAL))
   outdev->rectyp = indev->rectyp;
/*
  If flag is set to CHECK, then error check to insure the record types,
  record size, and block size of the input/output devices are compatiable.
*/
if (flag == CHECK)
   {
   if (indev->recsiz != outdev->recsiz) 
      ditto_error(" ",DITTO_RECSIZ,ERROR_VERIFY,UPDATE_FD);

   if (indev->blksiz != outdev->blksiz) 
      ditto_error(" ",DITTO_BLKSIZ,ERROR_VERIFY,UPDATE_FD);

   if (indev->rectyp != outdev->rectyp)
      ditto_error(" ",DITTO_RECTYP,ERROR_VERIFY,UPDATE_FD);
   }
return(E_SUCC);
}

/******************************************************************************
PURPOSE:	Closes the input or output devices.  

ALGORITHM 
    If input device is disk file then 
	update current record attribute
	update current file attribute
	close the disk file
    Else if input device is tape then 
	update current record attribute
	update current file attribute

    If output device is disk file then 
	update current record attribute
	update current file attribute
	close the disk file
    Else if output device is tape then 
	update current record attribute
	update current file attribute

ALGORITHM REFERENCES	none
*******************************************************************************/
void pits_close
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev	/* output DITTO device               */
)
{
extern struct DISK_FD idisk;		/* input disk file                   */
extern struct DISK_FD odisk;		/* output disk file                  */
extern struct TAPE_FD itape;		/* input tape                        */
extern struct TAPE_FD otape;		/* output tape                       */

long status;			/* function return status code       */

if (indev->device == DISK)		/* if input device is disk file then */
   {
   idisk.cur_rec = 0;			/*   update current record attribute */
   if (idisk.cur_file != NOVAL)		/*   update current file attribute   */
      idisk.cur_file++;	
   status = dclose(&idisk);		/*   close the disk file             */
   if (status != E_SUCC)
	ditto_error("INPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else					/* if input device is tape then      */
   {
   itape.cur_rec = 0;
   itape.cur_file++;
   }
if (outdev->device == DISK)		/* if output device is disk file then*/
   {
   odisk.cur_rec = 0;			/*   update current record attribute */
   odisk.cur_file++;			/*   update current file attribute   */
   status = dclose(&odisk);		/*   close the disk file             */
   if (status != E_SUCC)
	ditto_error("OUTPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else					/* if output device is tape then     */
   {					/*   update current record and file  */
   otape.cur_rec = 0;
   otape.cur_file++;
   }
return;
}

/******************************************************************************
PURPOSE:	Copies a specified number of records from the current
		position of the input device to the output device.  The
		record and block sizes of both devices should be equal.

		If parity error output a menu of options on how to continue.

*******************************************************************************/

int copy_pits
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev,	/* output DITTO device               */
    long *filnmr,			/* current file number being copied  */
    long msgtim			/* # of seconds between display msg  */
)
{
extern long currec;			/* current record for display msg    */
extern struct TAPE_FD otape;		/* output tape                       */
long dummy = -2;		/* dummy variable for display msg    */
long flag = GEN1;		/* flag indicating type display msg  */
long nbyte;			/* number of bytes read              */
long mode = HANDLE;		/* tape i/o error mode */
long one = 1;			/* value of one */
long prev_nbyte;		/* Previous record block size */
long status;			/* function status code */
unsigned char *bufptr;	/* pointer to I/O buffer             */

/*
  Allocate enough buffer space to read one input physical record as specified
  by the block size.
*/
bufptr = (unsigned char *) malloc(indev->blksiz);
if (bufptr == 0)
   {
   ditto_error(" ",DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }
prev_nbyte = 0;
currec = 0;
sprintf(Text2,"of file %d",*filnmr);
c_inittm(Prcnam,Verb,Subj,Text1,Text2,&dummy,&dummy,&msgtim,&flag);
c_startm();
for (currec=0; ; currec++,indev->cur_rec++,outdev->cur_rec++)
    {
    nbyte = pits_read_device(indev,bufptr);
    if (nbyte == COPY_EOV)
	break;
    if (nbyte < 0)
	nbyte = pits(indev,outdev,filnmr,msgtim,prev_nbyte,(char *) bufptr);
    if (nbyte == 0)			/* if at end of file break out*/
	break;
    else
	{
	prev_nbyte = nbyte;
        write_device(indev,outdev,bufptr,nbyte);
	}
   }
c_stoptm();
free(bufptr);
/*
  Update attributes
*/
indev->cur_file++;
outdev->cur_file++;
/*
  If the output device is tape write End of File marks
*/
if (outdev->device == TAPE)
   {
   status = c_tpweof(&otape.unit,&one,&mode);
   if (status != TAPE_SUCC)
	ditto_error("OUTPUT",DITTO_WEOF,ERROR_WARN,UPDATE_FD);
   }
return(E_SUCC);
}

int pits
(
     struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev,	/* output DITTO device               */
    long *filnmr,
    long msgtim,
    long prev_nbyte,
    char *bufptr 
)
{
extern struct TAPE_FD itape;		/* input tape descriptor            */
extern struct TAPE_FD otape;		/* output tape descriptor            */
long cnt;
long cont;
long mode = HANDLE;
long nbyte;
long option;
long skipopt;
long status;

c_stoptm();
nbyte = -1;
for (cont=TRUE; cont; )
    {
    printf("\n\n\nPITS Menu\n");
    printf("   1) Write zero fill record of previous record size (%d bytes)\n",
	prev_nbyte);
    printf("      to output device, and continue with next record.\n");
    printf("   2) Output tape mark on output device and skip to next\n");
    printf("      file on input device.\n");
    printf("   3) Write EOF on output device, copy disk file to output\n");
    printf("      device, and skip to next file on input device.\n");
    printf("   4) Backspace to beginning of file on output device, \n");
    printf("      copy disk file to output device, and skip to next\n");
    printf("      file on input device.\n");
    printf("   5) Terminate program\n");
    printf("\nEnter option: ");
    if (scanf("%d",&option) == 1)
	{
	switch (option)
	    {
	    case 1:
		cont = FALSE;
		skipopt = 1;
   		status = c_tpfsr(&itape.unit,&skipopt,&mode);
   		if (status != TAPE_SUCC)
		    {
		    c_errmsg("Unable to skip past bad record","ditto-pits",
			&nonfatal);
		    cont = TRUE;
   		    }
		else
		    {
		    for (cnt=0; cnt<prev_nbyte; cnt++)
		        *(bufptr+cnt) = 0;
   		    write_device(indev,outdev,(unsigned char *) bufptr,prev_nbyte);
		    nbyte = prev_nbyte;
		    }
		break;
	    case 2:
		cont = FALSE;
		skipopt = 0;
		status = c_tpbsf(&itape.unit,&skipopt,&mode);
   		if (status != TAPE_SUCC)
		    {
		    c_errmsg("Unable to back skip to beginning of file",
			"ditto-pits",&nonfatal);
		    cont = TRUE;
		    }
		skipopt = 1;
   		status = c_tpfsf(&itape.unit,&skipopt,&mode);
   		if (status != TAPE_SUCC)
		    {
		    c_errmsg("Unable to skip past bad file","ditto-pits",
			&nonfatal);
		    cont = TRUE;
		    }
		else
		    {
		    nbyte = 0;
		    }
		break;
	    case 3:
		cont = FALSE;
		skipopt = 0;
		status = c_tpbsf(&itape.unit,&skipopt,&mode);
   		if (status != TAPE_SUCC)
		    {
		    c_errmsg("Unable to back skip to beginning of input file",
			"ditto-pits",&nonfatal);
		    cont = TRUE;
		    }
		skipopt = 1;
   		status = c_tpfsf(&itape.unit,&skipopt,&mode);
   		if (status != TAPE_SUCC)
		    {
		    c_errmsg("Unable to skip past bad input file","ditto-pits",
			&nonfatal);
		    cont = TRUE;
		    }
		if (cont == FALSE)
		    {
		    ++(*filnmr);
		    indev->cur_file++;
		    outdev->cur_file++;
		    if (outdev->device == TAPE)
   		        {
		        skipopt = 1;
   		        status = c_tpweof(&otape.unit,&skipopt,&mode);
   		        if (status != TAPE_SUCC)
			    ditto_error("OUTPUT",DITTO_WEOF,ERROR_WARN,
				UPDATE_FD);
   		        }
		    status = pits_dcopy(indev,outdev,filnmr,msgtim);
		    if (status == E_SUCC)
		        nbyte = 0;
		    else
		        cont = TRUE;
		    }
		break;
	    case 4:
		cont = FALSE;
		skipopt = 0;
		status = c_tpbsf(&itape.unit,&skipopt,&mode);
   		if (status != TAPE_SUCC)
		    {
		    c_errmsg("Unable to back skip to beginning of input file",
			"ditto-pits",&nonfatal);
		    cont = TRUE;
		    }
		skipopt = 1;
   		status = c_tpfsf(&itape.unit,&skipopt,&mode);
   		if (status != TAPE_SUCC)
		    {
		    c_errmsg("Unable to skip past bad input file","ditto-pits",
			&nonfatal);
		    cont = TRUE;
		    }
		skipopt = 0;
		status = c_tpbsf(&otape.unit,&skipopt,&mode);
   		if (status != TAPE_SUCC)
		    {
		    c_errmsg("Unable to back skip to beginning of output file",
			    "ditto-pits",&nonfatal);
		    cont = TRUE;
		    }
		if (cont == FALSE)
		    {
		    status = pits_dcopy(indev,outdev,filnmr,msgtim);
		    if (status == E_SUCC)
		        nbyte = 0;
		    else
			cont = TRUE;
		    }
		break;
	    case 5:
		cont = FALSE;
		c_errmsg("PITS program aborted by user","ditto-copy",&fatal);
		break;
	    }
	}
    }
printf("\n");
c_startm();
return(nbyte);
}

int pits_dcopy
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev,	/* output DITTO device               */
    long *filnmr,
    long msgtim
)
{
extern long currec;		/* current record for display msg    */
long dummy = -2;		/* dummy variable for display msg    */
long flag = GEN1;		/* flag indicating type display msg  */
long fd;
long reclen;
long status;
char *buf;
char emsg[CMLEN];
char fname[CMLEN];

printf("Enter input disk file name: ");
scanf("%s",fname);
printf("Enter record length: ");
scanf("%d",&reclen);
buf = (char *)malloc(reclen);
if (buf == NULL)
    {
    perror("pits_dcopy");
    sprintf(emsg,"Error allocating %d bytes for i/o buffer",reclen);
    c_errmsg(emsg,"ditto-copy",&nonfatal);
    }
fd = open(fname,O_RDONLY,0);
if (fd == -1)
    {
    perror("pits_dcopy");
    sprintf(emsg,"Error opening file %s",fname);
    c_errmsg(emsg,"ditto-copy",&nonfatal);
    return(E_FAIL);
    }
currec = 0;
sprintf(Text2,"of file %d",*filnmr);
c_inittm(Prcnam,Verb,Subj,Text1,Text2,&dummy,&dummy,&msgtim,&flag);
c_startm();
indev->cur_rec = outdev->cur_rec = 0;
for (currec=0; ; currec++,indev->cur_rec++,outdev->cur_rec++)
    {
    status = read(fd,buf,reclen);
    if (status == 0)
	break;
    if (status == -1)
        {
        perror("pits_dcopy");
        sprintf(emsg,"Error reading file %s",fname);
        c_errmsg(emsg,"ditto-copy",&nonfatal);
        return(E_FAIL);
	}
    write_device(indev,outdev,(unsigned char *) buf,status);
    }
close(fd);
c_stoptm();
free(buf);
return(E_SUCC);
}

int pits_read_device
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    unsigned char *inbuf	/* buffer to read record into        */
)
{
extern struct DISK_FD idisk;		/* DITTO input disk file descriptor  */
extern struct TAPE_FD itape;		/* DITTO input tape descriptor       */
long mode = HANDLE;			/* tape I/O package output error msg */
long nbyte;				/* number of bytes read              */

if (indev->device == DISK)
   {
   nbyte = dread(&idisk,inbuf,idisk.blksiz);
   if (nbyte == E_FAIL)
      c_errmsg("Error reading input disk record","ditto-pits",&nonfatal);
   }
else
   {
   nbyte = c_tpread(&itape.unit,inbuf,&itape.blksiz,&mode);
   if (nbyte < 0)
      {
      switch(nbyte)
         {
         case 
	     TAPE_EOF:	return(0);
         case 
	     TAPE_EOV:	return(COPY_EOV);
         default:
             c_errmsg("Error reading input tape record","ditto-pits",&nonfatal);
	     break;
         }
      }
   }
return(nbyte);
}
