/******************************************************************************
FUNCTION:	PROC_ERROR

PURPOSE: 	Processes error conditions.  It saves the old DITTO
		input/output attributes, update the attributes to
		current processing point, and calls ditto_error to
		output the error message and to determine if processing
		should continue.  If control is returned from ditto_error,
		the old attributes are reassigned.

ALGORITHM 
  Get the input  device disk/tape attribute record
  Get the output device disk/tape attribute record
  Save the current attributes
  Stop the message processing utility
  Display the error message.  (Things will be restored if the error message
  was not fatal).
  Restart the message processing utility
  Restore the old attributes

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>   			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "copy.h"
#include "protos.h"

void proc_error
(
    register struct COPY_DEV *indev,	/* input copy device                 */
    register struct COPY_DEV *outdev,	/* output copy device                */
    register char *which,		/* which device (input or output)    */
    register long error_code		/* error code encountered            */
)
{
extern struct DISK_FD idisk;		/* DITTO input disk file             */
extern struct DISK_FD odisk;		/* DITTO output disk file            */
extern struct TAPE_FD itape;		/* DITTO input tape                  */
extern struct TAPE_FD otape;		/* DITTO output tape                 */

long   old_irec;			/* old input current record          */
long   old_orec;			/* old output current record         */
long   *inrec;				/* pointer to input current record   */
long   *outrec;				/* pointer to output current record  */

/*
  Get the input device disk/tape attribute record
*/
if (indev->device == DISK)
   inrec = &idisk.cur_rec;
else
   inrec = &itape.cur_rec;

/*
  Get the output device disk/tape attribute record
*/
if (outdev->device == DISK)
   outrec = &odisk.cur_rec;
else
   outrec = &otape.cur_rec;

/*
  Save the current attributes
*/
old_irec = *inrec;
old_orec = *outrec;

if (*inrec != NOVAL)
   *inrec += indev->cur_rec + 1;

if (*outrec != NOVAL)
   *outrec += outdev->cur_rec + 1;

/*
  Stop the message processing utility
*/
c_stoptm();

/*
  Display the error message.  Things will be restored if the error message
  was not fatal.
*/
ditto_error(which,error_code,ERROR_VERIFY,UPDATE_FD);

/*
  Restart the message processing utility
*/
c_startm();

/*
  Restore the old attributes
*/
*inrec = old_irec;
*outrec = old_orec;

return;
}
