/******************************************************************************
FUNCTION:	REBLOCK_COPY

PURPOSE: 	Reblocks a specified number of records of a single file from
		the input device, and copies them to the output device.  It
		pads or truncates each input logical record to the specified
		output logical record length.  The record and block sizes are
		retrieved from the TAE parameter block as global variables.

ALGORITHM 
    Retrieve the input parameters
    Open input and output DITTO devices
    Reblock and copy specified number of input records to the output device
    Close the input and output DITTO devices

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>   			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "copy.h"
#include "protos.h"

#define currec   curent			/* curent is used for display msg.   */

void reblock_copy
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice		/* output DITTO device (tape or disk)*/
)
{
extern struct PARBLK vblock;		/* TAE parameter block		     */

struct COPY_DEV indev;			/* input copy device descriptor      */
struct COPY_DEV outdev;			/* output copy device descriptor     */
struct VARIABLE *val;			/* pointer to TAE parameters         */

long   msgtim;				/* # seconds between display msg.    */
long   nrecords;			/* # input records to be processed   */
long   swap;				/* byte swap flag                    */

unsigned char pad_val;			/* pad value                         */

char  *pad;				/* pad character string              */
char  trans[10];			/* type of transformation            */

/*
  Retrieve number of records
*/
val = p_find(&vblock,"nrecs");
if (val->v_count > 0)
   nrecords = IVAL(*val,0);
else
   nrecords = ALL;

/*
  Retrieve transopt parameter
*/
val = p_find(&vblock,"tranopt");
strcpy(trans,SVAL(*val,0));

/*
  Retrieve number of bytes
*/
val = p_find(&vblock,"nbytes");
swap = IVAL(*val,0);

/*
  Retrieve fill value
*/
val = p_find(&vblock,"fillval");
pad = SVAL(*val,0);

/*
  Retrieve fill type
*/
val = p_find(&vblock,"filltype");
if (strcmp(SVAL(*val,0),"HEX") == 0)	/* if "type" set to hexidecimal then */
   hex2byt(pad,&pad_val);		/*    convert ascii hex to byte value*/
else					/* else                              */
   {					/*    assign first value of "pad"    */
   if (strlen(pad) > 1)
      ditto_error(" ",DITTO_PADCHR,ERROR_VERIFY,UPDATE_FD);
   pad_val = pad[0];
   }

/*
  Retrieve message time
*/
val = p_find(&vblock,"$msgtim");
msgtim = IVAL(*val,0);

/*
  Open input and output DITTO devices
*/
open_device(idevice,odevice,&indev,&outdev,NOCHECK);

/*
  Reblock and copy specified number of input records to the output device
*/
reblk_proc(&indev,&outdev,nrecords,trans,swap,pad_val,msgtim);

/*
  Close the input and output DITTO devices
*/
close_device(&indev,&outdev);

return;
}

/******************************************************************************
FUNCTION:	REBLK_PROC

PURPOSE: 	Reblock the input data and write it to the output device.
		The routine pads or truncates logical records to the
		specified length.  If the output carriage control is
		carriage return, a new line character is inserted as
		the last character of the output buffer.  If the output
		record type is variable length records, then all input
		records less than the maximum output record length are
		not padded, however all input records greater than the
		maximum output record length are truncated.  

ALGORITHM 
    Enable display message utility
    If transformation is to be performed
	set trans_flag equal to true.
    Allocate memory for the input and output buffers.  Each allocate number of
    bytes in one physical record (block size).
   Reblock input data and copy to output device.  For each physical input
   record the following steps are taken:
	read one input physical record
	if number of bytes read is zero, break from loop
	if input device has carriage return carriage control the newline 
	   character is replaced with pad value
	if trans_flag is true, then the transformation routine is called
	if output carriage control is carriage return AND
	   number of bytes read is less than output block size then
		write input buffer to the output device
		go to beginning of loop (read next input record)
	do for each byte of the input physical record
	   do forever (increment input and output # bytes for each iteration)
	      if output block size is reached then
		 call reblk_prewrite to write output buffer to output device
	      if output logical record size reached then
		 increment input # byte to end of logical input record to 
			truncate the record.
	      if input logical record size reached then
                if output logical record length NOT reached then
		    call reblk_pad to pad the logical record
		 reset input and output logical record length counts to zero
		 break out of "do forever" loop
	      if input physical record size reach then
		 break out of "do forever" loop
          end do forever
	end do
	if data in output buffer then
	   call reblk_prewrite to write output buffer to output device

    update device attributes
    Disable processing message utility
    If the output device is tape
	write EOF marks

ALGORITHM REFERENCES	none
*******************************************************************************/
void reblk_proc
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    long nrecords,			/* # of input records to be copied   */
    char *trans,			/* type of transformation            */
    long swap,				/* byte swapping flag                */
    unsigned char pad_val,		/* pad value                         */
    long msgtim			/* # seconds between display messages*/
)
{
extern struct TAPE_FD otape; 		/* DITTO tape descriptor             */
extern long currec;			/* current record for display msg    */
long blksiz;				/* actual block size read            */
long dummy = -2;			/* dummy variable                    */
long fatal = -1;			/* Fatal error code		     */
long flag = GEN1;			/* type of display message           */
long mode = HANDLE;			/* tape I/O writes error messages    */
long incur = 0;				/* current input byte of logical rec */
long intot = 0;				/* current input byte of physical rec*/
long one = 1;				/* value of one                      */
long outcur = 0;			/* current output byte of logical rec*/
long outtot = 0;			/* current output byte of physical rec*/
long status;				/* function return status code       */

long   trans_flag = 0;		/* transformation flag               */
unsigned char *inbuf;		/* input buffer                      */
unsigned char *outbuf;		/* output buffer                     */

/*
  enable display message utility
*/
c_inittm("DITTO COPY ","Processed","record #"," "," ",&dummy,&dummy,&msgtim,
	&flag);

c_startm();

/*
  If transformation is to be performed set trans_flag equal to true.
*/
if ((swap > 0) || (strcmp(trans,"NONE") != 0))
   trans_flag = 1;

/*  
  Allocate memory for the input and output buffers.  Each allocate number of
  bytes in one physical record (block size).
*/
inbuf = (unsigned char *) malloc(indev->blksiz);
if (inbuf == NULL)
   {
   ditto_error("INPUT",DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }

outbuf = (unsigned char *) malloc(outdev->blksiz);
if (outbuf == NULL)
   {
   ditto_error("OUTPUT",DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto copy-fatal",&fatal);
   }

/*
*   Reblock input data and copy to output device.  For each physical input
*   record the following steps are taken:
*	read one input physical record
*	if number of bytes read is zero, break from loop
*	if input device has carriage return carriage control the newline 
*	   character is replaced with pad value
*	if trans_flag is true, then the transformation routine is called
*	if output carriage control is carriage return AND
*	   number of bytes read is less than output block size then
*		write input buffer to the output device
*		go to beginning of loop (read next input record)
*	do for each byte of the input physical record
*	   do forever (increment input and output # bytes for each iteration)
*	      if output block size is reached then
*		 call reblk_prewrite to write output buffer to output device
*	      if output logical record size reached then
*		 increment input # byte to end of logical input record to 
*			truncate the record.
*	      if input logical record size reached then
*                if output logical record length NOT reached then
*		    call reblk_pad to pad the logical record
*		 reset input and output logical record length counts to zero
*		 break out of "do forever" loop
*	      if input physical record size reach then
*		 break out of "do forever" loop
*          end do forever
*	end do
*	if data in output buffer then
*	   call reblk_prewrite to write output buffer to output device
*/
for (currec=0; currec < nrecords; currec++,indev->cur_rec++)
   {
   blksiz = read_device(indev,outdev,inbuf);
   if (blksiz <= 0)
      break;

   if (indev->cc == CARRIAGE)
      *(inbuf + blksiz - 1) = pad_val;

   if (trans_flag)
      transform(inbuf,&blksiz,&swap,trans);

   if (outdev->rectyp == VAR) 
        {
	if (blksiz <= outdev->recsiz)
           {
           for(intot=0; intot<blksiz; intot++)
              *(outbuf + intot) = *(inbuf + intot);
           reblk_prewrite(indev,outdev,outbuf,&blksiz);
           }
        else
           {
           for (intot=0;intot<blksiz;)
              {
              for (; ; intot++,outtot++)
                 {
                 if (outdev->cc == CARRIAGE)
                   {
                   if (outtot == (outdev->blksiz - 1))
                      {
                      reblk_prewrite(indev,outdev,outbuf,&outdev->blksiz);
                      outtot = 0;
 		      } 

                   if (intot == (blksiz - 1))
                      {
                      outtot++;
                      reblk_prewrite(indev,outdev,outbuf,&outtot);
                      outtot = 0;
                      intot++;
                      break;
                      }
                   }
                 else
                   {
                   if (outtot == outdev->blksiz)
                      {
                      reblk_prewrite(indev,outdev,outbuf,&outtot);
                      outtot = 0;
 		      } 

                   if (intot == blksiz)
                      {
                      reblk_prewrite(indev,outdev,outbuf,&outtot);
                      outtot = 0;
                      break;
                      }
                   }

                 *(outbuf + outtot) = *(inbuf + intot);
                 }
              }
           }
        }
      else
        {
	if (blksiz <= outdev->recsiz)
           {
           for(intot=0; intot<blksiz; intot++)
              *(outbuf + intot) = *(inbuf + intot);
           if (outdev->cc == CARRIAGE)
              outtot = intot + 1;
           else
              outtot = intot;
           reblk_pad(indev,outdev,outbuf,pad_val,&intot,&outtot);
           outtot = 0;
           }
        else
           {
           for (intot=0;intot<blksiz;)
              {
              for (; ; incur++,intot++,outcur++,outtot++)
               {
               if(((outcur == (outdev->blksiz - 1)) && (outdev->cc == CARRIAGE))
		  || ((outcur == outdev->blksiz) && (outdev->cc != CARRIAGE)))
                    {
                    if (outdev->cc == CARRIAGE)
                        outcur++;
                    reblk_prewrite(indev,outdev,outbuf,&outcur);
                    if (intot != (blksiz- 1))
                       outcur = incur = 0;
                    else
                       {
                       outcur = incur = 0;
                       break;
                       }
                    }
                 else   
                 if (((intot == (blksiz - 1)) && (outdev->cc == CARRIAGE))
		     || ((intot == blksiz) && (outdev->cc != CARRIAGE)))
                    {
                    if (outcur < outdev->recsiz)
                      {
                      if (outdev->cc == CARRIAGE)
                          outcur--;
                      reblk_pad(indev,outdev,outbuf,pad_val,&outcur,&incur);
                      }
                    outtot = outcur = incur = 0;
                    intot++;
                    break;
                    }
                 else
                 if (intot == blksiz)
                      break;
                 *(outbuf + outcur) = *(inbuf + intot);
                 }
              }
           }
        }
   }

if (outtot > 0)
   {
   if ((outcur > 0) && (outcur < outdev->recsiz))
      reblk_pad(indev,outdev,outbuf,pad_val,&outcur,&outtot);

   reblk_prewrite(indev,outdev,outbuf,&outtot);
   }

if (blksiz <= 0)
   indev->cur_file++;			/* update device attributes          */
outdev->cur_file++;

c_stoptm();				/* Disable processing msg utility    */

/*
  If the output device is tape write EOF marks
*/
#ifndef vms
if (outdev->device == TAPE)
   {
   status = c_tpweof(&otape.unit,&one,&mode);
   if (status != TAPE_SUCC)
	ditto_error("OUTPUT",DITTO_WEOF,ERROR_WARN,UPDATE_FD);
   }
#endif

return;
}

/******************************************************************************
FUNCTION:	REBLK_PAD

PURPOSE: 	Pads a logical record with a specified padding value.
		The pad value is of data type byte.

ALGORITHM 
  Pad the output logical record until the output logical record size is 
  reached.
	If the output physical record size is reached (block size)
		the output record is written to the output device

ALGORITHM REFERENCES	none
*******************************************************************************/
void reblk_pad
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    unsigned char *outbuf,	/* output buffer to pad              */
    long pad_val,		/* the pad value                     */
    long *outcur,		/* current output byte               */
    long *outtot		/* total number of output bytes      */
)
{
/*
  Pad the output logical record until the output logical record size is 
  reached.  If the output physical record size is reached (block size), then
  the output record is written to the output device.
*/
if (outdev->cc != CARRIAGE)
    {
    outdev->recsiz++;
    }
for ( ; *outcur < outdev->recsiz ; (*outcur)++,(*outtot)++)
   {
   if (*outtot == outdev->blksiz)
      {
      if (outdev->cc != CARRIAGE)
    	 outdev->recsiz--;
      reblk_prewrite(indev,outdev,outbuf,outtot);
      }

   *(outbuf + *outcur) = (unsigned char) pad_val;
   }

return;
}

/******************************************************************************
FUNCTION:	REBLK_PREWRITE

PURPOSE: 	Prepares the output buffer for writing to the output device.
		If the output carriage control is CARRIAGE, a new line
		character is inserted at the end of the buffer.  The buffer
		is then written to the output device.  

ALGORITHM 
    If output carriage control is CARRIAGE
	 new line is inserted
    Write buffer to output device
    Increment output current record 
    Reset outtot to zero

ALGORITHM REFERENCES	none
*******************************************************************************/
void reblk_prewrite
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    unsigned char *outbuf,	/* output buffer                     */
    long *outtot		/* total # bytes in output record    */
)
{
if (outdev->cc == CARRIAGE)		/* if output carriage control is     */
   *(outbuf + *outtot - 1) = '\n';	/*    CARRIAGE, new line inserted    */

write_device(indev,outdev,outbuf,*outtot); /* write buffer to output device  */

outdev->cur_rec++;			/* increment output current record   */

return;
}
