/******************************************************************************
FUNCTION:	TRANSFORM

PURPOSE: 	Calls the appropriate pixel manipulation routine to perform
		the desired transformation.  Transformation that are allowed
		are:
		    1)  convert ascii to ebcidic
		    2)  convert ebcidic to ascii
		    3)  convert lower case to upper case (assumes ascii data)
		    4)  convert upper case to lower case (assumes ascii data)
		    5)  perform byte swapping
		    6)  convert to high bit on (PRIMOS ASCII)
		    7)  convert to high bit off (non PRIMOS ASCII)

ALGORITHM 
  Set initial success flag
  If the number of bytes in entity to be swapped not zero
	perform byte swapping
  If transformation flag is "ASTOEB"
	convert ascii to ebcidic
  If transformation flag is "EBTOAS"
	convert ebcidic to ascii
  If transformation flag is "UP2LOW"
	convert upper case to lower case/
  If transformation flag is "LOW2UP"
	convert lower case to upper case 
  if trransformation flag is "HION"
	convert to high bit on
  if transformation flag is "HIOFF"
	convert to high bit off

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "pixman.h"
#include "util.h"
#include "ditto.h"			/* ditto include file                */
#include "copy.h"
#include "protos.h"

static void hion
(
    unsigned char *bufptr,
    long *nbyte
);

static void hioff
(
    unsigned char *bufptr,
    long *nbyte
);


void transform
(
    unsigned char *bufptr,	/* buffer to perform transformation on*/
    long *nbyte,		/* number of bytes in buffer          */
    long *swap,		/* # bytes in entity to be swapped    */
    char *trans		/* type of transformation             */
)
{
long status;			/* function return status code        */
long ns;				/* number of entities to be swapped   */

status = E_SUCC;			/* Set initial success flag	      */

/*
  If the number of bytes in entity to be swapped not zero perform byte swapping
*/
if (*swap > 0)
   {
   /*
     Perform byte swapping
   */
   ns = *nbyte / *swap;
   status = c_pxswap(bufptr,&ns,swap);
   if (status != E_SUCC)
      ditto_error("INPUT\0",DITTO_SWAP,ERROR_VERIFY,UPDATE_FD);
   }

/*
  Convert ascii to ebcidic
*/
if (strcmp(trans,"ASTOEB") == 0)
   c_asc2eb((char *) bufptr,nbyte);

/*
  Convert ebcidic to ascii
*/
if (strcmp(trans,"EBTOAS") == 0)
   c_eb2asc((char *) bufptr,nbyte);

/*
  Convert uppercase to lower case
*/
if (strcmp(trans,"UP2LOW") == 0)
   c_up2low((char *) bufptr,nbyte);

/*
  Convert lower case to upper case
*/
if (strcmp(trans,"LOW2UP") == 0)
   c_low2up((char *) bufptr,nbyte);

if (strcmp(trans,"HION") == 0)
   hion(bufptr,nbyte);

if (strcmp(trans,"HIOFF") == 0)
   hioff(bufptr,nbyte);
return;
}

/* 
  HION sets the most significant bit of each byte in the buffer to 1.  This
  is a useful translation when converting non-PRIMOS ASCII data into PRIMOS
  ASCII data.
*/
static void hion
(
    unsigned char *bufptr,
    long *nbyte
)
{
long cnt;
unsigned char *ptr;

ptr = bufptr;
for (cnt=0; cnt < *nbyte; cnt++,ptr++)
    *ptr |= 0x80;	/* set high bit to 1, leave all other bits the same */
return;
}

/* 
  HIOFF sets the most significant bit of each byte in the buffer to 0.  This
  is a useful translation when converting PRIMOS ASCII data into non-PRIMOS
  ASCII data.
*/
static void hioff
(
    unsigned char *bufptr,
    long *nbyte
)
{
long cnt;
unsigned char *ptr;

ptr = bufptr;
for (cnt=0; cnt < *nbyte; cnt++,ptr++)
    *ptr &= 0x7f;	/* set high bit to 0, leave all other bits the same */
return;
}
