/******************************************************************************

FUNCTION:	DUMP

PURPOSE: 	Executable code for the DUMP DITTO command.  The dump
		command dumps the file contents of a specified DITTO
		device.  The dump may be printed to the terminal or
		print file in ASCII, EBCIDIC, or hexidecimal format.
		If the output is to be printed to a line printer, the
		TAE proc FILEUTIL is invoked.

	NOTE:  To dump the file in EBCIDIC format, the records are converted
	to ASCII format and then outputted to the appropriate destination.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0       1/86    K. Gacke		initial development
  1.1	    7/86    K. Gacke		LAS standards for PRINT parameter
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1	    9/92    T. Mittan		Made use of tapeid to include tapeid
				 	in printout.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	The input parameters are retrieved from the TAE parameter block.  The
	appropriate DITTO device is opened for read access.  The specified
	number of physical records are read and dumped to the terminal or
	print file.  The dumped file may be formated in ASCII, EBCIDIC,
 	or hexidecimal format.  

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

#define ALL      2147483647		/* all records of file to be dumped  */
#define TERM	0			/* index for terminal		     */
#define FIL	1			/* index for file		     */
#define LP	2			/* index for line printer	     */

struct PARBLK   vblock;			/* TAE parameter block               */
struct DISK_FD  disk;			/* disk file descriptor              */
struct TAPE_FD  tape;			/* tape file descriptor              */
struct DITTO_DEV ditto_device;		/* ditto I/O devices descriptors     */

int main
(void)
{
FILE      *lp_fp = NULL;			/* file pointer to line printer file */

long	  bytes_read;			/* number of bytes read              */
long	  currec;			/* current record being dumped       */
long	  nrecords;			/* number of records to be scanned   */
long	  status;			/* function return status code       */

unsigned char     *buf;			/* pointer to buffer storage         */

static char out[1][CMLEN];		/* output files for completion msg   */
static char prtfile[CMLEN];		/* Printer working file		     */
static char filename[CMLEN];		/* Output print file name	     */
char device[10];			/* DITTO device (tape or disk)       */
char header[100];			/* header message of output          */
char which[10];				/* which device (input or output     */
char tapeid[CMLEN];

long ebc_fmt = FALSE;			/* is file to be dumped in ebcidic   */
long hex_fmt  = FALSE;			/* is file to be dumped in hex       */
long print[3];				/* Flag for term, lp, or file output */

long nonfatal = 1;			/* Non-fatal error code	 	     */
long fatal = -1;			/* Fatal error code		     */
long outcnt = 0;			/* Number of output files	     */
long flag = 0;				/* PTERM flag			     */

/*
  Retrieve input parameters and, set boolean variables for the dump format
  (hex, ascii, ro ebcidic) and destination of the output (line printer,
  terminal or both)
*/
getpar(which,device,&nrecords,&ebc_fmt,&hex_fmt,tapeid);

/*
  Get the working output print file
*/
strcpy(prtfile,"dump");
status = c_setprt(&vblock,&print[0],&print[1],&print[2],prtfile,filename);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto dump-fatal",&fatal);

/*
  Open the temporary print file if going to the line printer or a file
*/
if (print[LP] || print[FIL])
    {
    if ((lp_fp = fopen(prtfile,"a+")) == NULL)
	{
	c_errmsg("Error opening the print file","ditto dump-open",&nonfatal);
	}

    /*
      Hold onto the print host file name so clean up can be done when an
      error occurs
    */
    strcpy(out[0],prtfile);
    outcnt=1;
    }

ditto_device.input_dev = NULL;		/* set input and output devices to   */
ditto_device.output_dev = NULL;		/*   no values                       */

/*
  The DITTO device is opened and buffer space allocated to read records into
*/
buf = open_device(which,device,header,tapeid);

/*
  Write header to the output file if specified
*/
if (print[LP] || print[FIL])
    fprintf(lp_fp,"\f%s\n",header);

/*
  Write header to the terminal
*/
if (print[TERM])
    c_pterm(header,&flag);

/*
  The specified number of records are read, and the appropriate routine is
  called to format the dump for the appropriate output destination.
*/
for (currec = 0; currec < nrecords; currec++)
   {
   bytes_read = read_device(device,currec,buf);
   if (bytes_read == 0)
      break;

   if (ebc_fmt)
      c_eb2asc((char *) buf,&bytes_read);

   if (hex_fmt)
      {
      if (print[LP] || print[FIL])
         lp_hex(lp_fp,buf,bytes_read,currec);

      if (print[TERM])
         term_hex(buf,bytes_read,currec);
      }
   else
      {
      if (print[LP] || print[FIL])
         lp_char(lp_fp,buf,bytes_read,currec);

      if (print[TERM])
         term_char(buf,bytes_read,currec);
      }
   }

if (print[LP] || print[FIL])
   {
   fflush(lp_fp);
   fclose(lp_fp);
   }
   
/*
  The DITTO device is closed 
*/
close_device(which,device,currec,bytes_read);

if ((nrecords != ALL) && (currec < nrecords))
   ditto_error(" ",DITTO_EOF,ERROR_WARN,UPDATE_FD);

/*
  Add the output print file to the list of created files.
*/
if (print[FIL])
    {
    strcpy(out[0],filename);
    outcnt = 1;
    }
else
    outcnt=0;

c_complt("DITTO DUMP",out,&outcnt);

return EXIT_SUCCESS;

}
/******************************************************************************
FUNCTION:	GETPAR

PURPOSE: 	Retrieves the input parameters and, set boolean variables for
		the dump format (hex, ascii, or ebcidic)

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
void getpar
(
    char *which,			/* which device (input or output)    */
    char *device,			/* device type (tape or disk)        */
    long *nrecords,			/* number of records to be dumped    */
    long *ebc_fmt,			/* is file dumped in ebcidic format  */
    long *hex_fmt,			/* is file dumped in hex format      */
    char *tapeid
)
{
extern struct PARBLK vblock;		/* TAE parameter block               */

struct VARIABLE *val;			/* pointer to TAE variable           */

long	std_out;			/* Fortran LUN for standard output   */

c_inlas(&vblock,&std_out);		/* initialize TAE                    */

val = p_find(&vblock,"devtype");	/* retrieve parameter "devtype"      */
strcpy(which,SVAL(*val,0));

val = p_find(&vblock,"nrecs");	 	/* retrieve parameter "nrecs"        */
if (val->v_count == 0)
   *nrecords = ALL;
else
   *nrecords = IVAL(*val,0);

val = p_find(&vblock,"dataform");	/* retrieve parameter "dataform"     */
if (strcmp(SVAL(*val,0),"HEX") == 0)	/*   dump file in hex format         */
   *hex_fmt  = TRUE;
else					/*   dump file in ascii format       */
   *hex_fmt = FALSE;

if (strcmp(SVAL(*val,0),"EBCIDIC") == 0)/*   records need to be converted    */
   *ebc_fmt = TRUE;			/*     from ebcidic to ascii         */
else					/*   no conversion is required       */
   *ebc_fmt = FALSE;

if (strcmp(which,"INPUT") == 0)       	/* retrieve input DITTO device       */
   {
   val = p_find(&vblock,"idevice");
   if (val->v_count == 0)
      ditto_error(which,DITTO_ALLOC,ERROR_SEV,NO_UPDATE_FD);

   strcpy(device,SVAL(*val,0));
   val = p_find(&vblock,"itapeid");	/* retrieve parameter "devtype"      */
   strcpy(tapeid,SVAL(*val,0));
   }
else				      	/* retrieve output DITTO device      */
   {
   val = p_find(&vblock,"odevice");   
   if (val->v_count == 0)
      ditto_error(which,DITTO_ALLOC,ERROR_SEV,NO_UPDATE_FD);

   strcpy(device,SVAL(*val,0));
   val = p_find(&vblock,"otapeid");	/* retrieve parameter "devtype"      */
   strcpy(tapeid,SVAL(*val,0));
   }

return;
}
