/******************************************************************************
FUNCTION:	OPEN_DEVICE

PURPOSE: 	Open the appropriate DITTO device that is to be dumped.  A
		header line that is to be written for each page of ouput is
		assigned to input parmater "header".  It also allocates buffer
		space to read records into.  The address of the buffer is			returned.

ALGORITHM 
  If device is a disk file
     Get the disk file attributes
     Open the disk file
     Allocate buffer space
     Print the header and file name
  Else Device is a tape
     Reopen the tape
     Alloc buffer memory
     Print header and drive name

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>  			/* C standard I/O include file       */
#include <string.h>
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

unsigned char *open_device
(
    char *which,			/* which device (input or output)    */
    char *device,			/* device type (tape or disk)        */
    char *header,			/* header message of each output page*/
    char *tapeid
)
{
extern struct DISK_FD disk;		/* Disk device structure	     */
extern struct TAPE_FD tape;		/* Tape device structure	     */
unsigned char   *buf;			/* Buffer			     */

long status;				/* Status variable		     */
long fatal = -1;			/* Fatal error code		     */ 

char file_name[CMLEN];			/* File name			     */
char ext[CMLEN];			/* Extention location		     */
char *adir;				/* address of directory within fname */
char *aext;				/* address of directory within fname */

/*
  If device is a disk file
*/
if (strcmp(device,"DISK") == 0)
   {
   status = get_disk_fd(which,&disk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto dump-fatal",&fatal);

   /*
     Get the disk file attributes
   */
   status = dstat(&disk,0);
   if (status == E_FAIL)
      {
      ditto_error(which,DITTO_DSTAT,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto dump-fatal",&fatal);
      }

   /*
     Open the disk file
   */
   status = dopen(&disk,DREAD);
   if (status == E_FAIL)
      {
      ditto_error(which,DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto dump-fatal",&fatal);
      }

   /*
     Allocate buffer space
   */
   buf = (unsigned char *) malloc(disk.blksiz);
   if (buf == 0)
      {
      ditto_error(which,DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto dump-fatal",&fatal);
      }

/*
  Check the current file and append it to the file name
*/
   strcpy(file_name,disk.fname);
   if (disk.cur_file != NOVAL)
	{
	adir = strrchr(file_name,HOST_DIR_END);
	if (adir == 0)
	    adir = file_name;
	aext = strchr(adir,HOST_EXT);

	if (disk.cur_file == NOVAL)
	    disk.cur_file = 1;

	if (aext == 0)
	    sprintf(file_name,"%s_%d",disk.fname,disk.cur_file);
	else
	    {
	    strcpy(ext,aext);
	    sprintf(aext,"_%d%s",disk.cur_file,ext);
	    }
	}

   /*
     Print the header and file name
   */
   sprintf(header,"********** Dump of disk file %s **********",file_name);
   }
else 				/* Device is a tape */
   {
   /*
     Reopen the tape
   */
   status = reopen_tape(which,NOREW,TREAD,&tape);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","ditto dump-fatal",&fatal);

   /*
     Alloc buffer memory
   */
   buf = (unsigned char *) malloc(tape.blksiz);

   /*
     Print header and drive name
   */
   sprintf(header,"********** Dump of tape %s mounted on DRIVE %s **********",
           tapeid,tape.drive);
   }

return(buf);
}
