/******************************************************************************
FUNCTION:	MAP_TAPE

PURPOSE:	Does a mapper on the specifed magnetic tape.  If the
		generated output is to be printed on the line printer,
		it writes the output to the disk file "mapper.prt".  If
		the generated output is to be printed to the terminal,
		it calls subroutine outmsg for each generated line of
		output.  If the generated output is to be printed to the
		terminal, it calls subroutine outmsg for each generated
		line of output.


ALGORITHM 
  Reopen the tape device
  Alloc buffer space
  Write out mapper header
  Read physical records from the tape until the end of volume is reached.
	For each file on tape there are two counters to keep track of
	the number of consecutive records with the same record size,
	and to keep track of the total number of records in the current
	file.
	    The output is written to the terminal, line printer
	    (file "mapper_time.prt"), or both depending on input
	     parameter "print".
  Close and rewind the tape

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <time.h>
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void map_tape
(
    char *which,			/* DITTO device (input or output)    */
    long *print,			/* destination of generated output   */
    FILE *lp_fp,			/* file pointer to line printer      */
    char *tapeid
)
{
struct TAPE_FD tape;			/* DITTO tape descriptor             */
struct TAPEDESC *tp;            	/* tape drive description */

long  bufsiz;				/* maximum block size of a tape rec  */
long  eof_count = 0;			/* number of consecutive EOF marks   */
long  file;				/* number of files                   */
long  last = 0;				/* number of bytes in last record    */
long  nonfatal = 1;			/* Non-fatal error code		     */
long  fatal = -1;			/* Fatal error code		     */
long  mode = HANDLE;			/* tape I/O package output error msg */
long  nbyte;				/* number of bytes read              */
long  rec;				/* count of records with equal size  */
long  status;				/* function return status code       */
long  totrec;				/* total number of records in file   */
long  trectape=0;			/* total number of records on tape   */

unsigned char *bufptr;			/* buffer to read tape records into  */
char  msg[ERRLEN];			/* buffer for generated output       */

long start, stop;                       /* start and stop time               */
float total_bytes=0.0;                  /* total bytes read                  */
float bytes_read;                       /* counter for bytes read            */

/*
  Reopen the tape device
*/
status = reopen_tape(which,REWIND,TREAD,&tape);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","ditto mapper-fatal",&fatal);

/*
  Alloc buffer space
*/
tp = (struct TAPEDESC *) tape.unit;
bufsiz = tp->maxbyte;
bufptr = malloc((bufsiz + 1));
if (bufptr == 0)
   {
   ditto_error(which,DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto mapper-fatal",&fatal);
   }

/*
  Write out mapper header
*/
sprintf(msg,"********** Mapper of tape %s mounted on DRIVE %s **********",
        tapeid,tape.drive);
outmsg(msg,print,lp_fp);


/*
  Start timer
*/
 start = time(NULL);


/*
  Read physical records from the tape until the end of volume is reached.
  For each file on tape there are two counters to keep track of the number 
  of consecutive records with the same record size, and to keep track of the 
  total number of records in the current file.  The output is written to the 
  terminal, line printer (file "mapper_time.prt"), or both depending on input 
  parameter "print".
*/
for (file=1;;file++)
   {
   for (rec=0,totrec=0;;totrec++)
      {
      /*
	Read a physical record from the tape
      */
      nbyte = c_tpread(&tape.unit,bufptr,&bufsiz,&mode);
      if ((nbyte == TAPE_EOF) || (nbyte == TAPE_EOV) || (nbyte == TAPE_EOT))
	 break;

      if (nbyte < 0)
         {
         sprintf(msg,"Read %d records",totrec);
         c_errmsg(msg,"ditto mapper-dread",&nonfatal);
	 ditto_error(which,DITTO_TPREAD,ERROR_VERIFY,UPDATE_FD);
         }

      if ((nbyte != last) && (rec != 0))
	 {
         sprintf(msg,"  %6d RECORDS %6d BYTES LONG",rec,last);
         outmsg(msg,print,lp_fp);
         rec = 0;
         }

      ++rec;
      last = nbyte;
      bytes_read = ((float) last) / 1000.0;
      total_bytes += bytes_read; 
      }

   if ((nbyte == 0) && (rec == 0))	/* VMS does not detect EOV           */
      {
      ++eof_count;
      if (eof_count >= 2)
         {
 	 nbyte = TAPE_EOV;
         c_errmsg("Two end of file marks encountered","ditto mapper-teof",
		&nonfatal);
	 break;
         }

      continue;
      }
/* 
   Total the records on the tape
*/
   trectape =trectape + totrec;

   sprintf(msg,"  %6d RECORDS %6d BYTES LONG",rec,last);
   outmsg(msg,print,lp_fp);

   sprintf(msg,"END OF FILE #%d >>>>> %d TOTAL RECORDS.",file,totrec);
   outmsg(msg,print,lp_fp);
   outmsg(" ",print,lp_fp);
   outmsg(" ",print,lp_fp);
   outmsg(" ",print,lp_fp);

   if ((nbyte == TAPE_EOV) || (nbyte == TAPE_EOT))
      break;
   }

/*
  Stop timer
*/
stop = time(NULL); 



if (nbyte == TAPE_EOV)
   strcpy(msg,"**********  END OF VOLUME  **********");
else
   strcpy(msg,"**********  END OF TAPE  **********");
outmsg(msg,print,lp_fp);
outmsg(" ",print,lp_fp);
outmsg(" ",print,lp_fp);

sprintf(msg,"--------------------------------");
outmsg(msg,print,lp_fp);
if (nbyte == TAPE_EOV)
   sprintf(msg,"       %d RECORDS IN VOLUME.",trectape);
else
   sprintf(msg,"       %d RECORDS ON TAPE.",trectape);
outmsg(msg,print,lp_fp);

/*
 Output transfer rate
*/

sprintf(msg,"Transfer rate is %d k/sec", ((unsigned long)total_bytes)/(stop-start));
outmsg(msg,print,lp_fp);


tape.cur_rec = 0;
tape.cur_file = 1;

/*
  Close and rewind the tape
*/
close_tape(which,&tape,REWIND,1);

/*
  Free buffer space
*/
free(bufptr);

return;
}
