/******************************************************************************
 
FUNCTION:	MAPPER

PURPOSE:	Executable part of the MAPPER DITTO command.  The mapper
		command produces a summary of the files, records, and
		record lengths on the specifed device.  If the mapper
		command is run on a magnetic tape, the tape is rewound
		before processing begins and rewound to beginning of
		tape at completion.  If the mapper command is run on a 
		disk file, processing begins at the beginning of the
		file, and after completion the file pointer is returned
		to beginning of file.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      10/85    K. Gacke    	initial development
  1.1	    5/86    K. Gacke		allow for multiple disk files
  1.2	    6/86    K. Gacke		LAS standards for PRlong parameter
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1	    2/92    T. Mittan	 	Array Bufptr needed 1 more cell for
					last character.	
  5.2	    9/92    T. Mittan		Made use of tapeid to include tapeid
					in printout.
  5.3       9/93    D.Etrheim           Changed so the maximum number of bytes 
					to be read from tape is based on 
					TAPEDESC rather than MAXTAPEWT
  5.4      12/98    D. Hames            Output transfer rate to the mapper. This will 
					give the user a guide for deciding the 
					density of a tape.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	From input parameter WHICH, it is determined which device the mapper
	is to be run on.  If the mapper command is run on a magnetic tape, the 
	tape is rewound before processing begins and rewound to beginning of 
	tape at completion.  If the mapper command is run on a disk file, 
	processing begins at the beginning of the file, and after completion 
	the file pointer is returned to beginning of file.  

	From input parameter PRlong, it is determined where the generated output
	is to be written.  If output is to be written to the terminal, it is
	done so by the executable code.  If output is to be written to the line
	printer, the executable code writes the output to the file named
	"MAPPER.PR" which is then inputted into the TAE proc FILEUTIL.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"
 
#define TERM	0			/* index for terminal		     */
#define FIL	1			/* index for file		     */
#define LP	2			/* index for line printer	     */

struct PARBLK   vblock;		/* TAE parameter block               */
struct DITTO_DEV ditto_device;	/* ditto I/O devices descriptors     */

int main
(void)
{
struct VARIABLE *val;			/* pointer to TAE variable           */

FILE *lp_fp = NULL;				/* file pointer to line printer      */

long    status;				/* Status variable		     */
long	std_out;			/* standard output -- Fortran LUN    */
long	outcnt;				/* Number of output files	     */
long	print[3];			/* Print destinations		     */
long    fatal = -1;			/* Fatal error code		     */
long    nonfatal = 0;			/* Non-Fatal error code		     */

static char out[1][CMLEN];		/* output files for completion msg   */
static char prtfile[CMLEN]; 		/* Printer working file		     */
static char filename[CMLEN];		/* Output print file name	     */
char    *device;			/* disk or tape DITTO device         */
char	*which;				/* input or output DITTO device      */
char	tapeid[CMLEN];

/*
  Initialize TAE
*/
c_inlas(&vblock,&std_out);

/*
  Set input and output devices to no value
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
  Retrieve device type
*/
val = p_find(&vblock,"devtype");
which = SVAL(*val,0);

/*
  Retrieve the input or output based on parameter "which"
*/
if (strcmp(which,"INPUT") == 0)
   {
   val = p_find(&vblock,"itapeid");
   strcpy(tapeid,SVAL(*val,0));
   val = p_find(&vblock,"idevice");
   }
else
   {
   val = p_find(&vblock,"otapeid");
   strcpy(tapeid,SVAL(*val,0));
   val = p_find(&vblock,"odevice");   
   }

if (val->v_count == 0)
    {
    ditto_error(which,DITTO_ALLOC,ERROR_WARN,NO_UPDATE_FD);
    c_errmsg("Fatal error encountered","ditto mapper-fatal",&fatal);
    }
device = SVAL(*val,0);

/*
  Get the working output print file
*/
strcpy(prtfile,"mapper");
status = c_setprt(&vblock,&print[0],&print[1],&print[2],prtfile,filename);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto mapper-fatal",&fatal);

/*
  Open the temporary print file if going to the line printer or a file
*/
if (print[LP] || print[FIL])
    {
    if ((lp_fp = fopen(prtfile,"a+")) == NULL)
	{
	c_errmsg("Error opening the print file","ditto mapper-open",&nonfatal);
	}

    /*
      Hold onto the print host file name so clean up can be done when an
      error occurs
    */
    strcpy(out[0],prtfile);
    outcnt = 1;
    }

if (strcmp(device,"DISK") == 0)		/* if disk file call disk mapper     */
   map_disk(which,print,lp_fp);
else					/* if tape call tape mapper          */
   map_tape(which,print,lp_fp,tapeid); 

/*
  Add the print file name to the list of output files created
*/
if (print[FIL])
    {
    strcpy(out[0],filename);
    outcnt = 1;
    }
else
    outcnt = 0;

c_complt("DITTO MAPPER",out,&outcnt);

return EXIT_SUCCESS;

}
