/******************************************************************************
FUNCTION:	OPEN_DEVICE

PURPOSE:	Opens the input or output DITTO device.  It also retrieves
		the file attributes of the DITTO device from the TAE param-
		eter block and allocates buffer space to read physical
		records into.  The pointer to the buffer space is returned
		to the calling routine.

ALGORITHM 
  If device is a disk file
     Retrieve the file attributes
     Insure that the file exists
     Open the disk file
     Allocate buffer space
     Write header and file name
  Else device is a tape
     Reopen the tape device
     Allocate buffer space
     Write header and drive
  Return pointer to buffer space

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <string.h>
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

unsigned char *open_device
(
    char *which,			/* which device (input or output)    */
    char *device,			/* device type  (tape or disk)       */
    char *header,			/* header line for report            */
    char *tapeid
)
{
extern struct DISK_FD disk;		/* disk file descriptor              */
extern struct TAPE_FD tape;		/* tape descriptor                   */
unsigned char   *buf;			/* buffer pointer to be returned     */

long     status;			/* function return status code       */
long     fatal = -1;			/* Fatal error code		     */

char file_name[CMLEN];			/* File name			     */
char ext[CMLEN];			/* Extention location		     */
char *adir;				/* address of directory within fname */
char *aext;				/* address of directory within fname */

/*
  Device is a disk file
*/
if (strcmp(device,"DISK") == 0)
   {
   /*
     Retrieve the file attributes
   */
   status = get_disk_fd(which,&disk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto scan-fatal",&fatal);

   /*
     Insure that the file exists
   */
   status = dstat(&disk,0);
   if (status == E_FAIL)
      {
      ditto_error(which,DITTO_DSTAT,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto scan-fatal",&fatal);
      }

   /*
     Open the disk file
   */
   status = dopen(&disk,DREAD);
   if (status == E_FAIL)
      {
      ditto_error(which,DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto scan-fatal",&fatal);
      }

   /*
     Allocate buffer space
   */
   buf = (unsigned char *) malloc(disk.blksiz);

/*
  Check the current file and append it to the file name
*/
   strcpy(file_name,disk.fname);
   if (disk.cur_file != NOVAL)
	{
	adir = strrchr(file_name,HOST_DIR_END);
	if (adir == 0)
	    adir = file_name;
	aext = strchr(adir,HOST_EXT);

	if (disk.cur_file == NOVAL)
	    disk.cur_file = 1;

	if (aext == 0)
	    sprintf(file_name,"%s_%d",disk.fname,disk.cur_file);
	else
	    {
	    strcpy(ext,aext);
	    sprintf(aext,"_%d%s",disk.cur_file,ext);
	    }
	}

   /*
     Write header and file name
   */
   sprintf(header,"********** Scan of disk file %s **********",file_name);
   }
else					/* Device is a tape		*/
   {
   /*
     Reopen the tape device
   */
   status = reopen_tape(which,NOREW,TREAD,&tape);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto scan-fatal",&fatal);

   /*
     Allocate buffer space
   */
   buf = (unsigned char *) malloc(tape.blksiz);

   /*
     Write header and drive
   */
   sprintf(header,"********** Scan of tape %s mounted on DRIVE %s **********",
           tapeid,tape.drive);
   }

/*
  Return pointer to buffer space
*/
return(buf);				/* return pointer to buffer space    */
}
