/******************************************************************************
 
FUNCTION:	SCAN

PURPOSE: 	Executable code for the SCAN DITTO command.  The scan
		command scans a specified number of physical records
		within a file for a specified string of bytes.  If a
		match is found, the record and byte offset are printed
		to the terminal or print file.  The scan string may be
		specified in character or hexidecimal format.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0       1/86    K. Gacke		initial development
  1.1	    7/86    K. Gacke		LAS standards for PRINT parameter
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1	    9/92    T. Mittan		Made use of tapeid parameter to
					display the tapeid.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	From input parameter WHICH, it is determined which device the scan
	is to be run on.  The scan string is converted from the character or
	hexidecimal format into a byte valued array.  The appropriate DITTO 
	device is opened.  Physical records are read until an EOF or the
	number of records specified is reached.  Each record is searched for
	the scan string starting at the specified offset.  If a match is found, 
	the record and byte offset are outputted.  Note that multiple matches 
	within records are found.  

	From input parameter PRINT, it is determined where the generated output
	is to be written.  If output is to be written to the terminal, it is
	done so by the executable code.  If output is to be written to the line
	printer, the executable code writes the output to the file named
	"SCAN_timecode.PR" and the PDF "scan" invokes the TAE proc FILEUTIL
	to print the file to the specified line printer.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"
 
#define ALL      2147483647		/* all records of file to be scanned */
#define TERM	0			/* print report to terminal screen   */
#define FIL	1			/* print report to disk file         */
#define LP	2			/* print report to line printer      */

struct PARBLK   vblock;		/* TAE parameter block               */
struct DISK_FD  disk;		/* disk file descriptor              */
struct TAPE_FD  tape;		/* tape file descriptor              */
struct DITTO_DEV ditto_device;	/* ditto I/O devices descriptors     */
FILE *lp_fp;			/* file pointer to line printer file */
char *scan;			/* scan string in char or hex format */
char *type;			/* specifies format of scan string   */

int main
(void)
{
extern FILE *lp_fp;			/* file pointer to line printer file */
long	bytes_read = 0;  	/* number of bytes read              */
long	currec;				/* current record being scanned      */
long    nmr_matches;			/* number of matches found in record */
long	nrecords;			/* number of records to be scanned   */
long	scan_length;			/* length of scan string             */
long	start;				/* starting offset of scan           */
long	status;				/* function return status code       */
long    outcnt;				/* Number of output files	     */
long	print[3];			/* output destination                */
long	total_matches = 0;		/* total number of matches found     */
long    fatal = -1;			/* Fatal error code		     */
long    nonfatal = 0;			/* Non-Fatal error code		     */

unsigned char   *buf;			/* pointer to buffer storage         */
unsigned char	*scan_string;		/* scan string                       */

static char out[1][CMLEN];		/* output files for completion msg   */
static char out_msg[100];		/* message to be outputted           */
static char hdr1[100];			/* header line one of report         */
static char hdr2[100];			/* header line two of report         */
static char header[100];		/* header line of report             */
static char prtfile[CMLEN];		/* Temporary print file name	     */
static char filename[CMLEN];		/* Host print file name		     */
static char device[10];			/* DITTO device (tape or disk)       */
static char which[10];			/* which device (input or output     */
char tapeid[CMLEN];

/*
  Get the inpur parameters
*/
getpar(which,device,&start,&nrecords,tapeid);

/*
  Get the working output print file
*/
strcpy(prtfile,"scan");
status = c_setprt(&vblock,&print[0],&print[1],&print[2],prtfile,filename);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto scan-fatal",&fatal);

/*
  Open the temporary print file if going to the line printer or a file
*/
if (print[LP] || print[FIL])
    {
    if ((lp_fp = fopen(prtfile,"a+")) == NULL)
	{
	c_errmsg("Error opening the print file","ditto scan-open",&nonfatal);
	}

    /*
      Hold onto the print host file name so clean up can be done when an
      error occurs
    */
    strcpy(out[0],prtfile);
    outcnt = 1;
    }

/*
  Set input and output devices to no values
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
*   Retrieve input parameters, convert the character/hexidecimal formatted 
*   scan string into a byte value array.  Read each record and determine if
*   any matches occur.
*   terminated.
*/
scan_string = compute_scan(&scan_length);
buf = open_device(which,device,header,tapeid);
sprintf(hdr1,"SCAN STRING = \"%s\"",scan);
sprintf(hdr2,"SCAN TYPE   = %s",type);

if (print[TERM])
   {
   status = t_highlight(header);
   status = t_highlight(hdr1);
   status = t_highlight(hdr2);
   }

outmsg(header,print,lp_fp);
outmsg(" ",print,lp_fp);
outmsg(hdr1,print,lp_fp);
outmsg(hdr2,print,lp_fp);
outmsg(" ",print,lp_fp);

for (currec = 0; currec < nrecords; currec++)
   {
   bytes_read = read_device(device,currec,buf);
   if (bytes_read == 0)
      break;

   nmr_matches = scanbuf(buf,start,bytes_read,currec,scan_string,
                          scan_length,print);
   total_matches += nmr_matches;
   }

free(buf);
sprintf(out_msg,"TOTAL NUMBER OF MATCHES --- %d",total_matches);

if (print[TERM])
   status = t_highlight(out_msg);

outmsg(" ",print,lp_fp);
outmsg(out_msg,print,lp_fp);

if ((print[LP]) || (print[FIL]))
   {
   fflush(lp_fp);
   fclose(lp_fp);
   }

if ((nrecords != ALL) && (currec < nrecords))
   ditto_error(" ",DITTO_EOF,ERROR_WARN,UPDATE_FD);

/*
  Close the device
*/
close_device(which,device,currec,bytes_read);

/*
  Add the output print file to the list of created files
*/
if (print[FIL])
    {
    strcpy(out[0],filename);
    outcnt = 1;
    }
else
    outcnt = 0;

c_complt("DITTO SCAN",out,&outcnt);

return EXIT_SUCCESS;

}

/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Retrieves the input parameters from the TAE parameter block.  

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
void getpar
(
    char *which,			/* which device (input or output)    */
    char *device,			/* device type  (tape or disk)       */
    long *start,			/* starting offset within record     */
    long *nrecords,			/* number of records to be processed */
    char *tapeid
)
{
extern struct PARBLK vblock;		/* TAE parameter block               */
extern char *scan;			/* scan string in char or hex format */
extern char *type;			/* specifies format of scan string   */

struct VARIABLE *val;			/* pointer to TAE variable           */

long    fatal = -1;			/* Fatal error code		     */
long	std_out;			/* Fortran LUN for standard output   */

c_inlas(&vblock,&std_out);		/* initialize TAE                    */

val = p_find(&vblock,"devtype");	/* retrieve parameter "devtype"      */
strcpy(which,SVAL(*val,0));

val = p_find(&vblock,"text");		/* retrieve parameter "text"         */
scan = SVAL(*val,0);

val = p_find(&vblock,"texttype");	/* retrieve parameter "texttype"     */
type = SVAL(*val,0);

val = p_find(&vblock,"offset");		/* retrieve parameter "offset"       */
*start = IVAL(*val,0);

val = p_find(&vblock,"nrecs");		/* retrieve parameter "nrecs"        */
if (val->v_count == 0)
   *nrecords = ALL;
else
   *nrecords = IVAL(*val,0);

if (strcmp(which,"INPUT") == 0)       	/* retrieve input DITTO device       */
   {
   val = p_find(&vblock,"itapeid");	/* retrieve parameter "devtype"      */
   strcpy(tapeid,SVAL(*val,0));
   val = p_find(&vblock,"idevice");
   }
else				      	/* retrieve output DITTO device      */
   {
   val = p_find(&vblock,"otapeid");	/* retrieve parameter "devtype"      */
   strcpy(tapeid,SVAL(*val,0));
   val = p_find(&vblock,"odevice");   
   }

if (val->v_count == 0)
   {
   ditto_error(which,DITTO_ALLOC,ERROR_WARN,NO_UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto scan-fatal",&fatal);
   }
strcpy(device,SVAL(*val,0));

return;
}
