/******************************************************************************
FUNCTION:	CLOSE_DEVICE

PURPOSE:	Closes the input and output devices currently allocated to
		the DITTO program.  It also updates the attributes of each
		device (i.e. current record and current file).

ALGORITHM 
  If input device is a disk file
     Update the current record and file attributes
     Update the attributes
     Close the input disk file
  Else if input device is a tape
     Update the current record and file attributes
     Close the input tape device

  If output device is a disk file
     Update the current record and file attributes
     Update the attributes
     Close the input disk file
  Else if output device is a tape
     Update the current record and file attributes
     Close the output tape device

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"			/* ditto include file                */
#include "verify.h"
#include "protos.h"

void close_device
(
    struct VERIFY_DEV *indev,		/* input DITTO device descriptor     */
    struct VERIFY_DEV *outdev		/* output DITTO device descriptor    */
)
{
extern struct DISK_FD idisk;		/* input disk file descriptor        */
extern struct DISK_FD odisk;		/* output disk file descriptor       */
extern struct TAPE_FD itape;		/* input tape file descriptor        */
extern struct TAPE_FD otape;		/* output tape file descriptor       */
long status;				/* function return status code       */

/*
  Device is a disk file
*/
if (indev->device == DISK)
   {
   /*
     Update the current record and file attributes
   */
   if ((indev->cur_file > 0) && (idisk.cur_file != NOVAL))
      {
      idisk.cur_file += indev->cur_file;
      idisk.cur_rec = 0;
      }
   else if (idisk.cur_rec != NOVAL)
      idisk.cur_rec += indev->cur_rec;

   /*
     Update the attributes
   */
   put_disk_fd("INPUT",&idisk);

   /*
     Close the input disk device
   */
   status = dclose(&idisk);
   if (status != E_SUCC)
	ditto_error("INPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }

else				/* Device is a tape device		*/
   {
   /*
     Update the current record and file attriutes
   */
   if (indev->cur_file > 0)
      {
      if (itape.cur_file != NOVAL)
         itape.cur_file += indev->cur_file;
      itape.cur_rec = 0;
      }
   else if (itape.cur_rec != NOVAL)
      itape.cur_rec += indev->cur_rec;

   /*
     Close the output device (no rewind)
   */
   close_tape("INPUT",&itape,NOREW,1);
   }

/*
  Output device is a disk device
*/
if (outdev->device == DISK)
   {
   /*
     Update current record and file attributes
   */
   if ((outdev->cur_file > 0) && (odisk.cur_file != NOVAL))
      {
      odisk.cur_file += outdev->cur_file;
      odisk.cur_rec = 0;
      }
   else if (odisk.cur_rec != NOVAL)
      odisk.cur_rec += outdev->cur_rec;

   /*
     Update attributes
   */
   put_disk_fd("OUTPUT",&odisk);

   /*
     Close the output disk device
   */
   status = dclose(&odisk);
   if (status != E_SUCC)
	ditto_error("INPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else				/* output device is a tape 	*/
   {
   /*
     Update current record and file attributes
   */
   if (outdev->cur_file > 0)
      {
      if (otape.cur_file != NOVAL)
         otape.cur_file += outdev->cur_file;

      otape.cur_rec = 0;
      }
   else if (otape.cur_rec != NOVAL)
      otape.cur_rec += outdev->cur_rec;

   /*
     Close the output device (no rewind)
   */
   close_tape("OUTPUT",&otape,NOREW,2);
   }

return;
}
