/******************************************************************************
FUNCTION:	FILE_VERIFY

PURPOSE:	Verifies the specified number of files on the input and output
		DITTO devices.  Since only one disk file can be allocated to
		a DITTO device at once, multiple files can only be processed
		if both DITTO devices are allocated to a magnetic tape.

ALGORITHM 
  Retrieve the number of files
  Retrieve the message time
  Open the input DITTO device
  Open the output DITTO device
  Verify each file
     Open the file
     Verify the current  file
     Increment the current input and output file counters
  Close the input DITTO device
  Close the output DITTO device

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"			/* ditto include file                */
#include "verify.h"
#include "protos.h"

void file_verify
(
    char *idevice,			/* input device (tape or disk)       */
    char *odevice			/* output device (tape or disk)      */
)
{
extern struct PARBLK vblock;		/* TAE parameter block               */
extern struct DISK_FD idisk;		/* input disk file                   */
extern struct DISK_FD odisk;		/* output disk file                  */
extern struct TAPE_FD itape;		/* input tape                        */
extern struct TAPE_FD otape;		/* output tape                       */

struct VARIABLE *val;			/* pointer to TAE parameters         */
struct VERIFY_DEV indev;		/* input device descriptor           */
struct VERIFY_DEV outdev;		/* output device descriptor          */

long  file;				/* current file being verified       */
long  msgtim;				/* # seconds between display messages*/
long  nfiles;				/* number of files to be verified    */
long  status;				/* function return status code       */
long  fatal = -1;			/* Fatal error code		     */

/*
  Retrieve the number of files
*/
val = p_find(&vblock,"nfiles");
if (val->v_count > 0)
   nfiles = IVAL(*val,0);
else
   nfiles = ALL;

/*
  Retrieve the message time
*/
val = p_find(&vblock,"$msgtim");
msgtim = IVAL(*val,0);

/*
  Open the input DITTO device
*/
if (strcmp(idevice,"TAPE") == 0)
   {
   status = reopen_tape("INPUT",NOREW,TREAD,&itape);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
   }
else
   {
   status = get_disk_fd("INPUT",&idisk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);

   if (idisk.cur_file == NOVAL)
      nfiles = 1;
   }

/*
  Open the output device
*/
if (strcmp(odevice,"TAPE") == 0)
   {
   status = reopen_tape("OUTPUT",NOREW,TREAD,&otape);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
   }
else
   {
   status = get_disk_fd("OUTPUT",&odisk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);

   if (odisk.cur_file == NOVAL)
      odisk.cur_file = 1;

   put_disk_fd("OUTPUT",&odisk);
   }

/*
  Verify each file
*/
for (file = 0; file < nfiles; file++)
   {
   /*
     Open the file
   */
   status = file_open(idevice,odevice,&indev,&outdev);
   if (status == VERIFY_EOV)
      break;

   /*
     Verify the current  file
   */
   status = verify_proc(&indev,&outdev,ALL,msgtim);

   /*
     Close the file
   */
   file_close(&indev,&outdev);

   /*
     Increment the current input and output file counters
   */
   ++indev.cur_file;
   ++outdev.cur_file;

   if (status == VERIFY_EOV)
      break;
   }

/*
  Close the input device
*/
if (strcmp(idevice,"TAPE") == 0)
   close_tape("INPUT",&itape,NOREW,1);
else
   put_disk_fd("INPUT",&idisk);

/*
  Close the output device
*/
if (strcmp(odevice,"TAPE") == 0)
   close_tape("OUTPUT",&otape,NOREW,1);
else
   put_disk_fd("OUTPUT",&odisk);

return;
}

/******************************************************************************
FUNCTION:	FILE_OPEN

PURPOSE:	Opens the input or output devices.  This function is called
		only if "copy-file" is being executed because the TAE
		parameter block cannot be updated for each file copied.  

ALGORITHM 
  Input device is a disk file
     Open the input disk file
     Initialize input descriptor
  Input device is a tape
     Initialize input descriptor

  Output device is a disk file
     Open the output disk file
     Initialize output descriptor
  Output device is a tape
     Initialize output descriptor

  Error check to insure the record types, record size, and block size of 
  the input/output devices are compatiable.

ALGORITHM REFERENCES	none
*******************************************************************************/

int file_open
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice,		/* output DITTO device (tape or disk)*/
    struct VERIFY_DEV *indev,	/* input device descriptor           */
    struct VERIFY_DEV *outdev	/* output device descriptor          */
)
{
extern struct DISK_FD idisk;		/* DITTO input disk file             */
extern struct DISK_FD odisk;		/* DITTO output disk file            */
extern struct TAPE_FD itape;		/* DITTO input tape                  */
extern struct TAPE_FD otape;		/* DITTO output tape                 */
long     status;			/* function return status code       */
long     fatal = -1;			/* Fatal error code		     */

/*
  Input device is a disk file
*/
if (strcmp(idevice,"DISK") == 0)	
   {
   /*
     Get the disk file attributes
   */
   status = dstat(&idisk,0);
   if (status == E_FAIL)
      {
      ditto_error("INPUT",DITTO_DSTAT,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
      }

   /*
     Open the input disk file
   */
   status = dopen(&idisk,DREAD);
   if (status == E_FAIL)
      return(VERIFY_EOV);

   /*
     Initialize input descriptor
   */
   indev->cc     = idisk.cc;
   indev->rectyp = idisk.rectyp;
   indev->recsiz = idisk.recsiz;
   indev->blksiz = idisk.blksiz;
   indev->device = DISK;
   indev->cur_rec = indev->cur_file = 0;
   }
else				/* Input device is a tape		*/
   {
   /*
     Initialize input descriptor
   */
   indev->cc     = itape.cc;
   indev->rectyp = itape.rectyp;
   indev->recsiz = itape.recsiz;
   indev->blksiz = itape.blksiz;
   indev->device = TAPE;
   indev->cur_rec = indev->cur_file = 0;
   }

/*
  Output device is a disk file
*/
if (strcmp(odevice,"DISK") == 0)
   {
   /*
     Get the disk file attributes
   */
   status = dstat(&odisk,0);
   if (status == E_FAIL)
      {
      ditto_error("INPUT",DITTO_DSTAT,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
      }

   /*
     Initialize output descriptor
   */
   if (odisk.cc == NOVAL)
      odisk.cc = indev->cc;

   if (odisk.recsiz == NOVAL)
      odisk.recsiz = indev->recsiz;

   if (odisk.blksiz == NOVAL)
      odisk.blksiz = indev->blksiz;

   if (odisk.rectyp == NOVAL)
      odisk.rectyp = indev->rectyp;

   /*
     Open the output device
   */
   status = dopen(&odisk,DREAD);
   if (status == E_FAIL)
      return(VERIFY_EOV);

   outdev->cc     = odisk.cc;
   outdev->rectyp = odisk.rectyp;
   outdev->recsiz = odisk.recsiz;
   outdev->blksiz = odisk.blksiz;
   outdev->device = DISK;
   outdev->cur_rec = outdev->cur_file = 0;
   }
else				/* Output device is a tape		*/
   {
   /*
     Intailize output descriptor
   */
   if (otape.cc == NOVAL)
      otape.cc = indev->cc;

   if ((otape.recsiz == NOVAL) || (otape.recsiz == MAXTAPEWT))
      otape.recsiz = indev->recsiz;

   if ((otape.blksiz == NOVAL) || (otape.blksiz == MAXTAPEWT))
      otape.blksiz = indev->blksiz;

   if (otape.rectyp == NOVAL)
      otape.rectyp = indev->rectyp;

   outdev->cc     = otape.cc;
   outdev->rectyp = otape.rectyp;
   outdev->recsiz = otape.recsiz;
   outdev->blksiz = otape.blksiz;
   outdev->device = TAPE;
   outdev->cur_rec = outdev->cur_file = 0;
   }

if ((indev->rectyp == NOVAL) && (outdev->rectyp != NOVAL))
   indev->rectyp = outdev->rectyp;
else if ((outdev->rectyp == NOVAL) && (indev->rectyp != NOVAL))
   outdev->rectyp = indev->rectyp;

/*
  Error check to insure the record types, record size, and block size of 
  the input/output devices are compatiable.
*/
if (indev->recsiz != outdev->recsiz) 
   ditto_error(" ",DITTO_RECSIZ,ERROR_VERIFY,UPDATE_FD);

if (indev->blksiz != outdev->blksiz) 
   ditto_error(" ",DITTO_BLKSIZ,ERROR_VERIFY,UPDATE_FD);

if (indev->rectyp != outdev->rectyp)
   ditto_error(" ",DITTO_RECTYP,ERROR_VERIFY,UPDATE_FD);

return(E_SUCC);
}

/******************************************************************************
FUNCTION:	FILE_CLOSE

PURPOSE:	Closes the input or output devices.  This function is called
		only if "copy-file" is being executed because the TAE
		parameter block cannot be updated for each file copied.  

ALGORITHM 
  Input device is disk file
     Update current record and file attributes
     Close the input device
  Input device is a tape
     Update the current record and file attributes

  Output device is disk file
     Update current record and file attributes
     Close the output device
  Output device is a tape
     Update the current record and file attributes

ALGORITHM REFERENCES	none
*******************************************************************************/

void file_close
(
    struct VERIFY_DEV *indev,	/* input DITTO device                */
    struct VERIFY_DEV *outdev	/* output DITTO device               */
)
{
extern struct DISK_FD idisk;		/* input disk file                   */
extern struct DISK_FD odisk;		/* output disk file                  */
extern struct TAPE_FD itape;		/* input tape                        */
extern struct TAPE_FD otape;		/* output tape                       */
long  status;			/* function return status code       */

/*
  Input device is disk file
*/
if (indev->device == DISK)
   {
   /*
     Update current record and file attributes
   */
   idisk.cur_rec = 0;
   if (idisk.cur_file != NOVAL)
      idisk.cur_file++;	

   /*
     Close the input device
   */
   status = dclose(&idisk);
   if (status != E_SUCC)
	ditto_error("INPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else					/* Input device is a tape	*/
   {
   /*
     Update the current record and file attributes
   */
   itape.cur_rec = 0;
   itape.cur_file++;
   }

/*
  Output device is a disk file
*/
if (outdev->device == DISK)		/* if output device is disk file then*/
   {
   /*
     Update the current record and file attributes
   */
   odisk.cur_rec = 0;
   odisk.cur_file++;

   /*
     Close the output device
   */
   status = dclose(&odisk);
   if (status != E_SUCC)
	ditto_error("OUTPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else					/* Output device is a tape	*/
   {
   /*
     Update current record and file attributes
   */
   otape.cur_rec = 0;
   otape.cur_file++;
   }

return;
}
