/******************************************************************************
FUNCTION:	OPEN_DEVICE

PURPOSE:	Opens the input and output DITTO devices.  It also retrieves
		the file attributes of the DITTO devices from the TAE
		parameter block.  Error checking is done to insure the
		record types, record sizes, and block sizes are compatiable.

ALGORITHM 
  Input device is a disk file
     Get the input device attributes
     Open the input device
     Update the current attributes
  Input device is a tape
     Reopen the input tape device
     Update the current attributes

  Output device is a disk file
     Get the output device attributes
     Open the output device
     Update the current attributes
  Output device is a tape
     Reopen the output tape device
     Update the current attributes
  Error check to insure record type, record size, and block sizes are the
  same.
ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"			/* ditto include file                */
#include "verify.h"
#include "protos.h"

void open_device
(
    char *idevice,			/* input device (tape or disk)       */
    char *odevice,			/* output device (tape or disk)      */
    struct VERIFY_DEV *indev,		/* input device descriptor           */
    struct VERIFY_DEV *outdev		/* output device descriptor          */
)
{
extern struct DISK_FD idisk;		/* input disk file descriptor        */
extern struct DISK_FD odisk;		/* output disk file descriptor       */
extern struct TAPE_FD itape;		/* input tape descriptor             */
extern struct TAPE_FD otape;		/* output tape descriptor            */
long status;				/* function return status code       */
long fatal = -1;			/* Fatal error code		     */

/*
  Input device is a disk file
*/
if (strcmp(idevice,"DISK") == 0)
   {
   /*
     Get the input device attributes
   */
    status = get_disk_fd("INPUT",&idisk);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);

   /*
     Get the disk file attributes
   */
   status = dstat(&idisk,0);
   if (status == E_FAIL)
      {
      ditto_error("INPUT",DITTO_DSTAT,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
      }

   /*
     Open the input device
   */
   status = dopen(&idisk,DREAD);
   if (status == E_FAIL)
      {
      ditto_error("INPUT",DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
      }

   /*
     Update the current attributes
   */
   indev->cc     = idisk.cc;
   indev->rectyp = idisk.rectyp;
   indev->recsiz = idisk.recsiz;
   indev->blksiz = idisk.blksiz;
   indev->device = DISK;
   indev->cur_rec = indev->cur_file = 0;
   }
else					/* Input device is a tape	*/
   {
   /*
     Reopen the input tape device
   */
   status = reopen_tape("INPUT\0",NOREW,TREAD,&itape);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);

   /*
     Update the current attributes
   */
   indev->cc     = itape.cc;
   indev->rectyp = itape.rectyp;
   indev->recsiz = itape.recsiz;
   indev->blksiz = itape.blksiz;
   indev->device = TAPE;
   indev->cur_rec = indev->cur_file = 0;
   }

/*
  The output device is a disk file
*/
if (strcmp(odevice,"DISK") == 0)
   {
   /*
     Get the output device attributes
   */
   status = get_disk_fd("OUTPUT\0",&odisk);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);

   /*
     Get the disk file attributes
   */
   status = dstat(&odisk,0);
   if (status == E_FAIL)
      {
      ditto_error("OUTPUT",DITTO_DSTAT,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
      }

   if (odisk.cc == NOVAL)
      odisk.cc = indev->cc;

   if (odisk.recsiz == NOVAL)
      odisk.recsiz = indev->recsiz;

   if (odisk.blksiz == NOVAL)
      odisk.blksiz = indev->blksiz;

   /*
     Open the output device
   */
   status = dopen(&odisk,DREAD);
   if (status == E_FAIL)
      {
      ditto_error("OUTPUT",DITTO_DOPEN,ERROR_SEV,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
      }

   /*
     Update the current output attributes
   */
   outdev->cc     = odisk.cc;
   outdev->rectyp = odisk.rectyp;
   outdev->recsiz = odisk.recsiz;
   outdev->blksiz = odisk.blksiz;
   outdev->device = DISK;
   outdev->cur_rec = outdev->cur_file = 0;
   }
else					/* Output device is a tape	*/
   {
   /*
     Reopen the tape device
   */
   status = reopen_tape("OUTPUT\0",NOREW,TREAD,&otape);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);

   /*
     Update the output device attributes
   */
   otape.cc = indev->cc;
   if ((otape.recsiz == NOVAL) || (otape.recsiz == MAXTAPEWT))
      otape.recsiz = indev->recsiz;

   if ((otape.blksiz == NOVAL) || (otape.blksiz == MAXTAPEWT))
      otape.blksiz = indev->blksiz;

   outdev->cc     = otape.cc;
   outdev->rectyp = otape.rectyp;
   outdev->recsiz = otape.recsiz;
   outdev->blksiz = otape.blksiz;
   outdev->device = TAPE;
   outdev->cur_rec = outdev->cur_file = 0;
   }

/*
  Error check to insure record type, record size, and block sizes are the
  same.
*/
if ((indev->rectyp == NOVAL) && (outdev->rectyp != NOVAL))
   indev->rectyp = outdev->rectyp;
else if ((outdev->rectyp == NOVAL) && (indev->rectyp != NOVAL))
   outdev->rectyp = indev->rectyp;

if (indev->recsiz != outdev->recsiz) 
   ditto_error(" ",DITTO_RECSIZ,ERROR_VERIFY,UPDATE_FD);

if (indev->blksiz != outdev->blksiz) 
   ditto_error(" ",DITTO_BLKSIZ,ERROR_VERIFY,UPDATE_FD);

if (indev->rectyp != outdev->rectyp)
   ditto_error(" ",DITTO_RECTYP,ERROR_VERIFY,UPDATE_FD);

return;
}
