/******************************************************************************
FUNCTION:	READ_DEVICE

PURPOSE:	Reads one physical record from the input and output DITTO
		device.

ALGORITHM 
  Set the block sizes
  The input device is a tape
     Read one physical record from the tape
     Check to see if the tape is at the EOV or EOT
        Backspace one record

  The output device is a tape
     Read one physical record from the tape
     Check to see if the tape is at the EOV or EOT
        Backspace one record

  The input device is a disk file
     Read one physical record from the file
  The output device is a disk file
     Read one physical record from the file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"   			/* ditto include file                */
#include "verify.h"
#include "protos.h"

int read_device
(
    struct VERIFY_DEV *indev,		/* Input device			*/
    struct VERIFY_DEV *outdev		/* Output device to verify	*/
)
{
extern struct DISK_FD idisk;		/* Iput disk structure		*/
extern struct DISK_FD odisk;		/* Output disk structure	*/
extern struct TAPE_FD itape;		/* Input tape structure		*/
extern struct TAPE_FD otape;		/* Output disk structure	*/

long	in_blksiz;			/* Input block size		*/
long	out_blksiz;			/* Output block size		*/
long	one = 1;			/* One				*/
long    mode = HANDLE;			/* Mode to handle errors	*/
long	return_status = 1;		/* Return status		*/

/*
  Set the block sizes
*/
in_blksiz = indev->blksiz;
out_blksiz = outdev->blksiz;

/*
  The input device is a tape
*/
if (indev->device == TAPE)
   {
   /*
     Read one physical record from the tape
   */
   indev->bytes_read = c_tpread(&itape.unit,indev->buf,&itape.blksiz,&mode);
   if (indev->bytes_read == TAPE_EOF)
      indev->bytes_read = 0;

   /*
     Check to see if the tape is at the EOV or EOT
   */
   if ((indev->bytes_read == TAPE_EOV) || (indev->bytes_read == TAPE_EOT))
      {
      /*
        Backspace one record
      */
      indev->bytes_read = 0;
      return_status = VERIFY_EOV;
      c_tpbsr(&itape.unit,&one,&mode);
      }

   if (indev->bytes_read < 0)
      proc_error(indev,outdev,"INPUT",DITTO_TPREAD);

#ifdef unix
   if (indev->bytes_read > 0)
      out_blksiz = indev->bytes_read;
#endif
   }

/*
  The output device is a tape
*/
if (outdev->device == TAPE)
   {
   /*
     Read one physical record from the output device
   */
   outdev->bytes_read = c_tpread(&otape.unit,outdev->buf,&otape.blksiz,&mode);
   if (outdev->bytes_read == TAPE_EOF)
      outdev->bytes_read = 0;

   /*
     Check to see if the tape is at EOV or EOT
   */
   if ((outdev->bytes_read == TAPE_EOV) || (outdev->bytes_read == TAPE_EOT))
      {
      /*
        Backspace one record
      */
      outdev->bytes_read = 0;
      return_status = VERIFY_EOV;
      c_tpbsr(&otape.unit,&one,&mode);
      }

   if (outdev->bytes_read < 0)
      proc_error(indev,outdev,"OUTPUT",DITTO_TPREAD);

#ifdef unix
   if (outdev->bytes_read > 0)
      in_blksiz = outdev->bytes_read;
#endif
   }

/*
  Input device is a disk file
*/
if (indev->device == DISK)
   {
   /*
     Read one physical record from the file
   */
   indev->bytes_read = dread(&idisk,indev->buf,in_blksiz);
   if (indev->bytes_read == E_FAIL)
      proc_error(indev,outdev,"INPUT",DITTO_DREAD);
   }

/*
  Output device is a disk file
*/
if (outdev->device == DISK)
   {
   /*
     Read one physical record from the file
   */
   outdev->bytes_read = dread(&odisk,outdev->buf,out_blksiz);
   if (outdev->bytes_read == E_FAIL)
      proc_error(indev,outdev,"OUTPUT",DITTO_DREAD);
   }

return(return_status);
}
