/******************************************************************************
FUNCTION:	VERIFY_PROC

PURPOSE:	Reads the specified number of records from each DITTO device
		and calls "verify_buf" to do a byte by byte compare of the
		two records.

ALGORITHM 
  Allocate input buffer space
  Allocate output buffer space
  Initialize the output message
  Process the number of records specified
     Read record from each device
     Check to see to see that something was read from each device
	break out if not
     If the number of bytes read are not equal
	stop message time for a bit
	Inform the user of the verify error found
        Reset the input number of bytes to verify as much as we can
	restart message time
     Verify the buffers
  Broke out of loop above to generate an error when reading input device
  Broke out of loop above to generate an error when reading output device
  Free the input and output buffer space

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "util.h"
#include "typlim.h"			/* Data type limit		     */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"			/* ditto include file                */
#include "verify.h"
#include "protos.h"

#define currec curent

int verify_proc
(
    struct VERIFY_DEV *indev,		/* Input device			*/
    struct VERIFY_DEV *outdev,		/* Output device		*/
    long nrecords,			/* Number of records		*/
    long msgtim			/* Message time			*/
)
{
extern long currec;			/* Current recod processing	*/

long	   max_ent = MAXLONG;		/* Maximum number of lines	*/
long	   max_cat = 1;			/* Maximum number of files	*/
long	   flag = GEN1;			/* flag				*/
long	   return_status = 1;		/* Return status		*/
long	   fatal = -1;			/* Fatal error code		*/

char	   msg[ERRLEN];			/* Message string		*/

/*
  Allocate input buffer space
*/
indev->buf = (unsigned char *) malloc(indev->blksiz);
if (indev->buf == 0)
   {
   ditto_error(" ",DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
   }

/*
  Allocate output buffer space
*/
outdev->buf = (unsigned char *) malloc(outdev->blksiz);
if (outdev->buf == 0)
   {
   ditto_error(" ",DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
   }

/*
  Initialize the output message
*/
currec = 0;
c_inittm("DITTO VERIFY ","Verified","record #"," "," ",&max_ent,&max_cat,
	 &msgtim,&flag);
c_startm();

/*
  Process the number of records specified
*/
for (currec=0; currec<nrecords; currec++, indev->cur_rec++, outdev->cur_rec++)
   {
   /*
     Read record from each device
   */
   return_status = read_device(indev,outdev);
   if (return_status == VERIFY_EOV)
	break;

   /*
     Check to see to see that something was read from each device
   */
   if ((indev->bytes_read == 0) || (outdev->bytes_read == 0))
	break;

   /*
     If the number of bytes read are not equal inform the user
   */
   if (indev->bytes_read != outdev->bytes_read)
      {
      c_stoptm();		/* stop the message time for a bit	*/

      sprintf(msg,"VERIFY ERROR at record %d",currec);
      m_msg(msg,"ditto-uneq");

      sprintf(msg,"    Number Bytes In Input Record  = %4d",indev->bytes_read);
      m_msg(msg,"ditto-uneq");

      sprintf(msg,"    Number Bytes in Output Record = %4d",outdev->bytes_read);
      m_msg(msg,"ditto-uneq");

      proc_error(indev,outdev," ",DITTO_UNEQ);

      c_startm();		/* Restart the message time		*/

      /*
        Reset the input number of bytes to verify as much as we can
      */
      if (outdev->bytes_read > indev->bytes_read)
         indev->bytes_read = outdev->bytes_read;
      }

   /*
     Verify the buffers
   */
   verify_buf(indev,outdev,indev->bytes_read);
   }

/*
  Broke out of loop above to generate an error when reading input device
*/
if (indev->bytes_read == 0)
   {
   if (outdev->bytes_read != 0)
      proc_error(indev,outdev," ",DITTO_INEOF);
   indev->cur_file++;
   }

/*
  Broke out of loop above to generate an error when reading output device
*/
if (outdev->bytes_read == 0)
   {
   if (indev->bytes_read != 0)
      proc_error(indev,outdev," ",DITTO_OUTEOF);
   outdev->cur_file++;
   }

/*
  Stop the message time
*/
c_stoptm();

/*
  Warn the user that all records may not have been processed
*/
if ((nrecords != ALL) && (currec < nrecords))
   ditto_error(" ",DITTO_EOF,ERROR_WARN,UPDATE_FD);

/*
  Free the input and output buffer space
*/
free(indev->buf);
free(outdev->buf);

return(return_status);
}
