#ifndef DDR_H
#define DDR_H

#include "tae.h"
#include "imageio.h"

/*This is the 'C' include file for the DDR routines                         */


struct DDR                   /* ddr for integer data                        */
{
    int  nl;                 /* number of lines in image                    */
    int  ns;                 /* number of samples in image                  */
    int  nbands;             /* number of bands in image                    */
    int  dtype;              /* data type of pixels:                        */
                             /*   =1:  unsigned char                        */
                             /*   =2:  short                                */
                             /*   =3:  long                                 */
                             /*   =4:  float                                */
    int  master_line;        /* line relative to master image               */
    int  master_sample;      /* sample relative to master image             */
    int  valid[8];           /* valid flags:                                */
                             /*   =0:  not valid                            */
                             /*   =1:  valid                                */
                             /*   =2:  unknown                              */
                             /* valid[0] = projection code                  */
                             /* valid[1] = zone code                        */
                             /* valid[2] = datum code                       */
                             /* valid[3] = projection coefficients          */
                             /* valid[4] = ground units                     */
                             /* valid[5] = ground distance                  */
                             /* valid[6] = corner coordinates               */
                             /* valid[7] = line/sample increments           */
    int  proj_code;          /* GCTP projection code                        */
                             /*  refer to GCTP document for valid codes     */
    int  zone_code;          /* UTM or State Plane zone                     */
                             /*  refer to GCTP document for valid codes     */
    int  datum_code;         /* GCTP datum code                             */
                             /*  refer to GCTP document for valid codes     */
    int  spare;              /* spare integer value; for future use or
                                expansion                                   */

    char system[12];         /* computer system data is on (with NULL)      */
    char proj_units[12];     /* Projection units (GCTP units+other) 
                                (with NULL)                                 */
    char last_used_date[12]; /* last access date        (with NULL)         */
                             /* NOTE: All ddr dates are stored as           */
                             /* "dd-mmm-yy".  For example, a date           */
                             /* of December 31, 1986 is stored as           */
                             /* "31-dec-86" with the month in lower         */
                             /* case.                                       */
    char last_used_time[12]; /* last access time        (with NULL)         */
                             /* NOTE: All ddr times are stored              */
                             /* using a twenty-four hour clock.             */
                             /* Seconds are separated by a colon.           */
                             /* Example: 1:05:55 pm is stored as            */
                             /* 1305:55                                     */

    double proj_coef[15];    /* GCTP projection parameters                  */
                             /* refer to GCTP document for field definitions*/
    double upleft[2];        /* Corner coordinates entered                  */
    double loleft[2];        /* in latitude/longitude or                    */
    double upright[2];       /* northing/easting order or                   */
    double loright[2];       /* projection coordinates (y/x)                */
    double pdist_y;          /* projection distance/pixel (y-direction)     */
    double pdist_x;          /* projection distance/pixel (x-direction)     */
    double line_inc;         /* line increment for sampling                 */
    double sample_inc;       /* sample increment for sampling               */
                             /* NOTE:  The line/sample increments are       */
                             /*   the values applied to the original        */
                             /*   image to obtain this image.  If re-       */
                             /*   or sub-sampling was not applied, the      */
                             /*   value contained in these fields will      */
                             /*   be 1.0                                    */
};

struct BDDR
{
    int  bandno;             /* band no. of the record                      */
    int  valid;              /* min/max validity flag of the band           */
                             /*   =0:  not valid                            */
                             /*   =1:  valid                                */
                             /*   =2:  bounded                              */

    double minval;           /* minimum value of the band                   */
    double maxval;           /* maximum value of the band                   */

    char source[32];         /* source of data (with NULL)                  */
    char instrument[32];     /* type of sensor (with NULL)                  */
    char direction[64];      /* direction of capture process(with NULL)     */
    char date[10];           /* capture date (with NULL)                    */
                             /* NOTE: All ddr dates are stored as           */
                             /* "dd-mmm-yy".  For example, a date           */
                             /* of December 31, 1986 is stored as           */
                             /* "31-dec-86" with the month in lower         */
                             /* case.                                       */
    char time[8];            /* capture time (with NULL)                    */
                             /* NOTE: All ddr times are stored              */
                             /* using a twenty-four hour clock.             */
                             /* Seconds are speperated by a colon.          */
                             /* Example: 1:05:55 pm is stored as            */
                             /* 1305:55                                     */
};

/*  Below are the constants to be used as keys within GETBDF and PUTBDF
----------------------------------------------------------------------------*/
#define DDBAND 100           /* band number                                 */
#define DDMMV 101            /* Min/max validity flag                       */
#define DDMIN 102            /* Minimum value                               */
#define DDMAX 103            /* Maximum value                               */
#define DDDIR 106            /* Direction                                   */
#define DDVFLG 6             /* Validity flag array                         */
#define DDSYS 10             /* System                                      */
#define DDSRC 104            /* Source                                      */
#define DDINST 105           /* Instrument                                  */
#define DDCDAT 107           /* Capture date                                */
#define DDCTIM 108           /* Capture time                                */

/*  Below are the constants to be used as keys within GETDDF and PUTDDF
----------------------------------------------------------------------------*/
#define DDNL 0               /* number of lines                             */
#define DDNS 1               /* number of samples                           */
#define DDNB 2               /* number of bands                             */
#define DDDTYP 3             /* Data type                                   */
#define DDML 4               /* Master line                                 */
#define DDMS 5               /* Master sample                               */
#define DDPCOD 7             /* Projection code                             */
#define DDZCOD 8             /* Zone code                                   */
#define DDDCOD 9             /* Datum code                                  */
#define DDUNIT 11            /* Projection Unit                             */
#define DDLDAT 12            /* Last use dated                              */
#define DDLTIM 13            /* Last used time                              */
#define DDPCF 14             /* Projection coeficients                      */
#define DDUL 15              /* Upper left                                  */
#define DDLL 16              /* Lower left                                  */
#define DDUR 17              /* Upper right                                 */
#define DDLR 18              /* Lower right                                 */
#define DDPDY 19             /* Projection distance-y                       */
#define DDPDX 20             /* Projection distance-x                       */
#define DDLINC 21            /* Line increment                              */
#define DDSINC 22            /* Sample increment                            */


/* Below are the constants to retrieve the correct validity flag from the
   validity flag array                                                   
----------------------------------------------------------------------------*/
#define DDPCV  0             /* projection code validity flag               */
#define DDZCV  1             /* zone code validity flag                     */
#define DDDCV  2             /* datum code validity flag                    */
#define DDPPV  3             /* projection parameters validity flag         */
#define DDPUV  4             /* ground units validity flag                  */
#define DDPDV  5             /* ground distance validity flag               */
#define DDCCV  6             /* corner coordinates validity flag            */
#define DDINCV 7             /* line/sample increments validity flag        */

/*  Below of Misc. DDR constants
----------------------------------------------------------------------------*/
#define E_PROT -2            /* return status of write protected            */
#define INVAL   0            /* invalid validity flag status                */
#define VALID   1            /* valid validity flag                         */
#define UNKNOW  2            /* unknown and equal validity flag             */
#define BOUND   2            /* bounded min/max validity flag               */
#define COMB    1            /* images are being combined into one band     */
#define NOCOMB  0            /* images are not being combined into one band */
#define DISIZE 18            /* number of integers in record 1 of DDR       */
#define DDSIZE 27            /* number of doubles in record 2 of DDR        */
#define DDSYLN 12            /* length of the SYSTEM character string       */
#define DDSTCT 4             /* No. of string in record 1 of DDR file       */
#define DDNVAL 8             /* No. of valid flags stored in the DDR file   */

/*  Below are constants used in the process of finding file corners
    when image corners are known
----------------------------------------------------------------------------*/
#define UPLEFT_Y 0           /* upper left y/line                           */
#define UPLEFT_X 1           /* upper left x/sample                         */
#define UPRIGHT_Y 2          /* upper right y/line                          */
#define UPRIGHT_X 3          /* upper right x/sample                        */
#define LOLEFT_Y 4           /* lower left y/line                           */
#define LOLEFT_X 5           /* lower left x/sample                         */
#define LORIGHT_Y 6          /* lower right y/line                          */
#define LORIGHT_X 7          /* lower right x/sample                        */

void las_calcor
(
   double sl,                /* I: Starting line                            */
   double ss,                /* I: Starting sample                          */
   double nl,                /* I: Number of lines                          */
   double ns,                /* I: Number of samples                        */
   const struct DDR *ddr,    /* I: Input DDR structure                      */
   double *upleft,           /* O: Upper left corner                        */
   double *upright,          /* O: Upper right corner                       */
   double *loleft,           /* O: Lower left corner                        */
   double *loright           /* O: Lower right corner                       */
);

int c_comsys
(
    const char *hostname,    /*  I: host file name                          */
    int   *flag              /*  O: output flag                             */
);

int las_getbdf
(
    const struct BDDR *bddr, /* I: input band depenent record               */
    int  key,                /* I: specified key of field needed            */
    void *value              /* O: output value of specified field          */
);

int las_getbdr
(
    const char *hname,       /* I: input host image name                    */
    struct BDDR *bddr,       /* O: output band dependent record             */
    int band                 /* I: specified band number                    */
);

int c_getddr
(
    const char *hname,       /* I: Host image file name                     */
    struct DDR *ddr          /* O: DDR structure                            */
);

void c_intbdr
(
    struct BDDR *bddr        /* I/O: bddr data                              */
);

void c_intddr
(
    struct DDR *ddr          /* I/O: ddr data in record 1 and 2             */
);


int las_merge_ddrs
(
    struct PARBLK *parblk,        /* I: TAE parameter block                 */
    const  char host_in[][CMLEN], /* I: array of input host names           */
    const  char *host_out,        /* I: Output host name                    */
    int    nmrimg,                /* I: Total number of input images        */
    const int window[][4],        /* I: Window specifications for each im   */
    const  int bands[][MAXBND +1],/* I: band numbers for each input image   */
    const  int nbands[],          /* I: number of bands for each input im   */
    double line_inc,              /* I: increment in the line direction     */
    double samp_inc,              /* I: increment in the sample direction   */
    int    out_nbands,            /* I: number of output bands              */
    int    cflag,                 /* I: combination of bands flag           */
    struct BDDR **out_bddr,       /* O: output band dependent ddr records   */
    struct DDR *out_ddr,          /* O: output ddr                          */
    char   upddr_flag             /* I: Flag: TRUE means called by upddr    */
);

int c_lused
(
    const char *hostname          /* I: host name of associated DDR       */
);

int las_minmax
(
    const char hname[][CMLEN],    /* I: string descriptor of host name      */
    int     nimg,                 /* I: total number of images              */
    int     window[][4],          /* I/O: starting line of image            */
    int     bands[][MAXBND + 1],  /* I/O: array of band numbers needed      */
    int     nbands,               /* I: number of bands for each image      */
    double  *minval,              /* O: minimum value array                 */
    double  *maxval               /* O: maximum value array                 */
);

int las_putbdf
(
    struct   BDDR *bddr,          /* I/O: input band dependent record       */
    int      key,                 /* I: specified key of field needed       */
    const void *value             /* I: output value of specified field     */
);

int c_putbdr
(
    const char *hname,            /* I: input host image name               */
    const struct BDDR *bddr       /* I: band data for the DDR               */
);

int las_putddf
(
    struct DDR *ddr,              /* I/O: input band dependent record       */
    int key,                      /* I: specified key of field needed       */
    const void *value             /* I: output value of specified field     */
);

int c_putddr
(
    const char *hname,            /* I: host image file name                */
    const struct DDR *ddr         /* I: ddr data in record 1 and 2          */
);

int c_recalc_pixel_size
(
    double *pixel_y,              /* I/O: Pixel size in the y direction     */
    double *pixel_x,              /* I/O: Pixel size in the x direction     */
    double upleft_y,              /* I: Upper left corner of image in y dir */
    double loleft_y,              /* I: Lower left corner of image in y dir */
    double loright_y,             /* I: Lower right corner of image in y di */
    double upleft_x,              /* I: Upper left corner of image in x dir */
    double loleft_x,              /* I: Lower left corner of image in x dir */
    double loright_x,             /* I: Lower right corner of image in x di */
    int nl,                       /* I: Total number of line in the image   */
    int ns                        /* I: Total number of samples in the image*/
);

int c_rotcor
(
    /* The following parameters with 8 elements are
       ordered as follows:
           [0] = upper left y
           [1] = upper left x
           [2] = lower left y
           [3] = lower left x
           [4] = upper right y
           [5] = upper right x
           [6] = lower right y
           [7] = lower right x    */

    const double proj[8],         /* I: Input image corner projection coords*/
    const int  image[8],          /* I: Input image corner line,sample coord*/
    const double window[4],       /* I: Window applied to image             */
    double proj_wind[8]           /* O: Output image corner projection coord*/
);

int las_rtcoef
(
    /* The following parameters with 8 elements are
       ordered as follows:
           [0] = upper left y
           [1] = upper left x
           [2] = lower left y
           [3] = lower left x
           [4] = upper right y
           [5] = upper right x
           [6] = lower right y
           [7] = lower right x    */

    const double proj[8],         /* I: Input image corner projection coord */
    const int  image[8],          /* I: Input image corner line,sample coor */
    double coef[6],               /* O: Transformation coefficients         */
    int flag                      /* I: Flag for error messages             */
);

int las_rtiddr
(
    struct DDR *inddr,            /* I/O: Input DDR structure to be updated */
    int rot                       /* I: Rotation amount in 90 degree incs: 
                                        1 =  90 degrees
                                        2 = 180 degrees
                                        3 = 270 degrees
                                        4 = Horizontal flip
                                        5 = Vertical flip                   */
);

int las_svmnmx
(
    const char *hname,            /* I: input host image name               */
    int totbnd,                   /* I: number of bands to be processed     */
    const double minval[],        /* I: minimum intensity values            */
    const double maxval[],        /* I: maximum intensity values            */
    const char *m_flag            /* I: min/max validity flag 
                                        ("YES"/"NO"/"BOUND")  
                                        from $MINMAX global                 */
);

int las_upddr
(
    struct PARBLK *parblk,        /* I: TAE parameter block                 */
    const char host_in[][CMLEN],  /* I: array of input host names           */
    const char *host_out,         /* I: Output host name                    */
    int  nmrimg,                  /* I: Total number of input images        */
    const int  window[][4],       /* I: Window specifications for each im   */
    const int  bands[][MAXBND +1],/* I: band numbers for each input image   */
    const int  nbands[],          /* I: number of bands for each input im   */
    double line_inc,              /* I: increment in the line direction     */
    double samp_inc,              /* I: increment in the sample direction   */
    int  out_nbands,              /* I: number of output bands              */
    int  cflag                    /* I: combination of bands flag           */
);

void set_capture
(
    const struct BDDR *in_bddr,   /* I: input band depentent DDR            */
    struct BDDR *out_bddr,        /* O: output band depentent DDR           */
    int  nimg                     /* I: number of images                    */
);

void set_corners
(
    int  corner_flag,             /* I: Processing corner validity flag     */
    int  proj_flag,               /* I: Processing projection validity flag */
    int  pdist_flag,              /* I: Processing proj. dist. validity flag*/
    int  zone_flag,               /* I: Processing zone validity flag       */
    int  ppar_flag,               /* I: Processing proj. parameters validity*/
    const struct DDR *in_ddr,     /* I: Input DDR structure                 */
    struct DDR *out_ddr,          /* O: Output DDR structure                */
    double line_inc,              /* I: Line increment                      */
    double samp_inc,              /* I: Sample increment                    */
    const int  window[]           /* I: Window specifications               */
);

void set_datum
(
    const struct DDR in_ddr[],    /* I: input DDRs                          */
    struct DDR *out_ddr,          /* O: Output DDR                          */
    int  nimg                     /* I: number of input images              */
);

void set_inc_flag
(
    struct DDR ddr[],             /* I: Input DDRs                          */
    int  *inc_flag,               /* O: Processing increment flag           */
    int  *inc_index,              /* O: Index to first DDR with a valid inc */
    int  nimg                     /* I: Total number of input DDRs          */
);

void set_master
(
    const struct DDR *in_ddr,     /* I: input DDR structure                 */
    struct DDR *out_ddr,          /* O: output DDR structure                */
    int  inc_flag,                /* I: increment flag                      */
    const int  window[]           /* I: window specification of input image */
);

void set_pdist
(
    int  pdist_flag,              /* I: input projection distance flag      */
    const struct DDR *in_ddr,     /* I: input DDR structure                 */
    struct DDR *out_ddr,          /* O: output DDR structure                */
    double line_inc,              /* I: input line increment value          */
    double samp_inc               /* I: input sample increment value        */
);

void las_set_ppar
(
    const struct DDR in_ddr[],    /* I: input DDRs                          */
    struct DDR *out_ddr,          /* O: output DDR                          */
    int  *ppar_flag,              /* I/O: Processing projection pararmeter  */
    int  proj_flag,               /* I: Projection flag                     */
    int  nimg                     /* I: Total number of input DDRs          */
);

void set_proj
(
    const struct DDR in_ddr[],    /* I: input DDRs                          */
    struct DDR *out_ddr,          /* O: output DDR                          */
    int *proj_flag,               /* O: Processing projection flag          */
    int  nimg                     /* I: number of input DDRs                */
);

void set_punit
(
    const struct DDR in_ddr[],    /* I: input DDRs                          */
    struct DDR *out_ddr,          /* O: output DDR                          */
    int  nimg                     /* I: number of input DDRs                */
);

void set_zone
(
    const struct DDR in_ddr[],    /* I: input DDRs                          */
    struct DDR *out_ddr,          /* O: output DDR                          */
    int  *zone_flag,              /* O: Processing zone validity flag       */
    int  nimg                     /* I: number of input DDRs                */
);

void set_corn_flag
(
    const struct DDR ddr[],       /* I: input integer DDR fields            */
    int  *corner_flag,            /* O: processing flag for the corner field*/
    const int  (*window)[4],      /* I: input window                        */
    int  *copy_index,             /* O: index of the first valid corn field */
    int  pdist_flag,              /* I: processing fl for the proj dist fld */
    double tgbl,                  /* I: tolerance global                    */
    int  pdist_index,             /* I: index of the 1st valid proj dist fd */
    int  gbl_flag,                /* I: TAE global validity flag            */
    int  nimg                     /* I: total number of input images        */
);


int set_increment
(
    int  inc_flag,                /* I: processing increment flag           */
    const struct DDR *in_ddr,     /* I: input DDR                           */
    struct DDR *out_ddr,          /* O: output DDR                          */
    double in_line_inc,           /* I: line increment                      */
    double in_samp_inc            /* I: sample increment                    */
);

void set_pdis_flag
(
    const struct DDR ddr[],       /* I: input DDRs                          */
    int  *pdist_flag,             /* O: Processing projection distance flag */
    int  *pdist_index,            /* O: Index of 1st DDR with a valid pdist */
    int  nimg                     /* I: Number of input DDRs                */
);


#ifndef NO_OBSOLETE_INTERFACE

void c_calcor
(
    const double *sl,             /* I: starting line                       */
    const double *ss,             /* I: starting sample                     */
    const double *nl,             /* I: number of lines                     */
    const double *ns,             /* I: number of samples                   */
    const struct DDR *ddr,        /* I: input DDR structure                 */
    double *upleft,               /* O: Upper left corner                   */
    double *upright,              /* O: Upper right corner                  */
    double *loleft,               /* O: Lower left corner                   */
    double *loright               /* O: Lower right corner                  */
);

int c_getbdf
(
    const struct BDDR *bddr,      /* I: input band depenent record          */
    const long *key,              /* I: specified key of field needed       */
    void *value                   /* O: output value of specified field     */
);

int c_getbdr
(
    const char *hname,            /* I: input host image name               */
    struct BDDR *bddr,            /* O: output band depenent record         */
    const long *band              /* I: specified band number               */
);


int c_merge_ddrs
(
    struct PARBLK *parblk,          /* I: TAE parameter block                 */
    const char host_in[][CMLEN],    /* I: array of input host names           */
    const char *host_out,           /* I: Output host name                    */
    const long *nmrimg,             /* I: Total number of input images        */
    const long window[][4],         /* I: Window specifications for each image*/
    const long bands[][MAXBND +1],  /* I: band numbers for each input image   */
    const long nbands[],            /* I: number of bands for each input image*/
    const double *line_inc,         /* I: increment in the line direction     */
    const double *samp_inc,         /* I: increment in the sample direction   */
    const long *out_nbands,         /* I: number of output bands              */
    const long *cflag,              /* I: combination of bands flag           */
    struct BDDR **out_bddr,         /* O: output band dependent ddr records   */
    struct DDR *out_ddr,            /* O: output ddr                          */
    char upddr_flag                 /* I: Flag: TRUE means called by upddr fun*/
);

int c_minmax
(
    const  char  hname[][CMLEN],    /* I: string descriptor of host name    */
    const  long  *nimg,             /* I: total number of images            */
    long   window[][4],             /* I/O: starting line of image          */
    long   bands[][MAXBND + 1],     /* I/O: array of band numbers needed    */
    const  long  *nbands,           /* I: number of bands for each image    */
    double *minval,                 /* O: minimum value array               */
    double *maxval                  /* O: maximum value array               */
);


int c_putbdf
(
    struct BDDR *bddr,              /* I/O: input band depenent record      */
    const long *key,                /* I: specified key of field needed     */
    const void *value               /* I: output value of specified field   */
);

int c_putddf
(
    struct DDR *ddr,                /* I/O: input band dependent record     */
    const long *key,                /* I:  specified key of field needed    */
    const void  *value              /* I:  output value of specified field  */
);

int c_rtcoef
(
    /* The following parameters with 8 elements are
       ordered as follows:
           [0] = upper left y
           [1] = upper left x
           [2] = lower left y
           [3] = lower left x
           [4] = upper right y
           [5] = upper right x
           [6] = lower right y
           [7] = lower right x    */

    const double proj[8],         /* I: Input image corner projection coord */
    const long   image[8],        /* I: Input image corner line,sample coor */
    double coef[6],               /* O: Transformation coefficients         */
    const long *flag              /* I: Flag for error messages             */
);

int c_rtiddr
(
    struct DDR *inddr,            /* I/O: Input DDR structure to be updated */
    const long *rot               /* I: Rotation amount in 90 degree incr:
                                        1 =  90 degrees
                                        2 = 180 degrees
                                        3 = 270 degrees
                                        4 = Horizontal flip
                                        5 = Vertical flip                   */
);

void set_ppar
(
    const struct DDR in_ddr[],    /* I: input DDRs                          */
    struct DDR *out_ddr,          /* O: output DDR                          */
    long  *ppar_flag,             /* I/O: Processing projection pararmeter  */
    const long *proj_flag,        /* I: Projection flag                     */
    int  nimg                     /* I: Total number of input DDRs          */
);

int c_upddr
(
    struct PARBLK *parblk,        /* I: TAE parameter block                 */
    const char host_in[][CMLEN],  /* I: array of input host names           */
    const char *host_out,         /* I: Output host name                    */
    const long *nmrimg,           /* I: Total number of input images        */
    const long  window[][4],      /* I: Window specifications for each image*/
    const long  bands[][MAXBND +1],/* I: band numbers for each input image  */
    const long  nbands[],         /* I: number of bands for each input image*/
    const double *line_inc,       /* I: increment in the line direction     */
    const double *samp_inc,       /* I: increment in the sample direction   */
    const long  *out_nbands,      /* I: number of output bands              */
    const long *cflag             /* I: combination of bands flag           */
);

int c_svmnmx
(
    const char *hname,            /* I: input host image name               */
    const long *totbnd,           /* I: number of bands to be processed     */
    const double minval[],        /* I: minimum intensity values            */
    const double maxval[],        /* I: maximum intensity values            */
    const char *m_flag            /* I: min/max validity flag 
                                        ("YES"/"NO"/"BOUND")   
                                        from $MINMAX global                 */
);

#endif /* obsolete interfaces */

#endif
