#ifndef LTABLE_H
#define LTABLE_H

#define FATAL    -1       /* Fatal error encountered */
#define NONFATAL  1       /* Nonfatal error encountered */

#ifndef FALSE
#define FALSE     0       /* Boolean value for FALSE */
#endif
#ifndef TRUE
#define TRUE      1       /* Boolean value for TRUE  */
#endif

#define MAX_SUB  20       /* Maximum number of associated LT files    */
#define TAE_VAL  50       /* Max number of TAE values for a parameter */

#define FIELD_SEP1 ','    /* Logical record field separator 1 */
#define FIELD_SEP2 '\n'   /* Logical record field separator 2 */
#define RECORD_SEP ';'    /* Logical record separator */
#define CHAR_DEL   '\"'   /* Character delimiter */

#define RSEP    2         /* Record separator for FORTRAN programs  */

#define LIN_SIZ         80 /* Number characters without a new line */
#define MAX_FIELD_LEN 1024 /* Max number of characters in a record field */
#define TAB_SIZ       8192 /* Labeled table physical record size */

#define C       0         /* Character data type */
#define I1      1         /* 8-bit integer data type */
#define I2      2         /* 16-bit integer data type */
#define I4      3         /* 32-bit integer data type */
#define R4      4         /* 32-bit real data type */
#define R8      5         /* 64-bit real data type */
#define LTDATE  6         /* Date stored as char string mm/dd/yyyy */
#define LTTIME  7         /* Time stored as char string hh:mm:ss      */
#define NDTYPE  8         /* Total number of data types */

#ifndef OPEN_TAB_C_
extern const long LT_DTYPE_SIZE[NDTYPE];
extern const char DATA_TYPE[NDTYPE][7];
#endif

#include "desc.h"

struct LT_DATE
{
    int month;
    int day;
    int year;
};

struct LT_TIME
{
    int hour;
    int minute;
    int second;
};

struct MATRIX             /* I: Structure defining the number of rows */
{                         /*    and columns of a matrix               */
    int nrow;
    int ncol;
};

struct VECTOR             /* Recursive structure defining the         */
{                         /* Label vectors of the Labeled Table.      */
    char *label;          /* I: Label name                            */
    int dtype;            /* I: Data type of the vector               */
    char *desc;           /* I: Label description                     */
    union                 /* Define a union which depends on the data */
    {                     /* type as to which pointer is currently    */
        char   *c;        /* valid.  There is a separate pointer      */
        char   *i1;       /* for each of the valid data types.        */
        short  *i2;
        int   *i4;
        float  *r4;
        double *r8;
    } ptr;
    struct MATRIX lt_size;/* I: Number of rows & cols in LT           */
    struct MATRIX size;   /* I: Actual number of rows & cols in data  */
    int null;             /* I: Flag indicating null fields in data   */
    char sep;             /* I: Type of separator                     */
    int flag;             /* I: Boolean flag used for APPEND access   */
    struct VECTOR *next;  /* I: Pointer to next vector                */
};

struct TAB_DEF
{
    char hname[CMLEN];    /* I: Host name of the Labeled Table       */
    char *subfile[MAX_SUB];/*I: List of files associated to this LT  */
    int nsubs;            /* I: Number of files associated to LT     */
    int fd;               /* I: File descriptor to LT                */
    int access;           /* I: Access the LT was opened for         */
    char *ftype;          /* I: File type of the LT                  */
    char *fdesc;          /* I: File description of the LT           */
    struct DESC buf;      /* I/O: Data buffer                        */
    int ncol;             /* I: Number of columns in the LT          */
    struct VECTOR *vector;/* I: Column vector                        */
    int nbytes;           /* I: Number of bytes in data buffer       */
    int cur;              /* I: Current byte offset in the data bufr */
    int currec;           /* I: Current  record being processed      */
    int label_flag;       /* I: Flag indicating data  written        */
};


void close_tab
(
    struct TAB_DEF *tab   /* I/O: Structure defining the LT files    */
);

void free_vec
(
    const struct VECTOR *vec  /* I: Labeled Table vector             */
);

void get_lt_date
(
    struct VECTOR *vec,  /* I/O: Defines the LT record element       */
    struct LT_DATE *date /* O: Date read from the LT I/O buffer      */
);

void get_lt_time
(
    struct VECTOR *vec,  /* I/O: Struct for the LT record element    */
    struct LT_TIME *tim  /* O: Time read from the LT I/O buffer      */
);

void get_matrix
(
    struct VECTOR *vec,  /* I/O: Struct defining the record to copy  */
    int odtype,          /* I: Data type of the output matrix        */
    void *matrix         /* O: Output matrix(any valid data type)    */
);

int get_record
(
    struct TAB_DEF *tab        /* I/O: Labeled Table to be processed */
);

void get_vector
(
    struct TAB_DEF *tab        /* I/O: Struct of LT file to process  */
);

struct VECTOR *get_field_ptr
(
    const struct TAB_DEF *tab, /* I: Defines the LT structure        */ 
    const char *name           /* I: Name defining Lt record field   */
);

char *make_lt_name
(
    char innam[],         /* I: Master LT filename                   */
    int nmr               /* I: Numeric value appended to master LT  */
);

void open_tab
(
    struct TAB_DEF *tab,  /* I/O: Struct of Labeled Table            */
    const char *fname,    /* I: Host name of the LT file to be opened*/
    int access            /* I: Type of access to open the file with */
                          /*    0 - read                             */
                          /*    1 - write                            */
                          /*    2 - append                           */
);

void put_matrix
(
    struct VECTOR *vec,        /* I: VECTOR structure                */
    int idtype,                /* I: input data type                 */
    const struct MATRIX *size, /* I: Matrix structure                */
    const void *matrix         /* I: The matrix to be written into   */
);

void put_record
(
    struct TAB_DEF *tab   /* I: Structure defining LT file           */
);

void put_i4_field
(
    struct TAB_DEF *tab,  /* I/O: Struct defining the LT file to be  */
    int val,              /* I: Field or record sep to be written    */
    int sep               /* I: The I/O buffer after the real field  */
);

void put_r4_field
(
    struct TAB_DEF *tab,  /* I/O: Struct for file to be processed    */
    double val,           /* I: Real value written into the I/O buf  */
    int sep               /* I: Field or record sep to be written    */
);

void put_r8_matrix
(
    struct TAB_DEF *tab,          /* I/O: Struct for file processed  */
    const struct MATRIX *in,      /* I: # of rows & col of in. matrix*/
    const struct MATRIX *lt_size, /* I: # of rows & cols of LT matrix*/
    double matrix[],      /* O: Double matrix written to the I/O buf */
    int sep               /* I: Field or record sep to be written    */
);

void put_vector
(
    struct TAB_DEF *tab,  /* I: Struct defining file to be processed*/
    const char label[],   /* I: Label name of the vector             */
    int dtype,            /* I: Data type of the label vector        */
    const char desc[],    /* I: Description of the label vector      */
    int nrow,             /* I: Number of rows                       */
    int ncol,             /* I: Number of columns                    */
    int sep               /* I: Sep(FIELD_SEP1,FIELD_SEP2,RECORD_SEP)*/
);

void put_null_field
(
    struct TAB_DEF *tab,  /* I/O: Defines LT file                    */
    int sep               /* I: Field or record separator            */
);

void put_null_matrix
(
    struct TAB_DEF *tab,       /* I/O: Struct for file to process    */
    const struct MATRIX *size, /* I: Matrix structure                */
    int sep                    /* I: The I/O buffer after real field */
);

void put_string_field
(
    struct TAB_DEF *tab,  /* I/O: Struct for file to be processed    */
    const char *val,      /* I: Char string  written into I/O buffer */
    int sep,              /* I: Field or record sep to be written    */
    int flag              /* I: Flag specifying string (delimited)   */
);

void upcase
(
    char *buf             /* I/O: Buffer to convert to upper case    */
);

void write_tab
(
    struct TAB_DEF *tab   /* I/O: Struct for file to be processed    */
);

#endif
