/******************************************************************************
FUNCTION:       GETALIAS

PURPOSE:        Searches the input string for a "$" (indicating an alias name),
                and returns the string with the alias text, if any, substituted
                for the alias name.
                Because the alias name has no termination character, it is
                assumed to be terminated by the end of the input string or
                by the occurrence of any non-alphanumeric character.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      10/86    C. Greenhagen       New Development
  1.1      12/87    B. Ailts            Change include dir. spec.
  5.0      04/88    D. Hollaren         changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed status code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
  5.1      09/88    B. Ailts            Expanded code to recognize the $$ as
                                        not an alias name
  5.2      03/89    B. Davis            commented out printf left from debugging
  5.3      10/90    T. Sturman          Added error checking for length of 
                                        output string before returning
  5.4      11/90    D. Etrheim          Corrected for alias of length 9.  Need
                                        ASZ+2 to handle lf and NULL chars.
                                        Changed declaration of addstring.
  5.1      12/90    D. Etrheim          Change nonfatal to 1.
  5.2      1/91     B. Davis            Standardized errmsg calls and status
                                        returns according to EDC guidelines.
  6.0      6/95     T. Wolbrink         Added string length checks prior
                                        to string concatenations to fix PR 33.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE 

PROJECT                 LAS

ALGORITHM 
Search for a "$" in the input string.
Copy any text preceeding the "$" into the output string.
Return if there were no aliases in the input string.
Get the user's ALIAS LIST and open it.
Parse the alias name from the input string
    Search the ALIAS LIST for a match
        substitute the alias text into the output string
    Copy any non-alias text into the output string
    Search for the next alias in the input string
Repeating the same steps for each alias in the input string.
Return the output string with all aliases substituted with there associated
alias text.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/file.h>
#include <fcntl.h>
#include "las.h"
#include "alias.h"
#include "local_protos.h"

/**
 * c_getalias:
 * @instring: I: The input string containing the alias
 *               name.  An alias name is indicated by a "$".
 * @outstring: O:  The output string which contains the
 *                 substituted value of the alias name.
 *
 * c_getalias() is used to retrieve an alias name and substitute
 * the value defined in an alias name table into the output string.
 * The input string is searched for a "$", if any are encountered
 * (indicating an alias name), then the retrieved alias text is
 * substituted for the alias name and returned in the output string.
 * The input string is returned in the output string when the input
 * string contains no alias name.  Because the alias name has no
 * termination character, it is assumed to be terminated by the end of
 * the input string or by the occurrence of any nonalphanumeric character.
 * An alias name must also be a maximum of nine characters long.
 * 
 * Returns:  E_SUCC: Successful substitution of alias name, if any. 
 *           E_FAIL: Unsuccessful, error in the alias name.
 */
int c_getalias
(
    const char *instring,               /* I: input string              */
    char *outstring                     /* O: output string             */
)
{
    #define ASZ     9                   /* max length of alias name     */
    #define TSZ     CMLEN-1             /* max length of alias text     */

    static char
        alphanumeric[] =                /* alphanumeric characters      */
        {"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"};

    char
        aliasfile[CMLEN],               /* name of user's alias file    */
        aliasname[ASZ+2],               /* alias name                   */
        aliastext[TSZ+1],               /* alias text                   */
        addstring[CMLEN],               /* string to accumulate text    */
        *tempstring;                    /* temporary work string        */
    const
        char *inptr;                    /* pointer to input string      */

    int
        fd,                             /* file descriptor              */
        len,                            /* string length                */
        morealias,                      /* more aliases to search for   */
        status;                         /* cmlock status                */


    char msg[ERRLEN+1];                 /* Error message string         */

    FILE *aliasf = NULL;                /* alias file                   */

    len = strcspn (instring,"$");               /* length of string     */
                                                /* preceding "$"        */
    if ((strlen (instring)) == len)             /* no alias used        */ 
    {
        if (len <  CMLEN) 
        {
            strcpy (outstring,instring);
            return (E_SUCC);
        }
        else
        {
            c_lasmsg (FALSE,"getalias-text","Text is too long");
            return   (E_FAIL);
        }
    }

    if (strlen(instring) >= CMLEN) 
    {
        c_lasmsg (FALSE,"getalias-text","Expanded alias text too long");
        return   (E_FAIL);
    }
    strncpy (addstring,instring,len);           /* copy 1st part of string  */
    addstring[len] = 0;

    bldalias (aliasfile);                       /* build alias file name   */
    status = cmlock(aliasfile,&fd,O_RDONLY,60); /* open alias file         */
    if (status == E_SUCC)
       aliasf = fdopen (fd,"r");

    if ((status != E_SUCC) || (aliasf == NULL))
    {
        sprintf  (msg,"Error opening file %s",aliasfile);
        c_lasmsg (FALSE,"getalias-open",msg);
        strcpy   (msg,"Error returned from cmlock");
        c_lasmsg (FALSE,"getalias-call",msg);
        return   (E_FAIL);
    }

    morealias = 1;
    inptr = instring;
    tempstring = strchr (inptr,'$');             /* get alias name */
    if (tempstring == 0)
        morealias = 0;

    while (morealias)
    {
        len = strspn (tempstring,"$");
        if (len == 2)
        {
            tempstring ++;
            len = strspn ((tempstring),"$");     /* name length    */
            if ((strlen(addstring) + len) >= CMLEN )
            {
                c_lasmsg (FALSE,"getalias-text",
                          "Expanded alias text too long");
                return   (E_FAIL);
            }
            strncat (addstring,tempstring,len);
            inptr = tempstring + len;
        }
        else
        {
            len = strspn ((tempstring+1),alphanumeric)+1;/* name length */
            if (len > ASZ)
            {
                c_lasmsg (FALSE,"getalias-name","Alias name too long");
                return   (E_FAIL);
            }

            strncpy (aliasname,tempstring,len);
            aliasname[len] = 0;
            inptr = tempstring + len;

            if (findalias(aliasf,aliasname,aliastext) == E_SUCC)
            {
                if ((strlen(addstring) + strlen(aliastext)) >= CMLEN)
                {
                    c_lasmsg (FALSE,"getalias-text",
                              "Expanded alias text too long");
                    return   (E_FAIL);
                }
                strcat (addstring,aliastext);
            }
            else
            {
                c_lasmsg (FALSE,"getalias-found",
                          "Specified alias not found");
                return   (E_FAIL);
             
            }
        }

            len = strcspn (inptr,"$");                /* copy non-alias text  */
            strncat (addstring,inptr,len);            /* to outstring         */

            tempstring = strchr (inptr,'$');          /* get next alias name  */
            if (tempstring == 0)
                morealias = 0;
            else
                rewind (aliasf);
    }

    /*move addstring to outstring before returning */
    strcpy (outstring,addstring);
    fclose (aliasf);

    return (E_SUCC);
}
