/******************************************************************************
FUNCTION:       FINDALIAS

PURPOSE:        Searches the user's alias file for the specified alias name.
                If the alias name is found, the associated text is returned.

PROGRAM HISTORY:
Version    Date     Author           Change Request
-------    -----    -------------    --------------
  1.0      10/86    C. Greenhagen     New Development
  1.1      12/87    B. Ailts          Change include dir. spec.
  5.0      04/88    D. Hollaren       changed include spec to las
                                      return E_SUCC/E_FAIL
  5.1      11/90    D. Etrheim        Corrected for alias of length 9.  Need
                                      ASZ+2 to handle lf and NULL chars.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE 
        Does NOT print any error messages prior to returning with E_FAIL.

PROJECT                 LAS

ALGORITHM 
Read each record from the user's ALIAS LIST file.
    When a match is found return the alias text.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/file.h>
#include "las.h"
#include "alias.h"

/**
 * findalias:
 * @aliasf: I: Alias file that is searched for
 *             the specified alias name. 
 * @aliasname: I: The specified alias name that is being
 *                searched for.
 * @aliastext: O: When a match is found for the specified
 *                alias name, the associated alias text is returned.
 *
 * findalias() searches the user's alias file for the specified
 * alias name. findalias() reads each record from the user's ALIAS LIST
 * file.  If the alias name is found, the associated alias text is 
 * returned.
 *
 * Note: The use of the findalias() routine by other
 * applications should be minimized. Only c_getalias()
 * should be used by most applications. 
 *
 * Returns E_SUCC: successful completion, a match is found, 
 *         E_FAIL: unsuccessful completion, no match found.
 */
int findalias 
(
    FILE *aliasf,                       /* I: alias file  (input)       */
    const char *aliasname,              /* I: alias name  (input)       */
    char *aliastext                     /* O: alias text  (output)      */
)
{
    #define ASZ     9                   /* max length of alias name     */
    #define TSZ     CMLEN-1             /* max length of alias text     */

    char
        buf[ASZ+2],                     /* buffer for reads             */
        *fptr;                          /* pointer to alias file        */
    int
        len,                            /* string length                */
        foundmatch;                     /* found alias name match t/f   */
    
    foundmatch = 0;

    while (foundmatch == 0)             /* read alias file              */
    {                                   /* search for alias name        */
        fptr = fgets (buf,ASZ+2,aliasf);
        if (fptr == NULL)
            return (E_FAIL);

        len = strlen (buf);
        buf[len-1] = 0;

        if (strcmp (aliasname,buf) == 0)        /* found a match,       */
        {                                       /* read alias text      */
            foundmatch = 1;
            fptr = fgets (aliastext,TSZ+1,aliasf);
            if (fptr == NULL)
                return (E_FAIL);
            else
            {
                len = strlen (aliastext);
                aliastext[len-1] = 0;
            }
        }
        else                                    /* not a match,         */
        {                                       /* skip alias text      */
            fptr = fgets (aliastext,TSZ+1,aliasf);
            if (fptr == NULL)
                return (E_FAIL);
        }
    }

    return (E_SUCC);
}
