/*******************************************************************************
FUNCTION:       LISTALIAS

PURPOSE:        Lists the specified aliases.
                If an alias name is specified, the alias is listed.
                If alias text is specified, all aliases assigned to the text
                are listed.
                If "$*" is specified, all aliases are listed.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      10/86    C. Greenhagen       New Development
  1.1      12/87    B. Ailts            Change include dir. spec.
  5.0      04/88    D. Hollaren         changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed status code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
  5.0       9/88    B. Davis            passed fp to allow user specified
                                        name for fileutil output
  5.1      11/90    D. Etrheim          Corrected for alias of length 9.  Need
                                        ASZ+2 to handle lf and NULL chars.
  5.2      12/90    D. Etrheim          Changed nonfatal to 1.
  5.3      1/91     B. Davis            Standardized errmsg calls and status
                                        returns according to EDC guidelines.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE 

PROJECT                 LAS

ALGORITHM 
Determine what operation is to be performed, set flags, and error check the
input string.
Open the user's ALIAS LIST file and an output print file.
Make sure that the ALIAS LIST file is not empty.
Read each line from the ALIAS LIST file.
    If input string matches the alias name, or if it matches the alias
    text or if all records were specified
        output the record to the print file.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/file.h>
#include <fcntl.h> 
#include "las.h"
#include "alias.h"
#include "local_protos.h"

/**
 * listalias: 
 * @pfile: I/O: An output file opened along with the user's  
 *            ALIAS LIST file.  If the input string matches
 *            the alias name, or if it matches the alias text,
 *            or if all records were specified, the record
 *            is outputted to the print file.
 * @instring: I: Input string compared with the alias name  
 *               or alias text. If it matches the specifations,
 *               or all records were specified, the record is 
 *               outputted to the print file.
 *
 * listalias() lists all specified aliases.  
 * If an alias name is specified, the alias is listed.
 * If alias text is specified, all aliases assigned to
 * the text are listed.  If "$*" is specified, all
 * aliases are listed.
 *
 * Note: The use of the listalias() routine by other
 * applications should be minimized. Only c_getalias()
 * should be used by most applications.
 *
 * Returns: E_SUCC:  successful completion (list of aliases),
 *          E_FAIL:  unsuccessful completion.
 */


int listalias 
(
    FILE *pfile,                /* I/O: print file               */
    const char *instring        /* I: input string               */
)
{
    #define ASZ     9                   /* max length of alias name      */
    #define TSZ     CMLEN-1             /* max length of alias text      */

    char
        aliasfile[CMLEN],               /* name of user's alias file     */
        aliasname[ASZ+2],               /* alias name                    */
        aliastext[TSZ+1],               /* alias text                    */
        msg[ERRLEN + 1];                /* Error message string          */

    int
        all=0,                          /* list all aliases (t=1,f=0)    */
        byname=0,                       /* select by name   (t=1,f=0)    */
        bytext=0,                       /* select by text   (t=1,f=0)    */
        fda,                            /* file descriptor of alias file */
        flags,                          /* flags for file open           */
        more,                           /* more records to read          */
        status;                         /* cmlock status                 */

    FILE *aliasf = NULL;                /* alias file                    */

/*************************************************************************/

    bldalias (aliasfile);               /* build alias file name         */

    if (strcmp (instring,"$*") == 0)    /* list all aliases     */
        all = 1;
    else if (instring[0] == '$')        /* select by alias name */
    {
        byname = 1;
        if (strlen (instring) > ASZ)
        {
            c_lasmsg (FALSE,"listalias-open","Alias name too long");
            return   (E_FAIL);
        }
    }
    else                                /* select by alias text */
        bytext = 1;

    flags = O_RDONLY;                   /* open alias file      */
    status = cmlock (aliasfile,&fda,flags,60);
    if (status == E_SUCC)
        aliasf = fdopen (fda,"r");

    if ((status != E_SUCC) || (aliasf == NULL))
    {
        sprintf  (msg,"Error opening file %s",aliasfile);
        c_lasmsg (FALSE,"listalias-open",msg);
        strcpy   (msg,"Error returned from cmlock");
        c_lasmsg (FALSE,"listalias-call",msg);
        return   (E_FAIL);
    }

    more = 1;                      /* read alias, file write selected records */
    if ((fgets (aliasname,ASZ+2,aliasf)) == NULL)
        more = 0;
    else if ((fgets (aliastext,TSZ+1,aliasf)) == NULL)
        more = 0;

    while (more)
    {
        aliasname[strlen(aliasname)-1] = 0;         /* replace lf by null   */
        aliastext[strlen(aliastext)-1] = 0;

        if ((byname && (strcmp (aliasname,instring)==0)) ||
            (bytext && (strcmp (aliastext,instring)==0)) || all)
        {                                       /* alias qualifies,     */
            fputs   (aliasname,pfile);              /* print it             */
            fprintf (pfile,"\n");
            fputs   (aliastext,pfile);
            fprintf (pfile,"\n\n");
        }

        if ((fgets (aliasname,ASZ+2,aliasf)) == NULL)
            more = 0;
        else if ((fgets (aliastext,TSZ+1,aliasf)) == NULL)
            more = 0;
    }

    fclose (aliasf);

    return (E_SUCC);
}
