/*******************************************************************************
FUNCTION:       PUTALIAS

PURPOSE:        Adds a new alias name and text to the user's alias file.
                A new alias file will be created if one does not exist.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      10/86    C. Greenhagen       New Development
  1.1      12/87    B. Ailts            Change include dir. spec.
  5.0      04/88    D. Hollaren         changed include spec to las
                                        change cerrmsg to c_errmsg and
                                        pass status code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
  5.1      03/89    B. Davis            Gave a more descriptive error message
                                        when the alias.file cannot be created
                                        because the users cm dir does not exist
  5.2      01/90    B. Davis            Allow a status from cmlock that means
                                        the directory does not exist to initiate
                                        the making of that directory
  5.3      1/91     B. Davis            Standardized errmsg calls and status
                                        returns according to EDC guidelines.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        MUST BE RUN UNDER TAE 

PROJECT                 LAS

ALGORITHM 
Get the user's ALIAS FILE directory and name spec.
If the file exists
    make sure that the alias name doesn't already exist.
Otherwise
    create a new ALIAS FILE for the user.
Add the new alias name to the user's ALIAS FILE.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/file.h>
#include <fcntl.h> 
#include "las.h"
#include "labserv.h"  /* c_lsstat prototype */
#include "alias.h"
#include "local_protos.h"

/**
 * putalias:
 * @aliasname: I: Name of alias to be added to the user's ALIAS FILE. 
 * @aliastext: I: Text of the alias to be added to the user's ALIAS FILE. 
 * @aliasfile: O: Name of user's ALIAS FILE.
 * @newfile:   O: If a file doesn't exit, one will be created. If one 
 *                was created: newfile = 1, if not newfile = 0.
 *
 * putalias() adds a new alias name and text to the user's alias file.
 * A new alias file will be created if one does not exist.
 *
 * Note: The use of the putalias() routine by other
 * applications should be minimized. Only c_getalias()
 * should be used by most applications.
 *
 * Returns E_SUCC: successful completion, (alias was added), 
 *         E_FAIL: unsuccessful completion (alias wasn't added).
 */
 
int putalias
(
    const char *aliasname,              /* I: name of alias add (input) */
    const char *aliastext,              /* I: text of alias add (input) */
    char *aliasfile,                    /* O: name of user's alias file */
    int *newfile                        /* O: new file created? y=1,n=0 */
)

{
    #define ASZ     9                   /* max length of alias name     */
    #define TSZ     CMLEN-1             /* max length of alias text     */

    int
        fd;                             /* file descriptor              */
    long
        flags,                          /* flags for file open          */
        status,                         /* cmlock status                */
        zero=0;                         /* zero (pass to lsstat)        */


    char buf[TSZ+1];                    /* buffer for reads             */
    char dirpath[CMLEN];                /* directory path               */
    char msg[ERRLEN + 1];               /* Error message string         */

    FILE *aliasf=NULL;                  /* alias file pointer           */

/************************************************************************/

    bldalias (aliasfile);               /* bld alias file name          */

    if (c_lsstat(aliasfile,&zero) == E_SUCC)
    {                                           /* open existing file   */
        *newfile = 0;
        flags = O_RDONLY;
        if ((status = cmlock (aliasfile,&fd,flags,60)) == E_SUCC)
            if ((aliasf = fdopen (fd,"r")) != NULL)
            {
                if (findalias (aliasf,aliasname,buf) == E_SUCC)
                {
                    c_lasmsg (FALSE,"putalias-exists",
                              "Alias name already exists");
                    return   (E_FAIL);
                }
                fclose (aliasf);
            }

        if ((status != E_SUCC) || (aliasf == NULL))
        {
            sprintf  (msg,"Error opening file %s",aliasfile);
            c_lasmsg (FALSE,"putalias-open",msg);
            strcpy   (msg,"Error returned from cmlock");
            c_lasmsg (FALSE,"putalias-call",msg);
            return   (E_FAIL);
        }
    }
    else
        *newfile = 1;                           /* create new file      */

                                                /* write to end of file */
    flags = O_WRONLY | O_CREAT | O_APPEND;
    status = cmlock (aliasfile,&fd,flags,60);

    if (status == ENOENT)
    {
        strcpy (dirpath,aliasfile);
        status = dirnm (dirpath);
        if (status == E_SUCC)
        {
            status = mkdr (dirpath);
            if (status == E_SUCC)
                status = cmlock (aliasfile,&fd,flags,60);
        }
    }

    if (status == E_SUCC)
        if ((aliasf = fdopen (fd,"a")) != NULL)
        {
            fputs   (aliasname,aliasf);
            fprintf (aliasf,"\n");
            fputs   (aliastext,aliasf);
            fprintf (aliasf,"\n");
        }

    if ((status != E_SUCC) || (aliasf == NULL))
    {
        sprintf  (msg,"Error opening file %s",aliasfile);
        c_lasmsg (FALSE,"putalias-open",msg);
        strcpy   (msg,"Error returned from cmlock");
        c_lasmsg (FALSE,"putalias-call",msg);
        if (status == -2)
        {
            strcpy   (msg,"A cm directory must exist under"
                     "the user's home directory.");
            c_lasmsg (FALSE,"putalias-open",msg);
            strcpy   (msg,"Alias names are stored in a Catalog"
                     "Manager file there.");
            c_lasmsg (FALSE,"putalias-open",msg);
            strcpy   (msg,"Error returned from cmlock");
            c_lasmsg (FALSE,"putalias-call",msg);
       }
       return(E_FAIL);
    }

    fclose (aliasf);
    return (E_SUCC);
}
