/*********************   Catalog Manager Routines   *************************
FUNCTION:       C_CMHOST

PURPOSE:        Given a TAE file name, c_cmhost returns the fully specified
                host file name. 

PROGRAM HISTORY:
 Version    Author     Date    Change Request
 --------   -------    -----   --------------
            K. Gacke    8/86    initial development
            K. Gacke    2/87    allow device names to be specified on VMS
            B. Ailts    12/87   replace descriptors with strings
                                change name to c_cmhost
    5.0     D. Hollaren 04/88   changed include spec to las
                                changed cerrmsg to c_errmsg and
                                passes status code by address
                                return status E_SUCC/E_FAIL
    5.0     B. Davis    09/88   clarified tae parent directory check
    5.0     D. Akkerman 05/90   Modified to allow a direct UNIX path
                                specification by entering a "%" as the first
                                character in the LAS path.
                                Modified to allow UNIX path name specifications
                                in the form of LAS path names where the UNIX
                                slash is replaced by the LAS ".".
    5.1     D.Etrheim   01/91   Standardized error message handling
    6.0     J.Fenno     03/94   Ensure the filename/path is all lowercase
    7.1     J.Janssen   01/99   Added the function c_cmhostcs to allow a
                                case sensitive version of c_cmhost.  c_cmhostcs
                                contains all of the code for the old c_cmhost
                                except the lines that make the path all 
                                lowercase. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
  If a "%" is the first character then:
         Parse the remainder of the input string as a straight UNIX path.
  Else:
        If the TAE file name has the following delimeters
                '[' --> begin directory specification
                ']' --> end directory specification
                '.' --> child directory
                '-' --> parent directory
        then a TAE file name will fall in one of four categories.
        1) fname            --> file in current working directory
        2) [.dir1 ...]fname --> file in child directory of current working 
           directory
        3) [- ...]fname     --> file in parent directory of current working 
           directory
        4) [user ...]fname  --> file in user default directory

  In the first three cases the current working directory is copied into 
  the host file array.  Each character of the TAE name is examined and 
  translated to the host file name.  

  In the fourth case the user name is extracted from the TAE file name.  On 
  UNIX systems the default directory is returned by the "get_def_dir" routine
  or the LAS path specification is taken as a direct UNIX path.  On VMS systems
  the user code is the default directory.  Each character after the user name 
  of the TAE name is examined and translated to the host file name.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "cm.h"
#include "local_protos.h"

/**
 * c_cmhost:
 * @tae: I: Input TAE file name.
 * @hname: I/O: Output host file name.
 *
 * Given a TAE file name, c_cmhost() returns the fully specified
 * host file name using the following algorithm: 
 * If a "%" is the first character then:
 *        Parse the remainder of the input string as a straight UNIX path.
 * Else:
 *       If the TAE file name has the following delimeters
 *               '[' --> begin directory specification
 *               ']' --> end directory specification
 *               '.' --> child directory
 *               '-' --> parent directory
 *       then a TAE file name will fall in one of four categories.
 *       1) fname            --> file in current working directory
 *       2) [.dir1 ...]fname --> file in child directory of current working 
 *          directory
 *       3) [- ...]fname     --> file in parent directory of current working 
 *          directory
 *       4) [user ...]fname  --> file in user default directory
 *       (... = may be more directories added on)
 *
 * In the first three cases the current working directory is copied into 
 * the host file array.  Each character of the TAE name is examined and 
 * translated to the host file name.  
 *
 * In the fourth case the user name is extracted from the TAE file name.  On 
 * UNIX systems the default directory is returned by the get_def_dir() routine
 * or the LAS path specification is taken as a direct UNIX path.  On VMS systems
 * the user code is the default directory.  Each character after the user name 
 * of the TAE name is examined and translated to the host file name.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation. 
 */

int c_cmhost
(
    const char *tae,     /* I: Input TAE file name                           */
    char *hname          /* I/O: Output host file name                       */
)
{

    char user_code[CMLEN];     /* user name                                  */
    char path[CMLEN];
    char *dirbeg;              /* pointer to TAE begin directory character   */
    char *dirend;              /* pointer to TAE end directory character     */
    char *hptr;                /* pointer to end of host file name           */
    char *ptr;                 /* temporary pointer for user code            */
    const char *tptr;          /* pointer to current position in TAE name    */


    /* If the LAS filename contains a left bracket or semicolon, the name is
       in traditional LAS format.  Otherwise assume host (Unix) format.      */
    if (strchr(tae, TAE_DIR_BEG) || strchr(tae, TAE_EXT))
    {
        /* Process a LAS path specification. Check for the dir delimeters.   */
        dirbeg = strchr(tae,TAE_DIR_BEG);
        dirend = strrchr(tae,TAE_DIR_END);
        if (((dirbeg == 0) && (dirend != 0)) || ((dirbeg != 0)
            && (dirend == 0)))
        {
            c_lasmsg (FALSE, "cmhost-invalid", 
                     "Invalid LAS file directory specification");
            return(E_FAIL);
        }

        if (dirbeg == 0)
        {
            /* Case 1 in which only file name was specified.   */
            getcwd(hname,CMLEN);
            tptr = tae;
        }
        else if ((*(dirbeg+1)==TAE_PARENT) || (*(dirbeg+1)==TAE_DIR_SEP))
        {
            /* Case 2 and 3 in which a parent or child directory
               from the current working directory was specified.             */
            getcwd(hname,CMLEN);
            tptr = tae + 1;
        }
        else
        {
            /* Case 4 in which the TAE user name was specified.              */
            for (tptr = dirbeg + 1, ptr = user_code; 
                (*tptr != TAE_DIR_SEP) && (*tptr != TAE_DIR_END); 
                *ptr++ = *tptr++);
                *ptr = '\0';

            if (get_def_dir(user_code,hname) == E_FAIL)
                sprintf(hname,"/%s",user_code);
        }

        hptr = hname + strlen(hname) - 1;

        /* Translate directory part of TAE name to the host file name.      */
        for (; tptr<dirend; tptr++)
        {
            switch (*tptr)
            {
                case TAE_PARENT:
                    /* Ensure that there is a parent directory. */
                    if ((hptr = strrchr(hname,HOST_DIR_SEP)) == 0)
                    {
                        c_lasmsg (FALSE, "cmhost-parent", 
                                 "Too many parent directories specified");
                        return(E_FAIL);
                    }
                    *hptr-- = '\0';
                    break;

                case TAE_DIR_SEP:
                    *(++hptr) = HOST_DIR_SEP;
                    break;

                case TAE_DIR_BEG:
                    *(++hptr) = HOST_DIR_BEG;
                    break;

                case TAE_DIR_END:
                    *(++hptr) = HOST_DIR_END;
                    break;

                default:
                    *(++hptr) = *tptr;
                    break;
            }
        }

        if (dirend != 0)
            ++tptr;

        /* Only append final "/" if a filename is included. */
        if (*tptr != 0)
            *(++hptr) = HOST_DIR_END;


        /* Translate the file name of the TAE name to the host file name     */
        for (; *tptr!='\0'; tptr++)
        {
            switch (*tptr)
            {
                case TAE_SEP: 
                    *(++hptr) = HOST_SEP; 
                    break;
                case TAE_EXT: 
                    *(++hptr) = HOST_EXT; 
                    break;
                default:      
                    *(++hptr) = *tptr;    
                    break;
            }
        }

        *(++hptr) = '\0';          /* null terminate the host file name      */
    }

    else   /* Unix filename syntax */
    {
        /* If the path begins with a tilde, the path is relative to a user's
           home directory.  Prepend the correct home path.                   */
        if (tae[0] == '~')
        {
            for (tptr = tae, ptr = user_code; *tptr != '/'; 
                *ptr++ = *tptr++);
                *ptr = '\0';

            if (strcmp(user_code, "~") == 0)
            {
                if (getmydef(hname) == E_FAIL)
                {
                    c_lasmsg (FALSE, "cmhost-invalid", 
                             "Invalid Las file directory specification");
                    return(E_FAIL);
                }
            }
            else
            {
                if (get_def_dir(&user_code[1], hname) == E_FAIL)
                {
                    c_lasmsg (FALSE, "cmhost-invalid", 
                             "Invalid Las file directory specification");
                    return(E_FAIL);
                }
            }
        }
        else if (tae[0] != '/')
        {
            if (getcwd(hname, CMLEN) == NULL)
            {
                c_lasmsg(FALSE, "cmhost-pathlen", "Path length too long.  "
                         "(Must be less than %d characters.)", CMLEN);
                return(E_FAIL);
            }    
            tptr = tae;
        }
        else
        {
            hname[0] = '\0';
            tptr = tae; 
        }
    
        /* Make sure that any relative paths, including . and .. are exanded
           into the full path.  Ensure that the path exists.                 */
        strcpy(path, tptr);
        tptr = strtok(path, "/");
        while (tptr != NULL)
        {
            if (strcmp(tptr, "..") == 0)
            {
                hptr = strrchr(hname, '/');
                *hptr = '\0';
            }
            else if ((*tptr != '\0') && (strcmp(tptr, ".") != 0))
            {
                strcat(hname, "/");
                strcat(hname, tptr);
            }
            tptr = strtok(NULL, "/");
        }
    }

    return(E_SUCC);
}

