#include <stdio.h>
#include <string.h>
#include "cm.h"
#include "ddr.h"
#include "imageio.h"     /* SL, SS, NL, NS definitions */
#include "las.h"
#include "ltable.h"      /* FATAL and NONFATAL definitions */
#include "pixman.h"
#include "util.h"
#include "util2.h"
#include "worgen.h"
#include "local_protos.h"
#include "labserv.h"

/*******************************************************************************
NAME:       c_copy_image


PURPOSE:    Copies an image into another image.  It allows the input
            image(s) to be sub-windowed and sub-banded when copied to
            the output image.  The input image may be BYTE, INTEGER*2,
            INTEGER*4, or REAL*4.  The output image may be converted
            to any one of the four data types (the default is no data
            conversion).

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      N.Donahue       4/83           CSC           Original Development

  4.0      F. Irani        8/87           SAR           Rename to COPY from
                                                        FILECOPY and modify to
                                                        LAS 4.0 standards

  5.0      B. Ailts        6/88           EDC           Add UPDDR routine
                                                        Take descriptors out
                                                        of calling sequences
                                                        Replace newlas.h with
                                                        las.h
                                                        Delete tae.h
                                                        Place 'c_' in front of
                                                        las subroutines
                                                        Place subroutines in a
                                                        seperate file
 5.1      B. Ailts         10/88         EDC            Deleted flag argument
                                                        from the call to hstout
 5.2       B.Ailts        12/88         EDC/CSB         Added subroutine RNDPROC                          
                                                        in order to round when
                                                        the parameter ROUNDOPT
                                                        is set to ROUND and
                                                        the data is being
                                                        converted from REAL to
                                                        a lower data type
 5.3     B.Ailts         04/88          EDC/CSB         Corrected problem with
                                                        rounding negative num.
 7.2     G.Schmidt       06/00          EDC/CSB         Turned this into a
                                                        C-callable library
                                                        routine.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:                        LAS

ALGORITHM DESCRIPTION:
        The input parameters and globals are retrieved.
        Error checking is done on the input image(s) to ensure that they all
        have equal window sizes.
        Each line of the input image is copied to the corresponding line of the
        output image.
        The associated history file is created to represent the output
        image.
        A completion message is written.

ALGORITHM REFERENCES:           none

NOTES:
1.  When subsampling an image, the output image origin is reset
    to (1,1).

2.  COPY may also be used to combine several separate LAS images
    into a single image.

3.  COPY truncates when converting from a higher (REAL*4) to
    lower (BYTE) data type using the ODTYPE parameter.  The MAP
    module should be used to rescale images.

4.  Some images may not compress and may actually expand when the
    COMPFLG is set to YES.  Some example images that do this are
    created by TESTGEN-RAN.  Images that expand generally are
    random and use most of the possible intensities.

******************************************************************************/
/******************************************************************************
NAME:       ci_getpar

PURPOSE:    To get the necessary information from the input parameters.  Also
            retrieves the TAE global parameters.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
 7.2     G.Schmidt       07/00          EDC/CSB         Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:                        LAS

ALGORITHM:

ALGORITHM REFERENCES:                none
******************************************************************************/


static int ci_getpar
(
    int  odtype,                  /* I:   Data type of output image
                                          (0=SAME, EBYTE=BYTE, EWORD=I*2,
                                          ELONG=I*4, EREAL=R*4)              */
    int  nmrimg,                  /* I:   Number of input images             */
    const char hosin[MAXIMG][CMLEN],    /* I:   Filenames of input images (Unix)
                                          hosin may be a single- or multi-band
                                          image and may contain BYTE, I*2,
                                          I*4, or R*4 pixel data.            */
    int  window[MAXIMG][4],       /* I/O: Window for each input image.
                                          (SL, SS, NL, NS)
                                          If all values are 0s then the entire
                                          image will be processed.  If the first
                                          window is all 0 then it is assumed the
                                          rest of the windows are all 0s. If the
                                          images are not of the same size then
                                          processing will be stopped.  On exit
                                          window will contain the values for
                                          each image.                        */
    int rtopt_in,                 /* I:   Round/truncation option
                                          TRUE = Round the input data
                                          FALSE = Truncate the input data    */
    int bands[MAXIMG][MAXBND+1], /* I/O:  Bands to process - all images
                                          If all 0s then all bands in all the
                                          images will be processed.  nbands
                                          also needs to be set to 0 for the
                                          default.                           */
    int nbands[MAXIMG],          /* I/O:  Number of bands per input image
                                          If bands are all 0s and nbands[i]
                                          is 0 then all bands in all the
                                          images will be processed           */
    int  *totbnd,                 /* I/O: Total number of bands.  Set to 0 if
                                          processing the default bands       */
    int *rtopt                    /* O:   Round/truncation option
                                          FALSE = TRUNC, TRUE = ROUND        */
)
{

    int i;
    int j;                      /* Looping variables                         */
    int window_specd = TRUE;    /* Flag to mark if the window was specified  */
    int bands_specd = TRUE;     /* Flag to mark if the bands were specified  */
    char tname[1][CMLEN];       /* TAE name of image specified               */
    char hname[1][CMLEN];       /* Host name of image specified              */
    int  in_dtype;              /* Input image data type                     */
    int  status;                /* Function return status code               */
    int  one = 1;               /* Value of one                              */
    int  nimg;                  /* Number of images for the c_getin2 call    */
    int  tmp_window[1][4];      /* Image window (sl ss nl ns) for c_getin2   */
    int  tmp_bands[1][MAXBND+1];/* Image bands for c_getin2 call             */
    int  tmp_nbands;            /* Number of bands in the image for c_getin2 */
    int  tmp_totbnd;            /* Total number of bands specified c_getin2  */
    int  same;                  /* 1 if all windows the same for c_getin2    */
    int  qflag[1][2];           /* 1 user defined window & bands else:
                                   0 if default window and bands for c_getin2*/

    /* Check the window to see if it was specified                           */
    if (window[0][SL] == 0 && window[0][SS] == 0 &&
        window[0][NL] == 0 && window[0][NS] == 0)
    window_specd = FALSE;

    /* Check the bands to see if they were specified                         */
    if (*totbnd == 0)
        bands_specd = FALSE;

    /* If the window was not specified, then the rest of the imagery needs
       to be checked to verify the window sizes are all the same.
       If the bands are all 0s then nbands also needs to be 0s and totbnd needs
       to be 0.  Check totbnd to see if it is 0.  If it is then each image
       needs to be processed to determine the bands and the number of bands. */
    if (!window_specd || !bands_specd)
    {
        for (i = 0; i < nmrimg; i++)
        {
            /* Parse each input image parameter pulling the bands, image names,
               and windowing information.  c_getin2 call requires TAE formatted
               image names                                                   */
            if (las_getin2 (hosin[i], tname, hname, &nimg, tmp_window, 
                tmp_bands,&tmp_nbands, &tmp_totbnd, one, &same, qflag)
                != E_SUCC)
            {
                c_lasmsg (FALSE, "copy-getin2", "Error in c_getin2 call");
                return (E_FAIL);
            }

            /* If the windowing of this image is a match, determine if the
               windowing across all the images match.  Only check if the
               window was not originally specified.                          */
            if (same != 0 && !window_specd)
            {
                if (i != 0 && ((window[i-1][NL] != tmp_window[0][NL]) ||
                   (window[i-1][NS] != tmp_window[0][NS])))
                {
                    same = 0;
                }
                else
                {
                    /* Set window to the tmp_window values                   */
                    for (j = SL; j <= NS; j++)
                        window[i][j] = tmp_window[0][j];
                }
            }

            /* Issue message if input image windows don't match.  Only check
               if the window was not originally specified.                   */
            if (same == 0 && !window_specd)
            {
                c_lasmsg (FALSE,"copy-window",
                          "Input images do not have same window sizes");
                return (E_FAIL);
            }

            /* Accumulate the bands and the total number of bands            */
            if (!bands_specd)
            {
                *totbnd += tmp_nbands;
                nbands[i] = tmp_nbands;
                for (j = 0; j < tmp_nbands; j++)
                     bands[i][j] = tmp_bands[0][j];
            }
        }
    }

    /* Determine whether or not to round or truncate                         */
    *rtopt = FALSE;
    if (odtype != EREAL)
    {
        if (rtopt_in)
        {
            /* Round the image */
            if (odtype != 0)
            {
                for (i = 0; i < nmrimg; i++)
                {
                    status = las_gettyp (hosin[i], one, &in_dtype);
                    if (status != E_SUCC)
                    {
                        c_lasmsg (FALSE, "copy_image-gettype",
                                 "Error getting the file type");
                        return (E_FAIL);
                    }
                    if (in_dtype == EREAL)
                        *rtopt = TRUE;
                }
            }
            else
            {
                /* If odtype == SAME rounding will be done if first input image
                   is NOT EREAL and one of the other input images is EREAL   */
                status = las_gettyp (hosin[0], one, &in_dtype);
                if (in_dtype != EREAL)
                {
                    for (i = 1; i < nmrimg; i++)
                    {
                        status = las_gettyp (hosin[i], one, &in_dtype);
                        if (status != E_SUCC)
                        {
                            c_lasmsg (FALSE, "copy_image-gettype",
                                      "Error getting the file type");
                            return (E_FAIL);
                        }
                        if (in_dtype == EREAL)
                            *rtopt = TRUE;
                    }
                }
            }  /* else if odtype != 0 */
        }  /*  if rtopt == ROUND  */
    }  /*  if odtype != EREAL  */

    return (E_SUCC);
}


/******************************************************************************
NAME:       process_finish

PURPOSE:    To cleanup/finish for process and rndproc routines.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
 7.2     G.Schmidt       07/00          EDC/CSB         Stripped common post-
                                                        processing code from
                                                        ci_process and
                                                        ci_rndproc.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:                        LAS

ALGORITHM:

ALGORITHM REFERENCES:                none
******************************************************************************/

static int ci_process_finish
(
    struct PARBLK *vblock,      /* I: TAE parameter block                   */
    const char hosin[][CMLEN],  /* I: host names of input images            */
    const char hosout[CMLEN],   /* I: host name of output image             */
    int  nmrimg,                /* I: number of input images                */
    int  window[][4],           /* I: window specification of input images  */
    int  bands[][MAXBND+1],     /* I: bands to be processed for all images  */
    int  nbands[],              /* I: number of bands per input image       */
    int  totbnd,                /* I: total number of bands to be processed */
    char minmax[],              /* I: min max flag                          */
    struct GDESC *gdesc,        /* I: group descriptor used for image I/O   */
    char *thebuf,               /* I: image I/O buffer                      */
    double minval[MAXBND],      /* I: min pixel value array                 */
    double maxval[MAXBND]       /* I: max pixel value array                 */
)

{
    double line_inc;    /* line increment          */
    double samp_inc;    /* sample increment        */
    int  upddr_flag;    /* flag for update DDR     */

    /* Final write      */
    if (c_estep (&gdesc) != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-step",
                 "Error stepping through the image");
        return (E_FAIL);
    }

    /* Turn off display messages */
    c_stoptm ();

    /* Update the DDR            */
    line_inc = (double) 1.0;
    samp_inc = (double) 1.0;
    upddr_flag = NOCOMB;
    if (las_upddr (vblock, hosin, hosout, nmrimg, window, bands, nbands,
        line_inc, samp_inc, totbnd, upddr_flag) != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-upddr",
                 "Error updating the DDR");
        return (E_FAIL);
    }

    /* Create associated min/max file  */
    if (las_svmnmx (hosout, totbnd, minval, maxval, minmax) != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-warn",
                 "Fatal error encountered saving the min/max file");
        return (E_FAIL);
    }

    /* Close all image files in the group                                    */
    if (c_egclse (&gdesc) != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-gclse",
                 "Fatal error encountered closing the image files");
        return (E_FAIL);
    }

    /* Free buffer   */
    free (thebuf);

    return (E_SUCC);
}

/******************************************************************************
NAME:       process_setup

PURPOSE:    To setup the variables needed for process and rndproc routines.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
 7.2     G.Schmidt       07/00          EDC/CSB         Stripped common pre-
                                                        processing code from
                                                        ci_process and
                                                        ci_rndproc.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:                        LAS

ALGORITHM:

ALGORITHM REFERENCES:                none
******************************************************************************/

static int ci_process_setup
(
    int  round,               /* I: TRUE is rndproc, FALSE is process        */
    long out_dtype,           /* I: output data type                         */
    int  opt_zero,            /* I: value for the data type in the option array
                                  for opening the output image               */
    const char hosin[][CMLEN],/* I: host names of input images               */
    const char hosout[CMLEN], /* I: host name of output image                */
    int comp,                 /* I: Compression flag
                                    TRUE = Compress output image
                                    FALSE = Don't compress output image
                                    EXT_COMP = Use external compres_opt
                                    or COMPRES1 environment variable         */
    int  nmrimg,              /* I: number of input images                   */
    int  window[][4],         /* I: window specification of input images     */
    long  bands[][MAXBND+1],  /* I: bands to be processed for all images     */
    int  nbands[],            /* I: number of bands per input image          */
    long totbnd,              /* I: total number of bands to be processed    */
    long  *odtype,            /* I/O: data type of output image (if SAME then
                                      odtype is changed to EBYTE, ELONG, EWORD,
                                      or EREAL depending on the input dtype  */
    int  msgtim,              /* I: elapsed time between display messages    */
    const char minmax[],      /* I: min max flag                             */
    long  *nl,                /* O: number of image lines                    */
    long  *ns,                /* O: number of image samples                  */
    struct FDESC *fdesc[],    /* O: image file descriptors used for image I/O*/
    struct GDESC **gdesc,     /* O: group descriptor used for image I/O      */
    long  offset1[],          /* O: offsets into "thebuf" for input bands    */
    long  offset2[],          /* O: offsets into "thebuf" for output bands   */
    int  *mmflag,             /* O: min/max flag                             */
    int  *size,               /* O: number of bytes per pixel                */
    char **thebuf,            /* O: image I/O buffer                         */
    double minval[],          /* O: min pixel value array                    */
    double maxval[]           /* O: max pixel value array                    */
)

{
    extern long curent;   /* current processing line             */
    long access;          /* image I/O access type               */
    int  band;            /* loop index counting the bands       */
    long bufsiz;          /* size of image I/O buffer            */
    int  image;           /* loop index counting input images    */
    int  msg_flag=ALLCAT; /* flag for processing message         */
    long one = 1;         /* one                                 */
    long opt[3];          /* option array                        */
    long sl;              /* starting image line                 */
    long ss;              /* starting image sample               */
    long zero = 0;        /* value of zero                       */

    /* Open input image(s) */
    access = IREAD;
    *nl = window[0][NL];
    *ns = window[0][NS];

    for (band = 0, image = 0; image < nmrimg; image++)
    {
        sl = window[image][SL];
        ss = window[image][SS];
        if (c_eopens (&fdesc[image], hosin[image], bands[image], &sl, &ss,
            nl, ns, &offset1[band], &access, &out_dtype, &zero, &zero)
            != E_SUCC)
        {
            c_lasmsg (FALSE, "copy_image-opens",
                     "Fatal error encountered opening the imagery");
            return (E_FAIL);
        }
        band += nbands[image];
    }

    /* Get the output data type if it is specified as SAME       */
    if (*odtype == SAME)
        *odtype = out_dtype;

    /* Open output image                                         */
    access = IWRITE;
    opt[0] = opt_zero;
    opt[2] = 0;
    if (comp == TRUE)
        opt[1] = COMPRES;
    else if (comp == FALSE)
        opt[1] = NO_COMPRES;
    else  /* comp == EXT_COMP */
        opt[1] = 1;  /* use external compres_opt or COMPRES1 environment var */

    if (c_eopens (&fdesc[nmrimg], hosout, &totbnd, &one, &one, nl, ns,
        offset2, &access, &out_dtype, &zero, opt) != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-opens",
                 "Fatal error encountered opening the output image");
        return (E_FAIL);
    }
    fdesc[nmrimg+1] = 0;

    /* Allocate I/O buffer                                       */
    if ((*thebuf = c_ealloc (fdesc, &bufsiz)) == NULL)
    {
        c_lasmsg (FALSE, "copy_image-alloc",
                 "Error allocating dynamic memory");
        return (E_FAIL);
    }

    /* Group images   */
    if (c_egroup (fdesc, gdesc, (unsigned char *) *thebuf, &bufsiz) != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-group", "Error grouping images");
        return (E_FAIL);
    }

    /* Setup for data type                       */
    if (round)
    {
        *size = 4;
    }
    else
    {
        switch (*odtype)
        {
            case EBYTE:
                *size = 1; 
                break;
            case EWORD:  
                *size = 2;
                break;
            case ELONG:  
                *size = 4; 
                break;
            case EREAL:  
                *size = 4; 
                break;
            default:
                c_lasmsg (FALSE, "copy_image-odtype",
                          "Only BYTE, WORD, LONG, and REAL data types"
                          " are supported");
                return (E_FAIL);
        }
    }

    /* Setup for processing message utility    */
    las_inittm ("copy_image", "processed", "line", "bands", " ", *nl, totbnd,
              msgtim, msg_flag);
    curent = 1;
    c_startm ();

    /* Initialize min/max arrays/mmflag is set to be used within the
       processing loop--quicker than strcmp                                  */
    if (strcmp (minmax, "YES") == 0)
    {
        las_inmnmx (*odtype, totbnd, minval, maxval);
        *mmflag = TRUE;
    }
    else
        *mmflag = FALSE;

    return (E_SUCC);
}



/******************************************************************************
NAME:       ci_process

PURPOSE:    To copy input image(s) into the output image. The input
            image(s) can be sub-windowed and sub-banded when copied to
            the output image.  The input image may also be converted to
            BYTE, INTEGER*2, INTEGER*4, or REAL*4 and written to the
            output image.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      N.Donahue       4/83           CSC           Original Development

  4.0      F. Irani        8/87           SAR           Rename to COPY from
                                                FILECOPY and modify to
                                                LAS 4.0 standards

  5.0      B. Ailts        6/88           EDC           Add UPDDR routine
                                                Take descriptors out
                                                the calling sequences
                                                Replace newlas.h with
                                                las.h
                                                Delete tae.h
                                                Place 'c_' in front of
                                                las subroutines
                                                Place subroutines in a
                                                seperate file
 7.2     G.Schmidt       06/00          EDC/CSB         Turned this into a
                                                        C-callable library
                                                        routine.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:                        LAS

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   The data is converted upon input to the data type of the output image
   The one line of each band and each image are processed at the same time.
   Save the minmax values if $MINMAX was set to "YES"
   Update the output ddr
   Close the input and output image files

ALGORITHM REFERENCES:           none
******************************************************************************/


static int ci_process
(
    struct PARBLK *vblock,      /* I: TAE parameter block                    */
    const char hosin[][CMLEN],  /* I: host names of input images             */
    const char hosout[CMLEN],   /* I: host name of output image              */
    int comp,                   /* I: Compression flag
                                      TRUE = Compress output image
                                      FALSE = Don't compress output image
                                      EXT_COMP = Use external compres_opt
                                      or COMPRES1 environment variable       */
    int  nmrimg,                /* I: number of input images                 */
    int  window[][4],           /* I: window specification of input images   */
    int  bands[][MAXBND+1],     /* I: bands to be processed for all images   */
    int  nbands[],              /* I: number of bands per input image        */
    int  totbnd,                /* I: total number of bands to be processed  */
    long  odtype,               /* I: data type of output image              */
    int  msgtim,                /* I: elapsed time between display messages  */
    char minmax[]               /* I: min max flag                           */
)

{
    struct FDESC *fdesc[MAXIMG+1];
                                /* image file descriptors used for image I/O */
    struct GDESC *gdesc = NULL; /* group descriptor used for image I/O       */
    long offset1[MAXIMG*MAXBND];/* offsets into thebuf for input bands       */
    long offset2[MAXBND];       /* offsets into "thebuf" for output bands    */
    extern long curent;         /* current processing line                   */

    double minval[MAXBND];      /* min pixel value array                     */
    double maxval[MAXBND];      /* max pixel value array                     */

    int  band;                  /* loop index counting the bands             */
    int  curlin;                /* current line being processed              */
    int  i;                     /* looping variable                          */
    int  mmflag;                /* min/max flag                              */
    long nl;                    /* number of image lines                     */
    long ns;                    /* number of image samples                   */
    int  size;                  /* number of bytes per pixel                 */

    char *thebuf = NULL;        /* image I/O buffer                          */

    /* Initialize fdesc to NULL */
    for (i = 0; i < MAXIMG+1; i++)
        fdesc[i] = NULL;

    /* Setup for processing */
    if (ci_process_setup (FALSE, odtype, 0, hosin, hosout, comp, nmrimg,
        window, bands, nbands, totbnd, &odtype, msgtim, minmax, &nl, &ns, fdesc,
        &gdesc, offset1, offset2, &mmflag, &size, &thebuf, minval, maxval)
        != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-process_setup",
                  "Error encountered setting up for process");
        return (E_FAIL);
    }

    /* Main processing loop */
    for (curlin = 0; curlin < nl; curlin++)
    {
        if (c_estep (&gdesc) != E_SUCC)
        {
            c_lasmsg (FALSE, "copy_image-step",
                      "Error stepping through the image");
            return (E_FAIL);
        }
        for (band = 0; band < totbnd; band++)
        {
            las_pixcopy ((unsigned char *) (thebuf + (*(offset1+band) * size)),
                         (unsigned char *) (thebuf + (*(offset2+band) * size)),
                          odtype, ns);
            if (mmflag)
            {
                if (las_pixminmax ((long *) (thebuf + (*(offset2+band) * size)),
                    ns, &minval[band], &maxval[band], odtype) != E_SUCC)
                {
                    c_lasmsg (FALSE, "copy_image-pxmnmx",
                              "Error encountered getting min/max");
                    return (E_FAIL);
                }
            }
        }
        curent++;
    }

    /* Finish processing and cleanup                                   */
    if (ci_process_finish (vblock, hosin, hosout, nmrimg, window, bands,
        nbands, totbnd, minmax, gdesc, thebuf, minval, maxval) != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-process_finish",
                 "Error encountered cleaning up for process");
        return (E_FAIL);
    }

    return (E_SUCC);
}


/******************************************************************************
NAME:       ci_rndproc

PURPOSE:    To round REAL*4 input image(s) and convert to ODTYPE. The input
            image(s) can be sub-windowed and sub-banded when copied to
            the output image.  The input image may also be converted to
            BYTE, INTEGER*2, or INTEGER*4  and written to the output image.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
 5.1     B.Ailts         12/88    EDC/CSB       Added this subroutine
                                                in order to round when
                                                the parameter ROUNDOPT
                                                is set to ROUND and
                                                the data is being
                                                converted from REAL to
                                                a lower data type
 5.2     B. Ailts        05/89    EDC/CSB       Changed code to round
                                                negative numbers
                                                correctly
 7.2     G.Schmidt       06/00    EDC/CSB       Turned this into a
                                                C-callable library
                                                routine.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:                        LAS

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   The data is converted upon input to the REAL*4 data type.
   The one line of each band and each image are processed at the same time.
   The data values have .5 added to each for acheive rounding
   Save the minmax values if $MINMAX was set to "YES"
   Update the output ddr
   Close the input and output image files

ALGORITHM REFERENCES:           none
******************************************************************************/

static int ci_rndproc
(
    struct PARBLK *vblock,      /* I: TAE parameter block                    */
    const char hosin[][CMLEN],  /* I: Host names of input images             */
    const char hosout[CMLEN],   /* I: Host name of output image              */
    int comp,                   /* I: Compression flag
                                       TRUE = Compress output image
                                       FALSE = Don't compress output image
                                       EXT_COMP = Use external compres_opt
                                       or COMPRES1 environment variable      */
    int  nmrimg,                 /* I: Number of input images                */
    int  window[][4],            /* I: Window specification of input images  */
    int  bands[][MAXBND+1],      /* I: Bands to be processed for all images  */
    int  nbands[],               /* I: Number of bands per input image       */
    int  totbnd,                 /* I: Total number of bands to be processed */
    long odtype,                 /* I: Data type of output image             */
    int  msgtim,                 /* I: Elapsed time between display messages */
    char minmax[]                /* I: Min max flag                          */
)

{
    struct FDESC *fdesc[MAXIMG+1];
                                 /* Image file descriptors used for image I/O*/
    struct GDESC *gdesc = NULL;  /* Group descriptor used for image I/O      */
    long offset1[MAXIMG*MAXBND]; /* Offsets into thebuf for input bands      */
    long offset2[MAXBND];        /* Offsets into "thebuf" for output bands   */
    extern long curent;          /* Current processing line                  */

    static double minval[MAXBND];/* Min pixel value array                    */
    static double maxval[MAXBND];/* Max pixel value array                    */

    float *rthebuf=NULL;         /* Pointer to thebuf                        */

    int  band;                   /* Loop index counting the bands            */
    int  curlin;                 /* Current line being processed             */
    int  dtype = EREAL;          /* Input data type                          */
    int  i;                      /* Looping variable                         */
    int  j;                      /* Counter -- current pixel                 */
    int  mmflag;                 /* Min/max flag                             */
    long nl;                     /* Number of image lines                    */
    long ns;                     /* Number of image samples                  */
    int  size;                   /* Number of bytes per pixel                */

    char *thebuf=NULL;           /* Image I/O buffer                         */

    /* Initialize fdesc to NULL  */
    for (i = 0; i < MAXIMG+1; i++)
        fdesc[i] = NULL;

    /* Setup for processing  */
    if (ci_process_setup (TRUE, dtype, odtype, hosin, hosout, comp, nmrimg,
        window, bands, nbands, totbnd, &odtype, msgtim, minmax, &nl, &ns, fdesc,
        &gdesc, offset1, offset2, &mmflag, &size, &thebuf, minval, maxval)
        != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-process_setup",
                 "Error encountered setting up for rndproc");
        return (E_FAIL);
    }

    /* Main processing loop  */
    rthebuf = (float *) thebuf;
    for (curlin = 0; curlin < nl; curlin++)
    {
        if (c_estep (&gdesc) != E_SUCC)
        {
            c_lasmsg (FALSE, "copy_image-step",
                     "Error stepping through the image");
            return (E_FAIL);
        }
        for (band = 0; band < totbnd; band++)
        {
            for (j = 0; j < ns; j++)
            {
                if (*(rthebuf + (*(offset1+band) + j)) >= 0.0)
                    *(rthebuf + (*(offset2+band) + j)) =
                    *(rthebuf + (*(offset1+band) + j)) + 0.5;
                else
                    *(rthebuf + (*(offset2+band) + j)) =
                    *(rthebuf + (*(offset1+band) + j)) + -0.5;
            }
            if (mmflag)
                if (las_pixminmax ((long *)(thebuf + (*(offset2+band) * size)),
                    ns, &minval[band], &maxval[band], dtype) != E_SUCC)
                {
                    c_lasmsg (FALSE, "copy_image-pxmnmx",
                             "Error encountered getting min/max");
                    return (E_FAIL);
                }
        }
        curent++;
    }

    /* Finish processing and cleanup  */
    if (ci_process_finish (vblock, hosin, hosout, nmrimg, window, bands,
        nbands, totbnd, minmax, gdesc, thebuf, minval, maxval) != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-process_finish",
                 "Error encountered cleaning up for rndproc");
        return (E_FAIL);
    }

    return (E_SUCC);
}



/**
 * c_copy_image: 
 * @vblock: I: TAE parameter block.
 * @hosin: I: Filenames of Unix input images.  Hosin may be a single or
 *            multi-band image and may contain BYTE, I*2, I*4, or R*8
 *            pixel data.
 * @hosout: I: Filename of Unix output image.
 * @comp: I: Compression flag:
 *           TRUE:     Compress output image,
 *           FALSE:    Don't compress output image,
 *           EXT_COMP: Use external compres_opt or COMPRES1     
 *                     environment variable.
 * @nmrimg: I: Number of input images.
 * @window: I: Window for each input image (SL,SS,NL,NS).  If all values are
 *             0's then the entire image will be processed.  If the images
 *             are not of the same size then processing will be stopped. 
 * @bands: I: Bands to process for all images.  If they are all 0's then all
 *            bands in all of the images will be processed. nbands also needs
 *            to be set to 0 for the default.
 * @nbands: I: Number of bands per input image.  If bands are all 0's and 
 *             nbands[i] is 0 then all bands in all of the images will be
 *             processed.
 * @totbnd: I: Total number of bands.  Set to 0 if processing the default
 *              bands.
 * @odtype: I: Data type of output image (0 = same, EBYTE = BYTE, EWORD = I*2,
 *            ELONG = I*4, EREAL = R*4)
 * @rtopt_in: I: Round/truncation option:
 *               TRUE:  Round the input data,
 *               FALSE: Truncate the input data.
 * @msgtim: I: Elapsed time between display messages for TAE.
 * @minmax: I: Minimum/maximum flag for TAE. 
 *
 * c_copy_image() copies an image into another image.  It allows the input
 * image(s) to be sub-windowed and sub-banded when copied to the output image.
 * The input image may be BYTE, INTEGER*2, INTEGER*4, or REAL*4.  The output
 * image may be converted to any one of the four data types (the default
 * is no data conversion).
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */  
int c_copy_image
(
    struct PARBLK *vblock,           /* I: TAE parameter block               */
    const char hosin[MAXIMG][CMLEN], /* I: Filenames of input images (Unix)
                                           hosin may be a single- or multi-band
                                           image and may contain BYTE, I*2,
                                           I*4, or R*4 pixel data.           */
    const char hosout[CMLEN],        /* I: Filename of output image (Unix)   */
    int comp,                        /* I: Compression flag
                                           TRUE = Compress output image
                                           FALSE = Don't compress output image
                                           EXT_COMP = Use external compres_opt
                                           or COMPRES1 environment variable  */
    int  nmrimg,                     /* I: Number of input images            */
    int  window[MAXIMG][4],          /* I: Window for each input image
                                          (SL, SS, NL, NS)
                                          If all values are 0s then the entire
                                          image will be processed.  If the
                                          images are not of the same size then
                                          processing will be stopped.        */
    int  bands[MAXIMG][MAXBND+1],    /* I: Bands to process - all images
                                           If all 0s then all bands in all the
                                           images will be processed.  nbands
                                           also needs to be set to 0 for the
                                           default.                          */
    int  nbands[MAXIMG],             /* I: Number of bands per input image
                                           If bands are all 0s and nbands[i]
                                           is 0 then all bands in all the
                                           images will be processed          */
    int  totbnd,                     /* I: Total number of bands.  Set to 0 if
                                           processing the default bands      */
    int  odtype,                     /* I: Data type of output image
                                           (0=SAME, EBYTE=BYTE, EWORD=I*2,
                                           ELONG=I*4, EREAL=R*4)             */
    int rtopt_in,                    /* I: Round/truncation option
                                           TRUE = Round the input data
                                           FALSE = Truncate the input data   */
    int  msgtim,                     /* I: Elapsed time between display msg 
                                           for TAE                           */
    char minmax[4]                   /* I: Min max flag for TAE              */
)

{
    int rtopt;                       /* Round/truncation option
                                        FALSE = TRUNC, TRUE = ROUND          */
    int mode = 0;                    /* Indicates LSSTAT is to check for
                                        the existence of the file */

    /* Make sure the output image doesn't already exist before continuing */
    if (c_labstat(hosout, mode) == E_SUCC)
    {
        c_lasmsg(FALSE, "copy_image", "Error, output file %s already exists",
                 hosout);
        return E_FAIL;
    }

    /* Sort out the input parameters and retrieve the TAE global variables   */
    if (ci_getpar (odtype, nmrimg, hosin, window, rtopt_in, bands, nbands,
        &totbnd, &rtopt) != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-params",
                  "Error getting the input parameters");
        las_clean_noexit (vblock, (const char (*)[256])hosout, 1);
        return (E_FAIL);
    }

    /* Process image -- round input data if needed  */
    if (rtopt)
    {
        if (ci_rndproc
           (vblock, hosin, hosout, comp, nmrimg, window, bands, nbands,
            totbnd, odtype, msgtim, minmax) != E_SUCC)
        {
            c_lasmsg (FALSE, "copy_image-rndproc", 
                     "Error processing the image");
            las_clean_noexit (vblock, (const char (*)[256])hosout, 1);
            return (E_FAIL);
        }
    }
    else
    {
        if (ci_process 
           (vblock, hosin, hosout, comp, nmrimg, window, bands, nbands,
            totbnd, odtype, msgtim, minmax) != E_SUCC)
        {
            c_lasmsg (FALSE, "copy_image-process", 
                      "Error processing the image");
            las_clean_noexit (vblock, (const char (*)[256])hosout, 1);
            return (E_FAIL);
        }
    }

    /* Update history file */
    if (las_hstout (vblock, hosout, hosin, nmrimg, " ") != E_SUCC)
    {
        c_lasmsg (FALSE, "copy_image-hstout",
                 "Nonfatal error encountered updating the history file");
    }

    return (E_SUCC);
}



