/*******************************************************************************
FUNCTION: c_getfiles

PURPOSE:  Given a file specification find all files that match this
          specification.  For example, /sg2/csb/user/mydir/  *.c for UNIX or
          [user.mydir]* for VMS.


Version  Date   Programmer       Modification
-------  -----  ---------------  -----------------------------------------------
  1.0    05/92  J.Fenno          New version of dirlist that will work 
                                 correctly on both UNIX and VMS systems.
  1.1    07/92  J.Fenno          Added code so that the pipe gets closed
                                 properly and does not leave defunct processes.
  1.2    07/92  T.Baltzer        Add locked file flag logic.
  1.3    12/93  J.Fenno          Added check for !first_call and file_ptr==NULL


PROJECT: LAS

ALGORITHM REFERENCES: LAS support dirlist
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "cm.h"


static FILE *file_ptr = NULL;/* Temp file ptr to the list of file names       */

/**
 * c_getfiles:
 * @filespec: I: Host file specification for files.
 * @file_names: O: Array of file names to be outputted.
 * @nfiles:  O: Number of files found matching file specifications.
 * @maxfiles: I: Maximum number of files to return all at one time.
 * @first_call:  I: Flag for first call to the c_getfiles() function.
 * @ck_lock: I: If ck_lock = 1: Return list of unlocked files only.
 *              If ck_lock = 0: Return list of all files found.
 *
 * Given a file specification, the c_getfiles() routine finds all files to
 * match this specification.  For example: 
 *          /sg2/csb/user/mydir/c_*.c 
 * finds all files beginning with a c_ for this specification.
 *
 * The c_getfiles() routine is a newer version of dirlist() and should be 
 * used instead.  It makes sure that the pipe gets closed properly when
 * needed and does not leave defunct processes.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_getfiles
(
    const char *filespec,       /* I: Host file specification for files       */
    char file_names[][CMLEN],   /* O: Array of file names to be output        */
    int  *nfiles,               /* O: Number of files found matching filespec */
    int  maxfiles,              /* I: Max number of files to return at once   */
    int  first_call,            /* I: Flag for first call to this function    */
    int  ck_lock                /* I: 1 = return list of unlocked files only, */
                                /*    0 = return list of all files found      */
)
{
    char filename[CMLEN];       /* Name of one file                           */
    char lockfile[CMLEN];       /* Name of one 'locked' file                  */
    char sys_command[CMLEN];    /* System command which produces file names   */

    int  status;                /* Return status for the wait command         */
    int  i        = 0;          /* Loop control                               */
    int  zero     = 0;          /* constant value                             */
    int  c;

    /* Ensure that maxfiles is at least one and initialize variables          */
    if (maxfiles < 1)
    {
        c_lasmsg (FALSE, "c_getfiles-num",
                  "Number of requested files must be greater than zero");
        return(E_FAIL);
    }
    *nfiles = 0;
    strcpy(filename, "");


    /* Create a file with the list of desired file names.  If the file
       specification uses an asterisk and no files are found, the file
       name will contain the given specification.  Therefore, if an
       asterisk is present then no files were found.                          */
    if (first_call)
    {       
        if (file_ptr != NULL)
        {
            pclose(file_ptr);
            file_ptr = NULL;
            wait(&status);
        }
        sprintf(sys_command, "echo %s", filespec);
        file_ptr = popen(sys_command, "r");
        if (file_ptr == NULL)
        {
            c_lasmsg (FALSE,"c_getfiles-open",
                     "Error opening temporary file");
            return(E_FAIL);
        }
    }
    else if (!first_call && (file_ptr == NULL))
    {
        /* If all files have already been returned then exit successfully     */
        *nfiles = 0;
        return(E_SUCC);
    }


    /* Get the desired number of file names from the temporary file           */
    do 
    {
        c = fgetc(file_ptr);

        if (c == '*')
        {
            *nfiles = 0;
            pclose(file_ptr);
            file_ptr = NULL;
            wait(&status);
            return(E_SUCC);
        }
        else if ((c != ' ') && (c != '\n'))
            filename[i++] = c;
        else if ((c == ' ') || (c == '\n') || ((c == EOF) && (i > 0)))
        {
            filename[i] = '\0';
            if (ck_lock)  /* Return the file only if it is not locked */
            {
                c_lsmknm(filename,"lock",lockfile);
                if (c_labstat(lockfile,zero) != E_SUCC)
                    strcpy(file_names[(*nfiles)++], filename);
            }
            else  /* Return the file whether it is locked or not */
                strcpy(file_names[(*nfiles)++], filename);
            strcpy(filename, "");
            i = 0;
        }
    } while ((c != EOF) && (*nfiles < maxfiles));


    /* If we only found one file then make sure this file really exists.  For
       example, if the user's file specification is /sg2/csb/user/myfile then
       the command "echo /sg2/csb/user/myfile" will do just that, echo
       "/sg2/csb/user/myfile", whether or not the file exists.  Thus we must
       check to see if the file does exist                                    */
    if (*nfiles == 1)
    {
        i = 0;
        if (c_labstat(file_names[0], i) != E_SUCC)
            *nfiles = 0;
    }


    /* Close the temporary file if done and return successfully               */
    if (*nfiles < maxfiles) 
    {
        pclose(file_ptr);
        file_ptr = NULL;
        wait(&status);
    }

    return (E_SUCC);
}
