/******************************************************************************
FUNCTION:       cmlock

PURPOSE:        Opens the specified file and attempts to lock it.
                If the file is already locked, cmlock will wait a little
                while, then try again, up to the specified time limit.
                Cmlock returns E_SUCC if successful, E_FAIL otherwise.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86    C. Greenhagen       New Development
  1.1      12/87    B. Ailts            include file changes
  5.0      04/88    D. Hollaren         changed include file to las
                                        return status of E_SUCC/E_FAIL
  5.1      03/89    B. Davis            Changed return from unsuccessful open
                                        to allow more descriptive error message
  5.2      01/90    B. Davis            Returned ENOENT when trying to create
                                        and lock a file that does not exist. In
                                        certain situations, when the last dir
                                        member of the path name does not exist,
                                        an attempt is made to create the dir,
                                        then call cmlock again. 
  5.3      08/94    D. Etrheim          Change 777 to 666

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Open the specified host file.
VMS automatically locks a file so nothing to be done
    error if the file was to be opened for output and it already exists.
In UNIX
   attempt to lock the file until it gets locked
   or the maximum wait time is reached.

ALGORITHM REFERENCES    none

Note: cmlock does not put out error messages before returning e_fail
*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <unistd.h>
#include "las.h"
#include "cm.h"


/**
 * cmlock:
 * @hname: Host name of file to be opened.
 * @fd: O: File descriptor.
 * @flags: I: File access. 
 * @maxtime: I/O: Maximum time to wait (seconds) while attempting
 *                to lock the file.
 *
 * cmlock() opens the specified file and attempts to lock it.  If
 * the file is already locked, cmlock() will wait a little while,
 * then try again, up to the specified time limit.  Closing the
 * file releases the lock on the file.
 *
 * Returns:  E_SUCC:  Successful completion of operation.
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int cmlock
(
    const char *hname,      /* I: Host name of file to open    */
    int *fd,                /* O: File descriptor              */
    int flags,              /* I: File access                  */
    int maxtime             /* I/O: Max time to wait (seconds) */
)
{
    mode_t oldmode;         /* return of umask                 */
    int gotit=0;            /* got the lock? (1=yes)           */

    /* On Silicon Graphics, read and write permission are required to 
       lock a file ( see flock() BUGS ). This fix will get around this bug.
       the open call sets mode to 0666 first, then any umask bits.  This is
       dangerous because a user umask could close off the file for write
       access to subsequent users.  We therefore set umask to be harmless,
       then change it back after opening the file.             */
#   if defined (sgi) || defined (sun)
    flags = O_CREAT | O_RDWR;
    oldmode = umask(000);
#   endif

    /* Flags consisting of O_RDONLY only apparently don't actually open 
     * a file in a manner suitable for locking under linux?  Some calls
     * to this function use that flag.                                 */
#   if defined (linux)
    flags = O_CREAT | O_RDWR;
#   endif

    *fd = open (hname,flags,0666);          /* open file - let mode default */
    if (*fd < 0)
    {
        if (errno == ENOENT)
            return(errno);
        if (*fd == -1)
            return(-2);
        else
            return (E_FAIL);
    }

#   if defined (sgi) || defined (sun)
    umask(oldmode);
#   endif

    /* use flock on unix systems    */                         
    while ((!gotit) && (maxtime>0))                       
    {                                                         
        if ( lockf(*fd,F_TLOCK,0) == 0)
            gotit = 1;                                         
        else                                                  
        {                                                    
            sleep(5);                                      
            maxtime -= 5;                                 
        }                                            
    }                                               

    if (maxtime <= 0)                                         
        return (E_FAIL);                                 

    return (E_SUCC);
}
