/******************************************************************************
FUNCTION:       copyfile

PURPOSE:        Makes a copy of the specified file with the specified name.
                E_SUCC is returned if the copy is successful, E_FAIL otherwise

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      11/86    C. Greenhagen       New Development
            6/87    K. Zanter           Changed so that if the
                                        file does exist on disk,
                                        it is not overwritten.
  1.1      12/87    B. Ailts            change calls to cmtae
                                        to reflect DESC to string
                                        changes
  5.0      04/88    D. Hollaren         changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed staus code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
  5.0      09/88    B. Davis            fixed bug, addr of char arrays were
                                        being sent to cmtae
  5.0      11/88    B. Davis            system calls to dk calls, also brought
                                        up to 5.0 standards
  5.1      01/91    D.Etrheim           Standardized error message handling
                                                        
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
If the TAE name is to reported to the user
    get the TAE file names from the host file names.
Make sure that the output host file doesn't already exist.
Copy the input host file to the output host file.
Check the report flag 
    report to the user the TAE file name or the host file name of the file
    copied.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"
#include "diskio.h"
#include "labserv.h"
#include "cm.h"

/**
 * copyfile:
 * @hname: I: Host name of existing file.
 * @hnewname: I: Host name of new file.
 * @report: I: Value determines the contents of the report file:
 *             0: tae, errors only,
 *             1: taename,
 *             2: host, error only,
 *             3: hostname.
 *
 * The copyfile() routine makes a copy of the specified file with
 * the specified name. Then reports to the user the hostfile name 
 * of the file copied.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int copyfile
(
    const char *hname,      /* I: Host name of existing file   */
    const char *hnewname,   /* I: Host name of new file        */
    int  report             /* I: Report file?                 */
                            /*    (0=tae,errs only, 1=taename, */
                            /*    2=host,errs only,3=hostname) */
)
{
    int   status;           /* Status returned              */
    int   zero = 0;         /* Zero (pass to lsstat)        */

    char  cmdstring[ERRLEN];/* Error message text string    */
    char  name[CMLEN];      /* TAE name of existing file    */
    char  newname[CMLEN];   /* TAE name of new file         */
    char  temp[CMLEN];      /* Temporary character array    */
   
/************************************************************************/

    strcpy (name,hname);
    strcpy (newname,hnewname);

    if (c_labstat(hnewname,zero) == E_SUCC)                 /* file exists, */
    {                                                       /* don't copy   */
        sprintf (temp,"Output file already exists: %s",newname);
        c_lasmsg (FALSE, "copyfile-exist", temp);
        return(E_FAIL);
    }

    status = c_dkcopy(hname,hnewname,NULL);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "copyfile-call",
                 "Error returned from c_dkcopy");
        sprintf (cmdstring,"Error copying file %s to %s",name,newname);
        c_lasmsg (FALSE, "copyfile-copy", cmdstring);
    }

    if ((status == E_SUCC) && ((report==1)||(report==3)))
    {
        sprintf (temp,"File created: %s",newname);
        c_lasmsg (FALSE," ", temp);
    }

    return (status);
}
