/******************************************************************************
FUNCTION:       deletefile

PURPOSE:        Deletes the specified file on disk,
                or disk directory.  If verify = 1, the user is prompted
                to confirm or deny the delete.
                E_SUCC is returned if the delete is successful,
                E_FAIL otherwise.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      11/86    C. Greenhagen       New Development
  1.1      12/87    B. Ailts            replace DESC with char. i
                                        strings
                                        change include file specs.
  5.0      04/88    D. Hollaren         changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed status code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
  5.0      09/88    B. Davis            updated dirlist call, use dkdel 
                                        instead of system calls
  5.1      05/90    B. Davis            Only check for the file in the user 
                                        tape catalog file if it was not found 
                                        on disk because catalog manager allows
                                        is to be only one place.  This saves 
                                        time in locking, reading, and unlocking
                                        the tapelist file.  Also, changed the 
                                        call for marking the tapelist file for 
                                        deleting an entry.
  5.2      11/90    D. Etrheim          Changes for offline deletions
  5.3      01/91    D. Etrheim          Standardized error message handling
  6.0      08/00    T. Beckmann         Removed tape catalog support since
                                        it was never fully implemented and
                                        is not used.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
When deleting a directory;
    translate the host directory name to a tae directory,
    make sure the host file is a directory
    optionally verify that the directory is to be deleted.
    Try to delete the directory report any errors encountered.

When deleting a file on disk;
    make sure the host name is not a directory,
    optionally verify the file is to be deleted
    delete the file.

ALGORITHM REFERENCES    none

Note:   This support module breaks the rule of calling errmsg before returning
        E_FAIL.  
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include "las.h"
#include "diskio.h"
#include "labserv.h"
#include "cm.h"

/**
 * deletefile:
 * @hname:  I: Host name of file or directory to be deleted. 
 * @type: I: Specifies whether item deleted will be a file or a
 *           directory.
 * @verify: I: If verify = 1, user is  prompted to confirm or deny the  
 *             deletion.
 * @report: I: If report = 1 and E_SUCC is returned, user is notified of the
 *             successful deletion.
 *
 * The deletefile() deletes the specified file on disk or the specified 
 * disk directory. When deleting a directory, it makes sure the host file is
 * a directory.  It optionally verifies that the directory is to be deleted,
 * then tries to delete the directory.  Any errors encountered are reported.
 * The same algorithm is followed for file deletion as well.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int deletefile
( 
    const char *hname,      /* I; Host name of file    */
    const char *type,       /* I: File or dir          */
    int  verify,            /* I: Verify? (yes=1)      */
    int  report             /* I: Report it? (yes=1)   */
)
{
    int  status;            /* File delete status      */
    int  zero=0;            /* For lsstat              */
    int  termcode;          /* Termination code        */

    char msgtxt[CMLEN];     /* Message text            */
    char name[CMLEN];       /* Tae name                */
    char tmpnam[CMLEN];     /* Tmp file name to hide   */

    struct stat buf;        /* Used for lstat inf      */

/************************************************************************/


    strcpy(tmpnam," ");

    status = E_SUCC;

    strcpy(msgtxt,hname);                        /* put host name in     */
                                                 /* msgtxt temporarily   */
    if (strcmp (type,"dir") == 0)
        strcat (msgtxt,"/");                      

    strcpy(name, msgtxt);

    /*  delete directory  */

    if (strcmp (type,"dir") == 0)
    {
        if (!isdir(name))
        {
            sprintf (msgtxt,"Error: %s is not a directory",name);
            c_lasmsg (FALSE, "deletefile-notdir", msgtxt);
            return (E_FAIL);
        }

        if (c_labstat(name,zero) == E_SUCC)     /* delete directory     */
        {
#ifndef NOTAE
            if (verify)
            {
                sprintf (msgtxt,"Directory %s",name);
                t_write (msgtxt,T_STDCC);
                t_write ("Delete? (Y/N)",T_PROMPT);
                t_read  (msgtxt,&termcode);
                if ((msgtxt[0] != 'Y') && (msgtxt[0] != 'y'))
                {
                    return (E_FAIL);
                }
            }
#endif
            if (name[strlen(name)-1] == '/')                  
                name[strlen(name)-1] = 0;                     

            if (c_dkdel (name) != E_SUCC)                     
            {                                                   
                c_lasmsg (FALSE, "deletefile-call", 
                          "Error returned from c_ckdel");
                sprintf (msgtxt,"Error deleting directory from disk: %s",name);
                c_lasmsg (FALSE, "deletefile-dir", msgtxt);  
            }                                                   
            else                                              
            {                                               
                sprintf (msgtxt,"Directory deleted: %s",name); 
                c_lasmsg (FALSE, " ", msgtxt);               
            }                                                

            return (status);
        }
        else
        {
            sprintf (msgtxt,"Specified directory not found: %s",name);
            c_lasmsg (FALSE, "deletefile-nodir", msgtxt);
            return (E_FAIL);
        }
    }                                           /* end delete directory */

    /*  delete file  */

    else if (strcmp (type,"file") == 0)          
    {
        if (isdir(name))
        {
            sprintf (msgtxt,"Error: %s is a directory",name);
            c_lasmsg (FALSE, "deletefile-isadir", msgtxt);
            return (E_FAIL);
        }

        if ((c_labstat (name,zero) == E_SUCC) || 
            (lstat(name,&buf) == 0))
        {                                       /* delete file from disk*/
#ifndef NOTAE
            if (verify && report)
            {
                sprintf (msgtxt,"File %s",name);
                t_write (msgtxt,T_STDCC);
                t_write ("Delete? (Y/N)",T_PROMPT);
                t_read  (msgtxt,&termcode);
                if ((msgtxt[0] != 'Y') && (msgtxt[0] != 'y'))
                {
                    return (E_FAIL);
                }
            }
#endif

            if (c_dkdel(name) != E_SUCC)
                status = E_FAIL;

            if (status != E_SUCC)
            {
                c_lasmsg (FALSE, "deletefile-call",
                         "Error returned from c_dkdel");
                sprintf (msgtxt,"Error deleting file from disk: %s",name);
                c_lasmsg (FALSE, "deletefile-disk", msgtxt);
            }

            if ((status == E_SUCC) && report)
            {
                sprintf (msgtxt,"File deleted from disk: %s",name);
                c_lasmsg (FALSE, " ", msgtxt);
            }
        }                                   /* end delete file from disk*/
        else
        {
            status = E_FAIL;
            sprintf (msgtxt,"Specified file not found: %s",name);
            c_lasmsg (FALSE, "deletefile-nofile", msgtxt);
        }
    }                                           /* end delete file      */

    return (status);
}
