/*******************************************************************************
FUNCTION:  getdirs

PURPOSE:   Get all of the subdirectories for a given file spec.

PROGRAM HISTORY:
Version  Date   Author          Change Request
-------  -----  -------------   --------------
         03/86  T.Butzer        Original Implementation
         12/87  B.Ailts         Changed the include file specifications
         04/88  K.Gacke
  5.0    04/88  D.Hollaren      Changed include spec to las.
                                Changed cerrmsg to c_errmsg and passes status
                                code by address.
                                Return status to E_SUCC/E_FAIL.
  5.1    01/90  D.Etrheim       Standardized error message handling
  5.1    10/91  J.Fenno         Added break condition in while loops for UNIX.
                                Corrected calls to VMS world support routine
                                genf.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "cm.h"

/**
 * getdirs:
 * @filespec:  I: Host description of path and file name.
 * @directories:  O: Array of directory names found.
 * @ndirs: O:  Number of directories found.
 * @maxdirs: I:  Maximum number of directories to look for at one time.
 * @startf:  I:  Number of directories already found.
 *
 * The getdirs() routine finds all of the subdirectories for a given file
 * specification.
 *
 * Returns: E_SUCC:  Successful completion of operation.
 *          E_FAIL:  Unsuccessful completion of operation.
 */
int getdirs
(
    const char *filespec,         /* I: Host description of path and filename*/
    char directories[][CMLEN],    /* O: Array of directory names found       */
    int *ndirs,                   /* O: Number of directories found          */
    int maxdirs,                  /* I: Maximum # of directories to look for */
    int startf                    /* I: Number of directories already found  */
)

{
    int   c;                /* temporary character                            */
    char  sysstr[CMLEN];    /* system command to produce a listing of files   */
    char  thisfile[CMLEN];  /* current file name which is being looked at     */
    char  localspec[CMLEN]; /* copy of the input string filespec              */
    char  *ptr;             /* character pointer                              */

    FILE  *fp;              /* file pointer to the opended pipe               */

    int   i;                /* string index                                   */
    int   count;            /* loop control                                   */


    strcpy(localspec,filespec);
    ptr = (char *)strrchr(localspec, '/');
    if (ptr != 0)
    {
        *ptr = '\0';
        strcat(localspec,"/*");
    }
    else
    {
        strcpy(localspec,"*");
    }

    /* open a pipe to the system command /bin/echo localspec, which
       will produce a listing of all files matching localspec.*/
    sprintf(sysstr, "/bin/echo %s",localspec);
    if ((fp = popen(sysstr, "r")) == NULL)
    {
        c_lasmsg (FALSE, "getdirs-open", "Error opening temporary file");
        return(E_FAIL);
    }

    /* If we have already processed "startf"
    directories, then skip past them. */
    i = 0;
    if (startf > 1)
    {
        count = 1;
        while ( ((c=getc(fp)) != EOF) && (count < startf))
        {
            if(c == ' ' || c == '\n')
            {
                thisfile[i] = '\0';
                if (isdir(thisfile))
                {
                    count++;
                }
                i = 0;
            }
            else
            {
                thisfile[i] = c;
                i++;
            }
            /* When using a pipe between two processes, which is created
               by popen, the pipe may not end with EOF.  For this application
               it will suffice to look for a newline character. */
            if (c == '\n')
                break;
        }
        thisfile[i] = c;  /* keep the last char picked up */
        i++;
    }

    *ndirs = 0;
    while ((c=getc(fp)) != EOF && (*ndirs < maxdirs))
    {
        if(c == ' ' || c == '\n')
        {
            thisfile[i] = '\0';
            if (isdir(thisfile))
            {
                strcpy(directories[*ndirs],thisfile);
                    (*ndirs)++;
            }
            i = 0;
        }
        else
        {
            thisfile[i] = c;
            i++;
        }
        /* When using a pipe between two processes, which is created by popen,
           the pipe may not end with EOF.  For this application it will suffice
           to look for a newline character. */
        if (c == '\n')
            break;
    }

    pclose(fp);
    return (E_SUCC);
}
