/******************************************************************************
FUNCTION:       processfile

PURPOSE:        performs set-up work for copying, renaming, or deleting files:
                it looks for image files if an "image set" was specified,
                calls bldname to construct the output filenames (wildcards
                may have been specified on output), checks for the existence
                of output files, and calls the appropriate function.


PROGRAM HISTORY:

Version    Date     Author         Change Request
-------    -----    -------------  --------------
  1.0      11/86    C. Greenhagen  New Development
  1.1      12/87    B. Ailts       change include dir. spec.
  5.0      04/88    D. Hollaren    change include spec to las SUCCESS/FAIL to
                                    E_SUCC/E_FAIL
  5.0      09/88    B. Davis       Changed dirlist call to accommodate update to
                                   dirlist.
  5.1      08/89    B. Davis       Made a call to hstout, in order to do so,
                                   vblock must be received.
  5.2      02/90    B.Davis/D.Hollaren
                                   Added imageflg, aftype array, account in
                                   order to copy all ass. files; or just img,
                                   ddr, and his; or just the ass. files in
                                   aftype; or some combination thereof.
  5.2       1/91    D. Etrheim     Added chkflg arg. to specify on-line/
                                   off-line conditional check for deletes.
                                   Other changes for offline deletions.
                                   Corrected to prompt for .his rename.
                                   Set verify based on runtype, if needed.
                                   Standardized error message handling.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM
Determine the function to be done:
    copy, rename, or delete a file or delete a directory.
Copy or rename a file;
    get the host file of the output image
    if associated files are to be copied
        get all of those names.
    From the input names
        build new names for the output files (TAE and host).
Call the routine to accomplish the desired task.

ALGORITHM REFERENCES    none

Note: processfile does not return E_FAIL to the calling routine.  Also
      do not return after deletefile, renamefile, or copyfile.  Try to
      process all the files in the list
*******************************************************************************/


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "cm.h"
#include "local_protos.h"

/**
 * las_processfile:
 * @vblock: I: Pointer to TAE parameter block.
 * @func: I: Function to perform.
 *           Legal function parameters include:
 *            cmcopy,
 *            cmrename,
 *            cmdeletedir,
 *            cmdeletefile.
 * @name:  I:  Current file name.
 * @newname: O: New file name.
 * @imgset: I:  If imgset = 1, it looks for image files associated with image.
 * @imageflg: I: Process image file.
 * @aftype: I: Associated file types.
 * @acount: I: Number of associated file types to search for.
 * @verify: O: If verify = 1, user is prompted to verify the operation 
 *             specified.  (copy, rename, or delete).
 * @nproc:  O: Number of files processed
 * @copyflg: I: Flag indicating if image data is copied or renamed.
 *
 * The las_processfile() routine performs set-up work for copying, renaming,
 * or deleting files. Then, it looks for image files if an "image set" was
 * specified, calls bldname to construct the output file names (wildcards 
 * may have been specified on output), checks for the existence of output
 * files, and calls the appropriate function.
 *
 */
void las_processfile
(
    struct PARBLK *vblock,        /* I: Ptr to TAE param block               */
    const char *func,             /* I: Function to perform                  */
    const char *name,             /* I: Current filename                     */
    char *newname,                /* O: New filename                         */
    int  imgset,                  /* I: Img & assoc? (1=yes)                 */
    int  imageflg,                /* I: Process image file                   */
    char aftype[][CMLEN],         /* I: Assoc file types                     */
    int  acount,                  /* I: # assoc file types                   */
    int  verify,                  /* O: Verify? (1=yes)                      */
    int  *nproc,                  /* O: # files processed                    */
    int  copyflg                  /* I: Flag if data is copied or renamed    */
)
{
#define MAXF    300               /* max # files to be                       */
                                  /* returned from dirlist                   */
struct VARIABLE *var;

    int  i;                      
    int  k;                       /* loop counters                           */
    int len1;                     /* length of hname                         */
    int more=1;                   /* more files indicator                    */
    int n_assoc = 0;              /* # assoc from dirlist                    */
    int n_img = 0;                /* # img from dirlist                      */
    int nfiles = 0;               /* # files from dirlist                    */
    int noreport=0;               /* report file? (n=0)                      */
    int ok=1;                     /* ok to rename (y=1)                      */
    int one=1;                    /* value of 1                              */
    int report=1;                 /* report file? (y=1)                      */
    int startf=1;                 /* dirlist start file                      */
    int status;                   /* function return stat                    */
    int zero = 0;
    int termcode;                 /* t_read term. code                       */
    int ifile; 
    int jfile; 
    int kfile;                    /* file loop counters                      */
    int copy_file=0;              /* cmcopy                                  */
    int delete_dir=0;             /* cmdel-dir                               */
    int delete_file=0;            /* cmdel-file                              */
    int rename_file=0;            /* cmrename                                */
    char crename[CMLEN];          /* name of file to create                  */
                                  /* (no wildcards)                          */
    static char fnames[MAXF][CMLEN];  /* array of host names                 */
    static char assocnm[MAXF][CMLEN]; /* array of assoc names                */
    char hname[CMLEN];                /* hostname of old file                */
    char hnewname[CMLEN];             /* hostname of new file                */
    char oldname[CMLEN];              /* tae name of old file                */
    char *p;                          /* name pointer                        */
    char temp[CMLEN];                 /* temporary char array                */
    char tmpnam[CMLEN];               /* tmp file to be hidden               */
    char subcmd[CMLEN];               /* contains subcommand                 */

/************************************************************************/

    strcpy(tmpnam," ");
    if (strcmp(func,"cmcopy") == 0)
        copy_file = 1;
    else if (strcmp(func,"cmrename") == 0)
        rename_file = 1;
    else if (strcmp(func,"cmdeletedir") == 0)
        delete_dir = 1;
    else if (strcmp(func,"cmdeletefile") == 0)
        delete_file = 1;

    var = p_find (vblock,"$RUNTYPE");
    if (strcmp(SVAL(*var,0),"INTERACTIVE") != 0)
        verify = 0;

    status = c_cmhost (name,hname);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "processfile-call", "Error returned from c_cmhost");
        return;
    }

    if (delete_dir)
    {
        len1 = (int)strlen(hname);
        if (hname[len1-1] == '/')                                   
            hname[len1-1] = 0;                                      
    }

    if (copy_file || rename_file)
    {
        status = c_cmhost (newname,hnewname);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "processfile-call",
                      "Error returned from c_cmhost");
            return;
        }
    }

    if (imgset)                                 /* image & assoc. files */
    {
        strcpy (temp,hname);                    /* look for img of image file */
        strcat (temp,"img");
        status = dirlist (tmpnam,temp,assocnm,&n_img,MAXF,startf);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "processfile-call",
                     "Error returned from dirlist");
            return;
        }

        if (n_img == 0)                         /* img does not exist,        */
        {
            c_lasmsg(FALSE, "processfile-exist","Image file %s does not exist.",
                     temp);
            more = 0;
        }
    }
                                                /* else get assoc files       */
                                                /* or all files if not imgset */
    while (more)
    {
        if (imgset)
        {
            nfiles = 0;
            for (i = 0; i < n_img; i++)
            {
                strcpy(temp, assocnm[i]);
                p = (char *)strrchr(temp,'.');
                p++;
                *p = '*';
                p++;
                *p = '\0';
                status = c_cmhost(temp,hname);
                if (status != E_SUCC)
                {
                    c_lasmsg (FALSE, "processfile-call",
                             "Error returned from c_cmhost");
                    return;
                }
                status = dirlist(tmpnam,hname,fnames[nfiles],&n_assoc,
                                 MAXF,startf);
                if (status != E_SUCC)
                {
                    c_lasmsg (FALSE, "processfile-call",
                             "Error returned from dirlist");
                    return;
                }
             
                /* Remove any redundant file names. */
                for (ifile=0; ifile<nfiles; ifile++)
                {
                    for (jfile=nfiles; jfile<nfiles+n_assoc; jfile++)
                    {
                        if (strcmp(fnames[ifile], fnames[jfile]) == 0)
                        {
                            n_assoc--;
                            for (kfile=jfile; kfile<nfiles+n_assoc; kfile++)
                                strcpy(fnames[kfile], fnames[kfile+1]);
                            break; /* because dirlist will not return identical
                                      filenames */
                        }    
                    }
                }

                nfiles = nfiles + n_assoc;
            }
        }
        else
        {
            status = dirlist (tmpnam,hname,fnames,&nfiles,MAXF,startf);
            if (status != E_SUCC)
            {
                c_lasmsg (FALSE, "processfile-call",
                         "Error returned from dirlist");
                return;
            }
        }

        /* Process each file in the list of file names */
        if (nfiles == 0)
        {
            if (delete_dir)
                c_lasmsg(FALSE, "processfile-dir", 
                        "Directory %s does not exist",hname);
            else
            {
                var = p_find (vblock,"_SUBCMD");
                strcpy(subcmd,SVAL(*var,0));
                for (i = 0, p = subcmd; i < strlen(subcmd); i++,p++) /* upcase
                                                                    subcommand*/
                    if (islower(*p))
                        *p = toupper(*p);
                    if (strcmp(subcmd,"DIR") != 0)
                        c_lasmsg(FALSE, "processfile-file", 
                                "File %s does not exist", hname);
            }
        }

        for (i = 0;i < nfiles;i++)
        {
            startf += 1;    
            if (copy_file || rename_file)
                bldname (fnames[i],hnewname,crename);
            else
                strcpy (crename,fnames[i]);

            if (copy_file || rename_file)
            {
                strcpy(newname, crename);
                strcpy(oldname, fnames[i]);
            }
            /* Retrieve the extension from the host file name */
            p = (char *)strrchr(fnames[i],HOST_EXT);
            p++;

            /* Do the specified operation */
            if (delete_dir)
            {
                /* Delete the directory */
                status = deletefile(fnames[i],"dir",verify,report);
            }
            else if (!imgset)
            {
                /* The input file was not an image file so just
                   process the file */
                if (copy_file)
                    status = copyfile(fnames[i],crename,report);
                else if (rename_file)
                    status = renamefile (fnames[i],crename,verify,
                                         report,copyflg);
                else if (delete_file)
                    status = deletefile (fnames[i],"file",verify,report);
            }
            else if ((imageflg) && ((strcmp(p,"img")==0)
                                || (strcmp(p,"ddr")==0)))
            {
                /* The input image is to be processed so check to see if the
                   file is the image or ddr and process it.  */
                if (copy_file)
                    status = copyfile(fnames[i],crename,report);
                else if (rename_file)
                    status = renamefile (fnames[i],crename,verify,
                                         report,copyflg);
                else if (delete_file)
                    status = deletefile (fnames[i],"file",verify,report);
            }
            else if ((imageflg) && (strcmp(p,"his") == 0))
            {
                /* The input image is to be processed so check to see if the
                   file is the history file and process it */
                if (copy_file)
                {
                    /* Copy the history file by calling lashstout to update the
                       output history from the input history.  This will also
                       insure that the history file will reflect the processing
                       from this routine */
                    if (c_labstat(crename,zero) == E_SUCC)   /* file exists, */
                    {                                        /* don't copy   */
                        sprintf (temp,"Output file already exists: %s",newname);
                        c_lasmsg (FALSE, "processfile-exist", temp);
                        status = E_FAIL;
                    }
                    else
                    {
                        status = las_hstout(vblock,crename,fnames[i],one," ");
                        if (status != E_SUCC)
                            c_lasmsg (FALSE, "processfile-call",
                                     "Error returned from las_hstout");
                        else
                        {
                            /* Inform the user of the file created */
                            sprintf (temp,"File created: %s",newname);
                            c_lasmsg (FALSE," ",temp);
                        }
                    }
                }
                else if (rename_file)
                {
                    /* Copy the history file by calling lashstout to update the
                       output history from the input history.  This will also
                       insure that the history file will reflect the processing
                       from this routine.  The old history file is then no
                       longer needed and is removed.  */
                    if (c_labstat(crename,zero) == E_SUCC)   /* file exists, */
                    {                                        /* don't copy   */
                        sprintf (temp,"Output file already exists: %s",newname);
                        c_lasmsg (FALSE,"processfile-exist",temp);
                        status = E_FAIL;
                        ok = 0;
                    }
                    else
                    {
                        if (verify)                          /* ask user */
                        {
                            sprintf (temp,"Rename %s",oldname);
                            t_write (temp,T_STDCC);
                            sprintf (temp,"To     %s",newname);
                            t_write (temp,T_STDCC);
                            t_write ("(Y/N)",T_PROMPT);
                            t_read (temp,&termcode);
                            if ((temp[0] != 'Y') && (temp[0] != 'y'))
                                ok = 0;
                        }
                    }

                    if (ok)
                    {
                        status = las_hstout(vblock,crename,fnames[i],one," ");
                        if (status != E_SUCC)
                            c_lasmsg (FALSE, "processfile-call",
                                     "Error returned from las_hstout");
                        else
                        {
                            /* Delete the old history file */
                            status = deletefile (fnames[i],"file",verify,
                                                 noreport);
                            if (status == E_SUCC)
                            {
                                /* Inform the user of the file renamed */
                                c_lasmsg (FALSE," "," ");
                                sprintf (temp,"File      : %s",oldname);
                                c_lasmsg (FALSE," ",temp);
                                sprintf (temp,"Renamed to: %s",newname);
                                c_lasmsg (FALSE," ",temp);
                            }
                        }
                    }
                }
                else if (delete_file)       /* Delete the history file */
                    status = deletefile (fnames[i],"file",verify,report);
            }
            else if ((strcmp(aftype[0],"*") == 0) && ((strcmp(p,"img") != 0) &&
                     (strcmp(p,"ddr") != 0) && (strcmp(p,"his") != 0)))
            {
                /* Any file associated with the image is to be processed.  The
                   image, ddr, and history files are processed seperately.  */
                if (copy_file)
                    status = copyfile(fnames[i],crename,report);
                else if (rename_file)
                    status = renamefile (fnames[i],crename,verify,report,
                                         copyflg);
                else if (delete_file)
                    status = deletefile (fnames[i],"file",verify,report);
            }
            else if (acount > 0)
            {
                /* Search the list of specified extention types for a match
                   to the file name.  If there is a match process the file */
                for (k = 0; k < acount; k++)
                {
                    if (strcmp(p,aftype[k]) == 0)
                    {
                        if (copy_file)
                            status = copyfile(fnames[i],crename,report);
                        else if (rename_file)
                            status = renamefile(fnames[i],crename,verify,report,
                                                copyflg);
                        else if (delete_file)
                            status = deletefile(fnames[i],"file",verify,report);
                    }
                }
            }
            else
                status = E_FAIL;

            /* Count the file as being processed */
            if (status == E_SUCC)
                *nproc += 1;
        }               /* end for              */

        /* If the max number of files is reached done processing for now */
        if (nfiles < MAXF)
            more = 0;
    }                   /* end while            */
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * processfile:
 * @vblock: I: Pointer to TAE parameter block.
 * @func: I: Function to perform.
 *           Legal function parameters include:
 *            cmcopy,
 *            cmrename,
 *            cmdeletedir,
 *            cmdeletefile.
 * @name:  I:  Current file name.
 * @newname: O: New file name.
 * @imgset: I:  If imgset = 1, it looks for image files associated with image.
 * @imageflg: I: Process image file.
 * @aftype: I: Associated file types.
 * @acount: I: Number of associated file types to search for.
 * @verify: O: If verify = 1, user is prompted to verify the operation 
 *             specified.  (copy, rename, or delete).
 * @nproc:  O: Number of files processed
 * @copyflg: I: Flag indicating if image data is copied or renamed.
 *
 * The processfile() routine performs set-up work for copying, renaming,
 * or deleting files. Then, it looks for image files if an "image set" was
 * specified, calls bldname to construct the output file names (wildcards 
 * may have been specified on output), checks for the existence of output
 * files, and calls the appropriate function.
 *
 * Deprecated. Use las_processfile() instead.
 */
void processfile
(
    struct PARBLK *vblock,           /* I: Ptr to TAE param block            */
    const char *func,                /* I: Function to perform               */
    const char *name,                /* I: Current filename                  */
    char *newname,                   /* O: New filename                      */
    int  imgset,                     /* I: Img & assoc? (1=yes)              */
    int  imageflg,                   /* I: Process image file                */
    char aftype[][CMLEN],            /* I: Assoc file types                  */
    const long *acount,              /* I: # assoc file types                */
    int  verify,                     /* O: Verify? (1=yes)                   */
    int  *nproc,                     /* O: # files processed                 */
    int  copyflg                     /* I: Flag: if data is copied or renamed*/
)
{
    las_processfile (vblock,func,name,newname,imgset,imageflg,aftype,
                     *acount,verify,nproc,copyflg);
}


#endif /* obsolete interfaces */


