/*****************************************************************************
FUNCTION:       renamefile

PURPOSE:        renames cataloged files (on disk only).

PROGRAM HISTORY:
Version    Date     Author           Change Request
-------    -----    -------------    --------------
  1.0      11/86    C. Greenhagen    New Development
  1.1      12/87    B. Ailts         change include dir. spec.
  5.0      04/88    D. Hollaren      changed include spec to las
                                     Changed cerrmsg to c_errmsg and
                                     passes status code by address
                                     SUCCESS/FAIL to E_SUCC/E_FAIL
  5.0      09/88    B. Davis         Added current host name to info message,
                                     system calls to dkmove, coded to 5.0 specs
  5.0      01/89    B. Davis         Fixed bug.  Check for where was not
                                     allowing confirmation.
  5.1      01/91    D. Etrheim       Corrected to not rename if file exists.
                                     Allow option to rename image file with
                                     symbolic link.
                                     Standardized error message handling.
  5.2      10/92    D. Etrheim       Corrected so that renaming across disks
                                     works.  Also set directories to NULL to
                                     fix error when it contained garbage. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Get the TAE names for the input and output host files.
Make sure the output files don't exist in the catalog or on the TAPEFILE.
Optionally verify that the file is to be renamed.
Rename the file.

ALGORITHM REFERENCES    none
*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/file.h>
#include "las.h"
#include "diskio.h"
#include "labserv.h"
#include "cm.h"

/**
 * renamefile:
 * @hname: I/O: Current host name.
 * @hnewname: I:  New host name.
 * @verify: I: If verify = 1, user will be prompted to verify the operation
 *             specified.
 * @report: I: If report = 1, user is notified of successful completion of
 *             operation.
 * @copyflg: I:  Flag indicating if image data is copied or renamed.
 *
 * The renamefile() routine renames cataloged files (on disk only).
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int renamefile
(
    char *hname,              /* I/O: Current host name                      */
    const char *hnewname,     /* I: New host name                            */
    int  verify,              /* I: Verify? (1=yes)                          */
    int  report,              /* I: Report file? (1=yes)                     */
    int  copyflg              /* I: Flag if image data is copied or renamed  */
)
{
    char temp[CMLEN];         /* temporary char array                        */

    char directories[CMLEN];  /* for dkmove                                  */
    int ok = 1;               /* ok to rename?                               */
    int status;               /* status variable                             */
    int zero = 0;             /* zero (pass to lsstat)                       */
    int termcode;             /* t_read term. code                           */

/************************************************************************/

    if (c_labstat(hnewname,zero) == E_SUCC)
    {
        sprintf (temp,"Output file already exists: %s",hnewname);
        c_lasmsg (FALSE, "renamefile-exist", temp);
        ok = 0;
    }
    else
    {
        if (verify)                                         /* ask user      */
        {
            sprintf (temp,"Rename %s",hname);
            t_write (temp,T_STDCC);
            sprintf (temp,"To     %s",hnewname);
            t_write (temp,T_STDCC);
            t_write ("(Y/N)",T_PROMPT);
            t_read (temp,&termcode);
            if ((temp[0] != 'Y') && (temp[0] != 'y'))
                ok = 0;
        }
        else if (!verify && report)                         /* don't ask     */
            c_lasmsg (FALSE," "," ");  /* print a space between reports      */
    }

    /* Set to NULL */ 
    directories[0]='\0';

    if (ok)                                                 /* ok to proceed */
    {
#ifdef UNIX_DISK
        if (!copyflg)
        {
            /* First attempt to rename the file.  The reason for doing this
               first is that files with symbolic links can be renamed faster
               then with dkmove (dkmove will actually do a copy if the link
               is across disks).  When this fix was implemented (1/91), it
               removed the capability to rename files across disks.  Therefore,
               if rename fails, we are going to assume that is was because we
               are attempting to rename across disks and will let dkmove 
               handle it. If the error occured for some other reason, dkmove
               should also handle it. Copyflg could probably be removed now. */ 

            status = rename (hname,hnewname);        /* rename symbolic link */
            if (status != E_SUCC)
            {
                status = c_dkmove (hname,hnewname,directories); /* file copied*/
                if (status != E_SUCC)
                    c_lasmsg (FALSE, "renamefile-call",
                             "Error returned from c_dkmove");
            }
        }
        else 
        {
            status = c_dkmove (hname,hnewname,directories);  /* file copied  */
            if (status != E_SUCC)
                c_lasmsg (FALSE, "renamefile-call",
                         "Error returned from c_dkmove");
        }
#else
        status = c_dkmove (hname,hnewname,directories);  /* file copied */
        if (status != E_SUCC)
            c_lasmsg (FALSE, "renamefile-call",
                     "Error returned from c_dkmove");
#endif
        if (status == E_SUCC)
        {
            if (report)
            {
                sprintf (temp,"File      : %s",hname);
                c_lasmsg (FALSE," ",temp);
                sprintf (temp,"Renamed to: %s",hnewname);
                c_lasmsg (FALSE," ",temp);
            }
            return (E_SUCC);
        }
        else
        {
            sprintf (temp,"Error renaming file %s to %s",hname,hnewname);
            c_lasmsg (FALSE,"renamefile-file", temp);
        }
    }

    return (E_FAIL);
}
