/*******************************************************************************
Purpose:
    This routines reads all lines fitting the user defined query.
    If clipping is to be performed, the bounding rectangle of each
    line is extracted on checked against the area of interest 
    boundaries.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
  5.2		07/93	D. Etrheim        CSB       Fix error messages
  5.3 		12/93	T. Rockvam	  CSB	    Make sure all files are 
						    closed upon exit; move close
						    of edge index file out of
						    loop
PROJECT:  LAS
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "dcw.h"

int dcw_readlines
(
    char *cdpath,	/* location of CD drive			      */
    char *lib,		/* DCW library (ex. noamer)		      */
    struct DCW_COVERAGE *coverage,	/* DCW coverage			      	      */
    struct DCW_LINE **dcw_line,	/* line data structure			      */
    int *dcw_numlines,	/* number of lines loaded into memory	      */
    int *dcw_maxline,	/* amount of space allocated for lines	      */
    char *tile,		/* pathname of tile			      */
    int *line,		/* list of line ID fitting query	      */
    int *type,		/* corresponding type of lines		      */
    int numline,	/* number of lines in query		      */
    double lllat,		/* lower left latitude			      */
    double lllon,		/* lower left longitude			      */
    double urlat,		/* upper right latitude			      */
    double urlon,		/* upper right longitude		      */
    int clip		/* flag denoting clipping to be performed     */
)
   {
   FILE		*fdedge, 	/* File descriptor of edge file		      */
		*fdedx, 	/* File descriptor of edge index	      */
		*fdebr;		/* File descriptor of bounding rectangle file */
   char edgepath[256];  /* full path of edge file */
   char edxpath[256];   /* full path of index file */
   char ebrpath[256];   /* full path of bounding rectangle */
   char edgepath_withdot[256]; /* full path of edge file with dot at end */
   char edxpath_withdot[256];  /* full path of index file with dot at end */
   char ebrpath_withdot[256];  /* full path of bounding rectangle with dot */
   unsigned char typebyte;	/* cross-tile triplet type byte		      */
   float	coord,		/* coordinate value			      */
		xmin,		/* minimun x value for a given line 	      */
		xmax,		/* maximum x value for a given line	      */
		ymin,		/* minumum y value for a given line	      */
		ymax;		/* maximum y value for a given line	      */
   int		clipped, 	/* denotes line was clipped		      */
		*cliptype,	/* types of clipped lines		      */
		clipx,		/* denotes clipping performed on x bounds     */
		clipy,		/* denotes clipping performed on y bounds     */
		endnode,	/* ending node in line			      */
		fpos, 		/* current file position		      */
		headrsize, 	/* size of header in bounding rectangle file  */
		id, 		/* id of line				      */
		lft_id, 	/* id of left area			      */
		nclip, 		/* number of lines clipped		      */
		npts, 		/* number of points in a given line    	      */
		ntriples, 	/* number of triples in a record     	      */
		startnode,	/* beginning node in the current line	      */
		tilellx, 	/* tile lower left x value		      */
		tilelly, 	/* tile lower left y value		      */
		temppos, 	/* temporary storage of file position	      */
		tripsize;	/* size of cross-tile triplet in bytes	      */
   int		i, j, k,	/* loop counters			      */
		nbytes,		/* size of record in edge file	              */
		*offset;	/* offset from beginning of edge file         */
   long	nonfatal = 1;	/* nonfatal error			      */
		

  /*
   *  *** generate full pathnames
   */
   strcpy(edgepath, cdpath);
   strcat(edgepath, "/dcw/");
   strcat(edgepath, lib);
   strcat(edgepath, "/");
   strcat(edgepath, coverage->name);
   strcat(edgepath, "/");
   strcat(edgepath, tile);
   strcat(edgepath, "/");

   strcpy(edxpath, edgepath);
   strcpy(ebrpath, edgepath);
   strcat(edxpath,  "edx");
   strcat(ebrpath,  "ebr");
   strcat(edgepath, "edg");
   /* build the edx, ebr, and edge path names with an extra dot at the end
      since some dcw libraries have an extra dot at the end */
   strcpy(edxpath_withdot,edxpath);
   strcpy(ebrpath_withdot,ebrpath);
   strcpy(edgepath_withdot,edgepath);
   strcat(edxpath_withdot,".");
   strcat(ebrpath_withdot,".");
   strcat(edgepath_withdot,".");

  /*
   *  *** open files
   */
   if (((fdedx = fopen(edxpath, "r")) == NULL) &&
       ((fdedx = fopen(edxpath_withdot, "r")) == NULL))
      return(E_FAIL);
 
   if (((fdedge = fopen(edgepath, "r")) == NULL) &&
       ((fdedge = fopen(edgepath_withdot, "r")) == NULL))
      {
      fclose(fdedx);
      return(E_FAIL);
      }

  /*
   *  *** if clipping, open bounding rectangle file
   */
   if (clip)
      {
      if ((((fdebr = fopen(ebrpath, "r")) == NULL) &&
          (((fdebr = fopen(ebrpath_withdot, "r")) == NULL))) ||
          (!dcw_read((unsigned char *)&headrsize, sizeof(long), 1, fdebr)))
         clip = FALSE;
      }
   
  /*
   *  *** get relative file offsets for each line
   */
   offset = (int*)malloc(numline*sizeof(int));

   nclip = 0;

  /*
   *  *** if clipping, make sure tile intersect area of interest (AOI)
   */
   if (clip)
      {
      tilellx = dcw_getlon(tile);
      tilelly = dcw_getlat(tile);

     /*
      *  *** if tile intersect AOI in x plane, set clipx
      */
      if ((lllon > tilellx) && (lllon < (tilellx+5)))
         clipx = TRUE;
      else if ((urlon > tilellx) && (urlon < (tilellx+5)))
         clipx = TRUE;
      else 
         clipx = FALSE;

     /*
      *  *** if tile intersect AOI in y plane, set clipy
      */
      if ((urlat > tilelly) && (urlat < (tilelly+5)))
         clipy = TRUE;
      else if ((urlat > tilelly) && (urlat < (tilelly+5)))
         clipy = TRUE;
      else 
         clipy = FALSE;
      }
   else
      {
      clipx = FALSE;
      clipy = FALSE;
      }

   headrsize += sizeof(int);
   cliptype = (int *)malloc(numline*sizeof(int));

  /*
   *  *** for each line, check if clipped and get file position of data record
   */
   for (i = 0; i < numline; i++)
      {
      clipped = FALSE;

     /*
      *  *** if clipping in x or y plane, get bounds and compare with AOI
      */
      if (clipx || clipy)
         {
         fseek(fdebr, headrsize+(line[i]-1)*20, 0);
         dcw_read((unsigned char *)&id, sizeof(long), 1, fdebr);
         dcw_read((unsigned char *)&xmin, sizeof(long), 1, fdebr);
         dcw_read((unsigned char *)&ymin, sizeof(long), 1, fdebr);
         dcw_read((unsigned char *)&xmax, sizeof(long), 1, fdebr);
         dcw_read((unsigned char *)&ymax, sizeof(long), 1, fdebr);

         if (clipx &&
            ((xmin > urlon) ||
             (xmax < lllon)))
            clipped = TRUE;
         else if (clipy &&
            ((ymin > urlat) ||
             (ymax < lllat)))
            clipped = TRUE;
         }

     /*
      *  *** if line was not clipped, get relative (to previous line)
      *          file position of each line record
      */
      cliptype[i-nclip] = type[i];

      if (!clipped)
         {
         fseek(fdedx, 8+(line[i]-1)*8, 0);
         dcw_read((unsigned char *)&offset[i-nclip], sizeof(long), 1, fdedx);
         temppos = offset[i-nclip];
   
         if ((i-nclip) > 0)
            offset[i - nclip] = offset[i - nclip] - fpos - nbytes;
   
         dcw_read((unsigned char *)&nbytes, sizeof(long), 1, fdedx);
         fpos = temppos; 
         }
      else
         nclip++;
      }

   fclose(fdedx);

   if (clip)
      fclose(fdebr);

  /*
   *  *** determine number of cross-tile triples based on topology level
   */
   if (coverage->level == 3)
      ntriples = 4;
   else
      ntriples = 2;

  /*
   *  *** read each unclipped line
   */
   for (i = 0; i < (numline - nclip); i++)
      {
      if (offset[i])
         fseek(fdedge, offset[i], 1);

      dcw_read((unsigned char *)&id, sizeof(long), 1, fdedge);

      dcw_read((unsigned char *)&lft_id, sizeof(long), 1, fdedge);
      dcw_read((unsigned char *)&startnode, sizeof(long), 1, fdedge);
      dcw_read((unsigned char *)&endnode, sizeof(long), 1, fdedge);

     /*
      *  *** skip over encrypted cross-tile triples
      */
      for (j = 0; j < ntriples; j++)
         {
         fread(&typebyte, 1, 1, fdedge);
         tripsize = (typebyte >> 6) + 
                    ((typebyte >> 4) & 0x03) +
                    ((typebyte >> 2) & 0x03);
         fseek(fdedge, tripsize, 1);
         }


      dcw_read((unsigned char *)&npts, sizeof(long), 1, fdedge);

     /*
      *  *** if maximum allocated, allocate new block
      */
      if (*dcw_numlines == *dcw_maxline)
         {
         *dcw_maxline += LINEBLOCK;
         *dcw_line = (struct DCW_LINE*)realloc(*dcw_line,
                               *dcw_maxline * sizeof(struct DCW_LINE));
	 if (*dcw_line == NULL) 
	     {
   	     free(offset);
   	     free(cliptype);
             fclose(fdedge);
	     c_errmsg("Error reallocating memory","dcw_rdlines-alloc",
                       &nonfatal);
	     return(E_FAIL);
	     }
         }

      
     /*
      *  *** set line attribute to vdt index
      */
      for (k = 0; k < coverage->numvdt; k++)
         {
         if (coverage->vdt[k].attribute == cliptype[i])
            {
            (*dcw_line)[*dcw_numlines].attrib = coverage->vdt[k].attindex;
            break;
            }
         }

     /*
      *  *** allocate x and y for each point
      */
      (*dcw_line)[*dcw_numlines].pts =
                          (double*)malloc(npts*2*sizeof(double));
      (*dcw_line)[*dcw_numlines].npts = npts;

     /*
      *  *** read each point in the line
      */
      for (j = 0; j < npts; j++)
         {
         dcw_read((unsigned char *)&coord, sizeof(float), 1, fdedge);
         (*dcw_line)[*dcw_numlines].pts[2*j] = (double)coord;

         dcw_read((unsigned char *)&coord, sizeof(float), 1, fdedge);
         (*dcw_line)[*dcw_numlines].pts[2*j+1] = (double)coord;
         }

      (*dcw_numlines)++;

     /*
      *  *** if maximum allocated, allocate new block
      */
      if (*dcw_numlines == *dcw_maxline)
         {
         *dcw_maxline += LINEBLOCK;
         (*dcw_line) = (struct DCW_LINE*) realloc(*dcw_line,
                                    *dcw_maxline * sizeof(struct DCW_LINE));
	 if (*dcw_line == NULL)
	     {
   	     free(offset);
   	     free(cliptype);
             fclose(fdedge);
	     c_errmsg("Error reallocating memory","dcw_rdlines-alloc",
                       &nonfatal);
	     return(E_FAIL);
	     }
         }
      }

   free(offset);
   free(cliptype);
   fclose(fdedge);
   return(E_SUCC);
   }


