/******************************************************************************

NAME:   CALCOR

FUNCTION:
        CALCOR calculates the projection coordinates of the corners of an 
        image.  It is called when an image is windowed and new corner 
        coordinates have to be calculated.  The image is assumed to be
        rectangular.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         10/87       B.Ailts      initial development
    1.1         12/87       B.Ailts     Place bridge routine in a seperate file
                                        Change the include file directories
    1.2         04/88       B.Ailts     Replace newlas.h with las.h
                                        Changed ddrdub to ddr
                                        Changed gdist to pdist
                                        Changed sl, ss, nl, and ns to double
                                        precision
    1.3         01/90       B.Ailts     Changed in order to handle arc-sec as
                                        a valid projection coordinate

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
                Run under TAE

PROJECT:        new LAS

ALGORITHM (FLOW):
        Each of the corner coordinates are calculated starting with the
        upper left corner.  The three other corners are calculated from
        this corner.  The algorithm for upper left coordinate is shown below.

        upper_left_northing = (ground_distance_in_line_direction * 
                              (starting_line - 1)) + 
                              current_upper_left_northing_coordinate

        upper_left_easting = (ground_distance_in_sample_direction *
                             (starting_sample - 1)) +
                             current_upper_left_easting_coordinate

ALGORITHM REFERENCES:

******************************************************************************/
#include "las.h"


/**
 * las_calcor:
 * @sl: I: Starting line of the output image.
 * @ss: I: Starting sample of the output image.
 * @nl: I: Number of lines of the output image.
 * @ns: I: Number of samples of the output image.
 * @ddr: I: A DDR structure.
 * @upleft: O: The upper-left coordinate pair, Y-X,  as calculated 
 *             from sl and the "upleft" corners of the DDR.
 * @upright: O: The upper-right cordinate pair, Y-X, as calculated from ss and
 *              the "upright" corners of the DDR.
 * @loleft: O: The lower-left coordinate pair, Y-X, as calculated from nl and
 *             the "loleft" corners of the DDR.
 * @loright: O: The lower-right coordinate pair, Y-X, as calculated from ns an
 *              the "loright" corners of the DDR.
 *
 * The las_calcor() call is used to calculate the four corner coordinates of the
 * window specified.  Each of the new corner coordinates will be calculated 
 * using the current coordinates in the DDR and the window specified in sl,
 * ss, nl, and ns.  Rotated projection coordinates cannot be input into 
 * las_calcor().
 */
void las_calcor
(
    double sl,                /*  I: starting line                          */
    double ss,                /*  I: starting sample                        */
    double nl,                /*  I: number of lines                        */
    double ns,                /*  I: number of samples                      */
    const struct DDR *ddr,    /*  I: input DDR structure                    */
    double *upleft,           /*  O: Upper left corner                      */
    double *upright,          /*  O: Upper right corner                     */
    double *loleft,           /*  O: Lower left corner                      */
    double *loright           /*  O: Lower right corner                     */
)

{

    /*  This is statement is placed in here to compensate for using 
        arc-seconds as a valid projection code.  Arc-seconds are reversed 
        in order compared to projection coordinates                         */
    if (ddr->loright[0] > ddr->upright[0])
    {
        *upleft = (ddr->pdist_y * (sl - 1.0)) + ddr->upleft[0];
        *loleft = (ddr->pdist_y * (nl - 1.0)) + *upleft;
    }
    else
    {
        *upleft = ddr->upleft[0] - (ddr->pdist_y * (sl - 1.0));
        *loleft = *upleft - (ddr->pdist_y * (nl - 1.0));
    }

    if (ddr->loright[1] < ddr->loleft[1])
    {
        *(upleft + 1) = ddr->upleft[1] - (ddr->pdist_x * (ss - 1.0));
        *(upright + 1) = *(upleft + 1) - (ddr->pdist_x * (ns - 1.0));
    }
    else
    {
        *(upleft + 1) = (ddr->pdist_x * (ss - 1.0)) + ddr->upleft[1];
        *(upright + 1) = (ddr->pdist_x * (ns - 1.0)) + *(upleft + 1);
    }

    *upright = *upleft;
    *loright = *loleft;
    *(loleft + 1) = *(upleft + 1);
    *(loright + 1) = *(upright + 1);

}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_calcor:
 * @sl: I: Starting line of the output image.
 * @ss: I: Starting sample of the output image.
 * @nl: I: Number of lines of the output image.
 * @ns: I: Number of samples of the output image.
 * @ddr: I: A DDR structure.
 * @upleft: O: The new upper-left coordinate pair, Y-X, as calculated from 
 *             sl and the "upleft" corners of the DDR.
 * @upright: O: The new upper-right cordinate pair, Y-X, as calculated from 
 *              ss and the "upright" corners of the DDR.
 * @loleft: O: The new lower-left coordinate pair, Y-X, as calculated from 
 *             nl and the "loleft" corners of the DDR.
 * @loright: O: The new lower-right coordinate pair, Y-X, as calculated from 
 *              ns and the "loright" corners of the DDR.
 *
 * The c_calcor() call is used to calculate the four corner coordinates of the
 * window specified.  Each of the new corner coordinates will be calculated 
 * using the current coordinates in the DDR and the window specified in 
 * sl, ss, nl, and ns.  Rotated projection coordinates cannot be input 
 * into c_calcor().
 *
 * Deprecated.  Use las_calcor() instead.
 */
void c_calcor
(
    const double *sl,             /*  I: starting line                      */
    const double *ss,             /*  I: starting sample                    */
    const double *nl,             /*  I: number of lines                    */
    const double *ns,             /*  I: number of samples                  */
    const struct DDR *ddr,        /*  I: input DDR structure                */
    double *upleft,               /*  O: Upper left corner                  */
    double *upright,              /*  O: Upper right corner                 */
    double *loleft,               /*  O: Lower left corner                  */
    double *loright               /*  O: Lower right corner                 */
)
{
    las_calcor (*sl,*ss,*nl,*ns,ddr,upleft,upright,loleft,loright);

}

#endif /* obsolete interfaces */


