/******************************************************************************
NAME:   COMSYS

FUNCTION:
        COMSYS compares the system that this software is being run on and
        the system that created the input ddr.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         10/87       B.Ailts     initial development
    1.1         12/87       B.Ailts     Change include directory specifications
                                        Place the bridge routine in a seperate
                                        file
                                        Replace DESC arguments with char. string
    1.2         04/88       B.Ailts     Replaced newlas.h with las.h
    5.0         03/89       D. Hollaren Added IEEE, PRIME, and IBM machines to
                                        the table.  Made table into static array
    5.1         01/90       D. Akkerman Modified to call c_getddr directly
                                        rather than using label services.
    5.2         12/90       B. Ailts    Updated the error messages

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
                Run under TAE

PROJECT:        LAS

ALGORITHM:
   Create a table of systems that LAS currently is tested on and the types of
    data that can be shared between each of them
   Retrieve the cpu system from the logical(environment variable) LASSYS
   Set the value of the variable sys
   Retreive the system field of the DDR
   Set the value of the variable ddrsys
   Based upon the table set the output flag
   Return

ALGORITHM REFERENCES:
  IEEE Fromat from document:
        "IEEE standard for Binary Floating-Point Arithmetic 754-1985"

  System lookup tape representing the differances in data type from machine
  to machine.

           _____________________________________________________________
           | GOULD_UTX |  SUN_BSD  |  VAX_ULT  |  VAX_VMS  | IEEE_STD  |
------------------------------------------------------------------------
|GOULD_UTX | SAME      | NOT_REAL  | ONLY_BYTE | ONLY_BYTE | NOT_REAL  |
------------------------------------------------------------------------
|SUN_BSD   | NOT_REAL  | SAME      | ONLY_BYTE | ONLY_BYTE | SAME      |
------------------------------------------------------------------------
|VAX_ULT   | ONLY_BYTE | ONLY_BYTE | SAME      | SAME      | ONLY_BYTE |
------------------------------------------------------------------------
|VAX_VMS   | ONLY_BYTE | ONLY_BYTE | SAME      | SAME      | ONLY_BYTE |
------------------------------------------------------------------------
|IEEE_STD  | NOT_REAL  | SAME      | ONLY_BYTE | ONLY_BYTE | SAME      |
------------------------------------------------------------------------
|HP_MPE    | ONLY_BYTE | ONLY_BYTE | ONLY_BYTE | ONLY_BYTE | ONLY_BYTE |
------------------------------------------------------------------------
|PRIM_OS   | NOT_REAL  | NOT_REAL  | ONLY_BYTE | ONLY_BYTE | NOT_REAL  |
------------------------------------------------------------------------
|IBM_AIX   | NOT_REAL  | NOT_REAL  | ONLY_BYTE | ONLY_BYTE | ONLY_BYTE |
------------------------------------------------------------------------
|IBM_MVS   | NOT_REAL  | NOT_REAL  | ONLY_BYTE | ONLY_BYTE | ONLY_BYTE |
------------------------------------------------------------------------
|OTHER_MSC | NONE      | NONE      | NONE      | NONE      | NONE      |
------------------------------------------------------------------------

           _____________________________________________________________
           |  HP_XXX   | PRIME_OS  |  IBM_AIX  |  IBM_MVS  | OTHER_MSC |
------------------------------------------------------------------------
|GOULD_UTX | ONLY_BYTE | ONLY_BYTE | ONLY_BYTE | ONLY_BYTE | NONE      |
------------------------------------------------------------------------
|SUN_BSD   | ONLY_BYTE | NOT_REAL  | NOT_REAL  | NOT_REAL  | NONE      |
------------------------------------------------------------------------
|VAX_ULT   | ONLY_BYTE | ONLY_BYTE | ONLY_BYTE | ONLY_BYTE | NONE      |
------------------------------------------------------------------------
|VAX_VMS   | ONLY_BYTE | ONLY_BYE  | ONLY_BYTE | ONLY_BYTE | NONE      |
------------------------------------------------------------------------
|IEEE_STD  | ONLY_BYTE | NOT_REAL  | NOT_REAL  | NOT_REAL  | NONE      |
------------------------------------------------------------------------
|HP_MPE    | SAME      | NOT_REAL  | NOT_REAL  | NOT_REAL  | NONE      |
------------------------------------------------------------------------
|PRIM_OS   | NOT_REAL  | SAME      | NOT_REAL  | NOT_REAL  | NONE      |
------------------------------------------------------------------------
|IBM_AIX   | NOT_REAL  | NOT_REAL  | SAME      | NOT_REAL  | NONE      |
------------------------------------------------------------------------
|IBM_MVS   | NOT_REAL  | NOT_REAL  | NOT_REAL  | SAME      | NONE      |
------------------------------------------------------------------------
|OTHER_MSC | NONE      | NONE      | NONE      | NONE      | SAME      |
------------------------------------------------------------------------

******************************************************************************/

#include <string.h>
#include "las.h"
#include "labserv.h"
#include "sysdef.h"
#include "util.h"

#define SAME            0
#define NOT_REAL        1
#define ONLY_BYTE       2
#define NONE            3

/**
 * c_comsys:
 * @hostname: I: Image host file name for which the system field will
 *               be read and compared with the current processing system.
 * @flag: O: An integer which is returned with one of the following values:
 *           0: Systems are the same for all data types.
 *           1: Systems are the same for Integer*2, and Integer*4 types.
 *           2: Systems are the same for BYTE only.
 *           3: Systems are not the same for any data types.
 *
 * The c_comsys() call is used to compare the system that created the "hname"
 * with the system which will process the image "hname".  c_comsys currently
 * checks for the following systems:
 *     gould-utx,
 *     sun-bsd,
 *     vax-ult,
 *     ieee-std,
 *     hp-mpe,
 *     prime-primos,
 *     ibm-aix,
 *     ibm-mvs,
 *     other-msc.
 *
 * Returns:   E_SUCC:   Successful completion of operation,
 *            E_FAIL:   Unsuccessful completion of operation.
 */
int c_comsys
(
    const char *hostname,  /*  I:  host file name                           */
    int  *flag             /*  O:  output flag                              */
)

{
    int   ddrsys;          /* system stored in the DDR                      */
    int   status;          /*  status                                       */
    int   sys;             /* system of the CPU                             */

    char ext[4];           /*  extension of file                            */
    char hname[CMLEN];     /*  host ddr file name                           */
    char system[11];       /*  system buffer                                */


    struct DDR ddr;        /*  Working DDR structure.                       */

    /* System look up table (Same as table above).                          */
static const long table[NMR_SYSTEMS][NMR_SYSTEMS] = 
{
    {SAME, NOT_REAL, ONLY_BYTE, ONLY_BYTE, NOT_REAL, ONLY_BYTE, ONLY_BYTE,
           ONLY_BYTE, ONLY_BYTE, NONE},
    {NOT_REAL, SAME, ONLY_BYTE, ONLY_BYTE, SAME, ONLY_BYTE, NOT_REAL, NOT_REAL,
           NOT_REAL, NONE},
    {ONLY_BYTE, ONLY_BYTE, SAME, SAME, ONLY_BYTE, ONLY_BYTE, ONLY_BYTE, 
          ONLY_BYTE, ONLY_BYTE, NONE},
    {ONLY_BYTE, ONLY_BYTE, SAME, SAME, ONLY_BYTE, ONLY_BYTE, ONLY_BYTE, 
          ONLY_BYTE, ONLY_BYTE, NONE},
    {NOT_REAL, SAME, ONLY_BYTE, ONLY_BYTE, SAME, ONLY_BYTE, NOT_REAL, NOT_REAL,
          NOT_REAL, NONE},
    {ONLY_BYTE, ONLY_BYTE, ONLY_BYTE, ONLY_BYTE, ONLY_BYTE, SAME, NOT_REAL,
          NOT_REAL, NOT_REAL, NONE},
    {NOT_REAL, NOT_REAL, ONLY_BYTE, ONLY_BYTE, NOT_REAL, NOT_REAL, SAME, 
          NOT_REAL, NOT_REAL, NONE},
    {NOT_REAL, NOT_REAL, ONLY_BYTE, ONLY_BYTE, ONLY_BYTE, NOT_REAL, NOT_REAL, 
          SAME, NOT_REAL, NONE},
    {NOT_REAL, NOT_REAL, ONLY_BYTE, ONLY_BYTE, ONLY_BYTE, NOT_REAL, NOT_REAL,
          NOT_REAL, SAME, NONE},
    {NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, SAME} 
};

    /* Initalization of character strings */
    strcpy(ext,"ddr");

    /* Get system currently on */
    status = c_getsys(system);
    if (status != E_SUCC) 
    {
        c_lasmsg (FALSE, "comsys-call", "Error returned from getsys");
        return(E_FAIL);
    }

    /* Lower case the system name */
    las_up2low(system);

    /* Set sys according to the system currently on */
    if (strcmp(system,UTX) == 0)
        sys = SYS_GOULD_UTX;
    else if (strcmp(system,BSD) == 0)
        sys = SYS_SUN_BSD;
    else if (strcmp(system,ULT) == 0)
        sys = SYS_VAX_ULT;
    else if (strcmp(system,VMS) == 0)
        sys = SYS_VAX_VMS;
    else if (strcmp(system,IEEE) == 0)
        sys = SYS_IEEE_STD;
    else if (strcmp(system,HP) == 0)
        sys = SYS_HP_MPE;
    else if (strcmp(system,PRIMOS) == 0)
        sys = SYS_PRIME_PRIMOS;
    else if (strcmp(system,MVS) == 0)
        sys = SYS_IBM_MVS;
    else if (strcmp(system,AIX) == 0)
        sys = SYS_IBM_AIX;
    else
        sys = SYS_OTHER_MSC;

    /* Create the DDR file name */
    c_lsmknm(hostname,ext,hname);

    /* Retrieve the DDR information.  */
    status = c_getddr(hname,&ddr);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "comsys-call", "Error returned from getddr");
        return(E_FAIL);
    }

    /* Lower case the ddr system name */
    las_up2low(ddr.system);

    /* Set DDRSY according to the system that created the DDR file */
    if (strcmp(ddr.system,UTX) == 0)
        ddrsys = SYS_GOULD_UTX;
    else if (strcmp(ddr.system,BSD) == 0)
        ddrsys = SYS_SUN_BSD;
    else if (strcmp(ddr.system,ULT) == 0)
        ddrsys = SYS_VAX_ULT;
    else if (strcmp(ddr.system,VMS) == 0)
        ddrsys = SYS_VAX_VMS;
    else if (strcmp(ddr.system,IEEE) == 0)
        ddrsys = SYS_IEEE_STD;
    else if (strcmp(ddr.system,HP) == 0)
        ddrsys = SYS_HP_MPE;
    else if (strcmp(ddr.system,PRIMOS) == 0)
        ddrsys = SYS_PRIME_PRIMOS;
    else if (strcmp(ddr.system,MVS) == 0)
        ddrsys = SYS_IBM_MVS;
    else if (strcmp(ddr.system,AIX) == 0)
        ddrsys = SYS_IBM_AIX;
    else
        ddrsys = SYS_OTHER_MSC;

    /* Set the return flag */
    *flag = table[sys][ddrsys];

    return(E_SUCC);
}
