/*******************************************************************************
NAME                           C_GETBDF

PURPOSE      Retrieve specified field of the band dependent record

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
B. Ailts              Apr. 1988         Original development
B. Ailts              DEC. 1990         Updated error messages

PROJECT      LAS

ALGORITHM 
   Based upon the input key retrieve the correct field of the BDDR structure.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "local_protos.h"

/**
 * las_getbdf:
 * @bddr: I: A BDDR structure.
 *           It contains the fields of the band record of the DDR.
 * @key: I: A constant used to designate the field of the DDR will be returned
 *          in the argument "value".  The constants are as follows:
 *          DDBAND: Band number,
 *          DDMMV:  Min/max validity flag,
 *          DDMIN:  Minimum value,
 *          DDMAX:  Maximum value,
 *          DDSRC:  Source,
 *          DDINST: Instrument,
 *          DDDIR:  Direction,
 *          DDCDAT: Capture date,
 *          DDCTIM: Capture time.
 * @value:  O: The value of bbdr that correspnds to "key".  It should consist
 *             of the same data type and length as the field of "bddr" 
 *             that is needed.
 *
 * The las_getbdf() call is used to return the "key" field of the 
 * band-dependent record "bddr".
 *
 * Returns:      E_SUCC:  Successful completion of operation,
 *               E_FAIL:  Unsuccessful completion of operation.
 */
int las_getbdf
(
    const struct BDDR *bddr, /* I: input band dependent record              */
    int  key,                /* I: specified key of field needed            */
    void *value              /* O: output value of specified field          */
)

{ /*  getbdf  */

    double *dblval;          /* temporary pointer to a double field         */

    int  *longval;           /* temporary pointer to a long field           */

    char *charval;           /* temporary pointer to a char field           */
    char errtxt[ERRLEN + 1]; /* buffer space for errmsg                     */

    /*  Assign the temporary pointers the address of the output value argum */
    dblval = (double *) value;
    longval = (int *) value;
    charval = (char *) value;

    /*  Based upon the value of the input key return the value of the 
        corresponding field of the DDR structure.   */
    switch (key)
    {
        case DDBAND: 
            *longval = bddr->bandno;
            break;
        case DDMMV:  
            *longval = bddr->valid;
             break;
        case DDMIN:     
            *dblval = bddr->minval;
            break;
        case DDMAX:     
            *dblval = bddr->maxval;
             break;
        case DDSRC:  
            strcpy(charval,bddr->source);
            break;
        case DDINST: 
            strcpy(charval,bddr->instrument);
            break;
        case DDDIR:     
            strcpy(charval,bddr->direction);
            break;
        case DDCDAT: 
            strcpy(charval,bddr->date);
            break;
        case DDCTIM: 
            strcpy(charval,bddr->time);
            break;
        default:        
            sprintf(errtxt,"%d is an invalid input key",key);
            c_lasmsg(FALSE, "getbdf-key", errtxt);
            return(E_FAIL);
    } /* switch (*key)  */

    return(E_SUCC);
} /*  getbdf  */



#ifndef NO_OBSOLETE_INTERFACE


/**
 * c_getbdf:
 * @bddr: I: A BDDR structure.
 *           It contains the fields of the band record of the DDR.
 * @key: I: A constant used to designate the field of the DDR will be returned
 *          in the argument "value".  The constants are as follows:
 *          DDBAND: Band number,
 *          DDMMV:  Min/max validity flag,
 *          DDMIN:  Minimum value,
 *          DDMAX:  Maximum value,
 *          DDSRC:  Source,
 *          DDINST: Instrument,
 *          DDDIR:  Direction,
 *          DDCDAT: Capture date,
 *          DDCTIM: Capture time.
 * @value:  O: The value of bbdr that correspnds to "key".  It should consist
 *             of the same data type and length as the field of "bddr" 
 *             that is needed.
 *
 * The c_getbdf() call is used to return the "key" field of the 
 * band-dependent record "bddr".
 *
 * Deprecated.  Use las_getbdf() instead.
 *
 * Returns:      E_SUCC:  Successful completion of operation,
 *               E_FAIL:  Unsuccessful completion of operation.
 */
int c_getbdf
(
    const struct BDDR *bddr,      /* I: input band dependent record         */
    const long *key,              /* I: specified key of field needed       */
    void  *value                  /* O: output value of specified field     */
)
{
    int status;

    status = las_getbdf (bddr,*key,value);

    return (status);

}

#endif /* obsolete interfaces */

