/*******************************************************************************
NAME                           C_GETBDR

PURPOSE      Retrieve specified band dependent record of a DDR file

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
K. Zanter             Feb. 1988         original development
B. Ailts              Apr. 1988         Replaced newlas.h with las.h
                                        Changed the calling sequence -- used
                                          only one structure
                                        Replaced ls___ status to E_SUCC
                                        Replaced cerrmsg with c_errmsg

D. Akkerman           Sept. 1989        Added code to convert the minval and
                                        maxval (double) portions of the BDR
                                        structure to the floating point format
                                        of the host system.  This occurs if the
                                        DDR is read from a "remote" (NFS)
                                        system or if the DDR was created from
                                        the "remote" (NFS) system and resides
                                        on the current host system.
B. Ailts              Dec. 1990         Updated error messages

PROJECT      LAS

ALGORITHM 
   Create the associated DDR file name from the input host image name.
   Retrieve the host system name.
   Open the DDR file for read access using label services.  
   Read the integer portion of the DDR (record 1).
   Retrieve the system name from the DDR.
   If the host system name and the DDR specified system name are not the same:
        Set the conversion flag to TRUE.
   Set up for and read the specified record the corresponds to the input band #.
   If the conversion flag is set:
        Retrieve the system code (integer) for the host name.
        Retrieve the system code (integer) for the DDR specified system.
        Convert the double precision elements of the BDR record to the floating
         point format of the host system.
   Close the DDR file.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "pixman.h"
#include "sysdef.h"
#include "util.h"
#include "local_protos.h"

/**
 * las_getbdr:
 * @hname: I: Image host file name for which the information is to be 
 *            retrieved.
 * @bddr: O: A BDDR structure.  It contains the fields of the band 
 *           record of the DDR.
 * @band: I: The band number of the DDR record to be retrieved.
 *
 * The las_getbdr() call is used to return the specified band-dependent
 * record of DDR information of image "hname".
 * 
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */

int las_getbdr
(
    const char *hname,    /* I: input host image name                       */
    struct BDDR *bddr,    /* O: output band dependent record                */
    int band              /* I: specified band number                       */
)

{
    struct DDR ddr;       /* working DDR structure                          */

    int   access;         /* file access type                               */
    int   action;         /* file close action flag                         */
    int   clen;           /* length of character part of record             */
    int   ctemp;          /* temporary storage for clen                     */
    int   dlen;           /* length of data part of record                  */
    int   dtemp;          /* temporary storage for dlen                     */
    int   fd;             /* file descriptor for opened file                */
    int   status;
    int   size = DBSIZE;  /* number of doubles in the BDR                   */
    int   flag = SYS_SET; /* flag that sets the doubles in the BDR to the
                               maximum size of the host machine if overflow
                               or underflow is detected in the conversion
                               process                                      */
    int   dtyp = EDOUBLE; /* data type of the buffer to be converted        */
    int   convflg = FALSE;/* working conversion flag                        */
    int   under; 
    int   over;           /* dummy underflow and overflow flags             */
    int   incode;         /* host system code                               */
    int   syscode;        /* DDR system code                                */
    int   sysflg = SETN;  /* system conversion flag (string to int)         */

    unsigned char *dbuf;  /* pointer to area where data is stuffed          */

    char dtype[3];        /* type of data portion of record                 */
    char hostddr[CMLEN];  /* host name of DDR file                          */
    char key[16];         /* record key                                     */
    char tempchar[DDTSLN];/* character portion of the record                */
    char *tempptr;
    char insys[DDSYLN];   /* host system name                               */
    char tmpstr1[5];      /* temp string for band number                    */
    char tmpstr2[3];      /* temp string for band valid number              */

    c_lsmknm(hname,"ddr",hostddr);     /* make associated DDR file name     */

    if (c_getsys(insys) != E_SUCC)     /* retrieve the host system name     */
    {
        c_lasmsg (FALSE, "getbdr-call", "Error returned from getsys");
        return(E_FAIL);
    }

    access = 0;                        /* open DDR file for read           */
    fd = c_labopen (hostddr, access);
    if (fd < 0) 
    {
        c_lasmsg (FALSE, "getbdr-call", "Error returned from lsopen");
        return(E_FAIL);
    }

    clen = DDSTCT * DDSYLN;            /* set up and read record 1 of  DDR */
    ctemp = clen - 1;
    dlen = dtemp = DISIZE * 4;
    dbuf = (unsigned char *) &ddr;
    strcpy(key,"DDRINT");
    status = c_labread (fd,key,&clen,&dlen,ddr.system,dbuf,dtype);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "getbdr-call", "Error returned from lsread");
        return(E_FAIL);
    }
    if ((clen != ctemp) || (dlen != dtemp))
    {
        c_lasmsg (FALSE, "getbdr-read", 
                  "Error reading record of associated DDR file");
        return(E_FAIL);
    }

    if (strcmp(insys,ddr.system) != 0) /* if the DDR system and the host    */
        convflg = TRUE;                /* system are not the same then set  */
                                       /* the conversion flag               */

    sprintf(key,"BAND%d",band);        /* set up for/read in spec band record*/
    clen = DDTSLN;
    ctemp = clen - 1;
    dlen = dtemp = DBSIZE * 8;
    dbuf = (unsigned char *) &(bddr->minval);
    status = c_labread (fd,key,&clen,&dlen,tempchar,dbuf,dtype);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "getbdr-call", "Error returned from lsread");
        return(E_FAIL);
    }

    if ( (clen != ctemp) || (dlen != dtemp) )
    {
        c_lasmsg (FALSE, "getbdr-read", 
                  "Error reading record of associated DDR file");
        return(E_FAIL);
    }

    if (convflg)                       /* if the conversion flag is set     */
    {
        /* retrieve the host system */
        if (las_sysset(insys,&incode,sysflg) != E_SUCC) 
        {
            c_lasmsg (FALSE, "getbdr-call", "Error returned from sysset");
            return(E_FAIL);
        }

        /* retrieve the DDR   */
        if (las_sysset(ddr.system,&syscode,sysflg) != E_SUCC) 
        {
            c_lasmsg (FALSE, "getbdr-call", "Error returned from sysset");
            return(E_FAIL);
        }

        if (c_pxsys(&syscode,&incode,dbuf,&dtyp,&size,&under,&over,
                &flag) != E_SUCC)      /* perform the conversion            */
        {
            c_lasmsg (FALSE, "getbdr-call", "Error returned from pxsys");
            return(E_FAIL);
        }
    }

    sscanf(tempchar,"%4c%2c",tmpstr1,tmpstr2);
    tmpstr1[4] = '\0';
    tmpstr2[2] = '\0';
    bddr->bandno = atoi(tmpstr1);
    bddr->valid = atoi(tmpstr2);
    tempptr = tempchar + DDBNLN + DDVLLN;
    strcpy(bddr->source,tempptr);
    tempptr += DDSRLN;
    strcpy(bddr->instrument,tempptr);
    tempptr += DDINLN;
    strcpy(bddr->direction,tempptr);
    tempptr += DDDRLN;
    strcpy(bddr->date,tempptr);
    tempptr += DDCDLN;
    strcpy(bddr->time,tempptr);

    action = 0;                        /* close associated DDR file         */
    if (c_labclose (fd, hostddr, action) != E_SUCC)
    {
        c_lasmsg (FALSE, "getbdr-call", "Error returned from labclose");
        return(E_FAIL);
    }

    return(E_SUCC);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_getbdr:
 * @hname: I: Image host file name for which the information is to be 
 *            retrieved.
 * @bddr: O: A BDDR structure.  It contains the fields of the band 
 *           record of the DDR.
 * @band: I: The band number of the DDR record to be retrieved.
 *
 * The c_getbdr() call is used to return the specified band-dependent
 * record of DDR information of image "hname".
 * 
 * Deprecated.  Use las_getbdr() instead.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_getbdr
(
    const char *hname,    /* I: input host image name                      */
    struct BDDR *bddr,    /* O: output band depenent record                */
    const long *band      /* I: specified band number                      */
)

{

    int status;

    status = las_getbdr (hname,bddr,*band);

    return (status);

}

#endif /* obsolete interfaces */


