/******************************************************************************

NAME:   LUSED

FUNCTION:
        LUSED is used to retrieve the current date and time and place 
        this data in the LAST_USED DATE and LAST_USED TIME fields of
        the ddr.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         10/87       B.Ailts     Initial development
    1.1         12/87       B.Ailts     Change include directory specifications
                                        Place bridge routines in a seperate file
                                        Replace DESC argument with char. str.
    1.2         04/88       B.Ailts     Replaced newlas.h with las.h
                                        Replaced the return status checks of
                                          the ls___ calls to E_SUCC
                                        Replaced cerrmsg with c_errmsg
                                        Standardized the err messages
    1.3         06/90       P.Pease     Removed long time(),my_time to fix
                                        warning that declaration of time
                                        conflicts with a previous declaration
    1.4         12/90       B.Ailts     Updated error messages
    1.5         07/98       T.Baltzer   Y2K compliant mods.
    1.6         10/98       D.Hames     Change %d to %02d. Need to ensure a 2digit year
                                        for Y2K.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
                Run under TAE

PROJECT:        LAS

ALGORITHM:
        LUSED calls the utilities TIME and LOCALTIME to retreive the current
        time from the system.  
        It then converts this information into the correct format.  
        This is then written to the ddr associated with the input host name.

ALGORITHM REFERENCES:
******************************************************************************/
#include <string.h>
#include <time.h>

#include "las.h"
#include "labserv.h"

/**
 * c_lused:
 * @hostname: I: Image host file name for which the last used date and
 *               time fields are updated.
 *
 * The c_lused() call is used to update the last used date and time fields
 * of the DDR associated with the image "hname".
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation,
 *            E_PROT:  DDR is write protected; No error message produced
 *                     from this error.
 */
int c_lused
(
    const char *hostname         /* I: host name of associated DDR        */
)

{ /* lused */

    struct time_struct           /*  structure of time from system          */
    {
        char hr[3];
        char minute[3];
        char sec[3];
    };
    struct time_struct time_tem; /*  Time returned from system              */
    struct tm *tm_ptr;           /* temporary pointer in time struct.       */
    struct DDR ddr;              /* ddr structure                           */
    int   access;                /* label services access                   */
    int   action;                /* action of lsclose                       */
    int   clen;                  /* character length                        */
    int   dlen;                  /* data length                             */
    int   fd;                    /* file descriptor                         */
    int   mode = 2;              /* type of open                            */
    int   status;                /* return status                           */
    time_t my_time;              /* time(), current time                    */

    char *ddrstr;                /* pointer to string part of DDR           */
    char dtype[3];               /* data type of label services record      */
    char hname[CMLEN];           /* host name of ddr                        */
    char key[16];                /* key of label services record            */
    char mon[4];                 /* current month                           */

    unsigned char *dbuf_buf;     /*  pointer to data buffer                 */

    c_lsmknm(hostname,"ddr",hname);

    status = c_labstat(hname,mode);
    if (status != E_SUCC)
        return(E_PROT);

    my_time = time(0);
    tm_ptr = localtime(&my_time);

    /* Calculate and format the time */
    sprintf (time_tem.hr,"%2d",tm_ptr->tm_hour);
    sprintf (time_tem.minute,"%2d",tm_ptr->tm_min);
    sprintf (time_tem.sec,"%2d",tm_ptr->tm_sec);
    if (tm_ptr->tm_hour < 10)
        time_tem.hr[0] = '0';
    if (tm_ptr->tm_min < 10)
        time_tem.minute[0] = '0';
    if (tm_ptr->tm_sec < 10)
        time_tem.sec[0] = '0';

    /* retrieve the correct string from the month provided */
    switch (tm_ptr->tm_mon)
    {
        case 0 : 
            sprintf (mon,"Jan");
            break;
        case 1 : 
            sprintf (mon,"Feb");
            break;
        case 2 : 
            sprintf (mon,"Mar");
            break;
        case 3 : 
            sprintf (mon,"Apr");
            break;
        case 4 : 
            sprintf (mon,"May");
            break;
        case 5 : 
            sprintf (mon,"Jun");
            break;
        case 6 : 
            sprintf (mon,"Jul");
            break;
        case 7 : 
            sprintf (mon,"Aug");
            break;
        case 8 : 
            sprintf (mon,"Sep");
            break;
        case 9 : 
            sprintf (mon,"Oct");
            break;
        case 10 : 
            sprintf (mon,"Nov");
            break;
        case 11 : 
            sprintf (mon,"Dec");
            break;
    }

    /*  Open the DDR file   */
    access = 3;
    fd = c_labopen(hname, access);
    if (fd < 0) 
    {
        c_lasmsg (FALSE, "lused-call", "Error returned from labopen");
        return(E_FAIL);
    }

    /*  Initialize the variables needed to read the DDR           */
    strcpy(dtype,"I4");
    strcpy(key,"DDRINT");
    clen= DDSTCT * DDSYLN;
    dlen= DISIZE * 4;
    dbuf_buf = (unsigned char *) &ddr;
    ddrstr = (char *)ddr.system;

    /*  Read the DDR record                                       */
    status = c_labread (fd,key,&clen,&dlen,ddrstr,dbuf_buf,dtype);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "lused-call", "Error returned from labread");
        return(E_FAIL);
    }

    /*  Place the values into the DDR structure                   */
    sprintf (ddr.last_used_date,"%d-%3s-%02d",tm_ptr->tm_mday,
             mon,tm_ptr->tm_year % 100);
    sprintf (ddr.last_used_time,"%2s%2s:%2s",time_tem.hr,time_tem.minute,
             time_tem.sec);

    /*  Replace the DDR record                                    */
    status = c_labreplace (fd,key,clen,dlen,ddrstr,(char *)dbuf_buf,dtype);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "lused-call", "Error returned from labreplace");
        return(E_FAIL);
    }

    /*  Close the DDR file                                        */
    action = 0;
    status = c_labclose (fd,hname,action);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "lused-call", "Error returned from labclose");
        return(E_FAIL);
    }

    return(E_SUCC);
}
