/*******************************************************************************
NAME                          C_MERGE_DDRS

PURPOSE      Combines a set of DDRs by correlating them into a single DDR

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------

PROJECT       LAS

ALGORITHM 
   Retreive the ouptut DDR band indepenent records

   If no. of input images is equal to one
      Retreive the input DDR band indepenent records
      Copy the fields to the output ddr
   Else
      Retreive the band indepenent records of all the input DDRs
      Compare validity flags and values to see if the values of the
        output DDR will be copied from the input DDR or made invalid
   End if

    Based on the flags set by the above comparisons or the validity
      flag if there was only one input image, copy or recalculate the
      individual fields of the output DDR.
    Write the band indepenent records to the output DDR file

    If cflag is set to COMB
       Read the band depenent data of the input DDRs
       Compare the fields to the correct band of the output DDR
       If the band dependent fields are equal then
          Copy the input band dependent fields to the output DDR
       Else
          Invalidate the band depenent fields of the output DDR
    Else
       Read the band depenent data of the input DDRs
       Copy the fields to the correct band of the output DDR
    Endif

    Return

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "local_protos.h"


/**
 * las_merge_ddrs:
 * @parblk: I: TAE parameter block passed by reference.  It is assumed that
 *             TAE has been initialized prior to calling las_merge_ddrs().
 * @host_in: I: Image host filename(s) from which the DDR values are to be
 *              retrieved and used to calculate the output DDR values.
 * @host_out: I: Output image host file name for which the DDR values are
 *              calculated and written to.
 * @nmrimg: I: Total number of input image(s).
 * @window: I: Two-dimensional array containing the window specification for
 *             each input image.  The first dimension is "imglimit" and the
 *             second one is 4.  The four elements of the window are SL,
 *             SS, NL, and NS.
 * @bands:  I: Two-dimensional array containing the band numbers of each of 
 *             the input images.  The first dimension is "imglimit" and the
 *             second is MAXBAND + 1.
 * @nbands: I: Number of bands specified for each input image.
 * @line_inc: I: Number of pixels of the input image(s) that one output pixel
 *               will represent in the line direction.  A positive fractional
 *               value greater than 0 and less than 1 specifies that the
 *               input image(s) have been expanded.  A positive integer
 *               value greater than 1 specifies that the input image(s) have
 *               been subsampled.  If no replication or subsampling of
 *               pixels was completed enter a value of 1.
 * @samp_inc: I: Number of pixels of the input image(s) that one output
 *               pixel will represent in the sample direction.  A positive
 *               fractional value greater than 0 and less than 1 specifies
 *               that the input image(s) havea been expanded.  A positive
 *               integer value greater than 1 specifies that the input
 *               image(s) have been subsampled.  If no replication or
 *               subsampling of pixels was completed enter a value of 1.
 * @out_nbands: I: Total number of bands the output image will contain.
 * @cflag:  I: Flag indicating whether the input images will be combined into
 *             one band of the output image.  This value should be set to
 *             COMB for functions like CONCAT, MATH where the number of 
 *             output bands is not the same as the number of input bands.
 *             Valid values are:
 *             COMB: images will be combined,
 *             NOCOMB: images will not be combined.
 * @out_bddr:  O: The output band dependent DDR records.
 * @out_ddr: O: The output DDR structure.
 * @upddr_flag: I: Flag indicating if called by las_upddr(). 
 *
 * The las_merge_ddrs() call reads "nmrimg" DDR files associated with the 
 * image host file names specified in "host_in".  Based upon the values of
 * "window, bands, nbands, line_inc, samp_inc, out_nbands, cflag", and the
 * values contained in the input DDR, the DDR associated with "host_out" will
 * contain values that are copied from the input DDRs, recalculated from the
 * input DDRs, or invalidated.
 *
 * This function does not actually write to an output DDR file.  That is 
 * performed by the las_upddr() function in lieu of calling this function.
 *
 * Returns:     E_SUCC:  Successful completion of operation,
 *              E_FAIL:  Unsuccessful completion of operation.
 */
 
int las_merge_ddrs
(
    struct PARBLK *parblk,          /* I: TAE parameter block               */
    const  char host_in[][CMLEN],   /* I: array of input host names         */
    const  char *host_out,          /* I: Output host name                  */
    int    nmrimg,                  /* I: Total number of input images      */
    const int window[][4],          /* I: Window specifications for each ima*/
    const  int  bands[][MAXBND +1], /* I: band numbers for each input image */
    const  int  nbands[],           /* I: number of bands for each input ima*/
    double line_inc,                /* I: increment in the line direction   */
    double samp_inc,                /* I: increment in the sample direction */
    int    out_nbands,              /* I: number of output bands            */
    int    cflag,                   /* I: combination of bands flag         */
    struct BDDR **out_bddr,         /* O: output band dependent ddr records */
    struct DDR *out_ddr,            /* O: output ddr                        */
    char   upddr_flag               /* I: Flag: TRUE means called by upddr  */
)
{ /* c_merge_ddrs.c  */

    struct VARIABLE *val;         /*  temporary pointer to  parameter value */

    struct DDR *in_ddr;           /* input ddrs                             */
    struct BDDR *in_bddr;         /* input band dependent ddr records       */

    float tgbl;                   /*  TAE tolerance global                  */

    int  corn_index;              /*  index to 1st  DDR with valid corner fl*/
    int  corner_flag;             /*  Processing corner validity flag       */
    int  curbnd;                  /*  loop counter -- current band          */
    int  dms = FALSE;             /*  dms flag                              */
    int  gbl_flag;                /*  TAE validity flag to use when not val */
    int  i;                       /*  loop counter                          */
    int  in_bands;                /*  total number of input bands           */
    int  inc_flag;                /*  Processing increment validity flag    */
    int  inc_index;               /*  index to first DDR with valid inc flag*/
    int  j;                       /*  loop counter                          */
    int  pdist_flag;              /*  Processing proj. distance validity flg*/
    int  pdist_index;             /*  index to first DDR with valid inc flag*/
    int  ppar_flag;               /*  Processing corner proj. parameter flag*/
    int  proj_flag;               /*  Processing corner projection flag     */
    int  status;                  /*  status flag                           */
    int  zone_flag;               /*  Processing corner zone flag           */

    char vflag[3];                /* ddr valid flag                         */

    /*  Allocate space for the input DDRs   */
    in_bands = 0;
    for (i = 0; i < nmrimg; i++)
        in_bands += nbands[i];
    in_ddr = (struct DDR *)calloc(nmrimg,sizeof(struct DDR));
    *out_bddr = (struct BDDR *)calloc(out_nbands,sizeof(struct BDDR));
    if (cflag == COMB)
        in_bddr = (struct BDDR *)calloc(in_bands,sizeof(struct BDDR));
    else
        in_bddr = (struct BDDR *)malloc(sizeof(struct BDDR));
    if ((in_ddr == NULL) || (*out_bddr == NULL) || (in_bddr == NULL))
    {
        c_lasmsg (FALSE, "merge_ddrs-alloc", "Error allocating dynamic memory");
        free (in_ddr);
        free (*out_bddr);
        *out_bddr = NULL;
        free (in_bddr);
        return (E_FAIL);
    }

    /*  Retrieve the DDR validity flag default from the parblk         */
    val = p_find(parblk,"$DDRVFLG");
    strcpy (vflag,SVAL(*val,0));
    if (strcmp(vflag,"IN") == 0)
        gbl_flag = INVAL;
    else
        gbl_flag = UNKNOW;

    /*  Retrieve the DDR validity flag default from the parblk         */
    val = p_find(parblk,"$PROJTOL");
    tgbl = RVAL(*val,0);

    /*  Retreive the output ddr information -- this is done so that the
        information placed in the ddr by eopen_ will not be lost      */
    status = c_getddr((upddr_flag == TRUE) ? host_out : host_in[0], out_ddr);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "merge_ddrs-call", "Error returned from getddr");
        free (in_ddr);
        free (*out_bddr);
        *out_bddr = NULL;
        free (in_bddr);
        return (E_FAIL);
    }

    /*  if only one input image copy needed information from the input ddr  */
    if (nmrimg == 1)
    {
        /* get input image DDR */
        status = c_getddr(host_in[0],&in_ddr[0]);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "merge_ddrs-call", "Error returned from getddr");
            free (in_ddr);
            free (*out_bddr);
            *out_bddr = NULL;
            free (in_bddr);
            return (E_FAIL);
        }
        /* if the projection units are DMS, convert to degrees for
           calculations                                           */
        if ((strncmp(in_ddr[0].proj_units,"DMS",3) == 0) ||
                (strncmp(in_ddr[0].proj_units,"dms",3) == 0))
        {
            if (convert_ddr(&in_ddr[0],"DEG") != E_SUCC) 
            {
                c_lasmsg (FALSE, "merge_ddrs-call",
                          "Error returned from convert_ddr");
                free (in_ddr);
                free (*out_bddr);
                *out_bddr = NULL;
                free (in_bddr);
                return (E_FAIL);
            }
            dms = TRUE;
        }

        /*  Set the validity flags used by set_corners routines    */
        corner_flag = in_ddr[0].valid[DDCCV];
        corn_index= 0;
        proj_flag = in_ddr[0].valid[DDPCV];
        pdist_flag = in_ddr[0].valid[DDPDV];
        pdist_index = 0;
        inc_flag = in_ddr[0].valid[DDINCV];
        inc_index = 0;
        zone_flag = in_ddr[0].valid[DDZCV];
        ppar_flag = in_ddr[0].valid[DDPPV];
        set_master(&in_ddr[0],out_ddr,inc_flag,window[0]);

        /*  Copy certain fields from the input DDR to the output DDR        */
        out_ddr->valid[DDPCV] = in_ddr[0].valid[DDPCV];
        out_ddr->valid[DDZCV] = in_ddr[0].valid[DDZCV];
        out_ddr->valid[DDDCV] = in_ddr[0].valid[DDDCV];
        out_ddr->valid[DDPPV] = in_ddr[0].valid[DDPPV];
        out_ddr->valid[DDPUV] = in_ddr[0].valid[DDPUV];
        out_ddr->proj_code    = in_ddr[0].proj_code;
        out_ddr->zone_code     = in_ddr[0].zone_code;
        out_ddr->datum_code    = in_ddr[0].datum_code;
        strcpy(out_ddr->proj_units,in_ddr[0].proj_units);

        for (i = 0; i < 15; i++)
            out_ddr->proj_coef[i] = in_ddr[0].proj_coef[i];

    }

    else  /* multiple input images */
    {
        /*  Read the ddr information for each input ddr                     */
        for (i = 0; i < nmrimg; i++)
        {
            status = c_getddr(host_in[i],&in_ddr[i]);
            if (status != E_SUCC)
            {
                c_lasmsg (FALSE,"merge_ddrs-call","Error returned from getddr");
                free (in_ddr);
                free (*out_bddr);
                *out_bddr = NULL;
                free (in_bddr);
                return (E_FAIL);
            }
            /*  if the projection units are DMS, convert to degrees
                for calculations                                            */
            if ((strncmp(in_ddr[i].proj_units,"DMS",3) == 0) ||
                    (strncmp(in_ddr[i].proj_units,"dms",3) == 0))
            {
                if (convert_ddr(&in_ddr[i],"DEG") != E_SUCC) 
                {
                    c_lasmsg (FALSE, "merge_ddrs-call",
                              "Error returned from convert_ddr");
                    free (in_ddr);
                    free (*out_bddr);
                    *out_bddr = NULL;
                    free (in_bddr);
                    return (E_FAIL);
                }
                dms = TRUE;
            }
        }
        /*  Copy or null the individual fields of the ddr                   */ 
        set_proj(in_ddr,out_ddr,&proj_flag,nmrimg);
        set_datum(in_ddr,out_ddr,nmrimg);
        set_zone(in_ddr,out_ddr,&zone_flag,nmrimg);
        las_set_ppar(in_ddr,out_ddr,&ppar_flag,proj_flag,nmrimg);
        set_punit(in_ddr,out_ddr,nmrimg);

        /*  Set the flags of the individual fields of the ddr which may need to
            be recalculated.                                                */
        set_pdis_flag(in_ddr,&pdist_flag,&pdist_index,nmrimg);
        set_inc_flag(in_ddr,&inc_flag,&inc_index,nmrimg);
        set_corn_flag(in_ddr,&corner_flag,&window[0],&corn_index,
                pdist_flag,tgbl,pdist_index,gbl_flag,nmrimg);
        out_ddr->master_line = 1;
        out_ddr->master_sample = 1;
    } /* else multiple input images */

    /*  Recalculate the individual fields of the ddr based upon their flags
        which were set above                                                */
    set_increment(inc_flag,&in_ddr[inc_index],out_ddr,line_inc,samp_inc);
    set_pdist(pdist_flag,&in_ddr[pdist_index],out_ddr,line_inc,samp_inc);
    set_corners(corner_flag,proj_flag,pdist_flag,zone_flag,ppar_flag,
            &in_ddr[corn_index],out_ddr,line_inc,samp_inc,
            window[corn_index]);

    /*  If projection units was DMS, convert out_ddr to DMS                 */
    if (dms)
    {
        if (convert_ddr(out_ddr,"DMS") != E_SUCC) 
        {
            free (in_ddr);
            free (*out_bddr);
            *out_bddr = NULL;
            free (in_bddr);
            return (E_FAIL);
        }
    }

    /*  Retreive the band DDR information for the output image so that
        the min, max, and valid values are not lost                         */
    if (upddr_flag == TRUE)
    {
        curbnd = 1;
        for ( i = 0; i < out_nbands; i++)
        {
            status = las_getbdr(host_out, &((*out_bddr)[i]),curbnd);
            if (status != E_SUCC)
            {
                c_lasmsg (FALSE,"merge_ddrs-call","Error returned from getbdr");
                free (in_ddr);
                free (*out_bddr);
                *out_bddr = NULL;
                free (in_bddr);
                return (E_FAIL);
            }
            curbnd++;
        }
    }

    /*  Compare the band dependent fields of each input ddr to check equality.
        This is done only when two or more individual bands will be combined
        into one band(i.e. applications like CONCAT,MATH)                   */
    if (cflag == COMB)
    {
        curbnd = 0;
        for (j = 0; j < nmrimg; j++)
        {
            for (i = 0; i < nbands[j]; i++)
            {

                /*  Retreive the band DDR information for each band of each
                    image                                                   */
                status = las_getbdr(host_in[j],&(in_bddr[curbnd]),bands[j][i]);
                if (status != E_SUCC)
                {
                    c_lasmsg (FALSE, "merge_ddrs-call",
                              "Error returned from getbdr");
                    free (in_ddr);
                    free (*out_bddr);
                    *out_bddr = NULL;
                    free (in_bddr);
                    return (E_FAIL);
                }
                curbnd++;
            }
        } /* for i < out_nbands */
        /*  Compare the fields for each band                                */  
        set_capture(&(in_bddr[0]),&((*out_bddr)[0]),in_bands);
    } /* cflag == COMB  */
    else /* there is one output band for each input band of each image      */
    { /* else cflag == NOCOMB */
        curbnd = 0;

        /* Check to ensure that the number of output bands == the number
           of total input bands for each image.  If we're going to assume it, 
           it might as well be checked to make sure.                        */

        if (in_bands != out_nbands)
            c_lasmsg (FALSE, "merge_ddrs-call",
                      "Number of output bands does not equal number of"
                      " input bands");

        for (j = 0; j < nmrimg; j++)
        {
            for (i = 0; i < nbands[j]; i++)
            {
                /*  Retreive the band DDR information for each band 
                    of each image                                           */
                status = las_getbdr(host_in[j],in_bddr,bands[j][i]);
                if (status != E_SUCC)
                {
                    c_lasmsg (FALSE, "merge_ddrs-call",
                              "Error returned from getbdr");
                    free (in_ddr);
                    free (*out_bddr);
                    *out_bddr = NULL;
                    free (in_bddr);
                    return (E_FAIL);
                }

                /*  Copy the band fields of the input DDR to the band fields 
                    of the output DDR except for minval, maxval, bandno, 
                    and validity                                            */
                strcpy((*out_bddr)[curbnd].source,in_bddr->source);
                strcpy((*out_bddr)[curbnd].instrument,in_bddr->instrument);
                strcpy((*out_bddr)[curbnd].direction,in_bddr->direction);
                strcpy((*out_bddr)[curbnd].date,in_bddr->date);
                strcpy((*out_bddr)[curbnd].time,in_bddr->time);
                curbnd++;

                /* Memory errors occur if we try to copy more input bands 
                   than we have output bands, hence the following error check */

                if (curbnd > out_nbands)
                    break;

            } /* for i to bands[j][i] */
        } /* for j to nmrimg */
    } /* else cflag == NOCOMB */
    free (in_ddr);
    free (in_bddr);
    return (E_SUCC);
} /*  c_merge_ddrs  */


#ifndef NO_OBSOLETE_INTERFACE



/**
 * c_merge_ddrs:
 * @parblk: I: TAE parameter block passed by reference.  It is assumed that
 *             TAE has been initialized prior to calling c_merge_ddrs().
 * @host_in: I: Image host filename(s) from which the DDR values are to be
 *             retrieved and used to calculate the output DDR values.
 * @host_out: I: Output image host file name for which the DDR values are
 *              calculated and written to.
 * @nmrimg: I: Total number of input image(s).
 * @window: I: Two-dimensional array containing the window specification for
 *             each input image.  The first dimension is "imglimit" and the
 *             second one is 4.  The four elements of the window are SL,
 *             SS, NL, and NS.
 * @bands:  I: Two-dimensional array containing the band numbers of each of 
 *             the input images.  The first dimension is "imglimit" and the
 *             second is MAXBAND + 1.
 * @nbands: I: Number of bands specified for each input image.
 * @line_inc: I: Number of pixels of the input image(s) that one output pixel
 *               will represent in the line direction.  A positive fractional
 *               value greater than 0 and less than 1 specifies that the
 *               input image(s) have been expanded.  A positive integer
 *               value greater than 1 specifies that the input image(s) have
 *               been subsampled.  If no replication or subsampling of
 *               pixels was completed enter a value of 1.
 * @samp_inc: I: Number of pixels of the input image(s) that one output
 *               pixel will represent in the sample direction.  A positive
 *               fractional value greater than 0 and less than 1 specifies
 *               that the input image(s) havea been expanded.  A positive
 *               integer value greater than 1 specifies that the input
 *               image(s) have been subsampled.  If no replication or
 *               subsampling of pixels was completed enter a value of 1.
 * @out_nbands: I: Total number of bands the output image will contain.
 * @cflag:  I: Flag indicating whether the input images will be combined into
 *             one band of the output image.  This value should be set to
 *             COMB for functions like CONCAT, MATH where the number of 
 *             output bands is not the same as the number of input bands.
 *             Valid values are:
 *             COMB: images will be combined,
 *             NOCOMB: images will not be combined.
 * @out_bddr:  O: The output band dependent DDR records.
 * @out_ddr: O: The output DDR structure.
 * @upddr_flag: I: Flag indicating if called by las_upddr(). 
 *
 * The c_merge_ddrs() call reads nmrimg DDR files associated with the 
 * image host file names specified in "host_in".  Based upon the values of
 * "window, bands, nbands, line_inc, samp_inc, out_nbands, cflag", and the
 * values contained in the input DDR, the DDR associated with "host_out" will
 * contain values that are copied from the input DDRs, recalculated from the
 * input DDRs, or invalidated.
 *
 * This function does not actually write to an output DDR file.  That is 
 * performed by the las_upddr() function in lieu of calling this function.
 *
 * Deprecated.  Use las_merge_ddrs() instead.
 *
 * Returns:     E_SUCC:  Successful completion of operation,
 *              E_FAIL:  Unsuccessful completion of operation.
 */
int c_merge_ddrs
(
    struct PARBLK *parblk,         /* I: TAE parameter block                 */
    const char host_in[][CMLEN],   /* I: array of input host names           */
    const char *host_out,          /* I: Output host name                    */
    const long *nmrimg,            /* I: Total number of input images        */
    const long window[][4],        /* I: Window specifications for each image*/
    const long bands[][MAXBND +1], /* I: band numbers for each input image   */
    const long nbands[],           /* I: number of bands for each input image*/
    const double *line_inc,        /* I: increment in the line direction     */
    const double *samp_inc,        /* I: increment in the sample direction   */
    const long *out_nbands,        /* I: number of output bands              */
    const long *cflag,             /* I: combination of bands flag           */
    struct BDDR **out_bddr,        /* O: output band dependent ddr records   */
    struct DDR *out_ddr,           /* O: output ddr                          */
    char upddr_flag                /* I: Flag: TRUE means called by upddr fun*/
)
{
    int status;

    status = las_merge_ddrs (parblk,host_in,host_out,*nmrimg,window,
                             bands,nbands,*line_inc,*samp_inc,*out_nbands,
                             *cflag,out_bddr,out_ddr,upddr_flag);

    return (status);

}

#endif /* obsolete interfaces */

