/******************************************************************************

NAME:   MINMAX

FUNCTION:
        MINMAX is an application callable function which is called to retreive
        or calculate the minimum and maximum pixel values of an image.  

PROGRAM HISTORY:
        Version         Date    Author          Request
        -------         ----    -----           -------
          1.0           5/86    B.Ailts         initial development
          2.0           9/86    K. Gacke        use host file names rather than
                                                tae file names
          3.0           7/88    B.Ailts         LAS 5.0 standards
                                                Added nimg and nbands as 
                                                  arguments
                                                Changed sl ss nl ss to window
          4.0           3/89    B.Ailts         Now does not return a fatal 
                                                error when the ddr could not
                                                be updated because of protection

          5.0           8/90    D. Akkerman     Added code to automatically 
                                                convert the image DDR to the 
                                                current system format if the 
                                                DDR is from an alien system.
                                                This solves the problem of 
                                                networked (NFS) DDRs that 
                                                reside on systems with 
                                                different real data formats.
          5.1           12/90   B. Ailts        Updated error messages
          5.2           10/92   T. Mittan       Malloced one more element to 
                                                cl_bands to correctly work on 
                                                MAXBND images.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
                Run under TAE

PROJECT:   LAS

ALGORITHM DESCRIPTION:

Check the DDR system fields and convert the DDRs to the local host
 format if necessary.
The space needed for the cl_bands array is dynamically allocated
The space needed for the mmbands array is dynamically allocated
For each input image
   The current image size is retreived
   The min and max arrays are initialized
   If the window specified are the full image
      CKMNMX is called to read the ddr file 
      If valid min and max values of the specified bands are not in the DDR file
         CLMNMX is called to calculate them for each band that is needed
         UPMNMX is called to update the min/max values in the DDR file
   Else
      CLMNMX is called to calculate them for each band that is needed
The memory for the cl_bands array is freed
The memory for the mmbands array is freed
These values are then passed back to the calling program.

ALGORITHM REFERENCES:  none

*****************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "sysdef.h"
#include "util.h"
#include "local_protos.h"


/******************************************************************************
 *                                                                            *
 *      Purpose:  This routine is designed to check the DDR of the specified  *
 *                image and insure that it has the same real data format as   *
 *                the current host system.  If the format is different, then  *
 *                the real data portions of the DDR are converted to the      *
 *                format of the current host system.  This allows the MINMAX  *
 *                support routine to function correctly in those instances    *
 *                when an "alien" DDR is selected for min/max update (i.e.    *
 *                NFS).                                                       *
 *                                                                            *
 *      By D. Akkerman, USGS/EROS Data Center, 8/90.                          *
 *                                                                            *
 *      Algorithm(flow):                                                      *
 *                                                                            *
 *      Create the associated DDR file name.                                  *
 *      Open the DDR for read.                                                *
 *      Read the main DDR record.                                             *
 *      Close the DDR file.                                                   *
 *      Retrieve the host system name.                                        *
 *      If the DDR system name and the host system name are not the same then *
 *       convert the DDR to the format of the host system.                    *
 *       Check the DDR file protection.                                       *
 *       Convert the first two records of the DDR.  Note that this conversion *
 *       relies on the c_getddr routine to convert the records when they are  *
 *       read.                                                                *
 *       Convert the band records of the DDR.  Note that this conversion also *
 *       relies on the c_getbdr routine to convert the records when they are  *
 *       read.                                                                *
 *       Now create a new ddr with the converted data fields.                 *
 *       Free the dynamically allocated memory.                               *
 *      Return.                                                               *
 *                                                                            *
 ******************************************************************************/
static int ddrconv
(
    const char *hostname        /* I: Host name of the image file.          */
)
{
    struct BDDR *bddr;          /* Pointer for the band data of the DDR.    */

    struct DDR ddr;             /* Main DDR record.                         */

    char  hostddr[CMLEN];       /* Host ddr name.                           */
    char  dtype[3];             /* Type of data portion of record.          */
    char insys[DDSYLN];         /* Host system name.                        */


    unsigned char *dbuf;        /* Pointer to area where data is stuffed    */

    int  mode;                  /* lsstat mode.                             */
    int  access;                /* File access type.                        */
    int  i;
    int  j;                     /* Working index variables.                 */
    int  fd;                    /* File descriptor for opened file.         */
    int  clen;                  /* Length of character part of record.      */
    int  ctemp;                 /* Temporary storage for clen.              */
    int  dlen;                  /* Length of data part of record.           */
    int  dtemp;                 /* Temporary storage for dlen.              */
    int  status;                /* Return status variable.                  */

    char key[16];

    /* Create the associated DDR file name.                                 */

    c_lsmknm(hostname,"ddr",hostddr);

    /* Open the DDR for read.                                               */

    access = 0;
    fd = c_labopen (hostddr, access);
    if (fd < 0) 
    {
        c_lasmsg (FALSE, "minmax-call", "Error returned from labopen");
        return(E_FAIL);
    }

    /* Read the main DDR record.                                            */

    clen = DDSTCT * DDSYLN;
    ctemp = clen - 1;
    dlen = dtemp = DISIZE * 4;
    dbuf = (unsigned char *)&ddr;

    strcpy(key,"DDRINT");
    if (c_labread(fd,key,&clen,&dlen,ddr.system,dbuf,dtype) != E_SUCC)
    {
        c_lasmsg (FALSE, "minmax-call", "Error returned from labread");
        return(E_FAIL);
    }

    if ((clen != ctemp) || (dlen != dtemp))
    {
        c_lasmsg (FALSE, "minmax-read",
                  "Error reading record 1 of associated DDR file");
        return(E_FAIL);
    }

    /* Close the DDR file.                                                  */

    if (c_labclose (fd,hostddr,access) != E_SUCC)
    {
        c_lasmsg (FALSE, "minmax-call", "Error returned from lsclos");
        return(E_FAIL);
    }

    /* Retrieve the host system name.                                       */

    if (c_getsys(insys) != E_SUCC)
    {
        c_lasmsg (FALSE, "minmax-call", "Error returned from getsys");
        return(E_FAIL);
    }

    /* If the DDR system name and the host system name are not the same then
       convert the DDR to the format of the host system.                    */

    if (strcmp(insys,ddr.system) != 0)
    {
        /* Check the DDR file protection.                                   */

        mode = 2;
        status = c_labstat(hostddr,mode);
        if (status != E_SUCC)
            return(E_PROT);

        /* Convert the first two records of the DDR.  Note that this 
           conversion relies on the c_getddr routine to convert the records 
           when they are read.                                              */

        status = c_getddr(hostname,&ddr);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "minmax-call", "Error returned from getddr");
            return(E_FAIL);
        }

        /* Allocate space to contain the band records.                      */

        bddr = (struct BDDR *)calloc(ddr.nbands,sizeof(struct BDDR));
        if (bddr == NULL)
        {
            c_lasmsg (FALSE, "minmax-alloc", "Error allocating dynamic memory");
            return(E_FAIL);
        }

        /* Convert the band records of the DDR.  Note that this conversion 
           also relies on the c_getbdr routine to convert the records when 
           they are read.                                                   */

        for (i = 0, j = 1; i < ddr.nbands; i++, j++)
        {
            status = las_getbdr(hostname,&bddr[i],j);
            if (status != E_SUCC)
            {
                c_lasmsg (FALSE, "minmax-call", "Error returned from getbdr");
                free (bddr);
                return(E_FAIL);
            }
        }

        /* Now create a new ddr with the converted data fields.             */

        status = c_putddr(hostname,&ddr);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "minmax-call", "Error returned from putddr");
            free (bddr);
            return(E_FAIL);
        }

        for (i = 0; i < ddr.nbands; i++)
        {
            status = c_putbdr(hostname,&bddr[i]);
            if (status != E_SUCC)
            {
                c_lasmsg (FALSE, "minmax-call", "Error returned from putbdr");
                free (bddr);
                return(E_FAIL);
            }
        }

        /* Free the dynamically allocated memory.                           */

        free (bddr);
    }

    return(E_SUCC);
}

/**
 * las_minmax:
 * @hname: I: A pointer to the host names of the input image(s).
 * @nimg: I: The number of input image(s).
 * @window: I: The window dimensions of the input image(s).
 * @bands: I: A two-dimensioned array of the specific bands for each input
 *            image that are to be calculated.  "Bands" needs to be a
 *            zero-terminated array.
 * @nbands: I: The number of bands for each of the input image(s).
 * @minval: O: A pointer to an array that will store the minimum values of each
 *             specific band of the image.
 * @maxval: O: A pointer to an array that will store the maximum values of
 *             each specific band of the image.
 *
 * The las_minmax() call is used to retrieve or calculate the minimum/maximum
 * values of nimg inpukt image(s) with host name(s) of "hname", window
 * dimensions of "window", and of bands which are specified in the array
 * "bands".  The minimum values are stored in the array "minval" and the
 * maximum values are stored in the array "maxval". 
 *
 * las_minmax() will update the minimum/maximum values in the input image(s)
 * DDR file if the window specified for that image is the full image size.
 * This is not dependen upon the value of the LAS global $MINMAX.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int las_minmax
(
    const char hname[][CMLEN],      /* I: string descriptor of host name */
    int        nimg,                /* I: total number of images         */
    int        window[][4],         /* I/O: starting line of image       */
    int        bands[][MAXBND + 1], /* I/O: array of band numbers needed */
    int        nbands,              /* I: number of bands for each image */
    double     *minval,             /* O: minimum value array            */
    double     *maxval              /* O: maximum value array            */
)

{
    struct  DDR ddr;            /*  ddr structure                           */
    int     band=0;             /*  total number of bands needed            */
    int     curbnd;             /*  current band recalculated min/max bands */
    int     i;                  /*  counter--current image                  */
    int     j;                  /*  counter--currnet band                   */
    int     k;                  /*  counter--pseudo band                    */
    int     *mmbands;           /*  bands of the min/max arrays to be used  */
    int     status;             /*  status flag                             */
    int     valid[MAXBND];      /*  valid flag of min/max array             */

    int     *cl_bands;          /*  bands that min/max has to be calculated */

    /*  Check the DDR system fields and convert the DDRs to the local host
        format if necessary.                                                */

    for (i = 0; i < nimg; i++)
    {
        status = ddrconv(hname[i]);
        if (status != E_SUCC)
            return(status);
    }

    /*  Allocate memory for cl_bands                                        */
    cl_bands = (int *)calloc(MAXBND + 1,4);
    if (cl_bands == 0)
    {
        c_lasmsg (FALSE, "minmax-alloc", "Error allocating dynamic memory");
        return(E_FAIL);
    }

    /*  Allocate memory for mmbands                                         */
    mmbands = (int *)calloc(MAXBND,4);
    if (mmbands == 0)
    {
        c_lasmsg (FALSE, "minmax-alloc", "Error allocating dynamic memory");
        free(cl_bands);
        return(E_FAIL);
    }

    for (i = 0; i < nimg; i++)
    {
        /* Retreive the full image size                                     */
        status = c_getddr(hname[i],&ddr);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "minmax-call", "Error returned from getddr");
            free(cl_bands);
            free(mmbands);
            return(E_FAIL);
        }

        /*  Initialize the min/max arrays for dtype of input image          */
        status = las_inmnmx(ddr.dtype,nbands,&minval[band],&maxval[band]);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "minmax-call", "Error returned from inmnmx");
            free(cl_bands);
            free(mmbands);
            return(E_FAIL);
        }

        if ((window[i][0] == 1) && (window[i][1] == 1) &&
                (window[i][2] == ddr.nl) && (window[i][3] == ddr.ns))
        {
            /*  Try to retreive valid min/max values from the DDR file      */
            status = ckmnmx (hname[i],bands[i],nbands,&minval[band],
                            &maxval[band], &valid[band]);
            if (status != E_SUCC)
            {
                free(cl_bands);
                free(mmbands);
                return(E_FAIL);
            }    
            /*  Create a temporary array of bands that the min/max values
                were not valid in the DDR file                              */
            for (k = 0, j = band, curbnd = 0; k < nbands; k++, j++)
                if (valid[j] != VALID)
                {
                    *(cl_bands + curbnd) = bands[i][k];
                    *(mmbands + curbnd) = k;
                    curbnd ++;
                }
            cl_bands[curbnd] = 0;

            /*  Calculate the min/max values for the specified bands        */
            status = clmnmx (hname[i],window[i],cl_bands,mmbands,&minval[band], 
                             &maxval[band]);
            if (status != E_SUCC)
            {
                free(cl_bands);
                free(mmbands);
                return(E_FAIL);
            }

            /*  Update the min/max and valid fields of the DDR file         */
            status = upmnmx (hname[i],cl_bands,mmbands,&minval[band],
                             &maxval[band]);
            if (status == E_FAIL)
            {
                free(cl_bands);
                free(mmbands);
                return(E_FAIL);
            }

        }
        else  /*  The full image was not specified  */
        {
            for (k = 0; k < nbands; k++)
            {
                *(cl_bands + k) = bands[i][k];
                *(mmbands + k) = k;
            }
            cl_bands[k] = 0;

            /*  Calculate the min/max values of the window specified        */
            status = clmnmx (hname[i],window[i],cl_bands,mmbands,
                             &minval[band], &maxval[band]);
            if (status != E_SUCC)
            {
                free(cl_bands);
                free(mmbands);
                return(E_FAIL);
            }
        }
        band += nbands;
    }

    /*  Free the space needed for the temporary array cl_bands              */
    free(cl_bands);

    /*  Free the space needed for the temporary array mmbands               */
    free(mmbands);

    return(E_SUCC);
}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_minmax:
 * @hname: I: A pointer to the host names of the input image(s).
 * @nimg: I: The number of input image(s).
 * @window: I: The window dimensions of the input image(s).
 * @bands: I: A two-dimensioned array of the specific bands for each input
 *            image that are to be calculated.  "Bands" needs to be a
 *            zero-terminated array.
 * @nbands: I: The number of bands for each of the input image(s).
 * @minval: O: A pointer to an array that will store the minimum values of each
 *             specific band of the image.
 * @maxval: O: A pointer to an array that will store the maximum values of
 *             each specific band of the image.
 *
 * The c__minmax() call is used to retrieve or calculate the minimum/maximum
 * values of nimg inpukt image(s) with host name(s) of "hname", window
 * dimensions of "window", and of bands which are specified in the array
 * "bands".  The minimum values are stored in the array "minval" and the
 * maximum values are stored in the array "maxval". 
 *
 * c_minmax() will update the minimum/maximum values in the input image(s)
 * DDR file if the window specified for that image is the full image size.
 * This is not dependent upon the value of the LAS global $MINMAX.
 *
 * Deprecated.  Use las_minmax() instead.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */

 int c_minmax
 (
    const char hname[][CMLEN],         /* I: string descriptor of host name */
    const   long  *nimg,               /* I: total number of images         */
    long    window[][4],               /* I/O: starting line of image       */
    long    bands[][MAXBND + 1],       /* I/O: array of band numbers needed   */
    const   long  *nbands,             /* I: number of bands for each imag*/
    double  *minval,                   /* O: minimum value array            */
    double  *maxval                    /* O: maximum value array            */
)
{
    int status;

    status = las_minmax (hname,*nimg,window,bands,*nbands,minval,maxval);

    return (status);

}    


#endif /* obsolete interfaces */


