/*******************************************************************************
NAME                           C_PUTBDF

PURPOSE      Place a value into a specified field of the band dependent record

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
B. Ailts              Apr. 1988         Original development
B. Ailts              Dec. 1990         Updated error messages

PROJECT      LAS

ALGORITHM 
   Based upon the input key place the input value into the correct field of 
   the BDDR structure.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "local_protos.h"

/**
 * las_putbdf:
 * @bddr: I/O: A BDDR structure.  It contains the fields of the band 
               record DDR.
 * @key: I: A constant used to designate the field of the DDR where "value"
 *          will be placed.  The constants are as follows:
 *          DDBAND:   Band number,
 *          DDMMV:    Min/max validity flag,
 *          DDMIN:    Minimum value,
 *          DDMAX:    Maximum value,
 *          DDSRC:    Source,
 *          DDINST:   Instrument,
 *          DDDIR:    Direction,
 *          DDCDAT:   Capture date,
 *          DDCTIM:   Capture time.
 * @value: I: Value that will be placed in the field of "bddr" that 
 *            corresponds to "key".  It should consist of the same
 *            data type and length as the field of "bddr" that is
 *            needed.
 *
 * The las_putbdf() call is used to place "value" in the field of "bddr"
 * designated by "key".
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */

int las_putbdf
(
    struct BDDR *bddr,         /* I/O:input band depenent record            */
    int key,                   /* I: specified key of field needed          */
    const void *value          /* I: output value of specified field        */
)

{ /*  putbdf  */

    double *dblval;            /* temporary pointer to a double field       */

    int  *longval;             /* temporary pointer to a long field         */

    char *charval;             /* temporary pointer to a char field         */
    char errtxt[ERRLEN + 1];   /* buffer space for errmsg                   */

    /*  Assign the temporary pointers the address of the output value arg   */
    dblval = (double *) value;
    longval = (int  *) value;
    charval = (char *) value;

    /*  Based upon the value of the input key return the value of the 
        corresponding field of the DDR structure.                           */
    switch (key)
    {
        case DDBAND: 
            bddr->bandno = *longval;
            break;
        case DDMMV:  
            bddr->valid = *longval;
            break;
        case DDMIN:     
            bddr->minval = *dblval;
            break;
        case DDMAX:     
            bddr->maxval = *dblval;
            break;
        case DDSRC:  
            strcpy(bddr->source,charval);
            break;
        case DDINST: 
            strcpy(bddr->instrument,charval);
            break;
        case DDDIR:     
            strcpy(bddr->direction,charval);
            break;
        case DDCDAT: 
            strcpy(bddr->date,charval);
            break;
        case DDCTIM: 
            strcpy(bddr->time,charval);
            break;
        default:        
            sprintf(errtxt,"%d is an invalid input key",key);
            c_lasmsg(FALSE,"putbdf-key",errtxt);
            return(E_FAIL);
    } /* switch (key)  */

    return(E_SUCC);
} /*  putbdf  */



#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_putbdf:
 * @bddr: I/O: A BDDR structure.  It contains the fields of the band 
               record DDR.
 * @key: I: A constant used to designate the field of the DDR where "value"
 *          will be placed.  The constants are as follows:
 *          DDBAND:   Band number,
 *          DDMMV:    Min/max validity flag,
 *          DDMIN:    Minimum value,
 *          DDMAX:    Maximum value,
 *          DDSRC:    Source,
 *          DDINST:   Instrument,
 *          DDDIR:    Direction,
 *          DDCDAT:   Capture date,
 *          DDCTIM:   Capture time.
 * @value: I: Value that will be placed in the field of "bddr" that 
 *            corresponds to "key".  It should consist of the same
 *            data type and length as the field of "bddr" that is
 *            needed.
 *
 * The c_putbdf() call is used to place "value" in the field of "bddr"
 * designated by "key".
 *
 * Deprecated.  Use las_putbdf() instead.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_putbdf
(
    struct BDDR *bddr,         /* I/O:  input band depenent record          */
    const long  *key,          /* I:  specified key of field needed         */
    const void *value          /* I:  output value of specified field       */  
)
{
    int status;

    status = las_putbdf (bddr, *key, value);

    return (status);

}

#endif /* obsolete interfaces */


