/*******************************************************************************
NAME                         C_PUTBDR

PURPOSE      Output specified band dependent record of a DDR file

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
K. Zanter             Feb. 1988         original development
B. Ailts              Apr. 1988         Replaced newlas.h with las.h
                                        Changed the calling sequence to use only
                                          one data structure -- BDDR
                                        Changed the return status codes of
                                          the ls___ calls to E_SUCC
D. Akkerman           Jan. 1990         Installed error checking for an
                                        incorrect band number.
B. Ailts              Dec. 1990         Updated error messages

PROJECT      LAS

ALGORITHM:
   Ensure that a valid band number was specified.
   Create the associated DDR file name from the input host image name.
   Open the DDR file for write access using label services.  
   Set up for and write the specified record.  
   Close the DDR file.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "local_protos.h"

/**
 * c_putbdr:
 * @hname: I: Image host file name for which the information is to be
 *            written.
 * @bddr: I: A BDDR structure.  It contains the fields of the band 
 *           record of the DDR.
 *
 * The c_putbdr() call is used to output the specified band-dependent
 * record of DDR information of image hname.
 *
 * The band field of the bddr structure must contain the band number of
 * the DDR record that this structure will be written to.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_putbdr
(
    const char *hname,       /* I: input host image name                    */
    const struct BDDR *bddr  /* I: band data for the DDR                    */
)

{
    int  access;             /* file access type                            */
    int  action;             /* file close action                           */
    int  clen;               /* length of char part of record               */
    int  dlen;               /* length of data part of record               */
    int  fd;                 /* file descriptor                             */ 

    unsigned char *dbuf;     /* pointer to area where data is stuffed       */

    char hostddr[CMLEN];     /* host name of DDR file                       */
    char key[16];            /* record key                                  */
    char tempchar[DDTSLN];   /* temp. storage of char. portion or record    */
    char *tempptr;

    /* Ensure that a valid band number was specified.                       */

    if ((bddr->bandno < 1) || (bddr->bandno > MAXBND))
    {
        c_lasmsg (FALSE, "putbdr-badnum",
                  "Incorrect band number specified for BDDR");
        return(E_FAIL);
    }

    c_lsmknm(hname,"ddr",hostddr);

    /* Open DDR file for write access                                       */
    access = 2;                          
    fd = c_labopen (hostddr, access);
    if (fd < 0) 
    {
        c_lasmsg (FALSE, "putbdr-call", "Error returned from labopen");
        return(E_FAIL);
    }

    /* Initialize the variables needed the write the band record            */
    clen = DDTSLN - 1;
    dlen = DBSIZE * 8;
    dbuf = (unsigned char *) &(bddr->minval);
    sprintf(key,"BAND%d",bddr->bandno);

    /*  Place structure members into a character buffer                     */
    sprintf(tempchar,"%4d%2d",bddr->bandno,bddr->valid);
    tempptr = tempchar + DDBNLN + DDVLLN;
    strcpy(tempptr,bddr->source);
    tempptr += DDSRLN;
    strcpy(tempptr,bddr->instrument);
    tempptr += DDINLN;
    strcpy(tempptr,bddr->direction);
    tempptr += DDDRLN;
    strcpy(tempptr,bddr->date);
    tempptr += DDCDLN;
    strcpy(tempptr,bddr->time);

    if (c_labwrite (fd,key,clen,dlen,tempchar,(char *)dbuf,"R8") != E_SUCC)
    {
        c_lasmsg (FALSE, "putbdr-call", "Error returned from labwrite");
        return(E_FAIL);
    }

    action = 0;                           /* close associated DDR file      */
    if (c_labclose (fd,hostddr,action) != E_SUCC)
    {
        c_lasmsg (FALSE, "putbdr-call", "Error returned from lsclos");
        return(E_FAIL);
    }

    return(E_SUCC);
}
