/*******************************************************************************
NAME                           C_PUTDDF

PURPOSE      Place a value in a specified field of the band independent record

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
B. Ailts              Apr. 1988         Original development
B. Ailts              Dec. 1990         Updated error messages

PROJECT      LAS

ALGORITHM 
   Based upon the input key, place a value into the correct field of the 
   DDR structure.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include "las.h"

/**
 * las_putddf:
 * @ddr: I/O: A DDR structure.  It contains the fields of the band 
 *            record for the DDR.
 * @key: I: A constant used to designate the field of the DDR where "value"
 *          will be placed.  The constants are as follows:
 *          DDNL:   Number of lines,
 *          DDNS:   Number of samples,
 *          DDDTYP: Data type,
 *          DDML:   Master line,
 *          DDMS:   Master sample,
 *          DDVFLG: Validity flag array,
 *          DDPCOD: Projection code,
 *          DDSYS:  System,
 *          DDUNIT: Projection Unit,
 *          DDLDAT: Last used date,
 *          DDLTIM: Last used time,
 *          DDPCF:  Projection coefficients,
 *          DDUL:   Upper left,
 *          DDLL:   Lower left,
 *          DDLR:   Lower right,
 *          DDUR:   Upper right,
 *          DDPDY:  Projection distance-y,
 *          DDPDX:  Projection distance-x,
 *          DDLINC: Line increment,
 *          DDSINC: Sample increment.
 * @value: I: Value that will be placed in the field of "ddr" that
 *            corresponds to "key".  It should consist of the same data
 *            type and length as the field of "ddr" that is needed.
 *
 * The las_putddf() call is used to place "value" in the field of "ddr"
 * designated by "key".
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int las_putddf
(
    struct DDR *ddr,           /* I/O: input band dependent record          */
    int key,                   /* I: specified key of field needed          */
    const void *value          /* I: output value of specified field        */
)

{ /*  putddf  */

    double *dblval;            /* temporary pointer to a double field       */

    int  i;                    /* counter                                   */
    int  *longval;             /* temporary pointer to a long field         */

    char *charval;             /* temporary pointer to a char field         */
    char errtxt[ERRLEN + 1];   /* buffer space for err message              */

    /*  Assign the temporary pointers the address of the output value arg   */
    dblval = (double *) value;
    longval = (int  *) value;
    charval = (char *) value;

    /*  Based upon the value of the input key return the value of the 
        corresponding field of the DDR structure.                           */
    switch (key)
    {
        case DDNL:
            ddr->nl = *longval;
            break;
        case DDNS:      
            ddr->ns = *longval;
            break;
        case DDNB:      
            ddr->nbands = *longval;
            break;
        case DDDTYP: 
            ddr->dtype = *longval;
            break;
        case DDML:      
            ddr->master_line = *longval;
            break;
        case DDMS:      
            ddr->master_sample = *longval;
            break;
        case DDVFLG: 
            for (i = 0; i < DDNVAL; i++)
                ddr->valid[i] = *(longval + i);
            break;
        case DDPCOD:    
            ddr->proj_code = *longval;
            break;
        case DDZCOD:    
            ddr->zone_code = *longval;
            break;
        case DDDCOD:    
            ddr->datum_code = *longval;
            break;
        case DDSYS:     
            strcpy( ddr->system , charval);
            break;
        case DDUNIT:    
            strcpy( ddr->proj_units , charval);
            break;
        case DDLDAT: 
            strcpy( ddr->last_used_date , charval);
            break;
        case DDLTIM: 
            strcpy( ddr->last_used_time , charval);
            break;
        case DDPCF:     
            for (i = 0; i < 15; i++)
                ddr->proj_coef[i] = *(dblval + i);
            break;
        case DDUL:      
            ddr->upleft[0] = *dblval;
            ddr->upleft[1] = *(dblval + 1);
            break;
        case DDLL:      
            ddr->loleft[0] = *dblval;
            ddr->loleft[1] = *(dblval + 1);
            break;
        case DDUR:      
            ddr->upright[0] = *dblval;
            ddr->upright[1] = *(dblval + 1);
            break;
        case DDLR:      
            ddr->loright[0] = *dblval;
            ddr->loright[1] = *(dblval + 1);
            break;
        case DDPDY:     
            ddr->pdist_y = *dblval;
            break; 
        case DDPDX:     
            ddr->pdist_x = *dblval;
            break;
        case DDLINC:    
            ddr->line_inc = *dblval; 
            break;
        case DDSINC:    
            ddr->sample_inc = *dblval; 
            break;
        default:        
            sprintf(errtxt,"%d is an invalid input key",key);
            c_lasmsg(FALSE,"putddf-key",errtxt);
            return(E_FAIL);
    } /* switch (*key)  */

    return(E_SUCC);
} /*  putddf  */




#ifndef NO_OBSOLETE_INTERFACE


/**
 * c_putddf:
 * @ddr: I/O: A DDR structure.  It contains the fields of the band 
 *            record for the DDR.
 * @key: I: A constant used to designate the field of the DDR where "value"
 *          will be placed.  The constants are as follows:
 *          DDNL:   Number of lines,
 *          DDNS:   Number of samples,
 *          DDDTYP: Data type,
 *          DDML:   Master line,
 *          DDMS:   Master sample,
 *          DDVFLG: Validity flag array,
 *          DDPCOD: Projection code,
 *          DDSYS:  System,
 *          DDUNIT: Projection Unit,
 *          DDLDAT: Last used date,
 *          DDLTIM: Last used time,
 *          DDPCF:  Projection coefficients,
 *          DDUL:   Upper left,
 *          DDLL:   Lower left,
 *          DDLR:   Lower right,
 *          DDUR:   Upper right,
 *          DDPDY:  Projection distance-y,
 *          DDPDX:  Projection distance-x,
 *          DDLINC: Line increment,
 *          DDSINC: Sample increment.
 * @value: I: Value that will be placed in the field of "ddr" that
 *            corresponds to "key".  It should consist of the same data
 *            type and length as the field of "ddr" that is needed.
 *
 * The c_putddf() call is used to place "value" in the field of "ddr"
 * designated by "key".
 *
 * Deprecated.  Use las_putddf() instead.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_putddf
(
    struct DDR *ddr,           /* I/O: input band depenent record           */
    const long *key,           /* I: specified key of field needed          */
    const void *value          /* I: output value of specified field        */
)
{
    int status;

    status = las_putddf (ddr,*key,value);

    return (status);

}


#endif /* obsolete interfaces */

