/*******************************************************************************
NAME                          C_PUTDDR

PURPOSE      Output records 1 and 2 of a DDR file

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
K. Zanter             Jan. 1988         Original development
B. Ailts              Apr. 1988         Replaced newlas.h with las.h
                                        Changed the calling sequence -- only
                                          one structure
                                        Replaced the ls___ status checks with
                                          the constat E_SUCC
                                        Replaced cerrmsg with c_errmsg
D. Akkerman           Jan. 1990         Installed code to ensure that
                                        required parameters are specified.
B. Ailts              Dec. 1990         Updeated error messages

PROJECT       LAS

ALGORITHM 
   Ensure that required parameters were specified.
   Create the associated DDR file name from the input host image name.
   Open the DDR file for write access using label services.  
   Write the DDRINT record (first record) 
   Write the DDRDUB record (second record).  
   Close the DDR file.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "diskio.h"
#include "labserv.h"
#include "local_protos.h"


/**
 * c_putddr:
 * @hname: I: Image host file name for which information is to be written.
 * @ddr: I: A DDR structure. It contains the first two records of the DDR.
 *
 * The c_putddr() call is used to output the first two records of DDR
 * information of image "hname".
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_putddr
(
    const char *hname,          /* I: host image file name                  */
    const struct DDR *ddr       /* I: ddr data in record 1 and 2            */
)

{
    int    access;              /* file access type                         */
    int    action;              /* file close action                        */
    int    clen;                /* length of char part of record            */
    int    dlen;                /* length of data part of record            */
    int    fd;                  /* file descriptor                          */
 
    unsigned char *dbuf;        /* pointer to area where data is stuffed    */

    char d_temp[DDSTCT][DDSYLN];/* temporary for squeezed strings           */
    char *temp;                 /* temporary for squeezed strings           */
    char hostddr[CMLEN];        /* host name of DDR file                    */
    char *junk_temp;

    /* Ensure that required parameters were specified.                      */

    if (ddr->nl < 1)
    {
        c_lasmsg (FALSE, "putddr-badnl", "Invalid number of lines specified");
        return(E_FAIL);
    }

    if (ddr->ns < 1)
    {
        c_lasmsg (FALSE, "putddr-badns", "Invalid number of samples specified");
        return(E_FAIL);
    }

    if ((ddr->nbands < 1) || (ddr->nbands > MAXBND))
    {
        c_lasmsg (FALSE, "putddr-badbnds", "Invalid number of bands specified");
        return(E_FAIL);
    }

    if ((ddr->dtype < 1) || (ddr->dtype > 4))
    {
        c_lasmsg (FALSE, "putddr_bdtype", "Invalid data type specified");
        return(E_FAIL);
    }

    if ((strcmp(ddr->system,"")) == 0)
    {
        c_lasmsg (FALSE, "putddr-badsys", "Bad system specified");
        return(E_FAIL);
    }

    c_lsmknm(hname,"ddr",hostddr);

    access = 1;                           /* open DDR file for write access */
    fd = c_labopen (hostddr, access);
    if (fd < 0)           
    {
        c_lasmsg (FALSE, "putddr-call", "Error returned from lsopen");
        return(E_FAIL);
    }

    /*  Place the string portion of the DDR into a temporary buffer         */
    junk_temp = (char *)&ddr->spare;
    strncpy(junk_temp,ddr->system,4);
    temp = squeeze(ddr->system,DDSYLN);
    strcpy(d_temp[0],temp);
    free(temp);
    temp = squeeze(ddr->proj_units,DDPULN);
    strcpy(d_temp[1],temp);
    free(temp);
    temp = squeeze(ddr->last_used_date,DDLDLN);
    strcpy(d_temp[2],temp);
    free(temp);
    temp = squeeze(ddr->last_used_time,DDLTLN);
    strcpy(d_temp[3],temp);
    free(temp);

    clen = DDSTCT * DDSYLN - 1;           /* set up and output record 1     */
    dlen = DISIZE * 4;
    dbuf = (unsigned char *) ddr;
    if (c_labwrite (fd,"DDRINT",clen,dlen,d_temp[0],(char *)dbuf,"I4") 
        != E_SUCC)
    {
        c_lasmsg (FALSE, "putddr-call", "Error returned from lswrit");
        return(E_FAIL);
    }

    /* Set up and output record 2. There is no character part to this record*/
    clen = 0;                             
    dlen = DDSIZE * 8;            
    dbuf = (unsigned char *) &(ddr->proj_coef[0]);
    if (c_labwrite (fd,"DDRDUB",clen,dlen,d_temp[0],(char *)dbuf,"R8") 
        != E_SUCC)
    {
        c_lasmsg (FALSE, "putddr-call", "Error returned from lswrit");
        return (E_FAIL);
    }

    action = 0;                           /* close associated DDR file      */
    if (c_labclose (fd,hostddr,action) != E_SUCC)   
    {
        c_lasmsg (FALSE, "putddr-call", "Error returned from labclose");
        return(E_FAIL);
    }
 
    return(E_SUCC);
}
