/*****************************************************************************

NAME:        c_recalc_pixel_size

PURPOSE:     Recalculate the pixel distance of an image before it was rotated
             or sheared. 

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
$log$

PROJECT:     LAS

NOTES:  

A = angle_A
B = angle_B

(y1,x1) = (upleft_y, upleft_x)
(y0,x0) = (loleft_y, loleft_x)
(y2,x2) = (loright_y, loright_x)


Example of rotated image:

NOTE: Y and X axes are perpendicular to each other in the rotated image

                   image axis y
                      (Line)
             Y         /|\         X
              \         |         /
               \        |        /<-- projection
                \       |(y1,x1)/     on x-axis 
                 \------o------/                 
                  \     |     /|  
                   \    |    / |
    projection ---> \   |   /  |<--- image (before rotation)
    on y-axis        \->|  /   |
                      \A| /B)  |(y2,x2)
               <--------o------o---> image axis x (Sample) 
                       /|\(y0,x0) 
                      / | \ 
                     /  |  \
                    /   |   \
                       \|/  


Example of sheared image:

                  image axis y
                     (Line)
                      /|\        Y 
                       |        / <----projection on y-axis                  
                       |       /(y1,x1)
                       |      o--------------/
                       |     /              /
                       |    /              /
                       |   /              / 
                       |<-/              /
                       |A/              /
                       |/              /(y2,x2)
                       o--------------o-------> image axis x (Sample) and
                      /|(y0,x0)                 projection on x-axis
                     / |
                    /  |
                      \|/


LIMITATIONS: This routine requires that the image has more than one line
             and sample.
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"

/**
 * c_recalc_pixel_size:
 * @pixel_y: I/O: Pixel size in the y direction.
 * @pixel_x: I/O: Pixel size in the x direction.
 * @upleft_y: I: Upper left corner of image in the y direction.
 * @loleft_y: I: Lower left corner of image in the y direction.
 * @loright_y: I: Lower right corner of image in the y direction.
 * @upleft_x: I:  Upper left corner of image in the x direction.
 * @loleft_x: I:  Lower left corner of image in the x direction.
 * @loright_x: I: Lower right corner of image in the x direction.
 * @nl: I: Total number of lines in the image.
 * @ns: I: Total number of samples in the image.
 *
 * The c_recalc_pixel_size() routine recalculates the pixel distance 
 * of an image before it was rotated or sheared.  It requires that the
 * image has more than one line and sample.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_recalc_pixel_size
(
    double *pixel_y,       /* I/O: Pixel size in the y direction            */
    double *pixel_x,       /* I/O: Pixel size in the x direction            */
    double upleft_y,       /* I: Upper left corner of image in y direction  */
    double loleft_y,       /* I: Lower left corner of image in y direction  */
    double loright_y,      /* I: Lower right corner of image in y direction */
    double upleft_x,       /* I: Upper left corner of image in x direction  */
    double loleft_x,       /* I: Lower left corner of image in x direction  */
    double loright_x,      /* I: Lower right corner of image in x direction */
    int nl,                /* I: Total number of line in the image          */
    int ns                 /* I: Total number of samples in the image       */
)

{
    double angle_A;              /* Angle of rotation from the unrotated 
                                    y axis                                  */
    double angle_B;              /* Angle of rotation from the unrotated
                                    x axis                                  */
    double projection_on_y_axis; /* y axis of the projected image           */
    double projection_on_x_axis; /* x axis of the projected image           */
    double sin_A;                /* sin value of angle_A                    */

    int switch_y_x_axis = FALSE; /* If TRUE calculate as if the y and x axis
                                    are flipped                             */
                                
    /* Make sure the number of lines and samples are greater than 1         */
    if (nl <= 1) 
    {
        c_lasmsg(FALSE, "recalc-pixel", "The number of lines %ld, "
                 "cannot be less than or equal to 1",nl);
        return (E_FAIL);
    }

    if (ns <= 1)
    {
        c_lasmsg(FALSE, "recalc-pixel", "The number of samples %ld, "
                 "cannot be less than or equal to 1",ns);
        return (E_FAIL);
    }

    /* Calculate the rotated angles of the projected image                  */
    angle_A = atan2((upleft_x - loleft_x), (upleft_y - loleft_y));
    angle_B = atan2((loright_y - loleft_y), (loright_x - loleft_x)); 


    /* Check the sine of angle_A. If the value is 0, then the image 
       is rotated by either 90 or 270 degrees.  The y and x axis of 
       the image are "switched".                                            */
    sin_A = sin(angle_A);

    if (fabs(sin_A) >= 0.5)
        switch_y_x_axis = TRUE;

    if (switch_y_x_axis != TRUE)
    {
        /* Calculate the projection of the y and x axis                     */
        projection_on_y_axis = (upleft_y - loleft_y) / cos(angle_A);
        projection_on_x_axis = (loright_x - loleft_x) / cos(angle_B);      

        /*Calculate the pixel size for y and x                              */
        *pixel_y = fabs(projection_on_y_axis / (nl - 1));
        *pixel_x = fabs(projection_on_x_axis / (ns - 1));
    }
    else
    {
        /* Calculate the projection of the y and x axis                     */
        projection_on_y_axis = (loright_y - loleft_y) / sin(angle_B);
        projection_on_x_axis = (upleft_x - loleft_x) / sin(angle_A);

        /* Calculate the pixel size for y and x                             */
        *pixel_y = fabs(projection_on_y_axis / (ns - 1));
        *pixel_x = fabs(projection_on_x_axis / (nl - 1));   
    }

    return (E_SUCC);
}

