/*****************************************************************************
NAME:                           C_RTIDDR

PURPOSE:        Updates the projection information in a DDR structure for
                rotations of 90 degree increments (ie. 90, 180, 270) and
                for horizontal and vertical flips.

PROGRAM HISTORY:
        By: D. Akkerman, USGS/EROS Data Center, 3/89
        By: B. Ailts, USGS/EROS Data Center, 12/90, Updated error messages

PROJECT:        LAS

ALGORITHM:

        Modify the DDR structure according to the input rotation value.

        If a 90 degree rotation:
         Rotate the corner coordinates 90 degrees.
         Swap the projection distance Y and X values.
         Swap the line and sample increment values.
         Swap the master line and sample values.

        Else if a 180 degree rotation:
         Rotate the corner coordinates 180 degrees.

        Else if a 270 degree rotation:
         Rotate the corner coordinates 90 degrees.
         Swap the projection distance Y and X values.
         Swap the line and sample increment values.
         Swap the master line and sample values.

        Else if a horizontal flip:
         Transpose the corner coordinates horizontally.

        Else if a vertical flip:
         Transpose the corner coordinates vertically.

        Else
         Output an error message.
         Return a bad status.

        Return.

*****************************************************************************/
#define Y       0       /* Y coordinate.                                    */
#define X       1       /* X coordinate.                                    */

#include "las.h"

/**
 * las_rtiddr:
 * @inddr: I/O: Input DDR structure to be updated.
 * @rot: I: Rotation amount in 90 degree increments.
 *          1:  90 degrees,
 *          2:  180 degrees,
 *          3:  270 degrees,
 *          4:  Horizontal flip,
 *          5:  Vertical flip.
 *
 * The las_rtiddr() call is used to update the projection information in a
 * DDR structure for rotations of 90 degree increments (ie. 90, 180, 270) and
 * for horizontal and vertical flips.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int las_rtiddr
(
    struct DDR *inddr,  /* I/O: Input DDR structure to be updated.          */
    int rot             /* I:  Rotation amount in 90 degree increments:
                                        1 =  90 degrees
                                        2 = 180 degrees
                                        3 = 270 degrees
                                        4 = Horizontal flip
                                        5 = Vertical flip                   */
)

{
    double  upright[2]; /* DDR corner coordinates.                          */
    double  loright[2];
    double  loleft[2];
    double  upleft[2];
    double  pdist_y;    /* Projection distance/pixel for Y.                 */
    double  pdist_x;    /* Projection distance/pixel for X.                 */
    double  line_inc;   /* DDR line increment for sampling.                 */
    double  sample_inc; /* DDR sample increment for sampling.               */

    int  master_line;   /* DDR line relative to master image.               */
    int  master_sample; /* DDR sample relative to master image.             */



    /* Modify the DDR structure according to the input rotation value.      */

    if (rot == 1)
    {
        /* Rotate the corner coordinates 90 degrees.                        */ 
        upright[Y] = inddr->upright[Y];
        upright[X] = inddr->upright[X];
        loright[Y] = inddr->loright[Y];
        loright[X] = inddr->loright[X];
        loleft[Y]  = inddr->loleft[Y];
        loleft[X]  = inddr->loleft[X];
        upleft[Y]  = inddr->upleft[Y];
        upleft[X]  = inddr->upleft[X];

        inddr->upright[Y] = upleft[Y];
        inddr->upright[X] = upleft[X];
        inddr->loright[Y] = upright[Y];
        inddr->loright[X] = upright[X];
        inddr->loleft[Y]  = loright[Y];
        inddr->loleft[X]  = loright[X];
        inddr->upleft[Y]  = loleft[Y];
        inddr->upleft[X]  = loleft[X];

        /* Swap the projection distance Y and X values.                     */

        pdist_y = inddr->pdist_y;
        pdist_x = inddr->pdist_x;

        inddr->pdist_y = pdist_x;
        inddr->pdist_x = pdist_y;

        /* Swap the line and sample increment values.                       */

        line_inc   = inddr->line_inc;
        sample_inc = inddr->sample_inc;

        inddr->line_inc   = sample_inc;
        inddr->sample_inc = line_inc;

        /* Swap the master line and sample values.                          */

        master_line   = inddr->master_line;
        master_sample = inddr->master_sample;

        inddr->master_line   = master_sample;
        inddr->master_sample = master_line;
    }

    else if (rot == 2)
    {
        /* Rotate the corner coordinates 180 degrees.                       */

        upright[Y] = inddr->upright[Y];
        upright[X] = inddr->upright[X];
        loright[Y] = inddr->loright[Y];
        loright[X] = inddr->loright[X];
        loleft[Y]  = inddr->loleft[Y];
        loleft[X]  = inddr->loleft[X];
        upleft[Y]  = inddr->upleft[Y];
        upleft[X]  = inddr->upleft[X];

        inddr->upright[Y] = loleft[Y];
        inddr->upright[X] = loleft[X];
        inddr->loright[Y] = upleft[Y];
        inddr->loright[X] = upleft[X];
        inddr->loleft[Y]  = upright[Y];
        inddr->loleft[X]  = upright[X];
        inddr->upleft[Y]  = loright[Y];
        inddr->upleft[X]  = loright[X];
    }

    else if (rot == 3)
    {
        /* Rotate the corner coordinates 270 degrees.                       */

        upright[Y] = inddr->upright[Y];
        upright[X] = inddr->upright[X];
        loright[Y] = inddr->loright[Y];
        loright[X] = inddr->loright[X];
        loleft[Y]  = inddr->loleft[Y];
        loleft[X]  = inddr->loleft[X];
        upleft[Y]  = inddr->upleft[Y];
        upleft[X]  = inddr->upleft[X];

        inddr->upright[Y] = loright[Y];
        inddr->upright[X] = loright[X];
        inddr->loright[Y] = loleft[Y];
        inddr->loright[X] = loleft[X];
        inddr->loleft[Y]  = upleft[Y];
        inddr->loleft[X]  = upleft[X];
        inddr->upleft[Y]  = upright[Y];
        inddr->upleft[X]  = upright[X];

        /* Swap the projection distance Y and X values.                     */

        pdist_y = inddr->pdist_y;
        pdist_x = inddr->pdist_x;

        inddr->pdist_y = pdist_x;
        inddr->pdist_x = pdist_y;

        /* Swap the line and sample increment values.                       */

        line_inc   = inddr->line_inc;
        sample_inc = inddr->sample_inc;

        inddr->line_inc   = sample_inc;
        inddr->sample_inc = line_inc;

        /* Swap the master line and sample values.                          */

        master_line   = inddr->master_line;
        master_sample = inddr->master_sample;

        inddr->master_line   = master_sample;
        inddr->master_sample = master_line;
    }

    else if (rot == 4)
    {
        /* Transpose the corner coordinates horizontally.                   */

        upright[Y] = inddr->upright[Y];
        upright[X] = inddr->upright[X];
        loright[Y] = inddr->loright[Y];
        loright[X] = inddr->loright[X];
        loleft[Y]  = inddr->loleft[Y];
        loleft[X]  = inddr->loleft[X];
        upleft[Y]  = inddr->upleft[Y];
        upleft[X]  = inddr->upleft[X];

        inddr->upright[Y] = upleft[Y];
        inddr->upright[X] = upleft[X];
        inddr->loright[Y] = loleft[Y];
        inddr->loright[X] = loleft[X];
        inddr->loleft[Y]  = loright[Y];
        inddr->loleft[X]  = loright[X];
        inddr->upleft[Y]  = upright[Y];
        inddr->upleft[X]  = upright[X];
    }

    else if (rot == 5)
    {
        /* Transpose the the corner coordinates vertically.                 */

        upright[Y] = inddr->upright[Y];
        upright[X] = inddr->upright[X];
        loright[Y] = inddr->loright[Y];
        loright[X] = inddr->loright[X];
        loleft[Y]  = inddr->loleft[Y];
        loleft[X]  = inddr->loleft[X];
        upleft[Y]  = inddr->upleft[Y];
        upleft[X]  = inddr->upleft[X];

        inddr->upright[Y] = loright[Y];
        inddr->upright[X] = loright[X];
        inddr->loright[Y] = upright[Y];
        inddr->loright[X] = upright[X];
        inddr->loleft[Y]  = upleft[Y];
        inddr->loleft[X]  = upleft[X];
        inddr->upleft[Y]  = loleft[Y];
        inddr->upleft[X]  = loleft[X];
    }

    else
    {
        /* An error condition exists; output an error message and return a
           bad status.                                                      */

        c_lasmsg(FALSE,"rtiddr-badrot","Invalid rotation angle");
        return(E_FAIL);
    }

    return(E_SUCC);
}



#ifndef NO_OBSOLETE_INTERFACE


/**
 * c_rtiddr:
 * @inddr: I/O: Input DDR structure to be updated.
 * @rot: I: Rotation amount in 90 degree increments.
 *          1:  90 degrees,
 *          2:  180 degrees,
 *          3:  270 degrees,
 *          4:  Horizontal flip,
 *          5:  Vertical flip.
 *
 * The c_rtiddr() call is used to update the projection information in a
 * DDR structure for rotations of 90 degree increments (ie. 90, 180, 270) and
 * for horizontal and vertical flips.
 *
 * Deprecated.  Use las_rtiddr() instead.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_rtiddr
(
    struct DDR *inddr,  /* I/O:  Input DDR structure to be updated.         */
    const long *rot     /* I:  Rotation amount in 90 degree increments:
                                        1 =  90 degrees
                                        2 = 180 degrees
                                        3 = 270 degrees
                                        4 = Horizontal flip
                                        5 = Vertical flip                   */
)
{
    int status;

    status = las_rtiddr (inddr,*rot);

    return (status);

}



#endif /* obsolete interfaces */


