/*******************************************************************************
NAME                           C_SVMNMX

PURPOSE      Output minimum and maximum intensity values and a
             corresponding validity flag for each band of an 
             output image to the associated DDR file.

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
K. Zanter             Feb. 1988         original development
B. Ailts              Apr. 1988         Replaced newlas.h with las.h
                                        Replaced return status codes of the
                                          ls___ calls with E_SUCC
                                        Replaced cerrmsg with c_errmsg
B.Ailts               Jul. 1988         Fixed updating of validity flag
B.Ailts               Sep. 1988         Fixed how the validity flag was updated
B.Ailts               Dec. 1988         If the min/max values are outside of the
                                        min/max range of the output data type,
                                        set min/max values to the min and max
                                        values of the data type
B.Ailts               Dec. 1990         Updated error messages

PROJECT         LAS

ALGORITHM:
   Create the associated DDR file name from the input host image name.
   Check for write access to the DDR file.  
   Open the DDR file for update access using label services.  
   Convert the flag string to INVAL(0) , VALID(1), or BOUND(2).
   For each output band 
       read in the specified record 
       change the min, max, and validity flag with the appropriate value
       replace the record
   Close the DDR file.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "typlim.h"
#include "util.h"
#include "local_protos.h"

/**
 * las_svmnmx:
 * @hname: I: Input host file name for which information is to be written.
 * @totbnd: I: Total number of bands in the output image.
 * @minval: I: Array of minimum intensity values.
 * @maxval: I: Array of maximum intensity values.
 * @m_flag: I: A flag ("YES", "NO", or "BOUNDED") indicating whether or not
 *             the min and max values were calculated/estimated.  An
 *             application program can pass the value retrieved from the
 *             $MINMAX global ("YES" the minmax was calculated or "NO" it 
 *             was not calculated).  An application program can pass the 
 *             value of "BOUNDED" when estimated values for minval are to be
 *             calculated.
 *
 * The las_svmnmx() call is used to output the specified minimum, maximum, and
 * validity flag values for each band record of the DDR file associated with
 * image "hname".
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation,
 *           E_PROT:  DDR is write protected, no error message is produced
 *                    from this error.
 */
int las_svmnmx
(
    const char *hname,     /* I: input host image name                      */
    int    totbnd,         /* I: number of bands to be processed            */
    const double minval[], /* I: minimum intensity values                   */
    const double maxval[], /* I: maximum intensity values                   */
    const char *m_flag     /* I: min/max validity flag ("YES"/"NO"/"BOUND") */
                           /*    from $MINMAX global                        */
)

{
    struct BDDR bddr;      /* band data for the DDR                         */

    int  access;           /* file access type                              */
    int  action;           /* file close action flag                        */
    int  clen;             /* length of character part of record            */
    int  ctemp;            /* temporary storage for clen                    */
    int  dlen;             /* length of data part of record                 */
    int  dtemp;            /* temporary storage for dlen                    */
    int  fd;               /* file descriptor for opened file               */
    int  i;                /* loop index                                    */
    int  idtype;           /* data type of input image                      */
    int  mode;             /* lsstat mode                                   */
    int  one = 1;          /* errmsg status code                            */
    int  status;           /* function return status code                   */
    int  v_flag;           /* validity flag in char form                    */

    unsigned char *dbuf;   /* pointer to area where data is stuffed         */

    char dtype[3];         /* type of data portion of record                */
    char hostddr[CMLEN];   /* host name of DDR file                         */
    char key[16];          /* record key                                    */
    char stemp[DDSRLN];    /* temporary storage of source                   */
    char tempchar[DDTSLN]; /* temporary char. portion of record             */
    char t_hname[1][CMLEN];/* temporary array to meet param. reqmt.         */

    strcpy(t_hname[0],hname);
    status = las_gettyp(t_hname,one,&idtype);
    if (status != E_SUCC)
    {
        c_lasmsg(FALSE,"svmnmx-call","Error returned from gettyp");
        return(E_FAIL);
    }
    c_lsmknm(hname,"ddr",hostddr);    /* make associated DDR file name   */

    mode = 2;                         /* check DDR file protection       */
    status = c_labstat(hostddr,mode);
    if (status != E_SUCC)
        return(E_PROT);

    access = 3;                       /* open DDR file for read/write    */
    fd = c_labopen(hostddr, access);
    if (fd < 0) 
    {
        c_lasmsg(FALSE,"svmnmx-call","Error returned from lsopen");
        return(E_FAIL);
    }

    /*  set output minmax validity flag */
    /*  only upper case string is possible */
    if (strcmp(m_flag,"YES") == 0)
        v_flag = VALID;
    else if (strcmp(m_flag,"BOUNDED") == 0)
        v_flag = BOUND;
    else
        v_flag = INVAL;

    for (i = 0; i < totbnd; i++)
    {
        /* read in current band dependent record */
        sprintf(key,"BAND%d",i+1);
        clen = DDTSLN;
        ctemp = clen - 1;
        dlen = dtemp = DBSIZE * 8;
        dbuf = (unsigned char *) &(bddr.minval);
        status = c_labread(fd,key,&clen,&dlen,tempchar,dbuf,dtype);
        if (status != E_SUCC)
        {
            c_lasmsg(FALSE,"svmnmx-call","Error returned from labread");
            return(E_FAIL);
        }

        if ( (clen != ctemp) || (dlen != dtemp) )
        {
            c_lasmsg(FALSE,"svmnmx-read", "Error reading the DDR file");
            return(E_FAIL);
        }

        /* update with new min, max, and validity flag */
        if (strcmp(m_flag,"NO") == 0)
        {
            bddr.minval = 0.0;
            bddr.maxval = 0.0;
        }
        else
        {
            /*  If the min/max values are outside of the range of the 
                min/max values of the image data type, set the min/max 
                values of the DDR to the min/max values of the image 
                data type                                                   */
            switch (idtype)
            {
                case EBYTE: 
                    if (minval[i] < MINUTINY)
                        bddr.minval = MINUTINY;
                    else
                        bddr.minval = (double)(unsigned char)minval[i];

                    if (maxval[i] > MAXUTINY)
                        bddr.maxval = MAXUTINY;
                    else
                        bddr.maxval = (double)(unsigned char)maxval[i];
                    break;
                case EWORD: 
                    if (minval[i] < MINWORD)
                        bddr.minval = MINWORD;
                    else
                        bddr.minval = (double)(short)minval[i];

                    if (maxval[i] > MAXWORD)
                        bddr.maxval = MAXWORD;
                    else
                        bddr.maxval = (double)(short)maxval[i];
                    break;
                case ELONG: 
                    if (minval[i] < MINLONG)
                        bddr.minval = MINLONG;
                    else
                        bddr.minval = (double)(long)minval[i];

                    if (maxval[i] > MAXLONG)
                        bddr.maxval = MAXLONG;
                    else
                        bddr.maxval = (double)(long)maxval[i];
                    break;
                case EREAL: 
                    if (minval[i] < MINREAL)
                        bddr.minval = MINREAL;
                    else
                        bddr.minval = (double)(float)minval[i];

                    if (maxval[i] > MAXREAL)
                        bddr.maxval = MAXREAL;
                    else
                        bddr.maxval = (double)(float)maxval[i];
                    break;
            }  /*  switch idtype  */
        }

        strcpy (stemp,(tempchar + DDBNLN + DDVLLN));
        sprintf((tempchar + DDBNLN),"%2d",v_flag);
        strcpy ((tempchar + DDBNLN + DDVLLN),stemp);

        /* replace record in associated DDR file */
        if (c_labreplace (fd,key,clen,dlen,tempchar,dbuf,dtype) 
                          != E_SUCC)
        {
            c_lasmsg(FALSE,"svmnmx-call","Error returned from labrepl");
            return(E_FAIL);
        }

    }  /* end of for loop */

    action = 0;                           /* close associated DDR file */
    if (c_labclose (fd,hostddr,action) != E_SUCC)   
    {
        c_lasmsg(FALSE,"svmnmx-call","Error returned from labclose");
        return(E_FAIL);
    }

    return(E_SUCC);
}


#ifndef NO_OBSOLETE_INTERFACE


/**
 * c_svmnmx:
 * @hname: I: Input host file name for which information is to be written.
 * @totbnd: I: Total number of bands in the output image.
 * @minval: I: Array of minimum intensity values.
 * @maxval: I: Array of maximum intensity values.
 * @m_flag: I: A flag ("YES", "NO", or "BOUNDED") indicating whether or not
 *             the min and max values were calculated/estimated.  An
 *             application program can pass the value retrieved from the
 *             $MINMAX global ("YES" the minmax was calculated or "NO" it 
 *             was not calculated).  An application program can pass the 
 *             value of "BOUNDED" when estimated values for minval are to be
 *             calculated.
 *
 * The c_svmnmx() call is used to output the specified minimum, maximum, and
 * validity flag values for each band record of the DDR file associated with
 * image "hname".
 *
 * Deprecated.  Use las_svmnmx() instead.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation,
 *           E_PROT:  DDR is write protected, no error message is produced
 *                    from this error.
 */
int c_svmnmx
(
    const char *hname,     /* I: input host image name                      */
    const long *totbnd,    /* I: number of bands to be processed            */
    const double minval[], /* I: minimum intensity values                   */
    const double maxval[], /* I: maximum intensity values                   */
    const char *m_flag     /* I: min/max validity flag ("YES"/"NO"/"BOUND") */
                           /*    from $MINMAX global                        */
)
{
    int status;

    status = las_svmnmx (hname,*totbnd,minval,maxval,m_flag);

    return (status);
    
}



#endif /* obsolete interfaces */


