/*******************************************************************************
NAME                          C_UPDDR

PURPOSE      Updates records 1 and 2 of a DDR file

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
B. Ailts              Jan. 1988         original development
B. Ailts              Apr. 1988         Replaced newlas.h with las.h
                                        Replaced cerrmsg with c_errmsg
                                        Changed calling sequences to the put
                                        and get ddr calls
                                        Added the rotated projection space
                                        subroutine
B. Ailts              jul. 1988         COMB option can now have more output
                                        bands than input bands
B. Ailts              Sept. 1988        Increased the buffer space for the 
                                        band records of the ddr when multiple
                                        bands are being combined
B.Ailts               Jan. 1989         If COMB, all bands of one input image
                                        are not compared with the corresponding
                                        bands of another input image,  all the
                                        input bands of all the input images are
                                        compared with one another
B.Ailts               Dec. 1989         Added code to free dynamic allocated 
                                        buffers
B.Ailts               Dec. 1990         Updated error messages
J.Janssen             Jun. 1998         Added the capability to handle LAS
                                        images with DMS projection units

PROJECT       LAS

ALGORITHM 
   Consolidate all of the input image DDRs into one
   Write the output DDR to the DDR file
   Write the output BDDRs to the DDR file

    Return

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <string.h>
#include "las.h"


/**
 * las_upddr:
 * @parblk: I: TAE parameter block passed by reference.  It is assumed
 *             that TAE has been initialized prior to calling las_upddr().
 * @host_in: I: Image host file name(s) from which the DDR values are to be
 *              retrieved and used to calculate the output DDR values.
 * @host_out: I: Output image host file name for which the DDR values are
 *               calculated and written to.
 * @nmrimg: I: Total number of input image(s).
 * @window: I: Two-dimensional array containing the window specification for
 *             each input image.  The first dimension is "imglimit" and the
 *             second is MAXBAND + 1.
 * @bands: I: Two-dimensional array containing the band numbers of each of 
 *            the input images.  The first dimension is "imglimit" and the
 *            second is MAXBND + 1.
 * @nbands: I: Number of bands specified for each input image.
 * @line_inc: I: Number of pixels of the input image(s) that one output pixel
 *               will represent in the line direction.  A positive fractional
 *               value greater than 0 and less than 1 specifies that the
 *               input image(s) have been expanded.  A positive integer
 *               value greater than 1 specifies that the input image(s) have
 *               been subsampled.  If no replication or subsampling of
 *               pixels was completed enter a value of 1.
 * @samp_inc: I: Number of pixels of the input image(s) that one output pixel
 *               will represent in the sample direction.  A positive fractional
 *               value greater than 0 and less than 1 specifies that the
 *               input image(s) have been expanded.  A positive integer
 *               value greater than 1 specifies that the input image(s) have
 *               been subsampled.  If no replication or subsampling of
 *               pixels was completed enter a value of 1.
 * @out_nbands: I: Total number of bands the output image will contain.
 * @cflag: I: Flag indicating whether the input images will be combined
 *            into one band of the output image.  This value should be set
 *            to COMB for functions like CONCAT, MATH where the number of
 *            output bands is not the same as the number of input bands.
 *            Valid values are:
 *            COMB:  Images will be combined,
 *            NOCOMB: Images will not be combined.
 *
 * The las_upddr() function calls c_merge_ddrs() which performs the bulk of
 * the processing.  las_upddr() then writes the updated DDR information,
 * including the individual band information, to an output DDR file.  The
 * las_merge_ddrs() call reads "nmrimg" DDR files associated with the image
 * host file names specified in "host_in".  Based upon the values of 
 * "window, bands, nbands, line_inc, samp_inc, out_nbands, cflag", and the
 * values in the input DDR, the DDR associated with "host_out" will contain
 * values that are copied from the input DDRs, recalculated from the input
 * DDRs, or invalidated.
 *
 * This function call must be placed after the call to c_eopens()/c_eopenr()
 * and before the call to las_svmnmx().
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
 
int las_upddr
(
    struct PARBLK *parblk,         /* I: TAE parameter block                */
    const  char host_in[][CMLEN],  /* I: array of input host names          */
    const  char *host_out,         /* I: Output host name                   */
    int    nmrimg,                 /* I: Total number of input images       */
    const  int  window[][4],       /* I: Window specifications for each im  */
    const  int  bands[][MAXBND +1],/* I: band numbers for each input image  */
    const  int  nbands[],          /* I: number of bands for each input im  */
    double line_inc,               /* I: increment in the line direction    */
    double samp_inc,               /* I: increment in the sample direction  */
    int    out_nbands,             /* I: number of output bands             */
    int    cflag                   /* I: combination of bands flag          */
)
{ 
    struct DDR out_ddr;          /* output ddr                              */
    struct BDDR *out_bddr;       /* output band dependent ddr record        */

    int  i;                      /*  loop counter                           */
    int  status;                 /*  status flag                            */

    /* Consolidate all of the image DDRs into one                           */
    status = las_merge_ddrs (parblk, host_in, host_out, nmrimg, window, 
                             bands, nbands, line_inc, samp_inc, out_nbands, 
                             cflag, &out_bddr, &out_ddr, TRUE);
    if (status != E_SUCC)
    {
        c_lasmsg(FALSE,"upddr-call","Error returned from merge_ddrs");
        return(E_FAIL);
    }

    /* Write the band indepenent information to the output ddr              */
    status = c_putddr(host_out,&out_ddr);
    if (status != E_SUCC)
    {
        c_lasmsg(FALSE,"upddr-call","Error returned from putddr");
        return(E_FAIL);
    }

    for (i = 0; i < out_nbands; i++)
    {
        /* Write each of the bands to the output ddr                        */
        status = c_putbdr(host_out,&(out_bddr[i]));
        if (status != E_SUCC)
        {
            c_lasmsg(FALSE,"upddr-call","Error returned from putbdr");
            return(E_FAIL);
        }
    }
    free(out_bddr);
    return (E_SUCC);
} /*  c_upddr  */

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_upddr:
 * @parblk: I: TAE parameter block passed by reference.  It is assumed
 *             that TAE has been initialized prior to calling c_upddr().
 * @host_in: I: Image host file name(s) from which the DDR values are to be
 *              retrieved and used to calculate the output DDR values.
 * @host_out: I: Output image host file name for which the DDR values are
 *               calculated and written to.
 * @nmrimg: I: Total number of input image(s).
 * @window: I: Two-dimensional array containing the window specification for
 *             each input image.  The first dimension is "imglimit" and the
 *             second is MAXBAND + 1.
 * @bands: I: Two-dimensional array containing the band numbers of each of 
 *            the input images.  The first dimension is "imglimit" and the
 *            second is MAXBND + 1.
 * @nbands: I: Number of bands specified for each input image.
 * @line_inc: I: Number of pixels of the input image(s) that one output pixel
 *               will represent in the line direction.  A positive fractional
 *               value greater than 0 and less than 1 specifies that the
 *               input image(s) have been expanded.  A positive integer
 *               value greater than 1 specifies that the input image(s) have
 *               been subsampled.  If no replication or subsampling of
 *               pixels was completed enter a value of 1.
 * @samp_inc: I: Number of pixels of the input image(s) that one output pixel
 *               will represent in the sample direction.  A positive fractional
 *               value greater than 0 and less than 1 specifies that the
 *               input image(s) have been expanded.  A positive integer
 *               value greater than 1 specifies that the input image(s) have
 *               been subsampled.  If no replication or subsampling of
 *               pixels was completed enter a value of 1.
 * @out_nbands: I: Total number of bands the output image will contain.
 * @cflag: I: Flag indicating whether the input images will be combined
 *            into one band of the output image.  This value should be set
 *            to COMB for functions like CONCAT, MATH where the number of
 *            output bands is not the same as the number of input bands.
 *            Valid values are:
 *            COMB:  Images will be combined,
 *            NOCOMB: Images will not be combined.
 *
 * The c_upddr() function calls c_merge_ddrs() which performs the bulk of
 * the processing.  c_upddr() then writes the updated DDR information,
 * including the individual band information, to an output DDR file.  The
 * las_merge_ddrs() call reads "nmrimg" DDR files associated with the image
 * host file names specified in "host_in".  Based upon the values of 
 * "window, bands, nbands, line_inc, samp_inc, out_nbands, cflag", and the
 * values in the input DDR, the DDR associated with "host_out" will contain
 * values that are copied from the input DDRs, recalculated from the input
 * DDRs, or invalidated.
 *
 * This function call must be placed after the call to c_eopens()/c_eopenr()
 * and before the call to c_svmnmx().
 *
 * Deprecated.  Use las_upddr() instead.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_upddr
(
    struct PARBLK *parblk,       /* I: TAE parameter block                  */
    const char host_in[][CMLEN], /* I: array of input host names            */
    const char *host_out,        /* I: Output host name                     */
    const long *nmrimg,          /* I: Total number of input images         */
    const long  window[][4],     /* I: Window specifications for each image */
    const long  bands[][MAXBND +1],/* I: band numbers for each input image  */
    const long  nbands[],        /* I: number of bands for each input image */
    const double *line_inc,      /* I: increment in the line direction      */
    const double *samp_inc,      /* I: increment in the sample direction    */
    const long *out_nbands,      /* I: number of output bands               */
    const long *cflag            /* I: combination of bands flag            */
)
{
    int status;

    status = las_upddr (parblk,host_in,host_out,*nmrimg,window,bands,nbands,
                        *line_inc,*samp_inc,*out_nbands,*cflag);

    return (status);

}

#endif /* obsolete interfaces */

