/******************************************************************************

NAME:   CLMNMX

FUNCTION: CLMNMX is function which is called to calculate the minimum and 
          maximum pixel values of an image.  It is called from MINMAX subroutine

PROGRAM HISTORY:
        Version         Date    Author          Request
        -------         ----    -----           -------
          1.0           5/86    B.Ailts         initial development
          2.0           9/86    K. Gacke        used host file names rather than
                                                TAE file names
          3.0           7/88    B.Ailts         LAS 5.0 standards
          4.0          12/90    B.Ailts         Updated error messages

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
                Run under TAE

PROJECT:   LAS

ALGORITHM DESCRIPTION:
The space needed for errtxt is dynamically allocated
An message is output to show the user what is going on
The image is opened
For each line of the input image
   For each band specified
      The min max values are calculated  
The image is closed
The space allocated for thebuf is freed
The space allocated for errtxt is freed
These values are then passed back to the calling program

ALGORITHM REFERENCES:  none
*****************************************************************************/
#include "las.h"
#include "pixman.h"
#include "local_protos.h"

int clmnmx
(
    const char *hostname,  /* I: host name of input image                   */
    int   *window,         /* O: window specified for the input image       */
    int   *cl_bands,       /* I: bands that the min/max values are needed   */
    const int *mmbands,    /* I: the correct place in the min/max arrays that
                                 the min/max values should be placed        */
    double *minval,        /* I/O: minimum value array                      */
    double *maxval         /* I/O: maximum value array                      */
)

{  /*  clmnmx  */

    int  band;             /*  current band                                 */
    struct FDESC *fdesc[2];/*  file descriptor array                        */
    struct GDESC *gdesc;   /*  group descriptor                             */
    int  line;             /*  current line                                 */
    int  size = 0;         /*  size of the data type in bytes               */
    int  status;           /*  error status                                 */

    char *errtxt;          /*  buffer of the error message                  */
    char *thebuf;          /*  buffer of the input image data               */
    long acc = IREAD;      /*  read access flag                             */
    long bufsize;          /*  size of the buffer                           */
    long dtype = 0;        /*  data type                                    */
    long offset[MAXBND];   /*  offsets into each band of the image          */
    long zero = 0;         /*  value of zero                                */

    if (cl_bands[0] == 0)
        return(E_SUCC);

    /*  Allocate the string buffers needed                                  */
    errtxt = (char *)malloc(ERRLEN + 1);
    if (errtxt == 0)
    {
        c_lasmsg (FALSE, "minmax-alloc", "Error allocating dynamic memory");
        return(E_FAIL);
    }

    /*  Output the informational message to the users                       */
    sprintf(errtxt,"Calculating the min and max values of %s",hostname);
    c_lasmsg (FALSE," ",errtxt);
    c_lasmsg (FALSE, " ", "Please wait...");

    /*  Open the input image                                                */
    status = c_eopens (&fdesc[0],hostname,(long *)cl_bands,(long *)&window[0],
                       (long *)&window[1],(long *)&window[2],(long *)&window[3],
                       offset,&acc, &dtype,&zero,&zero);
    if (status != E_SUCC)
    {
        sprintf (errtxt,"Error opening image %s",hostname);
        c_lasmsg (FALSE, "minmax-iopen", errtxt);
        free(errtxt);
        return(E_FAIL);
    }
    fdesc[1] = 0;

    switch (dtype)
    {
        case (EBYTE): 
            size = 1;  
            break;
        case (EWORD): 
            size = 2;  
            break;
        case (ELONG): 
            size = 4;  
            break;
        case (EREAL): 
            size = 4;  
            break;
    }

    thebuf = c_ealloc(fdesc,&bufsize);
    if (thebuf == NULL)
    {
        c_lasmsg (FALSE, "minmax-call", "Error returned from ealloc");
        free(errtxt);
        return(E_FAIL);
    }

    status = c_egroup(fdesc,&gdesc,(unsigned char *)thebuf,&bufsize);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "minmax-call", "Error returned from egroup");
        free(thebuf);
        free(errtxt);
        return(E_FAIL);
    }

    /*  loop for every line of every band needed 
        calculate the min/max values                                        */
    for (line = 0; line < window[2]; line++)
    {
        status = c_estep(&gdesc);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "minmax-call", "Error returned from estep");
            free(thebuf);
            free(errtxt);
            return(E_FAIL);
        }
        for (band = 0; cl_bands[band] != 0; band++)
            las_pixminmax ((int *)(thebuf + (offset[band] * size)),window[3],
                         &minval[mmbands[band]],&maxval[mmbands[band]],dtype);
    }

    /*  Close the image file                                                */
    status = c_egclse(&gdesc);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "minmax-call", "Error returned from egclse");
        free(thebuf);
        free(errtxt);
        return(E_FAIL);
    }

    free(thebuf);
    free(errtxt);

    return(E_SUCC);
} /* clmnmx  */
