/*******************************************************************************
NAME                           SET_PDIST

PURPOSE      Calculates the projection distance fields of the output DDR files

EXTERNAL ASSOCIATIONS
        This routine is called by C_UPDDR routine
        This routine is used in conjunction with SET_PDIS_FLAG

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
B. Ailts              Jan. 1988         original development
B. Ailts              Apr. 1988         Replace newlas.h with las.h
                                        Changed the calling sequence -- now only
                                          uses one DDR structure
                                        Update to use current constants
                                        
PROJECT       LAS

ALGORITHM 
   If the pdist_flag for equals INVAL
      Assign INVAL the the projection distance validity flag of the output DDR
      Assign a zero the the projection distance value of the output DDR

   else if the pdist_flag for equals UDIFF
      Assign UDIFF the the projection distance validity flag of the output DDR
      Assign a zero the the projection distance value of the output DDR

   else 
      If the pdist_flag for equals USAME
         Assign UKNOW the the projection distance validity flag of the 
           output DDR
      else 
         Assign VALID the the projection distance validity flag of the 
           output DDR
      endif
      Calculate the projection distance in the y direction of the output DDR 
        by multiplying the input DDR projection distance in the y direction 
        by the input line increment value

      Calculate the projection distance in the x direction of the output DDR
        by multiplying the input DDR projection distance in the x direction 
        by the input sample increment value

   endif
  

ALGORITHM REFERENCES    none
*******************************************************************************/
#include "las.h"
#include "local_protos.h" 




/**
 * set_pdist:
 * @pdist_flag: I: Input projection distance flag.
 * @in_ddr: I: Input DDR structure.
 * @out_ddr: O: Output DDR structure.
 * @line_inc: I: Number of pixels of the input image(s) that one output pixel
 *               will represent in the line direction.  A positive fractional
 *               value greater than 0 and less than 1 specifies that the
 *               input image(s) have been expanded.  A positive integer
 *               value greater than 1 specifies that the input image(s) have
 *               been subsampled.  If no replication or subsampling of
 *               pixels was completed enter a value of 1.
 * @samp_inc: I: Number of pixels of the input image(s) that one output pixel
 *               will represent in the sample direction.  A positive fractional
 *               value greater than 0 and less than 1 specifies that the
 *               input image(s) have been expanded.  A positive integer
 *               value greater than 1 specifies that the input image(s) have
 *               been subsampled.  If no replication or subsampling of
 *               pixels was completed enter a value of 1.
 *
 * The set_pdist() routine is called by the las_upddr() routine and 
 * is used in conjunction with the setpdistflag() routine.
 * The set_pdist() call calculates the projection distance fields of the 
 * output DDR files.
 */
void set_pdist
(
    int  pdist_flag,           /* I: input projection distance flag         */
    const struct DDR *in_ddr,  /* I: input DDR structure                    */
    struct DDR *out_ddr,       /* O: output DDR structure                   */
    double line_inc,           /* I: input line increment value             */
    double samp_inc            /* I: input sample increment value           */
)

{

    /*  Check the input projection distance flag for INVAL, UDIFF, USAME, 
        or VALID                                                            */
    if (pdist_flag == INVAL)
    {
        out_ddr->pdist_x = 0;  /* Null out the projection distance in x dir.*/
        out_ddr->pdist_y = 0;  /* Null out the projection distance in y dir.*/
        out_ddr->valid[DDPDV] = INVAL; /* Invalidate proj_dist validity flag*/
    }  /* if INVAL */

    else if (pdist_flag == UDIFF)  
        /* unknown flag, values not equal */
    {
        out_ddr->pdist_x = 0;   /*  Null out the projection distance in x dir.*/
        out_ddr->pdist_y = 0;   /*  Null out the projection distance in y dir.*/
        out_ddr->valid[DDPDV] = UNKNOW;
    } /* if UDIFF */

    else    /* USAME or VALID for flag, values equal */
    {
        /* set output corner validity flag */
        if (pdist_flag == USAME)
            out_ddr->valid[DDPDV] = UNKNOW;
        else
            out_ddr->valid[DDPDV] = VALID;

        /* set output ground distance values */
        out_ddr->pdist_y = in_ddr->pdist_y * line_inc;
        out_ddr->pdist_x = in_ddr->pdist_x * samp_inc;
    }
}  /* set_pdist */
