/*******************************************************************************
NAME                           UPMNMX

PURPOSE      Output minimum and maximum intensity values and a
             corresponding validity flag for each band specified
             of an image to the associated DDR file.

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
B. Ailts              Jul. 1988         original development
B. Ailts              Dec. 1990         Updated error messages
D. Etrheim            Oct. 1993         Corrected so that when the VALID flag
                                        is updated, tempchar does not get NULL
                                        terminated inadvertently by sprintf

PROJECT         LAS

ALGORITHM:
   Allocate space for the associated DDR file name
   Create the associated DDR file name from the input host image name.
   Check for write access to the DDR file.  
   Open the DDR file for update access using label services.  
   For each output band 
       read in the specified record 
       change the min, max, and validity flag with the appropriate value
       replace the record
   Close the DDR file.

ALGORITHM REFERENCES    Similar to the function C_SVMNMX
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "local_protos.h"

int upmnmx
(
    const char *hname,/* I: input host image name                           */
    int  *cl_bands,   /* I/O: bands of image that were processed            */
    int  *mmbands,    /* I/O: bands of min/max arrays to be used            */
    double minval[],  /* I/O: minimum intensity values                      */
    double maxval[]   /* I/O: maximum intensity values                      */
)

{
    struct BDDR bddr; /* band data for the DDR                              */

    int  access;      /* file access type                                   */
    int  action;      /* file close action flag                             */
    int  clen;        /* length of character part of record                 */
    int  ctemp;       /* temporary storage for clen                         */
    int  dlen;        /* length of data part of record                      */
    int  dtemp;       /* temporary storage for dlen                         */
    int  fd;          /* file descriptor for opened file                    */
    int  i;           /* loop index                                         */
    int  mode;        /* lsstat mode                                        */
    int  status;      /* function return status code                        */

    unsigned char *dbuf; /* pointer to area where data is stuffed           */

    char dtype[3];          /* type of data portion of record               */
    char *hostddr;          /*  host ddr name                               */
    char key[16];           /* record key                                   */
    char tempchar[DDTSLN];  /* temporary char. portion of record            */
    char tchar[3];          /* temporary string                             */

    /* if cl_bands[0] means that no bands need updating                     */
    if (cl_bands[0] == 0)
        return(E_SUCC);

    /* Allocate the space needed for the host ddr name                      */
    hostddr = (char *)malloc(CMLEN);
    if (hostddr == 0)
    {
        c_lasmsg (FALSE, "minmax-alloc", "Error allocating dynamic memory");
        return(E_FAIL);
    }

    c_lsmknm(hname,"ddr",hostddr);       /* make associated DDR file name   */

    mode = 2;                            /* check DDR file protection       */
    status = c_labstat(hostddr,mode);
    if (status != E_SUCC)
    {
        free (hostddr);
        return(E_PROT);
    }

    access = 3;                           /* open DDR file for read/write */
    fd = c_labopen (hostddr, access);
    if (fd < 0) 
    {
        c_lasmsg (FALSE, "minmax-call", "Error retruned from labopen");
        free (hostddr);
        return(E_FAIL);
    }

    for (i = 0; cl_bands[i] != 0; i++)
    {
        /* rewind to the beginning of the file each time in case the band 
           order is not from lowest to highest band */
        if (c_labrew(fd) != E_SUCC)
        {
            c_lasmsg(FALSE,"minmax-rew","Error rewinding the DDR file");
            return(E_FAIL);
        }
        
        /* read in current band dependent record                            */
        sprintf(key,"BAND%d",cl_bands[i]);
        clen = DDTSLN;
        ctemp = clen - 1;
        dlen = dtemp = DBSIZE * 8;
        dbuf = (unsigned char *) &(bddr.minval);
        status = c_labread (fd,key,&clen,&dlen,tempchar,dbuf,dtype);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "minmax-call", "Error retruned from lsread");
            free (hostddr);
            return(E_FAIL);
        }

        if ( (clen != ctemp) || (dlen != dtemp) )
        {
            c_lasmsg (FALSE, "minmax-read", "Error reading the DDR file");
            free (hostddr);
            return(E_FAIL);
        }

        /* update with new min, max, and validity flag                      */
        bddr.minval = minval[mmbands[i]];
        bddr.maxval = maxval[mmbands[i]];
        /*sprintf((tempchar + DDBNLN),"%2d",VALID);*/
        sprintf(tchar,"%2d",VALID);
        strncpy((tempchar + DDBNLN),tchar,2);

        /* replace record in associated DDR file                            */
        if (c_labreplace (fd,key,clen,dlen,tempchar,(char *)dbuf,dtype) 
                           != E_SUCC)
        {
            c_lasmsg (FALSE, "minmax-call", "Error retruned from lsrepl");
            free (hostddr);
            return(E_FAIL);
        }

    }  /* end of for loop */

    action = 0;                           /* close associated DDR file */
    if (c_labclose (fd,hostddr,action) != E_SUCC)   
    {
        c_lasmsg (FALSE, "minmax-call", "Error retruned from lsclos");
        free (hostddr);
        return(E_FAIL);
    }

    /*  Free the host DDR file name space                                   */
    free(hostddr);

    return(E_SUCC);
}
