/******************************************************************************
FUNCTION:       c_dkcopy

PURPOSE:        Copy a file to a new file

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0                                   Original development
  5.0      05/88    D. Hollaren         changed to use c_errmsg
  5.0      11/88    B. Davis            changed flags for vms open of output
                                        file to be compatible with vms dkcre
  5.0      10/90    T. Baltzer          clean up error msg's, return status's
  5.1      05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  5.2      11/94    D. Etrheim          Added call to dkfree to clean up
                                        disk reserve file correctly.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Get the size of the input file from the file status information
Open the input file
Create the output file the same size as the input file
Open the output file
Copy the input file to the output file
Close the files and free up the table reserved space.

ALGORITHM REFERENCES    none
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include "las.h"
#include "diskio.h"
#include "util.h"    /* c_sigblk/unblk prototypes */
#include "local_protos.h"


/**
 * c_dkcopy:
 * @from:  I: Specifies the input path/filename of the file to be copied.
 * @to: I: Specifies the output path/filename of the file to which the
 *         input file is to be copied.
 * @directories: I: Specifies a list of pathnames for directories to be
 *               searched for available space.
 *
 * If the specified file from does not exist, c_dkcopy() returns a status
 * of DISK_GONE. c_dkcopy() copies disk files.  It was written specifically
 * to copy disk files created by c_dkcre(). c_dkcopy() uses the same
 * algorithms to select a directory and allocate the file as c_dkcre().
 * The file "from" is copied and the file "to" becomes a symbolic link to the
 * actual file.  If the file "from" is a symbolic link, the file it points to
 * is copied and a symbolic link is created to point to the new file.
 *
 * Returns:  E_SUCC:        Successful completion,
 *           E_FAIL:        Failure status,
 *           DISK_GONE:     The specified file does not exit,
 *           DISK_NOSPACE:  The disk space specified was not available and
 *                          the calling function did not want to wait for it.
 */
int c_dkcopy
(
    const char *from,               /* I: Current file name                 */
    const char *to,                 /* I: New file name                     */
    char directories[][CMLEN]       /* I/O: List of destination directories */
)
{
    int sig_mask;
    int sz;                         /* Size of the buffer read              */
    int inp;                        /* Input opened file descriptor         */
    int out;                        /* Output opened file descriptor        */
    int flags;                      /* Flags indicating how to open the file*/
    int wait = 0;                   /* Time to wait when creating file      */
    int mode = 0644;                /* Mode to open the output file with    */
    long long filesize;             /* Size of the input file               */
    long nonfatal = 1;              /* Nonfatal error code                  */
    int  status;                    /* Return status                        */

    char buf[8192];                 /* Buffer used to copy with             */
    char msg[ERRLEN];               /* Error message text                   */

    struct stat statp;              /* File status structure                */

    c_sigblk(&sig_mask);

    /* Get status information about the input file  */
    if (stat(from,&statp) == -1)
    {
        if (errno == ENOENT)
        {
            c_sigunblk(&sig_mask);
            return(DISK_GONE);
        }
        else
        {
            c_errmsg("Error getting path status information","dkcopy-stat",
                     &nonfatal);
            c_sigunblk(&sig_mask);
            return(E_FAIL);
        }
    }

    /* Get the input device and the file size               */
    filesize = statp.st_size;

    /* Open the input file for read only                    */
    flags = O_RDONLY;
    if ((inp = open(from,flags,mode)) < 0)
    {
        sprintf(msg,"Error opening file %s",from);
        c_errmsg(msg,"dkcopy-open",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    /* Create the output file                               */
    if ((directories == NULL) || (strcmp(directories[0],"") == 0))
        status = c_dkcre(NULL,to,filesize,wait);
    else
        status = c_dkcre(directories,to,filesize,wait);
    switch (status)
    {
        case E_FAIL:    
            close(inp);
            c_sigunblk(&sig_mask);
            return(E_FAIL);
        case DISK_NOSPACE: 
            if ( close(inp) == -1 )
            {
                c_errmsg("Error closing file","dkcopy-close",
                          &nonfatal);
                c_sigunblk(&sig_mask);
                return(E_FAIL);
            }
            return(DISK_NOSPACE);
        case E_SUCC: 
            break;    /* do nothing */
        case DISK_EXISTS: 
            break;    /* do nothing */
    }

    /* Open the output file                                 */
    flags = O_RDWR | O_APPEND;
    if ((out = open(to,flags,mode)) < 0)
    {
        sprintf(msg,"Error opening file %s",to);
        c_errmsg(msg,"dkcopy-open",&nonfatal);
        close(inp);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    /* Read the data from the input file and write it to the output file */
    while ((sz = read(inp,buf,8192)) > 0)
    {
        status = write(out,buf,sz);
        if (status == E_FAIL)
        {
            sprintf(msg,"Error writing to file %s",to);
            c_errmsg(msg,"dkcopy-write",&nonfatal);
            close(out);
            close(inp);
            c_sigunblk(&sig_mask);
            return(E_FAIL);
        }
    }
    if (sz < 0)
    {
        sprintf(msg,"Error reading file %s",from);
        c_errmsg(msg,"dkcopy-read",&nonfatal);
        close(out);
        close(inp);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    /* Close the input and output files                     */
    if (( close(out) != 0 ) || ( close(inp) != 0 ))
    {
        c_errmsg("Error closing file","dkcopy-close",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    c_dkfree(to);
    c_sigunblk(&sig_mask);
    return(E_SUCC);
}
