
/****************************************************************************** FUNCTION:       c_dkcre

PURPOSE:        Create and reserve space for a file.  The file will be
                created and space reserved for the file in one of the
                specified directory paths.  The disk space available
                that can be best utilized for the creation of the file
                is used.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0                                   Original development
  5.0      05/88    D. Hollaren         Put in calles to c_errmsg
  5.1      07/88    D. Hollaren         Fixed bug that created two versions of
                                        an image on the vax
  5.2      10/90    T. Baltzer          Fix return status's, errmsg calls
  5.5      05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  5.6      08/91    B. Davis            longs changed to doubles for large disk
  5.7      09/91    D. Etrheim          Modified to do a first fit instead of
                                        a best fit for SG. PRF #6022
  5.8      02/92    D. Etrheim          Change to use bfree for DG.  This seems
                                        to better represent what can actually
                                        be allocated
  5.9      05/92    T. Mittan           Change to use bavail for DG.  This gives
                                        a better representation of availible
                                        disk space
  6.0      07/92    T. Mittan           type cast longs to doubles
  6.1      07/93    D. Etrheim          Modifed to do a first fit for all 
                                        systems
  6.2      05/96    K. Gacke            Add a buffer zone to ensure disk space
                                        is really available.
  6.3      01/97    D. Zokaites         Checked file existence with lstat()
                                        instead of access(). 
  6.4      11/98    T. Baltzer          Added logic in search() to call
                                        c_dkusage(), etc.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
  Make sure file dosen't already exist
  Allocate space for the current working directory,
    the DATAPATH,
    file status structure, and
    file systems statistics

  Get the current working directories
  Set an array of pointers to the directory paths
  Get the directory paths to be search for the creation of the file
    (if no directory paths were specified the current working directory is used)

  Loop until the file is created or try once if wait flag is not set
        Search the paths to determine the best location to create the file
        Try to create the file with that path
        If the the file was created break from the loop
        If the wait flag was not set and not enough disk space was avaliable
           break with an error
        Send a message to the operators that there is not enough disk space
        Wait for the specified amount of time
  End of loop; try to create the file again.

  Free up space allocated for;
        the current working directory,
        datapath, and
        file status structure

  return status

ALGORITHM REFERENCES
  The DATAPATH contains directories seperated by a colon ":"
  ie) "/software/test:/production/job:/production/batch/proj1"
******************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/statvfs.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>
#include "las.h"
#include "diskio.h"
#include "util.h"
#include "typlim.h"
#include "local_protos.h"


#define CONSOLE         "/dev/console"
#define DATAPATH        "DATAPATH"
#ifndef MIN
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#endif
#define PATHNAMESZ      1024
/* Define a buffer zone to ensure free disk is actually available.  On the
   SGI systems, the statfs on an XFS file system may return blocks available
   even though the file system is full.  This also allows the possibility of
   a user running a program outside of LAS that requires a small amount of
   disk space (eg. editing a file).                                         */

/* Private Globals                                                          */
static char            themsg[PATHNAMESZ];     /* used for cerrmsg          */
static char            *cwd;                   /* current working directory */
static char            *datapath;              /* DATAPATH environment      */
static char            **createdpaths=NULL;    /* pointer to paths created  */


#ifdef ALLOCATE
/******************************************************************************
FUNCTION:       allocate

PURPOSE:        Allocate the space on disk of a specified size

ALGORITHM
  Get file status information about the file
  using the block size write NULL's to the file
  until the specified file size is reached.

ALGORITHM REFERENCES    none
******************************************************************************/

static int allocate
(
    int fd,                  /* I: File descriptor              */
    long long filesize       /* I: Size to allocate             */
)
{
    int   blocksize;         /* Block size of file system    */
    struct stat statp;       /* File status structure        */
    int sig_mask;
        
    c_sigblk(&sig_mask);

#ifdef DEBUG
    fprintf(stderr,"allocating: %lld bytes\n",filesize);
#endif


    /* Get status information on the file from the file descriptor          */
    if (fstat(fd,&statp) == -1)
    {
        c_errmsg("Error getting status information on the file",
                 "dkcre-fstat",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    /* Get the block size of the file                                       */
    blocksize = statp.st_blksize;

    /* Write nulls to the file so that space is allocated for the file      */
    for (; filesize > 0; filesize -= blocksize)
    {
        /* Seek to the beginning of each block                              */
        if (lseek(fd,MIN(blocksize,filesize) - 1,1) == -1)
        {
            c_errmsg("Error seeking in file","dkcre-seek",&nonfatal);
            c_sigunblk(&sig_mask);
            return(E_FAIL);
        }

        /* Write NULL's to the file                                         */
        if (write(fd,"\0",1) == -1)
        {
            c_errmsg("Error writing to file","dkcre-write",&nonfatal);
            c_sigunblk(&sig_mask);
            return(E_FAIL);
        }
    }

    c_sigunblk(&sig_mask);
    return(E_SUCC);
}
#endif

/******************************************************************************
FUNCTION:       basename

PURPOSE:        Get the base file name from a path name

ALGORITHM 
  Look for the last slash in a directory specification
      If there are no slashes the path is a base name
  else
      return the address of the location past the slash

ALGORITHM REFERENCES    none
******************************************************************************/

static const char *basename
(
    const char *pathname          /* I: Pathname to delete prefix from */
)
{
    char   *dest;                /* O: Pointer to base file name      */

    /* Get the base file name                                               */
    if ((dest = strrchr(pathname,'/')) == NULL)
        return (pathname);
    else
        dest++;

    return(dest);
}

/******************************************************************************
FUNCTION:       create

PURPOSE:        create a file

ALGORITHM 
  If no path name was specified
        return

  If the path name is not in the current working directory
        Generate a fullpathname of the current working directory
        Create a temporary file for a soft link

  Create the file
  Software reserve the file size space
  Close the file created

  If not in the current working directory
        Genrate a soft link to the file created

ALGORITHM REFERENCES    none
******************************************************************************/

static int create
(
    const char *pathname,   /* I: Pathname where file should be created */
    const char *filename,   /* I: Name of file to be created            */
    long long filesize      /* I: Size of file to be created            */
)
{
    const char  *fullpathname;/* Full pathname of created file          */
    char *fpn;                /* Pointer to allocate and manipulate buf */
    char    msg[ERRLEN];      /* Error message string                   */
    int     fd;               /* File descriptor                        */
    long    nonfatal = 1;     /* Non-fatal error code                   */


    /* See if there is a pathname                                           */
    if (pathname == 0)
        return(0);

    /* Determine if the pathname is the current working directory           */
    if (strcmp(pathname,cwd) != 0 && strcmp(pathname,".") != 0)
    {
        /* Allocate space for the full path name                            */
        if ((fpn = malloc(PATHNAMESZ)) == NULL)
        {
            c_errmsg("Error allocating dynamic memory","dkcre-alloc",
                     &nonfatal);
            return(E_FAIL);
        }

        /* Generate the full path name                                      */
        sprintf(fpn,"%s/%s.XXXXXX",pathname,
                basename(filename));

        /* Create the file                                                  */
        if (mktemp(fpn) == NULL)
        {
            sprintf(msg,"Error opening file %s",fpn);
            c_errmsg(msg,"dkcre-open",&nonfatal);
            return(E_FAIL);
        }
        fullpathname = fpn;
    }
    else
        fullpathname = filename;


#ifdef DEBUG
    fprintf(stderr,"creating: %s\n",fullpathname);
#endif

    /* Create the file                                                      */
    if ((fd = open(fullpathname,O_WRONLY | O_CREAT | O_TRUNC, 0666)) == -1)
    {
        sprintf(msg,"Error creating file %s",fullpathname);
        c_errmsg(msg,"dkcre-create",&nonfatal);
        return(E_FAIL);
    }

#ifdef ALLOCATE
    if (!allocate(fd,filesize))
    {
        if ( delete(fullpathname) != E_SUCC)
        {
            sprintf(msg,"Error deleting file %s",fullpathname);
            c_errmsg(msg,"dkcre-delete",&nonfatal);
            return(E_FAIL);
        }
    }
#endif

    /* instead of actually allocating because that takes to long, a record
       is placed into a software disk reserve file specifing the space
        needed for the file                                                 */
    if ( c_dkresv(fullpathname, filesize) != E_SUCC )
    {
        sprintf(msg,"Error reserving space for file %s",fullpathname);
        c_errmsg(msg,"dkcre-reserve",&nonfatal);
        return(E_FAIL);
    }
    if ( close(fd) == -1 )
    {
        c_errmsg("Error closing file","dkcre-close",&nonfatal);
        return(E_FAIL);
    }

    if (strcmp(pathname,cwd) != 0 && strcmp(pathname,".") != 0)
    {
#ifdef DEBUG
        fprintf(stderr,"linking: %s\n",filename);
#endif

        /* Generate a symbolic link                                         */
        if (symlink(fullpathname,filename) == -1)
        {
            c_errmsg("Error generating a symbolic link",
                     "dkcre-symlink",&nonfatal);
            return(E_FAIL);
        }
    }

    return((int) pathname);
}

/******************************************************************************
FUNCTION:       createpaths

PURPOSE:        Get the directory paths from the external global DATAPATH

ALGORITHM 
  Determine the number of directory paths in the DATAPATH global string
  Allocate space for the path names
  Parse the DATAPATHs for each directory path

ALGORITHM REFERENCES
  DATAPATH has directory paths seperated by a colon between
  each directory path.  ie) /xxx/xxx/xxx:/sss/sss/sss:/nnn/nnn
******************************************************************************/

static char **createpaths
(void)
{
    int     npaths;         /* Number of pathnames                  */
    char    *pathname;      /* Pathname pointer                     */
    char    **path;         /* Path array pointer                   */
    long    nonfatal = 1;   /* Non-fatal erro code                  */

    /* Determine the number of path names specified                 */
    /* Path names are seperated by colons ":"                       */
    for (pathname = datapath, npaths = 2;
        (pathname = strchr(pathname,':')); npaths++, pathname++)
        ;

    /* Allocate space for the path directory names                  */
    if ((createdpaths = (char **)calloc(npaths,sizeof(char *))) == NULL)
    {
        c_errmsg("Error allocating dynamic memory","dkcre-alloc",
                 &nonfatal);
        return(NULL);
    }

    /* Parse the path names                                         */
    path = createdpaths;
    *path = strtok(datapath,":");
    for (path++; (*path = strtok((char *)0,":")); path++)
        ;

    return(createdpaths);
}

/******************************************************************************
FUNCTION:       freespace

PURPOSE:        Determine the amount of free space available on a file system

ALGORITHM 
  Determine if the pathname is a directory specification
  Get the file status information
  Determine the space available on the file system

ALGORITHM REFERENCES    none
******************************************************************************/

static long long freespace
(
    const char *path        /* I: Pathname to determine free space     */
)
{
    int buffer;
    long long avail;        /* Free space that is available         */
    long long savail;       /* Software space reserved              */
    struct statvfs statfsp; /* File system statistics               */

    /* Get the file status information to determine the space avaliable     */
    if (statvfs(path,&statfsp) == -1)
    {
        if ( errno == EOVERFLOW )
        {
            /* A kludge to get around very large filesystems */
            avail = 1000000000000; /* A little less than 1TB */
            return(avail);
        }
        else
        {
            c_lasmsg(FALSE,"dkcre-statfs", 
                     "Error getting file system status information for %s",
                      path);
            return(E_FAIL);
        }
    }

    /* Determine the amount of available space                              */
    avail = (long long)statfsp.f_bavail * statfsp.f_bsize;

#ifdef DEBUG
    fprintf(stderr,"freespace: %lld %s\n",avail,path);
#endif

    /* Subtract the amount of space that is software reserved               */
    if ((savail = c_dkrspac(path)) == E_FAIL)
        return(E_FAIL);
    if (c_dkbuf(&buffer) == E_FAIL)
        return(E_FAIL);
    avail = avail - savail - buffer;

#ifdef DEBUG
    fprintf(stderr,"actual freespace: %lld %s\n",avail,path);
#endif

    return(avail);
}

/******************************************************************************
FUNCTION:       getpaths

PURPOSE:        Get the path names that are used to determine were the
                file is to be created.

ALGORITHM 
  If pathnames were specified
        return them

  Get the paths from the external global DATAPATH
  If the DATAPATH global is empty or doesn't exist
        use the current working directory

  Parse the datapath string

ALGORITHM REFERENCES
  DATAPATH has directory paths seperated by a colon between
  each directory path.  ie) /xxx/xxx/xxx:/sss/sss/sss:/nnn/nnn
******************************************************************************/

static char **getpaths
(
    char **pathnames        /* I/O: Array of pathnames    */
)
{
    char   *datapathp;
    char   **currentp;
    struct stat statp;      /* File status structure      */
    int    valid;


    /* If pathnames were not specified, get them          */
    if (!*pathnames)
    {
        /* Get the path names from the DATAPATH or use the current working 
           directory */
        if ((datapathp = getenv(DATAPATH)))
            strcpy(datapath,datapathp);
        else
            strcpy(datapath,cwd); 

        pathnames=createpaths();
    }
            
    /* Remove any paths that are not valid */
    currentp = pathnames;
    while (currentp && *currentp)
    {
        valid = 1;
        if (stat(*currentp,&statp) == -1)
        {
            valid = 0;
            c_lasmsg(FALSE,"dkcre-stat", "Error accessing path %s",*currentp);
        }
        else if (!(statp.st_mode & S_IFDIR))
        {
            /* Error if the pathname is not a directory */
            valid = 0;
            c_lasmsg(FALSE,"dkcre-path", "Invalid pathname: %s",*currentp);
        }
        if (!valid)
        {
            /* invalid path, so remove it from the list */
            char **temp = currentp;
            char **temp1 = currentp;
            temp1++;
            while (*temp)
            {
                *temp = *temp1;
                temp++;
                temp1++;
            }
        }
        else
            currentp++;
    }

    return (pathnames);
}

/******************************************************************************
FUNCTION:       search

PURPOSE:        Search all paths to determine which file system will be best
                for the creation of the output file.

ALGORITHM 
  Determine the space available on each of the paths specified
      Use the file system that is large enough and is also the least utilized
      file system.

ALGORITHM REFERENCES    none
******************************************************************************/

static char *search
(
    char **paths,      /* I/O: Array of directories to search for space */
    long long size     /* I: Size of space to search for                */
)
{
    char    *searchp = NULL;       /* Pathname where space exists */
    int    i;                      /* Counter                     */
    int    usage_counter;          /* Usage counter               */
    int    searchc = NULL;         /* Usage counter for searchp   */

    /* Search each path for space available to determine the first/best path
       location to place the file into                                      */
    for (i = 0; *paths != NULL; paths++,i++)
    {
        if (freespace(*paths) >= size)
        {
            /* Have now found pathname which has enough space.
               Determine how many times this pathname is being
               already utilized.                               */
            if (c_dkusage(*paths,&usage_counter) != E_SUCC)
                return(NULL);

            /* If this is the very first candidate found, set
               the output path name to it.  If the output path
               name already has a value, replace it with only
               a less utilized path name.                      */
            if (searchp == NULL)
            {
                searchp = *paths;
                searchc = usage_counter;
            }
            else
            {
                /* Usage less than previous candidate */
                if (usage_counter < searchc)
                {
                    searchp = *paths;
                    searchc = usage_counter;
                }
            }
        }
    }

    return(searchp);
}

/******************************************************************************
FUNCTION:       sendops

PURPOSE:        Send the operator a message that we are looking for disk space

ALGORITHM 
  Get the user id
  Open a path to the operator's console
  Build a string telling the operator that X space is needed on the specified
    file system
  Send the message to the user and the operator

ALGORITHM REFERENCES    none
******************************************************************************/
static int sendops
(
    long long filesize,         /* I: Size of file to be created             */
    char **paths                /* I/O: Array of directories to search/space */
)
{
    struct passwd   *pwd;       /* Pointer to password entry                 */
    FILE            *stream;    /* File stream pointer                       */
    long nonfatal = 1;          /* Non-fatal error code                      */

    /* Get the User ID                                                       */
    if ((pwd = getpwuid(getuid())) == NULL)
    {
        c_errmsg("Who are you?","dkcre-uid",&nonfatal);
        return(E_FAIL);
    }

#ifdef DEBUG
    stream = stdout;
    #else
    /* Open a path to the Operator Console                          */
    if ((stream = fopen(CONSOLE,"w")) == NULL)
    {
        c_errmsg("Error opening CONSOLE","dkcre-consol",&nonfatal);
        return(E_FAIL);
    }
#endif

    /* Send the operator a message that we need space               */
    fprintf(stream,"PID: %ld %s waiting for %lld bytes on ",(long)getpid(),
            pwd->pw_name,filesize);

    sprintf(themsg, "PID: %ld %s waiting for %lld bytes on ",(long)getpid(),
            pwd->pw_name,filesize);

    for (; *paths; paths++)
    {
        if (strlen(*paths) == 1 && strcmp(*paths,".") == 0)
        {
            fprintf(stream,"%s:",cwd);
            strcat(themsg,cwd);
        }
        else
        {
            fprintf(stream,"%s:",*paths);
            strcat(themsg,*paths);
        }
    }

    fprintf(stream,"\n");
    c_errmsg(themsg," ",&nonfatal);

#ifndef DEBUG
    if ( fclose(stream) != 0 )
    {
        c_errmsg("Error closing stream","dkcre-fclose",&nonfatal);
        return(E_FAIL);
    }
#endif

    (void) endpwent();

    return(E_SUCC);
}

/**
 * c_dkcre:
 * @directories: I: List of pathnames for directories to be searched
 *                  for available space.
 * @filename: I: Pathname of the file to be created.
 * @filesize: I: Size, in bytes, of the file to be created.
 * @wait: I: Number of seconds between retries in searching for available
 *           space.  A value of zero indicates the calling function wants
 *           an immediate return status if the specified disk space is
 *           not available.
 *
 * If the specified filename already exists, c_dkcre() returns a status
 * of DISK_EXISTS.  c_dkcre() selects a directory on a best-fit basis.
 * The amount of available space in the directories specified in 
 * directories is determined, and the directory with the minimum amount
 * of free space that satisfies the filesize request is selected.  If a 
 * NULL value is passed for directories, the environment variable
 * DATAPATH is interrogated and is assumed to be a list of colon-separated
 * directories.  If the DATAPATH environment variable does not exist,
 * c_dkcre() only looks in the current working directory for space.
 * Filename is created in the directory selected.
 *
 * If the directory selected is not the current working directory,     
 * filename.XXXXXX is created, where XXXXXX is replaced with a pattern 
 * to make the filename unique.  Filename is created in the current working
 * directory as a symbolic link to the actual file.  After the best     
 * directory is selected, c_dkcre() allocates the file by writing
 * filesize and filename to a disk reserve.  If there is not enough
 * available space in any of the directories and the calling function
 * desires to wait for it, a message of the form "PID: 12345 username
 * waiting for 1234567 bytes on /usr/production: /usr/production2" is
 * displayed on the operator's console every wait seconds until the
 * process is killed or enough disk space becomes available. 
 *
 * Returns:  E_SUCC:        Successful completion,
 *           E_FAIL:        Failure status,
 *           DISK_EXISTS:   The specified disk already exists,
 *           DISK_NOSPACE:  The disk space specified was not available and
 *                          the calling function did not want to wait for it.
 */
int c_dkcre
(
    char directories[][CMLEN],      /* I/O: Array of directories to search    */
    const char *filename,           /* I: Name of file to be created          */
    long long filesize,             /* I: Size of file to be created          */
    int  wait                       /* I: Wait > 0, wait for space            */
                                    /*    Wait = 0, do not wait for space     */
)
{
    char    **paths;                /* Array of pointers to path names        */
    char    *dirs[MAXPATH];         /* Data paths                             */
    int     i;                      /* Looping variable                       */
    long    nonfatal = 1;           /* Non-fatal error code                   */
    long    status = E_SUCC;        /* Set successful status code             */
    int     rstat;                  /* Return status                          */
    struct stat lstat_buffer;       /* for lstat() results                    */

    /* Check to see if the file already exists.  access() does not check 
       for a symbolic link that points to nowhere.  So use lstat() instead.   */ 
    if (lstat (filename,&lstat_buffer) == 0)
        return(DISK_EXISTS);

    /* Allocate space for the current working directory name                  */
    if ((cwd = malloc(PATHNAMESZ)) == NULL)
    {
        c_errmsg("Error allocating dynamic memory","dkcre-alloc",
                 &nonfatal);
        return(E_FAIL);
    }

    /* Allocate space for the data path name */
    if ((datapath = malloc(PATHNAMESZ)) == NULL)
    {
        c_errmsg("Error allocating dynamic memory","dkcre-alloc",
                 &nonfatal);
        return(E_FAIL);
    }

    /* Get the current working directory     */
    if (getcwd(cwd,PATHNAMESZ) == NULL)
    {
        c_errmsg("Error getting the current working directory","dkcre-cwd",
                 &nonfatal);
        return(E_FAIL);
    }

    if (strrchr(filename,'/') != NULL)
    {
        strcpy(cwd,filename);
        *(strrchr(cwd,'/')) = '\0';
    }

    /* Set an array of pointers to the specified directory paths              */
    if (directories == NULL)
    {
        for (i = 0; i < MAXPATH; i++)
            dirs[i] = NULL;
    }
    else
    {
        for (i = 0; i < MAXPATH; i++)
        {
            if (strcmp(directories[i],"") == 0)
                dirs[i] = NULL;
            else
                dirs[i] = directories[i];
        }
    }

    /* Get the directories to search         */
    paths =  getpaths(dirs);

    if (*paths == NULL)
    {
        /* no valid paths to search          */
        c_lasmsg(FALSE,"dkcre-nopath","No valid paths specified");
        return (E_FAIL);
    }
    /* Try to create the file in one of the best fit directory locations      */
    for (;;)
    {
        rstat = create(search(paths,filesize),filename,filesize);
        if (rstat == E_FAIL)
            return(E_FAIL);          /* errmsg already called */
        if (rstat)
            break;

        /* Don't want to wait so get out of here    */
        if (!wait)
        {
            status = DISK_NOSPACE;
            break;
        }

        /* Send operators message not enough disk space */
        if (sendops(filesize,paths) != E_SUCC)
            return(E_FAIL);

        /* Wait for a specified amount of time      */
        sleep(wait);
    }

    /* Free up some space                           */
    free(cwd);
    free(datapath);
    if (createdpaths)
    {
        free(createdpaths);
        createdpaths=NULL;
    }
            
    if (!dirs)
        free(paths);

    return(status);
}
