/******************************************************************************
FUNCTION:       c_dkdel

PURPOSE:        Delete the specified file

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0                                   Original development
  5.0      05/88    D. Hollaren         added calles to c_errmsg
  5.0      11/88    B. Davis            added sys_errlist, allowed dir deletion
  5.0      10/90    T. Baltzer          fix return status checking, errmsg calls
  5.1      09/91    B. Ailts            fixed a spelling error

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
  Dynamically allocate space for the symbolic link name and file status struct
  Assume the file name is a symbolic link and try to find it
        If an error is found looking for the link
                return an error
        
  Otherwise; get the status information about the symbolic link file
        If an error occurs getting the status
                return an error
        else
                delete the linked file

  Delete the file name
  Free the space for the symbolic link name and file status structure

ALGORITHM REFERENCES    none
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include "las.h"
#include "diskio.h"
#include "local_protos.h"

#define PATHNAMESZ      1024


/**
 * c_dkdel:
 * @filename: I: Specifies the pathname of the file to be deleted.
 *
 * c_dkdel() deletes disk files.  It was written specifically to
 * delete disk files created by c_dkcre().  The file filename is
 * deleted.  If filename was a symbolic link, the file it pointed
 * at is also deleted.  If filename does not exist, c_dkdel() returns
 * a status of DISK_GONE.
 *
 * Returns:  E_SUCC:        Successful completion,
 *           E_FAIL:        Failure status,
 *           DISK_GONE:     The specified file does not exit,
 */
int c_dkdel
(
    const char *filename    /* I: Name of file to be deleted        */
)
{
    int sz;                /* Size of symbolic link name           */
    int status = E_SUCC;   /* Set successful status code           */
    char *linkname;        /* Symbolic link name                   */
    struct stat statp;     /* File status structure                */


    if ((linkname = malloc(PATHNAMESZ)) == NULL)
    {
        c_lasmsg(FALSE,"dkdel-alloc","Error allocating dynamic memory");
        return(E_FAIL);
    }

    /* Assume it is a symbolic link and read it */
    if ((sz = readlink(filename,linkname,PATHNAMESZ)) == -1)
    {
        if (errno == ENOENT)                        /* File does not exist */
        {
            status = DISK_GONE;
        }
        else if (errno != ENXIO && errno != EINVAL) /* Readlink error      */
        {
            c_lasmsg(FALSE,"dkdel-symlink",
                    "Error reading the symbolic link for %s",filename);
            free(linkname);
            return(E_FAIL);
        }
    }
    else                                    /* File is a symbolic link     */
    {
        linkname[sz] = '\0';                /* This is stupid!             */

        if (lstat(linkname,&statp) == -1)   /* Status of the linked file   */
        {
            if (errno != ENOENT)            /* Status error                */
            {
                c_lasmsg(FALSE,"dkdel-stat",
                        "Error getting status of linked file %s",linkname);
                free(linkname);
                return(E_FAIL);
            }
        }
        else if ( unlink(linkname) == -1 )   /* Delete the linked file      */
        {
            c_lasmsg(FALSE,"dkdel-delete","Error deleting the file %s",
                    linkname);
            free(linkname);
            return(E_FAIL);
        }
    }

    free(linkname);
    linkname = NULL;

    if (status != DISK_GONE)
    {
        /* stat the file so the file type can be determined */
        if (stat(filename,&statp) == -1)
        {
            if (errno != ENOENT)
            {
                c_lasmsg(FALSE,"dkdel-stat","Error getting status of file %s",
                         filename);
                return E_FAIL;
            }
        }

        if (S_ISDIR(statp.st_mode))
        {
            /* remove the directory */
            if (rmdir(filename) == -1)
            {
                c_lasmsg(FALSE,"dkdel-dir","Error removing directory %s",
                        filename);
                status = E_FAIL;
            }
        }
        else
        {
            /* Delete the file */
            if (unlink(filename) == -1)
            {
                c_lasmsg(FALSE,"dkdel-delete","Error deleting the file %s", 
                        filename);
                status = E_FAIL;
            }
        }
    }

    return(status);
}
