/******************************************************************************
FUNCTION:       c_dkfree

PURPOSE:        Free reserved space on a given pack

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0                                   Original development
  5.0      05/88    D. Hollaren         Broke into a seperate routine & document
  5.1      10/90    T. Baltzer          Check return status's, add errmsg calls
  5.2      11/90    T. Baltzer          enable recovery if DISKRSVD file gone
  5.5      05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  5.6      09/91    D. Etrheim          Modify to open the DISKRSVD file
                                        here instead of depending on cmlock
                                        to do it correctly. Ran thru Saber-C.
  5.5      04/92    T. Mittan           Modify to close file ptr from cmlock
                                        if DISKRSVD file is opened here.        
  5.6      12/92    D. Etrheim          Remove the O_APPEND flag for opening 
                                        the file.  We want to overwrite the
                                        record, but the append flag puts us to
                                        the end of the file on write.
  5.7      08/94    D. Etrheim          Change 777 to 666 on file open

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
  Get the current process ID
  Initailze the symbolic link name
  Open and lock the disk reserved file
  Loop until the end of the file
        Read each record of the disk reserved file
        If the record is label active
            If the process ID matches
                If the file name matches
                    Flag the record as Done or inactive
                    rewrite the record
  End loop until EOF

  Close and unlock the disk reserved file

ALGORITHM REFERENCES    none
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "las.h"
#include "diskio.h"
#include "util.h"   /* c_sigblk/unblk prototypes */
#include "diskdef.h"

/**
 * c_dkfree:
 * @path: I: Specifies the pathname on which the software reserved
 *           space is to be freed.
 *
 * c_dkfree() frees the software reserved space from the disk
 * reserve file for the specified pathname.  The space is 
 * software reserved when disk files are created by c_dkcre().
 *
 * Returns:  E_SUCC:        Successful completion,
 *           E_FAIL:        Failure status.
 */
int c_dkfree
(
    const char *path        /* I: Pathname to unreserve space for*/
)
{
    struct DISKRESERVE rsv_rec;     
    struct stat statp;      /* Status information structure for path*/
    int thispid;            /* This process id                      */
    int status;             /* Status varilable                     */
    int readstat;           /* Read status variable                 */
    int rfd;                /* File descriptor for the read         */
    int i;                  /* Looping variable                     */
    int fstat = E_SUCC;     /* Function status                      */
    long nonfatal = 1;      /* Non-fatal error code                 */
    char linkname[255];     /* Link name                            */
    char msg[ERRLEN];       /* Error message string                 */
    int existflg;           /* Does file exist status flag          */
    int sig_mask;
    off_t save_offset;      /* Saved file offset */

    c_sigblk(&sig_mask);

    /* Get the process ID                        */
    thispid = getpid();

    /* Initailize the link name                  */
    for (i=0; i<255; i++)
        linkname[i] = 0;

    /* Get the symbolic link                     */
    status = readlink(path, linkname, 255);
    if (status <  0)
    {       
        /* it probably was not a link */
        strcpy(linkname, path); /* so use path*/
    }
    else
    {
        char dir[CMLEN];
        char root[CMLEN];
        char ext[CMLEN];
          
        /* Make sure the link is a full path.  If it isn't, attach
           the original path to it */
        if (linkname[0] != '/')
        {
            c_splitname(path,dir,root,ext);
            strcat(dir,linkname);
            strcpy(linkname,dir);
        }
            
    }

    /* Determine whether the DISKRSVD file already exists */
    existflg = stat(DISKRSVD,&statp);

    /* Open and lock DISKRSVD file */
    if ( cmlock(DISKRSVD,&rfd,O_CREAT | O_RDWR,600) != E_SUCC)
    {
        sprintf(msg,"Error locking file %s",DISKRSVD);
        c_errmsg(msg,"dkfree-lock",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    if ( existflg < 0 )             /* File was just created */
    {
        if ( chmod(DISKRSVD, 0666) == -1 )
        {
            sprintf(msg,"Error changing file permissions for %s",DISKRSVD);
            c_errmsg(msg,"dkfree-chmod",&nonfatal);
            c_sigunblk(&sig_mask);
            return(E_FAIL);
        }
    }

    /* While not eof do */
    /* Read rsv_rec */
    do
    {
        /* Read the rsv_record      */
        save_offset = lseek(rfd,0,SEEK_CUR);
        readstat = read(rfd, &rsv_rec, sizeof(struct DISKRESERVE));
        if (readstat == -1)
        {
            c_errmsg("Error reading record","dkfree-read",&nonfatal);
            fstat = E_FAIL;
        }
        if (readstat > 0)
        {
            /* Found active record                  */
            if (rsv_rec.active == 'A')
            {
                /* Check the process ID             */
                if (thispid == rsv_rec.ppid)
                {
                    /* Check the file name          */
                    if (strcmp(linkname ,rsv_rec.filename) == 0)    
                    {
                        /* Flag the record as Done  */
                        rsv_rec.active = 'D';

                        /* Rewrite this record */
                        status = lseek(rfd,save_offset,SEEK_SET);
                        if ( status == -1 )
                        {
                            c_errmsg("Error seeking in file","dkfree-seek",
                                      &nonfatal);
                            c_sigunblk(&sig_mask);
                            return(E_FAIL);
                        }
                        status = write(rfd, &rsv_rec,
                                       sizeof(struct DISKRESERVE));
                        if ( status == -1 )
                        {
                            c_errmsg("Error writing to file","dkfree-write",
                                      &nonfatal);
                            c_sigunblk(&sig_mask);
                            return(E_FAIL);
                        }
                    }
                }
            }
        }
    } while (readstat > 0);
    /* End while not eof */

    /* Close and unlock /tmp/diskrsvd */
    if ( close(rfd) == -1 )
    {
        c_errmsg("Error closing file","dkfree-close",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    c_sigunblk(&sig_mask);
    return(fstat);
}
