/******************************************************************************
FUNCTION:       c_dkresv

PURPOSE:        Reserves space on a given pack

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 1.0                                    Original development
 5.0       05/88    D Hollaren          added calles to cerrmsg
 5.1       10/90    T. Baltzer          check return status's, add errmsg calls
 5.2       11/90    T. Baltzer          enable recovery if DISKRSVD file gone
 5.5       05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
 5.6       09/91    D. Etrheim          Modify to open the DISKRSVD file
                                        here instead of depending on cmlock
                                        to do it correctly. Ran thru Saber-C.
 5.7       04/92    D. Etrheim          Modify to close file ptr from cmlock
                                        if DISKRSVD file is opened here.
 5.8       08/94    D. Etrheim          Change 777 to 666 on file open
 5.9       05/95    K. Gacke            Added dev field to DISKRESERVE struct
                                        so that we do not need to stat the file
                                        while checking for space in c_dkrspac.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
  Get the current process id
  Initailize the file name
  Fill the disk reserve file record with the path name, process id, size, and
  active flag.
  open and lock the disk reserve file
  write the record
  close and unlock the disk reserve file

ALGORITHM REFERENCES    none
******************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>   /* Kill prototype */
#include "las.h"
#include "diskio.h"
#include "util.h"     /* c_sigblk/unblk prototype */
#include "diskdef.h"


/**
 * c_dkresv:
 * @path: I: Pathname to reserve space for.
 * @size: I: Size of file to reserve space for.
 *
 * The c_dkresv() routine reserves space on a given pack for the pathname.
 * It follows the algorithm of obtaining the current process id,    
 * initializing the file name, and filling the disk reserve file record
 * with the path name, process id, size, and active flag.
 *
 * Returns:  E_SUCC:        Successful completion,
 *           E_FAIL:        Failure status,
 */
int c_dkresv
(
    const char *path,           /* I: Pathname to reserve space for        */
    long long size              /* I: Size of the file to reserve space for*/
)
{
    struct DISKRESERVE rsv_rec; /* Software reserve file structure      */
    struct stat statp;          /* Status information structure for path*/
    int status;                 /* Status variable                      */
    int existflg;               /* does file exist status flag          */
    int rfd;                    /* File descriptor                      */
    long nonfatal = 1;          /* Non-fatal error code                 */
    char msg[ERRLEN];           /* Error message string                 */
    int sig_mask;
    char linkname[CMLEN];       /* Link name                            */
        
    c_sigblk(&sig_mask);

    /* Clear out the disk reservation structure */
    memset (&rsv_rec, 0, sizeof(struct DISKRESERVE));

    /* Get the process ID           */
    rsv_rec.ppid = getpid();

    /* Check if the path is a link */
    memset(linkname,0,CMLEN);
    status = readlink(path, linkname, CMLEN);
    if (status <  0)
    {       
        /* It probably was not a link */
        strcpy(linkname, path); /* so use path*/
    }
    else
    {
        char dir[CMLEN];
        char root[CMLEN];
        char ext[CMLEN];
            
        /* Make sure the link is a full path.  If it isn't, attach
           the original path to it */
        if (linkname[0] != '/')
        {
            c_splitname(path,dir,root,ext);
            strcat(dir,linkname);
            strcpy(linkname,dir);
        }
            
    }
        
    /* Set the structure for the disk reserve file record   */
    strcpy(rsv_rec.filename, linkname);
    rsv_rec.required_space = size;
    rsv_rec.active = 'A';
    status = stat(linkname,&statp);
    if (status != 0)
    {
        sprintf(msg,"Error getting status of %s",linkname);
        c_errmsg(msg,"dkresv-stat",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }
    rsv_rec.dev = statp.st_dev;

    /* Determine whether the DISKRSVD file already exists */
    existflg = stat(DISKRSVD,&statp);

    /* Open write lock DISKRSVD  */
    if ( cmlock(DISKRSVD,&rfd,O_CREAT | O_RDWR,600) != E_SUCC)
    {
        sprintf(msg,"Error locking file %s",DISKRSVD);
        c_errmsg(msg,"dkresv-lock",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }
       
    if ( existflg < 0 )             /* file was just created */
    {
        if ( chmod(DISKRSVD, 0666) == -1 )
        {
            sprintf(msg,"Error changing file permissions for %s",DISKRSVD);
            c_errmsg(msg,"dkresv-chmod",&nonfatal);
            c_sigunblk(&sig_mask);
            return(E_FAIL);
        }
    }

    /* Find an unused record to fill in */
    {
        struct DISKRESERVE search;
        off_t search_offset;
            
        do
        {
            search_offset = lseek(rfd,0,SEEK_CUR);
            status = read(rfd, &search, sizeof(struct DISKRESERVE));
            if (status > 0)
            {
                if ((search.active == 'D') || (kill(search.ppid,0) == -1))
                {
                    lseek(rfd,search_offset,SEEK_SET);
                    break;
                }
            }
        } while (status > 0);
    }    

    /* Write structure rsv_rec */
    status = write(rfd, &rsv_rec, sizeof(struct DISKRESERVE));
    if ( status == -1 )
    {
        c_errmsg("Error writing to file", "dkresv-write", &nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    /* Close unlock file /tmp/diskrsvd */
    if ( close(rfd) == -1 )
    {
        c_errmsg("Error closing file", "dkresv-close", &nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    c_sigunblk(&sig_mask);
    return(E_SUCC);
}
