/******************************************************************************
FUNCTION:       c_dkrspac

PURPOSE:        Returns how much space is presently software reserved

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0                                   Original development
  5.0      05/88    D Hollaren          added c_errmsg and las include
  5.1      03/89    D Hollaren          Now returns the correct amount of space
                                        used
  5.2      10/90    T. Baltzer          check return status's, add errmsg calls
  5.3      11/90    T. Baltzer          enable recovery if DISKRSVD file gone
  5.5      05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  5.6      09/91    D. Etrheim          Modify to open the DISKRSVD file
                                        here instead of depending on cmlock
                                        to do it correctly. Ran thru Saber-C.
  5.7      04/92    D. Etrheim          Modify to close file ptr from cmlock
                                        if DISKRSVD file is opened here.
  5.8      05/92    T. Mittan           Declare thisdev as an int for DG.
                                        The corresponding sturcture element
                                        statp.st_dev is defined as an int on DG.
  5.9      12/93    D. Etrheim          Modify calling sequence for cmlock so
                                        this is backword compatible for 
                                        Solaris 2.3.
  5.7      08/94    D. Etrheim          Change 777 to 666 on file open
  5.8      11/94    D. Etrheim          Added error message if stat call fails
                                        Declare thisdev as an int for SG.
                                        This evidently changed when the OS
                                        was upgraded to 5.x.  
  5.9      05/95    K. Gacke            Added dev field to DISKRESERVE struct
                                        so that we do not need to stat the file
                                        while checking for space in c_dkrspac.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Get status information on the specified file
Get the device the file is on
Open and lock the software disk reserve file

Read each record of the file
    If the record has an active file
        get the device the file is on from the status information
        if the device is the same as the device for the input file
            accumilate the software reserved space on that device
read next record until End Of File    

Close and unlock the software disk reserve file

ALGORITHM REFERENCES    none
******************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include "las.h"
#include "diskio.h"
#include "util.h"   /* c_sigblk/unblk prototypes */
#include "diskdef.h"

static int inmix
(
    pid_t pid
)
{
    int status;

    status = kill(pid,0);
    if (status == 0)
        return(pid);
    else if (errno != ESRCH)
        return(pid);
    return(0);
}


/**
 * c_dkrspac:
 * @path: I: Specifies the pathname to determine how much software
 *           reserved space has been allocated in the specified path.
 *
 * c_dkrspac() determines how much space is software reserved in a 
 * specified path.  It was written specifically to return the software
 * reserved space in the disk reserve file.  The space is software
 * reserved when disk files are created by c_dkcre().
 *
 * Returns:  Number of bytes that have been software reserved.
 *           E_FAIL:  Failure status.
 */
long long c_dkrspac
(
    const char *path        /* I: Pathname to determine free space       */
)
{
    struct DISKRESERVE rsv_rec;
    struct stat statp;      /* Status information structure for path     */
    long long rsvd;         /* Space that is reserved                    */
    long nonfatal = 1;      /* Nonfatal error code                       */
    int  rfd;               /* Read file descriptor                      */
    int  readstat;          /* Read status variable                      */
    dev_t thisdev;          /* The current device                        */
    int instat;             /* Integer return status                     */
    char msg[ERRLEN];       /* Error message string                      */
    int existflg;           /* Does file exist status flag               */
    int sig_mask;


    c_sigblk(&sig_mask);
    /* Get the status information about the pathname                     */
    if (stat(path,&statp) < 0)
    {
        c_errmsg("File doesn't exist","dkrspac-exist",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    thisdev = statp.st_dev;
    rsvd = 0;

    /* Determine whether the DISKRSVD file already exists */
    existflg = stat(DISKRSVD,&statp);


    /* Open and lock /tmp/diskrsvd */
    if ( cmlock(DISKRSVD,&rfd,O_CREAT | O_RDWR,600) != E_SUCC)
    {
        sprintf(msg,"Error locking file %s",DISKRSVD);
        c_errmsg(msg,"dkrspac-lock",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    if ( existflg < 0 )             /* file was just created */
    {
        if ( chmod(DISKRSVD, 0666) == -1 )
        {
            sprintf(msg,"Error changing file permissions for %s",DISKRSVD);
            c_errmsg(msg,"dkrspac-chmod",&nonfatal);
            c_sigunblk(&sig_mask);
            return(E_FAIL);
        }
    }

    /* While not eof do */
    do
    {
        /* read a the rsv_record */
        readstat = read(rfd, &rsv_rec, sizeof(struct DISKRESERVE));
        if (readstat == -1)
        {
            c_errmsg("Error reading from file","dkrspac-read",&nonfatal);
            c_sigunblk(&sig_mask);
            return(E_FAIL);
        }
        if (readstat > 0)
        {
            /* Check to see if this is an active record       */
            if ((rsv_rec.active == 'A') && (kill(rsv_rec.ppid,0) == 0))
            {
                if (rsv_rec.dev == thisdev)    
                {
                    /* Add up the space required       */
                    if ((instat = inmix(rsv_rec.ppid)) == E_FAIL)
                    {
                        c_errmsg("Error determining pid","dkrspac-inmix", 
                                  &nonfatal);
                        c_sigunblk(&sig_mask);
                        return(E_FAIL);
                    }
                    if (instat)
                        rsvd += rsv_rec.required_space;
                }
            } 
        }
    } while (readstat > 0);
    /* End while not eof */

    /* Close and unlock /tmp/diskrsvd */
    if ( close(rfd) == -1 )
    {
        c_errmsg("Error closing file","dkrspac-close",&nonfatal);
        c_sigunblk(&sig_mask);
        return(E_FAIL);
    }

    c_sigunblk(&sig_mask);
    return(rsvd);
}


