/******************************************************************************
FUNCTION:	GET_DISK_FD
  
PURPOSE:	Retrieve the current attributes of the DITTO input/output
		disk file.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      11/85    K. Gacke		initial development
  5.0      10/88    D. Hollaren		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE.  It is also assumed TAE has been initialized and
	running under the DITTO program (it uses several TAE global
	variables defined by the DITTO program).

PROJECT			LAS

ALGORITHM 
	From the input parameter "which", it is determined which DITTO device
	is to be processed.  Error checking is done to make sure the DITTO 
	device has been allocated to a disk file.  The disk file attributes
	are retrieved from TAE parameter block and assigned to the appropriate
	fields of the DITTO disk file descriptor.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "ditto.h"			/* ditto include file                */

int get_disk_fd
(
    char *which,			/* DITTO input/output device         */
    struct DISK_FD *disk 		/* disk descriptor                   */
)
{
extern struct DITTO_DEV ditto_device;
extern struct PARBLK vblock;		/* TAE parameter block               */

struct VARIABLE *val;			/* prointer to TAE variable          */

long  status;				/* function return status code       */

char *device;				/* DITTO input/output device         */

/*
  Input device
*/
if (strcmp(which,"INPUT") == 0)
   {
   /*
     Insure the DITTO device has been allocated to a disk file
   */
   ditto_device.in_disk = disk;
   ditto_device.input_dev = "DISK";

   /*
     Retrieve the input device
   */
   val = p_find(&vblock,"idevice");	
   if (val->v_count == 0)
      {
      ditto_error(which,DITTO_DALLOC,ERROR_WARN,NO_UPDATE_FD);
      return(E_FAIL);
      }
   device = SVAL(*val,0);

   /*
     Make sure the input device is a disk file
   */
   if (strcmp(device,"DISK") != 0)
      {
      ditto_error(which,DITTO_DALLOC,ERROR_WARN,UPDATE_FD);
      return(E_FAIL);
      }

   /*
     Retrieve disk host file name
   */
   val = p_find(&vblock,"ihost");
   strcpy(disk->fname,SVAL(*val,0));

   /*
     Retrieve block size
   */
   val = p_find(&vblock,"iblksiz");
   if (val->v_count == 0)
      disk->blksiz = NOVAL;
   else
      disk->blksiz = IVAL(*val,0);

   /*
     Retrieve record size
   */
   val = p_find(&vblock,"irecsiz");
   if (val->v_count == 0)
      disk->recsiz = NOVAL;
   else
      disk->recsiz = IVAL(*val,0);

   /*
     Retrieve record type
   */
   val = p_find(&vblock,"irectyp");
   if (val->v_count == 0)
      disk->rectyp = NOVAL; 
   else if (strcmp(SVAL(*val,0),"FIX") == 0)
      disk->rectyp = FIX;
   else if (strcmp(SVAL(*val,0),"VAR") == 0)
      disk->rectyp = VAR;

   /*
     Retrieve type of carrage control
   */
   val = p_find(&vblock,"icc");
   if (val->v_count == 0)
      disk->cc = NOVAL;
   else if (strcmp(SVAL(*val,0),"CARRIAGE") == 0)
      disk->cc = CARRIAGE;
   else if (strcmp(SVAL(*val,0),"FORTRAN") == 0)
      disk->cc = FORTRAN;
   else if (strcmp(SVAL(*val,0),"NONE") == 0)
      disk->cc = NONE;
   else
      disk->cc = PRINT;

   /*
     Retrieve current record
   */
   val = p_find(&vblock,"irec");
   disk->cur_rec = IVAL(*val,0);

   /*
     Retrieve current file
   */
   val = p_find(&vblock,"ifile");
   disk->cur_file = IVAL(*val,0);

   /*
     Retrieve disk file attributes
   */
   status = dstat(disk,0);
   if (status == E_FAIL)
      {
      ditto_error(which,DITTO_DSTAT,ERROR_WARN,UPDATE_FD);
      return(E_FAIL);
      }
   }

else				/* Output device		*/
   {
   ditto_device.out_disk = disk;
   ditto_device.output_dev = "DISK\0"; 

   /*
     Retrieve the output device
   */
   val = p_find(&vblock,"odevice");   	
   if (val->v_count == 0)
      {
      ditto_error(which,DITTO_DALLOC,ERROR_WARN,NO_UPDATE_FD);
      return(E_FAIL);
      }
   device = SVAL(*val,0);

   /*
     Insure the device is a disk file
   */
   if (strcmp(device,"DISK") != 0)
      {
      ditto_error(which,DITTO_DALLOC,ERROR_WARN,UPDATE_FD);
      return(E_FAIL);
      }

   /*
     Retrieve disk host file name
   */
   val = p_find(&vblock,"ohost");
   strcpy(disk->fname,SVAL(*val,0));

   /*
     Retrieve block size
   */
   val = p_find(&vblock,"oblksiz");
   if (val->v_count == 0)
      disk->blksiz = NOVAL;
   else
      disk->blksiz = IVAL(*val,0);

   /*
     Retrieve record size
   */
   val = p_find(&vblock,"orecsiz");
   if (val->v_count == 0)
      disk->recsiz = NOVAL;
   else
      disk->recsiz = IVAL(*val,0);

   /*
     Retrieve record type
   */
   val = p_find(&vblock,"orectyp");
   if (val->v_count == 0)
      disk->rectyp = NOVAL;
   else if (strcmp(SVAL(*val,0),"FIX") == 0)
      disk->rectyp = FIX;
   else if (strcmp(SVAL(*val,0),"VAR") == 0)
      {
      disk->rectyp = VAR;
      disk->blksiz = disk->recsiz;
      }

   /*
     Retrieve type of carriage control
   */
   val = p_find(&vblock,"occ");
   if (val->v_count == 0)
      disk->cc = NOVAL;
   else if (strcmp(SVAL(*val,0),"CARRIAGE") == 0)
      disk->cc = CARRIAGE;
   else if (strcmp(SVAL(*val,0),"FORTRAN") == 0)
      disk->cc = FORTRAN;
   else if (strcmp(SVAL(*val,0),"NONE") == 0)
      disk->cc = NONE;
   else
      disk->cc = PRINT;

   /*
     Retrieve current record
   */
   val = p_find(&vblock,"orec");
   disk->cur_rec = IVAL(*val,0);

   /*
     Retrieve current file
   */
   val = p_find(&vblock,"ofile");
   disk->cur_file = IVAL(*val,0);

   /*
     Retrieve disk file attributes
   status = dstat(disk,0);
   */
   }

return(E_SUCC);
}

/******************************************************************************
FUNCTION:	PUT_DISK_FD

PURPOSE:	Update the current attributes of the DITTO input/output disk
		file.

ALGORITHM 
	From the input parameter "which", it is determine which DITTO device
	is to be processed.  The attributes from the disk file descriptor are
	written back into the TAE parameter block.

ALGORITHM REFERENCES	none
*******************************************************************************/
void put_disk_fd
(
    char *which,			/* DITTO input/output device         */
    struct DISK_FD *disk 		/* disk descriptor                   */
)
{
extern struct PARBLK vblock;		/* TAE parameter block               */

char *cc[1];				/* carriage control                  */
char *device[1];			/* DITTO device (disk)               */
char *fname[1];				/* disk file name                    */
char *rectyp[1];			/* record type                       */

device[0] = "DISK";			/* device is disk                    */
fname[0] = disk->fname;			/* assign file name                  */

/*
  Assign record type
*/
switch (disk->rectyp)
   {
   case FIX:      rectyp[0] = "FIX";    break;

   case VAR:      rectyp[0] = "VAR"; break;
   }

/*
  Assign carriage control
*/
switch (disk->cc)
   {
   case CARRIAGE:  cc[0] = "CARRIAGE";  break;

   case FORTRAN:   cc[0] = "FORTRAN";   break;

   case PRINT:     cc[0] = "PRINT";     break;

   case NONE:      cc[0] = "NONE";      break;
   }

/*
  Input device
*/
if (strcmp(which,"INPUT") == 0)
   {
   /*
     Update global variables
   */
   q_string(&vblock,"idevice",1,device,P_UPDATE);
   q_string(&vblock,"ihost",1,fname,P_UPDATE);

   if (disk->rectyp != NOVAL)
      q_string(&vblock,"irectyp",1,rectyp,P_UPDATE);

   if (disk->cc != NOVAL)
      q_string(&vblock,"icc",1,cc,P_UPDATE);

   q_intg(&vblock,"irec",1,&disk->cur_rec,P_UPDATE);
   q_intg(&vblock,"ifile",1,&disk->cur_file,P_UPDATE);
   }
else				/* Output device		*/
   {
   /*
     Update the global variables
   */
   q_string(&vblock,"odevice",1,device,P_UPDATE);
   q_string(&vblock,"ohost",1,fname,P_UPDATE);
   if (disk->rectyp != NOVAL)
      q_string(&vblock,"orectyp",1,rectyp,P_UPDATE);

   if (disk->cc != NOVAL)
      q_string(&vblock,"occ",1,cc,P_UPDATE);

   q_intg(&vblock,"orec",1,&disk->cur_rec,P_UPDATE);
   q_intg(&vblock,"ofile",1,&disk->cur_file,P_UPDATE);
   }

/*
  Write the new parameter block
*/
q_out(&vblock);

return;
}
