/******************************************************************************
FUNCTION:	DSTAT

PURPOSE:	Part of the DITTO program which retrieves disk file attributes
		from the operating system.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      12/85    K. Gacke		initial development
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1 	    9/92    T. Mittan		Removed register declaration from three
					variables to eliminate warnings on 
					compile

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE.  It is also assumed that this routine is used
	only by the DITTO program.

PROJECT			LAS

ALGORITHM 
	From the disk file descriptor and mode it is determined if the file
	exists.  If it does not, a bad status code is returned.  The function
	"stat" is called to retrieve the file attributes from the operating
	system.  On UNIX it is also required to try to decide the type of file
	it is to retrieve the type of carriage control.

	If running under the VMS operating system, all files are opened using
	the file descriptor.  The file attribute list is built based on the
	input parameter "disk".

	If running under the UNIX operating system, files with carriage return
	carriage control are opened by file pointer (buffered I/O), otherwise
	it is opened by file descriptor.


ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <string.h>

#ifdef vms
#include <stat.h>			/* stat include file for VMS         */
#else
#include <sys/types.h>			/* stat include file for UNIX        */
#include <sys/stat.h>			/* stat include file for UNIX        */
#endif

#include "las.h"			/* LAS include file 		     */
#include "ditto.h"			/* DITTO include file                */

#ifndef vms
int dstat
(
	struct DISK_FD *fd, /* DITTO disk file descriptor        */
	long  mode          /* access mode                       */
)
{
struct stat stat_buf;			/* stat buffer                       */

FILE	*fp;				/* file pointer                      */

char	file_name[CMLEN];		/* file name                         */
char    fname[L_tmpnam];		/* temporary file name               */

char  name[CMLEN];			/* file name                         */
char  syscmd[CMLEN];			/* UNIX system command               */
char  type[CMLEN];				/* type of file                      */

char	 ext[CMLEN];			/* Extention location		     */
char	*adir;				/* address of directory within fname */
char	*aext;				/* address of directory within fname */
char 	*tptr;                    /* ptr to current position in file name */

/*
  Check to see if file exists
*/
if (access(fd->fname,mode) == 0)
   {
   strcpy(file_name,fd->fname);
   fd->cur_file = NOVAL;
   }
else
   {
   strcpy(file_name,fd->fname);
   adir = strrchr(file_name,HOST_DIR_END);
   if (adir == 0)
	adir = file_name;
   aext = strchr(adir,HOST_EXT);

   if (fd->cur_file == NOVAL)
       fd->cur_file = 1;

   if (aext == 0)
	sprintf(file_name,"%s_%d",fd->fname,fd->cur_file);
   else
	{
	strcpy(ext,aext);
	sprintf(aext,"_%d%s",fd->cur_file,ext);
	}

   if (access(file_name,mode) == -1)
      return(E_FAIL);
   }

/*
  Retrieve file attributes
*/
if (stat(file_name,&stat_buf) == -1)
   return(E_FAIL);

/*
  Assign block size if no value was specified
*/
if (fd->blksiz == NOVAL)
#ifdef sgi
   fd->blksiz = 8192;
#else
#ifdef DGUX
   fd->blksiz = 8192;
#else
   fd->blksiz = stat_buf.st_blksize;
#endif
#endif

/*
  Assign record size if no value was specified
*/
if (fd->recsiz == NOVAL)
   fd->recsiz = fd->blksiz;

if (fd->cc == NOVAL)			/* if carriage control = NOVAL then   */
   {					/*   try to determine from the UNIX   */
   tmpnam(fname);			/*   system command "file"            */
   sprintf(syscmd,"file %s >>%s",file_name,fname);
   system(syscmd);

   /*
     Open the file
   */
   fp = fopen(fname,"r");
   fscanf(fp,"%s%s",name,type);
   fclose(fp);
   unlink(fname);

   if ((strcmp(type,"character") == 0) || (strcmp(type,"c") == 0)      ||
       (strcmp(type,"fortran") == 0)   || (strcmp(type,"roff") == 0)   || 
       (strcmp(type,"English") == 0)   || (strcmp(type,"ascii") == 0))
	   fd->cc = CARRIAGE;
   else
	   fd->cc = NONE;

/* If the file is a LAS image, we need to reset the carriage control.  These 
   systems return a different value from the UNIX system command "file". 
   The above if statement may need to be refined for these systems and other
   System V UNIX platforms for other file types. */
#ifdef DGUX
  tptr = strrchr(name,HOST_EXT);
  if (tptr != NULL)
     {
     tptr++;
     if (strncmp(tptr,"img",3) == 0)
        fd->cc = NONE;
     }
#else
#ifdef sgi
  tptr = strrchr(name,HOST_EXT);
  if (tptr != NULL)
     {
     tptr++;
     if (strncmp(tptr,"img",3) == 0)
        fd->cc = NONE;
     }
#endif
#endif

   }

return(E_SUCC);
}
#else

int dstat
(
	struct DISK_FD *fd, /* DITTO disk file structure         */
	long  mode          /* access mode                       */
)
{
struct stat stat_buf;			/* stat buffer                       */

char     file_name[CMLEN];		/* file name                         */
char     msg[ERRLEN];			/* message buffer                    */
char	 ext[CMLEN];			/* Extention location		     */
char	*adir;				/* address of directory within fname */
char	*aext;				/* address of directory within fname */

long     status;			/* function return status code       */

if ((status = access(fd->fname,mode)) != -1)  /* check to see if file exists */
   {
   strcpy(file_name,fd->fname);
   fd->cur_file = NOVAL;
   }
else
   {
   strcpy(file_name,fd->fname);
   adir = strrchr(file_name,HOST_DIR_END);
   if (adir == 0)
	adir = file_name;
   aext = strchr(adir,HOST_EXT);

   if (fd->cur_file == NOVAL)
       fd->cur_file = 1;

   if (aext == 0)
	sprintf(file_name,"%s_%d",fd->fname,fd->cur_file);
   else
	{
	strcpy(ext,aext);
	sprintf(aext,"_%d%s",fd->cur_file,ext);
	}

   if ((status = access(file_name,mode)) == -1)
      return(E_FAIL);
   }

if ((status = stat(file_name,&stat_buf)) == -1)/* retrieve file attributes   */
   return(E_FAIL);

if (stat_buf.st_fab_mrs == 0)		/* KLUDGE -- does not obtain length  */
   stat_buf.st_fab_mrs = 512;		/*  of variable length records       */

if (fd->blksiz == NOVAL)		/* assign record and block sizes     */
   fd->blksiz = stat_buf.st_fab_mrs;

if (fd->recsiz == NOVAL)
   fd->recsiz = fd->blksiz;

switch (stat_buf.st_fab_rfm)		/* assign record format              */
   {
   case 1:
 	if (fd->rectyp == VAR)
	   ditto_error(" ",DITTO_DFIX,ERROR_WARN,NO_UPDATE_FD);
   	fd->rectyp = FIX;
	break;

   case 2:
 	if (fd->rectyp == FIX)
	   ditto_error(" ",DITTO_DVAR,ERROR_WARN,NO_UPDATE_FD);
      	fd->rectyp = VAR;
 	break;

   case 5:
  	if (fd->rectyp == NOVAL)
	   fd->rectyp = VAR;
	break;

   default:
	sprintf(msg,"Record Format = %d",stat_buf.st_fab_rfm);
	m_msg(msg,"DSTAT");
	break;
   }

switch (stat_buf.st_fab_rat)		/* assign carriage control           */
   {
   case 0:
   case 1:
	fd->cc = NONE;
	break;

   case 2:
	fd->cc = CARRIAGE;
	break;

   default:
	sprintf(msg,"Record Attributes = %d",stat_buf.st_fab_rat);
	m_msg(msg,"DSTAT");
	break;
   }

return(E_SUCC);
}
#endif
