/******************************************************************************
FUNCTION:	DITTO_ERROR

PURPOSE:	Called by DITTO modules to process error messages.  It
		output the error message to the standard output.  It also
		determines if processing is to continue by the sevrity
		code as follows:

		ERROR_WARN:   a non-fatal error in which processing continues
		ERROR_VERIFY: if error flag is set to "yes" then the user is
			      queried to see if processing is to continue
		ERROR_SEV:    a severe error in which processing is aborted.
			      The DITTO input/output device descriptors are
			      written to the TAE parameter block, and are
			      closed prior to terminating the process.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      10/85    K. Gacke		initial development
  5.0      10/88    D. Hollaren		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE.  It is also assumed that this routine is used
	only by the DITTO program.

PROJECT			LAS

ALGORITHM 
	It is first determined what severity the error is.  If the severity
	of the error is severe and the update flag is set, then the input
	and output DITTO device descriptors are updated and the devices 
	closed.  The appropriate error message is outputted to the standard
	output.  If the TAE global variable "errmod" is set to "yes", the
	user is queried to see if processing should continue.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */

void ditto_error
(
    char *which,			/* which device (input or output)    */
    long err_code,			/* error code                        */
    long severity,			/* error severity		     */
    long update			/* attribute update flag             */
)
{
extern struct PARBLK vblock;		/* TAE parameter block               */
extern struct DITTO_DEV ditto_device;	/* pointer to DITTO I/O devices      */

struct VARIABLE *val;			/* pointer to TAE variable           */

int 	term;				/* terminator code for terminal I/O  */
long	vrity;				/* actual severity code              */
long	nonfatal = 1;			/* Non-fatal error code		     */
long	fatal = -1;			/* Fatal error code		     */

char    msg[ERRLEN];			/* error message buffer              */
char	*errmod;			/* error flag                        */

val = p_find(&vblock,"errmode");
errmod = SVAL(*val,0);
switch(severity)			/* determine severity of error       */
   {					/*    1 --> warning                  */
   case ERROR_SEV: 			/*   -1 --> severe error             */
      vrity = -1;
      break;

   case ERROR_VERIFY:
      if (strcmp(errmod,"QUERY") == 0)
         vrity = 1;
      else 
	 vrity = -1;
      break;

   case ERROR_WARN:
      vrity = 1;
      break;
   }

/*
*   If severe error, then update the DITTO input/output attributes and close
*   the devices.
*/
if ((vrity == -1) && (update == UPDATE_FD))
   {
   if ((ditto_device.input_dev != NULL) && 
       (strcmp(ditto_device.input_dev,"DISK") == 0))
        	put_disk_fd("INPUT\0",ditto_device.in_disk);

   if ((ditto_device.output_dev != NULL) && 
       (strcmp(ditto_device.output_dev,"DISK") == 0))
      		put_disk_fd("OUTPUT\0",ditto_device.out_disk);

   if ((ditto_device.input_dev != NULL) &&
       (strcmp(ditto_device.input_dev,"TAPE") == 0))
      		close_tape("INPUT\0",ditto_device.in_tape,NOREW,1);

   if ((ditto_device.output_dev != NULL) &&
       (strcmp(ditto_device.output_dev,"TAPE") == 0))
      		close_tape("OUTPUT\0",ditto_device.out_tape,NOREW,1);
   }

/*
*   Based on error code, output the appropriate error message with the severity
*   determined above.
*/
switch(err_code)
   {
   case DITTO_DOUBLE:
      sprintf(msg,"DITTO %s device has already been allocated",which);
      c_errmsg(msg,"ditto-double",&vrity);
      break;

   case DITTO_DSTAT:
      sprintf(msg,"Error retrieving attributes of %s disk file",which);
      c_errmsg(msg,"ditto-dstat",&vrity);
      break;

   case DITTO_DFIX:
      sprintf(msg,"Input disk file does not have fixed length records");
      c_errmsg(msg,"ditto-dfix",&vrity);
      break;

   case DITTO_DVAR:
      sprintf(msg,"Input disk file does not have variable length records");
      c_errmsg(msg,"ditto-dvar",&vrity);
      break;

   case DITTO_TPOPEN:
      sprintf(msg,"Error mounting DITTO %s tape",which);
      c_errmsg(msg,"ditto-tpopen",&vrity);
      break;

   case DITTO_TPCLOS:
      sprintf(msg,"Error closing DITTO %s tape",which);
      c_errmsg(msg,"ditto-tpclos",&vrity);
      break;

   case DITTO_TPREOP:
      sprintf(msg,"Error reopening DITTO %s tape",which);
      c_errmsg(msg,"ditto-tpreop",&vrity);
      break;

   case DITTO_NALLOC:
      sprintf(msg,"The DITTO %s device was not allocated",which);
      c_errmsg(msg,"ditto-nalloc",&vrity);
      break;

   case DITTO_ALLOC:
      sprintf(msg,"The DITTO %s device is not allocated",which);
      c_errmsg(msg,"ditto-alloc",&vrity);
      break;

   case DITTO_CALLOC:
      sprintf(msg,"Error allocating dynamic buffer space");
      c_errmsg(msg,"ditto-calloc",&vrity);
      break;

   case DITTO_DOPEN:
      sprintf(msg,"Error opening DITTO %s disk file",which);
      c_errmsg(msg,"ditto-dopen",&vrity);
      break;

   case DITTO_DREAD:
      sprintf(msg,"Error reading DITTO %s disk file",which);
      c_errmsg(msg,"ditto-dread",&vrity);
      break;

   case DITTO_DCLOSE:
      sprintf(msg,"Error closing DITTO %s disk file",which);
      c_errmsg(msg,"ditto-dclose",&vrity);
      break;

   case DITTO_DWRITE:
      sprintf(msg,"Error writing record to DITTO %s disk file",which);
      c_errmsg(msg,"ditto-dwrite",&vrity);
      break;

   case DITTO_DALLOC:
      sprintf(msg,"DITTO %s device is not allocated to a disk file",which);
      c_errmsg(msg,"ditto-dalloc",&vrity);
      break;

   case DITTO_TPREAD:
      sprintf(msg,"Error reading DITTO %s tape",which);
      c_errmsg(msg,"ditto-tpread",&vrity);
      break;

   case DITTO_TPWRIT:
      sprintf(msg,"Error writing DITTO %s tape",which);
      c_errmsg(msg,"ditto-tpwrit",&vrity);
      break;

   case DITTO_TALLOC:
      sprintf(msg,"DITTO %s device is not allocated to a tape",which);
      c_errmsg(msg,"ditto-talloc",&vrity);
      break;

   case DITTO_BSF:
      sprintf(msg,"Error back spacing files on DITTO %s device",which);
      c_errmsg(msg,"ditto-bsf",&nonfatal);
      break;

   case DITTO_BSR:
      sprintf(msg,"Error back spacing records on DITTO %s device",which);
      c_errmsg(msg,"ditto-bsr",&nonfatal);
      break;

   case DITTO_FSF:
      sprintf(msg,"Error forward spacing files on DITTO %s device",which);
      c_errmsg(msg,"ditto-fsf",&nonfatal);
      break;

   case DITTO_FSR:
      sprintf(msg,"Error forward spacing records on DITTO %s device",which);
      c_errmsg(msg,"ditto-fsr",&nonfatal);
      break;

   case DITTO_TPREW:
      sprintf(msg,"Error rewinding DITTO %s device",which);
      c_errmsg(msg,"ditto-tprew",&vrity);
      break;

   case DITTO_RECSIZ:
      sprintf(msg,"Record sizes are NOT compatiable");
      c_errmsg(msg,"ditto-recsiz",&vrity);
      break;

   case DITTO_BLKSIZ:
      sprintf(msg,"Block sizes are NOT compatiable");
      c_errmsg(msg,"ditto-blksiz",&vrity);
      break;

   case DITTO_RECTYP:
      sprintf(msg,"Record types are not compatiable");
      c_errmsg(msg,"ditto-rectyp",&vrity);
      break;

   case DITTO_EOF:
      strcpy(msg,"EOF detected prior to processing all records");
      c_errmsg(msg,"ditto-eof",&vrity);
      break;

   case DITTO_WEOF:
      strcpy(msg,"Error writing EOF mark");
      c_errmsg(msg,"ditto-weof",&vrity);
      break;

   case DITTO_VERIFY:
      sprintf(msg,"VERIFY error -- Files are NOT the same");
      c_errmsg(msg,"ditto-verify",&vrity);
      break;

   case DITTO_UNEQ:
      sprintf(msg,"VERIFY error -- Input/Output record sizes are NOT EQUAL");
      c_errmsg(msg,"ditto-uneq",&vrity);
      break;

   case DITTO_INEOF:
      sprintf(msg,"VERIFY error -- Unexpected EOF detected in input device");
      c_errmsg(msg,"ditto-ineof",&vrity);
      break;

   case DITTO_OUTEOF:
      sprintf(msg,"VERIFY error -- Unexpected EOF detected in output device");
      c_errmsg(msg,"ditto-outeof",&vrity);
      break;

   case DITTO_INVMOD:
      strcpy(msg,"Invalid record number -- Cannot modify the specified record");
      c_errmsg(msg,"ditto-modeof",&vrity);
      break;

   case DITTO_MODLEN:
      sprintf(msg,"Requested to modify beyond end of record");
      c_errmsg(msg,"ditto-modlen",&vrity);
      break;

   case DITTO_MODREC:
      sprintf(msg,"%s%s","DITTO command \"copy-modrec\" is required to ",
              "run in interactive mode");
      c_errmsg(msg,"ditto-modrec",&vrity);
      break;

   case DITTO_HEXDIG:
      c_errmsg("Two hex digits is maximum pad value","ditto-hexdig",&vrity);
      break;

   case DITTO_HEXINV:
      c_errmsg("An invalid hex digit encountered","ditto-hexinv",&vrity);
      break;

   case DITTO_PADCHR:
      c_errmsg("One character is maximum pad length","ditto-padchr",&vrity);

   case DITTO_ABORT:
      c_errmsg("Process aborted by the user","ditto-abort",&fatal);
      break;

   default:
      sprintf(msg,"Invalid DITTO error code -- code = %d",err_code);
      c_errmsg(msg,"ditto-invcode",&vrity);
      break;
   }

/*
*   If error flag set to YES, then query user to see if processing should
*   continue.  If user wants to continue, return to the calling program.  
*   If the user wants to abort, call "ditto_error" with severity code 
*   ERROR_SEV to abort the process.
*/
if ((strcmp(errmod,"QUERY") == 0) && (severity == ERROR_VERIFY))
   {
   strcpy(msg,"\nDo you want to continue processing (Y/N):");
   t_highlight(msg);
   t_write(msg,T_PROMPT);
   t_read(msg,&term);

   if ((strncmp(msg,"Y",1) == 0) || (strncmp(msg,"y",1) == 0))
      return;
   else
      ditto_error(" ",DITTO_ABORT,ERROR_SEV,update);
   }

}
