/******************************************************************************
FUNCTION:	REOPEN_TAPE

PURPOSE:	Reopen a tape device for the DITTO program

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      11/85    K. Gacke		initial development
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1      09/96    D. Etrheim		Return fatal error if tpbsf fails

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	From the input parameter "which", it is determined which DITTO device
	is to be reopened.  Error checking is done to make sure the ditto 
	device has been allocated to a tape.  The DITTO global variable 
	specifying the number of records to back space on the open is retrieved
	from the TAE parameter block.  The tape is reopened and back spaced the
	specified number of records.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */

int reopen_tape
(
    char *which,			/* DITTO input/output device         */
    long rewopt,			/* rewind option on tape open        */
    long access,			/* I/O access to the tape            */
    struct TAPE_FD *tape		/* tape descriptor                   */
)
{
extern struct DITTO_DEV ditto_device;
extern struct PARBLK vblock;		/* TAE parameter block               */

struct VARIABLE *val;			/* prointer to TAE variable          */

long  bsf_skip;				/* number of tape marks to back skip */
long  mode = HANDLE;			/* tape I/O outputs error messages   */
long  skip = 0;				/* number of files to skip on open   */
long  status;				/* function return status code       */
long  nonfatal = 1;			/* Non-fatal error code		     */

char *device;				/* DITTO input/output device         */
char density[5];			/* tape density in BPI	             */
 
/*
  Input device
*/
if (strcmp(which,"INPUT") == 0)
   {
   /*
     Insure the DITTO device has been allocated to a tape device
   */
   ditto_device.in_tape = tape;
   ditto_device.input_dev = "TAPE\0";

   val = p_find(&vblock,"idevice");
   if (val->v_count == 0)
      {
      ditto_error(which,DITTO_TALLOC,ERROR_WARN,NO_UPDATE_FD);
      return(E_FAIL);
      }
   device = SVAL(*val,0);

   if (strcmp(device,"TAPE") != 0)
      {
      ditto_error(which,DITTO_TALLOC,ERROR_WARN,NO_UPDATE_FD);
      return(E_FAIL);
      }

   /*
     Retrieve physical tape drive name
   */
   val = p_find(&vblock,"ihost");
   strcpy(tape->drive,SVAL(*val,0));

   /*
     Retrieve density of tape in BPI
   */
   val = p_find(&vblock,"idensity");
   strcpy(density,SVAL(*val,0));

   /*
     Retrieve the number of tape marks to backspace when the tape is reopened
   */
   val = p_find(&vblock,"iskip");
   bsf_skip = IVAL(*val,0);

   /*
     Retrieve block size
   */
   val = p_find(&vblock,"iblksiz");
   if (val->v_count == 0)
      tape->blksiz = MAXTAPEWT;
   else
      tape->blksiz = IVAL(*val,0);

   /*
     Retrieve record size
   */
   val = p_find(&vblock,"irecsiz");
   if (val->v_count == 0)
      tape->recsiz = MAXTAPEWT;
   else
      tape->recsiz = IVAL(*val,0);

   /*
     Retrieve record type
   */
   val = p_find(&vblock,"irectyp");
   if (val->v_count == 0)
      tape->rectyp = NOVAL;
   else if (strcmp(SVAL(*val,0),"FIX") == 0)
      tape->rectyp = FIX;
   else if (strcmp(SVAL(*val,0),"VAR") == 0)
      tape->rectyp = VAR;

   /*
     Retrieve type of carriage control
   */
   val = p_find(&vblock,"icc");
   if (val->v_count == 0)
      tape->cc = NOVAL;
   else if (strcmp(SVAL(*val,0),"CARRIAGE") == 0)
      tape->cc = CARRIAGE;
   else if (strcmp(SVAL(*val,0),"FORTRAN") == 0)
      tape->cc = FORTRAN;
   else if (strcmp(SVAL(*val,0),"PRINT") == 0)
      tape->cc = PRINT;
   else
      tape->cc = NONE;

   /*
     Retrieve current record
   */
   val = p_find(&vblock,"irec");
   if (val->v_count == 0)
      tape->cur_rec = NOVAL;
   else
      tape->cur_rec = IVAL(*val,0);

   /*
     Retrieve current file
   */
   val = p_find(&vblock,"ifile");
   if (val->v_count == 0)
      tape->cur_file = NOVAL;
   else
      tape->cur_file = IVAL(*val,0);
   }
else					/* Output device		*/
   {
   /*
     Insure the DITTO device has been allocated to a tape device
   */
   ditto_device.out_tape = tape;
   ditto_device.output_dev = "TAPE\0";

   val = p_find(&vblock,"odevice"); 
   if (val->v_count == 0)
      {
      ditto_error(which,DITTO_TALLOC,ERROR_WARN,NO_UPDATE_FD);
      return(E_FAIL);
      }

   device = SVAL(*val,0);
   if (strcmp(device,"TAPE") != 0)
      {
      ditto_error(which,DITTO_TALLOC,ERROR_WARN,NO_UPDATE_FD);
      return(E_FAIL);
      }

   /*
     Retrieve physical drive name
   */
   val = p_find(&vblock,"ohost");
   strcpy(tape->drive,SVAL(*val,0));

   /*
     Retrieve tape density in BPI
   */
   val = p_find(&vblock,"odensity");
   strcpy(density,SVAL(*val,0));

   /*
     Retrieve the number of tape marks to backspace when tape is reopened
   */
   val = p_find(&vblock,"oskip");
   bsf_skip = IVAL(*val,0);

   /*
     Retrieve block size
   */
   val = p_find(&vblock,"oblksiz");
   if (val->v_count == 0)
      tape->blksiz = MAXTAPEWT;
   else
      tape->blksiz = IVAL(*val,0);

   /*
     Retrieve record size
   */
   val = p_find(&vblock,"orecsiz");
   if (val->v_count == 0)
      tape->recsiz = MAXTAPEWT;
   else
      tape->recsiz = IVAL(*val,0);

   /*
     Retrieve record type
   */
   val = p_find(&vblock,"orectyp");
   if (val->v_count == 0)
      tape->rectyp = NOVAL;
   else if (strcmp(SVAL(*val,0),"FIX") == 0)
      tape->rectyp = FIX;
   else if (strcmp(SVAL(*val,0),"VAR") == 0)
      tape->rectyp = VAR;

   /*
     Retrieve type of carriage control
   */
   val = p_find(&vblock,"occ");
   if (val->v_count == 0)
      tape->cc = NOVAL;
   else if (strcmp(SVAL(*val,0),"CARRIAGE") == 0)
      tape->cc = CARRIAGE;
   else if (strcmp(SVAL(*val,0),"FORTRAN") == 0)
      tape->cc = FORTRAN;
   else if (strcmp(SVAL(*val,0),"PRINT") == 0)
      tape->cc = PRINT;
   else
      tape->cc = NONE;

   /*
     Retrieve current record
   */
   val = p_find(&vblock,"orec");
   if (val->v_count == 0)
      tape->cur_rec = NOVAL;
   else
      tape->cur_rec = IVAL(*val,0);

   /*
     Retrieve current file
   */
   val = p_find(&vblock,"ofile");
   if (val->v_count == 0)
      tape->cur_file = NOVAL;
   else
      tape->cur_file = IVAL(*val,0);
   }

/*
  Reopen the tape device
*/
tape->access = access;
status=c_tpreop(&tape->unit,&skip,&mode,&rewopt,density,&access,tape->drive);
if (status != TAPE_SUCC)
   ditto_error(which,DITTO_TPREOP,ERROR_VERIFY,NO_UPDATE_FD);

if (rewopt == REWIND)
   bsf_skip = 0;

if (bsf_skip > 0)			/* back space the specified number   */
   {					/*  of records.                      */
   status = c_tpbsf(&tape->unit,&bsf_skip,&mode);
   if (status != TAPE_SUCC)
      {
      c_errmsg("Unable to back space over EOF mark","ditto-bsf",&nonfatal);
      return(E_FAIL);
      }
   }

return(E_SUCC);
}

/******************************************************************************
FUNCTION:	CLOSE_TAPE

PURPOSE:	Close a tape device for the DITTO program

ALGORITHM 
	The tape device is close.  The DITTO global variable specifying the
	number of records to be back spaced on the next open is updated 
	depending on the type of access to the tape.  If access was write, 
	"eofopt" tape marks were written to the tape on close, and the DITTO
	global variable is set to this value.  If access was read, no tape
	marks were written to the tape on close, and the DITTO global variable
	is set to zero.  The updated TAE parameter block is rewritten.

ALGORITHM REFERENCES	none
*******************************************************************************/
void close_tape
(
    char *which,			/* DITTO input/output device         */
    struct TAPE_FD *tape,		/* tape descriptor                   */
    long rewopt,			/* tape rewind option                */
    long eofopt			/* tape end of file option for close */
)
{
extern struct PARBLK vblock;		/* TAE parameter block               */
long  mode = HANDLE;			/* tape routine outputs error msg    */
long  back;				/* EOF marks to be back spaced       */
long  status;				/* function return status code       */

char *cc[1];				/* type of carriage control          */
char *rectyp[1];			/* record type                       */

/*
  Close the tape device
*/
status = c_tpclos(&tape->unit,&mode,&rewopt,&eofopt);
if (status != TAPE_SUCC)
   ditto_error(which,DITTO_TPCLOS,ERROR_WARN,NO_UPDATE_FD);

/* 
  If read access, no tape marks are written on close, so DITTO global if 
  is set to zero for appropriate device.  Else if write access, "eofopt" 
  tape  marks written on close, so DITTO global is set to "1" for the
  appropriate device.
*/
if (tape->access == TREAD)
   back = 0;
else
   back = 1;

/*
  Update the DITTO global variables defined in the TAE parameter block.
*/
switch (tape->rectyp)
   {
   case FIX: rectyp[0] = "FIX";	break;
   case VAR: rectyp[0] = "VAR";	break;
   }

switch (tape->cc)
   {
   case NONE:	  cc[0] = "NONE";	break;
   case CARRIAGE: cc[0] = "CARRIAGE";	break;
   case PRINT:	  cc[0] = "PRINT";	break;
   case FORTRAN:  cc[0] = "FORTRAN";	break;
   }

if (strcmp(which,"INPUT") == 0)	
   {
   if (tape->rectyp != NOVAL)
      status = q_string(&vblock,"irectyp",1,rectyp,P_UPDATE);
   else
      status = q_string(&vblock,"irectyp",0,rectyp,P_UPDATE);

   if (tape->cc != NOVAL)
      status = q_string(&vblock,"icc",1,cc,P_UPDATE);
   else 
      status = q_string(&vblock,"icc",0,cc,P_UPDATE);

   if (tape->cur_rec < 0)
      status = q_intg(&vblock,"irec",0,&tape->cur_rec,P_UPDATE);
   else
      status = q_intg(&vblock,"irec",1,&tape->cur_rec,P_UPDATE);

   if (tape->cur_file <= 0)
      status = q_intg(&vblock,"ifile",0,&tape->cur_file,P_UPDATE);
   else
      status = q_intg(&vblock,"ifile",1,&tape->cur_file,P_UPDATE);

   status = q_intg(&vblock,"iskip",1,&back,P_UPDATE);
   }
else
   {
   if (tape->rectyp != NOVAL)
      status = q_string(&vblock,"orectyp",1,rectyp,P_UPDATE);
   else
      status = q_string(&vblock,"orectyp",0,rectyp,P_UPDATE);

   if (tape->cc != NOVAL)
      status = q_string(&vblock,"occ",1,cc,P_UPDATE);
   else 
      status = q_string(&vblock,"occ",0,cc,P_UPDATE);

   if (tape->cur_rec < 0)
      status = q_intg(&vblock,"orec",0,&tape->cur_rec,P_UPDATE);
   else
      status = q_intg(&vblock,"orec",1,&tape->cur_rec,P_UPDATE);

   if (tape->cur_file <= 0)
      status = q_intg(&vblock,"ofile",0,&tape->cur_file,P_UPDATE);
   else
      status = q_intg(&vblock,"ofile",1,&tape->cur_file,P_UPDATE);

   status = q_intg(&vblock,"oskip",1,&back,P_UPDATE);
   }

/*
  Write the updated parameter block
*/
status = q_out(&vblock);

}
