/****************************************************************************
NAME:		C_CKDLTF

PURPOSE:	Checks the DLTF for duplicate entry names.  If duplicate
		entry names do not exist then the routine will open the
		DLTF for output.

PROGRAM HISTORY:
	By D. Akkerman, USGS/EROS Data Center, 9/88
	Modified by L. Huewe, USGS/EROS Data Center, 9/89
	    -  Modified to find and report duplicate entries correctly.
	Modified by B. Davis, USGS/EROS Data Center, 1/91
	    - Standardized errmsg and status returns according to EDC
	       support guidelines.

PROJECT:	LAS 5.0

ALGORITHM:
	Squeeze the host file name and the DLTF entry name for use with
	 FORTRAN routines.
	Append the DLTF extension to the host file name.
	Convert the DLTF entry name to upper case.
	Check for the existence of a duplicate DLTF entry name.
	If a duplicate entry exists then give an error and terminate processing.
	Assign the file access to write/append.
	Open the DLTF for output.
	Copy the "squeezed" values of the host file name and DLTF entry name
	 to the output parameters.
	Return.

*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "vll.h"
#include "util.h"
#include "labserv.h"
#include "dltf.h"

int c_ckdltf
(
    char *host,		/* Host file name of the DLTF.  The return
			   string has been "squeezed" for FORTRAN.
			   (I/O)				*/
    char *key,		/* DLTF entry name. The return string
			   contains the key in upper case, and
			   has been "squeezed" for FORTRAN.(I/O)*/
    long *fd		/* File descriptor of the DLTF. Assigned
			   only if the duplicate entry check is
			   unsuccessful. (O)			*/
)
{
long	access,		/* Determines file access.		*/
	status,		/* Status return variable.		*/
	host_len = CMLEN,
			/* Dimensioned length of the host file name
			   string; used in the squeeze call.	*/
	key_len = KEYLEN,
			/* Dimensioned length of the DLTF entry 
			   name string; used in the squeeze call.*/
	clen = 0,	/* Character length variable.		*/
	dlen = 0,	/* Data length variable.		*/
	slen,		/* Length of the DLTF entry name.	*/
	one = 1;	/* Indicates a nonfatal error.		*/

char	key1[KEYLEN+1],	/* Working key used to append an "@" to the
			   actual key.				*/
	*hostptr,	/* Working pointer used to squeeze the host
			   file name for FORTRAN.		*/
	*keyptr,	/* Working pointer used to squeeze the LAS
			   file name for FORTRAN.		*/
	*cbuf=NULL,		/* Character buffer.			*/
	type[DTYPL];	/* Dummy data type variable.		*/

unsigned char *dbuf=NULL;	/* Data buffer.				*/

/* Squeeze the host file name and the DLTF entry name for use with
   FORTRAN routines.						*/

hostptr = squeeze(host,host_len);
keyptr  = squeeze(key,key_len);

/* Convert the entry name to upper case, copy the entry name to a working
   variable and append an "@" to the working entry name.  This is used to
   prevent possible collisions with the names of the second type of DLTF
   record.								*/

slen = (long)strlen(keyptr);
c_low2up(keyptr,&slen);
sprintf(key1,"%s@",keyptr);

/* Check for a duplicate DLTF entry name.				*/

/* Assign the file access mode to read.  This is also used to test for
   the existence of the file.						*/

access = 0;

/* First check to see of the file exists; if it doesn't then we don't
   have to worry about duplicate records.				*/

status = c_lsstat(hostptr,&access);

if (status == E_SUCC)
  {
	status = c_lsopen(fd,hostptr,&access);
	if (status != E_SUCC)
	  {
		c_errmsg("Unable to open DLTF","ckdltf-open",&one);
		c_errmsg("Error returned from lsopen","ckdltf-call",&one);
		return(E_FAIL);
	  }

	status = c_lsread(fd,key1,&clen,&dlen,cbuf,dbuf,type);
	if (status == E_SMAL)
	  {
	  	cbuf = malloc(clen);
	  	if (cbuf == 0) 
		  {
			c_errmsg(
		"Error allocating buffer space for DLTF record","ckdltf-nonmem",
		                 &one);
			return(E_FAIL);
		  }
	  	status = c_lsread(fd,key1,&clen,&dlen,cbuf,dbuf,type);
	  }
	if ((status != E_FAIL) && (status != E_EOF))
	  {
		c_errmsg("Duplicate DLTF entry name","ckdltf-dupl",&one);
		return(E_FAIL);
	  }

	status = c_lsclos(fd,hostptr,&access);
	if (status != E_SUCC)
	  {
		c_errmsg("Unable to close DLTF","ckdltf-close",&one);
		c_errmsg("Error returned from lsclos","ckdltf-call",&one);
		return(E_FAIL);
	  }
  }

/* Assign write/append file access.					*/

access = 2;

/* If no duplicate entries exist then reopen the DLTF for output.	*/

status = c_lsopen(fd,hostptr,&access);

if (status != E_SUCC)
  {
	c_errmsg("Unable to open DLTF","ckdltf-open",&one);
	c_errmsg("Error returned from lsopen","ckdltf-call",&one);
	return(E_FAIL);
  }

/* Copy the "squeezed" values of the LAS file name and DLTF entry name
   to the output parameters.						*/

strcpy(host,hostptr);
strcpy(key,keyptr);

return(E_SUCC);
}
