/*****************************************************************************
NAME:		C_R1DLTF

PURPOSE:	This routine performs three main objectives: first, it is used
		to open the DLTF for input.  The file descriptor for the DLTF
		is assigned and is used for all subsequent reads.  The routine
		also checks for the existence of the DLTF entry, and validates 
		its data type.  

		Second, the routine reads the first type of DLTF record and 
		returns the fields as output parameters.  

		Finally, the routine reads the first band member of the entry 
		and returns the length of the data portion of the second DLTF 
		record type.  This is necessary because memory must be 
		allocated for the data portion of the second DLTF record 
		before the call to R2DLTF can be made.

PROGRAM HISTORY:
	By D. Akkerman, USGS/EROS Data Center, 9/88
	By D. Akkerman, USGS/EROS Data Center, 11/88 -- Use one for key instead
							of band no. for 1st rec.
	Modified by B. Davis, USGS/EROS Data Center, 1/91
	    - Standardized errmsg and status returns according to EDC
	       support guidelines.
	Modified by L. Huewe, USGS/EROS Data Center, 2/91
	    - Due to changes in the label services routines on the SUNs,
	      fseek must be used instead of lseek (on SUNs only). 

PROJECT:	LAS 5.0

ALGORITHM:
	Squeeze the DLTF entry name (key) for use with Fortran routines.
	Open the associated DLTF. 
	If it doesn't exist then issue an error message and return.
	Convert the entry name for the DLTF to upper case.
	Read the first DLTF record.
	Check for a read error.
	Save the file position for the rewind below.
	Validate the input data type.
	Convert the character portions in the DLTF record to integers.
	Copy the entry description to the output parameter.
	Create the name of the first band member of the DLTF entry.
	Read the first band member (second record type) and assign the
	  data length of the record to the output parameter.
	Check for a read error.
	Rewind the DLTF record so that the band members (second DLTF record
	  type) can be processed by R2DLTF.
	Copy the "squeezed" values of the DLTF entry name to the output parm.
	Return.

******************************************************************************/

#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "util.h"
#include "vll.h"
#include "dltf.h"

int c_r1dltf
(
    long *fd,		/* File descriptor for the DLTF. (O)		*/
    char *host,		/* Host file name of the DLTF. (I)		*/
    char *key,		/* DLTF entry name; converted to upper case. 
			   (I/O)					*/
    char *descr,		/* String containing a user specified description
			   of the DLTF entry. (O)			*/
    struct ASDLTF *asdltf,	/* Structure containing DLTF field information.
			   (O)						*/
    long *dtlen,		/* Total number of bytes required for the data 
			   portion of the second DLTF record type. (O)	*/
    char *dtype		/* Data type of the DLTF. (O)			*/
) 
{
struct CDLTF cdltf;	/* Input structure for the character portion of
			   first DLTF record.				*/

long	one = 1,	/* Indicates a nonfatal error.	     		*/
	clen,		/* Length of the character portion of the DLTF. */
	dlen,		/* Length of the data portion of the DLTF.	*/
	slen,		/* Length of the DLTF entry name.		*/
	key_len = KEYLEN,
			/* Dimensioned length of the DLTF entry name string;
			   used in the squeeze call.			*/
	pos,		/* File position used to rewind the DLTF.	*/
	status,		/* Return status variable.			*/
	i,  		/* Working index variables.			*/
	access = 0;	/* Indicates read access.			*/

char	key1[KEYLEN+1],	/* Working key name with the "@" extension; used
		  	   in the creation of the name for the first type
			   of DLTF record.				*/
	*cbuf=NULL,		/* Dummy character buffer used to read the second
			   DLTF record type.				*/
	*keyptr,	/* Working pointer used to squeeze the DLTF entry
			   name for Fortran.				*/
	keybnd[KEYTL];
			/* Band member name of the DLTF entry.		*/

unsigned char *dbuf=NULL;	/* Dummy data buffer used to read the first DLTF
			   record type.					*/

/* Squeeze the DLTF entry name (key) for use with Fortran routines.	*/

keyptr = squeeze(key,key_len);

/* Open the associated DLTF.						*/

status = c_lsopen(fd,host,&access);

/* Check for existence of the DLTF.  					*/

if (status != E_SUCC)
  {
	c_errmsg("Unable to open DLTF","r1dltf-open",&one);
	c_errmsg("Error returned from lsopen","r1dltf-call",&one);
        free(keyptr);
	return(status);
  }

/* Convert the entry name for the DLTF to upper case.  Note the use of
   the "@" below is to prevent possible collisions with the names of the
   second type of DLTF record.  This is used for the first type of DLTF
   record only.								*/

slen = (long)strlen(keyptr);
c_low2up(keyptr,&slen);
sprintf(key1,"%s@",keyptr);

/* Read the specified DLTF entry (first record type).  The lengths of the
   character and data portions of the DLTF entry are assigned because we
   don't know their actual lengths ahead of time.			*/

clen = MXDLTF;
dlen = 0;
status = c_lsread(fd,key1,&clen,&dlen,(char *)&cdltf,dbuf,dtype);

if ((status != E_SUCC) && (status != E_SMAL))
  {
	c_errmsg("Unable to read DLTF entry","r1dltf-read",&one);
	c_errmsg("Error returned from lsread","r1dltf-call",&one);
        free(keyptr);
	return(E_FAIL);
  }

/* Save the current file position for the rewind below.			*/

pos = (long)lseek(*fd,0,1);

/* Validate the input data type.					*/

if ((strcmp(dtype,"B") != 0) && (strcmp(dtype,"I2") != 0))
	{
	 c_errmsg("Invalid data type","r1dltf-invalid",&one);
         free(keyptr);
	 return(E_FAIL);
	}

/* Convert the character portions of the DLTF to integers in the output
   structure ASDLTF.							*/

sscanf(cdltf.numbnd,"%d",&asdltf->numbnd);

for (i = 0; i < asdltf->numbnd; i++)
	sscanf(cdltf.bands[i],"%d",&asdltf->band[i]);

for (i = 0; i < 4; i++)
	sscanf(cdltf.window[i],"%d",&asdltf->window[i]);

for (i = 0; i < 2; i++)
  {
	sscanf(cdltf.izoom[i],"%d",&asdltf->izoom[i]);
	sscanf(cdltf.gzoom[i],"%d",&asdltf->gzoom[i]);
	sscanf(cdltf.ishift[i],"%d",&asdltf->ishift[i]);
	sscanf(cdltf.gshift[i],"%d",&asdltf->gshift[i]);
  }

/* Copy the entry description to the output parameter.			*/

strcpy(descr,cdltf.descr);

/* Create the name of the first band member of the DLTF entry (this will
   always be 1). 							*/

sprintf(keybnd,"%s1",keyptr);

/* Set the character and data buffer lengths to one so that label
   services will return their actual lengths.				*/

clen = *dtlen = 1;

/* Read the first band member (second record type) of the DLTF entry;
   this will assign the data length so that memory can be allocated
   to process the second DLTF records.					*/

status = c_lsread(fd,keybnd,&clen,dtlen,cbuf,dbuf,dtype);

if ((status != E_SUCC) && (status != E_SMAL))
  {
	c_errmsg("Unable to read DLTF entry","r1dltf-read",&one);
	c_errmsg("Error returned from lsread","r1dltf-call",&one);
        free(keyptr);
	return(E_FAIL);
  }

/* Rewind the DLTF so that the second records can be processed.		*/

status = (long)lseek(*fd,pos,0);

if (status == -1)
  {
	c_errmsg("Unable to reset DLTF","r1dltf-norew",&one);
        free(keyptr);
	return(E_FAIL);
  }

/* Copy the "squeezed" value of the DLTF entry name to the output parm.	*/

strcpy(key,keyptr);
free(keyptr);

return(E_SUCC);
}
