/*****************************************************************************
NAME:		C_R2DLTF

PURPOSE:	This routine is designed to read the second type of DLTF 
		record type and return its fields as output parameters. 

PROGRAM HISTORY:
	By D. Akkerman, USGS/EROS Data Center, 10/88
	Modified by D. Akkerman, USGS/EROS Data Center, 3/89
	 - Changed the slope and offset parameters to double.
	Modified by B. Davis, USGS/EROS Data Center, 1/91
	    - Standardized errmsg and status returns according to EDC
	       support guidelines.
	Modified by L. Huewe, USGS/EROS Data Center, 2/91
	    - Due to changes in the label services routines on the SUNs,
	      fseek must be used instead of lseek (on SUNs only). 

PROJECT:	LAS 5.0

ALGORITHM:
	Save the current file position so that it can be reset.
	Create the band member name (second DLTF record name).
	Read the second DLTF record.
	Assign the slope and offset of the LUT.
	Assign the starting and ending points of the LUT.
	Shift the LUT in the data buffer to remove the starting and
	  ending points.
	Reset the file position.
	Return.

*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "vll.h"
#include "dltf.h"

int c_r2dltf
(
    long *fd,		/* File descriptor for the DLTF. (I)		*/
    char *key,		/* DLTF entry name. (I)				*/
    long *band,		/* Band number of member to be read. (I)	*/
    long *startpt,	/* Starting point of the LUT. (O)		*/
    long *endpt,		/* Ending point of the LUT. (O)			*/
    long *dlen,		/* Length of the data portion of the second type
			   of DLTF record. (I)				*/
    unsigned char *lutdata,	/* Buffer for the data portion of the DLTF
			   entry. (O)					*/
    double *slope,		/* Slope of the LUT. (O)			*/
    double *offset	/* Offset of the LUT. (O)			*/
)
{
long	pos,		/* File position used to reset the DLTF.	*/
	clen = RECSIZ2 + 1,	
			/* Length of the character portion of the DLTF
			   entry.					*/
	status,		/* Return status indicator.			*/
	one = 1,	/* Indicates a nonfatal error.			*/
	i,		/* Working variable.				*/
	nument;		/* Number of entries in the LUT.		*/

short	*shrtptr,	/* Working I*2 pointers.			*/
	*sshrtptr;

char	bndnam[KEYTL], 	/* String used in the creation of the band
			   member name.					*/
	cbuf[RECSIZ2+1],	
			/* Buffer for the character portion of the
			   DLTF entry.					*/
	*cptr,		/* Working pointer into the character buffer.	*/
	temp[19],	/* Working string for the conversion of the slope
			   and offset strings to numbers.		*/
	dtype[DTYPL];	/* Variable used to capture the data type
			   of the DLTF entry.				*/
unsigned char *byteptr,	/* Working BYTE pointers.			*/
	      *sbyteptr;

/* Save the current file position so that it can be reset.		*/

pos = (long)lseek(*fd,0,1);

/* Create the band member name (second DLTF record name). 		*/

sprintf(bndnam,"%s%d",key,*band);

/* Read the second DLTF record.						*/

status = c_lsread(fd,bndnam,&clen,dlen,cbuf,lutdata,dtype);

if ((status != E_SUCC) && (status != E_SMAL))
  {
	c_errmsg("Unable to read DLTF entry","r2dltf-read",&one);
	c_errmsg("Error returned from lsread","r2dltf-call",&one);
	return(E_FAIL);
  }

/* Assign the slope and offset of the LUT.  The 19 represents the 19 
   characters of the slope and offset.  See the Display and Look-up Table 
   File format document for details on the format of the second type of 
   DLTF record.								*/

cptr = cbuf;
strcpy(temp,cptr);
cptr += 19;
sscanf(temp,"%lf",slope);

strcpy(temp,cptr);
sscanf(temp,"%lf",offset);

/* Assign the starting and ending points of the LUT, and then shift
   the LUT in the data buffer to remove the starting and ending points.	*/

if ((strcmp(dtype,"B")) == 0)
  {
	byteptr = sbyteptr = lutdata;

	*startpt = (long)*byteptr;
	byteptr++;
	*endpt   = (long)*byteptr;
	byteptr++;

	nument = (*endpt - *startpt) + 1;
	for (i = 0; i < nument; i++, sbyteptr++, byteptr++)
		*sbyteptr = *byteptr;
  }

else
  {
	sshrtptr = shrtptr = (short *)lutdata;

	*startpt = (long)*shrtptr;
	shrtptr++;
	*endpt   = (long)*shrtptr;
	shrtptr++;

	nument = (*endpt - *startpt) + 1;
	for (i = 0; i < nument; i++, sshrtptr++, shrtptr++)
		*sshrtptr = *shrtptr;
  }

/* Reset the file position.						*/

status = (long)lseek(*fd,pos,0);

if (status == -1)
  {
	c_errmsg("Unable to reset DLTF","r2dltf-norew",&one);
	return(E_FAIL);
  }

return(E_SUCC);
}
