/****************************************************************************
NAME:		C_W1DLTF

PURPOSE:	Writes the first DLTF entry record to an associate DLTF.
		The function receives the necessary DLTF fields as parameters
		and stores them in an associate DLTF.

PROGRAM HISTORY:
	By D. Akkerman, USGS/EROS Data Center, 9/88
	Modified by B. Davis, USGS/EROS Data Center, 1/91
	    - Standardized errmsg and status returns according to EDC
	       support guidelines.

PROJECT:	LAS 5.0

ALGORITHM:
	Assign the DLTF fields for the first DLTF entry record.
	 - Check the input data type; if it is not BYTE or I*2 then
	     print an error message and terminate processing.
	 - Convert the number of bands to a string.
	 - Convert the number of input bands to a string.
	 - Calculate the number of bytes needed to store the band numbers
	   as strings.
	 - Convert the number of each band to a string.
	 - Convert the LAS image window to strings.
	 - Convert the zoom and shift factors to strings.
	 - Copy the description field to the output structure.
	Calculate the total number of bytes in the character portion of
	 the first DLTF entry record.
	Create the working key name.
	Write the first DLTF entry record.
	Return.

*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "vll.h"
#include "dltf.h"

int c_w1dltf
(
    long *fd,		/* File descriptor for the DLTF. (I)		*/
    char *key,		/* DLTF entry name. (I)				*/
    char *descr,		/* String containing a user specified description
			   of the DLTF entry. (I)			*/
    char *type,		/* String containing the data type of the LUT(s).
			   (I)						*/
    struct ASDLTF *asdltf	/* Structure containing DLTF field information.
			   (I)						*/
)
{
struct CDLTF cdltf;	/* Output structure for the character portion of
			   the DLTF.					*/

long	i,		/* Working variables.				*/
	one = 1,	/* Specifies a nonfatal error.			*/
	dlen = 0,	/* Length of the data portion of the first DLTF
			   record (always 0).				*/
	clen,		/* Length of the character portion of the first
			   DLTF record.					*/
	status,		/* Return status indicator.			*/
	bndsiz;		/* Buffer size needed to for the conversion of the
			   actual band numbers to strings.		*/

char	key1[KEYLEN + 1];
			/* Working key name variable.  An "@" is appended
			   to the key name to form this value.  This is
			   done to prevent possible collisions in key
			   names.					*/

unsigned char *dbuf=NULL;	/* Dummy buffer used to write the first record
			   of the DLTF entry.				*/

/* Assign the DLTF fields to the output structure.			*/

/* Check the input data type.						*/

if ((strcmp(type,"B") != 0) && (strcmp(type,"I2") != 0))
  {
	c_errmsg("Invalid data type","w1dltf-invalid",&one);
	return(E_FAIL);
  }

/* Convert the number of bands to a string.				*/

sprintf(cdltf.numbnd,"%d",asdltf->numbnd);

/* Calculate the number of bytes needed to store the band numbers as
   strings. (The 4 represents 4 chars.)					*/

bndsiz = asdltf->numbnd * 4;

/* Convert the band numbers to strings.					*/

for (i = 0; i < asdltf->numbnd; i++)
	sprintf(cdltf.bands[i],"%d",asdltf->band[i]);

/* Convert the LAS image window to strings.  The 4 here corresponds to
   SL,SS,NL,NS.								*/

for (i = 0; i < 4; i++)
	sprintf(cdltf.window[i],"%d",asdltf->window[i]);

/* Convert the zoom and shift factors to strings.			*/

sprintf(cdltf.izoom[0],"%d",asdltf->izoom[0]);
sprintf(cdltf.izoom[1],"%d",asdltf->izoom[1]);

sprintf(cdltf.gzoom[0],"%d",asdltf->gzoom[0]);
sprintf(cdltf.gzoom[1],"%d",asdltf->gzoom[1]);

sprintf(cdltf.ishift[0],"%d",asdltf->ishift[0]);
sprintf(cdltf.ishift[1],"%d",asdltf->ishift[1]);

sprintf(cdltf.gshift[0],"%d",asdltf->gshift[0]);
sprintf(cdltf.gshift[1],"%d",asdltf->gshift[1]);

/* Copy the description of the DLTF entry to the output structure.	*/

strcpy(cdltf.descr,descr);

/* Calculate the length of the character portion of the first DLTF entry
   record.								*/

clen = SZDLTF + bndsiz;

/* Copy the entry name to a working variable and append an "@" to the
   working entry name.  This is used to prevent possible collisions
   with the names of the second type of DLTF record.  Note also that after
   the call to c_ckdltf, which should occur previous to c_w1dltf, we
   can now safely assume that the key name is upper case.		*/

sprintf(key1,"%s@",key);

/* Write the first DLTF record.						*/

status = c_lswrit(fd,key1,&clen,&dlen,(char *)&cdltf,(char *)dbuf,type);

if (status != E_SUCC)
  {
	c_errmsg("Error writing to DLTF","w1dltf-write",&one);
	c_errmsg("Error returned from lswrit","w1dltf-call",&one);
	return(E_FAIL);
  }

return(E_SUCC);
}
