/****************************************************************************
NAME:		C_W2DLTF

PURPOSE:	Writes the second type of DLTF entry record to a DLTF.  The
		routine should be called once for each band entry that
		composes the DLTF entry.

PROGRAM HISTORY:
	By D. Akkerman, USGS/EROS Data Center, 9/88
	Modified by D. Akkerman, USGS/EROS Data Center, 3/89
	  - Changed the slope and offset parameters to double.
	Modified by B. Davis, USGS/EROS Data Center, 1/91
	    - Standardized errmsg and status returns according to EDC
	       support guidelines.

PROJECT:	LAS 5.0

ALGORITHM:
	Convert the slope and offset of the band entry to strings.
	Create the key for the second type of DLTF record.
	Assign the total number of bytes in the character portion of
	  the second type of DLTF record.
	Create the character portion.
	Find the numeric equivalent of the data type string.
	Calculate the total number of bytes in the data portion of
	  the second type of DLTF record.
	Allocate buffer space and create the data portion.
	Write the second type of DLTF record to the DLTF entry.
	Return.

*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "vll.h"
#include "dltf.h"

int c_w2dltf
(
    long *fd,		/* File descriptor for the DLTF. (I)		*/
    char *type,		/* String containing the data type of the LUT(s);
			   either BYTE or I2. (I)			*/
    char *key,		/* DLTF entry name. (I)				*/
    double *slope,		/* The slope of the image band that corresponds to
			   the LUT band. (I)				*/
    double *offset,	/* The offset of the image band that corresponds to
			   the LUT band. (I)				*/
    long *startpt,	/* The value of the input image that corresponds to
			   the first entry in the LUT data. (I)		*/
    long *endpt,		/* The value of the input image that corresponds to
			   the last entry in the LUT data. (I)		*/
    long *lutnum,	/* LUT number that corresponds to the image band. 
			   (I)						*/
    unsigned char *lutdata	/* The LUT to be written to the DLTF. (I)	*/
)
{

char	slstr[19],	/* String to contain the slope value.		*/
	ofstr[19],	/* String to contain the offset value.		*/
	ename[KEYTL],	/* DLTF entry name.				*/
	cbuf[2][19];	/* Buffer for the character portion of the second
			   DLTF record.					*/
long	clen,		/* Length of the character portion of the second
			   type of DLTF record.				*/
	dlen,		/* Length of the data portion of the second type
			   of DLTF record.				*/
	dtype,		/* Numeric equivalent to the data type string.	*/
	i,		/* Working loop variable.			*/
	one = 1,	/* Specifies a nonfatal error.			*/
	nument,		/* Number of entries in the LUT.		*/
	status;		/* Return status indicator.			*/

unsigned char *outbuf,	/* Buffer for output data portion of the
			   second type of DLTF record.			*/
	 *byteptr,	/* Working BYTE pointer into outbuf.		*/
	 *bytelut;	/* Working BYTE pointer into lutdata.		*/

short	*shrtptr,	/* Working I*2 pointer into outbuf.		*/
	*shrtlut;	/* Working I*2 pointer into lutdata.		*/

/* Convert the slope and offset of the band entry to strings.		*/

sprintf(slstr,"%.10f",*slope);
sprintf(ofstr,"%.10f",*offset);

/* Create the key for the second type of DLTF record.  This is accomplished
   by using the DLTF entry name (key) and appending the LUT number.	*/

sprintf(ename,"%s%d",key,*lutnum);

/* Assign the total number of bytes in the character portion of the
   second type of DLTF record.						*/

clen = RECSIZ2;

/* Create the character portion for the second type of DLTF record.	*/

strcpy(cbuf[0],slstr);
strcpy(cbuf[1],ofstr);

/* Find the numeric equivalent of the data type string.			*/

if (strcmp(type,"B") == 0)
	dtype = 1;
else if (strcmp(type,"I2") == 0)
	dtype = 2;

/* Calculate the total number of bytes in the data portion of the second
   type of DLTF record.							*/

nument = (*endpt - *startpt) + 1;

dlen = (2 * dtype) + (nument * dtype);

/* Allocate buffer space and create the data portion.			*/

if ((outbuf = (unsigned char *)malloc(dlen)) == 0)
  {
	c_errmsg("Error allocating buffer space for DLTF record",
		 "dltf-nonmem",&one);
	return(E_FAIL);
  }

/* Assign working pointers for type casting based upon the data type
   of the output DLTF record.						*/

if (dtype == 1)
  {
	byteptr = outbuf;
	bytelut = lutdata;
  }
else
  {
	shrtptr = (short *)outbuf;
	shrtlut = (short *)lutdata;
  }

/* Process the input LUT based upon its data type.			*/

if (dtype == 1)
  {
	*byteptr = (unsigned char)*startpt;
	byteptr++;
	*byteptr = (unsigned char)*endpt;
	byteptr++;

	for (i = *startpt; i < (*endpt + 1); i++, byteptr++, bytelut++)
		*byteptr = *bytelut;

  }
else
  {
	*shrtptr = (short)*startpt;
	shrtptr++;
	*shrtptr = (short)*endpt;
	shrtptr++;

	for (i = *startpt; i < (*endpt + 1); i++, shrtptr++, shrtlut++)
		*shrtptr = *shrtlut;

  }

/* Write the second type of DLTF record.				*/

status = c_lswrit(fd,ename,&clen,&dlen,(char *)cbuf,(char *)outbuf,type);

if (status != E_SUCC)
  {
	c_errmsg("Error writing to DLTF","w2dltf-write",&one);
	c_errmsg("Error returned from lswrit","w2dltf-call",&one);
	return(E_FAIL);
  }

return(E_SUCC);
}
