#include <math.h>
#include "worgen.h"
#include "proj.h"
#include "geompak.h"
#include "local_protos.h"

#define NOT_CHECKED     0 /* no error checking is necessary for the param.   */
#define UNDEFINED_FIELD 0 /* no error checking is necessary for the param.   */
#define AXIS_TYPE       1 /* the parameter is either the semi-major,
                             semi-minor or sphere value, each of which are
                             represented in meters.  They are validated by
                             simply checking to make sure they are not less
                             than zero.                                      */
#define DMS_TYPE        2 /* the parameter is an angle in packed
                             degrees/minutes/seconds (DDDMMMSSS.SS) format
                             and is validated using the check_dms routine.   */
#define PATH_TYPE       3 /* the parameter represents the landsat path # and
                             is currently present in only the SOM, format B
                             projection parameters.                          */
#define SATNUM_TYPE     4 /* the parameter represents the landsat satellite
                             # and is currently present in only the SOM,
                             format B projection parameters.                 */
#define ZERO_TYPE       5 /* flag value used with EQUIDC, HOM, and SOM
                             projections to indicate format A parameters     */
#define ONE_TYPE        6 /* flag value used with EQUIDC, HOM, and SOM
                             projections to indicate format B parameters     */
#define PSREV_TYPE      7 /* the parameter represents the period of the
                             satellite revolution in minutes and is currently
                             present in only the SOM, format A proj. params. */
#define LRAT_TYPE       8 /* the parameter represents the landsat ratio to
                             compensate for confusion at northern end of
                             orbit and is currently present in only the SOM,
                             format A projection parameters.                 */
#define PFLAG_TYPE      9 /* the parameter represents the end of path flag
                             and is currently present in only the SOM,
                             format A projection parameters.                 */
#define IZONE_TYPE     10 /* the parameter represents the number of zones in 
                             an integerized sinusoidal projection.  This 
                             must be >= 2 and even. */
#define RFLAG_TYPE     11 /* the parameter indicates how to justify the 
                             columns in the integerized sinusoidal projection.
                             It must be a 0, 1, or 2. */

/* These variables are used to locate the format B projection parameters
   which have been placed at the end of the table.
------------------------------------------------------------------------*/
#define EQUIDC_B   (MAXPROJ + 1)
#define HOM_B      (MAXPROJ + 2)
#define SOM_B      (MAXPROJ + 3)
#define TABLE_SIZE (MAXPROJ + 4)

/* Tolerance allowed */
#define TOL   .00000001
#define LRAT_TOL .0000001
#define ISIN_TOL 0.01

/* Landsat ratio to compensate for confusion at northern end of orbit
  (SOM, format A -- use 0.5201613) */
#define LRAT_VAL   0.5201613

/*
   projparm_valid - A table indicating the type of each projection parameter
                    to allow c_check_proj_params to do proper validation of
                    its value.
*/

/* This is size MAXPROJ + 3 because of the three projection types that have
   two formats (EQUIDC, HOM, and SOM). */
static const int projparm_valid[TABLE_SIZE][COEFCT] = {
{ UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 0: GEO */

{        DMS_TYPE,        DMS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 1: UTM */

{ UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 2: SPCS */

{       AXIS_TYPE,       AXIS_TYPE,        DMS_TYPE,        DMS_TYPE,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 3: ALBERS */

{       AXIS_TYPE,       AXIS_TYPE,        DMS_TYPE,        DMS_TYPE,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 4: LAMCC */

{       AXIS_TYPE,       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 5: MERCAT */

{       AXIS_TYPE,       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 6: PS */

{       AXIS_TYPE,       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 7: POLYC */

{       AXIS_TYPE,       AXIS_TYPE,        DMS_TYPE, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
        ZERO_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 8: EQUIDC A */

{       AXIS_TYPE,       AXIS_TYPE,     NOT_CHECKED, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 9: TM */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 10: STEREO */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 11: LAMAZ */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 12: AZMEQD */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 13: GNOMON */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 14: ORTHO */

{       AXIS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 15: GVNSP */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 16: SNSOID */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 17: EQRECT */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 18: MILLER */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 19: VGRINT */

{       AXIS_TYPE,       AXIS_TYPE,     NOT_CHECKED, UNDEFINED_FIELD,
  UNDEFINED_FIELD,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
         DMS_TYPE,        DMS_TYPE,        DMS_TYPE,        DMS_TYPE,
        ZERO_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 20: HOM A */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 21: ROBIN */

{       AXIS_TYPE,       AXIS_TYPE, UNDEFINED_FIELD,        DMS_TYPE,
         DMS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
       PSREV_TYPE,       LRAT_TYPE,      PFLAG_TYPE, UNDEFINED_FIELD,
        ZERO_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 22: SOM A */

{       AXIS_TYPE,       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 23: ALASKA */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 24: GOOD */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 25: MOLL */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 26: IMOLLW */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 27: HAMMER */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 28: WAGIV */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 29: WAGVII */

{       AXIS_TYPE, UNDEFINED_FIELD,               0,               0,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
                0, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 30: OBEQA */

{       AXIS_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         DMS_TYPE, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
       IZONE_TYPE, UNDEFINED_FIELD,      RFLAG_TYPE, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 31: ISINUS */

{       AXIS_TYPE,       AXIS_TYPE,        DMS_TYPE,        DMS_TYPE,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
         ONE_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 8: EQUIDC B */

{       AXIS_TYPE,       AXIS_TYPE,     NOT_CHECKED,        DMS_TYPE,
         DMS_TYPE,        DMS_TYPE,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         ONE_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 20: HOM B */

{       AXIS_TYPE,       AXIS_TYPE,     SATNUM_TYPE,       PATH_TYPE,
  UNDEFINED_FIELD, UNDEFINED_FIELD,     NOT_CHECKED,     NOT_CHECKED,
  UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD, UNDEFINED_FIELD,
         ONE_TYPE, UNDEFINED_FIELD, UNDEFINED_FIELD }, /* 22: SOM B */
};


/******************************************************************************

NAME:c_check_proj_params

PURPOSE: Verify projection parameters for each type of projection.

PROGRAM HISTORY:
  Version       Date       Author           Request
  -------       ----       ------           -------
    1.0         10/00      A.Shimitz        original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
        Runs under TAE

PROJECT: Landsat Pathfinder

ALGORITHM DESCRIPTION:

NOTE:
******************************************************************************/

int c_check_proj_params
(
   const double projparm[COEFCT],/* I: Array of COEFCT projection parameters */
   long inproj                   /* I: Input projection number               */
)
{
   int i;                    /* looping index                                */
   const int *table_ptr;     /* ptr to given projection's valdity array      */

   /* First thing to do is make sure we have a valid system number.
   ----------------------------------------------------------------*/
   if((inproj < 0) || (inproj > MAXPROJ))
      return(proj_err(1001));

   /* Get the appropriate projection validity array for reference.  We have
      3 special cases: EQUIDC, HOM, and SOM have both an "A" and "B" format.
      One parameter is used as a flag to indicate which version.  The flag is 
      at proj_valid[8] for EQUIDC and at proj_valid[12] for both HOM and SOM.
      0 - format "A" projection parameters
      1 - format "B" projection parameters
   --------------------------------------------------------------------------*/
   if((inproj == EQUIDC) && (projparm[8] == 1))
      table_ptr = projparm_valid[EQUIDC_B];
   else if((inproj == HOM) && (projparm[12] == 1))
      table_ptr = projparm_valid[HOM_B];
   else if((inproj == SOM) && (projparm[12] == 1))
      table_ptr = projparm_valid[SOM_B];
   else
      table_ptr = projparm_valid[inproj];

   /* Check for valid DMS angles using check_dms. Verify the semi-major and
      semi-minor axis (or sphere) is not less than zero.
   ------------------------------------------------------------------------*/
   for(i = 0; i < COEFCT; i++)
   {
      if(table_ptr[i] == DMS_TYPE)
      {
         if(check_dms(projparm[i]) != E_SUCC)
            return(proj_err(1116));
      }
      else if(table_ptr[i] == AXIS_TYPE)
      {
         if(projparm[i] < 0.0)
            return(proj_err(115));
      }
      else if(table_ptr[i] == SATNUM_TYPE)
      {
         if((projparm[i] != 1.0) && (projparm[i] != 2.0) &&
            (projparm[i] != 3.0) && (projparm[i] != 4.0) &&
            (projparm[i] != 5.0) && (projparm[i] != 7.0))
            return(proj_err(115));
      }
      else if(table_ptr[i] == PATH_TYPE)
      {
         if((projparm[i] < 1.0) || (projparm[i] > 251.0))
            return(proj_err(115));
      }
      else if(table_ptr[i] == ZERO_TYPE)
      {
         if((projparm[i] < (0.0 - TOL)) || (projparm[i] > (0.0 + TOL)))
            return(proj_err(115));
      }
      else if(table_ptr[i] == ONE_TYPE)
      {
         if((projparm[i] < (1.0 - TOL)) || (projparm[i] > (1.0 + TOL)))
            return(proj_err(115));
      }
      else if(table_ptr[i] == PSREV_TYPE)
      {
         if(projparm[i] <= 0.0)
            return(proj_err(115));
      }
      else if(table_ptr[i] == LRAT_TYPE)
      {
         if((projparm[i] < (LRAT_VAL - LRAT_TOL)) ||
            (projparm[i] > (LRAT_VAL + LRAT_TOL)))
            return(proj_err(115));
      }
      else if(table_ptr[i] == PFLAG_TYPE)
      {
         if((projparm[i] < (0.0 - TOL)) || (projparm[i] > (1.0 + TOL)) ||
            ((projparm[i] > (0.0 + TOL)) && (projparm[i] < (1.0 - TOL))))
            return(proj_err(115));
      }
      else if(table_ptr[i] == IZONE_TYPE)
      {
         /* make sure the zone number is an integer in the range expected 
            by the isin projection code */
         int nzone = (int)(projparm[i] + ISIN_TOL);
         if ((fabs(projparm[i] - nzone) > ISIN_TOL) || /*not close to integer*/
             (nzone < 2) || /* less than minimum */
             nzone > (360*3600)) /* greater than maximum number of zones */
         {
            return(proj_err(115));
         }
      }
      else if(table_ptr[i] == RFLAG_TYPE)
      {
         int rflag = (int)(projparm[i] + ISIN_TOL);
         if ((fabs(projparm[i] - rflag) > ISIN_TOL) || /*not close to integer*/
             (rflag < 0) || (rflag > 2)) /* not 0,1, or 2 */
         {
            return(proj_err(115));
         }
      }
   }
   return (E_SUCC);
}
