
/******************************************************************************
FUNCTION:       c_makegrid

PURPOSE:
    c_makegrid creates a grid relating output space to input space.  The number
    of lines and samples in the grid cells can be specified or set to zero to
    have the function select the number and size of the grid cells to meet 
    error tolerance specified.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      9/23/99  T. Beckmann         Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT:  LAS

*******************************************************************************/

#include <math.h>
#include "las.h"
#include "util.h"
#include "geompak.h"

#define MAX_GRID_CELLS 4095
/* Maximum number of cells in a row or column of the grid.  There is nothing 
   magical about this number, it was just selected as some sufficiently large
   to meet the current needs but small enough to not cause excessive 
   calculations */
   
#define MIN_CELL_SIZE  5
/* The smallest number of lines/samples in a grid cell */

#define SEARCH_GRID_SIZE 100
/* Defines the maximum rows and columns in the grid to test when determining
   how dense the grid should be made */
   
/* #define DEBUG */
/* uncomment to provide debug messages */


/*****************************************************************************

Function:  find_grid_spacing

Purpose:
    find_grid_spacing finds the grid density for the provided grid row or 
    column that allows it to meet or beat the error tolerance.

Inputs:
    - output space lines or samples
    - input space lines
    - input space samples
    - number of elements in the output space and input space line/sample arrays
    - maximum allowed error tolerance
    - maximum grid point spacing found so far
    
Outputs:
    - maximum grid point spacing that meets the error tolerance
    
Returns:
    - E_FAIL/E_SUCC

Algorithm:
    Set the current grid spacing to the maximum grid point spacing
    Repeat
        Initialize the max error to -1
        For each grid cell using the current spacing
            Calculate the error at each point in the maximum density grid
            Track the largest error
        If the largest error is greater than the error tolerance
            Estimate a new reduced grid spacing
    Until the maximum error found is with tolerance or the max spacing between
        grid points reaches the highest density
    Set the maximum grid point spacing to the current grid spacing
    
*****************************************************************************/


static int find_grid_spacing
(
    const int *output_space,    /* I: output space lines or samples array 
                                      (depending on whether columns or rows are 
                                      being reduced) */
    const double *ilines,       /* I: input space lines array */
    const double *isamps,       /* I: input space samples array */
    int elements,               /* I: number of elements in the arrays */ 
    double tolerance,           /* I: error tolerance */
    int *max_spacing            /* I/O: max spacing between grid points */
)
{
    int current_spacing = *max_spacing; /* default to previous limit */ 
    int start_index;        /* index of starting grid point for current range
                               calculating error for */
    int end_index;          /* index of ending grid point for current range 
                               calculating error for */
    int check_index;        /* index of grid point currently calculating error
                               for */
    int output_pixels;      /* output pixels in current range for which errors
                               are being calculated */
    double in_line_start;   /* cached input space start line for current 
                               range */
    double in_samp_start;   /* cached input space start sample for current 
                               range */
    double in_lines_per_out_pixel; /* input space lines for each output pixel 
                                      in current range */
    double in_samps_per_out_pixel; /* input space samples for each output pixel
                                      in current range */
    int out_space;          /* output space pixels to current point in range */
    double errory, errorx;  /* error between estimate and actuals */
    double linear_est_line; /* line linear estimate for grid point */
    double linear_est_samp; /* sample linear estimate for grid point */
    double maxerror;        /* max error found for the current cell spacing */
    int maxloc = 0;         /* grid location where maxerror was found */

    /* loop until the maximum error found using the current cell spacing 
       is within tolerance or the maximum spacing has hit the minimum */
    do
    {
        maxerror = -1.0;  /* initialize the maximum error */
        
        /* calculate the errors at each grid point using the current grid point
           spacing */
        for (start_index = 0; start_index < elements - 1; 
             start_index+=current_spacing)
        {
            /* set the end index for the current spacing, limiting it to be 
               within the number of elements in the arrays */
            end_index = start_index + current_spacing;
            if (end_index > elements - 1)
                end_index = elements - 1;

            /* calculate the output space pixels in the current range */
            output_pixels = output_space[end_index] - output_space[start_index];
            
            /* cache the input space line and sample at the start of the
               current range */
            in_line_start = ilines[start_index];
            in_samp_start = isamps[start_index];
            
            /* calculate the input space lines/samples per output pixel in the
               current range to use for doing the linear estimates of the error
               at each grid point */
            in_lines_per_out_pixel = (ilines[end_index] - in_line_start) /
                                     output_pixels;
            in_samps_per_out_pixel = (isamps[end_index] - in_samp_start) /
                                     output_pixels;

            /* calculate the error at each grid point between the start and
               end index in the current range */
            for (check_index = start_index + 1; check_index < end_index;
                 check_index++)
            {
                /* get the pixels in output space from the start of the range
                   to the current point in the range */
                out_space = output_space[check_index] - 
                                output_space[start_index];

                /* calculate the linear estimate for the line/sample at the 
                   current grid point using the two endpoints of the range */
                linear_est_line = in_line_start + (in_lines_per_out_pixel * 
                                  out_space);
                linear_est_samp = in_samp_start + (in_samps_per_out_pixel * 
                                  out_space);

                /* calculate the error between the linearly estimated input
                   space line/sample verses the actual input space line sample
                   calculated at the current grid point */
                errorx = fabs(linear_est_samp - isamps[check_index]);
                errory = fabs(linear_est_line - ilines[check_index]);
                
                /* keep track of the maximum error and where it occurred */
                if (errorx > maxerror)
                {
                    maxerror = errorx;
                    maxloc = check_index;
                }
                if (errory > maxerror)
                {
                    maxerror = errory;
                    maxloc = check_index;
                }
            }
        }

#ifdef DEBUG
        c_lasmsg(FALSE,"reduce-debug","error = %lf at %d with spacing of %d",
                 maxerror,maxloc,current_spacing);
#endif

        /* if max error isn't in tolerance, select new max spacing */
        if (maxerror > tolerance)
        {
            int new_spacing;
            double error_to_tol_ratio = maxerror/tolerance;
           
            /* guess what the new spacing should be */
            if (error_to_tol_ratio > 4)
            {
                /* the error is relatively large, so force the error to zero
                   at the size of spacing where the maximum error was found */
                new_spacing = maxloc % current_spacing;
            }
            else if (maxerror/tolerance > 2)
            {
                /* the max error is approaching the tolerance, so estimate the
                   new spacing by assuming the spacing needs to be reduced 
                   by twice the tolerance to error ratio */
                new_spacing = current_spacing / error_to_tol_ratio * 2.0;
            }
            else
            {
                /* the max error is very close to the tolerance, so assume 
                   the spacing can be reduce in a nearly linear fashion */
                new_spacing = current_spacing / error_to_tol_ratio + 1;
            }
            
            /* make sure the spacing actually changed so we converge to an 
               answer */
            if (new_spacing >= current_spacing)
                current_spacing--;
            else if ((new_spacing <= 1) && (current_spacing > 2))
                current_spacing = 2;
            else
                current_spacing = new_spacing;
        }

    } while ((maxerror > tolerance) && (current_spacing >= 2));

    /* return the current spacing as the new maximum allowed spacing */
    *max_spacing = current_spacing;
        
    return (E_SUCC);
}

/****************************************************************************

Function:  size_grid

Purpose:
    size_grid calculates the grid density needed to meet or beat a specified
    error tolerance.

Inputs:
    - window
    - ols2ils pointer
    - ols2ils "data" pointer
    - error tolerance

Outputs:
    - rows in grid
    - columns in grid
    - lines per grid cell
    - samples per grid cell

Returns:
    - E_FAIL/E_SUCC
    
Algorithm:
    Based on the minimum allowed grid cell size and the maximum number of 
        rows/columns in the grid, calculate the highest density grid to 
        consider
    Allocate space for a single row and column of the grid
    Initialize the output space grid line and sample arrays
    Calculate the spacing between the rows to check minimum grid size
    Initialize the minimum grid density to a single cell grid in the row 
        direction
    Repeat
        Select the next row to test
        Calculate the input space line and sample at each point in output
            space for that row
        Find the minimum grid density that meets the error tolerance for
            this row
        If the minimum grid density changed
            Save the current row as the location where the minimum grid 
                density was established
        Move to the next row index
    Until a grid spacing is found that meets the error tolerance for all
        the rows or until it is determined that the highest grid density 
        must be used
    Calculate the spacing between the columns to check minimum grid size
    Initialize the minimum grid density to a single cell grid in the column 
        direction
    Repeat
        Select the next column to test
        Calculate the input space line and sample at each point in output
            space for that column
        Find the minimum grid density that meets the error tolerance for
            this column
        If the minimum grid density changed
            Save the current column as the location where the minimum grid 
                density was established
        Move to the next column index
    Until a grid spacing is found that meets the error tolerance for all
        the columns or until it is determined that the highest grid density 
        must be used
    Free the memory allocated
    Calculate the rows and columns in the grid
    Calculate the lines and samples per grid cell

*****************************************************************************/

static int size_grid
(
    int window[4],          /* I: sl, ss, nl, ns in grid */
    int (*ols2ils)(void *data, int oline, int osamp, double *iline, 
                      double *isamp), /* I: function pointer for out line/sample
                                            to in line/sample */
    void *data,               /* I: data to pass to ols2ils (defined by 
                                    particular application) */
    double tolerance,         /* I: max error tolerance at each grid point */
    int *cal_rows,            /* O: rows calculated */
    int *cal_columns,         /* O: columns calculated */
    int *ret_lines_per_cell,  /* O: lines per grid cell */
    int *ret_samples_per_cell /* O: samples per grid cell */
)
{
    int *olines = NULL;     /* output space lines array pointer */
    int *osamps = NULL;     /* output space samples array pointer */
    double *ilines = NULL;  /* input space lines array pointer */
    double *isamps = NULL;  /* input space samples array pointer */
    int max_rows;           /* maximum number of rows to allow in the grid */
    int max_columns;        /* maximum number of columns to allow in the grid */
    int max_elements;       /* largest of max_rows and max_columns */
    int lines_per_cell;     /* lines included in a grid cell */
    int samps_per_cell;     /* samples included in a grid cell */
    int line;               /* current line */
    int sample;             /* current sample */
    int row;                /* current row */
    int column;             /* current column */
    int search_grid_size;   /* number of locations through the grid to check for
                               the mapping to be within tolerance */
    int search;             /* current search row/column in grid */
    double search_interval; /* interval between grid rows/columns to test for 
                               grid spacing to be in tolerance */
    int col_max_spacing;    /* max spacing found to meet tolerance requirements
                               for the grid columns */
    int row_max_spacing;    /* max spacing found to meet tolerance requirements
                               for the grid rows */
    int saved_max_spacing;  /* temporary storage for max spacing found */
    int last_space_change;  /* last search location where the spacing changed */

    /* issue a message that the optimum grid size is being determined */
    c_lasmsg(FALSE,"informational","Calculating optimum grid size...");

    /* initialize the returned parameters */
    *cal_rows = 0;
    *cal_columns = 0;
    *ret_lines_per_cell = 0;
    *ret_samples_per_cell = 0;
    
    /* calculate the maximum number of cells allowed based on the minimum 
       cell size.  Add (MIN_CELL_SIZE - 1) to the window size to force any 
       fractional rows to add a row.  Add 1 to account for the endpoint. */
    max_rows = (window[2] + MIN_CELL_SIZE - 1)/MIN_CELL_SIZE + 1;
    max_columns = (window[3] + MIN_CELL_SIZE - 1)/MIN_CELL_SIZE + 1;
    lines_per_cell = MIN_CELL_SIZE;
    samps_per_cell = MIN_CELL_SIZE;

    /* if the maximum rows/columns exceeds the maximum number of cells allowed 
       in the grid, resize the cells to a larger size and reduce the number of
       grid cells to match */
    if (max_rows > MAX_GRID_CELLS)
    {
        lines_per_cell =  window[2]/(MAX_GRID_CELLS - 1);
        max_rows = (window[2] + lines_per_cell - 1)/lines_per_cell + 1;
    }
    if (max_columns > MAX_GRID_CELLS)
    {
        samps_per_cell = window[3]/(MAX_GRID_CELLS - 1);
        max_columns = (window[3] + samps_per_cell - 1)/samps_per_cell + 1;
    }
   
    /* initialize the max spacing to be one less than the max columns/rows 
       since the arrays are zero based */
    col_max_spacing = max_columns-1;
    row_max_spacing = max_rows-1;

    /* allocate memory for a single row and column of the grid for the grid 
       resolution search (pick the largest for spacing for the input space 
       data so the same buffer can be used when calculating rows and columns) */
    if (max_columns > max_rows)
        max_elements = max_columns;
    else
        max_elements = max_rows;
    olines = (int *)malloc(max_rows * sizeof(int));
    osamps = (int *)malloc(max_columns * sizeof(int));
    ilines = (double *)malloc(max_elements * sizeof(double));
    isamps = (double *)malloc(max_elements * sizeof(double));
    if ((olines == NULL) || (osamps == NULL) || (ilines == NULL) || 
        (isamps == NULL))
    {
        free (olines);
        free (osamps);
        free (ilines);
        free (isamps);
        c_lasmsg(FALSE,"makegrid-size","Error allocating dynamic memory");
        return (E_FAIL);
    }
    
    /* initialize the output space lines and samples */
    for (row = 0; row < max_rows; row++)
    {
        olines[row] = window[0] + (row * lines_per_cell);
    }
    
    for (column = 0; column < max_columns; column++)
    {
        osamps[column] = window[1] + (column * samps_per_cell);
    }
    
    /* calculate the interval of rows to check in the grid for the errors 
       meeting the tolerance */
    search_grid_size = SEARCH_GRID_SIZE;
    search_interval = (double)max_rows/search_grid_size;
    if (search_interval < 1.0)
    {
        /* the grid isn't dense enough to support the SEARCH_GRID_SIZE, so 
           check every row that is there */
        search_interval = 1.0;
        search_grid_size = max_rows;
    }

    /* loop until all search intervals have been checked at the current spacing
       or until the spacing has been reduced down to every grid point */
    search = 0;
    last_space_change = search;
    do 
    {
#ifdef DEBUG        
        c_lasmsg(FALSE,"makegrid-debug","search interval %d",search);
#endif
        /* calculate the row in the grid to check on this pass */
        row = (int)(search * search_interval);
        if (row >= max_rows)
            row = max_rows-1;
            
        /* cache the current row's output line */
        line = olines[row];
        
        /* build the input space line/sample arrays for this row */
        for (column = 0; column < max_columns; column++)
        {
            if (ols2ils(data, line, osamps[column], &ilines[column],
                        &isamps[column]) != E_SUCC)
            {
                c_lasmsg(FALSE,"makegrid-trace","Error in ols2ils");
                return (E_FAIL);
            }
        }
        
        /* reduce the row to the lowest density grid spacing that meets the 
           tolerance */
        saved_max_spacing = col_max_spacing;
        if (find_grid_spacing(osamps,ilines,isamps,max_columns,tolerance,
                              &col_max_spacing) != E_SUCC)
        {
            c_lasmsg(FALSE,"makegrid-trace","Error in find_grid_spacing");
            return (E_FAIL);
        }

        /* if the spacing was changed in the reduction, keep track of the
           search index where the change occurred so all the search rows 
           down at a different spacing can be rechecked at the new spacing */
        if (saved_max_spacing != col_max_spacing)
            last_space_change = search;

        /* move to the next search location, wrapping when needed */
        search++;
        if (search >= search_grid_size)
            search = 0;
            
    } while ((search != last_space_change) && (col_max_spacing > 1));

    /* calculate the interval of columns to check in the grid for the errors 
       meeting the tolerance */
    search_grid_size = SEARCH_GRID_SIZE;
    search_interval = (double)max_columns/search_grid_size;
    if (search_interval < 1.0)
    {
        /* the grid isn't dense enough to support the SEARCH_GRID_SIZE, so 
           check every column that is there */
        search_interval = 1.0;
        search_grid_size = max_columns;
    }

    /* loop until all search intervals have been checked at the current spacing
       or until the spacing has been reduced down to every grid point */
    search = 0;
    last_space_change = search;
    do
    {
#ifdef DEBUG        
        c_lasmsg(FALSE,"makegrid-debug","search interval %d",search);
#endif
        /* calculate the column in the grid to check on this pass */
        column = (int)(search * search_interval);
        if (column >= max_columns)
            column = max_columns - 1;
        
        /* cache the current column's output sample */
        sample = osamps[column];
       
        /* build the input space line/sample arrays for this column */
        for (row = 0; row < max_rows; row++)
        {
            line = olines[row];

            if (ols2ils(data, line, sample, &ilines[row],&isamps[row]) 
                    != E_SUCC)
            {
                c_lasmsg(FALSE,"makegrid-trace","Error in ols2ils");
                return (E_FAIL);
            }
        }
        
        /* reduce the column to the lowest density grid spacing that meets 
           the tolerance */
        saved_max_spacing = row_max_spacing;
        if (find_grid_spacing(olines,ilines,isamps,max_rows,tolerance,
                              &row_max_spacing) != E_SUCC)
        {
            c_lasmsg(FALSE,"makegrid-trace","Error in find_grid_spacing");
            return (E_FAIL);
        }

        /* if the spacing was changed in the reduction, keep track of the
           search index where the change occurred so all the search rows 
           down at a different spacing can be rechecked at the new spacing */
        if (saved_max_spacing != row_max_spacing)
            last_space_change = search;
        
        /* move to the next search location, wrapping when needed */
        search++;
        if (search >= search_grid_size)
            search = 0;
            
    } while ((search != last_space_change) && (row_max_spacing > 1));
    
    /* free memory */
    free (olines); olines = NULL;
    free (osamps); osamps = NULL;
    free (ilines); ilines = NULL;
    free (isamps); isamps = NULL;
  
    /* calculate the grid size */
/*    *cal_rows = (max_rows + row_max_spacing - 1)/row_max_spacing + 1;
    *cal_columns = (max_columns + col_max_spacing - 1)/col_max_spacing + 1;
*/
    if (((max_rows - 1) % row_max_spacing) == 0)
        *cal_rows = (max_rows - 1)/row_max_spacing + 1;
    else
        *cal_rows = (max_rows - 1)/row_max_spacing + 2;
    if (((max_columns - 1) % col_max_spacing) == 0)
        *cal_columns = (max_columns - 1)/col_max_spacing + 1;
    else
        *cal_columns = (max_columns - 1)/col_max_spacing + 2;


    *ret_lines_per_cell = row_max_spacing * lines_per_cell;
    *ret_samples_per_cell = col_max_spacing * samps_per_cell;

#ifdef DEBUG
    c_lasmsg(FALSE,"makegrid-debug","Maximum column cell spacing %d",
             col_max_spacing);
    c_lasmsg(FALSE,"makegrid-debug","Samples per cell: %d",
             *ret_samples_per_cell);
    c_lasmsg(FALSE,"makegrid-debug","Columns in grid: %d",*cal_columns);
    c_lasmsg(FALSE,"makegrid-debug","Maximum row cell spacing %d",
             row_max_spacing);
    c_lasmsg(FALSE,"makegrid-debug","Lines per cell: %d",*ret_lines_per_cell);
    c_lasmsg(FALSE,"makegrid-debug","Rows in grid: %d",*cal_rows);
#endif

    return (E_SUCC);
}

/****************************************************************************

Function:  c_makegrid

Purpose:
    c_makegrid creates a grid relating output space to input space.  The number
    of lines and samples in the grid cells can be specified or set to zero to
    have the function select the number and size of the grid cells to meet 
    error tolerance specified.

Inputs:
    - window
    - lines per grid cell
    - samples per grid cell
    - pointer to function to convert output space line/sample into an
      input space line/sample (ols2ils)
    - "data" pointer to allow setup data to be passed to the ols2ils routine
      from the different gridding applications
    - error tolerance allowed in grid
    - pointer to a grid structure
    
Outputs:
    - calculated grid

Returns:
    - E_FAIL/E_SUCC

Algorithm:
    If lines or samples per cell are illegal or zero
        Calculate the optimum lines and samples per
            grid cell by calling size_grid
    Allocate space for the grid
    Build the output lines/samples arrays
    Calculate the input space line/sample at each grid point


*****************************************************************************/

int c_makegrid
(
    int window[4],              /* I: sl, ss, nl, ns in grid */
    int lines_per_cell,         /* I: user requested lines per grid cell
                                      (0 = find to within tolerance) */
    int samples_per_cell,       /* I: user requested samples per grid cell
                                      (0 = find to within tolerance) */
    int (*ols2ils)(void *data, int oline, int osamp, double *iline, 
                   double *isamp), /* I: pointer to function to convert output
                                         space line/sample to input space 
                                         line/sample */
    void *data,                 /* I: implementation specific data structure
                                      to pass to ols2ils function */
    double tolerance,           /* I: tolerance on errors in the grid (units
                                      is number of pixels) */
    char *application_name,     /* I: name of application for the status
                                      line */
    long message_time,          /* I: time between message updates, set to 
                                      zero if no messages desired */
    struct GEOGRID *grid        /* O: grid to make */
)
{
    int grid_rows;      /* rows in the grid */
    int grid_columns;   /* columns in the grid */
    int elements;       /* elements in a grid row/column */
    int count;          /* loop counter */
    int row, column;    /* loop counters */
    float temp_float;   /* temporary float variable */
    extern long curent; /* Current "line" element for processing message */
    long one = 1;       /* parameter for c_inittm */
    long three = 3;     /* parameter for c_inittm */
    
    /* Place the number of output lines and samples into the grid record */
    grid->lines = window[0] + window[2] - 1;
    grid->samples = window[1] + window[3] - 1;
    
    /* verify the lines and samples per cell give a valid number of rows
       and columns.  If not, the lines and samples per cell to zero and 
       size the grid. */
    if ((lines_per_cell > 0) && (samples_per_cell > 0))
    {
        temp_float = window[2] / (float)lines_per_cell + 1;
        grid_rows = (int)temp_float;
        if ((float)grid_rows != temp_float) grid_rows++;
        if (grid_rows > MAX_GRID_CELLS)
        {
            /* too may rows requested, force it to default */
            lines_per_cell = 0;
            c_lasmsg(FALSE,"makegrid-nrows",
                     "Number or grid rows exceeded maximum");
            c_lasmsg(FALSE,"makegrid-nrows",
                     "The optimum number of grid cells will be calculated");
            c_lasmsg(FALSE,"makegrid-warn","Nonfatal error encountered");
        }
        else
        {
            /* the number of grid rows is legal, so now check the grid columns
               (if the number of grid rows isn't legal, don't bother doing this
               since it will be overridden when the grid is automatically 
               sized */
            temp_float = window[3] / (float)samples_per_cell + 1;
            grid_columns = (int)temp_float;
            if ((float)grid_columns != temp_float) grid_columns++;
            if (grid_columns > MAX_GRID_CELLS)
            {
                /* too may rows requested, force it to default */
                samples_per_cell = 0;
                c_lasmsg(FALSE,"makegrid-ncols",
                         "Number or grid columns exceeded maximum");
                c_lasmsg(FALSE,"makegrid-ncols",
                         "The optimum number of grid cells will be calculated");
                c_lasmsg(FALSE,"makegrid-warn","Nonfatal error encountered");
            }
        }
    }

    /* if the lines or samples per cell is not specified, calculate the 
       optimum grid cell size */
    if ((lines_per_cell <= 0) || (samples_per_cell <= 0))
    {
        if (size_grid(window, ols2ils, data, tolerance, &grid_rows, 
                      &grid_columns, &lines_per_cell, &samples_per_cell) 
                      != E_SUCC)
        {
            c_lasmsg(FALSE,"makegrid-size",
                     "Error calculating the grid cell size");
            return (E_FAIL);
        }
    }

    /* set the grid rows and columns */
    grid->nrows = grid_rows;
    grid->ncols = grid_columns;
    
    /* allocate memory for the grid components */
    elements = grid_rows * grid_columns;
    grid->out_lines = (long *)malloc(grid_rows * sizeof(long));
    grid->out_samps = (long *)malloc(grid_columns * sizeof(long));
    grid->in_lines = (double *)malloc(elements * sizeof(double));
    grid->in_samps = (double *)malloc(elements * sizeof(double));
    if((grid->out_lines == NULL) || (grid->out_samps == NULL) ||
        (grid->in_lines == NULL) || (grid->in_samps == NULL)) 
    {
        free (grid->out_lines);  grid->out_lines = NULL;
        free (grid->out_samps);  grid->out_samps = NULL;
        free (grid->in_lines);   grid->in_lines = NULL;
        free (grid->in_samps);   grid->in_samps = NULL;
        c_lasmsg(FALSE,"makegrid-alloc","Error allocating dynamic memory");
        return(NO_SPACE);
    }
    
    /* Calculate the output grid intersections and place in geometric mapping
       grid record. */
    for (count = 0; count < grid_rows; count++)
    {
        grid->out_lines[count] = window[0] + (count * lines_per_cell);
    }

    for (count = 0; count < grid_columns; count++)
    {
       grid->out_samps[count] = window[1] + (count * samples_per_cell);
    }

    /* start the message timer */
    curent = 1;
    c_inittm(application_name,"Generated","grid element","","",&elements,&one,
             &message_time,&three);
    c_startm();
        
    /* build the full grid */
    for (row = 0; row < grid_rows; row++)
    {
        count = grid_columns * row;
        
        for (column = 0; column < grid_columns; column++, count++)
        {
            if (ols2ils(data, grid->out_lines[row], grid->out_samps[column],
                        &grid->in_lines[count], &grid->in_samps[count])
                        != E_SUCC)
            {
                c_stoptm();
                c_lasmsg(FALSE,"makegrid-trace","Error returned from ols2ils");
                free (grid->out_lines);  grid->out_lines = NULL;
                free (grid->out_samps);  grid->out_samps = NULL;
                free (grid->in_lines);   grid->in_lines = NULL;
                free (grid->in_samps);   grid->in_samps = NULL;
                return (E_FAIL);
            }
            curent++;  /* update display */
        }
    }
    
    c_stoptm();
    
    return (E_SUCC);
}


