/*******************************************************************************
NAME                          c_minor2recip

PURPOSE      Calculate recipricol flattening based on semi minor axis.  

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

REFERENCES

1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  "Appendix E Projection Transformation Package" of LAS 7.1 
    "Geometric Registration Overview"

*******************************************************************************/
#include <stdlib.h>
#include <math.h>
#include "tae.h"
#include "worgen.h"
#include "geompak.h"

int c_minor2recip
(
    double *semi_major,		/* I: semi major axis			*/
    double *semi_minor,		/* I: semi minor axis			*/
    double *recip_flat		/* O: calculated recipricol flattening  */
)
{

double flat = 0.0;		/* Flattening of the spheroid */
long nonfatal = 1;		/* nonfatal error code        */

/* If the value is greater than 1, it represents the semi-minor
 -------------------------------------------------------------------*/
if(*semi_minor > 1.0)
    {
    flat = 1.0 - ((*semi_minor)/(*semi_major));
    *recip_flat = (1.0/flat);
    }


/* If the value is less than 1, it represents the eccentricity squared
 --------------------------------------------------------------------*/
else if((*semi_minor < 1.0) && (*semi_minor != 0.0)) 
    {
    flat = fabs(sqrt(1.0-(*semi_minor))-1.0);
    *recip_flat = (1.0/flat);
    }


/* If the value is equal to 1, error 
----------------------------------*/
else if(*semi_minor == 1.0)
    {
    c_errmsg("Invalid value for Semi Minor Axis","recip-minor",&nonfatal);
    return (E_FAIL);
    }

/* If the value equals zero, it is a sphere
 -----------------------------------------*/
else
    {
    *recip_flat = 0.0;
    }

return(E_SUCC);
} 
