/******************************************************************************
NAME: c_projstr2num

PURPOSE: To convert the projection name (as a string) to its projection code.

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
  7.2    05/00  G.Schmidt        EDC   Original development
  7.3    12/00  A.Shimitz        EDC   Added free(pname) at end of routine
                                       (CR 4382)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

ALGORITHM DESCRIPTION:
  Input the projection name.
  Copy the projection name.
  Convert the projection name to lower case.
  Remove trailing blank spaces.
  Determine the corresponding projection code.
  Output the projection code.

NOTES:
  If the projection name is not found, then this routine will return -1.
  The projections listed in proj.h are used for the codes returned from
    this routine.  Some of the projection string comparisons may need to be
    updated to work with different abbreviations of the projection names.

******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "proj.h"

int c_projstr2num (
  char projname[])      /* I: projection name */
{
  int i;                /* looping variable                                */
  int pcode;            /* projection code for input projection name       */
  long strsize;         /* size of the pname string                        */
  long fatal = -1;      /* fatal constant                                  */
  long one = 1;         /* constant one                                    */
  char *pname = NULL;   /* copy of projname for the lower case conversion  */

  /* Copy projname and convert to lower case for the comparisons
   */
  pname = strdup (projname);
  if (pname == NULL)
  {
    c_errmsg ("Error duplication the projection string", "projstr2num-strdup",
      &one);
    c_errmsg ("Fatal error encountered", "projstr2num-projname", &fatal);
  }

  strsize = (long) strlen (pname);
  c_up2low (pname, &strsize);

  /* Remove the trailing blank spaces.
   */
  for (i = 0; i < strlen (pname); i++)
  {
    if (pname[i] == ' ')
    {
      pname[i] = '\0';
      break;
    }
  }

  /* Determine the projection code
   */
  if (!strcmp (pname, "geo"))
    pcode = GEO;
  else if (!strcmp (pname, "utm"))
    pcode = UTM;
  else if (!strcmp (pname, "spcs"))
    pcode = SPCS;
  else if (!strcmp (pname, "acea"))
    pcode = ALBERS;
  else if (!strcmp (pname, "albers"))
    pcode = ALBERS;
  else if (!strcmp (pname, "lcc"))
    pcode = LAMCC;
  else if (!strcmp (pname, "mer"))
    pcode = MERCAT;
  else if (!strcmp (pname, "ps"))
    pcode = PS;
  else if (!strcmp (pname, "pol"))
    pcode = POLYC;
  else if (!strcmp (pname, "pc"))
    pcode = POLYC;
  else if (!strcmp (pname, "ec"))
    pcode = EQUIDC;
  else if (!strcmp (pname, "tm"))
    pcode = TM;
  else if (!strcmp (pname, "sg"))
    pcode = STEREO;
  else if (!strcmp (pname, "laea"))
    pcode = LAMAZ;
  else if (!strcmp (pname, "ae"))
    pcode = AZMEQD;
  else if (!strcmp (pname, "gno"))
    pcode = GNOMON;
  else if (!strcmp (pname, "og"))
    pcode = ORTHO;
  else if (!strcmp (pname, "gvnp"))
    pcode = GVNSP;
  else if (!strcmp (pname, "sin"))
    pcode = SNSOID;
  else if (!strcmp (pname, "er"))
    pcode = EQRECT;
  else if (!strcmp (pname, "mc"))
    pcode = MILLER;
  else if (!strcmp (pname, "vdg"))
    pcode = VGRINT;
  else if (!strcmp (pname, "om"))
    pcode = HOM;
  else if (!strcmp (pname, "rob"))
    pcode = ROBIN;
  else if (!strcmp (pname, "robin"))
    pcode = ROBIN;
  else if (!strcmp (pname, "som"))
    pcode = SOM;
  else if (!strcmp (pname, "ac"))
    pcode = ALASKA;
  else if (!strcmp (pname, "alaska"))
    pcode = ALASKA;
  else if (!strcmp (pname, "good"))
    pcode = GOOD;
  else if (!strcmp (pname, "goode"))
    pcode = GOOD;
  else if (!strcmp (pname, "igh"))
    pcode = GOOD;
  else if (!strcmp (pname, "moll"))
    pcode = MOLL;
  else if (!strcmp (pname, "im"))
    pcode = IMOLL;
  else if (!strcmp (pname, "imoll"))
    pcode = IMOLL;
  else if (!strcmp (pname, "ham"))
    pcode = HAMMER;
  else if (!strcmp (pname, "hammer"))
    pcode = HAMMER;
  else if (!strcmp (pname, "wagiv"))
    pcode = WAGIV;
  else if (!strcmp (pname, "wagvii"))
    pcode = WAGVII;
  else if (!strcmp (pname, "obeqa"))
    pcode = OBEQA;
  else if (!strcmp (pname, "oea"))
    pcode = OBEQA;
  else
  {
    c_errmsg ("Invalid projection name", "projstr2num-projname", &one);
    pcode = -1;
  }

  free(pname);
  return (pcode);
}
