/****************************************************************************
NAME				c_putrwt

PURPOSE		Write a resampling weight table to disk

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
D. Steinwand	     	2/89		Original development
D. Etrheim		7/90		Standardized error message handling

PROJECT		LAS

ALGORITHM
   Check for existance of the output file.  Abort if already there.
   Open (create) the resampling weight table file for write access using
    label services.
   Write the resampling weight table file to disk
   Close the file
******************************************************************************/

#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "geompak.h"

int c_putrwt
(
    char *host_name,	/* Host file name of the rwt file */
    struct RWTAB *table	/* Resampling weight table (rwt) */
)
{
long one = 1;		/* constant of 1, used for non-fatal error status */
long mode = 0;		/* access mode for lsstat */
long access = 1;	/* access mode for lsopen, set to write access */
long fd;		/* file descriptor */
long clen;		/* number of bytes of character data per record */
long dlen;		/* number of bytes of data per record */
long action = 0;	/* action parameter for lsclos */
long tmpbuf[3];		/* Temporary buffer for write */
long i,j,k;		/* Loop counters */
double tmpdbl[528];	/* Temporary buffer for write--double */
char dummy[30];		/* Dummy argument for ls write */

/* Check to see if the file aready exists.  If it does exist, an error message
   is printed.
 -----------------*/
if (c_lsstat(host_name, &mode) == E_SUCC)
   {
   c_errmsg("Resampling weight table file already exists","putrwt-exist", &one);
   return(E_FAIL);
   }

/* Open the table file
 --------------------*/
if (c_lsopen(&fd, host_name, &access) != E_SUCC)
   {
   c_errmsg("Error opening resampling weight table file for write access",
            "putrwt-open", &one);
   return(E_FAIL);
   }

/* Write the first table file record
 ----------------------------------*/
clen = 0;
dlen = 12;
tmpbuf[0] = table->dim[0];
tmpbuf[1] = table->dim[1];
tmpbuf[2] = table->dim[2];

if (c_lswrit(&fd, "RWT_INT", &clen, &dlen, dummy, (char *)tmpbuf, 
             "I4") != E_SUCC)
   {
   c_errmsg("Error writing to the resampling weight table file", "putrwt-write",
             &one);
   return(E_FAIL);
   }

/* Write the second table file record
 -----------------------------------*/
clen = 0;
dlen = 33 * (long)sizeof(double) * table->dim[0];

for (k = 0, j = 0; j < 33; j++)
   for (i = 0; i < table->dim[0]; k++, i++)
	tmpdbl[k] = table->table1[j][i];

if (c_lswrit(&fd, "RWT_TAB1", &clen, &dlen, dummy, (char *)tmpdbl, 
             "R8") != E_SUCC)
   {
   c_errmsg("Error writing to the resampling weight table file", "putrwt-write",
             &one);
   return(E_FAIL);
   }

/* Write the third table file record
 ----------------------------------*/
clen = 0;
dlen = 33 * (long)sizeof(double) * table->dim[1];

for (k = 0, j = 0; j < 33; j++)
   for (i = 0; i < table->dim[1]; k++, i++)
	tmpdbl[k] = table->table2[j][i];

if (c_lswrit(&fd, "RWT_TAB2", &clen, &dlen, dummy, (char *)tmpdbl, 
             "R8") != E_SUCC)
   {
   c_errmsg("Error writing to the resampling weight table file", "putrwt-write",
             &one);
   return(E_FAIL);
   }

/* Write the fourth table file record
 -----------------------------------*/
clen = 0;
dlen = 33 * (long)sizeof(double) * table->dim[2];

for (k = 0, j = 0; j < 33; j++)
   for (i = 0; i < table->dim[2]; k++, i++)
	tmpdbl[k] = table->table3[j][i];

if (c_lswrit(&fd, "RWT_TAB3", &clen, &dlen, dummy, (char *)tmpdbl, 
             "R8") != E_SUCC)
   {
   c_errmsg("Error writing to the resampling weight table file", "putrwt-write",
             &one);
   return(E_FAIL);
   }

/* Close the resampling weight table file 
 ---------------------------------------*/
if (c_lsclos(&fd, host_name, &action) != E_SUCC)
   {
   c_errmsg("Error closing the resampling weight table file", "putrwt-close",
             &one);
   return(E_FAIL);
   }

return(E_SUCC);
}
