/*****************************************************************************
NAME:           c_string2unit

PURPOSE:        This routine is designed to check the string units 
                for coordinates as stored in the LAS ddr. It returns an
                integer code that corresponds to the string units and
                is capable of being used by the Projection Transformation
                Package.  The routine will return a bad status (E_FAIL)
                and an output unit code of -1 for undefined (user defined) 
                unit codes.

                The routine currently supports the following code strings:

                        Radians:
                        Feet:
                        Meters:
                        Seconds:        Seconds of arc or arc seconds
                        Degrees:        Degrees of arc or arc degrees
                        Dms:            Packed DMS (DDDMMMSSS.SS)

                Note also that the input unit code string is converted to
                upper case, thereby allowing the routine to handle case
                combinations of the above.

PROGRAM HISTORY:
        By D. Akkerman, USGS/EROS Data Center, 11/88
        D. Etrheim, CSB, 7/90, Standardized error message handling
        D. Akkerman, CSB, 7/91, Fixed off-by-one error in instr memory
                                allocation.
        D. Etrheim, CSB, 10/93, Free allocated space

PROJECT:        LAS

ALGORITHM:

        Convert the input unit code string to lower case.
        Assign the output unit code based on the input unit string.
        If the input unit code is undefined then
          return E_FAIL
        Return the output unit code

******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "util.h"

int c_string2unit
(
    const char *inunit        /* I: Input unit string                    */
)
{
    char instr[8];            /* Working input unit string               */
    int outunit;              /* Output unit code                        */
    long slen;                /* Size of input string                    */ 
    
    /* Convert the input unit code string to lower case
    ---------------------------------------------------*/
    slen = (long) strlen(inunit); 
    if (slen > 7)
        slen = 7;
    strncpy(instr,inunit,slen);
    instr[slen] = '\0';
    c_up2low(instr,&slen);

    /* Assign the output unit code based on the input unit string
    -------------------------------------------------------------*/
    if (strcmp(instr,"radians") == 0)
        outunit = 0;
    else if (strcmp(instr,"feet") == 0)
        outunit = 1;
    else if (strcmp(instr,"meters") == 0)
        outunit = 2;
    else if (strcmp(instr,"seconds") == 0)
        outunit = 3;
    else if (strcmp(instr,"degrees") == 0)
        outunit = 4;
    else if (strcmp(instr,"dms") == 0)
        outunit = 5;
    else
        return(E_FAIL);

    return(outunit);
}


/*****************************************************************************
NAME:           c_ckunit

PROJECT:        LAS

NOTE: The original c_ckunit was renamed to c_string2unit. This function is now
      a wrapper function for c_string2unit. This is needed since the old name
      c_ckunit is still used in some applications.

******************************************************************************/
#include "las.h"

#ifndef NO_OBSOLETE_INTERFACE

int c_ckunit
(
    const char *inunit,         /* I: Unit code string                      */
    long *outunit               /* O: Unit code                             */

)

{
    int status;

    status = c_string2unit(inunit);

    *outunit = status;

    return (E_SUCC);
}

#endif /* Obsolete interfaces */


