/****************************************************************************
NAME:				c_regmap

PURPOSE:   Registers a map to a digtizier

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development
  5.01	 7/89	D. Steinwand  CSB      Misc clean-up (prompt re-wording)
  5.02	 8/89	D. Steinwand  CSB      Fixed bug for SUN 4

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set the numeric "units" value based on the coordinate units string
	Convert the host mapfile name to a TAE name for reports
	Loop until a map has been registered correctly (or user quits)
	   Loop until map parameters have been found or entered correctly
		Prompt for map id
		Convert to lower case
		If QUIT, return
		Get map parameters from file
		IF not found, Enter new parameters
	   If manual mode, break out of this loop
	   Register map to digitizer (spark points, fit, report residuals)
	Set/Reset the coordinate units string based on the numeric "units"
	Return
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "worgen.h"
#include "tae.h"
#include "digitizer.h"
#include "ltable.h"
#include "cm.h"
#include "util.h"
#include "labserv.h"
#include "geompak.h"

#define NEG_PI	       -3.14159265359	/* -PI */
#define HALF_PI		1.5707963268	/* PI / 2 */
#define NEG_HALF_PI    -1.5707963268	/* -(PI / 2) */
#define FIRST	1			/* First order fit */
#define SECOND	2			/* Second order fit */
#define E_MAP_NOT_FOUND	100		/* Flag--mapid not found in file */
#define E_MAP_NOT_OK 	200		/* Flag--input not ok, do over */
#define NULL_UNITS	300		/* Coordinate units undefined */

struct MAP_PARMS 	/* Structure containing mapid file parameters */
   {
   double ul_x;		/* Upper left map corner X coordinate */
   double ul_y;		/* Upper left map corner Y coordinate */
   double lr_x;		/* Lower right map corner X coordinate */
   double lr_y;		/* Lower right map corner Y coordinate */
   double inc_x;	/* Increment in X */
   double inc_y;	/* Increment in Y */
   double verify_x;	/* Verify point X coordinate */
   double verify_y;	/* Verify point Y coordinate */
   long scale;		/* Annotation field--scale of map */
   long npts_x;		/* Number  of coordinates in X */
   long npts_y;		/* Number  of coordinates in Y */
   char datum[80];	/* Annotation field--map datum */
   char maptype[80];	/* Annotation field--type of map */
   char mapdate[10];	/* Annotation field--date of map */
   char mapdesc[80];	/* Annotation field--map description & notes */
   };


int read_mapfile
(
    char *mapfile,               /* Map file name */
    char *mapid,                 /* Map identifier -- used as key */
    struct MAP_PARMS *map_params /* Map registration parameters */
);

void wrlabtab
(
    char *hosout,                /* Map file host name */
    char *mapid,                 /* Map identifer */
    struct MAP_PARMS *map_params /* Map registration parameters */
);

static long get_new_map
(
    char *mapid,			/* Map identifier */
    char *mapfile,			/* Map parameter file */
    struct MAP_PARMS *map_params	/* Map registration parameters */
);

static long get_map_rec
(
    char *mapid,			/* Map identifier */
    char *mapfile,			/* Map parameter file name */
    struct MAP_PARMS *map_params	/* Map registration parameters */
);

static void edit
(
    long number,			/* Number of field to edit/enter */
    struct MAP_PARMS *map_params	/* Map registration parameters */
);

static void report
(
    char *mapid,			/* Map identifier */
    struct MAP_PARMS *map_params	/* Map registration parameters */
);

static void report_man
(
    char *mapid,			/* Map identifier */
    struct MAP_PARMS *map_params	/* Map registration parameters */
);

static long register_map
(
    struct MAP_PARMS *map_params	/* Map registration parameters */
);

static void report_fit
(
    long npts,			/* Number of points in fit */
    double x_coord[25],		/* X map coordinates */
    double y_coord[25],		/* Y map coordinates */
    long dig_x[25],			/* X digitizer coordinates */
    long dig_y[25],			/* Y digitizer coordinates */
    long dx,			/* Digitizer coordinate--verify pt X */
    long dy,			/* Digitizer coordinate--verify pt Y */
    struct MAP_PARMS *map_params	/* Map registration parameters */
);

static void get_latlon
(
    char *val,			/* String describing coordinate to enter */
    double *y,
    double *x			/* Y & X (lat & long) value to return */
);

static double dms_field
(
    long deg,			/* Degrees portion of angle */
    long minute,			/* Minutes portion of angle */
    double sec			/* Seconds portion of angle */
);

static long get_tp
(
    double *yx		/* YX value (array dimensioned to two) */
);

static long local_check_dms
(
    long deg,		/* Degrees portion of angle */
    long minute,		/* Minutes portion of angle */
    double sec,		/* Seconds portion of angle */
    long format		/* Format--LATITUDE or LONGITUDE */
);

static long find_deg
(
    double angle			/* Angle in total degrees */
);

static long find_min
(
    double angle			/* Angle in total degrees */
);

static double find_sec
(
    double angle			/* Angle in total degrees */
);

static char *format_angle_as_DMS
(
    char *output_buffer,  /* O: buffer to print angle to */
    const char *format,   /* I: format to use for outputing the DMS angle.  A
                                typical format would be "%d %d %f" */
    double angle          /* I: angle (in degrees) to format as DMS */
);

static double coef_y[6];	/* Coefficients mapping line coordinates */
static double coef_x[6];	/* Coefficients mapping sample coordinates */
static long manual;		/* Boolean--is input manual (not digitizer)? */
static long units ;		/* Type of coordinate unit */
static long order;		/* FIRST or SECOND order polynomial flag */
static char taenam[CMLEN];	/* TAE name of mapid file */

int c_regmap
(
    char *mapfile,			/* Name of map parameter file */
    char *mapid,			/* Map identifier */
    char *coord_units		/* Coordinate units */
)
{
struct MAP_PARMS map_params;	/* Map registration parameters */
long mess_len = 79;		/* Length of message string */
long status;			/* Function return status flag */
TEXT mess[80];			/* Text buffer for t_ calls */
CODE term;			/* Terminator flag for t_read call */

/* If coordinate units have already been set, assign the proper numeric
   code to the unit variable
   ------------------------*/
if (strcmp(coord_units, "\0") != 0)
	{
	if (strcmp(coord_units, "radians") == 0) units = 0;
	else if (strcmp(coord_units, "feet") == 0) units = 1;
	else if (strcmp(coord_units, "meters") == 0) units = 2;
	else if (strcmp(coord_units, "seconds") == 0) units = 3;
	else if (strcmp(coord_units, "degrees") == 0) units = 4;
	else if (strcmp(coord_units, "dms") == 0) units = 5;
	else if (strcmp(coord_units, "user_def") == 0) units = 6;
	else units = NULL_UNITS;
	}
else units = NULL_UNITS;

/* Convert the host mapfile name to the TAE name for reporting purposes
  --------------------------------------------------------------------*/
strcpy(taenam, mapfile);

/* Loop until a map has been registered correctly or until user quits
  ------------------------------------------------------------------*/
for (;;)
   {
   for (;;)
      {
      t_write(" ", T_STDCC);
      t_write("Enter map identifier>  ", T_PROMPT);
      t_read(mess, &term);
      strcpy(mapid, mess);
      c_up2low(mapid, &mess_len);
      if (strcmp(mapid, "quit") == 0) return(E_DONE);
      status = get_map_rec(mapid, mapfile, &map_params);
      if (status == E_MAP_NOT_FOUND)
 	status = get_new_map(mapid, mapfile, &map_params);
      if (status == E_FAIL) return(E_FAIL);
      if (status == E_SUCC) break;
      }
   if (manual) break;
   if(register_map(&map_params) == E_SUCC) break;
   }

/* Reset/Set the coordinate units variable based on the "units" numeric
   value
  ------*/
if (units == 0) strcpy(coord_units, "radians");
if (units == 1) strcpy(coord_units, "feet");
if (units == 2) strcpy(coord_units, "meters");
if (units == 3) strcpy(coord_units, "seconds");
if (units == 4) strcpy(coord_units, "degrees");
if (units == 5) strcpy(coord_units, "dms");
if (units == 6) strcpy(coord_units, "user_def");
if (units == NULL_UNITS) strcpy(coord_units, NULL); 

return(E_SUCC);
}

/****************************************************************************
NAME:				get_map_rec

PURPOSE:  Gets a map parameter record from file

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	If a mapfile isn't used, return(not found)
	Read the map file--search for mapid key; Return(not found) if not found
	Report map parameters to the user
	Loop until satisfactory input has been given
	   Prompt--are these parameters OK?
	   If yes, break out of loop
	   if no, return (map not ok)
	Return success
*****************************************************************************/
static long get_map_rec
(
    char *mapid,			/* Map identifier */
    char *mapfile,			/* Map parameter file name */
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
TEXT mess[80];			/* TAE message text */
char dms_buffer[80];    /* buffer for formatting angles as DMS */
char *dms_format = "%d %d %f"; /* DMS format to use */
CODE term;			/* Terminator flag for t_read call */
long mess_len = 79;		/* Message length */

/* Check--is map file being used?  Return if not
  ---------------------------------------------*/
if(strcmp(mapfile, "\0") == 0) return(E_MAP_NOT_FOUND);

/* Read map parameter record.  Use mapid as key.  Return if not found
  ------------------------------------------------------------------*/
if(read_mapfile(mapfile, mapid, map_params) != E_SUCC) return(E_MAP_NOT_FOUND);

/* Report map parameters to the user
  ---------------------------------*/
t_clear();
t_write( "                              REGISTER MAP--Read", T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Map parameter file:  %s", taenam);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Map identifier:  %s", mapid);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Upper left latitude, longitude:  %s  %s",
   format_angle_as_DMS(dms_buffer,dms_format,map_params->ul_y),
   format_angle_as_DMS(dms_buffer,dms_format,map_params->ul_x));
t_write(mess, T_STDCC);
sprintf(mess, "Lower right latitude, longitude:  %s  %s",
   format_angle_as_DMS(dms_buffer,dms_format,map_params->lr_y),
   format_angle_as_DMS(dms_buffer,dms_format,map_params->lr_x));
t_write(mess, T_STDCC);
sprintf(mess, "Increment in X:  %f degrees  (%d points)",
   fabs(map_params->inc_x), map_params->npts_x);
t_write(mess, T_STDCC);
sprintf(mess, "Increment in Y:  %f degrees  (%d points)",
   fabs(map_params->inc_y), map_params->npts_y);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Verify point latitude, longitude:  %s  %s",
   format_angle_as_DMS(dms_buffer,dms_format,map_params->verify_y),
   format_angle_as_DMS(dms_buffer,dms_format,map_params->verify_x));
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Map type:   %s", map_params->maptype);
t_write(mess, T_STDCC);
sprintf(mess, "Map scale:  1:%d", map_params->scale);
t_write(mess, T_STDCC);
sprintf(mess, "Map datum:  %s", map_params->datum);
t_write(mess, T_STDCC);
sprintf(mess, "Map date:   %s", map_params->mapdate);
t_write(mess, T_STDCC);
sprintf(mess, "Map description:  %s", map_params->mapdesc);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);

/* Ask user if this is the correct parameter record
  -------------------------------------------------*/
for(;;)
   {
   t_write("Are map parameters satisfactory [Y]/N?  ", T_PROMPT);
   t_read(mess, &term);
   c_up2low(mess, &mess_len);
   if ((strcmp(&(mess[0]), "\0") == 0) || (strcmp(&(mess[0]),"y") == 0)) break;
   if (strcmp(&(mess[0]),"n") == 0) 
	{
	units = NULL_UNITS;
	t_clear();
	t_write( "                               REGISTER MAP--Add", T_STDCC);
	t_write(" ", T_STDCC);
	return(E_MAP_NOT_OK);
	}
   }
return(E_SUCC);
}

/****************************************************************************
NAME:				read_mapfile

PURPOSE:   Reads a labeled table file of map parameters 

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		5/89	D. Steinwand	   CSB      New development

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	If the file exists, open it.  Otherwise return fail
	Retrieve file label
	Set up pointers to file fields
	Initialize boolean values
	Loop until key (mapid) is found or until EOF
	   Get a record
	   If found, pack map parameter fields and break out of loop
	Close the file
	Return success
*****************************************************************************/
int read_mapfile
(
    char *mapfile,			/* Map file name */
    char *mapid,			/* Map identifier -- used as key */
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
struct TAB_DEF tab;		/* Tabular structure of label table */
struct VECTOR *mapidptr;	/* Pointer to mapid field */
struct VECTOR *unitptr;		/* Pointer to units field */
struct VECTOR *ul_yptr;		/* Pointer to ul corner y field */
struct VECTOR *ul_xptr;		/* Pointer to ul corner x field */
struct VECTOR *lr_yptr;		/* Pointer to lr corner y field */
struct VECTOR *lr_xptr;		/* Pointer to lr corner x field */
struct VECTOR *ver_yptr;	/* Pointer to verify pt y field */
struct VECTOR *ver_xptr;	/* Pointer to verify pt x field */
struct VECTOR *npts_yptr;	/* Pointer to npts in y field */
struct VECTOR *npts_xptr;	/* Pointer to npts in x field */
struct VECTOR *scaleptr;	/* Pointer to scale field */
struct VECTOR *datumptr;	/* Pointer to datum field */
struct VECTOR *typeptr;		/* Pointer to map type field */
struct VECTOR *dateptr;		/* Pointer to map date field */
struct VECTOR *descptr;		/* Pointer to map description field */
long no_key;			/* Boolean--null mapid ? */
long status;			/* Function return status */
long mode = 0;			/* Assess mode for lsstat */
long npts;			/* Number of points-- x * y */
long count = 0;			/* Count of records read */
static long rec_num = 1;	/* Count of records found */

/* Open the labeled table file & retrieve the label
  ------------------------------------------------*/
if (c_lsstat(mapfile, &mode) != E_SUCC) return(E_FAIL);
open_tab(&tab, mapfile, IREAD);
get_vector(&tab);

/* Determine pointers to fields in the file
  ----------------------------------------*/
mapidptr = get_field_ptr(&tab, "ID");
if (mapidptr == NULL) { close_tab(&tab); return(E_FAIL); }
unitptr = get_field_ptr(&tab, "UNITS");
if (unitptr == NULL) { close_tab(&tab); return(E_FAIL); }
ul_yptr = get_field_ptr(&tab, "UL_Y");
if (ul_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
ul_xptr = get_field_ptr(&tab, "UL_X");
if (ul_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
lr_yptr = get_field_ptr(&tab, "LR_Y");
if (lr_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
lr_xptr = get_field_ptr(&tab, "LR_X");
if (lr_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
ver_yptr = get_field_ptr(&tab, "VER_Y");
if (ver_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
ver_xptr = get_field_ptr(&tab, "VER_X");
if (ver_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
npts_yptr = get_field_ptr(&tab, "NPTS_Y");
if (npts_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
npts_xptr = get_field_ptr(&tab, "NPTS_X");
if (npts_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
scaleptr = get_field_ptr(&tab, "SCALE");
if (scaleptr == NULL) { close_tab(&tab); return(E_FAIL); }
datumptr = get_field_ptr(&tab, "DATUM");
if (datumptr == NULL) { close_tab(&tab); return(E_FAIL); }
typeptr = get_field_ptr(&tab, "TYPE");
if (typeptr == NULL) { close_tab(&tab); return(E_FAIL); }
dateptr = get_field_ptr(&tab, "DATE");
if (dateptr == NULL) { close_tab(&tab); return(E_FAIL); }
descptr = get_field_ptr(&tab, "DESC");
if (descptr == NULL) { close_tab(&tab); return(E_FAIL); }

/* Initialize BOOLEAN variables
  ----------------------------*/
if (strcmp(mapid, "\0") == 0) no_key = TRUE;
else 
   {
   no_key = FALSE;
   rec_num = 1;
   }

/* Loop until key is found or EOF
 -------------------------------*/
for (;;)
   {
   status = get_record(&tab);
   if (status == E_FAIL) { close_tab(&tab); return(E_FAIL); }
   if (status == E_EOF) { rec_num = 1; close_tab(&tab); return(E_FAIL); }
   count++;
   if((strcmp(mapidptr->ptr.c, mapid) == 0) || ((no_key) && (count == rec_num)))
	{
	rec_num++;
	strcpy(mapid, mapidptr->ptr.c);
	units = *unitptr->ptr.i4;
	map_params->ul_y = *ul_yptr->ptr.r8;
	map_params->ul_x = *ul_xptr->ptr.r8;
	map_params->lr_y = *lr_yptr->ptr.r8;
	map_params->lr_x = *lr_xptr->ptr.r8;
	map_params->verify_y = *ver_yptr->ptr.r8;
	map_params->verify_x = *ver_xptr->ptr.r8;
	map_params->npts_y = *npts_yptr->ptr.i4;
	map_params->npts_x = *npts_xptr->ptr.i4;
        map_params->inc_y = -fabs(map_params->ul_y - map_params->lr_y) /
		(map_params->npts_y - 1);
        map_params->inc_x = fabs(map_params->ul_x - map_params->lr_x) / 
		(map_params->npts_x - 1);
        npts = map_params->npts_x * map_params->npts_y;
        if (npts < 7) order = FIRST;
        else order = SECOND;
	map_params->scale = *scaleptr->ptr.i4;
	strcpy(map_params->datum, datumptr->ptr.c);
	strcpy(map_params->maptype, typeptr->ptr.c);
	if (strcmp(map_params->maptype, "manual") == 0) manual = TRUE;
	else manual = FALSE;
	strcpy(map_params->mapdate, dateptr->ptr.c);
	strcpy(map_params->mapdesc, descptr->ptr.c);
        break;
	}
   }

/* Close the file & return
  -----------------------*/
close_tab(&tab);
return(E_SUCC);
}

/****************************************************************************
NAME:				get_new_map

PURPOSE:  Gets map parameters from user

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Clear screen and print headers
	Prompt for map type
	If map type is "manual",
	   Prompt for units
	   Set coordinate fields to zero
	Else
	   Prompt for coordinate type (geographic or xy)
	   Prompt for upper left coordinates
	   Prompt for lower right coordinates
	   Prompt for x & y increments
	   Prompt for verify point
	Prompt for map scale
	Prompt for map datum
	Prompt for map date
	Prompt for map description
	Report parameters back to user for final editing & verification
	Loop until satisfactory input has been given
	   Prompt--are map parameters OK?
	   If yes, break out of loop
	   Edit appropriate field
	Write map parameters to map file
	Return success
*****************************************************************************/
static long get_new_map
(
    char *mapid,			/* Map identifier */
    char *mapfile,			/* Map parameter file */
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
TEXT mess[80];			/* Message buffer for t_ calls */
CODE term;			/* Terminator flag for t_read call */
long mess_len = 80;		/* Message length */
long number;			/* Number of field to edit */
long lower;			/* Lower bounds on fields to edit */

/* Clear srceen and print header 
  -----------------------------*/
t_clear();
t_write( "                               REGISTER MAP--Add", T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Map parameter file:  %s", taenam);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Map identifier:  %s", mapid);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);

/* Enter type of map
  -----------------*/
edit(6, map_params);
if (strcmp(map_params->maptype, "quit") == 0) return(E_MAP_NOT_OK);

/* If map type is "manual", set-up manually entered coordinates
  ------------------------------------------------------------*/
if (strcmp(map_params->maptype,"manual") == 0) 
   {
   manual = TRUE;
   t_write(" ", T_STDCC);
   if (units == NULL_UNITS)
	{
   	t_write("Valid unit codes:", T_STDCC);
   	t_write("  (0) Radians", T_STDCC);
   	t_write("  (1) Feet", T_STDCC);
   	t_write("  (2) Meters", T_STDCC);
   	t_write("  (3) Seconds", T_STDCC);
   	t_write("  (4) Degrees", T_STDCC);
   	t_write("  (5) DMS", T_STDCC);
   	t_write("  (6) User-defined", T_STDCC);
   	for (;;)
	   {
   	   t_write("Enter unit code>  ", T_PROMPT);
	   t_read(mess, &term);
	   units = atoi(mess);
	   if ((units >= 0) && (units <= 6)) break;
	   }
	}
   map_params->ul_y = map_params->ul_x = 0.0;
   map_params->lr_y = map_params->lr_x = 0.0;
   map_params->verify_y = map_params->verify_x = 0.0;
   map_params->npts_y = map_params->npts_x = 0.0;
   map_params->inc_y = map_params->inc_x = 0.0;
   t_write(" ", T_STDCC);
   }
else
   {
   manual = FALSE;
   for (;;)
      {

/* Prompt for type of coordinates--geographic or user defined XY
  -------------------------------------------------------------*/
      t_write(
	"Enter coordinate units:  [1] geographic, (2) user-defined X,Y>  ",
	T_PROMPT);
      t_read(mess, &term);
      if (strcmp(&(mess[0]), "\0") == 0) number = 1;
      else number = atoi(mess);
      if ((number == 1) || (number == 2)) break;
      }
   if (number == 1) units = 4;
   else units = 6;

/* Prompt for the upper left map corner coordinates
  ------------------------------------------------*/
   t_write(" ",T_STDCC);
   t_write(
  "Note:  North Latitudes and East Longitudes are entered as positive numbers",
 	 T_STDCC);
   t_write(
  "Note:  South Latitudes and West Longitudes are entered as negative numbers",
 	 T_STDCC);
   t_write(" ",T_STDCC);
   edit(1, map_params);

/* Prompt for the lower right map corner coordinates
  ------------------------------------------------*/
   edit(2, map_params);

/* Prompt for the number of points in the Y direction
  --------------------------------------------------*/
   edit(3, map_params);

/* Prompt for the number of points in the X direction
  --------------------------------------------------*/
   edit(4, map_params);

/* Prompt for the verify point coordinates
  ---------------------------------------*/
   edit(5, map_params);
   }

/* Prompt for the map scale
  ------------------------*/
edit(7, map_params);

/* Prompt for the map datum
  ------------------------*/
edit(8, map_params);

/* Prompt for the map date
  -----------------------*/
edit(9, map_params);

/* Prompt for the map description
  ------------------------------*/
edit(10, map_params);

/* Report parameters to the user
  -----------------------------*/
if (manual) report_man(mapid, map_params);
else report(mapid, map_params);

/* Allow the user to edit any field entered.  Keep in this loop until all
   parameters are satisfactory or until the user "quits"
  ----------------------------------------------------*/
for(;;)
   {
   t_write("Are map parameters satisfactory [Y]/N?  ", T_PROMPT);
   t_read(mess, &term);
   c_up2low(mess, &mess_len);
   if ((strcmp(&(mess[0]), "\0") == 0) || (strcmp(&(mess[0]),"y") == 0)) break;
   if (strcmp(&(mess[0]),"n") == 0)
	{
	if (manual) lower = 7;
	else lower = 1;
	for (;;)
	   {
	   if(manual)t_write("Enter line number (7 - 10) to edit >  ",T_PROMPT);
	   else t_write("Enter line number (1 - 10) to edit >  ", T_PROMPT);
	   t_read(mess, &term);
           c_up2low(mess, &mess_len);
           if (strcmp(mess, "quit") == 0) return(E_MAP_NOT_OK);
	   number = atoi(mess);
	   if ((number >= lower) && (number <= 10)) break;
	   }
	edit(number, map_params);
	if (manual) report_man(mapid, map_params);
	else report(mapid, map_params);
	}
   }

/* Write the resulting parameters to the map parameter file
  --------------------------------------------------------*/
t_write(" ", T_STDCC);
if(strcmp(mapfile, "\0") == 0) return(E_SUCC);
t_write("Writing to Map parameter file", T_STDCC);
wrlabtab(mapfile, mapid, map_params);

return(E_SUCC);
}

/*****************************************************************************
NAME:				 WRLABTAB 

PURPOSE:  Opens and writes the parameters to the output labeled table file.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		5/89	D. Steinwand	   CSB      New development
  5.02		8/89	D. Steinwand	   CSB      Fixed bug for SUN 4

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initialize the variables needed.
	Open output label table file.
	Write the parameter values to the output matrix.
	Write the matrix record to the output labeled table file.
	Close the labeled table file
	Return
*****************************************************************************/
void wrlabtab
(
    char *hosout,			/* Map file host name */
    char *mapid,			/* Map identifer */
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
struct MATRIX size;		/* matrix structure of label table */
struct TAB_DEF tab;		/* tabular structure of label table */
struct VECTOR *ptr;		/* pointer to each vector of label table */
char fdescbuf[80];		/* Data buffer */
char ftypebuf[80];		/* Data buffer */

/* Initialize the variables needed.
-----------------------------------*/
tab.nsubs = 0;
strcpy(tab.subfile[0],"");
strcpy(ftypebuf, "MAPFILE");
tab.ftype = ftypebuf;
strcpy(fdescbuf, "Map identifier parameter file");
tab.fdesc = fdescbuf;
tab.ncol = 6;

/*  Open output label table file.
---------------------------------*/
open_tab(&tab,hosout,IUPDATE);

put_vector(&tab,"ID",C,"Unique map identifier",1,20,FIELD_SEP2);
put_vector(&tab,"UNITS",I4,"Coordinate unit code 4=deg 6=xy",1,1,FIELD_SEP2);
put_vector(&tab,"UL_Y",R8,"Upper left Y/latitude coordinate",1,1,FIELD_SEP2);
put_vector(&tab,"UL_X",R8,"Upper left X/longitude coordinate",1,1,FIELD_SEP2);
put_vector(&tab,"LR_Y",R8,"Lower right Y/latitude coordinate",1,1,FIELD_SEP2);
put_vector(&tab,"LR_X",R8,"Lower right X/longitude coordinate",1,1,FIELD_SEP2);
put_vector(&tab,"VER_Y",R8,"Verify pt Y/latitude coordinate",1,1,FIELD_SEP2);
put_vector(&tab,"VER_X",R8,"Verify pt X/longitude coordinate",1,1,FIELD_SEP2);
put_vector(&tab,"NPTS_Y",I4,"Number of tics in Y",1,1,FIELD_SEP2);
put_vector(&tab,"NPTS_X",I4,"Number of tics in X",1,1,FIELD_SEP2);
put_vector(&tab,"SCALE",I4,"Map scale in 1:x",1,1,FIELD_SEP2);
put_vector(&tab,"DATUM",C,"Map datum",1,80,FIELD_SEP2);
put_vector(&tab,"TYPE",C,"Map type",1,80,FIELD_SEP2);
put_vector(&tab,"DATE",C,"Map date",1,10,FIELD_SEP2);
put_vector(&tab,"DESC",C,"Map description",1,80,RECORD_SEP);

/*  Write the parameter values to the output matrix.
----------------------------------------------------*/
size.nrow = 1;
size.ncol = 20;
ptr = tab.vector;
strcpy(ptr->ptr.c,mapid);

size.ncol = 1;
ptr = ptr->next;
put_matrix(ptr,I4,&size,(unsigned char *)&units);
ptr = ptr->next;
put_matrix(ptr,R8,&size,(unsigned char *)&map_params->ul_y);
ptr = ptr->next;
put_matrix(ptr,R8,&size,(unsigned char *)&map_params->ul_x);
ptr = ptr->next;
put_matrix(ptr,R8,&size,(unsigned char *)&map_params->lr_y);
ptr = ptr->next;
put_matrix(ptr,R8,&size,(unsigned char *)&map_params->lr_x);
ptr = ptr->next;
put_matrix(ptr,R8,&size,(unsigned char *)&map_params->verify_y);
ptr = ptr->next;
put_matrix(ptr,R8,&size,(unsigned char *)&map_params->verify_x);
ptr = ptr->next;
put_matrix(ptr,I4,&size,(unsigned char *)&map_params->npts_y);
ptr = ptr->next;
put_matrix(ptr,I4,&size,(unsigned char *)&map_params->npts_x);
ptr = ptr->next;
put_matrix(ptr,I4,&size,(unsigned char *)&map_params->scale);
size.ncol = 80;
ptr = ptr->next;
strcpy(ptr->ptr.c,map_params->datum);
size.ncol = 80;
ptr = ptr->next;
strcpy(ptr->ptr.c,map_params->maptype);
size.ncol = 10;
ptr = ptr->next;
strcpy(ptr->ptr.c,map_params->mapdate);
size.ncol = 80;
ptr = ptr->next;
strcpy(ptr->ptr.c,map_params->mapdesc);

/*  Write the matrix record to the output labeled table file.
-------------------------------------------------------------*/
put_record(&tab);

/*  Close the labeled table file
--------------------------------*/
close_tab(&tab);
}

/****************************************************************************
NAME:				edit

PURPOSE:  Edit/enters individual fields in the map parameter record

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Switch on number entered
	Enter parameter field
	Return
*****************************************************************************/
static void edit
(
    long number,			/* Number of field to edit/enter */
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
TEXT mess[80];			/* Message buffer for t_ calls */
CODE term;			/* Terminator flag for t_read call */
long mess_len = 79;		/* Length of message buffer */
long npts;			/* Number of points--X * Y */

switch (number)
{
/*  Get type of map used (quad-map, ortho-photo, manual, etc)
  ----------------------------------------------------------*/
case 6:
   t_write("Enter type of map >  ", T_PROMPT);
   t_read(mess, &term);
   c_up2low(mess, &mess_len);
   strcpy(map_params->maptype, mess);
   break;

/* Get the upper left corner coordinates
  -------------------------------------*/
case 1: get_latlon("upper left", &map_params->ul_y, &map_params->ul_x); break;

/* Get the lower right corner coordinates
  -------------------------------------*/
case 2: get_latlon("lower right", &map_params->lr_y, &map_params->lr_x); break;

/* Get the number of points in the X direction.  Calculate the distance 
   increment as well
  -----------------*/
case 3: for (;;)
   {
   t_write("Enter number of points in the X direction (2-5) >  ", T_PROMPT);
   t_read(mess, &term);
   if (strcmp(mess, "\0") == 0) map_params->npts_x = 2;
   else map_params->npts_x = atoi(mess);
   if ((map_params->npts_x >= 2) && (map_params->npts_x <= 5)) break;
   t_bell();
   }
   map_params->inc_x = fabs(map_params->ul_x - map_params->lr_x) / 
		(map_params->npts_x - 1);
   npts = map_params->npts_x * map_params->npts_y;
   if (npts < 7) order = FIRST;
   else order = SECOND;
   break;

/* Get the number of points in the Y direction.  Calculate the distance 
   increment as well
  -----------------*/
case 4: for (;;)
   {
   t_write("Enter number of points in the Y direction (2-5) >  ", T_PROMPT);
   t_read(mess, &term);
   if (strcmp(mess, "\0") == 0) map_params->npts_y = 2;
   else map_params->npts_y = atoi(mess);
   if ((map_params->npts_y >= 2) && (map_params->npts_y <= 5)) break;
   t_bell();
   }
   map_params->inc_y = -fabs(map_params->ul_y - map_params->lr_y) /
		(map_params->npts_y - 1);
   npts = map_params->npts_x * map_params->npts_y;
   if (npts < 7) order = FIRST;
   else order = SECOND;
   break;

/* Get the coordinates of the verify point
  ---------------------------------------*/
case 5: 
   get_latlon("verify point", &map_params->verify_y, &map_params->verify_x);
   break;

/* Get the map scale
  -----------------*/
case 7: t_write("Enter map scale >  1:", T_PROMPT);
   t_read(mess, &term);
   if (strcmp(mess, "\0") == 0) map_params->scale = 1;
   else map_params->scale = atoi(mess);
   break;

/* Get the map datum
  -----------------*/
case 8: t_write("Enter map datum >  ", T_PROMPT);
   t_read(mess, &term);
   strcpy(map_params->datum, mess);
   break;

/* Get the map date
  ----------------*/
case 9: t_write("Enter map date >  ", T_PROMPT);
   t_read(mess, &term);
   strcpy(map_params->mapdate, mess);
   break;

/* Get the user-entered map description/notes field
  ------------------------------------------------*/
case 10: t_write("Enter map description >  ", T_PROMPT);
   t_read(mess, &term);
   strcpy(map_params->mapdesc, mess);
   break;
}
}

/****************************************************************************
NAME:				report

PURPOSE:   Reports map parameters just entered

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Clear screen
	Print map parameters
	Return
*****************************************************************************/
static void report
(
    char *mapid,			/* Map identifier */
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
TEXT mess[80];			/* Message text buffer */
char dms_buffer[80];    /* buffer for formatting angles as DMS */
char *dms_format = "%d %d %f"; /* DMS format to use */

t_clear();
t_write( "                             REGISTER MAP--Verify", T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Map parameter file:  %s", taenam);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Map identifier:  %s", mapid);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
if (units == 6) sprintf(mess, "1.   Upper left coordinates (y,x):   %f %f",
   map_params->ul_y, map_params->ul_x);
else sprintf(mess, "1.   Upper left latitude, longitude:  %s  %s",
   format_angle_as_DMS(dms_buffer,dms_format,map_params->ul_y),
   format_angle_as_DMS(dms_buffer,dms_format,map_params->ul_x));
t_write(mess, T_STDCC);
if (units == 6) sprintf(mess, "2.   Lower right coordinates (y,x):  %f %f",
   map_params->lr_y, map_params->lr_x);
else sprintf(mess, "2.   Lower right latitude, longitude:  %s  %s",
   format_angle_as_DMS(dms_buffer,dms_format,map_params->lr_y),
   format_angle_as_DMS(dms_buffer,dms_format,map_params->lr_x));
t_write(mess, T_STDCC);
if (units == 6) sprintf(mess, "3.  Increment in X:  %f  (%d points)",
   fabs(map_params->inc_x), map_params->npts_x);
else sprintf(mess, "3.   Increment in X:  %f degrees  (%d points)",
   fabs(map_params->inc_x), map_params->npts_x);
t_write(mess, T_STDCC);
if (units == 6) sprintf(mess, "4.  Increment in Y:  %f  (%d points)",
   fabs(map_params->inc_y), map_params->npts_y);
else sprintf(mess, "4.   Increment in Y:  %f degrees  (%d points)",
   fabs(map_params->inc_y), map_params->npts_y);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
if (units == 6) sprintf(mess, "5.   Verify point coordinates (y,x):  %f %f",
   map_params->verify_y, map_params->verify_x);
else sprintf(mess, "5.   Verify point latitude, longitude:  %s  %s",
   format_angle_as_DMS(dms_buffer,dms_format,map_params->verify_y),
   format_angle_as_DMS(dms_buffer,dms_format,map_params->verify_x));
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "6.   Map type:   %s", map_params->maptype);
t_write(mess, T_STDCC);
sprintf(mess, "7.   Map scale:  1:%d", map_params->scale);
t_write(mess, T_STDCC);
sprintf(mess, "8.   Map datum:  %s", map_params->datum);
t_write(mess, T_STDCC);
sprintf(mess, "9.   Map date:   %s", map_params->mapdate);
t_write(mess, T_STDCC);
sprintf(mess, "10.  Map description:  %s", map_params->mapdesc);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
}

/****************************************************************************
NAME:				report_man

PURPOSE:   Reports map parameters when a "manual" map is used

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Clear screen
	Print map parameters
	Return
*****************************************************************************/
static void report_man
(
    char *mapid,			/* Map identifier */
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
TEXT mess[80];			/* Message buffer for t_ calls */

t_clear();
t_write( "                             REGISTER MAP--Verify", T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Map parameter file:  %s", taenam);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "Map identifier:  %s", mapid);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
t_write("1.   Upper left latitude, longitude:     *** Not used ***", T_STDCC);
t_write("2.   Lower right latitude, longitude:    *** Not used ***", T_STDCC);
t_write("3.   Increment in X:                     *** Not used ***", T_STDCC);
t_write("4.   Increment in Y:                     *** Not used ***", T_STDCC);
t_write(" ", T_STDCC);
t_write("5.   Verify point latitude, longitude:   *** Not used ***", T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, "6.   Map type:   %s", map_params->maptype);
t_write(mess, T_STDCC);
sprintf(mess, "7.   Map scale:  1:%d", map_params->scale);
t_write(mess, T_STDCC);
sprintf(mess, "8.   Map datum:  %s", map_params->datum);
t_write(mess, T_STDCC);
sprintf(mess, "9.   Map date:   %s", map_params->mapdate);
t_write(mess, T_STDCC);
sprintf(mess, "10.  Map description:  %s", map_params->mapdesc);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
}

/****************************************************************************
NAME:				register_map

PURPOSE:   Registers map to digitizer

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Get points from digitizer
	Perform a first or second order fit (depending upon x & y npts)
	Get the verify point
	Report fit statistics
	Loop until satisfactory
	   Prompt--is registration satisfactory?
	   Edit
	   Report fit
	Return
*****************************************************************************/
static long register_map
(
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
double data[25][MAX_COEF];	/* Raw data array for model routine */
double alpha[2];		/* Enter & exit test significance levels */
double fabs();			/* Double precision absolute value function */
long dx, dy;			/* Digitizer X and Y values--verify point */
long i,j,k;			/* Loop counters */
long nvar;			/* Number of independent variables */
long mess_len = 79;		/* Message buffer length */
long dig_x[25];			/* Digitizer x coordinates */
long dig_y[25];			/* Digitizer y coordinates */
long npts;			/* Number of points to digitize */
long button;			/* Button pressed on digitizer puck */
long pass = 1;			/* Informational message flag */
double x_coord[25];		/* X coordinate values to spark points at */
double y_coord[25];		/* Y coordinate values to spark points at */
TEXT mess[80];			/* Message buffer for t_ calls */
CODE term;			/* Terminator flag for t_read calls */
char dms_buffer[80];    /* buffer for formatting angles as DMS */
char *dms_format = "%d %d %f"; /* DMS format to use */

/* Initializer alpha to (near) 1.  This approximates a standard least
   squares fit
  -----------*/
alpha[0] = alpha[1] = 0.999;
   
/* Get points from digitizer
  -------------------------*/
t_write(" ",T_STDCC);
for (i = 0, k = 0; i < map_params->npts_y; i++)
   for (j = 0; j < map_params->npts_x; j++, k++)
	{
	x_coord[k] = map_params->ul_x + (map_params->inc_x * j);
	y_coord[k] = map_params->ul_y + (map_params->inc_y * i);
	if (units == 6) sprintf(mess, "Spark point at (%f,%f) >  ",
	   y_coord[k], x_coord[k]);
	else sprintf(mess, "Spark point at %s, %s >  ", 
       format_angle_as_DMS(dms_buffer,dms_format,y_coord[k]),
       format_angle_as_DMS(dms_buffer,dms_format,x_coord[k]));
	t_write(mess, T_PROMPT);
	t_read(mess, &term);
	sscanf(mess, "%d %d %d",&button, &(dig_x[k]), &(dig_y[k]));
	data[k][0] = (double) dig_x[k];
	data[k][1] = (double) dig_y[k];
        if (order == FIRST) data[k][2] = x_coord[k];
	else
	   {
	   data[k][2] = (double) dig_x[k] * (double) dig_x[k];
	   data[k][3] = (double) dig_x[k] * (double) dig_y[k];
	   data[k][4] = (double) dig_y[k] * (double) dig_y[k];
           data[k][5] = x_coord[k];
  	   }
	}

/* Perform a least squares fit on the digitizer coordinates and the
   map coordinates.  This gives us polynomial coefficients mapping from
   digitizer coordinates to map coordinates
  ----------------------------------------*/
npts = map_params->npts_x * map_params->npts_y;
if (order == FIRST) nvar = 2;
else nvar = 5;
if(c_steplr(&npts, &nvar, data, coef_x, alpha) != E_SUCC) 
   c_errmsg("Least squares fit failed in X direction"," ",&pass);

for (i = 0, k = 0; i < map_params->npts_y; i++)
   for (j = 0; j < map_params->npts_x; j++, k++)
      data[k][nvar] = y_coord[k];

if(c_steplr(&npts, &nvar, data, coef_y, alpha) != E_SUCC) 
   c_errmsg("Least squares fit failed in Y direction"," ",&pass);

/* Get verify point
  ----------------*/
if (units == 6) sprintf(mess, "Spark verify point at (%f,%f) >  ",
   map_params->verify_y, map_params->verify_x);
else sprintf(mess, "Spark verify point at %s, %s >  ", 
   format_angle_as_DMS(dms_buffer,dms_format,map_params->verify_y),
   format_angle_as_DMS(dms_buffer,dms_format,map_params->verify_x));
t_write(mess, T_PROMPT);
t_read(mess, &term);
sscanf(mess, "%d %d %d",&button, &dx, &dy);

/* Calculate & report fit statistics
  ---------------------------------*/
report_fit(npts, x_coord, y_coord, dig_x, dig_y, dx, dy, map_params);

/* Allow editing on any point & re-calculate/re-fit statistics
  -----------------------------------------------------------*/
for(;;)
   {
   t_write("Is map registration satisfactory [Y]/N?  ", T_PROMPT);
   t_read(mess, &term);
   c_up2low(mess, &mess_len);
   if ((strcmp(&(mess[0]), "\0") == 0) || (strcmp(&(mess[0]),"y") == 0)) break;
   if (strcmp(&(mess[0]),"n") == 0)
	{
	for (;;)
	   {
 	   t_write("Enter point number to re-spark>  ", 
		T_PROMPT);
	   t_read(mess, &term);
           c_up2low(mess, &mess_len);
           if (strcmp(mess, "quit") == 0) return(E_MAP_NOT_OK);
	   k = atoi(mess);
	   if ((k >= 1) && (k <= (npts + 1))) break;
	   }
	k--;
	t_write(" ", T_STDCC);
	if (k == npts)
	   {
	   if (units == 6)sprintf(mess,"Re-spark verify point at (%f,%f) >  ",
	   	map_params->verify_y, map_params->verify_x);
	   else sprintf(mess,"Re-spark verify point at %s, %s >  ",
           format_angle_as_DMS(dms_buffer,dms_format,map_params->verify_y),
           format_angle_as_DMS(dms_buffer,dms_format,map_params->verify_x));
	   t_write(mess, T_PROMPT);
	   t_read(mess, &term);
	   sscanf(mess, "%d %d %d",&button, &dx, &dy);
	   }
	else
	   {
	   if (units == 6)sprintf(mess, "Re-spark point at (%f,%f) >  ",
	      y_coord[k], x_coord[k]);
	   else sprintf(mess, "Re-spark point at %s, %s >  ", 
           format_angle_as_DMS(dms_buffer,dms_format,y_coord[k]),
           format_angle_as_DMS(dms_buffer,dms_format,x_coord[k]));
	   t_write(mess, T_PROMPT);
	   t_read(mess, &term);
	   sscanf(mess, "%d %d %d",&button, &(dig_x[k]), &(dig_y[k]));
	   data[k][0] = (double) dig_x[k];
	   data[k][1] = (double) dig_y[k];
           if (order == FIRST) data[k][2] = x_coord[k];
	   else
	      {
	      data[k][2] = (double) dig_x[k] * (double) dig_x[k];
	      data[k][3] = (double) dig_x[k] * (double) dig_y[k];
	      data[k][4] = (double) dig_y[k] * (double) dig_y[k];
              data[k][5] = x_coord[k];
  	      }
	   for (i = 0, k = 0; i < map_params->npts_y; i++)
	      for (j = 0; j < map_params->npts_x; j++, k++)
	         data[k][nvar] = x_coord[k];

	   if(c_steplr(&npts, &nvar, data, coef_x, alpha) != E_SUCC) 
		printf("STEPLR ERROR\n");

	   for (i = 0, k = 0; i < map_params->npts_y; i++)
	      for (j = 0; j < map_params->npts_x; j++, k++)
	         data[k][nvar] = y_coord[k];

	   if(c_steplr(&npts, &nvar, data, coef_y, alpha) != E_SUCC) 
		printf("STEPLR ERROR\n");
	   }

	report_fit(npts, x_coord, y_coord, dig_x, dig_y, dx, dy, map_params);
	}
   }
t_write(" ", T_STDCC);
return(E_SUCC);
}

/****************************************************************************
NAME:				report_fit

PURPOSE:   Calculates & reports residuals

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Clear the screen & print headers
	For each point
	   Calculate the map point according to the polymonial coeffs
	   Calculate residual between actual and calculated map point
	   Report Actual pt, calculated pt, and residual
	Calculate the verify point according to the polynomial coeffs
	Calculate the residual between actual and calculated verify pt
	Report Actual verify pt, calculated verify pt, and residual
	Return
*****************************************************************************/
static void report_fit
(
    long npts,			/* Number of points in fit */
    double x_coord[25],		/* X map coordinates */
    double y_coord[25],		/* Y map coordinates */
    long dig_x[25],			/* X digitizer coordinates */
    long dig_y[25],			/* Y digitizer coordinates */
    long dx,			/* Digitizer coordinate--verify pt X */
    long dy,			/* Digitizer coordinate--verify pt Y */
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
double x,y;			/* Calcualated map coordinates */
double res_lat;			/* Residual in Y/latitude direction */
double res_lon;			/* Residual in X/longitude direction */
long i;				/* Loop counter */
double temp_x,temp_y;		/* Temporary x & y digitizer coordinates */
TEXT mess[80];			/* Message buffer for t_ calls */
char dms_buffer[80];    /* buffer for formatting angles as DMS */
char *lat_dms_format = "%3d %2d %4.1f"; /* DMS format to use */
char *long_dms_format = "%4d %2d %4.1f"; /* DMS format to use */

/* Report statistics
  -----------------*/
t_clear();
t_write("                           MAP REGISTRATION STATISTICS",T_STDCC);
t_write(" ", T_STDCC);
t_write(" #       Map Location             Calculated Location       Residual (seconds)", T_STDCC);
t_write("--- ------------------------- --------------------------- ---------------------",T_STDCC);

/* For each point, calculate the map coordinate with the polynomial.  Then,
   calculate the residual between the actual map point and the calculated
   map point.  Write the actual map point, the calculated map point, and
   the residual to the screen
  --------------------------*/
for (i = 0; i < npts; i++)
   {
   temp_x = (double)dig_x[i];
   temp_y = (double)dig_y[i];
   x = c_eval(&order, coef_x, &temp_x, &temp_y);
   y = c_eval(&order, coef_y, &temp_x, &temp_y);
   if (units == 6)
      {
      res_lat = fabs(y) - fabs(y_coord[i]);
      res_lon = fabs(x) - fabs(x_coord[i]);
      }
   else
      {
      res_lat = (fabs(y) - fabs(y_coord[i])) * 3600.0;
      res_lon = (fabs(x) - fabs(x_coord[i])) * 3600.0;
      }
   if (units == 6)sprintf(mess,"%2d. %12.1f %12.1f  %12.1f %12.1f  %8.2f %8.2f",
	(i + 1), y_coord[i], x_coord[i], y, x, res_lat, res_lon);
   else sprintf(mess, "%2d. %s  %s   %s  %s  %8.2f %8.2f",
        (i + 1), 
        format_angle_as_DMS(dms_buffer,lat_dms_format,y_coord[i]), 
        format_angle_as_DMS(dms_buffer,long_dms_format,x_coord[i]),
        format_angle_as_DMS(dms_buffer,lat_dms_format,y), 
        format_angle_as_DMS(dms_buffer,long_dms_format,x),
        res_lat,res_lon);
   t_write(mess, T_STDCC);
   }

/* Calculate the verify point coordinate with the polynomial.  Then,
   calculate the residual between the actual verify point and the calculated
   verify point.  Write the actual verify point, the calculated verify point, 
   and the residual to the screen
  ------------------------------*/
temp_x = (double)dx;
temp_y = (double)dy;
x = c_eval(&order, coef_x, &temp_x, &temp_y);
y = c_eval(&order, coef_y, &temp_x, &temp_y);
if (units == 6)
      {
      res_lat = fabs(y) - fabs(map_params->verify_y);
      res_lon = fabs(x) - fabs(map_params->verify_x);
      }
else
      {
      res_lat = (fabs(y) - fabs(map_params->verify_y)) * 3600.0;
      res_lon = (fabs(x) - fabs(map_params->verify_x)) * 3600.0;
      }
if (units == 6)sprintf(mess,"%2d. %12.1f %12.1f  %12.1f %12.1f  %8.2f %8.2f",
   (i + 1), map_params->verify_y, map_params->verify_x, y, x, res_lat, res_lon);
else sprintf(mess, "%2d. %s  %s   %s  %s  %8.2f %8.2f",
    (i + 1), 
    format_angle_as_DMS(dms_buffer,lat_dms_format,map_params->verify_y), 
    format_angle_as_DMS(dms_buffer,long_dms_format,map_params->verify_x),
    format_angle_as_DMS(dms_buffer,lat_dms_format,y), 
    format_angle_as_DMS(dms_buffer,long_dms_format,x),
    res_lat,res_lon);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
}

/****************************************************************************
NAME:				get_latlon

PURPOSE:   Gets a latitude, longitude value in DDD MM SS.SS from the user

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
  	If xy (user defined) coordinates are entered, prompt for those
	Else
	   Prompt for lat, long in DDD MM SS.SS
	   Get input from user
	   Check the validity of the latitude value
	   Check the validity of the longitude value
	Return
*****************************************************************************/
static void get_latlon
(
    char *val,			/* String describing coordinate to enter */
    double *y,
    double *x			/* Y & X (lat & long) value to return */
)
{
TEXT mess[80];			/* Message text buffer for t_ calls */
CODE term;			/* Terminator for t_read call */
float xsec, ysec;		/* Number of seconds in x & y */
long xdeg, ydeg;		/* Number of degrees in x & y */
long xmin, ymin;		/* Number of minutes in x & y */
long lon_stat;			/* Longitude validity flag */
long lat_stat;			/* Latitude validity flag */

/* If user defined (y,x) coordinates are being entered, prompt for those
  ---------------------------------------------------------------------*/
if (units == 6)
   {
   sprintf(mess,"Enter %s corner coordinate (y,x) >  ", val);
   t_write(mess, T_PROMPT);
   t_read(mess, &term);
   *y = *x = 0.0;
   sscanf(mess, "%lf %lf", y, x);
   }

/* If latitude, longitude coordinates are being entered, prompt for those.
   Check the validity of the values entered and convert to total degrees
  ---------------------------------------------------------------------*/
else 
  {
  for (;;)
     {
     ydeg = xdeg = ymin = xmin = 0;
     ysec = xsec = 0.0;
     sprintf(mess,"Enter %s latitude, longitude in DDD MM SS.SS >  ", val);
     t_write(mess, T_PROMPT);
     t_read(mess, &term);
     sscanf(mess, "%d %d %f %d %d %f",&ydeg,&ymin,&ysec,&xdeg,&xmin,&xsec);
     lat_stat = local_check_dms(ydeg, ymin, ysec, LATITUDE);
     lon_stat = local_check_dms(xdeg, xmin, xsec, LONGITUDE);
     if ((lat_stat == E_SUCC) && (lon_stat == E_SUCC)) break;
     }
  *y = dms_field(ydeg, ymin, ysec);
  *x = dms_field(xdeg, xmin, xsec);
  }
}

/****************************************************************************
NAME:				dms_field

PURPOSE:   Returns total degrees from input deg, min, sec values

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
*****************************************************************************/
static double dms_field
(
    long deg,			/* Degrees portion of angle */
    long minute,			/* Minutes portion of angle */
    double sec			/* Seconds portion of angle */
)
{
double angle;			/* Angle in total degrees (upon completion) */
long sign;			/* Sign of angle */

if (deg < 0) 
   {
   sign = -1;
   deg = abs(deg);
   }
else sign = 1;
angle = deg + (minute/ 60.0) + (sec / 3600.0);
angle *= sign;
return(angle);
}

/****************************************************************************
NAME:				c_digpt

PURPOSE:    Gets a map coordinate from the digitizer

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	If coordinate input is manual, call get_tp & return
	Instruct user to spark a map point
	Read the digitizer coordinate
	Apply the bivariate polynomial to the digitizer coordinates to 
		calculate a map coordinate
	Report calculated map coordinates to user
	Return
*****************************************************************************/
int c_digpt
(
    double *yx,			/* y,x cooridinates (usually lat/long) */
    long *flag			/* Boolean flag--TRUE includes new map option */
)
{
long button;			/* Button pressed on digitizer puck */
long dx,dy;			/* Digitizer x & y coordinates */
double temp_x, temp_y;		/* Double values of long digitizer coords */
TEXT mess[80];			/* Message buffer for t_ calls */
CODE term;			/* Terminator for t_read calls */
char dms_buffer[80];    /* buffer for formatting angles as DMS */
char *dms_format = "%d %d %6.3f"; /* DMS format to use */

/* If coordinate input is manual, call get_tp & return
  ---------------------------------------------------*/
if (manual) return(get_tp(yx));

/* Instruct user to spark a map point
  ----------------------------------*/
t_write(" ", T_STDCC);
if (*flag) sprintf(mess, 
"Spark map point; push (%d) to select, (%d) new map, (%d) to quit >  ",
	SELECT_PT,NEW_MAP,QUIT_DIG);
else sprintf(mess, "Spark map point; push (%d) to select, (%d) to abort >  ",
	SELECT_PT,QUIT_DIG);
t_write(mess, T_PROMPT);

/* Read the digitizer coordinate
  -----------------------------*/
t_read(mess, &term);
sscanf(mess, "%d %d %d",&button, &dx, &dy);
if (button != SELECT_PT) return(button);

/* Apply the bivariate polynomial to the digitizer coordinates to calculate a 
   map coordinate
  --------------*/
temp_x = (double)dx;
temp_y = (double)dy;
yx[1] = c_eval(&order, coef_x, &temp_x, &temp_y);
yx[0] = c_eval(&order, coef_y, &temp_x, &temp_y);

/* Report calculated map coordinates to user
  -----------------------------------------*/
if (units == 6) sprintf(mess,"(y,x):  %f, %f", yx[0], yx[1]);
else sprintf(mess,"latitude, longitude:  %s  %x",
    format_angle_as_DMS(dms_buffer,dms_format,yx[0]),
    format_angle_as_DMS(dms_buffer,dms_format,yx[1]));
t_write(mess,T_STDCC);
return(button);
}

/****************************************************************************
NAME:				get_tp

PURPOSE:  Gets a tie point coordinate from the user

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development
	 5/91   L. Huewe      CSB      Modified to differentiate between
				       positive and negative zero.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Do until statisfactory input has been given:
	   Print prompt
	   Get the input back from the user
	   If done, return EOF
	   For the appropriate units:
		Unpack the user input into Point_id, Y coord, X coord
		Check the valid range of Y coord & X coord
		If valid, break out of loop
		Prompt user to re-enter invalid input
	Return SUCCESS
*****************************************************************************/
static long get_tp
(
    double *yx		/* YX value (array dimensioned to two) */
)
{
TEXT mess[80];		/* Buffer for user messages and input */
char sdeg1[5],smin1[3],ssec1[80],sdeg2[5],smin2[3],ssec2[80];
			/* String equivalents for degrees, minutes and seconds
			   used to detect a -0 entry */
CODE term;		/* Terminal flag for t_io */
long x_ok, y_ok;	/* Boolean flags */
long deg1, min1;	/* Degrees & minutes part of first DMS coordinate */
long deg2, min2;	/* Degrees & minutes part of second DMS coordinate */
long status;		/* sscanf return status flag */
long mess_len = 79;	/* Length of message buffer for lower case routine */
float sec1, sec2;	/* Seconds parts of DMS coordatates */

/* Do until satisfactory user input is given
  -----------------------------------------*/
for (;;)
   {

/* Prompt user
  -----------*/
   if (units == 0) sprintf(mess,"Enter latitude and longitude in radians>  ");
   if (units == 1) sprintf(mess,"Enter (y,x) coordinates in feet>  ");
   if (units == 2) sprintf(mess,"Enter (y,x) coordinates in meters>  ");
   if (units == 3) sprintf(mess,"Enter latitude and longitude in seconds>  ");
   if (units == 4) sprintf(mess,"Enter latitude and longitude in degrees>  ");
   if (units == 5) sprintf(mess,"Enter latitude and longitude in DMS>  ");
   if (units == 6) sprintf(mess,"Enter (y,x) coordinates>  ");
   t_write(mess,T_PROMPT);

/* Read user input
  ---------------*/
   t_read(mess, &term);
   
/* Check for end of entry
  ----------------------*/
   if (strlen(mess) == 0) return(QUIT_DIG);
   c_up2low(mess, &mess_len);
   if (strcmp(mess, "quit") == 0) return(QUIT_DIG);

/* For the given units, unpack the fields and check validity.  Break out of
   loop if satisfactory input is given
  -----------------------------------*/
   x_ok = y_ok = FALSE;
   if (units == 0)  /* Radians */
	{
  	status = sscanf(mess, "%lf %lf", &(yx[0]), &(yx[1])); 
        if ((yx[0] >= NEG_HALF_PI) && (yx[0] <= HALF_PI)) y_ok = TRUE;
        if ((yx[1] >= NEG_PI) && (yx[1] <= PI)) x_ok = TRUE;
	if (status != 2) x_ok = y_ok = FALSE;
        }
   if ((units == 1) || (units == 2) || (units == 6))  /* Feet, meters, user */
	{
  	status = sscanf(mess, "%lf %lf", &(yx[0]), &(yx[1])); 
 	x_ok = y_ok = TRUE;
	if (status != 2) x_ok = y_ok = FALSE;
        }
   if (units == 3)  /* Seconds */
	{
  	status = sscanf(mess, "%lf %lf", &(yx[0]), &(yx[1])); 
        if ((yx[0] >= -324000.0) && (yx[0] <= 324000.0)) y_ok = TRUE;
        if ((yx[1] >= -648000.0) && (yx[1] <= 648000.0)) x_ok = TRUE;
	if (status != 2) x_ok = y_ok = FALSE;
        }
   if (units == 4)  /* Degrees */
	{
  	status = sscanf(mess, "%lf %lf", &(yx[0]), &(yx[1])); 
        if ((yx[0] >= -90.0) && (yx[0] <= 90.0)) y_ok = TRUE;
        if((yx[1] >= -180.0) && (yx[1] <= 180.0)) x_ok = TRUE;
	if (status != 2) x_ok = y_ok = FALSE;
        }
   if (units == 5)  /* Packed DMS */
	{
	deg1 = deg2 = min1 = min2 = 0;
	sec1 = sec2 = 0.0;
  	status = sscanf(mess, 
		"%d %d %f %d %d %f",&deg1,&min1,&sec1,&deg2,&min2,&sec2);
 	x_ok = y_ok = TRUE;
	if ((deg1 > 90) || (deg1 < -90)) y_ok = FALSE;
	if ((deg2 > 180) || (deg2 < -180)) x_ok = FALSE;
 	if ((min1 > 60) || (min2 > 60)) y_ok = FALSE;
	if ((sec1 > 60.0) || (sec2 > 60.0)) x_ok = FALSE;

	if (deg1 < 0) 
 	   yx[0] = ((double)(deg1 * -1000000) + (min1 * 1000) + sec1) * -1.0;
	else if (deg1 == 0)
	    {
	    sscanf(mess,"%s %s %s %s %s %s",sdeg1,smin1,ssec1,sdeg2,smin2,
		ssec2);
	    if (strcmp(sdeg1,"-0") == 0)
 	        yx[0] = ((double)(min1 * 1000) + sec1) * -1.0;
	    else
 	        yx[0] = (double)(min1 * 1000) + sec1;
 	    }
	else
	   yx[0] = (double)(deg1 * 1000000) + (min1 * 1000) + sec1;

	if (deg2 < 0)
	   yx[1] = ((double)(deg2 * -1000000) + (min2 * 1000) + sec2) * -1.0;
	else if (deg2 == 0)
	    {
	    sscanf(mess,"%s %s %s %s %s %s",sdeg1,smin1,ssec1,sdeg2,smin2,
		ssec2);
	    if (strcmp(sdeg2,"-0") == 0)
	        yx[1] = ((double)(min2 * 1000) + sec2) * -1.0;
	    else
	        yx[1] = (double)(min2 * 1000) + sec2;
	    }
 	else
	   yx[1] = (double)(deg2 * 1000000) + (min2 * 1000) + sec2;

	if (yx[0] > 90000000.00) y_ok = FALSE;
	if (yx[0] < -90000000.00) y_ok = FALSE;
	if (yx[1] > 180000000.00) x_ok = FALSE;
	if (yx[1] < -180000000.00) x_ok = FALSE;
	if (status < 4) x_ok = y_ok = FALSE;
        }
   if (x_ok && y_ok) break;
   if (!x_ok) t_write(" Illegal X/Longitude value entered; re-enter!", T_STDCC);
   if (!y_ok) t_write(" Illegal Y/Latitude value entered; re-enter!", T_STDCC);
   }
return(SELECT_PT);
}

/****************************************************************************
NAME:				local_check_dms

PURPOSE:  checks validity of a lat,long pair in DMS

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set flag to PASS
	Check range of degrees portion; set to FAIL if not ok
	Check range of minutes portion; set to FAIL if not ok
	Check range of seconds portion; set to FAIL if not ok
	Convert to total degrees
	Check range of total degrees; set to FAIL if not ok
	If FAIL, print error message & return FAIL
	Return PASS
*****************************************************************************/
static long local_check_dms
(
    long deg,		/* Degrees portion of angle */
    long minute,		/* Minutes portion of angle */
    double sec,		/* Seconds portion of angle */
    long format		/* Format--LATITUDE or LONGITUDE */
)
{
double temp;		/* Total degrees (used temporarily) */
long pass = 1;		/* Informational message */
long ok;		/* Boolean flags */

ok = TRUE;
if (format == LATITUDE)
   if ((deg > 90) || (deg < -90)) ok = FALSE;
if (format == LONGITUDE)
   if ((deg > 180) || (deg < -180)) ok = FALSE;

if ((minute > 60) || (minute > 60)) ok = FALSE;
if ((sec > 60.0) || (sec > 60.0)) ok = FALSE;

if (deg < 0.0) 
   temp = ((deg * -1000000) + (minute * 1000) + sec) * -1.0;
else
   temp = (deg * 1000000) + (minute * 1000) + sec;

if (format == LATITUDE)
   {
   if (temp > 90000000.00) ok = FALSE;
   if (temp < -90000000.00) ok = FALSE;
   }
if (format == LONGITUDE)
   {
   if (temp > 180000000.00) ok = FALSE;
   if (temp< -180000000.00) ok = FALSE;
   }
if (ok) return(E_SUCC);
if (format == LATITUDE) 
      c_errmsg("Illegal latitude value entered; re-enter!", " ", &pass);
if (format == LONGITUDE) 
      c_errmsg("Illegal longitude value entered; re-enter!", " ", &pass);
return(E_FAIL);
}

/****************************************************************************
NAME:			find_deg, find_min, find_sec	

PURPOSE:  Extracts deg, min, or sec portions of an angle

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 5/89	D. Steinwand  CSB      LAS 5.0 (original) Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Extract portion of angle
	Return
*****************************************************************************/
static long find_deg
(
    double angle			/* Angle in total degrees */
)
{
long sign;			/* Sign of angle */
long deg;			/* Degrees portion of angle */
long minute;			/* Minutes portion of angle */
double sec;			/* Seconds portion of angle */
deg = (long)angle;
sign = 1;
if (deg < 0) sign = -1;
deg = (long)fabs(angle);
minute =  (long)((fabs(angle) - deg) * 60.0);
sec =  (((fabs(angle) - deg) * 60.0) - minute) * 60.0;
if (sec >= 59.999) minute++;
if (minute >= 60) deg++;
deg *= sign;
return(deg);
}

static long find_min
(
    double angle			/* Angle in total degrees */
)
{
double sec;			/* Seconds portion of angle */
long minute;			/* Minutes portion of angle */
angle = fabs(angle);
angle -= (long)angle;
minute = (long)(angle * 60.0);
sec = ((angle * 60.0) - minute) * 60.0;
if (sec > 59.999) minute++;
return(minute);
}

static double find_sec
(
    double angle			/* Angle in total degrees */
)
{
angle = fabs(angle);
angle -= (long)angle;
angle *= 60.0;
angle -= (long)angle;
angle *= 60.0;
if (angle > 59.999) angle -= 60.0;
return(angle);
}

static char *format_angle_as_DMS
(
    char *output_buffer,  /* O: buffer to print angle to */
    const char *format,   /* I: format to use for outputing the DMS angle.  A
                                typical format would be "%d %d %f" */
    double angle          /* I: angle (in degrees) to format as DMS */
)
{
    char full_format[80];

    /* setup the format with the proper sign */
    strcpy(full_format,"");
    if (angle < 0.0)
    {
        strcat(full_format,"-");
    }
    strcat(full_format,format);
    
    /* format the angle as DMS */
    sprintf(output_buffer,full_format,
            find_deg(angle),find_min(angle),find_sec(angle));

    /* return the address of the buffer so this can be used in other 
       print statements */
    return (output_buffer);
}
