/*****************************************************************************
NAME:           C_ATTCHK

PURPOSE: Parses input attribute text for a valid NAME and VALUE.   
         The location of the NAME in the GOF header record        
         determines where the VALUE will be placed in the output   
         attribute array when the search flag is set to false        
         (ie. VALUE is placed in the same location as the NAME is in 
         the GOF header record).  When the search flag is set to     
         true, attribute values will be placed in sequential order   
         in the output attribute array and the location of each of   
         those values in a separate index array.                     
                                                                             
*****************************************************************************/
#include <ctype.h>
#include "las.h"
#include "util.h"
#include "gof.h"

int c_attchk
(
    struct HEADER *hdr,         /* Header information from the GOF. (I)       */
    char inatt[][ATTIN],        /* Input attribute string. NAME:VALUE
                                   NOTE: Any of the MAXATT input attributes
                                   that do not have a specified attribute
                                   must be assigned a null string. (I)        */
    char outatt[][ATTVAL],      /* Output attribute VALUES in the order
                                   defined by the header record from the 
                                   GOF. (O)                                   */
    int index[MAXATT],          /* Indices of attributes for searching. (O)   */
    int *search                 /* Flag indicating checking by a search func.
                                   (I)                                        */
)
{
    static  char    colon[] = {":"}, /* Colon to search for. */
    mess[60];                        /* Error message string. */

    int     i, j, k;                 /* Indexing variables. */
    int     loc;                     /* Location of the colon in the string. */
    int     end;                     /* Length of the value portion of the
                                        string. */
    int     pos = 0;                 /* Position in index and outatt arrays. */
    int     temp;                    /* Temporary conversion variable. */
    int     status;                  /* Status variable. */
    long    nonfatal = 1;            /* Nonfatal error flag. */
    long    len;                     /* Length of the input attribute string. */

    char    name[ATTNAM];            /* Name portion of the attribute string. */
    char    value[ATTVAL];           /* Value portion of the attribute string.*/
    char    type[7];                 /* Data type of the value. */

    int     fail;                    /* Failure flag. */

    /* Initalize the output attribute array. */
    for (i = 0; i < MAXATT; i++)
        strcpy(outatt[i],"");

    /* Parse the input attribute array into the NAME and VALUE portions.
       Check to make sure that the name is a valid NAME in the GOF header
       record and that the value is valid for the NAMEs data type. */
    for (i = 0; i < MAXATT; i++)
    {
        len = (long)strlen(inatt[i]);
        c_low2up (inatt[i],&len);
        if (strcmp(inatt[i],"") != 0)
        {
            len = (long)strlen(inatt[i]);
            loc = strcspn(inatt[i],colon);
            end = len - (loc + 1);
            /* Check for valid lengths for the NAME and VALUE strings. */
            if ((loc > ATTNAM) || (end > ATTVAL) || (loc == len))
            {
                sprintf(mess,"Invalid attribute name or value in %s",
                        &inatt[i][0]);
                c_errmsg(mess, "attchk-invatt", &nonfatal);
                return(E_FAIL);
            }
            /* Separate the NAME and the VALUE from the input attribute. */
            strncpy(name,inatt[i],loc);
            name[loc] = '\0';
            if (end != 0)
            {
                strncpy(value,&inatt[i][loc+1],end);
                value[end] = '\0';
            }
            else
                if (*search)
                    strcpy(value,"");
                else
                {
                    sprintf(mess,"Invalid attribute name or value in %s",
                            &inatt[i][0]);
                    c_errmsg(mess, "attchk-invatt", &nonfatal);
                    return(E_FAIL);
                }
            /* Check to make sure that the value for the NAME is valid for
               the data type defined in the GOF header. */
            fail = TRUE;
            for (j = 0; j < MAXATT; j++)
            {
                if ((hdr->mask[j] != 0) && (hdr->mask[j] == loc) &&
                        (strcmp(hdr->attrb[j],name) == 0))
                {
                    /* If the data type is not a character make sure that
                       all the characters are digits else its an error. */
                    if (strcmp(hdr->attyp[j],"CHA") != 0)
                    {
                        for (k = 0; k < end; k++)
                        {
                            status = isdigit(value[k]);
                            if ((status == 0) && (value[k]!= '-')
                                    && (value[k] != '+') && (value[k] != '.'))
                            {
                                if (strcmp(hdr->attyp[j],"INT") == 0)
                                    strcpy(type,"INTEGER");
                                else if (strcmp(hdr->attyp[j],"REA")==0)
                                    strcpy(type,"REAL");
                                else if (strcmp(hdr->attyp[j],"DOU")==0)
                                    strcpy(type,"DOUBLE");
                                else if (strcmp(hdr->attyp[j],"BYT")==0)
                                    strcpy(type,"BYTE");
                                sprintf(mess,
                           "Attribute value for %s must have a data type of %s",
                                        inatt[i],type);
                                c_errmsg(mess, "attchk-invtype", &nonfatal);
                                return(E_FAIL);
                            }
                            if (strcmp(hdr->attyp[j],"BYT") == 0)
                            {
                                sscanf (value, "%d", &temp);
                                if ((temp < 0) || (temp > 255))
                                {
                                    strcpy(type,"BYTE");
                                    sprintf(mess,
                           "Attribute value for %s must have a data type of %s",
                                            inatt[i],type);
                                    c_errmsg(mess, "attchk-invtype", &nonfatal);
                                    return(E_FAIL);
                                }
                            }
                        }
                    }
                    /* Checking was successful. If searching and no value was
                       specified, return a generic value. Else return the value
                       in the output attribute array in the location defined by
                       the attribute name located in the GOF header. */
                    fail = FALSE;
                    if ((end == 0) && (*search == TRUE))
                    {
                        strcpy(outatt[pos],ATHAT);
                        index[pos] = j;
                        pos++;
                    }
                    else
                        if (*search == TRUE)
                            /* Return the value in the output attribute array
                               in sequential order and set the index array. */
                        {
                            strcpy(outatt[pos],value);
                            index[pos] = j;
                            pos++;
                        }
                        else
                            /* Return the value in the output attribute array
                               in the location defined by the attribute name
                               located in the GOF header. */
                            strcpy(outatt[j],value);
                }
            }
            /* Display an error message if the input attribute string is not
               valid. */
            if (fail)
            {
                sprintf(mess,"Invalid attribute name or value in %s",
                        &inatt[i][0]);
                c_errmsg(mess, "attchk-invatt", &nonfatal);
                return(E_FAIL);
            }
            if (*search == TRUE)
                index[pos] = -1;
        }
    }
    return(E_SUCC);
}
