/******************************************************************************
NAME:           c_gofrd                                                                       *
PURPOSE:  Read the specified type of record from the GOF and put        
          the information into the defined structure.

PROGRAM HISTORY:
  Version       Date    Contrib         Change
  -------       ----    -------         ------
   1.0          3/86    L. Huewe        Intial code
   2.0          5/92    L. Huewe        Modified to be a more generic
                                        WORLD support routine.
   2.1          1/95    D. Eichacker    To read a full polygon or line 
                                        two read are required the 1st 
                                        call to this function should be 
                                        with HPOLIN the 2nd with XYPAIRS.
   3.0          9/03    A. Shimitz      Added support for portable GOFs

PROJECT: LAS

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "tae.h"
#include "worgen.h"
#include "ddr.h"
#include "gof.h"

int c_gofrd 
(
    int *fd,                /* File descriptor of the GOF to read (I). */
    int *obyte,             /* Number of bytes read (O).*/
    int *nbyte,             /* Number of bytes to next record (O).*/
    int *type,              /* Type of record being read (I).*/
    struct DDR *ddr,        /* DDR structure containing proj. info only (O). */
    union GOF *gofptr       /* Pointer to the proper structure (O). */
)
{
    struct  RWHDR   arec;         /* Attribute record structure */

    struct  PPLGOF  buf;          /* Point, polygon or line GOF record 
                                     structure */
    struct  ANOT1   a1buf;        /* Character portion of annotation record
                                     structure */
    struct  ANOT2   a2buf;        /* Float portion of annotation record
                                     structure */
    struct PORT_PROJ projbuf;     /* Struct of projection info */

    float   xypair[2];            /* XY pair for polygon or line. */
    double  projxypair[2];        /* proj X/Y pair for polygon or line. */

    int     clen,                 /* Character length */
            dlen,                 /* Data length */
            i,                    /* Loop index */
            numbyt;               /* Number of bytes read */
    long one = 1;                 /* Indicates nonfatal error */

    /* Read the attribute definition record. */ 
    if (*type == HDR)
    {
        *obyte = read (*fd, &arec, HDRLEN);
        if (*obyte < 0)
        {
            c_errmsg ("Error reading GOF","gofrd-read",&one);
            return (E_FAIL);
        }
        else if (*obyte == 0)
            return (E_EOF);
        /* Determine the number of bytes to the next record. */
        *nbyte = 0; 
        /* Copy the attribute mask into the structure. */
        for (i=0; i < MAXATT; i++)
            gofptr->hdrec.mask[i] = arec.mask[i];
        /* Copy the attribute names and types into the structure. */
        for (i=0; i < MAXATT; i++)
        {
            strcpy (gofptr->hdrec.attrb[i],arec.attrb[i]);
            strcpy (gofptr->hdrec.attyp[i],arec.attyp[i]);
        }
        /* Copy the number of records in the file into the structure. */
        sscanf (arec.rnum,"%d",&gofptr->hdrec.nrec);
        return (E_SUCC);
    }
    /* Read the appropriate number of bytes based on the type 
       of record being read. */ 
    switch (*type)
    {
        case HPOLIN :               /* Polygon or line header record */
        case PNT :                  /* Point record */
            *obyte = read (*fd, &buf, LENTYP + CHARLEN);
            if (*obyte < 0)
            {
                c_errmsg ("Error reading GOF","gofrd-read",&one);
                return (E_FAIL);
            }
            else if (*obyte == 0)
                return (E_EOF);
            numbyt = read (*fd, &(buf.x), XYLEN);
            *obyte += numbyt;
            break;
        case PORT_HPOLIN :          /* Portable polygon or line header record */
        case PORT_PNT :             /* Portable point record */
            *obyte = read (*fd, &buf, LENTYP + CHARLEN);
            if (*obyte < 0)
            {
                c_errmsg ("Error reading GOF","gofrd-read",&one);
                return (E_FAIL);
            }
            else if (*obyte == 0)
                return (E_EOF);
            numbyt = read (*fd, &(buf.x_proj), PROJXYLEN);
            *obyte += numbyt;
            break;
        case ANOT:                  /* Annotation record */
            *obyte = read (*fd,&a1buf,LENTYP);
            if (*obyte < 0)
            {
                c_errmsg ("Error reading GOF","gofrd-read",&one);
                return (E_FAIL);
            }
            else if (*obyte == 0)
                return (E_EOF);
            sscanf (a1buf.length,"%d/%d",&clen,&dlen);
            numbyt = read (*fd,a1buf.attrb,clen);
            *obyte = numbyt + LENTYP;
            numbyt = read (*fd,&a2buf,dlen);
            *obyte += numbyt;
            break;
        case PORT_ANOT:             /* Portable annotation record */
            *obyte = read (*fd,&a1buf,LENTYP);
            if (*obyte < 0)
            {
                c_errmsg ("Error reading GOF","gofrd-read",&one);
                return (E_FAIL);
            }
            else if (*obyte == 0)
                return (E_EOF);
            sscanf (a1buf.length,"%d/%d",&clen,&dlen);
            numbyt = read (*fd,a1buf.attrb,clen);
            *obyte = numbyt + LENTYP;
            numbyt = read (*fd,&a2buf,dlen);
            *obyte += numbyt;
            break;
        case XYPAIRS:               /* XY pairs of a line or polygon */
            buf.xpoints = (float *)malloc(gofptr->plrec.numpts * sizeof(
                        float));
            if (buf.xpoints == NULL)
            {
                c_errmsg ("Error allocating dynamic memory","gofrd-read", &one);
                return (E_FAIL);
            }
            buf.ypoints = (float *)malloc(gofptr->plrec.numpts * sizeof(
                        float));
            if (buf.ypoints == NULL)
            {
                c_errmsg ("Error allocating dynamic memory","gofrd-read", &one);
                return (E_FAIL);
            }
            for (i=0; i < gofptr->plrec.numpts; i++)
            {
                *obyte = read(*fd,xypair,XYLEN);
                if (*obyte < 0)
                    break;
                buf.xpoints[i] = xypair[0];
                buf.ypoints[i] = xypair[1];
            }
            break;
        case PROJXYPAIRS:             /* proj X/Y pairs of a line or polygon */
            buf.x_proj_points = (double *)malloc(gofptr->plrec.numpts * sizeof(
                        double));
            if (buf.x_proj_points == NULL)
            {
                c_errmsg ("Error allocating dynamic memory","gofrd-read",
                        &one);
                return (E_FAIL);
            }
            buf.y_proj_points = (double *)malloc(gofptr->plrec.numpts * sizeof(
                        double));
            if (buf.y_proj_points == NULL)
            {
                c_errmsg ("Error allocating dynamic memory","gofrd-read",
                        &one);
                return (E_FAIL);
            }
            for (i=0; i < gofptr->plrec.numpts; i++)
            {
                *obyte = read(*fd,projxypair,PROJXYLEN);
                if (*obyte < 0)
                    break;
                buf.x_proj_points[i] = projxypair[0];
                buf.y_proj_points[i] = projxypair[1];
            }
            break;
    }   /* End Switch */

    if (*obyte < 0)
    {
        c_errmsg ("Error reading GOF","gofrd-read",&one);
        return (E_FAIL);
    }
    else if (*obyte == 0) return (E_EOF);

    /* Read the projection information, if applicable (portable GOFs only) */
    if (*type == PORT_PNT || *type == PROJXYPAIRS || *type == PORT_ANOT)
    {
        /* Read the data from the file */
        numbyt = read (*fd,&projbuf,LENTYP);
        if (numbyt < 0)
        {
            c_errmsg ("Error reading portable GOF","gofrd-read",&one);
            return (E_FAIL);
        }
        else if (numbyt == 0)
            return (E_EOF);
        *obyte += numbyt;
        sscanf (projbuf.length,"%d/%d",&clen,&dlen);
        numbyt = read (*fd,projbuf.proj_units,clen);
        *obyte += numbyt;

        /* Read the non-character portion of the record */
        numbyt = read (*fd,&(projbuf.proj_code),PROJDUBLEN);
        if (numbyt < 0)
        {
            c_errmsg ("Error reading portable GOF","gofrd-read",&one);
            return (E_FAIL);
        }
        else if (numbyt == 0) return (E_EOF);
        *obyte += numbyt;

        numbyt = read (*fd,&(projbuf.zone_code),PROJDUBLEN);
        if (numbyt < 0)
        {
            c_errmsg ("Error reading portable GOF","gofrd-read",&one);
            return (E_FAIL);
        }
        else if (numbyt == 0) return (E_EOF);
        *obyte += numbyt;

        numbyt = read (*fd,&(projbuf.datum_code),PROJDUBLEN);
        if (numbyt < 0)
        {
            c_errmsg ("Error reading portable GOF","gofrd-read",&one);
            return (E_FAIL);
        }
        else if (numbyt == 0) return (E_EOF);
        *obyte += numbyt;

        for (i = 0; i < 15; i++)
        {
            numbyt = read (*fd,&(projbuf.proj_coef[i]),PROJDUBLEN);
            if (numbyt < 0)
            {
                c_errmsg ("Error reading portable GOF","gofrd-read",&one);
                return (E_FAIL);
            }
            else if (numbyt == 0) return (E_EOF);
            *obyte += numbyt;
        }
    }

    if ((*type == HPOLIN) || (*type == PORT_HPOLIN) || (*type == ANOT) ||
        (*type == PORT_ANOT) || (*type == PNT) || (*type == PORT_PNT))
    {
        /* Determine the total record length. */
        if (*type != ANOT)
            sscanf (buf.length,"%d/%d",&clen,&dlen);
        *nbyte = clen + dlen + LENTYP;
        /* Determine the number of bytes to the next record. */
        *nbyte = *nbyte - *obyte;
        /* Copy the label into the struture. */ 
        if (*type == PNT)
            strcpy (gofptr->ptrec.label,buf.label);
        else if (*type == HPOLIN)
            strcpy (gofptr->plrec.label,buf.label);
        else
            strcpy (gofptr->antrec.label,a1buf.label);
        /* Copy the point or annotation XY pair into the structure. */
        if (*type == PNT) 
        {
            gofptr->ptrec.x = buf.x;
            gofptr->ptrec.y = buf.y;
        }
        else if (*type == PORT_PNT) 
        {
            gofptr->ptrec.x_proj = buf.x_proj;
            gofptr->ptrec.y_proj = buf.y_proj;
        }
        else if (*type == ANOT)
        {
            gofptr->antrec.x = a2buf.x;
            gofptr->antrec.y = a2buf.y;
        }
        else if (*type == PORT_ANOT)
        {
            gofptr->antrec.x_proj = a2buf.x_proj;
            gofptr->antrec.y_proj = a2buf.y_proj;
        }
        /* Convert the number of points to integer. */
        else if (*type == HPOLIN)
            gofptr->plrec.numpts = (int)buf.x;
        else
            gofptr->plrec.numpts = (int)buf.x_proj;
        /* Assign the address of the point values into the structure. */ 
        if (*type == PNT)
            for (i=0; i < MAXATT; i++)
                strcpy (gofptr->ptrec.attrb[i],buf.attrb[i]);
        else if (*type == ANOT)
            for (i=0; i < MAXATT; i++)
                strcpy (gofptr->antrec.attrb[i],a1buf.attrb[i]);
        else
            for (i=0; i < MAXATT; i++)
                strcpy (gofptr->plrec.attrb[i],buf.attrb[i]);
    }
    /* Copy the XY point values into the structure. */
    if (*type == XYPAIRS)
    {
        *nbyte = 0;
        gofptr->plrec.x = buf.xpoints;
        gofptr->plrec.y = buf.ypoints;
    }
    /* Copy the proj X/Y point values into the structure. */
    if (*type == PROJXYPAIRS)
    {
        *nbyte = 0;
        gofptr->plrec.x_proj = buf.x_proj_points;
        gofptr->plrec.y_proj = buf.y_proj_points;
    }
    /* Copy the annotation string into the structure. */ 
    if ((*type == ANOT) || (*type == PORT_ANOT))
    {
        *nbyte = 0;
        gofptr->antrec.size = (int)a2buf.size;
        gofptr->antrec.angle = a2buf.angle;
        strcpy (gofptr->antrec.antext,a1buf.anotstr);
    }
    /* Copy the projection information into the DDR structure. */
    if (*type == PORT_PNT || *type == PORT_HPOLIN ||
        *type == PROJXYPAIRS || *type == PORT_ANOT)
    {
        ddr->proj_code = (int)projbuf.proj_code;
        ddr->zone_code = (int)projbuf.zone_code;
        ddr->datum_code = (int)projbuf.datum_code;
        strcpy(ddr->proj_units, (const char *)projbuf.proj_units);
        for (i = 0; i < 15; i++)
            ddr->proj_coef[i] = projbuf.proj_coef[i];
    }

    return (E_SUCC);
}
