/******************************************************************************
NAME:           c_gofwr

PURPOSE:  Write the specified type of record to the GOF from the 
          information received in the defined structure.

PROGRAM HISTORY:
Version         Date    Contrib         Change
-------         ----    -------         ------
 1.0            3/86    L. Huewe        Intial code
 2.0            5/92    L. Huewe        Modified to be a more generic
                                        WORLD support routine.
 3.0            9/03    A. Shimitz      Added support for portable GOFs

PROJECT: LAS

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "tae.h"
#include "worgen.h"
#include "ddr.h"
#include "gof.h"

int c_gofwr
(
    int *fd,            /* File descriptor of the GOF to read (I).      */
    int *nbyte,         /* Number of bytes to next record (O).          */
    int *type,          /* Type of record being read (I).               */
    struct DDR *ddr,    /* DDR structure with projection info only (I). */
    union GOF *gofptr,  /* Pointer to the appropriate structure (I).    */
    int *addrec         /* TRUE - Adding records to the GOF     
                           FALSE - Updating existing records (I).       */
)

{
    static struct     RWHDR   arec,   /* Attribute definition record structure*/
                              oldrec; /* Old header record                    */

    static struct     PPLGOF  buf;    /* Point, polygon or line GOF record 
                                         structure                            */
    static struct     ANOT1   a1buf;  /* Character portion of annotation record 
                                         structure                            */
    static struct     ANOT2   a2buf;  /* Float portion of annotation record 
                                         structure                            */
    static struct  PORT_PROJ  projbuf;/* Projection info needed for
                                         portable GOFs                        */

    float       xypair[2];            /* XY pair for polygon or line. */
    double      projxypair[2];        /* proj X/Y pair for polygon or line. */

    int anotchar,                  /* Lgth of the attributes and annot string */
        current,                   /* Current # of records in the GOF */
        dlen,                      /* Data length */
        i,                         /* Loop index */
        obyte,                     /* Number of bytes written */
        pos;                       /* Current file position */
    long one = 1;                  /* Indicates a nonfatal error */

    char        temp[LENL];        /* Temporary string containing character 
                                      and data lengths */

    /* Write a GOF header record. */
    if (*type == HDR)
    {
        obyte = read (*fd, &oldrec, HDRLEN);
        lseek (*fd,0,0);
        if (obyte < 0)
        {
            c_errmsg ("Error reading GOF","gofwr-read",&one);
            return (E_FAIL);
        }
        else if (obyte == 0) strcpy (oldrec.rnum,"0"); 

        /* Set up the length, type and number of records. */
        *nbyte = HDRLEN;
        /* Length is split into 2 parts.  The first is the length of the 
           character portion of the record, the second is the length of the
           data portion of the record. */
        sprintf (temp,"%d/%d",*nbyte - LENTYP - MAXATT,MAXATT);
        sprintf (arec.length,"%-*s",LENL,temp);
        sprintf (arec.type,"%-*s",TYPL,"B");
        strcpy (arec.key,HDRKEY);
        sprintf (arec.rnum,"%d",gofptr->hdrec.nrec);

        /* Copy the attributes to the output buffer. */ 
        for (i=0; i < MAXATT; i++)
        {
            arec.mask[i] = gofptr->hdrec.mask[i];
            strcpy (arec.attrb[i],gofptr->hdrec.attrb[i]);
            strcpy (arec.attyp[i],gofptr->hdrec.attyp[i]);
        }

        /* Write the number of specified bytes. */ 
        obyte = write (*fd, &arec, *nbyte);
        if ((obyte <= 0) || (obyte != *nbyte))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }

        /* Set the number of bytes to the next record. */
        *nbyte = 0;
        return (E_SUCC);
    }

    /* Determine the number of bytes to write and the number of bytes to
       the next record. */
    switch (*type)
    {
        case HPOLIN:    /* Polygon or line subheader */
            *nbyte = gofptr->plrec.numpts * XYLEN;
            /* Length is split into 2 parts.  The first is the length of the 
               character portion of the record, the second is the length of
               the data portion of the record. */
            dlen = XYLEN;
            sprintf (temp,"%d/%d",CHARLEN,gofptr->plrec.numpts * XYLEN + XYLEN);
            sprintf (buf.length,"%-*s",LENL,temp);
            /* Set up the data type of the record. */
            sprintf (buf.type,"%-*s",TYPL,"R4");
            /* Copy the label to the output buffer. */ 
            strcpy (buf.label,gofptr->plrec.label);
            break;
        case PORT_HPOLIN:       /* Portable polygon or line subheader */
            *nbyte = gofptr->plrec.numpts * PROJXYLEN;
            /* Length is split into 2 parts.  The first is the length of the 
               character portion of the record, the second is the length of
               the data portion of the record. */
            dlen = PROJXYLEN;
            sprintf (temp, "%d/%d", CHARLEN,
                     gofptr->plrec.numpts * PROJXYLEN + PROJXYLEN);
            sprintf (buf.length,"%-*s",LENL,temp);
            /* Set up the data type of the record. */
            sprintf (buf.type,"%-*s",TYPL,"R8");
            /* Copy the label to the output buffer. */ 
            strcpy (buf.label,gofptr->plrec.label);
            break;
        case FPOLIN:    /* Full polygon or line record */
            *nbyte = 0;
            /* Length is split into 2 parts.  The first is the length of the 
               character portion of the record, the second is the length of
               the data portion of the record. */
            dlen = gofptr->plrec.numpts * XYLEN + XYLEN;
            sprintf (temp,"%d/%d",CHARLEN,dlen);
            sprintf (buf.length,"%-*s",LENL,temp);
            /* Set up the data type of the record. */
            sprintf (buf.type,"%-*s",TYPL,"R4");
            /* Copy the label to the output buffer. */ 
            strcpy (buf.label,gofptr->plrec.label);
            break;
        case PORT_FPOLIN:       /* Portable full polygon or line record */
            *nbyte = 0;
            /* Length is split into 2 parts.  The first is the length of the 
               character portion of the record, the second is the length of
               the data portion of the record. */
            dlen = gofptr->plrec.numpts * PROJXYLEN + PROJXYLEN;
            sprintf (temp,"%d/%d",CHARLEN,dlen);
            sprintf (buf.length,"%-*s",LENL,temp);
            /* Set up the data type of the record. */
            sprintf (buf.type,"%-*s",TYPL,"R8");
            /* Copy the label to the output buffer. */ 
            strcpy (buf.label,gofptr->plrec.label);
            break;
        case ANOT:              /* Annotation record */
            *nbyte = 0;
            /* Length is split into 2 parts.  The first is the length of the 
               character portion of the record, the second is the length of
               the data portion of the record. */
            anotchar = CHARLEN + strlen(gofptr->antrec.antext) + 1;
            sprintf (temp,"%d/%d",anotchar,2 * XYLEN);
            sprintf (a1buf.length,"%-*s",LENL,temp);
            /* Set up the data type of the record. */
            sprintf (a1buf.type,"%-*s",TYPL,"R4");
            /* Copy the label to the output buffer. */ 
            strcpy (a1buf.label,gofptr->antrec.label);
            break;
        case PORT_ANOT:              /* Portable annotation record */
            *nbyte = 0;
            /* Length is split into 2 parts.  The first is the length of the 
               character portion of the record, the second is the length of
               the data portion of the record. */
            anotchar = CHARLEN + strlen(gofptr->antrec.antext) + 1;
            sprintf (temp,"%d/%d",anotchar,2 * PROJXYLEN);
            sprintf (a1buf.length,"%-*s",LENL,temp);
            /* Set up the data type of the record. */
            sprintf (a1buf.type,"%-*s",TYPL,"R8");
            /* Copy the label to the output buffer. */ 
            strcpy (a1buf.label,gofptr->antrec.label);
            break;
        case PNT:               /* Point record */
            *nbyte = 0;
            /* Length is split into 2 parts.  The first is the length of the 
               character portion of the record, the second is the length of
               the data portion of the record. */
            dlen = XYLEN;
            sprintf (temp,"%d/%d",CHARLEN,dlen);
            sprintf (buf.length,"%-*s",LENL,temp);
            /* Set up the data type of the record. */
            sprintf (buf.type,"%-*s",TYPL,"R4");
            /* Copy the label to the output buffer. */ 
            strcpy (buf.label,gofptr->ptrec.label);
            break;
        case PORT_PNT:               /* Portable point record */
            *nbyte = 0;
            /* Length is split into 2 parts.  The first is the length of the 
               character portion of the record, the second is the length of
               the data portion of the record. */
            dlen = PROJXYLEN;
            sprintf (temp,"%d/%d",CHARLEN,dlen);
            sprintf (buf.length,"%-*s",LENL,temp);
            /* Set up the data type of the record. */
            sprintf (buf.type,"%-*s",TYPL,"R8");
            /* Copy the label to the output buffer. */ 
            strcpy (buf.label,gofptr->ptrec.label);
            break;
        case XYPAIRS:   /* XY pairs of a polygon or line record */
            *nbyte = 0;
            dlen = gofptr->plrec.numpts * XYLEN;
            break;
        case PROJXYPAIRS:   /* Coordinate pairs of a polygon or line record */
            *nbyte = 0;
            dlen = gofptr->plrec.numpts * PROJXYLEN;
            break;
    }
    /* Copy the point XY or coordinate value to the output buffer. */
    if (*type == PNT) 
    {
        buf.x = gofptr->ptrec.x;
        buf.y = gofptr->ptrec.y;
        /* Copy the attributes to the output buffer. */ 
        for (i=0; i < MAXATT; i++)
            strcpy (buf.attrb[i],gofptr->ptrec.attrb[i]);
    }
    if (*type == PORT_PNT) 
    {
        buf.x_proj = gofptr->ptrec.x_proj;
        buf.y_proj = gofptr->ptrec.y_proj;
        /* Copy the attributes to the output buffer. */ 
        for (i=0; i < MAXATT; i++)
            strcpy (buf.attrb[i],gofptr->ptrec.attrb[i]);
    }
    else if (*type == ANOT)
    {
        a2buf.x = gofptr->antrec.x;
        a2buf.y = gofptr->antrec.y;
        for (i=0; i < MAXATT; i++)
            strcpy (a1buf.attrb[i],gofptr->antrec.attrb[i]);
        /* Copy the size, angle and annotation string into the output buffer. */
        a2buf.size = (float)gofptr->antrec.size;
        a2buf.angle = gofptr->antrec.angle;
        strcpy (a1buf.anotstr,gofptr->antrec.antext);
    }
    else if (*type == PORT_ANOT)
    {
        a2buf.x_proj = gofptr->antrec.x_proj;
        a2buf.y_proj = gofptr->antrec.y_proj;
        for (i=0; i < MAXATT; i++)
            strcpy (a1buf.attrb[i],gofptr->antrec.attrb[i]);
        /* Copy the size, angle and annotation string into the output buffer. */
        a2buf.size = (float)gofptr->antrec.size;
        a2buf.angle = gofptr->antrec.angle;
        strcpy (a1buf.anotstr,gofptr->antrec.antext);
    }
    /* Convert the number of points to float. */
    else if ((*type == HPOLIN) || (*type == FPOLIN)) 
    {
        buf.x = (float)gofptr->plrec.numpts;
        for (i=0; i < MAXATT; i++)
            strcpy (buf.attrb[i],gofptr->plrec.attrb[i]);
    }
    /* Convert the number of points to double. */
    else if ((*type == PORT_HPOLIN) || (*type == PORT_FPOLIN)) 
    {
        buf.x_proj = (double)gofptr->plrec.numpts;
        for (i=0; i < MAXATT; i++)
            strcpy (buf.attrb[i],gofptr->plrec.attrb[i]);
    }
    /* Assign the address of the XY point values into the structure. */
    if ((*type == XYPAIRS) || (*type == FPOLIN)) 
    {
        buf.xpoints = gofptr->plrec.x;
        buf.ypoints = gofptr->plrec.y;
    }
    if ((*type == PROJXYPAIRS) || (*type == PORT_FPOLIN)) 
    {
        buf.x_proj_points = gofptr->plrec.x_proj;
        buf.y_proj_points = gofptr->plrec.y_proj;
    }
    /* Write the number of specified bytes. */ 
    if ((*type == FPOLIN) || (*type == HPOLIN) || (*type == PNT))
    {
        obyte = write (*fd, &buf, LENTYP + CHARLEN);
        if ((obyte <= 0) || (obyte != LENTYP + CHARLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        obyte = write (*fd, &(buf.x), XYLEN);
        if ((obyte <= 0) || (obyte != XYLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        if (dlen > XYLEN)
        {
            for (i=0; i < buf.x; i++)
            {
                xypair[0] = buf.xpoints[i];
                xypair[1] = buf.ypoints[i];
                obyte = write (*fd, xypair, XYLEN);
                if ((obyte <= 0) || (obyte != XYLEN))
                {
                    c_errmsg ("Error writing to GOF","gofwr-write",&one);
                    return (E_FAIL);
                }
            }
        }
    }
    else if ((*type == PORT_FPOLIN) || (*type == PORT_HPOLIN) ||
             (*type == PORT_PNT))
    {
        obyte = write (*fd, &buf, LENTYP + CHARLEN);
        if ((obyte <= 0) || (obyte != LENTYP + CHARLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        obyte = write (*fd, &(buf.x_proj), PROJXYLEN);
        if ((obyte <= 0) || (obyte != PROJXYLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        if (dlen > PROJXYLEN)
        {
            for (i=0; i < buf.x_proj; i++)
            {
                projxypair[0] = buf.x_proj_points[i];
                projxypair[1] = buf.y_proj_points[i];
                obyte = write (*fd, projxypair, PROJXYLEN);
                if ((obyte <= 0) || (obyte != PROJXYLEN))
                {
                    c_errmsg ("Error writing to GOF","gofwr-write",&one);
                    return (E_FAIL);
                }
            }
        }
    }
    else if (*type == ANOT)
    {
        obyte = write (*fd, &a1buf, anotchar+LENTYP);
        if ((obyte <= 0) || (obyte != anotchar+LENTYP))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        obyte = write (*fd, &a2buf, 2 * XYLEN);
        if ((obyte <= 0) || (obyte != 2 * XYLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
    }
    else if (*type == PORT_ANOT)
    {
        obyte = write (*fd, &a1buf, anotchar+LENTYP);
        if ((obyte <= 0) || (obyte != anotchar+LENTYP))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        obyte = write (*fd, &a2buf, 2 * PROJXYLEN);
        if ((obyte <= 0) || (obyte != 2 * PROJXYLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
    }
    else if (*type == XYPAIRS)
    {
        obyte = write (*fd, buf.xpoints, dlen);
        if ((obyte <= 0) || (obyte != dlen))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
    }
    else if (*type == PROJXYPAIRS)
    {
        obyte = write (*fd, buf.x_proj_points, dlen);
        if ((obyte <= 0) || (obyte != dlen))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
    }

    /* If writing a portable GOF record, store the projection information too */
    if (*type == PORT_PNT || *type == PORT_ANOT || *type == PORT_FPOLIN ||
        *type == PROJXYPAIRS)
    {
        /* Length is split into 2 parts.  The first is the length of the 
           character portion of the record, the second is the length of
           the data portion of the record. */
        dlen = 18*PROJDUBLEN;
        sprintf (temp,"%d/%d",CHARLEN,dlen);
        sprintf (projbuf.length,"%-*s",LENL,temp);
        /* Set up the data type of the record. */
        sprintf (projbuf.type,"%-*s",TYPL,"R8");
        /* Copy the label to the output buffer. */ 
        strcpy (projbuf.label,gofptr->plrec.label);

        /* Copy projection info to local buffer */
        strcpy((char *)projbuf.proj_units, ddr->proj_units);
        projbuf.proj_code = ddr->proj_code;
        projbuf.zone_code = ddr->zone_code;
        projbuf.datum_code = ddr->datum_code;
        for (i = 0; i < 15; i++)
            projbuf.proj_coef[i] = ddr->proj_coef[i];

        /* Write to the GOF file */
        obyte = write (*fd, &projbuf, LENTYP + CHARLEN);
        if ((obyte <= 0) || (obyte != LENTYP + CHARLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        obyte = write (*fd, &(projbuf.proj_code), PROJDUBLEN);
        if ((obyte <= 0) || (obyte != PROJDUBLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        obyte = write (*fd, &(projbuf.zone_code), PROJDUBLEN);
        if ((obyte <= 0) || (obyte != PROJDUBLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        obyte = write (*fd, &(projbuf.datum_code), PROJDUBLEN);
        if ((obyte <= 0) || (obyte != PROJDUBLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        for(i = 0; i < 15; i++)
        {
            obyte = write (*fd, &(projbuf.proj_coef[i]), PROJDUBLEN);
            if ((obyte <= 0) || (obyte != PROJDUBLEN))
            {
                c_errmsg ("Error writing to GOF","gofwr-write",&one);
                return (E_FAIL);
            }
        }
    }

    /* Increment the number of records in the GOF. */
    if (*addrec)
    {
        pos = lseek (*fd,0,1);
        lseek (*fd,0,0);
        obyte = read (*fd, &oldrec, HDRLEN);
        if (obyte < 0)
        {
            c_errmsg ("Error reading GOF","gofwr-read",&one);
            return (E_FAIL);
        }
        sscanf (oldrec.rnum,"%d",&current);
        current++;
        sprintf (oldrec.rnum,"%d",current);
        lseek (*fd,0,0);
        obyte = write (*fd, &oldrec, HDRLEN);
        if ((obyte <= 0) || (obyte != HDRLEN))
        {
            c_errmsg ("Error writing to GOF","gofwr-write",&one);
            return (E_FAIL);
        }
        lseek (*fd,pos,0);
    }

    return (E_SUCC);
}
