#include <math.h>
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "fhdr.h"
#include "header.h"

/******************************************************************************

NAME: c_fhdrrd_fst

PURPOSE: Read a particular field in a Fast Format Header.

PROGRAM HISTORY:
  Version	Date       Author	       Request
  -------	----	   ------              -------
   1.0        11/92        K Sheffield    initial developement
   7.2        04/99        G.Schmidt      Updated to support Fast-L7A format
   7.2        07/99        G.Schmidt      Updated to support Space Imaging
                                          Fast-C

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	

PROJECT: Landsat Pathfinder

ALGORITHM DESCRIPTION:
   Determine the field to read.
   Check to see if the data pointer is valid.
   Recast the data pointer to the appropriate data type.
   Reformat the data if needed into a temporary string.
   Copy the data or temporary string into the supplied data address.
   If no data is available, return HUGE_VAL (infinity) for numerical values, 
   and a NULL string for ascii values.

NOTE: This is the same as the c_fhdrrd routine, but it has been updated to
      work with Fast-L7A and Space Imaging Fast-C headers.
******************************************************************************/

int c_fhdrrd_fst(
   char *header,              /* the pointer to the fast fmt header       */
   long field,                /* the field to be read                     */
   char *data,                /* address to place the data read           */
   char *hdrtype)             /* type of header to read                   */
                              /* valid types are A, B, C, L7A, SIC        */
{
   char *cptr;                  /* pointer to character data                */
   char tmp[361];               /* temporary string                         */
   double *dptr;                /* pointer to double data                   */
   long flen;                   /* length of the field format               */
   long fieldinfo[5];           /* array containing field information       */
   long i;
   long *lptr;                  /* pointer to integer data                  */
   long status;                 /* function return status                   */
   long one = 1;
   

   memset(tmp,' ',360);
   tmp[360] = '\0';
   
   if (c_fhdfld_fst(field,fieldinfo,hdrtype) == E_FAIL)
   {
      c_errmsg("Error returned from fhdfld","fhdfld-call",&one);
      return(E_FAIL);
   }
   
   switch (fieldinfo[FDTYPE]) 
   {
/* return the ascii fields */
/* the first set are just labels */
    case 'C':
/* these fields are ascii text with parameters/values in them */
    case 'A':
      if(data != NULL)
      {
	 flen = fieldinfo[FLEN1];
	 strncpy(data,&header[fieldinfo[FSBYTE]],flen);
	 data[flen] = 0;
	 if(strspn(data," ") == flen)
	   *data = 0;
      }
      else
      {
	 c_errmsg("Illegal data address","fhdrrd-addr",&one);
	 return(E_FAIL);
      }
      
      break;
      
/* these fields contain simple integer values */
    case 'I':
      if( data != NULL )
      {
	 lptr = (long *) data;
	 flen = fieldinfo[FLEN1];
	 strncpy(tmp, &header[fieldinfo[FSBYTE]],flen);
	 status = sscanf(tmp,"%ld",lptr);
	 if(status != 1)
	   *lptr = (long)HUGE_VAL;
      }
      else
      {
	 c_errmsg("Illegal data address","fhdrrd-addr",&one);
	 return(E_FAIL);
      }

      break;
      
/* these fields contain simple floating point values */
    case 'D':
      if( data != NULL )
      {
	 flen = fieldinfo[FLEN1];
	 dptr = (double *) data;
	 
	 strncpy(tmp, &header[fieldinfo[FSBYTE]],flen);
	 status = sscanf(tmp,"%lf",dptr);
	 if(status != 1)
	   *dptr = HUGE_VAL;
      }
      else
      {
	 c_errmsg("Illegal data address","fhdrrd-addr",&one);
	 return(E_FAIL);
      }
      break;
      
/* These fields are assumed to be latitudes 
   The format of these numbers are in the form ddmmss.ssssN or ddmmss.ssssS 
   the value passed in is to be in decimal degrees */
    case 'l':
      if ( data != NULL )
      {
	 flen = fieldinfo[FLEN1];
	 dptr = (double *) data;

	 strncpy(tmp, &header[fieldinfo[FSBYTE]],flen);
	 status = dms2deg(tmp,dptr,0);
	 if(status)
	   *dptr = HUGE_VAL;
      }
      else
      {
	 c_errmsg("Illegal data address","fhdrrd-addr",&one);
	 return(E_FAIL);
      }
      
      break;
      
      
/* These fields are assumed to be longitudes 
   The format of these numbers are in the form dddmmss.ssssE or dddmmss.ssssW 
   the value passed in is to be in decimal degrees */
    case 'L':
      if ( data != NULL )
      {
	 flen = fieldinfo[FLEN1];
	 dptr = (double *) data;

	 strncpy(tmp, &header[fieldinfo[FSBYTE]],flen);
	 status = dms2deg(tmp,dptr,1);
	 if(status)
	   *dptr = HUGE_VAL;
      }
      else
      {
	 c_errmsg("Illegal data address","fhdrrd-addr",&one);
	 return(E_FAIL);
      }

      break;

/* this field is for the USGS projection parameters.  These are a special
   case since the formatting has to be changed to FORTRAN specs */
    case 'G':
      if (data != NULL)
      {
	 flen = fieldinfo[FLEN1];
	 dptr = (double *) data;
	 
	 cptr = header + fieldinfo[FSBYTE];/* keep track of current location*/

	 
	 for(i=0;i<15;i++, cptr += 24)
	 {
	    strncpy(tmp,cptr,flen);
	    status = fstr2cdbl(tmp,dptr);
	    if(status)
	      *dptr = HUGE_VAL;
	    dptr++;
	 }
      }
      else
      {
	 c_errmsg("Illegal data address","fhdrrd-addr",&one);
	 return(E_FAIL);
      }

      break;
      
    default:
      c_errmsg("Illegal field data type","fhdrrd-field",&one);
      return(E_FAIL);
   }
   
   return(E_SUCC);
}
