#include <math.h>
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "util.h"
#include "fhdr.h"
#include "header.h"

/******************************************************************************

NAME: c_fhdrup

PURPOSE: Update a particular field in a Fast Format Header.

PROGRAM HISTORY:
  Version	Date       Author	       Request
  -------	----	   ------              -------
   1.0        11/92        K Sheffield    initial developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	

PROJECT: Landsat Pathfinder

ALGORITHM DESCRIPTION:
   If the field to be updated is supposed to be a constant, return E_FAIL
   Recast the data pointer to the appropriate data type.
   If the data pointer is NULL or equal to infinity, the field will be 
   blanked out.
   Reformat the data if needed into a temporary string.
   Copy the data or temporary string into the appropriate field.
   Return E_SUCC if successfull.

   Note: This function assumes that the data being passed in is in the 
         correct data type, (ie. ascii, long interger, float, double)
	 Only one field (#49 USGS projection params) needs an array of values.
	 
******************************************************************************/


int c_fhdrup
(
    char *header,               /* the pointer to the fast fmt header       */
    long field,                 /* the field to be updated                  */
    char *data                  /* pointer to the data to place in the field*/
)
{
   char *cptr;                  /* pointer to character data                */
   char fmtstr[80];             /* array used to build sprintf formats       */
   char tmp[361];               /* temporary string                          */
   double *dptr;                /* pointer to real data                      */
   long fieldinfo[5];           /* array containing field information        */
   long flen;                   /* length of the field format                */
   long *lptr;                  /* pointer to integer data                   */
   long one = 1;
   long status;                 /* function return status                    */
   long slen;                   /* length of a string                        */
   long i;
   
   memset(tmp,' ',360);
   tmp[360] = '\0';
   
   if(E_FAIL == c_fhdfld(field,fieldinfo))
   {
      c_errmsg("Error returned from fhdfld","fhdfld-call",&one);
      return(E_FAIL);
   }
   
   
   switch ( fieldinfo[FDTYPE] ) 
   {
/* return E_FAIL if trying to update a label or blank field 
   -------------------------------------------------------*/
    case 'C':
      c_errmsg("Cannot update a constant field","fhdrup-constant",&one);
      return(E_FAIL);

/* these fields are just ascii text 
   --------------------------------*/
    case 'A':
      if(data != NULL)
      {
	 slen = (long)strlen(data);
	 slen = (slen > 361) ? 360  : slen;
	 strncpy(tmp,data,slen);
	 c_low2up(tmp,&slen);     
      }
      
      strncpy(&header[fieldinfo[FSBYTE]],tmp,fieldinfo[FLEN1]);
      break;
      
/* these fields contain simple integer values 
   ------------------------------------------*/

    case 'I':
      flen = fieldinfo[FLEN1];
      lptr = (long *) data;
      if( data != NULL && *lptr != ((long) HUGE_VAL) )
      {
	 sprintf(fmtstr,"%%%dd ",flen);
	 sprintf(tmp,fmtstr ,*lptr);
      }
      strncpy(&header[fieldinfo[FSBYTE]],tmp,flen);
      break;
      
/* these fields contain simple floating point values 
   -------------------------------------------------*/
    case 'D':
      flen = fieldinfo[FLEN1];
      dptr = (double *) data;
      if( data != NULL && *dptr != HUGE_VAL )
      {
	 sprintf(fmtstr,"%%%d.%dlf ",flen, fieldinfo[FLEN2]);
	 sprintf(tmp,fmtstr ,*dptr);
      }
      strncpy(&header[fieldinfo[FSBYTE]],tmp,flen);
      break;
      
/* These fields are assumed to be latitudes 
   The format of these numbers are in the form ddmmss.ssssN or ddmmss.ssssS 
   the value passed in is to be in decimal degrees 
   ------------------------------------------------------------------------*/

    case 'l':
      flen = fieldinfo[FLEN1];
      dptr = (double *) data;
      
      if ( data != NULL && *dptr != HUGE_VAL )
      {
	 status = deg2dms(tmp,*dptr,0);
	 if(status)
	 {
	    c_errmsg("Error returned from deg2dms","deg2dms-call",&one);
	    return(E_FAIL);
	 }
      }
      strncpy(&header[fieldinfo[FSBYTE]],tmp,flen);
      break;
      
      
/* These fields are assumed to be longitudes 
   The format of these numbers are in the form dddmmss.ssssE or dddmmss.ssssW 
   the value passed in is to be in decimal degrees 
   --------------------------------------------------------------------------*/

    case 'L':
      flen = fieldinfo[FLEN1];
      dptr = (double *) data;
      if ( data != NULL && *dptr != HUGE_VAL )
      {
	 status = deg2dms(tmp,*dptr,1);
	 if(status)
	 {
	    c_errmsg("Error returned from deg2dms","deg2dms-call",&one);
	    return(E_FAIL);
	 }
      }
      strncpy(&header[fieldinfo[FSBYTE]],tmp,flen);
      break;

/* this field is for the 15 USGS GTCP projection parameters.  These are a 
   special case since the formatting has to be changed to FORTRAN specs 
   ---------------------------------------------------------------------*/

    case 'G':
      flen = fieldinfo[FLEN1];
      dptr = (double *) data;
      
      if (data != NULL && *dptr != HUGE_VAL )
      {
	 
	 cptr = header + fieldinfo[FSBYTE];/* keep track of current location*/

	 for(i=0;i<15;i++, cptr += flen)
	 {
	    status = cdbl2ftn(tmp,*dptr);
	    if(status)
	    {
	       c_errmsg("Error returned from cdbl2ftn","cdbl2ftn-call",&one);
	       return(E_FAIL);
	    }
	    strncpy(cptr,tmp,flen);
	    dptr++;
	 }
      }
      else
	strncpy(&header[fieldinfo[FSBYTE]],tmp,flen*15);

      break;
      
    default:
      c_errmsg("Illegal field data type","fhdrup-field",&one);
      return(E_FAIL);
   }
   
   return(E_SUCC);
}
