/******************************************************************************
NAME: convert_dms

PURPOSE: Convert all of the projection parameters from the newer GCTP DMS
	 format into the older format or vice versa.  The new format only 
	 uses 2 ascii characters for the minutes and seconds fields, while 
	 the older format uses 3.

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ----------------------------------------
    1    12/91  J.Fenno          EDC   Development
    2 	 11/92	T.Mittan	 EDC   Modified to convert from LAS DMS to
				       fast format 
    3    12/92  K.Sheffield      EDC   Modified to convert from either format
                                       to the other.
    4     7/93  D.Etrheim        EDC   Change to use projection constants

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

ASSUMPTIONS:
   
   All character strings have a length of 24 and are of the form:
      "  -0.xxxxxxxxxxxxxxxE+XX"

   New format to old format:
      
      "  -0.dddmmssssssssssE+07"   => insert 0 before minutes and seconds
      "  -0.ddmmsssssssssssE+06"   => insert 0 before minutes and seconds
      "  -0.dmmssssssssssssE+05"   => insert 0 before minutes and seconds
      "  -0.mmsssssssssssssE+04"   => insert 0 before seconds
      "  -0.mssssssssssssssE+03"   => insert 0 before seconds
      "  -0.sssssssssssssssE+02"   => do nothing
      "  -0.sssssssssssssssE+01"   => do nothing
      "  -0.sssssssssssssssE+00"   => do nothing
   
   Old format to new format:

      "  -0.dddmmmsssssssssE+09"   => remove 0 before minutes and seconds
      "  -0.ddmmmssssssssssE+08"   => remove 0 before minutes and seconds
      "  -0.dmmmsssssssssssE+07"   => remove 0 before minutes and seconds
      "  -0.mmmssssssssssssE+06"   => remove 0 before minutes and seconds
      "  -0.mmsssssssssssssE+05"   => remove 0 before seconds
      "  -0.mssssssssssssssE+04"   => remove 0 before seconds 
      "  -0.sssssssssssssssE+03"   => remove 0 before seconds
      "  -0.sssssssssssssssE+02"   => do nothing
      "  -0.sssssssssssssssE+01"   => do nothing
      "  -0.sssssssssssssssE+00"   => do nothing

   where:
   
      1. The leading sign can be either negative, positive or absent
      2. The character to the left of the decimal point is '0'
      3. The fifth character is '.'
      4. d, m, and s are characters from '0' to '9'
      5. E is always in the fourth position from the right
      6. The third position from the right is either '+' or '-'
      7. The exponent is '00' to '07' for the new format.  The exponent is
         '00' to '09' for the old format.  All others will be trapped

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "proj.h"
#include "header.h"

static long convert2new ();  /* forward declaration */
static long convert2old ();  /* forward declaration */

int convert_dms 
(
    char **strings,    /* pointer to strings     */
    long proj_number,  /* USGS projection number */
    long method        /* converstion method.  0 = old->new,  !0 = new->old */
)
{
   char temp_str[CMLEN]; /* temporary string                                */
   char errstr[CMLEN];   /* output error string                             */
   char errcall[40];     /* function call that generated the error          */
   long nonfatal =  1;
   long (*convert)();    /* pointer to conversion routine */
   
   /* determine what conversion method to use */

   if(method == 0)
   {
      convert = convert2new;
      strcpy(errstr,"Error returned from convert2new");
      strcpy(errcall,"convert2new-call");
   }
   else
   {
      convert = convert2old;
      strcpy(errstr,"Error returned from convert2old");
      strcpy(errcall,"convert2old-call");
   }
  
   if (proj_number == UTM)
   {
      /* UTM parameters 1 & 2 are DMS fields */
      if(E_FAIL == (*convert)(*(strings + 0))) 
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      
      if(E_FAIL == (*convert)(*(strings + 1)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
   }
   else if (proj_number == SPCS)
   {
      /* SPCS has no parameters that are in DMS format */
   }
   else if ((proj_number == ALBERS) || (proj_number == LAMCC) || 
            (proj_number == EQUIDC))
   {
      /* ACEA, LCC, & EC parms 3, 4, 5, & 6 are DMS fields */
      if(E_FAIL == (*convert)(*(strings + 2)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings + 3)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
   
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings + 4)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings + 5)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
   }
   else if ((proj_number ==  MERCAT) || (proj_number ==  PS) ||
	    (proj_number ==  POLYC) || (proj_number ==  TM) ||
	    (proj_number == STEREO) || (proj_number == LAMAZ) ||
	    (proj_number == AZMEQD) || (proj_number == GNOMON) ||
	    (proj_number == ORTHO) || (proj_number == GVNSP) ||
	    (proj_number == EQRECT) || (proj_number == VGRINT))
   {
      /* MER,PS,PC,TM,SG,LAEA,AE,GNO,OG,GVNP,ER, & VDG parms 5 & 6 are 
	 DMS fields*/
      if(E_FAIL == (*convert)(*(strings + 4)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings + 5)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
   }
   else if ((proj_number == SNSOID) || (proj_number == MILLER))
   {
      /* SIN & MC parameter 5 is a DMS field */
      if(E_FAIL == (*convert)(*(strings + 4)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
   }
   else if (proj_number == SOM)
   {
      /* SOM parameters 4 & 5 are DMS fields */
      if(E_FAIL == (*convert)(*(strings + 3)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings + 4)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
   }
   else if (proj_number == HOM)
   {
      /* OM parms 4, 5, 6, 9, 10, 11, & 12 are DMS fields */
      if(E_FAIL == (*convert)(*(strings +  3)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings +  4)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings +  5)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings +  8)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings +  9)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings + 10)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
      if(E_FAIL == (*convert)(*(strings + 11)))
      {
	 c_errmsg(errstr,errcall, &nonfatal);
	 return(E_FAIL);
      }
   }
   else
   {
      sprintf(temp_str,"Map projection number %d unknown",proj_number);
      c_errmsg(temp_str,"convert_dms-projection", &nonfatal);
      return(E_FAIL);
      
   }
   return(E_SUCC);
}


/***************************************************************************
NAME: convert2old
    
PURPOSE: Convert a single string from the newer GCTP DMS format into the 
         older format.  The new format only uses 2 ascii characters for 
         the minutes and seconds fields, while the older format uses 3.
    
PROGRAM HISTORY:
    
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ------------------------------------
1        11/91  J.Fenno          EDC   Development
2        12/92  K.Sheffield      EDC   Modified for a more general purpose
                                       convert_dms.
3     	  3/95  S.Nelson	 EDC   Modified error handling so that the
				       exact string is returned when an
				       error is encountered.
    
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.
    
PROJECT: LAS
    
ASSUMPTIONS:  See those for convert_dms
    
**************************************************************************/

static long convert2old (str)

char *str;     /* in/out - string containing a DDDMMSS longitude or latitude */

{
   char degree_str[4];    /* the degrees portion of the input and output     */
   char dms_str[16];      /* the dms portion of the input                    */
   char E_str[3];         /* the E+ or E- portion of the input and output    */
   char exponent_str[3];  /* the exponent portion of the input and output    */
   char minute_str[4];    /* the minutes portion of the input and output     */
   char second_str[16];   /* the seconds portion of the input and output     */
   char temp_str[CMLEN];  /* temporary string                                */
   
   long nonfatal =  1;
   
   if (strlen(str) < 24)
   {
      sprintf(temp_str,"DMS field length is %1d, expecting 24",strlen(str));
      c_errmsg(temp_str, "convert_dms-dmslen", &nonfatal);
      return(E_FAIL);
   }
   
   /* Extract the dms, E+ or E-, and the exponent portions of the input string
      -----------------------------------------------------------------------*/
   strncpy(dms_str,      (str +  5), 15);  dms_str[15]     = '\0';
   strncpy(E_str,        (str + 20),  2);  E_str[2]        = '\0';
   strncpy(exponent_str, (str + 22),  2);  exponent_str[2] = '\0';
   *(str + 5) = '\0';
   
   if ((strcmp(exponent_str,"07") == 0) ||
       (strcmp(exponent_str,"06") == 0) ||
       (strcmp(exponent_str,"05") == 0))
   {
      if (strcmp(exponent_str,"07") == 0)
      {
	 strncpy(degree_str, &dms_str[0], 3);  degree_str[3] = '\0';
	 strncpy(minute_str, &dms_str[3], 2);  minute_str[2] = '\0';
	 strncpy(second_str, &dms_str[5], 8);  second_str[8] = '\0';
	 strcpy (exponent_str, "09");
      }
      else if (strcmp(exponent_str,"06") == 0)
      {
	 strncpy(degree_str, &dms_str[0], 2);  degree_str[2] = '\0';
	 strncpy(minute_str, &dms_str[2], 2);  minute_str[2] = '\0';
	 strncpy(second_str, &dms_str[4], 9);  second_str[9] = '\0';
	 strcpy (exponent_str, "08");
      }
      else if (strcmp(exponent_str,"05") == 0)
      {
	 strncpy(degree_str, &dms_str[0],  1);  degree_str[1]  = '\0';
	 strncpy(minute_str, &dms_str[1],  2);  minute_str[2]  = '\0';
	 strncpy(second_str, &dms_str[3], 10);  second_str[10] = '\0';
	 strcpy (exponent_str, "07");
      }
      strcpy(temp_str,"0");           /* add zero before the minute field */
      strcat(temp_str, minute_str);
      strcpy(minute_str, temp_str);
      strcpy(temp_str,"0");           /* add zero before the second field */
      strcat(temp_str, second_str);
      strcpy(second_str, temp_str);
   }
   else if ((strcmp(exponent_str,"04") == 0) ||
	    (strcmp(exponent_str,"03") == 0))
   {
      strcpy (degree_str, "");
      if (strcmp(exponent_str,"04") == 0)
      {
	 strncpy(minute_str, &dms_str[0],  2);  minute_str[2]  = '\0';
	 strncpy(second_str, &dms_str[2], 12);  second_str[12] = '\0';
	 strcpy (exponent_str, "05");
      }
      else if (strcmp(exponent_str,"03") == 0)
      {
	 strncpy(minute_str, &dms_str[0],  1);  minute_str[1]  = '\0';
	 strncpy(second_str, &dms_str[1], 13);  second_str[13] = '\0';
	 strcpy (exponent_str, "04");
      }
      strcpy(temp_str,"0");           /* add zero before the second field */
      strcat(temp_str, second_str);
      strcpy(second_str, temp_str);
   }
   else if ((strcmp(exponent_str,"02") == 0) ||
	    (strcmp(exponent_str,"01") == 0) ||
	    (strcmp(exponent_str,"00") == 0))
   {
      strcpy(degree_str, "");
      strcpy(minute_str, "");
      strcpy(second_str, dms_str);
   }
   else
   {
      sprintf(temp_str, "Unexpected exponent %s found in DMS field", 
	      exponent_str);
      c_errmsg(temp_str, "convert_dms-dms", &nonfatal);

      /* replace with original value
        ----------------------------*/
      str[5] = dms_str[0];
      return(E_FAIL);
   }
   
   strcat(str, degree_str);
   strcat(str, minute_str);
   strcat(str, second_str);
   strcat(str, E_str);
   strncpy((str + 22), exponent_str, 2);

   return(E_SUCC);
   
}

/*****************************************************************************
NAME: convert2new

PURPOSE: Convert a single string from the older format to the new GCTP 
         DMS format.  This is only for the exponetial form.  The new format 
	 only uses 2 ascii characters for the minutes and seconds fields, 
	 while the older format uses 3.

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ----------------------------------------
    1    11/91  J.Fenno          EDC   Development
    2    11/92  T. Mittan	 EDC   Modified to work with fast format out
    2    12/92  K.Sheffield      EDC   Modified for a more general purpose
                                       convert_dms.
    3     3/95  S.Nelson	 EDC   Modified error handling so that the
				       exact string is returned when an
				       error is encountered.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

ASSUMPTIONS:  See those for convert_dms

*******************************************************************************/

static long convert2new (str)

char *str;     /* in/out - string containing a DDDMMSS longitude or latitude */

{
   char degree_str[4];    /* the degrees portion of the input and output     */
   char dms_str[16];      /* the dms portion of the input                    */
   char E_str[3];         /* the E+ or E- portion of the input and output    */
   char exponent_str[3];  /* the exponent portion of the input and output    */
   char minute_str[4];    /* the minutes portion of the input and output     */
   char second_str[16];   /* the seconds portion of the input and output     */
   char temp_str[CMLEN];  /* temporary string                                */
   
   long nonfatal =  1;
   
     if (strlen(str) < 24)
     {
	sprintf(temp_str,"DMS field length is %1d, expecting 24",strlen(str));
     	c_errmsg(temp_str, "convert_dms-dmslen", &nonfatal);
	return(E_FAIL);
	
     }
   
   /* Extract the dms, E+ or E-, and the exponent portions of the input string
      -----------------------------------------------------------------------*/
   strncpy(dms_str,      (str +  5), 15);  dms_str[15]     = '\0';
   strncpy(E_str,        (str + 20),  2);  E_str[2]        = '\0';
   strncpy(exponent_str, (str + 22),  2);  exponent_str[2] = '\0';
   *(str + 5) = '\0';
   
   if ((strcmp(exponent_str,"09") == 0) ||
       (strcmp(exponent_str,"08") == 0) ||
       (strcmp(exponent_str,"07") == 0) ||
       (strcmp(exponent_str,"06") == 0))
   {
      if (strcmp(exponent_str,"09") == 0)
      {
	 strncpy(degree_str, &dms_str[0], 3);  degree_str[3] = '\0';
	 strncpy(minute_str, &dms_str[4], 2);  minute_str[2] = '\0';
	 strncpy(second_str, &dms_str[7], 8); 
	 strncpy(&second_str[8], "00",2); second_str[10] = '\0';
	 strcpy (exponent_str, "07");
      }
      else if (strcmp(exponent_str,"08") == 0)
      {
	 strncpy(degree_str, &dms_str[0], 2);  degree_str[2] = '\0';
	 strncpy(minute_str, &dms_str[3], 2);  minute_str[2] = '\0';
	 strncpy(second_str, &dms_str[6], 9); 
	 strncpy(&second_str[9], "00",2); second_str[11] = '\0';
	 strcpy (exponent_str, "06");
      }
      else if (strcmp(exponent_str,"07") == 0)
      {
	 strncpy(degree_str, &dms_str[0],  1);  degree_str[1]  = '\0';
	 strncpy(minute_str, &dms_str[2],  2);  minute_str[2]  = '\0';
	 strncpy(second_str, &dms_str[5], 10); 
	 strncpy(&second_str[10], "00",2); second_str[12] = '\0';
	 strcpy (exponent_str, "05");
      }
      else if (strcmp(exponent_str,"06") == 0)
      {
	 strcpy (degree_str, "");
	 strncpy(minute_str, &dms_str[1],  2);  minute_str[2]  = '\0';
	 strncpy(second_str, &dms_str[4], 11);
	 strncpy(&second_str[11], "00",2); second_str[13] = '\0';
	 strcpy (exponent_str, "04");
      }
   }
   else if ((strcmp(exponent_str,"05") == 0) ||
	    (strcmp(exponent_str,"04") == 0) ||	
	    (strcmp(exponent_str,"03") == 0))
   {
      strcpy (degree_str, "");
      if (strcmp(exponent_str,"05") == 0)
      {
	 strncpy(minute_str, &dms_str[0],  2);  minute_str[2]  = '\0';
	 strncpy(second_str, &dms_str[3], 12); 
	 strncpy(&second_str[12], "00",1); second_str[13] = '\0';
	 strcpy (exponent_str, "04");
      }
      else if (strcmp(exponent_str,"04") == 0)
      {
	 strncpy(minute_str, &dms_str[0],  1);  minute_str[1]  = '\0';
	 strncpy(second_str, &dms_str[2], 13); 
	 strncpy(&second_str[13], "00",1); second_str[14] = '\0';
	 strcpy (exponent_str, "03");
      }
      else if (strcmp(exponent_str,"03") == 0)
      {
	 strcpy(minute_str, "");
	 strncpy(second_str, &dms_str[1], 13); 
	 strncpy(&second_str[14], "00",1); second_str[15] = '\0';
	 strcpy (exponent_str, "02");
      }
   }
   else if ((strcmp(exponent_str,"02") == 0) ||
	    (strcmp(exponent_str,"01") == 0) ||
	    (strcmp(exponent_str,"00") == 0))
   {
      strcpy(degree_str, "");
      strcpy(minute_str, "");
      strcpy(second_str, dms_str);
   }
   else
   {
      sprintf(temp_str, "Unexpected exponent %s found in DMS field", 
	      exponent_str);
      c_errmsg(temp_str, "convert_dms-dms", &nonfatal);

      /* replace with original value
        ----------------------------*/
      str[5] = dms_str[0];
      return(E_FAIL);
   }
   
   strcat(str, degree_str);
   strcat(str, minute_str);
   strcat(str, second_str);
   strcat(str, E_str);
   strncpy((str + 22), exponent_str, 2);
   return(E_SUCC);
}
