#include <math.h>
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "header.h"

/******************************************************************************

NAME: deg2dms

PURPOSE: To convert a latitude or longitude in decimal degees into a DMS
         format used in the fast format header.  This format is as follows:
	 dddmmss.ssssW for longitude or ddmmss.ssssN for latitude.

PROGRAM HISTORY:
  Version      Date       Author	        Request
  -------      ----       ------               -------
    1.0        11/92      K. Sheffield         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	

PROJECT: Landsat Pathfinder

ALGORITHM DESCRIPTION:
  Determine the direction (N, S, E or W)
  Extract the integer degrees from the input value.
  Calculate the number of minutes and seconds.
  Print these values to the return buffer 

******************************************************************************/


int deg2dms
(
    char *buffer,               /* the buffer to write the string to */
    double value,               /* the value to convert */
    long mode                   /* latitude = 0, longitude = 1 */
)
{
   
   long degrees;
   long minutes;
   float sec;
   char direct;                      /* direction either N, S, E or W */

   
   /* determine the direction */

   if(mode == 0)
     direct = (value > 0.0) ? 'N' : 'S';
   else
     direct = (value > 0.0) ? 'E' : 'W';

   if(value < 0.0)
     value *= -1.0;
   
/* extract the number of degrees */

   degrees = (long)value;
   value -= degrees;
   
/* extract the number of minutes */

   value *= 60;
   minutes = (long)value;
   value -= minutes;
   
/* extract the number of seconds */

   value *= 60;
   sec = value;
   
/* now create the output string */

   if(mode)   /* printing out longitudes */
     sprintf(buffer,"%03d%02d%07.4f%c ",degrees,minutes,sec,direct);
   else       /* printing out latitudes */
     sprintf(buffer,"%02d%02d%07.4f%c ",degrees,minutes,sec,direct);

/*   if(status == EOF)
      {
      c_errmsg("Unable to generate DMS string","deg2dms-sprintf",&one);
      return(E_FAIL);
      }
   else*/

     return(E_SUCC);
}

   

/******************************************************************************

NAME: dms2deg

PURPOSE: To convert a latitude or longitude in the DMS format used in the 
         fast format header to decimal degrees.  This format is as follows:
	 dddmmss.ssssW for longitude or ddmmss.ssssN for latitude.

PROGRAM HISTORY:
  Version      Date       Author	        Request
  -------      ----       ------               -------
    1.0        11/92      K. Sheffield         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	

PROJECT: Landsat Pathfinder

ALGORITHM DESCRIPTION:
  Parse the degrees, minutes, and seconds, and direction from the input string.
  Determine the sign of the value.

******************************************************************************/


int dms2deg
(
    char *buffer,               /* the buffer that contains the string */
    double *value,              /* the address to place the result */
    long mode                   /* latitude = 0, longitude = 1 */
)
{
   long status;
   int deg, min;
   float sec;
   char tmpbuf[10];
   long one = 1;
   
   
   char direction;
   
   if(mode)
   {
      /* reading longitude */
      status = sscanf(buffer,"%3d%2d%7s%c",&deg, &min, tmpbuf, &direction);
      if(status != 4)
      {
	 c_errmsg("Unable to parse input string","dms2deg-sscanf",&one);
	 return(E_FAIL);
      }
      
      
/* had to copy the float part into it's own string for some stupid reason */
      status = sscanf(tmpbuf,"%f",&sec);
      if(status != 1)
      {
	 c_errmsg("Unable to parse input string","dms2deg-sscanf",&one);
	 return(E_FAIL);
      }
      
      *value = deg + min / 60.0 + sec / 3600.0;
      if(direction == 'W')
	*value *= -1;
   }
   else
   {
      /* reading latitude */
      status = sscanf(buffer,"%2d%2d%7s%c",&deg, &min, tmpbuf, &direction);
      if(status != 4)
      {
	 c_errmsg("Unable to parse input string","dms2deg-sscanf",&one);
	 return(E_FAIL);
      }
      
/* had to copy the float part into it's own string for some stupid reason */
      status = sscanf(tmpbuf,"%f",&sec);
      if(status != 1)
      {
	 c_errmsg("Unable to parse input string","dms2deg-sscanf",&one);
	 return(E_FAIL);
      }
      
      *value = deg + min / 60.0 + sec / 3600.0;
      if(direction == 'S')
	*value *= -1;
   }
   
   return(E_SUCC);
   
}

   
      
   
