#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "fhdr.h"
#include "util.h"
#include "header.h"


/*
   fastfmt_fields - a table indicating the byte positions, type and format
                    of fields in the fast format header.  A 5 element vector
                    exists for each field.  The elements in that vector are
                    defined as follows:

    1st element - starting byte position ( zero relative )
    2nd element - ending byte position ( zero relative )
    3rd element - field type A = ascii, C = constant, I = integer, D = double
                  L = longitude, l = latitude, G = GTCP parameter
    4th element - format length
    5th element - format precision - only used for float and double
*/

/* Note: For Fast-L7A the admin, radiometric, and geometric headers are
 *       back to back in the header file.  Thus the first 1536 bytes are
 *       for the administrative header, the next 1536 bytes are for the
 *       radiometric header, and the last 1536 bytes are for the geometric
 *       header.  The administrative header contains 133 fields.  The
 *       radiometric header contains 65 fields.  The following table is set
 *       up so the programmer may follow the Fast-L7A documentation.
 *       The Fast-L7A fields are used for Space Imaging Fast-C too.
 */

static long fastL7_fields[NUM_ADMIN_FIELDS+NUM_RAD_FIELDS+NUM_GEOM_FIELDS][5]={
{ 0,	7,	'C',	8,	0 },  /* Administrative Record (Line 1) */
{ 8,	27,	'A',	20,	0 },
{ 28,	33,	'C',	6,	0 },
{ 34,	50,	'A',	17,	0 },
{ 51,	69,	'C',	19,	0 },
{ 70,	77,	'A',	8,	0 },
{ 78,	78,	'C',	1,	0 },
{ 79,	79,	'C',	1,	0 },
{ 80,	90,	'C',	11,	0 },  /* Line 2 */
{ 91,	100,	'A',	10,	0 },
{ 101,	109,	'C',	9,	0 },
{ 110,	119,	'A',	10,	0 },
{ 120,	133,	'C',	14,	0 },
{ 134,	139,	'A',	6,	0 },
{ 140,	152,	'C',	13,	0 },
{ 153,	158,	'D',	6,	2 },
{ 159,	159,	'C',	1,	0 },
{ 160,	182,	'C',	23,	0 },  /* Line 3 */
{ 183,	193,	'C',	11,	0 },
{ 194,	210,	'A',	17,	0 },
{ 211,	229,	'C',	19,	0 },
{ 230,	237,	'A',	8,	0 },
{ 238,	238,	'C',	1,	0 },
{ 239,	239,	'C',	1,	0 },
{ 240,	250,	'C',	11,	0 },  /* Line 4 */
{ 251,	260,	'A',	10,	0 },
{ 261,	269,	'C',	9,	0 },
{ 270,	279,	'A',	10,	0 },
{ 280,	293,	'C',	14,	0 },
{ 294,	299,	'A',	6,	0 },
{ 300,	312,	'C',	13,	0 },
{ 313,	318,	'D',	6,	2 },
{ 319,	319,	'C',	1,	0 },
{ 320,	342,	'C',	23,	0 },  /* Line 5 */
{ 343,	353,	'C',	11,	0 },
{ 354,	370,	'A',	17,	0 },
{ 371,	389,	'C',	19,	0 },
{ 390,	397,	'A',	8,	0 },
{ 398,	398,	'C',	1,	0 },
{ 399,	399,	'C',	1,	0 },
{ 400,	410,	'C',	11,	0 },  /* Line 6 */
{ 411,	420,	'A',	10,	0 },
{ 421,	429,	'C',	9,	0 },
{ 430,	439,	'A',	10,	0 },
{ 440,	453,	'C',	14,	0 },
{ 454,	459,	'A',	6,	0 },
{ 460,	472,	'C',	13,	0 },
{ 473,	478,	'D',	6,	2 },
{ 479,	479,	'C',	1,	0 },
{ 480,	502,	'C',	23,	0 },  /* Line 7 */
{ 503,	513,	'C',	11,	0 },
{ 514,	530,	'A',	17,	0 },
{ 531,	549,	'C',	19,	0 },
{ 550,	557,	'A',	8,	0 },
{ 558,	558,	'C',	1,	0 },
{ 559,	559,	'C',	1,	0 },
{ 560,	570,	'C',	10,	0 },  /* Line 8 */
{ 571,	580,	'A',	10,	0 },
{ 581,	589,	'C',	9,	0 },
{ 590,	599,	'A',	10,	0 },
{ 600,	613,	'C',	14,	0 },
{ 614,	619,	'A',	6,	0 },
{ 620,	632,	'C',	13,	0 },
{ 633,	638,	'D',	6,	2 },
{ 639,	639,	'C',	1,	0 },
{ 640,	653,	'C',	14,	0 },  /* Line 9 */
{ 654,	671,	'A',	18,	0 },
{ 672,	686,	'C',	15,	0 },
{ 687,	696,	'A',	10,	0 },
{ 697,	718,	'C',	22,	0 },
{ 719,	719,	'C',	1,	0 },
{ 720,	739,	'C',	20,	0 },  /* Line 10 */
{ 740,	750,	'A',	11,	0 },
{ 751,	763,	'C',	13,	0 },
{ 764,	765,	'A',	2,	0 },
{ 766,	798,	'C',	33,	0 },
{ 799,	799,	'C',	1,	0 },
{ 800,	818,	'C',	19,	0 },  /* Line 11 */
{ 819,	820,	'I',	2,	0 },
{ 821,	821,	'C',	1,	0 },
{ 822,	823,	'I',	2,	0 },
{ 824,	841,	'C',	18,	0 },
{ 842,	846,	'I',	5,	0 },
{ 847,	863,	'C',	17,	0 },
{ 864,	868,	'I',	5,	0 },
{ 869,	869,	'C',	1,	0 },
{ 870,	874,	'I',	5,	0 },
{ 875,	878,	'C',	4,	0 },
{ 879,	879,	'C',	1,	0 },
{ 880,	893,	'C',	14,	0 },  /* Line 12 */
{ 894,	898,	'I',	5,	0 },
{ 899,	916,	'C',	18,	0 },
{ 917,	918,	'I',	2,	0 },
{ 919,	930,	'C',	12,	0 },
{ 931,	939,	'I',	9,	0 },
{ 940,	952,	'C',	13,	0 },
{ 953,	958,	'D',	6,	2 },
{ 959,	959,	'C',	1,	0 },
{ 960,	982,	'C',	23,	0 },  /* Line 13 */
{ 983,	984,	'I',	2,	0 },
{ 985,	1010,	'C',	26,	0 },
{ 1011,	1012,	'I',	2,	0 },
{ 1013,	1038,	'C',	26,	0 },
{ 1039,	1039,	'C',	1,	0 },
{ 1040,	1054,	'C',	15,	0 },  /* Line 14 */
{ 1055,	1086,	'A',	32,	0 },
{ 1087,	1118,	'C',	32,	0 },
{ 1119,	1119,	'C',	1,	0 },
{ 1120,	1129,	'C',	10,	0 },  /* Line 15 */
{ 1130,	1158,	'A',	29,	0 },
{ 1159,	1168,	'C',	10,	0 },
{ 1169,	1197,	'A',	29,	0 },
{ 1198,	1198,	'C',	1,	0 },
{ 1199,	1199,	'C',	1,	0 },
{ 1200,	1209,	'C',	10,	0 },  /* Line 16 */
{ 1210,	1238,	'A',	29,	0 },
{ 1239,	1248,	'C',	10,	0 },
{ 1249,	1277,	'A',	29,	0 },
{ 1278, 1278,	'C',	1,	0 },
{ 1279,	1279,	'C',	1,	0 },
{ 1280,	1289,	'C',	10,	0 },  /* Line 17 */
{ 1290,	1318,	'A',	29,	0 },
{ 1319,	1328,	'C',	10,	0 },
{ 1329,	1357,	'A',	29,	0 },
{ 1358,	1358,	'C',	1,	0 },
{ 1359,	1359,	'C',	1,	0 },
{ 1360,	1438,	'C',	79,	0 },  /* Line 18 */
{ 1439,	1439,	'C',	1,	0 },
{ 1440,	1518,	'C',	79,	0 },  /* Line 19 */
{ 1519,	1519,	'C',	1,	0 },
{ 1520,	1531,	'C',	12,	0 },  /* Line 20 */
{ 1532,	1534,	'A',	3,	0 },
{ 1535,	1535,	'C',	1,	0 },
 
/* Radiometric Record */
{ 1536+0,	1536+49,	'C',	50,	0 },  /* Rad Record (Line 1) */
{ 1536+50,	1536+78,	'C',	29,	0 },
{ 1536+79,	1536+79,	'C',	1,	0 },
{ 1536+80,	1536+103,	'D',	24,	15 }, /* Line 2 */
{ 1536+104,	1536+104,	'C',	1,	0 },
{ 1536+105,	1536+128,	'D',	24,	15 },
{ 1536+129,	1536+158,	'C',	30,	0 },
{ 1536+159,	1536+159,	'C',	1,	0 },
{ 1536+160,	1536+183,	'D',    24,     15 }, /* Line 3 */
{ 1536+184,	1536+184,	'C',    1,      0 },
{ 1536+185,	1536+208,	'D',    24,     15 },
{ 1536+209,	1536+238,	'C',    30,     0 },
{ 1536+239,	1536+239,	'C',    1,      0 },
{ 1536+240,	1536+263,	'D',    24,     15 }, /* Line 4 */
{ 1536+264,	1536+264,	'C',    1,      0 },
{ 1536+265,	1536+288,	'D',    24,     15 },
{ 1536+289,	1536+318,	'C',    30,     0 },
{ 1536+319,	1536+319,	'C',    1,      0 },
{ 1536+320,	1536+343,	'D',    24,     15 }, /* Line 5 */
{ 1536+344,	1536+344,	'C',    1,      0 },
{ 1536+345,	1536+368,	'D',    24,     15 },
{ 1536+369,	1536+398,	'C',    30,     0 },
{ 1536+399,	1536+399,	'C',    1,      0 },
{ 1536+400,	1536+423,	'D',    24,     15 }, /* Line 6 */
{ 1536+424,	1536+424,	'C',    1,      0 },
{ 1536+425,	1536+448,	'D',    24,     15 },
{ 1536+449,	1536+478,	'C',    30,     0 },
{ 1536+479,	1536+479,	'C',    1,      0 },
{ 1536+480,	1536+503,	'D',    24,     15 }, /* Line 7 */
{ 1536+504,	1536+504,	'C',    1,      0 },
{ 1536+505,	1536+528,	'D',    24,     15 },
{ 1536+529,	1536+558,	'C',    30,     0 },
{ 1536+559,	1536+559,	'C',    1,      0 },
{ 1536+560,	1536+583,	'D',    24,     15 }, /* Line 8 */
{ 1536+584,	1536+584,	'C',    1,      0 },
{ 1536+585,	1536+608,	'D',    24,     15 },
{ 1536+609,	1536+638,	'C',    30,     0 },
{ 1536+639,	1536+639,	'C',    1,      0 },
{ 1536+640,	1536+663,	'D',    24,     15 }, /* Line 9 */
{ 1536+664,	1536+664,	'C',    1,      0 },
{ 1536+665,	1536+688,	'D',    24,     15 },
{ 1536+689,	1536+718,	'C',    30,     0 },
{ 1536+719,	1536+719,	'C',    1,      0 },
{ 1536+720,	1536+798,	'C',	79,	0 },  /* Line 10 */
{ 1536+799,	1536+799,	'C',	1,	0 },
{ 1536+800,	1536+878,	'C',	79,	0 },  /* Line 11 */
{ 1536+879,	1536+879,	'C',	1,	0 },
{ 1536+880,	1536+958,	'C',    79,     0 },  /* Line 12 */
{ 1536+959,	1536+959,	'C',    1,      0 },
{ 1536+960,	1536+1038,	'C',    79,     0 },  /* Line 13 */
{ 1536+1039,	1536+1039,	'C',    1,      0 },
{ 1536+1040,	1536+1118,	'C',    79,     0 },  /* Line 14 */
{ 1536+1119,	1536+1119,	'C',    1,      0 },
{ 1536+1120,	1536+1198,	'C',    79,     0 },  /* Line 15 */
{ 1536+1199,	1536+1199,	'C',    1,      0 },
{ 1536+1200,	1536+1278,	'C',    79,     0 },  /* Line 16 */
{ 1536+1279,	1536+1279,	'C',    1,      0 },
{ 1536+1280,	1536+1358,	'C',    79,     0 },  /* Line 17 */
{ 1536+1359,	1536+1359,	'C',    1,      0 },
{ 1536+1360,	1536+1438,	'C',    79,     0 },  /* Line 18 */
{ 1536+1439,	1536+1439,	'C',    1,      0 },
{ 1536+1440,	1536+1518,	'C',    79,     0 },  /* Line 19 */
{ 1536+1519,	1536+1519,	'C',    1,      0 },
{ 1536+1520,	1536+1534,	'C',    15,     0 },  /* Line 20 */
{ 1536+1535,	1536+1535,	'C',    1,      0 },

/* Note the USGS projection parameters are separated by blank spaces and
 * sometimes endl characters, thus for this header these values need to be
 * read in separately rather than the USGS projection parms (G).  In addition,
 * these values will need to be converted to the old LAS format, which is
 * done in ASCII, thus we will read them as ASCII characters and convert them
 * to doubles later.  (See process_hdr.c)
 */

/* Geometric Record */
{ 3072+0,	3072+13,	'C',    14,     0 },  /* Geom Record (Line 1) */
{ 3072+14,	3072+30,	'C',	17,	0 },
{ 3072+31,	3072+34,	'A',	4,	0 },
{ 3072+35,	3072+46,	'C',	12,	0 },
{ 3072+47,	3072+64,	'A',	18,	0 },
{ 3072+65,	3072+72,	'C',	8,	0 },
{ 3072+73,	3072+78,	'A',	6,	0 },
{ 3072+79,	3072+79,	'C',	1,	0 },
{ 3072+80,	3072+107,	'C',	28,	0 },  /* Line 2 */
{ 3072+108,	3072+108,	'C',	1,	0 },
{ 3072+109,	3072+132,	'D',	24,	15 },	
{ 3072+133,	3072+133,	'C',	1,	0 },
{ 3072+134,	3072+157,	'D',    24,	15 },   
{ 3072+158,	3072+158,	'C',    1,      0 },
{ 3072+159,	3072+159,	'C',	1,	0 },
{ 3072+160,	3072+183,	'D',    24,	15 }, /* Line 3 */
{ 3072+184,	3072+184,	'C',    1,      0 },
{ 3072+185,	3072+208,	'D',    24,	15 },    
{ 3072+209,	3072+209,	'C',    1,      0 },
{ 3072+210,	3072+233,	'D',    24,	15 },   
{ 3072+234,	3072+238,	'C',    5,      0 },
{ 3072+239,	3072+239,	'C',    1,      0 },
{ 3072+240,	3072+263,	'D',    24,	15 }, /* Line 4 */
{ 3072+264,	3072+264,	'C',    1,      0 },
{ 3072+265,	3072+288,	'D',    24,	15 },    
{ 3072+289,	3072+289,	'C',    1,      0 },
{ 3072+290,	3072+313,	'D',    24,	15 },   
{ 3072+314,	3072+318,	'C',    5,      0 },
{ 3072+319,	3072+319,	'C',    1,      0 },
{ 3072+320,	3072+343,	'D',    24,	15 }, /* Line 5 */
{ 3072+344,	3072+344,	'C',    1,      0 },
{ 3072+345,	3072+368,	'D',    24,	15 },    
{ 3072+369,	3072+369,	'C',    1,      0 },
{ 3072+370,	3072+393,	'D',    24,	15 },   
{ 3072+394,	3072+398,	'C',    5,      0 },
{ 3072+399,	3072+399,	'C',    1,      0 },
{ 3072+400,	3072+423,	'D',    24,	15 }, /* Line 6 */
{ 3072+424,	3072+424,	'C',    1,      0 },
{ 3072+425,	3072+448,	'D',    24,	15 },    
{ 3072+449,	3072+449,	'C',    1,      0 },
{ 3072+450,	3072+473,	'D',    24,	15 },   
{ 3072+474,	3072+478,	'C',    5,      0 },
{ 3072+479,	3072+479,	'C',    1,      0 },
{ 3072+480,	3072+503,	'D',    24,	15 }, /* Line 7 */
{ 3072+504,	3072+504,	'C',    1,      0 },
{ 3072+505,	3072+519,	'C',	15,	0 },
{ 3072+520,	3072+525,	'I',	6,	0 },  /* Line 8 */
{ 3072+526,	3072+558,	'C',	33,	0 },
{ 3072+559,	3072+559,	'C',	1,	0 },
{ 3072+560,	3072+563,	'C',	4,	0 },  /* Line 8 */
{ 3072+564,	3072+564,	'C',	1,	0 },
{ 3072+565,	3072+577,	'L',	13,	0 },
{ 3072+578,	3072+578,	'C',	1,	0 },
{ 3072+579,	3072+590,	'l',	12,	0 },
{ 3072+591,	3072+591,	'C',	1,	0 },
{ 3072+592,	3072+604,	'D',	13,	3 },
{ 3072+605,	3072+605,	'C',	1,	0 },
{ 3072+606,	3072+618,	'D',	13,	3 },
{ 3072+619,	3072+638,	'C',	20,	0 },
{ 3072+639,	3072+639,	'C',	1,	0 },
{ 3072+640,	3072+643,	'C',	4,	0 },  /* Line 9 */
{ 3072+644,	3072+644,	'C',	1,	0 },
{ 3072+645,	3072+657,	'L',	13,	0 },
{ 3072+658,	3072+658,	'C',	1,	0 },
{ 3072+659,	3072+670,	'l',	12,	0 },
{ 3072+671,	3072+671,	'C',	1,	0 },
{ 3072+672,	3072+684,	'D',	13,	3 },
{ 3072+685,	3072+685,	'C',	1,	0 },
{ 3072+686,	3072+698,	'D',	13,	3 },
{ 3072+699,	3072+718,	'C',	20,	0 },
{ 3072+719,	3072+719,	'C',	1,	0 },
{ 3072+720,	3072+723,	'C',	4,	0 },  /* Line 10 */
{ 3072+724,	3072+724,	'C',	1,	0 },
{ 3072+725,	3072+737,	'L',	13,	0 },
{ 3072+738,	3072+738,	'C',	1,	0 },
{ 3072+739,	3072+750,	'l',	12,	0 },
{ 3072+751,	3072+751,	'C',	1,	0 },
{ 3072+752,	3072+764,	'D',	13,	3 },
{ 3072+765,	3072+765,	'C',	1,	0 },
{ 3072+766,	3072+778,	'D',	13,	3 },
{ 3072+779,	3072+798,	'C',	20,	0 },
{ 3072+799,	3072+799,	'C',	1,	0 },
{ 3072+800,	3072+803,	'C',	4,	0 },  /* Line 11 */
{ 3072+804,	3072+804,	'C',	1,	0 },
{ 3072+805,	3072+817,	'L',	13,	0 },
{ 3072+818,	3072+818,	'C',	1,	0 },
{ 3072+819,	3072+830,	'l',	12,	0 },
{ 3072+831,	3072+831,	'C',	1,	0 },
{ 3072+832,	3072+844,	'D',	13,	3 },
{ 3072+845,	3072+845,	'C',	1,	0 },
{ 3072+846,	3072+858,	'D',	13,	3 },
{ 3072+859,	3072+878,	'C',	20,	0 },
{ 3072+879,	3072+879,	'C',	1,	0 },
{ 3072+880,	3072+887,	'C',	8,	0 },  /* Line 12 */
{ 3072+888,	3072+888,	'C',	1,	0 },
{ 3072+889,	3072+901,	'L',	13,	0 },
{ 3072+902,	3072+902,	'C',	1,	0 },
{ 3072+903,	3072+914,	'l',	12,	0 },
{ 3072+915,	3072+915,	'C',	1,	0 },
{ 3072+916,	3072+928,	'D',	13,	3 },
{ 3072+929,	3072+929,	'C',	1,	0 },
{ 3072+930,	3072+942,	'D',	13,	3 },
{ 3072+943,	3072+943,	'C',	1,	0 },
{ 3072+944,	3072+948,	'I',	5,	0 },
{ 3072+949,	3072+949,	'C',	1,	0 },
{ 3072+950,	3072+954,	'I',	5,	0 },
{ 3072+955,	3072+958,	'C',	4,	0 },
{ 3072+959,	3072+959,	'C',	1,	0 },
{ 3072+960,	3072+967,	'C',	8,	0 },  /* Line 13 */
{ 3072+968,	3072+973,	'I',	6,	0 },
{ 3072+974,	3072+993,	'C',	20,	0 },
{ 3072+994,	3072+999,	'D',	6,	2 },
{ 3072+1000,	3072+1038,	'C',	39,	0 },
{ 3072+1039,	3072+1039,	'C',	1,	0 },
{ 3072+1040,	3072+1060,	'C',	21,	0 },  /* Line 14 */
{ 3072+1061,	3072+1064,	'D',	4,	1 },
{ 3072+1065,	3072+1084,	'C',	20,	0 },
{ 3072+1085,	3072+1089,	'D',	5,	1 },
{ 3072+1090,	3072+1118,	'C',	29,	0 },
{ 3072+1119,	3072+1119,	'C',	1,	0 },
{ 3072+1120,	3072+1198,	'C',	79,	0 },  /* Line 15 */
{ 3072+1199,	3072+1199,	'C',	1,	0 },
{ 3072+1200,	3072+1278,	'C',	79,	0 },  /* Line 16 */
{ 3072+1279,	3072+1279,	'C',	1,	0 },
{ 3072+1280,	3072+1358,	'C',	79,	0 },  /* Line 17 */
{ 3072+1359,	3072+1359,	'C',	1,	0 },
{ 3072+1360,	3072+1438,	'C',	79,	0 },  /* Line 18 */
{ 3072+1439,	3072+1439,	'C',	1,	0 },
{ 3072+1440,	3072+1518,	'C',	79,	0 },  /* Line 19 */
{ 3072+1519,	3072+1519,	'C',	1,	0 },
{ 3072+1520,	3072+1534,	'C',	15,	0 },  /* Line 20 */
{ 3072+1535,	3072+1535,	'C',	1,	0 }};

static long fastfmt_fields[NUMFIELDS][5] = {
{ 0,	8,	'C',	9,	0 },
{ 9,	19,	'A',	11,	0 },
{ 20,	25,	'C',	6,	0 },
{ 26,	34,	'A',	9,	0 },
{ 35,	53,	'C',	19,	0 },
{ 54,	61,	'A',	8,	0 },
{ 62,	73,	'C',	12,	0 },
{ 74,	75,	'A',	2,	0 },
{ 76,	88,	'C',	13,	0 },
{ 89,	92,	'A',	4,	0 },
{ 93,	107,	'C',	15,	0 },
{ 108,	121,	'A',	14,	0 },
{ 122,	136,	'C',	15,	0 },
{ 137,	146,	'A',	10,	0 },
{ 147,	224,	'A',	78,	0 },
{ 225,	254,	'C',	30,	0 },
{ 255,	264,	'A',	10,	0 },
{ 265,	277,	'C',	13,	0 },
{ 278,	279,	'A',	2,	0 },
{ 280,	299,	'C',	20,	0 },
{ 300,	315,	'A',	16,	0 },
{ 316,	316,	'C',	1,	0 },
{ 317,	332,	'A',	16,	0 },
{ 333,	333,	'C',	1,	0 },
{ 334,	349,	'A',	16,	0 },
{ 350,	350,	'C',	1,	0 },
{ 351,	366,	'A',	16,	0 },
{ 367,	367,	'C',	1,	0 },
{ 368,	383,	'A',	16,	0 },
{ 384,	384,	'C',	1,	0 },
{ 385,	400,	'A',	16,	0 },
{ 401,	401,	'C',	1,	0 },
{ 402,	417,	'A',	16,	0 },
{ 418,	437,	'C',	20,	0 },
{ 438,	440,	'A',	3,	0 },
{ 441,	454,	'C',	14,	0 },
{ 455,	459,	'I',	5,	0 },
{ 460,	474,	'C',	15,	0 },
{ 475,	479,	'I',	5,	0 },
{ 480,	493,	'C',	14,	0 },
{ 494,	499,	'D',	6,	2 },
{ 500,	512,	'C',	13,	0 },
{ 513,	516,	'A',	4,	0 },
{ 517,	536,	'C',	20,	0 },
{ 537,	542,	'I',	6,	0 },
{ 543,	558,	'C',	16,	0 },
{ 559,	564,	'I',	6,	0 },
{ 565,	593,	'C',	29,	0 },
{ 594,	953,	'G',	24,	15 },
{ 954,	971,	'C',	18,	0 },
{ 972,	991,	'A',	20,	0 },
{ 992,	1009,	'C',	18,	0 },
{ 1010,	1020,	'D',	11,	3 },
{ 1021,	1038,	'C',	18,	0 },
{ 1039,	1049,	'D',	11,	3 },
{ 1050,	1062,	'A',	13,	0 },
{ 1063,	1067,	'D',	5,	2 },
{ 1068,	1084,	'C',	17,	0 },
{ 1085,	1089,	'I',	5,	0 },
{ 1090,	1106,	'C',	17,	0 },
{ 1107,	1111,	'I',	5,	0 },
{ 1112,	1115,	'C',	4,	0 },
{ 1116,	1128,	'L',	13,	0 },
{ 1129,	1129,	'C',	1,	0 },
{ 1130,	1141,	'l',	12,	0 },
{ 1142,	1142,	'C',	1,	0 },
{ 1143,	1155,	'D',	13,	3 },
{ 1156,	1156,	'C',	1,	0 },
{ 1157,	1169,	'D',	13,	3 },
{ 1170,	1173,	'C',	4,	0 },
{ 1174,	1186,	'L',	13,	0 },
{ 1187,	1187,	'C',	1,	0 },
{ 1188,	1199,	'l',	12,	0 },
{ 1200,	1200,	'C',	1,	0 },
{ 1201,	1213,	'D',	13,	3 },
{ 1214,	1214,	'C',	1,	0 },
{ 1215,	1227,	'D',	13,	3 },
{ 1228,	1231,	'C',	4,	0 },
{ 1232,	1244,	'L',	13,	0 },
{ 1245,	1245,	'C',	1,	0 },
{ 1246,	1257,	'l',	12,	0 },
{ 1258,	1258,	'C',	1,	0 },
{ 1259,	1271,	'D',	13,	3 },
{ 1272,	1272,	'C',	1,	0 },
{ 1273,	1285,	'D',	13,	3 },
{ 1286,	1289,	'C',	4,	0 },
{ 1290,	1302,	'L',	13,	0 },
{ 1303,	1303,	'C',	1,	0 },
{ 1304,	1315,	'l',	12,	0 },
{ 1316,	1316,	'C',	1,	0 },
{ 1317,	1329,	'D',	13,	3 },
{ 1330,	1330,	'C',	1,	0 },
{ 1331,	1343,	'D',	13,	3 },
{ 1344,	1359,	'C',	16,	0 },
{ 1360,	1366,	'A',	7,	0 },
{ 1367,	1384,	'C',	18,	0 },
{ 1385,	1388,	'I',	4,	0 },
{ 1389,	1404,	'C',	16,	0 },
{ 1405,	1409,	'I',	5,	0 },
{ 1410,	1425,	'C',	16,	0 },
{ 1426,	1427,	'I',	2,	0 },
{ 1428,	1441,	'C',	14,	0 },
{ 1442,	1444,	'I',	3,	0 },
{ 1445,	1452,	'C',	8,	0 },
{ 1453,	1465,	'L',	13,	0 },
{ 1466,	1466,	'C',	1,	0 },
{ 1467,	1478,	'l',	12,	0 },
{ 1479,	1479,	'C',	1,	0 },
{ 1480,	1492,	'D',	13,	3 },
{ 1493,	1493,	'C',	1,	0 },
{ 1494,	1506,	'D',	13,	3 },
{ 1507,	1512,	'I',	6,	0 },
{ 1513,	1518,	'I',	6,	0 },
{ 1519,	1526,	'C',	8,	0 },
{ 1527,	1530,	'I',	4,	0 },
{ 1531,	1534,	'C',	4,	0 },
{ 1535,	1535,	'A',	1,	0 },
};


/******************************************************************************

NAME:c_fhdfld_fst

PURPOSE: Extract information about a field from the fastfmt_fields or
  fastL7_fields array.

PROGRAM HISTORY:
  Version       Date       Author           Request
  -------       ----       ------           -------
    1.0         12/92      K.Sheffield      original development
    7.2         04/99      G.Schmidt        Updated to support Fast-L7A format

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
        Runs under TAE

PROJECT: Landsat Pathfinder

ALGORITHM DESCRIPTION:
    Determine if the header is a Fast-A/B/C, Fast-L7A, or Space Imaging Fast-C
      Format type.
    Check to see if the field referenced is a valid one.
    Copy the data.

NOTE: This is the same as the c_fhdfld routine, but it has been updated to
      work with Fast-L7A and Space Imaging Fast-C headers.
******************************************************************************/

int c_fhdfld_fst(
    long field,         /* field to get information about (one-based)  */
    long fdata[5],      /* array to fill with the information          */
    char *hdrtype)      /* type of header to read                      */
                        /* valid types are A, B, C, L7A, SIC           */
{
   long one=1;
   long *fptr;
   long slen;

   /* determine which type of header we are reading
      --------------------------------------------- */
   slen = (long) strlen (hdrtype);
   c_low2up (hdrtype, &slen);
   if (!strcmp (hdrtype, "A") || !strcmp (hdrtype, "B") ||
       !strcmp (hdrtype, "C"))
   {
      /* check to see if the field is ok
         ------------------------------- */
      if(field < 1 || field > NUMFIELDS)
      {
         c_errmsg("Invalid field reference","fhdfld-field",&one);
         return(E_FAIL);
      }

      /* transfer the data
         ------------------ */
      fptr = &fastfmt_fields[field-1][0];
   }
   else if (!strcmp (hdrtype, "L7A") || !strcmp (hdrtype, "SIC"))
   {
      /* check to see if the field is ok
         ------------------------------- */
      if (field < 1 ||
          field > NUM_ADMIN_FIELDS+NUM_RAD_FIELDS+NUM_GEOM_FIELDS)
      {
         c_errmsg("Invalid field reference","fhdfld-field",&one);
         return(E_FAIL);
      }

      /* transfer the data
         ------------------ */
      fptr = &fastL7_fields[field-1][0];
   }
   else
   {
      c_errmsg("Invalid header type", "fhdfld-hdr", &one);
      return (E_FAIL);
   }

   /* transfer the data
      ----------------- */
   fdata[0] = fptr[0];
   fdata[1] = fptr[1];
   fdata[2] = fptr[2];
   fdata[3] = fptr[3];
   fdata[4] = fptr[4];

   return(E_SUCC);
}
