/******************************************************************************

MODULE:		METADATA

PURPOSE:	To provide a simple, standardized way of writing out metadata
		files.  This approach allows programmers to insert fields into
		the metadata file in any random order they wish, writing out
		the file whenever it is necessary.  It also allows the format
		of the metadata file to be changed (for example, additional
		fields added) without breaking old code, since an enumeration
		is provided to reference the distinct fields mnemonically.

NOTES:		The metadata fields are listed in order in metadata.h.  If
		fields are added, existing code only needs to be recompiled
		to conform to a new format/order (so long as the code uses
		the mnemonic names and not numerical field references.)

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
1.0      6/96   C. Engebretson   Original development

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "metadata.h"
#include "header.h"


/* Implementation data for the module
-------------------------------------*/
static char *metadata[FIELDS];		/* Actual metadata fields	*/	
static FILE *fp;			/* General-purpose file pointer */
static char *filename;			/* Filename for .scene file	*/
static char *arfilename;		/* Filename for .archv file	*/
static long needwrite;			/* Does metadata need writing?	*/


/* meta_init(): Nullify all metadata fields and copy over the name of the
   initial output metadata file.  This must be the first function called.
-----------------------------------------------------------------------------*/
int meta_init
(
    char *file
)
{
	int i;

	/* Nullify all fields in the metadata structure.  A nulled field
	   will signify that meta_write() should insert a pound sign. */

	for (i = 0; i < FIELDS; i++) metadata[i] = NULL;
	needwrite = 0;

	/* Given the root filename, allocate memory for and copy in new
	   filenames with .scene and .archv suffixes added. */

	filename = (char *) malloc (strlen(file) + 7);
	if (!filename)
		c_lasmsg(TRUE, "metadata-alloc",
			 "Error allocating dynamic memory");
	arfilename = (char *) malloc (strlen(file) + 7);
	if (!arfilename)
		c_lasmsg(TRUE, "metadata-alloc",
			 "Error allocating dynamic memory");
	sprintf(filename, "%s.scene\0", file);
	sprintf(arfilename, "%s.archv\0", file);
	return E_SUCC;
}


/* meta_insert(): Insert a new item into the metadata structure.
   Strip out leading and trailing spaces if necessary.  If an item
   already exists, deallocate the memory that it is using first.
------------------------------------------------------------------*/
int meta_insert
(
    int field,
    char *data
)
{
	char *ptr;

	/* Check validity of field number. */

        if ((field >= FIELDS) || (field < 0))
        {
                c_lasmsg(FALSE, "metadata-invalid",
                         "Invalid field specified");
                return E_FAIL;
        }

	/* If something already exists for this field, release the memory
	   that has been allocated for it. */

	if (metadata[field]) free (metadata[field]);

	/* Strip out leading and trailing whitespace, which is not needed
	   and may very well end up causing problems down the road. */

	ptr = &data[strlen(data) - 1];
	while (*ptr == 32) ptr--;
	*(++ptr) = 0;
	ptr = &data[0];
	while (*ptr == 32) ptr++;

	/* Allocate space for field, copy it, and return. */

	metadata[field] = (char *) malloc (strlen(ptr) + 1);
	if (!metadata[field])
		c_lasmsg(TRUE, "metadata-alloc",
			 "Error allocating dynamic memory");
	strcpy(metadata[field], ptr);
	needwrite = 1;
	return E_SUCC;
}


/* meta_getfield(): Returns pointer to specified field.
-------------------------------------------------------*/
char *meta_getfield
(
    int field
)
{
	if ((field >= FIELDS) || (field < 0))
	{
		c_lasmsg(FALSE, "metadata-invalid",
			 "Invalid field specified");
		return 0;
	}
	else return metadata[field];
}


/* meta_setfile(): Allows code to specify a new output file without
   altering the contents of the metadata structure.
-----------------------------------------------------------------------*/
void meta_setfile
(
    char *file
)
{
	if (needwrite) meta_write();

	/* Reallocate the filename strings to hold the new filenames and
	   copy them over. */

	filename = (char *) realloc (filename, strlen(file) + 7);
	arfilename = (char *) realloc (arfilename, strlen(file) + 7);
	if ((!filename) || (!arfilename))
		c_lasmsg(TRUE, "metadata-alloc",
			 "Error allocating dynamic memory");
	sprintf(filename, "%s.scene\0", file);
	sprintf(filename, "%s.archv\0", file);
}


/* meta_write(): Write out the metadata file.  If a field is empty
   (NULLED), use a pound sign to signify "no value."
----------------------------------------------------------------------*/
int meta_write
(void)
{
	int i;

	fp = fopen(filename, "w");
	if (!fp)
		c_lasmsg(TRUE, "metadata-write",
			 "Error opening output file %s", filename);

	/* Write out fields.  Empty fields will be filled in with a pound
	   sign, signifying that data for this field is either not available
	   or does not apply. */

	for (i = 0; i < FIELDS; i++)
		fprintf(fp, "%s|", (metadata[i] ? metadata[i] : "#"));
	fprintf(fp, "\n");
	fclose(fp);
	needwrite = 0;

	return E_SUCC;
}


/* meta_writearchv(): Writes the corresponding archive file which
   describes the tape locations and data format.  The user supplies it with
   an array of character pointers, each which point to a tape location.
   The array must be null-terminated.
---------------------------------------------------------------------------*/
void meta_writearchv
(
    char *tapeloc[],
    char *format,
    int file_number
)
{
	int numtapes = 0;
	int curtape = 1;
	char *loc;

	/* Count the number of non-nulled strings passed in "tapeloc", which
	   will be the total number of tapes. */

	while (tapeloc[numtapes]) numtapes++;

	fp = fopen(arfilename, "w");
	if (!fp)
		c_lasmsg(TRUE, "metadata-write",
			 "Error opening output file %s", arfilename);
	loc = tapeloc[0];

	/* Write tape information one at a time. */

	while (loc)
	{
		fprintf(fp, "%s|%d|%d|%d|%s|\n", loc, file_number, curtape,
						numtapes, format);
		loc = tapeloc[curtape];
		curtape++;
	}
	fclose(fp);
}


/* meta_read(): Read in a metadata file that already exists.  Any existing
   data in the metadata structure will be wiped out.
--------------------------------------------------------------------------*/
void meta_read(void)
{
	char buffer[512];		/* Should be large enough */
	char tempstr[80];
	int i;

	/* Ensure that we free() all existing fields to avoid leaks.  Note
	   that a call it meta_write() if needwrite == 1 would be pointless
	   here. */

	for (i = 0; i < FIELDS; i++)
		if (metadata[i]) 
		{
			free(metadata[i]);
			metadata[i] = NULL;
		}

	fp = fopen(filename, "r");
	if (!fp)
	c_lasmsg(TRUE, "metadata-write",
		 "Error opening input file %s", filename);

	if (!(fgets(buffer, 512, fp)))
		c_lasmsg(TRUE, "metadata-read",
			 "Error reading data from %s", filename);
	for (i = 0; i < FIELDS; i++)
	{
		strcpy(tempstr, strtok(i ? (char *)NULL : buffer, "|"));
		if (buffer[0] != '#')
		{
			metadata[i] = (char *) malloc (strlen(tempstr) + 1);
			strcpy(metadata[i], tempstr);
		}
	}
}


/* meta_cleanup(): Deallocate all memory, and write out the metadata
   file if it has not been written since the last meta_insert().
------------------------------------------------------------------------*/
int meta_cleanup
(void)
{
	int i;

	if (needwrite) meta_write();
	for (i = 0; i < FIELDS; i++)
		if (metadata[i]) free(metadata[i]);
	free(arfilename);
	free(filename);
	return E_SUCC;
}
