/******************************************************************************
FUNCTION:       c_ealloc

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original development

  2.0      12/87    B.Ailts             Used raw 'C' types

  2.1      04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with c_errmsg
                                        Renamed ealloc to c_ealloc

  2.2      05/88    B.Ailts             Replaced errh with c_errmsg
                                        Took out error.h
  2.3      08/88    B.Ailts             Based memory allocation on total number
                                        of bytes and not number of lines
  2.4      01/89    B.Ailts             Now checks to be sure the buffer is
                                        large enough for UPDATE access
  2.5      02/91    B.Ailts             Standardized error messages
  2.6      06/93    K.Sheffield         Modified to allow the user-defined
                                        buffer sizes.
  2.7      08/99    T.Beckmann          Added support for files >2GB
                        
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT:                LAS

ALGORITHM: 
Calculate the maximum number of bytes needed to hold the largest line size 
of each of the images in the FDARRAY
Make sure buffer is large enough to hold at least on line of data
If LASBUFSZ is set
   Extract the buffer size in megabytes from LASBUFSZ
   Clamp this amount to the system allocation limit and store in maxbufsize.
Else
   Set maxbufsize to MAXBUFS
   
If maxbufsize is less than the number of bytes of whole image
   Set buffer size to maxbufsize
Else
   Set buffer size to number of bytes of whole image
Allocate the number of bytes calculated above

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdlib.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <limits.h>
#include "las.h"
#include "fdesc.h"


#define MBYTE 1048576
#define MAXMEG 256              /* maximum # of megabytes to allocate */

char *c_ealloc
(
    struct FDESC *fdarray[],    /* I: file descriptor array        */
    long         *buffersz      /* O: size of user buffer pool */
)
{
   struct rlimit limit;     /* resource limit structure             */
   long i = 0;              /* counter -- current image             */
   long tempacc = IREAD;    /* temporary access                     */
   long maxbufsize;         /* maximum buffer size                  */
   long maxbytes = 0;       /* maximum number of bytes needed for a line */
   long maxnbuf = 0;
   long nbytes;             /* number of bytes needed for a line    */
   long nonfatal = 1;       /* nonfatal error code                  */
   long long temp_bufsiz;   /* temporary buffer size                */
   long total_bands = 0;    /* total bands being allocated          */
   char *bufptr;            /* buffer pointer                       */
   char *envptr;            /* pointer to the LASBUFSZ variable     */
   
   
   while (fdarray[i] != 0)
   {
      total_bands = total_bands + fdarray[i]->nbands;
      if (fdarray[i]->acc == IUPDATE)
        tempacc = IUPDATE;
      if (fdarray[i]->acc == IREAD)
        nbytes = fdarray[i]->linsiz;
      else
      {
         if (fdarray[i]->cnvlsz != 0)
           nbytes = fdarray[i]->cnvlsz;
         else
           nbytes = fdarray[i]->linsiz;
      }
                
      if (fdarray[i]->nbuf > maxnbuf)
        maxnbuf = fdarray[i]->nbuf;
      if (nbytes > maxbytes)
        maxbytes = nbytes;
      i++;
   }  /* While fdarray[i] != 0  */

/*  this line is replaced by the lines below so that images smaller than
    BLKFACTOR lines will allocate less space and the whole image of smaller
    images can all be read into memory at once -- 8-1-88 B.Ailts
*buffersz = maxbytes * total_bands * BLKFACTOR;
-------------------------------------------------------------------------*/
   temp_bufsiz = (long long)maxbytes * total_bands * fdarray[0]->nl;

   
   if((envptr = getenv("LASBUFSZ")) == NULL)
     maxbufsize = MAXBUFS;         
   else
   {
      if(1 != sscanf(envptr,"%lu",&maxbufsize))
        maxbufsize = MAXBUFS;       /* invalid value, reset to MAXBUFS */
      else
      {
        
         if(maxbufsize > MAXMEG) maxbufsize = MAXMEG;
         
         maxbufsize *= MBYTE;       /* convert to megabytes */

         if(getrlimit(RLIMIT_DATA,&limit))
           maxbufsize = MAXBUFS;   /* unable to get resource limits */
         else
         {
            /* limit the rlim_cur to a long in case it is larger on some
               architectures (this limits the buffer to 2 GB, but that 
               shouldn't be a problem) */
            if (limit.rlim_cur > LONG_MAX)
                limit.rlim_cur = LONG_MAX;
                
            /* clamp to half of the soft resource limit  */
            if(maxbufsize > limit.rlim_cur)
              maxbufsize = (long)limit.rlim_cur / 2;

            /* don't let the user allocate any less than what would normally
               be allocated */

            if(maxbufsize < MAXBUFS)  
              maxbufsize = MAXBUFS;
         }
      }
   }
   

   if ( maxbufsize < temp_bufsiz)
     *buffersz = maxbufsize;
   else
     *buffersz = (long)temp_bufsiz;

   if (tempacc == IUPDATE)
   {
      /*  When IUPDATE access is specified the buffer is split into two buffers
          Each buffer will need to hold at least 1 line of data
      ------------------------------------------------------------------------*/
      if (*buffersz < (maxbytes * total_bands * 2)) 
        *buffersz = maxbytes * total_bands * 2 * (maxnbuf + 1);
   }
   else
   {
     /*  When IUPDATE access is NOT specified the buffer will need to hold 
         at least 1 line of data
     ------------------------------------------------------------------------*/
     if (*buffersz < (maxbytes * total_bands)) 
       *buffersz = maxbytes * total_bands * (maxnbuf + 1);  
   }
   
   /* allocate the buffer
   ----------------------*/
   bufptr = (char *)malloc(*buffersz);
   if (bufptr == NULL)
     c_errmsg("Error allocating dynamic memory","ealloc-alloc",&nonfatal);
        
   return(bufptr);
}
