/******************************************************************************
FUNCTION:       c_egclse

PURPOSE         closes the group after all processing is done

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      06/85    T. Butzer           Original implementation
  1.1      03/86    T. Butzer           dealloc the buffer space used
  1.2      03/86    K. Johnson          handle multiple file groups
  5.0      12/87    B. Ailts            change include directory specifications
                                        change to raw 'C' types
                                        place bridge routines is seperate file
  5.1      04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with c_errmsg
  5.2      10/88    B. Ailts            Added statements to free dynamic
                                        memory that was used internal to imageio
  5.3      02/91    B. Ailts            Standardized error messages
  5.4      10/92    T. Mittan           Added compression option
  5.5      12/92    T. Mittan           Freed memory that was not being freed.  
            1/94    K.Gacke             Modified to read/write dal images.  TAE
                                        images can be read, but all images are
                                        written as a BSQ DAL image.
  5.6      08/99    T.Beckmann          Added support for files >2GB

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM 
        For each file
           Close the file
           If access equals update
              Free the update and conversion buffers
           else
              Free the buffer and conversion buffers
           Free the file descriptors
        Free the group descriptors
        Return

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <unistd.h>     /* close prototype */
#include "las.h"
#include "imageio.h"    /* prototype for this routine */
#include "diskio.h"     /* c_dk routine prototypes */
#include "lasfileio.h"  /* c_fwrite prototype */
#include "taeimageio.h" /* tae image i/o prototypes */
#include "fdesc.h"


static long nonfatal = 1;               /* non-fatal error code */

int c_egclse
(
    struct GDESC **gdesc /* I: group desc from egroup */
)
{
    short i;
    short j;
    int disp;
    int return_value = E_SUCC;
    
    disp = I_SAVE;
    
    /* call xiclse for each file in group */
    
    for (i=0; i<(*gdesc)->nfiles; i++)
    {
        if ((*gdesc)->fdarray[i]->compres == TRUE)
        {
            if ((*gdesc)->fdarray[i]->acc == IWRITE)
            {
                if (FSEEK((*gdesc)->fdarray[i]->fp,18,SEEK_SET) != 0)
                {
                    if (return_value == E_SUCC) /* only print error once */
                    {
                        c_errmsg("error seeking in file","eread-seek",
                                 &nonfatal);
                        return_value = E_FAIL;
                    }
                }
                else if (c_fwrite((*gdesc)->fdarray[i]->offset_table,
                      (*gdesc)->fdarray[i]->file_offset_size,
                      ((*gdesc)->fdarray[i]->nl * (*gdesc)->fdarray[i]->nbands),
                      (*gdesc)->fdarray[i]->fp) != 
                      ((*gdesc)->fdarray[i]->nl * (*gdesc)->fdarray[i]->nbands))
                {
                    if (return_value == E_SUCC) /* only print error once */
                    {
                        c_errmsg("error writing file","eread-write",&nonfatal);
                        return_value = E_FAIL;
                    }
                }
                free((*gdesc)->fdarray[i]->offset_table);
                (*gdesc)->fdarray[i]->offset_table = NULL;
                free((*gdesc)->fdarray[i]->tempbuf);
                (*gdesc)->fdarray[i]->tempbuf = NULL;
            }
            else
            {
                free((*gdesc)->fdarray[i]->foffset);
                (*gdesc)->fdarray[i]->foffset = NULL;
            }
            fclose((*gdesc)->fdarray[i]->fp);
            (*gdesc)->fdarray[i]->fp = NULL;
            c_dkfree((*gdesc)->fdarray[i]->fname);
        } 
        else if ((*gdesc)->fdarray[i]->dal == TRUE)
        {
            close((*gdesc)->fdarray[i]->dalfd);
            c_dkfree((*gdesc)->fdarray[i]->fname);
        }
        else if ((*gdesc)->fdarray[i]->tae == TRUE)
        {
            i_clse(&((*gdesc)->fdarray[i]->ifcb), disp);
            c_dkfree((*gdesc)->fdarray[i]->fname);
        }
        
        /*  Free the update and conversion buffers
        -----------------------------------------*/
        if ((*gdesc)->fdarray[i]->acc == IUPDATE)
        {
            if ((*gdesc)->fdarray[i]->flags & CONVERSION)
            {
                for (j = 0; j < (*gdesc)->fdarray[i]->nbands; j++)
                    free((*gdesc)->fdarray[i]->upbuf[j].conbuf);
            }
            free((*gdesc)->fdarray[i]->upbuf);
            (*gdesc)->fdarray[i]->upbuf = NULL;
        }  /*  if access equals UPDATE  */
        else
        {
            if ((*gdesc)->fdarray[i]->flags & CONVERSION)
            {
                for (j = 0; j < (*gdesc)->fdarray[i]->nbands; j++)
                    free((*gdesc)->fdarray[i]->buf[j].conbuf);
            }
            free((*gdesc)->fdarray[i]->buf);
            (*gdesc)->fdarray[i]->buf = NULL;
        }  /*  else access not equals UPDATE  */
        
        /*  free the file descriptors  */
        free((*gdesc)->fdarray[i]);
        (*gdesc)->fdarray[i] = NULL;
    }  /*  for gdesc->nfiles  */
    
    /*  Free the group descriptor  */
    free(*gdesc);
    *gdesc = NULL;
    
    return(return_value);
}
