/******************************************************************************
FUNCTION:       c_eopens

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original development
  2.0      12/87    B.Ailts             Placed bridge routines in seperate file
                                        Renamed from eopens to c_eopens
  2.1      04/88    D.Hollaren          Replaced newlas.h with las.h
  2.2      05/88    B.Ailts             Replaced errh with c_errmsg
                                        Changed the calling sequence of the
                                        DDR routines -- one structure
                                        Added ddr.h
                                        Removed eerror.h
                                        Removed CHKSTAT
  2.3      08/88    B.Ailts             Now lets a user specify more bands than
                                        the image has as long as the band no.
                                        is within the range of bands
  2.4      09/88    B.Ailts             filesize is now only checked for READ 
                                        access  -- not for UPDATE access
  2.5      01/89    B.Ailts             Now calculates filesize correctly in
                                        all cases
  2.6      08/89    B.Ailts             Now if zero is specified for bands and
                                        access = IREAD or IUPDATE all the bands
                                        are specified
  2.7      10/89    B.Davis             Placed the call to lused inside of a
                                        check for read access.  We don't
                                        want to change the operating system
                                        time stamp of the ddr disk file when
                                        all we are doing is reading an image
                                        for input.  (for Catalog Manager).
  2.8      09/91    B.Ailts             Standardized error messages
  2.9      05/92    T. Mittan           Check entire filesize not just
                                        requested filesize
  3        10/92    T.Mittan            Added compression option
  4        11/93    T.Mittan            Modified compression option to be 
                                        set by a LAS global     
            1/94    K.Gacke             Modified to read/write dal images. TAE
                                        images can be read, but all images are
                                        written as a BSQ DAL image.
  5.0       8/94    D.Etrheim           Added check to verify image is below
                                        system limit of 2GB.
  5.1      11/98    T.Baltzer           Added call to c_dkresv() for read access
                                        (to be able to track disk utilization).
  5.2      08/99    T.Beckmann          Added support for files >2GB

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM 
If access is read or update
   Initialize the file descriptor members
   Open the image
   Get the ddr information and place in the fdesc structure
   Check the window specifications
   Check the band specifications
   Call c_dkresv() (to be able to track disk utilization)
Else write access
   Check the band specification
   Check the options argument
   If UNIX_DISK
      Call the disk allocation software
   Initialize the DDR fields
   Retrieve the system of the system and place in the DDR system field
   Fill in the DDR fields that are needed
   Write the band indepenant DDR fields
   Initialize the band depenant fields
   Write the band depentant fields
Write the last used date and time to the DDR

ALGORITHM REFERENCES    none

NOTES:
- Buffers are allocated in 512 byte blocks because VAX I/O had to be done
  in 512 byte blocks
*******************************************************************************/

#include <string.h>
#include <limits.h>  /* INT_MAX definition */
#include <stdlib.h>  /* getenv prototype */
#include <sys/types.h>
#include <sys/stat.h>

#define COMPRES1 "COMPRES"
#include "las.h"
#include "typlim.h"
#include "diskio.h"       /* c_dk routine prototypes */
#include "util.h"         /* c_getsys prototype */
#include "datasize.h"
#include "fdesc.h"
#include "local_protos.h"

extern long compres_opt;

int c_eopens
(
    struct FDESC **fdesc,  /* I/O: file descriptor array */
    const char *fname,     /* I: host file name */
    long bands[],          /* I/O: band array */
    const long *sl,        /* I: start line of window specification */
    const long *ss,        /* I: start sample of window specification */
    long *nl,              /* I/O: number of lines in window spec */
    long *ns,              /* I/O: number of samples in window spec */
    long offset[],         /* I: band offset array */
    const long *acc,       /* I: access mode */
    long *dtype,           /* I/O: data type */
    const long *nbuf,      /* I: number of previously read lines */
    const long opt[]       /* I: option array */
)
{
    struct DDR ddr;
    struct BDDR bddr;
    register struct FDESC *fd;
    
    long dflag;
    long long filesize;
    long i;                         /* counter -- current band              */
    long nbands;
    long nonfatal = 1;              /* nonfatal error code  */
    long stati;
    long linsiz;
    
    char errtxt[ERRLEN];
    char system[DDSYLN];
    char *compres;
    struct stat statstruct;
    
    *fdesc = (struct FDESC *)calloc(1,sizeof(struct FDESC));
    if (*fdesc == NULL)
    {
        c_errmsg("Error allocating dynamic memory","eopens-alloc",&nonfatal);
        return(E_FAIL);
    }
    
    fd = *fdesc;    /* used to simplify expressions */
    fd->nl = fd->img_nl = *nl;
    fd->ns = fd->img_ns = *ns;
    fd->sl = *sl;
    fd->ss = *ss;
    fd->offset = offset;
    fd->acc = *acc;
    fd->method = STEP;
    fd->dtype = *dtype;
    fd->nbuf = *nbuf;
    fd->linsiz = (*ns * datasize[*dtype] + 511) & ~0x1ff;
    strcpy(fd->fname, fname);
    fd->flags = 0; /* clear flags */
    fd->cnvlsz = 0; /* set conversion line size 0 */ 
    
    if ((fd->acc == IREAD) || (fd->acc == IUPDATE))
    {
        
        /* Compare the system that the data is stored on and the system that 
           will process the data.
        --------------------------------------------------------------------*/
        stati = c_comsys(fname,&dflag);
        if (stati == E_FAIL)
        {
            c_errmsg("Error returned from comsys","eopens-call",&nonfatal);
            return(E_FAIL);
        }
        
        stati = c_getddr(fname, &ddr);
        if (stati != E_SUCC)
        {
            c_errmsg("Error returned from getddr","eopens-call", &nonfatal);
            return(E_FAIL);
        }
        
        if ((dflag == 3) || 
            ((dflag == 1) && (ddr.dtype == EREAL)) ||
            ((dflag == 2) && (ddr.dtype != EBYTE)))
        {
            c_errmsg("Data type of image is not compatible with the processor",
                     "eopens-dtype",&nonfatal);
            return(E_FAIL);
        }
        
        linsiz = (ddr.ns * datasize[ddr.dtype] + 511) & ~0x1ff;
        
        /* if a zero was specified for bands all the bands are specified
        ----------------------------------------------------------------*/
        nbands = 0;
        if (bands[0] == 0)
        {
            for (i = 1; i <= ddr.nbands; i++)
            {
                fd->bands[nbands] = i;
                nbands++;
            }
        }
        else
        {
            while (bands[nbands] != 0)
            {
                if (bands[nbands] > ddr.nbands)
                {
                    sprintf(errtxt,"More bands specified than exist for %s",
                            fd->fname);
                    c_errmsg(errtxt,"eopens-nbands",&nonfatal);
                    return(E_FAIL);
                }
                fd->bands[nbands] = bands[nbands];
                nbands++;
            }
        }
        
        fd->nbands = nbands;
        
        stati = do_open(fd);    
        if (stati != E_SUCC)
        {
            c_errmsg("Error returned from doopen","eopens-call",&nonfatal);
            return(E_FAIL);
        }
        
#ifndef LARGE_FILES
        {
            long long file_overhead = 0;
            if (fd->compres)
            {
                /* size of line offset table */
                file_overhead += ddr.nl * sizeof(int) * ddr.nbands; 
                file_overhead += 18; /* overhead at start of file */
            }

            /* large file support is not available, so make sure a file larger 
               than 2GB isn't opened to read */
            if (((long long)ddr.ns * ddr.nl * datasize[ddr.dtype] * ddr.nbands
                + file_overhead) > INT_MAX)
            {
                /* attempted to read a file > 2GB without support */
                c_errmsg("Error: attempted reading file larger than 2GB on "
                         " a system that doesn't support large files",
                         "eopens-large", &nonfatal);
                /* TBD - close opened file?  not done anywhere else... */
                return(E_FAIL);
            }
        }
#endif
        
        if (*dtype == 0)
        {
            *dtype = ddr.dtype;
            fd->dtype = ddr.dtype;
        }
        
        if (ddr.dtype != fd->dtype)
        {
            fd->flags |= CONVERSION;
            fd->conv_type = ddr.dtype;
            if ((fd->tae) || (fd->compres))
                fd->cnvlsz = (ddr.ns * datasize[ddr.dtype] + 511) & ~0x1ff;
            else if (fd->dal)
                fd->cnvlsz = ddr.ns * datasize[ddr.dtype];
        }
        if (nbands == 0)
        {
            for(i=1; i<=ddr.nbands; i++)
                fd->bands[i-1] = i;
        }
        
        if (*nl == 0)
        {
            *nl = ddr.nl;
            fd->nl = *nl;
        }
        
        if (*ns == 0)
        {
            *ns = ddr.ns;
            fd->ns = *ns;
        }
        if ((fd->tae) || (fd->compres))
        {
            fd->linsiz = (ddr.ns * datasize[ddr.dtype] + 511) & ~0x1ff;
            linsiz = (ddr.ns * datasize[ddr.dtype] + 511) & ~0x1ff;
        }
        else if (fd->dal)
        {
            fd->linsiz = ddr.ns * datasize[ddr.dtype];
            linsiz = ddr.ns * datasize[ddr.dtype];
            fd->img_nl = ddr.nl;
            fd->img_ns = ddr.ns;
        }
        
        if (check_window(fd, &ddr) != E_SUCC)
        {
            sprintf(errtxt,"Invalid window for file %s",fd->fname);
            c_errmsg(errtxt,"eopens-invalwind",&nonfatal);
            return(E_FAIL);
        }
        if (fd->compres == TRUE)
        {
            fd->mns = ddr.ns;
            if (fd->acc == IUPDATE)
            {
                c_errmsg("Compresed image can not be opened for update access",
                         "eopens-update",&nonfatal);
                fclose(fd->fp);
                return(E_FAIL);
            }
        }
        
        if ((fd->acc == IREAD) && (fd->compres != TRUE))
        {
            if (fd->tae)
            {
                if (fd->flags & CONVERSION)
                {
                    if (((ddr.ns * datasize[ddr.dtype]) % 512) == 0)
                        filesize = 512 + (fd->cnvlsz * ddr.nl * ddr.nbands);
                    else
                        filesize = 512 + (fd->cnvlsz * ddr.nl * ddr.nbands) -
                                (512 - ((ddr.ns * datasize[ddr.dtype]) % 512));
                }
                else
                {
                    if (((ddr.ns * datasize[ddr.dtype]) % 512) == 0)
                        filesize = 512 + (linsiz * ddr.nl * ddr.nbands);
                    else
                        filesize = 512 + (linsiz * ddr.nl * ddr.nbands) -
                                (512 - ((ddr.ns * datasize[ddr.dtype]) % 512));
                }
            }
            else
            {
                if (fd->flags & CONVERSION)
                    filesize = (long long)fd->cnvlsz * ddr.nl * ddr.nbands;
                else
                    filesize = (long long)linsiz * ddr.nl * ddr.nbands;
            }
            
            if (stat(fd->fname, &statstruct) != 0)
            {
                c_errmsg("Error returned from stat","eopens-stat",&nonfatal);   
                return (E_FAIL);
            }
            if (statstruct.st_size < filesize)
            {
                sprintf(errtxt, "File %s is not %lld bytes big",
                        fd->fname,filesize);
                c_errmsg(errtxt,"eopens-size",&nonfatal);
                return(E_FAIL);
            }
        }  /*  if fd->acc == IREAD  */
        
        /* For read/update access, make entry in disk reserve file (0 space
           allocation required); entry required for disk utilization tracking.
        */
        stati = c_dkresv(fd->fname, 0);
        if (stati != E_SUCC)
        {
            c_errmsg("Error returned from dkresv","eopens-resv",&nonfatal);
            return(E_FAIL);
        }
        
    }  /*  if (fd->acc == IREAD) || (fd->acc == IUPDATE)  */
    else
    {  /*  else fd->acc == IWRITE  */
    
        fd->compres = fd->tae = FALSE;
        fd->dal = TRUE;
        nbands = bands[0];
        fd->nbands = nbands;
        
        if (nbands < 1)
        {
            sprintf(errtxt, "No Bands Specified for file %s", fd->fname);
            c_errmsg(errtxt,"eopens-nobands",&nonfatal);
            return(E_FAIL);
        }
        else
        {
            for (i=0; i<nbands; i++)
                fd->bands[i] = i + 1;
        }
        if (opt[1] == COMPRES)
        {
            fd->compres = TRUE;
        }
        else if (opt[1] == NO_COMPRES)
        {
            fd->compres = FALSE;
        }
        else if (compres_opt == COMPRES)
        {
            fd->compres = TRUE;
        }
        else if (compres_opt == NO_COMPRES)
        {
            fd->compres = FALSE;
        }
        else if ((compres = getenv(COMPRES1)) != NULL)
        {
            if ((strcmp(compres,"ON") == 0) || (strcmp(compres,"on") == 0))
            {
                fd->compres = TRUE;
            }
        }

        /* allocate a buffer for temporary use when encoding files if 
           compression is enable */
        if (fd->compres)
        {
            fd->tempbuf = (unsigned char *) calloc((fd->linsiz *2),
                                                   sizeof(unsigned char));
            if (fd->tempbuf == NULL)
            {
                c_errmsg("Error allocating dynamic memory","eopenr-tempbuf",
                         &nonfatal);
                return(E_FAIL);
            }
            
            fd->foffset = (long *) calloc(fd->nbands, sizeof(long));
            if (fd->foffset == NULL)
            {
                c_errmsg("Error allocating dynamic memory","eopenr-foffset",
                         &nonfatal);
                return(E_FAIL);
            }
        }

        {
            long long file_overhead = 0;
            long long max_file_size;
 
            if (fd->compres)
            {
                file_overhead += *nl * sizeof(int);/*size of line offset table*/
                file_overhead += 18; /* overhead at start of file */
                file_overhead += *nl * sizeof(unsigned char); /* overhead for 
                                       the compressed flag on each line */
            }
            max_file_size = ((long long)*ns * *nl * datasize[*dtype] * nbands 
                            + file_overhead);
#ifndef LARGE_FILES
            /* large file support is not available, so make sure a file larger 
               than 2GB isn't opened to write */
            if (max_file_size > INT_MAX)
            {
                /* attempted to read a file > 2GB without support */
                c_errmsg("Error: attempted writing file larger than 2GB on "
                         " a system that doesn't support large files",
                         "eopens-large", &nonfatal);
                return(E_FAIL);
            }

            /* select small compressed file format */
            fd->file_offset_size = sizeof(int);
#else
            /* large files are supported, so if compression is enabled for
               this file, determine whether the large or small compression
               format should be used */
            if (fd->compres)
            {
                if (max_file_size > INT_MAX)
                {
                    /* large compression format needed */
                    fd->file_offset_size = sizeof(long long);
                }
                else
                {
                    /* small compression format can be used for less overhead */
                    fd->file_offset_size = sizeof(int);
                }
            }
#endif
        }

        /* make sure the number of lines in all the bands does not exceed
           the size of an integer.  This check is probably not a concern
           any time soon since a file with this many lines wouldn't fit on
           any current file systems */
        if (((long long)*nl * nbands) > INT_MAX)
        {
            c_errmsg("Error: number of lines in all the bands exceeds machine "
                     "limits", "eopens-size", &nonfatal);
            return (E_FAIL);
        }

        if (opt[0] != 0)
        {
            if ((opt[0] > 0) && (opt[0] < NDTYPES))     
            {
                if (opt[0] != fd->dtype)
                {
                    fd->flags |= CONVERSION;
                    fd->conv_type = opt[0];
                    if ((fd->tae) || (fd->compres))
                        fd->cnvlsz = (*ns * datasize[opt[0]] +511) & ~0x1ff;
                    else if (fd->dal)
                        fd->cnvlsz = *ns * datasize[opt[0]];
                }
            }
            else
            {
                c_errmsg("Invalid option specified","eopens-option", &nonfatal);
                return(E_FAIL);
            }
        }
        
        if (fd->dal)
        {
            fd->linsiz = *ns * datasize[*dtype];
            filesize = (long long)fd->linsiz * fd->nl * fd->nbands;
            if (opt[0] != 0 && opt[0] != 5)
            {
                if (opt[0] != fd->dtype)
                {
                    filesize = (long long)fd->cnvlsz * fd->nl * fd->nbands;
                }
            }
        }
        else
        {
            /* This brack doesn't look as if it is every used */
            filesize = 11 * 512 + (long long)fd->linsiz * fd->nl * fd->nbands;
            if (opt[0] != 0 && opt[0] != 5)
            {
                if (opt[0] != fd->dtype)
                    filesize = 11 * 512 + (long long)fd->cnvlsz * fd->nl * 
                                          fd->nbands;
            }
        }
        stati = c_dkcre(0, fd->fname, filesize, 600);
        if (stati != E_SUCC)
        {
            c_errmsg("Error returned from dkcre","eopens-call",&nonfatal);
            return(E_FAIL);
        }
        
        stati = do_open(fd);    
        if (stati != E_SUCC)
        {
            c_errmsg("Error returned from doopen","eopens-call", &nonfatal);
            return(E_FAIL);
        }
        
        c_intddr(&ddr);
        ddr.nl = fd->nl;
        ddr.ns = fd->ns;
        ddr.nbands = fd->nbands;
        if (fd->flags & CONVERSION)
            ddr.dtype = fd->conv_type;
        else
            ddr.dtype = fd->dtype;
        
        stati = c_getsys(system);
        if (stati != E_SUCC)
            c_errmsg("Error returned from getsys","eopens-call", &nonfatal);
        else
            strcpy(ddr.system,system);
        
        stati = c_putddr(fname, &ddr);  
        if (stati != E_SUCC)
        {
            c_errmsg("Error returned from putddr","eopens-call", &nonfatal);
            return(E_FAIL);
        }
        
        for (i=1; i < nbands + 1; i++)
        {
            c_intbdr(&bddr);
            bddr.bandno = i;
            stati = c_putbdr(fname, &bddr);
            if (stati != E_SUCC)
            {
                c_errmsg("Error returned from putbdr","eopens-call", &nonfatal);
                return(E_FAIL);
            }
        } /* for i < nbands + 1  */
    }
    
    if (fd->acc != IREAD)
    {
        stati = c_lused(fname);
        if (stati == E_FAIL)
        {
            c_errmsg("Error returned from lused","eopens-call", &nonfatal);
            return(E_FAIL);
        }
    }
    
#ifdef DEBUG_IMAGEIO
    pfd(fd);
#endif
    if ((fd->tae) || (fd->compres))
    {
        fd->linsiz = (ddr.ns * datasize[*dtype] + 511) & ~0x1ff;
        fd->file_linsiz = (ddr.ns * datasize[ddr.dtype] + 511) & ~0x1ff;
    }
    else if (fd->dal)
    {
        fd->linsiz = ddr.ns * datasize[*dtype];
        fd->file_linsiz = ddr.ns * datasize[ddr.dtype];
    }
    return(E_SUCC);
}

#ifdef DEBUG_IMAGEIO

FUNCTION pfd(fd)

        struct FDESC *fd;
{
    printf("nl=%d\n", fd->nl);
    printf("ns=%d\n", fd->ns);
    printf("sl=%d\n", fd->sl);
    printf("offset=%x\n", fd->offset);
    printf("acc=%d\n", fd->acc);
    printf("method=%d\n", fd->method);
    printf("dtype=%d\n", fd->dtype);
    printf("nbuf=%d\n", fd->nbuf);
    printf("linsiz=%d\n", fd->linsiz);
    
    printf("conv_type=%d\n", fd->conv_type);
    printf("cnvlsz=%d\n", fd->cnvlsz);
    printf("file = %s\n", fd->fname);
    printf("nbands=%d\n", fd->nbands);
    printf("flags=%x\n", fd->flags);
}
#endif
