/******************************************************************************
FUNCTION:       c_eread

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original development
  2.0      12/87    B.Ailts             Placed bridge routine in seperate file
                                        Changed name from eread to c_eread
  2.1      04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with c_errmsg
  2.2      05/88    B.Ailts             Replaced errh with c_errmsg
                                        Changed calling sequences of the DDR
                                        routines -- only one structure
                                        Took out eerror.h
  2.3      08/88    L. Huewe            Added argument for number of lines to
                                        be read at one time
  2.4      12/88    B. Ailts            Output an error message when the data
                                        is being clipped
  2.5       2/88    B. Ailts            Replaced the 'ceil' function -- does
                                        not work correctly
  2.6      02/91    B. Ailts            Standardized error messages
  2.7      05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  2.8      10/92    T.Mittan            Added compression option
            1/94    K.Gacke             Modified to read/write dal images.  TAE
                                        images can be read, but all images are
                                        written as a BSQ DAL image.
  2.9      08/99    T.Beckmann          Added support for files >2GB


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM 
If conversion
   Read the line into temporary buffer
   Convert the data into the output buffer
Else
   Read the line into the output buffer
Return number of samples read

ALGORITHM REFERENCES    none

NOTES:
- VAX I/O is limited to being done in 512 byte blocks.  To support this,
  the lines for an image on a VAX are grouped into 512 byte blocks.
*******************************************************************************/

#include <math.h>
#include <unistd.h>
#include "las.h"
#include "util.h"         /* c_sigblk prototype */
#include "lasfileio.h"    /* c_read prototype */
#include "taeimageio.h"   /* tae image i/o prototypes */
#include "datasize.h"
#include "fdesc.h"
#include "pixman.h"
#include "local_protos.h"

#define MAXBUF 65536

static long read_dal
(
    struct FDESC *fd,       /*  file descriptor array               */
    long *eband,            /*  band number to be read              */
    long *eline,            /*  line number to be read              */
    unsigned char *euserbuf,/*  buffer                              */
    long *nlines
);

static long nonfatal = 1;               /* non-fatal error code */

long c_eread
(
    struct FDESC **fdesc,    /* I: file descriptor array */
    long *eband,             /* I: band number to be read */
    long *eline,             /* I: line number to be read */
    unsigned char *euserbuf, /* I/O: buffer */
    long *nlines             /* I: number of lines to read */
)
{
    struct FDESC *fd;
    
    int hcode;
    
    long status;
    long line;
    long nreads;
    long index;
    long i;
    long numlines;
    long lsize;
    long count = 1;
    long count1;
    FSEEK_OFF_T sample;
    void *position_end;
    unsigned char *tempbuf;
    unsigned char *tempbuf1;
    
    short big_first = TRUE;
    short small_first = TRUE;
    
    char errtxt[ERRLEN];
    unsigned char *cbufptr, *ubufptr;
    int sig_mask;
    
    fd = *fdesc;
    if (fd->dal)
    {
        status = read_dal(fd, eband, eline, euserbuf, nlines);
        return(status);
    }
    /*
    * Process Compress or TAE Image
    */
    c_sigblk(&sig_mask);
    line = *eline + fd->sl - 1;
    
#ifdef DEBUG
    printf("read line = %d nlines = %d\n", line, *nlines);
#endif
    
    if (fd->flags & CONVERSION)
        lsize = fd->cnvlsz;
    else
        lsize = fd->linsiz;
    
    if ((fd->flags & CONVERSION) || (fd->flags & MOVEDATA))
    {
        nreads = 1;
        numlines = *nlines;
        index = 0;

        for (i=0; i < nreads; i++)
        {
            /* check compression flag   
            -----------------------*/
            if (fd->compres)
            {
                if ((position_end = calloc((fd->nl * fd->mbands),
                            fd->file_offset_size))== NULL)
                {
                    c_errmsg("Error allocation dynamic memory","c_eread-alloc",
                             &nonfatal);
                    return(E_FAIL);
                }
                sample = 18 + ((line -1) * (FSEEK_OFF_T)fd->mbands * 
                         fd->file_offset_size);
                if (FSEEK(fd->fp,sample,SEEK_SET) !=  0)
                {
                    c_errmsg("error seeking in file","eread-seek",&nonfatal);
                    return(E_FAIL);
                }
                if (c_fread(position_end,fd->file_offset_size,
                       (fd->nl * fd->mbands), fd->fp) != (fd->nl * fd->mbands))
                {
                    c_errmsg("error reading file","eread-read",&nonfatal);
                    return(E_FAIL);
                }
                
                while (count <= numlines)
                {
                    if (fd->file_offset_size == sizeof(int))
                    {
                        sample = (FSEEK_OFF_T)((int *)position_end)
                                  [((count - 1)* fd->mbands) + (*eband - 1)];
                    }
                    else
                    {
                        sample = (FSEEK_OFF_T)((long long *)position_end)
                                  [((count - 1)* fd->mbands) + (*eband - 1)];
                    }
                    if (FSEEK(fd->fp,sample,SEEK_SET) != 0)
                    {
                        c_errmsg("error seeking in file","eread-seek",
                                 &nonfatal);
                        return(E_FAIL);
                    }
                    if (c_fread(&count1,sizeof(long),1,fd->fp) != 1)
                    {
                        c_errmsg("error reading file","eread-read",&nonfatal);
                        return(E_FAIL);
                    }
                    if ((tempbuf = (unsigned char *)calloc(count1,
                                    sizeof(unsigned char))) == NULL)
                    {
                        c_errmsg("Error allocating dynamic memory",
                                 "c_eread-alloc", &nonfatal);
                        return(E_FAIL);
                    }
                    if (c_fread(tempbuf,sizeof(char),count1,fd->fp) != count1)
                    {
                        c_errmsg("error reading file","eread-read",&nonfatal);
                        return(E_FAIL);
                    }
                    if (fd->conv_type == 0)
                    {
                        cbufptr = fd->conv_buf + index;
                        decode_76(tempbuf,cbufptr,fd->mns,datasize[fd->dtype]);
                    }
                    else
                    {
                        cbufptr = fd->conv_buf + index;
                        decode_76(tempbuf,cbufptr,fd->mns,
                                  datasize[fd->conv_type]);
                    }
                    index =  (count * lsize);
                    count++;
                    free(tempbuf);
                }
                free(position_end);
            }
            else if (fd->tae)          /* tae image i/o */
            {
                i_read(&(fd->ifcb),(GENPTR)&(fd->conv_buf[index]),*eband,line,
                         numlines);
                status = i_wait(&(fd->ifcb));
                if (status != SUCCESS)
                {
                    hcode = i_herr(&(fd->ifcb));
                    sprintf(errtxt,"Xiwait error %d encountered for file %s",
                            hcode,fd->fname);
                    c_errmsg(errtxt,"eread-xiwait",&nonfatal);
                    return(E_FAIL);
                }
            }
        }
        
        
        if (fd->flags & CONVERSION)
            cbufptr = fd->conv_buf + (fd->ss - 1) * datasize[fd->conv_type];
        else
            cbufptr = fd->conv_buf + (fd->ss - 1) * datasize[fd->dtype];
        ubufptr = euserbuf;
        
        for (i=0; i < *nlines; i++)
        {
            if (fd->flags & CONVERSION)
            {
                status = c_pxconv(&fd->conv_type, &fd->dtype,cbufptr, 
                                  ubufptr, &fd->ns);
                if (status != E_SUCC)
                {
                    if (((status & TOOBIG) == TOOBIG) && (big_first))
                    {
                        sprintf(errtxt,"%s%s", "Pixel values were larger than",
                                " maximum allowed by output data type");
                        c_errmsg(errtxt,"eread-pixel",&nonfatal);
                        big_first = FALSE;
                    }
                    else if (((status & TOOSMALL)  == TOOSMALL) && small_first)
                    {
                        sprintf(errtxt,"%s%s", "Pixel values were smaller than",
                                " minimum allowed by output data type");
                        c_errmsg(errtxt,"eread-pixel",&nonfatal);
                        small_first = FALSE;
                    }
                }
            }
            else
                c_pxcopy(cbufptr, ubufptr, &fd->dtype, &fd->ns);
            
            cbufptr += lsize;
            ubufptr += (fd->ns * datasize[fd->dtype]);
        }
    }
    else
    {
        nreads = 1;
        numlines = *nlines;
        index = 0;

        for (i=0; i < nreads; i++)
        {
            /* check compression flag   
            -----------------------*/
            if (fd->compres)
            {
                if ((position_end = calloc((fd->nl * fd->mbands),
                                    fd->file_offset_size)) == NULL)
                {
                    c_errmsg("Error allocation dynamic memory","c_eread-alloc",
                             &nonfatal);
                    return(E_FAIL);
                }
                sample = 18 + (line -1) * (FSEEK_OFF_T)fd->mbands * 
                         fd->file_offset_size;
                if (FSEEK(fd->fp,sample,SEEK_SET) != 0)
                {
                    c_errmsg("error seeking in file","eread-seek",&nonfatal);
                    return(E_FAIL);
                }
                if (c_fread(position_end,fd->file_offset_size,
                       (fd->nl * fd->mbands), fd->fp) != (fd->nl * fd->mbands))
                {
                    c_errmsg("error reading file","eread-read",&nonfatal);
                    return(E_FAIL);
                }
                while (count <= numlines)
                {
                    if (fd->file_offset_size == sizeof(int))
                    {
                        sample = (FSEEK_OFF_T)((int *)position_end)
                                  [((count - 1)* fd->mbands) + (*eband - 1)];
                    }
                    else
                    {
                        sample = (FSEEK_OFF_T)((long long *)position_end)
                                  [((count - 1)* fd->mbands) + (*eband - 1)];
                    }
                    if (FSEEK(fd->fp,sample,SEEK_SET) != 0)
                    {
                        c_errmsg("error reading file","eread-read",&nonfatal);
                        return(E_FAIL);
                    }
                    if (c_fread(&count1,sizeof(long),1,fd->fp) != 1)
                    {
                        c_errmsg("error reading file","eread-read",&nonfatal);
                        return(E_FAIL);
                    }
                    if ((tempbuf = (unsigned char *)calloc(count1,
                                    sizeof(unsigned char))) == NULL)
                    {
                        c_errmsg("Error allocation dynamic memory",
                                 "c_eread-alloc", &nonfatal);
                        return(E_FAIL);
                    }
                    if ((tempbuf1 = (unsigned char *)calloc((fd->mns * 
                        datasize[fd->dtype]), sizeof(unsigned char))) == NULL)
                    {
                        c_errmsg("Error allocation dynamic memory",
                                 "c_eread-alloc", &nonfatal);
                        return(E_FAIL);
                    }
                    if (c_fread(tempbuf,sizeof(char),count1,fd->fp) != count1)
                    {
                        c_errmsg("error reading file","eread-read",&nonfatal);
                        return(E_FAIL);
                    }
                    decode_76(tempbuf,tempbuf1,fd->mns, datasize[fd->dtype]);
                    c_pxcopy(&tempbuf1[fd->ss-1],&euserbuf[index],&fd->dtype,
                             &fd->ns);
                    index =  (count * lsize);
                    count++;
                    free(tempbuf);
                    free(tempbuf1);
                }
                free(position_end);
            }
            else if (fd->tae)          /* tae image */
            {
                i_read(&(fd->ifcb), (GENPTR)&euserbuf[index], *eband, line, 
                       numlines);
                
                status = i_wait(&(fd->ifcb));
                if (status != SUCCESS)
                {
                    hcode = i_herr(&(fd->ifcb));
                    sprintf(errtxt,"Xiwait error %d encountered for file %s",
                            hcode,fd->fname);
                    c_errmsg(errtxt,"eread-xiwait",&nonfatal);
                    return(E_FAIL);
                }
            }
        }
    }
    
    c_sigunblk(&sig_mask);
    return((fd->ns * (*nlines)));
}


static long read_dal
(
    struct FDESC *fd,       /*  file descriptor array               */
    long *eband,            /*  band number to be read              */
    long *eline,            /*  line number to be read              */
    unsigned char *euserbuf,/*  buffer                              */
    long *nlines
)
{
    static short big_first = TRUE;
    static short small_first = TRUE;
    long line;
    long lsize;
    long nbyte;
    long status;
    int sig_mask;
    LSEEK_OFF_T newpos;
    char errtxt[ERRLEN];
    unsigned char *ubufptr;
    
    c_sigblk(&sig_mask);
    lsize = (fd->flags & CONVERSION) ? fd->cnvlsz : fd->linsiz;
    if (fd->flags & CONVERSION) 
    {
        ubufptr = euserbuf;
        for (line=0; line<*nlines; line++)
        {
            /*
            * Need to seek in case a window is specified.
            */
            newpos = ((LSEEK_OFF_T)(*eband - 1) * fd->img_nl * lsize) +
                     ((*eline + fd->sl + line - 2) * (LSEEK_OFF_T)lsize) + 
                     ((fd->ss - 1) * (LSEEK_OFF_T)datasize[fd->conv_type]);
            if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
            {
                perror("c_eread-seek");
                c_errmsg("error seeking dal file","eread",&nonfatal);
                return(E_FAIL);
            }
            nbyte = fd->ns * datasize[fd->conv_type];
            status = c_read(fd->dalfd,fd->conv_buf,nbyte,
                            datasize[fd->conv_type]);
            if (status < nbyte)
            {
                perror("c_eread-read");
                c_errmsg("error reading dal file","eread",&nonfatal);
                return(E_FAIL);
            }
            status = c_pxconv(&fd->conv_type,&fd->dtype,fd->conv_buf,
                              ubufptr,&fd->ns);
            if (status != E_SUCC)
            {
                if (((status & TOOBIG) == TOOBIG) && (big_first))
                {
                    sprintf(errtxt,"%s%s", "Pixel values were larger than",
                            " maximum allowed by output data type");
                    c_errmsg(errtxt,"eread-pixel",&nonfatal);
                    big_first = FALSE;
                }
                else if (((status & TOOSMALL)  == TOOSMALL) && (small_first))
                {
                    sprintf(errtxt,"%s%s", "Pixel values were smaller than",
                            " minimum allowed by output data type");
                    c_errmsg(errtxt,"eread-pixel",&nonfatal);
                    small_first = FALSE;
                }
            }
            ubufptr += (fd->ns * datasize[fd->dtype]);
        }
    }
    else if (fd->flags & MOVEDATA)
    {
        ubufptr = euserbuf;
        for (line=0; line<*nlines; line++)
        {
            newpos = ((LSEEK_OFF_T)(*eband - 1) * fd->img_nl * lsize) +
                     ((*eline + fd->sl + line - 2) * (LSEEK_OFF_T)lsize) + 
                     ((fd->ss - 1) * (LSEEK_OFF_T)datasize[fd->dtype]);
            if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
            {
                perror("c_eread-seek");
                c_errmsg("error seeking dal file","eread",&nonfatal);
                return(E_FAIL);
            }
            nbyte = fd->ns * datasize[fd->dtype];
            status = c_read(fd->dalfd,ubufptr,nbyte,datasize[fd->dtype]);
            if (status < nbyte)
            {
                perror("c_eread-read");
                c_errmsg("error reading dal file","eread",&nonfatal);
                return(E_FAIL);
            }
            ubufptr += nbyte;
        }
    }
    else
    {
        ubufptr = euserbuf;
        newpos = ((LSEEK_OFF_T)(*eband - 1) * fd->img_nl * lsize) +
                 ((*eline + fd->sl - 2) * (LSEEK_OFF_T)lsize) + 
                 ((fd->ss - 1) * (LSEEK_OFF_T)datasize[fd->dtype]);
        if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
        {
            perror("c_eread-seek");
            c_errmsg("error seeking dal file","eread",&nonfatal);
            return(E_FAIL);
        }
        nbyte = lsize * *nlines;
        status = c_read(fd->dalfd,ubufptr,nbyte,datasize[fd->dtype]);
        if (status < nbyte)
        {
            perror("c_eread-read");
            c_errmsg("error reading dal file","eread",&nonfatal);
            return(E_FAIL);
        }
    }
    c_sigunblk(&sig_mask);
    return((fd->ns * (*nlines)));
}
