/*
*************************************************************************
NAME: dim_fbnd

PURPOSE: Determine the dimensions of the 'fill boundaries' (those areas
         within the input buffer boundaries which need to be set using
         mirroring or some other fill algorithm).  The values to be
         filled are presumably not available within the image, and 
         conseqeuntly need to be filled by mirroring or whatever.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1           11/89   T. Baltzer   

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
        The following function assumes that boundary values available
        within the image window will be read from the image.
        Unavailable boundary values (values outside of the image
        window) will be filled using some fill technique.

*****************************************************************
        arguments:
        name            description
        ----            -----------
        slcb            center buffer origin line specification (input)
        sscb            center buffer origin sample specification (input)
        nlcb            number of lines in the center buffer (input)
        nscb            number of samples in the center buffer (input)
        nlb             number of lines in the boundary (input)
        nsb             number of samples in the boundary (input)
        sli             starting line of the image window (input)
        ssi             starting sample of the image window (input)
        nli             number of lines in the image window (input)
        nsi             number of samples in the image window (input)
        nltf            number of lines in top boundary to fill (output)
        nlbf            number of lines in bottom boundary to fill (output)
        nslf            number of samples in left boundary to fill (output)
        nsrf            number of samples in right boundary to fill (output)

************************************************************************
*/

#include "las.h"
#include "local_protos.h"

static void dim_fbnd
(
    long    slcb,
    long    sscb,
    long    nlcb,
    long    nscb,
    long    nlb,
    long    nsb,
    long    sli,
    long    ssi,
    long    nli,
    long    nsi,
    long    *nltf,
    long    *nlbf,
    long    *nslf,
    long    *nsrf
)
{
    
    /* Determine the index of the leftmost sample in the left boundary. */
    /* If the index is less than the starting sample of the image       */
    /* window, filling is required.  Like-wise with the top boundary.   */
    
    if (sscb - nsb < ssi)
        *nslf =  ssi - sscb + nsb;
    else
        *nslf = 0;
    
    if (slcb - nlb < sli)
        *nltf =  sli - slcb + nlb;
    else
        *nltf = 0;
    
    /* Determine the index of the rightmost sample in the right         */
    /* boundary (sscb+nscb-1+nsb).  This value may be larger than it    */
    /* should be in the event that a whole buffer with nscb samples is  */
    /* not available.  Subtract this index from the index of the right- */
    /* most index of the image window.  If this difference is greater   */
    /* than zero, some or all of the boundary lies to the right of the  */
    /* image window and needs to be filled.  If nscb samples is not     */
    /* available, this difference may exceed nsb (nsrf should never     */
    /* exceed nsb).  If this difference is not greater than zero, no    */
    /* columns in the rightmost  boundary need to be filled.  Likewise  */
    /* for the bottom boundary.                                         */
    
    *nsrf = sscb + nscb + nsb - ssi - nsi;
    if (*nsrf > 0)
    {
        if (*nsrf > nsb)
        {
            *nsrf = nsb; 
        }
    }
    else
    {
        *nsrf = 0;
    }
    
    *nlbf = slcb + nlcb + nlb - sli - nli;
    if (*nlbf > 0)
    {
        if (*nlbf > nlb)
        {
            *nlbf = nlb; 
        }
    }
    else
    {
        *nlbf = 0;
    }
    
}     /* end of dim_fbnd()   */

/*
*************************************************************************
NAME: dim_rbf

PURPOSE: determine the origin and dimensions of the 'read buffer'.  The
         'read  buffer' is that section of the input buffer which is
         read from the input image (the remainder of the input buffer
         is filled by mirroring data or some other fill algorithm).

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           11/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
         The 'read  buffer' is that section of the input buffer which
         is read from the input image (the remainder of the input buffer
         is filled by mirroring data or some other fill algorithm).
         The assumption of this algorithm is that all of the data that
         can be read from the image will be (mirroring or whatever will
         not occur unless actual data is not available).  The matter is
         complicated by the fact that 'the image' (nli x nsi) may very
         well be a window within a much larger 'total image' (nlti x nsti).
         The boundary surrounding the nli x nsi image will NOT be read in
         from the total image even though it actually exists.
         The 'center buffer' referenced below refers to the "inner" buffer
         of the input buffer (the input buffer less the boundary pixels).
         So the input buffer consists of this center buffer plus the 
         boundary pixels, where the boundary pixels exist only to help 
         determine the replacement value of the pixels within the center
         buffer.  This function assumes that the origin of the 'center
         buffer' does lie within the image window and does not error
         check for it.
         The following algorithm considers the case in which:
         (1) all, or part, or none of the boundary pixels may be read in
             from the image.
         (2) a full 'center buffer' of data exists within the image, or
             only a partial 'center buffer' of data is available (must
             be at the bottom or far right of the image window).

*****************************************************************
        arguments:
        name            description
        ----            -----------
        slcb            center buffer origin line specification (input)
        sscb            center buffer origin sample specification (input)
        nlcb            number of lines in the center buffer (input)
        nscb            number of samples in the center buffer (input)
        nlb             number of lines in the boundary (input)
        nsb             number of samples in the boundary (input)
        sli             starting line of the image window (input)
        ssi             starting sample of the image window (input)
        nli             number of lines in the image window (input)
        nsi             number of samples in the image window (input)
        slrb            read buffer origin line specification (output)
        ssrb            read buffer origin sample specification (output)
        nlrb            number of lines in the read buffer (output)
        nsrb            number of samples in the read buffer (output)

************************************************************************
*/

static void dim_rbf
(
    long    slcb,
    long    sscb,
    long    nlcb,
    long    nscb,
    long    nlb,
    long    nsb,
    long    sli,
    long    ssi,
    long    nli,
    long    nsi,
    long    *slrb,
    long    *ssrb,
    long    *nlrb,
    long    *nsrb
)
{
    
    /* Begin by locating the starting line (origin) of the read buffer. */
    /* Note that this value cannot lie outside of the user specified    */
    /* image window, since no values outside of the image window are to */
    /* be read.  Likewise with the samples.                             */
    
    if (slcb - nlb < sli )
        *slrb = sli;                     /* top edge of image window */
    else
        *slrb = slcb - nlb;              /* center buffer less boundary */
    
    if (sscb - nsb < ssi )
        *ssrb = ssi;                       /* left edge of image window */
    else
        *ssrb = sscb - nsb;              /* center buffer less boundary */
    
    
    /* Now calculate the number of lines in the read buffer.  Add the   */
    /* top boundary (if any), the center buffer, and the lower boundary.*/
    /* If this sum places one below the image window dimensions, then   */
    /* back-up to the last line within the image window.  Likewise with */
    /* the samples.                                                     */
    
    *nlrb = (slcb - *slrb) + nlcb + nlb;
    if ( *slrb + *nlrb > sli + nli )
        *nlrb = sli + nli - *slrb;
    
    *nsrb = (sscb - *ssrb) + nscb + nsb;
    if ( *ssrb + *nsrb > ssi + nsi )
        *nsrb = ssi + nsi - *ssrb;
    
}     /* end of dim_rbf()   */

/*
************************************************************************
NAME: err_get

PURPOSE: error check the input parameters to get_buf()

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1           12/89   T. Baltzer   
  2.4           02/91    B.Ailts        Standardized error messages



COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

*****************************************************************
        arguments:
name            description
----            -----------
nlbuf           total number of lines in 'buffer' (input)
nsbuf           total number of samples in 'buffer' (input)
sli             starting line of the image window (input)
ssi             starting sample of the image window (input)
nli             number of lines in the image window (input)
nsi             number of samples in the image window (input)
nlb             number of lines in the boundary (input)
nsb             number of samples in the boundary (input)
slcb            starting line of center buffer (input)
sscb            starting sample of center buffer (input)
nextflg         flag to indicate first call to get_buf(), and
                flag to indicate whether the buffer origin should be
                incremented before reading the data: (input)
                  INIT  (0)  this is the first call to get_buf() for
                             this image at this origin
                  CURNT (1)  don't move the buffer origin
                  NEXT  (2)  do move the origin to the next data block,
                             and reopen the image when appropriate
band            number of the band within the image to read (input)
fill            fill technique specification:       (input)
                   NOFIL (0)  don't do any boundary filling
                   MIRR  (1)  fill by mirroring adjoining image data
dtype           data type of buffer

************************************************************************
*/

static int err_get
(
    long    nlbuf,
    long    nsbuf,
    long    sli,
    long    ssi,
    long    nli,
    long    nsi,
    long    nlb,
    long    nsb,
    long    slcb,
    long    sscb,
    long    nextflg,
    long    band,
    long    fill,
    long    dtype
)
{
    
    long    nonfatal = 1; /* nonfatal error status           */
    
    
    /* Begin by checking the values of the individual parameters.     */
    
    
    if ( nlbuf < 1 || nsbuf < 1 )
    {
        c_errmsg("Invalid input buffer size","get_buf-bufsiz",&nonfatal);
        return(E_FAIL);
    }
    if ( sli   < 1 || ssi   < 1 || nli  < 1 || nsi  < 1 )
    {
        c_errmsg("Invalid image window specified","get_buf-wind",&nonfatal);
        return(E_FAIL);
    }
    if ( nextflg < 0 || nextflg > 2 || band < 1 ||
         fill < 0 || fill > 1 || dtype < 0 || dtype > 4)
    {
        c_errmsg("Invalid input parameter","get_buf-input",&nonfatal);
        return(E_FAIL);
    }
    if ( nlb   < 0 || nsb   < 0 || slcb < 1 || sscb < 1 )
    {
        c_errmsg("Invalid computed input parameter","get_buf-comput",&nonfatal);
        return(E_FAIL);
    }
    
    /* Error checking is NOT performed to make certain that the       */
    /* specified window lies within the total image (if this is the   */
    /* case, error messages will result when the random image i/o     */
    /* routines are called).                                          */
    
    
    /* Make sure the starting origin is inside the specified window.  */
    
    if ( slcb < sli || slcb > sli + nli - 1 ||
         sscb < ssi || sscb > ssi + nsi - 1 )
    {
        c_errmsg("Starting origon outside image window","get_buf-orig",
                 &nonfatal);
        return(E_FAIL);
    }
    
    
    /* Make sure the specified boundaries aren't so big that they take*/
    /* up the entire buffer.                                          */
    
    if ( 2 * nlb >= nlbuf || 2 * nsb >= nsbuf )
    {
        c_errmsg("Buffer too small to load full kernel","get_buf-smallb",
                 &nonfatal);
        return(E_FAIL);
    }
    
    
    /* Make sure that the specified window isn't so small that the    */
    /* boundary values cannot be filled properly.                     */
    
    if ( nli < nlb || nsi < nsb )
    {
        c_errmsg("Window too small to load full kernel","get_buf-smallw",
                 &nonfatal);
        return(E_FAIL);
    }
    
    
    return(E_SUCC);
    
}    /*  end of err_get()   */

/*
************************************************************************
NAME: mirr_byt

PURPOSE: Fill the designated boundary pixels by mirroring the
         appropriate image data (where the buffer of data is of
         type CHAR).


PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           11/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS


*****************************************************************
        arguments:
        name    description
        ----    -----------
        buf     buffer of data under consideration (input/output)
        nlav    number of actual valid lines in buf (input)
        nsav    number of actual valid samples in buf (input)
        nsbuf   number of total samples in buf (input)
        nltf    number of lines in top boundary to fill (input)
        nlbf    number of lines in bottom boundary to fill (input)
        nslf    number of samples in left boundary to fill (input)
        nsrf    number of samples in right boundary to fill (input)

************************************************************************
*/

static void mirr_byt
(
    unsigned char   buf[],
    long    nlav,
    long    nsav,
    long    nsbuf,
    long    nltf,
    long    nlbf,
    long    nslf,
    long    nsrf
)
{
    
    long    i;      /* index counter */
    long    j;      /* index counter */
    long    k;      /* constant variable */
    unsigned char   *froma; /* 'from address' */
    unsigned char   *toadd; /* 'to address'   */
    unsigned char   *frtmp; /* temporary address */
    unsigned char   *totmp; /* temporary address */
    
    /* Mirror the boundaries in the following randomly-choosen order:   */
    /* (1) left, (2) right, (3) top, (4) bottom                         */
    /* Pixels in the corners (wherein two boundaries lap) will be set   */
    /* only in steps (3) and (4) above.                                 */
    /* Boundary fill dimensions of zero (0) will be handled by this     */
    /* algorithm.                                                       */
    
    /* Mirror the left boundary less the corner pixels.   */
    /* In essence, buf[j][i] = buf[j][k-i], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = (nslf * 2);
    totmp = buf + (nltf * nsbuf);
    for (i = 0; i < nslf; i++)
    {
        toadd = totmp;
        froma = toadd + k - i - i;
        totmp++;
        for (j = nltf; j < nlav - nlbf; j++)
        {
            *toadd = *froma;
            toadd += nsbuf;
            froma += nsbuf;
        }
    }
    
    /* Mirror the right boundary less the corner pixels.  */
    /* In essence, buf[j][i] = buf[j][k-i], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = ((nsav - nsrf) * 2) - 2;
    totmp = buf + (nltf * nsbuf) + nsav - nsrf;
    for (i = nsav - nsrf; i < nsav; i++)
    {
        toadd = totmp;
        froma = toadd + k - i - i;
        totmp++;
        for (j = nltf; j < nlav - nlbf; j++)
        {
            *toadd = *froma;
            toadd += nsbuf;
            froma += nsbuf;
        }
    } 
    
    /* Mirror the top boundary.                           */
    /* In essence, buf[i][j] = buf[k-i][j], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = (nltf * 2);
    totmp = buf;
    frtmp = buf + (k * nsbuf);
    for (i = 0; i < nltf; i++)
    {
        toadd = totmp;
        froma = frtmp;
        totmp += nsbuf;
        frtmp -= nsbuf;
        for (j = 0; j < nsav; j++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    /* Mirror the bottom boundary.                        */
    /* In essence, buf[i][j] = buf[k-i][j], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = ((nlav - nlbf) * 2) - 2;
    totmp = buf + ((nlav - nlbf) * nsbuf);
    frtmp = buf + ((k - (nlav - nlbf)) * nsbuf);
    for (i = nlav - nlbf; i < nlav; i++)
    {
        toadd = totmp;
        froma = frtmp;
        totmp += nsbuf;
        frtmp -= nsbuf;
        for (j = 0; j < nsav; j++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    
}     /* end of mirr_byt()   */

/*
************************************************************************
NAME: mirr_flt

PURPOSE: Fill the designated boundary pixels by mirroring the
         appropriate image data (where the buffer of data is of
         type FLOAT).


PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           11/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS


*****************************************************************
        arguments:
        name    description
        ----    -----------
        buf     buffer of data under consideration (input/output)
        nlav    number of actual valid lines in buf (input)
        nsav    number of actual valid samples in buf (input)
        nsbuf   number of total samples in buf (input)
        nltf    number of lines in top boundary to fill (input)
        nlbf    number of lines in bottom boundary to fill (input)
        nslf    number of samples in left boundary to fill (input)
        nsrf    number of samples in right boundary to fill (input)

************************************************************************
*/

static void mirr_flt
(
    float   buf[],
    long    nlav,
    long    nsav,
    long    nsbuf,
    long    nltf,
    long    nlbf,
    long    nslf,
    long    nsrf
)
{
    
    long    i;      /* index counter */
    long    j;      /* index counter */
    long    k;      /* constant variable */
    float   *froma; /* 'from address' */
    float   *toadd; /* 'to address'   */
    float   *frtmp; /* temporary address */
    float   *totmp; /* temporary address */
    
    /* Mirror the boundaries in the following randomly-choosen order:   */
    /* (1) left, (2) right, (3) top, (4) bottom                         */
    /* Pixels in the corners (wherein two boundaries lap) will be set   */
    /* only in steps (3) and (4) above.                                 */
    /* Boundary fill dimensions of zero (0) will be handled by this     */
    /* algorithm.                                                       */
    
    /* Mirror the left boundary less the corner pixels.   */
    /* In essence, buf[j][i] = buf[j][k-i], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = (nslf * 2);
    totmp = buf + (nltf * nsbuf);
    for (i = 0; i < nslf; i++)
    {
        toadd = totmp;
        froma = toadd + k - i - i;
        totmp++;
        for (j = nltf; j < nlav - nlbf; j++)
        {
            *toadd = *froma;
            toadd += nsbuf;
            froma += nsbuf;
        }
    }
    
    /* Mirror the right boundary less the corner pixels.  */
    /* In essence, buf[j][i] = buf[j][k-i], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = ((nsav - nsrf) * 2) - 2;
    totmp = buf + (nltf * nsbuf) + nsav - nsrf;
    for (i = nsav - nsrf; i < nsav; i++)
    {
        toadd = totmp;
        froma = toadd + k - i - i;
        totmp++;
        for (j = nltf; j < nlav - nlbf; j++)
        {
            *toadd = *froma;
            toadd += nsbuf;
            froma += nsbuf;
        }
    } 
    
    /* Mirror the top boundary.                           */
    /* In essence, buf[i][j] = buf[k-i][j], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = (nltf * 2);
    totmp = buf;
    frtmp = buf + (k * nsbuf);
    for (i = 0; i < nltf; i++)
    {
        toadd = totmp;
        froma = frtmp;
        totmp += nsbuf;
        frtmp -= nsbuf;
        for (j = 0; j < nsav; j++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    /* Mirror the bottom boundary.                        */
    /* In essence, buf[i][j] = buf[k-i][j], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = ((nlav - nlbf) * 2) - 2;
    totmp = buf + ((nlav - nlbf) * nsbuf);
    frtmp = buf + ((k - (nlav - nlbf)) * nsbuf);
    for (i = nlav - nlbf; i < nlav; i++)
    {
        toadd = totmp;
        froma = frtmp;
        totmp += nsbuf;
        frtmp -= nsbuf;
        for (j = 0; j < nsav; j++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    
}     /* end of mirr_flt()   */

/*
************************************************************************
NAME: mirr_lng

PURPOSE: Fill the designated boundary pixels by mirroring the
         appropriate image data (where the buffer of data is of
         type LONG).


PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           11/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS


*****************************************************************
        arguments:
        name    description
        ----    -----------
        buf     buffer of data under consideration (input/output)
        nlav    number of actual valid lines in buf (input)
        nsav    number of actual valid samples in buf (input)
        nsbuf   number of total samples in buf (input)
        nltf    number of lines in top boundary to fill (input)
        nlbf    number of lines in bottom boundary to fill (input)
        nslf    number of samples in left boundary to fill (input)
        nsrf    number of samples in right boundary to fill (input)

************************************************************************
*/

static void mirr_lng
(
    long    buf[],
    long    nlav,
    long    nsav,
    long    nsbuf,
    long    nltf,
    long    nlbf,
    long    nslf,
    long    nsrf
)
{
    
    long    i;      /* index counter */
    long    j;      /* index counter */
    long    k;      /* constant variable */
    long    *froma; /* 'from address' */
    long    *toadd; /* 'to address'   */
    long    *frtmp; /* temporary address */
    long    *totmp; /* temporary address */
    
    /* Mirror the boundaries in the following randomly-choosen order:   */
    /* (1) left, (2) right, (3) top, (4) bottom                         */
    /* Pixels in the corners (wherein two boundaries lap) will be set   */
    /* only in steps (3) and (4) above.                                 */
    /* Boundary fill dimensions of zero (0) will be handled by this     */
    /* algorithm.                                                       */
    
    /* Mirror the left boundary less the corner pixels.   */
    /* In essence, buf[j][i] = buf[j][k-i], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = (nslf * 2);
    totmp = buf + (nltf * nsbuf);
    for (i = 0; i < nslf; i++)
    {
        toadd = totmp;
        froma = toadd + k - i - i;
        totmp++;
        for (j = nltf; j < nlav - nlbf; j++)
        {
            *toadd = *froma;
            toadd += nsbuf;
            froma += nsbuf;
        }
    }
    
    /* Mirror the right boundary less the corner pixels.  */
    /* In essence, buf[j][i] = buf[j][k-i], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = ((nsav - nsrf) * 2) - 2;
    totmp = buf + (nltf * nsbuf) + nsav - nsrf;
    for (i = nsav - nsrf; i < nsav; i++)
    {
        toadd = totmp;
        froma = toadd + k - i - i;
        totmp++;
        for (j = nltf; j < nlav - nlbf; j++)
        {
            *toadd = *froma;
            toadd += nsbuf;
            froma += nsbuf;
        }
    } 
    
    /* Mirror the top boundary.                           */
    /* In essence, buf[i][j] = buf[k-i][j], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = (nltf * 2);
    totmp = buf;
    frtmp = buf + (k * nsbuf);
    for (i = 0; i < nltf; i++)
    {
        toadd = totmp;
        froma = frtmp;
        totmp += nsbuf;
        frtmp -= nsbuf;
        for (j = 0; j < nsav; j++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    /* Mirror the bottom boundary.                        */
    /* In essence, buf[i][j] = buf[k-i][j], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = ((nlav - nlbf) * 2) - 2;
    totmp = buf + ((nlav - nlbf) * nsbuf);
    frtmp = buf + ((k - (nlav - nlbf)) * nsbuf);
    for (i = nlav - nlbf; i < nlav; i++)
    {
        toadd = totmp;
        froma = frtmp;
        totmp += nsbuf;
        frtmp -= nsbuf;
        for (j = 0; j < nsav; j++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    
}     /* end of mirr_lng()   */

/*
************************************************************************
NAME: mirr_wrd

PURPOSE: Fill the designated boundary pixels by mirroring the
         appropriate image data (where the buffer of data is of
         type SHORT).


PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           11/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS


*****************************************************************
        arguments:
        name    description
        ----    -----------
        buf     buffer of data under consideration (input/output)
        nlav    number of actual valid lines in buf (input)
        nsav    number of actual valid samples in buf (input)
        nsbuf   number of total samples in buf (input)
        nltf    number of lines in top boundary to fill (input)
        nlbf    number of lines in bottom boundary to fill (input)
        nslf    number of samples in left boundary to fill (input)
        nsrf    number of samples in right boundary to fill (input)

************************************************************************
*/

static void mirr_wrd
(
    short   buf[],
    long    nlav,
    long    nsav,
    long    nsbuf,
    long    nltf,
    long    nlbf,
    long    nslf,
    long    nsrf
)
{
    
    long    i;      /* index counter */
    long    j;      /* index counter */
    long    k;      /* constant variable */
    short   *froma; /* 'from address' */
    short   *toadd; /* 'to address'   */
    short   *frtmp; /* temporary address */
    short   *totmp; /* temporary address */
    
    /* Mirror the boundaries in the following randomly-choosen order:   */
    /* (1) left, (2) right, (3) top, (4) bottom                         */
    /* Pixels in the corners (wherein two boundaries lap) will be set   */
    /* only in steps (3) and (4) above.                                 */
    /* Boundary fill dimensions of zero (0) will be handled by this     */
    /* algorithm.                                                       */
    
    /* Mirror the left boundary less the corner pixels.   */
    /* In essence, buf[j][i] = buf[j][k-i], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = (nslf * 2);
    totmp = buf + (nltf * nsbuf);
    for (i = 0; i < nslf; i++)
    {
        toadd = totmp;
        froma = toadd + k - i - i;
        totmp++;
        for (j = nltf; j < nlav - nlbf; j++)
        {
            *toadd = *froma;
            toadd += nsbuf;
            froma += nsbuf;
        }
    }
    
    /* Mirror the right boundary less the corner pixels.  */
    /* In essence, buf[j][i] = buf[j][k-i], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = ((nsav - nsrf) * 2) - 2;
    totmp = buf + (nltf * nsbuf) + nsav - nsrf;
    for (i = nsav - nsrf; i < nsav; i++)
    {
        toadd = totmp;
        froma = toadd + k - i - i;
        totmp++;
        for (j = nltf; j < nlav - nlbf; j++)
        {
            *toadd = *froma;
            toadd += nsbuf;
            froma += nsbuf;
        }
    } 
    
    /* Mirror the top boundary.                           */
    /* In essence, buf[i][j] = buf[k-i][j], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = (nltf * 2);
    totmp = buf;
    frtmp = buf + (k * nsbuf);
    for (i = 0; i < nltf; i++)
    {
        toadd = totmp;
        froma = frtmp;
        totmp += nsbuf;
        frtmp -= nsbuf;
        for (j = 0; j < nsav; j++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    /* Mirror the bottom boundary.                        */
    /* In essence, buf[i][j] = buf[k-i][j], only perform  */
    /* the index arithmetic yourself.                     */
    
    k = ((nlav - nlbf) * 2) - 2;
    totmp = buf + ((nlav - nlbf) * nsbuf);
    frtmp = buf + ((k - (nlav - nlbf)) * nsbuf);
    for (i = nlav - nlbf; i < nlav; i++)
    {
        toadd = totmp;
        froma = frtmp;
        totmp += nsbuf;
        frtmp -= nsbuf;
        for (j = 0; j < nsav; j++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    
}     /* end of mirr_wrd()   */

/*
*************************************************************************
NAME: mirr_bnd

PURPOSE: Call the proper routine to fill the designated boundary pixels
         by mirroring the appropriate image data.

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

*****************************************************************
        arguments:
        name    description
        ----    -----------
        buf     buffer of data under consideration (input/output)
        dtype   data type of buf (input)
        nlav    number of actual valid lines in buf (input)
        nsav    number of actual valid samples in buf (input)
        nsbuf   number of total samples in buf (input)
        nltf    number of lines in top boundary to fill (input)
        nlbf    number of lines in bottom boundary to fill (input)
        nslf    number of samples in left boundary to fill (input)
        nsrf    number of samples in right boundary to fill (input)

************************************************************************
*/

static void mirr_bnd
(
    unsigned char   *buf,
    long    dtype,
    long    nlav,
    long    nsav,
    long    nsbuf,
    long    nltf,
    long    nlbf,
    long    nslf,
    long    nsrf
)
{
    
    switch (dtype)
    {
        case EREAL:
            mirr_flt((float*)buf,nlav,nsav,nsbuf,nltf,nlbf,nslf,nsrf);
            break;
        case ELONG:
            mirr_lng((long*)buf,nlav,nsav,nsbuf,nltf,nlbf,nslf,nsrf);
            break;
        case EWORD:
            mirr_wrd((short*)buf,nlav,nsav,nsbuf,nltf,nlbf,nslf,nsrf);
            break;
        case EBYTE:
            mirr_byt(buf,nlav,nsav,nsbuf,nltf,nlbf,nslf,nsrf);
            break;
    }
    
}     /*  end of mirr_bnd()   */

/*
*************************************************************************
NAME: next_org

PURPOSE:
        The origin can be thought of as the upper left-hand corner of
        a block of data.  An image is broken-up into blocks of data
        each having an origin.  This function is called to calculate
        the origin of the next block of data in the image.


PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           11/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
        The origin can be thought of as the upper left-hand corner of
        a block of data.  An image is broken-up into blocks of data
        each having an origin.  This function is called to calculate
        the origin of the next block of data in the image.  This
        'next origin' is derived on a column by row basis:
                ---------------------------
                |    1    |    4    |    7| 
                ---------------------------
                |    2    |    5    |    8| 
                ---------------------------
                |    3    |    6    |    9| 
                ---------------------------
        The data blocks mentioned above coinside with the 'center
        buffer' of the input buffer (the 'center buffer' is the 
        input buffer less the boundary pixels).

*****************************************************************
        arguments:
        name            description
        ----            -----------
         sli            starting line of the image window (input)
         ssi            starting sample of the image window (input)
         nli            number of lines in the image window (input)
         nsi            number of samples in the image window (input)
         nlcb           number of lines per data block (input), or
                        number of lines in the center buffer
         nscb           number of samples per data block (input), or
                        number of samples in the center buffer
         slcb           line specification of previous origin (input)
                        line specification of next origin (output)
                        (starting line of center buffer)
         sscb           sample specification of previous origin (input)
                        sample specification of next origin (output)
                        (starting sample of center buffer)

************************************************************************
*/

static int next_org
(
    long    sli,
    long    ssi,
    long    nli,
    long    nsi,
    long    nlcb,
    long    nscb,
    long    *slcb,
    long    *sscb
)
{
    
    
    if ( *slcb + nlcb <= sli + nli - 1) /* bottom of the current column? */
    {                             /* no, so                           */
        *slcb = *slcb + nlcb;         /* get next line in current column  */
    }
    else                  /* already at the bottom of the current column */ 
    {
        if ( *sscb + nscb <= ssi + nsi - 1) /* is this the last column?   */
        {                          /* no, so                           */
            *sscb = *sscb + nscb;      /* go to the top of the next column */
            *slcb = sli;
        }
        else               /* at the bottom of the last column already    */
        {
            return(E_FAIL);
        }
    }
    
    return(E_SUCC);
    
}    /*  end of next_org()   */

/*
************************************************************************
NAME: shft_byt

PURPOSE: shift the data in 'buffer' by an offset in the horizontal
         and vertical direction (where buffer is CHAR data).

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        down and to the right by the specified offsets.  The data will
        be moved in a sequence such that no data is overwritten.  Note
        that the offsets may be zero (leave the data in its current
        location).  While it is easy to imagine the data as a 2-D
        array, one must remember that the data is actually stored in
        contiguous memory locations.
        This algorithm also fixes the problem wherein the number of
        actual samples read in (nvsb) is less than the number of samples
        in the total buffer (nsbuf).  This situation is a problem
        since the read will pass back a contiguous block of data
        with no imbedded spaces left for the unread samples (thinking
        of the data as a 2-D array).

*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        nvlb            number of valid lines of data in buffer (input)
        nvsb            number of valid samples of data in buffer(input)
        loff            number of lines to offset by (input)
        soff            number of samples to offset by (input)
        nsbuf           number of samples in the total buffer (input)

************************************************************************
*/

static int shft_byt
(
    unsigned char   *buffer,
    long    nvlb,
    long    nvsb,
    long    loff,
    long    soff,
    long    nsbuf
)
{
    
    long    i;              /* index counter */
    long    j;              /* index counter */
    unsigned char   *froma;         /* 'from address' */
    unsigned char   *toadd;         /* 'to address'   */
    unsigned char   *totemp;        /* temporary address */
    
    
    /* In order to return successfully without doing anything            */
    /*    (1) there must be no line offset, and                          */
    /*    (2) there must be no sample offset, and                        */
    /*    (3) no shifting on account of 'sample padding' is neccessary.  */
    
    if (loff == 0 && soff == 0 && nvsb == nsbuf)
        return(E_SUCC);
    
    
    /* With the following indexing scheme, one can think of the counters */
    /* j and i as indexes to the 'to location' of buffer:                */
    /*      buffer[j][i] = .....    (thinking of buffer as a 2-D array)  */
    /* The 'from address' of buffer is decremented by one each time      */
    /* because the valid buffer data coming into this routine is stored  */
    /* in contiguous memory locations.                                   */
    
    froma = buffer + (nvlb * nvsb) - 1;        /* initial 'from address' */
    
    totemp = buffer + ((nvlb - 1 + loff) * nsbuf) + nvsb - 1 + soff;
    for (j = nvlb - 1; j >= 0; j-- )
    {
        toadd = totemp;
        totemp -= nsbuf;
        for (i = nvsb - 1; i >= 0; i-- )
        {
            *toadd = *froma;
            froma--;
            toadd--;
        }
    }
    
    return(E_SUCC);
    
}     /*  end of shft_byt()   */

/*
************************************************************************
NAME: shft_flt

PURPOSE: shift the data in 'buffer' by an offset in the horizontal
         and vertical direction (where buffer is FLOAT data).

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        down and to the right by the specified offsets.  The data will
        be moved in a sequence such that no data is overwritten.  Note
        that the offsets may be zero (leave the data in its current
        location).  While it is easy to imagine the data as a 2-D
        array, one must remember that the data is actually stored in
        contiguous memory locations.
        This algorithm also fixes the problem wherein the number of
        actual samples read in (nvsb) is less than the number of samples
        in the total buffer (nsbuf).  This situation is a problem
        since the read will pass back a contiguous block of data
        with no imbedded spaces left for the unread samples (thinking
        of the data as a 2-D array).

*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        nvlb            number of valid lines of data in buffer (input)
        nvsb            number of valid samples of data in buffer(input)
        loff            number of lines to offset by (input)
        soff            number of samples to offset by (input)
        nsbuf           number of samples in the total buffer (input)

************************************************************************
*/

static int shft_flt
(
    float   *buffer,
    long    nvlb,
    long    nvsb,
    long    loff,
    long    soff,
    long    nsbuf
)
{
    
    long    i;              /* index counter */
    long    j;              /* index counter */
    float   *froma;         /* 'from address' */
    float   *toadd;         /* 'to address'   */
    float   *totemp;        /* temporary address */
    
    
    /* In order to return successfully without doing anything            */
    /*    (1) there must be no line offset, and                          */
    /*    (2) there must be no sample offset, and                        */
    /*    (3) no shifting on account of 'sample padding' is neccessary.  */
    
    if (loff == 0 && soff == 0 && nvsb == nsbuf)
        return(E_SUCC);
    
    
    /* With the following indexing scheme, one can think of the counters */
    /* j and i as indexes to the 'to location' of buffer:                */
    /*      buffer[j][i] = .....    (thinking of buffer as a 2-D array)  */
    /* The 'from address' of buffer is decremented by one each time      */
    /* because the valid buffer data coming into this routine is stored  */
    /* in contiguous memory locations.                                   */
    
    froma = buffer + (nvlb * nvsb) - 1;        /* initial 'from address' */
    
    totemp = buffer + ((nvlb - 1 + loff) * nsbuf) + nvsb - 1 + soff;
    for (j = nvlb - 1; j >= 0; j-- )
    {
        toadd = totemp;
        totemp -= nsbuf;
        for (i = nvsb - 1; i >= 0; i-- )
        {
            *toadd = *froma;
            froma--;
            toadd--;
        }
    }
    
    return(E_SUCC);
    
}     /*  end of shft_flt()   */

/*
************************************************************************
NAME: shft_lng

PURPOSE: shift the data in 'buffer' by an offset in the horizontal
         and vertical direction (where buffer is LONG data).

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        down and to the right by the specified offsets.  The data will
        be moved in a sequence such that no data is overwritten.  Note
        that the offsets may be zero (leave the data in its current
        location).  While it is easy to imagine the data as a 2-D
        array, one must remember that the data is actually stored in
        contiguous memory locations.
        This algorithm also fixes the problem wherein the number of
        actual samples read in (nvsb) is less than the number of samples
        in the total buffer (nsbuf).  This situation is a problem
        since the read will pass back a contiguous block of data
        with no imbedded spaces left for the unread samples (thinking
        of the data as a 2-D array).

*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        nvlb            number of valid lines of data in buffer (input)
        nvsb            number of valid samples of data in buffer(input)
        loff            number of lines to offset by (input)
        soff            number of samples to offset by (input)
        nsbuf           number of samples in the total buffer (input)

************************************************************************
*/

static int shft_lng
(
    long    *buffer,
    long    nvlb,
    long    nvsb,
    long    loff,
    long    soff,
    long    nsbuf
)
{
    
    long    i;              /* index counter */
    long    j;              /* index counter */
    long    *froma;         /* 'from address' */
    long    *toadd;         /* 'to address'   */
    long    *totemp;        /* temporary address */
    
    
    /* In order to return successfully without doing anything            */
    /*    (1) there must be no line offset, and                          */
    /*    (2) there must be no sample offset, and                        */
    /*    (3) no shifting on account of 'sample padding' is neccessary.  */
    
    if (loff == 0 && soff == 0 && nvsb == nsbuf)
        return(E_SUCC);
    
    
    /* With the following indexing scheme, one can think of the counters */
    /* j and i as indexes to the 'to location' of buffer:                */
    /*      buffer[j][i] = .....    (thinking of buffer as a 2-D array)  */
    /* The 'from address' of buffer is decremented by one each time      */
    /* because the valid buffer data coming into this routine is stored  */
    /* in contiguous memory locations.                                   */
    
    froma = buffer + (nvlb * nvsb) - 1;        /* initial 'from address' */
    
    totemp = buffer + ((nvlb - 1 + loff) * nsbuf) + nvsb - 1 + soff;
    for (j = nvlb - 1; j >= 0; j-- )
    {
        toadd = totemp;
        totemp -= nsbuf;
        for (i = nvsb - 1; i >= 0; i-- )
        {
            *toadd = *froma;
            froma--;
            toadd--;
        }
    }
    
    return(E_SUCC);
    
}     /*  end of shft_lng()   */

/*
************************************************************************
NAME: shft_wrd

PURPOSE: shift the data in 'buffer' by an offset in the horizontal
         and vertical direction (where buffer is SHORT data).

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        down and to the right by the specified offsets.  The data will
        be moved in a sequence such that no data is overwritten.  Note
        that the offsets may be zero (leave the data in its current
        location).  While it is easy to imagine the data as a 2-D
        array, one must remember that the data is actually stored in
        contiguous memory locations.
        This algorithm also fixes the problem wherein the number of
        actual samples read in (nvsb) is less than the number of samples
        in the total buffer (nsbuf).  This situation is a problem
        since the read will pass back a contiguous block of data
        with no imbedded spaces left for the unread samples (thinking
        of the data as a 2-D array).

*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        nvlb            number of valid lines of data in buffer (input)
        nvsb            number of valid samples of data in buffer(input)
        loff            number of lines to offset by (input)
        soff            number of samples to offset by (input)
        nsbuf           number of samples in the total buffer (input)

************************************************************************
*/

static int shft_wrd
(
    short   *buffer,
    long    nvlb,
    long    nvsb,
    long    loff,
    long    soff,
    long    nsbuf
)
{
    
    long    i;              /* index counter */
    long    j;              /* index counter */
    short   *froma;         /* 'from address' */
    short   *toadd;         /* 'to address'   */
    short   *totemp;        /* temporary address */
    
    
    /* In order to return successfully without doing anything            */
    /*    (1) there must be no line offset, and                          */
    /*    (2) there must be no sample offset, and                        */
    /*    (3) no shifting on account of 'sample padding' is neccessary.  */
    
    if (loff == 0 && soff == 0 && nvsb == nsbuf)
        return(E_SUCC);
    
    
    /* With the following indexing scheme, one can think of the counters */
    /* j and i as indexes to the 'to location' of buffer:                */
    /*      buffer[j][i] = .....    (thinking of buffer as a 2-D array)  */
    /* The 'from address' of buffer is decremented by one each time      */
    /* because the valid buffer data coming into this routine is stored  */
    /* in contiguous memory locations.                                   */
    
    froma = buffer + (nvlb * nvsb) - 1;        /* initial 'from address' */
    
    totemp = buffer + ((nvlb - 1 + loff) * nsbuf) + nvsb - 1 + soff;
    for (j = nvlb - 1; j >= 0; j-- )
    {
        toadd = totemp;
        totemp -= nsbuf;
        for (i = nvsb - 1; i >= 0; i-- )
        {
            *toadd = *froma;
            froma--;
            toadd--;
        }
    }
    
    return(E_SUCC);
    
}     /*  end of shft_wrd()   */

/*
*************************************************************************
NAME: shft_buf

PURPOSE: shift the data in the buffer by an offset in the horizontal
         and vertical direction.

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        down and to the right by the specified offsets.
        A read to the disk will return a buffer of data.  Viewing this
        buffer as a two-dimensional array, the data is always returned
        in the upper-left-hand corner of the buffer (if the buffer is
        larger than the data read into it).  The data read into the
        buffer may need to be moved to the 'center' of the 2-D array
        so that mirroring around the boundaries can take place.  Some
        of the boundary values may be part of the data read into the
        buffer.  At any rate, this function will take the data within
        the buffer and shift it down and to the right by the specified
        offsets.  The data will be moved in a sequence such that no
        data is lost.  Note that the offsets may be zero (leave the
        data in its current location).

*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        dtype           data  type (input)
        nvlb            number of valid lines of data in buffer (input)
        nvsb            number of valid samples of data in buffer (input)
        loff            number of lines to offset by (input)
        soff            number of samples to offset by (input)
        nsbuf           number of samples in the total buffer (input)

************************************************************************
*/

static void shft_buf
(
    unsigned char   *buffer,
    long    dtype,
    long    nvlb,
    long    nvsb,
    long    loff,
    long    soff,
    long    nsbuf
)
{
    
    
    switch (dtype)
    {
        case EREAL:
            shft_flt((float*)buffer,nvlb,nvsb,loff,soff,nsbuf);
            break;
        case ELONG:
            shft_lng((long*)buffer,nvlb,nvsb,loff,soff,nsbuf);
            break;
        case EWORD:
            shft_wrd((short*)buffer,nvlb,nvsb,loff,soff,nsbuf);
            break;
        case EBYTE:
            shft_byt(buffer,nvlb,nvsb,loff,soff,nsbuf);
            break;
    }
    
}     /*  end of shft_buf()   */

/*
************************************************************************
NAME: get_buf

PURPOSE: get a buffer full of data from an image in a 'sliding window'
         fashion.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1           11/89   T. Baltzer   
    1.1         02/91   B. Ailts        Standardized error messages


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

*****************************************************************
        arguments:
name            description
----            -----------
buffer          data buffer (output)
nlbuf           total number of lines in 'buffer' (input)
nsbuf           total number of samples in 'buffer' (input)
nlav            number of lines of actual valid data in buffer(output)
nsav            number of samples of actual valid data in buffer(output)
sli             starting line of the image window (input)
ssi             starting sample of the image window (input)
nli             number of lines in the image window (input)
nsi             number of samples in the image window (input)
nlb             number of lines in the boundary (input)
nsb             number of samples in the boundary (input)
slcb            previous starting line of image data (input)
                current starting line of image data (output)
                (also starting line of 'center buffer')
sscb            previous starting sample of image data (input)
                current starting sample of image data (output)
                (also starting sample of 'center buffer')
nextflg         flag to indicate first call to get_buf(), and
                flag to indicate whether the buffer origin should be
                incremented before reading the data: (input)
                  INIT  (0)  this is the first call to get_buf() for
                             this image at this origin
                  CURNT (1)  don't move the buffer origin
                  NEXT  (2)  do move the origin to the next data block,
                             and reopen the image when appropriate
band            number of the band within the image to read (input)
fill            fill technique specification:       (input)
                   NOFIL (0)  don't do any boundary filling
                   MIRR  (1)  fill by mirroring adjoining image data
fdesc           file descriptor
host            host name of the image file
dtype           data type of buffer (input/output)

NOTE:  Starting line/sample numbers are relative to one (1).

************************************************************************
*/

int get_buf
(
    unsigned char   *buffer,
    long    *nlbuf,
    long    *nsbuf,
    long    *nlav,
    long    *nsav,
    long    *sli,
    long    *ssi,
    long    *nli,
    long    *nsi,
    long    *nlb,
    long    *nsb,
    long    *slcb,
    long    *sscb,
    long    *nextflg,
    long    *band,
    long    *fill,
    struct FDESC **fdesc,
    char    *host,
    long    *dtype
)
{
    
    long    nltf;           /* number of lines to fill, top boundary     */
    long    nlbf;           /* number of lines to fill, bottom boundary  */
    long    nslf;           /* number of samples to fill, left boundary  */
    long    nsrf;           /* number of samples to fill, right boundary */
    long    nlcb;           /* number of lines in the center buffer      */
    long    nscb;           /* number of samples in the center buffer    */
    long    slrb;           /* starting line of the read buffer          */
    long    ssrb;           /* starting sample of the read buffer        */
    long    nlrb;           /* number of lines in the read buffer        */
    long    nsrb;           /* number of samples in the read buffer      */
    long    loff;           /* number of lines to offset image data      */
    long    soff;           /* number of samples to offset image data    */
    long    nlred;          /* number of lines read                      */
    long    nsred;          /* number of samples read                    */
    long    status;         /* return status flag                        */
    long    bands = 0;      /* number of bands in the image              */
    long    nonfatal = 1;
    long    openflg = 0;    /* open image flag indicator                 */
    long    oldss;          /* remember old starting sample number       */
    long    access = 0;     /* file access specification                 */
    
    
    /* Error check the input parameters.                                 */
    
    status = err_get(*nlbuf,*nsbuf,*sli,*ssi,*nli,*nsi,*nlb,*nsb,
                     *slcb,*sscb,*nextflg,*band,*fill,*dtype);
    if (status == E_FAIL)
        return(E_FAIL);
    
    
    /* Determine the center buffer dimensions.  The center buffer is     */
    /* the input buffer 'buffer' less the boundaries.                    */
    
    nlcb = *nlbuf - (*nlb * 2);
    nscb = *nsbuf - (*nsb * 2);
    
    
    /* Determine the origin of the buffer of data to be read (based on   */
    /* the previous origin, the 'next flag', and the image window        */
    /* dimensions specified by sli, ssi, nli, nsi).  Also set the        */
    /* 'open image flag' (openflg).                                      */
    
    switch (*nextflg)
    {
        case  INIT:             /* first call to get_buf()      */
            openflg = 1;   /* open image for first time    */
            break;
        case  CURNT:            /* read at current origin       */
            openflg = 0;   /* do not need to open image    */
            break;
        case  NEXT:             /* get next origin before reading  */
            oldss = *sscb;
            status = next_org(*sli,*ssi,*nli,*nsi,nlcb,nscb,slcb,sscb);
            if (status == E_FAIL)
                return(E_EOIW);     /* origin passed image window   */
            if ( oldss != *sscb)
            {
                openflg = 2;        /* need to reopen image */
            }
            else
                openflg = 0;        /* don't need to reopen image */
            break;
    }  /* end of switch construct */
    
    
    /* Determine the dimensions of the read buffer.  The read buffer is  */
    /* the buffer of data which will be read from the disk.  This would  */
    /* include a full or partial center buffer of data (whatever's       */
    /* available), plus all of the available boundary pixels.            */
    
    dim_rbf(*slcb,*sscb,nlcb,nscb,*nlb,*nsb,*sli,*ssi,*nli,*nsi,
            &slrb,&ssrb,&nlrb,&nsrb);
    
    
    /* Make sure that the image file is opened at the correct location.  */
    /* open_img() may or may not open the image at a different location. */
    
    status = open_img(fdesc,host,openflg,&bands,ssrb,0,nsrb,
                      *nlbuf,access,dtype,0);
    if (status == E_FAIL)
        return(E_FAIL);
    
    
    /* Read the available data from the image.                           */
    
    nlred = nlrb;
    nsred = c_eread(fdesc,band,&slrb,buffer,&nlred);
    if (nsred == E_FAIL)
    {
        c_errmsg("Error returned from eread","get_buf-call",&nonfatal);
        return(E_FAIL);
    }
    
    nsred = nsred / nlred;          /* calculate samples per line */
    
    
    /* Shift the data within the input buffer by the appropriate offsets */
    
    loff = slrb - *slcb + *nlb;
    soff = ssrb - *sscb + *nsb;
    shft_buf(buffer,*dtype,nlred,nsred,loff,soff,*nsbuf);
    
    
    /* Determine the dimensions of the 'fill boundaries'.  The fill      */
    /* boundaries are the number of rows/columns on the boundaries that  */
    /* were not read from the image and need to be filled using some     */
    /* fill algorithm.                                                   */
    
    dim_fbnd(*slcb,*sscb,nlcb,nscb,*nlb,*nsb,*sli,*ssi,*nli,*nsi,
             &nltf,&nlbf,&nslf,&nsrf);
    
    
    /* There is now sufficient data available to determine the number of */
    /* lines/samples of actual valid data in the input buffer 'buffer'.  */
    /* Note that if 'fill' = NOFIL (no fill), the number of valid lines/ */
    /* samples will be set as if these values were filled using some     */
    /* fill algorithm.                                                   */
    
    *nlav = nlred + nltf + nlbf;
    *nsav = nsred + nslf + nsrf;
    
    
    /* Fill the boundary pixel values using some fill algorithm.         */
    
    switch (*fill)
    {
        case NOFIL:
            break;
        case MIRR :
            mirr_bnd(buffer,*dtype,*nlav,*nsav,*nsbuf,nltf,nlbf,nslf,nsrf);
            break;
    }
    
    
    return(E_SUCC);
    
}    /*  end of get_buf()   */
