/******************************************************************************
FUNCTION:       process_it

PURPOSE:        Process_it is an internal imageio function called by
                estep for each line.  Do to the complexity of asynchronous I/O
                this routine gets pretty hairy.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      06/85    T. Butzer           Original implementation

  1.1      03/86    T. Butzer           Return status

  1.2      03/86    K. Johnson          Handle multiple file groups

  2.0      12/87    B. Ailts            Changed include directory specification
                                        Changed to raw 'C' types
                                        Moved bridge routines to seperate file

  2.1      04/88    D.Hollaren          Replaced cerrmsg with c_errmsg
                                        Replaced newlas.h with las.h

  2.2      05/88    B.Ailts             Replaced errh with c_errmsg
                                        Replaced xi____ with i____ calls
                                        Removed eerror.h
  2.3      08/88    B.Ailts             Removed the status checks from the
                                        i_read and i_write calls -- they do
                                        not return any values
                                        Added better error checking
  2.4      12/88    B.Ailts             Output an error message when output
                                        values are clipped
  2.5      02/91    B.Ailts             Standardized error messages
  2.6      05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  2.7      10/92    T. Mittan           Added compression option
            1/94    K.Gacke             Modified to read/write dal images.  TAE
                                        images can be read, but all images are
                                        written as a BSQ DAL image.
  2.8      08/99    T.Beckmann          Added support for files >2GB

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM:
If read access
    If  
        If conversion
           Read into the conversion buffer
           Recalculate the offsets
           Convert the data into the output buffer
        Else
           Read into the output buffer
        Recalculate pointers and offsets
Else if write access
   Set up pointers
   If buffer needs to be written
   Recalculate the starting line and number of lines in buffer
   If conversion
      Recalculate buffers
      Convert the input buffer into the conversion buffer
      Write the conversion buffer
   Else
      Write the input buffer
   Recalculate pointers
Else if update accesss
   Set up pointers
   If conversion
      Output an error message
   Else
      Read in the lines
      If current line is greater than or equal to the number of lines in buffer
         Write the lines
   Recalculate the offsets
   
ALGORITHM REFERENCES    none
*******************************************************************************/

#include <sys/types.h>
#include <unistd.h>
#include "las.h"
#include "util.h"         /* c_sigblk prototype */
#include "lasfileio.h"    /* endian neutral I/O routines */
#include "taeimageio.h"   /* tae image i/o prototypes */
#include "datasize.h"
#include "fdesc.h"
#include "pixman.h"
#include "local_protos.h"

#define NLINES gd->gbl_nlines
/**************
#define DEBUG 1
**************/

int process_it
(
    struct FDESC *fd,  /*  I/O: file descriptor */
    long cline,        /*  I: current line */
    long cband,        /*  I: current band */
    struct GDESC *gd   /*  I: group descrptor */
)
{
    long herr;
    long line, nlines;
    long status;
    long nonfatal = 1;          /* non-fatal error code        */
    long offset;                /* offset in compressed buffer    */
    long count1;                /* number of values to read       */
    long count2;                /* # bytes to read/write for dal image files */
    long j;                     /* loop counter                   */
    long one = 1;
    FSEEK_OFF_T position;       /* file position                  */
    long *position_end;
    LSEEK_OFF_T newpos;         /* file position for dal image files */
    
    static short big_first = TRUE;
    static short small_first = TRUE;
    
    unsigned char *coffset, *inoffset;
    unsigned char *tempbuf;           /* temporary offset buffer        */
    char errtxt[ERRLEN + 1];
    long i;
    int sig_mask;
    
    c_sigblk(&sig_mask);
    if (fd->acc == IREAD)
    {
        if (fd->buf[cband].cnt == 0)
        {
            line = cline + fd->sl - fd->nbuf;
            if (line < fd->sl)
                line = fd->sl;
            if ((cline + NLINES) > fd->nl)
                nlines = fd->nl - cline;
            else
                nlines = NLINES;
            
            if (fd->flags & CONVERSION)
            {
                if (cline < fd->nl)
                {
                    /* check compression flag   
                    -----------------------*/
                    if (fd->compres == TRUE)
                    {
                        fd->foffset[cband] = 0;
                        offset = 0;
                        /* read in nlines at a time
                        --------------------------*/
                        if ((position_end = calloc((NLINES * fd->mbands), 
                            fd->file_offset_size)) == NULL)
                        {
                            c_errmsg("Error allocation dynamic memory",
                                     "Processit-alloc", &nonfatal);
                            return(E_FAIL);
                        }
                        position = 18 + ((fd->sl + cline -1)* 
                             (FSEEK_OFF_T)fd->mbands) * fd->file_offset_size;
                        if (FSEEK(fd->fp,position,SEEK_SET) != 0)
                        {
                            c_errmsg("Error seeking in file","estep-seek",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                        if (c_fread(position_end,fd->file_offset_size,(nlines * 
                                  fd->mbands),fd->fp) != (nlines * fd->mbands))
                        {
                            c_errmsg("Error reading file","eread-read",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                        for (j = 0; j < nlines; j++) 
                        {
                            /* find position in file
                            ----------------------*/
                            if (fd->file_offset_size == sizeof(int))
                            {
                              position = (FSEEK_OFF_T)((int *)position_end)
                                    [(j * fd->mbands) + fd->bands[cband] -1];
                            }
                            else
                            {
                              position =(FSEEK_OFF_T)((long long *)position_end)
                                    [(j * fd->mbands) + fd->bands[cband] -1];
                            }
                            /* Read in number of values and values and send to 
                               decompression routine
                            -------------------------------------------------*/
                            if (FSEEK(fd->fp,position,SEEK_SET) != 0)
                            {
                                c_errmsg("Error seeking in file","estep-seek",
                                         &nonfatal);
                                return(E_FAIL);
                            }
                            if (c_fread(&count1,sizeof(long),1,fd->fp) != 1)
                            {
                                c_errmsg("error reading file","eread-read",
                                         &nonfatal);
                                return(E_FAIL);
                            }
                            if ((tempbuf = (unsigned char *)calloc(count1,
                                              sizeof(unsigned char))) == NULL)
                            {
                                c_errmsg("Error allocation dynamic memory",
                                         "Processit-alloc", &nonfatal);
                                return(E_FAIL);
                            }
                            if (c_fread(tempbuf ,sizeof(char),count1,fd->fp)
                                    != count1)
                            {
                                c_errmsg("error reading file","eread-read",
                                         &nonfatal);
                                return(E_FAIL);
                            }
                            decode_76(tempbuf,&fd->buf[cband].conbuf[offset],
                                      fd->mns, datasize[fd->conv_type]);
                            fd->foffset[cband] += fd->linsiz; 
                            offset += fd->cnvlsz;
                            free(tempbuf);
                        }
                        free(position_end);
                    }
                    else if (fd->dal)
                    {
                        newpos = ((LSEEK_OFF_T)(fd->bands[cband] - 1) * 
                                  fd->img_nl * fd->cnvlsz) + 
                                  ((line - 1) * (LSEEK_OFF_T)fd->cnvlsz);
                        if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
                        {
                            perror("c_eread-seek");
                            c_errmsg("error seeking dal file","eread",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                        count2 = fd->cnvlsz * nlines;
                        if (c_read(fd->dalfd,fd->buf[cband].conbuf,count2,
                                   datasize[fd->conv_type]) < 
                                count2)
                        {
                            perror("c_eread-read");
                            c_errmsg("error reading dal file","eread",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                    }
                    else if (fd->tae)
                    {
                        
                        /* the following xiwait is for vms tae 1.2 i believe it
                           allocates the same event flag for all files this 
                           problem may be fixed in tae 1.3 further 
                           investigation is needed on this. */
                        status = i_wait(&(fd->ifcb));
                        if (status != SUCCESS)
                        {
                            herr = i_herr(&(fd->ifcb));
                            c_lasmsg(FALSE,"estep-xiwait",
                                    "Xiwait error %d encountered for file %s",
                                    herr,fd->fname);
                            return (E_FAIL);
                        }
#ifdef DEBUG
                        printf("read line = %d nlines = %d\n", line, nlines);
#endif
                        /* read in nlines of data
                        ----------------------- */
                        i_read(&(fd->ifcb), (GENPTR)fd->buf[cband].conbuf, 
                               fd->bands[cband], line, nlines);
                        
                        status = i_wait(&(fd->ifcb));
                        if (status != SUCCESS)
                        {
                            herr = i_herr(&(fd->ifcb));
                            c_lasmsg(FALSE,"estep-xiwait",
                                    "Xiwait error %d encountered for file %s",
                                    herr,fd->fname);
                            return (E_FAIL);
                        }
                    }               
                    inoffset = (unsigned char *) (fd->buf[cband].conbuf
                            + (fd->ss - 1) * datasize[fd->conv_type]);
                    coffset = (unsigned char *) (fd->buf[cband].nxt_buf
                            + (fd->ss - 1) * datasize[fd->dtype]);
                    for (i=0; i<nlines; i++)
                    {
                        status = c_pxconv(&fd->conv_type, &fd->dtype, 
                                          inoffset, coffset, &fd->ns);
                        if (status != E_SUCC)
                        {
                            if (((status & TOOBIG) == TOOBIG) && (big_first))
                            {
                                sprintf(errtxt,"%s%s",
                                        "Pixel values were larger than",
                                        " maximum allowed by output data type");
                                c_errmsg(errtxt,"estep-pixel",&nonfatal);
                                big_first = FALSE;
                            }
                            else if (((status & TOOSMALL) == TOOSMALL) 
                                     && (small_first))
                            {
                                sprintf(errtxt,"%s%s",
                                        "Pixel values were smaller than",
                                        " minimum allowed by output data type");
                                c_errmsg(errtxt,"estep-pixel",&nonfatal);
                                small_first = FALSE;
                            }
                        }
                        coffset += fd->linsiz;
                        inoffset += fd->cnvlsz;
                    }
                } /* end of if cline < fd->nl */
            }
            else
            {
                if (cline < fd->nl)
                {
                    /* check compression flag   
                    -----------------------*/
                    if (fd->compres == TRUE)
                    {
                        fd->foffset[cband] = 0;
                        if ((position_end = calloc((NLINES * fd->mbands),
                            fd->file_offset_size)) == NULL)
                        {
                            c_errmsg("Error allocation dynamic memory",
                                     "Processit-alloc", &nonfatal);
                            return(E_FAIL);
                        }
                        position = 18 + ((fd->sl + cline -1)* 
                             (FSEEK_OFF_T)fd->mbands) * fd->file_offset_size;
                        if (FSEEK(fd->fp,position,SEEK_SET) != 0)
                        {
                            c_errmsg("Error seeking in file","eread-read",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                        if (c_fread(position_end,fd->file_offset_size,(nlines * 
                                  fd->mbands), fd->fp) != (nlines * fd->mbands))
                        {
                            c_errmsg("error reading file","eread-read",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                        /* read in nlines of data
                        ------------------------*/
                        for (j = 0; j < nlines; j++) 
                        {
                            /* find position in file
                            ----------------------*/
                            if (fd->file_offset_size == sizeof(int))
                            {
                              position = (FSEEK_OFF_T)((int *)position_end)
                                  [(j * fd->mbands) + fd->bands[cband] -1];
                            }
                            else
                            {
                              position =(FSEEK_OFF_T)((long long *)position_end)
                                  [(j * fd->mbands) + fd->bands[cband] -1];
                            }

                            /* Read in number of values and values and send to 
                               decompression routine
                            -------------------------------------------------*/
                            if (FSEEK(fd->fp,position,SEEK_SET) != 0)
                            {
                                c_errmsg("Error seeking in file","eread-read",
                                         &nonfatal);
                                return(E_FAIL);
                            }
                            if (c_fread(&count1,sizeof(long),1,fd->fp) != 1)
                            {
                                c_errmsg("error reading file","eread-read",
                                         &nonfatal);
                                return(E_FAIL);
                            }
                            if ((tempbuf = (unsigned char *)calloc(count1,
                                           sizeof(unsigned char))) == NULL)
                            {
                                c_errmsg("Error allocation dynamic memory",
                                         "Processit-alloc", &nonfatal);
                                return(E_FAIL);
                            }
                            if (c_fread(tempbuf ,sizeof(char),count1,fd->fp)
                                   != count1)
                            {
                                c_errmsg("error reading file","eread-read",
                                         &nonfatal);
                                return(E_FAIL);
                            }
                            decode_76(tempbuf,&fd->buf[cband].nxt_buf[
                                      fd->foffset[cband]], fd->mns,
                                      datasize[fd->dtype]);
                            fd->foffset[cband] +=  fd->linsiz;
                            free(tempbuf);
                        }
                        free(position_end);
                    }
                    else if (fd->dal)
                    {
                        newpos = ((LSEEK_OFF_T)(fd->bands[cband] - 1) * 
                                  fd->img_nl * fd->linsiz) + 
                                  ((line - 1) * (LSEEK_OFF_T)fd->linsiz);
                        if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
                        {
                            perror("c_eread-seek");
                            c_errmsg("error seeking dal file","eread",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                        count2 = fd->linsiz * nlines;
                        if (c_read(fd->dalfd,fd->buf[cband].nxt_buf,count2,
                                   datasize[fd->dtype]) < count2)
                        {
                            perror("c_eread-read");
                            c_errmsg("error reading dal file","eread",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                    }
                    else if (fd->tae)
                    {
                        status = i_wait(&(fd->ifcb));
                        if (status != SUCCESS)
                        {
                            herr = i_herr(&(fd->ifcb));
                            c_lasmsg(FALSE,"estep-xiwait",
                                    "Xiwait error %d encountered for file %s",
                                    herr,fd->fname);
                            return (E_FAIL);
                        }
                        
#ifdef DEBUG
                        printf("read line = %d nlines = %d\n", line, nlines);
#endif
                        i_read(&(fd->ifcb), (GENPTR)fd->buf[cband].nxt_buf, 
                               fd->bands[cband], line, nlines);
                    }
                } /* end if cline */
            }
            fd->buf[cband].cnt = NLINES;
            fd->buf[cband].nxt_buf += (fd->linsiz * NLINES);
            if (fd->buf[cband].nxt_buf >= fd->buf[cband].buf_end)
            {
                fd->buf[cband].nxt_buf = fd->buf[cband].buf_start;
                if (fd->compres)
                    fd->foffset[cband] = 0;
            }
        }
        fd->offset[cband] = (fd->buf[cband].nxt_offset + (fd->ss - 1) *
                datasize[fd->dtype] - gd->thebuf) / datasize[fd->dtype];
        fd->buf[cband].nxt_offset += fd->linsiz;
        if (fd->buf[cband].nxt_offset >= fd->buf[cband].buf_end)
        {
            fd->buf[cband].nxt_offset = fd->buf[cband].buf_start + 
                (fd->nbuf * fd->linsiz);
        }
        
        fd->buf[cband].cnt--;
    }
    else
    if (fd->acc == IWRITE)
    {
        fd->offset[cband] = (fd->buf[cband].nxt_offset + (fd->ss - 1) *
                datasize[fd->dtype] - gd->thebuf) / datasize[fd->dtype];
        fd->buf[cband].nxt_offset += fd->linsiz;
        if (fd->buf[cband].nxt_offset >= fd->buf[cband].buf_end)
            fd->buf[cband].nxt_offset = fd->buf[cband].buf_start;
        if ((fd->buf[cband].cnt == 0) || (cline >= fd->nl) || 
            (fd->compres && cline != 0))
        {
            line = cline + fd->sl - NLINES;
            if (fd->compres)
                if (cline <= fd->nl)
                    nlines = 1;
                else
                    nlines = 0;
            else
            {
                if (cline >= fd->nl)
                {
                    nlines = NLINES - fd->buf[cband].cnt;
                    line = cline + fd->sl - (NLINES - fd->buf[cband].cnt);
                }
                else
                    nlines = NLINES;
                
                if (fd->tae)
                {
                    status = i_wait(&(fd->ifcb));
                    if (status != SUCCESS)
                    {
                        herr = i_herr(&(fd->ifcb));
                        c_lasmsg(FALSE,"estep-xiwait",
                                "Xiwait error %d encountered for file %s",
                                herr,fd->fname);
                        return (E_FAIL);
                    }
                }
            }
#ifdef DEBUG
            printf("write line = %d nlines = %d\n", line, nlines);
#endif
            if (fd->flags & CONVERSION)
            {
                if (fd->compres)
                {
                    inoffset = (unsigned char *)
                        (&fd->buf[cband].nxt_buf[fd->foffset[cband]]
                        + (fd->ss - 1) * datasize[fd->dtype]);
                }
                else
                {
                    inoffset = (unsigned char *) (fd->buf[cband].nxt_buf
                                + (fd->ss - 1) * datasize[fd->dtype]);
                }
                coffset = (unsigned char *) (fd->buf[cband].conbuf
                          + (fd->ss - 1) * datasize[fd->conv_type]);
                for (i=0; i < nlines; i++)
                {
                    status = c_pxconv(&fd->dtype, &fd->conv_type, inoffset, 
                                      coffset, &fd->ns);
                    if (status != E_SUCC)
                    {
                        if (((status & TOOBIG) == TOOBIG) && (big_first))
                        {
                            c_errmsg("Pixel values were larger than maximum "
                                     "allowed by output data type",
                                     "estep-pixel", &nonfatal);
                            big_first = FALSE;
                        }
                        else if (((status & TOOSMALL)  == 
                                 TOOSMALL) && (small_first))
                        {
                            c_errmsg("Pixel values were smaller than minimum "
                                     "allowed by output data type",
                                     "estep-pixel", &nonfatal);
                            small_first = FALSE;
                        }
                    }
                    coffset += fd->cnvlsz;
                    inoffset += fd->linsiz;
                }
                if ((fd->compres == TRUE) && (nlines > 0))
                {
                    if (FSEEK(fd->fp,0,SEEK_END) != 0)
                    {
                        c_errmsg("Error seeking in file","estep-write",
                                 &nonfatal);
                        return(E_FAIL);
                    }

                    /* set the file offset for the current line in the offset
                       table */
                    if (fd->file_offset_size == sizeof(long long))
                    {
                        ((long long *)fd->offset_table)[((cline - 1) * 
                                fd->nbands) + cband] = FTELL(fd->fp);
                    }
                    else
                    {
                        ((int *)fd->offset_table)[((cline - 1) * fd->nbands) +
                                cband] = (int)FTELL(fd->fp);
                    }
                    
                    encode_76(fd->buf[cband].conbuf,fd->tempbuf,fd->ns, &count1,
                              datasize[fd->conv_type]);
                    if (c_fwrite(&count1,sizeof(long), one,fd->fp) != one)
                    {
                        c_errmsg("error writing file","estep-write",&nonfatal);
                        return(E_FAIL);
                    }
                    if (c_fwrite(fd->tempbuf,sizeof(char),count1,fd->fp)
                            !=count1)
                    {
                        c_errmsg("error writing file","estep-write",&nonfatal);
                        return(E_FAIL);
                    }
                    fd->foffset[cband] += fd->linsiz;
                }
                else if (fd->dal)
                {
                    newpos = ((LSEEK_OFF_T)(fd->bands[cband] - 1) * fd->img_nl 
                             * fd->cnvlsz) + 
                             ((line - 1) * (LSEEK_OFF_T)fd->cnvlsz);
                    if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
                    {
                        perror("c_estep-seek");
                        c_errmsg("error seeking dal file", "estep",&nonfatal);
                        return(E_FAIL);
                    }
#ifdef DEBUG
                    printf("processit: dal band=%2d  newpos=%5d  "
                           "img_nl=%d  linsiz=%d\n",
                           fd->bands[cband],newpos,fd->img_nl,fd->linsiz);
#endif
                    count2 = fd->cnvlsz * nlines;
                    if (c_write(fd->dalfd, fd->buf[cband].conbuf,count2,
                                datasize[fd->conv_type]) < 0)
                    {
                        perror("c_ewrite-write");
                        c_errmsg("error writing dal file", "ewrite",&nonfatal);
                        return(E_FAIL);
                    }
                }
                else if (fd->tae)
                {
                    i_write(&(fd->ifcb), (GENPTR)fd->buf[cband].conbuf,
                            fd->bands[cband],line,nlines);
                }
            }
            else
            {
                if ((fd->compres == TRUE) && (nlines > 0))
                {
                    if (FSEEK(fd->fp,0,SEEK_END) != 0)
                    {
                        c_errmsg("Error seeking in file","estep-write",
                                 &nonfatal);
                        return(E_FAIL);
                    }
                    
                    /* set the file offset for the current line in the offset
                       table */
                    if (fd->file_offset_size == sizeof(long long))
                    {
                        ((long long *)fd->offset_table)[((cline - 1) * 
                                fd->nbands) + cband] = FTELL(fd->fp);
                    }
                    else
                    {
                        ((int *)fd->offset_table)[((cline - 1) * fd->nbands) +
                                cband] = (int)FTELL(fd->fp);
                    }
                            
                    encode_76(&fd->buf[cband].nxt_buf[fd->foffset[cband]],
                              fd->tempbuf,fd->ns,&count1, datasize[fd->dtype]);
                    if (c_fwrite(&count1,sizeof(long), one,fd->fp) != one)
                    {
                        c_errmsg("error writing file","estep-write",&nonfatal);
                        return(E_FAIL);
                    }
                    if (c_fwrite(fd->tempbuf,sizeof(char), count1,fd->fp)
                            !=count1)
                    {
                        c_errmsg("error writing file","estep-write",&nonfatal);
                        return(E_FAIL);
                    }
                    fd->foffset[cband] += fd->linsiz;
                }
                else if (fd->dal)
                {
                    newpos = ((LSEEK_OFF_T)(fd->bands[cband] - 1) * fd->img_nl 
                              * fd->linsiz) + 
                              ((line - 1) * (LSEEK_OFF_T)fd->linsiz);
                    if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
                    {
                        perror("c_estep-fseek");
                        c_errmsg("error seeking dal file", "c_estep",&nonfatal);
                        return(E_FAIL);
                    }
#ifdef DEBUG
                    printf("processit: dal band=%2d  newpos=%5d  "
                           "img_nl=%d  linsiz=%d\n",
                           fd->bands[cband],newpos,fd->img_nl,fd->linsiz);
#endif
                    count2 = fd->linsiz * nlines;
                    if (c_write(fd->dalfd, fd->buf[cband].nxt_buf,count2,
                                datasize[fd->dtype]) < 0)
                    {
                        perror("c_ewrite-write");
                        c_errmsg("error writing dal file", "ewrite",&nonfatal);
                        return(E_FAIL);
                    }
                }
                else if (fd->tae)
                {
                   i_write(&(fd->ifcb), (GENPTR)fd->buf[cband].nxt_buf,
                           fd->bands[cband],line,nlines);
                }
            }
            
            if ((fd->buf[cband].cnt == 0) || (cline >= fd->nl)) 
            {
                fd->buf[cband].cnt = NLINES;
                fd->buf[cband].nxt_buf += (fd->linsiz * NLINES);
                if (fd->buf[cband].nxt_buf >= fd->buf[cband].buf_end)
                {
                    fd->buf[cband].nxt_buf = fd->buf[cband].buf_start;
                    if (fd->compres)
                        fd->foffset[cband] = 0;
                }
            }
        }
        
        fd->buf[cband].cnt--;
    }
    else if (fd->acc == IUPDATE)
    {
        if (fd->upbuf[cband].rdcnt == 0)
        {
            line = cline + fd->sl;
            if ((cline + NLINES) > fd->nl)
                nlines = fd->nl - cline;
            else
                nlines = NLINES;
            
            if (fd->flags & CONVERSION)
            {
                sprintf(errtxt,
                    "Conversions not allowed with update access of file %s",
                    fd->fname);
                c_errmsg(errtxt,"estep-updconv",&nonfatal);
                return(E_FAIL);
            }
            else
            {
                if (cline < fd->nl)
                {
#ifdef DEBUG
                    printf("read line = %d nlines = %d\n", line, nlines);
#endif
                    if (fd->dal)
                    {
                        newpos = ((LSEEK_OFF_T)(fd->bands[cband] - 1) * 
                                  fd->img_nl * fd->linsiz) + 
                                  ((line - 1) * (LSEEK_OFF_T)fd->linsiz);
                        if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
                        {
                            perror("c_eread-seek");
                            c_errmsg("error seeking dal file","eread",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                        count2 = fd->linsiz * nlines;
                        if (c_read(fd->dalfd, fd->upbuf[cband].nxtrd_buf,count2,
                                   datasize[fd->dtype])
                              < 0)
                        {
                            perror("c_eread-read");
                            c_errmsg("error reading dal file","eread",
                                     &nonfatal);
                            return(E_FAIL);
                        }
                    }
                    if (fd->tae)
                    {
                        status = i_wait(&(fd->ifcb));
                        if (status != SUCCESS)
                        {
                            herr = i_herr(&(fd->ifcb));
                            c_lasmsg(FALSE,"estep-xiwait",
                               "Xiwait error %d encountered for file %s",
                                herr,fd->fname);
                            return (E_FAIL);
                        }
                        i_read(&(fd->ifcb), (GENPTR)fd->upbuf[cband].nxtrd_buf, 
                               fd->bands[cband],line, nlines);
                    }
                } /* end if cline */
            }
            fd->upbuf[cband].rdcnt = NLINES;
            fd->upbuf[cband].nxtrd_buf += (fd->linsiz * NLINES);
            if (fd->upbuf[cband].nxtrd_buf >= fd->upbuf[cband].buf_end)
                fd->upbuf[cband].nxtrd_buf = fd->upbuf[cband].buf_start;
        }
        
        if ((fd->upbuf[cband].wtcnt == 0) || (cline >= fd->nl))
        {
            line = cline + fd->sl - NLINES;
            if (cline >= fd->nl)
            {
                nlines = NLINES - fd->upbuf[cband].wtcnt;
                line = cline + fd->sl - (NLINES - fd->upbuf[cband].wtcnt);
            }
            else
                nlines = NLINES;
            
#ifdef DEBUG
            printf("write line = %d nlines = %d\n", line, nlines);
#endif
            if (fd->dal)
            {
                newpos = ((LSEEK_OFF_T)(fd->bands[cband] - 1) * fd->img_nl * 
                         fd->linsiz) + ((line - 1) * (LSEEK_OFF_T)fd->linsiz);
                if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
                {
                    perror("c_ewrite-seek");
                    c_errmsg("error seeking dal file", "ewrite",&nonfatal);
                    return(E_FAIL);
                }
                count2 = fd->linsiz * nlines;
                if (c_write(fd->dalfd, fd->upbuf[cband].nxtwt_buf,count2,
                            datasize[fd->dtype]) < 0)
                {
                    perror("c_ewrite-write");
                    c_errmsg("error writing dal file", "ewrite",&nonfatal);
                    return(E_FAIL);
                }
            }
            if (fd->tae)
            {
                status = i_wait(&(fd->ifcb));
                if (status != SUCCESS)
                {
                    herr = i_herr(&(fd->ifcb));
                    c_lasmsg(FALSE,"estep-xiwait","Xiwait error %d encountered "
                             "for file %s",herr,fd->fname);
                    return (E_FAIL);
                }
                i_write(&(fd->ifcb),(GENPTR)fd->upbuf[cband].nxtwt_buf,
                        fd->bands[cband], line, nlines);
            }
            
            fd->upbuf[cband].wtcnt = NLINES;
            fd->upbuf[cband].nxtwt_buf += (fd->linsiz * NLINES);
            if (fd->upbuf[cband].nxtwt_buf >= fd->upbuf[cband].buf_end)
                fd->upbuf[cband].nxtwt_buf = fd->upbuf[cband].buf_start;
        }
        
        fd->offset[cband] = (fd->upbuf[cband].nxt_offset + (fd->ss - 1)
                            * datasize[fd->dtype] - gd->thebuf)
                            / datasize[fd->dtype];
        fd->upbuf[cband].nxt_offset += fd->linsiz;
        if (fd->upbuf[cband].nxt_offset >= fd->upbuf[cband].buf_end)
            fd->upbuf[cband].nxt_offset = fd->upbuf[cband].buf_start;
        
        fd->upbuf[cband].rdcnt--;
        fd->upbuf[cband].wtcnt--;
    }
    c_sigunblk(&sig_mask);
    return(E_SUCC);
}
