/*
************************************************************************
NAME: put_buf

PURPOSE: write a buffer full of data to an image in a 'sliding window'
         fashion.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1           12/89   T. Baltzer   
    1.1         02/91   B.Ailts         Standardized error messages



COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

*****************************************************************
        arguments:
name            description
----            -----------
buffer          data buffer (input)
nlbuf           total number of lines in 'buffer' (input)
nsbuf           total number of samples in 'buffer' (input)
nlav            number of lines of actual valid data in buffer(input)
nsav            number of samples of actual valid data in buffer(input)
slbuf           starting line of the data on the image (input)
ssbuf           starting sample of the data on the image (input)
nlti            number of lines in the total image (imput)
nsti            number of samples in the total image (imput)
nextflg         flag to indicate first call to put_buf(), and
                flag to indicate whether the buffer origin should be
                incremented before writing the data: (input)
                  CREAT (-1) this is the first call to put_buf() for
                             this image at this origin (NEW FILE)
                  INIT  (0)  this is the first call to put_buf() for
                             this image at this origin (EXISTING FILE)
                  CURNT (1)  don't move the buffer origin
                  NEXT  (2)  do move the origin to the next data block,
                             and reopen the image when appropriate
band            number of the band within the image to write (input)
bands           number of bands within the image (input)
fdesc           file descriptor
host            host name of the image file
dtype           data type of buffer
wtype           data type that data should be converted to and written
                as, SAME (0) if no conversion required (input).

NOTE:  Starting line/sample numbers are relative to one (1). 

************************************************************************
*/

#include "las.h"
#include "fdesc.h"
#include "local_protos.h"

static int err_put
(
    long    nlbuf,
    long    nsbuf,
    long    nlav,
    long    nsav,
    long    slbuf,
    long    ssbuf,
    long    nlti,
    long    nextflg,
    long    band,
    long    bands,
    long    dtype,
    long    wtype
);

static int next_wrt
(
    long nlti,
    long nlbuf,
    long nsbuf,
    long *slbuf,
    long *ssbuf
);

static void pack_buf
(
    unsigned char *buffer,
    long nsbuf,
    long dtype,
    long nlav,
    long nsav
);

static int pack_byt
(
    unsigned char *buffer,
    long nsbuf,
    long nlav,
    long nsav
);

static int pack_flt
(
    float *buffer,
    long  nsbuf,
    long  nlav,
    long  nsav
);

static int pack_lng
(
    long *buffer,
    long nsbuf,
    long nlav,
    long nsav
);

static int pack_wrd
(
    short *buffer,
    long  nsbuf,
    long  nlav,
    long  nsav
);


int put_buf
(
    unsigned char *buffer,
    long    *nlbuf,
    long    *nsbuf,
    long    *nlav,
    long    *nsav,
    long    *slbuf,
    long    *ssbuf,
    long    *nlti,
    long    *nsti,
    long    *nextflg,
    long    *band,
    long    *bands,
    struct FDESC **fdesc,
    char    *host,
    long    *dtype,
    long    *wtype
)
{
    long    nlwrt;          /* number of lines to write                  */
    long    nswrt;          /* number of samples written                 */
    long    status;         /* return status flag                        */
    long    nonfatal = 1;
    long    oldss;          /* remember old starting sample number       */
    long    openflg=0;      /* open image flag                           */
    long    access=0;       /* file access specification                 */
    
    
    /* Error check the input parameters.                                 */
    
    status = err_put(*nlbuf,*nsbuf,*nlav,*nsav,*slbuf,*ssbuf,*nlti,
                     *nextflg,*band,*bands,*dtype,*wtype);
    if (status == E_FAIL)
        return(E_FAIL);
    
    
    /* Determine the origin of the buffer of data to be written (based   */
    /* on the previous origin and the 'next flag'.  Also set the         */
    /* 'open image flag' (openflg).                                      */
    
    switch (*nextflg)
    {
        case  CREAT:            /*  create image                */
            openflg = 1;   /* open image for first time    */
            access = 1;    /* write access only            */
            status = open_img(fdesc,host,openflg,bands,*ssbuf,*nlti,*nsti,
                              *nlbuf,access,dtype,*wtype);
            if (status == E_FAIL)
            {
                c_errmsg("Error returned from open_img","put_buf-call",
                         &nonfatal);
                return(E_FAIL);
            }
            openflg = 2;   /* need to reopen image */
            access = 2;    /* read/write access    */
            break;
        case  INIT:             /* first call to put_buf()      */
            openflg = 1;   /* open image for first time    */
            access = 2;    /* read/write access            */
            break;
        case  CURNT:            /* write at current origin      */
            openflg = 0;   /* do not need to open image    */
            break;
        case  NEXT:             /* get next origin before writing  */
            oldss = *ssbuf;
            next_wrt(*nlti,*nlbuf,*nsbuf,slbuf,ssbuf);
            if ( oldss != *ssbuf)
            {
                openflg = 2;        /* need to reopen image */
                access = 2;         /* read/write access    */
            }
            else
                openflg = 0;        /* don't need to reopen image */
            break;
    }  /* end of switch construct */
    
    
    
    /* Make sure that the image file is opened at the correct location.  */
    /* open_img() may or may not open the image at a different location. */
    
    status = open_img(fdesc,host,openflg,bands,*ssbuf,*nlti,*nsav,
                      *nlbuf,access,dtype,*wtype);
    if (status == E_FAIL)
    {
        c_errmsg("Error returned from open_img","put_buf-call",&nonfatal);
        return(E_FAIL);
    }
    
    
    /* Remove any gaps in the data due to the fact that nsav may be less */
    /* than nsbuf.                                                       */
    
    pack_buf(buffer,*nsbuf,*dtype,*nlav,*nsav);
    
    
    /* Write the available data to the image.                            */
    
    nlwrt = *nlav;
    nswrt = c_ewrite(fdesc,band,slbuf,buffer,&nlwrt);
    if (nswrt == E_FAIL)
    {
        c_errmsg("Error returned from ewrite","put_buf-call",&nonfatal);
        return(E_FAIL);
    }
    
    
    return(E_SUCC);
    
}    /*  end of put_buf()   */

/*
************************************************************************
NAME: err_put

PURPOSE: error check the input parameters to put_buf()

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1           12/89   T. Baltzer   
    1.1         02/91   B.Ailts         Standardized error messages


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

*****************************************************************
        arguments:
name            description
----            -----------
nlbuf           total number of lines in 'buffer' (input)
nsbuf           total number of samples in 'buffer' (input)
nlav            number of actual valid lines in 'buffer' (input)
nsav            number of actual valid samples in 'buffer' (input)
slbuf           starting line of buffer (input)
ssbuf           starting sample of buffer (input)
nlti            number of lines in the total image (input)
nextflg         flag to indicate first call to put_buf(), and
                flag to indicate whether the buffer origin should be
                incremented before writing the data: (input)
                  CREAT (-1) this is the first call to put_buf() for
                             this image at this origin (NEW FILE)
                  INIT  (0)  this is the first call to put_buf() for
                             this image at this origin (EXISTING FILE)
                  CURNT (1)  don't move the buffer origin
                  NEXT  (2)  do move the origin to the next data block,
                             and reopen the image when appropriate
band            number of the band within the image to write (input)
bands           number of bands within the image (input)
dtype           data type of buffer
wtype           data type that data should be converted 
                to and written as (input).

************************************************************************
*/

static int err_put
(
    long    nlbuf,
    long    nsbuf,
    long    nlav,
    long    nsav,
    long    slbuf,
    long    ssbuf,
    long    nlti,
    long    nextflg,
    long    band,
    long    bands,
    long    dtype,
    long    wtype
)
{
    
    long    nonfatal = 1; /* nonfatal error status           */
    
    
    /* Begin by checking the values of the individual parameters.     */
    
    if ( nlbuf < 1 || nsbuf < 1 || nlav < 1 || nsav < 1 )
    {
        c_errmsg("Invalid buffer dimensions","put_buf-bufdim",&nonfatal);
        return(E_FAIL);
    }
    if ( slbuf < 1 || ssbuf < 1 )
    {
        c_errmsg("Invalid write origon","put_buf-worig",&nonfatal);
        return(E_FAIL);
    }
    if ( dtype < 0 || dtype > 4 || wtype < 0 || wtype > 4 )
    {
        c_errmsg("Invalid data type specified","put_buf-dtype",&nonfatal);
        return(E_FAIL);
    }
    if (  nlti < 1 || nextflg < -1 || nextflg > 2 || 
          band < 1 || bands < 1 )
    {
        c_errmsg("Invalid input parameter","put_buf-input",&nonfatal);
        return(E_FAIL);
    }
    
    /* Make sure the specified actual value dim do not exceed the     */
    /* specified buffer size dimensions.                              */
    
    if ( nlav > nlbuf || nsav > nsbuf )
    {
        c_errmsg("Actual data exceeds buffer","put_buf-overf",&nonfatal);
        return(E_FAIL);
    }
    
    /* Make sure the specified lines in buffer does not exceed the    */
    /* lines in the total image.                                      */
    
    if ( nlav > nlti )
    {
        c_errmsg("Lines in buffer exceeds lines in image","put_buf-lines",
                 &nonfatal);
        return(E_FAIL);
    }
    
    return(E_SUCC);
    
}    /*  end of err_put()   */

/*
*************************************************************************
NAME: next_wrt

PURPOSE:
        The origin can be thought of as the upper left-hand corner of
        a block of data.  An image is broken-up into blocks of data
        each having an origin.  This function is called to calculate
        the origin of the next block of data in the image TO WRITE
        ( the next WRITE origin).


PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
        The origin can be thought of as the upper left-hand corner of
        a block of data.  An image is broken-up into blocks of data
        each having an origin.  This function is called to calculate
        the origin of the next block of data in the image TO WRITE.
        This 'next origin' is derived on a column by row basis:
                ---------------------------
                |    1    |    4    |    7| 
                ---------------------------
                |    2    |    5    |    8| 
                ---------------------------
                |    3    |    6    |    9| 
                ---------------------------
        The function next_org() calculates the next origin for a
        read image, the function next_wrt() calculates the next origin
        for a write image.

*****************************************************************
        arguments:
        name            description
        ----            -----------
         nlti           number of lines in the total image (input)
         nlbuf          number of lines per data block (input)
         nsbuf          number of samples per data block (input)
         slbuf          line specification of previous origin (input)
                        line specification of next origin (output)
         ssbuf          sample specification of previous origin (input)
                        sample specification of next origin (output)

************************************************************************
*/

static int next_wrt
(
    long nlti,
    long nlbuf,
    long nsbuf,
    long *slbuf,
    long *ssbuf
)
{
    
    
    if ( *slbuf + nlbuf <= nlti )    /* bottom of the current column?    */
    {                             /* no, so                           */
        *slbuf = *slbuf + nlbuf;      /* get next line in current column  */
    }
    else                  /* already at the bottom of the current column */ 
    {
        *ssbuf = *ssbuf + nsbuf;      /* go to the top of the next column */
        *slbuf = 1;
    }
    
    return(E_SUCC);
    
}    /*  end of next_wrt()   */

/*
*************************************************************************
NAME: pack_buf

PURPOSE: shift the data in the buffer such that all of the valid data
         is in contiguous memory locations.

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        such that all of the valid data will lie in contiguous memory
        locations within the buffer.  The buffer has a total of 'nlbuf'
        lines and 'nsbuf' samples.  The actual valid data lies within
        a 'nlav' by 'nsav' 2-D rectangle.  This function shifts the 
        actual valid data to the front contiguous memory locations within
        buffer.  Since 'C' stores 2-D arrays by row, data will only be 
        shifted in the event that 'nsbuf' and 'nsav' are not equal.
        The function of pack_buf() is the reverse of that of shft_buf().


*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        nsbuf           number of samples in the total buffer (input)
        dtype           data  type (input)
        nlav            number of valid lines of data in buffer (input)
        nsav            number of valid samples of data in buffer (input)

************************************************************************
*/

static void pack_buf
(
    unsigned char *buffer,
    long nsbuf,
    long dtype,
    long nlav,
    long nsav
)
{
    
    
    switch (dtype)
    {
        case EREAL:
            pack_flt((float *)buffer,nsbuf,nlav,nsav);
            break;
        case ELONG:
            pack_lng((long*)buffer,nsbuf,nlav,nsav);
            break;
        case EWORD:
            pack_wrd((short *)buffer,nsbuf,nlav,nsav);
            break;
        case EBYTE:
            pack_byt(buffer,nsbuf,nlav,nsav);
            break;
    }
    
}     /*  end of pack_buf()   */

/*
************************************************************************
NAME: pack_byt

PURPOSE: shift the data in 'buffer' such that all of the valid data
         lies in contiguous memory locations (where buffer is CHAR
         data).

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        such that all of the valid data will lie in contiguous memory
        locations at the front of buffer.  The buffer is 'nlbuf' lines
        by 'nsbuf' samples big.  The actual valid data within buffer
        lies within a 2-D array which is 'nlav' lines by 'nsav' samples
        big.  Since 'C' stores arrays by row, if 'nsav' is less than
        'nsbuf', gaps in the data will result.  These 'gaps' will be
        removed by this function.

*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        nsbuf           number of samples in the total buffer (input)
        nlav            number of valid lines of data in buffer (input)
        nsav            number of valid samples of data in buffer(input)

************************************************************************
*/

static int pack_byt
(
    unsigned char *buffer,
    long nsbuf,
    long nlav,
    long nsav
)
{
    
    long    i;              /* index counter */
    long    j;              /* index counter */
    unsigned char   *froma;         /* 'from address' */
    unsigned char   *toadd;         /* 'to address'   */
    unsigned char   *frtmp;         /* temporary address */
    
    
    /* If no gaps in the data is present, return without doing anything. */
    
    if ( nsav == nsbuf )
        return(E_SUCC);
    
    
    /* With the following indexing scheme, one can think of the counters */
    /* j and i as indexes to the 'from location' of buffer:              */
    /*   ..... =  buffer[j][i]      (thinking of buffer as a 2-D array)  */
    /* The 'to address' of buffer is incremented by one each time.       */
    
    toadd = buffer;
    frtmp = buffer;
    for (j = 0; j < nlav; j++)
    {
        froma = frtmp;
        frtmp += nsbuf;
        for (i = 0; i < nsav; i++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    return(E_SUCC);
    
}     /*  end of pack_byt()   */

/*
************************************************************************
NAME: pack_flt

PURPOSE: shift the data in 'buffer' such that all of the valid data
         lies in contiguous memory locations (where buffer is FLOAT
         data).

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        such that all of the valid data will lie in contiguous memory
        locations at the front of buffer.  The buffer is 'nlbuf' lines
        by 'nsbuf' samples big.  The actual valid data within buffer
        lies within a 2-D array which is 'nlav' lines by 'nsav' samples
        big.  Since 'C' stores arrays by row, if 'nsav' is less than
        'nsbuf', gaps in the data will result.  These 'gaps' will be
        removed by this function.

*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        nsbuf           number of samples in the total buffer (input)
        nlav            number of valid lines of data in buffer (input)
        nsav            number of valid samples of data in buffer(input)

************************************************************************
*/

static int pack_flt
(
    float *buffer,
    long  nsbuf,
    long  nlav,
    long  nsav
)
{
    
    long    i;              /* index counter */
    long    j;              /* index counter */
    float   *froma;         /* 'from address' */
    float   *toadd;         /* 'to address'   */
    float   *frtmp;         /* temporary address */
    
    
    /* If no gaps in the data is present, return without doing anything. */
    
    if ( nsav == nsbuf )
        return(E_SUCC);
    
    
    /* With the following indexing scheme, one can think of the counters */
    /* j and i as indexes to the 'from location' of buffer:              */
    /*   ..... =  buffer[j][i]      (thinking of buffer as a 2-D array)  */
    /* The 'to address' of buffer is incremented by one each time.       */
    
    toadd = buffer;
    frtmp = buffer;
    for (j = 0; j < nlav; j++)
    {
        froma = frtmp;
        frtmp += nsbuf;
        for (i = 0; i < nsav; i++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    return(E_SUCC);
    
}     /*  end of pack_flt()   */

/*
************************************************************************
NAME: pack_lng

PURPOSE: shift the data in 'buffer' such that all of the valid data
         lies in contiguous memory locations (where buffer is LONG
         data).

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        such that all of the valid data will lie in contiguous memory
        locations at the front of buffer.  The buffer is 'nlbuf' lines
        by 'nsbuf' samples big.  The actual valid data within buffer
        lies within a 2-D array which is 'nlav' lines by 'nsav' samples
        big.  Since 'C' stores arrays by row, if 'nsav' is less than
        'nsbuf', gaps in the data will result.  These 'gaps' will be
        removed by this function.

*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        nsbuf           number of samples in the total buffer (input)
        nlav            number of valid lines of data in buffer (input)
        nsav            number of valid samples of data in buffer(input)

************************************************************************
*/

static int pack_lng
(
    long *buffer,
    long nsbuf,
    long nlav,
    long nsav
)
{
    
    long    i;              /* index counter */
    long    j;              /* index counter */
    long    *froma;         /* 'from address' */
    long    *toadd;         /* 'to address'   */
    long    *frtmp;         /* temporary address */
    
    
    /* If no gaps in the data is present, return without doing anything. */
    
    if ( nsav == nsbuf )
        return(E_SUCC);
    
    
    /* With the following indexing scheme, one can think of the counters */
    /* j and i as indexes to the 'from location' of buffer:              */
    /*   ..... =  buffer[j][i]      (thinking of buffer as a 2-D array)  */
    /* The 'to address' of buffer is incremented by one each time.       */
    
    toadd = buffer;
    frtmp = buffer;
    for (j = 0; j < nlav; j++)
    {
        froma = frtmp;
        frtmp += nsbuf;
        for (i = 0; i < nsav; i++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    return(E_SUCC);
    
}     /*  end of pack_lng()   */

/*
************************************************************************
NAME: pack_wrd

PURPOSE: shift the data in 'buffer' such that all of the valid data
         lies in contiguous memory locations (where buffer is SHORT
         data).

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           12/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
        This function will take the data within the buffer and shift it
        such that all of the valid data will lie in contiguous memory
        locations at the front of buffer.  The buffer is 'nlbuf' lines
        by 'nsbuf' samples big.  The actual valid data within buffer
        lies within a 2-D array which is 'nlav' lines by 'nsav' samples
        big.  Since 'C' stores arrays by row, if 'nsav' is less than
        'nsbuf', gaps in the data will result.  These 'gaps' will be
        removed by this function.

*****************************************************************
        arguments:
        name            description
        ----            -----------
        buffer          data buffer under consideration (input/output)
        nsbuf           number of samples in the total buffer (input)
        nlav            number of valid lines of data in buffer (input)
        nsav            number of valid samples of data in buffer(input)

************************************************************************
*/

static int pack_wrd
(
    short *buffer,
    long  nsbuf,
    long  nlav,
    long  nsav
)
{
    
    long    i;              /* index counter */
    long    j;              /* index counter */
    short   *froma;         /* 'from address' */
    short   *toadd;         /* 'to address'   */
    short   *frtmp;         /* temporary address */
    
    
    /* If no gaps in the data is present, return without doing anything. */
    
    if ( nsav == nsbuf )
        return(E_SUCC);
    
    
    /* With the following indexing scheme, one can think of the counters */
    /* j and i as indexes to the 'from location' of buffer:              */
    /*   ..... =  buffer[j][i]      (thinking of buffer as a 2-D array)  */
    /* The 'to address' of buffer is incremented by one each time.       */
    
    toadd = buffer;
    frtmp = buffer;
    for (j = 0; j < nlav; j++)
    {
        froma = frtmp;
        frtmp += nsbuf;
        for (i = 0; i < nsav; i++)
        {
            *toadd = *froma;
            toadd++;
            froma++;
        }
    }
    
    return(E_SUCC);
    
}     /*  end of pack_wrd()   */
