/*********************   Label Services Subsystem   ***************************
FUNCTION:            lsclos
                
PURPOSE:             Close an associated file.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           04/86    C. Greenhagen       initial development
           07/86    K. Gacke            newLAS development
           12/87    B. Ailts            change include directory specifications
                                        move bridge routines to seperate file
                                        replace DESC arguments with char strings
                                        use raw 'C' types
   5.0     04/88    D. Hollaren         changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passes error code by address
                                        return values E_SUCC/E_FAIL
                                        added dk routines to create a file
  5.1     11/88    B. Ailts             Replaced las.h with worgen.h so these
                                        routines can be placed in world
  5.2     11/88    B. Ailts             Now does not create a new version when
                                        compacting a file
  5.3     07/90    D. VanderZee         Standardized error handling
  5.4     01/91    K. Gacke             Changed i/o to buffered i/o for sun --
                                        sun has a problem with direct i/o when
                                        the close takes to long to be executed
  5.5     05/90    B. Ailts             Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  5.6     09/92    T. Mittan            Type cast constant 0 to (char *) to 
                                        eliminate warnings on compile.
  5.7     03/93    D. Etrheim           Changed pmode to 666 for file open.
                                        This will allow users to use umask
                                        appropriately. Execute permision is not
                                        needed for any files created by LAS.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT              LAS

ALGORITHM 
There are three ways that an associated file can be closed (normal, crunch, and
delete).
Case 0:
    Normal close of the file
    report any errors closing

Case 1:
    Seek to the beginning of the file
    Open a temporary file
    For each record in the assoicated file
        read the header information
        check to see if it has the "DELETED" flag
        Skip any records that are deleted or
        read the data and write it to the temp output file
   If the temp output file is not empty
        rename it to input file name
   else
        delete the associated and temp files

Case 2:
    Delete the associated file
    Report any errors deleting the file.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "tae.h"
#include "vll.h"
#include "worgen.h"
#include "util.h"
#include "labserv.h"

static int lsclose
(
    int fd,                 /* I: File descriptor of opened file */
    const char *hostname,   /* I: host file name */
    int action              /* I: action flag: 0=close, 1=crunch, 2=delete */
)
{
    long pmode = 0666;
    int  clen, dlen;
    long total_len, empty, fdtemp, flags, more, nbytes;
    long buf_size = 1024;
    char tempfile[CMLEN];
    char *buffer;
    char dtype[TYPL];
    char header[HDRL];
    char *key;
    char *ptr;
    int  temp;

    switch (action)
    {
        case 0:
            /* normal closing action */
            if (close(fd) != 0)
            {
                c_lasmsg(FALSE,"lsclos-close",
                         "Error closing label services file");
                return(E_FAIL);
            }
            break;

        case 1:
            /* close the file, eliminating deleted records */
            if (lseek(fd, 0, SEEK_SET) != 0)
            {
                c_lasmsg(FALSE,"lsclos-seek",
                         "Error seeking in label services file");
                return(E_FAIL);
            }

            strcpy(tempfile,hostname);
            strcat(tempfile,"_temp");

            /* open file for a temporary copy */
            flags = O_WRONLY | O_TRUNC | O_APPEND | O_CREAT;
            fdtemp = open (tempfile, flags, pmode);
            if (fdtemp == -1)
            {
                c_lasmsg(FALSE,"lsclos-temp","Error opening temporary file %s",
                         tempfile);
                return (E_FAIL);
            }

            if ((buffer = malloc((unsigned)buf_size)) == NULL)
            {
                c_lasmsg(FALSE,"lsclos-alloc",
                         "Error allocating dynamic memory");
                return(E_FAIL);
            }

            empty = 1;
            more = 1;
            while (more)
            {
                nbytes = read(fd,header,HDRL);
                if (nbytes == 0)
                    break;

                if (nbytes == -1)
                {
                    free(buffer);
                    c_lasmsg(FALSE,"lsclos-read","Error reading header "
                             "record from label services file");
                    return(E_FAIL);
                }

                clen = dlen = 0;
                if (((ptr = strchr(header,'/')) != NULL) && 
                    (ptr - header < LENL))
                    sscanf(header,"%d/%d%s",&clen,&dlen,dtype); 
                else
                    sscanf(header,"%d%s",&dlen,dtype); 

                total_len = clen + dlen;

                /* check if the key for this record is "DELETED" */
                key = squeeze(header+LENL+TYPL,(long)strlen(header+LENL+TYPL));
                temp = strcmp(key,"DELETED");
                free (key);
                key = NULL;

                if (temp != 0)          /* copy record  */
                {                                       /* to new file  */
                    if (total_len > buf_size)
                    {
                        free(buffer);
                        buf_size = total_len;

                        if ((buffer = malloc((unsigned)buf_size)) == NULL)
                        {
                            c_lasmsg(FALSE,"lsclos-alloc",
                                     "Error allocating dynamic memory");
                            return(E_FAIL);
                        }
                    }

                    nbytes = read(fd, buffer, total_len);
                    if (nbytes != total_len)
                    {
                        free(buffer);
                        c_lasmsg(FALSE,"lsclos-read","Error reading record "
                                 "from label services file");
                        return(E_FAIL);
                    }

                    if ((write(fdtemp,header,HDRL) != HDRL) ||
                        (write(fdtemp,buffer,nbytes) != nbytes))
                    {
                        free(buffer);
                        c_lasmsg(FALSE,"lsclos-write",
                              "Error writing to temporary file %s", tempfile);
                        return(E_FAIL);
                    }

                    empty = 0;
                }
                else                                    /* skip record  */ 
                {
                    if (lseek(fd, (long)total_len, SEEK_CUR) == -1)
                    {
                        free(buffer);
                        c_lasmsg(FALSE,"lsclos-seek",
                                 "Error seeking in label services file");
                        return(E_FAIL);
                    }
                }
            }

            free(buffer);

            if (close(fd) != 0)
            {
                c_lasmsg(FALSE,"lsclos-close",
                         "Error closing label services file");
                return(E_FAIL);
            }

            if (close(fdtemp) != 0)
            {
                c_lasmsg(FALSE,"lsclos-close","Error closing temporary file %s",
                         tempfile);
                return(E_FAIL);
            }

            if (unlink(hostname) == -1)
            {
                c_lasmsg(FALSE,"lsclos-delete",
                         "Error deleting label services file %s",hostname);
                return(E_FAIL);
            }

            if (!empty)                                 /***  delete file   ***/
            {
                if (rename(tempfile,hostname) != E_SUCC)
                {
                    c_lasmsg(FALSE,"lsclos-rename",
                            "Error renaming file %s to %s",tempfile,hostname);
                    return(E_FAIL);
                }
            }
            else if (unlink(tempfile) == -1)
            {
                c_lasmsg(FALSE,"lsclos-delete",
                         "Error deleting label services file %s",hostname);
                return(E_FAIL);
            }

            break;

        case 2:
            /* delete file */
            if (unlink(hostname) == -1)
            {
                c_lasmsg(FALSE,"lsclos-delete",
                         "Error deleting label services file %s",hostname);
                return(E_FAIL);
            }
            break;
        default:
            c_lasmsg(FALSE,"lsclos-action","Illegal action: %d",action);
            return (E_FAIL);
    }

    return(E_SUCC);
}

/**
 * c_labclose:
 * @fd: I: File descriptor of opened label services file. 
 *         File descriptor is set by c_lsopen() and
 *         should not be altered.
 * @hostname: I: Host file name of the label services file.
 * @action: I: Action Flag: Valid values  are:
 *             0 =  Normal close.  
 *             1 =  Physically remove records marked for deletion.   
 *             2 =  Delete file.  
 *
 * The c_labclose() call is used to close the label services file.  If the
 * action flag is set to 1, the records that are not marked for deletion are
 * copied into a temporary file.  After all valid records have been copied,
 * the temporary file is renamed to the original label services filename. 
 *
 * Returns:      E_SUCC   =  successful completion,
 *               E_FAIL   =  operation failed
 */ 
int c_labclose
(
    int fd,               /* I: File descriptor of opened file */
    const char *hostname, /* I: host file name */
    int action            /* I: action flag: 0=close, 1=crunch, 2=delete */
)
{
    int sig_mask;
    int status;

    c_sigblk(&sig_mask);

    status = lsclose(fd, hostname, action);

    c_sigunblk(&sig_mask);

    return (status);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_lsclos:
 * @fd: I: File descriptor of opened label services file.
 *         File descriptor is set by c_lsopen() and
 *         should not be altered.
 * @hostname: I: Host file name of the label services file.
 * @action: I: Action flag.
 *             Valid values are:
 *             0 =  Normal close.   
 *             1 =  Physically remove records marked for deletion.
 *             2 =  Delete file.  
 *
 * The c_lsclos() call is used to close the label services file.  If the
 * action flag is set to 1, the records that are not marked for deletion are
 * copied into a temporary file.  After all valid records have been copied, the
 * temporary file is renamed to the original label services filename. 
 *
 * Deprecated.  Use c_labclos().
 *
 * Returns:          E_SUCC   =  successful completion,
 *                   E_FAIL   =  operation failed
 */
int c_lsclos
(
    const long *fd,         /* I: File descriptor of opened file */
    const char *hostname,   /* I: host file name */
    const long *action      /* I: action flag: 0=close, 1=crunch, 2=delete */
)
{
    int sig_mask;
    int status;

    c_sigblk(&sig_mask);

    status = lsclose(*fd, hostname, *action);

    c_sigunblk(&sig_mask);

    return (status);
}

#endif /* Obsolete interfaces */
