/*********************   Label Services Subsystem   *********************
FUNCTION:       lsdel

PURPOSE:        Delete specified record in the file.
                If the key is not blank, read the header and key area of
                each record of the file, looking for a matching key;
                write "DELETED" in the key area of the record to indicate
                that the record is to be deleted.

                If the key is blank or null, ???
                Don't know just how to handle duplicate keys -
                How do we delete the right record of many with duplicate keys?

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           04/86    C. Greenhagen       initial development
           07/86    K. Gacke            newLAS development
           12.87    B. Ailts            change include directory specifications
                                        place bridge routines in seperate
                                        routines
                                        replace DESC arguments with char strings
                                        use raw 'C' types
  5.0      04/88    D. Hollaren         change include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed error code by address
                                        return E_SUCC/E_FAIL
  5.1     11/88    B. Ailts             Replaced las.h with worgen.h so these
                                        routines can be placed in world
  5.2     07/90    D. VanderZee         Standardized error handling
  5.4     01/91    K. Gacke             Changed i/o to buffered i/o for sun --
                                        sun has a problem with direct i/o when
                                        the close takes to long to be executed
  5.5     05/90    B. Ailts             Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  5.6     09/92    T. Mittan            Type cast constant 0 to (char *) to 
                                        eliminate warnings on compile. Also
                                        placed vll.h after tae.h to eliminate
                                        min redefine warning.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Read each record in the associated file until a key match is found
    If the record key matches the specified key
        write "DELETED" in the key field
        backup up the file pointer to the begining of the record

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <string.h>
#include <sys/types.h>
#include "tae.h"
#include "vll.h"
#include "worgen.h"
#include "util.h"
#include "labserv.h"

static int lsdel
(
    int fd,             /* I: file descriptor */
    const char *inkey   /* I: key of record to be deleted */
)
{
    int   charlen, datalen;
    long  nbytes;
    long  offset;
    long  nonfatal = 1;
    short keymatch = FALSE;
    char  delkey[KEYL];
    char  header[HDRL],dtype[TYPL];
    char  *ptr, *out_key;

    /* create the deleted key */
    sprintf(delkey,"%-*s",KEYL-1,"DELETED");

    while (!keymatch)
    {
        if ((nbytes = read (fd, header, HDRL)) == 0)
        {
            c_errmsg("EOF detected without a key match","lsdel-eof",&nonfatal);
            return(E_FAIL);
        }

        if (nbytes != HDRL)
        {
            c_errmsg("Error reading header record of label services file",
                     "lsdel-read",&nonfatal);
            return(E_FAIL);
        }

        charlen = datalen = 0;
        if (((ptr = strchr(header,'/')) != NULL) && (ptr - header < LENL))
            sscanf(header,"%d/%d%s",&charlen,&datalen,dtype);
        else
            sscanf(header,"%d%s",&datalen,dtype); 

        out_key = squeeze(header+LENL+TYPL,(long)strlen(header+LENL+TYPL));

        if ((strlen(inkey) == 0) || (strcmp(inkey," ") == 0) 
                                  || (strcmp(inkey,out_key) == 0))
            keymatch = TRUE;

        if (strcmp(out_key,"DELETED") == 0)
            keymatch = FALSE;

        free(out_key);
        out_key = NULL;

        if (keymatch) 
        {
            offset = -KEYL;
            if (lseek(fd, offset, SEEK_CUR) == -1) /* reposition at key */
            {
                c_errmsg("Error seeking in label services file","lsdel-seek",
                         &nonfatal);
                return(E_FAIL);
            }

            if (write(fd,delkey,KEYL) != KEYL) /* mark record deleted */
            {
                c_errmsg("Error writing deletion flag to label services file",
                         "lsdel-delflag",&nonfatal);
                return(E_FAIL);
            }

            offset = -HDRL;
            if (lseek(fd, offset, SEEK_CUR) == -1) /* reposition at record */
            {
                c_errmsg("Error seeking in label services file","lsdel-seek",
                         &nonfatal);
                return(E_FAIL);
            }
        }
        else
        {/* go to next record */
            offset = charlen + datalen;
            if (lseek(fd, offset, SEEK_CUR) == -1)
            {
                c_errmsg("Error seeking in label services file","lsdel-seek",
                         &nonfatal);
                return(E_FAIL);
            }
        }
    }

    return(E_SUCC);
}

/**
 * c_labdelete:
 * @fd: I: File descriptor of an opened label services file.
 *         The file descriptor is set by c_lsopen() and
 *         should not be altered.
 * @inkey: I: Key of record to be deleted.  Spaces may be
 *            embedded in the key; however, right justified
 *            spaces are ignored.  If the key is NULL or a blank,
 *            the current record within the file is marked for
 *            deletion.  If a value is  specified for the key, the
 *            first record found with the same key is marked for 
 *            deletion.
 *
 * The c_labdelete() call is used to mark a record for deletion.
 * The c_labdelete() function requires the label services file
 * to be opened for read/write access. A record is marked for
 * deletion by replacing the key with the string "DELETED"
 * ("DELETED" is an invalid key specification).
 *
 * Returns:   E_SUCC   =  successful completion, 
 *            E_FAIL   =  operation failed
 */
int c_labdelete
(
    int fd,             /* I: file descriptor */
    const char *inkey   /* I: key of record to be deleted */
)
{
    int sig_mask;
    int status;

    c_sigblk(&sig_mask);

    status = lsdel(fd, inkey);

    c_sigunblk(&sig_mask);

    return (status);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_lsdel:
 * @fd: I: File descriptor of an opened label services file.
 *         File descriptor is set by c_lsopen() and
 *         should not be altered.
 * @inkey: I: Key of record to be deleted.  Spaces may be
 *            embedded in the key; however, right justified
 *            spaces are ignored.  If the key is NULL or a blank,
 *            the current record within the file is marked for
 *            deletion.  If a value is specified for the key, the
 *            first record found with the same key is marked for 
 *            deletion.
 *
 *
 * The c_lsdel() call is used to mark a record for deletion.
 * The c_labdelete() function requires the label services file to
 * be opened for read/write access. A record is marked for
 * deletion by replacing the key with the string "DELETED"
 * ("DELETED" is an invalid key specification).
 *
 * Deprecated. Use c_labdelete().
 *
 * Returns:       E_SUCC  =  successful completion,
 *                E_FAIL  =  operation failed
 */
int c_lsdel
(
    const long *fd,     /* I: file descriptor */
    const char *inkey   /* I: key of record to be deleted */
)
{
    int sig_mask;
    int status;

    c_sigblk(&sig_mask);

    status = lsdel(*fd, inkey);

    c_sigunblk(&sig_mask);

    return (status);
}

#endif /* Obsolete interfaces */
