/*********************   Label Services Subsystem   ***************************
FUNCTION:       lsopen

PURPOSE:        Open an associated file.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           04/86    C. Greenhagen       initial development
           07/86    K. Gacke            newLAS development
           10/86    T. Butzer           delete file on write if exists (VMS)
           12.87    B. Ailts            change include file directory spec
                                        replace DESC arguments with char strings
                                        use raw 'C' types
                                        place bridge routines in a seperate file
  5.0      04/88    D. Hollaren         changed include spec to las
  5.1     11/88    B. Ailts             Replaced las.h with worgen.h so these
                                        routines can be placed in world
                                        Replaced c_delfil with delete so this
                                        routine can be placed in world
  5.2     07/90    D. VanderZee         Standardized error handling
  5.3     01/91    K. Gacke             Changed i/o to buffered i/o for sun --
                                        sun has a problem with direct i/o when
                                        the close takes to long to be executed
  5.4     03/93    D. Etrheim           Changed pmode to 666 for file open.
                                        This will allow users to use umask 
                                        appropriately. Execute permision is not
                                        needed for any files created by LAS.
  5.5     05/96    D. Etrheim           On update mode, added check for file 
                                        existance before assigning O_CREAT flag.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Set the open flags according to the specified mode
In VMS
    If writing to the file and the file already exists
        delete the file
Open the file

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "tae.h"
#include "worgen.h"
#include "vll.h"
#include "labserv.h"

/* returns the opened file descriptor, or E_FAIL */
/**
 * c_labopen:
 * @hname: I: TAE host filename of the label services file.
 * @rwmode: I: Type of read/write access the file is to be opened
 *             with.  Valid values are: 
 *             Read,
 *             Write (existing file is overwritten),
 *             Write (append to end of existing file),
 *             Read/write (replace a record).
 *
 * The c_labopen() call is used to open the associated label services file. 
 *
 * Returns:     E_SUCC   =  successful completion,
 *              E_FAIL   =  operation failed
 */
int c_labopen
(
    const char *hname,  /* I: TAE host file name */
    int rwmode          /* I: File access: READ, WRITE, APPEND, or RW */
)
{
    int pmode = 0666;
    int flags;
    int fd;

    if (rwmode == READ) /* set flags: */
        flags = O_RDONLY; /* read only */

    else if (rwmode == WRITE) /* write only - append */
    {
        flags = O_WRONLY | O_TRUNC | O_APPEND | /*   may create new file      */
                O_CREAT;                 /*   existing file truncated  */
    }

    else if (rwmode == APPEND) /* write only - append */
        flags = O_WRONLY | O_APPEND | O_CREAT; /*   may create new file      */

    else if (rwmode == RW) /* read/write */
    {
        if (access(hname,F_OK) == 0)
           flags = O_RDWR; 
        else
           flags = O_RDWR | O_CREAT; /* may create new file */
    }
    else
    {
        c_lasmsg(FALSE,"lsopen-mode","Invalid open mode specified: %d",rwmode);
        return(E_FAIL);
    }

    fd = open(hname, flags, pmode);
    if (fd == -1)
    {
        c_lasmsg(FALSE,"lsopen-open","Error opening file %s",hname);
        return(E_FAIL);
    }

    return(fd);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_lsopen:
 * @fd: O: File descriptor to be used by other label services functions. 
 *         File descriptor is set  by c_lsopen() and should not be altered.
 * @hname: I: TAE host file name
 * @rwmode: I: Type of read/write access the file is to be opened with.
 *              Valid values are: 
 *              Read,
 *              Write (existing file is overwritten),
 *              Write (append to end of existing file),
 *              Read/write (replace a record).
 *
 * The c_lsopen() call is used to open the  associated label services file. 
 *
 * Deprecated. Use c_labopen().
 *
 * Returns:     E_SUCC   =  successful completion,
 *              E_FAIL   =  operation failed
 */
int c_lsopen
(
    long *fd,           /* O: File descriptor of open file */
    const char *hname,  /* I: TAE host file name */
    const long *rwmode  /* I: File access: READ, WRITE, APPEND, or RW */
)
{
    *fd = c_labopen(hname, *rwmode);

    if (*fd == E_FAIL)
        return E_FAIL;

    return E_SUCC;
}

#endif /* Obsolete interfaces */
