/*********************   Label Services Subsystem   *********************
FUNCTION:       lsrepl

PURPOSE:        Replace (rewrite) the record just read in the associated file.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           04/86    C. Greenhagen       initial development
           07/86    K. Gacke            newLAS development
           12/87    B. Ailts            change the include directory spec
                                        use raw 'C' types
                                        replace DESC arguments with char strings
                                        place bridge routines in a seperate file
  5.0      04/88    D. Hollaren         change include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed error code by address
                                        return E_SUCC/E_FAIL
  5.1     11/88    B. Ailts             Replaced las.h with worgen.h so these
                                        routines can be placed in world
  5.2     07/90    D. VanderZee         Standardized error handling
  5.3     01/91    K. Gacke             Changed i/o to buffered i/o for sun --
                                        sun has a problem with direct i/o when
                                        the close takes to long to be executed
  5.4     05/90    B. Ailts             Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  5.5     10/91    T. Mittan            Added fseek before return for sun's.
                                        This prevents c_lread from reading past
  5.6     09/92    T. Mittan            Typecast constant 0 to (char *) to 
                                        eliminate warnings on compile.
                                        the end of the file.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Truncate the key field if it's to long
Position to the start of the previous record
read header information
Make sure the length of the data to be re-written is the same as
the record length.
Backup the file pointer to the beginning of the record
Update the header information
Write the new header information
Write any character information
Write any other data information

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include "tae.h"
#include "worgen.h"
#include "vll.h"
#include "util.h"
#include "lasfileio.h"
#include "labserv.h"
#include "local_protos.h"

static int lsreplace
(
    int fd,             /* I: file descriptor */
    const char *key,    /* I: key to be written */
    int clen,           /* I: number of char bytes to be written */
    int dlen,           /* I: number of data bytes to be written */
    const char *cbuf,   /* I: char buffer to be written */
    const void *dbuf,   /* I: data buffer to be written */
    const char *dtype   /* I: data type of data record (B,I2,I4,R4,R8) */
)
{
    int  charlen, datalen;
    long offset;
    long nonfatal = 1;
    char header[HDRL], len[LENL], odtype[TYPL];
    char *out_key;
    char *ptr;
    char local_key[KEYL];
    
    strncpy(local_key, key, KEYL);
    local_key[KEYL - 1] = '\0';
    if (strlen(key) >= KEYL)
    {
        c_errmsg("Key length too long.  Key has been truncated.","lsrepl-key",
                 &nonfatal);
    }

    offset = -(clen + dlen + HDRL);/* position at start of record  */
    if (lseek(fd, offset, SEEK_CUR) == -1)/* before current position */
    {
        c_errmsg("Error seeking in label services file","lsrepl-seek",
                 &nonfatal);
        return(E_FAIL);
    }

    if (read (fd, header, HDRL) == -1)
    {
        c_errmsg("Error reading header record of label services file",
                 "lsrepl-read",&nonfatal);
        return(E_FAIL);
    }

    charlen = datalen = 0;
    if (((ptr = strchr(header,'/')) != NULL) && (ptr - header < LENL))
        sscanf(header,"%d/%d%s",&charlen,&datalen,odtype);
    else
        sscanf(header,"%d%s",&datalen,odtype);

    if (charlen + datalen != clen + dlen)
    {
        c_errmsg("Error replacing record within label services file",
                 "lsrepl-replace",&nonfatal);
        c_lasmsg(FALSE,"labserv-bytes",
                 "    %d bytes requested, record contains %d",
                 clen + dlen, datalen);
        return(E_FAIL);
    }

    out_key = squeeze(header+LENL+TYPL,KEYL - 1);
    if ((strlen(local_key) != 0) && (strcmp(local_key," ") != 0))
        sprintf(out_key,"%-*s",KEYL-1,local_key);

    if ((strlen(dtype) != 0) && (strcmp(dtype," ") != 0))
        sprintf(odtype,"%-*s",TYPL-1,dtype);

    if (clen > 0)
       sprintf(len,"%-d/%-d",clen,dlen);
    else
       sprintf(len,"%-d",dlen);

    sprintf(header,"%-*s%-*s%-*s",LENL,len,TYPL,odtype,KEYL-1,out_key);
    free(out_key);

    offset = -(HDRL);
    if (lseek(fd, offset, SEEK_CUR) == -1)
    {
        c_errmsg("Error seeking in label services file","lsrepl-seek",
                 &nonfatal);
        return(E_FAIL);
    }

    if (write(fd, header, HDRL) != HDRL)
    {
        c_errmsg("Error writing header to label services file","lsrepl-write",
                 &nonfatal);
        return(E_FAIL);
    }


    if (clen > 0)
    {
        if (write(fd, cbuf, clen) != clen)
        {
            c_errmsg("Error writing char record to label services file",
                     "lsrepl-write",&nonfatal);
            return(E_FAIL);
        }
    }

    if (dlen > 0)
    {
        /* determine the number of bytes per data element */
        int bytes_per_element = labserv_datasize(dtype);
        
        if (bytes_per_element == 0)
        {
            c_lasmsg(FALSE,"lswrit-dtype","%s is an unrecognized data type",
                     dtype);
            return(E_FAIL);
        }

        if (c_write (fd, dbuf, dlen, bytes_per_element) != dlen) /* data */
        {
            c_errmsg("Error writing data record to label services file",
                     "lsrepl-write",&nonfatal);
            return(E_FAIL);
        }
    }

    return(E_SUCC);
}

/**
 * c_labreplace:
 * @fd: I: File descriptor of an opened label services file.
 *         File descriptor is set by c_lsopen() and should not
 *         be altered.
 * @key: I: Key of the record to be replaced.  Spaces may be
 *          embedded in the key, however, right justified spaces
 *          are ignored. If the key is NULL or blank, the key
 *          of the record is unchanged.  If a value is specified
 *          for the key, the existing key is overwritten.
 * @clen: I: Number of bytes in the character portion of the record to be
 *           written to the label services file.
 * @dlen: I: Number of bytes in the data portion of the record to be written
 *           to the label services file.
 * @cbuf: I: Character buffer to be written to the character portion of
 *           the label services record.
 * @dbuf: I: Data buffer to be written to the data portion of the record
 *           to be written to the label services record.
 * @dtype:   I: Data type of the data portion of the data record.
 *              Valid data types are: 
 *              B  = unsigned byte data,
 *              I2 = signed two byte data,
 *              I4 = signed four byte data,
 *              R4 = four byte floating point data, 
 *              R8 = eight byte floating point data.
 *
 * The c_labreplace() call is used to replace the record just read in the
 * label services file.  The c_labreplace() function requires the file to be
 * opened with read/write access. 
 *
 * A key value of "DELETED" is invalid.
 *
 * The new record must contain the exact same number of bytes as the
 * existing record.  The character length and data length may be
 * modified as long as the total is the same.
 *
 * Returns:  E_SUCC   = successful completion,  
 *           E_FAIL   = operation failed
 */
int c_labreplace
(
    int fd,             /* I: file descriptor */
    const char *key,    /* I: key to be written */
    int clen,           /* I: number of char bytes to be written */
    int dlen,           /* I: number of data bytes to be written */
    const char *cbuf,   /* I: char buffer to be written */
    const void *dbuf,   /* I: data buffer to be written */
    const char *dtype   /* I: data type of data record (B,I2,I4,R4,R8) */
)
{
    int sig_mask;
    int status;

    c_sigblk(&sig_mask);

    status = lsreplace(fd, key, clen, dlen, cbuf, dbuf, dtype);

    c_sigunblk(&sig_mask);

    return (status);
}

#ifndef NO_OBSOLETE_INTERFACE
/**
 * c_lsrepl:
 * @fd: I: File descriptor of an opened label services file.
 *         File descriptor is set by c_lsopen() and should
 *         not be altered.
 * @key: I: Key of the record to be replaced.  Spaces may be
 *          embedded in the key, however, right justified
 *          spaces are ignored. If they key if NULL or blank,
 *          the key of the record is unchanged. If a value is
 *          specified for the key, the existing key is
 *          overwritten.
 * @clen: I: Number of bytes in the character portion of the record to be
 *           written to the label services file.
 * @dlen: I: Number of bytes in the data portion of the record to be
 *           written to the label services file.
 * @cbuf: I: Character buffer to be written to the character portion of
 *           the label services record.
 * @dbuf: I: Data buffer to be written to the data portion of the record
 *           to be written to the label services record.
 * @dtype: I: Data type of the data portion of the data record.
 *            Valid data types are: 
 *            B  = unsigned byte data,
 *            I2 = signed two byte data,
 *            I4 = signed four byte data,
 *            R4 = four byte floating point data, 
 *            R8 =  eight byte floating point data.
 *
 * The c_lsrepl() call is used to replace the record just read in the
 * label services file. The c_lsrepl() function requires the file to be
 * opened with read/write access. 
 *
 * A key value of "DELETED" is invalid.
 *
 * The new record must contain the exact same number of bytes as the
 * existing record.  The character length and data length may be
 *modified as long as the total is the same.
 *
 *
 * Deprecated.  Use c_labreplace().
 *
 * Returns:      E_SUCC   = successful completion,  
 *               E_FAIL   = operation failed
 */
int c_lsrepl
(
    const long *fd,     /* I: file descriptor */
    const char *key,    /* I: key to be written */
    const long *clen,   /* I: number of char bytes to be written */
    const long *dlen,   /* I: number of data bytes to be written */
    const char *cbuf,   /* I: char buffer to be written */
    const void *dbuf,   /* I: data buffer to be written */
    const char *dtype   /* I: data type of data record (B,I2,I4,R4,R8) */
)
{
    int sig_mask;
    int status;

    c_sigblk(&sig_mask);

    status = lsreplace(*fd, key, *clen, *dlen, cbuf, dbuf, dtype);

    c_sigunblk(&sig_mask);

    return (status);
}

#endif /* Obsolete interfaces */
