/*********************   Label Services Subsystem   *********************
FUNCTION:       lsstat

PURPOSE:        Ascertain file presence and requested access permissions.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           04/86    C. Greenhagen       initial development
           07/86    K. Gacke            newLAS development
           12/87    B. Ailts            place bridge routines in seperate file
                                        use raw 'C' types
                                        change include directory specifications
                                        replace DESC arguments with char strings
  5.0      04/88    D. Hollaren         change include spec to las
                                        return E_SUCC/E_FAIL
  5.1     11/88    B. Ailts             Replaced las.h with worgen.h so these
                                        routines can be placed in world

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Set the return status to E_FAIL if the specified mode can not be done

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <unistd.h>
#include "worgen.h"
#include "labserv.h"

/**
 * c_labstat:
 * @hostname: I: Host file name of the label services file.
 * @mode:     I: Type of access mode to check:
 *               F_OK = Tests to see if the file exists, 
 *               X_OK = Execute, 
 *               W_OK = Write (implies delete access), 
 *               R_OK = Read.
 *         
 * The c_labstat() call is used to ascertain file presence and
 * requested access permissions. A return status of E_SUCC
 * implies that test is valid.  For example: if mode=2 and a
 * status of E_SUCC is returned, the file has write access.
 * 
 * Returns:         E_SUCC   = successful completion,
 *                  E_FAIL   = operation failed (no message)
 */
int c_labstat
(
    const char *hostname,   /* I: host file name */
    int mode                /* I: type of access mode to check */
                            /*  F_OK = Tests to see if the file exists */
                            /*  X_OK = Execute */
                            /*  W_OK = Write (implies delete access) */
                            /*  R_OK = Read */
)
{
    if (access(hostname,mode) != 0)
        return(E_FAIL);

    return(E_SUCC);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_lsstat: 
 * @hostname: I: Host file name of the label services file.
 * @mode:    I: Type of access mode to check:
 *              0 = Tests to see if the file exists,
 *              1 = Execute,
 *              2 = Write (implies delete access),
 *              4 = Read.
 *
 * The c_lsstat() call is used to ascertain file presence and
 * requested access permissions. A return status of E_SUCC
 * implies that test is valid.  For example: if mode=2 and a
 * status of E_SUCC is returned, the file has write access.
 *
 * Deprecated.  Use c_labstat().
 *
 * Returns:           E_SUCC   = successful completion,
 *                    E_FAIL   = operation failed (no message)
 */
int c_lsstat
(
    const char *hostname,   /* I: host file name */
    const long *mode        /* I: type of access mode to check */
                            /*   0 = Tests to see if the file exists */
                            /*   1 = Execute */
                            /*   2 = Write (implies delete access) */
                            /*   4 = Read */
)
{
    if (access(hostname,*mode) != 0)
        return(E_FAIL);

    return(E_SUCC);
}

#endif /* Obsolete interfaces */
