/*********************   Label Services Subsystem   *********************
FUNCTION:       lswrit

PURPOSE:        Write a record to the associated file.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           04/86    C. Greenhagen       initial development
           07/86    K. Gacke            newLAS development
           12/87    B. Ailts            change include directory specifcations
                                        replace DESC arguments with cha. strings
                                        use raw 'C' types
                                        place bridge routines in seperate file
  5.0      04/88   D. Hollaren          change include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed error code by address
                                        return E_SUCC/E_FAIL
  5.1     11/88    B. Ailts             Replaced las.h with worgen.h so these
                                        routines can be placed in world
  5.2     07/90    D. VanderZee         Standardized error handling
  5.3     01/91    K. Gacke             Changed i/o to buffered i/o for sun --
                                        sun has a problem with direct i/o when
                                        the close takes to long to be executed
  5.4     05/90    B. Ailts             Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Truncate the key if it's to long
Go to the end of the file
Write the header information
Write the character information
Write the data information

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include "tae.h"
#include "worgen.h"
#include "vll.h"
#include "util.h"
#include "lasfileio.h"
#include "labserv.h"
#include "local_protos.h"


static int lswrite
(
    int fd,             /* I: file descriptor */
    const char *key,    /* I: key of record to be written */
    int clen,           /* I: number of char bytes to be written */
    int dlen,           /* I: number of data bytes to be written */
    const char *cbuf,   /* I: character buffer to be written */
    const void *dbuf,   /* I: data buffer to be written */
    const char *dtype   /* I: datatype of data record (B,I2,I4,R4,R8) */
)
{
    long nonfatal = 1;
    char header[HDRL];
    char len[LENL];
    char local_key[KEYL];

    strncpy(local_key, key, KEYL);
    local_key[KEYL - 1] = '\0';
    if (strlen(key) >= KEYL)
    {
        c_errmsg("Error: Key length too long.  Key has been truncated.",
                 "lswrit-key", &nonfatal);
    }

    /* append to the end of the file */
    if (lseek (fd, 0, SEEK_END) == -1)
    {
        c_errmsg("Error seeking in label services file","lswrit-seek",
                 &nonfatal);
        return(E_FAIL);
    }

    /* TBD - potential overflow bug here? */
    if (clen > 0)
        sprintf(len,"%-d/%-d",clen,dlen);
    else
        sprintf(len,"%-d",dlen);

    sprintf (header,"%-*s%-*s%-*s",LENL,len,TYPL,dtype,KEYL-1,local_key);
    if (write (fd, header, HDRL) != HDRL)
    {
        c_errmsg("Error writing header to label services file","lswrit-write",
                 &nonfatal);
        return(E_FAIL);
    }

    if (clen > 0)
    {
        if (write (fd, cbuf, clen) != clen)
        {
            c_errmsg("Error writing char record to label services file",
                     "lswrit-write",&nonfatal);
            return(E_FAIL);
        }
    }

    if (dlen > 0)
    {
        /* determine the number of bytes per data element */
        int bytes_per_element = labserv_datasize(dtype);
        
        if (bytes_per_element == 0)
        {
            c_lasmsg(FALSE,"lswrit-dtype","%s is an unrecognized data type",
                     dtype);
            return(E_FAIL);
        }

        if (c_write (fd, dbuf, dlen, bytes_per_element) != dlen) /* data */
        {
            c_errmsg("Error writing data record to label services file",
                     "lswrit-write",&nonfatal);
            return(E_FAIL);
        }
    }
    return(E_SUCC);
}

/**
 * c_labwrite:
 * @fd: I: File descriptor of an opened label services file.
 *         File descriptor is set by c_lsopen() and should not be altered.
 * @key: I: Key of record to be written.  Spaces may be embedded
 *          in the key; however, right justified spaces are ignored.
 *          If the key is NULL or a blank, the key written is a blank.
 * @clen: I: Number of bytes (not counting the NULL terminator)in the
 *           character portion of the record to be written to the label
 *           services file.
 * @dlen: I: Number of bytes in the data portion of the record to be written
 *           to the label services file.
 * @cbuf: I: Character buffer to be written to the character portion of the
 *           label services record.
 * @dbuf: I: Data buffer to be written to the data portion of the label
 *           services record.
 * @dtype: I: Datatype of data portion of the record.
 *            Valid datatypes are:  
 *            B  = unsigned byte data,
 *            I2 = signed two byte data,
 *            I4 = signed four byte data,
 *            R4 = four byte floating point data,
 *            R8 = eight byte floating point data.
 *
 * The c_labwrite() call is used to record to the associated label
 * services file. The c_labwrite() function requires the file to be
 * opened with write access.
 *
 * A key value of "DELETED" is invalid.
 *
 * Returns:    E_SUCC   = successful completion,               
 *             E_FAIL   = operation failed
 */
int c_labwrite
(
    int fd,             /* I: file descriptor */
    const char *key,    /* I: key of record to be written */
    int clen,           /* I: number of char bytes to be written */
    int dlen,           /* I: number of data bytes to be written */
    const char *cbuf,   /* I: character buffer to be written */
    const void *dbuf,   /* I: data buffer to be written */
    const char *dtype   /* I: datatype of data record (B,I2,I4,R4,R8) */
)
{
    int sig_mask;
    int status;

    c_sigblk(&sig_mask);

    status = lswrite(fd, key, clen, dlen, cbuf, dbuf, dtype);

    c_sigunblk(&sig_mask);
    return (status);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_lswrit:
 * @fd: I: File descriptor of an opened label services file.
 *         File descriptor is set by c_lsopen() and should not be altered.
 * @key: I: Key of record to be written.  Spaces may be embedded in
 *          the key; however, right justified spaces are ignored.  If
 *          the key is NULL or a blank, the key written is a blank.
 * @clen: I: Number of bytes (not counting the NULL terminator) in the
 *           character portion of the record to be written to the
 *           label services file.
 * @dlen: I: Number of bytes in the data portion of the record to be
 *           written to the label services file.
 * @cbuf: I: Character buffer to be written to the character portion
 *           of the label services record.
 * @dbuf: I: Data buffer to be written to the data portion of the label
 *           services record.
 * @dtype: I: Datatype of data portion of the record.
 *            Valid datatypes are:  
 *            B  = unsigned byte data,
 *            I2 = signed two byte data,
 *            I4 = signed four byte data,
 *            R4 = four byte floating point data,
 *            R8 = eight byte floating point data.
 * 
 * The c_lswrit() call is used to one record to the associated label
 * services file. The c_lswrit() function requires the file to be
 * opened with write access.
 *
 * Deprecated.  Use c_labwrite().
 *
 * Returns:      E_SUCC   = successful completion,               
 *               E_FAIL   = operation failed
 */
int c_lswrit
(
    const long *fd,     /* I: file descriptor */
    const char *key,    /* I: key of record to be written */
    const long *clen,   /* I: number of char bytes to be written */
    const long *dlen,   /* I: number of data bytes to be written */
    const char *cbuf,   /* I: character buffer to be written */
    const void *dbuf,   /* I: data buffer to be written */
    const char *dtype   /* I: datatype of data record (B,I2,I4,R4,R8) */
)
{
    int sig_mask;
    int status;

    c_sigblk(&sig_mask);

    status = lswrite(*fd, key, *clen, *dlen, cbuf, dbuf, dtype);

    c_sigunblk(&sig_mask);
    return (status);
}

#endif /* Obsolete interfaces */
