/*******************************************************************************
FUNCTION:       close_tab

PURPOSE:        Closes Labeled Table files

PROGRAM HISTORY:
  Version  Date   Author        Request
  -------  ----   ------        -------
    1      12/86  K. Gacke      Original development
    2       3/87  K. Gacke      Modified open_tab to allow directories to be 
                                specified for the subfiles.  Replaced "strcat" 
                                calls with "strcpy" to make the code more 
                                efficient.  Replaced "sscanf" calls with "atol"
                                and "atof" to make the code more efficient.
    3       5/87  K. Gacke      Modified "put_vector" to search the link list 
                                for each attribute defined.
    4       6/87  K. Gacke      Allow the LT to be opened for append.  Modified
                                "close_tab", "get_vector", "open_tab", and 
                                "put_vector".
    5       7/87  K. Gacke      changed the calls malloc and free to xalloc and
                                xree for the sel.  These routines are the C run
                                time routines renamed and put into the newlas.a
                                library to avoid linking the malloc and free in
                                the fortran libraries.
    6       8/87  K. Gacke      added the DATE and TIME data types.  Modified
                                the routines get_vector, get_record, put_record,
                                and put_vector.  Created the routines
                                get_lt_date, get_lt_time, put_lt_date, and
                                put_lt_time.
    7       9/87  K. Gacke      removed mod for "version 5" after installing
                                new OS (UTX 2.0)
    8      12/87  B. Ailts      Change include directory structure
    9      05/88  B. Ailts      Standardized err messages
                                Split routines into separate files
                                Added worgen.h
    9      12/92  J. Hemmer     Freed vector data pointer in free_vec.  Also
                                freed vector root node.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

ALGORITHM:
If opened for write access, flush the I/O buffers prior to the close.  
Close the file
Free the dynamic memory used for the I/O buffers and the logical record
linked list.
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "worgen.h"
#include "ltable.h"
#include "local_protos.h"

/******************************************************************************
FUNCTION:       cltab

ALGORITHM:      Closes the Labeled Table file
******************************************************************************/
static int cltab
(
    int fd                          /* I: File descriptor created by "optab" */
)
{
    if (close(fd) == -1)
        return(E_FAIL);             /* Error closing Labeled Table file      */

    return(E_SUCC);
}

/******************************************************************************
FUNCTION:       free_vec

ALGORITHM:      
Free the dynamic linked list of the LT vector.  
It is called when the LT file is closed.  
NOTE: That this is a recursive
function, which follows the linked list freeing the label
name and file description.  Once the end of the linked list
is reached, the linked list node is freed.
******************************************************************************/
void free_vec
(
    const struct VECTOR *vec        /* I: Labeled Table vector               */
)
{
    if (vec == NULL)
        return;

    if (vec->label != NULL)
        free(vec->label);

    if (vec->desc != NULL)
        free(vec->desc);

    free(vec->ptr.c);

    if (vec->next != NULL)
    {
        free_vec(vec->next);
        free(vec->next);
    }
}


/*
 * close_tab:
 * @tab:  I/O: Pointer to a structure defining the LT structure.  The structure
 *             is inialized by the open_tab() utility.
 *
 * close_tab() closes the LT file.  If the LT file is opened for write access,  
 * it flushes the I/O buffers prior to the close.  In addition, it frees all
 * dynamic memory used to manipulate the LT file; therefore, the linked
 * list defined by structure VECTOR and the LT data I/O buffer should
 * not be referenced after calling close_tab().
 */
void close_tab
(
    struct TAB_DEF *tab             /* I/O: Structure defining the LT files  */
)
{
    char msg[ERRLEN];

    if ((tab->access == 1) || (tab->access == 2)) /* Opened for write access */
        write_tab(tab);                           /* Flush the I/O buffers   */

    if (cltab(tab->fd) == E_FAIL)                 /* Close the LT files      */
    {
        sprintf(msg,"Error closing Labeled Table file %s.",tab->hname);
        tab_error(tab,TAB_LT_CLOSE,FATAL,msg,"close_tab");
        return;
    }

    free(tab->buf.addr);                          /* Free dynamic I/O buffers*/
    free_vec(tab->vector);                        /* Free dynamic linked list*/
    free(tab->vector);
}
