/*******************************************************************************
FUNCTION:       get_matrix

PURPOSE:        Copies the matrix from the LT record structure to the
                application programs buffer.  The matrix is converted to
                the specified data type when copied to the output buffer.

PROGRAM HISTORY:
  Version  Date   Author        Request
  -------  ----   ------        -------
    1      12/86  K. Gacke      Original development
    2       3/87  K. Gacke      Modified open_tab to allow directories to be 
                                specified for the subfiles.  Replaced "strcat" 
                                calls with "strcpy" to make the code more 
                                efficient.  Replaced "sscanf" calls with "atol"
                                and "atof" to make the code more efficient.
    3       5/87  K. Gacke      Modified "put_vector" to search the link list 
                                for each attribute defined.
    4       6/87  K. Gacke      Allow the LT to be opened for append.  Modified
                                "close_tab", "get_vector", "open_tab", and 
                                "put_vector".
    5       7/87  K. Gacke      changed the calls malloc and free to xalloc and
                                xree for the sel.  These routines are the C run
                                time routines renamed and put into the newlas.a
                                library to avoid linking the malloc and free in
                                the fortran libraries.
    6       8/87  K. Gacke      added the DATE and TIME data types.  Modified
                                the routines get_vector, get_record, put_record,
                                and put_vector.  Created the routines
                                get_lt_date, get_lt_time, put_lt_date, and
                                put_lt_time.
    7       9/87  K. Gacke      removed mod for "version 5" after installing
                                new OS (UTX 2.0)
    8      12/87  B. Ailts      Change include directory structure
    9      05/88  B. Ailts      Standardized err messages
                                Split routines into separate files
                                Added worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

ALGORITHM:
Determine the input and output data types
Copy or convert matrix from the VECTOR structure into the output buffer

*******************************************************************************/
#include "worgen.h"
#include "pixman.h"
#include "ltable.h"
#include "local_protos.h"

/*
 * get_matrix:
 * @vec: I/O: Pointer to a structure defining a field within an LT logical
 *            record.  Note that a field is required to be a numerical matrix.
 *            Note that vec is an element of the linked list defining the LT
 *            record.
 * @odtype: I: Specifies the data type of parameter matrix.  If the matrix
 *             is stored in the LT as a different data type, the matrix is
 *             converted to the specified data type.  Note that data type
 *             conversion may lead to unexpected results when converting from
 *             a large data type (e.g.,R*4) to a small data type(e.g.,I*1).
 *             Valid data type values are:
 *             I1:   Unsigned byte data,
 *             I2:   Signed two byte  data,
 *             I4:   Signed four byte data,
 *             R4:   Four byte floating point data,
 *             R8:   Eight byte floating point data. 
 * @matrix: O: The matrix read from the LT.  If required, the matrix is 
 *             converted to the desired data type specified by the parameter
 *             odtype dimension array.
 *
 * The get_matrix() routine retrieves a matrix from the LT. and returns an
 * I*1, I*2, I*4, R*4, or R*8 matrix read from the LT into parameter vec
 * dimension array in row major.  The data type of the returned matrix is
 * determined by parameter odtype.  
 *
 * If null values are encountered within the matrix, the structure field is set
 * to represent the size of the matrix where the first null value was 
 * detected.  It is required that an application program call get_vector() and
 * get_record() prior to calling get_matrix().            
 */
void get_matrix
(
    struct VECTOR *vec,       /* I/O: Structure defining the record to copy  */
    int odtype,               /* I: Data type of the output matrix           */
    void *matrix              /* O: Output matrix(any valid matrix data type)*/
)
{
    struct TAB_DEF dummy;
    int in_dtype;
    int nentity;
    int out_dtype;

    switch (vec->dtype)
    {
        case I1:
        case I2:
        case I4:
        case R4:
        case R8:
            in_dtype = vec->dtype;
            break;
        default: 
            tab_error(&dummy,TAB_LT_DTYPE,FATAL,
                      "Invalid data type for a matrix","getmatrix");
            return;
    }

    switch (odtype)
    {
        case I1:
        case I2:
        case I4:
        case R4:
        case R8:
            out_dtype = odtype;
            break;
        default: 
            tab_error(&dummy,TAB_LT_DTYPE,FATAL,
                      "Invalid data type for a matrix","getmatrix");
            return;
    }

    nentity = vec->size.nrow * vec->size.ncol;

    if (in_dtype == out_dtype)
        las_pixcopy(vec->ptr.i1,matrix,out_dtype,nentity);
    else
        las_pixconvert (in_dtype,out_dtype,vec->ptr.i1,matrix,nentity);

}
