/*******************************************************************************
FUNCTION:       open_tab

PURPOSE:        Opens the Labeled Table file with the specified access.
                It initializes the TAB_DEF structure by allocating the
                I/O buffers and assigning appropriate values to each of
                the structure fields.  If opened for append and the file
                already exists, error checking is done to ensure the
                subfile list and LT file types are the same.
PROGRAM HISTORY:
  Version  Date   Author        Request
  -------  ----   ------        -------
  5.0     ?????   ?????
  5.1     03/93   D. Etrheim    Changed pmode to 666 for file open.
                                This will allow users to use umask
                                appropriately. Execute permision is not
                                needed for any files created by LAS.


*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h> 
#include "tae.h"
#include "worgen.h"
#define OPEN_TAB_C_  /* define so the globals do not get declared as externs */
#include "ltable.h"
#include "local_protos.h"

/*-------------------------------- GLOBAL VARIABLES ---------------------*/

const long LT_DTYPE_SIZE[NDTYPE] = {1,1,2,4,4,8,1,1};
const char DATA_TYPE[NDTYPE][7] = {"C","I1","I2","I4","R4","R8","DATE","TIME"};

/*-----------------------------------------------------------------------*/

/******************************************************************************
FUNCTION:       optab

ALGORITHM:      Opens the Labeled Table file with the specified access.
******************************************************************************/
static int optab
(
    int *fd,                 /* I: File descriptor array                     */
    const char *hname,       /* I: Host file name of Labeled Table           */
    int mode                 /* I: File acess                                */
                             /*    0 - read access                           */
                             /*    1 - write access                          */
                             /*    2 - update access                         */
)
{
    struct TAB_DEF dummy;
    int    flags;
    int    pmode = 0666;
    long   nonfatal = 1;     /* Non-fatal error code                         */
    char   msg[ERRLEN];

    switch (mode)            /* Set up the access flags for the open call    */
    {
        case 0:
            flags = O_RDONLY;   
            break;
        case 1:
            flags = O_WRONLY | O_TRUNC | O_APPEND | O_CREAT;     
            break;
        case 2: 
            flags = O_RDWR | O_CREAT;                            
            break;
        default:
            tab_error(&dummy,TAB_LT_ACCESS,NONFATAL,"","open_tab");
            return(E_FAIL);
    }
    if ((mode == 1) && (access(hname,0) != -1))
    {
        sprintf(msg,"Labeled Table \"%s\" already exists",hname);
        c_errmsg(msg,"open_tab-exists",&nonfatal);
        return(E_FAIL);
    }
    if ((*fd = open(hname,flags,pmode)) == -1) /* Open the LT file           */
        return(E_FAIL);

    return(E_SUCC);
}



/*
 * open_tab:
 * @tab: I/O: Pointer to a structure defining the LT file structure.  The
 *            other LT utility routines access various fields within this
 *            structure.
 * @fname:  I: Host file of the LT file.  Directory specifications may be
 *             included using the host computer filenaming conventions.
 * @access: I: Specifies the type of access the LT is being opened for.  
 *             Valid values are:
 *             0:  Read access,
 *             1:  Write access,
 *             2:  Append (read/write) access.
 *
 * The open_tab() routine opens the LT file with the specified access.  It
 * initializes the data structures by allocating the I/O buffers and assigning
 * appropriate values to each of the structure fields.  The following fields of
 * parameter tab are assigned for all types of access:
 *             1.  LT host filename (fname),
 *             2.  Type of file access (access),
 *             3.  File descriptor (fd),
 *             4.  Allocate LT I/O buffer (buf).
 * If the LT is opened for read access, it reads the first two logical records
 * of the LT file.  The following fields of parameter tab are assigned:
 *             1.  Number of associated files (nsubs),
 *             2.  Array of associated filenames (subfile),
 *             3.  LT file type (ftype) 
 *             4.  LT file description (fdesc).
 * If the LT is opened for write access, it writes the first two logical records
 * into the LT.  It expects the following fields of parameter tab to be assigned
 * prior to calling open_tab():
 *             1.  Number of associated files (nsubs),
 *             2.  Array of associated filenames (subfile),
 *             3.  LT file type (ftype)
 *             4.  LT file description (fdesc).
 * If the LT is opened for append and the file does not exist, it is treated
 * exactly as if it were being opened for write access.  That is it writes the
 * first two logical records into the LT and it expects the following fields of
 * parameter tab to be assigned prior to calling open_tab():
 *              1.  Number of associated files (nsubs),
 *              2.  Array of associated filenames (subfile)
 *              3.  LT file type (ftype)
 *              4.  LT file description (fdesc).
 * IF the LT is opened for append and the file already exists, error checking
 * is done to ensure the subfile(s) and file type correspond between the calling 
 * program and the existing LT.  The function get_vector() is called to retrieve
 * the current label vector and the file pointer is positioned to the end of the 
 * file.  It expects the following fields of parameter tab to be assigned prior
 * to calling open_tab():
 *              1.  Number of associated files (nsubs),
 *              2.  Array of associated filenames (subfile),
 *              3.  LT file type (ftype),
 *              4.  LT file description (fdesc).
 */
void open_tab
(
    struct TAB_DEF *tab,    /* I/O: Struct of Labeled Table                  */
    const char *fname,      /* I: Host name of the LT file to be opened      */
    int access              /* I: Type of access to open the file with       */
                            /*    0 - read                                   */
                            /*    1 - write                                  */
                            /*    2 - append                                 */
)

{
#define DIR_END '/'

    int dirlen;
    int eor;
    int flag;
    int index;
    int index1;
    int nsubs;
    char *ftype;
    char msg[ERRLEN];
    char name[CMLEN];
    char sep;
    char *subfile[MAX_SUB];
    char *val;

    if (optab(&tab->fd,fname,access) != E_SUCC)  /* Open the LT file         */
    {
        sprintf(msg,"Error opening Labeled Table file \"%s\".",fname);
        tab_error(tab,TAB_LT_OPEN,FATAL,msg,"open_tab");
        return;
    }

    strcpy(tab->hname,fname);           /* Initialize the TAB_DEF structure  */
    tab->access = access;
    if ((tab->buf.addr = malloc(TAB_SIZ)) == NULL)
    {
        tab_error(tab,TAB_ALLOC,FATAL,\
                 "Error allocating Label Table I/O buffer.","open_tab");
        return;
    }

    *(tab->buf.addr) = '\0';
    tab->buf.length = TAB_SIZ;
    tab->vector = NULL;
    tab->nbytes = 0;
    tab->cur = 0;
    tab->currec = 2;
    flag = access;
    if (access == 2)                   /* If opened for append access, issue */
    {                                  /* a physical read.  If the file is   */
         read_tab(tab);                /* empty, treat it as if it is being  */
         if (tab->nbytes == 0)         /* opened for write access.           */
             flag = 1;
    }

    if (flag == 0)                     /* If opened for read access          */
    {                                  /* read the list of subfiles, attach  */
        read_tab(tab);                 /* the directory name, and store in   */
        if (tab->nbytes == 0)          /* LT structure.                      */
        {
            tab_error(tab,TAB_LT_OPEN,FATAL,
                "Labeled Table is opened for read access and contains no data.",
                "open_tab");
            return;
        }
        strcpy(name,fname);
        if ((val = strrchr(name,DIR_END)) == NULL)
            *name = '\0';
        else
            *(val + 1) = '\0';
        dirlen = (long)strlen(name);
        for (index=0; index<MAX_SUB; index++)
            tab->subfile[index] = NULL;
        for (eor=FALSE,tab->nsubs=0; eor!=TRUE; tab->nsubs++)
        {
            val = get_string_field(tab,&eor);
            if (*val == '\0')
            break;
            if ((tab->subfile[tab->nsubs] =
                 malloc(dirlen + strlen(val) + 1)) == NULL)
            {
                tab_error(tab,TAB_ALLOC,FATAL,
                    "Error allocating Label Table I/O buffer.","open_tab");
                return;
            }
            sprintf(tab->subfile[tab->nsubs],"%s%s",name,val);
            free(val);
        }
        tab->ftype = get_string_field(tab,&eor); /* Store LT file type       */
        tab->fdesc = get_string_field(tab,&eor); /* Store LT file description*/
        tab->label_flag = TRUE;                  /* Label vector exists      */
    }
    else if (flag == 1)                 /* If opened for write access        */
    {                                   /* remove directory specification    */
        if (tab->nsubs == 0)            /* from the subfile names and write  */
        put_null_field(tab,RECORD_SEP); /* them to the master LT             */
        else
        {
            for (sep=FIELD_SEP1,index=0; index<tab->nsubs; index++)
            {
                if (index+1 == tab->nsubs)
                sep = RECORD_SEP;
                strcpy(name,tab->subfile[index]);
                if ((val = strrchr(name,DIR_END)) == NULL)
                    val = name;
                else
                    ++val;
                put_string_field(tab,val,sep,TRUE);
            }
            for (index=tab->nsubs; index<MAX_SUB; index++)
            tab->subfile[index] = NULL;
        }
        put_string_field(tab,tab->ftype,FIELD_SEP1,TRUE); /* Write file type  */
        put_string_field(tab,tab->fdesc,RECORD_SEP,TRUE); /* Write  file desc */
        tab->label_flag = FALSE;                /* Label vector does not exist*/
    }
    else if (flag == 2)                 /* If opened for append access       */
    {
        for (index=tab->nsubs; index<MAX_SUB; index++)
            tab->subfile[index] = NULL;
        for (eor=FALSE,nsubs=0; eor!=TRUE; nsubs++)
        {
            subfile[nsubs] = get_string_field(tab,&eor);
            if (*subfile[nsubs] == '\0')
                break;
        }
        for (index=0; index<tab->nsubs; index++)  /* Error check the list of */
        {                                         /* subfiles to ensure no   */
            for (index1=0; index1<nsubs; index1++)/* new ones are specified  */
            {
                if (strcmp(tab->subfile[index],subfile[index1]) == 0)
                break;
            }
            if (index1 == nsubs)
            {
                sprintf(msg,"%s is not a subfile of the Labeled Table \"%s\".",
                tab->subfile[index],tab->hname);
                tab_error(tab,TAB_LT_SUBFIL,FATAL,msg,"open_tab");
                return;
            }
        }
        ftype = get_string_field(tab,&eor);     /* Error check the file type */
        if (strcmp(tab->ftype,ftype) != 0)
        {
            sprintf(msg,"%s\"%s\"%s\"%s\"%s\"%s\".",
                   "Labeled Table ",tab->hname," has a file type of ",ftype,
                   ", but the program expects a file type of ",tab->ftype);
            tab_error(tab,TAB_LT_FTYPE,FATAL,msg,"open_tab");
            return;
        }
        tab->fdesc = get_string_field(tab,&eor);
        get_vector(tab);                        /* Retrieve the label vector */
        tab->label_flag = TRUE;                 /* label vector exists       */

        if (lseek(tab->fd, 0, SEEK_END) == -1)  /* Skip to end of file       */
        {
            tab_error(tab,TAB_LT_SEEK,FATAL,"","open_tab");
            return;
        }

        *(tab->buf.addr) = '\0';
        tab->nbytes = 0;
        tab->cur = 0;
    }

}



