/******************************************************************************
FUNCTION:       put_x_matrix            

PURPOSE:        Write matrix into Labeled Table I/O buffer

PROGRAM HISTORY:
  Version  Date   Author        Request
  -------  ----   ------        -------
    1      12/86  K. Gacke      Original development
    2       3/87  K. Gacke      Modified open_tab to allow directories to be 
                                specified for the subfiles.  Replaced "strcat" 
                                calls with "strcpy" to make the code more 
                                efficient.  Replaced "sscanf" calls with "atol"
                                and "atof" to make the code more efficient.
    3       5/87  K. Gacke      Modified "put_vector" to search the link list 
                                for each attribute defined.
    4       6/87  K. Gacke      Allow the LT to be opened for append.  Modified
                                "close_tab", "get_vector", "open_tab", and 
                                "put_vector".
    5       7/87  K. Gacke      changed the calls malloc and free to xalloc and
                                xree for the sel.  These routines are the C run
                                time routines renamed and put into the newlas.a
                                library to avoid linking the malloc and free in
                                the fortran libraries.
    6       8/87  K. Gacke      added the DATE and TIME data types.  Modified
                                the routines get_vector, get_record, put_record,
                                and put_vector.  Created the routines
                                get_lt_date, get_lt_time, put_lt_date, and
                                put_lt_time.
    7       9/87  K. Gacke      removed mod for "version 5" after installing
                                new OS (UTX 2.0)
    8      12/87  B. Ailts      Change include directory structure
    9      05/88  B. Ailts      Standardized err messages
                                Split routines into separate files
                                Added worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

ALGORITHM:
PUT_MATRIX copies the matrix from the application programs buffer into the
LT record structure.  The matrix is converted to the correct data type when
copied to the LT record structure.
    Arguments:
        *vec    VECTOR    I/O   structure defining the record structure to copy
                                the matrix into
        idtype  I*4        I    data type of the input matrix 
        size    MATRIX     I    structure defining the size of the input matrix
        matrix  I*1        I    input matrix (may be any valid matrix data type)
*******************************************************************************/
#include <stdio.h>
#include <ctype.h>
#include "worgen.h"
#include "pixman.h"
#include "ltable.h"
#include "local_protos.h"

/*
 * put_matrix: 
 * @vec: I: Pointer to a structure defining a field within an LT logical
 *          record.  Note that a field is required to be a numerical matrix.
 * @idtype: I: Specifies the data type of parameter matrix.  If the matrix
 *             is stored in the LT as as different data type, the matrix is
 *             converted to the specified data type.  Note that data type
 *             conversion may lead to unexpected results when converting
 *             from a large data type(e.g., R*4) to a small data type
 *             (e.g., I*1). 
 *             Valid data type values are:
 *             I1:  Unsigned byte data,
 *             I2:  Signed two byte data,
 *             I4:  Signed four byte data,
 *             R4:  Four byte floating point data,
 *             R8:  Eight byte floating point data. 
 * @size: I: Number of rows and columns in the input matrix.  If the input
 *           matrix size is smaller than the actual number of rows and 
 *           columns in the LT, null fields will be written to the lower-right
 *           region of the matrix.
 * @matrix: I: The matrix to be written into the LT.  If required, the matrix
 *             is converted to the data type specified by the label vector.
 *
 * The put_matrix() routine places a matrix into the LT.  Writes an I*1, I*2,
 * I*4, R*4, or R*8 matrix into the LT parameter vec array in row major.  The
 * data type of the input matrix is determined by parameter idtype.
 *
 * Note that if parameter vec has a larger number of rows or columns, a fatal
 * error is generated.  If the matrix to be written has a smaller number of
 * rows or columns, null values are written into the lower-right region of the
 * matrix and a nonfatal error is outputted.
 *
 * It is required that an application program call put_vector() prior to 
 * calling put_matrix().
 */ 
void put_matrix
(
    struct VECTOR *vec,         /*  I: VECTOR structure                      */
    int idtype,                 /*  I: Input data type                       */
    const struct MATRIX *size,  /*  I: Matrix structure                      */
    const void *matrix          /*  I: The matrix to be written to           */
)
{
    struct TAB_DEF dummy;
    int in_dtype;
    int nentity;
    int out_dtype;

    switch (idtype)
    {
        case I1:
        case I2:
        case I4: 
        case R4: 
        case R8: 
            in_dtype = idtype;       
            break;
        default: 
            tab_error(&dummy,TAB_LT_DTYPE,FATAL,
                      "Invalid data type for a matrix",
                      "put+matrix");
            return;
    }
    switch (vec->dtype)
    {
        case I1:
        case I2: 
        case I4: 
        case R4: 
        case R8: 
            out_dtype = vec->dtype;      
            break;
        default: 
             tab_error(&dummy,TAB_LT_DTYPE,FATAL,
                       "Invalid data type for a matrix",
                       "put+matrix");
            return;
    }
    nentity = size->nrow * size->ncol;
    if (in_dtype == out_dtype)
        las_pixcopy (matrix,vec->ptr.i1,in_dtype,nentity);
    else
        las_pixconvert (in_dtype,out_dtype,matrix,vec->ptr.i1,nentity);
    vec->size.nrow = size->nrow;
    vec->size.ncol = size->ncol;
}
