/*******************************************************************************
FUNCTIONS:      put_vector              
PURPOSE:        Write label vector into Labeled Table I/O buffer

PROGRAM HISTORY:
  Version  Date   Author        Request
  -------  ----   ------        -------
    1      12/86  K. Gacke      Original development
    2       3/87  K. Gacke      Modified open_tab to allow directories to be 
                                specified for the subfiles.  Replaced "strcat" 
                                calls with "strcpy" to make the code more 
                                efficient.  Replaced "sscanf" calls with "atol"
                                and "atof" to make the code more efficient.
    3       5/87  K. Gacke      Modified "put_vector" to search the link list 
                                for each attribute defined.
    4       6/87  K. Gacke      Allow the LT to be opened for append.  Modified
                                "close_tab", "get_vector", "open_tab", and 
                                "put_vector".
    5       7/87  K. Gacke      changed the calls malloc and free to xalloc and
                                xree for the sel.  These routines are the C run
                                time routines renamed and put into the newlas.a
                                library to avoid linking the malloc and free in
                                the fortran libraries.
    6       8/87  K. Gacke      added the DATE and TIME data types.  Modified
                                the routines get_vector, get_record, put_record,
                                and put_vector.  Created the routines
                                get_lt_date, get_lt_time, put_lt_date, and
                                put_lt_time.
    7       9/87  K. Gacke      removed mod for "version 5" after installing
                                new OS (UTX 2.0)
    8      12/87  B. Ailts      Change include directory structure
    9      05/88  B. Ailts      Standardized err messages
                                Split routines into separate files
                                Added worgen.h
   10      02/90  B. Ailts      Checked for allocation errors

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

ALGORITHM:
    PUT_VECTOR writes the label vector to the Labeled Table file.
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "worgen.h"
#include "ltable.h"
#include "local_protos.h"


/*
 * put_vector:
 * @tab: I/O: Pointer to a structure defining the LT file structure.  The
 *            structure is initialized by the open_tab() utility.
 * @label: I: Attribute name for the current element of the vector linked
 *            list.  It is assigned to the vector field label.
 * @dtype: I: Data type for the current element of the vector linked list.
 *            It is assigned to the vector field dtype.  Valid values are:
 *            C:  Character data,
 *            I1: Unsigned byte data,
 *            I2: Signed two byte data,
 *            I4: Signed four byte data,
 *            R4: Four byte floating point data,
 *            R8: Eight byte floating point data.
 * @desc: I: Description for the current element of the vector linked list.
 *           It is assigned to the vector field desc.
 * @ncol: I: Number of columns in the matrix for the current element of the 
 *           vector linked list.  It is assigned to the vector field 
 *           lt_size.ncol.  If the data type is character, this value should
 *           be set to the maximum character string length to be written
 *           to the lt. 
 * @nrow: I: Number of rows in the matrix for the current element of the 
 *           vector linked list.  It is assigned to the vector field
 *           lt_size.nrow.  If the data type is character, this value should
 *           be set to 1 (one).
 * @sep: I: Type of separator to be written to terminate the label vector.  If
 *          this is to be the last element of the linked list, sep to the
 *          constant FIELD_SEP1 or FIELD_SEP2.
 *         
 * The put_vector() routine places an element of the label vector (defined
 * by structure VECTOR) into the LT.  The label vector is written as 
 * specified by the input parameters.  Each time put_vector() is called, it
 * adds an element to the vector linked list.  The following fields are
 * assigned to the new vector element:
 *            1.  Name of the attribute (label),
 *            2.  Data type of the attribute (dtype), 
 *            3.  Description of the attribute (desc),
 *            4.  Number of rows and columns in the matrix (lt_size).
 * If the LT was opened for append and the LT already exists, error checking
 * is done to ensure the label vector of the LT is exactly the same as the
 * label vector to be processed by the program.
 *
 * In addition, put_vector() allocates dynamic memory to store logical records
 * to be written to the LT; therefore, it is required that an application
 * program to call put_vector() prior to calling put_matrix() and 
 * put_record().
 */

void put_vector
(
    struct TAB_DEF *tab,     /* I: Struct defining LT file to be processed   */
    const char label[],      /* I: Label name of the vector                  */
    int dtype,               /* I: Data type of the label vector             */
    const char desc[],       /* I: Description of the label vector           */
    int nrow,                /* I: Number of rows                            */
    int ncol,                /* I: Number of columns                         */
    int sep                  /* I: Separator(FIELD_SEP1,FIELD_SEP2,RECORD_SEP)*/
)
{
    struct VECTOR *next_vec;
    struct VECTOR *temp;
    struct VECTOR *vec;
    int size;
    char msg[ERRLEN];

    if ((tab->label_flag == TRUE) && (tab->access != 2))
    {
        sprintf(msg,
               "Tried to write to the label vector"
               " after data was written to LT \"%s\".",tab->hname);
        tab_error(tab,TAB_LT_PUTVEC,FATAL,msg,"put_vector");
        return;
    }
    if (tab->label_flag == FALSE)
    {
        if (sep == RECORD_SEP)
        {
            ++tab->currec;
            tab->label_flag = TRUE;
        }
        ++tab->ncol;
        put_string_field(tab,label,FIELD_SEP1,TRUE);
        put_string_field(tab,DATA_TYPE[dtype],FIELD_SEP1,TRUE);
        put_string_field(tab,desc,FIELD_SEP1,TRUE);
        put_i4_field(tab,nrow,FIELD_SEP1);
        put_i4_field(tab,ncol,sep);

        if ((next_vec = (struct VECTOR *) malloc(sizeof(struct VECTOR))) 
             == NULL)
        {
            tab_error(tab,TAB_ALLOC,FATAL," ","put_vector");
            return;
        }
        if ((next_vec->label = malloc(strlen(label) + 1)) == NULL)
        {
            tab_error(tab,TAB_ALLOC,FATAL," ","put_vector");
            return;
        }
        strcpy(next_vec->label,label);
        next_vec->dtype = dtype;
        if ((next_vec->desc = malloc(strlen(desc) + 1)) == NULL)
        {
            tab_error(tab,TAB_ALLOC,FATAL," ","put_vector");
            return;
        }
        strcpy(next_vec->desc,desc);
        next_vec->sep = FIELD_SEP1;
        size = nrow * ncol;
        switch (dtype)                 /* Allocate memory to hold values for */
        {                              /* the record field                   */
            case I1:
                if ((next_vec->ptr.i1 = malloc(sizeof(char) * size)) == NULL)
                {
                    tab_error(tab,TAB_ALLOC,FATAL," ","put_vector");
                    return;
                }
                break;
            case I2:
                if ((next_vec->ptr.i2 =
                    (short *) malloc(sizeof(short) * size)) == NULL)
                {
                    tab_error(tab,TAB_ALLOC,FATAL," ","put_vector");
                    return;
                }
                break;
            case I4:
                if ((next_vec->ptr.i4 =
                    (int *) malloc(sizeof(int) * size)) == NULL)
                {
                    tab_error(tab,TAB_ALLOC,FATAL," ","put_vector");
                    return;
                }
                break;
            case R4:
                if ((next_vec->ptr.r4 =
                    (float *) malloc(sizeof(float) * size)) == NULL)
                {
                    tab_error(tab,TAB_ALLOC,FATAL," ","put_vector");
                    return;
                }
                break;
            case R8:
                if ((next_vec->ptr.r8 =
                    (double *) malloc(sizeof(double) * size))==NULL)
                {
                    tab_error(tab,TAB_ALLOC,FATAL," ","put_vector");
                    return;
                }
                break;
            case C:
            case LTDATE:
            case LTTIME:
                if ((next_vec->ptr.c = malloc(MAX_FIELD_LEN)) == NULL)
                {
                    tab_error(tab,TAB_ALLOC,FATAL," ","put_vector");
                    return;
                }
                break;
            default:
                tab_error(tab,TAB_LT_DTYPE,FATAL,"","put_vector");
                return;
        }
        next_vec->lt_size.nrow = nrow;
        next_vec->lt_size.ncol = ncol;
        next_vec->next = NULL;
        if (tab->vector == NULL)        /* Assign the current vector to the  */
            tab->vector = next_vec;     /* end of the linked list            */
        else
        {
            for (vec=tab->vector; vec->next!=NULL; vec=vec->next)
                ;
            vec->next = next_vec;
        }
    }
    else
    {
        temp = get_field_ptr(tab,label);
        if (temp == NULL)
        {
            sprintf(msg,"Label name \"%s\" does not exist in the LT \"%s\".",
                    label,tab->hname);
            tab_error(tab,TAB_LT_PUTVEC,FATAL,msg,"put_vector");
            return;
        }
        if (temp->dtype != dtype)
        {
            sprintf(msg,
                   "Data type for label \"%s\" does not correspond with LT"
                   "  \"%s\".",label,tab->hname);
            tab_error(tab,TAB_LT_PUTVEC,FATAL,msg,"put_vector");
            return;
        }
        if ((temp->lt_size.nrow != nrow) || (temp->lt_size.ncol != ncol))
        {
            sprintf(msg,"%s\"%s\"%s\"%s\".",
                    "Number of rows or columns for label ",
                    label," does not correspond with LT ",tab->hname);
            tab_error(tab,TAB_LT_PUTVEC,FATAL,msg,"put_vector");
            return;
        }
        temp->flag = TRUE;
        if (sep == RECORD_SEP)
        {
            for (temp=tab->vector; temp!=NULL; temp=temp->next)
            {
                if (temp->flag != TRUE)
                {
                    sprintf(msg,"%s\"%s\"%s\"%s\"%s","Label name ",temp->label,
                           " exists in LT ",tab->hname,
                           ", but is not known to the program.");
                    tab_error(tab,TAB_LT_PUTVEC,FATAL,msg,"put_vector");
                    return;
                }
            }
        }
    }
}
