/******************************************************************************
NAME                    pxbtan

PURPOSE         Perform a bitwise and of each element of two input arrays
                generating a third output array which is the "anded" result.
                The input array can be of type byte, signed 2 byte, signed 4
                byte, or real. The output array will be of the same type as the
                input array.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        D.GORDON        7/85    Original development for NEWLAS
        B.Ailts         12/87   Place bridge routines in a seperate file
                                change include directory specifications
                                Use raw 'C' types
        B.AILTS         3/88    replaced newlas.h with worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system.

PROJECT                         NEWLAS

ALGORITHM

receive:
        -the input array and its type
        -the input mapping array
        -the output array
        -the number of samples of the input array.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        Assign the output array pixel the bitwise and of the input array and
        the mapping array.
return

ALGORITHM REFERENCES            none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void bbtan
(
    const unsigned char *in,
    const unsigned char *map,
    unsigned char *out,
    int ns
)
{
    for (; ns; ns--,in++,map++,out++)
        *out = *in & *map;
}

static void lbtan
(
    const int *in,
    const int *map,
    int *out,
    int ns
)
{
    for (; ns; ns--,in++,map++,out++)
        *out = *in & *map;
}

static void wbtan
(
    const short *in,
    const short *map,
    short *out,
    int ns
)
{
    for (; ns; ns--,in++,map++,out++)
        *out = *in & *map;
}

/**
 * las_pixbitand:
 * @in:  I: The input array.
 * @map: I: The mapping array.
 * @out: O: The output "anded" array.
 * @ns:  I: The number of samples to be scanned from the buffer.
 * @dtype: I: An integer value that represents the type of data
 *            in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 *
 * The las_pixbitand() call is used to perform a bitwise AND of an input
 * array with a mapping array to generate an output array.
 *
 * Returns:  E_SUCC:   Successful completion,
 *           TYPERR:   Invalid data type specified.
 */
int las_pixbitand
(
    const void *in,         /* I:  input array                           */
    const void *map,        /* I:  input mapping array                   */
    void *out,              /* O:  output "anded" array                  */
    int ns,                 /* I:  number of samples                     */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)   */
)
{
    int status = E_SUCC;        /* return status for pxadd */

    switch (dtype)
    {
        case (EBYTE): 
            bbtan((unsigned char *)in,(unsigned char *)map,
                  (unsigned char *)out,ns); 
            break;
        case (EWORD): 
            wbtan((short *)in,(short *)map,(short *)out,ns); 
            break;
        case (ELONG): 
        case (EREAL): 
            lbtan((int *)in,(int *)map,(int *)out,ns); 
            break;
        default:      
            status = TYPERR;
            break;

    } /* switch */

    return (status);

} /* pxbtan */


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_pxbtan:
 * @in:  I: The input array.
 * @map: I: The mapping array.
 * @out: O: The output "anded" array.
 * @ns:  I: The number of samples to be scanned from the buffer.
 * @dtype: I: An integer value that represents the type of data
 *            in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 *
 * The c_pxbtan() call is used to perform a bitwise AND of an input
 * array with a mapping array to generate an output array.
 *
 * Deprecated.  Use las_pxbtan() instead.
 *
 * Returns:  E_SUCC:   Successful completion,
 *           TYPERR:   Invalid data type specified.
 */
int c_pxbtan
(
    const void *in,   /* I:  input array                                 */
    const void *map,  /* I:  input mapping array                         */
    void *out,        /* O:  output "anded" array                        */
    const long *ns,   /* I:  number of samples                           */
    const long *dtype /* I:  data type (EBYTE,EWORD,ELONG,EREAL)         */
)

{
    int    status = E_SUCC;        /* return status for pxadd */

    status = las_pixbitand (in,map,out,*ns,*dtype);
    
    return (status);

} /* pxbtan */

#endif /* Obsolete interfaces */



