/*************************************************************
NAME                    pxbtor

PURPOSE         Perform a bitwise "or" of each element of two input arrays
                generating a third output array which is the "or"'d result.
                The input array can be of type byte, signed 2 byte, signed 4
                byte, or real. The output array will be of the same type as the
                input array.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        R.MCKINNEY      9/87    Original development for NEWLAS
        B.Ailts         4/88    Replaced newlas.h with worgen.h
                                Moved the fortran bridge to a seperate file
                                Used raw C types
                                added include pixman.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system 

PROJECT                         LAS

ALGORITHM 

receive:
        -the input array and its type
        -the input mapping array
        -the output array
        -the number of samples of the input array.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        Assign the output array pixel the bitwise "or" of the input array and
        the mapping array.
return

ALGORITHM REFERENCES            none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void bbtor
(
    const unsigned char *in,
    const unsigned char *map,
    unsigned char *out,
    int ns
)
{
    for (; ns; ns--,in++,map++,out++)
        *out = *in | *map;
}

static void wbtor
(
    const short *in,
    const short *map,
    short *out,
    int ns
)
{
    for (; ns; ns--,in++,map++,out++)
        *out = *in | *map;
}

static void lbtor
(
    const int *in,
    const int *map,
    int *out,
    int ns
)
{
    for (; ns; ns--,in++,map++,out++)
        *out = *in | *map;
}

/**
 * las_pixbitor:
 * @in: I: The input array.
 * @map: I: The input mapping array.
 * @out: O: The output "ored" array.
 * @ns: I: The number of samples. 
 * @dtype: I: An integer value that represents the
 *            type of data in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 *
 * The las_pixbitor() call is used to perform a bitwise OR of an input
 * array with a mapping array to generate an output array.
 *
 * Returns:  E_SUCC:  Successful completion,
 *           TYPERR:  Invalid data type specified.
 */
int las_pixbitor
(
    const void *in,         /* I:  input array                           */
    const void *map,        /* I:  input mapping array                   */
    void *out,              /* O:  output "ored" array                   */
    int ns,                 /* I:  number of samples                     */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)   */
)
{
    int status = E_SUCC;        /* return status for pxor */

    switch (dtype)
    {
        case (EBYTE): 
            bbtor((unsigned char *)in,(unsigned char *)map,
                  (unsigned char *)out,ns); 
            break;
        case (EWORD): 
            wbtor((short *)in,(short *)map,(short *)out,ns); 
            break;
        case (ELONG): 
        case (EREAL): 
            lbtor((int *)in,(int *)map,(int *)out,ns); 
            break;
        default:
            status = TYPERR;
            break;
    } /* switch */

    return (status);

} /* pxbtor */



#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_pxbtor:
 * @in: I: The input array.
 * @map: I: The input mapping array.
 * @out: O: The output "ored" array.
 * @ns: I: The number of samples. 
 * @dtype: I: An integer value that represents the
 *            type of data in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 *
 * The c_pxbtor() call is used to perform a bitwise OR of an input
 * array with a mapping array to generate an output array.
 *
 * Deprecated.  Use las_pixbitor() instead.
 *
 * Returns:  E_SUCC:  Successful completion,
 *           TYPERR:  Invalid data type specified.
 */
int c_pxbtor
(
    const void *in,   /* I:  input array                                 */
    const void *map,  /* I:  input mapping array                         */
    void *out,        /* O:  output "ored" array                         */
    const long *ns,   /* I:  number of samples                           */
    const long *dtype /* I:  data type (EBYTE,EWORD,ELONG,EREAL)         */
)

{
    int status = E_SUCC;        /* return status for pxor */

    status = las_pixbitor (in,map,out,*ns,*dtype);

    return (status);

} /* pxbtor */

#endif /* obsolete interface */
