/*************************************************************
NAME                    c_pxcomp

PURPOSE         Perform a one's complement of each element of the input 
                array generating a second output array which is the 
                result.  The input array can be of type byte, signed 2 byte, 
                signed 4 byte, or real. The output array will be of the same 
                type as the input array.

EXTERNAL ASSOCIATIONS           none

FILE REFERENCES                 none

EXTERNAL VARIABLES REFERENCED   none

DATA STRUCTURES none

SUBPROGRAMS CALLED 
NAME            DESCRIPTION
----            -----------
bbcomp          performs pixel ones complement on unsigned byte data
wbcomp          performs pixel ones complement on signed integer data
lbcomp          performs pixel ones complement on signed long data or real
                data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS      none.

FATAL ERROR CONDITIONS          TYPERR 66 (choices: EBYTE,EWORD,ELONG,or EREAL)

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        M.MAZZELLA      6/87    Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                c_pxbxor must be linked with the imageio routines.

PROJECT                         NEWLAS

LANGUAGE                        C

ALGORITHM (FLOW)

receive:
        -the input array and its type
        -the output array
        -the number of samples of the input array.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        Assign the output array pixel the one's complement of the input 
        array.
return

ALGORITHM REFERENCES            none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void bbcomp
(
    const unsigned char *in,
    unsigned char *out,
    int ns
)
{
    for (; ns; ns--,in++,out++)
        *out = ~*in;
}

static void wbcomp
(
    const short *in,
    short *out,
    int ns
)
{
    for (; ns; ns--,in++,out++)
        *out = ~*in;
}

static void lbcomp
(
    const int *in,
    int *out,
    int ns
)
{
    for (; ns; ns--,in++,out++)
        *out = ~*in;
}


/**
 * las_pixcomplement:
 * @in: I: The input array to be complemented. Must be same type as
 *         the output array.
 * @out: O: The output array to hold result.  Must be the same type
 *          as the input array.
 * @ns: I: Number of elements in the input array.
 * @dtype: I: An integer value that represents the type of data
 *            in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 *
 * The las_pixcomplement() call performs a one's complements on each element
 * of an input array.  The input array and output array must be
 * of the same type.
 *
 * Returns:  E_SUCC:  Successful completion,
 *           TYPERR:  Invalid type specified.
 */

int las_pixcomplement
(
    const void *in,         /* I:  input array                            */
    void *out,              /* O:  output array                           */
    int ns,                 /* I:  number of samples                      */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
)
{
    int status = E_SUCC;        /* return status for pxadd */

    switch (dtype)
    {
        case (EBYTE):
            bbcomp((unsigned char *)in,(unsigned char *)out,ns); 
            break;
        case (EWORD):
            wbcomp((short *)in,(short *)out,ns); 
            break;
        case (ELONG): 
        case (EREAL):
            lbcomp ((int *)in,out,ns);
            break;
        default:
            status = TYPERR;
            break;
    } /* switch */

    return (status);

} /* las_pxcomp */


 
#ifndef NO_OBSOLETE_INTERFACE


/**
 * c_pxcomp:
 * @in: I: The input array to be complemented. Must be same type as
 *         the output array.
 * @out: O: The output array to hold result.  Must be the same type
 *          as the input array.
 * @ns: I: Number of elements in the input array.
 * @dtype: I: An integer value that represents the type of data
 *            in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 *
 * The c_pxcomp() call performs a one's complements on each element
 * of an input array.  The input array and output array must be
 * of the same type.
 *
 * Deprecated.  Use las_pixcomplement() instead.
 *
 * Returns:  E_SUCC:  Successful completion,
 *           TYPERR:  Invalid type specified.
 */
int c_pxcomp
(
    const void *in,   /* I:  input array                                  */
    void *out,        /* O:  output array                                 */
    const long *ns,   /* I:  number of samples                            */
    const long *dtype /* I:  data type (EBYTE,EWORD,ELONG,EREAL)          */
)

{
    int status = E_SUCC;        /* return status for pxadd */

    status = las_pixcomplement (in,out,*ns,*dtype);

    return (status);

} /* pxcomp */

#endif /* obsolete interface */
