/*****************************************************************************
NAME                    pxconv

PURPOSE         Convert an input array from one data type to another.  The 
                output array will be of the same type as 'totyp'.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        D.GORDON        5/7/85   Original development for NEWLAS
        J.REED          11/12/85 Optimized byte portion of routine.
        D.GORDON        11/20/85 Optimized all routines.
        B.Ailts          12/87   Changed include directory specifications
                                 Use raw 'C' types
                                 Place bridge routines in a seperate file
        B.Ailts          04/88   replaced newlas.a with las.a
        B.Ailts          12/88   When converting to a smaller data type, the
                                 output value is set to the max or min of the
                                 output data type if the input value is above
                                 or below the max or min of the output data
                                 type

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system.

PROJECT                         LAS

ALGORITHM

receive:
        -the input array and its data type
        -the output array and its data type
        -the number of samples of the input array contains.
depending on the type of input array values
        call a function for that particular type of conversion

for each pixel of the input arrays
        each the pixels from each input array are type cast into the output 
        array
return

ALGORITHM REFERENCES            none
*******************************************************************************/
#include "worgen.h"
#include "pixman.h"
#include "typlim.h"

static void byte_word
(
    const unsigned char *frombuf,
    short *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (short) *frombuf++)
        ;
}

static void byte_long
(
    const unsigned char *frombuf,
    int *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (int) *frombuf++)
        ;
}

static void byte_real
(
    const unsigned char *frombuf,
    float *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (float) *frombuf++)
        ;
}

static void byte_double
(
    const unsigned char *frombuf,
    double *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (double) *frombuf++)
        ;
}

static int word_byte
(
    const short *frombuf,
    unsigned char *tobuf,
    int size
)
{
    int flag;

    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXUTINY)
        {
            *tobuf = MAXUTINY;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINUTINY)
        {
            *tobuf = MINUTINY;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (unsigned char)*frombuf;
    }
    return (flag);
}

static void word_long
(
    const short *frombuf,
    int *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (int)*frombuf++)
        ;
}

static void word_real
(
    const short *frombuf,
    float *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (float)*frombuf++)
        ;
}

static void word_double
(
    const short *frombuf,
    double *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (double)*frombuf++)
        ;
}

static int long_byte
(
    const int *frombuf,
    unsigned char *tobuf,
    int size
)
{
    int flag;

    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXUTINY)
        {
            *tobuf = MAXUTINY;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINUTINY)
        {
            *tobuf = MINUTINY;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (unsigned char)*frombuf;
    }
    return (flag);
}

static int long_word
(
    const int *frombuf,
    short *tobuf,
    int size
)
{
    int flag;

    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXWORD)
        {
            *tobuf = MAXWORD;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINWORD)
        {
            *tobuf = MINWORD;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (short)*frombuf;
    }
    return (flag);
}

static void long_real
(
    const int *frombuf,
    float *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (float)*frombuf++)
        ;
}

static void long_double
(
    const int *frombuf,
    double *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (double)*frombuf++)
        ;
}

static int real_byte
(
    const float *frombuf,
    unsigned char *tobuf,
    int size
)
{
    int flag;

    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXUTINY)
        {
            *tobuf = MAXUTINY;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINUTINY)
        {
            *tobuf = MINUTINY;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (unsigned char)*frombuf;
    }
    return (flag);
}

static int real_word
(
    const float *frombuf,
    short *tobuf,
    int size
)
{
    int flag;

    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXWORD)
        {
            *tobuf = MAXWORD;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINWORD)
        {
            *tobuf = MINWORD;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (short)*frombuf;
    }
    return (flag);
}

static int real_long
(
    const float *frombuf,
    int *tobuf,
    int size
)
{
    int flag;

    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXLONG)
        {
            *tobuf = MAXLONG;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINLONG)
        {
            *tobuf = MINLONG;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (int)*frombuf;
    }
    return (flag);
}

static void real_double
(
    const float *frombuf,
    double *tobuf,
    int size
)
{
    for (; size--; *tobuf++ = (double)*frombuf++)
        ;
}

static int double_byte
(
    const double *frombuf,
    unsigned char *tobuf,
    int size
)
{
    int flag;

    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXUTINY)
        {
            *tobuf = MAXUTINY;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINUTINY)
        {
            *tobuf = MINUTINY;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (unsigned char)*frombuf;
    }
    return (flag);
}

static int double_word
(
    const double *frombuf,
    short *tobuf,
    int size
)
{
    int flag;

    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXWORD)
        {
            *tobuf = MAXWORD;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINWORD)
        {
            *tobuf = MINWORD;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (short)*frombuf;
    }
    return (flag);
}

static int double_long
(
    const double *frombuf,
    int *tobuf,
    int size
)
{
    int flag;
 
    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXLONG)
        {
            *tobuf = MAXLONG;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINLONG)
        {
            *tobuf = MINLONG;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (int)*frombuf;
    }
    return (flag);
}

static int double_real
(
    const double *frombuf,
    float *tobuf,
    int size
)
{
    int flag;

    for (; size--; tobuf++, frombuf++)
    {
        if (*frombuf > MAXREAL)
        {
            *tobuf = MAXREAL;
            flag |= TOOBIG;
        }
        else if (*frombuf < MINREAL)
        {
            *tobuf = MINREAL;
            flag |= TOOSMALL;
        }
        else
            *tobuf = (float)*frombuf;
    }
    return (flag);
}

/**
 * las_pixconvert:
 * @ftyp: I: An integer representing the data type of the source
 *           buffer.  Legal values include:
 *           EBYTE:   Unsigned byte data,
 *           EWORD:   Signed two byte data,
 *           ELONG:   Signed four byte data,
 *           EREAL:   Four byte floating point data.
 * @totyp: I: An integer representing the data type of the
 *            destination buffer.  Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Signed four byte data,
 *            EREAL:  Four byte floating point data.
 * @frombuf: I: The  source buffer of the data to be converted;
 *              this data is of type ftyp. 
 * @tobuf: O: The destination buffer of the converted data;
 *            this data will be of type totyp.
 * @size: I: The number of pixels to be converted in the input buffer. 
 *
 * The las_pixconvert() call is used to convert size pixels of type ftyp to
 * totyp.  If the two data types are the same, c_pxcopy() is called.
 *
 * Returns:  E_SUCC:   Successful completion,
 *           TOOBIG:   Exceeded upper limit of data type,
 *           TOOSMALL: Exceeded lower limit of data type.
 */
int las_pixconvert
(
    int ftyp,               /* I:  from type                              */
    int totyp,              /* I:  to type                                */
    const void *frombuf,    /* I:  from buffer                            */
    void *tobuf,            /* O:  to buffer                              */
    int size                /* I:  number of samples in the input buffer  */
)
{ /* pxconv */
    int status = 0;

    switch (ftyp)
    {
        case EBYTE:
            switch (totyp)
            {
                case EBYTE:   
                    las_pixcopy(frombuf,tobuf,totyp,size);
                    break;
                case EWORD:   
                    byte_word((unsigned char *)frombuf,
                              (short *)tobuf,size);
                    break;
                case ELONG:   
                    byte_long((unsigned char *)frombuf,
                              (int *)tobuf,size);
                    break;
                case EREAL:   
                    byte_real((unsigned char *)frombuf,
                              (float *)tobuf,size);
                    break;
                case EDOUBLE: 
                    byte_double((unsigned char *)frombuf,
                                (double *)tobuf,size);
                    break;
            } 
            break;
        case EWORD:
            switch (totyp)
            {
                case EBYTE:   
                    status = word_byte((short *)frombuf,tobuf,size);
                    break;
                case EWORD:   
                    las_pixcopy(frombuf,tobuf, totyp, size);
                    break;
                case ELONG:   
                    word_long((short *)frombuf,(int *)tobuf,size);
                    break;
                case EREAL:   
                    word_real((short *)frombuf,(float *)tobuf,
                               size);
                    break;
                case EDOUBLE: 
                    word_double((short *)frombuf,
                                (double *)tobuf,size);
                    break;
            }
            break;
        case ELONG:
            switch (totyp)
            {
                case EBYTE:   
                    status = long_byte((int *)frombuf,tobuf,size);
                    break;
                case EWORD:   
                    status = long_word((int *)frombuf,(short *)tobuf,size);
                    break;
                case ELONG:   
                    las_pixcopy(frombuf, tobuf, totyp, size);
                    break;
                case EREAL:   
                    long_real((int *)frombuf,(float *)tobuf,size);
                    break;
                case EDOUBLE: 
                    long_double((int *)frombuf,(double *)tobuf,
                                 size);
                    break;
            }
            break;
        case EREAL:
            switch (totyp)
            {
                case EBYTE:   
                    status = real_byte((float *)frombuf,tobuf,size);
                    break;
                case EWORD:   
                    status = real_word((float *)frombuf,(short *)tobuf,
                                        size);
                    break;
                case ELONG:   
                    status = real_long((float *)frombuf,(int *)tobuf,size);
                    break;
                case EREAL:   
                    las_pixcopy(frombuf, tobuf, totyp, size);
                    break;
                case EDOUBLE: 
                    real_double((float *)frombuf,(double *)tobuf,
                                 size);
                    break;
            }
            break;
        case EDOUBLE:
            switch (totyp)
            {
                case EBYTE:   
                    status = double_byte((double *)frombuf,tobuf,size);
                    break;
                case EWORD:   
                    status = double_word((double *)frombuf,(short *)tobuf,
                                          size);
                    break;
                case ELONG:   
                    status = double_long((double *)frombuf,(int *)tobuf,
                                          size);
                    break;
                case EREAL:   
                    status = double_real((double *)frombuf,(float *)tobuf,
                                 size);
                    break;
                case EDOUBLE: 
                    las_pixcopy(frombuf, tobuf, totyp, size);
                    break;
            }
            break;
        }

    if (!status)
        status = E_SUCC;

    return (status);
}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_pxconv:
 * @ftyp: I: An integer representing the data type of the source
 *           buffer.  Legal values include:
 *           EBYTE:  Unsigned byte data,
 *           EWORD:  Signed two byte data,
 *           ELONG:  Signed four byte data,
 *           EREAL:  Four byte floating point data.
 * @totyp: I: An integer representing the data type of the
 *            destination buffer.  Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Signed four byte data,
 *            EREAL:  Four byte floating point data.
 * @frombuf: I: The  source buffer of the data to be converted;
 *              this data is of type ftyp. 
 * @tobuf: O: The destination buffer of the converted data;
 *            this data will be of type totyp.
 * @size: I: The number of pixels to be converted in the input buffer. 
 *
 * The c_pxconv() call is used to convert size pixels of type ftyp to
 * totyp.  If the two data types are the same, c_pxcopy() is called.
 *
 * Deprecated.  Use las_pixconvert() instead.
 *
 * Returns:  E_SUCC:   Successful completion,
 *           TOOBIG:   Exceeded upper limit of data type,
 *           TOOSMALL: Exceeded lower limit of data type.
 */
int c_pxconv
(
    const long *ftyp,       /* I: from type                              */
    const long *totyp,      /* I: to type                                */
    const void *frombuf,    /* I: from buffer                            */
    void *tobuf,            /* O: to buffer                              */
    const long *size        /* I: number of samples in the input buffer  */
)

{ /* pxconv */
    int status = 0;

    status = las_pixconvert (*ftyp,*totyp,frombuf,tobuf,*size); 

    return (status);
}
#endif /* Obsolete interfaces */

