/******************************************************************************
NAME                    pxdiv

PURPOSE         Divide two input arrays into one output array.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        D.GORDON        5/7/85  Original development for NEWLAS
        D.GORDON        11/20/85 Optimumized all routines.
        B.Ailts            12/87        Placed bridge routines in seperate file
                                        Used raw 'C' types
                                        Changed include directory specifications
        B.Ailts         03/88    Replaced newlas.h with worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system.

PROJECT                         LAS

ALGORITHM 

receive:
        -the input arrays and their type
        -the output array
        -the number of samples of the input arrays to divide.
        -the scaleing factors and the offset.
        -the value to be returned in the case of divide by zero.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        if the denominator is going to be a zero then
                set the status to DIVZERO
                set the output value to errval (specified by user)
        otherwise:

        divide the pixels from input arrays and store result in output array
            (ie. output = inputarray1 / inputarray2 * scale + offset)
        if output is smaller than the lower limit of the data type then 
                output = the lower limit of data type and error is returned
        if output is larger than the higher limit of the data type then
                output = the higher limit of data type and error is returned.
return
*note:  Each operation is performed with variables one data type larger than the
        desired dta type. This makes checking for the operation exceding the
        bounds of the data type very easy while maintaining a fast algorithm.

*****************************************************************************/
#include "worgen.h"
#include "pixman.h"
#include "typlim.h"

#define DIVZERO 44

static int bdiv
(
    int ns,
    const unsigned char *in1,
    const unsigned char *in2,
    unsigned char *out,
    float scale,
    float offset,
    const unsigned char *errval
    
)

{
    int status;
    int tempin1;
    int tempin2;
    int tempout;

    if ((scale == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if(*in2 == 0)
            {
                status = DIVZERO;
                tempout = *errval;
            }
            else if ((tempout = tempin1 / tempin2) > MAXUTINY)
            {
                tempout = MAXUTINY;
                status = TOOBIG;
            }
            else if (tempout < MINUTINY)
            {
                tempout = MINUTINY;
                status = TOOSMALL;
            }
            *out = tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if(*in2 == 0)
            {
                status = DIVZERO;
                tempout = *errval;
            }
            else tempout = (tempin1 / tempin2) * scale + offset;
            if (tempout > MAXUTINY)
            {
                tempout = MAXUTINY;
                status = TOOBIG;
            }
            else    if (tempout < MINUTINY)
            {
                tempout = MINUTINY;
                status = TOOSMALL;
            }
            *out = tempout;
        }
    }
    return (status);
}

static int wdiv
(
    int ns,
    const short *in1,
    const short *in2,
    short *out,
    float scale,
    float offset,
    const short *errval
)

{
    int status;
    int tempin1;
    int tempin2;
    int tempout;

    if ((scale == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if(*in2 == 0)
            {
                status = DIVZERO;
                tempout = *errval;
            }
            else if ((tempout = tempin1 / tempin2) > MAXWORD)
            {
                tempout = MAXWORD;
                status = TOOBIG;
            }
            else  if (tempout < MINWORD)
            {
                tempout = MINWORD;
                status = TOOSMALL;
            }
            *out = (short)tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if(*in2 == 0)
            {
                status = DIVZERO;
                tempout = *errval;
            }
            else tempout = (long)((float)(tempin1 / tempin2) * scale 
                            + offset);
                if (tempout > MAXWORD)
                {
                    tempout = MAXWORD;
                    status = TOOBIG;
                }
                else    if (tempout < MINWORD)
                {
                    tempout = MINWORD;
                    status = TOOSMALL;
                }
                *out = (short)tempout;
        }
    }
    return (status);
}

static int ldiv
(
    int ns,
    const int *in1,
    const int *in2,
    int *out,
    float scale,
    float offset,
    const int *errval
)
{
    double status;
    double tempin1;
    double tempin2;
    double tempout;

    if ((scale == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if(*in2 == 0)
            {
                status = DIVZERO;
                tempout = *errval;
            }
            else if ((tempout = tempin1 / tempin2) > MAXLONG)
            {
                tempout = MAXLONG;
                status = TOOBIG;
            }
            else  if (tempout < MINLONG)
            {
                tempout = MINLONG;
                status = TOOBIG;
            }
            *out = (long)tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if(*in2 == 0)
            {
                status = DIVZERO;
                tempout = *errval;
            }
            else tempout = (tempin1 / tempin2) * scale + offset;
                if (tempout > MAXLONG)
                {
                    tempout = MAXLONG;
                    status = TOOBIG;
                }
                else    if (tempout < MINLONG)
                {
                    tempout = MINLONG;
                    status = TOOBIG;
                }
                *out = (long)tempout;
        }
    }
    return (status);
}

static int rdiv
(
    int ns,
    const float *in1,
    const float *in2,
    float *out,
    float scale,
    float offset,
    const float *errval
)
{
    int status;
    double tempin1;
    double tempin2;
    double tempout;

    if ((scale == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if(*in2 == 0)
            {
                status = DIVZERO;
                tempout = *errval;
            }
            else tempout = (tempin1 / tempin2) * scale + offset;
            if (tempout > MAXREAL)
            {
                tempout = MAXREAL;
                status = TOOBIG;
            }
            else    if (tempout < MINREAL)
            {
                tempout = MINREAL;
                status = TOOBIG;
            }
            *out = tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if(*in2 == 0)
            {
                status = DIVZERO;
                tempout = *errval;
            }
            else tempout = (tempin1 / tempin2) * scale + offset;
            if (tempout > MAXREAL)
            {
                tempout = MAXREAL;
                status = TOOBIG;
            }
            else    if (tempout < MINREAL)
            {
                tempout = MINREAL;
                status = TOOBIG;
            }
            *out = tempout;
        }
    }
    return (status);
}


/**
 * las_pixdivide:
 * @ns:  I: The number of samples to be processed.
 * @in1: I: An array of pixel values to be divided (numerator).
 * @in2: I: An array of divisor pixel values (denominator).
 * @out: O:  An array created by dividing, scaling, and
 *           offsetting the two input arrays.
 *           out = (in1 / in2 ) * scale + offset
 * @scale: I: The scale factor to be applied after division.
 * @offset: I: The offset to be applied after division.
 * @dtype: I: An integer value that represents the
 *            type of data in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 * @errval: I: An integer value to be placed in the output array
 *             at all locations where a divide by zero was attempted.
 *             The errval should be of the same type as the in and 
 *             out buffers.
 *
 * The las_pixdivide() call is used to create an output array by dividing,
 * scaling, and offsetting ns pixel values of two input pixel arrays
 * in1 and in2 of type dtype. 
 *
 * Returns:    E_SUCC:   Successful completion,
 *             TOOBIG:   Exceeded upper limit of data type,
 *             TOOSMALL: Exceeded lower limit of data type,
 *             TYPERR:   Invalid data type specified,
 *             DIVZERO:  Divide by zero.
*/
int las_pixdivide
(
    int ns,                 /* I:  number of samples                      */
    const void *in1,        /* I:  input array one                        */
    const void *in2,        /* I:  input array two                        */
    void *out,              /* O:  output array                           */
    float scale,            /* I:  scale applied to in. arrays after mult */
    float offset,           /* I:  offset added in after sum              */
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    const void *errval      /* I:  value to place in array where div zero */
)
{
    int status = E_SUCC;        /* return status for pxmul */

    switch (dtype)
    {
        case (EBYTE): 
            status = bdiv(ns,(unsigned char *)in1,(unsigned char *)in2,
                     (unsigned char *)out,scale,offset,
                     (unsigned char *)errval);
            break;
        case (EWORD): 
            status = wdiv(ns,(short *)in1,(short *)in2,(short *)out,scale,
                     offset,(short *)errval); 
            break;
        case (ELONG): 
            status = ldiv(ns,(int *)in1,(int *)in2,(int *)out,scale,
                     offset,(int *)errval);
            break;
        case (EREAL): 
            status = rdiv(ns,(float *)in1,(float *)in2,(float *)out,scale,
                     offset,(float *)errval); 
            break;
        default:      
            status = TYPERR;
            break;

    } /* switch */

    return (status);

} /* pxdiv */

#ifndef NO_OBSOLETE_INTERFACE
/**
 * c_pxdiv:
 * @ns:  I: The number of samples to be processed.
 * @in1: I: An array of pixel values to be divided (numerator).
 * @in2: I: An array of divisor pixel values (denominator).
 * @out: O:  An array created by dividing, scaling, and
 *           offsetting the two input arrays.
 *           out = (in1 / in2 ) * scale + offset
 * @scale: I: The scale factor to be applied after division.
 * @offset: I: The offset to be applied after division.
 * @dtype: I: An integer value that represents the
 *            type of data in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 * @errval: I: An integer value to be placed in the output array
 *             at all locations where a divide by zero was attempted.
 *             The errval should be of the same type as the in and
 *             out buffers.
 *
 * The c_pxdiv() call is used to create an output array by dividing,
 * scaling, and offsetting ns pixel values of two input pixel arrays
 * in1 and in2 of type dtype. 
 *
 * Deprecated.  Use las_pixdivide() instead.
 *
 * Returns:    E_SUCC:   Successful completion,
 *             TOOBIG:   Exceeded upper limit of data type,
 *             TOOSMALL: Exceeded lower limit of data type,
 *             TYPERR:   Invalid data type specified,
 *             DIVZERO:  Divide by zero.
*/
int c_pxdiv
(
    const long *ns,      /* I: number of samples                          */
    const void *in1,     /* I: input array one                            */
    const void  *in2,    /* I: input array two                            */
    void *out,           /* O: output array                               */
    const float *scale,  /* I: scale applied to input arrays after mult.  */
    const float *offset, /* I: offset added in after sum                  */
    const long *dtype,   /* I: data type (EBYTE,EWORD,ELONG,EREAL)        */
    const long *errval   /* I: value to place in array where div zero     */
)

{
    int status = E_SUCC;        /* return status for pxmul */

    status = las_pixdivide (*ns,in1,in2,out,*scale,*offset,*dtype,errval);

    return (status);

} /* pxdiv */

#endif /* obsolete interfaces */

