/*****************************************************************************
NAME                    pxfill

PURPOSE         Fill an output array with a specified value.  The 
                output array will be of the same type as 'dtype'.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        D.GORDON        5/7/85   Original development for NEWLAS
        J.REED          11/12/85 Optimized byte portion of routine.
        B.Ailts          12/87   Changed include directory specifications
                                 Use raw 'C' types
                                 Place bridge routines in a seperate file
        B.Ailts          04/88   replaced newlas.a with las.a

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system.

PROJECT                         NEWLAS

ALGORITHM 

receive:
        -the output array and its data type
        -the number of samples to write to the output array
        -the value to be placed within the output array

Call the routine which corresponds to the input data type

for each sample of the output array
        copy the specified value to the output array
return

ALGORITHM REFERENCES            none
*******************************************************************************/

#include "worgen.h"
#include "pixman.h"


static void realfill
(
    float *rtoptr,
    int ns,
    const float *fillval
)
{
    for (; ns--; *rtoptr++ = *fillval)
        ;
}

static void bytefill
(
    unsigned char *btoptr,
    int ns,
    const unsigned char *fillval
)
{
    for (;ns--; *btoptr++ = *fillval)
        ;
}

static void wordfill
(
    short *wtoptr,
    int ns,
    const short *fillval
)
{
    for (; ns--; *wtoptr++ = *fillval)
        ;
}

static void longfill
(
    int *ltoptr,
    int ns,
    const int *fillval
)
{
    for (; ns--; *ltoptr++ = *fillval)
        ;
}

/**
 * las_pixfill:
 * @tobuf: O: The buffer to be filled with fillval.
 * @dtype: I: An integer representing the data type of the
 * fill value.  Legal values include the following:
 *      EBYTE:  Unsigned byte data,
 *      EWORD:  Signed two byte data,
 *      ELONG:  Four byte floating point data,
 *      EREAL:  Four byte floating point data.
 * @ns: I: The number of elements or pixels to be filled.
 * @fillval: I: The fill value for the buffer.  The fill
 * value must be the same data type as the buffer.
 *
 * The las_pixfill() call is used to fill ns pixels of buffer
 * tobuf with constant value fillval of type dtype.
 *
 */

 
void las_pixfill
(
    void *tobuf,            /* O: the buffer to be filled with fillval   */
    int dtype,              /* I: data type (EBYTE,EWORD,ELONG,EREAL)    */
    int ns,                 /* I: number of elements/pixels to be filled */
    const void *fillval     /* I: fill value for the buffer              */
)
{

    switch (dtype)
    {
        case EBYTE: 
            bytefill(tobuf, ns, fillval);
            break;
        case EWORD: 
            wordfill((short *)tobuf, ns, (short *)fillval);
            break;
        case ELONG: 
            longfill((int *)tobuf, ns, (int *)fillval);
            break;
        case EREAL: 
            realfill((float *)tobuf, ns, (float *)fillval);
            break;
    }
}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_pxfill:
 * @tobuf: O: The buffer to be filled with fillval.
 * @dtype: I: An integer representing the data type of the
 * fill value.  Legal values include the following:
 *      EBYTE:  Unsigned byte data,
 *      EWORD:  Signed two byte data,
 *      ELONG:  Four byte floating point data,
 *      EREAL:  Four byte floating point data.
 * @ns: I: The number of elements or pixels to be filled.
 * @fillval: I: The fill value for the buffer.  The fill
 * value must be the same data type as the buffer.
 *
 * The c_pxfill() call is used to fill ns pixels of buffer
 * tobuf with constant value fillval of type dtype.
 *
 * Deprecated.  Use las_pixfill() instead.
 *
 */
void c_pxfill
(
    void *tobuf,        /* O: the buffer to be filled with fillval      */
    const long *dtype,  /* I: data type (EBYTE,EWORD,ELONG,EREAL)       */
    const long *ns,     /* I: number of elements or pixels to be filled */
    const void *fillval /* I: fill value for the buffer                 */
)

{
    las_pixfill (tobuf,*dtype,*ns,fillval);

}

#endif /* Obsolete interfaces */
