/*************************************************************
NAME                    c_pxindr

PURPOSE                 Searches an array from end to beginning
                        to find the first occurence not equal to  a specified 
                        value.

EXTERNAL ASSOCIATIONS           none

FILE REFERENCES                 none

EXTERNAL VARIABLES REFERENCED   none

DATA STRUCTURES none

SUBPROGRAMS CALLED 
NAME            DESCRIPTION
----            -----------
bindx           performs pixel search on unsigned byte data
windx           performs pixel search on signed integer data
lindx           performs pixel search on signed long data 
rindx           performs pixel search on real data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS      none.

FATAL ERROR CONDITIONS          TYPERR 66 (choices: EBYTE,EWORD,ELONG,or EREAL)

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        M. Mazzella     7/87    original development for NEWLAS
        T. Baltzer      9/90    make E_SUCC the default return status
        D. Etrheim      4/96    Make zero relative and correct pointer
                                arithmetic

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
        none

PROJECT                         NEWLAS

LANGUAGE                        C

ALGORITHM (FLOW)

receive:
        -the input array and its type
        -the number of samples of the input array.
        -the number to be searched for
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        Assign the output variable 'index' the index of the input array of the 
        first occurence not equal to 'val', beginning at the end of the array
return

ALGORITHM REFERENCES            none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void bindr
(
    const unsigned char *in,
    int val,
    int *index,
    int ns
)
{
    for (*index = (ns - 1); *index && in[*index] == val; (*index)--)
        ;
}

static void windr
(
    const short *in,
    int val,
    int *index,
    int ns
)
{
    for (*index = (ns - 1); *index && in[*index] == val; (*index)--)
        ;
}

static void lindr
(
    const int *in,
    int val,
    int *index,
    int ns
)
{
    for (*index = (ns - 1); *index && in[*index] == val; (*index)--)
        ;
}

static void rindr
(
    const float *in,
    double val,
    int *index,
    int ns
)
{
    for (*index = (ns - 1); *index && in[*index] == val; (*index)--)
        ;
}

/**
 * las_pxindr:
 * @in: I: The input array.
 * @val: I: Value of elements to be skipped.  Must be same type as in.
 * @index: O: Index into array of first value not equal to val.
 * @ns: I: Number of elements in the input array.
 * @dtype: I: Integer value representing the type of data.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 *
 * las_pxindr() searches an array to find the first value in the
 * array which is not equal to the value specified in the variable
 * val.  The array is searched from the end back.
 * 
 * Returns: E_SUCC:   Successful completion,
 *          TYPERR:   Invalid type specified.
 */
int las_pxindr
(
    const void *in,         /* I:  input array                            */
    int val,                /* I:  value to be searched for               */
    int ns,                 /* I:  number of samples                      */
    int *index,             /* 0:  index to be returned                   */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
)
{
    int status = E_SUCC;

    switch (dtype)
    {
        case (EBYTE): 
            bindr((unsigned char *)in,val,index,ns); 
            break;
        case (EWORD): 
            windr((short *)in,val,index,ns); 
            break;
        case (ELONG): 
            lindr((int *)in,val,index,ns); 
            break;
        case (EREAL): 
            rindr((float *)in,val,index,ns); 
            break;
        default:      
            status = TYPERR;
            break;
    } /* switch */

    return (status);

} /* pxindr */


/**
 * c_pxindr:
 * @in: I: The input array.
 * @val: I: Value of elements to be skipped.  Must be same type as in.
 * @index: O: Index into array of first value not equal to val.
 * @ns: I: Number of elements in the input array.
 * @dtype: I: Integer value representing the type of data.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 *
 * c_pxindr() searches an array to find the first value in the
 * array which is not equal to the value specified in the variable
 * val.  The array is searched from the end back.
 * 
 * Deprecated.  Use las_pxindr() instead.
 * 
 * Returns:  E_SUCC:   Successful completion,
 *           TYPERR:   Invalid type specified.
 */
int c_pxindr
(
    const void *in,    /* I: input array                                  */
    const long *val,   /* I: value to be searched for                     */
    const long *ns,    /* I: number of samples                            */
    long *index,       /* O: index to be returned                         */
    const long *dtype  /* I: data type (EBYTE,EWORD,ELONG,EREAL)          */
)

{
    int status = E_SUCC;

    status = las_pxindr(in,*val,*ns,(int *)index,*dtype);

    return (status);

} /* pxindr */
