/*************************************************************
NAME                    c_pxline

PURPOSE         Perform a linear transformation of each element of an input 
                array generating an output array which is the result.
                The input array can be of type byte, signed 2 byte, signed 4
                byte, or real. The output array will be of the same type as the
                input array.

EXTERNAL ASSOCIATIONS           none

FILE REFERENCES                 none

EXTERNAL VARIABLES REFERENCED   none

DATA STRUCTURES none

SUBPROGRAMS CALLED 
NAME            DESCRIPTION
----            -----------
blin            performs linear transformation on unsigned byte data
wlin            performs linear transformation on signed integer data
llin            performs linear transformation on signed long data 
flin            performs linear transformation on real data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS      none.

FATAL ERROR CONDITIONS          TYPERR 66 (choices: EBYTE,EWORD,ELONG,or EREAL)

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        M. Mazzella     6/87    Original development for NEWLAS
        B. Ailts        12/88   SCALE + OFFSET arguments are now FLOAT

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                c_pxlin must be linked with the imageio routines.

PROJECT                         NEWLAS

LANGUAGE                        C

ALGORITHM (FLOW)

receive:
        -the input array and its type
        -the output array
        -the number of samples of the input array.
        -the sampling factor used
        -the offset used
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        Assign the output array pixel the product of the input array and
        the scale factor plus the offset.
return

ALGORITHM REFERENCES            none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void  blin
(
    const unsigned char *in,
    unsigned char *out,
    int ns,
    float scale,
    float offset
)
{
    for (; ns; ns--,in++,out++)
        *out = *in * scale + offset;
}

static void wlin
(
    const short *in,
    short *out,
    int ns,
    float scale,
    float offset
)
{
    for (; ns; ns--,in++,out++)
        *out = *in * scale + offset;
}

static void llin
(
    const int *in,
    int *out,
    int ns,
    double scale,
    double offset
)
{
    for (; ns; ns--,in++,out++)
        *out = (int)(*in * scale + offset);
}

static void flin
(
    const float *in,
    float *out,
    int ns,
    float scale,
    float offset
)
{
    for (; ns; ns--,in++,out++)
        *out = *in * scale + offset;
}


/**
 * las_pixlinear:
 * @in: I: Input array to be transformed.  Must be same type as out.
 * @out: O: Output array to hold result.  Must be same type as in.
 * @ns: I: The number of elements in the input array.
 * @dtype: Integer value representing the type of data.
 *         Legal values include:
 *         EBYTE:  Unsigned byte data,
 *         EWORD:  Signed two byte data,
 *         ELONG:  Four byte floating point data,
 *         EREAL:  Four byte floating point data.
 * @scale: I: Factor by which each element in the array
 *            will be multiplied. 
 * @offset: I: Number to be added to each element of the array.
 *
 * las_pixlinear() performs a linear transformation on each element of
 * an input array.  The input array and output array must be of
 * the same type.
 *
 * Returns:   E_SUCC:   Successful completion,
 *            TYPERR:   Invalid type specified.
 */
int las_pixlinear
(
    const void *in,         /* I:  input array                            */
    void *out,              /* O:  output array                           */
    int ns,                 /* I:  number of samples                      */
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    float scale,            /* I:  scale factor                           */
    float offset            /* I:  offset to be added                     */
)
{
    int status = E_SUCC;        /* return status for c_pxline */

    switch (dtype)
    {
        case (EBYTE): 
            blin((unsigned char *)in,(unsigned char *)out,ns,scale,
                  offset); 
            break;
        case (EWORD): 
            wlin((short *)in,(short *)out,ns,scale,offset);
            break;
        case (ELONG): 
            llin((int *)in,out,ns,scale,offset);
            break;
        case (EREAL): 
            flin((float *)in,(float *)out,ns,scale,offset);
            break;
        default:      
            status = TYPERR;
            break;

    } /* switch */

    return (status);

} /* c_pxline */

/**
 * c_pxline:
 * @in: I: Input array to be transformed.  Must be same type as out.
 * @out: O: Output array to hold result.  Must be same type as in.
 * @ns: I: The number of elements in the input array.
 * @dtype: Integer value representing the type of data.
 *         Legal values include:
 *         EBYTE:  Unsigned byte data,
 *         EWORD:  Signed two byte data,
 *         ELONG:  Four byte floating point data,
 *         EREAL:  Four byte floating point data.
 * @scale: I: Factor by which each element in the array
 *            will be multiplied. 
 * @offset: I: Number to be added to each element of the array.
 *
 * c_pxline() performs a linear transformation on each element of
 * an input array.  The input array and output array must be of
 * the same type.
 *
 * Deprecated.  Use las_pixlinear() instead.
 *
 * Returns:   E_SUCC:   Successful completion,
 *            TYPERR:   Invalid type specified.
 */
int c_pxline
(
    const void *in,     /* I: input array                                 */
    void *out,          /* O: output array                                */
    const long *ns,     /* I: number of samples                           */
    const long *dtype,  /* I: data type (EBYTE,EWORD,ELONG,EREAL)         */
    const float *scale, /* I: scale factor                                */
    const float *offset /* I: offset to be added                          */
)

{
    int status = E_SUCC;        /* return status for c_pxline */

    status = las_pixlinear (in,out,*ns,*dtype,*scale,*offset);

    return (status);

} /* c_pxline */
