
/*************************************************************
NAME                    c_pxlkup

PURPOSE         Perform a lut transformation of each element of an input 
                array generating an output array which is the result.
                The input array can be of type byte or signed 2 byte.
                The output array will be of the same type as the lut array.

EXTERNAL ASSOCIATIONS           none

FILE REFERENCES                 none

EXTERNAL VARIABLES REFERENCED   none

DATA STRUCTURES none

SUBPROGRAMS CALLED 
NAME            DESCRIPTION
----            -----------
blutb           performs byte lut transformation on unsigned byte data
blutw           performs integer lut transformation on unsigned byte data
wlutw           performs integer lut transformation on integer data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS      none.

FATAL ERROR CONDITIONS          TYPERR 66 (choices: EBYTE or EWORD)

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        M. Mazzella     6/87    Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                c_pxlkup must be linked with the imageio routines.

PROJECT                         NEWLAS

LANGUAGE                        C

ALGORITHM (FLOW)

receive:
        -the input array and its type
        -the output array
        -the number of samples of the input array.
        -the lookup table.
        -the lookup table type.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        Assign the output array pixel the lut value of the input array 
return

ALGORITHM REFERENCES            none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void blutb
(
    const unsigned char *in,
    unsigned char *out,
    int ns,
    const unsigned char *lut
)
                
{
    for (; ns; ns--,in++,out++)
        *out = lut[*in];
}

static void blutw
(
    const unsigned char *in,
    short *out,
    int ns,
    const short *lut
)
                
{
    for (; ns; ns--,in++,out++)
        *out = lut[*in];
}

static void wlutw
(
    const short *in,
    short *out,
    int ns,
    const short *lut
)
                
{
    for (; ns; ns--,in++,out++)
        *out = lut[*in];
}

/** 
 * las_pxlookup:
 * @in: I: The input array.
 * @out: O: The output array to hold the result.
 * @ns: I: Number of elements in the input arrays.
 * @dtype: I: Integer value representing the type of data in
 *            input array.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 * @lut: I: Factor by which each element in the array will be multiplied.
 * @luttype: I: Integer value representing the type of data in
 *              look-up table.  Legal values include:
 *              EBYTE:   Unsigned byte data,
 *              EWORD:   Signed 2byte data.  
 *
 * las_pxlookup() performs a look-up table transformation on each
 * element of an input array.
 *
 * Returns:  E_SUCC:   Successful completion,
 *           TYPERR:   Invalid type specified.
 */
int las_pxlookup
(
    const void *in,         /* I:  input array                            */
    void *out,              /* O:  output array                           */
    int ns,                 /* I:  number of samples                      */
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    const void *lut,        /* I:  lookup table array                     */
    int luttype             /* I:  data type of lookup table              */
)
{
    int status = E_SUCC;        /* return status for c_pxlkup */
    switch (dtype)
    {
        case (EBYTE):
        {
            if (luttype == EBYTE)
            {
                 blutb((unsigned char *)in,(unsigned char *)out,ns,
                       (unsigned char *)lut);
                 break;
            }
            else
                if (luttype == EWORD)
                {
                    blutw((unsigned char *)in,(short *)out,ns,
                                   (short *)lut);
                    break;
                }
                else
                {
                    status = TYPERR;
                    break;
                }
        }
        case (EWORD): 
            wlutw((short *)in,(short *)out,ns,(short *)lut); 
            break;
        case (ELONG):
        case (EREAL):
        default:
            status = TYPERR;
            break;
    } /* switch */

    return (status);
} /* c_pxlkup */


#ifndef NO_OBSOLETE_INTERFACE

/** 
 * c_pxlkup:
 * @in: I: The input array.
 * @out: O: The output array to hold the result.
 * @ns: I: Number of elements in the input arrays.
 * @dtype: I: Integer value representing the type of data in
 *            input array.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 * @lut: I: Factor by which each element in the array will be multiplied.
 * @luttype: I: Integer value representing the type of data in
 *              look-up table.  Legal values include:
 *              EBYTE:   Unsigned byte data,
 *              EWORD:   Signed 2byte data.  
 *
 * c_pxlkup() performs a look-up table transformation on each
 * element of an input array.
 *
 * Deprecated.  Use las_pxlkup() instead. 
 *
 * Returns: E_SUCC:   Successful completion,
 *          TYPERR:   Invalid type specified.
 */
int c_pxlkup
(
    const void *in,     /* I: input array                                 */
    void *out,          /* O: output array                                */
    const long *ns,     /* I: number of samples                           */
    const long *dtype,  /* I: data type (EBYTE,EWORD,ELONG,EREAL)         */
    const void *lut,    /* I: lookup table array                          */
    const long *luttype /* I: data type of lookup table                   */
)

{
    int status = E_SUCC;        /* return status for c_pxlkup */

    status = las_pxlookup(in,out,*ns,*dtype,lut,*luttype);

    return (status);

} /* c_pxlkup */

#endif /* obsolete interfaces */

