/*************************************************************
NAME                    pxmax


PURPOSE         Determine the maximum values of either the input array or
                the mapping array.  Assign the maximum value to the output
                array.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        R. MCKINNEY     9/87    Original development for NEWLAS
        B. Ailts        4/88    Replaced newlas.h with worgen.h
                                Used raw C types -- removed stdedc.h
                                Moved fortran bridge to seperate file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system 

PROJECT                         LAS

ALGORITHM 

receive:
        -the input array and its type
        -the input mapping array
        -the output array
        -the number of samples of the input array.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        Determine the maximum pixel values.
        Assign the output array pixel the maximum of the input array and
                the mapping array.
return

ALGORITHM REFERENCES            none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void bmax
(
    const unsigned char *in,
    const unsigned char *map,
    unsigned char *out,
    int ns
)
{
    for(; ns; ns--,in++,map++,out++)
    {
        if(*in >= *map) *out = *in;
        if(*in < *map) *out = *map;
    }
}

static void wmax
(
    const short *in,
    const short *map,
    short *out,
    int ns
)
{
    for(; ns; ns--,in++,map++,out++)
    {
        if(*in >= *map) *out = *in;
        if(*in < *map) *out = *map;
    }
}

static void lmax
(
    const int *in,
    const int *map,
    int *out,
    int ns
)
{
    for(; ns; ns--,in++,map++,out++)
    {
        if(*in >= *map) *out = *in; 
        if(*in < *map) *out = *map;
    }
}

static void rmax
(
    const float *in,
    const float *map,
    float *out,
    int ns
)
{
    for(; ns; ns--,in++,map++,out++)
    {
        if(*in >= *map) *out = *in;
        if(*in < *map) *out = *map;
    }
}

/**
 * las_pixmaximum:
 * @in: I: The input array.
 * @map: I: The input mapping array.
 * @out: O: The output array.
 * @ns: I: The number of samples to be scanned from the buffer.
 * @dtype: I: An integer value that represents the type of data in
 *            the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 * 
 * The las_pixmaximum() call is used to determine the maximum of an input
 * array with a mapping array to generate an output array.
 *
 * Returns:  E_SUCC:   Successful completion,
 *           TYPERR:   Invalid data type specified.
 */
int las_pixmaximum
(
    const void *in,         /* I:  input array                            */
    const void *map,        /* I:  input mapping array                    */
    void *out,              /* O:  output array                           */
    int ns,                 /* I:  number of samples                      */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
)
{
    int status = E_SUCC;        /* return status for pxmax */

    switch (dtype)
    {
        case (EBYTE): 
            bmax((unsigned char *)in,(unsigned char *)map,
                 (unsigned char *)out,ns);
            break;
        case (EWORD): 
            wmax((short *)in,(short *)map,(short *)out,ns);
            break;
        case (ELONG): 
            lmax((int *)in,(int *)map,out,ns);
            break;
        case (EREAL): 
            rmax((float *)in,(float *)map,(float *)out,ns);
            break;
        default:      
            status = TYPERR;
            break;
    } /* switch */

    return (status);

} /* pxmax */

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_pxmax:
 * @in: I: The input array.
 * @map: I: The input mapping array.
 * @out: O: The output array.
 * @ns: I: The number of samples to be scanned from the buffer.
 * @dtype: I: An integer value that represents the type of data in
 *            the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 * 
 * The c_pxmax() call is used to determine the maximum of an input
 * array with a mapping array to generate an output array.
 *
 * Deprecated.  Use las_pixmaximum() instead.
 *
 * Returns:  E_SUCC:   Successful completion,
 *           TYPERR:   Invalid data type specified.
 */
int c_pxmax
(
    const void *in,   /* I: input array                                  */
    const void *map,  /* I: input mapping array                          */
    void *out,        /* O: output  array                                */
    const long *ns,   /* I: number of samples                            */
    const long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)          */
)

{
    int status = E_SUCC;        /* return status for pxmax */

    status = las_pixmaximum (in,map,out,*ns,*dtype);

    return (status);

} /* pxmax */

#endif /* obsolete interfaces */

