/*************************************************************
NAME                    pxmin


PURPOSE         Determine the minimum values of either the input array or
                the mapping array.  Assign the minimum value to the output
                array.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        R. MCKINNEY    9/87     Original development for NEWLAS
        B. Ailts        4/88    Replaced newlas.h with worgen.h
                                Placed fortran bridges in a seperate file
                                Used raw C types -- deleted stdedc.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of LAS system 

PROJECT                         LAS

ALGORITHM

receive:
        -the input array and its type
        -the input mapping array
        -the output array
        -the number of samples of the input array.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        Determine the minimum pixel values.
        Assign the output array pixel the minimum of the input array and
                the mapping array.
return

gLGORITHM REFERENCES            none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void bmin
(
    const unsigned char *in,
    const unsigned char *map,
    unsigned char *out,
    int ns
)
{
    for(; ns; ns--,in++,map++,out++)
    {
        if(*in <= *map) *out = *in;
        if(*in > *map) *out = *map;
    }
}

static void wmin
(
    const short *in,
    const short *map,
    short *out,
    int ns
)
{
    for(; ns; ns--,in++,map++,out++)
    {
        if(*in <= *map) *out = *in;
        if(*in > *map) *out = *map;
    }
}

static void lmin
(
    const int *in,
    const int *map,
    int *out,
    int ns
)
{
    for(; ns; ns--,in++,map++,out++)
    {
        if(*in <= *map) *out = *in;
        if(*in > *map) *out = *map;
    }
}

static void rmin
(
    const float *in,
    const float *map,
    float *out,
    int ns
)
{
    for(; ns; ns--,in++,map++,out++)
    {
        if(*in <= *map) *out = *in;
        if(*in > *map) *out = *map;
    }
}

/**
 * las_pixminimum:
 * @in: I: The input array.
 * @map: I: The input mapping array.
 * @out: O: The output array.
 * @ns: I: The number of samples to be scanned from the buffer.
 * @dtype: I: An integer value that represents the type of data
 *            in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Signed four byte data,
 *            EREAL:  Real*4 data.
 *
 * The las_pixminimum() call is used to determine the minimum of an input
 * array with a mapping array to generate an output array. 
 *
 * Returns:  E_SUCC:   Successful completion,
 *           TYPERR:   Invalid data type specified.
 */
int las_pixminimum
(
    const void *in,         /* I:  input array                           */
    const void *map,        /* I:  input mapping array                   */
    void *out,              /* O:  output array                          */
    int ns,                 /* I:  number of samples                     */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)   */
)

{
    int status = E_SUCC;        /* return status for pxmin */

    switch (dtype)
    {
        case (EBYTE): 
            bmin((unsigned char *)in,(unsigned char *)map,
                 (unsigned char *)out,ns);
            break;
        case (EWORD): 
            wmin((short *)in,(short *)map,(short *)out,ns);
            break;
        case (ELONG): 
            lmin((int *)in,(int *)map,out,ns);
            break;
        case (EREAL): 
            rmin((float *)in,(float *)map,(float *)out,ns);
            break;
        default:      
            status = TYPERR;
            break;
    } /* switch */

    return (status);

} /* pxmin */
 


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_pxmin:
 * @in: I: The input array.
 * @map: I: The input mapping array.
 * @out: O: The output array.
 * @ns: I: The number of samples to be scanned from the buffer.
 * @dtype: I: An integer value that represents the type of data
 *            in the input and output arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Signed four byte data,
 *            EREAL:  Real*4 data.
 *
 * The c_pxmin() call is used to determine the minimum of an input
 * array with a mapping array to generate an output array. 
 *
 * Deprecated.  Use las_pixminimum() instead.
 *
 * Returns:           E_SUCC:   Successful completion,
 *                    TYPERR:   Invalid data type specified.
 */
int c_pxmin
(
    const void *in,   /* I: input array                                  */
    const void *map,  /* I: input mapping array                          */
    void *out,        /* O: output  array                                */
    const long *ns,   /* I: number of samples                            */
    const long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)          */
)

{
    int status = E_SUCC;        /* return status for pxmin */

    status = las_pixminimum (in,map,out,*ns,*dtype);

    return (status);

} /* pxmin */

#endif /* obsolete interfaces */

