/*******************************************************************************
NAME                            C_PXMNMX

PURPOSE         Determine the minimum and maximum values of the input array and
                the minimum and maximum parameters themselves.  The input array
                can be of type byte, signed 2 byte, signed 4 byte, or real.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        D.GORDON        7/85    Original development for NEWLAS
        B.Ailts         12/87   Changed include directory specifications
                                moved bridge routines into a separate file
                                Used raw 'C' types
        K. Zanter        2/88   changed min and max arguments to be double
        B. Ailts        03/88   replaced newlas.h with worgen.h
        B. Davis        06/90   added double capibility

PROJECT                         new LAS

ALGORITHM 

receive:
        -the input array and its type
        -the number of samples of the input array.
        -the minimum and maximum pixel values.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        Determine the minimum and maximum pixel values.
        if the array's min and max values excede the min and max parameters
                the parameters min and/or max are reassigned 
return

ALGORITHM REFERENCES            none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void bmnmx
(
    const unsigned char *in,
    int ns,
    unsigned char *bmin,
    unsigned char *bmax
)
{
    for (; ns; ns--,in++)
    {
        if(*in < *bmin) *bmin = *in;
        if(*in > *bmax) *bmax = *in;
    }
}

static void wmnmx
(
    const short *in,
    int ns,
    short *wmin,
    short *wmax
)
{
    for (; ns; ns--,in++)
    {
        if(*in < *wmin) *wmin = *in;
        if(*in > *wmax) *wmax = *in;
    }
}

static void lmnmx
(
    const int *in,
    int ns,
    int *lmin,
    int *lmax
)
{
    for (; ns; ns--,in++)
    {
        if(*in < *lmin) *lmin = *in;
        if(*in > *lmax) *lmax = *in;
    }
}

static void rmnmx
(
    const float *in,
    int ns,
    float *fmin,
    float *fmax
)
{
    for (; ns; ns--,in++)
    {
        if(*in < *fmin) *fmin = *in;
        if(*in > *fmax) *fmax = *in;
    }
}

static void dblmnmx
(
    const double *in,
    int ns,
    double *dblmin,
    double *dblmax
)
{
    for (; ns; ns--,in++)
    {
        if(*in < *dblmin) *dblmin = *in;
        if(*in > *dblmax) *dblmax = *in;
    }
}


/**
 * las_pixminmax:
 * @in: I: The array for which minimum and maximum pixel
 *         values are to be determined.
 * @ns: I: The number of samples to be scanned from the buffer.
 * @dmin: I/O: The minimum pixel value.
 * @dmax: I/O:  The maximum pixel value.
 * @dtype: I: A value that represents the type of data to be scanned.
 *
 * The las_pixminmax() call is used to determine the minimum and maximum
 * values of the input array #in and the #min and #max parameters
 * themselves.
 *
 * Returns: E_SUCC:   Successful completion,
 *          TYPERR:   Invalid data type specified.
 */
int las_pixminmax
(
    const void *in,         /* I:  input array                            */
    int ns,                 /* I:  number of samples                      */
    double *dmin,           /* I/O: minimum pixel value                   */
    double *dmax,           /* I/O: maximum pixel value                   */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
)

{
    double  dblmin;         /* double min value */
    double  dblmax;         /* double max value */

    float   fmin;           /* float min value */
    float   fmax;           /* float max value */
    int     status = E_SUCC;        /* return status for pxmnmx */

    int    lmin;            /* int min value  */
    int    lmax;            /* int max value  */

    short   wmin;            /* short min value */
    short   wmax;            /* short max value */

    unsigned char  bmin;    /* byte min value  */
    unsigned char  bmax;    /* byte max value  */

    switch (dtype)
    {
        case (EBYTE):   
            bmin = *dmin;
            bmax = *dmax;
            bmnmx((unsigned char *)in,ns,&bmin,&bmax); 
            *dmin = bmin;
            *dmax = bmax;
            break;
        case (EWORD):   
            wmin = *dmin;
            wmax = *dmax;
            wmnmx((short *)in,ns,&wmin,&wmax); 
            *dmin = wmin;
            *dmax = wmax;
            break;
        case (ELONG):   
            lmin = (int)*dmin;
            lmax = (int)*dmax;
            lmnmx(in,ns,&lmin,&lmax); 
            *dmin = lmin;
            *dmax = lmax;
            break;
        case (EREAL):   
            fmin = *dmin;
            fmax = *dmax;
             rmnmx((float *)in,ns,&fmin,&fmax); 
            *dmin = fmin;
            *dmax = fmax;
            break;
        case (EDOUBLE): 
            dblmin = *dmin;
            dblmax = *dmax;
            dblmnmx((double *)in,ns,&dblmin,&dblmax); 
            *dmin = dblmin;
            *dmax = dblmax;
            break;
        default:        
            status = TYPERR;
            break;
    } /* switch */

    return (status);

} /* pxmnmx */


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_pxmnmx:
 * @in: I: The array for which minimum and maximum pixel
 *         values are to be determined.
 * @ns: I: The number of samples to be scanned from the buffer.
 * @dmin: I/O: The minimum pixel value.
 * @dmax: I/O:  The maximum pixel value.
 * @dtype: I: A value that represents the type of data to be scanned.
 *
 * The c_pxmnmx() call is used to determine the minimum and maximum
 * values of the input array #in and the #min and #max parameters
 * themselves.
 *
 * Deprecated.  Use las_pixminmax() instead.
 * 
 * Returns:         E_SUCC:   Successful completion,
 *                  TYPERR:   Invalid data type specified.
 */
int c_pxmnmx
(
    const void  *in,    /* I: input array                                 */
    const long *ns,     /* I: number of samples                           */
    double *dmin,       /* I/O: minimum pixel value                       */
    double *dmax,       /* I/O: maximum pixel value                       */
    const long *dtype   /* I: data type (EBYTE,EWORD,ELONG,EREAL)         */
)
                        /*  EBYTE = unsigned 1 byte  */
                        /*  EWORD = signed 2 byte  */
                        /*  ELONG = signed 4 byte  */
                        /*  EREAL = real 4 byte */
                        /*  EDOUBLE = real 8 byte */

{

    int     status = E_SUCC;        /* return status for pxmnmx */

    status = las_pixminmax (in,*ns,dmin,dmax,*dtype);

    return (status);

} /* pxmnmx */

#endif /* obsolete interfaces */
