/*****************************************************************************
NAME                    pxmul

PURPOSE         Multiply two input arrays into one output array.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        D.GORDON        5/7/85  Original development for NEWLAS
        J.REED          11/12/85 Optimized byte subroutine.
        D.GORDON        11/20/85 Optimized all subroutines.
        D.GORDON        9/10/86 Corrected bug in byte optimization algorithm.
        B.Ailts           12/87 Change the include directory specifications
                                Use raw 'C' types
                                Place bridge routines in a seperate file
        B.Ailts         03/88   Replaced newlas.h with worgen.h
        L.Huewe         04/90   Modified the routine bmul to leave the scale
                                factor a float variable to maintain accuracy

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system.

PROJECT                         NEWLAS

ALGORITHM

receive:
        -the input arrays and their type
        -the output array
        -the number of samples of the input arrays to multiply.
        -the scaling factors and the offset.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        multiply the pixels from input arrays and store result in output array
            (ie. output = inputarray1 * inputarray2 * scale + offset)
        if output is smaller than the lower limit of the data type then 
                output = the lower limit of data type and error is returned
        if output is larger than the higher limit of the data type then
                output = the higher limit of data type and error is returned.
return

*note:  Each operation is performed with variables one data type larger than the
        desired dta type. This makes checking for the operation exceding the
        bounds of the data type very easy while maintaining a fast algorithm.

ALGORITHM REFERENCES            none
*******************************************************************************/
#include "worgen.h"
#include "pixman.h"
#include "typlim.h"

static int bmul
(
    int ns,
    const unsigned char *in1,
    const unsigned char *in2,
    unsigned char *out,
    float scale,
    float offset
)
{
    int status;
    int tempout;

    if((scale == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            if ((tempout = *in1 * *in2) > MAXUTINY)
            {
                *out = MAXUTINY;
                status = TOOBIG;
            }
            else    if (tempout < MINUTINY)
            {
                *out = MINUTINY;
                status = TOOSMALL;
            }
            else    *out = (unsigned char)tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempout = (int)((*in1 * *in2 * (scale)) + (offset));
            if (tempout > MAXUTINY)
            {
                *out = MAXUTINY;
                status = TOOBIG;
            }
            else    if (tempout < MINUTINY)
            {
                *out = MINUTINY;
                status = TOOSMALL;
            }
            else    *out = (unsigned char)tempout;
        }
    }
    return (status);
} /* bmul  */

static int wmul
(
    int ns,
    const short *in1,
    const short *in2,
    short *out,
    float scale,
    float offset
)
{
    int status;
    int tempin1;
    int tempin2;
    int tempout;

    if ((scale == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if ((tempout = tempin1 * tempin2) > MAXWORD)
            {
                tempout = MAXWORD;
                status = TOOBIG;
            }
            else    if (tempout < MINWORD)
            {
                tempout = MINWORD;
                status = TOOSMALL;
            }
            *out = (short)tempout;
            }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            tempout = (int)((float)(tempin1 * tempin2) * scale + 
                              offset);
            if (tempout > MAXWORD)
            {
                tempout = MAXWORD;
                status = TOOBIG;
            }
            else    if (tempout < MINWORD)
            {
                tempout = MINWORD;
                status = TOOSMALL;
            }
            *out = (short)tempout;
        }
    }
    return (status);
}

static int lmul
(
    int ns,
    const int *in1,
    const int *in2,
    int *out,
    float scale,
    float offset
)
{
    int status;
    double tempin1;
    double tempin2;
    double tempout;

    if ((scale == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if ((tempout = tempin1 * tempin2) > MAXLONG)
            {
                tempout = MAXLONG;
                status = TOOBIG;
            }
            else    if (tempout < MINLONG)
            {
                tempout = MINLONG;
                status = TOOBIG;
            }
            *out = (int)tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            tempout = (tempin1 * tempin2) * scale + offset;
            if (tempout > MAXLONG)
            {
                tempout = MAXLONG;
                status = TOOBIG;
            }
            else    if (tempout < MINLONG)
            {
                tempout = MINLONG;
                status = TOOBIG;
            }
            *out = (int)tempout;
        }
    }
    return (status);
}

static int rmul
(
    int ns,
    const float *in1,
    const float *in2,
    float *out,
    float scale,
    float offset
)
{
    int status;
    double tempin1;
    double tempin2;
    double tempout;

    if ((scale == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if ((tempout = tempin1 * tempin2) > MAXREAL)
            {
                tempout = MAXREAL;
                status = TOOBIG;
            }
            else    if (tempout < MINREAL)
            {
                tempout = MINREAL;
                status = TOOBIG;
            }
            *out = tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            tempout = (tempin1 * tempin2) * scale + offset;
            if (tempout > MAXREAL)
            {
                tempout = MAXREAL;
                status = TOOBIG;
            }
            else    if (tempout < MINREAL)
            {
                tempout = MINREAL;
                status = TOOBIG;
            }
            *out = tempout;
        }
    }
    return (status);
}

/**
 * las_pixmultiply:
 * @ns: I: The number of samples to be multiplied.
 * @in1: I: An array of pixel values to be multiplied.
 * @in2: I: An array of pixel values to  be multiplied.
 * @out: O: An array created by multipying, scaling, and
 *          offsetting the two input arrays.
 *          out = (in1 * in2) * scale + offset
 * @scale: I: The scale factor to be applied after multiplication.
 * @offset: I: The offset to be applied after multiplication.
 * @dtype: I: An integer value that represents the type of data to
 *            be multiplied.  The output array is of the same data
 *            type as the input arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Signed four byte data,
 *            EREAL:  Four byte floating point data.
 *
 * The las_pixmultiply() call is used to create an output array out by
 * multiplying, scaling, and offsetting the values of ns samples of
 * two input arrays in1 and in2 of type #dtype.
 *
 * Returns:     E_SUCC:   Successful completion,
 *              TOOBIG:   Exceeded upper limit of data type,
 *              TOOSMALL: Exceeded lower limit of data type,
 *              TYPERR:   Invalid data type specified. 
 */
int las_pixmultiply
(
    int ns,                 /* I:  number of samples                     */
    const void *in1,        /* I:  input array one                       */
    const void *in2,        /* I:  input array two                       */
    void *out,              /* I:  output array                          */
    float scale,            /* I:  scale applied to in. arrays after mul */
    float offset,           /* I:  offset added in after sum             */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)   */
)  

{
    int status = E_SUCC;        /* return status for pxmul */

    switch (dtype)
    {
        case (EBYTE): 
            status = bmul(ns,(unsigned char *)in1,(unsigned char *)in2,
                         (unsigned char *)out,scale,offset);
            break;
        case (EWORD): 
            status = wmul(ns,(short *)in1,(short *)in2,(short *)out,scale,
                          offset);
            break;
        case (ELONG): 
            status = lmul(ns,(int *)in1,(int *)in2,(int *)out,scale,offset);
            break;
        case (EREAL): 
            status = rmul(ns,(float *)in1,(float *)in2,(float *)out,scale,
                          offset);
            break;
        default:      
            status = TYPERR;
            break;
    } /* switch */

    return (status);

} /* pxmul */

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_pxmul:
 * @ns: I: The number of samples to be multiplied.
 * @in1: I: An array of pixel values to be multiplied.
 * @in2: I: An array of pixel values to  be multiplied.
 * @out: O: An array created by multipying, scaling, and
 *          offsetting the two input arrays.
 *          out = (in1 * in2) * scale + offset
 * @scale: I: The scale factor to be applied after multiplication.
 * @offset: I: The offset to be applied after multiplication.
 * @dtype: I: An integer value that represents the type of data to
 *            be multiplied.  The output array is of the same data
 *            type as the input arrays.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Signed four byte data,
 *            EREAL:  Four byte floating point data.
 *
 * The c_pxmul() call is used to create an output array out by
 * multiplying, scaling, and offsetting the values of ns samples of
 * two input arrays in1 and in2 of type #dtype.
 *
 * Deprecated.  Use las_pixmultiply() instead.
 *
 * Returns:     E_SUCC:   Successful completion,
 *              TOOBIG:   Exceeded upper limit of data type,
 *              TOOSMALL: Exceeded lower limit of data type,
 *              TYPERR:   Invalid data type specified. 
 */
int c_pxmul
(
    const long *ns,      /* I: number of samples                         */
    const void *in1,     /* I: input array one                           */
    const void  *in2,    /* I: input array two                           */
    void *out,           /* O: output array                              */
    const float *scale,  /* I: scale applied to arrays after mult        */
    const float *offset, /* I: offset added in after sum                 */
    const long *dtype    /* I: data type (EBYTE,EWORD,ELONG,EREAL)       */
)

{
    int status = E_SUCC;        /* return status for pxmul */

    status = las_pixmultiply (*ns,in1,in2,out,*scale,*offset,*dtype);

    return (status);
} /* pxmul */

#endif /* obsolete interfaces */
