/*****************************************************************************
NAME                    pxsub

PURPOSE         Subtract two input arrays into one output array. Each input
                array can be scaled and an offset can be applied to the
                difference. The input arrays can be of type byte, signed 2 byte,
                signed 4 byte, or real. The output array will be of the same
                type as the input arrays.

PROGRAM HISTORY
        PROGRAMMER      DATE     REASON
        ----------      ----     ------
        D.GORDON        5/7/85   Original development for NEWLAS
        J.REED          11/12/85 Optimized byte subroutine.
        D.GORDON        11/20/85 Optomized all subroutines.
        B.Ailts            12/87 Change include directory specifications
                                 Use raw 'C' data types
                                 Place bridge routines in a seperate file
        B.Ailts         03/88    Replaced newlas.h with worgen.h
        B.Ailts         07/88    Fixed bug in R*4 and I*2 data
        L.Huewe         04/90   Modified the routine bsub to leave the scale
                                factor a float variable to maintain accuracy

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system.

PROJECT                         NEWLAS

ALGORITHM

receive:
        -the input arrays and their type
        -the output array
        -the number of samples of the input arrays to subtract.
        -the scaleing factors and the offset.
depending on the type of array values
        call a function for that particular type that performs the following:

for each pixel of the input arrays
        aubtract the pixels from each input array and store difference in output
        array (ie. output= inputarray1 * scale1 - inputarray2 * scale2 + offset)
        if the output values excede the boundaries of the data type then
                the output is reassigned that boundary that has been exceeded,
                and an error code is returned, either TOOBIG or TOOSMALL.
return
*note:  All operations are performed with a variable one data type larger than
        the specified data type. This is to allow for easy error checking of
        operation results exceding the bounds of the desired data type while
        maintaining a reasonably fast pixal manipulation routine.


ALGORITHM REFERENCES            none
******************************************************************************/
#include "worgen.h"
#include "pixman.h"
#include "typlim.h"

static int bsub
(
    int ns,
    const unsigned char *in1,
    const unsigned char *in2,
    unsigned char *out,
    float scale1,
    float scale2,
    float offset
)

{
    int status;
    int tempout;

    if((scale1 == 1) && (scale2 == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {       
            if(MAXUTINY < (tempout = *in1 - *in2 ))
            {
                *out = MAXUTINY;
                status = TOOBIG;
            }
            else if (tempout < MINUTINY)
            {
                *out = MINUTINY;
                status = TOOSMALL;
            }
            *out = (unsigned char)tempout;
        }
    }
    else 
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempout = (int)((*in1 * scale1) - (*in2 * scale2) + 
                             offset);
            if (tempout > MAXUTINY)
            {
                *out = MAXUTINY;
                status = TOOBIG;
            }
            else if (tempout < MINUTINY)
            {
                *out = MINUTINY;
                status = TOOSMALL;
            }
            *out = (unsigned char)tempout;
        }
    }
    return (status);
}

static int  wsub
(
    int ns,
    const short *in1,
    const short *in2,
    short *out,
    float scale1,
    float scale2,
    float offset
)
{
    int status;
    int tempin1;
    int tempin2;
    int tempout;

    if ((scale1 == 1) && (scale2 == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if ((tempout = tempin1 - tempin2)  > MAXWORD)
            {
                tempout = MAXWORD;
                status = TOOBIG;
            }
            else if (tempout < MINWORD)
            {
                tempout = MINWORD;
                status = TOOSMALL;
            }
            *out = (short)tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            tempout = (int)(((float)tempin1 * scale1) - 
                             ((float)tempin2 * scale2) + offset);
            if (tempout > MAXWORD)
            {
                tempout = MAXWORD;
                status = TOOBIG;
            }
            else if (tempout < MINWORD)
            {
                tempout = MINWORD;
                status = TOOSMALL;
            }
            *out = (short)tempout;
        }
    }
    return (status);
}       

static int lsub
(
    int ns,
    const int *in1,
    const int *in2,
    int *out,
    float scale1,
    float scale2,
    float offset
)
{       
    double tempin1;
    double tempin2;
    double tempout;
    int status;

    if ((scale1 == 1) && (scale2 == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if ((tempout = tempin1 - tempin2)  > MAXLONG)
            {
                tempout = MAXLONG;
                status = TOOBIG;
            }
            else if (tempout < MINLONG)
            {
                tempout = MINLONG;
                status = TOOSMALL;
            }
            *out = (int)tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            tempout = (tempin1 * scale1) - (tempin2 * scale2) + offset;
            if (tempout > MAXLONG)
            {
                tempout = MAXLONG;
                status = TOOBIG;
            }
            else if (tempout < MINLONG)
            {
                tempout = MINLONG;
                status = TOOSMALL;
            }
            *out = (int)tempout;
        }
    }
    return (status);
}

static int rsub
(
    int ns,
    const float *in1,
    const float *in2,
    float *out,
    float scale1,
    float scale2,
    float offset
)
{       
    double tempin1;
    double tempin2;
    double tempout;
    int status;

    if ((scale1 == 1) && (scale2 == 1) && (offset == 0))
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            if ((tempout = tempin1 - tempin2) > MAXREAL)
                if (tempout > MAXREAL)
                {
                    tempout = MAXREAL;
                    status = TOOBIG;
                }
                else if (tempout < MINREAL)
                {
                    tempout = MINREAL;
                    status = TOOSMALL;
                }
                *out = tempout;
        }
    }
    else
    {
        for(; ns; ns--,out++,in1++,in2++)
        {
            tempin1 = *in1;
            tempin2 = *in2;
            tempout = (tempin1 * scale1) - (tempin2 * scale2) + offset;
            if (tempout > MAXREAL)
            {
                tempout = MAXREAL;
                status = TOOBIG;
            }
            else if (tempout < MINREAL)
            {
                tempout = MINREAL;
                status = TOOSMALL;
            }
            *out = tempout;
        }
    }
    return (status);
}

/**
 * las_pixsubtract:
 * @ns: I: The number of pixels to be subtracted.
 * @in1: I: An array of pixels values to subtract from.
 * @in2: I: An array of pixel values to subtract.
 * @out: O: An array created by scaling, subtracting, and offsetting
 *          the two input arrays.
 *          out = in1 * scale1 - in2 * scale2 + offset
 * @scale1: I: The  scale factor to be applied to in1 subtraction.
 * @scale2: I: The scale factor to be applied to in2 subtraction.
 * @offset: I: An offset added to the calculation after the subtraction.
 * @dtype: I: An integer value that represents the type of data to be
 *            subtracted.  The output array is of the same data type
 *            as the input arrays.
 *            Legal values include:
 *            EBYTE:   Unsigned byte data,
 *            EWORD:   Signed two byte data,
 *            ELONG:   Signed four byte data,
 *            EREAL:   Four byte floating point data.
 *
 * The las_pixsubtract() call is used to create an output array out by
 * scaling, subtracting, and offsetting the values of ns samples of two
 *  input pixel arrays in1 and in2 of type dtype.
 * 
 * Returns:   E_SUCC:   Successful completion,
 *            TOOBIG:   Exceeded upper limit of data type,
 *            TOOSMALL: Exceeded lower limit of data type,
 *            TYPERR:   Invalid data type specified.
 */
int las_pixsubtract
(
    int ns,                 /* I:  number of samples in input buffer      */
    const void *in1,        /* I:  input array one                        */
    const void *in2,        /* I:  input array two                        */
    void *out,              /* O:  output array                           */
    float scale1,           /* I: scale applied to in. array1 before sub  */
    float scale2,           /* I: scale applied to in. array2 before sub  */
    float offset,           /* I: offset added in after difference        */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
)

{
    int status = E_SUCC;        /* return status for pxsub */

    switch (dtype)
    {
        case (EBYTE): 
            status = bsub(ns,(unsigned char *)in1,(unsigned char *)in2,
                         (unsigned char *)out,scale1,scale2,offset);
            break;
        case (EWORD): 
            status = wsub(ns,(short *)in1,(short *)in2,(short *)out,scale1,
                          scale2,offset);
            break;
        case (ELONG): 
            status = lsub(ns,(int *)in1,(int *)in2,(int *)out,scale1,scale2,
                          offset);
            break;
        case (EREAL): 
            status = rsub(ns,(float *)in1,(float *)in2,(float *)out,
                          scale1, scale2,offset);
            break;
        default:      
            status = TYPERR;
            break;
    } /* switch */

    return (status);

} /* pxsub */

/**
 * c_pxsub:
 * @ns: I: The number of pixels to be subtracted.
 * @in1: I: An array of pixels values to subtract from.
 * @in2: I: An array of pixel values to subtract.
 * @out: O: An array created by scaling, subtracting, and offsetting
 *          the two input arrays.
 *          out = in1 * scale1 - in2 * scale2 + offset
 * @scale1: I: The  scale factor to be applied to in1 subtraction.
 * @scale2: I: The scale factor to be applied to in2 subtraction.
 * @offset: I: An offset added to the calculation after the subtraction.
 * @dtype: I: An integer value that represents the type of data to be
 *            subtracted.  The output array is of the same data type
 *            as the input arrays.
 *            Legal values include:
 *            EBYTE:   Unsigned byte data,
 *            EWORD:   Signed two byte data,
 *            ELONG:   Signed four byte data,
 *            EREAL:   Four byte floating point data.
 *
 * The c_pxsub() call is used to create an output array out by scaling,
 * subtracting, and offsetting the values of ns samples of two input pixel
 * arrays in1 and in2 of type dtype.
 * 
 * Deprecated.  Use las_pixsubtract() instead.
 *
 * Returns:   E_SUCC:   Successful completion,
 *            TOOBIG:   Exceeded upper limit of data type,
 *            TOOSMALL: Exceeded lower limit of data type,
 *            TYPERR:   Invalid data type specified.
 */
int c_pxsub
(
    const long *ns,      /* I: number of samples                        */
    const void *in1,     /* I: input array one                          */
    const void *in2,     /* I: input array two                          */
    void *out,           /* O: output array                             */
    const float *scale1, /* I: scale applied to array1  before subtract */
    const float *scale2, /* I: scale applied to array2 before subtract  */
    const float *offset, /* I: offset added in after difference         */
    const long *dtype    /* I: data type (EBYTE,EWORD,ELONG,EREAL)      */
)

{
    int status = E_SUCC;        /* return status for pxadd */

    status = las_pixsubtract (*ns,in1,in2,out,*scale1,*scale2,
                              *offset,*dtype);
    
    return (status);

} /* pxsub */
