/*******************************************************************************
NAME:    c_pxswap

PURPOSE: Swap bytes of an input array

HISTORY:
Programmer       Date   Reason
---------------  -----  --------------------------------------------------------
D.Gordon         05/85  Original development for NEWLAS
J.Reed           11/85  Optimized byte portion of routine.
B.Ailts          12/87  Changed include directory specifications.
                        Use raw 'C' types.
                        Placed bridge routines in a seperate file.
B.Ailts          04/88  Replaced newlas.h with worgen.h
T.Baltzer        09/90  Add c_errmsg call prior to returning E_FAIL
J.Fenno          04/92  Optimized code.  Removed the swap function that was
                        called for each element that was to be swapped.
                        Speed up of 30% to 55%

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Part of the LAS system.

PROJECT: LAS

*******************************************************************************/
#include "tae.h"
#include "worgen.h"
#include "pixman.h"

/**
 * las_pixswap:
 * @buf: I/O: The data buffer which is to be byte swapped.  The
 *            parameter ns specifies the number of entities in
 *            the data buffer, and parameter size specifies the
 *            size of each entity.
 * @ns: I: The number of entities to be byte swapped in the buffer.
 * @size: I: The number of bytes per entity.  Parameter size is
 *           required to be an even value.
 * 
 * The las_pixswap() call is used to byte swap values within a data
 * buffer.  The byte swapping algorithm for any entity which occupies
 * 2N bytes is: byte 1 is swapped with byte 2N, byte 2 is swapped with
 * byte 2N-1, bytes 3 is swapped with bytes 2N-2, and so on, ending
 * with bytes N and N + 1.
 * 
 * Returns:    E_SUCC:   Successful completion,
 *             E_FAIL:   Odd number of bytes per entity specified. 
 */
int las_pixswap
(
    void *buf,              /* I/O: data buffer which will be byte swapped*/
    int ns,                 /* I:  number of entities in the buffer       */
    int size                /* I:  number of bytes per entity             */
)

{
    unsigned char *bottom;
    unsigned char *byte_buf;
    unsigned char *top;
    unsigned char temp;
    int count;
    long fatal = 1;

    byte_buf=(unsigned char*)buf;

    if (size == 1)
       return (E_SUCC);
    else if ((size % 2) != 0)
    {
        c_errmsg ("Odd number of bytes per entity specified",
                  "pxswap-oddnum", &fatal);
        return (E_FAIL);
    }
    else
    {
        count = ns;
        for (; count--; byte_buf += size)
        {
            bottom = byte_buf;
            top    = bottom + size - 1;
            while (bottom < top)
            {
                temp = *bottom;
                *(bottom++) = *top;
                *(top--) = temp;
            }
        }
    }
    return (E_SUCC);
}


/**
 * c_pxswap:
 * @buf: I/O: The data buffer which is to be byte swapped.  The
 *            parameter ns specifies the number of entities in
 *            the data buffer, and parameter size specifies the
 *            size of each entity.
 * @ns: I: The number of entities to be byte swapped in the buffer.
 * @size: I: The number of bytes per entity.  Parameter size is
 *           required to be an even value.
 * 
 * The c_pxswap() call is used to byte swap values within a data
 * buffer.  The byte swapping algorithm for any entity which occupies
 * 2N bytes is: byte 1 is swapped with byte 2N, byte 2 is swapped with
 * byte 2N-1, bytes 3 is swapped with bytes 2N-2, and so on, ending
 * with bytes N and N + 1.
 * 
 * Deprecated.  Use las_pixswap() instead.
 *
 * Returns:    E_SUCC:   Successful completion,
 *             E_FAIL:   Odd number of bytes per entity specified. 
 */
int c_pxswap
(
    void *buf,        /* I/O: data buffer which is to be byte swapped     */
    const long *ns,   /* I: number of entities in the buffer              */
    const long *size  /* I: number of bytes per entity                    */
)

{
    int status = E_SUCC;

    status = las_pixswap (buf,*ns,*size);
   
    return (status); 
}
