/******************************************************************************
FUNCTION:       C_PXSYS

PURPOSE:        Convert data from source computer system format to destination
                computer format.  Data types handled are ECHAR, EBYTE, EWORD,
                EREAL, and EDOUBLE.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1        01/89    K. Gacke            Original development
  2         8/89    K. Gacke            Added ECHAR data type to convert ASCII
                                        with the high bit on/off.
  3         8/89    K. Gacke            Fixed bug in vax_r4_conv to check value
                                        of zero prior to applying exp bias
  4         2/90    K. Gacke            Fixed bug in align_buf to correctly
                                        free and assign bufptr.
  5         8/90    D. Akkerman         Modified to check the insys and outsys
                                        codes for the possibility of a system
                                        change between invocations.  This better
                                        accommodates interactive processing.
  6        10/90    T. Baltzer          check return status's, add errmsg calls
  7        12/90    K. Gacke            Add cray y-mp conversion routines
  8         4/91    B. Davis            Added ibm aix.  (Emulate sun-bsd)
  9         3/91    K. Gacke            Fixed bug in pn_r8_conv routine to
                                        properly compute the 2's complement
                                        on a negative number.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
         
         Floating Point Conversion Algorithm (function pxsys):
             o Supported computer systems:
                     Computer          Referenced in this paper
                     --------          ------------------------
                     DEC VAX/VMS       DEC
                     Gould Power Node  Gould
                     SUN II, III, IV   IEEE
                     IBM AIX           IEEE
                     IBM MVS           IBM
                     PRIME Primos      PRIME 
                     SILICON GRAPHICS  IEEE
                     DATA GENERAL      IEEE
         
             o Convert remote host float number directly into binary 
               format of local host.
         
             o Assume application program will read binary float data 
               created on local or remote host, but writes binary 
               float data only in the local host format, or IEEE 
               standard format.
         
             o Algorithm description to read data created on remote 
               host:  Mask sign bit, exponent field, and fraction 
               field of the float number created on remote host.  Do 
               the floating point arithmetic on the local host to 
               create a valid float number.  If over flow or under 
               flow is detected, conversion routine can optionally 
               assign fill values or return a bad status code.
         
             o Algorithm description to write data into IEEE standard 
               format:  From float number on local host, compute the 
               sign bit, exponent field, and fraction field to make 
               an IEEE float number.  The sign bit is unchanged.  The 
               exponent is computed by taking the natural log of the 
               local host's float number divided by natural log 2, 
               and rounding up to next highest integer (exponent is 
               the smallest integer number such that 2 raised to the 
               power of it is greater than the float number).  The 
               fraction is computed by dividing the float number by 2 
               raised to the power of exponent and multiplying by 
               0x1000000 to create the integer fraction field.
         
             o The conversion algorithms require the use of the power 
               function and the natural logarithm function, both of 
               which are CPU intensive.  To improve the performance, 
               the conversion routines make use of a static array 
               containing values for 2 raised to the power of -126 
               through 127 (IEEE standard).  The exponent field is 
               computed by performing a binary search on the power 
               table.  If the value is outside of the table range, 
               the math functions are used.  The performance was 
               improved by a factor of 2 on the PN9050 and VAX.
         

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>

#ifdef unix
#include <math.h>
#endif

#ifdef vms
#include <math.h>
#endif

#include "las.h"
#include "sysdef.h"
#include "ieee.h"
#include "pixman.h"
#include "util.h"

#define NEG_ONE -1

#define PXSYS_OK        1
#define PXSYS_ERROR     2
#define PXSYS_RESET     3

#define PXSYS_IEEE              1
#define PXSYS_DEC_VAX           2
#define PXSYS_GOULD_PN          3
#define PXSYS_PRIME             4
#define PXSYS_HP_MPE            5
#define PXSYS_IBM_MVS           6
#define PXSYS_CRAY              7
#define PXSYS_IBM_AIX           8
#define PXSYS_SGI_IRX           9
#define PXSYS_DG_UX             10
#define PXSYS_HION              11
#define PXSYS_HIOFF             12

struct MACH
{
    long bswap;
    long c;
    long r4;
    long r8;
};


#define LSH17 (IEEE_pow[17 + IEEE_R4_EXCESS])   /*           0x20000 */
#define LSH21 (IEEE_pow[21 + IEEE_R4_EXCESS])   /*          0x200000 */
#define LSH32 (IEEE_pow[32 + IEEE_R4_EXCESS])   /*       0x100000000 */
#define LSH48 (IEEE_pow[48 + IEEE_R4_EXCESS])   /*   0x1000000000000 */
#define LSH53 (IEEE_pow[53 + IEEE_R4_EXCESS])   /*  0x20000000000000 */
#define LSH56 (IEEE_pow[56 + IEEE_R4_EXCESS])   /* 0x100000000000000 */

static unsigned char *align_buf (unsigned char *buf, const long *dtype, 
                                 const long *size,  long *cpflg);
static int cray_r8_conv (const long *outsys, unsigned char *buf, const 
                         long *size, const double *under, const double
                         *over, const long *flag);
static int get_mach     (long, long, struct MACH *isystem, 
                         struct MACH *osystem);
static int ibm_r4_conv  (const long *outsys, unsigned long *buf, 
                         const long *size, const float *under,
                         const float *over, 
                         const long *flag);
static int ibm_r8_conv  (const long *outsys, unsigned char *buf, 
                         const long *size, const double *under,
                         const double *over, 
                         const long *flag);
static int ieee_r4_conv (const long *outsys, unsigned long *buf, 
                         const long *size, const float *under, 
                         const float *over,
                         const long *flag);
static int ieee_r8_conv (const long *outsys, unsigned char *buf, 
                         const long *size, const double *under,
                         const double *over, const long *flag);
static int ieee_r4_out  (const long *outsys, float *buf, const long *size, 
                         const float *under, const float *over, 
                         const long *flag);
static int ieee_r8_out  (const long *outsys, double *buf, const long *size, 
                         const double *under, const double *over, 
                         const long *flag);
static int pn_r4_conv   (const long *outsys, unsigned long *buf, 
                         const long *size, const float *under,
                         const float *over, const long *flag);
static int pn_r8_conv   (const long *outsys, unsigned char *buf,
                         const long *size, const double *under,
                         const double *over, const long *flag);
static int prime_r4_conv(const long *outsys, unsigned long *buf, 
                         const long *size, const float *under,
                         const float *over, const long *flag);
static int prime_r8_conv(const long *outsys, unsigned char *buf,
                         const long *size, const double *under,
                         const double *over, const long *flag);
static int vax_r4_conv  (const long *outsys, unsigned long *buf,
                         const long *size, const float *under, 
                         const float *over, const long *flag);
static int vax_r8_conv  (const long *outsys, unsigned char *buf, 
                         const long *size, const double *under, 
                         const double *over, const long *flag);
static int pxsys_r4err  (const long *outsys, const float *under, 
                         const float *over, const long *flag,  
                         long *exp,  long *frac, float *ival);
static int pxsys_r8err  (const long *outsys, const double *under, 
                         const double *over, const long *flag, 
                         long *exp, long *frac1, long *frac2,
                         double *val);

static void hioff (unsigned char *bufptr, const long *nbyte);
static void hion  (unsigned char *bufptr, const long *nbyte);


static long datasize[] = {0,1,2,4,4,8,1};
static char *sys_name[] =
{
    UTX, BSD, ULT, VMS, IEEE, HP, PRIMOS, AIX, MVS, IRX, DG_UX, MSC, ""
};



int c_pxsys
(
    const long *insys,  /* I: input computer system                         */
    const long *outsys, /* I: output computer system                        */
    unsigned char *buf, /* I/O: array of real numbers to be converted       */
    const long *dtype,  /* I: data type (EBYTE,EWORD,ELONG,EREAL)           */
    const long *size,   /* I: number of elements in array                   */
    const void *under,  /* I: value assigned if underflow detected          */
    const void *over,   /* I: value assigned if overflow detected           */
    const long *flag    /* I: flag used for error detection                 */
)
                        /*        SYS_ERR => Error on underflow/overflow    */
                        /*        SYS_SET => Set to min/max for system      */
                        /*        SYS_USE => Set underflow/overflow values  */
{
    static struct MACH isystem;
    static struct MACH osystem;
    static long inarch = -1;
    static long outarch = -1;
    static long locsys = -1;
    long cpflg;
    long nelem;
    long status;
    long sysflg;
    long nonfatal = 1;
    char errstr[ERRLEN+1];
    char system[CMLEN];
    unsigned char *bufptr;

    status = E_SUCC;
    cpflg = FALSE;
    bufptr = buf;
    if ((inarch != *insys) || (outarch != *outsys))
    {
        if (get_mach(*insys,*outsys,&isystem,&osystem) != E_SUCC)
            return(E_FAIL);
        inarch = *insys;
        outarch = *outsys;
        if (c_getsys(system) != E_SUCC)
            return(E_FAIL);
        sysflg = SETN;
        if (c_sysset(system,&locsys,&sysflg) != E_SUCC)
            return(E_FAIL);
    }
    switch (*dtype)
    {
        case ECHAR:
            if (isystem.c != osystem.c)
            {
                if (osystem.c == PXSYS_HION)
                    hion(bufptr,size);
                else
                    hioff(bufptr,size);
            }
        case EBYTE:
            break;
        case EWORD:
            if (isystem.bswap ^ osystem.bswap)
                status = c_pxswap(bufptr,size,&datasize[EWORD]);
            break;
        case ELONG:
            if (isystem.bswap ^ osystem.bswap)
                status = c_pxswap(bufptr,size,&datasize[ELONG]);
            break;
        case EREAL:
            if (isystem.r4 == osystem.r4)           /* same system */
                break;
            if ((isystem.r4 == PXSYS_CRAY) || (osystem.r4 == PXSYS_CRAY))
            {
                c_errmsg("Cray does not support 32-bit real"," ",&nonfatal);
                return(E_FAIL);
            }
            if ((bufptr = align_buf(buf,dtype,size,&cpflg)) == NULL)
                return(E_FAIL);
            if ((*outsys == SYS_IEEE_STD) && (locsys != SYS_IEEE_STD))
            {
                status = ieee_r4_out(outsys,(float *)bufptr,size,
                                    (float *)under,(float *)over,flag);
                if (status == E_FAIL)   
                    break;
                if (isystem.bswap)
                    status = c_pxswap(bufptr,size,&datasize[ELONG]);
            }
            else
            {
                switch (isystem.r4)           /* convert from the specified */
                {                             /* input computer system to   */
                    case PXSYS_IEEE:          /* the host computer          */
                        if (osystem.bswap)
                        {
                            status = c_pxswap(bufptr,size,&datasize[ELONG]);
                            if ( status == E_FAIL )  
                                break;
                        }
                        status = ieee_r4_conv(outsys,(unsigned long *)bufptr,
                                              size,(float *)under,
                                             (float *)over, flag);
                        break;
                    case PXSYS_GOULD_PN:
                        if (osystem.bswap)
                        {
                            status = c_pxswap(bufptr,size,&datasize[ELONG]);
                            if ( status == E_FAIL )   
                                break;
                        }
                        status = pn_r4_conv(outsys,(unsigned long *)bufptr,
                                            size, (float *)under,
                                           (float *)over,flag);
                        break;
                    case PXSYS_PRIME:
                        if (osystem.bswap)
                        {
                            status = c_pxswap(bufptr,size,&datasize[ELONG]);
                            if ( status == E_FAIL )   break;
                        }
                        status = prime_r4_conv(outsys,(unsigned long *)bufptr,
                                               size,(float *)under,
                                              (float *)over, flag);
                        break;
                    case PXSYS_DEC_VAX:
                        if (!osystem.bswap)
                        {
                            status = c_pxswap(bufptr,size,&datasize[ELONG]);
                            if ( status == E_FAIL )   
                                break;
                        }
                        status = vax_r4_conv(outsys,(unsigned long *)bufptr,
                                             size,(float *)under,
                                            (float *)over, flag);
                        break;
                    case PXSYS_IBM_MVS:
                        if (osystem.bswap)
                        {
                            status = c_pxswap(bufptr,size,&datasize[ELONG]);
                            if ( status == E_FAIL )   
                                break;
                        }
                        status = ibm_r4_conv(outsys,(unsigned long *)bufptr,
                                             size, (float *)under,
                                            (float *)over,flag);
                        break;
                    default:
                        sprintf(errstr,"Computer <%s> not supported for R*4",
                                sys_name[*insys]);
                        c_errmsg(errstr,"pxsys-noreal4",&nonfatal);
                        status = E_FAIL;
                        break;
                }
            }
            break;
        case EDOUBLE:
            if (isystem.r8 == osystem.r8)           /* same system */
                break;
            if ((bufptr = align_buf(buf,dtype,size,&cpflg)) == NULL)
                return(E_FAIL);
            if ((*outsys == SYS_IEEE_STD) && (locsys != SYS_IEEE_STD))
            {
                status = ieee_r8_out(outsys,(double *)bufptr,size,
                                    (double *)under,(double *)over,flag);
                if (status == E_FAIL)   
                    break;
                if (isystem.bswap)
                {
                    nelem = *size * 2;
                    status = c_pxswap(bufptr,&nelem,&datasize[ELONG]);
                }
            }
            else
            {
                nelem = *size * 2;
                switch (isystem.r8)            /* convert from the specified*/
                {                              /* input computer system to  */
                    case PXSYS_IEEE:           /* the host computer         */
                        if (osystem.bswap)
                        {
                            status = c_pxswap(bufptr,&nelem,&datasize[ELONG]);
                            if ( status == E_FAIL )   break;
                        }
                        status = ieee_r8_conv(outsys,bufptr,size,
                                             (double *)under,(double *)over,
                                              flag);
                        break;
                    case PXSYS_GOULD_PN:
                        if (osystem.bswap)
                        {
                            status = c_pxswap(bufptr,&nelem,&datasize[ELONG]);
                            if ( status == E_FAIL )   break;
                        }
                        status = pn_r8_conv(outsys,bufptr,size,
                                           (double *)under,(double *)over,
                                            flag);
                        break;
                    case PXSYS_PRIME:
                        if (osystem.bswap)
                        {
                            status = c_pxswap(bufptr,&nelem,&datasize[ELONG]);
                            if ( status == E_FAIL )   break;
                        }
                        status = prime_r8_conv(outsys,bufptr,size,
                                              (double *)under,(double *)over,
                                               flag);
                        break;
                    case PXSYS_DEC_VAX:
                        if (!osystem.bswap)
                        {
                            status = c_pxswap(bufptr,&nelem,&datasize[ELONG]);
                            if ( status == E_FAIL )   
                                break;
                        }
                        status = vax_r8_conv(outsys,bufptr,size,
                                            (double *)under,(double *)over,
                                             flag);
                        break;
                    case PXSYS_IBM_MVS:
                        if (osystem.bswap)
                        {
                            status = c_pxswap(bufptr,size,&datasize[ELONG]);
                            if ( status == E_FAIL )   
                                break;
                        }
                        status = ibm_r8_conv(outsys,bufptr,size,
                                            (double *)under,(double *)over,
                                             flag);
                        break;
                    case PXSYS_CRAY:
                        if (osystem.bswap)
                        {
                            status = c_pxswap(bufptr,&nelem,&datasize[ELONG]);
                            if ( status == E_FAIL )   
                                break;
                        }
                        status = cray_r8_conv(outsys,bufptr,size,
                                             (double *)under,(double *)over,
                                              flag);
                        break;
                    default:
                        sprintf(errstr,"Computer <%s> not supported for R*8",
                                sys_name[*insys]);
                        c_errmsg(errstr,"pxsys-noreal8",&nonfatal);
                        status = E_FAIL;
                        break;
                }
            }
            break;
        default:
            sprintf(errstr,"Invalid data type, dtype = %d",*dtype);
            c_errmsg(errstr,"pxsys-badtype",&nonfatal);
            status = E_FAIL;
            break;
    }
    if (cpflg)                         /* if input buffer not aligned, copy */
    {                                  /* data back into the input buffer   */
        c_pxcopy(bufptr,buf,dtype,size);
    }
    return(status);
}





/*
*  Ensure the data buffer is aligned on proper word boundary for the 
*  specified data type.
*/
static unsigned char *align_buf
(
    unsigned char *buf,
    const long *dtype,
    const long *size,
    long *cpflg
)
{
    static long max_size = 0;
    static unsigned char *bufptr;
    long nbyte;
    unsigned int bufaddr;
    long nonfatal = 1;

    bufaddr = (int) buf;
    if (bufaddr % datasize[*dtype] != 0)/* if input buffer is not aligned  */
    {                                   /* copy data into a temp work buffer*/
        *cpflg = NEG_ONE;
        nbyte = *size * datasize[*dtype];
        if (nbyte > max_size)
        {
            if (max_size > 0)
                free(bufptr);
            max_size = nbyte;
            bufptr = (unsigned char *) malloc(max_size);
            if (bufptr == NULL)
            {
                c_errmsg("Error allocating alignment buffer","pxsys-malloc",
                         &nonfatal);
                return(bufptr);
            }
        }
        c_pxcopy(buf,bufptr,dtype,size);
    }
    else
    {
        *cpflg = FALSE;
        if (max_size > 0)
        {
            free(bufptr);
            max_size = 0;
        }
        bufptr = buf;
    }
    return(bufptr);
}

/* This routine converts an array of eight byte Cray float numbers
*  into an array of float numbers of the host computer.
*
*  The internal representation of the Cray 8-byte float number is:
*      63            48 47                    0
*       ---------------------------------------
*       |S|S|exponent  |   fraction           |
*       ---------------------------------------
*
*  Bit(s)       Description
*  ------       -----------
*   63          Sign bit of the fraction. 1=>negative, 0=>positive
*   62          Sign bit of the exponent. 1=>positive, 0=>negative
*  48-61        exponent is a power of 2
*   0-47        fraction
*
*/
static int cray_r8_conv
(
    const long *outsys,       /* (I) output computer system */
    unsigned char *buf,       /* (I/O) array of floats to be converted */
    const long *size,         /* (I) number of elements in array */
    const double *under,      /* (I) under flow fill value */
    const double *over,       /* (I) over flow fill value */
    const long *flag          /* (I) error flag */
)
{
    double frac;
    double *newval;
    long cur;
    long rstat;
    long sign;
    long status;
    long  exp;
    unsigned long  fmask1;
    unsigned long  fmask2;
    unsigned long  *ptr1;
    unsigned long  *ptr2;
    unsigned char *curptr;

    rstat = E_SUCC;
    curptr = buf;
    newval = (double *) buf;
    for (cur = 0; cur < *size; cur++,curptr+=8,newval++)
    {
        ptr1 = (unsigned long *) curptr;
        ptr2 = (unsigned long *) (curptr + 4);
        sign = (*ptr1 & 0x80000000) ? -1 : 1;
        exp = (long)((*ptr1 & 0x3fff0000) >> 16);
        if (!(*ptr1 & 0x40000000))        /* check exp sign bit 0=>negative */
            exp -= 0x4000;
        fmask1 = (*ptr1 & 0x0000ffff);
        fmask2 = (*ptr2 & 0xffffffff);
        if ((*ptr1 == 0) && (*ptr2 == 0))   /* value = 0 */
        {
            *newval = 0;
            continue;
        }
        status = pxsys_r8err(outsys,under,over,flag,&exp,(long *)&fmask1,
                            (long *)&fmask2,newval);
        switch (status)
        {
            case PXSYS_OK:  
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:
                continue;
        }
        frac = ((fmask1 * (double) LSH32) + fmask2) / (double) LSH48;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *newval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *newval = frac * pow((double) 2.0,(double) exp) * sign;
    }
    return(rstat);
}

/* Assign to the MACH structure the computer system's characteristics.  */
static int get_mach
(
    long insys,
    long outsys,
    struct MACH *isystem,
    struct MACH *osystem
)
{
    long status;
    long nonfatal = 1;
    char errstr[ERRLEN+1];

    status = E_SUCC;
    switch (insys)
    {
        case SYS_IEEE_STD:
        case SYS_SUN_BSD:
        case SYS_IBM_AIX:
        case SYS_SGI_IRX:
        case SYS_DG_UX:
            isystem->bswap = FALSE;
            isystem->c  = PXSYS_HIOFF;
            isystem->r4 = PXSYS_IEEE;
            isystem->r8 = PXSYS_IEEE;
            break;
        case SYS_GOULD_UTX:
            isystem->bswap = FALSE;
            isystem->c  = PXSYS_HIOFF;
            isystem->r4 = PXSYS_GOULD_PN;
            isystem->r8 = PXSYS_GOULD_PN;
            break;
        case SYS_PRIME_PRIMOS:
            isystem->bswap = FALSE;
            isystem->c  = PXSYS_HION;
            isystem->r4 = PXSYS_PRIME;
            isystem->r8 = PXSYS_PRIME;
            break;
        case SYS_VAX_VMS:
            isystem->bswap = NEG_ONE;
            isystem->c  = PXSYS_HIOFF;
            isystem->r4 = PXSYS_DEC_VAX;
            isystem->r8 = PXSYS_DEC_VAX;
            break;
        case SYS_IBM_MVS:
            isystem->bswap = FALSE;
            isystem->c  = PXSYS_HIOFF;
            isystem->r4 = PXSYS_IBM_MVS;
            isystem->r8 = PXSYS_IBM_MVS;
            break;
        case SYS_HP_MPE:
            isystem->bswap = FALSE;
            isystem->c  = PXSYS_HIOFF;
            isystem->r4 = PXSYS_HP_MPE;
            isystem->r8 = PXSYS_HP_MPE;
            break;
        case SYS_CRAY_UNICOS:
            isystem->bswap = FALSE;
            isystem->c  = PXSYS_HIOFF;
            isystem->r4 = PXSYS_CRAY;
            isystem->r8 = PXSYS_CRAY;
            break;
        default:
            sprintf(errstr,"Input computer system <%s> not supported",
                    sys_name[insys]);
            c_errmsg(errstr,"pxsys-badinsys",&nonfatal);
            status = E_FAIL;
            break;
    }
    switch (outsys)
    {
        case SYS_IEEE_STD:
        case SYS_SUN_BSD:
        case SYS_IBM_AIX:
        case SYS_SGI_IRX:
        case SYS_DG_UX:
            osystem->bswap = FALSE;
            osystem->c  = PXSYS_HIOFF;
            osystem->r4 = PXSYS_IEEE;
            osystem->r8 = PXSYS_IEEE;
            break;
        case SYS_GOULD_UTX:
            osystem->bswap = FALSE;
            osystem->c  = PXSYS_HIOFF;
            osystem->r4 = PXSYS_GOULD_PN;
            osystem->r8 = PXSYS_GOULD_PN;
            break;
        case SYS_PRIME_PRIMOS:
            osystem->bswap = FALSE;
            osystem->c  = PXSYS_HION;
            osystem->r4 = PXSYS_PRIME;
            osystem->r8 = PXSYS_PRIME;
            break;
        case SYS_VAX_VMS:
            osystem->bswap = NEG_ONE;
            osystem->c  = PXSYS_HIOFF;
            osystem->r4 = PXSYS_DEC_VAX;
            osystem->r8 = PXSYS_DEC_VAX;
            break;
        case SYS_IBM_MVS:
            osystem->bswap = FALSE;
            osystem->c  = PXSYS_HIOFF;
            osystem->r4 = PXSYS_IBM_MVS;
            osystem->r8 = PXSYS_IBM_MVS;
            break;
        case SYS_HP_MPE:
            osystem->bswap = FALSE;
            osystem->c  = PXSYS_HIOFF;
            osystem->r4 = PXSYS_HP_MPE;
            osystem->r8 = PXSYS_HP_MPE;
            break;
        case SYS_CRAY_UNICOS:
            isystem->bswap = FALSE;
            isystem->c  = PXSYS_HIOFF;
            isystem->r4 = PXSYS_CRAY;
            isystem->r8 = PXSYS_CRAY;
            break;
        default:
            sprintf(errstr,"Output computer system <%s> not supported",
                    sys_name[insys]);
            c_errmsg(errstr,"pxsys-badoutsys",&nonfatal);
            status = E_FAIL;
            break;
    }
    return(status);
}

/* HION sets the most significant bit of each byte in the buffer to 1.  This
   is a useful translation when converting non-PRIMOS ASCII data into PRIMOS
   ASCII data.                                                              */
static void hion
(
    unsigned char *bufptr,
    const long *nbyte
)
{
    long cnt;
    unsigned char *ptr;

    ptr = bufptr;
    for (cnt=0; cnt < *nbyte; cnt++,ptr++)
        *ptr |= 0x80;   /* set high bit to 1, leave all other bits the same */
}

/* HIOFF sets the most significant bit of each byte in the buffer to 0.  This
   is a useful translation when converting PRIMOS ASCII data into non-PRIMOS
   ASCII data.                                                              */
static void hioff
(
    unsigned char *bufptr,
    const long *nbyte
)
{
    long cnt;
    unsigned char *ptr;

    ptr = bufptr;
    for (cnt=0; cnt < *nbyte; cnt++,ptr++)
        *ptr &= 0x7f;   /* set high bit to 0, leave all other bits the same */
}

/* This routine converts an array of four byte IBM MVS float numbers
*  into an array of float numbers of the host computer.
*
*  The internal representation of the IBM MVS 4-byte float number is:
*       0  1           7 8                    31
*       ----------------------------------------
*       |S |  exponent  |   fraction           |
*       ----------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 7-bit binary number with a bias of 64 to which the
*       base 16 is raised
*  Fraction is a 24-bit hexadecimal normalized number with a radix point to
*       the left of the highest order fraction bit (bit 8)
*
*  Algorithm consists of masking the sign bit (bit 0), exponent field
*  (bits 1 - 7), and fraction field (bits 8 - 31).  After obtaining the 
*  exponent and fraction the floating point arithmetic of the host 
*  computer is used to compute the float number.
*/

static int ibm_r4_conv
(
    const long *outsys,   /* (I) output computer system */
    unsigned long *buf,   /* (I/O) array of floats to be converted */
    const long *size,     /* (I) number of elements in array */
    const float *under,   /* (I) under flow fill value */
    const float *over,    /* (I) over flow fill value */
    const long *flag      /* (I) error flag */
)
{
    float frac;
    float *newval;
    long cur;
    long rstat;
    long status;
    long sign;
    long  exp;
    unsigned long  ival;
    unsigned long  fmask;

    rstat = E_SUCC;
    newval = (float *) buf;
    for (cur = 0; cur < *size; cur++,++newval)
    {
        ival = (unsigned long) *(buf + cur);
        sign = (ival & 0x80000000) ? -1 : 1;
        exp = (long)((ival & 0x7f000000) >> 24);
        fmask = ival & 0x00ffffff;
        if ((exp == 0) && (fmask == 0))
        {
            *newval = 0.0;
            continue;
        }
        exp = (exp - 64) * 4;             /* convert from base 16 to base 2 */
        status = pxsys_r4err (outsys,under,over,flag,&exp,(long *)&fmask,
                              newval);
        switch (status)
        {
            case PXSYS_OK:   
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        frac = (float) fmask / 0x1000000;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *newval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *newval = frac * (float) pow((double) 2.0,(double) exp) *
                      (float)sign;
    }
    return(rstat);
}

/* This routine converts an array of eight byte IBM MVS double numbers
*  into an array of double numbers of the host computer.
*
*  The internal representation of the IBM MVS 8-byte float number is:
*       0 1           7 8                    63
*       ---------------------------------------
*       |S|  exponent  |   fraction           |
*       ---------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 7-bit binary number with a bias of 64 to which the
*       base 16 is raised
*  Fraction is a 56-bit hexadecimal normalized number with a radix point to
*       the left of the highest order fraction bit (bit 8)
*
*  Algorithm consists of masking the sign bit (bit 0), exponent field
*  (bits 1 - 7), and fraction field (bits 8 - 63).  After obtaining the 
*  exponent and fraction the floating point arithmetic of the host 
*  computer is used to compute the float number.
*/

static int ibm_r8_conv
(
    const long *outsys,   /* (I) output computer system */
    unsigned char *buf,   /* (I/O) array of floats to be converted */
    const long *size,     /* (I) number of elements in array */
    const double *under,  /* (I) under flow fill value */
    const double *over,   /* (I) over flow fill value */
    const long *flag      /* (I) error flag */
)
{
    double frac;
    double *newval;
    long cur;
    long rstat;
    long sign;
    long status;
    long  exp;
    unsigned long fmask1;
    unsigned long fmask2;
    unsigned long *ptr1;
    unsigned long *ptr2;
    unsigned char *curptr;

    rstat = E_SUCC;
    curptr = buf;
    newval = (double *) buf;
    for (cur = 0; cur < *size; cur++,curptr+=8,newval++)
    {
        ptr1 = (unsigned long *) curptr;
        ptr2 = (unsigned long *) (curptr + 4);
        sign = (*ptr1 & 0x80000000) ? -1 : 1;
        exp = (long)((*ptr1 & 0x7f000000) >> 24);
        fmask1 = *ptr1 & 0x00ffffff;
        fmask2 = *ptr2 & 0xffffffff;
        if ((exp == 0) && (fmask1 == 0) && (fmask2 == 0))
        {
            *newval = 0.0;
            continue;
        }
        exp = (exp - 64) * 4;             /* convert from base 16 to base 2 */
        status = pxsys_r8err (outsys,under,over,flag,&exp,(long *)&fmask1,
                             (long *)&fmask2,newval);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        frac = (double) ((fmask1 * (double) LSH32) + fmask2) / (double) LSH56;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *newval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *newval = frac * pow((double) 2.0,(double) exp) * sign;
    }
    return(rstat);
}

/* This routine converts an array of four byte IEEE float numbers
*  into an array of float numbers of the host computer.
*
*  The internal representation of the IEEE 4-byte float number is:
*      31 30         23 22                    0
*       ---------------------------------------
*       |S|  exponent  |   fraction           |
*       ---------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 8-bit real number with an excess of 127 to which the
*       base 2 is raised
*  Fraction is a 23-bit hexadecimal normalized number with an implicit
*       1 in bit 24
*
*  Algorithm consists of masking the sign bit (bit 31), exponent field
*  (bits 30 - 23), and fraction field (bits 22 - 0).  After obtaining the
*  exponent and fraction, the floating point arithmetic of the host computer
*  is used to compute the float number.
*/

static int ieee_r4_conv
(
    const long *outsys,    /* (I) output computer system */
    unsigned long *buf,    /* (I/O) array of floats to be converted */
    const long *size,      /* (I) number of elements in array */
    const float *under,    /* (I) under flow value */
    const float *over,     /* (I) over flow value */
    const long *flag       /* (I) error detection flag */
)
{
    float frac;
    float *outval;
    long  cur;
    long  exp;
    long  sign;
    long  rstat;
    long  status;
    unsigned long fmask;
    unsigned long ival;

    outval = (float *) buf;
    rstat = E_SUCC;
    for (cur=0; cur<*size; cur++)
    {
        ival = *(buf + cur);
        sign = (ival & 0x80000000) ? -1 : 1;
        exp = (long)((ival & 0x7f800000) >> 23);
        fmask = (ival & 0x007fffff);
        if (exp == 0)               /* value = 0, normal and denormalized */
        {
            *(outval + cur) = 0.0;
            continue;
        }
        exp -= IEEE_R4_EXCESS;
        status = pxsys_r4err (outsys,under,over,flag,&exp,(long *)&fmask,
                             (outval+cur));
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        exp += IEEE_R4_EXCESS;
        fmask |= 0x00800000;
        frac = (float) fmask / 0x1000000;
        *(outval + cur) = frac * (float) IEEE_pow[exp] * (float)sign;
    }
    return(rstat);
}

/* This routine converts an array of eight byte IEEE standard double numbers
*  into an array of double numbers of the host computer.
*
*  The internal representation of the IEEE standard 8-byte double number is:
*      63 62         52 51                    0
*       ---------------------------------------
*       |S|  exponent  |   fraction           |
*       ---------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 11-bit real number with an excess of 1023 to which the
*       base 2 is raised
*  Fraction is a 52-bit hexadecimal normalized number with an implicit
*       1 in bit 53
*
*  Algorithm consists of masking the sign bit (bit 63), exponent field
*  (bits 62 - 52), and fraction field (bits 51 - 0).  After obtaining the
*  exponent and fraction, the floating point arithmetic of the host computer
*  is used to compute the float number.                                     */

static int ieee_r8_conv
(
    const long *outsys,           /* (I) output computer system */
    unsigned char *buf,     /* (I) array of floats to be converted */
    const long *size,           /* (I) number of elements in array */
    const double *under,          /* (I) under flow fill value */
    const double *over,           /* (I) over flow fill value */
    const long *flag           /* (I) error flag */
)
{
    double frac;
    double *newval;
    long cur;
    long rstat;
    long sign;
    long status;
    long  exp;
    unsigned long  fmask1;
    unsigned long  fmask2;
    unsigned long  *ptr1;
    unsigned long  *ptr2;
    unsigned char *curptr;

    rstat = E_SUCC;
    curptr = buf;
    newval = (double *) buf;
    for (cur = 0; cur < *size; cur++,curptr+=8,newval++)
    {
        ptr1 = (unsigned long *) curptr;
        ptr2 = (unsigned long *) (curptr + 4);
        sign = (*ptr1 & 0x80000000) ? -1 : 1;
        exp = (long)((*ptr1 & 0x7ff00000) >> 20);
        fmask1 = (*ptr1 & 0x000fffff);
        fmask2 = (*ptr2 & 0xffffffff);
        if (exp == 0)               /* value = 0, normal and denormalized */
        {
            *newval = 0;
            continue;
        }
        exp -= 1022;                        /* exponent bias */
        status = pxsys_r8err (outsys,under,over,flag,&exp,(long *)&fmask1,
                             (long *)&fmask2,newval);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        fmask1 |= 0x00100000;
        frac = ((fmask1 * (double) LSH32) + fmask2) / (double) LSH53;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *newval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *newval = frac * pow((double) 2.0,(double) exp) * sign;
    }
    return(rstat);
}

/* This routine converts an array of four byte float numbers in the local
*  host format to an array of float numbers in IEEE standard format.
*
*  The internal representation of the IEEE 4-byte float number is:
*      31 30         23 22                    0
*       ---------------------------------------
*       |S|  exponent  |   fraction           |
*       ---------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 8-bit real number with an excess of 127 to which the
*       base 2 is raised
*  Fraction is a 23-bit hexadecimal normalized number with an implicit
*       1 in bit 24
*
*  Algorithm:
*       From float number on local host, compute the sign bit, exponent
*       field, and fraction field to make an IEEE float number.  The sign
*       bit is unchanged.  The exponent is computed by taking the natural
*       log of the local host's float number divided by natural log 2, and
*       rounding up to next highest integer (exponent is the smallest integer
*       number such that 2 raised to the power of it is greater than the
*       float number).  The fraction is computed by dividing the float number
*       by 2 raised to the power of the exponent and multiplying the quotient
*       by 0x1000000 to create an integer value.  The fields are shifted
*       such into a four byte entity to create the IEEE float number.
*
*  Notes:
*       To improve the performance, a static array containing the values for
*       2 raised to the power of -126 through 127 is used.  The exponent
*       field is computed by performing a binary search of this array.     */
static int ieee_r4_out
(
    const long *outsys,   /* (I) output computer system */
    float *buf,           /* (I/O) array of floats to be converted */
    const long *size,     /* (I) number elements in array */
    const float *under,   /* (I) under flow fill value */
    const float *over,    /* (I) over flow fill value */
    const long *flag      /* (I) error detection flag */
)
{
    double dval;
    float ival;
    long  cur;
    long  rstat;
    long  status;
    long   sign;
    long   exp, lexp, hexp;
    unsigned long *curptr;
    unsigned long frac;

    rstat = E_SUCC;
    curptr = (unsigned long *) buf;
    for (cur=0; cur<*size; cur++,curptr++)
    {
        ival = *(buf + cur);
        retry:
        sign = 0;
        if (ival == 0)              /* if value is zero IEEE value is zero */
        {
            *curptr = 0;
            continue;
        }
        if (ival < 0)               /* if negative, set sign bit and take */
        {                           /*      absolute value of float number */
            sign = 1;
            ival *= -1;
        }
        if (ival > IEEE_pow[IEEE_POW_HEXP])         /* over flow */
            exp = IEEE_POW_HEXP + 1;
        else if (ival < IEEE_pow[IEEE_POW_LEXP])    /* under flow */
            exp = IEEE_POW_LEXP - 1;
        else                                    /* perform binary search to */
        {                                       /*   assign exponent value */
            for (lexp=IEEE_POW_LEXP,hexp=IEEE_POW_HEXP; lexp+1<hexp; )
            {
                exp = (lexp + hexp) / 2;
                if (IEEE_pow[exp] > ival)
                    hexp = exp;
                else
                    lexp = exp;
            }
            if (IEEE_pow[exp] <= ival)
                ++exp;
            dval = IEEE_pow[exp];
            frac = (unsigned long)(((float) (ival / dval)) * 0x1000000);
        }
        exp -= IEEE_R4_EXCESS;
        status = pxsys_r4err(outsys,under,over,flag,&exp,(long *)&frac,&ival);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                goto retry;
        }
        exp += IEEE_R4_EXCESS;
        *curptr = (sign << 31) | (exp << 23) | (frac & 0x7fffff);
    }
    return(rstat);
}

/* This routine converts an array of eight byte float numbers in the local
*  host format to an array of eight byte float numbers in IEEE standard format.
*
*  The internal representation of the IEEE 4-byte float number is:
*      64 63         52 51                    0
*       ---------------------------------------
*       |S|  exponent  |   fraction           |
*       ---------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 11-bit real number with an excess of 1023 to which the
*       base 2 is raised
*  Fraction is a 52-bit hexadecimal normalized number with an implicit
*       1 in bit 53
*
*  Algorithm:
*       From float number on local host, compute the sign bit, exponent
*       field, and fraction field to make an IEEE float number.  The sign
*       bit is unchanged.  The exponent is computed by taking the natural
*       log of the local host's float number divided by natural log 2, and
*       rounding up to next highest integer (exponent is the smallest integer
*       number such that 2 raised to the power of it is greater than the
*       float number).  The fraction is computed by dividing the float number
*       by 2 raised to the power of the exponent and multiplying the quotient
*       to create an integer value.  The fields are shifted such into a eight
*       byte entity to create the IEEE float number.
*
*  Notes:
*       Unlike function ieee_r4_out, this function does not make use of
*       the static array, therefore this routine is CPU intensive.  Also
*       note the large numbers used to compute the fraction field results
*       in rounding errors.                                                */
static int ieee_r8_out
(
    const long *outsys,     /* (I) output computer system */
    double *buf,            /* (I/O) array of floats to be converted */
    const long *size,       /* (I) number of elements in array */
    const double *under,    /* (I) under flow fill value */
    const double *over,     /* (I) over flow fill value */
    const long *flag        /* (I) error detection flag */
)
{
    double dexp;
    double dval;
    double frac;
    double ival;
    long  cur;
    long  rstat;
    long  status;
    long  sign;
    long  exp;
    unsigned long *curptr;
    unsigned long fmask1;
    unsigned long fmask2;

    rstat = E_SUCC;
    curptr = (unsigned long *) buf;
    for (cur=0; cur<*size; cur++,curptr+=2)
    {
        ival = *(buf + cur);
        retry:
        sign = 0;
        if (ival == 0)                  /* if input value is zero, then IEEE */
        {                               /*   value is zero */
            *curptr = *(curptr + 1) = 0;
            continue;
        }
        if (ival < 0)
        {
            sign = 1;
            ival *= -1;
        }
/******************************
*****   CPU INTENSIVE *********
******************************/
        dexp = log(ival) / log((double) 2.0);
        exp = (int) ceil(dexp);                /* exponent is next highest */
        dval = pow((double) 2.0, (double) exp);
        if (dval <= ival)                      /* may need to increment exp */
        {                                      /* if rounding error occured */
            ++exp;                             /* in ceil function, or if   */
            dval *= 2;                         /* number is a power of 2.   */
        }
        frac = ival / dval;                    /* may result in rounding    */
        fmask1 = (unsigned long)floor(frac * LSH21); /*   errors            */
        fmask2 = (unsigned long)(((frac * LSH21) - fmask1) * LSH32);
/********************************/
        status = pxsys_r8err (outsys,under,over,flag,&exp,(long *)&fmask1,
                             (long *)&fmask2,&ival);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
               goto retry;
        }
        exp += 1022;                                /* exponent bias */
        *curptr = (sign << 31) | (exp << 20) | (fmask1 & 0x0fffff);
        *(curptr + 1) = fmask2;
    }
    return(rstat);
}

/* This routine converts an array of four byte Gould Power Node float numbers
*  into an array of float numbers of the host computer.
*
*  The internal representation of the Gould 4-byte float number is:
*       0  1           7 8                    31
*       ----------------------------------------
*       |S |  exponent  |   fraction           |
*       ----------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 7-bit binary number with a bias of 64 to which the
*       base 16 is raised
*  Fraction is a 24-bit hexadecimal normalized number with a radix point to
*       the left of the highest order fraction bit (bit 8)
*  Positive real value is converted to a negative real value by taking the
*       two's complement of the positive fraction and the one's complement
*       of the biased exponent.
*
*  Algorithm consists of masking the sign bit (bit 0), exponent field
*  (bits 1 - 7), and fraction field (bits 8 - 31).  If the sign is negative
*  take the one's complement of the exponent and the two's complement of the
*  fraction to get the positive real value.  After obtaining the exponent
*  and fraction the floating point arithmetic of the host computer is used
*  to compute the float number.                                            */

static int pn_r4_conv
(
    const long *outsys,  /* (I) output computer system */
    unsigned long *buf,  /* (I/O) array of floats to be converted */
    const long *size,    /* (I) number of elements in array */
    const float *under,  /* (I) under flow fill value */
    const float *over,   /* (I) over flow fill value */
    const long *flag     /* (I) error flag */
)
{
    float frac;
    float *newval;
    long cur;
    long rstat;
    long status;
    long sign;
    long  exp;
    unsigned long  ival;
    unsigned long  fmask;

    rstat = E_SUCC;
    newval = (float *) buf;
    for (cur = 0; cur < *size; cur++,++newval)
    {
        ival = (unsigned long) *(buf + cur);
        sign = (ival & 0x80000000) ? -1 : 1;
        if (sign < 0)
        {
            exp = (long)((~(ival) & 0x7f000000) >> 24);
            fmask = (~(ival) + 1) & 0x00ffffff;
        }
        else
        {
            exp = (long)((ival & 0x7f000000) >> 24);
            fmask = ival & 0x00ffffff;
        }
        if ((exp == 0) && (fmask == 0))
        {
            *newval = 0.0;
            continue;
        }
        exp = (exp - 64) * 4;            /* convert from base 16 to base 2 */
        status = pxsys_r4err (outsys,under,over,flag,&exp,(long *)&fmask,
                              newval);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        frac = (float) fmask / 0x1000000;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *newval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *newval = frac * (float)pow((double)2.0,(double)exp) * 
                             (float)sign;
    }
    return(rstat);
}

/* This routine converts an array of eight byte Gould Power Node double numbers
*  into an array of double numbers of the host computer.
*
*  The internal representation of the Gould 8-byte float number is:
*       0 1           7 8                    63
*       ---------------------------------------
*       |S|  exponent  |   fraction           |
*       ---------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 7-bit binary number with a bias of 64 to which the
*       base 16 is raised
*  Fraction is a 56-bit hexadecimal normalized number with a radix point to
*       the left of the highest order fraction bit (bit 8)
*  Positive real value is converted to a negative real value by taking the
*       two's complement of the positive fraction and the one's complement
*       of the biased exponent.
*
*  Algorithm consists of masking the sign bit (bit 0), exponent field
*  (bits 1 - 7), and fraction field (bits 8 - 63).  If the sign is negative
*  take the one's complement of the exponent and the two's complement of the
*  fraction to get the positive real value.  After obtaining the exponent
*  and fraction the floating point arithmetic of the host computer is used
*  to compute the float number.                                            */

static int pn_r8_conv
(
    const long *outsys,   /* (I) output computer system */
    unsigned char *buf,   /* (I/O) array of floats to be converted */
    const long *size,     /* (I) number of elements in array */
    const double *under,  /* (I) under flow fill value */
    const double *over,   /* (I) over flow fill value */
    const long *flag      /* (I) error flag */
)
{
    double frac;
    double *newval;
    long cur;
    long rstat;
    long sign;
    long status;
    long  exp;
    unsigned long fmask1;
    unsigned long fmask2;
    unsigned long *ptr1;
    unsigned long *ptr2;
    unsigned char *curptr;

    rstat = E_SUCC;
    curptr = buf;
    newval = (double *) buf;
    for (cur = 0; cur < *size; cur++,curptr+=8,newval++)
    {
        ptr1 = (unsigned long *) curptr;
        ptr2 = (unsigned long *) (curptr + 4);
        sign = (*ptr1 & 0x80000000) ? -1 : 1;
        if (sign < 0)
        {
            exp = (long)((~(*ptr1) & 0x7f000000) >> 24);
            fmask1 = (~(*ptr1)) & 0x00ffffff;
            fmask2 = (~(*ptr2)) & 0xffffffff;
            if (fmask2 == 0xffffffff)
            {
                fmask2 = 0x0;
                ++fmask1;
            }
            else
            {
                ++fmask2;
            }
        }
        else
        {
            exp = (long)((*ptr1 & 0x7f000000) >> 24);
            fmask1 = *ptr1 & 0x00ffffff;
            fmask2 = *ptr2 & 0xffffffff;
        }
        if ((exp == 0) && (fmask1 == 0) && (fmask2 == 0))
        {
            *newval = 0.0;
            continue;
        }
        exp = (exp - 64) * 4;             /* convert from base 16 to base 2 */
        status = pxsys_r8err (outsys,under,over,flag,&exp,(long *)&fmask1,
                             (long *)&fmask2,newval);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        frac = (double) ((fmask1 * (double) LSH32) + fmask2) / (double) LSH56;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *newval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *newval = frac * pow((double) 2.0,(double) exp) * sign;
    }
    return(rstat);
}

/* This routine converts an array of four byte PRIME float numbers
*  into an array of float numbers of the host computer.
*
*  The internal representation of the PRIME 4-byte float number is:
*       1 2                24 25             32
*       ---------------------------------------
*       |S|  fraction        |   exponent     |
*       ---------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 8-bit unsigned number with an excess of 128 to which the
*       base 2 is raised
*  Fraction is a 23-bit hexadecimal normalized number.  The fraction and
*  sign are treated as a two's complement number.
*
*  Algorithm consists of masking the sign bit (bit 1), exponent field
*  (bits 25 - 32), and fraction field (bits 2 - 24).  If the sign bit is
*  set, apply two's complement to the fraction field.  After obtaining the
*  exponent and fraction, the floating point arithmetic of the host computer
*  is used to compute the float number.                                    */

static int prime_r4_conv
(
    const long *outsys,  /* (I) output computer system */
    unsigned long *buf,  /* (I/O) array of floats to be converted */
    const long *size,    /* (I) number of elements in array */
    const float *under,  /* (I) under flow value */
    const float *over,   /* (I) over flow value */
    const long *flag     /* (I) error detection flag */
)
{
    float frac;
    float *outval;
    long  cur;
    long  exp;
    long  sign;
    long  rstat;
    long  status;
    unsigned long fmask;
    unsigned long ival;

    outval = (float *) buf;
    rstat = E_SUCC;
    for (cur=0; cur<*size; cur++,outval++)
    {
        ival = *(buf + cur);
        sign = (ival & 0x80000000) ? -1 : 1;
        exp = (long)(ival & 0x000000ff);
        fmask = (ival & 0xffffff00) >> 8;
        if (sign == -1)                         /* if negative, apply two's */
            fmask = (~fmask + 1) & 0x00ffffff;  /*   complement to fraction */
        if ((exp == 0) && (fmask == 0))
        {
            *outval = 0.0;
            continue;
        }
        exp -= 128;
        status = pxsys_r4err (outsys,under,over,flag,&exp,(long *)&fmask,
                              outval);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        frac = (float) fmask / 0x800000;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *outval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *outval = frac * pow((double) 2.0,(double) exp) * sign;
    }
    return(rstat);
}

/* This routine converts an array of eight byte PRIME double float numbers
*  into an array of double float numbers of the host computer.
*
*  The internal representation of the PRIME double float number is:
*       1 2                 48 49             64
*       ---------------------------------------
*       |S|  fraction         |   exponent    |
*       ---------------------------------------
*  Sign bit is a 1 if the number is negative
*  Exponent is a 15-bit signed integer number with an excess of 128 to
*       which the base 2 is raised
*  Fraction is a 48-bit hexadecimal normalized number.  If the sign bit is
*       set, apply two's complement.
*
*  Algorithm consists of masking the sign bit (bit 1), exponent field
*  (bits 49 - 64), and fraction field (bits 2 - 48).  If the sign bit is
*  set, apply two's complement to the fraction field.  If the high bit of
*  the exponent is set, apply two's complement to the exponent field.
*  After obtaining the exponent and fraction, the floating point arithmetic
*  of the host computer is used to compute the float number.               */

static int prime_r8_conv
(
    const long *outsys,        /* (I) output computer system */
    unsigned char *buf,        /* (I/O) array of floats to be converted */
    const long *size,          /* (I) number of elements in array */
    const double *under,       /* (I) under flow fill value */
    const double *over,        /* (I) over flow fill value */
    const long *flag           /* (I) error flag */
)
{
    double frac;
    double *newval;
    long cur;
    long rstat;
    long sign;
    long status;
    long exp;
    unsigned long  fmask1;
    unsigned long  fmask2;
    unsigned long  *ptr1;
    unsigned long  *ptr2;
    unsigned char *curptr;

    rstat = E_SUCC;
    curptr = buf;
    newval = (double *) buf;
    for (cur = 0; cur < *size; cur++,curptr+=8,newval++)
    {
        ptr1 = (unsigned long *) curptr;
        ptr2 = (unsigned long *) (curptr + 4);
        sign = (*ptr1 & 0x80000000) ? -1 : 1;
        exp = (long)(*ptr2 & 0x00007fff);
        if (exp & 0x00004000)                    /* exponent is negative */
            exp = ((~exp + 1) & 0x00007fff) * -1;
        fmask1 = (*ptr1 & 0xffffffff);
        fmask2 = (*ptr2 & 0xffff8000) >> 15;
        if (sign == -1)                        /* if sign bit is set, apply */
        {                                      /*   two's complement to the */
            fmask1 = ~fmask1;                  /*   fraction */
            fmask2 = (~fmask2 + 1) & 0x0001ffff;;
        }
        if ((exp == 0) && (fmask1 == 0) && (fmask2 == 0))
        {
            *newval = 0;
            continue;
        }
        exp -= 128;                        /* exponent bias */
        status = pxsys_r8err (outsys,under,over,flag,&exp,(long *)&fmask1,
                             (long *)&fmask2,newval);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        frac = ((fmask1 * (double) LSH17) + fmask2) / (double) LSH48;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *newval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *newval = frac * pow((double) 2.0,(double) exp) * sign;
    }
    return(rstat);
}

/* This routine converts an array of four byte VAX float numbers
*  into an array of float numbers of the host computer.
*
*  The internal representation of the VAX 4-byte float number is:
*        1 1
*        5 4             7 6           0
*       --------------------------------
*       |S|  exponent     |   fraction |  :A
*       --------------------------------
*       |           fraction           |  :A+2
*       --------------------------------
*
*  Sign bit is a 1 if the number is negative
*  Exponent is a 8-bit real number with an excess of 128 to which the
*       base 2 is raised
*  Fraction is a 23-bit hexadecimal normalized number with an implicit
*       1 in bit 24
*  Within the fraction, bits of increasing significance go from 16
*       through 31 and 0 through 6
*
*  Algorithm consists of masking the sign bit (bit 15), exponent field
*  (bits 14 - 7), and fraction field (bits 6 - 0, and 31 - 16).  After
*  obtaining the exponent and fraction, the floating point arithmetic of
*  the host computer is used to compute the float number.                  */

static int vax_r4_conv
(
    const long *outsys,   /* (I) output computer system */
    unsigned long *buf,   /* (I/O) array of floats to be converted */
    const long *size,     /* (I) number of elements in array */
    const float *under,   /* (I) under flow fill value */
    const float *over,    /* (I) over flow fill value */
    const long *flag      /* (I) error flag */
)
{
    float   frac;
    float   *newval;
    long    cur;
    long    rstat;
    long    status;
    long    sign;
    long    exp;
    unsigned long   fmask;
    unsigned long   ival;

    rstat = E_SUCC;
    newval = (float *) buf;
    for (cur = 0; cur < *size; cur++,++newval)
    {
        ival = (unsigned long) *(buf + cur);
        exp = (long)((ival & 0x7f80) >> 7);               /* bits 7 - 14 */
        sign = (ival & 0x8000) ? -1 : 1;            /* bit 15 */
        fmask = ((ival & 0x7f) << 16) | ((ival & 0xffff0000) >> 16);
        if ((exp == 0) && (fmask == 0))
        {
            *newval = 0.0;
            continue;
        }
        exp -= 128;                                     /* exponent bias */
        status = pxsys_r4err (outsys,under,over,flag,&exp,(long *)&fmask,
                              newval);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        fmask |= 0x800000;                  /* implicit fraction bit */
        frac = (float) fmask / 0x1000000;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *newval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *newval = frac * (float)pow((double)2.0,(double)exp) * 
                             (float)sign;
    }
    return(rstat);
}

/* This routine converts an array of eight byte VAX float numbers
*  into an array of eight byte float numbers of the host computer.
*
*  The internal representation of the VAX 4-byte float number is:
*        1 1
*        5 4             7 6           0
*       --------------------------------
*       |S|  exponent     |   fraction |  :A
*       --------------------------------
*       |           fraction           |  :A+2
*       --------------------------------
*       --------------------------------
*       |           fraction           |  :A+4
*       --------------------------------
*       |           fraction           |  :A+6
*       --------------------------------
*
*  Sign bit is a 1 if the number is negative
*  Exponent is a 8-bit real number with an excess of 128 to which the
*       base 2 is raised
*  Fraction is a 39-bit hexadecimal normalized number with an implicit
*       1 in bit 40
*  Within the fraction, bits of increasing significance go from 48
*       through 63, 32 through 47, 16 through 31 and 0 through 6
*
*  Algorithm consists of masking the sign bit (bit 15), exponent field
*  (bits 14 - 7), and fraction field (bits 6 - 0, 31 - 16, 32 - 47, and
*  48 - 63).  After *  obtaining the exponent and fraction, the floating
*  point arithmetic of the host computer is used to compute the float number.  */

static int vax_r8_conv
(
    const long *outsys,   /* (I) output computer system */
    unsigned char *buf,   /* (I/O) array of floats to be converted */
    const long *size,     /* (I) number of elements in array */
    const double *under,  /* (I) under flow fill value */
    const double *over,   /* (I) over flow fill value */
    const long *flag      /* (I) error flag */
)
{
    double  frac;
    double  *newval;
    long    cur;
    long    rstat;
    long    sign;
    long    status;
    long    exp;
    unsigned long   fmask1;
    unsigned long   fmask2;
    unsigned long   *ptr1;
    unsigned long   *ptr2;
    unsigned char   *curptr;

    rstat = E_SUCC;
    curptr = buf;
    newval = (double *) buf;
    for (cur = 0; cur < *size; cur++,curptr+=8,newval++)
    {
        ptr1 = (unsigned long *) curptr;
        ptr2 = (unsigned long *) (curptr + 4);
        exp = (long)((*ptr1 & 0x7f80) >> 7);               /* bits 7 - 14 */
        sign = (*ptr1 & 0x8000) ? -1 : 1;                   /* bit 15 */
        fmask1 = ((*ptr1 & 0x7f) << 16) | ((*ptr1 & 0xffff0000) >> 16);
        fmask2 = ((*ptr2 & 0x0000FFFF) << 16) | ((*ptr2 & 0xFFFF0000) >> 16);
        if ((exp == 0) && (fmask1 == 0) && (fmask2 == 0))
        {
            *newval = 0.0;
            continue;
        }
        exp -= 128;                                        /* exponent bias */
        status = pxsys_r8err (outsys,under,over,flag,&exp,(long *)&fmask1,
                             (long *)&fmask2,newval);
        switch (status)
        {
            case PXSYS_OK:                          
                break;
            case PXSYS_ERROR:                       
                return(E_FAIL);
            case PXSYS_RESET:       
                continue;
        }
        fmask1 |= 0x00800000;                      /* implicit fraction bit */
        frac = ((fmask1 * (double) LSH32) + fmask2) / (double) LSH56;
        if ((IEEE_MINEXP <= exp) && (exp <= IEEE_MAXEXP))
            *newval = frac * IEEE_pow[exp+IEEE_R4_EXCESS] * sign;
        else
            *newval = frac * (float) pow((double) 2.0,(double) exp) * sign;
    }
    return(rstat);
}

static int pxsys_r4err
(
    const long *outsys,
    const float *under,
    const float *over,
    const long *flag,
    long *exp,
    long *frac,
    float *ival
)
{
    long rstat;
    long nonfatal = 1;
    char errstr[ERRLEN+1];

    rstat = PXSYS_OK;
    switch (*outsys)
    {
        case SYS_IEEE_STD:
        case SYS_SUN_BSD:
        case SYS_IBM_AIX:
        case SYS_SGI_IRX:
        case SYS_DG_UX:
            if ((*exp < -126) || (*exp > 127))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 127)
                        {
                            *exp = 127;
                            *frac = 0x7fffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *ival = *exp > 127 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        case SYS_GOULD_UTX:
            if ((*exp < -256) || (*exp > 252))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 252)
                        {
                            *exp = 252;
                            *frac = 0xffffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *ival = *exp > 252 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        case SYS_PRIME_PRIMOS:
            if ((*exp < -128) || (*exp > 127))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp >127)
                        {
                            *exp = 127;
                            *frac = 0x7fffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *ival = *exp > 127 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        case SYS_VAX_VMS:
            if ((*exp < -128) || (*exp > 127))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 127)
                        {
                            *exp = 127;
                            *frac = 0x7fffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *ival = *exp > 127 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        case SYS_IBM_MVS:
            if ((*exp < -63) || (*exp > 64))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 64)
                        {
                            *exp = 64;
                            *frac = 0xffffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *ival = *exp > 64 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        default:
            sprintf (errstr,"Invalid system name <%s>",sys_name[*outsys]);
            c_errmsg (errstr,"pxsys-invalsys",&nonfatal);
            rstat = PXSYS_ERROR;
            break;
    }
    return(rstat);
}

static int pxsys_r8err
(
    const long *outsys,
    const double *under,
    const double *over,
    const long *flag,
    long *exp,
    long *frac1,
    long *frac2,
    double *val
)
{
    long rstat;
    long nonfatal = 1;
    char errstr[ERRLEN+1];

    rstat = PXSYS_OK;
    switch (*outsys)
    {
        case SYS_IEEE_STD:
        case SYS_SUN_BSD:
        case SYS_IBM_AIX:
        case SYS_SGI_IRX:
        case SYS_DG_UX:
            if ((*exp < -1022) || (*exp > 1023))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 1023)
                        {
                            *exp = 1023;
                            *frac1 = 0x000fffff;
                            *frac2 = 0xffffffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac1 = *frac2 = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *val = *exp > 1023 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        case SYS_GOULD_UTX:
            if ((*exp < -256) || (*exp > 252))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 252)
                        {
                            *exp = 252;
                            *frac1 = 0x00ffffff;
                            *frac2 = 0xffffffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac1 = *frac2 = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *val = *exp > 252 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        case SYS_PRIME_PRIMOS:
            if ((*exp < -32896) || (*exp > 32639))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 32639)
                        {
                            *exp = 32639;
                            *frac1 = 0x7fffffff;
                            *frac2 = 0x0001ffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac1 = *frac2 = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *val = *exp > 32639 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        case SYS_VAX_VMS:
            if ((*exp < -128) || (*exp > 127))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 127)
                        {
                            *exp = 127;
                            *frac1 = 0x007fffff;
                            *frac2 = 0xffffffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac1 = *frac2 = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *val = *exp > 127 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        case SYS_IBM_MVS:
            if ((*exp < -63) || (*exp > 64))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 64)
                        {
                            *exp = 64;
                            *frac1 = 0x00ffffff;
                            *frac2 = 0xffffffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac1 = *frac2 = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *val = *exp > 64 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        case SYS_CRAY_UNICOS:
            if ((*exp < -16384) || (*exp > 16383))
            {
                switch (*flag)
                {
                    case SYS_ERR:
                        c_errmsg ("Under/over flow error",
                                  "pxsys-flow",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                    case SYS_SET:
                        if (*exp > 16383)
                        {
                            *exp = 16383;
                            *frac1 = 0x0000ffff;
                            *frac2 = 0xffffffff;
                            rstat = PXSYS_OK;
                        }
                        else
                        {
                            *exp = *frac1 = *frac2 = 0;
                            rstat = PXSYS_OK;
                        }
                        break;
                    case SYS_USE:
                        *val = *exp > 16383 ? *over : *under;
                        rstat = PXSYS_RESET;
                        break;
                    default:
                        c_errmsg ("Invalid under/over flow flag",
                                  "pxsys-invalflg",&nonfatal);
                        rstat = PXSYS_ERROR;
                        break;
                }
            }
            break;
        default:
            sprintf (errstr,"Invalid system name <%s>",sys_name[*outsys]);
            c_errmsg (errstr,"pxsys-invalsys",&nonfatal);
            rstat = PXSYS_ERROR;
            break;
    }
    return(rstat);
}
