/*
                          STATIO package

*/

#include <stdio.h>
#include <ctype.h>
#ifdef vms
#include <string.h>
#endif
#include "desc.h"
#include "las.h"
#include "util.h"
#include "statio.h"

/*--------------------------- #defines -------------------------------------*/

#define   REC_SIZE         512
#define   NUM_DATA_BYTES   440
/*
#define   TRUE               1   */ /* In C all non-zero is true */
/*
#define   FALSE              0   */ /* Only zero is false */
#define   R_MODE           "r"    /* read only mode */
#define   RU_MODE         "r+"    /* read update mode (our write mode) */
#define   WU_MODE         "w+"    /* write update mode. (open new file) */
#define   AU_MODE         "a+"    /* append update mode */
#define   PROTMODE        0755    /* R-W-E for owner, R-E for others */
#define   NULL_OFF          -1
#define   SITES_PER_RECORD   100
#define   CLASSES_PER_RECORD 100

/*
   The following are codes that the local routines return.
   These are NOT codes that the user callables return.
*/
#define   NO_ERR             0   /* no error */
#define   ERR_CLASS          1   /* invalid class */
#define   ERR_SITE           2   /* invalid site */
#define   ERR_DATA           3   /* invalid data */
#define   NEW_CLASS          4   /* new class created */
#define   NEW_SITE           5   /* new site created */
#define   NEW_DATA           6   /* new data entered */
#define   OLD_DATA           7   /* old data replaced */
#define   ERR_IO            10   /* I/O error */


/*--------------------------- typedefs -------------------------------------*/

/* NOTE: Each of these structures must be REC_SIZE bytes in size.  It is
         assumed to be the case.  Many of the compiler warnings generated
         by this module are due to interchanging the record types at will
         without proper regard to the types.  Unfortunately, it is beyond 
         the scope to fix right now */

typedef struct first_im_rec {     /* The first record (image level) */
   char            data_type[28];       /* the data type */
   long            cont_off;            /* offset of continued record */
   long            next_off;            /* offset of next rec at this level */
   SHAPE_VEC       shape;               /* the shape vector */
   unsigned long   num_class;          	/* the number of classes */
   long            class_off[CLASSES_PER_RECORD];   /* offsets to all classes */
   long            del_off;             /* offset of first deleted rec. */
   char            empty[32];
   } FIRST_IM_REC;

typedef struct first_cl_rec {     /* The first class level record */
   char            data_type[28];
   long            cont_off;
   long            next_off;
   SHAPE_VEC       shape;
   unsigned long   num_site;            /* the number of sites */
   long            site_off[SITES_PER_RECORD];  /* offsets to all sites */
   char            empty[36];
   } FIRST_CL_REC;

typedef struct data_record {     /* One data record */
   char            data_type[28];
   long            cont_off;
   long            next_off;
   SHAPE_VEC       shape;
   unsigned char   rec_data[NUM_DATA_BYTES];
   } FIRST_SI_REC, DATA_REC;

typedef struct generic_rec {
   unsigned char   buf[REC_SIZE];
   } GENERIC_REC;


/*--------------------------- externals ------------------------------------*/

/*
   These are external to all the Stat I/O software.
*/

static  FILE                 *curr_file_ptr = NULL;

/*
   The following externals are used by stnode.  stclse must reset
   last_ptr when the file that was last accessed by stnode is closed.
   last_ptr is also reset if the file is site is altered by any other
   functions (stput, stdldt, stdlnd).
*/

static  long   last_class = -1, last_site = -1, last_data= -1;
static	FILE   *last_ptr = NULL;
static  long   next_off = NULL_OFF;



/*----------------------------------------------------------------------*/
/*------------------------ Local Prototypes ----------------------------*/
/*----------------------------------------------------------------------*/

void copy_rec
(
    GENERIC_REC *rec_ptr1,              /* output record                      */
    GENERIC_REC *rec_ptr2               /* input record                       */
);

void init_class_list
(
    FIRST_IM_REC *rec_ptr               /* class-list record                  */
);

void init_site_list
(
    FIRST_CL_REC *rec_ptr               /* site-list record                   */
);

void init_data_rec
(
    GENERIC_REC *rec_ptr                /* a data record                      */
);

int read_rec
(
    long offset,                        /* offset of record to read           */
    GENERIC_REC *rec_ptr                /* pointer to the record read         */
);

int write_rec
(
    long offset,                        /* offset to write record to          */
    GENERIC_REC *rec_ptr                /* pointer to record to be written    */
);

long get_empty_off
(
    long *off,                          /* offset to an empty record          */
    long *del_off                       /* new offset to deleted record list  */
);

long get_class_off
(
    long ic,                            /* class index                        */
    long *icoff                         /* class offset                       */
);

long get_site_off
(
    long ic,                            /* class index                        */
    long is,                            /* site index                         */
    long *isoff                         /* site offset                        */
);

long get_shape_buff
(
    DATA_REC *rec,              /* pointer to record structure                */
    SHAPE_VEC shape,            /* shape vector read in                       */
    unsigned char *buf          /* the actual data read in                    */
);

long search_list
(
    long roff,                  /* record offset of first record      */
    char *data_type,            /* data type to find                  */
    DATA_REC *rec_found,                /* record found matching data type    */
    long *off_found             /* offset of the record in the file   */
);

long get_name
(
    long offset,                /* offset of first record in list     */
    char name[]                 /* the name found                     */
);

long get_first_image_rec
(
    FIRST_IM_REC *rec_ptr       /* pointer to first record structure  */
);

long get_first_class_rec
(
    long ic,                    /* class index                        */
    FIRST_CL_REC *rec_ptr       /* pointer to record structure read in*/
);

long get_first_site_rec
(
    long ic,                    /* class index                        */
    long is,                    /* site index                         */
    FIRST_SI_REC *rec_ptr       /* pointer to first site record struct*/
);

long get_class_off_list
(
    long *numc,                 /* number of classes in file                  */
    long *clist,                /* list of class offsets                      */
    long *delptr                /* offset of first deleted record             */
);

long get_site_off_list
(
    long ic,                    /* class index                        */
    long *nums,                 /* number of sites                    */
    long *slist                 /* list of site offsets in file       */
);

long get_site_list
(
    FIRST_CL_REC *rec1,         /* first site list record             */
    long *nums,                 /* number of sites                    */
    long *slist                 /* list of site offsets in file       */
);

long get_image_level
(
    char *data_type,            /* data type to search for            */
    SHAPE_VEC shape,            /* shape of data read in              */
    unsigned char *buf          /* buffer to read data into           */
);

long get_class_level
(
    long ic,                    /* class index                        */
    char *data_type,            /* data type to search for            */
    SHAPE_VEC shape,            /* shape of data read in              */
    unsigned char *buf          /* buffer to read data into           */
);

long get_site_level
(
    long ic,                    /* class index                        */
    long is,                    /* site index                         */
    char *data_type,            /* data type to search for            */
    SHAPE_VEC shape,            /* shape of data read in              */
    unsigned char *buf          /* buffer to read data into           */
);

long calc_num_bytes
(
    SHAPE_VEC shape,            /* shape vector                       */
    long *num_bytes             /* number of bytes                    */
);

long rewrite_data
(
    long offset,                /* offset of record to be replaced    */
    SHAPE_VEC shape,            /* shape of new data                  */
    unsigned char *buf          /* data to be written to file         */
);

long write_new_data
(
    DATA_REC *rec_prev,         /* the previous record                */
    long off_prev,              /* offset of previous record          */
    char *data_type,            /* data type of data to be written    */
    SHAPE_VEC shape,            /* shape of data to be written        */
    unsigned char *buf          /* data to be written                 */
);

long add_data
(
    long offset,                /* first record of list               */
    char *data_type,            /* data type of data to be written    */
    SHAPE_VEC shape,            /* the shape of the data to be written*/
    unsigned char *buf          /* the data to be written             */
);

long create_class
(
    FIRST_IM_REC *rec1,         /* first record in file               */
    FIRST_CL_REC *new_rec,      /* new site list record               */
    long *new_off               /* offset of site list record         */
);

long new_class
(
    FIRST_IM_REC *rec1,         /* first record in file               */
    char *data_type,            /* data type of data to be written    */
    SHAPE_VEC shape,            /* shape of data to be written        */
    unsigned char *buf,         /* the data to be written             */
    FIRST_CL_REC *new_rec,      /* site list record                   */
    long *new_off               /* offset of site list record         */
);

long new_site
(
    FIRST_CL_REC *rec1,         /* site list record                   */
    long rec1_off,              /* offset of site list record         */
    char *data_type,            /* data type of data to be written    */
    SHAPE_VEC shape,            /* shape of data to be written        */
    unsigned char *buf          /* data to be written                 */
);

long put_image_level
(
    char *data_type,            /* data type of data to be written    */
    SHAPE_VEC shape,            /* shape of data to be written        */
    unsigned char *buf          /* data to be written                 */
);

long put_class_level
(
    long ic,                    /* class index                        */
    char *data_type,            /* data type of data to be written    */
    SHAPE_VEC shape,            /* shape of data to be written        */
    unsigned char *buf          /* data to be written                 */
);

long put_site_level
(
    long ic,                    /* class index                        */
    long is,                    /* site index                         */
    char *data_type,            /* data type of data to be written    */
    SHAPE_VEC shape,            /* shape of data to be written        */
    unsigned char *buf          /* data to be written                 */
);

long delete_class2
(
    long ic                     /* class index                        */
);

long delete_site2
(
    long ic,                    /* class index                        */
    long is                     /* site index                         */
);

long delete_linked_list
(
    long offset                 /* first record of list to delete     */
);

long delete_using_cont
(
    long offset,                /* offset of first record in list     */
    long *del_off               /* offset of first deleted record     */
);

long delete_data
(
    long roff,                  /* offset of first record of list     */
    char *data_type,            /* data type of record to delete      */
    long *new_off               /* new offset of first record of list */
);

long class_offset_from_index
(
    FIRST_IM_REC *rec,          /* first record of file               */
    long ic                     /* class index in array               */
);

long site_offset_from_index
(
    FIRST_CL_REC *rec,          /* first record of class              */
    long is                     /* site index in array                */
);

int write_class_list
(
    FIRST_IM_REC *rec,          /* first record of class list         */
    long offset,                /* offset of above record             */
    long clist[],               /* the list of class offsets          */
    long numc                   /* number of classes                  */
);

int write_site_list
(
    FIRST_CL_REC *rec,          /* first record of site list          */
    long offset,                /* offset of above record             */
    long slist[],               /* the list of site offsets           */
    long nums                   /* number of sites                    */
);

void upcase2
(
    char *c1,                   /* output string                      */
    char *c2                    /* input string                       */
);

/*----------------------------------------------------------------------*/
/*------------------------ User Callables ------------------------------*/
/*----------------------------------------------------------------------*/

/*--------------------------- c_stopen ---------------------------------*/

long c_stopen 
(
    FILE **fd,          /* file descriptor for stats file     */
    char hos[],         /* host name of stats file            */
    long *access,       /* type of open access (read, write,..*/
    long *nclass        /* number of classes in file          */
)
/******************************************************************************
NAME            stopen

PURPOSE         Open a stats file.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion
   B. Ailts/EDC	      07-Mar-89    Fixed vms problem of creating new versions
				   when file was opened for read/write

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *fd		   FILE		 O	 file descriptor
   *hos            char		 I       stat file host name
   *access         long		 I       type of access to open file with
   *nclass         long          O       number of classes in file

RETURN VALUES

   value        Meaning
   -----        -------
     0          old file successfully opened
     1          new file successfully created - no old file
     2          general I/O error - unable to open or create

ALGORITHM (PDF)

   open file with desired access type
   if doesn't work then return
   if old file opened then
      read first record for nclass
   else
      write first record (CLASS-LIST)
   endif
   return
   
*/
{
short           newfile;		/* indicates file is new              */
long            ir;			/* return status                      */
FIRST_IM_REC    rec;			/* first record in file               */
#ifdef vms
char vers[] = ";1";
#endif

/* first open the file according to the access specified */

switch (*access)
   {
   case IUPDATE:
      /*
         open old file for read and write
      */
#ifdef vms
      if (strchr(hos,';') == NULL)
         strcat(hos,vers);
#endif
      if ((*fd = fopen (hos, RU_MODE)) == NULL)
	 return (E_GNLIO);
      newfile = FALSE;
      break;
   case IREAD:
      /*
         open old file for read only access
      */
      if ((*fd = fopen (hos, R_MODE)) == NULL)
	 return (E_GNLIO);
      newfile = FALSE;
      break;
   case IWRITE:
      /*
         open new file with read and write access
      */
      if ((*fd = fopen (hos, WU_MODE)) == NULL)
	 return (E_GNLIO);
      newfile = TRUE;
      break;
   case IUNKNOWN:
      /*
         try to open old file for read and write
      */
      if ((*fd = fopen (hos, RU_MODE)) == NULL)
	 {
#ifdef vms
      if (strchr(hos,';') == NULL)
         strcat(hos,vers);
#endif
	 /* old file open didn't work. Try opening new file. */
	 if ((*fd = fopen (hos, WU_MODE)) == NULL)
	    return (E_GNLIO);
	 else
	    newfile = TRUE;
	 }
      else
	 newfile = FALSE;
      break;
   default:
      return (E_GNLIO);
   }

curr_file_ptr = *fd;

if (!newfile)
   {
   /*
      File already exists. Get number of classes
   */
   if (get_first_image_rec (&rec) == ERR_IO)
      return (E_GNLIO);
   *nclass = rec.num_class;
   ir = E_OPOLD;
   }
else
   {
   /*
      New file. Initialize a class-list record
   */
   init_class_list (&rec);
   /* Write the class-list record */
   if (write_rec ((long) 0, (GENERIC_REC *) &rec) == ERR_IO)
      return (E_GNLIO);
   *nclass = 0;
   ir = E_OPNEW;
   }
return (ir);
}


/*---------------------------- c_stclse --------------------------------*/

int c_stclse 
(
    FILE **fd       /* file descriptor for stats file     */
)
/*
NAME            stclse

PURPOSE         Close a stats file.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *fd		   FILE		 O	 file descriptor

ALGORITHM (PDF)

   close the file
   return

*/
{
if (fclose (*fd) == EOF)
   return (E_FAIL);
last_ptr = (last_ptr == *fd) ? NULL : last_ptr;
curr_file_ptr = NULL;
return (E_SUCC);
}


/*---------------------------- c_stget ---------------------------------*/

int c_stget 
(
    FILE **fd,          /* file descriptor for stats file     */
    char data_type[],   /* name of data to be read            */
    long *ic,           /* class index of node to read from   */
    long *is,           /* site index of node to read from    */
    SHAPE_VEC shape,    /* shape of data read in              */
    unsigned char *buf  /* the actual data read in            */
)
/*
NAME            stget

PURPOSE         Read a specified data item associated with a given node.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *fd		   FILE		 O	 file descriptor
   *data_type      char		 I       the data type to read
   *ic             long          I       the class to be read from
   *is             long          I       the site to be read from
   shape           SHAPE_VEC     O       the shape vector for the data
   *buf            unsigned char O       the buffer to read data into

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer

RETURN VALUES

   value        Meaning
   -----        -------
     0          successful read
     1          class *ic not found  - no action
     2          site *is not found   - no action
     3          data-type not found  - no action
     4          general I/O error    - no action

ALGORITHM (PDF)

   if image level data requested then
      if NCL requested then
         get num. of classes from first rec.
      elseif CLASS-LIST requested then
         return error
      else
         get image level data
      endif
   else if class level data requested then
      if NSI requested then
         get num. of sites from first rec. of class
      elseif SITE-LIST requested then
         return error
      else
         get class level data
      endif
   else
      get site level data
   endif
   return

*/
{
long    ierr,			/* status from local routines         */
		tlen;			/* temp len for calling low2up        */
int ir = E_FAIL;                 /* return status                      */
FIRST_IM_REC    rec_im;			/* class-list record                  */
FIRST_CL_REC    rec_cl;			/* site-list record                   */

tlen = (long)strlen (data_type);
c_low2up (data_type, &tlen);

curr_file_ptr = *fd;

if ((*ic < 0) || (*ic > MAXCLS))
   ierr = ERR_CLASS;
else if (*ic == 0)
   {
   if (*is == 0)
      {
      /*
         Image level data requested
      */
      if (strcmp (data_type, "NCLASSES") == 0)
	 {
	 ierr = get_first_image_rec (&rec_im);
	 *((int *) buf) = rec_im.num_class;
         shape[0] = ELONG;
	 shape[1] = 1;
	 shape[2] = 1;
	 }
      else if (strcmp (data_type, "CLASS-LIST") == 0)
	 ierr = ERR_DATA;
      else
	 ierr = get_image_level (data_type, shape, buf);
      }
   else
      ierr = ERR_SITE;
   }
else if ((*is < 0) || (*is > MAXSIT))
   ierr = ERR_SITE;
else if (*is == 0)
   {
   /*
      Class level data requested
   */
   if (strcmp (data_type, "NSITES") == 0)
      {
      ierr = get_first_class_rec (*ic, &rec_cl);
      *((int *) buf) = rec_cl.num_site;
      shape[0] = ELONG;
      shape[1] = 1;
      shape[2] = 1;
      }
   else if (strcmp (data_type, "SITE-LIST") == 0)
      ierr = ERR_DATA;
   else
      ierr = get_class_level (*ic, data_type, shape, buf);
   }
else
   /*
      Site level data requested
   */
   ierr = get_site_level  (*ic, *is, data_type, shape, buf);

switch (ierr)
   {
   case NO_ERR:
      ir = E_SUCC;
      break;
   case ERR_CLASS:
      ir = E_NOCL;
      break;
   case ERR_SITE:
      ir = E_NOSI;
      break;
   case ERR_DATA:
      ir = E_NODT;
      break;
   case ERR_IO:
      ir = E_GNLIO;
      break;
   }
return (ir);
}


/*---------------------------- c_stput ---------------------------------*/

int c_stput 
(
    FILE **fd,          /* file descriptor for stats file     */
    char data_type[],   /* name of data to write              */
    long *ic,           /* class index to write data to       */
    long *is,           /* site index to write data to        */
    SHAPE_VEC shape,    /* shape of data to be written        */
    unsigned char *buf  /* the actual data to be written      */
)
/*
NAME            stput

PURPOSE         Write a specified data item into a given node.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *fd		   FILE		 O	 file descriptor
   *data_type      char		 I       the data type to write
   *ic             long          I       the class to be written to
   *is             long          I       the site to be written to
   shape           SHAPE_VEC     O       the shape vector for the data
   *buf            unsigned char O       the buffer to write data from

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer
   last_class      long          I       last class stnode read from
   last_site       long          I       last site stnode read from
   last_ptr        long         I/O      ptr of last file stnode read from

RETURN VALUES

   value        Meaning
   -----        -------
     0          old data replaced
     1          new data entered
     2          no class *ic - node (ic, is) created and data entered
     3          no site *is  - node (ic, is) created and data entered
     4          class *ic not found.  *ic > num_class + 1.  no action
     5          site *is not found.   *is > num_site + 1.   no action
     6          error in data_type.  - no action
     7          general I/O error    - no action

ALGORITHM (PDF)

   if image level data given then
      if CLASS-LIST given then
         return error
      else
         put image level data
      endif
   else if class level data given then
      if SITE-LIST given then
         return error
      else
         put class level data
      endif
   else
      put site level data
   endif
   return

*/
{
long	ierr,				/* status from local routines         */
	tlen;				/* temp len for calling low2up        */
int ir = E_FAIL;        /* return status                      */

tlen = (long)strlen (data_type);
c_low2up (data_type, &tlen);

if ((*ic == last_class) && (*is == last_site) && (last_ptr == *fd))
   {
   last_ptr == NULL;
   }

curr_file_ptr = *fd;
if (strcmp (data_type, "NAME") == 0)
   {
   c_low2up (&buf[0], &shape[2]);
   if (shape[2] > STNAME)
      return (E_INVNM);
   }

if ((*ic < 0) || (*ic > MAXCLS))
   ierr = ERR_CLASS;
else if (*ic == 0)
   {
   if (*is == 0)
      {
      /*
         Image level data given
      */
      if (strcmp (data_type, "CLASS-LIST") == 0)
	 ierr = ERR_DATA;
      else
	 ierr = put_image_level (data_type, shape, buf);
      }
   else
      ierr = ERR_SITE;
   }
else if ((*is < 0) || (*is > MAXSIT))
   ierr = ERR_SITE;
else if (*is == 0)
   {
   /*
      Class level data given
   */
   if (strcmp (data_type, "SITE-LIST") == 0)
      ierr = ERR_DATA;
   else
      ierr = put_class_level (*ic, data_type, shape, buf);
   }
else
   /*
      Site level data given
   */
   ierr = put_site_level (*ic, *is, data_type, shape, buf);

switch (ierr)
   {
   case OLD_DATA:
      ir = E_DTOLD;
      break;
   case NEW_DATA:
      ir = E_DTNEW;
      break;
   case NEW_CLASS:
      ir = E_NOCL;
      break;
   case NEW_SITE:
      ir = E_NOSI;
      break;
   case ERR_CLASS:
      ir = E_INVCL;
      break;
   case ERR_SITE:
      ir = E_INVSI;
      break;
   case ERR_DATA:
      ir = E_INVDT;
      break;
   case ERR_IO:
      ir = E_GNLIO;
      break;
   }
return (ir);
}


/*---------------------------- c_stfdcl --------------------------------*/

int c_stfdcl 
(
    FILE **fd,          /* file descriptor for stats file     */
    char class_name[],  /* class name to find                 */
    long *ic            /* class index of class               */
)
/*
NAME            stfdcl

PURPOSE         Locate the class index given the class name.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *fd		   FILE		 O	 file descriptor
   *class_name     char		 I       the class name to search for
   *ic             long          O       the class index

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer

RETURN VALUES

   value        Meaning
   -----        -------
     0          successful search
     1          class name not found  - no action
     2          general I/O error     - no action

ALGORITHM (PDF)

   get list of class offsets
   for each offset
      try to get the name of the class
      if name exists then
         if name matches requested name then
            return with class index
         endif
      endif
   endfor
   return

*/
{
long	clist[MAXCLS],			/* list of class offsets in file      */
	delptr,				/* offset of first deleted record     */
	i,				/* loop variable                      */
	ierr,				/* status from local routines         */
	numc,				/* number of classes                  */
	tlen;				/* temp len for calling low2up        */
int ir = E_FAIL;		/* return status                      */
char	cname[NUM_DATA_BYTES];		/* a class name                       */

tlen = (long)strlen (class_name);
c_low2up (class_name, &tlen);
if (tlen > STNAME)
   return (E_INVNM);

curr_file_ptr = *fd;

/*
   get the class list
*/
if ((ierr = get_class_off_list (&numc, clist, &delptr)) == NO_ERR)
   {
   for (i = 0; i < numc; i++)
      {
      /*
         for each class do
         get the name of the class if name exists
      */
      if ((ierr = get_name (clist[i], cname)) == NO_ERR)
	 if (strcmp (cname, class_name) == 0)
	    {
	    /*
               name matches requested name
            */
	    *ic = i + 1;
	    return (E_SUCC);
	    }
      }
   /*
      class name not found
   */
   ierr = ERR_DATA;
   }

switch (ierr)
   {
   case NO_ERR:
      ir = E_SUCC;
      break;
   case ERR_DATA:
      ir = E_NOCLN;
      break;
   case ERR_IO:
      ir = E_GNLIO;
      break;
   }
return (ir);
}


/*---------------------------- c_stfdsi --------------------------------*/

int c_stfdsi 
(
    FILE **fd,          /* file descriptor for stats file     */
    char class_name[],  /* class name to be searched          */
    char site_name[],   /* site name to be searched           */
    long *ic,           /* class index found                  */
    long *is            /* site index found                   */
)
/*
NAME            stfdsi

PURPOSE         Locate the class and site index given the class name and
                site name.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *fd		   FILE		 O	 file descriptor
   *class_name     char		 I       the class name to search for
   *site_name      char		 I       the site name to search for
   *ic             long          O       the class index
   *is             long          O       the site index

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer

RETURN VALUES

   value        Meaning
   -----        -------
     0          successful search
     1          class name not found  - no action
     3          site name not found   - no action
     4          general I/O error     - no action

ALGORITHM (PDF)

   get the index of the class
   get the site offset list for the class
   for each offset
      try to get the name of the site
      if name exists then
         if name matches requested name then
            return with class and site indices
         endif
      endif
   endfor
   return

*/
{
long	i,				/* loop variable                      */
	ierr,				/* status from local routines         */
	nums,				/* number of sites                    */
	slist[MAXSIT],			/* list of site offsets in file       */
	tlen;				/* temp len for calling low2up        */
int ir; 				/* return status                      */
char	sname[NUM_DATA_BYTES];		/* a site name                        */

tlen = (long)strlen (class_name);
c_low2up (class_name, &tlen);
if (tlen > STNAME)
   return (E_INVNM);
tlen = (long)strlen (site_name);
c_low2up (site_name, &tlen);
if (tlen > STNAME)
   return (E_INVNM);

curr_file_ptr = *fd;

/*
   first get the class index
*/
if ((ir = c_stfdcl (fd, class_name, ic)) == 0)
   {
   /*
      get the site list
   */
   if ((ierr = get_site_off_list (*ic, &nums, slist)) == NO_ERR)
      {
      for (i = 0; i < nums; i++)
	 {
	 /*
            for each site do
	    get the name of the site if the name exists
         */
	 if ((ierr = get_name (slist[i], sname)) == NO_ERR)
            {
	    if (strcmp (sname, site_name) == 0)
	       {
	       /*
                  name matches requested name
               */
	       *is = i + 1;
	       return (E_SUCC);
	       }
	    }
	 }
      /*
         site name not found
      */
      ierr = ERR_DATA;
      }

   switch (ierr)
      {
      case NO_ERR:
	 ir = E_SUCC;
	 break;
      case ERR_CLASS:
	 ir = E_NOCLN;
	 break;
      case ERR_DATA:
	 ir = E_NOSIN;
	 break;
      case ERR_IO:
	 ir = E_GNLIO;
	 break;
      }
   return (ir);
   }
else
   return (ir);
}


/*---------------------------- c_stdldt --------------------------------*/

int c_stdldt 
(
    FILE **fd,          /* file descriptor for stats file     */
    char data_type[],   /* name of data to be deleted         */
    long *ic,           /* class index                        */
    long *is            /* site index                         */
)
/*
NAME            stdldt

PURPOSE         Delete a specified data item from a given node.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *fd		   FILE		 O	 file descriptor
   *data_type      char		 I       the data type to be deleted
   *ic             long          I       the class which the data belongs to
   *is             long          I       the site which the data belongs to

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer
   last_class      long          I       last class stnode read from
   last_site       long          I       last site stnode read from
   last_ptr        long         I/O      ptr of last file stnode read from

RETURN VALUES

   value        Meaning
   -----        -------
     0          successful delete
     1          class *ic not found  - no action
     2          site *is not found   - no action
     3          data-type not found  - no action
     4          general I/O error    - no action

ALGORITHM (PDF)

   if image level data to be deleted then
      if CLASS-LIST then
         return error
      else
         delete image level data
      endif
   elseif class level data to be deleted then
      if SITE-LIST then
         return error
      else
         delete class level data
      endif
   else
      delete site level data
      if site now has no records then
         remove site from site list
      elseif site now has new first record then
         update site list to point to new record
      endif
   endif
   return

*/
{
long            i,			/* loop variable                      */
		icoff,			/* class offset in file               */
		ierr,			/* status from local routines         */
		new_off,		/* new offset of next data item       */
		nums,			/* number of sites in class           */
		offset,			/* an offset into the file            */
		slist[MAXSIT],		/* list of site offsets               */
		tlen;			/* temp len for calling low2up        */
int ir = E_FAIL;		/* return status                      */

FIRST_CL_REC    cl_rec;			/* site-list record                   */

tlen = (long)strlen (data_type);
c_low2up (data_type, &tlen);

curr_file_ptr = *fd;

if ((*ic == last_class) && (*is == last_site) && (last_ptr == *fd))
   {
   last_ptr == NULL;
   }

if ((*ic < 0) || (*ic > MAXCLS))
   ierr = ERR_CLASS;
else if (*ic == 0)
   {
   if (*is == 0)
      {
      /*
         Image level data to be deleted
      */
      if (strcmp (data_type, "CLASS-LIST") == 0)
	 ierr = ERR_DATA;
      else
         /*
            delete at image level
         */
	 ierr = delete_data (0, data_type, &new_off);
      }
   else
      ierr = ERR_SITE;
   }
else if ((*is < 0) || (*is > MAXSIT))
   ierr = ERR_SITE;
else if (*is == 0)
   {
   /*
      delete at class level
   */
   if (strcmp (data_type, "SITE-LIST") == 0)
      ierr = ERR_DATA;
   else
      if ((ierr = get_class_off (*ic, &offset)) == NO_ERR)
	 ierr = delete_data (offset, data_type, &new_off);
   }
else
   {
   /*
      delete at site level
   */
   if ((ierr = get_site_off (*ic, *is, &offset)) == NO_ERR)
      {
      if ((ierr = delete_data (offset, data_type, &new_off)) == NO_ERR)
	 {
	 if (new_off == NULL_OFF)
	    {
	    /*
               the site now has no records.
	       update site-list to remove the site
            */
	    if ((ierr = get_class_off (*ic, &icoff)) == NO_ERR)
	       if ((ierr = read_rec (icoff, &cl_rec)) == NO_ERR)
		  {
                  if ((ierr = get_site_list (&cl_rec, &nums, slist)) == NO_ERR)
                     {
		     cl_rec.num_site--;
		     nums = cl_rec.num_site;
		     if (cl_rec.num_site != 0)
                        {
			for (i = *is - 1; i < cl_rec.num_site; i++)
			   slist[i] = slist[i + 1];
/*
			   cl_rec.site_off[i] = cl_rec.site_off[i + 1];
*/
                        }
		     else
		        slist[0] = NULL_OFF;
                     ierr = write_site_list (&cl_rec, icoff, slist, nums);
/*
		     ierr = write_rec (icoff, &cl_rec);
*/
                     }
		  }
	    }
	 else if (new_off != offset)
	    {
	    /*
               site has new first record. update site-list
            */
	    if ((ierr = get_class_off (*ic, &icoff)) == NO_ERR)
	       if ((ierr = read_rec (icoff, &cl_rec)) == NO_ERR)
                  if ((ierr = get_site_list (&cl_rec, &nums, slist)) == NO_ERR)
		  {
		  slist[*is - 1] = new_off;
                  ierr = write_site_list (&cl_rec, icoff, slist, nums);
/*
		  ierr = write_rec (icoff, &cl_rec);
*/
		  }
	    }
	 }
      }
   }

switch (ierr)
   {
   case NO_ERR:
      ir = E_SUCC;
      break;
   case ERR_CLASS:
      ir = E_NOCL;
      break;
   case ERR_SITE:
      ir = E_NOSI;
      break;
   case ERR_DATA:
      ir = E_NODT;
      break;
   case ERR_IO:
      ir = E_GNLIO;
      break;
   }
return (ir);
}


/*---------------------------- c_stdlnd --------------------------------*/

int c_stdlnd 
(
    FILE **fd,      /* file descriptor for stats file     */
    long *ic,       /* class index                        */
    long *is        /* site index                         */
)
/*
NAME            stdlnd

PURPOSE         Delete a specified node.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *fd		   FILE		 O	 file descriptor
   *ic             long          I       the class index of node to delete
   *is             long          I       the site index of node to delete

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer
   last_class      long          I       last class stnode read from
   last_site       long          I       last site stnode read from
   last_ptr        long         I/O      ptr of last file stnode read from

RETURN VALUES

   value        Meaning
   -----        -------
     0          successful delete of node (*ic, *is)
     1          class *ic not found   - no action
     2          site *is not found    - no action
     3          general I/O error     - no action

ALGORITHM (PDF)

   if class to be deleted then
      delete class
   elseif site to be deleted then
      delete site
   endif
   return

*/
{
long    ierr;		/* status from local routines         */
int     ir = E_FAIL;/* return status                      */

curr_file_ptr = *fd;

if ((*ic == last_class) && (*is == last_site) && (last_ptr == *fd))
   {
   last_ptr == NULL;
   }

if (*ic <= 0)
   ierr = ERR_CLASS;
else if (*is < 0)
   ierr = ERR_SITE;
else if (*is == 0)
   ierr = delete_class2 (*ic);
else
   ierr = delete_site2 (*ic, *is);

switch (ierr)
   {
   case NO_ERR:
      ir = E_SUCC;
      break;
   case ERR_CLASS:
      ir = E_NOCL;
      break;
   case ERR_SITE:
      ir = E_NOSI;
      break;
   case ERR_IO:
      ir = E_GNLIO;
      break;
   }
return (ir);
}


/*---------------------------- c_stnode --------------------------------*/

int c_stnode 
(
    FILE **fd,          /* file descriptor for stats file     */
    long *ic,           /* class index                        */
    long *is,           /* site index                         */
    long *id,           /* data item index within node        */
    char data_type[],   /* name of the next data              */
    SHAPE_VEC shape     /* shape of data                      */
)
/*
NAME            stnode

PURPOSE         Return the data_type name and shape vector at a specified
                node given the index of the data_type.  This function is
                called repetitively to get all the data_types at a given
                node so we wil keep an offset to the next record in static
                space.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *fd		   FILE		 O	 file descriptor
   *ic             long          I       the class index of the node
   *is             long          I       the site index of the node
   *id             long          I       the index of the data item
   *data_type      char		 O       the data type of the data found
   shape           SHAPE_VEC     O       the shape vector of data found

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer
   last_class      long         I/O      last class stnode read from
   last_site       long         I/O      last site stnode read from
   last_data       long         I/O      last data item index stnode read
   last_ptr        long         I/O      ptr of last file stnode read from
   next_off        long         I/O      offset of next record to read by stnode

RETURN VALUES

   value        Meaning
   -----        -------
     0          successful read
     1          no more data items in node
     2          class *ic not found   - no action
     3          site *is not found    - no action
     4          general I/O error     - no action

NOTE

   This routine accesses some global static variables.  They are used to keep
   track of which file and node the previuos call to stnode accessed.  One
   also keeps the offset to the next record at that node.  If the next call
   to stnode is the same file and node and the next consecutive data item,
   then that saved offset is used to reduce the number of reads required.

ALGORITHM (PDF)

   if next item requested then
      do nothing
   elseif some other data requested then
      get record for beginning of node
      step through node the number of records indicated by data index
      update the global variables
   endif
   read the record
   copy the shape and data type
   return

*/
{
long            ierr,			/* status from local routines         */
		i;			/* loop variable                      */
int		ir = E_FAIL; /* return status                      */

DATA_REC        rec;			/* a data record from the file        */

curr_file_ptr = *fd;

if ((*ic == last_class) && (*is == last_site) && (*id == last_data + 1)
   && (last_ptr == *fd))
   {
   /*
      this is a request for the next consecutive record in the node
      we already have the offset of the record requested from
      previous call
   */
   ierr = NO_ERR;
   }
else
   {
   /*
      this is not a request for the next consecutive record in the node
      we must get the offset of the record requested

      first go to the beginning of the node
   */
   ierr = NO_ERR;
   if ((*ic < 0) || (*ic > MAXCLS))
      ierr = ERR_CLASS;
   else if (*ic == 0)
      {
      if (*is == 0)
	 {
	 ierr = get_first_image_rec (&rec);
	 next_off = rec.next_off;
	 }
      else
	 ierr = ERR_SITE;
      }
   else if ((*is < 0) || (*is > MAXSIT))
      ierr = ERR_SITE;
   else if (*is == 0)
      {
      ierr = get_first_class_rec (*ic, &rec);
      next_off = rec.next_off;
      }
   else
      ierr = get_site_off (*ic, *is, &next_off);
   /*
      now go forward the number of items indicated by id
   */
   if (ierr == NO_ERR)
      {
      i = 1;
      while ((i < *id) && (next_off != NULL_OFF))
	 {
	 ierr = read_rec (next_off, &rec);
	 next_off = rec.next_off;
	 i++;
	 }
      }
   /*
      set the global variables
   */
   last_class = *ic;
   last_site  = *is;
   last_ptr   = *fd;
   }

/*
   now that we have the offset do the read
*/
if (ierr == NO_ERR)
   {
   if (next_off != NULL_OFF)
      {
      ierr = read_rec (next_off, &rec);
      /*
         copy shape and data_type
      */
      for (i = 0; i < 9; i++)
	 shape[i] = rec.shape[i];

      strcpy (data_type, rec.data_type);
      /*
         update last_data and next_off
      */
      last_data = *id;
      next_off = rec.next_off;
      }
   else
      ierr = ERR_DATA;
   }

switch (ierr)
   {
   case NO_ERR:
      ir = E_SUCC;
      break;
   case ERR_DATA:
      ir = E_NODT;
      break;
   case ERR_CLASS:
      ir = E_NOCL;
      break;
   case ERR_SITE:
      ir = E_NOSI;
      break;
   case ERR_IO:
      ir = E_GNLIO;
      break;
   }
return (ir);
}


/*----------------------------------------------------------------------*/
/*------------------------- Local Routines -----------------------------*/
/*----------------------------------------------------------------------*/

/*---------------------------- copy_rec --------------------------------*/

void copy_rec
(
    GENERIC_REC *rec_ptr1,		/* output record                      */
    GENERIC_REC *rec_ptr2		/* input record                       */
)
/*
NAME            copy_rec

PURPOSE         Copies record pointed to by rec_ptr2 to rec_ptr1.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec_ptr1       GENERIC_REC   O       record to recieve data
   *rec_ptr2       GENERIC_REC   I       record which contains data

RETURN VALUES   none

*/
{
long            i;			/* loop variable                      */

for (i = 0; i < REC_SIZE; i++)
   rec_ptr1->buf[i] = rec_ptr2->buf[i];
}


/*---------------------------- init_class_list -------------------------*/

void init_class_list
(
    FIRST_IM_REC *rec_ptr		/* class-list record                  */
)
/*
NAME            init_class_list

PURPOSE         Initialize a record which will be the first record in a
                stats file. It will contain offsets to all classes
                (class-list record).

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec_ptr        FIRST_IM_REC  O       record to be initialized

RETURN VALUES   none

*/
{
long	i;				/* loop variable                      */

strcpy (rec_ptr->data_type, "CLASS-LIST");
for (i = 10; i < 28; i++)
   rec_ptr->data_type[i] = '\0';
rec_ptr->cont_off = NULL_OFF;
rec_ptr->next_off = NULL_OFF;
rec_ptr->shape[0] = ELONG;
rec_ptr->shape[1] = 1;
rec_ptr->shape[2] = MAXCLS;
for (i = 3; i < 9; i++)
   rec_ptr->shape[i] = 0;
rec_ptr->num_class = 0;
for (i = 0; i < CLASSES_PER_RECORD; i++)
   rec_ptr->class_off[i] = NULL_OFF;
rec_ptr->del_off = NULL_OFF;
for (i = 0; i < 32; i++)
   rec_ptr->empty[i] = '\0';
}


/*---------------------------- init_site_list --------------------------*/

void init_site_list
(
    FIRST_CL_REC *rec_ptr		/* site-list record                   */
)
/*
NAME            init_site_list

PURPOSE         Initialize a record which will be the first record for a
                class in a stats file.  It will contain offsets to all
                sites of a particular class (site-list record).

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec_ptr        FIRST_CL_REC  O       record to be initialized

RETURN VALUES   none

*/
{
long	i;				/* loop variable                      */

strcpy (rec_ptr->data_type, "SITE-LIST");
for (i = 9; i < 28; i++)
   rec_ptr->data_type[i] = '\0';
rec_ptr->cont_off = NULL_OFF;
rec_ptr->next_off = NULL_OFF;
rec_ptr->shape[0] = ELONG;
rec_ptr->shape[1] = 1;
rec_ptr->shape[2] = MAXSIT;
for (i = 3; i < 9; i++)
   rec_ptr->shape[i] = 0;
rec_ptr->num_site = 0;
for (i = 0; i < SITES_PER_RECORD; i++)
   rec_ptr->site_off[i] = NULL_OFF;
for (i = 0; i < 36; i++)
   rec_ptr->empty[i] = '\0';
}


/*---------------------------- init_data_rec ---------------------------*/

void init_data_rec
(
    GENERIC_REC *rec_ptr		/* a data record                      */
)
/*
NAME            init_data_rec

PURPOSE         Initialize a data record.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec_ptr        GENERIC_REC   O       record to be initialized

RETURN VALUES   none

*/
{
long	i;			/* loop variable                              */

for (i = 0; i < REC_SIZE; i++)
   rec_ptr->buf[i] = '\0';
}


/*---------------------------- read_rec --------------------------------*/

int read_rec
(
    long offset,			/* offset of record to read           */
    GENERIC_REC *rec_ptr		/* pointer to the record read         */
)
/*
NAME            read_rec

PURPOSE         Read a record given its offset and return a pointer to the
                record.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion
   B. Ailts	      06-May-91    Added c_sigblk and c_sigunblk in order
 				   that the interupts produced by the 
				   processing message does corrupt the I/O.

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   offset          long          I       the offset of the record to be read
   *rec_ptr        GENERIC_REC   O       the record to read into

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer

RETURN VALUES   NO_ERR, ERR_IO

ALGORITHM (PDF)

   move to offset specified
   read the record starting there

*/
{
int sig_mask;

c_sigblk(&sig_mask);
/* DEBUG  rewind (curr_file_ptr);*/
/* DEBUG  printf ("\n");*/
/* DEBUG  printf ("read_rec: fseek to position %d.\n", offset);*/

if (fseek (curr_file_ptr, offset, SEEK_SET) == 0)
   {
/* DEBUG  fpos = ftell (curr_file_ptr);*/
/* DEBUG  printf ("read_rec: Actual file position is %d.\n", fpos);*/
 
/* DEBUG  num_bytes = REC_SIZE;*/
   if (fread (rec_ptr, (long) REC_SIZE, 1, curr_file_ptr) == 1)
      {

/* DEBUG  printf ("read_rec: data_type is %s.\n", &rec_ptr[0]); */
/* DEBUG fpos = ftell (curr_file_ptr);*/ 
/* DEBUG printf ("read_rec: file position after write is %d.\n", fpos);*/ 
/* DEBUG printf ("\n");*/ 
 
      c_sigunblk(&sig_mask);
      return (NO_ERR);
      }
   else
      return (ERR_IO);
   }
else
   return (ERR_IO);
}


/*---------------------------- write_rec -------------------------------*/

int write_rec
(
    long offset,			/* offset to write record to          */
    GENERIC_REC *rec_ptr		/* pointer to record to be written    */
)
/*
NAME            write_rec

PURPOSE         Write a record given its offset and a pointer to the
                record.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion
   B. Ailts           06-May-91	   Added c_sigblk and c_sigunblk in order
 				   that the interupts produced by the 
				   processing message does corrupt the I/O.

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   offset          long          I       the offset of the record to be read
   *rec_ptr        GENERIC_REC   O       the record to be written out

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer

RETURN VALUES   NO_ERR, ERR_IO

ALGORITHM (PDF)

   move to offset specified
   write a record there

*/
{
int sig_mask;

c_sigblk(&sig_mask);
/* DEBUG rewind (curr_file_ptr);*/ 
/* DEBUG printf ("\n");*/ 
/* DEBUG printf ("write_rec: fseek to position %d.\n", offset);*/ 

if (fseek (curr_file_ptr, offset, SEEK_SET) == 0)
   {
/* DEBUG fpos = ftell (curr_file_ptr);*/ 
/* DEBUG printf ("write_rec: Actual file position is %d.\n", fpos);*/ 
/* DEBUG printf ("write_rec: data_type is %s.\n", &rec_ptr[0]); */ 
/* DEBUG num_bytes = REC_SIZE;*/ 
   if (fwrite (rec_ptr, (long) REC_SIZE, (long) 1, curr_file_ptr) == 1)
      {

/* DEBUG fpos = ftell (curr_file_ptr);*/ 
/* DEBUG printf ("write_rec: file position after write is %d.\n", fpos);*/ 
/* DEBUG printf ("\n");*/ 
 
      c_sigunblk(&sig_mask);
      return (NO_ERR);
      }
   else
      return (ERR_IO);
   }
else
   return (ERR_IO);
}


/*---------------------------- get_empty_off ---------------------------*/

long get_empty_off
(
    long *off,				/* offset to an empty record          */
    long *del_off			/* new offset to deleted record list  */
)
/*
NAME            get_empty_off

PURPOSE         Return either the offset of a record from the deleted
                record list or the offset of a new record at the end of
                file.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *off            long          O       the offset to an empty record
   *del_off        long          O       offset of first record in deleted list

EXTERNAL VARIABLE REFERENCES

   name            type         I/O      description
   ----            ----         ---      -----------
   curr_file_ptr   long          I       current file pointer

RETURN VALUES   NO_ERR, ERR_IO

ALGORITHM (PDF)

   read in first record
   if it doesn't point to a deleted record then
      position to end of file
      get the offset there
      write a blank record there
      return
   else there is a deleted record then
      get the offset from first record
      update first record
      return
   endif

*/
{
long		ierr,			/* return status                      */
		pos;			/* current position in file           */
FIRST_IM_REC	rec1;			/* class-list record                  */
DATA_REC	rec2;			/* site list record                   */

if ((ierr = get_first_image_rec (&rec1)) == NO_ERR)
   {
   *del_off = rec1.del_off;
   if (rec1.del_off == NULL_OFF)
      {
      /*
         there are no empty records in the file
         position to end of file and get offset
      */
/* DEBUG printf ("\n");*/ 
/* DEBUG printf ("get_empty_off: Go to bottom of file.\n");*/ 

/* DEBUG rewind (curr_file_ptr);*/ 
      if (fseek (curr_file_ptr, 0, SEEK_END) == 0)
	 {
	 if ((pos = ftell (curr_file_ptr)) == -1)
	    return (ERR_IO);
	 else
	    {
/* DEBUG printf ("get_empty_off: Says bottom of file is %d.\n", pos);*/ 
/* DEBUG printf ("\n");*/ 

	    *off = pos;
	    init_data_rec (&rec2);
	    if ((ierr = write_rec (pos, &rec2)) == NO_ERR)
	       return (NO_ERR);
	    else
	       return (ierr);
	    }
	 }
      else
	 return (ERR_IO);
      }
   else
      {
      /*
         get the offset of deleted record and update the offset in
	 the first record to make it undeleted
      */
      *off = rec1.del_off;
      if ((ierr = read_rec (*off, (GENERIC_REC *) &rec2)) != NO_ERR)
	 return (ierr);
      rec1.del_off = rec2.next_off;
      *del_off = rec1.del_off;
      return (write_rec ((long) 0, (GENERIC_REC *) &rec1));
      }
   }
else
   return (ierr);
}


/*---------------------------- get_class_off ---------------------------*/

long get_class_off
(
    long ic,				/* class index                        */
    long *icoff				/* class offset                       */
)
/*
NAME            get_class_off

PURPOSE         Get the offset for the first class level record for
                requested class. This is the site-list record for the
                class.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   *icoff          long          O       the offset to the class

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS

ALGORITHM (PDF)

   get first record
   get offset from table in the record
   return

*/
{
long		ierr;			/* return status                      */
FIRST_IM_REC	rec1;			/* class-list record                  */

/*
   read first record
*/
if ((ierr = get_first_image_rec (&rec1)) == NO_ERR)
   {
   /*
      return class offset for 'ic'
   */
   if ((ic > rec1.num_class) || (ic <= 0))
      {
      /*
         ic too high or too low
      */
      *icoff = NULL_OFF;
      return (ERR_CLASS);
      }
   else
      {
      *icoff = class_offset_from_index (&rec1, ic - 1);
/*
      *icoff = rec1.class_off[ic - 1];
*/
      return (NO_ERR);
      }
   }
else
   {
   *icoff = NULL_OFF;
   return (ierr);
   }
}


/*---------------------------- get_site_off ----------------------------*/

long get_site_off
(
    long ic,				/* class index                        */
    long is,				/* site index                         */
    long *isoff				/* site offset                        */
)
/*
NAME            get_site_off

PURPOSE         Get the offset for the first site level record for
                requested class and site.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   is              long          I       the site index
   *isoff          long          O       the offset to the site

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS, ERR_SITE

ALGORITHM (PDF)

   read the first record for the class
   get the offset from the table in the record
   return

*/
{
long ierr;			/* return status                      */
FIRST_CL_REC	rec_cl;			/* site-list record                   */

/*
   read class record using offset for ic
*/
if ((ierr = get_first_class_rec (ic, &rec_cl)) == NO_ERR)
   {
   /*
      return site offset for 'is'
   */
   if ((is > rec_cl.num_site) || (is <= 0))
      {
      /*
         is too high or too low
      */
      *isoff = NULL_OFF;
      return (ERR_SITE);
      }
   else
      {
      *isoff = site_offset_from_index (&rec_cl, is - 1);
/*
      *isoff = rec_cl.site_off[is - 1];
*/
      return (NO_ERR);
      }
   }
else
   {
   *isoff = NULL_OFF;
   return (ierr);
   }
}


/*---------------------------- get_shape_buff --------------------------*/

long get_shape_buff
(
    DATA_REC *rec,		/* pointer to record structure                */
    SHAPE_VEC shape,		/* shape vector read in                       */
    unsigned char *buf		/* the actual data read in                    */
)
/*
NAME            get_shape_buff

PURPOSE         Given a record pointer read in the shape and data buffer
                using the continuation offset.  The data may or may not be
                continued on other records.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion
   T. Sturman	      30-Nov-90    Fixed problem of copying Zero bytes 
				   if previous record contained the max 
				   number of bytes per record. 

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec            DATA_REC      I       the record to copy data from
   shape           SHAPE_VEC     O       the shape vector of the data
   *buf            unsigned char O       the data that is read in

RETURN VALUES   NO_ERR, ERR_IO

ALGORITHM (PDF)

   copy the shape from the record
   copy the data from the record
   while the data is continued on another record
      read the continuation record
      copy the data over
   endwhile
   return

*/
{
DATA_REC	rec2;			/* a data record                      */
long		bpos,			/* temp bosition in data buffer       */
		i,			/* loop counter                       */
		ierr,			/* error status                       */
		num_bytes,		/* number of bytes to be read in      */
		num_to_copy,		/* number of bytes to get in a record */
		offset;			/* offset of a record in the file     */
/*
   get the shape
*/
for (i = 0; i < 9; i++)
   shape[i] = rec->shape[i];
/*
   Calculate the total number of bytes to be copied
*/
if (!(calc_num_bytes (shape, &num_bytes)))
   return (ERR_DATA);
/*
   Now calculate the number of bytes on the last record
*/
num_bytes %= NUM_DATA_BYTES;
/*
   now get the data
*/
num_to_copy =
 (((rec->cont_off == NULL_OFF)&&(num_bytes != 0)) ? num_bytes : NUM_DATA_BYTES);
bpos = 0;
for (i = 0; i < num_to_copy; i++)
   buf[bpos++] = rec->rec_data[i];

offset = rec->cont_off;
while (offset != NULL_OFF)     /* data is continued */
   {
   /*
      keep reading continuation records
   */
   if ((ierr = read_rec (offset, (GENERIC_REC *) &rec2)) == NO_ERR)
      {
/*    num_to_copy = 
	 ((rec2.cont_off == NULL_OFF) ? num_bytes : NUM_DATA_BYTES);
*/
      num_to_copy = 
 (((rec2.cont_off == NULL_OFF)&&(num_bytes != 0)) ? num_bytes : NUM_DATA_BYTES);
      for (i = 0; i < num_to_copy; i++)
	 buf[bpos++] = rec2.rec_data[i];
      offset = rec2.cont_off;
      }
   else
      return (ierr);
   }
return (NO_ERR);
}


/*---------------------------- search_list -----------------------------*/

long search_list
(
    long roff,			/* record offset of first record      */
    char *data_type,		/* data type to find                  */
    DATA_REC *rec_found,		/* record found matching data type    */
    long *off_found		/* offset of the record in the file   */
)
/*
NAME            search_list

PURPOSE         Search a linked list of records looking for the data type.
                If found, return the record which contains it and its
                offset.  If the data_type is not found off_found is the
                offset of the last record in the list and rec_found points
                to a copy of that record.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   roff            long          I       offset of first record in list
   *data_type      char          I       the data_type to search for
   *rec_found      DATA_REC      O       the the record which contains the data
   *off_found      long          O       the offset of the record found

RETURN VALUES   NO_ERR, ERR_IO, ERR_DATA

ALGORITHM (PDF)

   while not at the end of the linked list
      read the next record
      if data types match then
         return
      endif
   endwhile
   return error

*/
{
long            ierr;			/* error status                       */
DATA_REC        temp_rec;		/* a temporary data record            */

*off_found = NULL_OFF;

/*
   Step through using pntr 'next_off' looking for data_type
   roff is the offset of the first record to look at
*/
while (roff != NULL_OFF)
   {
   if ((ierr = read_rec (roff, (GENERIC_REC *) &temp_rec)) != NO_ERR)
      return (ierr);
   copy_rec ((GENERIC_REC *) rec_found, (GENERIC_REC *) &temp_rec);
   *off_found = roff;
   if (strcmp (temp_rec.data_type, data_type) == 0)
      return (NO_ERR);
   roff = temp_rec.next_off;
   }
/*
   data_type not found
*/
return (ERR_DATA);
}


/*---------------------------- get_name --------------------------------*/

long get_name
(
    long offset,			/* offset of first record in list     */
    char name[]			/* the name found                     */
)
/*
NAME            get_name

PURPOSE         Given the offset of the first record of a list of records,
                search through list looking for data_type "NAME" and return
                the name.  The list will be either class level or site
                level.  The name should be less than 441 characters.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   offset          long          I       offset of first record in list
   name[]          char          O       the name if found

RETURN VALUES   NO_ERR, ERR_IO, ERR_DATA

ALGORITHM (PDF)

   while not at the end of the linked list
      read the next record
      if data type is NAME then
         return with the name
      endif
   endwhile
   return error

*/
{
DATA_REC        rec;			/* a data record                      */
long ierr;			/* error status                       */

while (offset != NULL_OFF)
   {
   if ((ierr = read_rec (offset, (GENERIC_REC *) &rec)) != NO_ERR)
      return (ierr);
   if (strcmp (rec.data_type, "NAME") == 0)
      {
      strcpy (name, rec.rec_data);
      return (NO_ERR);
      }
   else
      offset = rec.next_off;
   }
return (ERR_DATA);
}


/*---------------------------- get_first_image_rec ---------------------*/

long get_first_image_rec
(
    FIRST_IM_REC *rec_ptr		/* pointer to first record structure  */
)
/*
NAME            get_first_image_rec

PURPOSE         Read the first image level record which is the first record
                in the file. This is the class-list record.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec_ptr        FIRST_IM_REC  O       the class-list record read in

RETURN VALUES   NO_ERR, ERR_IO
*/
{
return (read_rec (0, (GENERIC_REC *) rec_ptr));
}


/*---------------------------- get_first_class_rec ---------------------*/

long get_first_class_rec
(
    long ic,			/* class index                        */
    FIRST_CL_REC *rec_ptr		/* pointer to record structure read in*/
)
/*
NAME            get_first_class_rec

PURPOSE         Get a pointer to the first record of a class given the
                class number. This is a site-list record.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   *rec_ptr        FIRST_CL_REC  O       the site-list record read in

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS

ALGORITHM (PDF)

   get the offset of the class
   read the record
   return

*/
{
long            coff,			/* class offset in file               */
		ierr;			/* error status                       */

if ((ierr = get_class_off (ic, &coff)) == NO_ERR)
   ierr = read_rec (coff, (GENERIC_REC *) rec_ptr);
return (ierr);
}


/*---------------------------- get_first_site_rec ----------------------*/

long get_first_site_rec
(
    long ic,			/* class index                        */
    long is,			/* site index                         */
    FIRST_SI_REC *rec_ptr		/* pointer to first site record struct*/
)
/*
NAME            get_first_site_rec

PURPOSE         Get a pointer to the first record of a site given the class
                number.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   is              long          I       the site index
   *rec_ptr        FIRST_SI_REC  O       the data record read in

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS, ERR_SITE

ALGORITHM (PDF)

   get the offset of the site
   read the record
   return

*/
{
long            soff,			/* site offset                        */
		ierr;			/* error status                       */

if ((ierr = get_site_off (ic, is, &soff)) == NO_ERR)
   ierr = read_rec (soff, (GENERIC_REC *) rec_ptr);
return (ierr);
}


/*---------------------------- get_class_off_list ----------------------*/

long get_class_off_list
(
    long *numc,			/* number of classes in file                  */
    long *clist,			/* list of class offsets                      */
    long *delptr		/* offset of first deleted record             */
)
/*
NAME            get_class_off_list

PURPOSE         Get the number of classes and the offset to all the
                classes. This is taken from the class-list record.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *numc           long          O       the number of classes in the file
   *clist          long          O       offsets to all classes
   *delptr         long          O       offset of first deleted record

RETURN VALUES   NO_ERR, ERR_IO

ALGORITHM (PDF)

   read the first record
   copy the list of offsets
   return

*/
{
FIRST_IM_REC    rec1;			/* pointer to class-list record struct*/
long            i,			/* loop counter                       */
		ierr,			/* error status                       */
   		index;			/* index into class list              */

if ((ierr = get_first_image_rec (&rec1)) == NO_ERR)
   {
   *delptr = rec1.del_off;
   *numc = rec1.num_class;
   index = 0;
   do {
      for (i = 0; (i < CLASSES_PER_RECORD) && (index < *numc); i++)
	 clist[index++] = rec1.class_off[i];
      if (index < *numc)
         {
	 if ((ierr = read_rec (rec1.cont_off, (GENERIC_REC *) &rec1)) != NO_ERR)
	    return (ierr);
         }
      } while (index < *numc);
/*
      clist[i] = rec1.class_off[i];
*/
   return (NO_ERR);
   }
else
   {
   *numc = 0;
   return (ierr);
   }
}


/*---------------------------- get_site_off_list -----------------------*/

long get_site_off_list
(
    long ic,			/* class index                        */
    long *nums,			/* number of sites                    */
    long *slist			/* list of site offsets in file       */
)
/*
NAME            get_site_off_list

PURPOSE         Get the number of sites for a particular class, and get all
                the offsets for the sites.  This is taken from the
                site-list record.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   *nums           long          O       the number of site for the class
   *slist          long          O       offsets to all sites of this class

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS

ALGORITHM (PDF)

   get the first record for the class
   copy the list of offsets
   return

*/
{
FIRST_CL_REC    rec1;			/* site list record                   */
long            i,			/* loop counter                       */
		ierr,			/* error status                       */
   		index;			/* index into site list               */

	if ((ierr = get_first_class_rec  (ic, &rec1)) == NO_ERR)
	   {
   *nums = rec1.num_site;
   index = 0;
   do {
      for (i = 0; (i < SITES_PER_RECORD) && (index < *nums); i++)
	 slist[index++] = rec1.site_off[i];
      if (index < *nums)
         {
	 if ((ierr = read_rec (rec1.cont_off, (GENERIC_REC *) &rec1)) != NO_ERR)
	    return (ierr);
         }
      } while (index < *nums);
/*
   for (i = 0; i < *nums; i++)
      slist[i] = site_offset_from_index (&rec1, i);
      slist[i] = rec1.site_off[i];
*/
   return (NO_ERR);
   }
else
   {
   *nums = 0;
   return (ierr);
   }
}


/*---------------------------- get_site_list ---------------------------*/

long get_site_list
(
    FIRST_CL_REC *rec1,			/* first site list record             */
    long *nums,			/* number of sites                    */
    long *slist			/* list of site offsets in file       */
)
{
FIRST_CL_REC    temp_rec;		/* site list record                   */
long            i,			/* loop counter                       */
		ierr,			/* error status                       */
   		index;			/* index into site list               */

*nums = rec1->num_site;
copy_rec (&temp_rec, rec1);
index = 0;
do {
   for (i = 0; (i < SITES_PER_RECORD) && (index < *nums); i++)
      slist[index++] = temp_rec.site_off[i];
   if (index < *nums)
      {
      if ((ierr = read_rec (temp_rec.cont_off, (GENERIC_REC *) &temp_rec)) != NO_ERR)
	 return (ierr);
      }
   } while (index < *nums);
return (NO_ERR);
}

/*---------------------------- get_image_level -------------------------*/

long get_image_level
(
    char *data_type,		/* data type to search for            */
    SHAPE_VEC shape,			/* shape of data read in              */
    unsigned char *buf			/* buffer to read data into           */
)
/*	
NAME            get_image_level

PURPOSE         Get image level data given the data type.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *data_type      char          I       the type of data to be read
   shape           SHAPE_VEC     O       the shape vector of data read in
   *buf            unsigned char O       the data read in

RETURN VALUES   NO_ERR, ERR_IO, ERR_DATA

ALGORITHM (PDF)

   search image level data for data type
   if data type is found then
      get shape and data
   endif
   return

*/
{
FIRST_IM_REC    rec1;			/* class list record                  */
long            ierr,			/* error status                       */
		off_rec1;		/* offset of record found             */

if ((ierr = search_list (0, data_type, &rec1, &off_rec1)) == NO_ERR)
   /*
      copy shape and buf from rec1
   */
   ierr = get_shape_buff (&rec1, shape, buf);

return (ierr);
}


/*---------------------------- get_class_level -------------------------*/

long get_class_level
(
    long ic,			/* class index                        */
    char *data_type,		/* data type to search for            */
    SHAPE_VEC shape,			/* shape of data read in              */
    unsigned char *buf			/* buffer to read data into           */
)
/*
NAME            get_class_level

PURPOSE         Get class level data given the class and data type.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   *data_type      char          I       the type of data to be read
   shape           SHAPE_VEC     O       the shape vector of data read in
   *buf            unsigned char O       the data read in

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS, ERR_DATA

ALGORITHM (PDF)

   get offset of first record of the class
   search class level data for data type
   if data type is found then
      get shape and data
   endif
   return

*/
{
FIRST_CL_REC    rec1;			/* site list record                   */
long            ierr,			/* error status                       */
		icoff,			/* offset of class record             */
		off_rec1;		/* offset of record found             */

/*
   Find first class record for desired class
*/
if ((ierr = get_class_off (ic, &icoff)) != NO_ERR)
   return (ierr);

if ((ierr = search_list (icoff, data_type, &rec1, &off_rec1)) == NO_ERR)
   /*
      copy shape and buf
   */
   ierr = get_shape_buff (&rec1, shape, buf);

return (ierr);
}


/*---------------------------- get_site_level --------------------------*/

long get_site_level
(
    long ic,			/* class index                        */
    long is,			/* site index                         */
    char *data_type,		/* data type to search for            */
    SHAPE_VEC shape,			/* shape of data read in              */
    unsigned char *buf			/* buffer to read data into           */
)
/*
NAME            get_site_level

PURPOSE         Get site level data given the class, site and data type.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   is              long          I       the site index
   *data_type      char          I       the type of data to be read
   shape           SHAPE_VEC     O       the shape vector of data read in
   *buf            unsigned char O       the data read in

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS, ERR_SITE, ERR_DATA

ALGORITHM (PDF)

   get offset of first record of the site
   search site level data for data type
   if data type is found then
      get shape and data
   endif
   return

*/
{
FIRST_SI_REC    rec1;			/* first record of a site             */
long            ierr,			/* error status                       */
		isoff,			/* offset of the site record          */
		off_rec1;		/* offset of record found             */

/*
   Find first site record for desired class and site
*/
if ((ierr = get_site_off (ic, is, &isoff)) != NO_ERR)
   return (ierr);

if ((ierr = search_list (isoff, data_type, &rec1, &off_rec1)) == NO_ERR)
   /*
      copy shape and buf
   */
   ierr = get_shape_buff (&rec1, shape, buf);

return (ierr);
}


/*---------------------------- calc_num_bytes --------------------------*/

long calc_num_bytes
(
    SHAPE_VEC shape,			/* shape vector                       */
    long *num_bytes		/* number of bytes                    */
)
/*
NAME            calc_num_bytes

PURPOSE         Using the shape vector calculate the number of bytes the
                data takes up.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   shape           SHAPE_VEC     I       the shape vector of data
   *num_bytes      long          O       the number of bytes data requires

RETURN VALUES

   value        meaning
   -----        -------
   TRUE         calculation completed successfully
   FALSE        calculation could not be made. error in shape.

ALGORITHM (PDF)

   get bytes per data from first shape element
   get number of indices from second element
   for each index
      multiply
   endfor
   return

*/
{
long	i,				/* loop counter                       */
	size;				/* size of data element               */

*num_bytes = 0;

/*
   Put the number of bytes per item into size
*/
switch (shape[0])
   {
   case EBYTE:   size = sizeof (unsigned char); break;
   case EWORD:   size = sizeof (short);         break;
   case ELONG:   size = sizeof (long);          break;
   case EREAL:   size = sizeof (float);         break;
   case EDOUBLE: size = sizeof (double);        break;
   case ECOMP:   size = 2 * sizeof (float);     break;
   case EDCOMP:  size = 2 * sizeof (double);    break;
   default: return (FALSE);
   }

if (shape[1] < 0 || shape[1] > 7)
   return (FALSE);
*num_bytes = size;

/*
   multiply by each of the ranges of the indices
*/
for (i = 1; i <= shape[1]; i++)
   *num_bytes *=  shape[i + 1];
return (TRUE);
}


/*---------------------------- rewrite_data ----------------------------*/

long rewrite_data
(
    long offset,			/* offset of record to be replaced    */
    SHAPE_VEC shape,			/* shape of new data                  */
    unsigned char *buf			/* data to be written to file         */
)
/*
NAME            rewrite_data

PURPOSE         Replace data which is already in the file.  New records
                might have to be allocated or some may have to be returned
                to the deleted record list.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   offset          long          I       the offset of record to be replaced
   shape           SHAPE_VEC     I       the shape of the data
   *buf            unsigned char I       the data to be written

RETURN VALUES   NO_ERR, ERR_IO, ERR_DATA

ALGORITHM (PDF)

   calculate number of records needed for data
   read the first existing record to be replaced
   initialize the record
   for number of records needed
      put data into record
      if another record will be needed then
         if we have existing record to replace then
            read the record to be replaced and its offset so
            it can be pointed to by current record
         else
            get an empty record offset so current record can
            point to it
         endif
      else
         current record is last so it will point nowhere
      endif
      write out the record
   endfor
   delete any remaining old records
   return

*/
{
long            bpos,			/* current position in data buffer    */
		del_off,		/* offset of deleted record           */
		i,			/* loop counter                       */
		ierr,			/* error status                       */
		j,			/* loop counter                       */
		new_off,		/* offset of next record to be written*/
		new_off2,		/* offset of record to write          */
		num_bytes,		/* total number of bytes to write     */
		num_recs,		/* number of records required for data*/
		num_to_copy,		/* number of bytes on current record  */
		remainder;		/* number of bytes on last record     */
DATA_REC        new_rec,		/* record to be written               */
		temp_rec;		/* existing record read in            */

/*
   calculate the number of records required
*/
calc_num_bytes (shape, &num_bytes);
remainder = num_bytes % NUM_DATA_BYTES;
num_recs = (num_bytes / NUM_DATA_BYTES) + (remainder > 0);
num_to_copy = NUM_DATA_BYTES;

bpos = 0;
ierr = ERR_DATA;

if ((offset != NULL_OFF) && (num_recs > 0))
   {
   /*
      use existing record offset
   */
   new_off = offset;
   if ((ierr = read_rec (offset, (GENERIC_REC *) &temp_rec)) != NO_ERR)
      return (ierr);
   offset = temp_rec.cont_off;
   }
else
   return (ierr);
/*
   copy data_type and shape
*/
init_data_rec (&new_rec);
strcpy (new_rec.data_type, temp_rec.data_type);
for (j = 0; j < 9; j++)
   new_rec.shape[j] = shape[j];

new_rec.next_off = temp_rec.next_off;

for (i = 1; i <= num_recs; i++)
   {
   /*
      Put data into record
   */
   if (i == num_recs)
      num_to_copy = (remainder == 0) ? num_to_copy : remainder;
   for (j = 0; j < num_to_copy; j++)
      new_rec.rec_data[j] = buf[bpos++];
   /*
      if another record needed then put its offset into new_rec
   */
   if (i < num_recs)
      {
      if (offset != NULL_OFF)
	 {
         /*
            have existing record of this data
            use existing record offset
         */
	 new_off2 = offset;
	 if ((ierr = read_rec (offset, (GENERIC_REC *) &temp_rec)) != NO_ERR)
	    return (ierr);
	 offset = temp_rec.cont_off;
	 }
      else
	 {
         /*
            have run out of old records of this data
	    get new record offset
         */
	 if ((ierr = get_empty_off (&new_off2, &del_off)) != NO_ERR)
	    return (ierr);
	 }
      }
   else
      /*
         no more records needed
      */
      new_off2 = NULL_OFF;

   new_rec.cont_off = new_off2;
   /*
      write out the record
   */
   if ((ierr = write_rec (new_off, (GENERIC_REC *) &new_rec)) == NO_ERR)
      {
      new_off = new_off2;
      init_data_rec (&new_rec);
      new_rec.next_off = NULL_OFF;
      }
   else
      return (ierr);
   }

/*
   offset now points to the first record which needs to be deleted
   Return any remaining records to the deleted record list.
*/
return (delete_using_cont (offset, &del_off));
}


/*---------------------------- write_new_data --------------------------*/

long write_new_data
(
    DATA_REC *rec_prev,		/* the previous record                */
    long off_prev,		/* offset of previous record          */
    char *data_type,		/* data type of data to be written    */
    SHAPE_VEC shape,			/* shape of data to be written        */
    unsigned char *buf			/* data to be written                 */
)
/*
NAME            write_new_data

PURPOSE         Write new data out with the previous record's next offset
                pointing to it.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec_prev       DATA_REC      I       the record which will point to new record
   off_prev        long          I       offset of above record
   *data_type      char          I       data_type of new data
   shape           SHAPE_VEC     I       the shape of the data
   *buf            unsigned char I       the data to be written

RETURN VALUES   NO_ERR, ERR_IO, ERR_DATA

ALGORITHM (PDF)

   calculate number of records needed for data
   if we have data to put in and can get an empty record then
      have previous record point to new record
      initialize the record
      for number of records needed
         put data into record
         if another record will be needed then
            get an empty record offset so current record can
            point to it
         else
            current record is last so it will point nowhere
         endif
         write out the record
      endfor
      return
   else
      return
   endif

*/
{
long            bpos,			/* current position in data buffer    */
		del_off,		/* offset of deleted record           */
		i,			/* loop counter                       */
		ierr,			/* error staus                        */
		j,			/* loop counter                       */
		new_off,		/* offset of next record to be written*/
		new_off2,		/* offset of record to write          */
		num_bytes,		/* total number of bytes to write     */
		num_recs,		/* number of records required for data*/
		num_to_copy,		/* number of bytes on current record  */
		remainder;		/* number of bytes on last record     */

DATA_REC        new_rec;		/* record to be written               */
FIRST_IM_REC    *rec1_ptr;		/* first record in file               */

if (strlen (data_type) > 25)
   return (ERR_DATA);

/*
   calculate the number of records required
*/
calc_num_bytes (shape, &num_bytes);
remainder = num_bytes % NUM_DATA_BYTES;
num_recs = (num_bytes / NUM_DATA_BYTES) + (remainder > 0);
num_to_copy = NUM_DATA_BYTES;

bpos = 0;
ierr = ERR_DATA;

if (num_recs > 0 && (ierr = get_empty_off (&new_off, &del_off)) == NO_ERR)
   {
   /*
      have the previous record point to the new data
   */
   rec_prev->next_off = new_off;
   if (off_prev == 0)
      {
      rec1_ptr = ((FIRST_IM_REC *) rec_prev);
      rec1_ptr->del_off = del_off;
      }
   if ((ierr = write_rec (off_prev, (GENERIC_REC *) rec_prev)) != NO_ERR)
      return (ierr);
   /*
      copy data_type and shape
   */
   init_data_rec (&new_rec);
   strcpy (new_rec.data_type, data_type);
   for (j = 0; j < 9; j++)
      new_rec.shape[j] = shape[j];

   for (i = 1; i <= num_recs; i++)
      {
      /*
         Put data into record
      */
      if (i == num_recs)
	 num_to_copy = (remainder == 0) ? num_to_copy : remainder;
      for (j = 0; j < num_to_copy; j++)
	 new_rec.rec_data[j] = buf[bpos++];
      /*
         if another record needed then put its offset into new_rec
      */
      if (i < num_recs)
	 {
	 if ((ierr = get_empty_off (&new_off2, &del_off)) != NO_ERR)
	    return (ierr);
	 }
      else
         /*
            no more records needed
         */
	 new_off2 = NULL_OFF;
      new_rec.cont_off = new_off2;
      /*
         write out the record
      */
      new_rec.next_off = NULL_OFF;
      if ((ierr = write_rec (new_off, (GENERIC_REC *) &new_rec)) == NO_ERR)
	 {
	 new_off = new_off2;
	 init_data_rec (&new_rec);
	 }
      else
	 return (ierr);
      }
   return (NO_ERR);
   }
else
   return (ierr);
}


/*---------------------------- add_data --------------------------------*/

long add_data
(
    long offset,			/* first record of list               */
    char *data_type,		/* data type of data to be written    */
    SHAPE_VEC shape,			/* the shape of the data to be written*/
    unsigned char *buf			/* the data to be written             */
)
/*
NAME            add_data

PURPOSE         Put data at any level given the offset at the beginning of
                the linked list and the data type.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   offset          long          I       offset of first record of linked list
   *data_type      char          I       data_type of new data
   shape           SHAPE_VEC     I       the shape of the data
   *buf            unsigned char I       the data to be written

RETURN VALUES   NEW_DATA, OLD_DATA, ERR_IO, ERR_DATA

ALGORITHM (PDF)

   search for the data type starting at offset given
   if data type is found then
      rewrite old data
   elseif data type is not found then
      write new data
   endif
   return

*/
{
FIRST_IM_REC    rec_found;		/* first record in file               */
long		ierr,			/* error status                       */
		off_found;		/* offset of record found             */

/*
   look for the data type at the level specified
*/
ierr = search_list (offset, data_type, &rec_found, &off_found);
if (ierr == NO_ERR)
   {
   /*
      exising data found
   */
   if ((ierr = rewrite_data (off_found, shape, buf)) == NO_ERR)
      ierr = OLD_DATA;
   }
else if (ierr == ERR_DATA)
   {
   /*
      no existing data found
   */
   if ((ierr = write_new_data (&rec_found, off_found, data_type, shape, buf))
      == NO_ERR)
      ierr = NEW_DATA;
   }
return (ierr);
}


/*---------------------------- create_class ----------------------------*/

long create_class
(
    FIRST_IM_REC *rec1,			/* first record in file               */
    FIRST_CL_REC *new_rec,		/* new site list record               */
    long *new_off		/* offset of site list record         */
)
/*
NAME            create_class

PURPOSE         Create a new class.  Write a site-list record and update
                the class-list record.  Rec1 is the class-list record,
                new_rec is the site-list record, and new_off is the offset
                of new_rec.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec1           FIRST_IM_REC  I       the first (class-list) record
   *new_rec        FIRST_CL_REC  O       the new site-list record
   *new_off        long          O       offset of new site-list record

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS

ALGORITHM (PDF)

   if max number of classes exist then
      return
   endif
   write an initialized site list record
   update the class list record
   return

*/
{
long		clist[MAXCLS],		/* list of class offsets              */
		del_off,		/* offset of deleted record           */
		ierr,			/* error status                       */
   		numc;			/* number of classes                  */

if (rec1->num_class == MAXCLS)
   return (ERR_CLASS);

/*
   write an initialized site-list record
*/
init_site_list (new_rec);
if ((ierr = get_empty_off (new_off, &del_off)) != NO_ERR)
   return (ierr);

if ((ierr = write_rec (*new_off, (GENERIC_REC *) new_rec)) != NO_ERR)
   return (ierr);

/*
   update class-list record
*/
rec1->del_off = del_off;
if ((ierr = get_class_off_list (&numc, clist, &del_off)) == NO_ERR)
   {
   clist[rec1->num_class++] = *new_off;
   numc = rec1->num_class;
/*
   rec1->class_off[rec1->num_class++] = *new_off;
*/
   ierr = write_class_list (rec1, 0, clist, numc);
/*
   ierr = write_rec ((long) 0, (GENERIC_REC *) rec1);
*/
   }
return (ierr);
}


/*---------------------------- new_class -------------------------------*/

long new_class
(
    FIRST_IM_REC *rec1,			/* first record in file               */
    char *data_type,		/* data type of data to be written    */
    SHAPE_VEC shape,			/* shape of data to be written        */
    unsigned char *buf,			/* the data to be written             */
    FIRST_CL_REC *new_rec,		/* site list record                   */
    long *new_off		/* offset of site list record         */
)
/*
NAME            new_class

PURPOSE         Create a new class and put new data at the class level.
                Write a site-list record and update the class-list record.
                Write the new data to that class using add_data. Rec1 is the
                class-list record, new_rec is the site-list record, and
                new_off is the offset of new_rec.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec1           FIRST_IM_REC  I       the first (class-list) record
   *data_type      char          I       data_type of data to be written
   shape           SHAPE_VEC     I       shape of the data
   *buf            unsigned char I       the data to be written
   *new_rec        FIRST_CL_REC  O       the new site-list record
   *new_off        long          O       offset of new site-list record

RETURN VALUES   NO_ERR, ERR_IO, ERR_DATA, ERR_CLASS

ALGORITHM (PDF)

   create the new class
   add data at the class level
   return

*/
{
long ierr;			/* eror status                        */

/*
   create a new class
*/
if ((ierr = create_class (rec1, new_rec, new_off)) != NO_ERR)
   return (ierr);

/*
   insert data at class level
*/
ierr = add_data (*new_off, data_type, shape, buf);
if ((ierr != ERR_IO) && (ierr != ERR_DATA))
   return (NO_ERR);
return (ierr);
}


/*---------------------------- new_site --------------------------------*/

long new_site
(
    FIRST_CL_REC *rec1,			/* site list record                   */
    long rec1_off,		/* offset of site list record         */
    char *data_type,		/* data type of data to be written    */
    SHAPE_VEC shape,			/* shape of data to be written        */
    unsigned char *buf			/* data to be written                 */
)
/*
NAME            new_site

PURPOSE         Create a new site and add new data at the site level.
                Update the site-list record and get an offset to a new record
                which will be the first record for the new site.  First it will
                write a record with only the data_type and then it will write
                the rest of the data using add_data.  Rec1 is the site-list
                record and rec1_off is its offset.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *rec1           FIRST_CL_REC  I       the site-list record for the class
   rec1_off        long          I       the offset of the site-list record
   *data_type      char          I       data_type of data to be written
   shape           SHAPE_VEC     I       shape of the data
   *buf            unsigned char I       the data to be written

RETURN VALUES   NO_ERR, ERR_IO, ERR_DATA, ERR_SITE

ALGORITHM (PDF)

   if max number of sites exist then
      return
   endif
   write out an empty record with desired data type
   add data starting at the empty record
   update site list record to point to new site
   return

*/
{
long		del_off,		/* offset of deleted record           */
		ierr,			/* error status                       */
   		nums,			/* number of sites                    */
		offset,			/* offset of record written           */
   		slist[MAXSIT];		/* site offset list                   */
DATA_REC        rec_data;		/* data record written                */

if (rec1->num_site == MAXSIT)
   return (ERR_SITE);

/*
   write the data record with nothing but data_type in it
*/
if (strlen (data_type) > 25)
   return (ERR_DATA);

if ((ierr = get_empty_off (&offset, &del_off)) != NO_ERR)
   return (ierr);
rec_data.next_off = rec_data.cont_off = NULL_OFF;
strcpy (rec_data.data_type, data_type);
if ((ierr = write_rec (offset, (GENERIC_REC *) &rec_data)) != NO_ERR)
   return (ierr);

/*
   now add the data at the site level
*/
ierr = add_data (offset, data_type, shape, buf);

if ((ierr != ERR_IO) && (ierr != ERR_DATA))
   {
   /*
      update site-list record
   */
   if ((ierr = get_site_list (rec1, &nums, slist)) == NO_ERR)
      {
/*
      rec1->site_off[rec1->num_site++] = offset;
      ierr = write_rec (rec1_off, (GENERIC_REC *) rec1);
*/
      slist[rec1->num_site++] = offset;
      nums = rec1->num_site;
      ierr = write_site_list (rec1, rec1_off, slist, nums);
      }
   }
return (ierr);
}


/*---------------------------- put_image_level -------------------------*/

long put_image_level
(
    char *data_type,		/* data type of data to be written    */
    SHAPE_VEC shape,			/* shape of data to be written        */
    unsigned char *buf			/* data to be written                 */
)
/*
NAME            put_image_level

PURPOSE         Put image level data given the data type.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   *data_type      char          I       data_type of data to be written
   shape           SHAPE_VEC     I       shape of the data
   *buf            unsigned char I       the data to be written

RETURN VALUES   NEW_DATA, OLD_DATA, ERR_IO, ERR_DATA

*/
{
return (add_data (0, data_type, shape, buf));
}


/*---------------------------- put_class_level -------------------------*/

long put_class_level
(
    long ic,			/* class index                        */
    char *data_type,		/* data type of data to be written    */
    SHAPE_VEC shape,			/* shape of data to be written        */
    unsigned char *buf			/* data to be written                 */
)
/*
NAME            put_class_level

PURPOSE         Put class level data given the class and data type.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   *data_type      char          I       data_type of data to be written
   shape           SHAPE_VEC     I       shape of the data
   *buf            unsigned char I       the data to be written

RETURN VALUES   NEW_CLASS, NEW_DATA, OLD_DATA, ERR_IO, ERR_CLASS, ERR_DATA

ALGORITHM (PDF)

   if new class is to be created then
      create new class and add data
   else
      add the data
   endif
   return

*/
{
long            ierr,			/* error status                       */
		num_classes,		/* numbre of classes in file          */
		offset,			/* offset of class                    */
		rec2_off;		/* offset of record written           */
FIRST_IM_REC    rec1;			/* class list record                  */
FIRST_CL_REC    rec2;			/* site list record                   */

if ((ierr = get_first_image_rec (&rec1)) != NO_ERR)
   return (ierr);
num_classes = rec1.num_class;

if (ic == num_classes + 1)
   {
   /*
      new class must be created
   */
   if ((ierr = new_class (&rec1, data_type, shape, buf, &rec2, &rec2_off))
      == NO_ERR)
      ierr = NEW_CLASS;
   }
else if ((ic <= num_classes) && (ic > 0))
   {
   /*
      class exists. just add data
   */
   offset = class_offset_from_index (&rec1, ic - 1);
/*
   offset = rec1.class_off[ic - 1];
*/
   ierr = add_data (offset, data_type, shape, buf);
   }
else
   ierr = ERR_CLASS;

return (ierr);
}


/*---------------------------- put_site_level --------------------------*/

long put_site_level
(
    long ic,			/* class index                        */
    long is,			/* site index                         */
    char *data_type,		/* data type of data to be written    */
    SHAPE_VEC shape,			/* shape of data to be written        */
    unsigned char *buf			/* data to be written                 */
)
/*
NAME            put_site_level

PURPOSE         Put site level data given the class, site and data type.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   is              long          I       the site index
   *data_type      char          I       data_type of data to be written
   shape           SHAPE_VEC     I       shape of the data
   *buf            unsigned char I       the data to be written

RETURN VALUES   NEW_CLASS, NEW_SITE,  NEW_DATA, OLD_DATA,
                ERR_IO,    ERR_CLASS, ERR_SITE, ERR_DATA
ALGORITHM (PDF)

   get class list record
   if new class is to be created then
      if site index is 1 then
         create new class
         create new site and add data
      else
         return error
      endif
   else
      get number of sites in class
      if new site is to be created then
         create new site and add data
      else
         add data at site level
      endif
   endif
   return

*/
{
long            ierr,			/* error status                       */
		num_classes,		/* number of classes in file          */
		num_sites,		/* number of sites in class           */
		offset,			/* offset of site                     */
		rec2_off;		/* offset of record written           */
FIRST_IM_REC    rec1;			/* class list record                  */
FIRST_CL_REC    rec2;			/* site list record                   */

if ((ierr = get_first_image_rec (&rec1)) != NO_ERR)
   return (ierr);
num_classes = rec1.num_class;

if (ic == num_classes + 1)
   {
   if (is == 1)
      {
      /*
         new class and new site must be created
      */
      if ((ierr = create_class (&rec1, &rec2, &rec2_off)) == NO_ERR)
	 if ((ierr = new_site (&rec2, rec2_off, data_type, shape, buf))
	    == NO_ERR)
	    ierr = NEW_CLASS;
      }
   else
      ierr = ERR_SITE;
   }
else if (ic <= num_classes)
   {
   /*
      get number of sites
   */
/*
   if (ierr = (read_rec (rec1.class_off[ic - 1], (GENERIC_REC *) &rec2)) != NO_ERR)
*/
   if ((ierr = (read_rec (class_offset_from_index (&rec1, ic - 1), (GENERIC_REC *) &rec2))) != NO_ERR)
      return (ierr);
   num_sites = rec2.num_site;

   if (is == num_sites + 1)
      {
      /*
         new site must be created
      if ((ierr = new_site (&rec2, rec1.class_off[ic - 1], data_type,
      */
      if ((ierr = new_site (&rec2, class_offset_from_index (&rec1, ic - 1),
           data_type, shape, buf)) == NO_ERR)
	 ierr = NEW_SITE;
      }
   else if (is <= num_sites)
      {
      offset = site_offset_from_index (&rec2, is - 1);
/*
      offset = rec2.site_off[is - 1];
*/
      ierr = add_data (offset, data_type, shape, buf);
      }
   else
      ierr = ERR_SITE;
   }
else
   ierr = ERR_CLASS;
return (ierr);
}


/*---------------------------- delete_class2 ----------------------------*/

long delete_class2
(
    long ic			/* class index                        */
)
/*
NAME            delete_class2

PURPOSE         Delete an entire class given its index.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS

ALGORITHM (PDF)

   get class list record
   if class index is invalid then
      return error
   else
      update class list record
   endif
   read site list record
   for the number of sites
      delete the site
   endfor
   delete the class level data
   return

*/
{
long		clist[MAXCLS],		/* list of class offsets              */
		del_off,		/* offset of deleted record           */
		i,			/* loop counter                       */
		icoff,			/* class offset                       */
		ierr,			/* error status                       */
		is,			/* site index                         */
		numc,			/* number of classes                  */
		num_sites,		/* number of sites in class           */
		slist[MAXSIT];		/* list of site offsets               */
FIRST_CL_REC    cl_rec;			/* site list record                   */
FIRST_IM_REC    rec1;			/* class list record                  */

/*
   Update class-list record
*/
if ((ierr = get_first_image_rec (&rec1)) != NO_ERR)
   return (ierr);

if (ic > rec1.num_class)
   return (ERR_CLASS);
else
   {
   /*
      update class-list record first
   */
   if ((ierr = get_class_off_list (&numc, clist, &del_off)) != NO_ERR)
      return (ierr);
   icoff = clist [ic - 1];
/*
   icoff = rec1.class_off[ic - 1];
*/
   rec1.num_class--;
   numc = rec1.num_class;
   if (rec1.num_class != 0)
      {
      for (i = ic - 1; i < rec1.num_class; i++)
         clist[i] = clist [i + 1];
/*
	 rec1.class_off[i] = rec1.class_off[i + 1];
*/
      }
   else
      clist[0] = NULL_OFF;
   if ((ierr = write_class_list (&rec1, 0, clist, numc)) != NO_ERR)
      return (ierr);
/*
   if ((ierr = write_rec ((long) 0, &rec1)) != NO_ERR)
      return (ierr);
*/
   }

if ((ierr = read_rec (icoff, (GENERIC_REC *) &cl_rec)) != NO_ERR)
   return (ierr);
num_sites = cl_rec.num_site;
if ((ierr = get_site_list (&cl_rec, &num_sites, slist)) != NO_ERR)
   return (ierr);
/*
   delete all the sites for the class
*/
for (is = 0; is < num_sites; is++)
   delete_linked_list (slist[is]);
/*
   delete_linked_list (cl_rec.site_off[is - 1]);
*/

/*
   delete the class level data
*/
ierr = delete_linked_list (icoff);
return (ierr);
}


/*---------------------------- delete_site2 -----------------------------*/

long delete_site2
(
    long ic,			/* class index                        */
    long is			/* site index                         */
)
/*
NAME            delete_site2

PURPOSE         Delete an entire site given indecies for the class and the
                site.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   ic              long          I       the class index
   is              long          I       the site index

RETURN VALUES   NO_ERR, ERR_IO, ERR_CLASS, ERR_SITE

ALGORITHM (PDF)

   get the site list record
   if site index is invalid then
      return error
   else
      update site list record
   endif
   delete the site level data
   return

*/
{
long            i,			/* loop counter                       */
		ierr,			/* error status                       */
		icoff,			/* class offset                       */
		isoff,			/* site offset                        */
		nums,			/* number of sites                    */
		slist[MAXSIT];		/* list of site offsets               */
FIRST_CL_REC    cl_rec;			/* site list record                   */

/*
   get the site-list record for this class
*/
if ((ierr = get_class_off (ic, &icoff)) != NO_ERR)
   return (ierr);
if ((ierr = read_rec (icoff, &cl_rec)) != NO_ERR)
   return (ierr);

if (is > cl_rec.num_site)
   return (ERR_SITE);
else
   {
   /*
      update site-list record first
   */
   if ((ierr = get_site_list (&cl_rec, &nums, slist)) != NO_ERR)
      return (ierr);
   isoff = slist[is - 1];
/*
   isoff = cl_rec.site_off[is - 1];
*/
   cl_rec.num_site--;
   nums = cl_rec.num_site;
   if (cl_rec.num_site != 0)
      {
      for (i = is - 1; i < cl_rec.num_site; i++)
         slist[i] = slist [i + 1];
/*
	 cl_rec.site_off[i] = cl_rec.site_off[i + 1];
*/
      }
   else
      slist[0] = NULL_OFF;
   if ((ierr = write_site_list (&cl_rec, icoff, slist, nums)) != NO_ERR)
      return (ierr);
/*
   if ((ierr = write_rec (icoff, &cl_rec)) != NO_ERR)
      return (ierr);
*/
   }
/*
   now delete the data
*/
ierr = delete_linked_list (isoff);
return (ierr);
}


/*---------------------------- delete_linked_list ----------------------*/

long delete_linked_list
(
    long offset			/* first record of list to delete     */
)
/*
NAME            delete_linked_list

PURPOSE         Delete a linked list of records connected by next_off
                pointer.  These records also may have other records coming
                off them from cont_off pointers.  These must also be
                deleted.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   offset          long          I       offset of first record of list

RETURN VALUES   NO_ERR, ERR_IO

ALGORITHM (PDF)

   while not at end of linked (by next_off) list
      delete any continuation records off all records in list
   endwhile
   add the linked list of records to deleted record list
   return

*/
{
long            ierr,			/* error status                       */
		del_off,		/* offset of deleted record           */
		first_off,		/* first record deleted               */
		last_off;		/* last record deleted                */
DATA_REC        curr_rec;
FIRST_IM_REC    rec1;

if (offset == 0)
   return (ERR_IO);

first_off = offset;

/*
   first delete all the continuation records
   off the records in the list
*/
while (offset != NULL_OFF)
   {
   if ((ierr = read_rec (offset, (GENERIC_REC *) &curr_rec)) != NO_ERR)
      return (ierr);

   if ((ierr = delete_using_cont (curr_rec.cont_off, &del_off)) != NO_ERR)
      return (ierr);
   last_off = offset;
   offset = curr_rec.next_off;
   curr_rec.cont_off = NULL_OFF;
   }

/*
   now add the list to the deleted record list
   update first record using first_off and last_off.
*/
if (first_off != NULL_OFF)
   {
   if ((ierr = get_first_image_rec (&rec1)) != NO_ERR)
      return (ierr);
   curr_rec.next_off = rec1.del_off;
   if ((ierr = write_rec (last_off, (GENERIC_REC *) &curr_rec)) != NO_ERR)
      return (ierr);
   rec1.del_off = first_off;
   if ((ierr = write_rec ((long) 0, (GENERIC_REC *) &rec1)) != NO_ERR)
      return (ierr);
   }
return (NO_ERR);
}


/*---------------------------- delete_using_cont -----------------------*/

long delete_using_cont
(
    long offset,			/* offset of first record in list     */
    long *del_off		/* offset of first deleted record     */
)
/*
NAME            delete_using_cont

PURPOSE         Delete a list of records linked by cont_off pointers.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   offset          long          I       offset of first record of list
   *del_off        long          O       offset of first record in deleted list

RETURN VALUES   NO_ERR, ERR_IO

ALGORITHM (PDF)

   while not end of linked (by cont_off) list
      read the record
      change link from cont_off to next_off
   endwhile
   add the new linked list to the deleted record list
   return

*/
{
long            first_off,		/* first record deleted               */
		ierr,			/* error status                       */
		last_off;		/* last record deleted                */
DATA_REC        new_rec;		/* last record in the list            */
FIRST_IM_REC    rec1;			/* class list record                  */

/*
   The variable 'offset' points to the first record in a list of records
   to be added to the deleted record list. That list is linked by the
   cont_off pointer. It needs to be changed to next_off pointer for the
   deleted record list.
*/

if ((ierr = get_first_image_rec (&rec1)) != NO_ERR)
   return (ierr);
*del_off = rec1.del_off;

first_off = offset;
while (offset != NULL_OFF)
   {
   if ((ierr = read_rec (offset, (GENERIC_REC *) &new_rec)) != NO_ERR)
      return (ierr);
   new_rec.next_off = new_rec.cont_off;
   new_rec.cont_off = NULL_OFF;
   if ((ierr = write_rec (offset, (GENERIC_REC *) &new_rec)) != NO_ERR)
      return (ierr);
   last_off = offset;
   offset = new_rec.next_off;
   }

/*
   last_off points to the last record in the list. first_off points
   to the first record in the list and new_rec contains a copy of the
   last record in the list.
*/

if (first_off != NULL_OFF)
   {
   new_rec.next_off = rec1.del_off;
   if ((ierr = write_rec (last_off, (GENERIC_REC *) &new_rec)) != NO_ERR)
      return (ierr);
   rec1.del_off = first_off;
   *del_off     = first_off;
   if ((ierr = write_rec ((long) 0, (GENERIC_REC *) &rec1)) != NO_ERR)
      return (ierr);
   }
return (NO_ERR);
}


/*---------------------------- delete_data -----------------------------*/

long delete_data
(
    long roff,			/* offset of first record of list     */
    char *data_type,		/* data type of record to delete      */
    long *new_off		/* new offset of first record of list */
)
/*
NAME            delete_data

PURPOSE         Search a linked list of records looking for the data type.
                If found, delete that record and any any continuation
                records from that record.  Update the previous record to
                point to the correct record.

PROGRAM HISTORY

   programmer         date         reason
   ----------         ----         ------
   N. Deodhar/SAR     28-Nov-86    Unix conversion

PARAMETERS

   name            type         I/O      description
   ----            ----         ---      -----------
   roff            long          I       offset of first record of list
   *data_type      char          I       the data_type to be deleted
   *new_off        long          O       new offset of first record of same list

RETURN VALUES   NO_ERR, ERR_IO, ERR_DATA

ALGORITHM (PDF)

   while not at end of linked list
      read a record
      if data types match then
         delete this record and all of its continuation records
         if deleted record was first in linked list then
            no previous record to update
         else
            update the previous record's next_off
         endif
         return
      endif
   endwhile
   return error

*/
{
long            del_off,		/* offset of first deleted record     */
		ierr,			/* error status                       */
		prev_off;		/* offset of record prev to on deleted*/
DATA_REC        prev_rec,		/* record previous to one deleted     */
		temp_rec;		/* a temporary record                 */
FIRST_IM_REC    *rec1_ptr;		/* class list record                  */

prev_off = roff;
*new_off = roff;

/*
   Step through using pntr 'next_off' looking for data_type
   roff is the offset of the first record to look at
*/
while (roff != NULL_OFF)
   {
   if ((ierr = read_rec (roff, (GENERIC_REC *) &temp_rec)) != NO_ERR)
      return (ierr);

   if (strcmp (temp_rec.data_type, data_type) == 0)
      {
      /*
         record to be deleted found
      */
      if ((ierr = delete_using_cont (roff, &del_off)) != NO_ERR)
	 return (ierr);

      if (*new_off == roff)
	 /*
            record to be deleted is first record in list so no
	    previous record to update
         */
	 *new_off = temp_rec.next_off;
      else
	 {
	 /*
            record to be deleted is not the first record in list so
	    update prev record's next_off
         */
	 prev_rec.next_off = temp_rec.next_off;
	 if (prev_off == 0)
	    {
	    rec1_ptr = ((FIRST_IM_REC *) &prev_rec);
	    rec1_ptr->del_off = del_off;
	    }
	 if ((ierr = write_rec (prev_off, &prev_rec)) != NO_ERR)
	    return (ierr);
	 }

      return (NO_ERR);
      }

   prev_off = roff;
   copy_rec ((GENERIC_REC *) &prev_rec, (GENERIC_REC *) &temp_rec);
   roff = temp_rec.next_off;
   }
/*
   data_type not found in list
*/
return (ERR_DATA);
}


/*---------------------------------------------------------------------------*/
long class_offset_from_index
(
    FIRST_IM_REC *rec,			/* first record of file               */
    long ic			/* class index in array               */
)
{
FIRST_IM_REC	temp_rec;		/* temporary record to read into      */
long		cont_off;		/* next continuation record           */

cont_off = rec->cont_off;
copy_rec (&temp_rec, rec);
while (ic >= CLASSES_PER_RECORD)
   {
   if (read_rec (cont_off, (GENERIC_REC *) &temp_rec) != NO_ERR)
      return (NULL_OFF);
   ic -= CLASSES_PER_RECORD;
   cont_off = temp_rec.cont_off;
   }
return (temp_rec.class_off[ic]);
}


/*---------------------------------------------------------------------------*/
long site_offset_from_index
(
    FIRST_CL_REC *rec,			/* first record of class              */
    long is			/* site index in array                */
)
{
FIRST_CL_REC	temp_rec;		/* temporary record to read into      */
long		cont_off;		/* next continuation record           */

cont_off = rec->cont_off;
copy_rec (&temp_rec, rec);
while (is >= SITES_PER_RECORD)
   {
   if (read_rec (cont_off, (GENERIC_REC *) &temp_rec) != NO_ERR)
      return (NULL_OFF);
   is -= SITES_PER_RECORD;
   cont_off = temp_rec.cont_off;
   }
return (temp_rec.site_off[is]);
}


/*---------------------------------------------------------------------------*/
int write_class_list
(
    FIRST_IM_REC *rec,			/* first record of class list         */
    long offset,			/* offset of above record             */
    long clist[],		/* the list of class offsets          */
    long numc			/* number of classes                  */
)
{
long		cont_off = NULL_OFF;	/* next continuation record           */
long		index;			/* index into class list              */
long		i;			/* loop counter                       */
long		ierr;			/* error status                       */
long		curr_off;		/* offset of current record           */
long		del_off;		/* first deleted record               */
FIRST_IM_REC	temp_rec;		/* a record of class list             */
FIRST_IM_REC	temp_rec2;		/* a record of class list             */

cont_off = rec->cont_off;
curr_off = offset;
index = 0;
copy_rec (&temp_rec, rec);
do {
   for (i = 0; (i < CLASSES_PER_RECORD && index < numc); i++)
      temp_rec.class_off[i] = clist[index++];
   for (i = i; i < CLASSES_PER_RECORD; i++)
      temp_rec.class_off[i] = NULL_OFF;
   if (index == numc)	/* no more records needed */
      {
      temp_rec.cont_off = NULL_OFF;
      write_rec (curr_off, &temp_rec);
      if (cont_off != NULL_OFF)
         delete_using_cont (cont_off, &del_off);
      if (curr_off == 0)
         temp_rec.del_off = del_off;
      return (NO_ERR);
      }
   else
      {
      if (cont_off != NULL_OFF)
         {		/* record is already continued */
         temp_rec.cont_off = cont_off;
         write_rec (curr_off, &temp_rec);
         curr_off = cont_off;
         if ((ierr = read_rec (curr_off, &temp_rec2)) != NO_ERR)
            return (ierr);
         cont_off = temp_rec2.cont_off;
         }
      else		/* record needs to be continued */
         {
         get_empty_off (&cont_off, &del_off);
         temp_rec.cont_off = cont_off;
	 if (curr_off == 0)
	    temp_rec.del_off = del_off;
         write_rec (curr_off, &temp_rec);
	 curr_off = cont_off;
	 cont_off = NULL_OFF;
         }
      }
   temp_rec.next_off = NULL_OFF;
   } while (index < numc);
return (NO_ERR);
}


/*---------------------------------------------------------------------------*/
int write_site_list
(
    FIRST_CL_REC *rec,			/* first record of site list          */
    long offset,			/* offset of above record             */
    long slist[],		/* the list of site offsets           */
    long nums			/* number of sites                    */
)
{
long		cont_off = NULL_OFF;	/* next continuation record           */
long		index;			/* index into site list               */
long		i;			/* loop counter                       */
long		ierr;			/* error status                       */
long		curr_off;		/* offset of current record           */
long		dummy;			/* dummy variable for passing         */
FIRST_CL_REC	temp_rec;		/* a record of site list              */
FIRST_CL_REC	temp_rec2;		/* a record of site list              */

cont_off = rec->cont_off;
curr_off = offset;
index = 0;
copy_rec (&temp_rec, rec);
do {
   for (i = 0; (i < SITES_PER_RECORD && index < nums); i++)
      temp_rec.site_off[i] = slist[index++];
   for (i = i; i < SITES_PER_RECORD; i++)
      temp_rec.site_off[i] = NULL_OFF;
   if (index == nums)	/* no more records needed */
      {
      temp_rec.cont_off = NULL_OFF;
      write_rec (curr_off, &temp_rec);
      if (cont_off != NULL_OFF)
         delete_using_cont (cont_off, &dummy);
      return (NO_ERR);
      }
   else
      {
      if (cont_off != NULL_OFF)
         {		/* record is already continued */
	 temp_rec.cont_off = cont_off;
         write_rec (curr_off, &temp_rec);
         curr_off = cont_off;
         if ((ierr = read_rec (curr_off, &temp_rec2)) != NO_ERR)
            return (ierr);
         cont_off = temp_rec2.cont_off;
         }
      else		/* record needs to be continued */
         {
         get_empty_off (&cont_off, &dummy);
         temp_rec.cont_off = cont_off;
         write_rec (curr_off, &temp_rec);
	 curr_off = cont_off;
	 cont_off = NULL_OFF;
         }
      }
   temp_rec.next_off = NULL_OFF;
   } while (index < nums);
return (NO_ERR);
}


/*---------------------------------------------------------------------------*/
void upcase2
(
    char *c1,			/* output string                      */
    char *c2			/* input string                       */
)
/*
   Make string c2 uppercase and copy into string c1.
*/
{
int	i,				/* counter in output string           */
	j;				/* counter in input string            */

i = j = 0;
while ((c1[i++] = toupper (c2[j++])) != '\0')
   ;
}


#ifdef _IBMR2
#define unix
#endif

#ifdef unix
/*----------------------------------------------------------------------*/
/*-------------------- Unix FORTRAN Bridge routines --------------------*/
/*----------------------------------------------------------------------*/

/*---------------------------- stopen_ ---------------------------------*/

#ifdef _IBMR2
#define stopen_ stopen
#endif

long stopen_ 
(
    FILE **fd,
    char hos[],
    long *access,
    long *nclass,
    long hoslen
)
/*
   This is a bridge routine for calling from FORTRAN on Unix.
*/
{
return (c_stopen (fd, hos, access, nclass));
}


/*---------------------------- stclse_ ---------------------------------*/

#ifdef _IBMR2
#define stclse_ stclse
#endif

int stclse_ 
(
    FILE **fd
)
/*
   This is a bridge routine for calling from FORTRAN on Unix.
*/
{
return (c_stclse (fd));
}


/*---------------------------- stget_ ----------------------------------*/

#ifdef _IBMR2
#define stget_ stget
#endif

int stget_ 
(
    FILE **fd,
    char data_type[],
    long *ic,
    long *is,
    SHAPE_VEC shape,
    unsigned char *buf,
    long data_type_len
)
/*
   This is a bridge routine for calling from FORTRAN on Unix.
*/
{
return (c_stget (fd, data_type, ic, is, shape, buf));
}


/*---------------------------- stput_ ----------------------------------*/

#ifdef _IBMR2
#define stput_ stput
#endif

int stput_ 
(
    FILE **fd,
    char data_type[],
    long *ic,
    long *is,
    SHAPE_VEC shape,
    unsigned char *buf,
    long data_type_len
)
/*
   This is a bridge routine for calling from FORTRAN on Unix.
*/
{
return (c_stput (fd, data_type, ic, is, shape, buf));
}

/*---------------------------- stfdcl_ ---------------------------------*/

#ifdef _IBMR2
#define stfdcl_ stfdcl
#endif

int stfdcl_ 
(
    FILE **fd,
    char class_name[],
    long *ic,
    long class_name_len
)
/*
   This is a bridge routine for calling from FORTRAN on Unix.
*/
{
return (c_stfdcl (fd, class_name, ic));
}


/*---------------------------- stfdsi_ ---------------------------------*/

#ifdef _IBMR2
#define stfdsi_ stfdsi
#endif

int stfdsi_ 
(
    FILE **fd,
    char class_name[],
    char site_name[],
    long *ic,
    long *is,
    long class_name_len,
    long site_name_len
)
/*
   This is a bridge routine for calling from FORTRAN on Unix.
*/
{
return (c_stfdsi (fd, class_name, site_name, ic, is));
}


/*---------------------------- stdldt_ ---------------------------------*/

#ifdef _IBMR2
#define stdldt_ stdldt
#endif

int stdldt_ 
(
    FILE **fd,
    char data_type[],
    long *ic,
    long *is,
    long data_type_len
)
/*
   This is a bridge routine for calling from FORTRAN on Unix.
*/
{
return (c_stdldt (fd, data_type, ic, is));
}


/*---------------------------- stdlnd_ ---------------------------------*/

#ifdef _IBMR2
#define stdlnd_ stdlnd
#endif

int stdlnd_ 
(
    FILE **fd,
    long *ic,
    long *is
)
/*
   This is a bridge routine for calling from FORTRAN on Unix.
*/
{
return (c_stdlnd (fd, ic, is));
}


/*---------------------------- stnode_ ---------------------------------*/

#ifdef _IBMR2
#define stnode_ stnode
#endif

int stnode_ 
(
    FILE **fd,
    long *ic,
    long *is,
    long *id,
    char *data_type[],
    SHAPE_VEC shape,
    long data_type_len
)
/*
   This is a bridge routine for calling from FORTRAN on Unix.
*/
{
return (c_stnode (fd, ic, is, id, data_type, shape));
}
#endif		/* end of Unix bridges */


#ifdef vms
/*----------------------------------------------------------------------*/
/*-------------------- VMS FORTRAN Bridge routines ---------------------*/
/*----------------------------------------------------------------------*/

/*---------------------------- stopen ----------------------------------*/

long stopen 
(
    FILE **fd,
    struct DESC	*d_hos,
    long *access,
    long *nclass
)
/*
   This is a bridge routine for calling from FORTRAN on VMS.
*/
{
char	*hos;

hos = squeeze (d_hos->addr, d_hos->length);

return (c_stopen (fd, hos, access, nclass));

free (hos);
}


/*---------------------------- stclse ----------------------------------*/

int stclse 
(
    FILE		**fd
)
/*
   This is a bridge routine for calling from FORTRAN on VMS.
*/
{
return (c_stclse (fd));
}


/*---------------------------- stget -----------------------------------*/

int stget 
(
    FILE **fd,
    struct DESC	*d_data_type,
    long *ic,
    long *is,
    SHAPE_VEC shape,
    unsigned char *buf
)
/*
   This is a bridge routine for calling from FORTRAN on VMS.
*/
{
char	*data_type;

data_type = squeeze (d_data_type->addr, d_data_type->length);

return (c_stget (fd, data_type, ic, is, shape, buf));

free (data_type);
}


/*---------------------------- stput -----------------------------------*/

int stput 
(
    FILE **fd,
    struct DESC	*d_data_type,
    long *ic,
    long *is,
    SHAPE_VEC shape,
    unsigned char *buf
)
/*
   This is a bridge routine for calling from FORTRAN on VMS.
*/
{
char	*data_type;

data_type = squeeze (d_data_type->addr, d_data_type->length);

return (c_stput (fd, data_type, ic, is, shape, buf));

free (data_type);
}

/*---------------------------- stfdcl ----------------------------------*/

int stfdcl 
(
    FILE **fd,
    struct DESC	*d_class_name,
    long *ic
)
/*
   This is a bridge routine for calling from FORTRAN on VMS.
*/
{
char	*class_name;

class_name = squeeze (d_class_name->addr, d_class_name->length);

return (c_stfdcl (fd, class_name, ic));

free (class_name);
}


/*---------------------------- stfdsi ----------------------------------*/

int stfdsi 
(
    FILE **fd,
    struct DESC	*d_class_name,
    struct DESC	*d_site_name,
    long *ic,
    long *is
    )
/*
   This is a bridge routine for calling from FORTRAN on VMS.
*/
{
char	*class_name,
	*site_name;

class_name = squeeze (d_class_name->addr, d_class_name->length);
site_name  = squeeze (d_site_name->addr, d_site_name->length);

return (c_stfdsi (fd, class_name, site_name, ic, is));

free (class_name);
free (site_name);
}


/*---------------------------- stdldt ----------------------------------*/

int stdldt
(
    FILE **fd,
    struct DESC	*d_data_type,
    long *ic,
    long *is
)
/*
   This is a bridge routine for calling from FORTRAN on VMS.
*/
{
char	*data_type;

data_type = squeeze (d_data_type->addr, d_data_type->length);

return (c_stdldt (fd, data_type, ic, is));

free (data_type);
}


/*---------------------------- stdlnd ----------------------------------*/

int stdlnd 
(
    FILE **fd,
    long *ic,
    long *is
)
/*
   This is a bridge routine for calling from FORTRAN on VMS.
*/
{
return (c_stdlnd (fd, ic, is));
}


/*---------------------------- stnode ----------------------------------*/

int stnode 
(
    FILE **fd,
    long *ic,
    long *is,
    long *id,
    struct DESC	*d_data_type,
    SHAPE_VEC shape
)
/*
   This is a bridge routine for calling from FORTRAN on VMS.
*/
{
return (c_stnode (fd, ic, is, id, d_data_type->addr, shape));
}
#endif		/* end of VMS bridges */



/*----------------------------------------------------------------------*/
/* debug routine */
/*----------------------------------------------------------------------*/

#ifdef unix
void prtoff(FILE **fd);

void prtoff_ 
(
    FILE	**fd
)
{
    prtoff (fd);
}
#endif

void prtoff 
(
    FILE	**fd
)
{
FIRST_IM_REC   *im_rec_ptr;
FIRST_CL_REC   *cl_rec_ptr;
DATA_REC       dat_rec;
long           i, offset, rec_no, minimum;

curr_file_ptr = *fd;
offset = 0;
rec_no = 1;
while (read_rec (offset, &dat_rec) == NO_ERR)
   {
/*   printf ("\n"); */
   printf ("\nOffset = %5ld      Data_type = %-26s      Rec # %3ld\n",
	   offset, dat_rec.data_type, rec_no);
   printf (  "  next = %5ld           cont = %5ld\n",
           dat_rec.next_off, dat_rec.cont_off);
   if (strcmp (dat_rec.data_type, "CLASS-LIST") == 0)
      {
      im_rec_ptr = ((FIRST_IM_REC *) &dat_rec);
      printf ("     .del  = %5ld\n", im_rec_ptr->del_off);
      printf ("     Number of classes = %5ld.\n", im_rec_ptr->num_class);
      if (im_rec_ptr->num_class > 0)
         {
         minimum = (CLASSES_PER_RECORD < im_rec_ptr->num_class) ?
                    CLASSES_PER_RECORD : im_rec_ptr->num_class;
	 for (i = 1; i <= minimum; i++)
	    printf ("          class %4ld: offset = %5ld\n",
		    i, im_rec_ptr->class_off[i - 1]);
         }
      }
   else
      {
      if (strcmp (dat_rec.data_type, "SITE-LIST") == 0)
	 {
/*	 printf ("\n"); */
	 cl_rec_ptr = ((FIRST_CL_REC *) &dat_rec);
	 printf ("     Number of sites   = %5ld.\n", cl_rec_ptr->num_site);
	 if (cl_rec_ptr->num_site > 0)
            {
            minimum = (SITES_PER_RECORD < cl_rec_ptr->num_site) ?
                       SITES_PER_RECORD : cl_rec_ptr->num_site;
	    for (i = 1; i <= minimum; i++)
	       printf ("          site %4ld: offset = %5ld\n",
		       i, cl_rec_ptr->site_off[i - 1]);
            }
	 }
      else
	 {
/*	 printf ("\n"); */
	 }
      }
   offset += 512;
   rec_no++;
   }
printf ("\n");
}
