/******************************************************************************
FUNCTION:       c_errmsg

PURPOSE:
        c_errmsg prints error messages to the TAE session log and to
        TAE's standard output.  If the error is fatal, c_errmsg will
        stop all processing.

EXTERNAL ASSOCIATIONS
        If the error key passed to c_errmsg is to be interpreted as a
        valid TAE error key, a valid TAE error message file should
        exist which contains descriptive text for that error.  If
        the error key is not a TAE error key, then the program
        assumes that it is either a null string or one of "UNIX",
        "unix", "VMS", or "vms."  In each case, c_errmsg creates an
        appropriate key which begins with the facility identification
        "ERROR."  The associated message file for this facility should
        reside in the same directory as the executable code.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      03/85    P. Mumford          Original development.
  2.0      05/86    K. Gacke            changed TAE C callables to 
                                        TAE fortran callables to solve
                                        batch log problems.
  3.0      12/87                        change DESC to char. strings
                                        change name to c_errmsg
  5.0      02/89    B. Davis            fixed bug in for loop of print_mes that
                                        searches for white space to continue
                                        message on next line.  Also, to help
                                        accommodate long file names, if no
                                        white space is found in a line-widths-
                                        worth of message, a line wrap is
                                        allowed to occur at a / for unix host
                                        file names, at a . for vms host file
                                        names, and at a ] for tae file names. 
  5.1      03/89    B. Davis            Refrained from changing the value of
                                        vrity before passing it to z_exit.  It
                                        is possible for a fortran routine to
                                        pass 0, but the address being passed by
                                        fortran recieved by C is not necessarily
                                        an address that is allowed to be written
                                        to.  We will therefore assign a value to
                                        a local variable for passing to z_exit.

                                        In addition, print_mes was calling m_msg
                                        first, then doing subsequent printf's 
                                        when the message was longer than one
                                        line, which corrupted batch log files.
                                        Changed printf to m_msg call.
   5.2      04/89    B.Ailts            Took out the nested comments
   5.3      04/89    B.Ailts            Make sure a -1 is sent to z_exit when
                                        a fatal error code is passed in
   6.0      06/95    T. Wolbrink        Fixed bugs found in PR 36.
                                           -'\0' are now treated same as ' '
                                           -lines now wrap correctly by 
                                                subtracting 1 from back
                                           -input messages are now unlimited in
                                                size constraints        

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run in conjunction with TAE.

PROJECT                 LAS

ALGORITHM 
        If the key is null then check the severity.  If the severity
        is 0 then the key is set to "ERROR-FATAL" and the message
        "Fatal error encountered." is printed, and processing continues.
        Otherwise, the key is "ERROR-NON-FATAL" and the message "Non-fatal
        error encountered." is printed.  If the key is "UNIX" then print
        the message "UNIX error code X" where X is the error code and
        set the key to "ERROR-UNIX".  Likewise, if the key is "VMS",
        print "VMS error code X" where X now represents the VMS error
        code and set the key to "ERROR-VMS."  In all cases, processing
        continues and the error message is written out.

        To print the message, divide it into sections of 80 characters
        or less being sure to end the section on white space.  Print
        out each section seperately to both standard output and the
        session log.  After the message is written out, check the
        severity.

        If the severity is fatal, then stop the processing of the
        proc.  Otherwise, return to the caller.


ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h> 
#include "tae.h"
#include "worgen.h"

static void print_mes
(
    const char *ms,
    const char *ky
);


/**
 * c_errmsg:
 * @message: I: Character string to be displayed.  If the message cannot fit
 *              on an 80 column line it will be reformatted into multiple
 *              lines where the message is split at a "white space"
 *              character (blanks or tabs).
 * @key: I: Message key.  The type of message being displayed.  A null
 *          or a blank key indicates a general error.  The default key
 *          for a general fatal error is "[general-error]", and the key
 *          for a general nonfatal error is "[informational]".  Any other
 *          value is interpreted as a key to be used by the TAE message
 *          facility.  The maximum length of a message key is 17 characters.
 * @vrity: I: Severity level flag.  Zero or negative values indicate
 *            a fatal error and terminate the process.  Positive values
 *            indicate a nonfatal condition and return control to the
 *            calling process. 
 *
 * Deprecated: Use c_lasmsg.c() instead.
 *
 * The c_errmsg() call is used to output an error message in a standard
 * format to the terminal screen and to the TAE session log. If the error
 * is fatal, c_errmsg will stop all processing.
 *
 */             
void c_errmsg
(
    const char *message, /* I: Message associated with the error              */
    const char *key,     /* I: LAS error key, null string, "UNIX" or          */
    const long *vrity    /* I: Severity:                                      */
                                /*    <= 0   fatal                            */
                                /*     > 0   non-fatal                        */
)
{
    int     vflag;              /* int for changing the value of vrity to be  */
                                /* pass to z_exit                             */
    char    os_mes[81];         /* message for OS dependent error             */

    /* Resolve the type of key. */
    if ( *key == '\0' || *key == ' '  )     /* null-string/blank check */
    {
        if ( *vrity <= 0 )                         /* fatal error */
            print_mes(message,"general-error");
        else
            print_mes(message,"informational");
    }
    else if ( (strcmp(key,"unix")==0) || (strcmp(key,"UNIX")==0) )
    {
        sprintf(os_mes,"%s%s%d."," UNIX "," error code ",*vrity); 
        print_mes(os_mes,"ERROR-UNIX");
    }

    else if ( (strcmp(key,"vms")==0) || (strcmp(key,"VMS")==0) )
    {
        sprintf(os_mes,"%s%s%d."," VMS "," error code ",*vrity); 
        print_mes(os_mes,"ERROR-VMS");
    }
    else
        print_mes( message, key );

    /*  Decide whether or not to continue processing. 
-------------------------------------------------*/
    if ( *vrity <= 0 )
    {
        vflag = -1;
        z_exit(vflag,key);
    }

}

    #define MSG_PART_LEN 80
    static void print_mes
(
    const char *ms,
    const char *ky
)

{
    const char *newms;      /* string to pass recursively to print_mes */
    char    restms[CMLEN];  /* pointer to remaining part of ms */
                            /* if it is too long for a one line */
    short   back;           /* counter to break up string MS */

    /* If the length of MS is not greater than MSG_PART_LEN - the length of 
       the key then print it 
    */
    if ( strlen(ms) <= (MSG_PART_LEN-3)-strlen(ky) )
        m_msg(ms,ky);
    else
    {
        for ( back = (MSG_PART_LEN-3) - strlen(ky) - 1;
            ((back>=0) && (!isspace(ms[back])));
              back--)
              ;               /* find w. space to def substring */
        if ( back <= 0 )
        {
            for ( back = (MSG_PART_LEN-3) - strlen(ky) - 1;
                ((back>=0) && (ms[back] != '/') &&
                              (ms[back] != '.') &&
                              (ms[back] != ']'));
                  back--)
                  ;            /* no w. space, find host path separator */
            if ( back <= 0 )
                 back = (MSG_PART_LEN-3) - strlen(ky) - 1;
        }                   /*no path separator, just wrap after 80th char*/
        newms = (ms + back + 1);
        strncpy(restms, ms, back + 1);
        restms[back + 1] = '\0';  /* define substr with null */
        m_msg(restms,ky);
        print_mes( newms, ky );         /* print rest of string */
    }
}
