/*******************************************************************************
FUNCTION:       c_lasmsg

PURPOSE:        Write both nonfatal and fatal error messages to the terminal.
                If a fatal error is issued, this function will automatically
                issue the standard "[xxxxx-fatal] Fatal error encountered" after
                printing the specified error message.


Programmer       Date   Modifications
---------------  -----  --------------------------------------------------------
D.Zokaites       01/95  Development
D.Zokaites       04/95  Update to LAS standards.
D.Zokaites       05/95  Modified to write two error messages when a fatal error
                        key is supplied.  Applications do not need to issue the
                        "[xxxxx-fatal] Fatal error encountered" message any
                        longer.
*******************************************************************************/
#include <string.h>
#include <stdarg.h>     /* For variable number of arguments */
#include "las.h"


#define MSGLEN 1024     /* Maximum message length                            */


/**
 * c_lasmsg:
 * @fatal_flag: I: Flag indicating if the message is a fatal message.
 *                 A nonzero value (TRUE) indicates a fatal message.
 *                 The specified message will be printed to the the
 *                 terminal followed by a standard LAS fatal error
 *                 message.  The program will terminate.  A value of
 *                 zero (FALSE) indicates a nonfatal message.
 *                 The specified message will be printed to the terminal
 *                 and the function will complete successfully.
 * @key: I: LAS message key.  This string will be enclosed in square
 *          brackets at the beginning of the line and should be no more
 *          than 17 characters in length.  If key is an empty string (" ")
 *          or if it contains nothing but spaces, a key of "informational"
 *          will be printed.  Standard message keys are of the form
 *          FUNC-KEY, where FUNC is the name of the function and KEY is
 *          a unique key. 
 * @fmt: I: Message format like that passed to the C function, printf().
 *          If any '%' format specifiers are included in fmt, an additional
 *          argument must be supplied to the function for each specifier.
 *          Please see manual pages for printf().  
 *          The maximum length of the message must be less than 1024
 *          characters.  Longer messages will result in the
 *          over-writing of memory. 
 * @...: I: Additional format arguments in printf format.
 *
 * The c_lasmsg() routine provides the functionality of the C function
 * printf() for printing standard LAS messages.  It calls the LAS
 * support routine c_errmsg() to print all messages to the terminal
 * and save them in the TAE session log.
 *
 * If fatalflag is TRUE, the specified message is printed to the terminal
 * and is examined to  determine if the standard LAS fatal error message
 * should be printed.  If key ends with "-fatal" and the specified message
 * is "Fatal error encountered", the standard fatal error message will not
 * be printed.  If either of these conditions are false, the standard
 * message will be printed using a message key of the form "FUNC-fatal".
 * In either case, the program will terminate.
 *
 * Returns: E_SUCC:  Successful completion, 
 *                   Does not return when fatalflag==TRUE.
 */
int c_lasmsg 
(
    int fatal_flag, 
    const char *key, 
    const char *fmt, 
    ...
)
{

    va_list argumentp;   /* Varible number of arguments for 'format' string  */

    char msg[MSGLEN];    /* Message text sent to c_errmsg()                  */
    char mkey[MSGLEN];   /* Local copy of the message key                    */
    char *ptr;           /* Ptr to a character                               */

    long fatal = -1;     /* Fatal error flag */
    long nonfatal =  1;  /* Nonfatal error flag */

    /* Get the arguments to this function     */
    va_start(argumentp,fmt);
    va_end(argumentp);


    /* Create the desired output message      */
    vsprintf (msg, fmt, argumentp);


    /* Is the key a default                   */
    if ((strcmp(key, "") == 0) || (strcmp(key, " ") == 0))
        strcpy (mkey, "informational");
    else
        strcpy (mkey, key);


    /* If this is a fatal error and the message is the standard "Fatal
       error encountered" message, call c_errmsg with a fatal error
       key.  In such cases there is no need to print an additional
       standard fatal error message.          */
    if (fatal_flag)
    {
        ptr = (char *)strchr(mkey, '-');
        if ((ptr != NULL) && (strcmp(ptr, "-fatal") == 0) &&
            (strcmp(msg, "Fatal error encountered") == 0))
            c_errmsg (msg, mkey, &fatal);
    }


    /* Print the message using c_errmsg()     */
    c_errmsg (msg, mkey, &nonfatal);


    /* If this is a fatal error, issue the standard LAS fatal error message. */
    if (fatal_flag)
    {
        /* The message key should be of the form "function-key".  Transform
        this into the standard key, "function-fatal".  */
        ptr = (char *)strchr(mkey, '-');
        if (ptr == NULL)
        {
            c_errmsg ("Invalid message key", "c_lasmsg-key", &nonfatal);
            strcpy (mkey, "fatal-err");
        }
        else
            strcpy (++ptr, "fatal");


        /* Issue the fatal error              */
        c_errmsg ("Fatal error encountered", mkey, &fatal);
    }

    return (E_SUCC);
}
