/******************************************************************************
NAME:   SQUEEZE

PURPOSE:
        SQUEEZE returns a pointer to a null terminated character string.  It
        searches for the first non-blank character of the input parameter
        "str" from the right.  If the first non-blank character is not
        the null value (0), then the next character is assigned to the null
        value.  This routine is used to null terminate a fortran character
        string.

        Type definitions are defined as follows:
                unsigned char   ==  unsigned 8-bit integer
                short   ==  signed 16-bit integer
                int     ==  signed 32-bit integer
                float   ==  signed 32-bit real
                char    ==  character

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         5/86       K. Gacke     initial development
    1.1        10/86       K. Gacke     if a " " is inputted, the returned
                                        string is null
    1.2        12/87       B. Ailts     change to raw 'C' types
                                        change include directory specifications
    1.2        12/92       T. Mittan    Corrected problem in for loop.  If
                                        a string of blanks are used ptr would
                                        be trying to look before the allocated
                                        string.  Corrected error flag   

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
                Run under TAE

PROJECT:        LAS

ALGORITHM:
        Search from the right of a character string for the first non blank
        character.  If this character is not the null value, then it is 
        assigned to the null value.  This routine is used to null terminate
        fortran character strings.

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"

/**
 * squeeze:
 * @str: I: Character string to be null terminated. 
 * @len: I: Maximum length of the character string (str)
 *          which is to be null terminated.
 *
 * The squeeze() routine is used by FORTRAN callable bridge routines.
 * Any time a C subroutine inputs a character argument from a FORTRAN
 * main routine, the C subroutine should call squeeze() to null
 * terminate the string.  squeeze() truncates all blanks from the right
 * and inserts the null character.  It returns a pointer to the new null
 * terminated character string.
 *
 * Returns: A pointer to the newly null terminated character string.
 * NULL is returned if memory could not be allocated.
 * The caller is responsible for freeing the string that is returned 
 * to prevent a memory leak.
 */
char *squeeze
(
    const char *str,
    int len
)
{
    char *newstr;
    char *ptr;

    newstr = malloc (len+1);
    if (newstr == NULL)
    {
        c_lasmsg (FALSE,"err-alloc","Error allocating dynamic memory");
        return   (NULL);
    }
    strncpy (newstr,str,len);
    for (ptr = newstr + len - 1; ((ptr >= newstr) && (*ptr == ' ')); ptr--)
        ;
    *(ptr + 1) = '\0';
    return (newstr);
}
