/*********************   TAPE I/O Subsystem   *********************************
FUNCTION:	tplsrd

PURPOSE:	Read a label services record from associated file on tape

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
           01/89    D. Hollaren         Modified version of LSREAD to read from
					tape.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
Read each record in the file until EOF or key match is found
    Read header record
    look for a match to the specified key else skip the record
	if the specified character length is less than the actual character
	data length or the specified data length is less than the actual
	data length
	    reset the character and/or data lengths to the actual
	    backup to the beginning of the record
	read the character data
	read the other data
Verify that the lengths of the data requested is the same as the data
lengths to be transferred.
Free up any allocated space.

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <stdio.h>
#include <string.h>

#ifdef vms
#include <file.h>
#else
#include <sys/types.h>
#include <sys/file.h>
#endif

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#endif

#include "vll.h"
#include "tapeio.h"

#define MAX_BLKSIZ 32767

long c_tplsrd
(
    long *unit,		/* tape unit number		(input)		*/
    char *key,		/* input key of record to be read (in/out)	*/
    long *clen,		/* number of char bytes to read	(in/out)	*/
    long *dlen,		/* number of data bytes to read	(in/out)	*/
    char *cbuf,		/* buffer to read char data into  (output)	*/
    unsigned char *dbuf,	/* data buffer to read data into		*/
    char *dtype		/* datatype of data record (I2,B,R4,...) (out)	*/
)
{
static long  charlen;	/* length of the character portion		*/
static long  datalen;	/* length of the data portion			*/
static long  obyte;	/* number of bytes read from tape		*/
static long  count;	/* Looping variable				*/
static long  status;	/* Status variable				*/
long  one = 1;		/* value of one					*/
long  nonfatal = 1;	/* Non-fatal error code				*/
long  mode = HANDLE;	/* mode to handle errors on tape I/O		*/
long  nbyte = MAX_BLKSIZ;/* Maximum size of a label services record	*/

short keymatch = FALSE;	/* Flag for matching the input key		*/

char   *in_key;		/* Key to search for				*/
char   *out_key;	/* Key from the buffer read			*/
char   *ptr;		/* Temp pointer					*/

static char msgtxt[80];	/* Error message text				*/
static char header[HDRL];/* Header information				*/

static unsigned char inbuf[MAX_BLKSIZ];	/* Input buffer			*/
unsigned char *hptr;	/* Pointer for the header information		*/
unsigned char *tptr;	/* Temporary pointer to the input buffer	*/

/*
  Remove any blanks from the input key
*/
in_key = squeeze(key,KEYL);

do
    {
    /*
      Read the label services record from tape
    */
    charlen = datalen = 0;
    obyte = c_tpread(unit,inbuf,&nbyte,&mode);
    if ((obyte == TAPE_EOF) || (obyte == TAPE_EOV) || (obyte == TAPE_EOT))
	return(obyte);
    else if (obyte <= 0)
	{
	c_errmsg("Error reading label services record from tape",
		 "labserv-tpread",&nonfatal);
	return(TAPE_FAIL);
	}

    /*
      Retrieve the header information from the input buffer
    */
    hptr = (unsigned char *) header;
    tptr = inbuf;
    for (count = HDRL; count--; tptr++, hptr++)
	*hptr = *tptr;

    hptr -= HDRL;

    if (((ptr = strchr(header,'/')) != NULL) && (ptr - header < LENL))
        sscanf(header,"%d/%d%s",&charlen,&datalen,dtype);
    else
        sscanf(header,"%d%s",&datalen,dtype);

    out_key = squeeze(header+LENL+TYPL,(long)strlen(header+LENL+TYPL));
    if ((strlen(in_key) == 0) || (strcmp(in_key," ") == 0) 
                              || (strcmp(in_key,out_key) == 0))
        keymatch = TRUE;

    if (strcmp(out_key,"DELETED") == 0)
	keymatch = FALSE;

    if (keymatch) 
	{
	strcpy(key,out_key);

	/*
	  Check to see if buffers are to small
	*/
	if (((*clen <= charlen) && (charlen > 0)) || (*dlen < datalen)) 
	    {
	    /*
	      Set the return character length to the actual character length 
	      from the header plus one for the null terminator.
	    */
	    if (*clen <= charlen)
		*clen = charlen + 1;

	    /*
	      Set the return data length to the actual data length from the
	      header file.
	    */
	    if (*dlen < datalen)
		*dlen = datalen;

	    /*
	      Back space to the beginning of the record so a retry can be done
	    */
	    status = c_tpbsr(unit,&one,&mode);
	    if ((status != TAPE_SUCC) && (status != TAPE_BOT))
	        {
	        c_errmsg("Error back spacing in label services record on tape",	
			 "labserv-tpbsr",&nonfatal);
	        }

	    /*
	      Return status that input buffers are two small for actual record
	    */
	    return(TAPE_SMAL);
    	    }

	 /*
	   Retrieve the character portion from the input buffer
	 */
	if (charlen > 0)
	    {
	    /*
	      Copy the character portion
	    */
	    for (count = charlen; count--; tptr++, cbuf++)
		*cbuf = *tptr;

	    /*
	      rewind the character buffer
	    */
	    cbuf -= charlen;

	    /*
	      Set the return character length with a null terminator
	    */
	    *clen = charlen;
	    *(cbuf + *clen) = '\0';  
	    }
        else
	    *clen = 0;

	/*
	  Retrieve the data portion from the input buffer
	*/
	if (datalen > 0)
	    {
	    /*
	      Copy the data protion
	    */
	    for (count = datalen; count--; tptr++, dbuf++)
		*dbuf = *tptr;

	    /*
	      rewind the data buffer
	    */
	    dbuf -= datalen;

	    /*
	      Set the return data length
	    */
	    *dlen = datalen;
	    }
	else
	    *dlen = 0;
	}
    }
while (!keymatch);

/*
  Check to be sure all the data desired was read
*/
if (*clen + *dlen != charlen + datalen)
    {
    c_errmsg("Error reading from label services file on tape","labserv-tpread",
		&nonfatal);
    sprintf(msgtxt,"    %d bytes requested, %d bytes transferred",
             charlen + datalen,*clen + *dlen);
    c_errmsg(msgtxt,"labserv-bytes",&nonfatal);
    return(TAPE_FAIL);
    }

/*
  Free space allocated for the input key
*/
free(in_key);

/*
  Free space allocated for the output key
*/
free(out_key);

return(TAPE_SUCC);
}
