/*******************************************************************************
NAME:    c_tapelabel

PURPOSE: Using the given information and some system information, create a tape
	 label in free format or standard format.

PROGRAM HISTORY:
VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    T.Baltzer        06/92  EDC   Development
  1.1    D.Etrheim        07/92  EDC   #A6125 Send msg to ops on print failure
  1.2    J.Fenno          08/92  EDC   Modified to truncate hostnames to 4 chars
  1.3    D.Etrheim	  09/92  EDC   Add OPS field as it was in the original 
                                       version
  1.4    D.Etrheim        10/92  EDC   Change thebuf to be an array of pointers.
				       Use c_delfil to delete the temporary 
                                       file name.
  1.5    D.Etrheim        07/94  EDC   Modified to call new function c_tplblpr
  1.6    D.Etrheim        10/94  EDC   Modified to use lp (lpr) for printing
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS		                
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#include "util.h"
#endif

#include "local_protos.h"


static void lib_form 
(
    char *code,        /* IN - Code for the tape                           */
    char *density,     /* IN - Density of the tape                         */
    char *description, /* IN - Description of the tape. Up to 26 chars     */
    char *drive,       /* IN - Tape drive used                             */
    char *project,     /* IN - Project the tape was created for            */
    char *requestor,   /* IN - Initials of the person creating the tape    */
    long retain,	   /* IN - Months to retain past current date          */
    char *sequence,    /* IN - Sequence number of the tape                 */
    char *tapeid,      /* IN - Tape ID number                              */
    char *ops,         /* IN - error flag                                  */
    FILE *file_ptr     /* OUT - Temp file which will contain each label    */
);

static void fre_form 
(
    char *thebuf[11], /* IN - Buffer of information                         */
    FILE *file_ptr    /* IN - Temp file which will contain each label       */
);

/* Define the success status returned from the system command
-------------------------------------------------------------*/
#ifdef unix
#define SYS_SUCC 0
#else
#define SYS_SUCC 1
#endif

int c_tapelabel
(
    char *thebuf[11],  /* IN - buffer of information for LIBRARY FORMAT   */
                       /*      [00] code for the tape                     */
                       /*      [01] density of the tape                   */
                       /*      [02] description of the tape               */
                       /*      [03] drive used                            */
                       /*      [04] project the tape was created for      */
                       /*      [05] requestor.  Initials of tape creator  */
                       /*      [06] sequence number of the tape           */
                       /*      [07] tapeid                                */
                       /*      [08] error flag                            */
                       /*      [09] unused                                */
                       /*      [10] unused                                */
                       /* IN - buffer of information for FREE FORMAT      */
                       /*      [0-10] text in free format                 */

    long retain,   /* IN - Months to retain (used in library format only) */
                   /*      0    = permanent storage in archive            */
                   /*      1-60 = number of months to retain in arhcive   */
                   /*      > 60 = reset to 60                             */
    long printflg, /* IN - Print flag                                     */
                   /*      FALSE = do not print                           */
                   /*      TRUE  = print                                  */
    long formflg,  /* IN - File format flag (0 = FREE, 1 = STANDARD)      */

    char tae_name[] /* OUT - TAE name of file containing the tape label    */
)
{
char file_name[CMLEN]; /* Name of a tmp file used for printing the label      */
char cmd[CMLEN];       /* System command to print the tape label              */
char printer[CMLEN];   /* String containing the printer to use                */
char *ptr;             /* String ptr                                          */
char string[CMLEN];    /* Temporary string                                    */

long nonfatal = 1;     /* Nonfatal error flag                                 */

FILE   *file_ptr;      /* Temp file which will contain each label             */

/* Error check the input parameters if appropriate
---------------------------------------------------*/
if (formflg)
   {
   if (retain < 0)
      {
      c_errmsg("Invalid retension period specified", "c_tapelabel-ret",
	 &nonfatal);
      return(E_FAIL);
      }
   if (retain > 60)
      {
      retain = 60;
      c_errmsg("Retension period reduced to 60 months", " ", &nonfatal);
      }
   }
if ((formflg != 0) && (formflg != 1))
   {
   c_errmsg("Invalid format flag specified", "c_tapelabel-frm", &nonfatal);
   return(E_FAIL);
   }

/* Create a unique file name for printing the tape label.
---------------------------------------------------------*/
ptr = (char *)tmpnam(NULL);
if (ptr == NULL)
   sprintf(file_name, "tplbl.prt");
else
   sprintf(file_name, "%s.prt", ptr);

/* Return the tae name of the temporary file
--------------------------------------------*/
strcpy(tae_name, file_name);

/* Open the tmp file
--------------------*/
file_ptr = fopen(file_name, "w");
if (file_ptr == NULL)
   {
   c_errmsg("Error opening temporary label file","tapelabel-tmplbl",&nonfatal);
   return(E_FAIL);
   }

/* Create a standard or free formatted label
---------------------------------------------*/
if (formflg)
   lib_form(thebuf[0], thebuf[1], thebuf[2], thebuf[3], thebuf[4], thebuf[5],
      retain, thebuf[6], thebuf[7], thebuf[8], file_ptr);
else
   fre_form(thebuf,file_ptr);
fclose(file_ptr);

/* Print the tape label or just issue a message where it is located
-------------------------------------------------------------------*/
if (!printflg)
   {
   c_errmsg("Tape label will not be printed", " ", &nonfatal);
   sprintf(string, "Tape label is contained in %s", tae_name);
   c_errmsg(string, " ", &nonfatal);
   }
else
   {
   /* Retrieve the printer from the configuration file
   ---------------------------------------------------*/
   if (c_tplblpr(printer) != E_SUCC)
      {
      c_errmsg("Error returned from c_tplblpr","tapelabel-call",&nonfatal);
      return(E_FAIL);
      }

   /* If not a standard library form then change the
      printer so that it is not formatted by print
   -------------------------------------------------*/
   if (!formflg)
      printer[6] = '\0';	/* remove the -t from the printer name */

   /* Create system commands to print the tmp file
   ---------------------------------------------------*/
   sprintf(cmd, "lp -c -s -d%s %s", printer, file_name);

   /* Print the file and let the user and ops know if an error
      occurred.  If the print succeeds, delete the file.
   -----------------------------------------------------------*/
   if (system(cmd) != SYS_SUCC)
      {
      c_errmsg("Error occurred printing tape label","tapelabel-print",
	 &nonfatal);
      sprintf(string,"Error occurred printing tape label for tape %s\n",
         thebuf[7]);
      tpmsg(string);
      sprintf(string, "Tape label is contained in %s", tae_name);
      c_errmsg(string, " ", &nonfatal);
      tpmsg(string);
      return(E_FAIL);
      }
   c_delfil(file_name);
   }

return(E_SUCC);
}

/*******************************************************************************
NAME:    LIB_FORM

PURPOSE: Using the given information and some system information, create a tape
         label in standard library archive format.

PROJECT: LAS		                
*******************************************************************************/

static void lib_form 
(
    char *code,        /* IN - Code for the tape                           */
    char *density,     /* IN - Density of the tape                         */
    char *description, /* IN - Description of the tape. Up to 26 chars     */
    char *drive,       /* IN - Tape drive used                             */
    char *project,     /* IN - Project the tape was created for            */
    char *requestor,   /* IN - Initials of the person creating the tape    */
    long retain,	   /* IN - Months to retain past current date          */
    char *sequence,    /* IN - Sequence number of the tape                 */
    char *tapeid,      /* IN - Tape ID number                              */
    char *ops,         /* IN - error flag                                  */
    FILE *file_ptr     /* OUT - Temp file which will contain each label    */
)
{
char cur_time[5];	    /* Current time in 24 hour format HHMM              */
char host_name[CMLEN];  /* Name of the current host                         */
char info_str[CMLEN];
char *ptr;              /* String ptr                                       */
char today[9];		    /* Today's date in YYYYMMDD format                  */
char scratch[9];        /* Scratch date in YYYYMMDD format                  */

long day;		        /* day of month				      */
time_t current_time;    /* current time holder */
long i;                 /* temp value */
long month;		        /* the month 					      */
long ndays[12];         /* number of days in each month (non leap year)     */ 
long nmonths;		    /* number of months to find the number of days in   */
long num_years;	        /* number of years 				      */
long prev_nmonths;	    /* a backup of the number of months		      */
long nonfatal = 1;      /* Nonfatal error flag                              */
long year;	            /* the year					      */

struct tm *date;        /* Structure containing current date and time       */


ndays[ 0] = 31;
ndays[ 1] = 28;
ndays[ 2] = 31;
ndays[ 3] = 30;
ndays[ 4] = 31;
ndays[ 5] = 30;
ndays[ 6] = 31;
ndays[ 7] = 31;
ndays[ 8] = 30;
ndays[ 9] = 31;
ndays[10] = 30;
ndays[11] = 31;

/* Determine the current date and time
--------------------------------------*/
current_time = time(NULL);
date = localtime(&current_time);
date->tm_mon += 1;    /* month is zero indexed */

year = date->tm_year + 1900; /* tm_year is the number of years since 1900 */
month= date->tm_mon;
day = date->tm_mday;

sprintf(today, "%04d%02d%02d",year,month,day);
sprintf(cur_time,  "%02d%02d", date->tm_hour, date->tm_min);


/* determining the "new date" (which will be 
   the date to scratch the tape 
--------------------------------------------*/

nmonths = month + retain;

if (nmonths > 12)
    {
    /* if you go past the end of the year */
    prev_nmonths = nmonths;
    /* finding the month in the new year to scratch the tape */ 
    nmonths = nmonths%12;
    if (nmonths == 0)
	nmonths = month;
    /* number of years to add */
    num_years = (prev_nmonths - nmonths)/12;
    year = year + num_years;
    }

/* Determining if the year is a leap year
   c_isleap will return a 1 if it is a leap year
	                  0 if it is not */
if (c_isleap(&year))
    ndays[1] = 29;

/* checking to see if you can keep the same number of
   the day in the month or not: ie normally 
   1/21/95 would goto 3/31/95 if retain = 2
   BUT if you are at 1/31/95 and we'll go to 
   2/28/95 if retain = 1 and 2/29/96 if retain
   = 13 */
if (day > ndays[nmonths -1])
    day = ndays[nmonths-1];

month = nmonths;


/* Determine the scratch date
-----------------------------*/
if (retain == 0)
   strcpy(scratch,"99999999");
else
   sprintf(scratch,"%04d%02d%02d",year,month,day);


/* Determine the host name.  Check for remotely mounted
   tapes, drive names of the form host@drive.
-------------------------------------------------------*/
ptr = (char *)strchr(drive, '@');
if (ptr != NULL)
   {
   *(ptr++) = '\0';
   strcpy(host_name, drive);
   strcpy(drive, ptr);
   }
else 
   {
   gethostname(host_name, CMLEN);
   }

i = (long)strlen(host_name);
c_up2low(host_name, &i);

/* If the host name is edcxxxx, strip off the edc portion */
if (strncmp(host_name,"edc",3) == 0)
    strcpy(host_name,&host_name[3]);

/* Hostnames must be 4 characters or less.  Translate erosX to vaxX.
--------------------------------------------------------------------*/
if (strncmp(host_name, "eros", 4) == 0)
   {
   strncpy(host_name, "vax", 3);
   host_name[3] = host_name[4];
   host_name[4] = '\0';
   }

/* Drop 1 character out of system name to keep hostnames at 4 characters
------------------------------------------------------------------------*/
else if((strncmp(host_name,"sgs",3) == 0) || (strncmp(host_name,"snw",3) == 0))
   {
   host_name[2] = host_name[3];
   host_name[3] = host_name[4];
   host_name[4] = '\0';
   }
else if ((strncmp(host_name,"sgw",3) == 0))
   {
   host_name[1] = host_name[2];
   host_name[2] = host_name[3];
   host_name[3] = host_name[4];
   host_name[4] = '\0';
   }
else if (strlen(host_name) > 4)
   {
   sprintf(info_str, "Hostname has been truncated from %s to ", host_name);
   host_name[4] = '\0';
   strcat(info_str, host_name);
   c_errmsg(info_str, " ", &nonfatal);
   }

/* Create a tape label file
---------------------------*/
fprintf(file_ptr, "CODE:        %s\n", code);
fprintf(file_ptr, "CREATE:      %s\n", today);
fprintf(file_ptr, "DENSITY:     %s\n", density);
fprintf(file_ptr, "DESCRIPTION: %s\n", description);
fprintf(file_ptr, "DRIVE:       %s\n", drive);
/* This will flag if error creating tape 
----------------------------------------*/
if (strcmp(ops, "ERR") == 0)
   fprintf(file_ptr, "OPERATOR:    ERR\n");
else
   fprintf(file_ptr, "OPERATOR:       \n");
fprintf(file_ptr, "PROJECT:     %s\n", project);
fprintf(file_ptr, "REQUESTOR:   %s\n", requestor);
fprintf(file_ptr, "SCRATCH:     %s\n", scratch);
fprintf(file_ptr, "SEQUENCE:    %s\n", sequence);
fprintf(file_ptr, "SLOT:        %s\n", tapeid);
fprintf(file_ptr, "SYSTEM:      %s\n", host_name);
fprintf(file_ptr, "TIME:        %s\n", cur_time);
}

/*******************************************************************************
NAME:    FREE_FORM

PURPOSE: Using the given information, create a tape label in free format.

*******************************************************************************/

static void fre_form 
(
    char *thebuf[11], /* IN - Buffer of information                         */
    FILE *file_ptr    /* IN - Temp file which will contain each label       */
)
{
long i; /* Loop control	*/

/* Create a tape label file
---------------------------*/
fprintf(file_ptr, "%s\n", "         ");
for (i = 0; i < 11; i++) 
   fprintf(file_ptr, "%s\n", thebuf[i]);
}
