/**************************************************************************
*
*    Function:  tpansi
*
*    Purpose:   Check for ANSI label tape.
*               and inform user of the drive name.
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0         5/97    K. Gacke        Original development
*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*		if not at BOT then
*			return TAPE_SUCC
*		if tapeid equal "CCT" then
*			return TAPE_SUCC (assume unlabeled tape)
*		if access equal TWRITE then
*			write new ANSI label
*		if access equal TUPDATE then
*			read/verify ANSI label
*			forward skip to end of media and backspace one file mark
*		if access equal TREAD then
*			if ansi label present then
*				read/verify ANSI label
*				forward skip one file mark
*			else
*				rewind tape to BOT
*		return TAPE_SUCC
*
*       ALGORITHM REFERENCES:
*		/usr/local/src/ansi
*              ANSI X3.27-1978  American National Standard Magnetic
*              Tape Labels and File Structure for Information Interchange
*                      pp 13-18
*
******************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#include "util.h"
#endif

#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <ssdef.h>
#include <iodef.h>
#endif

/*
 *      Volume-Header #1 Layout
 */
typedef struct
        {
        char    label_id[3];            /* "VOL" */
        char    label_no;               /* "1" */
        char    volume_ident[6];        /* volume identifier */
        char    accessability;          /* accessability */
        char    vol1_spaces1[26];       /* spaces */
        char    owner_ident[14];        /* owner identifier */
        char    vol1_spaces2[28];       /* spaces */
        char    label_version;          /* label-standard version - "3" */
        } VOL1;

#define LABEL_VERSION   '3'
#define ANSI_PREFIX "VOL"
#define ANSI_LABNO  '1'

static VOL1 vol1;

static void init_labels
(
    char *volume_ident,			/* volume identifier	*/
    char *owner_ident			/* owner identifier	*/
);

static void make_label
(
    char *label,			/* address of desired label	*/
    int labelsz,		/* size of label		*/
    char *label_info		/* desired label information	*/
);

static void strnset
(
    char *dest,		/* destination pointer */
    int srce,		/* source data */
    int sz		    /* number of characters to set */
);

int c_tpansi
(
    long *unit,		/* (I/O) pointer to tape descriptor */
    long *mode,		/* (I)   error mode */
    char *dens,     /* (I)   tape density */
    long *inout,	/* (I)   tape access (TWRITE,TUPDATE,TREAD) */
    char *message,  /* (I/O) message to the operator */
    char *tapeid	/* (I/O) tape ID */
)
{
struct TAPEDESC *drive;		/* tape descriptor structure */
long count;			/* count paramter */
long eofopt;			/* number of tape marks */
long lblflg;			/* label flag -- TRUE if ANSI label is present*/
long len;			/* string length */
long nonfatal = 1;		/* non fatal error message code */
long rewopt = REWIND;		/* tape rewind option */
long rstat;			/* return status code */
long skip;			/* number of files to skip */
long status;			/* function status code */
long tidflg;			/* tape id flag -- TRUE if tape matches label */
char emsg[256];			/* error message buffer */
char *ptr;			/* character pointer */
char mntmsg[256];		/* tape mount message */
char tpid[256];			/* local buffer to store tape ID */
char username[256];		/* user name to be owner of the tape */

rstat = TAPE_SUCC;
drive = (struct TAPEDESC *) *unit;
getusern(username);
strcpy(tpid,tapeid);
len = (long)strlen(tpid);
c_low2up(tpid,&len);
/*
*  If tape is not at BOT, do not process the ANSI label.
*/
if (!testbot(drive))
    goto xit;
/*
*  If tape is a non-archive tape, do not process the ANSI Label.
*/
if (strncmp(tpid,"CCT",3) == 0)
    goto xit;
if (*inout == TWRITE)
    {
    init_labels (tpid,username);
    len = sizeof(vol1);
    /* NOTE: this treatment of vol1 as a character buffer is extremely risky
             and non-portable since it treats a structure as a character 
             buffer in a known order.  Compilers are free to reorder structures
             as they see fit, so this may not end up in the expected order.
             I'm not going to fix it since it's been this way forever... */
    status = c_tpwrit(unit,(unsigned char*)&vol1,&len,mode);
    if (status != len)
	{
	sprintf(emsg,"Error writing ANSI label, status=%d",status);
	tp_errmsg(emsg," ",&nonfatal);
	rstat = TAPE_FAIL;
	goto xit;
	}
    count = 1;
    status = c_tpweof(unit,&count,mode);
    if (status != TAPE_SUCC)
	{
	sprintf(emsg,"Error writing tape mark after ANSI label, status=%d",
		status);
	tp_errmsg(emsg," ",&nonfatal);
	rstat = TAPE_FAIL;
	goto xit;
	}
    }
else 
    {
    lblflg = TRUE;
    tidflg = TRUE;
    strcpy(tpid,tapeid);
    len = (long)strlen(tpid);
    c_low2up(tpid,&len);
    status = c_tpread(unit,(unsigned char *)drive->rah,(long *)&drive->maxbyte,mode);
    if (status != sizeof(vol1))
        lblflg = FALSE;
    if (lblflg)
	{
	memcpy((char *) &vol1,drive->rah,status);
	if (strncmp(vol1.label_id,ANSI_PREFIX,strlen(ANSI_PREFIX)) != 0)
	    {
	    lblflg = FALSE;
	    if (*mode == HANDLE)
		{
		len = (long)strlen(ANSI_PREFIX);
	        sprintf(emsg,"ANSI Label has %*.*s rather than %s prefix",
			len,len,vol1.label_id,ANSI_PREFIX);
	        tp_errmsg(emsg," ",&nonfatal);
		}
	    }
	if (vol1.label_no != ANSI_LABNO)
	    {
	    lblflg = FALSE;
	    if (*mode == HANDLE)
		{
	        sprintf(emsg,"ANSI Label has %c rather than %c label no",
			vol1.label_no,ANSI_LABNO);
	        tp_errmsg(emsg," ",&nonfatal);
		}
	    }
	if (vol1.label_version != LABEL_VERSION)
	    {
	    lblflg = FALSE;
	    if (*mode == HANDLE)
		{
	        sprintf(emsg,"ANSI Label has %c rather than %c label version",
			vol1.label_version,LABEL_VERSION);
	        tp_errmsg(emsg," ",&nonfatal);
		}
	    }
	}
    if (lblflg) 
	{
	ptr = strchr(vol1.volume_ident,' ');
	if (ptr == NULL)
	    len = (long)strlen(tpid);
	else
	    len = ptr - vol1.volume_ident;
	if (strncmp(vol1.volume_ident,tpid,len) != 0)
	    {
            tidflg = FALSE;
	    if (*mode == HANDLE)
		{
	        sprintf(emsg,"ANSI Label tape IDs do not match %*.*s != %s",
			len,len,vol1.volume_ident,tpid);
	        tp_errmsg(emsg," ",&nonfatal);
		}
	    }
	ptr = strchr(vol1.owner_ident,' ');
	if (ptr == NULL)
	    len = (long)strlen(username);
	else
	    len = ptr - vol1.owner_ident;
	if (strncmp(vol1.owner_ident,username,len) != 0)
	    {
	    if (*mode == HANDLE)
		{
	        sprintf(emsg,"ANSI Label tape owner does not match %*.*s != %s",
			len,len,vol1.owner_ident,username);
	        tp_errmsg(emsg," ",&nonfatal);
		}
	    }
	}
    /*
    *  No ANSI label; therefore, rewind tape to BOT
    */
    if (!lblflg)
        {
        rewopt = REWIND;
        c_tprew(unit, mode, &rewopt);
        if (*inout != TREAD)
	    {
            sprintf(emsg,"TapeID %s does not have ANSI Label ID; %s",
		tapeid,"therefore, unable to append data to tape");
	    tp_errmsg(emsg," ",&nonfatal);
	    rstat = TAPE_FAIL;
	    goto xit;
	    }
        }
    /*
    *  ANSI label present, but it does not match tape ID entered by 
    *  operator; therefore; dismount the tape, re-issue mount request,
    *  and re-call tpansi.
    */
    else if (!tidflg)
        {
        rewopt = REWOFFL;
	eofopt = 0;
        c_tpclos(unit, mode, &rewopt, &eofopt);
	skip = 0;
	rewopt = REWIND;
	for (count=0,rstat=TAPE_FAIL; (count<3) && (rstat!=TAPE_SUCC); count++)
	    {
	    sprintf(mntmsg,"tpansi: WRONG tape mounted -- %s",message);
	    rstat = c_tpopen(unit,&skip,mode,&rewopt,dens,inout,mntmsg,tapeid);
	    if (rstat == TPOPEN_NOAVAIL)
		break;
	    }
	if (rstat == TAPE_SUCC)
	    {
	    rstat = c_tpansi(unit,mode,dens,inout,message,tapeid);
	    }
	goto xit;
    	}
    /*
    *  ANSI label is present and it matches tape ID entered by operator;
    *  therefore; 
    *    if open for READ access -- forward skip to beginning of next file
    *    if open for TUPDATE access -- forward skip to end of media.
    */
    else
        {
        if (*inout == TREAD)
	    count = 1;
        else
            count = -1;
        status = c_tpfsf(unit,&count,mode);
        if ((status != TAPE_SUCC) && 
	    (status != TAPE_EOV)  && 
	    (status != TAPE_EOT)  && 
	    (status != TAPE_EOF))
	    {
	    sprintf(emsg,"Error forward spacing to EOM, status = %d",status);
            tp_errmsg(emsg," ",&nonfatal);
            rewopt = REWIND;
            c_tprew(unit, mode, &rewopt);
	    rstat = TAPE_FAIL;
	    goto xit;
	    }
        }
    }
xit:
return(rstat);
}


static void init_labels
(
    char *volume_ident,			/* volume identifier	*/
    char *owner_ident			/* owner identifier	*/
)
{

/***   Initialize VOL1 Label   ***/

make_label ((char *) &vol1,sizeof(vol1),"VOL1");
strncpy ((char *)vol1.volume_ident,volume_ident,
	MIN(sizeof(vol1.volume_ident),strlen(volume_ident)));
strncpy ((char *)vol1.owner_ident,owner_ident,
	MIN(sizeof(vol1.owner_ident),strlen(owner_ident)));
vol1.label_version = LABEL_VERSION;
}


static void make_label
(
    char *label,			/* address of desired label	*/
    int labelsz,		/* size of label		*/
    char *label_info		/* desired label information	*/
)
{
strnset (label,' ',labelsz);
strncpy (label,label_info,MIN(sizeof(label),strlen(label_info)));
}

/**************************************************************************/
/***   strnset fills dest with sz occurrances of the character in src   ***/
/**************************************************************************/

static void strnset
(
    char *dest,		/* destination pointer */
    int srce,		/* source data */
    int sz		/* number of characters to set */
)
{
for (; sz--; *dest++ = srce)
	;
}
