/**************************************************************************
*
*    Function:  tpopen
*
*    Purpose:   Allocate the tape drive specified by operator or system default
*               and inform user of the drive name.
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name. 
					Replaced DESC arguments with char 
					strings. Standardized error msg.
    2.1         6/89    K.Gacke         Port to SUN 4
    2.2         2/90    B.Ailts         Checked for allocation errors
    2.3         6/90    K. Gacke        Modified tape I/O so that if the tape 
					is opened for write access, it is 
					actually opened for read/write access
    2.4		7/90	K.Gacke		output system status code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    4.0		9/91	K.Gacke		Add tapeid parameter 
    4.1	       11/93	K.Gacke		Add tape length to mount request
*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       Squeeze the message
*       Do
*          Increment number of tries
*          If number of tries exceeds ten then return with error message
*          Retreive the tape number from the operator
*          Open the tape drive
*          Output correct error msg depending upon status of open
*       While status != TAPE_SUCC
*       Rewind tape is specified
*       Skip the number of files specified
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#endif

#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <ssdef.h>
#include <iodef.h>
#endif

char Tape_error[256]="";	/* tape error buffer */
long Tapelen = NULLVAL;		/* tape length */

int c_tpopen
(
    long *unit,             /*  (O)   tape unit */
    long *skip,             /*  (I)   files to be skipped */
    long *mode,             /*  (I)   error mode */
    long *opt1,             /*  (I)   rewind flag */
    char *dens,             /*  (I)   tape density */
    long *inout,            /*  (I)   type of access */
    char *message,          /*  (I/O) message to the operator */
    char *tapeid           /*  (O)   tape id */
)
{
    char tapenam[CMLEN];
    char cmd[256];
    long loc_stat;
    long local_mode;
    long nonfatal = 1;
    long ntries;
    int  sig_mask;
    long status;
    struct TAPEDESC *drive;
    char cerr[ERRLEN];
    char locmsg[CMLEN];
    char lmsg[CMLEN];
    char *mount_msg;
    int i;

    mount_msg = squeeze(message,CMLEN);
    strcpy(locmsg,mount_msg);
    free(mount_msg);
    mount_msg = NULL;
    drive = (struct TAPEDESC *) malloc(sizeof(struct TAPEDESC));
    if (drive == NULL)
        return(tperror(drive,TPOPEN_OPEN,*mode));
    *unit = (long)drive;
    drive->bot = drive->eof_flg = drive->eof = drive->eot = drive->err = 0;
    drive->fd = drive->flags = drive->inout = drive->maxbyte = 0;
    drive->minbyte = drive->dev_type = drive->netflg = 0;
    drive->lhost[0] = drive->name[0] = '\0';
    drive->thost[0] = drive->density[0] = '\0';
    local_mode = ~HANDLE;
    ntries = 0;

    for (status=TAPE_FAIL; status!=TAPE_SUCC; )
    {
        ++ntries;
        if (ntries > 10)
            return(TPOPEN_OPEN);
        status = get_tape_nam(tapenam, tapeid, &Tapelen, locmsg);
        if (status != TAPE_SUCC)
        {
            tperror(drive,status,HANDLE);
            return(status);
        }
        status = tprdconf(drive,tapenam,dens,*inout);

        if (status == TPOPEN_BADENS)
            return(tperror(drive,status,HANDLE));
        else if (status != TAPE_SUCC)
        {
            tperror(drive,status,HANDLE);
            continue;
        }
        if (!drive->netflg)			/* open local tape drive */
        {
            tpsig_blk(&sig_mask);
            status = open_tape(drive,TOPEN);
            tpsig_unblk(&sig_mask);
        }
        else				/* open remote tape drive */
        {
#ifdef NETWORK
            status = nt_tpopen(drive,tapenam);
            if (strcmp(drive->dtype,"cd") == 0)
            {
                sprintf(cerr,"Operator assigned CD drive %s\n", tapenam);
                sprintf(cmd,"/usr/local/bin/mntcd_nfs %s",drive->lname);
                loc_stat = system(cmd);
                if (loc_stat != 0)
                    return(TPOPEN_OPEN);
            }
#else
            status = TP_NONET;
#endif
        }
        if (status != TAPE_SUCC)
        {
            tperror(drive,status,*mode);
            switch (status)
            {
                case TPOPEN_OFFLINE:
                    sprintf(lmsg,"\n\n%s %s %s",
                            "Remount tape on drive",tapenam,
                            "and ensure drive is online");
                    break;
                case TPOPEN_WLOCK:
                    sprintf(lmsg,
                            "\n\nRemount tape on drive %s %s %s",tapenam,
                            "with a write ring and ensure tape drive",
                            "is online");
                    break;
                case TAPE_FAIL:
                case TPOPEN_OPEN:
                default:
                    sprintf(lmsg,
                            "\n\nCheck drive %s and remount the tape.",tapenam);
                    break;
            }
            tpmsg(lmsg);
        }
    }
    for (i=0; i<CMLEN; i++)
        message[i] = ' ';
    sprintf(message,"%s",tapenam);
    if (strcmp(drive->dtype,"cd") == 0)
        sprintf(cerr,"Operator assigned CD drive %s\n", tapenam);
    else
        sprintf(cerr,"Operator assigned tape drive %s\n", tapenam);
    tp_errmsg(cerr," ",&nonfatal);

    if ((*opt1 == REWIND) && (las_is_tape(drive)))
    {
        status = c_tprew(unit, &local_mode, opt1);
        if (status != TAPE_SUCC)
        {
            tperror(drive,TPOPEN_REW,*mode);
            return(TPOPEN_REW);
        }
    }
    if (*skip > 0)
    {
        status = c_tpfsf(unit, skip, &local_mode);
        if ((status != TAPE_SUCC) && (status != TAPE_EOV))
        {
            tperror(drive,TPOPEN_SKIP,*mode);
            return(TPOPEN_SKIP);
        }
    }
    tplen(drive);
    return(TAPE_SUCC);
}

/* 
* FUNCTION c_tpstacker assumes an SGI tape library can load/unload
* tapes via the "stacker" command.
*/
int c_tpstacker
(
    char *dname,		/* drive name */
    char *opcode,		/* operation code	*/
				/*    l => load		*/
				/*    u => unload	*/
    long *slot,             	/* library slot         */
    long drvnmr			/* drive number 	*/
)
{
FILE *fp;
struct TAPEDESC drive;
long crflg;
long lslot;
long status;
char cmd[1024];
char fname[1024];
char *ptr;

if ((strcmp(opcode,"l") != 0) && (strcmp(opcode,"u") != 0))
    {
    fprintf(stderr,"tpstacker: opcode must be 'l' or 'u' for load/unload\n");
    return(TAPE_FAIL);
    }
#ifdef NETWORK
if (strchr(dname,'@') != NULL)	
    {
    status = nt_tpstacker(dname,opcode,slot,drvnmr);
    return(status);
    }
#endif
lslot = *slot;
status = tprdconf(&drive,dname,"0",TREAD);
if (status != TAPE_SUCC)
    return(tperror(&drive,status,HANDLE));
ptr = strrchr(drive.stacker,'/');
if (ptr == NULL)
    ptr = drive.stacker;
sprintf(fname,"/tmp/%s",ptr);
if (access(fname,0) != 0)
    {
    crflg = TRUE;
    fp = fopen(fname,"w+");
    }
else
    {
    crflg = FALSE;
    fp = fopen(fname,"r+");
    }
if (fp == NULL)
    {
    fprintf(stderr,"tpstacker: error opening slot file <%s>\n",fname);
    return(TAPE_FAIL);
    }
if (crflg)
    chmod(fname,0666);
if (lslot < 1)
    {
    if (fscanf(fp,"%d",&lslot) != 1)
	{
	fprintf(stderr,"Error reading slot from slot file\n");
	lslot = 0;
	}
    if (strcmp(opcode,"l") == 0) 
        ++lslot;
    }
fseek(fp,0,SEEK_SET);
fprintf(fp,"%d\n",lslot);
fclose(fp);
if (drvnmr < 1)
    drvnmr = 1;
sprintf(cmd,"stacker -%s %d -d%d %s",opcode,lslot,drvnmr,drive.stacker);
/* fprintf(stderr,"tpstacker: cmd=<%s>\n",cmd); */
status = system(cmd);
if (status != 0)
    {
    fprintf(stderr,"tpstacker: error in system command <%s>\n",cmd);
    return(TAPE_FAIL);
    }
else
    {
    *slot = lslot;
    return(TAPE_SUCC);
    }
}
