/**************************************************************************
*	
*	NAME: tprdconf
*
*	PURPOSE:  
*		Read the tape configuration file, which defines the tape
*		drives for all computer systems.  The file name is 
*		"tape.config" in a directory defined by the WORCONFIG 
*		global variable.
*
   Program History:
   Date	Contrib		Change
   ----	-------		------
   1/91	K.Gacke		tape config file
   3/91	K.Gacke		Add Silicon Graphics
   7/91	K.Gacke		Add drive type field to tape.config
   5/93	K.Gacke		Add sgi jag (diff scsi)
  11/93	K.Gacke		Added CDROM support.  Modified error msg 
   6/95	K.Gacke		Fixed bug in allowing # of tape drives > 10
   8/95 J.Fenno         Changed WORCONFIG to LASTABLES
   8/95 K.Gacke		IRIX 5.3 support, added cdr as device type
   9/95 K.Gacke		DGUX 5.4R3.10 support
  11/95 K.Gacke         Modified for IRIX 5.3 (Fugitsu 9-track)
   1/96 K.Gacke         Port to Solaris
   2/97 K.Gacke		CD-R flags for writer speed and availability of Rimage
			printer

*
*	PROJECT: LAS
*
*	ALGORITHM DESCRIPTION:
*
*	ALGORITHM REFERENCES:
*
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#include "util.h"
#endif

#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <ssdef.h>
#include <iodef.h>
#endif

#define E_ESCAPE 100

/*
*  Valid Tape Densities
*	0	Default density setting (dendef)
*	800	9-track 800bpi
*	1600	9-track 1600bpi
*	6250	9-track 6250bpi
*	LO	Low density, no compression
*	LOC	Low density, compression
*	MED	Medium density
*	HI	High density, no compression
*	HIC	High density, compression
*/
char *_TPDENS[] = {"0", "800", "1600", "6250", "LO", "MED", "HI", "LOC", "HIC"};

/*
*  For Linux, the density and compression flags are set when the tape drive
*  is opened with an mtio command.  Need to save the tape density index.
*/
long TPDEN_Index;
long TPDEN_Flg;

int tprdconf
(
    struct TAPEDESC *drive,
    char *tapenam,
    char *density,
    long inout
)
{
struct TPREC tprec;
FILE *fp;
long cnt;
long cnt1;
long errflg;
long found;
long len;
long nonfatal = 1;
long rstat;
long status;
char emsg[CMLEN];
char *ptr;
char tpconfig[CMLEN];
char buf[1024];
char dname[CMLEN];
char *device[NMR_TAPE_DEV];
char tname[CMLEN];

device[VAX_GEN]      = "vax_gen";
device[VAX_FSF]      = "vax_fsf";
device[SOLARIS_SCSI] = "solaris_scsi";
device[SGI_XY] 	     = "sgi_xy";
device[IRIX_EXABYTE] = "irix_exabyte";
device[IRIX_SCSI]    = "irix_scsi";
device[DG54_GEN]     = "dg54_gen";
device[CD]	     = "cd";
device[CDR]	     = "cdr";
device[IRIX53_EXABYTE] = "irix53_exabyte";
device[IRIX53_SCSI]  = "irix53_scsi";
device[DVD]	     = "dvd";
device[DVDR]	     = "dvdr";
device[LINUX_SCSI]   = "linux_scsi";
device[LINUX_TS]     = "linux_ts";
drive->inout = (inout == TWRITE) ? TUPDATE : inout;
drive->err = 0;
strcpy(drive->stacker,"");
strcpy(tname,tapenam);
len = (long)strlen(tname);
c_up2low(tname,&len);
if (gethostname(drive->lhost,CMLEN) != 0)
    return(TPOPEN_LHOST);
len = (long)strlen(drive->lhost);
c_up2low(drive->lhost,&len);
ptr = strchr(tname,'@');
if (ptr == NULL)
    {
    strcpy(drive->thost,drive->lhost);
    strcpy(dname,tname);
    }
else
    {
    *ptr = '\0';
    strcpy(drive->thost,tname);
    strcpy(dname,ptr+1);
    }
drive->netflg = (strcmp(drive->lhost,drive->thost) == 0) ? FALSE : TRUE;
if (tpgetconf(tpconfig) == TAPE_FAIL)
    return(TAPE_FAIL);
fp = fopen(tpconfig,"r");
if (fp == NULL)
    {
    printf("Error opening tape configuration file %s\n",tpconfig);
    return(TAPE_FAIL);
    }
for (found=FALSE; !found; )
    {
    errflg = FALSE;
    init_tprec(&tprec);
    for (cnt=0; ; cnt++)
        {
        status = tpconfig_getfield(fp,buf);
        if ((status == E_EOL) || (status == E_EOF))
	    break;
        switch (cnt)
            {
            case 0:		/* first field is logical drive name */
                strcpy(tprec.drive,buf);
	        break;
            case 1:
                strcpy(tprec.host,buf);
	        break;
            case 2:
                strcpy(tprec.dtype,buf);
	        break;
            case 3:
                for (cnt1=0; cnt1<NMR_TAPE_DEV; cnt1++)
	            {
	            if (strcmp(device[cnt1],buf) == 0)
		        {
		        tprec.dev_type = cnt1;
			if (tprec.dev_type == IRIX53_EXABYTE)
			    tprec.dev_type = IRIX_EXABYTE;
			if (tprec.dev_type == IRIX53_SCSI)
			    tprec.dev_type = IRIX_SCSI;
		        break;
		        }
	            }
	        if (cnt1 == NMR_TAPE_DEV)
	            {
	            sprintf(emsg,
		      "tprdconf: invalid device type <%s> in config file",buf);
	            errflg = TRUE;
	            }
	        break;
            case 4:
                strcpy(tprec.devnam,buf);
	        break;
            default:
	        ptr = strchr(buf,'=');
	        if (ptr == NULL)
	            get_tpflag(buf,&tprec);
	        else
		    {
		    if (strncmp(buf,"stacker",7) != 0)
	                get_density(buf,&tprec);
		    else
			strcpy(drive->stacker,ptr+1);
		    }
	        break;
            }
        }
    if (status == E_EOF)
	break;
    if ((strcmp(drive->thost,tprec.host)==0) &&
        (strcmp(dname,tprec.drive)==0))
        {
        found = TRUE;
        }
    }
fclose(fp);
if (found)
    {
    if (errflg)
	{
	tp_errmsg(emsg," ",&nonfatal);
	return(TAPE_FAIL);
	}
    strcpy(drive->density,density);
    drive->flags = tprec.flags;
    drive->dev_type = tprec.dev_type;
    strcpy(drive->lname,tapenam);
    strcpy(drive->dtype,tprec.dtype);
    rstat = tpconfig_assdrv(drive);
    if (rstat != TAPE_SUCC)
	return(rstat);
    rstat = tpconfig_assden(drive,&tprec);
    if (rstat != TAPE_SUCC)
	return(rstat);
    }
if (!found)
    return(TPOPEN_DRNUM);
return(rstat);
}


void init_tprec
(
    struct TPREC *tprec
)
{
long cnt;

for (cnt=0; cnt<DENSCT; cnt++)
    tprec->drvnmr[cnt][0] = '\0';
tprec->flags = 0x0;
tprec->devnam[0] = tprec->drive[0] = tprec->host[0] = tprec->dtype[0] = '\0';
}


void get_density
(
    char *cmd,
    struct TPREC *tprec
)
{
char *ptr;

ptr = strchr(cmd,'=');
if (ptr == NULL)
    return;
*ptr = '\0';
if (strcmp(cmd,"dendef") == 0)
    sscanf(ptr+1,"%s",tprec->drvnmr[DEN_DEF]);
else if (strcmp(cmd,"den800") == 0)
    sscanf(ptr+1,"%s",tprec->drvnmr[DEN_800]);
else if (strcmp(cmd,"den1600") == 0)
    sscanf(ptr+1,"%s",tprec->drvnmr[DEN_1600]);
else if (strcmp(cmd,"den6250") == 0)
    sscanf(ptr+1,"%s",tprec->drvnmr[DEN_6250]);
else if (strcmp(cmd,"denlo") == 0)
    sscanf(ptr+1,"%s",tprec->drvnmr[DEN_LO]);
else if (strcmp(cmd,"denmed") == 0)
    sscanf(ptr+1,"%s",tprec->drvnmr[DEN_MED]);
else if (strcmp(cmd,"denhi") == 0)
    sscanf(ptr+1,"%s",tprec->drvnmr[DEN_HI]);
else if (strcmp(cmd,"denloc") == 0)
    sscanf(ptr+1,"%s",tprec->drvnmr[DEN_LO_C]);
else if (strcmp(cmd,"denhic") == 0)
    sscanf(ptr+1,"%s",tprec->drvnmr[DEN_HI_C]);
else
    printf("%s=%s is an invalid density specification\n",cmd,ptr+1);
return;
}

/******************************************************************************
	Assign the tape density.  

	On VAX/VMS the same device name is used for all valid 
	densities.  For these two systems it is assumed the actual density 
	value is stored in the tprec.drvnmr array.  The density field is
	then passed as a parameter to the tape open subroutine.

	On Unix machines a different device name is used for each valid
	density.  For each tape device, the device name is appended with
	the value stored in the tprec.drvnmr array to obtain the correct
	density.
******************************************************************************/
int tpconfig_assden
(
    struct TAPEDESC *drive,
    struct TPREC *tprec
)
{
extern char *_TPDENS[];
char density[8];
long found;
long rstat;

rstat = TAPE_SUCC;
if (strlen(drive->density) == 0)
    strcpy(drive->density,"0");
switch (drive->dev_type)
    {
    case VAX_GEN:		/* vms and primos system device name is the  */
    case VAX_FSF:		/* same for all densities.  Density is passed*/
    				/* as a parameter to the tape open call.     */
	found = FALSE;
	for (TPDEN_Index=0; (!found) && (TPDEN_Index<DENSCT); TPDEN_Index++)
	    {
	    if (strcmp(drive->density,_TPDENS[TPDEN_Index]) == 0)
	        {
	        if (tprec->drvnmr[TPDEN_Index][0] == '\0')
		    {
		    printf("Invalid density value specified for drive %s\n",
			        drive->name);
		    rstat = TPOPEN_BADENS;
		    }
	        sscanf(tprec->drvnmr[TPDEN_Index],"%s",density);
		found = TRUE;
	        }
	    }
	if (!found)
	    {
	    printf("Invalid density value specified for drive %s\n",
		        drive->name);
	    rstat = TPOPEN_BADENS;
	    }
	strcpy(drive->name,tprec->devnam);
	strcpy(drive->density,density);
	break;
    case CD:
    case CDR:
    case DVD:
    case DVDR:
	strcpy(drive->name,tprec->devnam);
	break;
    case LINUX_SCSI:
	found = FALSE;
	for (TPDEN_Index=0; (!found) && (TPDEN_Index<DENSCT); TPDEN_Index++)
	    {
	    if (strcmp(drive->density,_TPDENS[TPDEN_Index]) == 0)	
	        {
	        if (tprec->drvnmr[TPDEN_Index][0] == '\0')
		    {
		    printf("Invalid density value specified for drive %s\n",
			        drive->name);
		    rstat = TPOPEN_BADENS;
		    }
	        strcpy(drive->name,tprec->devnam);
		if (sscanf(tprec->drvnmr[TPDEN_Index],"%x",&TPDEN_Flg) != 1)
		    TPDEN_Flg = 0;
		found = TRUE;
	        }
	    }
	if (!found)
	    {
	    printf("Invalid density value specified for drive %s\n",
		        drive->name);
	    rstat = TPOPEN_BADENS;
	    }
	if (strcmp(drive->density,"0") == 0)
	    {
	    for (TPDEN_Index=1; TPDEN_Index<DENSCT; TPDEN_Index++)
		{
		if (strcmp(tprec->drvnmr[0],tprec->drvnmr[TPDEN_Index]) == 0)
		    {
		    strcpy(drive->density,_TPDENS[TPDEN_Index]);
		    break;
		    }
		}
	    }
	break;
    default:		/* unix systems density specified by device name */
	found = FALSE;
	for (TPDEN_Index=0; (!found) && (TPDEN_Index<DENSCT); TPDEN_Index++)
	    {
	    if (strcmp(drive->density,_TPDENS[TPDEN_Index]) == 0)	
	        {
	        if (tprec->drvnmr[TPDEN_Index][0] == '\0')
		    {
		    printf("Invalid density value specified for drive %s\n",
			        drive->name);
		    rstat = TPOPEN_BADENS;
		    }
	        sprintf(drive->name,"%s%s",tprec->devnam,tprec->drvnmr[TPDEN_Index]);
		found = TRUE;
	        }
	    }
	if (!found)
	    {
	    printf("Invalid density value specified for drive %s\n",
		        drive->name);
	    rstat = TPOPEN_BADENS;
	    }
	if (strcmp(drive->density,"0") == 0)
	    {
	    for (TPDEN_Index=1; TPDEN_Index<DENSCT; TPDEN_Index++)
		{
		if (strcmp(tprec->drvnmr[0],tprec->drvnmr[TPDEN_Index]) == 0)
		    {
		    strcpy(drive->density,_TPDENS[TPDEN_Index]);
		    break;
		    }
		}
	    }
	break;
    }
return(rstat);
}

void get_tpflag
(
    char *cmd,
    struct TPREC *tprec
)
{
if (strcmp(cmd,"nobsf") == 0)
    tprec->flags |= NOBSF;
else if (strcmp(cmd,"nobsr") == 0)
    tprec->flags |= NOBSR;
else if (strcmp(cmd,"nofsf") == 0)
    tprec->flags |= NOFSF;
else if (strcmp(cmd,"nofsr") == 0)
    tprec->flags |= NOFSR;
else if (strcmp(cmd,"noeot") == 0)
    tprec->flags |= NOEOT;
else if (strcmp(cmd,"noapp") == 0)
    tprec->flags |= NOAPP;
else if (strcmp(cmd,"cdrpr") == 0)
    tprec->flags |= CDRPR;
else if (strcmp(cmd,"cdr2x") == 0)
    tprec->flags |= CDR2X;
else if (strcmp(cmd,"cdr4x") == 0)
    tprec->flags |= CDR6X;
else if (strcmp(cmd,"cdr6x") == 0)
    tprec->flags |= CDR6X;
else if (strcmp(cmd,"cdr8x") == 0)
    tprec->flags |= CDR8X;
else
    printf("get_tpflag: %s is an invalid tape flag\n",cmd);
}

int tpconfig_assdrv
(
    struct TAPEDESC *drive
)
{

switch (drive->dev_type)
    {
    case IRIX_EXABYTE:
	drive->bot = BOT_IRIX_EXABYTE;
	drive->eof = EOF_IRIX_EXABYTE;
	drive->eot = EOT_IRIX_EXABYTE;
	drive->maxbyte = MAX_IRIX_EXABYTE;
	drive->minbyte = MIN_IRIX_EXABYTE;
	break;
    case IRIX_SCSI:
	drive->bot = BOT_IRIX_SCSI;
	drive->eof = EOF_IRIX_SCSI;
	drive->eot = EOT_IRIX_SCSI;
	drive->maxbyte = MAX_IRIX_SCSI;
	drive->minbyte = MIN_IRIX_SCSI;
	break;
    case LINUX_TS:
    case LINUX_SCSI:
	drive->bot = BOT_LINUX_SCSI;
	drive->eof = EOF_LINUX_SCSI;
	drive->eot = EOT_LINUX_SCSI;
	drive->maxbyte = MAX_LINUX_SCSI;
	drive->minbyte = MIN_LINUX_SCSI;
	break;
    case SGI_XY:
	drive->bot = BOT_SGI_XY;
	drive->eof = EOF_SGI_XY;
	drive->eot = EOT_SGI_XY;
	drive->maxbyte = MAX_SGI_XY;
	drive->minbyte = MIN_SGI_XY;
	break;
    case DG54_GEN:
	drive->bot = BOT_DG_GEN;
	drive->eof = EOF_DG_GEN;
	drive->eot = EOT_DG_GEN;
	drive->maxbyte = MAX_DG_GEN;
	drive->minbyte = MIN_DG_GEN;
	break;
    case SOLARIS_SCSI:
	drive->bot = BOT_SOLARIS_SCSI;
	drive->eof = EOF_SOLARIS_SCSI;
	drive->eot = EOT_SOLARIS_SCSI;
	drive->maxbyte = MAX_SOLARIS_SCSI;
	drive->minbyte = MIN_SOLARIS_SCSI;
	break;
    case CD:
    case CDR:
    case DVD:
    case DVDR:
	drive->maxbyte = 1;
	drive->minbyte = 1;
	break;
    case VAX_GEN:
    case VAX_FSF:
	drive->bot = BOT_VAX_GEN;
	drive->eof = EOF_VAX_GEN;
	drive->eot = EOT_VAX_GEN;
	drive->maxbyte = MAX_VAX_GEN;
	drive->minbyte = MIN_VAX_GEN;
	break;
    default:
	printf("tprdconf: unknown device type %d\n",drive->dev_type);
	return(TAPE_FAIL);
    }
if ((drive->rah = malloc(drive->maxbyte)) == NULL)
    {
    printf("Error allocating %d bytes for tape i/o work buffer\n",
	drive->maxbyte);
    return(TAPE_FAIL);
    }
drive->rah_nb = 0;
return(TAPE_SUCC);
}

int tpconfig_getchar
(
	FILE *fp,
	int  *ch
)
{
int rstat;

rstat = E_SUCC;
*ch = fgetc(fp);
if (*ch == '\\')			/* escape char, skip next char */
    {
    *ch = fgetc(fp);
    if (*ch == '\n')
        *ch = fgetc(fp);
    else
	return(E_ESCAPE);
    }
if ((*ch == ' ') || (*ch == '\t'))
    rstat = tpconfig_getchar(fp,ch);
return(rstat);
}

int tpconfig_getfield
(
	FILE *fp,
	char *buf
)
{
long rstat;
long status;
int  ch;
char *bufptr;

rstat = E_SUCC;
*buf = '\0';
status = tpconfig_getchar(fp,&ch);
if (ch == EOF)
    return(E_EOF);
if ((ch == '\n') && (status != E_ESCAPE))
    return(E_EOL);
else if ((ch == '#') && (status != E_ESCAPE))	/* rest of line is comment */
    {
    for (rstat=E_SUCC; rstat==E_SUCC; )
	{
	status = tpconfig_getchar(fp,&ch);
	if (ch == EOF)
	    rstat = E_EOF;
	if (ch == '\n')
	    rstat = E_EOL;
	}
    }
else
    {
    bufptr = buf;
    for (;;)
	{
        *bufptr++ = ch;
	status = tpconfig_getchar(fp,&ch);
	if (ch == EOF)
	    {
	    printf("Unexpected EOF encountered in tape configuration file\n");
	    rstat = E_EOF;
	    break;
	    }
	if (((ch == '\n') || (ch == '#')) && (status != E_ESCAPE))
	    {
	    if (ungetc(ch,fp) == EOF)
		{
		perror("tprdconf");
		rstat = E_FAIL;
		}
	    ch = ':';
	    }
        if ((ch == ':') && (status != E_ESCAPE))
	    break;
	}
    *bufptr = '\0';
    }
return(rstat);
}

int tpgetconf
(
    char *tpconfig
)
{
	char *tpconfig_ptr;
#ifdef vms
	char temp[256];

        if (get_logical("LASTABLES",temp) == -1)
            {
            printf("Global variable LASTABLES not defined\n");
            return(TAPE_FAIL);
            }
        sprintf(tpconfig,"%stape.config",temp);
#endif
#ifdef unix
        tpconfig_ptr = getenv("LASTABLES");
        if (tpconfig_ptr == NULL)
            {
            printf("Global variable LASTABLES not defined\n");
            return(TAPE_FAIL);
            }
        sprintf(tpconfig,"%s/tape.config",tpconfig_ptr);
#endif
	return(TAPE_SUCC);
}
