/******************************************************************************
FUNCTION:	HOMPRM

PURPOSE:	Fills the 15 parameter values necessary for the HOM projection

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0				L. Huewe	Original Development
  5.0		4/89		D. Hollaren	Conversion to 5.0
  6.0		12/90		BALTZER		Check return status's
  6.1		9/92		D. Etrheim	Modify to use the zone number's
						as zero relative.  Zone numbers 
						range from 1 to 10 (inclusive) 
						as set by homzon. Updated
						algorithm reference.
  6.2           5/93		T. Mittan	Modified so that correct
						longitude is found

ALGORITHM 
  Initialize the HOM parameter values

ALGORITHM REFERENCES
	Landsat Data Users Handbook, USGS, 1979, Appendix D

*******************************************************************************/

#include <math.h>
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "tm.h"


int homprm
(
    long *path,		/* WRS path designator				*/
    long *zone,		/* HOM zone code				*/
    double parms[]		/* HOM projection parameters			*/
)
{
double var;		/* Inclination of orbit at ascending node	*/

static double lat[] = { 67.0983333,  36.0, .0002777, -36.0, -67.0983333,
		       -67.0983333, -36.0, .0002777,  36.0,  67.0983333 };

static double lng[] = { -81.970, -99.275, -108.5069, -117.7388, -135.0438,
			 85.122,  67.817,  58.5851,   49.3532,   32.0482 };

static double az[] = { 24.7708181,  14.3394883,  13.001443,   14.3394883,
		       24.7708181, -24.7708181, -14.3394883, -13.001443,
		      -14.3394883, -24.7708181 };

long i;			/* Loop index					*/
long ivar;		/* Integer variable				*/
long in_fatal = 1;

if ((*zone < 1) || (*zone > 10))
   {
   c_errmsg("Invalid zone number","homprm-zone",&in_fatal);
   return(E_FAIL);
   }

/*
  Initialize the parameter values
*/
for (i = 0; i < COEFCT; i++)
    parms[i] = 0.0;

/*
  Scale factor at center of projection
*/
parms[2] = 1.0;

/*
  Azimuth angle east of north of center line
*/
if ( c_degdms(&az[(*zone - 1)],&parms[3],"DEG","CIR") != E_SUCC )
   return(E_FAIL);

/*
  Longitude of point on centeral meridian where azimuth occurs
*/
var = lng[(*zone - 1)] - (360.0 / 251.0) * (*path - 31);
ivar = (long)(var / 360.0);

var -= (ivar * 360.0);

if (var < -180.0)
    var +=360.0;
else
if (var > 180.0)
    var -=360.0;

if ( c_degdms(&var,&parms[4],"DEG","LON") != E_SUCC )
   return(E_FAIL);

/*
  Latitude of projection origin
*/
if ( c_degdms(&lat[(*zone - 1)],&parms[5],"DEG","LAT") != E_SUCC )
   return(E_FAIL);

/*
  False Eastings
*/
parms[6] = 500000.0;

/*
  False Northing
*/
parms[7] = 500000.0;

/*
  Set to any non-zero number
*/
parms[12] = 1.0;

return(E_SUCC);
}
