/******************************************************************************
FUNCTION:	LS2PROJ

PURPOSE:	This routine does basically the reverse process of LINSAM to
		convert from line/sample to projection coordinates.

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0		4/89		HOLLAREN	Original development

ALGORITHM 
  Initialize variables
  Reapply center value and pixel size
  Solve for Xp and Yp using the ing transformation equations
  Calculate projection coordinates based on full scene center

ALGORITHM REFERENCES
This routine uses the algorithm described in the paper entitled
"Geodetic Accuracy of LANDSAT 4 Multispectral Scanner and Thematic Mapper Data"
written by J. M. Thormodsgard and D. J. DeVries.
*******************************************************************************/

#include <math.h>
#include "las.h"
#include "proj.h"
#include "tm.h"


void ls2proj
(
    double *angle,		/* Display rotation angle			      */
    long *proj_code,	/* Map projection type (SOM or UTM)		      */
    double fcproj[],	/* Full scene center projection coords (y,x)	      */
    double fcenls[],	/* Full scene center line/sample		      */
    double proj[],		/* Projection coordinates calculated (y,x)	      */
    long linsamp[]		/* Line/sample values calculated		      */
)
{
double lp;		/* Line value after coordinate to be converted is     */
			/* subtracted from center line and multiplied by pixel*/
			/* size.					      */

double sp;		/* Sample value after center sample is subtracted from*/
			/* sample to be converted and multiplied by pixel size*/

double pixsiz;		/* Pixel size in meters				      */
double xp, yp;		/* Intermediate value after translation is applied    */

/********************************************************
   CORRECT ORIENTATION ANGLE FOR TM DATA IF TAPE
   PROCESSED BEFORE 1ST WEEK IN OCTOBER.
   I haven't needed to do this as of yet (LLH 11/87)

if (boct)
    a = pi - fabs(*angle);
*********************************************************/

/*
  Initialize variables
*/
*angle = fabs(*angle);
pixsiz = 28.50;

/*
  Reapply center value and pixel size
*/
sp = ((double)linsamp[1] - fcenls[1]) * pixsiz;
lp = (fcenls[0] - (double)linsamp[0]) * pixsiz;

/*
  Solve for Xp and Yp using the following transformation equations:
*/
xp =  sp * cos(*angle) + lp * sin(*angle);
yp = -sp * sin(*angle) + lp * cos(*angle);

/*
  Calculate projection coordinates based on full scene center

  NOTE: Coordinate axes for the Space Oblique Mercator (SOM) is
        the reversed of the other projection coordinate axes.
*/
if (*proj_code == SOM)
    {
    proj[0] = yp - fcproj[1];
    proj[1] = xp + fcproj[0];
    }
else
    {
    proj[0] = yp + fcproj[0];
    proj[1] = xp + fcproj[1];
    }
}

/******************************************************************************
FUNCTION:	IMAGPROJ

PURPOSE:	This routine does basically the reverse process of LINSAM to
		convert from line/sample to projection coordinates.

ALGORITHM 
  Calculate the transformation coefficients needed to generate projection
  coordinates from a line/sample
  Calculate the projection coordinate from the transformation coefficients
++

ALGORITHM REFERENCES:
*******************************************************************************/
int imgproj
(
    double pwin[],		/* Input image corner projection coordinates	      */
    long iwin[],		/* Input image corner line/sample coordinates	      */
    long linsamp[],		/* Line/sample values calculated		      */
    double proj[]		/* Projection coordinates calculated (y,x)	      */
)
{
double coef[6];		/* Transformation coefficients			      */
long prt_flag = FALSE;	/* Print flag					      */
long status;		/* Status variable				      */

prt_flag = FALSE;

/*
  Calculate the transformation coefficients needed to generate projection
  coordinates from a line/sample
*/
status = c_rtcoef(pwin,iwin,coef,&prt_flag);
if (status != E_SUCC)
    return(E_FAIL);

/*
  Calculate the projection coordinate from the transformation coefficients
*/
proj[0] = (coef[0] * (double)linsamp[0]) + (coef[1] * (double)linsamp[1]) +
	   coef[2];
proj[1] = (coef[3] * (double)linsamp[0]) + (coef[4] * (double)linsamp[1]) +
	   coef[5];

return(E_SUCC);
}
