/****************************************************************************

NAME: c_block_cleanup

PURPOSE: Mask the SIGTERM signal so that it is blocked until the critical
  section of code has completed.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    01/01  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Requires a system that provides POSIX signal handling.

PROJECT: LAS

NOTES:
  The c_unblock_cleanup routine should be called to reset this signal once
  the critical section of code has completed.

****************************************************************************/

#include <stdio.h>
#include <signal.h>
#include "las.h"
#include "worgen.h"
#include "util.h"

/**
 * c_block_cleanup:
 * @oldmask: O: Pointer to sigset_t variable that contains the status
 *              of the signal mask before SIGTERM was masked to be blocked.
 *
 * The c_block_cleanup() routine is used to mask the SIGTERM signal so that
 * it is blocked until the critical section of code has been completed.
 * The routine c_unblock_cleanup() can be used to unblock the SIGTERM
 * signal after the critical section of code has been completed.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_block_cleanup
(
    sigset_t *oldmask   /* O: status of signal mask before SIGTERM was masked
                              to be blocked                                 */
)

{
    sigset_t newmask;   /* mask for blocking SIGTERM                        */

    /* Set the mask for SIGTERM                                             */
    sigemptyset (&newmask);
    sigaddset (&newmask, SIGTERM);

    /* Block SIGTERM and save the current signal mask                       */
    if (sigprocmask (SIG_BLOCK, &newmask, oldmask) < 0)
    {
        c_lasmsg (FALSE, "block-cleanup",
                 "Warning: Cannot block the SIGTERM signal");
        return (E_FAIL);
    }
    else
        return (E_SUCC);
}
