/******************************************************************************
NAME:                           C_BNDCHK

FUNCTION:    Checks the band specification against the actual image.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         ??         ??
    2.0         9/86       K. Gacke     Added parameter "nbands" to return
                                        number of bands specified.
    3.0       Dec. 1987    K. Zanter    1.  switched to raw C types
                                        2.  changed include file specification
                                        3.  changed DESC argument to be a
                                            character string
                                        4.  renamed source from bndchk to
                                            c_bndchk
    4.0         12/87      B. Ailts     implemented new DDR structure and calls
    5.0         04/87      D. Hollaren  changed include spec to las
                                        change cerrmsg to c_errmsg and
                                        pass status code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
     5.0        07/90      D. VanderZee Standardized error handling 
     6.0        12/90      B. Ailts     Changed check for invalid band numbers
                                        Now will return a bad status when the
                                        first band is invalid
     7.0        3/91      B. Davis      put increment of band index of while
                                        loop at end of loop, not at beginning.
                                        the check was not working correctly for
                                        multi-image, multi-band input, but not
                                        erring out either, causing disastrous 
                                        delayed side effects if the input
                                        specification was incorrect.
    7.1         9/92        T. Mittan   Removed ; after last } to meet standards

PROJECT:        LAS

ALGORITHM:
        The DDR for the image is retrieved.  The DDR contains the actual number
        of bands of the image.  The input band specification is checked against
        the actual bands of the image.

ALGORITHM REFERENCES:  none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"

/**
 * c_bndchk:
 * @host: I: Input host file name.
 * @inbnds: I/O: Number of bands specified.
 * @vlbnds: O: Valid window values for bands.
 * @nbands: I: Number of bands per image.
 *
 * The c_bndchk() routine checks the band specifications against the 
 * actual image.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_bndchk
(
    const char *host,            /* I: Input host filename                  */
    int  *inbnds,                /* I/O: Number of bands specified          */
    int  *vlbnds,                /* O: Valid window values for bands        */
    int  *nbands                 /* O: Number of bands per image            */
)
{
    struct DDR ddr;
    int    band;                 /* counter and index for bands             */
    int    bands;                /* number of bands in image                */
    char   err_text[ERRLEN+1];   /* error message text                      */

    if (c_getddr(host,&ddr) != E_SUCC)
    {
        c_lasmsg (FALSE, "bndchk-call", "Error returned from getddr");
        return (E_FAIL);
    }

    bands = ddr.nbands;
    if ( bands > MAXBND )
    {
        sprintf( err_text, "%s %d %s %d %s",
                "Error: Image has too many bands.  It has",
                 bands, ".  The limit is", MAXBND, ".");
        c_lasmsg (FALSE,"bndchk-maxband",err_text);
        return (E_FAIL);
    }

    if ( inbnds[0] == 0 )
    {
        for ( band=0; band<bands; band++ ) 
            vlbnds[band] = band+1;
            vlbnds[bands] = 0;
    }
    else
    {
        band = 0;
        while ((inbnds[band]>0) && (band<MAXBND))
        {
            vlbnds[band] = inbnds[band];
            if ( inbnds[band] < 0 )
            {
                sprintf (err_text, "%s %d %s %s",
                        "Error: Band", band, "is invalid.",
                        " All band numbers must be positive.");
                c_lasmsg (FALSE,"bndchk-invband",err_text);
                return (E_FAIL);
            }

            else if ( inbnds[band] > bands )
            {
                sprintf (err_text, "%s %d %s%d%s",
                        "Error: Band", inbnds[band],
                        "is greater than the number of bands (",
                         bands, ") in the image" );
                c_lasmsg (FALSE,"bndchk-invband",err_text);
                return (E_FAIL);
            }

            else if ( (band>=MAXBND) && (inbnds[band]!=0) )
            {
                sprintf( err_text, "%s %d %s",
                        "Error: Too many bands were specified.  Only",
                         MAXBND, "bands can be specified." );
                c_lasmsg (FALSE,"bndchk-invband",err_text);
                return (E_FAIL);
            };
            band++;
        }
        vlbnds[band] = 0;
    };

    *nbands = band;
    return (E_SUCC);
}
