/******************************************************************************
NAME:    c_calc_bound_ls_coords

PURPOSE: Determine the output projection space minimum and maximum line
         and sample values that contain the entire input window.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
  Date      Programmer                        Changes
--------  --------------  -----------------------------------------------------
7/01      A. Shimitz      Original development (separated from
                          c_window_proj_to_ls to allow trancoord to share
                          same code)

ALGORITHM:
   Call c_mapedg to calculate the min and max projection coordinates for
      given upper left and lower right lat, long coords
   Round the min/max values to the nearest multiple of the x and y pixel
      sizes that includes the computed min/max value
   
******************************************************************************/

#include <math.h>
#include "las.h"
#include "geompak.h"

/**
 * c_calc_bound_ls_coords
 * @ulx: I/O: Upper left x coordinate.
 * @lrx: I/O: Lower right x coordinate.
 * @uly: I/O: Upper left y coordinate.
 * @lry: I/O: Lower right y coordinate.
 * @units: I: Window units.
 * @proj_code: I: GCTP projection code.
 * @zone_code: I: Zone code.
 * @proj_coef: I: GCTP projection parameters.
 * @out_units: I: Output units.
 * @datum_code: I: GCTP datum code.
 * @pdist_y: I: Projecton distance/pixel in the y direction.
 * @pdist_x: I: Projection distance/pixel in the x direction.
 * @pxmin: O: Minimum x value.
 * @pxmax: O: Maximum x value.
 * @pymin: O: Minimum y value.
 * @pymax: O: Maximum y value.
 *
 * The c_calc_bound_ls_coords() routine determines the output projection
 * space minimum and maximum line and sample values that contain the entire
 * input window.  The c_calc_bound_ls_coords()routine  calls c_mapedg() 
 * to calculate the minimum and maximum projection coordinates for 
 * given upper left and lower right latitude and longitude coordinates.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_calc_bound_ls_coords
(
    double *ulx,                /* I/O: upper left x                        */
    double *lrx,                /* I/O: lower right x                       */
    double *uly,                /* I/O: upper left y                        */
    double *lry,                /* I/O: lower right y                       */
    int  units,                 /* I: window units                          */
    int  proj_code,             /* I: GCTP projection code                  */
    int  zone_code,             /* I: zone code                             */
    const double proj_coef[15], /* I: GCTP projection parameters            */
    int  out_units,             /* I: output units                          */
    int  datum_code,            /* I: GCTP datum code                       */
    double pdist_y,             /* I: projection distance/pixel(y-direction)*/
    double pdist_x,             /* I: projection distance/pixel(x-direction)*/
    double *pxmin,              /* O: minimum x value                       */
    double *pxmax,              /* O: maximum x value                       */
    double *pymin,              /* O: minimum y value                       */
    double *pymax               /* O: maximum y value                       */
)
{
    /* Compute the minimum output window projection coordinates
       that contain the entire input window                                 */
    if(c_mapedg(ulx, lrx, uly, lry, &units, &proj_code, &zone_code, proj_coef,
                &out_units, pxmin, pxmax, pymin, pymax, &datum_code) != E_SUCC)
        return(E_FAIL);

    /* Round to the nearest multiple of the pixel size, ensuring that
       the computed min/max values are included                             */
    *uly = (int )(*pymax/pdist_y) * pdist_y;
    *ulx = (int)(*pxmin/pdist_x) * pdist_x;
    *lry = (int)(*pymin/pdist_y) * pdist_y;
    *lrx = (int)(*pxmax/pdist_x) * pdist_x;

    if(fmod(*pymax, pdist_y) > 0.0)
        *uly += pdist_y;
    if(fmod(*pxmin, pdist_x) < 0.0)
        *ulx -= pdist_x;
    if(fmod(*pymin, pdist_y) < 0.0)
        *lry -= pdist_y;
    if(fmod(*pxmax, pdist_x) > 0.0)
        *lrx += pdist_x;

    /* Set min and max values to the rounded values                         */
    *pymax = *uly;
    *pxmin = *ulx;
    *pymin = *lry;
    *pxmax = *lrx;

    return(E_SUCC);
}
