/*******************************************************************************
NAME                            C_GETIN

PURPOSE         Break input specification into individual image specifications.

PROGRAM HISTORY
        PROGRAMMER        DATE          REASON
        ----------        ----          ------
        P. Mumford       3/15/85        Original development for NEWLAS.
        K. Gacke         9/09/85        Updated the include files and fixed a
                                        bug involving parameter "same_window"
        L. Huewe        10/17/85        Reimplemented QFLAG parameter.
        K. Gacke         9/15/86        Added parameters "nbands" and "totbnd"
                                        to return number of bands per image and
                                        total number of bands.
        K. Gacke        12/12/86        Allow user to name images with any
                                        extension and not just "img"
        K. Gacke         2/2/87         Modified to allow device to be specified
                                        on VMS.  The colon character has two
                                        meanings depending on it position: 
                                        1) separator of device and directory
                                        2) separator of window and bands
        K. Gacke         2/2/87         Ensure no spaces trail the image name
        K. Zanter        12/87          1.  switch to raw C types 
                                        2.  strings descriptor arguments to 
                                            character strings
                                        3.  changed include file specification
                                        4.  renamed source from "getin" to
                                            "c_getin"
                                        5.  moved bridge routine to separate
                                            source file
        D. Hollaren      04/88          changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed the status code by address
        B. Ailts        07/88           Added better error checking
        B. Ailts        09/88           Returns *images as zero if the
                                        image_name[*element] was not specified
        B. Ailts        09/88           Returns a null values in the host and
                                        LAS names if the image_name[*element] 
                                        was not found in the parameter block
        D. VanderZee    07/90           Standardized error handling 
        D. Etrheim      09/90           Allow a band range to be specified
        T. Mittan       09/92           type cast pos and left to ints
                                        in comparision statments for ANSII
                                        standard compilers
        T. Mittan       09/92           Changed type casts and placed 
                                        (char *) on constants instead
        T. Mittan       07/93           Changed PRS_LST to double, added
                                        new windowing options, added 
                                        make_proj
        D. Bridges      10/93           Split c_getin into c_getin and c_getin2
                                        where c_getin is called from TAE and
                                        c_getin2 is called from outside TAE
        D. Etrheim      10/93           Return status from c_getin2
        D. Lloyd        08/98           Moved imgprs and its associated
                                        functions to a separate file and
                                        renamed to c_imgprs.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
        C_GETIN must be run under TAE.

PROJECT                         LAS

ALGORITHM:
        Get the value of the input image parameter
        If the count of the input image names are less than *element + 1
           String copy a blank into the tae file names
           String copy a blank into the host file names
           Set *images to zero
           Return
        Substitute any aliases in the input image parameter
        Parse the string and return the name, window, and bands in an array,
                also the number of images
        For each image
                Get the full TAE name
                Get the host name
                Check the window for accuracy and default value
                Check the bands for accuracy and default value
        End for


ALGORITHM REFERENCES
        This algorithm is roughly based on the GET_IN utility in LAS.

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h> 
#include "las.h"
#include "util.h"
#include "alias.h"
#include "labserv.h"
#include "local_protos.h"

/*****************************************************************************
    Function:  c_getin2_nocheck
    Purpose:
        Parse image names, bands, and windows.  The image need not exist.
        May be called by applications that do not use the TAE interface.
        For the applications that do use TAE, c_getin_nocheck must be called.
*****************************************************************************/
static int c_getin2_nocheck
(
    const char inparm[],      /* I:  Input image string with                */
    char  tnames[][CMLEN],    /* O:  TAE names of images specified          */
    char  hnames[][CMLEN],    /* O:  host names of images specified         */
    int   *images,            /* O:  Number of images                       */
    int   (*window)[4],       /* O:  Image windows (sl ss nl ns)            */
    int   (*bands)[MAXBND+1], /* O:  Image bands                            */
    int   *nbands,            /* O:  Number of bands per image              */
    int   *totbnd,            /* O:  total number of bands specified        */
    int   imglimit,           /* I:  Maximum images caller can handle       */
    int   *same_window,       /* O:  1 if all windows the same              */
    int   (*qflag)[2]         /* O:  1 user defined window & bands else:    */
                              /*     0 if default window and bands          */
)
{
    static struct PRS_LST p_list[MAXIMG];/* Image list returned fr imgprs*/

    char    outparm[CMLEN];    /* Output parm spec without aliases          */
    char    err_text[ERRLEN+1];/* error message text                        */
    char    tae[CMLEN];        /* current TAE name                          */
    char    *ptr;
    int     status;            /* status code returned by CM                */
    int     temp;
    unsigned short  image;     /* image counter and index                   */

    /* Substitute any alias                                                 */
    if (c_getalias(inparm,outparm) != E_SUCC)
    {
        c_lasmsg (FALSE, "getin-call", "Error returned from getalias");
        return(E_FAIL);
    }

    if( E_FAIL == c_imgprs(outparm,p_list,images))
    {
        c_lasmsg (FALSE, "getin-parse",
                  "Error: Input image parse was unsuccessful");
        return(E_FAIL);
    };

    /* Determine whether or not the user exceeded the maximum number of
       input images.                                                        */
    if( *images > imglimit )
    {
        sprintf( err_text, "%s %d %s %d %s.", "Error: There were", *images,
                "parsed which is greater than the limit of",
                 imglimit, "images");
        c_lasmsg (FALSE, "getin-imglmt", err_text);
        return(E_FAIL);
    }

    for ( image = 0; image < *images; image++ )
    {
        strcpy(tae,p_list[image].name);

        if (strchr(tae, TAE_DIR_BEG) || strchr(tae, TAE_EXT))
        {
            if ((ptr = strrchr(tae,TAE_EXT)) == 0)
                sprintf(&tnames[image][0],"%s%c%s",tae,TAE_EXT,"img");
            else 
            {
                if (strchr(tae,TAE_EXT) != ptr)
                {
                    c_lasmsg (FALSE, "getin-name",
                            "Error: Invalid LAS image name: Multiple extensions"
                            " not allowed");
                    return(E_FAIL);
                }
                strcpy(&tnames[image][0],tae);
            }
        }
        else
        {
            temp = (int)strlen(tae);
            if (strcmp(&tae[temp-4], ".img") != 0)
                sprintf(tnames[image],"%s%c%s", tae, HOST_EXT, "img");
            else 
                strcpy(tnames[image], tae);
        }
    }

    /* Get the full TAE and host names.                                     */
    for ( image = 0; image < *images; image++ )
    {
        if (c_cmhost(&tnames[image][0],&hnames[image][0]) != E_SUCC)
        {
            c_lasmsg (FALSE, "getin-call", "Error returned from cmhost");
            sprintf(err_text,"Error retrieving host file name for %s",
                    &tnames[image][0]);
            c_lasmsg (FALSE, "getin-cmhost", err_text);
            return(E_FAIL);
        }

       /* Copy name to TAE string, since we are still carrying around
          both host and LAS forms (that are now identical internally).      */
        strcpy(tnames[image], hnames[image]);
    }

    /* Check the windows and bands.                                         */
    *totbnd = 0;
    *same_window = 1;
    for ( image = 0; image < *images; image++ )
    {

        /* This is where projection coordinates can be converted to line
           sample                                                           */
        if (p_list[image].proj[0] != 0)
        {
            status = c_wndconv(&hnames[image][0],p_list[image].window,
                               p_list[image].proj);
            if (status != E_SUCC)
            {
                c_lasmsg (FALSE, "getin-call", "Error returned from wndconv");
                return(E_FAIL);
            }
        }

        status = c_wndchk(&hnames[image][0],p_list[image].window,
                          window[image]);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "getin-call", "Error returned from wndchk");
            sprintf (err_text,"Error: Window specified for %s was invalid",
                    &tnames[image][0]);
            c_lasmsg (FALSE, "getin-invwnd",err_text);
            return(E_FAIL);
        }

        if (image > 0)
            if ((window[image][NL] != window[image-1][NL]) ||
                (window[image][NS] != window[image-1][NS]) )
                *same_window = 0;

        status = c_bndchk(&hnames[image][0],p_list[image].bands,
                          bands[image],&nbands[image]);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "getin-call", "Error returned from bndchk");
            sprintf (err_text,"Error: Bands specified for %s were invalid",
                    &tnames[image][0]);
            c_lasmsg (FALSE, "getin-invbnds", err_text);
            return (E_FAIL);
        }

        *totbnd += nbands[image];
        qflag[image][0] = p_list[image].qflag[0];
        qflag[image][1] = p_list[image].qflag[1];
    }

    if (*totbnd > MAXBND)
    {
        c_lasmsg (FALSE, "getin-invband",
                  "Error: Too many bands specified in input image(s)");
        return(E_FAIL);
    }
    return(E_SUCC);
}


/* FIXME!                                                                   */
/* The las versions of the routines should only be returning a single
   version of the name                                                      */
/***************************************************************************** 
    Function:  las_getin
    Purpose:
       Parse image names, bands, and windows.  The image will be confirmed to
       exist.  This is used by applications that use the TAE interface.
       Calls c_getin2 to do the actual parse.
*****************************************************************************/
/**
 * las_getin:
 * @block: I: TAE parameter block passed by reference.  It is assumed that
 *            TAE has been initialized prior to calling las_getin().
 * @param_buf: I: Defines the parameter name in the procedure which
 *                contains the input image(s) specification.  In most cases
 *                the value of this parameter is IN.
 * @element: I: Defines which element of the input parameter array
 *              specified by "param_buf" will be 0 (zero) in 'C' or 1 (one)
 *              in FORTRAN for input parameters that contain only a
 *              single element.
 * @tnames: O: Array of character strings dimensioned to be the same size
 *             as parameter "imglimit".  It contains the TAE names of the
 *             images specified in parameter "param_buf".
 * @hnames: O: Array of character strings dimensioned to be the same size
 *             as parameter "imglimit".  It contains the host names of the
 *             images specified in the parameter "param_buf".
 * @images: O: Number of images defined in the image specification.
 * @window: O: Two-dimensional array containing the window specification
 *             for each image specified.  The first dimension is "imglimit"
 *             and the second is 4.  The four elements of the window are
 *             SL, SS, NL, and NS.
 * @bands: O: Two-dimensional array containing the band numbers selected
 *            for each image specified.  The first dimension is "imglimit"
 *            and the second is MAXBND + 1.  One is added to maxbnd in
 *            order to allow for the zero terminator which is needed by
 *            c_eopens().
 * @nbands: O: Number of bands specified for each input image.  This array
 *             is useful when calling the function c_eopens() or c_eopenr()
 *             to open the input images.
 * @totbnd: O: Total number of bands to be processed.  This value is
 *             useful for the processing message utility c_inittm().
 * @imglimit: I: Maximum number of images the calling routine can handle
 *               when las_getin() is called.
 * @same_window: O: Signals whether or not multiple images have the same
 *                  window size.  A value of 0 (zero) means that the
 *                  images do not all have the same window size while a
 *                  value of 1 (one) indicates that all windows are of the
 *                  same size.
 * @qflag: O: Two-dimensional array which will contain flags for each
 *            image indicating whether the user specified a window or
 *            band option or if window and band options were allowed to
 *            default to the full image.  A 1 (one) in the first element
 *            indicates that the user specified a window option and a 1
 *            (one) in the second element indicates that the user specified
 *            a band option.  A 0 (zero) in either element indicates the
 *            default option has been supplied.  The first dimension is
 *            "imglimit", the second dimension is 2.
 *
 * The las_getin() routine is used to parse an input image specification.
 * Note:  No error message is printed if the user specifies multiple images
 * with different window sizes.  Although most programs will typically
 * consider this an error, there may be some modules which will want to
 * allow multiple images to be specified with different windows.  Thus, it
 * is up to the individual program to decide whether images with different
 * window sizes are valid or not.  If not, the program will need to do its
 * own error handling.
 *
 * Returns:    E_SUCC:  Successful completion of operation,
 *             E_FAIL:  Unsuccessful completion of operation.
 */
int las_getin
(
    struct PARBLK *block,      /* I: TAE parameter block                    */
    const char *param_buf,     /* I: Input image parameter name             */
    int   element,             /* I: element array number of param.         */
    char  tnames[][CMLEN],     /* O: TAE names of images specified          */
    char  hnames[][CMLEN],     /* O: host names of images specified         */
    int   *images,             /* O: Number of images                       */
    int   (*window)[4],        /* O: Image windows (sl ss nl ns)            */
    int   (*bands)[MAXBND+1],  /* O: Image bands                            */
    int   *nbands,             /* O: Number of bands per image              */
    int   *totbnd,             /* O: total number of bands specified        */
    int   imglimit,            /* I: Maximum images caller can handle       */
    int   *same_window,        /* O: 1 if all windows the same              */
    int   (*qflag)[2]          /* O: 1 user defined window & bands else:    */
                               /* 0: if default window and bands            */
)
{
    struct  VARIABLE *i_spec;  /* Input image parameter value               */

    char    inparm[CMLEN];     /* Input image string with                   */
                               /* optional window specification             */
    int     status;            /* return status                             */

    /* Get image specification and parse it.                                */
    if( NULL == (i_spec = p_find( block, param_buf )))
    {
        c_lasmsg (FALSE, "getin-param",
                  "Error: Input image parameter does not exist");
        return(E_FAIL);
    };

    if (i_spec->v_count < element + 1)
    {
        *images = 0;
        strcpy(hnames[0],"\0");
        strcpy(tnames[0],"\0");
        return(E_SUCC);
    }
    strcpy(inparm,SVAL(*i_spec,element));


    status = las_getin2 (inparm,tnames,hnames,images,window,bands,nbands,
                         totbnd,imglimit,same_window,qflag);
    return(status);

}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_getin:
 * @block: I: TAE parameter block passed by reference.  It is assumed that
 *            TAE has been initialized prior to calling c_getin().
 * @param_buf: I: Defines the parameter name in the procedure which
 *                contains the input image(s) specification.  In most cases
 *                the value of this parameter is IN.
 * @element: I: Defines which element of the input parameter array
 *              specified by "param_buf" will be 0 (zero) in 'C' or 1 (one)
 *              in FORTRAN for input parameters that contain only a
 *              single element.
 * @tnames: O: Array of character strings dimensioned to be the same size
 *             as parameter "imglimit".  It contains the TAE names of the
 *             images specified in parameter "param_buf".
 * @hnames: O: Array of character strings dimensioned to be the same size
 *             as parameter "imglimit".  It contains the host names of the
 *             images specified in the parameter "param_buf".
 * @images: O: Number of images defined in the image specification.
 * @window: O: Two-dimensional array containing the window specification
 *             for each image specified.  The first dimension is "imglimit"
 *             and the second is 4.  The four elements of the window are
 *             SL, SS, NL, and NS.
 * @bands: O: Two-dimensional array containing the band numbers selected
 *            for each image specified.  The first dimension is "imglimit"
 *            and the second is MAXBND + 1.  One is added to maxbnd in
 *            order to allow for the zero terminator which is needed by
 *            c_eopens().
 * @nbands: O: Number of bands specified for each input image.  This array
 *             is useful when calling the function c_eopens() or c_eopenr()
 *             to open the input images.
 * @totbnd: O: Total number of bands to be processed.  This value is
 *             useful for the processing message utility c_inittm().
 * @imglimit: I: Maximum number of images the calling routine can handle
 *               when c_getin() is called.
 * @same_window: O: Signals whether or not multiple images have the same
 *                  window size.  A value of 0 (zero) means that the
 *                  images do not all have the same window size while a
 *                  value of 1 (one) indicates that all windows are of the
 *                  same size.
 * @qflag: O: Two-dimensional array which will contain flags for each
 *            image indicating whether the user specified a window or
 *            band option or if window and band options were allowed to
 *            default to the full image.  A 1 (one) in the first element
 *            indicates that the user specified a window option and a 1
 *            (one) in the second element indicates that the user specified
 *            a band option.  A 0 (zero) in either element indicates the
 *            default option has been supplied.  The first dimension is
 *            "imglimit", the second dimension is 2.
 *
 * The c_getin() routine is used to parse an input image specification.
 * Note:  No error message is printed if the user specifies multiple images
 * with different window sizes.  Although most programs will typically
 * consider this an error, there may be some modules which will want to
 * allow multiple images to be specified with different windows.  Thus, it
 * is up to the individual program to decide whether images with different
 * window sizes are valid or not.  If not, the program will need to do its
 * own error handling.
 *
 * Deprecated.  Use las_getin() instead.
 *
 * Returns:    E_SUCC:  Successful completion of operation,
 *             E_FAIL:  Unsuccessful completion of operation.
 */
int c_getin
(
    struct PARBLK *block,        /* I: TAE parameter block                  */
    const  char *param_buf,      /* I: Input image parameter name           */
    const  long *element,        /* I: Element array number of param        */
    char   tnames[][CMLEN],      /* O: TAE names of images specified        */
    char   hnames[][CMLEN],      /* O: Host names of images specified       */
    long    *images,             /* O: Number of images                     */
    long    (*window)[4],        /* O: Image windows (sl ss nl ns)          */
    long    (*bands)[MAXBND+1],  /* O: Image bands                          */
    long    *nbands,             /* O: Number of bands per image            */
    long    *totbnd,             /* O: Total number of bands specified      */
    const  long *imglimit,       /* I: Maximum images caller can handle     */
    long    *same_window,        /* O: 1 if all windows the same            */
    long    (*qflag)[2]          /* O: 1 user defined window & bands else
                                        if default window and bands         */
)
{
    int status;
    status = las_getin (block,param_buf,*element,tnames,hnames,images,
                        window,bands,nbands,totbnd,*imglimit,
                        same_window,qflag);
    return (status);

}

#endif /* obsolete interfaces */

/***************************************************************************** 
    Function:  las_getin_nocheck
    Purpose:
       Parse image names, bands, and windows.  The image need not exist.
       This is used by applications that use the TAE interface.
       Calls c_getin2_nocheck to do the actual parse.
*****************************************************************************/
/**
 * las_getin_nocheck:
 * @block: I: TAE parameter block passed by reference.  It is assumed that
 *            TAE has been initialized prior to calling las_getin().
 * @param_buf: I: Defines the parameter name in the procedure which
 *                contains the input image(s) specification.  In most cases
 *                the value of this parameter is IN.
 * @element: I: Defines which element of the input parameter array
 *              specified by "param_buf" will be 0 (zero) in 'C' or 1 (one)
 *              in FORTRAN for input parameters that contain only a
 *              single element.
 * @tnames: O: Array of character strings dimensioned to be the same size
 *             as parameter "imglimit".  It contains the TAE names of the
 *             images specified in parameter "param_buf".
 * @hnames: O: Array of character strings dimensioned to be the same size
 *             as parameter "imglimit".  It contains the host names of the
 *             images specified in the parameter "param_buf".
 * @images: O: Number of images defined in the image specification.
 * @window: O: Two-dimensional array containing the window specification
 *             for each image specified.  The first dimension is "imglimit"
 *             and the second is 4.  The four elements of the window are
 *             SL, SS, NL, and NS.
 * @bands: O: Two-dimensional array containing the band numbers selected
 *            for each image specified.  The first dimension is "imglimit"
 *            and the second is MAXBND + 1.  One is added to maxbnd in
 *            order to allow for the zero terminator which is needed by
 *            c_eopens().
 * @nbands: O: Number of bands specified for each input image.  This array
 *             is useful when calling the function c_eopens() or c_eopenr()
 *             to open the input images.
 * @totbnd: O: Total number of bands to be processed.  This value is
 *             useful for the processing message utility c_inittm().
 * @imglimit: I: Maximum number of images the calling routine can handle
 *               when las_getin() is called.
 * @same_window: O: Signals whether or not multiple images have the same
 *                  window size.  A value of 0 (zero) means that the
 *                  images do not all have the same window size while a
 *                  value of 1 (one) indicates that all windows are of the
 *                  same size.
 * @qflag: O: Two-dimensional array which will contain flags for each
 *            image indicating whether the user specified a window or
 *            band option or if window and band options were allowed to
 *            default to the full image.  A 1 (one) in the first element
 *            indicates that the user specified a window option and a 1
 *            (one) in the second element indicates that the user specified
 *            a band option.  A 0 (zero) in either element indicates the
 *            default option has been supplied.  The first dimension is
 *            "imglimit", the second dimension is 2.
 *
 * The las_getin_nocheck() routine is used to parse an input image 
 * specification and is used by applications that use the TAE interface.
 * The las_getin_nocheck() routine calls the c_getin2_nocheck() routine
 * to do the actual parsing.  The only difference between the 
 * las_getin_nocheck() routine and the las_getin() routine is that the
 * las_getin_nocheck() does not check to see if the image actually exists.
 *
 * Note:  No error message is printed if the user specifies multiple images
 * with different window sizes.  Although most programs will typically
 * consider this an error, there may be some modules which will want to
 * allow multiple images to be specified with different windows.  Thus, it
 * is up to the individual program to decide whether images with different
 * window sizes are valid or not.  If not, the program will need to do its
 * own error handling.
 *
 * Returns:    E_SUCC:  Successful completion of operation,
 *             E_FAIL:  Unsuccessful completion of operation.
 */
int las_getin_nocheck
(
    struct PARBLK *block,       /* I: TAE parameter block                   */
    const  char *param_buf,     /* I: Input image parameter name            */
    int    element,             /* I: Element array number of param.        */
    char   tnames[][CMLEN],     /* O: TAE names of images specified         */
    char   hnames[][CMLEN],     /* O: Host names of images specified        */
    int    *images,             /* O: Number of images                      */
    int    (*window)[4],        /* O: Image windows (sl ss nl ns)           */
    int    (*bands)[MAXBND+1],  /* O: Image bands                           */
    int    *nbands,             /* O: Number of bands per image             */
    int    *totbnd,             /* O: Total number of bands specified       */
    int    imglimit,            /* I: Maximum images caller can handle      */
    int    *same_window,        /* O: 1 if all windows the same             */
    int    (*qflag)[2]          /* O: 1 user defined window & bands else
                                          if default window and bands       */
)
{
    struct  VARIABLE *i_spec;   /* Input image parameter value              */

    char    inparm[CMLEN];      /* Input image string with                  */
                                /* optional window specification            */
    int  status;                /* return status                            */

    /* Get image specification and parse it. */
    if( NULL == (i_spec = p_find( block, param_buf )))
    {
        c_lasmsg (FALSE, "getin-param",
                  "Error: Input image parameter does not exist");
        return(E_FAIL);
    };

    if (i_spec->v_count < element + 1)
    {
        *images = 0;
        strcpy(hnames[0],"\0");
        strcpy(tnames[0],"\0");
        return(E_SUCC);
    }
    strcpy(inparm,SVAL(*i_spec,element));

    status = c_getin2_nocheck (inparm,tnames,hnames,images,window,bands,
                                 nbands,totbnd,imglimit,same_window,qflag);
    return(status);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_getin_nocheck:
 * @block: I: TAE parameter block passed by reference.  It is assumed that
 *            TAE has been initialized prior to calling las_getin().
 * @param_buf: I: Defines the parameter name in the procedure which
 *                contains the input image(s) specification.  In most cases
 *                the value of this parameter is IN.
 * @element: I: Defines which element of the input parameter array
 *              specified by "param_buf" will be 0 (zero) in 'C' or 1 (one)
 *              in FORTRAN for input parameters that contain only a
 *              single element.
 * @tnames: O: Array of character strings dimensioned to be the same size
 *             as parameter "imglimit".  It contains the TAE names of the
 *             images specified in parameter "param_buf".
 * @hnames: O: Array of character strings dimensioned to be the same size
 *             as parameter "imglimit".  It contains the host names of the
 *             images specified in the parameter "param_buf".
 * @images: O: Number of images defined in the image specification.
 * @window: O: Two-dimensional array containing the window specification
 *             for each image specified.  The first dimension is "imglimit"
 *             and the second is 4.  The four elements of the window are
 *             SL, SS, NL, and NS.
 * @bands: O: Two-dimensional array containing the band numbers selected
 *            for each image specified.  The first dimension is "imglimit"
 *            and the second is MAXBND + 1.  One is added to maxbnd in
 *            order to allow for the zero terminator which is needed by
 *            c_eopens().
 * @nbands: O: Number of bands specified for each input image.  This array
 *             is useful when calling the function c_eopens() or c_eopenr()
 *             to open the input images.
 * @totbnd: O: Total number of bands to be processed.  This value is
 *             useful for the processing message utility c_inittm().
 * @imglimit: I: Maximum number of images the calling routine can handle
 *               when las_getin() is called.
 * @same_window: O: Signals whether or not multiple images have the same
 *                  window size.  A value of 0 (zero) means that the
 *                  images do not all have the same window size while a
 *                  value of 1 (one) indicates that all windows are of the
 *                  same size.
 * @qflag: O: Two-dimensional array which will contain flags for each
 *            image indicating whether the user specified a window or
 *            band option or if window and band options were allowed to
 *            default to the full image.  A 1 (one) in the first element
 *            indicates that the user specified a window option and a 1
 *            (one) in the second element indicates that the user specified
 *            a band option.  A 0 (zero) in either element indicates the
 *            default option has been supplied.  The first dimension is
 *            "imglimit", the second dimension is 2.
 *
 * The c_getin_nocheck() routine is used to parse an input image 
 * specification and is used by applications that use the TAE interface.
 * The c_getin_nocheck() routine calls the las_getin_nocheck() routine
 * which then calls the c_getin2_nocheck() routine to do the actual parsing.
 * The only difference between the las_getin_nocheck() routine and the 
 * las_getin() routine is that the las_getin_nocheck() does not check 
 * to see if the image actually exists.
 *
 * Note:  No error message is printed if the user specifies multiple images
 * with different window sizes.  Although most programs will typically
 * consider this an error, there may be some modules which will want to
 * allow multiple images to be specified with different windows.  Thus, it
 * is up to the individual program to decide whether images with different
 * window sizes are valid or not.  If not, the program will need to do its
 * own error handling.
 *
 * Deprecated.  Use las_getin_nocheck() instead.
 *
 * Returns:    E_SUCC:  Successful completion of operation,
 *             E_FAIL:  Unsuccessful completion of operation.
 */
int c_getin_nocheck
(
    struct PARBLK *block,       /* I:  TAE parameter block                  */
    const  char *param_buf,     /* I:  Input image parameter name           */
    const  long *element,       /* I:  element array number of param.       */
    char   tnames[][CMLEN],     /* O:  TAE names of images specified        */
    char   hnames[][CMLEN],     /* O:  host names of images specified       */
    long   *images,             /* O:  Number of images                     */
    long   (*window)[4],        /* O:  Image windows (sl ss nl ns)          */
    long   (*bands)[MAXBND+1],  /* O:  Image bands                          */
    long   *nbands,             /* O:  Number of bands per image            */
    long   *totbnd,             /* O:  total number of bands specified      */
    const  long *imglimit,      /* I:  Maximum images caller can handle     */
    long   *same_window,        /* O:  1 if all windows the same            */
    long   (*qflag)[2]          /* O:  1 user defined window & bands else
                                       if default window and bands          */
)
{
    int status;

    status = las_getin_nocheck(block,param_buf,*element,tnames,hnames,
                               images,window,bands,nbands,totbnd,
                               *imglimit,same_window,qflag);
    return (status);
}

#endif /* obsolete interfaces */

/**
 * las_getin2:
 * @inparm: I: Input image string associated with optional window 
 *             specification.
 * @tnames: O: Array of character strings dimensioned to be the same size as
 *             parameter "imglimit".  It contains the TAE names of the images
 *             specified in parameter "param_buf".
 * @hnames: O: Array of character strings dimensioned to be the same size as
 *             parameter "imglimit".  It contains the host names of the 
 *             images specified in parameter "param_buf".
 * @images: O: Number of images defined in the image specification.
 * @window: O: Two-dimensional array containing the window specification
 *             for each image specified.  The first dimension is "imglimit"
 *             and the second is 4.  The four elements of the window are
 *             SL, SS, NL, and NS.
 * @bands: O: Two-dimensional array containing the band numbers selected
 *            for each image specified.  The first dimension is "imglimit"
 *            and the second is MAXBND + 1.  One is added to maxbnd in
 *            order to allow for the zero terminator which is needed by
 *            c_eopens().
 * @nbands: O: Number of bands specified for each input image.  This array
 *             is useful when calling the function c_eopens() or c_eopenr()
 *             to open the input images.
 * @totbnd: O: Total number of bands to be processed.  This value is
 *             useful for the processing message utility c_inittm().
 * @imglimit: I: Maximum number of images the calling routine can handle
 *               when las_getin2() is called.
 * @same_window: O: Signals whether or not multiple images have the same
 *                  window size.  A value of 0 (zero) means that the
 *                  images do not all have the same window size while a
 *                  value of 1 (one) indicates that all windows are of the
 *                  same size.
 * @qflag: O: Two-dimensional array which will contain flags for each
 *            image indicating whether the user specified a window or
 *            band option or if window and band options were allowed to
 *            default to the full image.  A 1 (one) in the first element
 *            indicates that the user specified a window option and a 1
 *            (one) in the second element indicates that the user specified
 *            a band option.  A 0 (zero) in either element indicates the
 *            default option has been supplied.  The first dimension is
 *            "imglimit", the second dimension is 2.
 *
 * The las_getin2() routine is used to parse an input image specification.
 *
 * The difference between las_getin() and las_getin2() is that las_getin2()
 * is designed to be called from outside of TAE.
 *
 * Note:  No error message is printed if the user specifies multiple images
 * with different window sizes.  Although most programs will typically
 * consider this an error, there may be some modules which will want to
 * allow multiple images to be specified with different windows.  Thus, it
 * is up to the individual program to decide whether images with different
 * window sizes are valid or not.  If not, the program will need to do its
 * own error handling.
 *
 * Returns:    E_SUCC:  Successful completion of operation,
 *             E_FAIL:  Unsuccessful completion of operation.
 */
int las_getin2
(
    const  char inparm[],       /* I:  Input image string with              */
    char   tnames[][CMLEN],     /* O:  TAE names of images specified        */
    char   hnames[][CMLEN],     /* O:  Host names of images specified       */
    int    *images,             /* O:  Number of images                     */
    int    (*window)[4],        /* O:  Image windows (sl ss nl ns)          */
    int    (*bands)[MAXBND+1],  /* O:  Image bands                          */
    int    *nbands,             /* O:  Number of bands per image            */
    int    *totbnd,             /* O:  Total number of bands specified      */
    int    imglimit,            /* I:  Maximum images caller can handle     */
    int    *same_window,        /* O:  1 if all windows the same            */
    int    (*qflag)[2]          /* O:  1 user defined window & bands else
                                           if default window and bands      */
)
{
    unsigned short  image;      /* image counter and index                  */
    int  mode = 0;              /* file access                              */


    if (c_getin2_nocheck (inparm,tnames,hnames,images,window,bands,nbands,
                          totbnd,imglimit,same_window,qflag) != E_SUCC)
        return (E_FAIL);

    /*                  Verify that the images exist                        */
    for (image = 0; image < *images; image++)
    {
        if (c_labstat(hnames[image],mode) != E_SUCC)
        {
            c_lasmsg(FALSE, "getin-exist", "Image file %s does not exist",
                     tnames[image]);
            return (E_FAIL);
        }
    }

    return (E_SUCC);
}

#ifndef NO_OBSOLETE_INTERFACE

/*****************************************************************************
    Function:  c_getin2
    Purpose:
        Parse image names, bands, and windows.  The image will be confirmed to
        exist.  May be called by applications that do not use the TAE
        interface.  For the applications that do use TAE, c_getin must be
        called.  Calls c_getin2_nocheck to do the actual parse.
*****************************************************************************/

/**
 * c_getin2:
 * @inparm: I: Input image string associated with optional window 
 *             specification.
 * @tnames: O: Array of character strings dimensioned to be the same size as
 *             parameter "imglimit".  It contains the TAE names of the images
 *             specified in parameter "param_buf".
 * @hnames: O: Array of character strings dimensioned to be the same size as
 *             parameter "imglimit".  It contains the host names of the 
 *             images specified in parameter "param_buf".
 * @images: O: Number of images defined in the image specification.
 * @window: O: Two-dimensional array containing the window specification
 *             for each image specified.  The first dimension is "imglimit"
 *             and the second is 4.  The four elements of the window are
 *             SL, SS, NL, and NS.
 * @bands: O: Two-dimensional array containing the band numbers selected
 *            for each image specified.  The first dimension is "imglimit"
 *            and the second is MAXBND + 1.  One is added to maxbnd in
 *            order to allow for the zero terminator which is needed by
 *            c_eopens().
 * @nbands: O: Number of bands specified for each input image.  This array
 *             is useful when calling the function c_eopens() or c_eopenr()
 *             to open the input images.
 * @totbnd: O: Total number of bands to be processed.  This value is
 *             useful for the processing message utility c_inittm().
 * @imglimit: I: Maximum number of images the calling routine can handle
 *               when c_getin2() is called.
 * @same_window: O: Signals whether or not multiple images have the same
 *                  window size.  A value of 0 (zero) means that the
 *                  images do not all have the same window size while a
 *                  value of 1 (one) indicates that all windows are of the
 *                  same size.
 * @qflag: O: Two-dimensional array which will contain flags for each
 *            image indicating whether the user specified a window or
 *            band option or if window and band options were allowed to
 *            default to the full image.  A 1 (one) in the first element
 *            indicates that the user specified a window option and a 1
 *            (one) in the second element indicates that the user specified
 *            a band option.  A 0 (zero) in either element indicates the
 *            default option has been supplied.  The first dimension is
 *            "imglimit", the second dimension is 2.
 *
 * The c_getin2() routine is used to parse an input image specification.
 *
 * The difference between c_getin() and c_getin2() is that c_getin2()
 * is designed to be called from outside of TAE.
 *
 * Note:  No error message is printed if the user specifies multiple images
 * with different window sizes.  Although most programs will typically
 * consider this an error, there may be some modules which will want to
 * allow multiple images to be specified with different windows.  Thus, it
 * is up to the individual program to decide whether images with different
 * window sizes are valid or not.  If not, the program will need to do its
 * own error handling.
 *
 * Deprecated.  Use las_getin2() instead.
 *
 * Returns:    E_SUCC:  Successful completion of operation,
 *             E_FAIL:  Unsuccessful completion of operation.
 */
int c_getin2
(
    const  char inparm[],       /* I: Input image string with               */
    char   tnames[][CMLEN],     /* O: TAE names of images specified         */
    char   hnames[][CMLEN],     /* O: Host names of images specified        */
    long    *images,            /* O: Number of images                      */
    long    (*window)[4],       /* O: Image windows (sl ss nl ns)           */
    long    (*bands)[MAXBND+1], /* O: Image bands                           */
    long    *nbands,            /* O: Number of bands per image             */
    long    *totbnd,            /* O: Total number of bands specified       */
    const  long *imglimit,      /* I: Maximum images caller can handle      */
    long    *same_window,       /* O: 1 if all windows the same             */
    long    (*qflag)[2]         /* O: 1 user defined window & bands else
                                        if default window and bands         */
)
{
    int status;

    status = las_getin2 (inparm,tnames,hnames,images,window,bands,nbands,
                         totbnd,*imglimit,same_window,qflag);

    return (status);

}

#endif /* obsolete interfaces */

