/******************************************************************************
NAME:   GETNAM

FUNCTION:
        GETNAM is a routine which retrieves a file name specification from
        the TAE parameter block.  It appends the specified extension to the 
        name if the extension was not specified.  If the file is to be opened
        for output, a bad status is returned if the file already exists on disk.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0        12/86       K. Gacke     initial development
    1.1        12/87       B. Ailts     Change include directory specifications
                                        Replace DESC arguments with char. string
                                        Use raw 'C' types
                                        Place bridge routines in seperate file
    5.0        04/88       D. Hollaren  changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed status flag by address
    5.1         09/88      B. Ailts     Now returns a null value in the host
                                        name when the parameter has a null value
    5.2         10/88      B. Ailts     Added argument to access multiple
                                        dimensioned parameters
    5.3         07/90      D. VanderZee Standardized error handling 
    5.4         10/04      D. Etrheim   Added check to verify that file has
                                        read permission

PROJECT:         LAS

ALGORITHM:
        Retrieve the file name from the TAE parameter block.
        Replace any aliases withing the TAE parameter
        If it does not have the specified extension,
            the extension is appended.
        The full host file name is retrieved.
        If the file is to be opened for output and the file already exists,
            a bad status code is returned.
        If the file is to be opened for input and the file does not exist,
            a bad status code is returned.

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "alias.h"
#include "labserv.h"

/**
 * las_getnam:
 * @vblock:  I: TAE parameter block passed by reference.  It is assumed that
 *              TAE has been initialized prior to calling las_getnam().
 * @param: I: Defines the TAE parameter name in the TAE parameter block
 *            which contains the file specifications.
 * @element: I: Defines which element of the input parameter array specified
 *              by "param" will be 0 (zero) in 'C' for input parameters 
 *              that contain only a single element.
 * @fname: O: Character string which contains the host name of the file
 *            specified in TAE parameter "param".
 * @ext: I: Character string denoting the default extensions for the file.
 *          If the user does not specify an extension (;), this extension
 *          is appended to the file name.  If the user specified an
 *          extension, the file name is not changed.
 * @access: I: Denotes the type of access for which the file is to opened.
 *             If access is for input, a check is made to ensure the file
 *             exists on disk.  If access is for output, a check is made to
 *             ensure the file does not exist.  If access is for update, no
 *             check is made if the file exists or not.
 *             Valid values for access are:
 *             IREAD:   Read access,
 *             IWRITE:  Write access,
 *             IUPDATE: Update access.
 *
 * The las_getnam() routine is used to return the host file name of the
 * file specifications defined by TAE parameter "param".
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int las_getnam
(
    struct PARBLK *vblock,  /* I: TAE parameter block                       */
    const char *param,      /* I: TAE parameter name                        */
    int element,            /* I: element of parameter                      */
    char *fname,            /* O: Host file name to be returned             */
    const char *ext,        /* I: default extension                         */
    int access              /* I: Type of I/O access                        */
)

{
    struct VARIABLE *val;
    int   mode = 0;
    char  msg[ERRLEN+1];
    char  tfull[CMLEN];
    char  inparm[CMLEN];
    char  tname[CMLEN];
    char  exten[CMLEN];
    char  *ptr;             /* temporary character pointer                  */

    /* copy the extension to a temporary variable since gcc 2.95.1 seems
       to have a bug and puts an immediate string passed in on the "ext" 
       param in constant memory space.  This then causes a core dump
       in c_up2low. */
    strcpy(exten,ext);

    val = p_find(vblock,param);

    /*  Return a blank if the parameter has a null value                    */
    if (val->v_count < element + 1)
    {
        strcpy(fname,"\0");
        return(E_SUCC);
    }

    strcpy(inparm,SVAL(*val,element));
    if (c_getalias(inparm,tname) != E_SUCC)
    {
        c_lasmsg (FALSE, "getnam-call", "Error returned from getalias");
        return (E_FAIL);
    }

    /* Determine whether name is in VMS or Unix syntax.  Check for 
       appropriate extension, and add if missing.                           */

    if (strchr(tname, TAE_DIR_BEG) || strchr(tname, TAE_EXT))  /* VMS */
    {
        /* If no extension, add default.  If no default, add nothing.       */
        if (strrchr(tname, TAE_EXT) == NULL && (strlen(exten) != 0))
            sprintf(tfull, "%s%c%s", tname, TAE_EXT, exten);
        else
            strcpy(tfull, tname);
    }
    else /* Unix */
    {
        /* Since dots can reside in the path as well as the filename,
           we need to ensure that we are checking for dots (as extensions)
           after any path that may be included.                             */

        if ((ptr = strrchr(tname, HOST_DIR_SEP)) == NULL)
            ptr = tname;

        /* If no extension, add default.  If no default, add nothing.       */

        if ((strrchr(ptr, HOST_EXT) == NULL) && (strlen(exten) != 0))
            sprintf(tfull, "%s%c%s", tname, HOST_EXT, exten);
        else
            strcpy(tfull, tname);
    }

    if (c_cmhost(tfull,fname) != E_SUCC)
    {
        c_lasmsg (FALSE, "getnam-call", "Error returned from cmhost");
        sprintf (msg,"Error retrieving host file name for %s",tfull);
        c_lasmsg (FALSE, "getnam-cmhost", msg);
        return (E_FAIL);
    }

    if (access == IREAD)           /* if file is opened for read access     */
    {                              /*   the file must exist                 */
        if (c_labstat(fname,mode) != E_SUCC)
        {
            sprintf (msg,"Error: %s does not exist",tfull);
            c_lasmsg (FALSE,"getnam-exist", msg);
            return (E_FAIL);
        }
        /* Verify that the file has read permission */
        mode = 4;
        if (c_labstat(fname,mode) != E_SUCC)
        {
            sprintf (msg,"Error: %s does not have read permission",tfull);
            c_lasmsg (FALSE, "getnam-read", msg);
            return (E_FAIL);
        }
    }

    if (access == IWRITE)         /* if file is opened for write access     */
    {                             /*   the file cannot exist                */
        if (c_labstat(fname,mode) == E_SUCC)
        {
            sprintf (msg,"Error: %s already exists",tfull);
            c_lasmsg (FALSE, "getnam-exists", msg);
            return (E_FAIL);
        }
    }

    return(E_SUCC);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_getnam:
 * @vblock:  I: TAE parameter block passed by reference.  It is assumed that
 *              TAE has been initialized prior to calling c_getnam().
 * @param: I: Defines the TAE parameter name in the TAE parameter block
 *            which contains the file specifications.
 * @element: I: Defines which element of the input parameter array specified
 *              by "param" will be 0 (zero) in 'C' for input parameters 
 *              that contain only a single element.
 * @fname: O: Character string which contains the host name of the file
 *            specified in TAE parameter "param".
 * @ext: I: Character string denoting the default extensions for the file.
 *          If the user does not specify an extension (;), this extension
 *          is appended to the file name.  If the user specified an
 *          extension, the file name is not changed.
 * @access: I: Denotes the type of access for which the file is to opened.
 *             If access is for input, a check is made to ensure the file
 *             exists on disk.  If access is for output, a check is made to
 *             ensure the file does not exist.  If access is for update, no
 *             check is made if the file exists or not.
 *             Valid values for access are:
 *             IREAD:   Read access,
 *             IWRITE:  Write access,
 *             IUPDATE: Update access.
 *
 * The c_getnam() routine is used to return the host file name of the
 * file specifications defined by TAE parameter "param".
 *
 * Deprecated. Use las_getnam() instead.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_getnam
(
    struct PARBLK *vblock,  /* I: TAE parameter block                       */
    const char *param,      /* I: TAE parameter name                        */
    const long *element,    /* I: Element of parameter                      */
    char *fname,            /* O: Host file name to be returned             */
    const char *ext,        /* I: Default extension                         */
    const long *access      /* I: Type of I/O access                        */
)
{
    int status;
    
    status = las_getnam (vblock,param,*element,fname,ext,*access);

    return (status);

}
#endif /* obsolete interfaces */

