/******************************************************************************
NAME:   GETTYP

FUNCTION:
        GETTYP is a routine which returns the data type for an array of 
        images.  It returns a bad status code if the DDR cannot be read,
        or if the data types of the images are not the same.  No error 
        message is output.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         8/86       K. Gacke     initial development
    1.1        12/87       B. Ailts     Change include directory specifications
                                        Use raw 'C' types
                                        Replace DESC arguments with char. desc.
                                        Place bridge routines in a seperate file
    1.2        12/87       B.Ailts      Implemented new DDR structure and call
    5.0        04/88       D. Hollaren  changed include spec to las
                                        removed the call to squeeze routine
                                        return E_SUCC/E_FAIL
    5.1        08/88       B. Ailts     Fixed the declaration of the hostname
     5.2        07/90      D. VanderZee Standardized error handling 


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

ALGORITHM:
        For each image 
            "c_getddr" is called to get the data type of the image.
            If the data type of the current image is not the same as the data 
            type of the previous image,
                 a bad status is returned.
        Exit

ALGORITHM REFERENCES:

******************************************************************************/

#include "las.h"
#include "util.h"

/**
 * las_gettyp:
 * @hosin: I: Array of image host file names for which type is to be returned.
 * @nmrimg: I: Number of images specified in parameter "hosin".  In most cases,
 *             the parameter "nmrimg" is returned by the function c_getin().
 * @odtype: O: Data type of the image(s) to be returned to the calling program.
 * 
 * The las_gettyp() routine is used to return the data type of the image(s).  
 * It returns a bad status code if the DDR cannot be read or if the data types
 * of the images are not the same.  No error message is output.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int las_gettyp
(
    const char hosin[][CMLEN],  /* I: input host file names                 */
    int nmrimg,                 /* I: number of input images                */
    int  *odtype                /* O: output data type, type of input img   */
)

{
    struct DDR ddr;         /* DDR record                                   */
    int  image;             /* loop index for current input image           */
    int  idtype;            /* input image data type                        */

    /* Retrieve the DDR for each image.  If the data types of the input
       images are not the same, a bad status code is returned.  Otherwise
       the input data type is assigned to "odtype" which is returned to
       the calling program.                                                 */

    for (image = 0; image < nmrimg; image++)
    {
        if (c_getddr(hosin[image],&ddr) != E_SUCC)
        {
            c_lasmsg (FALSE, "gettyp-call", "Error returned from getddr");
            return(E_FAIL);
        }
        idtype = ddr.dtype;
        if (image == 0)
            *odtype = idtype;
        else if (idtype != *odtype)
        {
            c_lasmsg (FALSE, "gettyp-dtype",
                      "Error: Data types of images are not the same");
            return(E_FAIL);
        }
    }

    return(E_SUCC);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_gettyp:
 * @hosin: I: Array of image host file names for which type is to be returned.
 * @nmrimg: I: Number of images specified in parameter "hosin".  In most cases,
 *             the parameter "nmrimg" is returned by the function c_getin().
 * @odtype: O: Data type of the image(s) to be returned to the calling program.
 * 
 * The c_gettyp() routine is used to return the data type of the image(s).  It
 * returns a bad status code if the DDR cannot be read or if the data types
 * of the images are not the same.  No error message is output.
 *
 * Deprecated.  Use las_gettyp() instead.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_gettyp
(
    const char hosin[][CMLEN], /* I: input host file names                  */
    const long *nmrimg,        /* I: number of input images                 */
    long  *odtype              /* O: output data type, type of input img    */
)
{
    int status;
    int i_odtype;

    status = las_gettyp (hosin,*nmrimg,&i_odtype);
    *odtype = i_odtype;

    return (status);

}
#endif /* obsolete interfaces */

