/******************************************************************************
NAME:   C_HSREAD

FUNCTION: Reads a record of a history file.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
     1        3-Jan-89  K. A. Narayanan
     2       27-Jun-89  K. A. Narayanan  freed buffer after use
     3       16-Oct-89  D. Akkerman      modified to allocate a buffer 
                                         adequate for the history record and 
                                         return a pointer to the buffer
     5.0        07/90      D. VanderZee Standardized error handling 


PROJECT:        LAS

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "labserv.h"


/**
 * las_history_read:
 * @fd: I: File descriptor of an input image history file.
 * @key: I/O: Key of record to be read, or key read if any key requested
 *            Spaces may be embedded within the key: however, right
 *            justified spaces are ignored.  If the key is NULL or a
 *            blank, the current record within the file is read.
 *            The key value of the record read is returned in this parameter.
 * @buf: O: History buffer.
 * @len: I/O: Character length of label service record to be read.
 *
 * The las_history_read() routine calls c_labread() routine to read a record of
 * a history file.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int las_history_read
(
    int  fd,                /* I: File descriptor of in image hist          */
    char key[],             /* I/O: Key of record to be read                */
    char **buf,             /* O: History buffer                            */
    int  *len               /* I/O: Char length of label service rec        */
)

{
    int  dlen = 0;          /* data length of label service record          */
    int  stat;
    int  status;            /* return status code                           */

    unsigned char *dbuf = NULL;  /* data buffer pointer                     */
    char    dtype[4];            /* data type buffer                        */
    char    *tptr = NULL;        /* working character pointer               */

    status = c_labread (fd,key,len,&dlen,tptr,dbuf,dtype);
    if (status == E_SMAL)
    {
        if ((*buf = (char *)malloc(*len)) == NULL)
        {
            c_lasmsg (FALSE, "hsread-alloc", "Error allocating dynamic memory");
            return (E_FAIL);
        }
        tptr = *buf;
        stat = c_labread (fd,key,len,&dlen,tptr,dbuf,dtype);
        if (stat == E_FAIL)
        {
            c_lasmsg (FALSE, "hsread-call", "Error returned from labread");
            return (stat);
        }
    }
    else if (status == E_FAIL)
        c_lasmsg (FALSE, "hsread-call", "Error returned from labread");
   
    return (status);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_hsread:
 * @fd: I: File descriptor of an input image history file.
 * @key: I/O: Key of record to be read, or key read if any key requested
 *            Spaces may be embedded within the key: however, right
 *            justified spaces are ignored.  If the key is NULL or a
 *            blank, the current record within the file is read.
 *            The key value of the record read is returned in this parameter.
 * @buf: O: History buffer.
 * @len: I/O: Character length of label service record to be read.
 *
 * The c_hsread() routine calls c_labread() routine to read a record of
 * a history file.
 *
 * Deprecated.  Use las_history_read() instead.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_hsread
(
    const long *fd,   /* I:  file descriptor of input images's history      */
    char  key[],      /* I/O: key of history record to be read              */
    char  **buf,      /* O:  history buffer                                 */
    long  *len        /* I/O:  char length of label service record          */
)
{
    int status;
    int ilen = *len;

    status = las_history_read (*fd,key,buf,&ilen);
    *len = ilen;

    return (status);
}

#endif /* obsolete interfaces */


