/*******************************************************************************
NAME                            C_GETIN

PURPOSE         Break input specification into individual image specifications.

PROGRAM HISTORY
        PROGRAMMER        DATE          REASON
        ----------        ----          ------
        P. Mumford       3/15/85        Original development for NEWLAS.
        K. Gacke         9/09/85        Updated the include files and fixed a
                                        bug involving parameter "same_window"
        L. Huewe        10/17/85        Reimplemented QFLAG parameter.
        K. Gacke         9/15/86        Added parameters "nbands" and "totbnd"
                                        to return number of bands per image and
                                        total number of bands.
        K. Gacke        12/12/86        Allow user to name images with any
                                        extension and not just "img"
        K. Gacke         2/2/87         Modified to allow device to be specified
                                        on VMS.  The colon character has two
                                        meanings depending on it position: 
                                        1) separator of device and directory
                                        2) separator of window and bands
        K. Gacke         2/2/87         Ensure no spaces trail the image name
        K. Zanter        12/87          1.  switch to raw C types 
                                        2.  strings descriptor arguments to 
                                            character strings
                                        3.  changed include file specification
                                        4.  renamed source from "getin" to
                                            "c_getin"
                                        5.  moved bridge routine to separate
                                            source file
        D. Hollaren      04/88          changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed the status code by address
        B. Ailts        07/88           Added better error checking
        B. Ailts        09/88           Returns *images as zero if the
                                        image_name[*element] was not specified
        B. Ailts        09/88           Returns a null values in the host and
                                        LAS names if the image_name[*element] 
                                        was not found in the parameter block
        D. VanderZee    07/90           Standardized error handling 
        D. Etrheim      09/90           Allow a band range to be specified
        T. Mittan       09/92           type cast pos and left to ints
                                        in comparision statments for ANSII
                                        standard compilers
        T. Mittan       09/92           Changed type casts and placed 
                                        (char *) on constants instead
        T. Mittan       07/93           Changed PRS_LST to double, added
                                        new windowing options, added 
                                        make_proj
        D. Bridges      10/93           Split c_getin into c_getin and c_getin2
                                        where c_getin is called from TAE and
                                        c_getin2 is called from outside TAE
        D. Etrheim      10/93           Return status from c_getin2
        D. Lloyd        08/98           Split from c_getin and renamed c_imgprs


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
        C_GETIN must be run under TAE.

PROJECT                         LAS

ALGORITHM:
        Get the value of the input image parameter
        If the count of the input image names are less than *element + 1
           String copy a blank into the tae file names
           String copy a blank into the host file names
           Set *images to zero
           Return
        Substitute any aliases in the input image parameter
        Parse the string and return the name, window, and bands in an array,
                also the number of images
        For each image
                Get the full TAE name
                Get the host name
                Check the window for accuracy and default value
                Check the bands for accuracy and default value
        End for


ALGORITHM REFERENCES
        This algorithm is roughly based on the GET_IN utility in LAS.

******************************************************************************/

#include <ctype.h> 
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"

static void compress
(
    const char *filled,/* I: Image specifications received from c_imgprs()  */
    char *small        /* O: Char buffer to hold compressed string          */
)
{
    int   i;
    int   j;
    enum    states
    {
        digit, non_digit
    }   p_state;

    /* Copy all characters except white space.  If a digit was just
       read, copy a space and then set back to a non-digit state.           */
    p_state = non_digit;
    i = j = 0;
    while ( filled[i] != '\0' )
    {
        if ( isspace(filled[i]) )
        {
            if ( p_state == digit )
            {
                small[j++] = ' ';
                i++;
            }
            else
            {
                i++;
            }
            p_state = non_digit;
        }
        else if ((filled[i] == ','))
        {
            {
                small[j++] = ' ';
                i++;
            }
            p_state = non_digit;
        }
        else if ( (j>0) && (filled[i]==':') && (small[j-1]==' ') )
        {
            small[j-1] = ':';
            i++;
            p_state = non_digit;
        }
        else if ( (j>0) && (filled[i]==')') && (small[j-1]==' ') )
        {
            small[j-1] = ')';
            i++;
            p_state = non_digit;
        }
        else
        {
            if ( isdigit(filled[i]) )       
                { p_state = digit; }
            else   
                { p_state = non_digit; };
                small[j++] = filled[i++];
        }
    };
    small[j] = '\0';

}


static int make_window
(
    char *numerals,             /* I/O: Window components: SL, SS, NL, NS   */
    double window[]             /* O: Array for valid window data           */ 
)
{
    unsigned short  wind;       /* window counter and index                 */
    char  *n_begin;             /* first numeral in window component        */
    char  *n_end;               /* the space following the numeral          */

    wind = 0;
    n_begin = numerals;
    do
    {
        n_end = strchr( n_begin, ' ' );
        if ( n_end == 0 )
        {
            sscanf(n_begin,"%lf",&window[wind++]);
        }
        else
        {
            *n_end = '\0';
            sscanf(n_begin,"%lf",&window[wind++]);
            *n_end = ' ';
            n_begin = n_end + 1;
        }
    } while ( (wind<4) && (n_end!=0) );

    if ( (wind==4) && (n_end!=0) )
    {
        c_lasmsg (FALSE, "getin-invwnd",
                  "Error: There are too many window components");
        return (E_FAIL);
    }
    else if ( (wind<4) && (n_end==0) )
    {
        c_lasmsg (FALSE, "getin-invwnd",
                 "Error: There are not enough window components");
        return (E_FAIL);
    }
    return E_SUCC;
}


static int make_bands
(
    char *numerals,         /* I/O: Band components                         */
    int  bands[]            /* 0: Array for valid band data                 */
)
{
    unsigned short  band;   /* band counter and index                       */
    char  *n_begin;         /* first numeral in band component              */
    char    *n_end;         /* the space following the numeral              */
    char    *n_dash;        /* postion of the '-' for band range            */
    int     end;
    int     i;
    int     start;          /* loop variables for band range                */

    band = 0;
    n_begin = numerals;
    do
    {
        n_end = strchr( n_begin, ' ' );
        n_dash = strchr( n_begin, '-');
        if ( n_end == n_dash )
        {
            bands[band++] = atoi( n_begin );
        }
        else if (((n_end < n_dash) && (n_end !=0)) || n_dash==0 )
        {
            *n_end = '\0';
            bands[band++] = atoi( n_begin );
            *n_end = ' ';
            n_begin = n_end + 1;
        }
        else
        {
            start = atoi( n_begin );
            n_begin = n_dash + 1;
            end = atoi( n_begin );
            for (i = start; i <= end; i++)
                bands[band++] = i;
            n_begin = n_end + 1;
        }
    } while ( (band<=MAXBND) && (n_end!=0) );

    if ( band>MAXBND ) 
    {
        c_lasmsg (FALSE, "getin-invbnds", 
                  "Error: There are too many band components");
        bands[band] = 0;
        return (E_FAIL);
    }

    bands[band] = 0;
    return(E_SUCC);
}


static int make_proj
(
    char *numerals,         /* I/O: Projection components                   */  
    int  proj[]             /* O: Array for valid projection data           */
)
{
    unsigned short  band;   /* band counter and index                       */
    char  *n_begin;         /* first numeral in band component              */
    char  *n_end;           /* the space following the numeral              */
    char  *n_dash;          /* postion of the '-' for band range            */

    band = 0;
    n_begin = numerals;
    do
    {
        n_end = strchr( n_begin, ' ' );
        n_dash = strchr( n_begin, '-');
        if ( n_end == n_dash )
        {
            if ((strcmp(n_begin,"LS") == 0) || 
                (strcmp(n_begin,"ls") == 0))
            {
                proj[0] = 0;
                band++;
            }
            else if ((strcmp(n_begin,"DEG") == 0) || 
                     (strcmp(n_begin,"deg") == 0))
            {
                proj[0] = 4;
                band++;
            }
            else if ((strcmp(n_begin,"DMS") == 0) || 
                     (strcmp(n_begin,"dms") == 0))
            {
                proj[0] = 5;
                band++;
            }
            else if ((strcmp(n_begin,"PRO") == 0) || 
                     (strcmp(n_begin,"pro") == 0))
            {
                proj[0] = 3;
                band++;
            }
            else
                proj[0] = 0;
        }
    } while ( (band == 0) && (n_end!=0) );

    if ( band > 1 ) 
    {
        c_lasmsg (FALSE, "getin-invbnds",
                  "Error: There are too many projection components");
        proj[band] = 0;
        return (E_FAIL);
    }

    proj[band] = 0;
    return(E_SUCC);
}

/**
 * image_info:
 * @spec: I: Single image specification.
 * @i_record: O: Record for name, window, and bands.
 *
 * The image_info() routine calls make_window() and make_proj() to check the
 * window and bands for accuracy and default values.  It then parses the
 * string and returns the name, window, and band in "i_record". 
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
static int image_info
(
    char *spec,                 /* I: single image specification            */
    struct PRS_LST *i_record    /* O: record for name, window, & bands      */
)
{
    char  *left;          /* position of left parenthesis                   */
    char  *colon;         /* position of colon                              */
    char  *right;         /* position of right parenthesis                  */
    char  *semi;          /* position of ';'                                */
    char  *ptr;           /* temporary pointer                              */
    char  err_text[ERRLEN];
    unsigned short  wind;           /* window index and counter             */

    /* Find position of (, :, and ).                                        */
    left = strchr( spec, '(' );
    if (left != NULL)
    {
        colon = strchr( left, ':' );
        semi = strchr( left, ';' );
    }
    else
    {
        colon = NULL;
        semi = NULL;
    }

    right = strchr( spec, ')' );
    
    /* make sure multiple images with parenthesis are properly joined with
       +'s.  If they are, the character after the ')' will be a null */
    if (right)
    {
        if (right[1] != '\0')
        {
            sprintf(err_text, "Error: Multiple images not joined by a '+' "
                    "in {%s}.", spec );
            c_lasmsg (FALSE, "getin-multimg", err_text);
            return(E_FAIL);
        }
    }
    
    if ( (left==0) && (colon==0) && (right==0) && (semi == 0))
                                    /* no window, bands or projection 
                                           specified */
    {
        strcpy( i_record->name, spec);
        for ( wind=0; wind<4; wind++ ) 
            i_record->window[wind] = 0;
        i_record->bands[0] = 0;
        i_record->qflag[0] = i_record->qflag[1] = 0;
        i_record->proj[0] = 0;
        
    }
    else if (( left > NULL ) && (semi == NULL))
    {
        i_record->proj[0] = 0;
        *left = '\0';
        strcpy( i_record->name, spec );
        *left = '(';
        if ( (colon==0) && (left<right) )
            /*      IMAGE(WINDOW)   */
        {
            *right = '\0';
            if (make_window(left+1,i_record->window) != E_SUCC)
            { 
                *right = ')';
                sprintf( err_text,
                        "Error: Window format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invwnd", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->bands[0] = 0;
            i_record->qflag[0] = 1;
        }
        else if ( (left<colon) && (colon+1==right) )
                                        /*      IMAGE(WINDOW:)  */
        {
            *colon = '\0';
            if (make_window(left+1,i_record->window) != E_SUCC)
            {
                *colon = ':';
                sprintf( err_text,
                        "Error: Window format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invwnd", err_text);
                return(E_FAIL);
            }

            *colon = ':';
            i_record->bands[0] = 0;
            i_record->qflag[0] = 1;
        }
        else if ( (left+1==colon) && (colon<right) )
                                        /*      IMAGE(:BANDS)   */
        {
            for ( wind=0; wind<4; wind++ )
                i_record->window[wind] = 0;
            *right = '\0';
            if (make_bands(colon+1,i_record->bands) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Band format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[1] = 1;
        }
        else if ( (left<colon) && (colon<right) )
                                        /*      IMAGE(WINDOW:BANDS)     */
        {
            *colon = '\0';
            if (make_window(left+1,i_record->window) != E_SUCC)
            {
                *colon = ':';
                sprintf( err_text,
                        "Error: Window format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invwnd", err_text);
                return(E_FAIL);
            }

            *colon = ':';
            *right = '\0';
            if (make_bands(colon+1,i_record->bands) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Band format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[0] = i_record->qflag[1] = 1;
        }
        else
                                        /* cannot parse window and bands    */
        {
            sprintf( err_text,
                    "Error: Specification %s cannot be parsed",spec );
            c_lasmsg (FALSE, "getin-invspec", err_text);
            return(E_FAIL);
        }
    }
    else if (( left > NULL  ) && (colon == NULL))
    {
        i_record->bands[0] = 0;
        *left = '\0';
        strcpy( i_record->name, spec );
        *left = '(';
        if ( (left<semi) && (semi+1==right) )
                                        /*      IMAGE(WINDOW;)              */
        {
            *semi = '\0';
            if (make_window(left+1,i_record->window) != E_SUCC)
            {
                *semi = ';';
                sprintf ( err_text,
                         "Error: Window format error was generated by %s",
                          spec );
                c_lasmsg (FALSE,"getin-invwnd", err_text);
                return(E_FAIL);
            }

            *semi = ';';
            i_record->proj[0] = 0;
            i_record->qflag[0] = 1;
        }
        else if ( (left+1==semi) && (semi<right) )
                                        /*      IMAGE(;PROJECTION           */
        {
            for ( wind=0; wind<4; wind++ )
                i_record->window[wind] = 0;
            *right = '\0';
            if (make_proj(semi+1,i_record->proj) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Projection format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[1] = 1;
        }
        else if ( (left<semi) && (semi<right) )
                              /*      IMAGE(WINDOW;PROJECTION               */
        {
            *semi = '\0';
            if (make_window(left+1,i_record->window) != E_SUCC)
            {
                *semi = ';';
                sprintf( err_text,
                        "Error: Window format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invwnd", err_text);
                return(E_FAIL);
            }

            *semi = ';';
            *right = '\0';
            if (make_proj(semi+1,i_record->proj) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Projection format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[0] = i_record->qflag[1] = 1;
        }
        else
                                        /* cannot parse window and bands    */
        {
            sprintf( err_text,
                    "Error: Specification %s cannot be parsed",
                     spec );
            c_lasmsg (FALSE, "getin-invspec", err_text);
            return(E_FAIL);
        }
    }
    else if (( left > NULL ) && (semi != NULL) && 
             (colon != NULL))
    {
        i_record->proj[0] = 0;
        *left = '\0';
        strcpy( i_record->name, spec );
        *left = '(';
        if ( (left<colon) && (colon+1==semi) && (semi+1==right) )
                                        /*      IMAGE(WINDOW:;)             */
        {
            *colon = '\0';
            *semi = '\0';
            if (make_window(left+1,i_record->window) != E_SUCC)
            {
                *colon = ':';
                *semi = ';';
                sprintf( err_text,
                        "Error: Window format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invwnd", err_text);
                return(E_FAIL);
            }

            *colon = ':';
            *semi = ';';
            i_record->bands[0] = 0;
            i_record->qflag[0] = 1;
            i_record->proj[0] = 0;
        }
        else if ( (left+1==colon) && (colon+1==semi) && (semi<right))
                                       /*      IMAGE(:;PROJECTION)          */
        {
            for ( wind=0; wind<4; wind++ )
                i_record->window[wind] = 0;
            *right = '\0';
            if (make_proj(semi+1,i_record->proj) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Projection format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[1] = 1;
            i_record->bands[0] = 0;
        }
        else if ( (left+1==colon) && (colon<semi) && (semi+1==right))
                                        /*      IMAGE(:BANDS;)              */
        {
            for ( wind=0; wind<4; wind++ )
                i_record->window[wind] = 0;
            *right = '\0';
            if (make_bands(colon+1,i_record->bands) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Band format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[1] = 1;
            i_record->proj[0] = 0;
        }
        else if ( (left+1==colon) && (colon<semi) && (semi<right))
                                       /*      IMAGE(:BANDS;PROJECTION)     */
        {
            for ( wind=0; wind<4; wind++ )
                i_record->window[wind] = 0;
            *right = '\0';
            if (make_bands(colon+1,i_record->bands) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Band format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }
            if (make_proj(semi+1,i_record->proj) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Projection format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[1] = 1;
        }
        else if ( (left<colon) && (colon<semi) && (semi+1==right) )
                                       /*      IMAGE(WINDOW:BANDS;)         */
        {
            *colon = '\0';
            if (make_window(left+1,i_record->window) != E_SUCC)
            {
                *colon = ':';
                sprintf( err_text,
                        "Error: Window format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invwnd", err_text);
                return(E_FAIL);
            }

            *colon = ':';
            *right = '\0';
            if (make_bands(colon+1,i_record->bands) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Band format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[0] = i_record->qflag[1] = 1;
            i_record->proj[0] = 0;
        }
        else if ( (left<colon) && (colon+1==semi) && (semi<right) )
                                        /*      IMAGE(WINDOW:;PROJECTION)   */
        {
            *colon = '\0';
            if (make_window(left+1,i_record->window) != E_SUCC)
            {
                *colon = ':';
                sprintf( err_text,
                        "Error: Window format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invwnd", err_text);
                return(E_FAIL);
            }

            *colon = ':';
            *right = '\0';
            if (make_proj(semi+1,i_record->proj) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Band format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[0] = i_record->qflag[1] = 1;
            i_record->bands[0] = 0;
        }
        else if ( (left<colon) && (colon<semi) && (semi<right) )
                                       /*     IMAGE(WINDOW:BANDS;PROJECTION)*/
        {
            *colon = '\0';
            if (make_window(left+1,i_record->window) != E_SUCC)
            {
                *colon = ':';
                sprintf( err_text,
                        "Error: Window format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invwnd", err_text);
                return(E_FAIL);
            }

            *colon = ':';
            *right = '\0';
            if (make_bands(colon+1,i_record->bands) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Band format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }
            if (make_proj(semi+1,i_record->proj) != E_SUCC)
            {
                *right = ')';
                sprintf( err_text,
                        "Error: Projection format error was generated by %s",
                         spec );
                c_lasmsg (FALSE, "getin-invbnds", err_text);
                return(E_FAIL);
            }

            *right = ')';
            i_record->qflag[0] = i_record->qflag[1] = 1;
        }
        else
                                        /* cannot parse window and bands    */
        {
            sprintf( err_text,
                    "Error: Specification %s cannot be parsed",
                     spec );
            c_lasmsg (FALSE, "getin-invspec", err_text);
            return(E_FAIL);
        }
    }
    else
                                        /* cannot parse, no left paren      */
    {
        sprintf (err_text,"Error: No left parenthesis in {%s}.", spec );
        c_lasmsg (FALSE, "getin-lparen", err_text);
        return(E_FAIL);
    }

    ptr = i_record->name + strlen(i_record->name) - 1;
    for (; (*ptr == ' ') && (ptr > i_record->name); ptr--)
        *ptr = '\0';
    return E_SUCC;
}

/**
 * c_imgprs:
 * @i_specs: I: Image specifications.
 * @list: I: Structure containing list of input image names.
 * @images: O: Total number of input images.
 *
 * The c_imgprs() routine parses an ascii list of input image names into
 * a list of names, which includes the image name, the bands requested, and
 * the window requested. If the user inputs "image1(1,1,100,100:2,3) + image2",
 * c_imgprs() will return a list that include a record for image1 and image2.
 * The image1 record will indicate that a window at 1,1 that is 100 lines
 * and samples wide was requested and bands 2 and 3 were requested as well.
 * 
 * Returns:   E_SUCC:   Successful completion of operation,
 *            E_FAIL:   Unsuccessful completion of operation.
 */
int c_imgprs
(
    char *i_specs,          /* I/O: Image specifications                    */
    struct PRS_LST *list,   /* I/O: List of input image names               */
    int *images             /* O: Total number of input images              */
)

{
    char  line[CMLEN+1];          /* holds compressed i_specs               */
    char  *spec_begin;            /* address of next image spec             */
    char  *pos;                   /* position of '+'                        */
    char  err_text[ERRLEN+1];     /* text of error message                  */

    /* Compress i_specs so that there is no white space except a single space
       between numbers                                                      */
    compress( i_specs, line );

    /* For each image specification (separated by '+'s), find the image name,
       window, and bands.*/
    spec_begin = line;
    *images = 0;
    do
    {
        if ( *images > MAXIMG )
        {
            sprintf( err_text," %s %s %d %s.",
                    "Error: Too many images were specified.",
                    "  Only",MAXIMG,"images are allowed");
            c_lasmsg (FALSE, "getin-maximg", err_text);
            return (E_FAIL);
        }

        pos = strchr( spec_begin, '+' );
        if (pos != NULL )
            *pos = '\0';

        if (image_info(spec_begin,&list[*images]) != E_SUCC)
            return (E_FAIL);

        if (pos != NULL ) 
            spec_begin = pos + 1;
        (*images)++;
    }
    while ( pos > NULL );
    return(E_SUCC);
}
