/****************************************************************************

NAME: c_install_cleanup_handler

PURPOSE: Set the signal handler to call the sig_hand function for a standard
  set of signals thrown by TAE.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    01/01  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Requires a system that provides POSIX signal handling.

PROJECT: LAS

NOTES:
  1) At this time this library handles the SIGTERM and SIGABRT signals.
  2) This function should be called from your main function as soon as you
     desire to start catching the SIGTERM and SIGABRT signals.
  3) If the SIGTERM or SIGABRT signals get thrown for any reason, this
     routine will set your program up to catch these signals and call the
     function that is passed in as sig_hand.  Instead of stopping execution
     when these signals are thrown, which is the default, your sig_hand
     function can unmount tapes and CDs and clean up any files that may
     need to be deleted.  Your sig_hand function should then exit with the
     following call 'exit (EXIT_FAILURE);'
  4) signal_cleanup.c in L7HDF2LAS is an example of a cleanup function that
     was written for the L7HDF2LAS application.  Use it as an example for
     other signal handler functions if needed.
  5) It is important to prevent race conditions when using a signal handler.
     Your application should know the exact status of anything that needs to
     be cleaned up, and should be prepared to handle a signal that is thrown
     while cleaning up your files or unmounting your tape/CD.  When deleting
     an array of files, delete them from the end of the list and decrement
     the file counter.  If a signal is thrown while deleting the files, the
     status of the file counter should be correct.
  6) The signal handlers set by this routine should be reset to their
     original status by using the c_remove_signal_handler routine, once they
     are no longer needed.  c_remove_signal_handler will reset the status
     of the install_initialized variable to specify that the cleanup has
     taken place.
  7) This routine should not be called more than once without first resetting
     the signal handlers using c_remove_signal_handler.

****************************************************************************/

#include <stdio.h>
#include <signal.h>
#include "worgen.h"
#include "util.h"
#include "local_protos.h"

static int install_initialized = FALSE;  /* used to specify if the instal
                                            function has been called so
                                            that c_remove_cleanup_handler can
                                            process safely */

/**
 * c_install_cleanup_handler:
 * @sig_hand: I: Signal handler function.
 *
 * The c_install_cleanup_handler() routine is used to change the way
 * signals are handled.  This function resets the signal handler to call the
 * specified function when the signal is thrown by the application. 
 * NOTES:
 * 1) At this time this library handles the SIGTERM and SIGABRT signals.
 * 2) This function should be called from your main function as soon as you
 *    desire to start catching the SIGTERM and SIGABRT signals.
 * 3) If the SIGTERM or SIGABRT signals get thrown for any reason, this
 *    routine will set your program up to catch these signals and call the
 *    function that is passed in as sig_hand.  Instead of stopping execution
 *    when these signals are thrown, which is the default, your sig_hand
 *    function can unmount tapes and CDs and clean up any files that may
 *    need to be deleted.  Your sig_hand function should then exit with the
 *    following call 'exit (EXIT_FAILURE);'
 * 4) signal_cleanup.c in L7HDF2LAS is an example of a cleanup function that
 *    was written for the L7HDF2LAS application.  Use it as an example for
 *    other signal handler functions if needed.
 * 5) It is important to prevent race conditions when using a signal handler.
 *    Your application should know the exact status of anything that needs to
 *    be cleaned up, and should be prepared to handle a signal that is thrown
 *    while cleaning up your files or unmounting your tape/CD.  When deleting
 *    an array of files, delete them from the end of the list and decrement
 *    the file counter.  If a signal is thrown while deleting the files, the
 *    status of the file counter should be correct.
 * 6) The signal handlers set by this routine should be reset to their
 *    original status by using the c_remove_signal_handler routine, once they
 *    are no longer needed.  c_remove_signal_handler will reset the status
 *    of the install_initialized variable to specify that the cleanup has
 *    taken place.
 * 7) This routine should not be called more than once without first resetting
 *    the signal handlers using c_remove_signal_handler.
 *
 * Returns:   E_SUCC:   Successful completion of operation,
 *            E_FAIL:   Unsuccessful completion of operation.
 */
int c_install_cleanup_handler
(
    void (*sig_hand)(int)  /* I: signal handler function                    */
)

{
    int status;            /* return status from sigaction                  */
    struct sigaction sigabrt_act;
                           /* specifies new action to be taken for sigabrt  */
    struct sigaction sigterm_act;
                           /* specifies new action to be taken for sigterm  */
    struct sigaction *sigabrt_oact;
                           /* specifies the action previously associated
                              with sigabrt so it can be restored            */
    struct sigaction *sigterm_oact;
                           /* specifies the action previously associated
                              with sigterm so it can be restored            */

    /* This routine should not be called more than once without first
       resetting the signal handlers.                                       */
    if (install_initialized)
    {
        c_lasmsg (FALSE, "install-sighand",
                  "Warning: c_install_cleanup_handler cannot be called"
                  " more than once without first resetting the signal"
                  " handlers with c_remove_cleanup_handler");
        return (E_SUCC);
    }

    /* Set up the act structure to call our signal handler for SIGTERM      */
    sigterm_act.sa_handler = sig_hand;  /* set the signal handling function */
    sigemptyset (&sigterm_act.sa_mask); /* don't mask any extra signals     */
    sigterm_act.sa_flags = 0;           /* no special flags                 */

    /* Call our signal handler for SIGTERM                                  */
    sigterm_oact = c_get_sigterm_struct ();   
    status = sigaction (SIGTERM, &sigterm_act, sigterm_oact);
    if (status != 0)
    {
        c_lasmsg (FALSE, "install-sighand",
                  "Error setting the signal handler for SIGTERM");
        return (E_FAIL);
    }

    /* Set up the act structure to call our signal handler for SIGABRT      */
    sigabrt_act.sa_handler = sig_hand;   /* set the signal handling function*/
    sigemptyset (&sigabrt_act.sa_mask);  /* don't mask any extra signals    */
    sigabrt_act.sa_flags = 0;            /* no special flags                */

    /* Call our signal handler for SIGABRT                                  */
    sigabrt_oact = c_get_sigabrt_struct ();   
    status = sigaction (SIGABRT, &sigabrt_act, sigabrt_oact);
    if (status != 0)
    {
        c_lasmsg (FALSE, "install-sighand",
                  "Error setting the signal handler for SIGABRT");

        /* Better reset the SIGTERM handler                                 */
        if (c_reset_handler (SIGTERM, sigterm_oact) != E_SUCC)
        {
            c_lasmsg (FALSE, "install_sighand",
                      "Error resetting the SIGTERM signal handler");
        }

        return (E_FAIL);
    }

    install_initialized = TRUE;
    return (E_SUCC);
}


/****************************************************************************

NAME: c_get_install_initialized

PURPOSE: Get the boolean variable to specify if c_install_cleanup_handler
         was called.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
TRUE            c_install_cleanup_handler has been called successfully
FALSE           c_install_cleanup_handler was not called successfully

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    01/01  G.Schmidt        EDC   Original development

PROJECT: LAS

****************************************************************************/

int c_get_install_initialized (void)
{
    return install_initialized;
}


/****************************************************************************

NAME: c_reset_install_initialized

PURPOSE: Reset the boolean variable for c_install_cleanup_handler to specify
         that the signal handlers have been reset.

RETURN VALUE:
Type = none

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    01/01  G.Schmidt        EDC   Original development

PROJECT: LAS

****************************************************************************/

void c_reset_install_initialized (void)
{
    install_initialized = FALSE;
}
