/******************************************************************************
NAME:   MKIMG

FUNCTION:  MKIMG is a routine which takes a root file name, appends an
           input string to the file name, followed by the image extension.

           It is assumed that the calling program retrieved the file name
           specification from the TAE parameter block prior to calling MKIMG.

           The utility optionally checks if the image file already 
           exists on disk.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         11/86      K. Zanter    initial development
    1.1         12/87      B. Ailts     Change include directory specifictions
                                        Use raw 'C' types
                                        Replace DESC arguments with char. str.
                                        Place bridge routines in seperate file
    5.0         04/88      D. Hollaren  changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed status key by address
                                        return E_SUCC/E_FAIL
     5.1        07/90      D. VanderZee Standardized error handling 


PROJECT:        LAS

ALGORITHM:
   Check if the image extension is in the input file name.
   If it is not,
        concatenate the root file name with the input extension, and
        then append the image extension to create the new TAE name.
  Otherwise,
        put a null terminator in place of the image extension in
        the input file name and create the new TAE name as above.
  The full host file name is then retrieved, and an optional check is
  made to see if the host file exists.
  Finally, the full TAE name is retrieved.

ALGORITH REFERENCES:   GETOUT utility
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"

/**
 * las_mkimg:
 * @in_string: I: Character string which contains the root image file name.
 *                It is assumed that the calling program retrieved this
 *                file name specification from the TAE parameter block
 *                prior to calling las_mkimg().
 * @ext: I: Defines the character string to be appended to the root filename.
 *          If nothing is to be appended, pass a blank string for this
 *          argument.
 * @hname: O: Character string which contains the host name of the image.
 * @flag: I: Flag indicating whether las_mkimg() should check if the
 *           image file exists.
 *           0:  Do not check if image file exists,
 *           1:  Check if image file exists.
 *
 * The las_mkimg() routine is used to make and return the TAE and host
 * file names of the image.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int las_mkimg
(
    const char *in_string,  /* I: Character string containing filename      */
    const char *ext,        /* I: Defines string to be appended             */
    char *hname,            /* O: String containing hostname of image       */
    int flag                /* I: Flag indicating existence of file         */
)

{
    int     mode=0;
    int     temp;

    char    in_temp[CMLEN];
    char    newtae[CMLEN];
    char    *ptr;
    char    msg[ERRLEN+1];


    strcpy(in_temp,in_string);

    if (strchr(in_temp, TAE_DIR_BEG) || strchr(in_temp, TAE_EXT))
    {
        if ((ptr = strrchr(in_temp,TAE_EXT)) == 0)
        {
            strcat(in_temp,ext);
            sprintf(newtae,"%s%c%s",in_temp,TAE_EXT,"img");
        }
        else
        {
            if (strchr(in_temp,TAE_EXT) != ptr)
            {
                c_lasmsg( FALSE, "mkimg-name",
                    "Error: Invalid LAS image name: Multiple extensions"
                    " not allowed");
                return(E_FAIL);
            }

            if (strcmp(ptr+1,"img") != 0)
            {
                c_lasmsg (FALSE, "mkimg-name",
                          "Error: Invalid LAS image name: Extension"
                          " must be 'img'");
                return(E_FAIL); 
            }

            *ptr = '\0';
            strcat(in_temp,ext);
            sprintf(newtae,"%s%c%s",in_temp,TAE_EXT,"img");
        }
    }
    else
    {
        temp = (int)strlen(in_temp);
        if (strcmp(&in_temp[temp-4], ".img") == 0)
            in_temp[temp-4] = '\0';
        strcat(in_temp,ext);
        sprintf(newtae, "%s%c%s", in_temp, HOST_EXT, "img");
    }

    if (c_cmhost(newtae,hname) != E_SUCC)
    {
        c_lasmsg (FALSE, "mkimg-call", "Error returned from cmhost");
        sprintf (msg,"Error retrieving host file name for %s",newtae);
        c_lasmsg (FALSE, "mkimg-cmhost", msg);
        return (E_FAIL);
    }

    if (flag == 1)
    {
        if (c_labstat(hname,mode) == E_SUCC)
        {
            sprintf (msg,"Error: %s already exists",newtae);
            c_lasmsg (FALSE, "mkimg-exists", msg);
            return (E_FAIL);
        }
    }


    return(E_SUCC);
}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_mkimg:
 * @in_string: I: Character string which contains the root image file name.
 *                It is assumed that the calling program retrieved this
 *                file name specification from the TAE parameter block
 *                prior to calling las_mkimg().
 * @ext: I: Defines the character string to be appended to the root filename.
 *          If nothing is to be appended, pass a blank string for this
 *          argument.
 * @tname: O: Character string which contains the TAE name of the image.
 * @hname: O: Character string which contains the host name of the image.
 * @flag: I: Flag indicating whether las_mkimg() should check if the
 *           image file exists.
 *           0:  Do not check if image file exists,
 *           1:  Check if image file exists.
 *
 * The c_mkimg() routine is used to make and return the TAE and host
 * file names of the image.
 *
 * Deprecated.  Use las_mkimg() instead.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_mkimg
(
    const char *in_string,  /* I: Character string containing filename      */
    const char *ext,        /* I: Defines string to be appended             */
    char *tname,            /* O: String containing TAE name of image       */
    char *hname,            /* O: String containing hostname of image       */
    const long *flag        /* I: Flag indicating existence of file         */
)
{
    int status;

    status = las_mkimg (in_string,ext,hname,*flag);

    if (status != E_SUCC)
        return (E_FAIL);
    
    strcpy(tname, hname);

    return (E_SUCC);


}

#endif /* obsolete interfaces */

