/******************************************************************************
PROGRAM:        MKNAM

PURPOSE:        Retrieves a file name specification. It appends the specified
                extension to the name if the extension was not specified.  If
                the file is to be opened for output, a bad status is returned
                if the file already exists on disk.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    5.0        04/88       D. Hollaren  Initail development taken from GETNAM
    5.0         07/90      D. VanderZee Standardized error handling 
                                

PROJECT:         LAS

ALGORITHM:
        Solve any aliases within the input name
        If it does not have the specified extension,
            the extension is appended.
        The full host file name is retrieved.
        If the file is to be opened for output and the file already exists,
            a bad status code is returned.
        If the file is to be opened for input and the file does not exist,
            a bad status code is returned.

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "alias.h"
#include "labserv.h"


/**
 * las_mknam:
 * @filename: I: Character string which contains the file name that is
 *               to have the extension added to it.
 * @outfile: O: Character string which contains the host name of the file
 *              specified "filename".
 * @ext: I: Character string denoting the default extension for the file.
 *          If the user does not specify an extension (;), the file name
 *          is not changed.
 * @access: I: Denotes the type of access for which the file is to be 
 *             opened.  If access is for input, a check is made to ensure the
 *             file exists on disk.  If access is for output, a check is
 *             made to ensure the file does not exist.  If access is for 
 *             update, no check is made to see if the file exists or not.
 *             IREAD:   Read access,
 *             IWRITE:  Write access,
 *             IUPDATE: Update access.
 *
 * The las_mknam() routine is used to return the host file name of "filename".
 * It retrieves a file name specification.  Then it appends the specified
 * extension to the name if the extension was not specified.  If the file
 * is to be opened for output, a bad status is returned if the file already
 * exists on disk.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int las_mknam
(
    const char *filename,        /* I: Input file name                      */
    char *outfile,               /* O: Host file name to be returned        */
    const char *ext,             /* I: Default extension                    */
    int access                   /* I: Type of I/O access                   */
)

{
    int  mode = 0;
    int  stat;
    char msg[ERRLEN+1];
    char tfull[CMLEN];
    char tname[CMLEN];
    char *ptr;                   /* temporary character pointer             */


    stat = c_getalias(filename,tname);
    if (stat != E_SUCC)
    {
        c_lasmsg (FALSE, "mknam-call", "Error returned from getalias");
        return (E_FAIL);
    }

    /* Determine whether name is in VMS or Unix syntax.  Check for appropriate
       extension, and add if missing. */
    if (strchr(tname, TAE_DIR_BEG) || strchr(tname, TAE_EXT))  /* VMS */
    {
        if (strrchr(tname, TAE_EXT) == NULL)   /* if no extension add default*/
            sprintf(tfull, "%s%c%s", tname, TAE_EXT, ext);
        else
            strcpy(tfull, tname);
    }
    else /* Unix */
    {
        /* Since dots can reside in the path as well as the filename,
           we need to ensure that we are checking for dots (as extensions)
           after any path that may be included.  */
        if ((ptr = strrchr(tname, HOST_DIR_SEP)) == NULL)
            ptr = tname;
        if (strrchr(ptr, HOST_EXT) == NULL)  /* if no extension add default */
            sprintf(tfull, "%s%c%s", tname, HOST_EXT, ext);
        else
            strcpy(tfull, tname);
    }

    if (c_cmhost(tfull,outfile) != E_SUCC)
    {
        c_lasmsg (FALSE, "mknam-call", "Error returned from cmhost");
        sprintf(msg,"Error retrieving host file name for %s",tfull);
        c_lasmsg (FALSE, "mknam-cmhost", msg);
        return (E_FAIL);
    }

    if (access == IREAD)               /* if file is opened for read access */
    {                                  /*   the file must exist             */
        if (c_labstat(outfile,mode) != E_SUCC)
        {
            sprintf (msg,"Error: %s does not exist",tfull);
            c_lasmsg (FALSE, "mknam-exist", msg);
            return (E_FAIL);
        }
    }

    if (access == IWRITE)             /* if file is opened for write access */
    {                                 /*   the file cannot exist            */
        if (c_labstat(outfile,mode) == E_SUCC)
        {
            sprintf (msg,"Error: %s already exists",tfull);
            c_lasmsg (FALSE, "mknam-exists", msg);
            return (E_FAIL);
        }
    }

    return(E_SUCC);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_mknam:
 * @filename: I: Character string which contains the file name that is
 *               to have the extension added to it.
 * @outfile: O: Character string which contains the host name of the file
 *              specified "filename".
 * @ext: I: Character string denoting the default extension for the file.
 *          If the user does not specify an extension (;), the file name
 *          is not changed.
 * @access: I: Denotes the type of access for which the file is to be 
 *             opened.  If access is for input, a check is made to ensure the
 *             file exists on disk.  If access is for output, a check is
 *             made to ensure the file does not exist.  If access is for 
 *             update, no check is made to see if the file exists or not.
 *             IREAD:   Read access,
 *             IWRITE:  Write access,
 *             IUPDATE: Update access.
 *
 * The c_mknam() routine is used to return the host file name of "filename".
 * It retrieves a file name specification.  Then it appends the specified
 * extension to the name if the extension was not specified.  If the file
 * is to be opened for output, a bad status is returned if the file already
 * exists on disk.
 *
 * Deprecated.  Use las_mknam() instead.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_mknam
(
    const char *filename,        /* I: Input file name                      */
    char *outfile,               /* O: Host file name to be returned        */
    const char *ext,             /* I: Default extension                    */
    const long *access           /* I: Type of I/O access                   */
)
{
    int status;

    status = las_mknam (filename,outfile,ext,*access);

    return (status);

}

#endif /* obsolete interfaces */

