/****************************************************************************

NAME: c_remove_cleanup_handler

PURPOSE: Reset the signal handlers set by c_install_cleanup_handler to their
  original state.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    01/01  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Requires a system that provides POSIX signal handling.

PROJECT: LAS

NOTES:
  1) At this time this library handles the SIGTERM and SIGABRT signals.
  2) Should be used to cleanup the signal handlers set by
     c_install_cleanup_handler.  If c_install_cleanup_handler has not been
     called (or called successfully), then this routine will not try to
     clean up the signal handlers.
  3) Once the signal handlers are reset, then information is sent to
     c_install_cleanup_handler so that it knows the signal handlers have
     been reset.
  4) If the first signal cannot be reset, try to reset the next signal
     before returning.

****************************************************************************/

#include <stdio.h>
#include <signal.h>
#include "worgen.h"
#include "util.h"
#include "local_protos.h"

static struct sigaction sigabrt_oact; /* specifies the action previously
                                      associated with sigabrt so it can be
                                      restored                              */
static struct sigaction sigterm_oact; /* specifies the action previously
                                      associated with sigterm so it can be
                                      restored                              */

/**
 * c_remove_cleanup_handler:
 *
 * The c_remove_cleanup_handler() routine is used to reset the way signals
 * are handled to their state prio to calling c_install_cleanup_handler().
 * NOTES:
 * 1) At this time this library handles the SIGTERM and SIGABRT signals.
 * 2) Should be used to cleanup the signal handlers set by
 *    c_install_cleanup_handler.  If c_install_cleanup_handler has not been
 *    called (or called successfully), then this routine will not try to
 *    clean up the signal handlers.
 * 3) Once the signal handlers are reset, then information is sent to
 *    c_install_cleanup_handler so that it knows the signal handlers have
 *    been reset.
 * 4) If the first signal cannot be reset, try to reset the next signal
 *    before returning.
 * 
 * Returns:   E_SUCC:   Successful completion of operation,
 *                      Signal handler was successfully reset.
 *            E_FAIL:   Unsuccessful completion of operation,
 *                      An error occurred resetting the signal handler.
 */
int c_remove_cleanup_handler (void)
{
    int status = E_SUCC;    /* return status                                */

    /* Don't remove the cleanup handlers if c_install_cleanup_handler has
       not been successfully called.                                        */
    if (!c_get_install_initialized ())
    {
        c_lasmsg (FALSE, "remove_sighand",
                  "Warning: c_install_cleanup_handler was not called"
                  " successfully, therefore the signal handlers cannot"
                  " be removed");
        return (status);
    }

    /* Reset SIGTERM signal                                                 */
    if (c_reset_handler (SIGTERM, &sigterm_oact) != E_SUCC)
    {
        c_lasmsg (FALSE, "remove_sighand",
                  "Error resetting the SIGTERM signal handler");
        status = E_FAIL;
    }

    /* Reset SIGABRT signal                                                 */
    if (c_reset_handler (SIGABRT, &sigabrt_oact) != E_SUCC)
    {
        c_lasmsg (FALSE, "remove_sighand",
                 "Error resetting the SIGABRT signal handler");
        status = E_FAIL;
    }

    if (status == E_SUCC)
        c_reset_install_initialized ();

    return (status);
}


/****************************************************************************

NAME: c_get_sigabrt_struct

PURPOSE: Get a pointer to the sigabrt structure.

RETURN VALUE:
Type = struct sigaction *
Value           Description
-----           -----------
> 0             Successful completion
NULL            An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    01/01  G.Schmidt        EDC   Original development

PROJECT: LAS

****************************************************************************/

struct sigaction *c_get_sigabrt_struct (void)
{
    return &sigabrt_oact;
}


/****************************************************************************

NAME: c_get_sigterm_struct

PURPOSE: Get a pointer to the sigterm structure.

RETURN VALUE:
Type = struct sigaction *
Value           Description
-----           -----------
> 0             Successful completion
NULL            An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    01/01  G.Schmidt        EDC   Original development

PROJECT: LAS

****************************************************************************/

struct sigaction *c_get_sigterm_struct (void)
{
    return &sigterm_oact;
}
