/******************************************************************************
*    Name:      setprt
*
*    Purpose:   Set up boolean flags for the destination of generated output
*               to the terminal, file, or line printer.
*
*    Program history:
*       K. Gacke        8/86    initial development
*       B. Ailts        12/87   Change include directory specifications
*                               Place bridge routines in a seperate file
*                               Use raw 'C' types
*       B. Ailts        10/88   Called squeeze for prtfile to be accessable
*                               from FORTRAN
*       B. Ailts        11/88   Fixed pointer problem to prtfile
*       B. Ailts        12/88   Renamed the original SETPRT to HSETPR -- this
*                               version of SETPRT will now return LAS name
*                               It calls HSETPR
        D. VanderZee    07/90   Standardized error handling 

*    Algorithm: 
*       Call HSETPR to set term, file, prtfile, and lp
*       Call MKNAM to check validity of input name
*       Call CMTAE to calculate the TAE name
*
*    Algorithm References               none
*******************************************************************************/

#include <string.h>
#include "las.h"                 /* las include file                     */
#include "util.h"


/**
 * c_setprt:
 * @vblock: I: TAE parameter  block passed by reference.  It is assumed
 *             that TAE has been initialized prior to calling c_setprt().
 * @term: O: Integer flag set if the user has specified TERM as one of the
 *           "print" values.
 *            0:  Terminal was not specified in "print",
 *            1:  Terminal was specified in "print".
 * @file: O: Integer flag set if the user has specified a file name as one of
 *           the "print" values.
 *           0:  A file name was not specified in "print",
 *           1:  A file name was specified in "print".
 * @lp: O: Integer flag set if the user has specified LP as one of the 
 *         "print" values.
 *          0:  Line printer was not specified in "print",
 *          1:  Line printer was specified in "print".
 * @prtfile: I/O: A file name should be input (normally the program name).
 *                The input file name will be returned with "xxxxxx.pr"
 *                (xxxxxx is a time stamp) appended to the end of it.  This
 *                file name will also be placed in a local variable of the
 *                TAE parameter block called "prtfile".
 * @filename: O: The file name specified by the user in the "print"
 *               parameter.
 *
 * The c_setprt() routine is used to set integer values from the "print"
 * parameter.  It is assumed the parameter vblock contains the TAE parameter
 * print.  It is also assumed the parameter vblock contains a local variable
 * called "prtfile".  The file name returned to the local parameter will
 * then need to be sent to the "infile" parameter of the routine fileutil().
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_setprt
(
    struct PARBLK *vblock,  /* I: TAE parameter block                       */
    int  *term,             /* O: Boolean for terminal output               */
    int  *file,             /* O: Boolean for file output                   */
    int  *lp,               /* O: Boolean for line printer output           */
    char *prtfile,          /* I/O: File name or lp working file            */
    char *filename          /* O: File name or lp output file               */
)

{
    int   mode;            /* access mode                                   */
    int   status;          /* error status                                  */

    char *t_hname;         /* temporary pointer to prtfile argument         */
    char *hname;           /* temporary pointer to prtfile argument         */

 
    t_hname = (char *)malloc(CMLEN);
    if (t_hname == NULL)
    {
        c_lasmsg (FALSE, "setprt-alloc", "Error allocating dynamic memory");
        return(E_FAIL);
    }

    status = c_hsetpr(vblock,term,file,lp,prtfile,t_hname);
    if (status != E_SUCC)
    {
        c_lasmsg (FALSE, "setprt-call", "Error returned from hsetpr");
        free (t_hname);
        return(E_FAIL);
    }

    if (*file)
    {
        hname = (char *)malloc(CMLEN);
        if (hname == NULL)
        {
            c_lasmsg (FALSE, "setprt-alloc", "Error allocating dynamic memory");
            free (t_hname);
            return(E_FAIL);
        }

        mode = IWRITE;
        status = las_mknam(t_hname,hname,"prt",mode);
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "setprt-call" , "Error returned from mknam");
            free (t_hname);
            free (hname);
            return(E_FAIL);
        }

        strcpy(filename, hname);

        free(hname);
    }

    free(t_hname);

    return(E_SUCC);
}  /*  setprt  */
