/******************************************************************************
NAME:    c_window_proj_to_ls

PURPOSE: Convert window specifications to line sample using an input DDR
         structure.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
  Date      Programmer                        Changes
--------  --------------  -----------------------------------------------------
12/00     G. Schmidt      Original development (based from the original
                          c_wndconv.c routine)

NOTE: c_wndconv will convert window specifications to line/sample using an
  existing DDR file.  It actually reads the DDR file into a DDR structure
  and calls this routine.
******************************************************************************/

#include <math.h>
#include <string.h>  /* for call to strcmp() */ 
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "cproj.h"

/**
 * c_window_proj_to_ls:
 * @ddr: I: Pointer to the input DDR structure.
 * @window: I/O: Image window to convert to line/sample.
 *               On input: Upper-left y, Upper-left x,  Lower-right y, Lower
 *                         right x.
 *               On output: Starting line, Starting sample, Number of lines,
 *                          Number of samples.
 * @units: Window units (DEG, DMG, PRO, LS).
 *
 * The c_window_proj_to_ls() routine is used to convert input window 
 * coordinates in units other than line/sample to line/sample units.
 * The routine c_wndconv() is a similar routine which also converts an
 * input window to line/sample coordinates.  However, c_wndconv() requires a
 * DDR file to be present, whereas c_window_proj_to_ls() only requires a
 * DDR structure to be passed to this routine.  c_wndconv() actually calls 
 * c_window_proj_to_ls() once it has read in the DDR from the file.
 *
 * Returns: E_SUCC:  Successful completion of operation.
 *          E_FAIL:  Unsuccessful completion of operation.
 */
int c_window_proj_to_ls
(
    const struct DDR *ddr,  /* I/O: pointer to DDR structure                */
    double window[],        /* I/O: Image window
                                    on input: uly, ulx, lry, lrx
                                    on output: sl, ss, nl, ns               */
    const int units[]       /* I: window units                              */ 
)
{
    double pxmax;                         /* maximum x value              */
    double pymax;                         /* maximum y value              */
    double pxmin;                         /* minimum x value              */
    double pymin;                         /* minimum y value              */
    double coef[6];                       /* rotation coefficient matrix  */
    double determinant;                   /* matrix determinant           */
    double uly = window[0]; 
    double ulx = window[1];               /* upper left window coords     */
    double lry = window[2]; 
    double lrx = window[3];               /* lower right  window coords   */ 

    long corners[8];                      /* image corners                */
    long ounits;                          /* output units for GCTP        */
    long flag = FALSE;                    /* flag for c_rtcoef            */

    /* Verify the DDR corner coordinates are valid */
    if (ddr->valid[6] != VALID)
    {
        c_lasmsg (FALSE, "wndconv-valid", "DDR has invalid fields");
        return (E_FAIL);
    }

    if ((units[0] == DEGREE) || (units[0] == DMS))
    {
        /* Verify the DDR projection code, zone code, datum code, projection
           coefficients, ground units, and ground distance are valid        */
        if ((ddr->valid[0] != VALID) || (ddr->valid[1] != VALID) ||
                (ddr->valid[2] != VALID) || (ddr->valid[3] != VALID) ||
                (ddr->valid[4] != VALID) || (ddr->valid[5] != VALID))
        {
            c_lasmsg (FALSE, "wndconv-valid", "DDR has invalid fields");
            return (E_FAIL);
        }

        ounits = c_string2unit(ddr->proj_units);
        if (ounits == E_FAIL)
        {
            c_lasmsg (FALSE, "wndconv-projunits",
                      "DDR has invalid proj_units: %s", ddr->proj_units);
            return (E_FAIL);
        } 

        /* Compute the minimium output window projection coordinates that
           contain the entire input window.                                 */
        if (c_calc_bound_ls_coords(&ulx, &lrx, &uly, &lry, units[0],
                                   ddr->proj_code, ddr->zone_code,
                                   ddr->proj_coef, ounits, ddr->datum_code,
                                   ddr->pdist_y, ddr->pdist_x, &pxmin, 
                                   &pxmax, &pymin, &pymax) != E_SUCC)
            return (E_FAIL);
    }

    /* Assign corner values to pass to c_rtcoef as specified in the
       documentation                                                        */
    corners[0] = 1;
    corners[1] = 1;
    corners[2] = ddr->nl;
    corners[3] = 1;
    corners[4] = 1;
    corners[5] = ddr->ns;
    corners[6] = ddr->nl;
    corners[7] = ddr->ns;

    las_rtcoef (ddr->upleft, corners, coef, flag);

    /* Compute the determinant of the coefficient matrix.  If the determinant
       is zero, the matrix is singular.  This should only happen if we are
       dealing with an image with only one line or one sample.
       (In this context, a singular matrix means the the window is mapped to
       a straight line.)                                                    */
    determinant = coef[0]*coef[4] - coef[3]*coef[1];
    if (fabs (determinant) > 1e-16)
    {
        /* Compute the window line/sample values based on the coefficients.
           1.0 is added to account for the 1-based line/sample indexing.
           0.5 is added to produce proper rounding during integer truncation*/
        window[SL] = (int) (((uly - coef[2])*coef[4] -
                (ulx - coef[5])*coef[1]) / determinant + 0.5);
        window[SS] = (int) (((ulx - coef[5])*coef[0] -
                (uly - coef[2])*coef[3]) / determinant + 0.5);
        window[NL] = (int) (((lry - coef[2])*coef[4] -
                (lrx - coef[5])*coef[1]) / determinant + 0.5) - window[SL] + 1;
        window[NS] = (int) (((lrx - coef[5])*coef[0] -
                (lry - coef[2])*coef[3]) / determinant + 0.5) - window[SS] + 1;
    }
    else
    {
        window[SL] = (int) (((ddr->upleft[0] - uly) / ddr->pdist_y) + 1.5); 
        window[SS] = (int) (((ulx - ddr->upleft[1]) / ddr->pdist_x) + 1.5); 
        window[NL] = (int) (((ddr->upleft[0] - lry) / ddr->pdist_y) + 1.5 -
                window[SL] + 1);
        window[NS] = (int) (((lrx - ddr->upleft[1]) / ddr->pdist_x) + 1.5 -
                window[SS] + 1);
    }

    /* Error check:  entire window outside of image.
       The number of pixels in the window is SL+NL-1 or SS+NS-1.  These
       quantities must be at least one or there is no data in the window.   */
    if ((int) (window[SL] + window[NL] - 1) < 1)
    {
        c_lasmsg (FALSE, "wndconv-invld",
                 "Window line values outside of image: %d %d %d %d",
                  window[SL], window[SS], window[NL], window[NS]);
        return (E_FAIL);
    }
    if ((int) (window[SS] + window[NS] - 1) < 1)
    {
        c_lasmsg (FALSE, "wndconv-invld",
                 "Window sample values outside of image: %d %d %d %d",
                  window[SL], window[SS], window[NL], window[NS]);
        return (E_FAIL);
    }

    /* Error check:  SL/SS outside of image.  
       SL and SS are compared to 1 because image windows are 1-relative.    */
    if ((int) (window[SL]) < 1)
    {
        c_lasmsg (FALSE, "wndconv-invld", 
                  "Starting line %d outside of bounds, reset to 1", window[SL]);
        window[NL] -= 1.0 - window[SL];
        window[SL]  = 1.0;
    }
    if ((int) (window[SS]) < 1)
    {
        c_lasmsg (FALSE, "wndconv-invld", 
                 "Starting sample %d outside of bounds, reset to 1", 
                  window[SS]);
        window[NS] -= 1.0 - window[SS];
        window[SS]  = 1.0;
    }

    return (E_SUCC);
}
