/*******************************************************************************
NAME: C_CHECK_WINDOW

PURPOSE: Validate an image window using a DDR structure.

PROGRAM HISTORY
PROGRAMMER        DATE          REASON
----------        ----          ------
P. Mumford        05/85         Original development for NEWLAS.
K. Zanter         12/87         1.  switched to raw C types 
                                2.  changed include file specification
                                3.  changed DESC argument to be a character
                                    string
                                4.  renamed source from wndchk to c_wndchk
B. Ailts          12/87         Implemented new DDR structure and call
D. Hollaren       04/88         Changed include spec las
                                Changed cerrmsg to c_errmsg and passed
                                status code by address
                                SUCCESS/FAIL to E_SUCC/E_FAIL
D. VanderZee      07/90         Standardized error handling 
T. Mittan         07/93         Changed inwind to double for new windowing
                                options

                                        
PROJECT: LAS

ALGORITHM:
        open the image in random mode and get the number of lines and samples
        if the window is 0 0 0 0 then return 1 1 #lines and #samples
        otherwise check that the starting line <= #lines
                                 starting sample <= #samples
                                 starting line + number lines -1 <= #lines
                                 starting sample + number samples -1 <= #samps

ALGORITHM REFERENCES: None
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"

/**
 * c_check_window:
 * @ddr:  I:  Pointer to DDR structure.
 * @inwind: I: Defines the input window to be checked.
 *             The four elements of the window are SL, SS, NL, and NS.
 * @vlwind: O: Defines the output window with window values that are
 *             valid.
 *
 * The c_check_window() routine is used to check an input window using a
 * DDR structure instead of an existing DDR file.  
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */

int c_check_window
(
    const struct DDR *ddr, /* I: pointer to DDR structure                   */
    const double inwind[], /* I: Input window to be checked                 */
    int  vlwind[]          /* O: Valid window                               */
)
{
    int  tmp_wind[4];      /* Holds the rounded input window                */
    int  i;                /* Loop control variable                         */


    /* Round the input window coordinates.  Anything above
      .50 will round up.  This because we are using the
      center of the pixel                                                   */
    for (i = 0; i < 4; i++)
        tmp_wind[i] = (int)(inwind[i]+.5);

    /* If no input window specified, set the
       starting line/sample to 1, and the
       number of lines/samples to input image                               */
    if ((inwind[0]==0) && (inwind[1]==0) && (inwind[2]==0) && (inwind[3]==0))
    {
        vlwind[0] = 1;
        vlwind[1] = 1;
        vlwind[2] = ddr->nl;
        vlwind[3] = ddr->ns;
    }
    else
    {
        /* Check the input starting line                                    */
        if (tmp_wind[0] > 0)
        {
            if (tmp_wind[0] <= ddr->nl)
                vlwind[0] = tmp_wind[0];
            else
            {
                c_lasmsg(FALSE,"wndchk-invwnd",
                 "Starting line %d greater than number of lines %d",
                  tmp_wind[0],ddr->nl);
                return(E_FAIL);
            }
        }
        else
        {
            c_lasmsg(FALSE,"wndchk-invwnd",
                    "Starting line %d must be positive",tmp_wind[0]);
            return(E_FAIL);
        }

        /* Check the input starting sample                                  */
        if (tmp_wind[1] > 0)
        {
            if (tmp_wind[1] <= ddr->ns)
                vlwind[1] = tmp_wind[1];
            else
            {
                c_lasmsg(FALSE,"wndchk-invwnd",
                     "Starting sample %d greater than number of samples %d",
                      tmp_wind[1],ddr->ns);
                return(E_FAIL);
            }
        }
        else
        {
            c_lasmsg(FALSE,"wndchk-invbnd",
                    "Starting sample %d must be positive",inwind[1]);
            return(E_FAIL);
        }

        /* Check the input number of lines                                  */
        if (tmp_wind[2] > 0)
        {
            if ((tmp_wind[0] + tmp_wind[2] -1) <= ddr->nl)
                vlwind[2] = tmp_wind[2];
            else
            {
                vlwind[2] = ddr->nl - tmp_wind[0] + 1;
                c_lasmsg(FALSE,"wndchk-invwnd","Starting line %d",tmp_wind[0]);
                c_lasmsg(FALSE,"wndchk-invwnd","and number of lines %d",
                         tmp_wind[2]);
                c_lasmsg(FALSE,"wndchk-invwnd",
                        "places the window outside of image");
                c_lasmsg(FALSE,"wndchk-invwnd","Number of lines reset to",
                         vlwind[2]);
            }
        }
        else
        {
            c_lasmsg(FALSE,"wndchk-invwnd",
                     "Number of lines %d must be positive",tmp_wind[2]);
            return(E_FAIL);
        }

        /* Check the input number of samples                                */
        if (tmp_wind[3] > 0)
        {
            if ((tmp_wind[1] + tmp_wind[3] -1) <= ddr->ns)
                vlwind[3] = tmp_wind[3];
            else
            {
                vlwind[3] = ddr->ns - tmp_wind[1] + 1;
                c_lasmsg(FALSE,"wndchk-invwnd","Starting sample %d",
                         tmp_wind[1]);
                c_lasmsg(FALSE,"wndchk-invwnd","and number of samples %d",
                         tmp_wind[3]);
                c_lasmsg(FALSE,"wndchk-invwnd",
                         "places the window outside of image");
                c_lasmsg(FALSE,"wndchk-invwnd","Number of samples reset to",
                         vlwind[3]);
            }
        }
        else
        {
            c_lasmsg(FALSE,"wndchk-invwnd",
                     "Number of samples %d must be positive",tmp_wind[3]);
            return(E_FAIL);
        }
    }
    return(E_SUCC);
}

/******************************************************************************
NAME:     c_wndchk

PURPOSE:  Validate an image window using an existing DDR file.

******************************************************************************/

/**
 * c_wndchk:
 * @host: I: Input DDR file name in host format.  The number of lines and
 *           samples are used from the file.
 * @inwind: I: Defines the input window to be checked.
 *             The four elements of the window are SL, SS, NL, and NS.
 * @vlwind: O: Defines the output window with window values that are
 *             valid.
 *
 * The c_wndchk() routine is used to check an input window. It retrieves
 * the DDR information from the DDR file then calls c_check_window()
 * to do the actual checking of the window.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_wndchk
(
    const char *host,     /* I: Host name                                   */
    const double inwind[],/* I: Input window to be checked                  */
    int  vlwind[]         /* O: Valid window                                */
)
{
    struct DDR ddr;       /* DDR structure                                  */

    /* Get the ddr information                                              */
    if (c_getddr(host,&ddr) != E_SUCC)
    {
        c_lasmsg(FALSE,"wndchk-call","Error returned from getddr");
        return(E_FAIL);
    }

    /* Check the window                                                     */
    if (c_check_window (&ddr, inwind, vlwind) != E_SUCC)
    {
        c_lasmsg (FALSE, "wndchk-window", "Error checking the window");
        return (E_FAIL);
    }

    return (E_SUCC);

}

