/******************************************************************************
NAME:    c_wndconv

PURPOSE: Convert window specifications to line sample using an existing DDR
         file.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
  Date      Programmer                        Changes
--------  --------------  -----------------------------------------------------
07/93     T. Mittan	  Original development
02/94     T. Mittan	  Modified to adjust negative line and sample numbers 
                          to 1.	
12/94     T. Mittan	  Modified to Fatal error if entire window is outside 
                          of the image.
04/95     S. Nelson	  Changed the order in which the window specifications 
                          are sent to the function c_rtcoef.  This was done 
                          because errors were found when running COPY with 
                          certain images.
May 96    David Zokaites  Adjusted checks for window being outside the image,
                          changed list of include files, called c_lasmsg()
			  instead of c_errmsg()
02/00     L. Johnson      Corrected line/sample window computations.
                          Replaced fixed number values with defined values
                          for projection types.
12/00     G. Schmidt      Changed c_wndconv to call c_window_proj_to_ls
                          which does all the work c_wndconv used to do.

NOTE: c_wndconv calls c_window_proj_to_ls after reading the existing DDR file.
  If the user is looking for a routine to convert the window from an
  existing DDR structure (rather than DDR file), then c_window_proj_to_ls is
  the routine to use.
******************************************************************************/

#include "las.h"
#include "util.h"

/**
 * c_wndconv:
 * @hname: I: Host name of the image specified.
 * @window: I: Imagae window to convert to line/sample.
 *             On input: Upper-left y, Upper-left x, Lower-right y,
 *                       Lower-right x.
 *             On output: Starting line, Starting sample, Number of lines,
 *                        Number of samples.
 * @units: I: Window units (DEG, DMS, PRO, LS).
 *
 * The c_wndconv() routine is similar to the c_window_proj_to_ls() routine.
 * It also converts an input window to line/sample coordinates.  However,
 * c_wndconv() requires a DDR file to be present, whereas c_window_proj_to_ls()
 * only requires a DDR structure to be passed.  c_wndconv actually calls
 * c_window_proj_to_ls() once it has read in the DDR from the file.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_wndconv
(
    const char hname[], /* I: host name of image specified                  */
    double window[],    /* I/O: image window
                                on input: uly, ulx, lry, lrx
                                on output: sl, ss, nl, ns                   */
    const int units[]  /* I: window units                                   */ 
)

{
    struct DDR ddr;     /* DDR structure                                    */

    /* Get the DDR                                                          */
    if (c_getddr (hname, &ddr) != E_SUCC)
    {
        c_lasmsg (FALSE, "wndconv-ddr", "Error retrieving DDR for %s", hname); 
        return (E_FAIL);
    }

    /* Do the window conversion                                             */
    if (c_window_proj_to_ls (&ddr, window, units) != E_SUCC)
    {
        c_lasmsg (FALSE, "wndconv-window", "Error converting the window");
        return (E_FAIL);
    }

    return (E_SUCC);
}
