/*****************************************************************************

NAME    inittm , startm, stoptm, cinittm

PURPOSE 
    inittm
        Initializes the application program for display messages.  The input
        parameters specify the format of the display message, and how often
        the message is to be outputted to the terminal.

    startm
        Starts the display message utility.

    stoptm
        Stop display message utility.  The utility may be restarted with 
        startm.

    cinittm
        C callable which builds string descriptors for its input arguments
        and calls "inittm" to initialize the display message utility.

    NOTE:
        The catalog manager routines do NOT like to be interrupted (it brings
        the catalog manager down).  Therefore no catalog manager routines 
        should be called between the inittm and posttm calls.

DATA STRUCTURES 
  struct static_val --> a structure containing all static fields of the display
                        message.  This includes the subject, verb, entity,
                        category, text, current entity, maximum entity, current
                        category, maximum category, number of seconds between 
                        display messages, and a flag indicating the format of 
                        the message.

PROGRAM HISTORY
        PROGRAMMER        DATE          REASON
        ----------        ----          ------
        K. Gacke          11/85         Initial development of unix-based
                                        raster processing system (newlas).
        K. Gacke           2/86         Modified so that inittm does not 
                                        output the first display message.
        K. Gacke           5/86         changed t_write call to xtwrit to 
                                        solve batch log problem with fortran
                                        programs.
        K. Gacke           6/86         made the display message utility more
                                        flexible by adding additional parameters
                                        to INITTM.  These parameters are:
                                        CAT:  Type of category (e.g. 'bands')
                                        TEXT: Addition text appended to the msg.
                                        CUR_ENTITY:  Current entity 
                                        CUR_CAT:  Current category 
        K. Gacke           6/86         Removed CUR_ENTITY and CUR_CAT from 
                                        parameter list and made them globals.
                                        This was done to solve the problem on
                                        VMS of putting the variables in a
                                        register.
        K. Gacke           8/86         Made internal functions to be "static"
        K. Gacke          11/86         Added function "cinittm" which is a
                                        C callable to build the string 
                                        descriptors and calls "inittm".  Also
                                        initialize global variables curent and
                                        curcat to 1.
        K. Gacke                        Created a new function called setmsg
                                        and placed curent and curcat as
                                        arguments because ALTOS would not
                                        support them as globals
        B. Ailts          12/87         Changed include directory specifications
                                        Replaced DESC arguments with char. str.
                                        Place bridge routines in a seperate file
                                        Use raw 'C' types
        B. Ailts          06/88         Placed curent and curcat as globals 
                                        again.  ALTOS is not going to be 
                                        supported
        B. Ailts          10/88         Fixed bug with linking to Fortran
        B. Ailts          01/89         Eliminated a processing message if
                                        seconds is equal to zero -- this will
                                        assure that the batch log are not 
                                        corrupted by the processing message --
                                        msgtim is set to zero in batch
        K. Gacke          01/90         Fixed bug in which parameters to the
                                        t_init call were wrong.  (PRF 5343)
        D. VanderZee    07/90           Standardized error handling
        D. Etrheim      09/92           Change reset_time from long to void


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        none.

PROJECT:        LAS

ALGORITHM:
    INITTM
        INITTM is called by all application programs before image processing
        begins.  The input parameters specify the format of the display
        message which are assigned to a global structure.  Whenever it is time
        to output a display message, the structure is referenced.  The input 
        parameter 'flag' specifies whether all bands are processed at once, 
        or if one band is processed at a time.  

    STARTM
        STARTM is called by application programs to enable the processing
        message utility.  STARTM sets the alarm for the number the specified
        number of seconds.  When the alarm interrupt is encountered, the 
        function SETTIM is called to output the processing message.

    STOPTM
        STOPTM is called by application programs to disable the processing
        message utility.  It turns off the alarm interrupt and outputs the
        final processing message.

    NOTE:  It was required to develop a separate version for the VAX VMS
        computer and the unix computer.  In the unix environment, fortran
        and C modules can call each other without causing any side effects.
        In the VMS environment, a fortran module cannot always call a C
        function.  The reason for this is that a C program initiates a
        process name C$MAIN which sets up the program environment such as
        interrupt signals.  A fortran program does not do this, and therefore
        interrupt signals are not caught.  For this reason, the VMS version
        is required to call system service routines directly to set the system
        timer and to call the display message routine each time the timer goes
        off.

ALGORITHM REFERENCES
        Basic idea was suggested by Ken Johnson.

******************************************************************************/
#ifdef sgi
#ifdef _CFE
#ifndef _BSD_COMPAT
#define _BSD_COMPAT
#endif
#endif
#endif
#include <stdio.h>              /* C standard I/O library                    */
#include <signal.h>             /* Interrupt include file                    */
#include "las.h"                /* LAS include file                          */
#include "desc.h"
#include "util.h"
#include "util2.h"


struct static_val               /* Structure defining the static protions of */
{                               /*   the display message                     */
    const char *subject;
    const char *verb;
    const char *entity;
    const char *cat;
    const char *text;
    int   max_entity;
    int   max_cat;

    int   msgtim;
    int   flag;
};

    int  curcat;
    int  curent;
    static int  _TOTLIN;                 /* Number of lines on terminal      */
    static int  _TOTCOL;                 /* Number of columns on terminal    */
    static struct static_val msg_values; /* Global display/line struct */
    static void output_line(void);
    static void reset_time(int signalnum);


/**
 * las_inittm:
 * @subject: I: Contains the program name of the calling module.  To conform
 *              with standards, it should be all upper-case characters.
 * @verb: I:  The type of action being performed.  To conform with standards
 *            the first character of the string is upper case, the rest is
 *            lower case.  The verb is used in the past tense.  Some examples
 *            are: Processed, Histogrammed, Calculated, and Printed.
 * @entity: I: The entity being processed.  To conform with standards, it 
 *             should be all lower-case characters.  Some examples are: line,
 *             section, and page.
 * @cat: I: The category being processed.  To conform with standards, it
 *          should be all lower-case characters.  Some examples are: band, and
 *          page.
 * @text: I: An optional text string to be appended to the processing message.
 *           This text is normally blank.  An example is 
 *           "for relation AARCXYS".
 * @max_entity: I: The maximum number of entities described in parameter 
 *                 "entity".
 * @max_cat: I: The maximum number of entities described in parameter
 *                   "cat."
 * @seconds: I: The number of seconds to elapse between processing messages.
 *              This should be the value of the TAE global $MSGTIM.
 * @flag: I: Flag determining the format of the output message.  Valid values 
 *           are:
 *           ALLCAT (0): BIL message format (all categories processed
 *                                             at once),
 *           ONECAT (1): BSQ message format (categories processed separately),
 *           GEN1   (2): General message format one,
 *           GEN2   (3): General message format two.
 *
 * The las_inittm() routine is used to initialize the processing message
 * utility.  The utility routines c_startm() and c_stoptm() are required to
 * enable and disable the processing message utility.
 *
 * There are two external variables that the application program is required
 * to reference. They are:
 *        curent:  current entity being processed,
 *        curcat:  current category being processed.
 *
 * The external variables are referenced from "C" by the following statements:
 * extern int curent, and extern int curcat.
 *
 * The application program may modify the processing message format by 
 * calling las_inittm() which outputs the new line character so that the
 * processing message will scroll.
 *
 * The processing message is displayed on the last terminal line.  If text
 * already exists on this line, it is overwritten.  The processing message
 * is appended with blanks so that the full terminal line is overwritten.  If
 * the processing message is greater than the number of columns on the 
 * terminal, the results are undefined.
 * Example:
 * las_inittm('COPY','Processed', 'line', 'band', ' ', &hundred, &three,
 *            &msgtim, &flag)
 * FLAG:
 *       0:  "COPY Processed line + of 100 for all 3 bands",
 *       1:  "COPY Processed line _ of 100 for band_ of 3",
 *       2:  "COPY Processed line_",
 *       3:  "COPY Processed line_ of 100".
 */
void las_inittm
(
    const char  *subject,    /* I: Subject of message (e.g. process name)     */
    const char  *verb,       /* I: Verb of message (e.g. 'Processing')        */
    const char  *entity,     /* I: Entity of display message (e.g. 'line')    */
    const char  *cat,        /* I: Category of message (e.g. 'band')          */
    const char  *text,       /* I: Appended text string of processing msg.    */
    int   max_entity,        /* I: Maximum number of 'entity' processed       */
    int   max_cat,           /* I: Maximum number of bands processed          */
    int   seconds,           /* I: Number of seconds between messages         */
    int   flag               /* I: Specifies format of display message.
                                   = ALLCAT -- processing all bands at once
                                   = ONECAT -- processing one band at a time
                                   = GEN1   -- general display message one
                                   = GEN2   -- general display message two    */
)
{
    char    outmsg[80];      /* Message to be outputted to terminal     */
    int     cc = T_STDCC;    /* Type of carriage control                */
    int     status;          /* Function return status code             */
    int     term_type;       /* Terminal type                           */

    msg_values.msgtim = seconds;
    status = t_init(&_TOTLIN,&_TOTCOL,&term_type);
    if (status != SUCCESS)
        c_lasmsg (FALSE, "inittm-init", "Error initializing terminal");
    if (term_type != T_CRT)
    {
        _TOTLIN = 1;
        _TOTCOL = 80;
    }

    /* Assign the input parameters to the global structure of msg_values.
       These values are used for all display messages.  Ensure the null
       character is appended to each input string parameter.        */

    msg_values.subject = subject;
    msg_values.verb = verb; 
    msg_values.entity = entity;
    msg_values.cat = cat;
    msg_values.text = text;
    msg_values.max_entity = max_entity;
    msg_values.max_cat = max_cat;
    msg_values.flag = flag;

    strcpy(outmsg," ");
    t_write(outmsg,cc);
}


#ifndef NO_OBSOLETE_INTERFACE


/**
 * c_inittm:
 * @subject: I: Contains the program name of the calling module.  To conform
 *              with standards, it should be all upper-case characters.
 * @verb: I:  The type of action being performed.  To conform with standards
 *            the first character of the string is upper case, the rest is
 *            lower case.  The verb is used in the past tense.  Some examples
 *            are: Processed, Histogrammed, Calculated, and Printed.
 * @entity: I: The entity being processed.  To conform with standards, it 
 *             should be all lower-case characters.  Some examples are: line,
 *             section, and page.
 * @cat: I: The category being processed.  To conform with standards, it
 *          should be all lower-case characters.  Some examples are: band, and
 *          page.
 * @text: I: An optional text string to be appended to the processing message.
 *           This text is normally blank.  An example is 
 *           "for relation AARCXYS".
 * @max_entity: I: The maximum number of entities described in parameter 
 *                 "entity".
 * @max_cat: I: The maximum number of entities described in parameter
 *                   "cat."
 * @seconds: I: The number of seconds to elapse between processing messages.
 *              This should be the value of the TAE global $MSGTIM.
 * @flag: I: Flag determining the format of the output message.  Valid values 
 *           are:
 *           ALLCAT (0): BIL message format (all categories processed
 *                                             at once),
 *           ONECAT (1): BSQ message format (categories processed separately),
 *           GEN1   (2): General message format one,
 *           GEN2   (3): General message format two.
 *
 * The c_inittm() routine is used to initialize the processing message
 * utility.  The utility routines c_startm() and c_stoptm() are required to
 * enable and disable the processing message utility.
 *
 * There are two external variables that the application program is required
 * to reference. They are:
 *        curent:  current entity being processed,
 *        curcat:  current category being processed.
 *
 * The external variables are referenced from "C" by the following statements:
 * extern int curent, and extern int curcat.
 *
 * The application program may modify the processing message format by 
 * calling c_inittm() which outputs the new line character so that the
 * processing message will scroll.
 *
 * The processing message is displayed on the last terminal line.  If text
 * already exists on this line, it is overwritten.  The processing message
 * is appended with blanks so that the full terminal line is overwritten.  If
 * the processing message is greater than the number of columns on the 
 * terminal, the results are undefined.
 * Example:
 * c_inittm('COPY','Processed', 'line', 'band', ' ', &hundred, &three,
 *            &msgtim, &flag)
 * FLAG:
 *       0:  "COPY Processed line + of 100 for all 3 bands",
 *       1:  "COPY Processed line _ of 100 for band_ of 3",
 *       2:  "COPY Processed line_",
 *       3:  "COPY Processed line_ of 100".
 *
 * Deprecated.  Use las_inittm() instead.
 */
void c_inittm
(
    const char  *subject,    /* I: Subject of message (e.g. process name)     */
    const char  *verb,       /* I: Verb of message (e.g. 'Processing')        */
    const char  *entity,     /* I: Entity of display message (e.g. 'line')    */
    const char  *cat,        /* I: Category of message (e.g. 'band')          */
    const char  *text,       /* I: Appended text string of processing msg.    */
    const long  *max_entity, /* I: Maximum number of 'entity' processed       */
    const long  *max_cat,    /* I: Maximum number of bands processed          */
    const long  *seconds,    /* I: Number of seconds between messages         */
    const long  *flag        /* I: Specifies format of display message.
                                   = ALLCAT -- processing all bands at once
                                   = ONECAT -- processing one band at a time
                                   = GEN1   -- general display message one
                                   = GEN2   -- general display message two    */
)
{

    las_inittm (subject,verb,entity,cat,text,*max_entity,
                *max_cat,*seconds,*flag);


}

#endif /* obsolete interfaces   */



    /* Reset the system timer to start displaying processing messages.  */
/**
 * c_startm:
 *
 * The c_startm() routine is used to enable the processing message utility.
 * The function c_inittm() must first be called to initialize the
 * processing message utility.  The function c_stoptm() must be called to
 * disable the processing message utility.
 */
void c_startm(void)
{

    if (msg_values.msgtim != 0)
        output_line();
    signal(SIGALRM,reset_time);     /* Call reset_time on next interrupt     */
    alarm(msg_values.msgtim);       /* Teset the alarm to appropriate number */
}



    /* STOPTM is called to temporarly disable the interrupt signal.  It 
       outputs the current line processed.  It is assumed "startm" will
       be called to restart the display message utility.  */
/**
 * c_stoptm:
 *
 * The c_stoptm() routine is used to disable the processing message utility. 
 * The function c_inittm() must first be called to initialize the processing
 * utility.  The function c_startm() must first be called to enable the
 * processing message utility.
 */
void c_stoptm(void)
{
    char outmsg[80];                /* Message to be written to terminal     */
    int    cc = T_STDCC;            /* Terminal carriage control             */

    alarm(0);                       /* Disable the alarm                     */
    if (msg_values.msgtim != 0)
    {
        if (curent - 1 == msg_values.max_entity)
            --curent;
        if (curcat - 1 == msg_values.max_cat)
            --curcat;
        output_line();

        strcpy(outmsg,"");
        t_write(outmsg,cc);

    } /*  if msg_values.msgtim != 0  */
}


static void output_line(void)
{
    char    line[200];
    char    *ptr;
    int     len;

    if (msg_values.flag == ALLCAT)  /* Output appropriate display message    */
        sprintf( line, "%-8s %s %s %d of %d for all %d %s %s",
                 msg_values.subject, 
                 msg_values.verb,msg_values.entity,curent,
                 msg_values.max_entity,msg_values.max_cat,msg_values.cat,
                 msg_values.text);
    else if (msg_values.flag == ONECAT)
        sprintf( line, "%-8s %s %s %d of %d for %s %d of %d %s",
                 msg_values.subject, 
                 msg_values.verb,msg_values.entity,curent,
                 msg_values.max_entity,msg_values.cat,curcat,
                 msg_values.max_cat,msg_values.text);
    else if (msg_values.flag == GEN1)
        sprintf( line, "%-8s %s %s %d %s",msg_values.subject,msg_values.verb, 
                 msg_values.entity,curent,msg_values.text);
    else if (msg_values.flag == GEN2)
        sprintf( line, "%-8s %s %s %d of %d %s",msg_values.subject,
                 msg_values.verb, 
                 msg_values.entity,curent,msg_values.max_entity,
                 msg_values.text);
    len = (int)strlen(line);
    if (len < _TOTCOL)
    {
        ptr = line + len;
        sprintf(ptr,"%*s",_TOTCOL-len-1," ");
    }
    t_pos(_TOTLIN,1);
    t_write(line,T_NULL);
}


    /* Reset_time is the function that is called when the alarm interrupt
       occurs. It outputs the appropriate display message, resets the
       alarm so that on the next alarm interrupt function 'reset_time'
       is called again.  */
static void reset_time(int signalnum)
{

    if (curent - 1 == msg_values.max_entity)
        --curent;
    if (curcat - 1 == msg_values.max_cat)
        --curcat;
    output_line();
    signal(SIGALRM,reset_time);     /* Call reset_time on next interrupt     */
    alarm(msg_values.msgtim);       /* Reset the alarm to appropriate number */
}



