/*******************************************************************************
NAME: c_fread

PURPOSE:  Provides an implementation of the C library fread function that will
          convert data read to the local endian.

PROGRAM HISTORY:
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        T. Beckmann     11/00   Original development

ALGORITHM DESCRIPTION:
   Read the data using fread
   Convert the data to the local endian

ALGORITHM REFERENCES:
   none
******************************************************************************/
#include <stdio.h>      /* fread prototype */
#include "las.h"        /* standard LAS defines */
#include "lasendian.h"  /* byte swapping routines */
#include "lasfileio.h"  /* prototype for this function */

/**
 * c_fread:
 * @buf: I/O: Pointer to a buffer for the data read from the file.  The data
 *            is converted from big endian byte order to the local
 *            byte order.
 * @size: I: Specifies the size, in bytes, of each element read from the file.
 *           This is limited to 1, 2, 4, or 8 bytes to match the size of 
 *           standard data types.
 * @count: I: Specifies the number of elements of "size" bytes to read
 *            from file.
 * @stream: I: Pointer to the file to read.  This file should be opened by
 *             the standard C fopen() call.
 *
 * All data files for LAS are written using big endian byte order.  This is the
 * byte order used by Mips, Sparc, and Motorola processors.  The c_fread()
 * routine is the equivalent of the standard C fread() routine that will
 * perform byte swapping on platforms that use little endian byte order.
 *
 * Returns: Number of items successfully read,
 *          E_FAIL:  Unsuccessful completion of operation, invalid
 *                   size or count parameters.
 */
size_t c_fread
(
    void *buf,               /* I/O: pointer to buffer to read the data into */
    size_t size,             /* I: size of each element read */
    size_t count,            /* I: number of elements to read */
    FILE *stream             /* I: stream to read */
)
{
    size_t items_read;  /* number of items read */

    /* make sure the size of each member is a power of two up to the number 
       of bytes in a 64-bit word */
    if (((size > 1) && (size & 0x01)) || (size > 8) || (size == 6))
    {
        c_lasmsg(FALSE,"fread-size","Element size of %d bytes is not a legal "
                 "word size", size);
        return E_FAIL;
    }

    if (count < 1)
    {
        c_lasmsg(FALSE,"fread-count","Element count of %d is not a legal",
                 count);
        return E_FAIL;
    }

    items_read = fread(buf, size, count, stream);

    /* convert the data read to the local byte order */
    localendian_buf(buf, size, items_read);

    return items_read;
}
