#include <stdio.h>
#include "tae.h"
#include "las.h"
#include "desc.h"
#include "util.h"

/**
 * las_getnum:
 * @vblock: I: TAE parameter block passed by reference.  It is assumed that
 *             TAE has been initialized prior to calling las_getnum().
 * @parname: I: Defines the TAE parameter name in the TAE parameter block
 *              which contains the integer or real value(s).
 * @count: I: Defines the number of entries in the parameter array
 *            specified by "parname" to copy into "buff".
 * @dtype: I: The parameter "dtype" describes the type of "buf" to write
 *            into.  If the PDF declares type INTEGER then "dtype"
 *            can be EBYTE, EWORD, or ELONG.  If the PDF declares type
 *            REAL then "dtype" can be EREAL or EDOUBLE.
 * @buf: I: The integer or real value(s) in the TAE parameter block specified
 *          by "param" are copied into this part of memory.  The "buf" is
 *          written to in a format specified by "dtype".  It is assumed this
 *          memory has been allocated prior to calling las_getnum().
 *
 * The las_getnum() routine returns TAE integer or real parameters in the
 * TAE parameter block under the parameter name "parname".  It checks to see
 * that the parameter name exists and does error checking on the data type.
 * It does NOT return from these errors.
 *
 * Returns: Actual number of entries read in.
 *          0:  parameter has a "null value",
 *          -1: parameter has "no value".
 */
int las_getnum
(
    struct PARBLK *vblock,  /* I: TAE parameter block                       */
    const char parname[],   /* I: The parameter name                        */
    int   count,            /* I: Number of entries to read in              */
    int   dtype,            /* I: NEWLAS data type                          */
    unsigned char  buf[]    /* O: Buffer to read into                       */
)
{
    short   *w_buf;
    int     *l_buf;
    float   *r_buf;
    double  *d_buf;

    int     actual_count;
    int     i;
    struct  VARIABLE *var;  /* Pointer into variable block                  */
    char    msg[ERRLEN+1];

    /* Get the parameter                                                    */
    var = p_find (vblock, parname);
    /* Check data type                                                      */
    if (((dtype == EBYTE || dtype == EWORD || dtype == ELONG)
            && var->v_type != V_INTEGER) || ((dtype == EREAL ||
                    dtype == EDOUBLE) && var->v_type != V_REAL))
    {
        sprintf (msg, "Error: Unexpected parameter type for parameter %s", 
                parname);
        c_lasmsg (TRUE, "getnum-fatal", msg);
    }

    w_buf = (short  *) buf;
    l_buf = (int   *) buf;
    r_buf = (float  *) buf;
    d_buf = (double *) buf;

    /* Get the min of count requested and count in vblock                   */
    actual_count = (count <= var->v_count) ? count : var->v_count;

    for (i = 0; i < actual_count; i++)
    {
        switch (dtype)
        {
            case (EBYTE):
                buf[i] = IVAL (*var, i);
                break;
            case (EWORD):
                w_buf[i] = IVAL (*var, i);
                break;
            case (ELONG):
                l_buf[i] = IVAL (*var, i);
                break;
            case (EREAL):
                r_buf[i] = RVAL (*var, i);
                break;
            case (EDOUBLE):
                d_buf[i] = RVAL (*var, i);
                break;
        }
    }
    return (actual_count);
}

#ifndef NO_OBSOLETE_INTERFACE


/**
 * c_getnum:
 * @vblock: I: TAE parameter block passed by reference.  It is assumed that
 *             TAE has been initialized prior to calling c_getnum().
 * @parname: I: Defines the TAE parameter name in the TAE parameter block
 *              which contains the integer or real value(s).
 * @count: I: Defines the number of entries in the parameter array
 *            specified by "parname" to copy into "buff".
 * @dtype: I: The parameter "dtype" describes the type of "buffer" to write
 *            into.  If the PDF declares type INTEGER then "dtype"
 *            can be EBYTE, EWORD, or ELONG.  If the PDF declares type
 *            REAL then "dtype" can be EREAL or EDOUBLE.
 * @buf: I: The integer or real value(s) in the TAE parameter block specified
 *          by "param" are copied into this part of memory.  The buffer is
 *          written to in a format specified by "dtype".  It is assumed this
 *          memory has been allocated prior to calling c_getnum().
 *
 * The c_getnum() routine returns TAE integer or real parameters in the
 * TAE parameter block under the parameter name "parname".  It checks to see
 * that the parameter name exists and does error checking on the data type.
 * It does NOT return from these errors.
 *
 * Deprecated.  Use las_getnum() instead.
 *
 * Returns: Actual number of entries read in.
 *          0:  parameter has a "null value",
 *          -1: parameter has "no value".
 */
int c_getnum
(
    struct PARBLK *vblock,      /* I: TAE parameter block              */
    const char parname[],       /* I: The parameter name               */
    const long *count,          /* I: Number of entries to read in     */
    const long *dtype,          /* I: NEWLAS data type                 */
    unsigned  char buf[]        /* O: Buffer to read into              */
)
{
    int status;

    status = las_getnum (vblock, parname, *count, *dtype, buf);

    return (status);

}

#endif /* obsolete interfaces   */


