/*******************************************************************************
NAME:    C_GETPARM

PURPOSE: Extract TAE parameters from a project parameter file.
         This routine will extract the nth occurrence of a parameter and send
         all of its values back to the calling routine.  If successful
         c_getparm will return the number of values found for the named
         parameter, if it is unable to find the nth occurrence of the parameter
         it will return 0, and E_FAIL for all errors.  To find the last
         occurrence of a parameter use a position of 0.


PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Development
  1.1    D.Etrheim        11/94  EDC   Use c_lsmknm to get file name

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   C_GETPARM must run under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION: Initial code copied from the LAS function GETPARM and
                       then modified to retrieve the nth occurrence of a
                       parameter in a ppf file.
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "worgen.h"
#include "util.h"


/**
 * c_getparm:
 * @filename: I: Name of the project parameter file.
 * @parm_name: I: Name of the variable to be retrieved.
 * @string_value: O: String values of variable returned.
 * @type: O: Type of the parameter being returned.
 * @double_value: O: Double values of the variable returned.
 * @long_value: O: Variable being returned.
 * @position: I: The nth occurrence to retrieve.
 *
 * The c_getparm() routine extracts TAE parameters from a project parameter
 * file.  It will extract the nth occurrence of a parameter and send all of
 * its values back to the calling routine.  If successful, c_getparm will
 * return the number of values found for the named parameter, if it is
 * unable to find the nth occurrence of the parameter it will return 0, and
 * E_FAIL for all errors.  To find the last occurrence of a parameter use
 * a position of 0.
 *
 * Returns: Number of values found for the named parameter,
 *          E_FAIL: Unsuccessful completion of operation.
 */
int c_getparm
(
    const char *filename,       /* I: Name of the project parameter file    */
    const char *parm_name,      /* I: Name of the variable to be retrieved  */
    char string_value[][CMLEN], /* O: String values of variable returned    */
    char *type,                 /* O: Type of the parameter being returned  */
    double double_value[],      /* O: Double values of the variable returned*/
    int  long_value[],          /* O: Variable being returned               */
    int  position               /* I: The nth occurrence to retrieve        */
)

{
    char   pname[CMLEN];         /* Temporary parameter name for conversion */ 
    char   tmp_filename[CMLEN];  /* Filename with the .ppf extension        */
    char   tmp_name[CMLEN];      /* Temporary parameter name                */
    char   tmp_string[CMLEN];    /* Temporary string variable               */
    char   tmp_type[2];          /* Temporary type variable                 */

    double tmp_double;           /* Temporary double value of a variable    */

    int    dimension;            /* Dimension of the requested parameter    */
    int    i;                    /* Loop control                            */
    int    j;                    /* Loop control                            */
    int    tmp_dimension;        /* Temp dimension of the current parameter */
    int    tmp_long;             /* Temporary long value of a variable      */

    FILE   *parm_file;           /* Project parameter file                  */


    /* Check for a valid position value                                     */
    if (position < 0)
    {
        c_lasmsg (FALSE, "c_getparm-pos",
                "Position must be greater than or equal to zero");
        return(E_FAIL);
    }

    /* Add the .ppf extension onto filename                                 */
    c_lsmknm(filename,"ppf",tmp_filename);

    /* Open the project parameter file for reading                          */
    if ((parm_file = fopen(tmp_filename, "r")) == NULL)
    {
        c_lasmsg (FALSE, "c_getparm-open", "Error opening file");
        return(E_FAIL);
    }

    /* Convert the requested parameter name to all uppercase                */
    strcpy (pname, parm_name);
    las_low2up(pname);

    i = 0;
    dimension = 0;
    while (TRUE)
    {
        /* Read a parameter name, its type, and its number of values        */
        j = fscanf(parm_file, "%s %s %d", tmp_name, tmp_type, &tmp_dimension);
        if ((j == EOF) && (position == 0))
        {
            fclose(parm_file);
            return(dimension);
        }
        else if (j == EOF)
        {
            fclose(parm_file);
            return(0);
        }
        else if (j != 3)
        {
            fclose(parm_file);
            c_lasmsg (FALSE, "c_getparm-format", "Invalid file format");
            return(E_FAIL);
        }

        /* Convert the parameter name and type to all uppercase             */
        las_low2up(tmp_name);
        las_low2up(tmp_type);

        /* If this is the requested parameter                               */
        if (strcmp(pname, tmp_name) == 0)
        {
            i++;
            strcpy(type, tmp_type);
            dimension = tmp_dimension;

            /* Read the indicated number of values for the current parameter*/
            if (strcmp(type, "I") == 0)
            {
                for (j = 0; j < dimension; j++)
                    if (fscanf(parm_file, "%d", &long_value[j]) != 1)
                    {
                        fclose(parm_file);
                        c_lasmsg (FALSE, "c_getparm-format",
                                "Invalid file format" );
                        return(E_FAIL);
                    }
            }
            else if (strcmp(type, "S") == 0)
            {
                for (j = 0; j < dimension; j++)
                    if (fscanf(parm_file, "%s", string_value[j]) != 1)
                    {
                        fclose(parm_file);
                        c_lasmsg (FALSE, "c_getparm-format",
                                "Invalid file format");
                        return(E_FAIL);
                    }
            }
            else if (strcmp(type, "R") == 0)
            {
                for (j = 0; j < dimension; j++)
                    if (fscanf(parm_file, "%lf", &double_value[j]) != 1)
                    {
                        fclose(parm_file);
                        c_lasmsg (FALSE, "c_getparm-format",
                                "Invalid file format");
                        return(E_FAIL);
                    }
            }
            else
            {
                fclose(parm_file);
                sprintf(tmp_string, "Invalid parameter type %s", type);
                c_lasmsg (FALSE, "c_getparm-type", tmp_string);
                return(E_FAIL);
            }

            /* If this is the requested occurrence the return the dimension */
            if (i == position)
            {
                fclose(parm_file);
                return(dimension);
            }
        }
        else
        {
            /* Skip past the indicated number of values for the current
               parameter                                                    */
            if (strcmp(tmp_type, "I") == 0)
            {
                for (j = 0; j < tmp_dimension; j++)
                    if (fscanf(parm_file, "%d", &tmp_long) != 1)
                    {
                        fclose(parm_file);
                        c_lasmsg (FALSE, "c_getparm-format",
                                "Invalid file format");
                        return(E_FAIL);
                    }
            }
            else if (strcmp(tmp_type, "S") == 0)
            {
                for (j = 0; j < tmp_dimension; j++)
                    if (fscanf(parm_file, "%s", tmp_string) != 1)
                    {
                        fclose(parm_file);
                        c_lasmsg (FALSE, "c_getparm-format",
                                "Invalid file format");
                        return(E_FAIL);
                    }
            }
            else if (strcmp(tmp_type, "R") == 0)
            {
                for (j = 0; j < tmp_dimension; j++)
                    if (fscanf(parm_file, "%lf", &tmp_double) != 1)
                    {
                        fclose(parm_file);
                        c_lasmsg (FALSE, "c_getparm-format",
                                "Invalid file format");
                        return(E_FAIL);
                    }
            }
            else
            {
                fclose(parm_file);
                sprintf(tmp_string, "Invalid parameter type %s", tmp_type);
                c_lasmsg (FALSE, "c_getparm-type", tmp_string);
                return(E_FAIL);
            }
        }
    }
}
