#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"

/**
 * las_getstr:
 * @vblock: I: TAE parameter block passed by reference.  It is assumed that
 *             TAE has been initizialied prior to calling las_getstr().
 * @parname: I: Defines the TAE parameter name in the TAE parameter block
 *              which contains the string(s).
 * @count: I: Defines the number of entries in the parameter array specified
 *            by "parname" to copy into "buf".
 * @slen: I: Defines the offset between the beginnings of strings as they are
 *           copied into the buffer.  This parameter should be at least the
 *           size of the string in TAE plus one.  The buffer will be filled in
 *           every "slen" bytes.
 * @buf: O: The strings in the TAE parameter block specified by "parname" are
 *          copied into this part of memory.  This buffer must have allocated
 *          to it at least (count * slen)bytes.  It is assumed this memory
 *          has been allocated prior to calling las_getstr().  Be warned that
 *          if "slen" is less than or equal to the length specified in the TAE
 *          parameter block, las_getstr() will possibly try to write past the
 *          end of the allocated buffer.
 *
 * The las_getstr() routine retrieves TAE string parameters located in the
 * TAE parameter block under the name "parname".  It checks to see that the
 * parameter name exists and does error checking on the data type.  It does
 * NOT return from these errors.
 *
 * Returns:  Actual number of entries read in.
 *           0:  Parameter has a "null value",
 *           -1: Parmater has "no value".
 */
int las_getstr
(
    struct PARBLK *vblock, /* I: TAE parameter block                        */
    const char parname[],  /* I: The parameter name                         */
    int   count,           /* I: Number of entries to read in               */
    int   slen,            /* I: Dimensioned length of string(s)            */
    char buf[]             /* O: Buffer to read into                        */
)
{
    int   actual_count;
    int   i;
    struct  VARIABLE *var; /* Pointer into variable block                   */
    char    msg[ERRLEN+1];

    /* Get the parameter                                                    */
    var = p_find (vblock, parname);
    /* Check data type                                                      */
    if (var->v_type != V_STRING)
    {
        sprintf (msg, "Error: Unexpected parameter type for parameter %s", 
                 parname);
        c_lasmsg (TRUE, "getstr-fatal", msg);
    }

    /* Get the min of count requested and count in vblock                   */
    actual_count = (count <= var->v_count) ? count : var->v_count;

    for (i = 0; i < actual_count; i++)
    {
        strcpy (buf + (i * (slen)), SVAL (*var, i));
    }
    return (actual_count);
}    




#ifndef NO_OBSOLETE_INTERFACE


/**
 * c_getstr:
 * @vblock: I: TAE parameter block passed by reference.  It is assumed that
 *             TAE has been initizialied prior to calling c_getstr().
 * @parname: I: Defines the TAE parameter name in the TAE parameter block
 *              which contains the string(s).
 * @count: I: Defines the number of entries in the parameter array specified
 *            by "parname" to copy into "buf".
 * @slen: I: Defines the offset between the beginnings of strings as they are
 *           copied into the buffer.  This parameter should be at least the
 *           size of the string in TAE plus one.  The buffer will be filled in
 *           every "slen" bytes.
 * @buf: O: The strings in the TAE parameter block specified by "parname" are
 *          copied into this part of memory.  This buffer must have allocated
 *          to it at least (count * slen)bytes.  It is assumed this memory
 *          has been allocated prior to calling c_getstr().  Be warned that
 *          if "slen" is less than or equal to the length specified in the TAE
 *          parameter block, c_getstr() will possibly try to write past the
 *          end of the allocated buffer.
 *
 * The c_getstr() routine retrieves TAE string parameters located in the
 * TAE parameter block under the name "parname".  It checks to see that the
 * parameter name exists and does error checking on the data type.  It does
 * NOT return from these errors.
 *
 * Deprecated.  Use las_getstr() intead.
 *
 * Returns:  Actual number of entries read in.
 *           0:  Parameter has a "null value",
 *           -1: Parmater has "no value".
 */
int c_getstr
(
    struct PARBLK *vblock,  /* I: TAE parameter block                       */
    const char parname[],   /* I: The parameter name                        */
    const long *count,      /* I: Number of entries to read in              */
    const long *slen,       /* I: Dimensioned length of string(s)           */
    char buf[]              /* O: Buffer to read into                       */
)
{
    int status;

    status = las_getstr (vblock, parname, *count, *slen, buf);

    return (status);

}


#endif /* obsolete interfaces   */

