/******************************************************************************
NAME:                           C_INLAS

FUNCTION:
        C_INLAS is a support routine which initializes TAE and the LAS
        program.  It is used so that all programs (FORTRAN and C) use the
        C TAE I/O routines.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         5/86       K. Gacke     initial development
    2.0        11/87       K. Gacke     change to use C TAE I/O (instead
                                        of FORTRAN) 
    3.0        12/87       K. Zanter    1.  switched to raw C types
                                        2.  changed include file specification
                                        3.  renamed source from inlas to 
                                            c_inlas
                                        4.  moved bridge routines to separate
                                            source file
    3.1        12/91       D. Etrheim   Check for ASYNC runtype.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
   C_INLAS must be run under TAE.

PROJECT:        LAS

ALGORITHM:
        TAE is initialized by calling the "z_init".  It also
        saves the beginning execution time in the global "begin_time". 

******************************************************************************/
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "tae.h"

#define STD_OUT 8

int begin_time;

/**
 * initialize_inlas:
 * @vblock: O: TAE parameter block passed by reference.  An array to receive
 *             the initial TAE parameter block from the TAE monitor.  "Vblock"
 *             should have a dimension sufficient to receive a TAE parameter
 *             block (ie. struct PARBLK is recommended for C).
 *
 * The initialize_las() routine is used to initialize a LAS program. The 
 * LAS program may be written in FORTRAN or C. The initialize_las() call 
 * should be the first executable statement of the application program 
 * to properly set up for the chosen run type.
 */
void initialize_las
(
    struct PARBLK *vblock       /* O: TAE parameter block                   */
)
{

    struct VARIABLE *val;       /* pointer to a TAE variable within v-block */
    int  zero = 0;
    char *runtype;

    z_init(vblock,P_ABORT);         /* initialize TAE */

    begin_time = 0;

    val = p_find(vblock,"$RUNTYPE");            /* if running in batch then  */
    runtype = SVAL(*val,0);                     /*   reset $MSGTIM to zero   */
    if ((strcmp(runtype,"BATCH") == 0) || (strcmp(runtype,"ASYNC") == 0))
    {
        q_intg(vblock,"$MSGTIM",1,&zero,P_UPDATE);
        q_out(vblock);
    }
}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_inlas:
 * @vblock: O: TAE parameter block passed by reference.  An array to receive
 *             the initial TAE parameter block from the TAE monitor.  "Vblock"
 *             should have a dimension sufficient to receive a TAE parameter
 *             block (ie. struct PARBLK is recommended for C).
 * @sout: O: Logical unit number (LUN) used in opening the standard output.
 *           "sout" is assigned the value of 8.
 *
 * The c_inlas() routine is used to initialize a LAS program. The LAS program
 * may be written in FORTRAN or C. The c_inlas() call should be the 
 * first executable statement of the application program to properly set 
 * up for the chosen run type.
 *
 * Deprecated.  Use initialize_las() instead.
 */
void c_inlas
(
    struct PARBLK *vblock,       /* O:  TAE parameter block                 */
    long *sout                   /* O:  unit number for opening stdout      */
)
{ 
    *sout = STD_OUT;            /* initialize standard out */

    initialize_las (vblock);

}


#endif /* obsolete interfaces   */

