#include "las.h"
#include "util.h"
/******************************************************************************
NAME:           C_JULGRE

PURPOSE:        Convert a Julian date into a Gregorian date

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  2.0           2/91    D. Hollaren        CSB      Conversion to LAS5.0

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
******************************************************************************/
static int  days1[13] = {0,31,59,90,120,151,181,212,243,273,304,334,365};
static int  days2[13] = {0,31,60,91,121,152,182,213,244,274,305,335,366};

/**
 * las_julgre:
 * @year: I: The year.
 * @jdate: I: The Julian date in the given year.  The Julian date is the
 *            numerical position of the date in the year, starting with 
 *            January 1 at 1 and ending with December 31 at 365 (366 in a
 *            leap year).
 * @month: O: The number of the month, where January = 1 and December = 12.
 * @day: O: The day of the month.
 *
 * The las_julgre() routine converts the Julian date of the given year to a
 * Gregorian date(month and day).  las_julgre() is located in the 
 * file julian.c.
 */
void las_julgre
(
    int  year,         /* I:  Year                                          */
    int  jdate,        /* I:  Julian date                                   */
    int  *month,       /* O:  Gregorian month                               */
    int  *day          /* O:  Gregorian day                                 */
)
{
    int  index;        /* Index into the days arrays                        */

    /* Calculate the gregorian date */
    if (las_isleap(year))
    {
        /* Find the gregorian date in a leap year */
        for (index = 1; index < 13; index++)
        {
            if (jdate <= days2[index])
            {
                *day = jdate - days2[index-1];
                *month = index;
                break;
            }
        }
    }
    else
    {
        /* Find the gregorian date in a non-leap year */
        for (index = 1; index < 13; index++)
        {
            if (jdate <= days1[index])
            {
                *day = jdate - days1[index-1];
                *month = index;
                break;
            }
        }
    }
}



#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_julgre:
 * @year: I: The year.
 * @jdate: I: The Julian date in the given year.  The Julian date is the
 *            numerical position of the date in the year, starting with 
 *            January 1 at 1 and ending with December 31 at 365 (366 in a
 *            leap year).
 * @month: O: The number of the month, where January = 1 and December = 12.
 * @day: O: The day of the month.
 *
 * The c_julgre() routine converts the Julian date of the given year to a
 * Gregorian date(month and day).  c_julgre() is located in the 
 * file julian.c.
 *
 * Deprecated.  Use las_julgre() instead.
 */
void c_julgre
(
    const long *year,           /* I: Year                              */
    const long *jdate,          /* I: Julian date                       */
    long *month,                /* O: Gregorian month                   */
    long *day                   /* O: Gregorian day                     */
)
{
    int imonth, iday;

    las_julgre (*year, *jdate, &imonth, &iday);

    *month = imonth;
    *day = iday;
}


#endif /* obsolete interfaces   */


/******************************************************************************
NAME:           C_GREJUL

PURPOSE:        Convert a Gregorian date to a Julian date.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  2.0           2/91    D. Hollaren        CSB      Conversion to LAS5.0

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
******************************************************************************/
/**
 * las_grejul:
 * @year: I: The year.
 * @month: O: The number of the month, where January = 1 and December = 12.
 * @day: O: The day of the month.
 * @jdate: I: The Julian date in the given year.  The Julian date is the
 *            numerical position of the date in the year, starting with 
 *            January 1 at 1 and ending with December 31 at 365 (366 in a
 *            leap year).
 *
 * The las_grejul() routine converts the Gregorian date (month and day)
 * to the Julian date of the given year
 *
 */
void las_grejul
(
    int year,          /* I:  Year                                          */
    int month,         /* I:  Gregorian month                               */
    int day,           /* I:  Gregorian day                                 */
    int *jdate         /* O:  Julian date                                   */
)
{

    /* Calculate the julian date                                            */
    if (las_isleap(year))
        *jdate = days2[month-1] + day;            /* Leap year             */
    else
        *jdate = days1[month-1] + day;            /* Non-leap year         */
}



#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_grejul:
 * @year: I: The year.
 * @month: O: The number of the month, where January = 1 and December = 12.
 * @day: O: The day of the month.
 * @jdate: I: The Julian date in the given year.  The Julian date is the
 *            numerical position of the date in the year, starting with 
 *            January 1 at 1 and ending with December 31 at 365 (366 in a
 *            leap year).
 *
 * The c_grejul() routine converts the Gregorian date (month and day)
 * to the Julian date of the given year
 *
 * Deprecated use las_grejul() instead.
 *
 */
void c_grejul
(
    const long *year,  /* I: Year                                           */
    const long *month, /* I: Gregorian month                                */
    const long *day,   /* I: Gregorian day                                  */
    long *jdate        /* O: Julian date                                    */
)
{
    int ijdate;

    las_grejul (*year, *month, *day, &ijdate);

    *jdate = ijdate;
}


#endif /* obsolete interfaces   */

