/*****************************************************************************
NAME            C_PTERM

PURPOSE         Outputs data to the terminal screen allowing the user to
                issue commands.

DATA STRUCTURES 
  struct static_val --> a structure containing all static fields of the display
                        message.  This includes the subject, verb, entity,
                        category, text, current entity, maximum entity, current
                        category, maximum category, number of seconds between 
                        display messages, and a flag indicating the format of 
                        the message.

PROGRAM HISTORY
        PROGRAMMER        DATE          REASON
        ----------        ----          ------
        K. Gacke          6/86          initial development
        D. Hollaren      04/88          changed include spec to worgen
                                        changed cerrmsg to c_errmsg and
                                        passed the status code as an address
                                        added option to continue to proccess
                                        without writing output to the terminal
                                        redefined quit command to really quit
        B. Ailts        08/89           checked for upper case before calling
                                        tolower
        D. VanderZee    07/90           Standardized error handling
        D. Etrheim      09/92           Typecast the constant after call to
                                        strchr for ANSI standards.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        none.

PROJECT:        LAS

ALGORITHM:
  On the first call the TAE terminal initialization routine is called
  to determine the number of lines and columns on the terminal screen.
  If the process is being executed in batch
        the current command is set to quit (terminal is an invalid option)
  otherwise
        the current command is set to page
  On the first and subsequent calls the text line is outputted to the
  terminal screen and to a temporary file.
  The user is queried to input terminal commands to page, scroll, start from
   beginning, or search for specified character string.

ALGORITHM REFERENCES:   none.

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "las.h"
#include "util.h"

#define _MAXVAL 200

static int   _FIRST_PTERM = TRUE; /* Flag indicating first call to C_PTERM   */
static int   _CURLIN =  1;        /* Current terminal line                   */
static int   _TOTLIN;             /* Total number of terminal lines          */
static int   _TOTCOL;             /* Total number of terminal columns        */
static char  _CURCMD[2];          /* Current terminal command                */
static char  _SCAN[100] = {""};   /* Scan string                             */
static FILE  *_FP;                /* Temporary file pointer                  */


static void read_command (void);
static void read_temp (void);

/******************************************************************************
FUNCTION:       LIST_HELP

PURPOSE:        Outputs the list of valid terminal commands to the user's
                terminal screen.

ALGORITHM 
  Write a heading on the user's terminal
  Write each help line to the user's terminal

ALGORITHM REFERENCES    none
******************************************************************************/
static void list_help
(void)
{
    char    msg[100];                   /* buffer to be written to terminal  */

    /* Write a heading on the user's terminal                                */
    strcpy(msg,"\n\nValid Terminal Commands");
    t_highlight(msg);
    t_write(msg,T_STDCC);

    /* Write each help line to the user's terminal                           */
    t_write("   h : list valid terminal commands",T_STDCC);
    t_write("   l : output one line of data to the terminal",T_STDCC);
    t_write("   p : terminal data is to be paged",T_STDCC);
    t_write("   s : terminal data is to be scrolled",T_STDCC);
    t_write("   b : terminal data is outputted starting at the beginning",
            T_STDCC);
    t_write("   / : search for the specified string",T_STDCC);
    t_write("   c : continue processing without writing output to"
            " the terminal",T_STDCC);
    t_write("   q : quit processing",T_STDCC);
}



/******************************************************************************
FUNCTION:       SCAN_BUF

PURPOSE:        Scans the record for matches

ALGORITHM 
  Get the scan string size and buffer record length/bytes
  Search through the buffer starting at the current position for a string
  match
        Search for an exact match
        Found a match return a successful status

ALGORITHM REFERENCES    none
******************************************************************************/
static int  scan_buf
(
    const char   *buf,  /* buffer containing the record                     */
    int     start       /* starting offset to begin scan                    */
)
{
    int    count;       /* byte offset of a match                           */
    int    length;      /* length of scan string                            */
    int    match;       /* number of bytes that have matched                */
    int    nbytes;      /* actual # of bytes to be scanned                  */
    int    recsiz;      /* number of bytes in record                        */

    /* Get the scan string size and buffer record length/bytes              */
    length = (int)strlen(_SCAN);
    recsiz = (int)strlen(buf);
    nbytes = recsiz - length + 1;

    /* Search through the buffer starting at the current position for a string
       match                                                                */
    for (buf += start,count = start; count < nbytes; buf++,count++)
    {
        if (*buf == *_SCAN)
        {
            /* Search for an exact match                                    */
            for (match = 1;
                    (match < length) && (*(buf + match) == *(_SCAN + match));
                    match++)
                ;

            /* Found a match return a successful status                     */
            if (match == length)
            {
                strcpy(_CURCMD,"/");
                return(E_SUCC);
            }
        }
    }

    return(E_FAIL);
}


/******************************************************************************
FUNCTION:       READ_COMMAND

PURPOSE:        Gets the next command from the user.  It then process that
                command.

ALGORITHM 
  User is queried for new command only if the current command is output by
  line, page, or string search was a success.
     Do until valid command is entered
        Output promt to the user
        Read the command and convert it to lower case
        if command is null
            use the previous command
        Process the command
           list help, restart at beginning, list line at time, page, scroll,
           search, and quit.

ALGORITHM REFERENCES    none
******************************************************************************/
static void read_command
(void)
{
    long fatal = -1;           /* Fatal code for c_errmsg */
    int   term;                /* specifies how string was terminate         */
    char  command[100];        /* command entered by the user                */
    char  outmsg[100];         /* message to be outputted                    */
    short cont = TRUE;         /* TRUE until valid command entered           */

    /* User is queried for new command only if the current command is output
       by line, page, or string search was a success.                        */
    if ((_CURCMD[0] == 'l') || (_CURCMD[0] == 'p') || (_CURCMD[0] == '/'))
    {
        /* Do until valid command is entered */
        for (;cont;)
        {
            /* Output promt to the user */
            sprintf(outmsg,"\nPlease enter command [%s]: ",_CURCMD);
            t_highlight(outmsg);
            t_write(outmsg,T_PROMPT);

            /* Read the command and convert it to lower case */
            t_read(command,&term);
            if (isupper(command[0]))
                command[0] = tolower(command[0]);

            /* If command is null, use the previous command */
            if (strlen(command) == 0)
                strcpy(command,_CURCMD);

            /* Process the command */
            switch (command[0])
            {
                case 'h':
                    list_help();              /* list help                  */
                    break;

                case 'b': 
                    strcpy(_CURCMD,"p");      /* start from beginning       */
                    _CURLIN = 1;
                    t_clear();
                    read_temp();
                    cont = FALSE;
                    break;

                case 'l': 
                    strcpy(_CURCMD,"l");     /* output one line at a time  */
                    _CURLIN = _TOTLIN;
                    t_lclear(_TOTLIN+1,1); 
                    t_lclear(_TOTLIN+2,1);
                    t_pos(_TOTLIN,1); 
                    cont = FALSE;
                    break;

                case 'p': 
                    strcpy(_CURCMD,"p");      /* paging command             */
                    _CURLIN = 1;
                    t_lclear(_TOTLIN+1,1); 
                    t_lclear(_TOTLIN+2,1);
                    t_pos(_TOTLIN,1); 
                    cont = FALSE;
                    break;

                case 's':  
                    strcpy(_CURCMD,"s");      /* scroll command             */
                    _CURLIN = 1;
                    t_lclear(_TOTLIN+1,1); 
                    t_lclear(_TOTLIN+2,1);
                    t_pos(_TOTLIN,1); 
                    cont = FALSE;
                    break;

                case '/': 
                    strcpy(_CURCMD,"|");     /* search command             */
                    _CURLIN = _TOTLIN;
                    if (strlen(command) > 1)
                        strcpy(_SCAN,&command[1]);

                    if (strlen(_SCAN) < 1)
                    {
                        c_lasmsg (FALSE, "pterm-search",
                                  "Error: No search pattern specified");
                        strcpy(_CURCMD,"l");
                        break;
                    }

                    t_lclear(_TOTLIN+1,1); 
                    t_lclear(_TOTLIN+2,1);
                    t_pos(_TOTLIN,1); 
                    sprintf(outmsg,"\n ... searching for string \"%s\"",_SCAN);
                    t_write(outmsg,T_STDCC);
                    cont = FALSE;
                    break;

                case 'c': 
                    strcpy(_CURCMD,"c");      /* continue command           */
                    _CURLIN = 1;
                    cont = FALSE;
                    break;

                case 'q': 
                    strcpy(_CURCMD,"q");      /* quit command               */
                    _CURLIN = 1;
                    c_errmsg("Process terminated by user","pterm-quit",&fatal);
                    cont = FALSE;
                    break;

                default :                     /* default to list help       */
                    list_help();
                    break;
            }
        }
    }
}

/******************************************************************************
FUNCTION:       READ_TEMP

PURPOSE:        Outputs the terminal data starting from the beginning.  It
                positions the file pointer to the beginning of the temporary
                file and read until end of file is reached.

ALGORITHM 
  Flush temporary file
  Seek to the beginning of the temp file
  Read from the temp file until the End Of File is reached
     Get a record from the temporary file until the end of the file
     Note:  it is required to take the new line character out of the record
     Get the command from the user
     Process the command
        quit, scroll, search or write buffer to the terminal

ALGORITHM REFERENCES    none
******************************************************************************/
static void read_temp
(void)
{
    long fatal = -1;                    /* Fatal code for c_errmsg */
    char buf[_MAXVAL];                  /* I/O buffer */
    char *ptr;                          /* pointer to character string */

    /* Flush temporary file */
    fflush(_FP);

    /* Seek to the beginning of the temp file */
    if (fseek(_FP,0,0) == EOF)
        c_lasmsg (FALSE, "pterm-seek",
                  "Error seeking to beginning of temporary file");

    /* Read from the temp file until the End Of File is reached.  Each 
       record is written to the terminal.  Note:  it is required to take
       the new line character out of the record.                             */
    for (;;)
    {
        /* Get a record from the temporary file until the end of the file */
        ptr = fgets(buf,_MAXVAL,_FP);
        if (ptr == NULL)
            break;

        /* Get the command from the user */
        *(buf + strlen(buf) - 1) = '\0';
        if (_CURLIN > _TOTLIN)
            read_command();

        /* Process the command */
        switch (_CURCMD[0])
        {
            case 'c': 
                break;                      /* Continue                   */

            case 'q':                       /* Quit                       */
                c_errmsg("User terminated processing","pterm-quit",&fatal);
                break;

            case '|':
                if (scan_buf(buf,0) == E_SUCC) /* Search                     */
                {
                    t_write(buf,T_STDCC);      /* write buffer at found loc  */
                    _CURLIN++;
                }
                break;

            case 's':
                t_write(buf,T_STDCC);          /* Scroll                     */
                break;

            default:
                t_write(buf,T_STDCC);          /* Write to the terminal      */
                _CURLIN++;
                break;
        }
    }
}


/******************************************************************************
FUNCTION:       TERM_OUT

PURPOSE:        Outputs the text line to the terminal and to the temporary
                file.

ALGORITHM 
  Process the user's command by
        Continue, doing nothing       or
        Search for the next string    or
        Write buffer to the terminal  or
        Write buffer to the terminal and temporary file

ALGORITHM REFERENCES    none
*******************************************************************************/

static void term_out
(
    char *buf               /* buffer to be written to terminal             */
)
{

    /* Process the user's command                                           */
    switch (_CURCMD[0])
    {
        case 'c':    
            break;                    /* Continue, do nothing      */

        case '|':    
            fprintf(_FP,"%s\n",buf);  /* Search for a string       */
            if (scan_buf(buf,0) == E_SUCC)
            {
                t_write(buf,T_STDCC);
                _CURLIN++;
            }
            break;

        case 's':    
            t_write (buf,T_STDCC);     /* Write buffer to terminal  */
            break;

        default:     
            fprintf(_FP,"%s\n",buf);  /* Write buffer to term & file*/
            t_write(buf,T_STDCC);
            _CURLIN++;
            break;
    }

}

/*  FIXME - buf parameter in las_pterm should be const and recursion removed */

/** 
 * las_pterm:
 * @buf: I/O: Line of text to be written to terminal screen.
 * @flag: I: Flag allowing the application program to control the paging
 *           mechanism for terminal output.
 *              0:  Do not page,
 *           <> 0:  After text is written, query user.
 *
 * The las_pterm() routine is used to output one line of text to the terminal
 * screen allowing the user to issue the following commands:
 *               h: list valid terminal commands,
 *               l: output one line of data to terminal,
 *               p: terminal data is to be paged,
 *               s: terminal data is to be scrolled,
 *               b: terminal data is output starting at the beginning,
 *               /: search for the specified string,
 *               c: continue processing without writing output to terminal,
 *               q: quit processing.
 * las_pterm() will automatically page, no matter which "flag" is set.
 */
void las_pterm
(
    char *buf,         /* I/O: character buffer to output                   */
    int  flag          /* I: flag to query user for command                 */
)
{

    int     one = 1;   /* value of one                                      */
    int     term_type; /* terminal type                                     */
    char    *ptr;      /* pointer to character string                       */

    /* If first call initialize TAE terminal package and open the temporary
       file                                                                 */
    if (_FIRST_PTERM)
    {
        _FIRST_PTERM = FALSE;
        t_init(&_TOTLIN,&_TOTCOL,&term_type);

        /* Check to make sure the terminal is a supported CRT               */
        if (term_type == T_NOTCRT)
            c_lasmsg (FALSE, "pterm-crt",
                    "Error: Terminal is not a supported CRT.");
        else if (term_type == T_NOTTERM) /* if program is in batch the      */
        {                                /* terminal option is invalid      */
            strcpy(_CURCMD,"c");         /* current command is set to cont  */
            _TOTLIN = _MAXVAL;           /* if it is desired to go to the   */
        }                                /* batch log current command       */
        else                             /* should be set to scroll.        */
        {
            _TOTLIN -= 2;
            strcpy(_CURCMD,"p");
            _FP = tmpfile();
            if (_FP == NULL)
                c_lasmsg (FALSE, "pterm-open", "Error opening temporary file");
        }

        if (*buf == '\f')                /* if first char of first line is  */
            *buf = ' ';
    }

    /* Search for the form feed character in the input buffer.  If one
       is found recursivly call this routines to process each portion
       of the buffer                                                        */
    if ((ptr = strchr(buf,'\f')) != NULL)
    {
        *ptr = '\0';
        las_pterm(buf,one);
        buf = ptr + 1;
    }

    /* Until the current line <= total lines, get the next command from
       the user                                                             */
    for (; _CURLIN > _TOTLIN; )
        read_command();

    /* Output line to the terminal                                          */
    term_out(buf);

    /* If the flag is set get next command from the user                    */
    if (flag)
        read_command();
}

/** 
 * c_pterm:
 * @buf: I/O: Line of text to be written to terminal screen.
 * @flag: I: Flag allowing the application program to control the paging
 *           mechanism for terminal output.
 *              0:  Do not page,
 *           <> 0:  After text is written, query user.
 *
 * The c_pterm() routine is used to output one line of text to the terminal
 * screen allowing the user to issue the following commands:
 *               h: list valid terminal commands,
 *               l: output one line of data to terminal,
 *               p: terminal data is to be paged,
 *               s: terminal data is to be scrolled,
 *               b: terminal data is output starting at the beginning,
 *               /: search for the specified string,
 *               c: continue processing without writing output to terminal,
 *               q: quit processing.
 * c_pterm() will automatically page, no matter which "flag" is set.
 *
 * Deprecated.  Use las_pterm() instead.
 */
void c_pterm
(
    char *buf,         /* I/O: Character buffer to output                   */
    const long *flag   /* I: Flag to query user for command                 */
)
{
    las_pterm (buf, *flag);
}


