/*******************************************************************************
NAME:    C_PUTPARM

PURPOSE: Write a TAE parameter to a project parameter file

PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Development
  1.1    D.Etrheim        11/94  EDC   Use c_lsmknm to get file name

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PUTPARM must run under TAE

PROJECT: LAS

*******************************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "util.h"


/**
 * c_putparm:
 * @filename: I: Hostname of the project parameter file.
 * @parmname: I: Parameter name to write to file.
 * @string_value: I: String values of the parameter.
 * @type: I: Type of parameter to write (I, R, S,).
 * @double_value: I: The double values of the parameter.
 * @dimension: I: Dimension of the parameter.
 * @long_value: The long values of the parameter.
 *
 * The c_putparm() routine writes a TAE parameter to a project parameter file.
 *
 * Returns:  E_SUCC:  Sccessful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_putparm
(
   const char *filename,            /* I: Hostname of the proj parameter file */
   const char *parmname,            /* I: Parameter name                      */
   const char string_value[][CMLEN],/* I: String values of the parameter      */
   const char *type,                /* I: Type of parameter to write (I, R, S)*/
   const double double_value[],     /* I: The double values of the parameter  */
   int dimension,                   /* I: Dimension of the parameter          */
   const int long_value[]           /* I: The long values of the parameter    */
)

{
    char pname[CMLEN];              /* Temporary parameter name for convert */
    char tmp_type[CMLEN];           /* Tempory type of parameter for convert*/
    char tmp_filename[CMLEN];       /* Actual file name with the .ppf ext   */
    int  i;                         /* Loop control                         */
    int  j;                         /* Loop control                         */

    FILE *parm_file;                /* File pointer to the proj parameter   */


    /* Make sure there is a positive dimension                              */
    if (dimension <= 0)
        return(E_SUCC);

    /* Add the .ppf extension to filename                                   */
    c_lsmknm(filename,"ppf",tmp_filename);      

    /* Open the project parameter file                                      */
    if ((parm_file = fopen(tmp_filename, "a")) == NULL)
    {
        c_lasmsg (FALSE, "c_putparm-open",
                  "Error opening project parameter file");
        return(E_FAIL);
    }

    /* Convert the parameter name and type to uppercase                     */
    strcpy (pname, parmname);
    las_low2up (pname);
    strcpy (tmp_type, type);
    las_low2up (tmp_type);

    /* Write the parameter to disk                                          */
    if (strcmp(tmp_type, "I") == 0)
    {
        fprintf(parm_file, "%-20s I %d", pname, dimension);
        for (i = 0; i < dimension; i++)
            fprintf(parm_file," %d", long_value[i]);
        fprintf(parm_file, "\n");
    }
    else if (strcmp(tmp_type, "R") == 0)
    {
        fprintf(parm_file, "%-20s R %d", pname, dimension);
        for (i = 0; i < dimension; i++)
            fprintf(parm_file," %f", double_value[i]);
        fprintf(parm_file, "\n");
    }
    else if (strcmp(tmp_type, "S") == 0)
    {
        fprintf(parm_file, "%-20s S %d", pname, dimension);
        for (i = 0; i < dimension; i++)
        {
            for (j = 0; j < strlen(string_value[i]); j++)
                if (isspace(string_value[i][j]))
                {
                    c_lasmsg (FALSE, "c_putparm-space",
                              "Strings cannot contain white space");
                    return(E_FAIL);
                }
            fprintf(parm_file," %s", string_value[i]);
        }
        fprintf(parm_file, "\n");
    }
    else
    {
        fclose(parm_file);
        c_lasmsg (FALSE, "c_putparm-type", "Invalid parameter type");
        return(E_FAIL);
    }

    fclose(parm_file);
    return(E_SUCC);
}
