/*******************************************************************************
NAME: c_read

PURPOSE:  Provides an implementation of the unix standard read function that 
          will convert data read to the local endian.

PROGRAM HISTORY:
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        T. Beckmann     11/00   Original development

ALGORITHM DESCRIPTION:
   Read the data using read
   Convert the data to the local endian

ALGORITHM REFERENCES:
   none
*******************************************************************************/
#include <unistd.h>     /* read prototype */
#include "las.h"        /* standard LAS defines */
#include "lasendian.h"  /* byte swapping routines */
#include "lasfileio.h"  /* prototype for this function */

/**
 * c_read:
 * @fd: I: File descriptor of the file to read.  This file should be opened
 *         by the standard unix open() call.
 * @buf: I/O: Pointer to a buffer for the data read from the file. The data is
 *            converted from big endian  byte order to the local byte order.
 * @count: I: Specifies the number of bytes to read from the file.
 * @size: I: Specifies the size, in bytes, of each element read from the file.
 *           This is limited to 1, 2, 4, or 8 bytes to match the size of
 *           standard data types.  The "count" parameter must be an even
 *           multiple of the "size" parameter.
 *
 * All data file for LAS are written using big endian byte order.  This is the
 * byte order used by Mips, Sparc, and Motorola processors.  The c_read() 
 * routine is the equivalent of the standard Unix read() routine, with the
 * added "size" parameter, that will perform byte swapping on platforms that
 * use little endian byte order.
 *
 * Returns: Number of items successfully read,
 *          E_FAIL:  Unsuccessful completion of operation, invalid
 *                   size or count parameters.
 */
ssize_t c_read
(
   int fd,                  /* I: file descriptor to read */
   void *buf,               /* I/O: pointer to buffer to read the data into */
   size_t count,            /* I: total number of bytes to read */
   size_t size              /* I: size of each element read so they can be
                                  byte swapped if needed */
)
{
    ssize_t items_read;  /* number of items read */

    /* make sure the size of each member is a power of two up to the number 
       of bytes in a 64-bit word and the count is an even multiple of size */
    if (((size > 1) && (size & 0x01)) || (size > 8) || (size == 6) || 
        ((count % size) != 0) || (count < size))
    {
        c_lasmsg(FALSE,"read-size","The combination of element count %d and "
                "size %d are not legal",count,size);
        return E_FAIL;
    }

    items_read = read(fd, buf, count);

    /* make sure read didn't return an error and that an even multiple of 
       elements have been read */
    if ((items_read < 1) || ((items_read % size) != 0))
        return E_FAIL;

    /* convert the data read to the local byte order */
    localendian_buf(buf, size, items_read/size);

    return items_read;
}
