/*******************************************************************************
NAME: c_write

PURPOSE:  Provides an implementation of the unix standard write function that 
          will convert data written to the export endian.

PROGRAM HISTORY:
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        T. Beckmann     11/00   Original development

ALGORITHM DESCRIPTION:
   Convert the data to the export endian
   Write the data using write

ALGORITHM REFERENCES:
   none
*******************************************************************************/
#include <string.h>     /* memcpy prototype */
#include <unistd.h>     /* write prototype */
#include "las.h"        /* standard LAS defines */
#include "lasendian.h"  /* byte swapping routines */
#include "lasfileio.h"  /* prototype for this function */

/**
 * c_read:
 * @fd: I: File descriptor of the file to be written.  This file should be 
 *         opened by the standard unix open() call.
 * @buf: I/O: Pointer to a buffer for the data written to the file. The data is
 *            converted to big endian byte order before it is written.
 * @count: I: Specifies the number of bytes to write to the file.
 * @size: I: Specifies the size, in bytes, of each element written to the file.
 *           This is limited to 1, 2, 4, or 8 bytes to match the size of
 *           standard data types.  The "count" parameter must be an even
 *           multiple of the "size" parameter.
 *
 * All data file for LAS are written using big endian byte order.  This is the
 * byte order used by Mips, Sparc, and Motorola processors.  The c_write()
 * routine is the equivalent of the standard Unix write() routine, with the
 * added "size" parameter, that will perform byte swapping on platforms that
 * use little endian byte order.
 *
 * Returns: Number of items successfully written,
 *          E_FAIL:  Unsuccessful completion of operation, invalid
 *                   size or count parameters.
 */
ssize_t c_write
(
    int fd,             /* I/O: file descriptor to write to */
    const void *buf,    /* I: pointer to the buffer of data to write */
    size_t count,       /* I: total number of bytes to write */
    size_t size         /* I: size of each element read so they can be
                              byte swapped if needed */
)
{
    /* make sure the size of each member is a power of two up to the number 
       of bytes in a 64-bit word and the count is an even multiple of size */
    if (((size > 1) && (size & 0x01)) || (size > 8) || (size == 6) ||
            ((count % size) != 0) || (count < size))
    {
        c_lasmsg(FALSE,"write-size","The combination of element count %d and "
                "size %d are not legal",count,size);
        return E_FAIL;
    }

#ifdef LAS_STANDARD_ENDIAN
    /* use the normal write function if standard endian machine */
    return (write(fd, buf, count));
#else
#define BUFSIZE 4096
    if (size == 1)
    {
        /* single byte data, so no byte swapping needed */
        return (write(fd, buf, count));
    }
    else
    {
        unsigned char swapbuf[BUFSIZE];
        size_t index;
        size_t total_elements = count/size;
        size_t elements = BUFSIZE/size;
        ssize_t bytes_written = 0;

        /* convert the data read to the export byte order */
        for (index = 0; index < total_elements; index += elements)
        {
            /* calculate the elements in the last sub-buffer */
            if ((index + elements) > total_elements)
                elements = total_elements - index;

            /* copy the buffer since the original buffer of data shouldn't be 
               modified, then swap the bytes */
            memcpy(swapbuf, &((char *)buf)[index * size], elements * size);
            exportendian_buf(swapbuf, size, elements);

            /* write the data */
            bytes_written = write(fd, swapbuf, elements * size);

            /* check for an error returned from the write */
            if (bytes_written < 0)
                return (E_FAIL);

            if (bytes_written != elements * size)
            {
                return index * size + bytes_written;
            }
        }

        return count;
    }
#endif

}
