/******************************************************************************
NAME:   logit

FUNCTION:
        Logs the program name into a log file.  On VAX/VMS it references the
        logical name APPLOG which must be in the process table.  On unix 
        it references the environment variable APPLOG.  The log file entry 
        consists of the date, time, user code, and program name.

PROGRAM HISTORY:
  Version       Date    Author          Request
  -------       ----    ------          -------
    1            --     T. Butzer       original development
    2           11/87   K. Gacke        rename the log file after reaching a
                                        max file size.  Added the APPLOG logical
                                        name (or environment variable).
    3           12/87   K. Zanter       changed DESC argument to be a pointer
                                        to a character string
    4            6/90   P. Pease        removed declaration of time to
                                        eliminate warning that its declaration
                                        conflicts with a previous declaration
    5.          07/90   D. VanderZee    Standardized error handling
    5.1         08/94   D. Etrheim      Corrected arguments to size_check
    5.2         08/94   D. Etrheim      Change 777 to 666 on file open
    5.3         10/98   D. Hames        Add code to comply
                                        with localtime & y2k


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

ALGORITHM:
        Translate the APPLOG logical name or environment variable into a
        text string and append the "applog.log" file name.  Open the log
        file and check the current size.  If the file size is greater than
        the constant MAXFSIZ, the log file is renamed with the date appended
        and function logit is called recursively.  Otherwise the program is
        logged.  The log file entry consists of the date, time, user code,
        and program name.

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include "desc.h"
#include "las.h"
#include "util.h"

#define MAXFSIZ 1024 * 500      /* new file after 500 K bytes */
#define FNAME   "applog.log"

static void size_check
(
    const char *progname,
    char *lasfile,
    FILE *fp,
    struct tm *tm_ptr 
)
{
    char newnam[256];

    fseek(fp,0,2);
    if (ftell(fp) > MAXFSIZ)
    {
        fclose(fp);
        sprintf(newnam,"%s_%2d%02d%02d",lasfile,tm_ptr->tm_year%100,
                tm_ptr->tm_mon+1,tm_ptr->tm_mday);
        rename(lasfile,newnam);
        logit(progname);
    }
}

/**
 * logit:
 * @progname: I: Program name to be logged.
 *
 * The logit() routine logs the program name into a log file.  On unix it
 * references the environment variable APPLOG.  The log file entry consists
 * of the date, time, user code, and program name.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int logit
(
    const char *progname    /* I:  Program name to be logged                */
)
{
    time_t my_time;
    struct tm  *tm_ptr;
    char username[100];
    char lasfile[256];
    char env_val[256];
    char *ptr;
    FILE *fp = NULL;
    int  count;
    int  year;

    /* Translate the APPLOG logical name or environment variable into a
       text string and append the "applog.log" file name.                   */
    if ((ptr = getenv("APPLOG")) == NULL)
    {
        c_lasmsg (FALSE, "logit-retrieve", "Error retrieving value of APPLOG");
        return(E_FAIL);
    }
    strcpy(env_val,ptr);
    sprintf(lasfile,"%s/%s",env_val,FNAME);

    my_time=time(0);
    tm_ptr=localtime(&my_time); 

    /* Open the log file and check the current size.  If the file size 
       is greater than the constant MAXFSIZ, the log file is renamed 
       with the date appended and function logit is called recursively.  
       Otherwise the program is logged.                                     */
    for (count=0; count<10; count++)
    {
        fp = fopen(lasfile, "a");
        if (fp != NULL)
            break;
        sleep(1);
    }

    if (count >= 10)
    {
        c_lasmsg (FALSE, "logit-open", "Error opening log file");
        return(E_FAIL);
    }
    chmod(lasfile,0666);

    size_check(progname,lasfile,fp,tm_ptr);
    getusern(username);

    /* The log file entry consists of the date, time, user code, and 
       program name.                                                        */

    year = tm_ptr->tm_year; 
    /* used to retrieve 2 year digits for dates after the year 2000         */
    year = year%100;

    fprintf(fp,"%2d/%02d/%02d %02d:%02d:%02d    "
            ,tm_ptr->tm_mon+1,tm_ptr->tm_mday,year
            ,tm_ptr->tm_hour,tm_ptr->tm_min,tm_ptr->tm_sec);
    fprintf(fp,"USER = %-12s PROC = %-12s\n",username, progname);

    fclose(fp);
    return(E_SUCC);
}



