
/*******************************************************************************
NAME:     image.c 
PURPOSE:  Simplify image IO for application developers by providing a data
          structure driven image IO implementation.  Multiple processing modes
          are supported, no global variables are required, full LAS
          functionality is assured.  Application developers may select some of
          these IO routines;  a few applications require simple revisions to
          these IO routines.

SOURCE CODE HISTORY
--------------------------------------------------------------------------------
  Date       Programmer                          Changes 
---------  ---------------  ----------------------------------------------------
 Dec 1992  David Zokaites   Developed imageio application:  functions and data
                            structures for sequential IO.
 Feb 1993  David Zokaites   Enabled random IO by writing openrin(), openrout(),
			    rread(), rwrite(), rimage();  created global vars
			    for easier error messages;  wrote minmax().
 May 1993  David Zokaites   Enabled block IO by writing bread(), bwrite(),
                            bimage();  wrote make_ddr().
 Jul 1993  David Zokaites   Renamed application from imageio to template;
			    eliminated functions getin2() and getout2();  wrote
			    help for template.
 Aug 1993  David Zokaites   Allowed for data conversion upon read and write;
                            wrote convert[srb]().
 May 1994  David Zokaites   Wrote openbin(), openbout(), minmax[srb](),
			    parse_file();  expanded help in template.pdf;
			    removed make.com;  added tae parms iomode,
			    readtype, procmeth, writconv.
 Aug 1994  David Zokaites   Prototyped all las support functions via
			    makep.{c,csh};  changed variable names to keep with
			    las conventions;  put errclean() in a separate file
			    to minimize global vars;  wrote Lintfile to run
			    lint;  eliminated functions real_parm(),
			    string_parm(), integer_parm(), parse_file();
			    renamed global data structures for las images;
			    wrote functions open[sr]out2().
 Nov 1994  David Zokaites   Improved error messages;  renamed ddr_his() to
                            finish_outi2().
 Jan 1995  David Zokaites   Enabled errclean() to process a variable number of
			    arguments;  wrote function errmsg();  (errmsg later
			    became c_lasmsg which later became a support);
			    reduced # of global vars, deleted begin_las();
			    reconstructed program history;  updated prototypes
			    of las support functions.
 Feb 1995  David Zokaites   Updated Makefile format.
 Jun 1995  David Zokaites   Tweaked error messages, called c_lasmsg() instead 
                            of c_errmsg() or errmsg(). 
 Jul 1995  David Zokaites   For fatal errors, errclean() can now be called from 
			    any function;  added status flags to image data
			    structures;  pblock is no longer global;
			    open[sr]out() now just call open[sr]out2();  changed
			    parms to open[sr]out2();  changed indenting per las
			    conventions;  renamed make_image.c to proces.c;
			    deleted c_lasmsg.c as this function is now a
			    support;  changed data structure and function names;
			    wrote read_bil() and write_bil(). 
 Aug 1995  David Zokaites   Defined macros for image data sizes;  used random
			    supports instead of sequential for BIL processing;
			    combined functions to open and close images;
			    deleted tae parm writconv;  simplified error 
			    messages to keep with las convention;  changed 
			    indentation per las convention;  replaced
			    if "c_function()" with if "c_function()!=E_SUCC".
 Oct 1995  David Zokaites   Added struct OUTINFO to simplify programmer
                            interface to open_outimage(), added functions
			    in_flags(), out_flags(). 
 Oct 1995  David Zokaites   Deleted functions not needed by CLRWHEEL. 
*******************************************************************************/


#include <math.h>      /* for call to ceil() */ 
#include <string.h>    /* for call to strcmp() */ 
#include <stdlib.h>    /* for call to malloc(), free() */ 
#include "cwheel.h"    /* application header file */
#include "util.h"
#include "pixman.h"


/******************************************************************************
NAME:     out_flags()
PURPOSE:  Initialize status flags for an output image 
******************************************************************************/
FUNCTION void out_flags (outimage)
   struct OUTIMAGE *outimage;   /* output: initialized flags */ 
{
   outimage->opened = FALSE;  /* the image has not been opened */ 
   outimage->closed = FALSE;  /* the image has not been closed */ 
   /* delflg is set in the function that opens the image */ 
}






/*******************************************************************************
NAME:     open_outimage
PURPOSE:  general function to open an output image and allocate data buffers,
          does not rely on an input image 
*******************************************************************************/

FUNCTION void open_outimage (pblock, outinfo, outimage)
   struct PARBLK *pblock;     /* input:  tae parameter block */ 
   struct OUTINFO *outinfo;   /* input:  processing parameters for outimage */ 
   struct OUTIMAGE *outimage; /* output: opened output image */
{

/* declare variables */ 
   char *odtype;            /* tae parm odtype, output data type */ 
   long accessm=IWRITE,     /* image access mode, parm to c_eopenr() */ 
	bandnum,            /* loop index, image band */ 
	linenum,            /* loop index, line in a block */ 
        one=1,              /* value of one */ 
	opt[2];             /* optional parm to c_eopenr(), used for data 
			       type conversion */ 
   struct VARIABLE *tparm;  /* points to a tae parm in pblock */ 

/* get tae globals:  $delflg, $minmax, $msgtim */ 
   /* delflg */ 
      tparm = p_find (pblock, "$delflg");   
      if ( !strcmp (SVAL(*tparm,0), "YES"))
	 outimage->delflg = TRUE;
      else
	 outimage->delflg = FALSE;
   /* minmax */ 
      tparm = p_find (pblock, "$minmax");   
      if ( !strcmp (SVAL(*tparm,0), "YES"))
	 outimage->minmax = TRUE;
      else 
	 outimage->minmax = FALSE;
   /* msgtim */ 
      tparm = p_find (pblock, "$msgtim");   
      outimage->msgtim = IVAL (*tparm, 0); 

/* get output image name */ 
   if (outinfo->parmname[0] != '\0')
      {
      if (c_getout (pblock, outinfo->parmname, outimage->taename[0], 
		   outimage->hostname[0]) !=E_SUCC)
	 errclean ("cwheel-fatal", "Fatal error encountered"); 
      }
   else  /* copy hostname and get tae name */ 
      {
      strcpy (outimage->hostname[0], outinfo->hostname);
      strcpy(outimage->taename[0], outimage->hostname[0]);
      }

/* set dtypeout, output data type */ 
   if (outinfo->dtypeout == NOTAPPLY)
      {
      /* get tae parm odtype */ 
	 tparm = p_find (pblock, "odtype");    
	 odtype = SVAL(*tparm, 0); 
      /* error check:  odtype=same and dtypein==notapply */ 
	 if (!strcmp (odtype, "SAME") && outinfo->dtypein == NOTAPPLY)
	    errclean ("cwheel-fatal", 
	             "Program error:  ODTYPE=SAME but dtypein=NOTAPPLY"); 
      /* set dtypeout */ 
	 if      (!strcmp (odtype, "SAME")) outimage->dtypeout=outinfo->dtypein;
	 else if (!strcmp (odtype, "BYTE")) outimage->dtypeout=EBYTE; 
	 else if (!strcmp (odtype, "I*2"))  outimage->dtypeout=EWORD; 
	 else if (!strcmp (odtype, "I*4"))  outimage->dtypeout=ELONG; 
	 else if (!strcmp (odtype, "R*4"))  outimage->dtypeout=EREAL; 
	 else errclean ("cwheel-fatal", 
	               "Program error:  parm ODTYPE has an illegal value"); 
      } 
   else
      outimage->dtypeout = outinfo->dtypeout; 

/* set dtypeap, data type as processed by the application */ 
   if (outinfo->dtypeap == NOTAPPLY)
      outimage->dtypeap = outimage->dtypeout;
   else
      outimage->dtypeap = outinfo->dtypeap; 

/* set dsizeap, data size as processed by the application */ 
   switch (outimage->dtypeap)
      {
      case EBYTE:  outimage->dsizeap=EBYTESIZE;  break; 
      case EWORD:  outimage->dsizeap=EWORDSIZE;  break; 
      case ELONG:  outimage->dsizeap=ELONGSIZE;  break; 
      case EREAL:  outimage->dsizeap=EREALSIZE;  break; 
      default:  errclean ("cwheel-fatal", 
		         "Program error:  unknown image data type");
      }

/* set variables in data structure */ 
   outimage->nl = outinfo->nl; 
   outimage->ns = outinfo->ns; 
   outimage->totalbands = outinfo->nbands; 
   outimage->iomode = outinfo->iomode; 
   outimage->truncate = FALSE; /* output data was tentatively not truncated */ 

/* optionally set misc variables for block processing */ 
   if (outimage->iomode == BLOCKIO)
      {
      outimage->blocksize = outinfo->blocksize;
      outimage->lblocks = (long) ceil ( (double)outimage->nl /
			  outimage->blocksize );
      outimage->sblocks = (long) ceil ( (double)outimage->ns /
			  outimage->blocksize );
      outimage->blocknl = outimage->blocksize * outimage->lblocks;
      outimage->blockns = outimage->blocksize * outimage->sblocks;
      outimage->fillns  = outimage->blockns - outimage->ns;
      }

/* open image files after setting opt to allow for data conversion on output */ 
   opt[0] = opt[1] = 0;  /* initialize array */ 
   if (outimage->dtypeap != outimage->dtypeout)
      opt[0] = outimage->dtypeout; 

   if (c_eopenr (outimage->fdarray, outimage->hostname[0], 
		&outimage->totalbands, &one, &one, &outimage->nl, &outimage->ns,
		&accessm, &outimage->dtypeap, opt, &one) !=E_SUCC)
      errclean ("cwheel-fatal", "Fatal error encountered"); 

/* allocate memory for image buffers */ 
   switch (outimage->iomode)
      {
      case BSQIO:  
	 /* buffer holds one line for one band */ 
	 if (( outimage->buffer = (unsigned char *) malloc 
	 (outimage->ns * outimage->dsizeap) ) == NULL)
	    errclean ("cwheel-memory", 
	             "Unable to allocate memory for input image buffer"); 
	 break; 

      case BILIO:  
	 /* linebuf holds one line for each band */ 
	 for (bandnum=0;  bandnum<outimage->totalbands;  bandnum++)
	    if (( outimage->linebuf[bandnum] = (unsigned char *) malloc 
	    (outimage->ns * outimage->dsizeap) ) == NULL)
	       errclean ("cwheel-memory", 
	                "Unable to allocate memory for input image buffer"); 
	    break; 

      case BLOCKIO:  
	 /* buffer holds a block of lines for one band */ 
	    if ((outimage->buffer = (unsigned char *) malloc 
	    (outimage->blocksize * outimage->blockns * outimage->dsizeap) ) 
	    ==NULL)
	       errclean ("cwheel-memory", 
	                "Unable to allocate memory for input image buffer"); 
	 /* bufloc points to each line in the block */ 
	    if ((outimage->bufloc = (unsigned char **) 
	    malloc ( sizeof(unsigned char *) * outimage->blocksize)) == NULL)
	       errclean ("cwheel-memory", 
	                "Unable to allocate memory for bufloc variable"); 
	    for (linenum=0;  linenum<outimage->blocksize;  linenum++)
	       outimage->bufloc[linenum] = outimage->buffer + 
	       linenum * outimage->dsizeap * outimage->blockns; 
	 break; 

      default:  
	 errclean ("cwheel-fatal", 
	          "Program error:  iomode has a bad value in open_in()"); 
      }

/* initialize min max arrays */ 
   if (outimage->minmax && 
   c_inmnmx (&outimage->dtypeap, &outimage->totalbands, 
            outimage->minval, outimage->maxval) !=E_SUCC)
      errclean ("cwheel-fatal", "Fatal error encountered"); 

/* set flag, image was opened */ 
   outimage->opened = TRUE; 
}





/*******************************************************************************
NAME:     write_bil 
PURPOSE:  write data to disk image, using BIL IO  
*******************************************************************************/

FUNCTION void write_bil (curline, outimage)
   long curline;           /* input:  starting line of block of data to write, 
			              0 relative */ 
   struct OUTIMAGE *outimage;  /* input:  data structure of image to write */
{

/* declare variable */ 
   long bandnum,  /* loop index, image band */ 
	curband,  /* current band relative to c_ewrite() */ 
        one=1;    /* n lines to write at a time */ 

/* convert curline from 0 relative to 1 relative */ 
   curline++; 

/* loop for all bands */ 
   for (bandnum=0;  bandnum<outimage->totalbands;  bandnum++)
      {

      /* convert bandnum from 0 relative to 1 relative */ 
	 curband = bandnum + 1; 

      /* write data to disk */ 
	 if (c_ewrite (outimage->fdarray, &curband, &curline, 
	              outimage->linebuf[bandnum], &one) != outimage->ns)
	    errclean ("cwheel-fatal", "Fatal error encountered"); 
      }
}




/*******************************************************************************
NAME:     minmax_bil 
PURPOSE:  optionally update minmax for a line of image data for BIL IO
*******************************************************************************/

FUNCTION void minmax_bil (outimage)
   struct OUTIMAGE *outimage; /* input and output:  structure of image data to 
			     update min/max for */ 
{

/* declare variable */ 
   long band;              /* loop index to image bands */ 

/* return if min/max does not need updated */ 
   if (!outimage->minmax)
      return; 

/* update minmax */ 
   for (band=0;  band<outimage->totalbands;  band++)
      if (c_pxmnmx (outimage->linebuf[band], &outimage->ns, 
                   outimage->minval+band, outimage->maxval+band, 
		   &outimage->dtypeap) !=E_SUCC)
	 errclean ("cwheel-fatal", "Fatal error encountered"); 
}





/*******************************************************************************
NAME:     close_outimage 
PURPOSE:  close output image files and free buffers  
*******************************************************************************/

FUNCTION void close_outimage (outimage)
   struct OUTIMAGE *outimage;   /* input:  close these files */ 
{

/* declare variable */ 
   long bandnum;  /* loop index, image band */ 

/* close image file, close file descriptor */ 
   (void) c_eclose (outimage->fdarray);

/* free buffers */ 
   switch (outimage->iomode)
      {
      case BSQIO:    free (outimage->buffer); 
		     break;  
      case BILIO:    for (bandnum=0;  bandnum<outimage->totalbands;  bandnum++)
		        free (outimage->linebuf[bandnum]); 
		     break;
      case BLOCKIO:  free (outimage->buffer); 
		     free (outimage->bufloc);
		     break;
      }

/* set status flag, image was closed */ 
   outimage->closed = TRUE; 
}





/*******************************************************************************
NAME:     finish_out2
PURPOSE:  finalize processing of output image:  create ddr and history file.  
	  If there are input images, call finish_out() instead of 
	  finish_out2().  The former puts valid projection information into 
	  the image ddr while the latter does not.
*******************************************************************************/

FUNCTION void finish_out2 (pblock, outimage)
   struct PARBLK *pblock;   /* input:  tae parameter block */ 
   struct OUTIMAGE *outimage;   /* input:  specs for output image */ 
{

/* declare variables */ 
   char notused[1][CMLEN];  /* used for dynamic tutor sessions, not applicable
                               here, required for c_hstout() */
   long bandnum,            /* loop index, band number */ 
        zero=0;             /* value of zero */ 
   struct BDDR bddr;        /* band-dependent record of ddr */ 
   struct DDR ddr;          /* image ddr */ 

/* don't check for data type truncation, 
   if there is no input image, data won't be converted. */ 

/* create ddr */ 
   c_intddr (&ddr);               /* initialize ddr including last used fields*/
   ddr.nl     = outimage->nl;         /* number of lines */ 
   ddr.ns     = outimage->ns;         /* number of samples */  
   ddr.nbands = outimage->totalbands; /* number of bands */ 
   ddr.dtype  = outimage->dtypeout;   /* data type */ 
   ddr.master_line   =1;          /* line relative to master image */
   ddr.master_sample =1;          /* sample relative to master image */
   ddr.line_inc      =1.0;        /* line increment for sampling */
   ddr.sample_inc    =1.0;        /* sample increment for sampling */
   ddr.valid[DDINCV] = VALID;     /* line/sample increments validity flag */
   if (c_getsys (ddr.system)!=E_SUCC || c_putddr(outimage->hostname[0],&ddr) 
   !=E_SUCC) 
      errclean ("cwheel-fatal", "Fatal error encountered"); 

/* create bddr's and update with band number */ 
   c_intbdr (&bddr);          /* initialize bddr */ 
   for (bandnum=1; bandnum<=outimage->totalbands; bandnum++)
      if (bddr.bandno=bandnum,  c_putbdr(outimage->hostname[0],&bddr) !=E_SUCC)
	 errclean ("cwheel-fatal", "Fatal error encountered"); 

/* update ddr with min max, must come after writing ddr to disk */ 
   if (outimage->minmax && 
   c_svmnmx (outimage->hostname[0], &outimage->totalbands, outimage->minval, 
	    outimage->maxval, "YES") !=E_SUCC)
      errclean ("cwheel-fatal", "Fatal error encountered"); 

/* create history file */ 
   if (c_hstout (pblock, outimage->hostname[0], notused, &zero, "") !=E_SUCC)
      errclean ("cwheel-fatal", "Fatal error encountered"); 
}




