
/*******************************************************************************
NAME:     image.h
PURPOSE:  header file for image IO 
	  includes data structures for LAS images, 
	  symbolic constants and function prototypes
*******************************************************************************/


/* macro:  use this value when a function parameter does not apply.  
   must be 0 for the supports*/
#define NOTAPPLY 0 


/* define macros for image data sizes, macros really belong in worgen.h */
#define EBYTESIZE (sizeof(unsigned char))
#define EWORDSIZE (sizeof(short))
#define ELONGSIZE (sizeof(long))
#define EREALSIZE (sizeof(float))



/* types of image IO allowed for, they all use LAS random IO.  random supports
   seem to work a little better than sequential supports */ 
   enum iomodes 
      {
      BSQIO,   /* BSQ processing */ 
      BILIO,   /* BIL processing */ 
      BLOCKIO  /* in blocks      */ 
      };



/* a note on data types: 
   An image can potentially have 3 different data types:
   dtypein:  data type as exists in input disk image
   dtypeap:  data type as processed internally by the application
   dtypeout:  data type as will exist in output disk image.  Data types
   can be EBYTE, EWORD, ELONG, EREAL. */ 



/* data structure for one las output image, used to facilitate IO, 
   structure includes some tae globals */ 
struct OUTIMAGE
{
      char hostname[1][CMLEN],     /* host name for image file, supporting
                                      routines expect a 2-d array */ 
           taename[1][CMLEN];      /* tae name for image, supporting 
				      routines expect a 2-d array */
      struct FDESC *fdarray[1+1];           /* file descriptor array */ 
      long blocksize,              /* size of blocks for BLOCKIO */ 
	   blocknl, blockns,       /* image size after expansion for blocking,
				      for BLOCKIO */
 	   dsizeap,                /* # bytes per pixel as seen by the app */
 	   dtypeap,                /* image data type as seen by the app */ 
 	   dtypeout,               /* image data type to write = EBYTE, ... */
	   fillns,                 /* ns needed to fill last block, for 
				      BLOCKIO */
           iomode,                 /* mode of io, = BILIO, BSQIO, BLOCKIO */ 
           lblocks,                /* number of blocks in line direction, 
				      for BLOCKIO */
           minmax,                 /* tae global $minmax, flag, determine 
				      min/max for ddr? = T,F */
	   msgtim,                 /* tae global $msgtim, time interval for
	                              processing message */ 
           nl, ns,                 /* size of output image */ 
           sblocks,                /* number of blocks in sample direction, for
				      BLOCKIO */ 
           totalbands,             /* total number of bands */ 
	   truncate;               /* flag, was the output data truncated to
	                              fit in the allowed range, used only if
				      c_pxconv() is called */ 
      unsigned char **bufloc,      /* location in data buffer for each line of
	                              data, for BLOCKIO */ 
	       *buffer,            /* holds one image line for BSQIO */ 
	       *linebuf[MAXBND];   /* holds one line for each band.  for BILIO 
	                              and BLOCKIO */ 
      double minval[MAXBND],       /* min pixel value, used to update ddr */ 
             maxval[MAXBND];       /* max pixel value, used to update ddr */ 
      /* status flags used for error handing, = true/false */ 
      long closed,                 /* has the image been closed */ 
	   delflg,                 /* tae global $delflg, delete images if err*/
	   opened;                 /* has the image been opened */
}; 



/* data structure giving processing parameters for output image, 
   simplifies the programmer interface for open_outimage() */ 
   struct OUTINFO 
      {
      char hostname[CMLEN], /* host name for output image, use "" if image 
			       name is to be found in pblock */ 
           *parmname; /* tae parameter name for output image, used to get name
                         in pblock */ 
      long dtypein,   /* data type of input image, use NOTAPPLY if there is 
			 no input image */ 
           dtypeap,   /* output image data type as processed by the application,
			 use NOTAPPLY if the app handles data type conversion 
			 and conversion does not occur upon writing to disk */ 
           dtypeout,  /* image data type as output to disk image.  use
			 NOTAPPLY if dtypeout should be based on parm odtype */ 
           iomode,    /* mode of io. = BSQIO, BILIO, or BLOCKIO */ 
           blocksize, /* size of blocks for BLOCKIO.  ignored if not using 
			 block io */ 
           nl,        /* nl for output image */ 
           ns,        /* ns for output image */ 
           nbands;    /* nbands for output image */ 
      };



/* function prototypes.  ifdef allows for old compilers which do not support 
   prototype checking */ 
   /* function in errclean.c */ 
void errclean (char *mkey, char *format, ...); 
   /* functions in image.c */
void out_flags (struct OUTIMAGE *outimage);
void open_outimage (struct PARBLK *pblock, struct OUTINFO *outinfo, 
	   struct OUTIMAGE *outimage); 
void write_bil (long curline, struct OUTIMAGE *outimage);
void minmax_bil (struct OUTIMAGE *outimage);
void close_outimage (struct OUTIMAGE *outimage);
void finish_out2 (struct PARBLK *pblock, struct OUTIMAGE *outimage);
