
/******************************************************************************
NAME:     iparms.c
PURPOSE:  functions to determine image generation parameters 
          most of these functions are only called if a chip (not a smooth) 
	  image is created
******************************************************************************/
#include "cwheel.h"  /* application header file */ 
#include <math.h>    /* for various trig functions and M_PI macro */ 
#include <stdio.h>   /* for file io */ 





/******************************************************************************
NAME:     image_parms() 
PURPOSE:  Determine parameters of image generation.
          Some parameters were set in getpar().
******************************************************************************/
FUNCTION void image_parms (uparms, igen)
   struct UPARMS *uparms;  /* input:   user tae parameters */ 
   struct IGEN   *igen;    /* output:  image generation parameters */ 
{
/* declare variables */ 
   double scalar;   /* a scalar used to set ringsat */ 
   long ringnum;    /* loop index, color ring number */ 

/* convert intensities to range needed by rgb2his() */ 
   igen->backints  = uparms->backints * MAXINTS;
   igen->bdrints   = uparms->bdrints  * MAXINTS;
   igen->wheelints = uparms->intens   * MAXINTS;

/* set wheelmaxrad, max radius for color wheel */ 
   if (igen->chipsflag)
      {
      if (igen->nrings)
	 igen->wheelmaxrad = igen->rwidth / 2.0    /* for central ring */
	 + igen->rwidth * (igen->nrings -1.0);     /* for  outer rings */
      else
	 igen->wheelmaxrad = 0; 
      }
   else
      igen->wheelmaxrad = uparms->wwidth / 2.0; 

/* set bdrmaxrad, max radius for color wheel border */ 
   igen->bdrmaxrad = igen->wheelmaxrad + uparms->bdrwidth;

/* calculate image center, 0 relative */ 
   igen->center = ( (double)igen->nlns - 1.0) /2.0; 

/* set ringlim lut.  index is ring number, lut value is the max radius for that 
   color ring */ 
   if (igen->chipsflag)
      for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
	 igen->ringlim[ringnum] = igen->rwidth / 2.0
				+ igen->rwidth * ringnum;  

/* set ringsat lut.  index is ring number, lut value is saturation for that 
   color ring.  saturation is 0 for the central ring, MAXSAT for the outer 
   ring */ 

   if (igen->chipsflag)
      {
	 if (igen->nrings > 1)
	    {
	    scalar = MAXSAT / (igen->nrings -1.0); 
	    for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
	       igen->ringsat[ringnum] = scalar * ringnum; 
	    }
	 else if (igen->nrings == 1) /* only one ring, saturation is zero */ 
	    igen->ringsat[0] = 0; 
      }
}






/******************************************************************************
NAME:     set_r2c()
PURPOSE:  set ring2chip  , number of chips in each color ring  
          set ring2chipp1, number of chips in each color ring plus 1 
******************************************************************************/
FUNCTION void set_r2c (uparms, igen)
   struct UPARMS *uparms;  /* input:   user tae parameters */ 
   struct IGEN   *igen;    /* output:  image generation paramters, 
				       ring2chip and ring2chipp1 are updated */ 
{
/* declare variables */ 
   long ringnum;  /* loop index, ring number */ 
   double aarg,   /* argument to asin() */ 
	  nchips; /* intermediate value for ring2chip */ 

/* calculate ring2chip.  derivation is at the bottom of this file */ 
   igen->ring2chip[0] = 1; /* 1 grey chip will be created at the central ring */
   for (ringnum=1;  ringnum<igen->nrings;  ringnum++)
      {
      /* basic calculation of nchips */ 
	 aarg = uparms->aspect / (2.0 * ringnum);
	 if (aarg > 1.0)   /* would result in a domain error for asin() */ 
	    nchips = 0;
	 else 
	    nchips = M_PI / asin (uparms->aspect / (2.0 * ringnum));

      /* quantize nchips */ 
	 if      (nchips +.333 > PRIHUES)
	    igen->ring2chip[ringnum] 
	    = (  (long) ((float)nchips/PRIHUES   +.333)  ) * PRIHUES; 
	 else if (nchips +.333 > PRIMARIES)
	    igen->ring2chip[ringnum] 
	    = (  (long) ((float)nchips/PRIMARIES +.333)  ) * PRIMARIES; 
	 else
	    igen->ring2chip[ringnum] = PRIMARIES; 
      } 

/* set ring2chipp1 */ 
   for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
      igen->ring2chipp1[ringnum] = igen->ring2chip[ringnum] + 1; 

/* set totalchips, total number of chips */ 
   for (igen->totalchips=0, ringnum=0;  ringnum<igen->nrings;  ringnum++)
      igen->totalchips += igen->ring2chip[ringnum]; 

/* user message */ 
   c_lasmsg (FALSE, "informational", 
   "OUT will have %d color chips", igen->totalchips); 

/* dump chips per ring to a file for debugging */ 
#ifdef DEBUG 
   {
      /* open file */ 
	 FILE *filep;
	 long totalchips=0;  
	 if ((filep = fopen ("debug.dat", "w")) == NULL)
	    errclean ("cwheel-debug", "Unable to open file debug.dat"); 
      /* table heading */ 
	 fprintf (filep, "Ring    Chips  Total \n"); 
	 fprintf (filep, "Number         Chips \n"); 
	 fprintf (filep, "------  -----  ----- \n"); 
      /* print table */ 
	 for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
	    {
	    totalchips += igen->ring2chip[ringnum];
	    fprintf (filep, "%4d %7d %6d\n", ringnum, igen->ring2chip[ringnum], 
	    totalchips);
	    }
      /* finish up */ 
	 fclose (filep); 
	 c_lasmsg (FALSE, "cwheel-debug", 
	 "Created file debug.dat listing chips per ring"); 
   } 
#endif 
}





/******************************************************************************
NAME:     set_ah()
PURPOSE:  set anglelim, max angle (in radians) for each color chip.
	  set huelut,   hue for each color chip. 
******************************************************************************/
FUNCTION void set_ah (igen)
   struct IGEN *igen; /* output:  image generation parameters,
				  anglelim and huelut are updated */
{
/* declare variables */ 
   double hueperchip, /* hues per chip, hues scaled for his2rgb() */ 
	  radperchip; /* radians per chip */ 
   long chipnum,      /* loop index, color chip */ 
        ringnum;      /* loop index, color ring */ 

/* calculate anglelim and hue lut.  anglelim has factor of .5 in calculation to 
   rotate color chips so that the first chip is centered at an axis instead of 
   starting at an axis */ 

   for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
      {
      hueperchip = MAXHUE       / igen->ring2chip[ringnum]; 
      radperchip = (2.0 * M_PI) / igen->ring2chip[ringnum]; 

      for (chipnum=0;  chipnum<igen->ring2chip[ringnum];  chipnum++)
	 {
	 igen->anglelim[ringnum][chipnum] = radperchip * (chipnum + 0.5); 
	 igen->huelut  [ringnum][chipnum] = hueperchip *  chipnum       ;
	 }

      /* The first chip is centered vertically, instead of starting at the
         vertical axis.  To prevent this first chip from being divided into two
	 chips, anglelim and huelut have one extra value.  The last value for
	 huelut = the first value;  last value for anglelim is set high.
	 When determining chip number from angle in hue_sat(), I test for
	 angle < anglelim.  Increase the last anglelim so that the last few
	 angles will always be less than anglelim */ 

	 igen->huelut  [ringnum][chipnum] = igen->huelut  [ringnum][0]; 
	 igen->anglelim[ringnum][chipnum] = igen->anglelim[ringnum][chipnum-1] 
					    * 2;  
      }

/* dump anglelim and huelut to a file for debugging */ 
#ifdef DEBUG 
   {
      /* open file */ 
	 FILE *filep;
	 if ((filep = fopen ("debug.dat", "a")) == NULL)
	    errclean ("cwheel-debug", "Unable to open file debug.dat"); 
      /* table heading */ 
	 fprintf (filep, "Ring  Chip  Anglelim  Huelut \n"); 
	 fprintf (filep, "----  ----  --------  ------ \n"); 
      /* print table */ 
	 for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
	    {
	    for (chipnum=0;  chipnum<igen->ring2chipp1[ringnum];  chipnum++)
	       fprintf (filep, "%3d %5d   %6.4f   %6.1f \n", 
	       ringnum, chipnum, igen->anglelim[ringnum][chipnum],
	       igen->huelut[ringnum][chipnum]); 
	    fprintf (filep, "\n");
	    }
      /* finish up */ 
	 fclose (filep); 
	 c_lasmsg (FALSE, "cwheel-debug", 
	 "Appended to file debug.dat to list anglelim and huelut"); 
   }
#endif 
}














#ifdef derivation_for_nchips

   On a particular ring of chips, the angle subtended by each chip is given by

      1) angle = 2 * pi 
		 ------
		 nchips 

   If each chip is divided in half vertically, then the chip sections form a 
   right triangle, the base angle of which is 

      2) angle = 2 * pi 
		 ----------
		 2 * nchips 

      or 

      3) angle = pi 
		 ------
		 nchips 


   The hypotenuse of this right triangle is given by the zero-relative formula

      4) hypotenuse = ringnumber * ringwidth 

   Aspect ratio for a chip is given by:

      5) aspect = width 
		  -----
		  height 

   Since the height of a chip = ringwidth, 

      6) aspect = width 
		  -----
		  ringwidth 

   Rearranging terms in 6) gives

      7) width = aspect * ringwidth 

   For right triangles, 

      8) sin (angle) = opposite 
		       ---------
		       hypotenuse 

   The opposite side is half the width of the chip and substituting 4) gives

      9) sin (angle) = width/2 
		       ----------------------
                       ringnumber * ringwidth

   Substituting 7) for width gives

      10) sin (angle) = aspect * ringwidth/2 
			----------------------
			ringnumber * ringwidth

   Cancelling out ringwidth and rearranging gives

      11) sin (angle) = aspect 
			--------------
			2 * ringnumber

   Substituting in 3) for angle gives

      12) sin (pi/nchips) = aspect 
			    --------------
			    2 * ringnumber

   Solving for nchips ...

      13) pi/nchips = asin (aspect / (2 * ringnumber)) 

      14) nchips = pi 
		   --------------------------------
		   asin (aspect / (2 * ringnumber)) 

   Equation 14) is the main equation for nchips.  However, nchips must be
   quantized to a multiple of PRIHUES or PRIMARIES.  Chips that are too small
   then to visually blend together, hence quantization favors making chips
   bigger.  This favoring means I use a factor of .333 instead of .5 for
   round-off adjustment.  The resulting round off formulas are in the code. 
#endif 


