
/******************************************************************************
NAME:     make_pdf.c 
PURPOSE:  functions to make a pdf that annotates brightness values in the 
          color chips 
******************************************************************************/
#include <math.h>    /* for M_PI macro */ 
#include <stdio.h>   /* for file io */ 
#include <stdlib.h>  /* for calling malloc() */ 
#include "cwheel.h"  /* application header file */ 



/* data structure for info on calls to annotate-inplace */ 
struct ANTEXT 
{
    int *bv,         /* brightness value to annotate */ 
        *sl,         /* starting line for text */ 
        *ss,         /* starting sample for text */ 
        *chargrey,   /* character grey value for text */ 
         scalfact,   /* annotation scale factor */ 
         nstrings,   /* number of strings to annotate */ 
         curstring,  /* current string being annotated */ 
         curcall,    /* current call to annotate */ 
         totalcalls; /* total number of calls to annotate */ 
};




/* prototype local functions */ 
static void antext_alloc (struct ANTEXT *antext); 
static void antext_free (struct ANTEXT *antext); 
static void text_locations (struct IGEN *igen, struct ANTEXT *antext); 
static void write_call (FILE *outpdf, char *imagename, 
       struct ANTEXT *antext); 



/******************************************************************************
NAME:     make_pdf()
PURPOSE:  create a pdf that calls annotate-inplace many times 
******************************************************************************/
FUNCTION void make_pdf (igen, anproc, imagename)
   struct IGEN *igen;   /* input:  image generation parameters */ 
   char anproc[CMLEN];  /* input:  host name for output pdf */ 
   char *imagename;     /* input:  las name for output image */ 
{
/* declare variables */ 
   FILE *outpdf;          /* points to output pdf */ 
   struct ANTEXT antext;  /* info on calls to annotate-inplace */ 

/* set miscellaneous antext variables */ 
   antext.scalfact = igen->scalfact; 
   /* processing starts at string # 0 and annotate-inplace call # 0 */ 
   antext.curstring = antext.curcall = 0;    
   /* at each chip, each band has a brightness value to annotate */
   antext.nstrings = igen->totalchips * NBANDS;  
   /* annotate-inplace allows for MAXASTRINGS input strings */ 
   antext.totalcalls = (double)antext.nstrings / MAXASTRINGS
		       + .99999;  /* .99999 to round up */

#ifdef OLDWAY 
   /* use one contrasting character grey value for all strings */ 
   if (igen->wheelints >= 0.5 * MAXINTS)
      antext.chargrey = BYTEMIN;
   else 
      antext.chargrey = BYTEMAX;
#endif 

/* allocate memory for antext */ 
   antext_alloc (&antext); 

/* determine locations for the annotation text */ 
   text_locations (igen, &antext);

/* open output file */ 
   if ((outpdf = fopen (anproc, "w")) == NULL)
      errclean ("cwheel-fatal", "Unable to open file %s for writing", anproc);

/* write pdf header */ 
   fprintf (outpdf, "\n!annotate brightness values in the color wheel chips\n");
   fprintf (outpdf, "procedure \n"); 
   fprintf (outpdf, "body \n\n"); 

/* write antext */ 
   while (antext.curcall < antext.totalcalls) 
      write_call (outpdf, imagename, &antext); 

/* finish the pdf */ 
   fprintf (outpdf, "end-proc\n"); 
   fclose (outpdf); 

/* free antext memory */
   antext_free (&antext); 
}






/******************************************************************************
NAME:     antext_alloc()
PURPOSE:  allocate memory for antext structure 
******************************************************************************/
FUNCTION static void antext_alloc (antext)
   struct ANTEXT *antext;  /* output:  allocated memory */
{
/* allocate */ 
   antext->bv       = (int *) malloc (sizeof (long) * antext->nstrings); 
   antext->chargrey = (int *) malloc (sizeof (long) * antext->nstrings); 
   antext->sl       = (int *) malloc (sizeof (long) * antext->nstrings); 
   antext->ss       = (int *) malloc (sizeof (long) * antext->nstrings); 

   if (antext->bv == NULL || antext->chargrey == NULL || 
       antext->sl == NULL || antext->ss       == NULL  )
       errclean ("cwheel-memory", "Unable to allocate memory for antext");
}





/******************************************************************************
NAME:     antext_free()
PURPOSE:  free memory for antext structure 
******************************************************************************/
FUNCTION static void antext_free (antext)
   struct ANTEXT *antext;  /* input:  free this memory */
{
/* free */ 
   free (antext->bv); 
   free (antext->chargrey); 
   free (antext->sl);
   free (antext->ss);
}






/******************************************************************************
NAME:     text_locations()
PURPOSE:  determine locations and values for the annotation text. 
          some of the logic in this function was taken from hue_sat() 
******************************************************************************/
FUNCTION static void text_locations (igen, antext)
   struct IGEN   *igen;    /* input:  image generation info */
   struct ANTEXT *antext;  /* output: text locations and values */ 
{
/* declare variables */
   long band,          /* loop index, image band */ 
	charbv,        /* brightness value for characters */ 
        chipnum,       /* loop index, color chip on this ring of chips */ 
        string=0,      /* current string being processed */
	ringnum;       /* loop index, color ring */ 
   double angle,       /* polar coordinate of this chip */ 
	  averagebv,   /* average of RGB brightness values in a chip */ 
          radschip,    /* radians per each color chip */ 
          hue,         /* HIS color to convert to RGB and annotate */ 
          radius,      /* polar coordinate of this chip */ 
          saturation,  /* HIS color to convert to RGB and annotate */ 
          xfactor,     /* shifts text horizontally */ 
	  xcenter,     /* x center of the color chip */ 
	  ycenter,     /* y center of the color chip */ 
	yshift[NBANDS];/* shifts text vertically, allows for NBANDS of text */ 

   static double twopi = M_PI * 2.0;  /* prevents repeated calculation */ 
   unsigned char *bvpoint[NBANDS], /* points to bvmem, parameter to his2rgb() */
                  bvmem[NBANDS];   /* allocates memory for bvpoint */ 

/* initialize bvpoint */ 
   for (band=0;  band<NBANDS;  band++)
      bvpoint[band] = bvmem + band; 

/* initialize yshift.  
   Text_locations() starts calculating text position at the center of the block
   of text.  To position the middle row of text, one starts at the y coordinate
   of the center of the text block and subracts half the character height
   (scalfact * CHARHEIGHT /2) to get SL of the annotation text.  Hence the
   calculation for yshift[1].  The factor of +1.0 compensates for the bottom
   blank row of pixels in CHARHEIGHT.  yshift[0] and yshift[2] allow for
   spacers between rows of text. */

   yshift[0] = igen->scalfact * (-CHARHEIGHT -SPACER -CHARHEIGHT/2.0 + 1.0); 
   yshift[1] = igen->scalfact * (                    -CHARHEIGHT/2.0 + 1.0); 
   yshift[2] = igen->scalfact * (             SPACER +CHARHEIGHT/2.0 + 1.0); 

/* loop for all rings */ 
   for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
   {
   /* set scalars to convert to polar coordinates */ 
      radschip = twopi / igen->ring2chip[ringnum]; 
      radius = igen->rwidth * ringnum;  

   /* loop for all chips */ 
      for (chipnum=0;  chipnum<igen->ring2chip[ringnum];  chipnum++)
	 {
	 /* get polar coordinates for the center of the chip */ 
	    angle = radschip * chipnum; 

	 /* rotate angle so that red is at the top of the hue circle */ 
	    angle += M_PI_2;      

	 /* for some unknown reason, I need to rotate angle some more */ 
	    angle += M_PI;      

	 /* get ring number and saturation from radius */ 
	    for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
	       if (radius < igen->ringlim[ringnum])
		  break; 
	       saturation = igen->ringsat[ringnum]; 

         /* get hue */ 
	    hue = igen->huelut[ringnum][chipnum]; 

	 /* check for greys.  his2rgb() expects NOHUE for greys */ 
	    if (saturation == 0.0)
	       hue = NOHUE; 
	 
	 /* convert from hue, intensity, saturation to RGB brightness value */ 
	    his2rgb (hue, igen->wheelints, saturation, bvpoint); 

	 /* get cartesian coordiates.  radius * sin or cos converts to cartesian
	    coordinates centered on the color wheel.  adding igen->center 
	    shifts to line, sample coordinates */ 

	    xcenter = radius * cos(angle) + igen->center;
	    ycenter = radius * sin(angle) + igen->center; 

	 /* find average bv for this chip, determine charbv for this text */
	    for (averagebv=0,  band=0;  band<NBANDS;  band++)
	       averagebv += bvmem[band];
	    averagebv /= NBANDS;
	    if (averagebv > (BYTEMAX + BYTEMIN) / 2.0)
	       charbv = BYTEMIN;
	    else
	       charbv = BYTEMAX;

	 /* loop for all bands */ 
	    for (band=0;  band<NBANDS;  band++)
	    {
	    /* save bv calculated by his2rgb() */ 
	       antext->bv[string] = bvmem[band];

	    /* save charbv calculated above */ 
	       antext->chargrey[string] = charbv; 

	    /* set sl of text */ 
	       antext->sl[string] = ycenter + yshift[band] + 0.5;  

	    /* set ss, make text right justified, adjust for number of digits */
	       if      (antext->bv[string] > 99) xfactor = -1.5; /* 3 digits */
	       else if (antext->bv[string] >  9) xfactor = -0.5; /* 2 digits */
	       else                              xfactor =  0.5; /* 1 digit  */

	       antext->ss[string] = xcenter + 
				    xfactor * CHARWIDTH * igen->scalfact + 
				    0.5;  /* .5 for round off */ 

	    /* process the next string */ 
	       string++;
	    }
	 } 
   }
}












/******************************************************************************
NAME:     write_call()
PURPOSE:  write a call to ANNOTATE-INPLACE
******************************************************************************/
FUNCTION static void write_call (outpdf, imagename, antext)
   FILE *outpdf;          /* input:  write to this file */ 
   char *imagename;       /* input:  name of image to annotate */ 
   struct ANTEXT *antext; /* input:  info on calls to annotate
		             output:  curstring and curcall are incremented */ 
{
/* declare variables */ 
   int laststring,  /* loop limit, last string to print */ 
       index;       /* loop index */ 

/* set laststring */ 
   laststring = antext->curstring + MAXASTRINGS; 
   if (laststring > antext->nstrings)
      laststring = antext->nstrings;

/* write a call to putmsg */ 
   fprintf (outpdf, "putmsg \"Executing ANNOTATE for time %d of %d times\" ", 
           antext->curcall+1, antext->totalcalls); 
   fprintf (outpdf, "informational\n"); 

/* start call to annotate-inplace */ 
   fprintf (outpdf, "  annotate-inplace in=%s + \n  annote=  ( ", imagename);

/* write bv */ 
   for (index=antext->curstring;  index<laststring;  index++)
      {
      fprintf (outpdf, "%3d,", antext->bv[index]); 
      if (index != laststring-1) 
	 fprintf (outpdf, ",");
      }

/* sl */ 
   fprintf (outpdf, ") +  \n  sl=      (");
   for (index=antext->curstring;  index<laststring;  index++)
      fprintf (outpdf, "%4d ", antext->sl[index]); 

/* ss */ 
   fprintf (outpdf, ") + \n  ss=      ("); 
   for (index=antext->curstring;  index<laststring;  index++)
      fprintf (outpdf, "%4d ", antext->ss[index]); 

/* scalfact */ 
   fprintf (outpdf, ") + \n  scalfact=(");
   for (index=antext->curstring;  index<laststring;  index++)
      fprintf (outpdf, "%4d ", antext->scalfact); 

/* chargrey */ 
   fprintf (outpdf, ") + \n  chargrey=(");
   for (index=antext->curstring;  index<laststring;  index++)
      fprintf (outpdf, "%4d ", antext->chargrey[index]); 

/* end the call */ 
   if (fprintf (outpdf, ") \n\n") < 0)
      {
      fclose (outpdf);
      errclean ("cwheel-write", "Error while writing to annotation pdf"); 
      }

/* increment counters */ 
   antext->curcall++;
   antext->curstring = laststring; 
}




