
/******************************************************************************
NAME:     target.c
PURPOSE:  Wite a pdf to make a target consisting of multiple color wheels. 
          I expect this program to be used by myself briefly on edcdgw31, 
	  and then probably never used again. 

PROGRAM HISTORY
-------------------------------------------------------------------------------
Date      Programmer       Changes
--------  ---------------  ----------------------------------------------------
Nov 1995  David Zokaites   Original development 
******************************************************************************/


#include <stdio.h>   /* for file output */ 
#include <stdlib.h>  /* for calling exit() */ 
#include <string.h>  /* for calling strcmp() */ 


/* these symbolic constants would be parameters if this program had a user
   interface.  I selected 3 sets of "parameters" for targets for the qlp, 
   a display, and the colorfire.  The parameter sets follow.  */ 
#define OUTNAME "colorfire"  /*  qlp  display  colorfire       old qlp */ 
#define ADDTEXT       "yes"  /*  yes      no         yes           yes */ 
#define BACKINTS       0.0   /*  1.0     0.0         0.0           1.0 */ 
#define BORDER           6   /*    6       3           6             5 */ 
#define RING1          113   /*  128      30         113           100 */
#define RINGWIDTH       78   /*   90      20          78            70 */ 
#define SPACER          23   /*   28       8          23            20 */ 


/* description of what the parameters are 
   OUTNAME   part of name for output pdf and image 
   ADDTEXT   option to add text to the wheels 
   BACKINTS  background intensity 
   BORDER    width of optional border on color wheels in pixels 
   RING1     ring width for wheels with one color ring 
   RINGWIDTH ring width for wheels with more than one color ring 
   SPACER    number of blank lines between each wheel in a column */ 


/* symbolic constants that don't change */ 
#define BDRINTS   0.5  /* border intensity, bounded by 0 and 1 */ 
#define COL1        5  /* put 5 wheels in column 1 of the composite image */ 
#define COL2        1  /* put 1 wheels in column 2 of the composite image */ 
#define COL3        5  /* put 5 wheels in column 3 of the composite image */ 
#define ERROR      -1  /* exit error code */ 
#define NCOLS       3  /* there are 3 columns of wheels in the composite */ 
#define NIMAGES    11  /* composite image will have 11 color wheels */ 
#define TEXTSIZE 70.0  /* size of the text as % of chip height */ 
#define BORDERED NIMAGES /* provide a contrasting border for all color wheels */


/* function prototypes */ 
   int  main        (void);
   void set_vars    (void);
   void write_part1 (void); 
   void set_cols    (void); 
   void write_cols  (long maskval);
   void merge_cols  (long maskval);
   void rotate_image(void); 
   void end_pdf     (void); 






/* global variables describing the color wheels */ 
   long bdrwidth [NIMAGES],  /* width of the border around the color wheel */ 
	colnum   [NIMAGES],  /* gives which column this color wheel is in */ 
        imagesize[NIMAGES],  /* image size (nl and ns) of the color wheel */ 
        nrings   [NIMAGES],  /* number of rings on each wheel */ 
	rwidth   [NIMAGES],  /* ring width for each color wheel */ 
	sl       [NIMAGES],  /* sl for each color wheel in the column */ 
	ss       [NIMAGES];  /* ss for each color wheel in the column */ 

/* other globals */ 
   char outname[100];        /* name for output pdf */ 
   long colheight[NCOLS],    /* height of a column of wheels in pixels */ 
        colwidth [NCOLS];    /* width  of a column of wheels in pixels */ 
   float intensity[NIMAGES]; /* intensity for each color wheel */ 
   FILE *outpdf;  /* points to output file */ 






/******************************************************************************
NAME:     main()
PURPOSE:  program main 
******************************************************************************/
int main (void)
{
/* declare variable */ 
   long maskval; /* mask value for calling concat-manual */ 

/* error check */ 
   if (COL1 + COL2 + COL3 != NIMAGES)
      {
      printf ("Program error in main() \n");
      return (ERROR);
      }

/* set maskval */ 
   if (BACKINTS == 0.0)
      maskval = 0;
   else if (BACKINTS == 1.0)
      maskval = 255;
   else
      {
      printf ("BACKINTS is neither 0 nor 1 \n");
      exit (ERROR);
      }

/* set outname */ 
   sprintf (outname, "make_%s.pdf", OUTNAME); 

/* set basic variables describing the color wheels */ 
   set_vars (); 

/* determine the placement of each color wheel in the column of color wheels */ 
   set_cols (); 

/* write first part of the pdf */ 
   write_part1 ();

/* write calls to concat-manual to make the columns of wheels */ 
   write_cols (maskval); 

/* merge the columns via concat-manual */ 
   merge_cols (maskval); 

/* rotate the composite image for the qlp */ 
   if (!strcmp (OUTNAME, "qlp"))
      rotate_image(); 

/* finish the pdf */ 
   end_pdf (); 

/* completion message */ 
   printf ("Target completed successfully and created %s \n", outname); 

/* return success */ 
   return (0); 
}








/******************************************************************************
NAME:     set_vars()
PURPOSE:  set variables describing the color wheels 
******************************************************************************/
void set_vars (void)
{
/* declare variables */ 
   long wheelnum,    /* loop index, color wheel number */ 
	topwheel,    /* loop index counting down from highest index */ 
        darkborder,  /* number of low  intensity wheels that get a border */ 
        lightborder; /* number of high intensity wheels that get a border */ 

/* set darkborder and lightborder */ 
   darkborder = lightborder = 0; 
   if (BACKINTS == 0.0)
      darkborder = BORDERED;
   if (BACKINTS == 1.0)
      lightborder = BORDERED; 

/* set nrings, it goes 1,2,4,6, ..., max value, ..., 6,4,2,1 */ 
   for (wheelnum=0;  wheelnum<=NIMAGES/2;  wheelnum++)
      {
	 if (wheelnum == 0) 
	    nrings[wheelnum] = 1; 
	 else
	    nrings[wheelnum] = wheelnum * 2; 

	 topwheel = NIMAGES - wheelnum -1; 

	 if (topwheel == NIMAGES-1) 
	    nrings[topwheel] = 1; 
	 else
	    nrings[topwheel] = wheelnum * 2; 
      }

/* set most variables */ 
   for (wheelnum=0;  wheelnum<NIMAGES;  wheelnum++)
      {
      /* rwidth, first and last rings have different value than the rest */ 
	 if (wheelnum==0 || wheelnum==NIMAGES-1) 
	    rwidth[wheelnum] = RING1; 
	 else 
	    rwidth[wheelnum] = RINGWIDTH;

      /* intensity, goes from 1 to 0, white to black */ 
	 intensity[wheelnum] = 1.0 - wheelnum * 1.0 / (NIMAGES - 1.0);

      /* bdrwidth, must follow intensity calculation */ 
	 bdrwidth[wheelnum] = 0;   /* default value */ 

	 if (wheelnum < lightborder && intensity[0] == 1.0 ||
	     wheelnum < darkborder  && intensity[0] == 0.0  )
	    bdrwidth[wheelnum] = BORDER;

	 topwheel = NIMAGES - wheelnum -1; 

	 if (topwheel < lightborder && intensity[NIMAGES-1] == 1.0 ||
	     topwheel < darkborder  && intensity[NIMAGES-1] == 0.0  )
	    bdrwidth[wheelnum] = BORDER;

      /* imagesize, must follow nrings, rwidth, bdrwidth */ 
	 imagesize[wheelnum] = rwidth[wheelnum] + rwidth[wheelnum] * 2 
			  * (nrings[wheelnum] -1)
			  + bdrwidth[wheelnum] * 2; 

      /* colnum */ 
	 if      (wheelnum < COL1     )       colnum[wheelnum] = 0;
	 else if (wheelnum < COL1+COL2)       colnum[wheelnum] = 1;
	 else  /* wheelnum < COL1+COL2+COL3*/ colnum[wheelnum] = 2; 
      }
} 






/******************************************************************************
NAME:     set_cols()
PURPOSE:  determine the placement of each wheel in the column of color wheels 
******************************************************************************/
void set_cols (void)
{
/* declare variables */ 
   long cindex,   /* loop index, column of color wheels */ 
	wheelnum, /* loop index, color wheel number */ 
	sltemp;   /* intermediate value of sl */ 
   float center;  /* central pixel in a column */ 

/* loop for all columns */ 
   for (cindex=0;  cindex < NCOLS;  cindex++)
   {

   /* set column width = max image size for color wheels in this column */ 
      colwidth[cindex] = 0; 
      for (wheelnum=0;  wheelnum<NIMAGES;  wheelnum++)
	 if (colnum[wheelnum] == cindex && imagesize[wheelnum] > colwidth[cindex])
	    colwidth[cindex] = imagesize[wheelnum];

   /* determine sl and ss for color wheels in this column */ 
      center = 1.0 + colwidth[cindex] / 2.0;   /* las is 1 relative */ 
      sltemp = 1; 
      for (wheelnum=0;  wheelnum<NIMAGES;  wheelnum++)
	 {
	 if (colnum[wheelnum] != cindex)
	    continue;

	 sl[wheelnum] = sltemp; 
	 ss[wheelnum] = center - imagesize[wheelnum] / 2.0   +0.5;
	 sltemp += imagesize[wheelnum] + SPACER; 
	 }

   /* set colheight, no need for space after the last image */ 
      colheight[cindex] = sltemp - SPACER;  
   }
}






/******************************************************************************
NAME:     write_part1() 
PURPOSE:  write part 1 of the pdf 
******************************************************************************/
void write_part1 (void)
{
/* declare variables */ 
   long wheelnum;      /* loop index, color wheel number */ 
   float textsize;     /* size of text as a % of chip height */ 

/* open file */ 
   outpdf = fopen (outname, "w"); 
   if (outpdf == NULL)
      {
      printf ("Unable to open file %s \n", outname);
      exit (ERROR);
      }

/* write opening section of the pdf */ 
   fprintf (outpdf, "\n!Create a target consisting of %d color wheels \n",
                    NIMAGES);
   fprintf (outpdf, "!Target image is designed for the %s \n", OUTNAME); 
   fprintf (outpdf, "procedure \n"); 
   fprintf (outpdf, "body \n"); 
   fprintf (outpdf, "\n"); 

/* create color wheels */ 
   fprintf (outpdf, "!Create color wheels \n"); 
   for (wheelnum=0;  wheelnum<NIMAGES;  wheelnum++)
      {
      /* reduce textsize for big chips so all wheels have same size text */ 
	 if (wheelnum==0 || wheelnum==NIMAGES-1)
	    textsize = TEXTSIZE / RING1 * RINGWIDTH; 
	 else
	    textsize = TEXTSIZE;

      /* create wheels */ 
	 fprintf (outpdf, "   clrwheel-chips out=i%.2f ", intensity[wheelnum]); 
	 fprintf (outpdf, "intens=%.2f ", intensity[wheelnum]); 
	 fprintf (outpdf, "nrings=%2d ", nrings[wheelnum]); 
	 fprintf (outpdf, "rwidth=%3d ", rwidth[wheelnum]); 
	 fprintf (outpdf, "aspect=1.0 + \n");
	 fprintf (outpdf, "   textsize=%.1f ", textsize); 
	 fprintf (outpdf, "addtext=%s ", ADDTEXT);
	 fprintf (outpdf, "bdrwidth=%d ", bdrwidth[wheelnum]);
	 fprintf (outpdf, "bdrints=%.2f ", BDRINTS); 
	 fprintf (outpdf, "backints=%.2f \n", BACKINTS); 
      }
   fprintf (outpdf, "\n"); 
}







/******************************************************************************
NAME:     write_cols()
PURPOSE:  write calls to concat-manual to make the columns of wheels 
******************************************************************************/
void write_cols (long maskval)  /* input:  mask value for concat-manual */ 
{
/* declare variables */ 
   long cindex,   /* loop index, column of color wheels */ 
	wheelnum; /* loop index, color wheel number */ 

/* comment for the pdf */ 
   fprintf (outpdf, "!Form the columns of color wheels \n"); 

/* loop for all columns */ 
   for (cindex=0;  cindex < NCOLS;  cindex++)
   {

   /* start the call */ 
      fprintf (outpdf, "   concat-manual out=col%d nl=%d ns=%d + \n", 
	 cindex, colheight[cindex], colwidth[cindex]); 

   /* print in */
      fprintf (outpdf, "   in=("); 
      for (wheelnum=0;  wheelnum<NIMAGES;  wheelnum++)
	 if (colnum[wheelnum] == cindex)
	    fprintf (outpdf, "i%.2f ", intensity[wheelnum] ); 
      fprintf (outpdf, ") + \n");

   /* sl */
      fprintf (outpdf, "   sl=("); 
      for (wheelnum=0;  wheelnum<NIMAGES;  wheelnum++)
	 if (colnum[wheelnum] == cindex)
	    fprintf (outpdf, "%5d ", sl[wheelnum] ); 
      fprintf (outpdf, ") + \n");

   /* ss */
      fprintf (outpdf, "   ss=("); 
      for (wheelnum=0;  wheelnum<NIMAGES;  wheelnum++)
	 if (colnum[wheelnum] == cindex)
	    fprintf (outpdf, "%5d ", ss[wheelnum] ); 
      fprintf (outpdf, ") + \n");

   /* maskval */
      fprintf (outpdf, "   ma=("); 
      for (wheelnum=0;  wheelnum<NIMAGES;  wheelnum++)
	 if (colnum[wheelnum] == cindex)
	    fprintf (outpdf, "%5d ", maskval ); 
      fprintf (outpdf, ") \n");

   fprintf (outpdf, "\n"); 
   }
}






/******************************************************************************
NAME:     merge_cols ()  
PURPOSE:  write a call to concat that will merge the columns of color wheels 
******************************************************************************/
void merge_cols (long maskval)  /* input:  mask value for concat-manual */ 
{
/* declare variables */ 
   long cindex,    /* loop index, column of color wheels */ 
	compwidth, /* width of the overall composite in pixels */ 
	maxcol,    /* nl in column with the most lines */ 
        startl,    /* starting line   of a column in the composite */ 
	starts;    /* starting sample of a column in the composite */ 
   float center;   /* central line in the composite */ 

/* determine which column has the most lines */ 
   maxcol = 0; 
   for (cindex=0;  cindex<NCOLS;  cindex++)
      if (colheight[cindex] > maxcol)
	 maxcol = colheight[cindex];

/* set center */ 
   center = maxcol / 2.0 + 1;  /* las is one relative */ 

/* determine width of the composite */ 
   compwidth =0; 
   for (cindex=0;  cindex<NCOLS;  cindex++)
      compwidth += colwidth[cindex]; 

/* comment for the pdf */ 
   fprintf (outpdf, "!Combine the columns of color wheels \n"); 

/* start the call */ 
   fprintf (outpdf, "   concat-manual out=%s%s nl=%d ns=%d + \n", 
   OUTNAME, 
   !strcmp(OUTNAME,"qlp") ? "0" : "",  
   maxcol, compwidth); 

/* print in */
   fprintf (outpdf, "   in=("); 
   for (cindex=0;  cindex<NCOLS;  cindex++)
      fprintf (outpdf, " col%d ", cindex); 
   fprintf (outpdf, ") + \n");

/* sl */
   fprintf (outpdf, "   sl=("); 
   for (cindex=0;  cindex<NCOLS;  cindex++)
      {
      startl = center - colheight[cindex]/2.0  + 0.5; 
      fprintf (outpdf, "%5d ", startl); 
      }
   fprintf (outpdf, ") + \n");

/* ss */
   starts = 1;  
   fprintf (outpdf, "   ss=("); 
   for (cindex=0;  cindex<NCOLS;  cindex++)
      {
      fprintf (outpdf, "%5d ", starts); 
      starts += colwidth[cindex] ; 
      }
   fprintf (outpdf, ") + \n");

/* maskval */
   fprintf (outpdf, "   ma=("); 
   for (cindex=0;  cindex<NCOLS;  cindex++)
      fprintf (outpdf, "%5d ", maskval ); 
   fprintf (outpdf, ") \n");

fprintf (outpdf, "\n"); 
}






/******************************************************************************
NAME:     rotate_image()
PURPOSE:  rotate the composite image so it fits better on the qlp 
******************************************************************************/
void rotate_image (void)
{
   fprintf (outpdf, "!Rotate the image so it fits better on the qlp \n");
   fprintf (outpdf, "   flip-90 in=qlp0 out=qlp \n\n");
}







/******************************************************************************
NAME:     end_pdf ()
PURPOSE:  finish the pdf 
******************************************************************************/
void end_pdf (void)
{
/* declare variables */ 
   long cindex,    /* loop index, column */ 
	wheelnum;  /* loop index, color wheel number */ 

/* delete intermediate images */ 
   /* comment for the pdf */ 
      fprintf (outpdf, "!Delete intermediate images \n");

   /* delete color wheels */ 
      fprintf (outpdf, "   cmdel-file infile= +\n   ("); 
      for (wheelnum=0;  wheelnum<NIMAGES;  wheelnum++)
	 fprintf (outpdf, "i%.2f ", intensity[wheelnum]); 
      fprintf (outpdf, ") + \n"); 
      fprintf (outpdf, "   conflg=no aftype=none \n\n"); 

   /* delete columns */ 
      fprintf (outpdf, "   cmdel-file infile=("); 
      for (cindex=0;  cindex<NCOLS;  cindex++)
	 fprintf (outpdf, "col%d ", cindex); 
      fprintf (outpdf, ") "); 
      fprintf (outpdf, "conflg=no aftype=none \n\n"); 

   /* optionally delete initial qlp composite */ 
      if (!strcmp (OUTNAME, "qlp"))
	 fprintf(outpdf,"   cmdel-file infile=qlp0 conflg=no aftype=none \n\n");

/* write successful completion message to user */ 
   fprintf (outpdf, "!Write successful completion message to user \n"); 
   fprintf (outpdf, "   write \n");
   fprintf (outpdf, "   putmsg \"%s created output image:  %s\" ", 
		    outname, OUTNAME); 
   fprintf (outpdf, "informational\n\n",
	   OUTNAME);

/* finish up */ 
   fprintf (outpdf, "end-proc \n");
   fclose (outpdf);
}





