#include "convolve.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
*****************************************************************************
FUNCTION convmf

PURPOSE: Perform convolution.

PROGRAM HISTORY:
VERSION   DATE      AUTHOR       CODE/CONTR.   CHANGE REQUEST
1	05/28/92  K. Sheffield    EDC/STX   recoded the convzf routine to
					    perform edge mirroring
2	06/01/92  K. Sheffield    EDC/STX   Changed circular buffer organization
                                            and dynamically allocated all 
                                            buffers.

PDL:

Begin convmf

	Locate weight grid center.
	Invert the weight matrix for convolution.
	Allocate local work space
	Write first output lines as zeros.
	Read first lines of input image and mirror the edges
	Point to top line of circular buffer with isl:
	for each set of nbr lines
		Perform convolution
		Copy out to output buffer
	Read the next line into the circular buffer and mirror the edges
	For each new output line
		for each set of nbr lines
			Perform convolution 
		Copy out to output buffer
	Free workspace

End convmf

********************************************************************
*/

void convmf
(
    char hosout[][CMLEN],         /* output image host name       	*/
    float wgtfct[],         /* Weight array 			*/
    long nbr,               /* Number of rows in wgtfct 		*/
    long nbc,               /* Number of colums in wgtfct 		*/
    long image,             /* Input image number			*/
    long band_num,          /* Band number 				*/
    long nl,		/* Number of samples in image		*/
    long ns,		/* Number of samples in image		*/
    long intype            /* Input band type 			*/
)

{
float  *circ_buf; 	/* pointer to circular buffer workspace	*/
float  *kernal;		/* pointer to convolution kernal	*/
float  biv[15][15];	/* Local convolution workspace		*/
float  x; 		

float	*kptr;		/* working pointer to convolution kernal*/
float	*cptr;		/* working pointer to circular buffer   */
float	*optr;		/* working pointer to output buffer     */

			/* Local convolution workspace		*/
long   inc = 0;		/* Matrix inversion indicie		*/
long   i, j;		/* Loop counters			*/
long   incount  = 1;	/* Input line counter			*/
long   isl;		/* Convolution kernal index		*/
long   nblk = 1;	/* Imageio lines-per-read parameter	*/
long   outtype = EREAL; /* Output data type			*/
long   outcount = 1;	/* Output line counter			*/
long   pixnum; 		/* Convolution loop counter		*/
long   size;		/* Pixel data type size in bits		*/
long   u, k;		/* Loop counters			*/
long   xc;		/* Convolution grid center indicie	*/
long   yc;		/* Convolution grid center indicie	*/
long   ksize;		/* number of elements in the kernal     */
long   ns_mir;		/* number of pixels in the mirrored scanline */ 

unsigned char *io_buf; 	/* I/O buffer */
unsigned char *conv_buf; /* conversion workspace		*/
float *conv_ptr;	/* start of the first non mirrored pixel */

curent = 1;

/*
Locate weight grid center.
--------------------------						*/
xc = nbr >> 1;
yc = nbc >> 1;

ns_mir = ns + (yc << 1);

/* Allocate memory resources needed by this program */

size = sizeof(float);
/* allocate I/O buffer */
io_buf =  (unsigned char *) malloc(ns * size);

/* allocate conversion buffer */
conv_buf  = (unsigned char *) malloc(ns_mir * size);

ksize = nbr * nbc;
/* allocate convolution kernal */
kernal  = (float *) malloc(size * ksize);

/* allocate space for circular buffer */
circ_buf = (float *) malloc(ns_mir * size * nbr);


if( (io_buf == NULL) || (conv_buf == NULL) || (kernal == NULL) || 
	(circ_buf == NULL) ) 
{
	if (io_buf != NULL) free(io_buf);
	if (kernal != NULL) free(kernal);
	if (conv_buf != NULL) free(conv_buf);
	if (circ_buf != NULL) free(circ_buf);

	c_abort(&vblock, "Unable to allocate memory needed.",
			 "convolve-malloc", prog_name, hosout, &one);
}

conv_ptr = (float *) conv_buf;
conv_ptr += yc;

/*
Invert the weight matrix for convolution.
-----------------------------------------				*/
for (i = 0; i < nbr; ++i)
{
	for (j = 0; j < nbc; ++j, ++inc)
		biv[i][j] = wgtfct[inc];
} 

kptr = kernal;
for ( i = (nbr - 1); i >= 0; --i)
{
	for (j = 0 ; j < nbc; j++)
		*(kptr++) = biv[j][i];
} 


/*
Read 1st nbr/2 lines of input image and store in buffer "a"
--------------------------------------------------------		*/

u = ( (nbr - 1) >> 1) + 1;
for(i = 1; i <= u; ++i)
{
   if (c_eread  (&fdesc_in[image], &band_num, &incount, io_buf, &nblk) 
   == E_FAIL)
   {
	c_abort(&vblock, "Error reading input image", 
		"convolve-in", prog_name, hosout, &one);
   }
   c_pxconv (&intype,&outtype,io_buf,(unsigned char *)conv_ptr,&ns);
   mir_data((float *)conv_buf, nbc, ns);
   cirfil ((float *)conv_buf, circ_buf, nbr - i, ns_mir, nbr);
   cirfil ((float *)conv_buf, circ_buf, i - 1, ns_mir, nbr);
   ++incount;
}

/*
Point to top line of circular buffer with isl:
------------------------------------------------			*/
isl = nbr - 1;


optr = (float *) io_buf;

/*
Perform convolution for each set of nbr lines
---------------------------------------------			*/
for(pixnum = 0; pixnum<ns; ++pixnum)
{
	x = 0.0;
	cptr = circ_buf + (nbr * pixnum);

	kptr = kernal;
	/*
	Calculate output cell value
	---------------------------                                     */
	for(u = 0; u < ksize; u++,kptr++,cptr++)
	{
		x += ( (*kptr) * (*cptr) );
	}
	*(optr++) = x;
} 

/*
Copy out to output buffer
------------------------					*/
if (c_ewrite(&fdesc_out, &curcat, &outcount, io_buf, &nblk) == E_FAIL)
{
	c_abort(&vblock, "Error writing output image", 
		"convolve-out", prog_name, hosout, &one);
}
++outcount;
++curent;

/*
Process new output lines.
-------------------------						*/
for(i = 1; i < (nl-xc); ++i)
{


	/*
	Get next line in a
	------------------						*/
	if (c_eread (&fdesc_in[image], &band_num, &incount, 
	    io_buf, &nblk) == E_FAIL)
	{
		c_abort(&vblock, "Error reading input image", 
			"convolve-in", prog_name, hosout, &one);
	}
   	c_pxconv (&intype,&outtype,io_buf,(unsigned char *)conv_ptr,&ns);
   	mir_data((float *)conv_buf, nbc, ns);
   	cirfil ((float *)conv_buf, circ_buf, isl, ns_mir, nbr);
	shiftk(kernal,nbc,nbr);

	++incount;
	/*
	Point to top line of circular buffer with isl:
	----------------------------------------------                  */
	--isl;
	if (isl < 0 )
		isl = nbr - 1;


	optr = (float *) io_buf;
	/*
	Perform convolution for each set of nbr lines
	---------------------------------------------			*/
	for(pixnum = 0; pixnum<ns; pixnum++)
	{
		x = 0.0;
		cptr = circ_buf + (nbr * pixnum);

		kptr = kernal;
	/*
	Calculate output cell value
	---------------------------					*/
		for(u = 0; u < ksize; u++,kptr++,cptr++)
		{
			x += ( (*kptr) * (*cptr) );
		} 
		*(optr++) = x;
	} 

	/*
	Copy out to output buffer
	------------------------					*/
	if (c_ewrite(&fdesc_out, &curcat, &outcount, io_buf, &nblk) == E_FAIL)
	{
		c_abort(&vblock, "Error writing output image", 
			"convolve-out", prog_name, hosout, &one);
	}
	++outcount;
	++curent;

}    


isl++;
if (isl > (nbr - 1) )
	isl = 0;

k = isl + 1;
if(nbr & 1)
	k++;
/*
Process remaining output lines.
-------------------------						*/
for(i = 0; i < xc; ++i)
{
	if(k > (nbr - 1))
		k = k - nbr;

	/*
	Point to top line of circular buffer with isl:
	----------------------------------------------			*/
	if (isl < 0 )
		isl = (nbr - 1) ; 

	circ_cpy (circ_buf, k++, isl, ns_mir,nbr);
	shiftk(kernal,nbc,nbr);
	--isl;

	/*
	Point to top line of circular buffer with isl:
	----------------------------------------------			*/
	if (isl < 0 )
		isl = (nbr - 1) ; 


	optr = (float *) io_buf;
	/*
	Perform convolution for each set of nbr lines
	---------------------------------------------			*/
	for(pixnum = 0; pixnum<ns; ++pixnum)
	{
		x = 0.0;
		cptr = circ_buf + (nbr * pixnum);

		kptr = kernal;
	/*
	Calculate output cell value
	---------------------------					*/
		for(u = 0; u < ksize; u++,kptr++,cptr++)
		{
			x += ( (*kptr) * (*cptr) );
		} 
		*(optr++) = x;
	} 

	/*
	Copy out to output buffer
	------------------------					*/
	if (c_ewrite(&fdesc_out, &curcat, &outcount, io_buf, &nblk) == E_FAIL)
	{
		c_abort(&vblock, "Error writing output image", 
			"convolve-out", prog_name, hosout, &one);
	}
	++outcount;
	++curent;
}

free(io_buf);
free(circ_buf);
free(conv_buf);
free(kernal);

}

void mir_data
(
    float *buf,
    long k,
    long ns
)
{
	float *src, *dst;
	long i, k_half;

	k_half = k >> 1;

	src = buf + k - 1;
	dst = buf;

	for(i=0;i<k_half;i++)
		*(dst++) = *(src--);

	dst = buf + ns + (k_half<<1);
	src = dst - k;
	
	
	for(i=0;i<k_half;i++)
		*(--dst) = *(src++);
}

void circ_cpy
(
    float *buffer,
    long from,
    long to,
    long num,
    long rows
)
{
	float *src, *dst;
	long i;

	src = dst = buffer;

	src += from;
	dst += to;

	if ( num < 1) return;

	i = 0;
	do 
	{
		*(dst) = *(src);
		dst += rows;
		src += rows;
		i++;
	} while (i < num);
}
		
	
