#include "convolve.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
*****************************************************************************
FUNCTION convzf

PURPOSE: Perform convolution.

PROGRAM HISTORY:
VERSION   DATE      AUTHOR       CODE/CONTR.   CHANGE REQUEST
1	6/24/85   F. IRANI        SAR         	NON-AP VERSION
2	10/15/89  J. McGuthry	  STX		C Conversion 
3	10/15/90  F. IRANI        STX	    Make handle multiple input images
4	1/29/91	  D. Akkerman	  EDC	    Changed large float declarations to
					    static float for Power Node.
5	06/01/92  K. Sheffield    EDC/STX   Changed circular buffer organization
                                            and dynamically allocated all 
                                            buffers.

PDL:

Begin convzf

	Locate weight grid center.
	Invert the weight matrix for convolution.
	Allocate local work space
	Write first output lines as zeros.
	Read first lines of input image
	Point to top line of circular buffer with isl:
	for each set of nbr lines
		Perform convolution
		Pad unprocessed output cells with zeros.
		Copy out to output buffer
	Read the next line in "a".
	For each new output line
		for each set of nbr lines
			Perform convolution 
		Pad unprocessed output cells with zeros.
		Copy out to output buffer
	Pad unprocessed lines with zeros
	Free workspace

End convzf

********************************************************************
*/

void convzf
(
    char hosout[][CMLEN],         /* output image host name       	*/
    float wgtfct[],         /* Weight array 			*/
    long nbr,               /* Number of rows in wgtfct 		*/
    long nbc,               /* Number of colums in wgtfct 		*/
    long image,             /* Input image number			*/
    long band_num,          /* Band number 				*/
    long nl,		/* Number of samples in image		*/
    long ns,		/* Number of samples in image		*/
    long intype            /* Input band type 			*/
)

{
float  *circ_buf; 	/* pointer to circular buffer workspace	*/
float  *kernal;		/* pointer to convolution kernal	*/
float  biv[15][15];	/* Local convolution workspace		*/
float  fzero = 0.0;	/* Zero constant			*/
float  x; 		

float	*kptr;		/* working pointer to convolution kernal*/
float	*cptr;		/* working pointer to circular buffer   */
float	*optr;		/* working pointer to output buffer     */

long   inc = 0;		/* Matrix inversion indicie		*/
long   i, j, ii;	/* Loop counters			*/
long   incount  = 1;	/* Input line counter			*/
long   isl;		/* Convolution kernal index		*/
long   nblk = 1;	/* Imageio lines-per-read parameter	*/
long   outtype = EREAL; /* Output data type			*/
long   outcount = 1;	/* Output line counter			*/
long   pixnum; 		/* Convolution loop counter		*/
long   size;		/* Pixel data type size in bits		*/
long   u,  k;		/* Loop counters			*/
long   xc;		/* Convolution grid center indicie	*/
long   yc;		/* Convolution grid center indicie	*/
long   ksize;		/* number of elements in the kernal     */

unsigned char *io_buf; /* Local convolution workspace		*/
unsigned char *conv_buf; /* Local convolution workspace		*/

curent = 1;

/*
Locate weight grid center.
--------------------------						*/
xc = nbr >> 1;
yc = nbc >> 1;

/* Allocate memory resources needed by this program */

size = sizeof(float);
/* allocate I/O buffer */
io_buf =  (unsigned char *) malloc(ns * size);

/* allocate conversion buffer */
conv_buf  = (unsigned char *) malloc(ns * size);

ksize = nbr * nbc;
/* allocate convolution kernal */
kernal  = (float *) malloc(size * ksize);

/* allocate space for circular buffer */
circ_buf = (float *) malloc(ns * size * nbr);

if( (io_buf == NULL) || (conv_buf == NULL) || (kernal == NULL) || 
	(circ_buf == NULL) ) 
{
	if (io_buf != NULL) free(io_buf);
	if (kernal != NULL) free(kernal);
	if (conv_buf != NULL) free(conv_buf);
	if (circ_buf != NULL) free(circ_buf);

	c_abort(&vblock, "Unable to allocate memory needed.",
			 "convolve-malloc", prog_name, hosout, &one);
}

/*
Invert the weight matrix for convolution.
-----------------------------------------				*/
for (i = 0; i < nbr; ++i)
{
	for (j = 0; j < nbc; ++j, ++inc)
		biv[i][j] = wgtfct[inc];
} 

kptr = kernal;
for ( i = (nbr - 1); i > -1; --i)
{
	for (j = 0 ; j < nbc; ++j)
		*(kptr++) = biv[j][i];
} 

/*
Write first xc output lines as zeros.
-------------------------------------					*/
for(i = 0; i < xc; ++i)
{
   c_pxfill (io_buf, &outtype, &ns, &fzero);
   if ((c_ewrite(&fdesc_out, &curcat, &outcount, io_buf, &nblk))
       == E_FAIL)
   {
	c_abort(&vblock, "Error writing output image", 
		"convolve-out", prog_name, hosout, &one);
   }
   ++outcount;
   ++curent;
}   
/*
Read 1st nbr lines of input image and store in buffer "a"
--------------------------------------------------------		*/
for(i = 1; i <= nbr; ++i)
{
   if (c_eread  (&fdesc_in[image], &band_num, &incount, io_buf, &nblk) 
   == E_FAIL)
   {
	c_abort(&vblock, "Error reading input image", 
		"convolve-in", prog_name, hosout, &one);
   }
   c_pxconv (&intype,&outtype,io_buf,conv_buf,&ns);
   cirfil ((float *)conv_buf, circ_buf, nbr - i, ns, nbr);
   ++incount;
}

/*
Point to top line of circular buffer with isl:
------------------------------------------------			*/
isl = nbr - 1;

optr = (float *) io_buf + yc;
/*
Perform convolution for each set of nbr lines
---------------------------------------------				*/
for (pixnum = yc; pixnum < (ns - yc); pixnum++)
{
	x = 0.0;
	cptr = circ_buf + (nbr * (pixnum - yc));

	kptr = kernal;
	/*
	Calculate output cell value
	---------------------------					*/
	for(u = 0; u < ksize; u++,kptr++,cptr++)
	{
		x += ( (*kptr) * (*cptr) );
	} 
	*(optr++) = x;
}  

/*
Pad unprocessed output cells with zeros.
---------------------------------------					*/
for (ii = 0;ii < 2; ++ii) 
{ 
	k = (ns - yc);
	optr = (float *) io_buf;
	optr += (ii * k);
	for (j = 0;j < yc; ++j) 
		*(optr++) = 0.;
}

/*
Copy out to output buffer
-------------------------						*/
if ( c_ewrite   (&fdesc_out, &curcat, &outcount, io_buf, &nblk) == E_FAIL)
{
	c_abort(&vblock, "Error writing output image", 
		"convolve-out", prog_name, hosout, &one);

}

++outcount;
++curent;

/*
Read the next line in "a".
--------------------------						*/
if (c_eread (&fdesc_in[image], &band_num, &incount, io_buf, &nblk) == E_FAIL)
{
	c_abort(&vblock, "Error reading input image", 
		"convolve-in", prog_name, hosout, &one);
}
c_pxconv (&intype, &outtype, io_buf, conv_buf, &ns);
cirfil ((float *)conv_buf, circ_buf, isl, ns, nbr);
shiftk(kernal,nbc,nbr);
++incount;

/*
Process new output lines.
-------------------------						*/
for(i = nbr; i < nl; ++i)
{

	/*
	Point to top line of circular buffer with isl:
	----------------------------------------------			*/
	--isl;
	if (isl < 0 ) 
		isl = nbr - 1;

	optr = (float *) io_buf + yc;
	/*
	Perform convolution for each set of nbr lines
	---------------------------------------------			*/
	for(pixnum = yc; pixnum<(ns-yc); ++pixnum)
	{
		x = 0.0;
		cptr = circ_buf + (nbr * (pixnum - yc));

		kptr = kernal;
	/*
	Calculate output cell value
	---------------------------					*/
		for(u = 0; u < ksize; u++,kptr++,cptr++)
		{
			x += ( (*kptr) * (*cptr) );
		} 
		*(optr++) = x;
	} 

	/*
	Pad unprocessed output cells with zeros.
	----------------------------------------			*/
	for(ii = 0; ii < 2; ++ii)
	{
		k = (ns - yc);
		optr = (float *) io_buf;
		optr += (ii * k);
		for (j = 0;j < yc; ++j) 
			*(optr++) = 0.;
	} 
	/*
	Copy out to output buffer
	------------------------					*/
	if (c_ewrite(&fdesc_out, &curcat, &outcount, io_buf, &nblk) == E_FAIL)
	{
		c_abort(&vblock, "Error writing output image", 
			"convolve-out", prog_name, hosout, &one);
	}
	++outcount;
	++curent;

	/*
	Get next line in a
	------------------						*/
	if ( i != (nl - 1) )
	{
	if (c_eread (&fdesc_in[image], &band_num, &incount, 
	    io_buf, &nblk) == E_FAIL)
	{
		c_abort(&vblock, "Error reading input image", 
			"convolve-in", prog_name, hosout, &one);
	}
	c_pxconv (&intype, &outtype, io_buf, conv_buf, &ns);
	cirfil ((float *)conv_buf, circ_buf, isl, ns, nbr);
	shiftk(kernal,nbc,nbr);
	++incount;
   }
}    

/*
Pad unprocessed lines with zeros
--------------------------------					*/
for(i = 0; i < xc; ++i)
{
	c_pxfill (io_buf, &outtype, &ns, &fzero);
	if (c_ewrite (&fdesc_out, &curcat, &outcount, io_buf, &nblk) 
	== E_FAIL)
	{
		c_abort(&vblock, "Error writing output image", 
			"convolve-out", prog_name, hosout, &one);
	}
	++outcount;
	if ( curent < ns ) ++curent;
}  	   

free(io_buf);
free(conv_buf);
free(kernal);
free(circ_buf);

}
