
/* dct.c by David Zokaites 5-93.  These functions compute forward and reverse 
Discrete Cosine Transforms (DCTs) */ 
#include <stdlib.h>
#include <math.h>   /* for call to cos(), sqrt() */ 
#include <stdio.h>  /* for definition of NULL */ 
#include "las.h"
#include "dctfil.h"
#include "protos.h"




/* global structure for dct constants, the pointers are used to create 1-D and
   2-D luts */ 
   struct dconstant 
   {
      float **cosval,    /* cos( dcon.twoip1[line] * dcon.ipidn2[samp] ) */
	    **fscalar,   /* scalar for the forward dct */ 
	      fscalar0,  /* intermediate in creating **fscalar */ 
             *ipidn2,    /* integer times pi divided by ntimes2 */ 
	      pi,        /* the familiar constant */ 
	      rec2,      /* 1 / by the square root of 2 */ 
	    **rscalar;   /* scalar for the reverse dct */ 
      long    bsize,     /* the size of dct blocks */
	      ntimes2,   /* the block size (n) times 2 */ 
             *twoip1;    /* 2 * integer + 1 */ 
   } dcon; 





/* FUNCTION:  initialize constants for DCT calculations */ 
void set_const
(
    long bsize           /* input:   size of dct blocks */
) 
			 /* output:  dcon global structure */ 
{ 
/* declare variables */ 
   long line, samp;      /* for loop indices */ 
/* allocate memory for luts */ 
   if (
   (dcon.cosval  = (float **) malloc (sizeof(float *) * bsize) ) == NULL ||
   (dcon.fscalar = (float **) malloc (sizeof(float *) * bsize) ) == NULL ||
   (dcon.rscalar = (float **) malloc (sizeof(float *) * bsize) ) == NULL ||
   (dcon.ipidn2  = (float  *) malloc (sizeof(float  ) * bsize) ) == NULL || 
   (dcon.twoip1  = (long   *) malloc (sizeof(long   ) * bsize) ) == NULL  )
      clean2 ("Unable to allocate memory for DCT luts."); 
/* initialize misc constants */ 
   dcon.bsize    = bsize; 
   dcon.ntimes2  = bsize * 2;           
   dcon.pi       = 3.1415926536;       
   dcon.rec2     = 1./ sqrt (2.); 
   dcon.fscalar0 = 4. / (bsize * bsize);
/* set 1-D luts */ 
   for (samp=0; samp<bsize; samp++)
   {
      dcon.ipidn2[samp] = (float) samp * dcon.pi / ((float) dcon.ntimes2); 
      dcon.twoip1[samp] = 2 * samp + 1; 
   }
/* set 2-D luts */ 
   for (line=0; line<bsize; line++)
   {
   /* allocate memory for cosval, fscalar, rscalar */ 
      if (
      (dcon.cosval[line] = (float *) malloc (sizeof(float) * bsize) ) == NULL ||
      (dcon.fscalar[line]= (float *) malloc (sizeof(float) * bsize) ) == NULL ||
      (dcon.rscalar[line]= (float *) malloc (sizeof(float) * bsize) ) == NULL  )
	 clean2 ("Unable to allocate memory for DCT luts."); 
   /* set lut values */ 
      for (samp=0; samp<bsize; samp++)
      {
	 dcon.cosval[line][samp] = 
                            cos( (float) dcon.twoip1[line] * dcon.ipidn2[samp]);
	 dcon.fscalar[line][samp] = dcon.fscalar0; 
	 dcon.rscalar[line][samp] = 1.; 
	 if (!line)
	 {
	    dcon.fscalar[line][samp] *= dcon.rec2; 
	    dcon.rscalar[line][samp] *= dcon.rec2; 
	 }
	 if (!samp)
	 {
	    dcon.fscalar[line][samp] *= dcon.rec2; 
	    dcon.rscalar[line][samp] *= dcon.rec2; 
	 }
      }
   }
/* dump variables 
   printf ("bsize=%d ntimes2=%d pi=%f rec2=%f scalar0=%f \n", 
   dcon.bsize, dcon.ntimes2, dcon.pi, dcon.rec2, dcon.fscalar0);
   printf ("samp  ipidn2  twoip1 \n"); 
   for (samp=0; samp<bsize; samp++)
      printf ("%d %f %d \n", samp, dcon.ipidn2[samp], dcon.twoip1[samp]); 
   printf ("line samp cosval fscalar rscalar \n"); 
   for (line=0; line<bsize; line++)
      for (samp=0; samp<bsize; samp++)
	 printf ("%d %d %f %f %f \n", 
	 line, samp, dcon.cosval[line][samp], dcon.fscalar[line][samp], 
	 dcon.rscalar[line][samp]); */ 
}





/* FUNCTION:  perform forward DCT */ 
void fdct
(
    float **imaged,     /* input:  perform dct on this image data */
    float **dctdata    /* output: dct */
) 
{
/* declare variables */ 
   long dline, dsamp,  /* loop indices to output Dct data (dctdata) */ 
        iline, isamp;  /* loop indices to input Image data (imaged) */ 
/* loop for all pixels */ 
   for (dline=0; dline<dcon.bsize; dline++)
      for (dsamp=0; dsamp<dcon.bsize; dsamp++)
      {
      /* initialize the sum of dct components */ 
	 dctdata[dline][dsamp] = 0.; 
      /* compute the sum */ 
	 for (iline=0; iline<dcon.bsize; iline++)
	    for (isamp=0; isamp<dcon.bsize; isamp++)
	       dctdata[dline][dsamp] += imaged[iline][isamp]    * 
	       dcon.cosval[iline][dline] * dcon.cosval[isamp][dsamp];
      /* scale the sum */ 
	 dctdata[dline][dsamp] *= dcon.fscalar[dline][dsamp]; 
      }
}





/* FUNCTION:  apply filter weights to DCT */ 
void app_weights
(
    float **fweights,   /* input:  filter weights */
    float **dctdata    /* input as dct, output as filtered dct */
)
{
/* declare variables */ 
   long line,          /* loop index to fweights */ 
        samp;          /* loop index to fweights */ 
/* loop for all pixels */ 
   for (line=0; line<dcon.bsize; line++)
      for (samp=0; samp<dcon.bsize; samp++)
	 dctdata[line][samp] *= fweights[line][samp]; 
}





/* FUNCTION:  perform reverse DCT */ 
void rdct
(
    float **dctdata,    /* input:  filtered dct */
    float **imaged     /* output: filtered image data */
) 
{
/* declare variables */ 
   long dline, dsamp,  /* loop indices to input Dct data (dctdata) */ 
        iline, isamp;  /* loop indices to output imaged */ 
/* loop for all pixels */ 
   for (iline=0; iline<dcon.bsize; iline++)
      for (isamp=0; isamp<dcon.bsize; isamp++)
      {
      /* initialize the sum of dct components */ 
	 imaged[iline][isamp] = 0.; 
      /* compute the sum */ 
	 for (dline=0; dline<dcon.bsize; dline++)
	    for (dsamp=0; dsamp<dcon.bsize; dsamp++)
	       imaged[iline][isamp] += dctdata[dline][dsamp] * 
	       dcon.cosval[iline][dline] * dcon.cosval[isamp][dsamp] * 
               dcon.rscalar[dline][dsamp]; 
      }
}





/* FUNCTION:  correct for truncation (round-off) error when converting from 
   REAL*4 to integer image data types */ 
void fix_truncate
(
    float **imaged   /* input and output:  image data to correct */
) 
{
/* declare variables */ 
   long line, samp;  /* loop indices */ 
/* loop for all pixels */ 
   for (line=0; line<dcon.bsize; line++)
      for (samp=0; samp<dcon.bsize; samp++)
      {
	 if (imaged[line][samp] >  (float)0.)
	     imaged[line][samp] += (float) .5; 
	 else 
	 if (imaged[line][samp] <  (float)0.)
	     imaged[line][samp] -= (float) .5; 
      }
}





/* FUNCTION:  free the memory in the global data structure */ 
void free_struct
(void)
{
/* declare variables */ 
   long line;      /* for loop index */ 
/* free memory */ 
   for (line=0; line<dcon.bsize; line++)
   {
      free (dcon.cosval[line]);
      free (dcon.fscalar[line]);
      free (dcon.rscalar[line]);
   }
   free (dcon.cosval);
   free (dcon.fscalar); 
   free (dcon.rscalar); 
   free (dcon.ipidn2); 
   free (dcon.twoip1); 
}
