
/*
*************************************************************************
NAME: GETPAR

PURPOSE: gets the user-specified parameters and tae globals

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************

	arguments:
	name			description
	----			-----------
	bands                   band specification
	high			high end of valid data range
	hosin  		        host names of input images
	hosout			host name of output image
	invld			option to use for invalid samples
	ksamps			number of samples per line in kernel
	klines			number of lines in kernel
	low			low end of valid data range
	minmax			indicator for whether the user wants the
				min and max values determined for each band
	msgtim			elapsed time in seconds between display messages
	nbands 			number of bands in each image
	nl			number of lines to be processed
	nmrimg			number of input images
	ns			number of samples per line to be processed
	otype			output data type
	subcmd			specified subcommand
	taeout			tae name of output image
	totbnd			total number of output bands
	window			window specification
	kernel			kernel dimensions
	length			length specification
	qflag			flag indicating if a subwindow was specified
	same			flag indicating if images have same window size
	status			function return status code
	stdout			logical unit number for standard output
	taein			tae names of input images
	nstd			number of standard deviations
	tol			tolerance value
************************************************************************
*/
#include "despike.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char hosin[][CMLEN],
    char taeout[],
    char hosout[],
    long *totbnd,
    long bands[][MAXBND+1],
    long nbands[],
    long *nl,
    long *ns,
    long window[][4],
    long *nmrimg,
    char subcmd[],
    long *ksamps,
    long *klines,
    float *low,
    float *high,
    long *otype,
    long *invld,
    float *nstd,
    float *tol
)

{
struct	VARIABLE	*var;

long	ftype = EREAL;
long	itype = ELONG;
long	kernel[2];
long	length;
long	maximg = MAXIMG;
long	qflag[MAXIMG][2];
long	same;
long	status;
long	std_out;
long	two = 2;
long    fatal_err = -1;
long    zero = 0;
long    one = 1;

char	minmax[4];
char	odtype[5];
char	taein[MAXIMG][CMLEN];
/*
	initialization
	--------------
*/
c_inlas (&parblk, &std_out);

/*
	get globals                     
	-----------
*/
var = p_find( &parblk, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &parblk, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

/*
	get subcommand
	--------------
*/
c_getstr (&parblk, "_SUBCMD", &one, &length, subcmd);

/*
	get input file information
	--------------------------
*/
status = c_getin(&parblk, "IN", &zero, taein, hosin, nmrimg, window, bands, 
                 nbands, totbnd, &maximg, &same, qflag);
if (status  !=  E_SUCC) 
   c_errmsg ("Fatal error encountered", "filter-fatal", &fatal_err);

if (same  !=  1) 
   c_errmsg ("Input images do not have the same window sizes", 
             "filter-window", &fatal_err);


*nl = window[0][NL];
*ns = window[0][NS];

/*
	get output file name
	--------------------
*/
status = c_getout (&parblk, "OUT", (char (*)[256])taeout,
                      (char (*)[256])hosout);
if (status  !=  E_SUCC) 
   c_errmsg ("Fatal error encountered", "filter-fatal", &fatal_err);

/*
	get kernel dimensions
	---------------------
*/
c_getnum (&parblk, "KERNDIM", &two, &itype, (unsigned char *)kernel);
*klines = kernel[0];
*ksamps = kernel[1];

/*
	validate kernel dimensions
	--------------------------
*/
if ((*klines % 2)  !=  1  ||  (*ksamps % 2)  !=  1)
   c_errmsg
      ("Invalid kernel size -- Kernel values must be odd", 
       "filter-odd", &fatal_err);
else if(*ksamps + *klines  <  4) 
   c_errmsg ("Invalid kernel size -- Too small", 
             "filter-small", &fatal_err);
else if((*ksamps  >  *ns)  ||  (*klines  >  *nl)) 
   c_errmsg ("Invalid kernel size -- Too large", 
             "filter-large", &fatal_err); 

/*
	get output data type
	--------------------
*/
c_getstr (&parblk, "ODTYPE", &one, &length, odtype);
if (strcmp (odtype, "SAME") == 0) *otype = 0;
else if (strcmp (odtype, "BYTE") == 0) *otype = 1;
else if (strcmp (odtype, "I*2") == 0) *otype = 2;
else if (strcmp (odtype, "I*4") == 0) *otype = 3;
else if (strcmp (odtype, "R*4") == 0) *otype = 4;
if (*otype  ==  0) 
   {
   status = c_gettyp(hosin, nmrimg, otype);
   if (status  !=  E_SUCC) 
      c_errmsg ("Input images are not of same data type", 
                "filter-input", &fatal_err);
   } 

/*
	 get remaining parameters
	 ------------------------
*/
   c_getnum (&parblk, "MINVAL", &one, &ftype, (unsigned char *)low);
   c_getnum (&parblk, "MAXVAL", &one, &ftype, (unsigned char *)high);
   if (*high  <=  *low) 
      c_errmsg ("Invalid values for MIN and MAX", 
                "filter-minmax", &fatal_err);
   c_getnum (&parblk, "INVALOPT", &one, &itype, (unsigned char *)invld);
   c_getnum (&parblk, "NSTD",&one, &ftype, (unsigned char *)nstd);
   c_getnum (&parblk, "TOL", &one, &ftype, (unsigned char *)tol);
}
