/*
*************************************************************************
NAME: HILO 

PURPOSE: applies a high or low pass filter to an image band.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    3.1          4/89   D. Hollaren   Dynamically allocated large arrays in
				      order to compile on PN9050
    4            9/99   D. Hames      Remove references to MAXNL/MAXNS and
                                      replace with nl and ns to support
                                      images >2gig. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************

	arguments:
	name		description
	----		-----------
	bndin		current input band number
	bndout		output band number
	high		high end of valid data range
	hosout		out image host name
	invld		option to use for invalid samples
	kersum		data sum of the kernel
	ksamps		number of samples per line in kernel
	low		low end of valid data range
	maxval		maximum values for each band
	minval		minimum values for each band
	nl		number of lines to be processed
	ns		number of samples per line to be processed
	otype		output data type
	valsum		the valid number of samples which contributed
			to the kernel sum
	xhalf		half of the number of samples in the kernel
	yhalf		half of the number of lines in the kernel

	internal variables:
	name		description
	----		-----------
	addval		value to be added in to the kernel sum
	addvld		value to be added to the valid count
	addlin		counter for lines to be added sequentially
			to bottom of kernel
	bindex		the index into the "bottom" array
	bottom		the lines to read to mirror at the bottom 
	dellin		counter for lines to be deleted sequentially
			from the top of the kernel
	lpos		position for left edge
	mean		mean of kernel
	outval		computed output value (in real*4)
	oval4		computed output value (in integer*4)
	subval		value to be subtracted from the kernel sum
	subvld		value to be subtracted from the valid count
	status		function return status code
	tindex		the index into the "top" array
	top		the lines to read to mirror at the top
************************************************************************
*/
#include "despike.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void hilo
(
    char hosout[],
    long nl,
    long *ns,
    long otype,
    long *bndin,
    long *bndout,
    long xhalf,
    long yhalf,
    long ksamps,
    double high,
    double low,
    float *varsum,
    float *kersum,
    long *valsum,
    long invld,
    double minval[],
    double maxval[],
    double nstd,
    double tol
)
{
extern	float	*sum;
extern	short	*vsum;
extern  float   *sqsum;
extern  float   *sum_buf;
extern  float   *sqsum_buf;
extern  short   *vsum_buf;

float		addval;
float		divv;
float		mean;
float		mydiff;
float		outval;
float           variance;
float		subval;

long    delta;
long	addlin;
long	bindex;
static long	bottom[KERNEL_SIZE];
long	curlin;
long	dellin;
long	ereal = EREAL;
long	j;
long	k;
long	l;
long	lpos;
long	oval4;
long	status;
long	tindex;
long	nbytes;			/* Number of bytes to allocate memory for */
long	*top;
long    inf_err = 1;

long    one = 1;
char    *prog_name = "DESPIKE"; /* program name            */

short	addvld;
short	subvld;

float *addbuf;
float *delbuf;
float *inbuf;
float *outbuf;

nbytes = (*ns + KERNEL_SIZE) * sizeof(float); 

/*
  Allocate memory for the add buffer
*/
top = (long *) malloc(KERNEL_SIZE * sizeof(long));
if (top == NULL)
    {
    c_eclose (&outfd);
    c_errmsg("Error allocating dynamic memory.","despike-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name, (char (*)[256])hosout, &one);
    }

addbuf = (float *) malloc(nbytes);
if (addbuf == NULL)
    {
    c_eclose (&outfd);
    c_errmsg("Error allocating dynamic memory.","despike-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name, (char (*)[256])hosout, &one);
    }

/*
  Allocate memory for the delete buffer
*/
delbuf = (float *) malloc(nbytes);
if (delbuf == NULL)
    {
    c_eclose (&outfd);
    c_errmsg("Error allocating dynamic memory.","despike-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name, (char (*)[256])hosout, &one);
    }

/*
  Allocate memory for the input buffer
*/
inbuf = (float *) malloc(nbytes);
if (inbuf == NULL)
    {
    c_eclose (&outfd);
    c_errmsg("Error allocating dynamic memory.","despike-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name,(char (*)[256])hosout, &one);
    }

/*
  Allocate memory for the output buffer
*/
outbuf = (float *) malloc(nbytes);
if (outbuf == NULL)
    {
    c_eclose (&outfd);
    c_errmsg("Error allocating dynamic memory.","despike-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name, (char (*)[256])hosout, &one);
    }

/*
	initialization
	--------------
*/
for (j = 0; j < yhalf; j++)
   {
   top[j] = yhalf + 1 - j;
   bottom[j]= nl - j - 1;
   }

tindex = 0;
bindex = 0;
dellin = 1;
addlin = yhalf + 2;


curlin = curent = 1;
c_startm ();

for (curlin = 1; curlin <= nl; curlin++, curent++)
   {

/*
	   get the input line 
	   ------------------
*/
   status = c_eread (&infd, bndin, &curlin, (unsigned char*)inbuf, &one);
   if (status  !=  *ns) 
      {
      c_eclose (&outfd);
      c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
               prog_name, (char (*)[256])hosout, &one);
      }

   lpos = 1 - xhalf;

   for (k = 0; k < *ns; k++)
      {
      divv = (float)*valsum -1;
            if ((inbuf[k] <  low) || (inbuf[k] > high)) 
               {
               delta = 0;
               }
      if ((inbuf[k] < low) || (inbuf[k] > high))
         {
          divv = (float)*valsum ;
         }

      if (divv  ==  0) 
         mean = 0;
      else
         mean = (*kersum - inbuf[k]) / divv;

/*
    variance = ((*varsum - (inbuf[k] * inbuf[k]))/divv) - 
               (((*kersum - inbuf[k]) * (*kersum - inbuf[k]))/(divv * divv));
*/
    variance = ((*varsum)/((double)*valsum))-
               ((*kersum * (*kersum))/((double)*valsum * ((double)*valsum)));
    mydiff = inbuf[k] - mean;
/* change nstd to nstd*nstd for check 6/25/90
*/
    if (((mydiff * mydiff)  > ((nstd*nstd) * variance)) && (abs(mydiff) > tol )) 
       {
       if (invld == 1) outval = 0;
       if (invld == 2) outval = mean;
       }
     else
       outval = inbuf[k];

/*
	      convert the computed output value to the sepcified type
	      -------------------------------------------------------
*/
      oval4 = (long)(outval + 0.5); 
      if (otype  ==  EREAL) 
         outbuf[k] = (float)outval;
      else if (otype  ==  ELONG) 
         outbuf[k] = (float)oval4;
      else if (otype  ==  EWORD) 
         {
         if (oval4  <  -32768)  oval4 = -32768;
         if (oval4  >   32767)  oval4 =  32767;
         outbuf[k] = (float)oval4;
         }
      else
         {
         if (oval4  <  0)   oval4 = 0;
         if (oval4  >  255) oval4 = 255;
         outbuf[k] = (float)oval4;
         } 

/*
	      reset kernel sum and sum valid values
	      -------------------------------------
*/
      if (k  <  *ns - 1) 
         {
         *kersum -= sum[lpos];
         *kersum += sum[lpos + ksamps];
         *valsum -= vsum[lpos];
         *valsum += vsum[lpos + ksamps];
         *varsum -= sqsum[lpos];
         *varsum += sqsum[lpos + ksamps];
         lpos++;
         } 
      }

/*
	   determine min and max values of output line
	   -------------------------------------------
*/
   if (min_max)
      c_pxmnmx ((long *)outbuf, ns, &minval[*bndout - 1], &maxval[*bndout - 1],
                  &ereal);

/*
	   write completed line to output image
	   ------------------------------------
*/
   status = c_ewrite (&outfd, bndout, &curlin, (unsigned char*)outbuf, &one);
   if (status  !=  *ns) 
      {
      c_eclose (&outfd);
      c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
               prog_name, (char (*)[256])hosout, &one);
      }
   
   if (curlin  <  nl) 
      {
/*
	      get next line to be deleted from the top of the kernel
	      ------------------------------------------------------
*/
      if (curlin  >=  yhalf + 1) 
         {
         status = c_eread (&infd, bndin, &dellin, (unsigned char*)delbuf, &one);
         if (status  !=  *ns) 
            {
            c_eclose (&outfd);
            c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name, (char (*)[256])hosout, &one);
            }
         dellin++;
         }
      else
         {
         status = c_eread (&infd, bndin, &top[tindex], (unsigned char*)delbuf, &one);
         if (status  !=  *ns) 
            {
            c_eclose (&outfd);
            c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name, (char (*)[256])hosout, &one);
            }
         tindex++;
         } 

/*
	      get next line to be added to the bottom of the kernel
	      -----------------------------------------------------
*/
      if ( (curlin + 1)  <=  (nl - yhalf) ) 
         {
         status = c_eread (&infd, bndin, &addlin, (unsigned char*)addbuf, &one);
         if (status  !=  *ns) 
            {
            c_eclose (&outfd);
            c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name, (char (*)[256])hosout, &one);
            }
         addlin++;
         }
      else
         {
         status = c_eread (&infd, bndin, &bottom[bindex], (unsigned char*)addbuf, &one);
         if (status  !=  *ns) 
            {
            c_eclose (&outfd);
            c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name, (char (*)[256])hosout, &one);
            }
         bindex++;
         } 

/*
	      reset the sum array and kernel sum value
	      ----------------------------------------
*/
      *varsum = 0;
      *kersum = 0;
      *valsum = 0;
      for (k = 1 - xhalf; k <= *ns + xhalf; k++)
         {
         addval = 1;
         subval = 1;
         addvld = 1;
         subvld = 1;
         set (k, &l, *ns);

/*
	         determine the value to be added
	         -------------------------------
*/
         addval = addbuf[l];
            if ((addval  <  low)  ||  (addval  >  high)) 
               {
               addval = 0;
               addvld = 0;
               } 

/*
	         determine the value to be subtracted
	         ------------------------------------
*/
         subval = delbuf[l];
            if ((subval  <  low)  ||  (subval  >  high)) 
               {
               subval = 0;
               subvld = 0;
               } 

         sum[k] = sum[k] - subval + addval;
         vsum[k] = vsum[k] - subvld + addvld;
         sqsum[k] = sqsum[k] - (subval * subval) + (addval * addval);
         if (k  <=  xhalf + 1) 
            {
            *kersum += sum[k];
            *valsum += vsum[k];
            *varsum += sqsum[k];
            } 
         }
      } 
   }

c_stoptm ();

free(addbuf);
free(delbuf);
free(inbuf);
free(outbuf);
free(sum_buf);
sum_buf = NULL;
free(sqsum_buf);
sqsum_buf = NULL;
free(vsum_buf); 
vsum_buf = NULL;
}
