/*
*************************************************************************
NAME: MEDIAN 

PURPOSE: applies a median filter to an image band.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    3.1          4/89   D. Hollaren   Added static to compile on PN9050
    4            9/99   D. Hames      Remove references to MAXNS and replace
                                      with ns to support images >2gig.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************

	arguments:
	name			description
	----			-----------
	bndin			current input band number
	bndout			output band number
	dtype			data type of input image
	hosout			output image host name
	ksamps			number of samples per line in kernel
	nl			number of lines to be processed
	ns			number of samples per line to be processed
	totpix			total number of pixels in the kernel
	maxval			maximum values for each band
	minval			minimum values for each band
	xhalf			half of the number of samples in the kernel
	yhalf			half of the number of lines in the kernel

	internal variables:
	name			description
	----			-----------
	bindex			the index into the "bottom" array
	bottom			the lines to read to miror at the bottom
	cntr			accumulates count of values
	count			count returned from pxhist
	hiend			high end array position
	iinline			counter for lines to be added sequentially
				to bottom of kernel
	loend			low end array position
	mid			middle of list
	status			function return status code
************************************************************************
*/
#include "despike.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

extern	long	*hist;
extern	long	*hist_buf;

extern	short	**image;
extern	short	*image_buf;
void median
(
    char hosout[],
    long nl,
    long *ns,
    long *bndin,
    long *bndout,
    long xhalf,
    long yhalf,
    double totpix,
    long *ksamps,
    long dtype,
    double minval[],
    double maxval[]
)
{
long	one = 1;
long    zero = 0;

char    *prog_name = "DESPIKE"; /* program name            */


long	bindex;
long	bottom[(KERNEL_SIZE_MED - 1) / 2];
long	cntr;
long	count;
long	curlin;
long	elong = ELONG;
long	eword = EWORD;
long	hiend;
long	hist_size;
long	iinline;
long	i;
long	j;
long	k;
long	length;
long	loend;
long	m;
long	mid;
long	status;

short	*medin;
short	*medout;

medin = (short *) malloc(*ns * sizeof(short));
if (medin  ==  NULL) 
   {
   c_eclose (&outfd);
   c_errmsg("Error allocating dynamic memory","despike-alloc",&one);
   c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
            prog_name, (char (*)[256])hosout, &one);
   }

medout = (short *) malloc(*ns * sizeof(short));
if (medout  ==  NULL) 
   {
   c_eclose (&outfd);
   c_errmsg("Error allocating dynamic memory","despike-alloc",&one);
   c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
            prog_name, (char (*)[256])hosout, &one);
   }
/*
	initialization
	--------------
*/
for (i = 0; i < yhalf; i++)
   bottom[i] = nl - i;

bindex = 0;

if (dtype  ==  EBYTE) 
   {
   loend = 0;
   hiend = 255;
   hist_size = 256;
   }
else
   {
   loend = -32768;
   hiend = 32767;
   hist_size = HIST_SIZE;
   } 

iinline = yhalf + 2;
mid = ((int) totpix - 1) / 2 + 1;

curlin = curent = 1;
c_startm ();

for (curlin = 1; curlin <= nl; curlin++, curent++)
   {
   for (k = 1; k <= *ns; k++)
      {
/*
	      determine the median value
	      --------------------------
*/
      cntr = 0;
      for (j = loend; j <= hiend; j++)
         {
         cntr = cntr + hist[j];
         if (cntr  >=  mid) break;
         }
      if (cntr  ==  0) j = 0;
      medout[k-1] = (short)j;

/*
	      reset for the next kernel
	      (for each line in the kernel, subtract the leftmost
	       pixel and add the next pixel to the right)
	      ---------------------------------------------------
*/
      if (k != *ns)
         for (j = 1 - yhalf; j <= 1 + yhalf; j++)
            {
            hist[image[j][k - xhalf]] = hist[image[j][k - xhalf]] - 1;
            hist[image[j][k+xhalf+1]] = hist[image[j][k+xhalf+1]] + 1;
            }
      }

/*
	   determine min and max values of output line
	   -------------------------------------------
*/
   if (min_max)
      c_pxmnmx ((long *)medout, ns, &minval[*bndout - 1], &maxval[*bndout - 1], 
		&eword);

/*
	   write completed line to output image
	   ------------------------------------
*/
   status = c_ewrite (&outfd, bndout, &curlin, (unsigned char*)medout, &one);
   if (status  !=  *ns) 
      {
      c_eclose (&outfd);
      c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
               prog_name, (char (*)[256])hosout, &one);
      }

   if (curlin  <  nl) 
      {
/*
	     shift the lines ahead in the array  
            removing the top line of the kernel
	     -----------------------------------
*/
      length = *ns + *ksamps - 1;
      for (j = 1 - yhalf; j <= yhalf; j++)
         c_pxcopy ((unsigned char *)&image[j + 1][1 - xhalf], 
			(unsigned char *)&image[j][1 - xhalf], &eword, &length);

/*
	     get the line to be added to the bottom of kernel
	     ------------------------------------------------
*/
      if (curlin  <  (nl - yhalf)) 
         {
         status = c_eread (&infd, bndin, &iinline, (unsigned char*)medin, &one);
         iinline++;
         }
      else
         {
         status = c_eread (&infd, bndin, &bottom[bindex], (unsigned char*)medin, &one);
         bindex++;
         } 
      if (status  !=  *ns) 
         {
         c_eclose (&outfd);
         c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                  prog_name, (char (*)[256])hosout, &one);
         }

/*
	     copy image line into array
	     --------------------------
*/
      c_pxcopy ((unsigned char *)medin, (unsigned char *)&image[yhalf + 1][1], 
			&eword, ns);

/*
	     do the left edge mirror
	     -----------------------
*/
      for (m = 1 - xhalf; m <= 0; m++)
         image[j][m] = image[j][abs(m - 2)];

/*
	     do the right edge mirror
	     ------------------------
*/
      for (m = *ns + 1; m <= *ns + xhalf; m++)
         image[j][m] = image[j][(2 * *ns) - m];

/*
	     do the histogram for this new kernel
	     ------------------------------------
*/
      if (dtype == EBYTE)
         c_pxfill ((unsigned char *)hist, &elong, &hist_size, 
                   	(unsigned char *)&zero);
      else
         c_pxfill ((unsigned char *)hist_buf, &elong, &hist_size, 
			(unsigned char *)&zero);

       for (k = 1 - yhalf; k <= yhalf + 1; k++)
         {
         status = c_pxhist(ksamps, (long *)&image[k][1 - xhalf], &eword, 
                           hist, &count);
         if (status  !=  E_SUCC) 
            {
            c_eclose (&outfd);
            c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
                     prog_name, (char (*)[256])hosout, &one);
      }
         }

      } 
   }

c_stoptm ();
free (hist_buf);
free (image_buf);
}
