/*
*************************************************************************
NAME: PREHL 

PURPOSE: prepares the initial kernel for high or low filters.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    3.1          4/89   D. Hollaren   Added static to compile on PN9050
    4            9/99   D. Hames      Remove references to MAXNS and replace
                                      with ns to support images >2gig.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************

	arguments:
	name		description
	----		-----------
	curbnd		current input band number
	high		high end of valid data range
	hosout		out image host name
	kersum		data sum of the kernel
	low		low end of valid data range
	ns		number of samples per line to be processed
	valsum		the valid number of samples which contributed
			to the kernel sum
	varsum		sum of squared data in kernel
	kersum		sum of data in the kernel
	xhalf		half of the number of samples in the kernel
	yhalf		half of the number of lines in the kernel

	internal variables:
	name		description
	----		-----------
	buffer		input buffer
	status		function return status code
************************************************************************
*/
#include "despike.h"
#include "util.h"
#include "protos.h"

float	*sum;                           /* pointer to the middle of sum   */
                                        /* array                          */
float	*sum_buf;  /* array of kernel column sums     */

float   *sqsum_buf; /* array of kernel column square sums */
float   *sqsum;                         /* pointer to the middle of sqsum */
short	*vsum;                          /* pointer to the middle of vsum  */
                                        /* array                          */
short	*vsum_buf; /* array of the number of valid    */
                                        /* pixels for the kernel columns   */


void prehl
(
    char hosout[],
    long xhalf,
    long yhalf,
    long *ns,
    long *curbnd,
    double high,
    double low,
    float *varsum,
    float *kersum,
    long *valsum
)
{
long    one = 1;

char    *prog_name = "DESPIKE"; /* program name            */

float	*buffer;

long	j;
long	k;
long	l;
long	status;

sum_buf = (float *) malloc((*ns + KERNEL_SIZE) * sizeof(float));
if (sum_buf == NULL)
   {
   c_eclose (&outfd);
   c_errmsg("Error allocating dynamic memory","despike-alloc",&one);
   c_abort (&parblk, "Fatal error encountered", "despike-fatal",
            prog_name, (char (*)[256])hosout, &one);
   }
sum = &sum_buf[(KERNEL_SIZE - 1) / 2];

vsum_buf = (short *) malloc((*ns + KERNEL_SIZE) * sizeof(short));
if (vsum_buf == NULL)
   {
   c_eclose (&outfd);
   c_errmsg("Error allocating dynamic memory","despike-alloc",&one);
   c_abort (&parblk, "Fatal error encountered", "despike-fatal",
            prog_name, (char (*)[256])hosout, &one);
   }
vsum = &vsum_buf[(KERNEL_SIZE - 1) / 2];

sqsum_buf = (float *) malloc((*ns + KERNEL_SIZE) * sizeof(float));
if (vsum_buf == NULL)
   {
   c_eclose (&outfd);
   c_errmsg("Error allocating dynamic memory","despike-alloc",&one);
   c_abort (&parblk, "Fatal error encountered", "despike-fatal",
            prog_name, (char (*)[256])hosout, &one);
   }
sqsum = &sqsum_buf[(KERNEL_SIZE -1) / 2];

buffer = (float *) malloc(*ns * sizeof(float));
if (buffer  ==  NULL) 
   {
   c_eclose (&outfd);
   c_errmsg("Error allocating dynamic memory","despike-alloc",&one);
   c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
            prog_name, (char (*)[256])hosout, &one);
   }

sum = &sum_buf[(KERNEL_SIZE - 1) / 2];
vsum = &vsum_buf[(KERNEL_SIZE - 1) / 2];
sqsum = &sqsum_buf[(KERNEL_SIZE -1) / 2];
/*
	initialization 
	--------------
*/
*kersum = 0.0;
*valsum = 0;
*varsum = 0.0;
for (j = -xhalf; j <= *ns + xhalf; j++)
   {
   sum[j] = 0.0;
   vsum[j] = 0;
   sqsum[j] = 0.0;
   }

/*
	sum up the valid column values for the initial kernel (including
       mirroring), and keep track of the number of valid values found
       for each column
	----------------------------------------------------------------
*/
for (j = 1; j <= yhalf + 1; j++)
   {
   status = c_eread (&infd, curbnd, &j, (unsigned char*)buffer, &one);
   if (status  !=  *ns) 
      {
      c_eclose (&outfd);
      c_abort (&parblk, "Fatal error encountered", "despike-fatal", 
               prog_name, (char (*)[256])hosout, &one);
      }

   for (k = 1 - xhalf; k <= *ns + xhalf; k++)
      {
      set (k, &l, *ns);
      if (buffer[l] >=  low  &&  buffer[l] <=  high) 
         {
         sum[k] += buffer[l];
         vsum[k]++;
         sqsum[k] += (buffer[l] * buffer[l]);
         if (j  !=  1) 
            {
            sum[k] += buffer[l];
            vsum[k]++;
            sqsum[k] += (buffer[l]*buffer[l]);
            } 
         } 
      }
   }

/*
	calculate the sum of the kernel and the number of valid values
	--------------------------------------------------------------
*/
for (k = 1 - xhalf; k <= xhalf + 1; k++)
   {
   *kersum += sum[k];
   *valsum += vsum[k];
   *varsum += sqsum[k];
   }
free(buffer);
}
