/******************************************************************************
NAME:   			PROCES 

PURPOSE:    To map specified values within an image. The input 
	    image(s) can be sub-windowed and sub-banded when copied to 
	    the output image. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   R. McKinney	   9/90 	  EDC		Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   FASTMAP must be run under TAE.

PROJECT:			LAS

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   The one line of each band and each image are processed at the same time.
   Save the minmax values if $MINMAX was set to "YES"
   Update the output ddr
   Close the input and output image files
   ALGORITHM REFERENCES:		none
******************************************************************************/

#include "las.h"
#include "util.h"
#include "protos.h"

void proces
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*window)[4],		/* window specification of input images      */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long totbnd,			/* total number of bands to be processed     */
    long odtype,                 /* data type of the image                    */
    long msgtim,			/* elapsed time between display messages     */
    long from,                   /* data value to be mapped                   */
    long to                     /* value to be inserted                      */
)

{

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
extern struct FDESC *fdesc[MAXIMG+1];	
                                 /* image file descriptors used for image I/O */
extern struct GDESC *gdesc;		 /* group descriptor used for image I/O       */
extern long offset1[MAXIMG*MAXBND];/* offsets into thebuf for input bands    */
extern long curent;		/* current processing line		     */

long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long	image;			/* loop index counting input images          */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long    sample;                 /* sample                                    */
long	size = 0;			/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    value;                  /* current pixel value                       */
long	zero = 0;		/* value of zero                             */

char    *thebuf;		/* image I/O buffer                          */

/* Open input image(s)
----------------------*/
access = IUPDATE;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&odtype,&zero,&zero);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","fastmap-fatal",&fatal);
   band += nbands[image];
   }

fdesc[nmrimg+1] = 0;

/* Allocate I/O buffer
----------------------*/
if ((thebuf = c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","fastmap-alloc",&nonfatal);
   c_clean(vblock,"fastmap",hosout,&zero);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   c_clean(vblock,"fastmap",hosout,&zero);

/* Setup for data type
----------------------*/
switch(odtype)
   {
   case EBYTE:  size = 1; break;
   case EWORD:  size = 2; break;
   case ELONG:	size = 4; break;
   case EREAL:  size = 4; break;
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("FASTMAP","Processed","line","bands"," ",&nl,
	&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();


/* Main processing loop
-----------------------*/
for (curlin = 0;curlin < nl;curlin++)
   {
   if (c_estep(&gdesc) != E_SUCC)
      c_clean(vblock,"fastmap",hosout,&zero);
   for (band=0;band<totbnd;band++)
      {
      for (sample=0;sample<ns;sample++)
          {
          value = *(thebuf + (*(offset1+band)+sample) * size);
          if (value == from)
             *(thebuf + (*(offset1+band)+sample) * size) = (char) to;
          }
      }
   curent++;
   }

/* Final write
--------------*/
if (c_estep(&gdesc) != E_SUCC)
      c_clean(vblock,"fastmap",hosout,&zero);

/* Turn off display messages
----------------------------*/
c_stoptm();


/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","fastmap-warn",&one);

/* Free buffer
--------------*/
free(thebuf);

return;
}
