/****************************************************************************
NAME:                           GEOTPL

PURPOSE:  Creates a TPL geo_coord from ref_coord & valid DDR proj info

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  5.0    1/90   D. Steinwand  CSB      LAS 5.0 C conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
        Initialize LAS processing
        Get parameters from the user

        Close all files
        Call complete & exit
*****************************************************************************/
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "proj.h"
#include "geotpl.h"

void main
(
  void
)

{
    struct PARBLK vblk;             /* TAE parameter block */
    struct TPLHEAD tpl_header;      /* TPL file header record */
    struct TPLDATA tpl_data;        /* Tie point location data record */
    double pcoef[15];
    double zero_coef[15];
    double corners[8];
    double coef[6];
    double proj_y,proj_x;
    int *in_tpl;                    /* Input tie point location file desc */
    int *out_tpl;                   /* Output tie point location file desc */
    long ptyp = TPL;                /* Type of tie point file */
    long acc;                       /* File access mode */
    long i;                         /* Loop counter */
    long one = 1;                   /* Constant of 1 */
    long zero = 0;
    long mode;
    long fatal = -1;
    long status;
    long image[8];
    long null_zone = 62;
    long code;
    long units;
    long zone;
    long datum;
    long prtprm[2];                 /* Print flags for c_transinit */
    long four = 4;                  /* Constant of 4 */
    long std_out;                   /* Dummy variable to standard output */
    long flag = TRUE;
    char fname[CMLEN];              /* Input Tie Point Location file name */
    char out[CMLEN];                /* Output Tie Point Location file name */


    c_inlas(&vblk, &std_out);

    /* Get processing parameters from the user
       ---------------------------------------*/
    mode = IREAD;
    if(c_getnam(&vblk, "INTL", &zero, fname, "tpl", &mode) != E_SUCC)
        c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);

    mode = IWRITE;
    if(c_getnam(&vblk, "OUTTL", &zero, out, "tpl", &mode) != E_SUCC)
        c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);

    /* Get DDR projection & location info
       ----------------------------------*/
    if(use_ddr_info(&vblk,&code,&units,&zone,&datum,pcoef,corners,image) 
                   != E_SUCC)
        c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);

    /* Calculate coefficients mapping from image coords to proj coords
       ---------------------------------------------------------------*/
    if(c_rtcoef(corners, image, coef, &flag) != E_SUCC)
        c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);

    /* Process header information
       --------------------------*/
    acc = IREAD; ptyp = TPL;
    if(c_ptopen((struct PTDESC **)&in_tpl, fname, &ptyp, &acc, 
                (long *)&tpl_header) != E_SUCC)
        c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);

    /* Make projection info GEOGRAPHIC
       -------------------------------*/
    for (i = 0; i < 8; i++) tpl_header.corners[i] = 0.0;
    tpl_header.pdist[0] = tpl_header.pdist[1] = 0.0;
    for (i = 0; i < 15; i++) tpl_header.projprms[i] = 0.0;
    tpl_header.proj_valid = GEO_VALID;
    tpl_header.code = 0;
    tpl_header.zone = 62;
    tpl_header.datum = 0;
    strcpy(tpl_header.units,"degrees");

    /* Report projection info to the user
       ----------------------------------*/
    for(i=0; i<15; i++) zero_coef[i]=0.0;
    prtprm[GEO_TERM] = TRUE;
    prtprm[GEO_FILE] = FALSE;
    if(c_transinit(&code, &units, &zone, &datum, pcoef, &zero, &four, 
            &null_zone, &datum, zero_coef, prtprm, "\0") != E_SUCC) 
        c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);

    /* Open new output file and write modified header to it
       ----------------------------------------------------*/
    acc = IWRITE;
    if(c_ptopen((struct PTDESC **)&out_tpl, out, &ptyp, &acc, 
               (long *)&tpl_header) != E_SUCC)
        c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);

    for (;;)
    {
        status = c_ptio((struct PTDESC **)&in_tpl, (long *)&tpl_data);
        if (status == E_EOF) break;
        if (status == E_FAIL) 
            c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);

        proj_y = (coef[0] * tpl_data.ref_coord[0]) + 
            (coef[1] * tpl_data.ref_coord[1]) + coef[2];
        proj_x = (coef[3] * tpl_data.ref_coord[0]) + 
            (coef[4] * tpl_data.ref_coord[1]) + coef[5];

        if(c_trans(&code, &units, &zero, &four,  
                &proj_x, &proj_y, &(tpl_data.geo_coord[1]), 
                &(tpl_data.geo_coord[0])) != E_SUCC) 
            c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);

        if(c_ptio((struct PTDESC **)&out_tpl, (long *)&tpl_data) != E_SUCC)
            c_errmsg("Fatal error encountered","geotpl-fatal",&fatal);
    }

    c_ptclse((struct PTDESC **)&out_tpl);
    c_ptclse((struct PTDESC **)&in_tpl);
    c_complt("GEOTPL", (char (*)[256])out, &one);
}

/*****************************************************************************
NAME                              use_ddr_info

PURPOSE    Reads projection & framing information from an image DDR

PROGRAM HISTORY
VERSION    PROGRAMMER      DATE     CODE/CONT   REASON
-------    ----------      ----     ---------   ------
  5.0      D. Steinwand    7/89        EDC      Original development

ALGORITHM DESCRIPTION
        Retrieve the name of the input image via getin()
        Get the DDR information
        If DDR information is valid,
           Pack output projection parameters from the DDR projection information
        Return
*****************************************************************************/
int use_ddr_info
(
    struct PARBLK *vblk,        /* TAE parameter block */
    long *code,
    long *units,
    long *zone,
    long *datum,
    double *pcoef,
    double *corners,            /* Corner projection coordinates */
    long *image                 /* Corner image coordinates */
)

{
    struct DDR ddr;         /* DDR structure */
    long in_wind[4];        /* Image window specification */
    long i;                 /* Loop counters */
    long zero = 0;          /* Constant of zero */
    long one = 1;           /* Constant of one for c_errmsg routine */
    long num_img;           /* Number of images input:  0 or 1 */
    long bands[MAXBND+1];   /* Band numbers */
    long nbands;            /* Number of bands */
    long totbands;          /* Total number of bands */
    long same;              /* Same window flag */
    long qflag[2];          /* Option flags */
    char in_image[CMLEN];   /* Image file name */
    char tdum[CMLEN];       /* Tae name */

    /* Check for projection information and place it in the pro structure
       if it exists.  
       -------------*/
    if(c_getin(vblk,"IN",&zero,(char (*)[256])tdum,(char (*)[256])in_image,
            &num_img,(long (*)[4])in_wind,(long (*)[257])bands,&nbands,
            &totbands,&one,&same,(long (*)[2])qflag) != E_SUCC) return(E_FAIL);
            if (c_getddr(in_image, &ddr) != E_SUCC) 
            {
                c_errmsg("Projection information not available from DDR",
                        "remap-noproj",&one);
                return(E_FAIL);
            }
            else
            {
                image[0] = image[1] = 1;
                image[2] = ddr.nl;
                image[3] = 1;
                image[4] = 1;        
                image[5] = ddr.ns;
                image[6] = ddr.nl;
                image[7] = ddr.ns;

                if ((ddr.valid[0] == INVAL) || (ddr.valid[3] == INVAL) || 
                        (ddr.valid[4] == INVAL))
                {
                    c_errmsg("Projection information not available from DDR",
                            "remap-noproj",&one);
                    return(E_FAIL);
                }
                else
                {
                    *code = ddr.proj_code;
                    if (ddr.valid[1] == INVAL) *zone = 62;
                    else *zone = ddr.zone_code;
                    if (ddr.valid[2] == INVAL) *datum = 0;
                    else *datum = ddr.datum_code;
                    *units = c_string2unit(ddr.proj_units);
                    if (*units == E_FAIL)
                    {
                        c_lasmsg(FALSE, "geotpl-warn", 
                                "Projection units %s are invalid", 
                                 ddr.proj_units);
                    }

                    for (i = 0; i < 15; i++)
                        pcoef[i] = ddr.proj_coef[i]; 

                    if (ddr.valid[6] == INVAL)
                        for (i = 0; i < 8; i++)
                            corners[i] = 0.0;
                    else
                    {
                        corners[0] = ddr.upleft[0];
                        corners[1] = ddr.upleft[1];
                        corners[2] = ddr.loleft[0];
                        corners[3] = ddr.loleft[1];
                        corners[4] = ddr.upright[0];
                        corners[5] = ddr.upright[1];
                        corners[6] = ddr.loright[0];
                        corners[7] = ddr.loright[1];
                    }
                }
            }

            return(E_SUCC);
}
