/*****************************************************************************
NAME				 fit

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

int fit
(
    struct GEOGRID *grid		/* Geometric mapping grid */
)
{
long row,col,i;
long index;
long size;
double *reflines,*refsamps,*srchlines,*srchsamps;

size = (long) (grid->nrows*grid->ncols*sizeof(double));
reflines = (double *)malloc(size);
refsamps = (double *)malloc(size);
srchlines = (double *)malloc(size);
srchsamps = (double *)malloc(size);

for (i=0, row = 0; row < grid->nrows; row++)
   for (col = 0; col < grid->ncols; i++,col++)
     {
     index = row * grid->ncols + col;
     reflines[i] = grid->out_lines[row],
     refsamps[i] = grid->out_samps[col],
     srchlines[i] = grid->in_lines[index],
     srchsamps[i] = grid->in_samps[index];
     }
fitpts(reflines,refsamps,srchlines,srchsamps,grid->nrows*grid->ncols,4);
free(reflines);
free(refsamps);
free(srchlines);
free(srchsamps);
return(E_SUCC);
}


/****************************************************************************
NAME:				FITPTS

PURPOSE:  Preforms a least-squares fit and calculates residual errors

****************************************************************************/
int fitpts
(
    double *refl,
    double *refs,
    double *srcl,
    double *srcs,
    long npts,			/* Number of points in TPL file */
    long degree			/* Degree of polynomial used */
)
{
double *v;			/* Work buffer for LSQ fit */
double *b1;			/* Vector containing X coordinates */
double *b2;			/* Vector containing Y coordinates */
double *A;			/* Matrix A (used for LSQ fit) */
double x,y;			/* Calculated x & y locactions */
double max_x,max_y;
double xloc = 0.0,yloc = 0.0;
double delta_x,delta_y;		/* Absolute value of x & y residuals */
double rms_acc; 		/* Accumulator for RMSe calc */
double x_residual;		/* Residual in X */
double y_residual;		/* Residual in Y */
double rmse;
long nl=2340;
long ns=3264;
long npts_used;
long i,k;			/* Loop counters */
long size;			/* Size of vectors (based on poly degree) */

if(degree==1)size=3;
else if(degree==2)size=6;
else if(degree==3)size=10;
else if(degree==4)size=15;
else {degree=1;size=3;}
A = (double *)malloc(2 * npts * size * sizeof(double));
if (A == NULL) return(NO_SPACE);
b1 = (double *)malloc(npts * sizeof(double));
if (b1 == NULL) return(NO_SPACE);
b2 = (double *)malloc(npts * sizeof(double));
if (b2 == NULL) return(NO_SPACE);
v = (double *)malloc(npts * sizeof(double));
if (v == NULL) return(NO_SPACE);

/* Loop on tie point pairs.  If the tie point is active, use it in the
   fit calculations; first for x, then for y.
  ------------------------------------------*/
for(k=0,i=0; i < npts; i++)
      {
      A[k] =               1.0;
      A[k+npts] =     refs[i];
      A[k+(2*npts)] = refl[i];
      if(degree>1)
           {
           A[k+(3*npts)]=refs[i]*refs[i];
           A[k+(4*npts)]=refs[i]*refl[i];
           A[k+(5*npts)]=refl[i]*refl[i];
           }
      if(degree>2)
	   {
           A[k+(6*npts)]=refs[i]*refs[i]*refs[i];
           A[k+(7*npts)]=refs[i]*refs[i]*refl[i];
           A[k+(8*npts)]=refs[i]*refl[i]*refl[i];
           A[k+(9*npts)]=refl[i]*refl[i]*refl[i];
           }
      if(degree>3)
           {
           A[k+(10*npts)]=refs[i]*refs[i]*refs[i]*refs[i];
           A[k+(11*npts)]=refs[i]*refs[i]*refs[i]*refl[i];
           A[k+(12*npts)]=refs[i]*refs[i]*refl[i]*refl[i];
           A[k+(13*npts)]=refs[i]*refl[i]*refl[i]*refl[i];
           A[k+(14*npts)]=refl[i]*refl[i]*refl[i]*refl[i];
           }

      b1[k] = srcs[i];
      b2[k] = srcl[i];
      k++;
      }
QR(A, npts, size, v);
QR_solve(A, npts, size, v, b1, 0);
QR_solve(A, npts, size, v, b2, 0);

/* Calculate Residuals
  -------------------*/
rms_acc = 0.0;

/* Loop on tie point pairs.  If the tie point is active, evaluate the
   polynomial and calculate residuals & rms errors
  -----------------------------------------------*/
for(npts_used = 0, max_x = max_y = i = 0; i < npts; i++)
   {
   if(srcs[i] > 0 && srcl[i] > 0 && srcs[i] < ns && srcl[i] < nl)
       {
       x = c_eval(&degree,b1,&(refs[i]),&(refl[i]));
       y = c_eval(&degree,b2,&(refs[i]),&(refl[i]));
       x_residual = srcs[i] - x;
       y_residual = srcl[i] - y;
       delta_x = fabs(x_residual);
       delta_y = fabs(y_residual);
       if (delta_x > max_x) {max_x = delta_x; xloc=x;}
       if (delta_y > max_y) {max_y = delta_y; yloc=y;}
       rms_acc += (delta_x * delta_x) + (delta_y * delta_y);
       npts_used++;
       }
   }

/* Calculate average residuals
  ---------------------------*/
rmse = sqrt(rms_acc / npts_used);

printf("Stats are reported over the input image area only\n");
printf("RMSe: %f\n", rmse);
printf("Max errors X:  %f at %f  Y:  %f at %f\n",max_x,xloc,max_y,yloc);
printf("Polynomial mapping samples:\n");
for(i=0;i<size;i++)printf("a%1d:  %e\n",i,b1[i]);
printf("Polynomial mapping lines:\n");
for(i=0;i<size;i++)printf("b%1d:  %e\n",i,b2[i]);
free(A); free(b1); free(b2); free(v);
return(E_SUCC);
}
