/* hazebias.c by David Zokaites.  Reads a histogram file to determine haze 
bias.  Haze bias = min histogram spike.  This routine is based on the EDIPS 
haze bias routine called "spike detection" in EDIPS module IPRAC.  "Spike 
detection" means finding the start of significant amounts of data in the lower 
part of a histogram.  

For Edips, SPIKE_THRESHOLD = pixels_processed / 100,000 * 6.  pixels_processed 
refers to uncorrected pixels, is about 7.8 million, and varies some from 
scene to scene.  The number of pixels in scenes output from Edips is 
2983 * 3548.  Inserting this into the above equation gives SPIKE_THRESHOLD =
635.  */ 
	

#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"

/* define constants */
#define NBANDS            3  /* these tapes have 3 bands */ 
#define NSPIKES           4  /* 4 consecutive spikes or brightness values*/
#define SPIKE_THRESHOLD 635  /* each bv must have >= 635 pixels */ 
#define TABLE_LIMIT     127  /* max entry in histogram is 127 */ 

/* declare global vars */ 
    long  local_histogram[NBANDS][128]; /* histograms of all bands on tape */ 
    long  fatal = -1;                   /* fatal error */
    long  success = 1;                  /* successful completion */ 
    char  message[100];                 /* message for user */ 

int readhist
(
    char *infile  /* name of input histogram file */
);

int getbias
(
    char *outfile,  /* output file name */
    long band      /* band of image to process */
);


int writebias
(
    long bias,      /* haze bias */
    char *outfile  /* output file name */
);


int main
(void)
{

/* declare vars needed by tae */ 
    long   std_out;            /* standard output */ 
    struct PARBLK   vblock;    /* variable block */ 
    struct VARIABLE *in;       /* pointer to var within parameter block */

/* declare parms passed to this routine.  length = length in calling proc + 1
    for null termination.  */ 
    char  infile[101];    /* input histogram file */ 
    char  outbase[101];   /* base of output file names */ 

/* declare local variables */ 
    char  outfiles[NBANDS][CMLEN];  /* output file names */ 
    long  nbands = NBANDS;        /* output files, and bands to process */ 
    long  index;                  /* for loop index */ 

/* las initialization */
    c_inlas (&vblock, &std_out); 

/* get parameters from tae */
    in = p_find (&vblock, "infile");   strcpy (infile,SVAL(*in,0) );
    in = p_find (&vblock, "outbase");  strcpy (outbase,SVAL(*in,0) );

/* read histograms from file */
    if (readhist (infile) < 0)  return (fatal);  

/* determine haze bias, write bias to file */
    for (index=0; index<NBANDS; index++)
    {
	sprintf ( outfiles[index], "%s%d%s", outbase, index+1, ".txt"); 
        if (getbias (outfiles[index], index) < 0 )  return (fatal);  
    }

/* end program */ 
    c_complt("HAZEBIAS", outfiles, &nbands); 

    return EXIT_SUCCESS;
}




/* function to read histograms from input file */
int readhist
(
    char *infile  /* name of input histogram file */
) 
{
    FILE  *pointer;    /* input file pointer */ 
    long  band, bv;    /* band and bv we are reading */
    char  string[132]; /* string in file */ 

    /* open input file */ 
        pointer = fopen (infile, "r");
	if (pointer == NULL)
	{
	    sprintf (message, "Unable to open file %s for read access.", 
	     infile); 
            c_errmsg (message, "HAZEBIAS-FATAL", &fatal); 
	    return (fatal); 
	}

    /* read histograms */ 
        for (band=0;  band<NBANDS;  band++)
	{
	/* read over junk */
	    for ( ; fgets(string,132,pointer) != NULL; )
		if ( strncmp(string, "   -----",8) == 0 )
		    break;
	/* read histograms */ 
	    for (bv=0;  fscanf(pointer,"%*d %d %*s %*s %*s",  
	     &(local_histogram[band][bv]))!= EOF && bv<=TABLE_LIMIT;  bv++);
	}
    return (success);
}



/* function to find bias from histogram */
int getbias
(
    char *outfile,  /* output file name */
    long band      /* band of image to process */
) 
{
/* declare local variables */
    long ftemp, ftemp1; /* # of pixels in two histogram bins */
    long spike_threshold = SPIKE_THRESHOLD * 2; /* # pixels in pair of spikes */
    long min_spike = TABLE_LIMIT;               /* min histogram spike */ 
    long cutoff_value;                          /* haze bias */ 
    long index, index1;                         /* for loop indices */ 
    long limit;                                 /* limit to for loop */ 

/* determine min spike or haze bias */ 
    for  (index=1; index<=TABLE_LIMIT; index+=2) 
    {
	ftemp = local_histogram[band][index] + local_histogram[band][index+1];
	if (ftemp >= spike_threshold) 
	{
	    min_spike = index; 
	    index = TABLE_LIMIT; 
	    limit = min_spike + NSPIKES + 1;
	    for (index1=min_spike; index1<=limit; index1+=2)
	    {
		ftemp1 = local_histogram[band][index1] + 
		  local_histogram[band][index1+1]; 
		if (ftemp1 < spike_threshold)
		{
		    index = index1;
		    min_spike = TABLE_LIMIT;
		    break;
		}
	    }
	}
    }
    cutoff_value = min_spike -1; 

    /* write bias to file */
	if (writebias(cutoff_value,outfile) < 0) return (fatal); 

    /* tell user what haze shift was */ 
	sprintf (message, "Determined a haze bias of %d for band %d", 
	 cutoff_value, band+1); 
	c_errmsg(message, "HAZEBIAS-INFO", &success); 

    return (success); 
}





/* function to write haze biases to a file */
int writebias
(
    long bias,      /* haze bias */
    char *outfile  /* output file name */
)
{
    long  from1, from2;  /* from values for map proc */
    long  to1,   to2;    /* to values for map proc */ 
    FILE  *pointer;      /* file pointer */ 

    /* calculate from and to values */ 
	from1 = bias;  
	to1   = 0;     /* shifts bv with a value of bias to 0 */ 
	from2 = TABLE_LIMIT;          /* shift all image data */ 
	to2   = TABLE_LIMIT - from1;  /* shift data by value of bias */ 

    /* open output file */
	pointer = fopen (outfile, "w");
	if (pointer == NULL)
	{
	    sprintf (message, "Unable to open temp file %s of map values",
	    outfile); 
	    c_errmsg (message, "HAZEBIAS-FATAL", &fatal); 
	    return (fatal); 
        }

    /* write to file */
	fprintf(pointer,"%3d %3d \n",from1,to1);
	fprintf(pointer,"%3d %3d \n",from2,to2);
	  
     /* close file */
	 if (fclose(pointer) == EOF)
	 {
	     sprintf (message, "Error closing temp file %s of map values",
	      outfile); 
	     c_errmsg (message, "HAZEBIAS-FATAL", &fatal); 
 	     return (fatal);
	 }

    return (success); 
}


