C     *************************************************************************
C     NAME        	    IBIS2GBL3  		
C
C     PURPOSE      Retrieves particular values specified by the row and 
C		   column index from an IBIS tabular file and sets
C	  	   the indicated global variables with the appropriate
C		   values.  The current values of the globals ($OFFL,
C		   $OFFS, $XNL, $XNS, $MEANZ) are also saved in a
C		   global parameter file so that if needed they can be
C		   restored in a later session by running SETGBLS.
C
C     PROGRAM HISTORY
C	PROGRAMMER	  DATE		REASON
C	----------	  ----		------
C	K. Zanter       Dec. 1986       PRF #3092	 
C       K. Zanter       March l987      A change in how the parameter file 
C					is created was made so that the file
C					can be transferred using NCP.
C	B. Ailts	Nov. 1988	LAS 5.0 retrocode
C       E. Hames        Jan  2000       Remove 50000 limits from parameters
C                                       $MOSNL,$MOSNS,$MOSOFFL,$MOSOFFS,$INL and
C					$INS in lamgblr.pdf. Modified to support
C					images > 2gig. SRF #893
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	IBIS2GBL3 must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins by getting the user-specified parameters
C     and the current values of the global variables.  The specified
C     values are then retrieved from the IBIS file, and the necessary
C     globals are updated.  The values of all the globals are then
C     written to a parameter file and passed out to the Terminal Monitor.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
	program ibis2gbl3

	include 'las.fin'

	include 'lams.fin'

	integer*4	column(5)
        integer*4	frame
	real*4	ibisvl(5)
	real*4	meanz(MMXIMG)
	integer*4	ncols
	integer*4	ngbls
	integer*4	nrows
	real*4	offl(MMXIMG)
	real*4	offs(MMXIMG)
	integer*4	row(5)
	integer*4 	stdout

      	integer*4 vblock(XPRDIM)

	real*4	xnl(MMXIMG)
	real*4	xns(MMXIMG)

	character*(CMLEN)	infile
	character*(CMLEN)	pfile
	character*(CMLEN)	tname

	character*6	gbls(5)

C	Initialize the parameter block
C	------------------------------
	call inlas(vblock,stdout)

C	get user-specified parameters and global variables
C	--------------------------------------------------
	call getpar(vblock,infile,frame,pfile,gbls,ngbls,row,nrows,column,
     +		    ncols,offl,offs,xnl,xns,meanz)

C	get values from the IBIS file
C	------------------------------
	call rdibis(infile,row,nrows,column,ncols,ibisvl)

C	update the specified globals
C	----------------------------
	call setup(ibisvl,gbls,ngbls,frame,offl,offs,xnl,xns,meanz)

C	create the parameter file
C	-------------------------
	call makepf(vblock,pfile,offl,offs,xnl,xns,meanz)

C	pass out the globals 
C	--------------------
	call gblout(offl,offs,xnl,xns,meanz)

C	output completion message
C	-------------------------
        tname = pfile
	call complt('IBIS2GBL3',tname,1)
	end
C	***********************************************************************
C	   GETPAR gets the user-specified parameters and the values of
C	   the global variables.
C	***********************************************************************
	subroutine getpar(parblk,infile,frame,pfile,gbls,ngbls,row,
     .			  nrows,column,ncols,offl,offs,xnl,xns,meanz)

	include 'las.fin'

	include 'lams.fin'

	integer*4	column(1)
	integer*4	count
        integer*4	frame
	integer*4	getnam
	integer*4	length(5)
	real*4	meanz(1)
	integer*4	mode
	integer*4	ncols
	integer*4	ngbls
	integer*4	nrows
	real*4	offl(1)
	real*4	offs(1)
        integer*4	parblk(1)
	integer*4	row(1)
	integer*4	status
	real*4	xnl(1)
	real*4	xns(1)

	character*(CMLEN)	msg
	character*(CMLEN)	infile
	character*(CMLEN)	pfile

	character*6	gbls(1)

C	get the input IBIS file name 
C	----------------------------
	mode = IREAD
	status = getnam(parblk,'INFILE',1,infile,'ibis_tab',mode)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','ibis2gbl3-fatal',-1)
	endif

C	get the frame number
C       --------------------
	call xrintg(parblk,'FRAMNUM',1,frame,count,status)

C	get the global parameter file name
C       ----------------------------------
	mode = IUPDAT
	status = getnam(parblk,'PARFILE',1,pfile,'txt',mode)

C	get the globals names and row,column locations
C	----------------------------------------------
	call xrstr(parblk,'PARNAME',5,gbls,length,ngbls,status)
	call xrintg(parblk,'ROWNUM',5,row,nrows,status)
	call xrintg(parblk,'COLNUM',5,column,ncols,status)

C	verify that the number of rows, number of columns, and
C       number of global names are all the same
C	------------------------------------------------------
100	FORMAT('The number of values specified for the ROWS, COLNUM',
     +	       ' and PARNAME parameters are not equal')

	if (ncols .ne. nrows) then
	   write (msg,100)
	   call errmsg(msg,'ibis2gbl3-unequal',1)
	   call errmsg('Fatal error encountered','ibis2gbl3-fatal',-1)
	endif

	if (ngbls .ne. nrows) then
	   write (msg,100)
	   call errmsg(msg,'ibis2gbl3-unequal',1)
	   call errmsg('Fatal error encountered','ibis2gbl3-fatal',-1)
	endif

C       get current values of the IBIS globals
C	--------------------------------------
	call xrreal(parblk,'$OFFL',MMXIMG,offl,count,status)
	call xrreal(parblk,'$OFFS',MMXIMG,offs,count,status)
	call xrreal(parblk,'$XNL',MMXIMG,xnl,count,status)
 	call xrreal(parblk,'$XNS',MMXIMG,xns,count,status)
	call xrreal(parblk,'$MEANZ',MMXIMG,meanz,count,status)

	return
	end

C	************************************************************************
C	   RDIBIS retrieves the specified values from the IBIS tabular file.
C	************************************************************************
	subroutine rdibis(infile,row,nrows,column,ncols,ibisvl)

	include 'las.fin'

	include 'lams.fin'

	real*4		data(MMXCOL)

	integer*4	column(1)
	integer*4	i
	real*4	ibisvl(1)
	integer  	fdesc
	integer*4	maxcol
	integer*4	maxrow
	integer*4	ncols
	integer*4	nrows
	integer*4	row(1)
	integer*4	status
	integer*4	tbclos
	integer*4	tbopen
	integer*4	tbrrow

	character*(ERRLEN)	msg
	character*(CMLEN)	infile

C	open the input IBIS file
C	------------------------
	status = tbopen(fdesc,infile,maxrow,maxcol,0)
	if (status .lt. ESUCC) then
	   call errmsg('Error opening input file','ibis2gbl3-open',1)
	   call errmsg('Fatal error encountered','ibis2gbl3-fatal',-1)
	endif

C	check that all specified columns are valid
C	------------------------------------------
10	FORMAT ('There is no column ', I5, ' in the IBIS file')
20	FORMAT ('The total number of columns is ',I5)

	do 30 i = 1,ncols
	   if (column(i) .gt. maxcol) then
	      write (msg,10) column(i)
	      call errmsg(msg,'ibis2gbl3-column',1)
	      write (msg,20) maxcol
              call errmsg(msg,'ibis2gbl3-column',1)
	      call errmsg('Fatal error encountered','ibis2gbl3-fatal',-1)
	   endif
30	continue

C	check that all specified rows are valid
C	---------------------------------------
40	FORMAT ('There is no row ', I5, ' in the IBIS file')
50	FORMAT ('The total number of rows is ',I5)
	do 60 i = 1,nrows
	   if (row(i) .gt. maxrow) then
	      write (msg,40) row(i)
	      call errmsg(msg,'ibis2gbl3-row',1)
	      write (msg,50) maxrow
	      call errmsg(msg,'ibis2gbl3-row',1)
	      call errmsg('Fatal error encountered','ibis2gbl3-fatal',-1)
	   endif
60	continue

C	get the values from the specified locations
C	-------------------------------------------

	do 100 i = 1,nrows
	   status = tbrrow(fdesc,column,ncols,row(i),data)
	   if (status .ne. ESUCC) then
	      call errmsg('Error reading input file',
     +	                  'ibis2gbl3-read',1)
	      call errmsg('Fatal error encountered','ibis2gbl3-fatal',-1)
	   endif
	   ibisvl(i) = data(i)
100	continue

C	close the IBIS file
C	-------------------
	status = tbclos(fdesc,infile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','ibis2gbl3-close',1)
	endif

	return
	end

C	************************************************************************
C	   SETUP sets the indicated global variable(s) with the values from
C          the IBIS tabular file.
C	************************************************************************
	subroutine setup(ibisvl,gbls,ngbls,frame,offl,offs,
     +			  xnl,xns,meanz)

	include 'las.fin'

	include 'lams.fin'

	integer*4	frame
	integer*4	i
	real*4	ibisvl(1)
	real*4	meanz(1)
	integer*4	ngbls
	real*4	offl(1)
	real*4	offs(1)
	real*4	xnl(1)
	real*4	xns(1)

	character*6	gbls(1)

	do 10 i = 1,ngbls

	   if (gbls(i) .eq. '$OFFL') then
	      offl(frame) = ibisvl(i)
	   else if (gbls(i) .eq. '$OFFS') then
	      offs(frame) = ibisvl(i)
	   else if (gbls(i) .eq. '$XNL') then
	      xnl(frame) = ibisvl(i)
	   else if (gbls(i) .eq. '$XNS') then
	      xns(frame) = ibisvl(i)
	   else if (gbls(i) .eq. '$MEANZ') then
	      meanz(frame) = ibisvl(i)
	   else
	      call errmsg('Error determining global name','ibis2gbl3-name',1)
	      call errmsg('Fatal error encountered','ibis2gbl3-fatal',-1)
	   endif

10	continue

	return
	end

C	************************************************************************
C	   MAKEPF makes the global parameter file.
C	************************************************************************
	subroutine makepf(vblock,pfile,offl,offs,xnl,xns,meanz)

	include 'las.fin'

	include 'lams.fin'

	integer*4	i
	integer*4	lsstat
	real*4	meanz(1)
	integer*4	mode
	real*4	offl(1)
	real*4	offs(1)
	integer*4	status

      	integer*4 vblock(1)

	real*4	xnl(1)
	real*4	xns(1)

	character*(CMLEN)	pfile

C	check if the specified parameter file exists
C       if it does, then delete it
C	--------------------------------------------
	mode = 0
	status = lsstat(pfile,mode)
	if (status .eq. ESUCC) then
	   call delfil(pfile)
	endif

C	open the parameter file
C	-----------------------
C       (NOTE:  There were problems with transferring (using NCP) the
C		ASCII parameter file (created with FORTRAN) from the
C		VAX/VMS system to the 1 systems.
C 	        
C	        To make the parameter file portable so it can be sent
C		through the network copy program (NCP), the file must
C		be opened on VAX with the carriagecontrol='list' qualifier
C	        and a blank must be written at the beginning of each line.)
C
C		The output file name can not be over 255 characters for VMS
C		and must not contain any null characters

	open(unit=3, file=pfile, status='new', access='sequential',
     +	     form='formatted', iostat=status)

	if (status .ne. 0) then
	   call errmsg('Error opening parameter file','ibis2gbl3-open',1)
	   close (unit=3)
	   call clean(vblock,'ibis2gbl3',pfile,1)
	endif

C	output $OFFL values to the parameter file
C	-----------------------------------------
	write (unit=3, fmt='(A)', iostat=status) ' $OFFL'
	if (status .ne. 0) then
	   close (unit=3)
	   call clean(vblock,'ibis2gbl3',pfile,1)
        endif
	do 10 i = 1,MMXIMG
	   write (unit=3, fmt='(A,f11.4)', iostat=status) ' ',offl(i)
	   if (status .ne. 0) then
	      close (unit=3)
	      call clean(vblock,'ibis2gbl3',pfile,1)
	   endif
10	continue

C	output $OFFS values to the parameter file
C	-----------------------------------------
	write (unit=3, fmt='(A)', iostat=status) ' $OFFS'
	if (status .ne. 0) then
	      close (unit=3)
	      call clean(vblock,'ibis2gbl3',pfile,1)
	   endif
	do 20 i = 1,MMXIMG
	   write (unit=3, fmt='(A,f11.4)', iostat=status) ' ',offs(i)
	   if (status .ne. 0) then
	      close (unit=3)
	      call clean(vblock,'ibis2gbl3',pfile,1)
	   endif
20	continue

C	output $XNL values to the parameter file
C	----------------------------------------
	write (unit=3, fmt='(A)', iostat=status) ' $XNL'
	if (status .ne. 0) then
	      close (unit=3)
	      call clean(vblock,'ibis2gbl3',pfile,1)
	   endif
	do 30 i = 1,MMXIMG
	   write (unit=3, fmt='(A,f11.4)', iostat=status) ' ',xnl(i)
	   if (status .ne. 0) then
	      close (unit=3)
	      call clean(vblock,'ibis2gbl3',pfile,1)
	   endif
30	continue

C	output $XNS values to the parameter file
C	----------------------------------------
	write (unit=3, fmt='(A)', iostat=status) ' $XNS'
	if (status .ne. 0) then
	      close (unit=3)
	      call clean(vblock,'ibis2gbl3',pfile,1)
	   endif
	do 40 i = 1,MMXIMG
	   write (unit=3, fmt='(A,f11.4)', iostat=status) ' ',xns(i)
	   if (status .ne. 0) then
	      close (unit=3)
	      call clean(vblock,'ibis2gbl3',pfile,1)
	   endif
40	continue

C	output $MEANZ values to the parameter file
C	------------------------------------------
	write (unit=3, fmt='(A)', iostat=status) ' $MEANZ'
	if (status .ne. 0) then
	      close (unit=3)
	      call clean(vblock,'ibis2gbl3',pfile,1)
	   endif
	do 50 i = 1,MMXIMG
	   write (unit=3, fmt='(A,f11.4)', iostat=status) ' ',meanz(i)
	   if (status .ne. 0) then
	      close (unit=3)
	      call clean(vblock,'ibis2gbl3',pfile,1)
	   endif
50	continue

C	close the parameter file
C	------------------------
	close (unit=3)

	return
	end

C	************************************************************************
C	   GBLOUT uses the XQ calls to pass out the updated globals.
C	************************************************************************
	subroutine gblout(offl,offs,xnl,xns,meanz)

	include 'las.fin'

	include 'lams.fin'

	real*4	meanz(1)
	integer*4	numel
	real*4	offl(1)
	real*4	offs(1)
	integer*4	status

      	integer*4 vblock(XPRDIM)

	real*4	xnl(1)
	real*4	xns(1)

	numel = MMXIMG
	call xqini(vblock,XPRDIM,XABORT)
 	call xqreal(vblock,'$OFFL',numel,offl,XADD,status)
	call xqreal(vblock,'$OFFS',numel,offs,XADD,status)
	call xqreal(vblock,'$XNL',numel,xnl,XADD,status)
	call xqreal(vblock,'$XNS',numel,xns,XADD,status)
	call xqreal(vblock,'$MEANZ',numel,meanz,XADD,status)
	call xqout(vblock,status)

	return
	end
