/**************************************************************************
NAME:	OMGR2MGR

PURPOSE:
To read old IBIS graphics file and write it to a new IBIS graphics file. 

PROGRAM HISTORY:
Version		Date	Author		Code/Conr	Request
-------		----	------		--------	-------
  1.0		4/87	B. Ailts 	  CSB		PRF 3427
  2.0		1/89	B. Ailts	EDC/CSB		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        OMGR2MGR must be run under TAE.

PROJECT:	LAS(LAMS)		

ALGORITHM DESCRIPTION:
The program begins by initializing TAE.
The user-specified parameters and the LAS global variables 
are then retrieved.  These parameters consist of:
1.  Infile--the input TAE file name of the IBIS graphics file
2.  Outfile--the output TAE file name of the labeled table file

After reading the input parameters, INFILE is opened and the 
polygon vertices are read.  These vertices are then written 
to the file specified in OUTFILE in the new IBIS graphics file
format.  
After all the polygon vertices are written, all the 
files are closed and the completion message is displayed.

ALGORITHM REFERENCES:	none

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "lams.h"

main()

{ /* main */
struct	PARBLK	parb;

long 	one = 1;
long	std;

char	taeout[CMLEN];
char	hosin[CMLEN];
char	hosout[CMLEN];


/*  Initialize tae 
------------------*/
c_inlas (&parb,&std);

init(&parb,taeout,hosin,hosout);

proces(&parb,hosin,hosout);

c_complt("omgr2mgr",taeout,&one);

} /* main */

/****************************************************************************
*
*	INIT gets the TAE parameters
*
*****************************************************************************/

FUNCTION init(parb,taeout,hosin,hosout)


struct	PARBLK	*parb;
char	*taeout;
char	*hosin;
char	*hosout;


{  /* beginning of init*/
long	access;			/* access mode				*/
long	fatal = -1;		/* fatal error code			*/
long	status;
long	zero = 0;		/*  value of zero			*/

/*  Retreive the TAE paramters
------------------------------*/
access = IREAD;
status = c_getnam(parb,"INFILE",&zero,hosin,"ibis_graph",&access);
if (status != E_SUCC) 
   c_errmsg("Fatal error encountered","omgr2mgr-fatal",&fatal);

access = IWRITE;
status = c_getnam(parb,"OUTFILE",&zero,hosout,"ibis_graph",&access);
if (status != E_SUCC) 
   c_errmsg("Fatal error encountered","omgr2mgr-fatal",&fatal);

/*  TAE name is calculated for the output file in order to be used for the
    completion message
--------------------------------------------------------------------------*/
strcpy(taeout, hosout);

return;

}  /*end of init*/

/**************************************************************************
*
*	proces reads an old IBIS graphics file and write in the new
*	IBIS graphics file format
*
***************************************************************************/

FUNCTION proces(parb,hosin,hosout)

struct	PARBLK	*parb;
char	*hosin;
char	*hosout;

{  /* beginning of proces  */
struct IFCB ifcb1;

float	*buffer;		/*  input buffer			*/

long	access;			/*  access mode				*/
long	fatal = -1;		/*  fatal error code			*/
int	fdesc1;			/*  file descriptor of output file	*/
long	i;			/*  counter -- current record		*/
long	lun;
long	nl;
long	ns;
long	one = 1;		/*  value of one			*/
long	rec;			/*  current record			*/
long	status;			/*  return status of calls		*/


/*  PROCES opens and reads the old IBIS graphics file and puts the polygon
    coordinates into the new IBIS graphis file
------------------------------------------------------------------------*/

/*  initialization of variables  
-------------------------------*/
rec = 1;
lun = 10;

/*  Open the output file
------------------------*/
access = I_INPUT;
status = i_opin(&ifcb1,lun,hosin,access);
if (status != SUCCESS)
   {
   c_errmsg("Error opening input file","omgr2mgr-open",&one);
   c_errmsg("Fatal error encountered","omgr2mgr-fatal",&fatal);
   }

/*  Read and write the input records
------------------------------------*/
nl = ifcb1.i_lines;
ns = ifcb1.i_linsz / 4;
if (nl * ns > MEMXPT)
   {
   c_errmsg("Input file contains more points than output file can handle",
            "omgr2mgr-points",&one);
   c_errmsg("Fatal error encountered","omgr2mgr-fatal",&fatal);
   }

buffer = (float *) malloc(ns * 4);
if (buffer == NULL)
   {
   c_errmsg("Error allocing dynamic memory","omgr2mgr-alloc",&one);
   c_errmsg("Fatal error encountered","omgr2mgr-fatal",&fatal);
   }

access = 1;
status = c_gropen(&fdesc1,hosout,&ns,&nl,&access);
if (status != E_SUCC)
   {
   status = c_grclos(&fdesc1,hosout);
   c_clean(parb,"omgr2mgr",hosout,&one);
   }

for (i = 0 ; i < nl; i ++)
   {
   status = i_read(&ifcb1,buffer,one,rec,one);
   if (status != SUCCESS)
      {
      c_errmsg("Error reading input file","omgr2mgr-read",&one);
      status = c_grclos(&fdesc1,hosout);
      c_clean(parb,"omgr2mgr",hosout,&one);
      }
   status = i_wait(&ifcb1);
   if (status != SUCCESS)
      {
      c_errmsg("Error reading input file","omgr2mgr-read",&one);
      status = c_grclos(&fdesc1,hosout);
      c_clean(parb,"omgr2mgr",hosout,&one);
      }

   status = c_grwrit(&fdesc1,&rec,buffer);
   if (status != E_SUCC)
      {
      c_errmsg("Error writing to the output file","omgr2mgr-write",&one);
      status = c_grclos(&fdesc1,hosout);
      c_clean(parb,"omgr2mgr",hosout,&one);
      }

   rec = rec + 1;
   } /* for i */

/*  Close the input and output files
------------------------------------*/
access = I_SAVE;
i_clse(&ifcb1,access);

status = c_grclos(&fdesc1,hosout);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","omgr2mgr-warn",&one);

return;

}  /*  end of process */
