/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the input parameters

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,     /* I: TAE parameter block                      */
    char          infiles[][CMLEN],/* O: Input LAS file(s)                    */
    long          *num_infiles,/* O: number of input image files              */
    char          outfile[],   /* O: Output host TIFF file                    */
    char          worldfile[], /* O: World file name                          */
    long          *imagelength,/* O: Common length of all input images        */
    long          *imagewidth, /* O: Common width  of all input images        */
    long          *sl,         /* O: Starting line of each LAS input image    */
    long          *ss,         /* O: Starting sample of each LAS input image  */
    long          *nbands,     /* O: Num bands to read from each input image  */
    long          *msgtim,     /* O: # of seconds between processing messages */
    long          bands[][MAXBND+1],/* O: Array to hold the bands for
                                     each input image                         */
    long          window[][4], /* O: Array to hold the windows (sl ss nl ns)  */
    long          *total_band_count/* O: total number of bands in image list  */
)
{
    struct VARIABLE *val;         /* pointer to a TAE variable                */

    char  *q;                     /* String pointer                           */
    char  *ptr        = NULL;     /* pointer to substring                     */
    char  taeoutfile[CMLEN];      /* TAE output tiff file                     */
    char  taeinfiles[MAXIMG][CMLEN];/* TAE input LAS file name                */

    long  cur_image;              /* Loop variable: input image counter       */
    long  imglimit    = MAXIMG;   /* maximum images caller can handle         */
    long  line_index  = 0;        /* Loop variable: index for input line count*/
    long  nimg        = 0;        /* number of images                         */
    long  qflag[MAXIMG][2];       /* 1 user defined window and bands else:
                                     0 is default window and bands            */
    long  same_window;            /* 1 if all windows are the same            */
    long  totbnds;                /* total number of bands specified          */
    long  totimg      = 0;        /* sum total image count                    */

    /* Get the TAE globals needed
       -------------------------*/
    val = p_find(vblock, "$MSGTIM");
    *msgtim = IVAL(*val, 0);

    /* Get the input image list count
       ------------------------------*/
    if(c_getcnt(vblock, "IN", num_infiles) != E_SUCC)
    {
        c_lasmsg(FALSE, "las2geotiff-getpar", "Error reading parameter IN");
        return E_FAIL;
    }

    /* Get input parameters
       --------------------*/
    for(line_index = 0 ; line_index < *num_infiles ; line_index++)
    {
        if ( c_getin( vblock, "IN", &line_index, &taeinfiles[totimg],
                &infiles[totimg], &nimg, &window[totimg], &bands[totimg],
                &nbands[totimg], &totbnds, &imglimit, &same_window,
                qflag) != E_SUCC)
        {
            c_lasmsg(FALSE, "las2geotiff-getpar","Error reading parameter IN");
            return E_FAIL;
        }

        /* Handle one or more image files per input line
           --------------------------------------------- */
        for (cur_image=totimg; cur_image<totimg+nimg; cur_image++)
        {
            /* Save window parameters
               ---------------------*/
            sl[cur_image] = window[cur_image][SL];
            ss[cur_image] = window[cur_image][SS];
            if (cur_image)
            {
                if (window[cur_image][NL] != window[0][NL] ||
                        window[cur_image][NS] != window[0][NS]  )
                {
                    c_lasmsg( FALSE, "las2geotiff-fatal",
                            "Input windows specified must be same size");
                    return E_FAIL;
                }
            }
        }
        totimg += nimg;
    }

    /* Since some tiff viewers (like xv) will only read 1 or 3 band
       images, check the number of bands and warn the user
       ------------------------------------------------------------*/
    *total_band_count = 0;
    for(line_index = 0 ; line_index < totimg ; line_index++)
        *total_band_count += nbands[line_index];
    if ((*total_band_count != 1) && (*total_band_count != 3))
    {
        c_lasmsg( FALSE, "las2geotiff-bands",
                "Some geoTIFF viewers cannot handle %ld band images",
                *total_band_count);
        c_lasmsg( FALSE, "las2geotiff-bands",
                "and may not be able to view this geoTIFF image\n");
    }

    /* Update input image file count
       ----------------------------- */
    *num_infiles = totimg;

    /* Save common input image size
       ---------------------------- */
    *imagelength = window[0][NL];
    *imagewidth  = window[0][NS];

    /* Retrieve output file name
       ------------------------- */
    val = p_find(vblock,"OUTFILE");
    strcpy(taeoutfile,SVAL(*val,0));
    c_cmhost(taeoutfile, outfile);

    /* For the outfile, if the user specified a filename extension
       use it, otherwise set it to tif.
       ----------------------------------------------------------- */
    ptr = strrchr(outfile,'.');  /* Find the right-most dot               */
    if (ptr == NULL)             /* IF no dot found                       */
        strcat(outfile,".tif");   /*    add extension                      */
    else                         /* ELSE                                  */
    {
        ptr = strchr(ptr,'/');    /*    Look for directory to right of dot */
        if (ptr != NULL)          /*    IF dir at or to right of dot       */
            strcat(outfile,".tif");/*       add extension                   */
    }

    /* Read the world file option.
       ------------------------------*/
    val = p_find(vblock,"WOPT");
    if (strcmp(SVAL(*val,0),"YES") == 0)
    {
        /* Set the world file root name the same as the output file.
           ------------------------------------------------------------*/
        strcpy(worldfile, outfile);
        if ((q = strchr(worldfile,'.')) != NULL)
            *q = '\0';
        strcat(worldfile,".tfw");
        printf("WORLDFILE : %s\n",worldfile);
    }
    else
        worldfile[0] = '\0';
    return E_SUCC;
}
