/*****************************************************************************
NAME: LAS2GEOTIFF

PURPOSE:  To convert LAS image(s) to a geoTIFF file.

PROGRAM HISTORY:

Version  Date  Programmer       Code  Reason
-------  ----- ----------       ----  ------
 0.0     09/97 B.Olson           C    Code borrowed from LAS2TIFF
 1.0     04/98 B.Olson           C    Recreated with geoTIFF extensions
 1.1     08/98 L.Johnson         C    Modified to only use UL corner in
                                      ModelTiepointTag.  Added option for
                                      saving the affine transformation in
                                      an ascii "world" file.
 1.2     10/99 D.Hames		 C    Modify utmkeyset and get_utmzone to
				      accept LAS ddr datum code > 100 and
				      output the key value to the geoTIFF.
				      Added prototypes. Reference SRF #2897.
 1.3     01/00 D.Hames		 C    Modified getpar.c because the variables of
				      imagewidth & imagelength were "switched" 
				      when declared.
 1.4	 03/00 D.Hames	         C    Modify getpar. The values of imagewidth
 				      and imagelength were "switched" when
                                      subwindowing was applied to the image. 
                                      Also remove startx and starty. These
				      variables were not being used. Finally
                                      fix the declaration and freeing of 
				      tempstr. CR#3422
 1.5	 03/00 D.Hames		 C    Modify warning "non-standard number of
				      bands" in getpar.  CR#2474
 1.6     05/00 D.Hames           C    Modify to support PLANARCONFIG_CONTIG
                                      instead of PLANARCONFIG_SEPARATE. Removed
                                      call to translas. Modify to support only
                                      LAS byte data, since output TIFF files 
                                      can only be 8 bitspersample. CR#3678

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS

ALGORITHM:
  Retrieve the parameters from the parameter block
  Open the LAS file(s) for read access
  Open the geoTIFF file for writing
  Write the TIFF image fields from ddr information into the geoTIFF
  Write the geoTIFF image fields from ddr information into the geoTIFF
  IF world file requested 
    IF input image(s) geographically valid 
      IF input image(s) match geographically (within tolerance value)
        Create world file 
  Allocate buffers for LAS image and TIFF image
  Initialize processing message
  Read LAS image(s), one line at a time and write to geoTIFF in BIP format
  Write out geoTIFF keys
  Close the geoTIFF and LAS file(s) and free buffers
  Stop processing message
  Return successful completion message back to TAE

*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "tiffio.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "util.h"
#include "protos.h"
#include "diskio.h"

#define software "LAS2GEOTIFF v1.0"

long curent;             /* current processing line                   */
long curcat;             /* current processing band                   */

static void close_files  /* Function closes files with file descriptors */
(
    struct FDESC *fdesc[],  /* List of file descriptors                 */
    int          file_count /* The number of files in list              */
)
{
    int ii;

    for (ii=0; ii<file_count; ii++)
        c_eclose(&fdesc[ii]);
} /* close_files */


void main
(void)
{
    struct FDESC *fdesc[MAXIMG];  /* File descriptors                     */
    struct PARBLK vblk;           /* TAE parameter block                  */

    char infiles[MAXIMG][CMLEN];  /* Input  LAS filename(s)               */
    char outfile[CMLEN];          /* Output TIFF file                     */
    char worldfile[CMLEN];        /* World file                           */

    struct DDR ddr;               /* Input image data descriptor record   */
    struct BDDR *bddr    = NULL;  /* output band dependent ddr record     */

    GTIF *gtif;                   /* GeoKey-level descriptor              */

    TIFF *out;                    /* Output file pointer                  */
    FILE *wf;                     /* World file pointer                   */

    long acc=IREAD;               /* Image file open access               */
    long bandnums[MAXIMG][MAXBND+1];/* Array to hold the band numbers in
                                       each input image                   */
    long curband;                 /* current processing band              */
    long curline         = 1;     /* current processing line              */
    long dtype=0;                 /* Data type                            */
    long flag = ALLCAT;           /* Display message flag                 */
    long imagelength     = 0;     /* Length of image                      */
    long imagewidth      = 0;     /* width of image                       */
    long line=0;                  /* Line counter                         */
    long msgtim;                  /* # of seconds between processing messages*/
    long nbands[MAXIMG]  = {0};   /* List of bands in each input image    */
    long nbands_out      = 0;     /* sum of nbands for all input images   */ 
    long num_infiles     = 0;     /* number of input image files          */
    long one = 1;                 /* Value of one                         */
    int  outband;                 /* current band relative to output image*/
    long samp;                    /* Current sample read, used for looping*/
    long size;                    /* # of array elements in tiepoints     */
    long sl[MAXIMG];              /* Starting line of each LAS image      */
    long ss[MAXIMG];              /* Starting sample of each LAS image    */
    long status;                  /* Process status                       */
    long stdlun=0;                /* Logical unit number for std output   */

    static  long  opt[3] ={0,0,0};/* Image file open option               */

    double colrot=0.;             /* (required only for world file)       */
    double pixelscale[3];         /* coordinate array for PIXELSCALE      */
    double rowrot=0.;             /* rotation terms for row and column    */
    double tiepoints[4][6];       /* coordinate array for GEOTIEPOINTS    */

    unsigned char *lasbuf;        /* input line buffer from LAS image     */
    long num_requested;           /* The number of bands requested; 0 = all*/
    unsigned char *tiffbuf;       /* line buffer for tiff image           */

    uint16 bitspersample=8;       /* Number of bits per sample            */

    int  file_index     = 0;      /* file counter                         */

    long  total_band_count = 0;   /* total number of bands in image list  */
    long  window[MAXIMG][4];      /* Array to hold the windows (sl ss nl ns)*/

    struct DDR best_ddr = {0};    /* the most recent or latest valid ddr  */
    double line_inc     = 1.0;    /* line increment                       */
    double samp_inc     = 1.0;    /* sample increment                     */
    long   upddr_flag   = NOCOMB; /* flag for update DDR: no combined bands*/
    char   valid_ddr    = 0;      /* flag (!0) means a valid ddr was found*/

    /*  Initialize TAE interface
        ------------------------ */
    c_inlas (&vblk,&stdlun);

    /* Retrieve the input parameters
       ----------------------------- */

    if (getpar( &vblk, infiles, &num_infiles, outfile, worldfile, &imagelength,
            &imagewidth, sl, ss, nbands, &msgtim, bandnums, window,
            &total_band_count) != E_SUCC)
        c_lasmsg(TRUE, "las2geotiff-open","Error opening output image file %s",
                outfile);

    /* Open each input LAS file for read access
       ---------------------------------------- */
    for (file_index = 0; file_index < num_infiles; file_index++)
    {
        num_requested = 0;

        /* Read in the data type from the ddr file
           --------------------------------------- */
        if(c_getddr(infiles[file_index], &ddr) != E_SUCC)
            c_lasmsg(TRUE, "las2geotiff-ddr",
                    "Unable to read DDR for file %s", infiles[file_index]);
        dtype = ddr.dtype;
        if (ddr.dtype != EBYTE)
            c_lasmsg( TRUE, "las2geotiff-bits",
                    "Input image %s is not of 8-bit sample size",
                    infiles[file_index]);

        /* Get the most recent (VALID) ddr for later 
           comparison to all previous ddrs in the list
           ------------------------------------------- */
        if (ddr.valid[5] && ddr.valid[6])
        {
            best_ddr = ddr;
            valid_ddr = 1;
        }

        status = c_eopenr( &fdesc[file_index], infiles[file_index],
                &num_requested, &sl[file_index], &ss[file_index],
                &imagelength, &imagewidth, &acc, &dtype, opt, &one);
        if ( status != E_SUCC)
            c_lasmsg(TRUE,"las2geotiff-las","Error opening LAS image file %s",
                    infiles[file_index]);

        /* Keep track of total number of bands for tiff output
           --------------------------------------------------- */
        nbands_out += nbands[file_index];
    }

    /* Set ddr to best one found in list, whether valid or not.
       This will be compared to all other ddrs in the list for
       possible update.
       ------------------------------------------------------- */ 
    ddr = best_ddr;

    /* Set the number of bits per sample from the dtype in the ddr
       ----------------------------------------------------------- */
    switch (dtype)
    {
        case EBYTE: bitspersample = 8;       /* unsigned char            */
                    break;
        default: close_files(fdesc, num_infiles);
                 c_lasmsg(FALSE, "las2geotiff-bits",
                         "Each LAS image in list must be of 8-bit sample size");
                 c_lasmsg(TRUE, "las2geotiff-bits",
                         "because the output geoTIFF file can only be 8-bit.");
                 break;
    }

    /* Update the DDR
       -------------- */
    if (valid_ddr)
    {
        /*  Consolidate all of the image DDRs into one
            ------------------------------------------*/
        status = c_merge_ddrs( &vblk, infiles, outfile, &num_infiles, window, 
                bandnums, nbands, &line_inc, &samp_inc, 
                &total_band_count, &upddr_flag, 
                &bddr, &ddr, FALSE);
        free(bddr);      /* not used in this application, so free memory now */
        if (status != E_SUCC)
            c_lasmsg (TRUE, "las2geotiff-ddr", "Error updating the DDR");
    }

    /* Check to see if the file already exists, and also to see if
       the DATAPATH variable is set. Assume 16384 bytes of overhead 
       for the tiff format plus another 16 bytes per line.  Note 
       that this is just an estimate, but it should be large enough
       to estimate the size with a safety margin.
       ----------------------------------------------------------*/
    if(c_dkcre(NULL, outfile, ((long long)imagewidth * imagelength * 
            (bitspersample/8) * nbands_out + 16384 + 16 * imagelength), 0) 
            == DISK_EXISTS)
    {
        c_lasmsg(TRUE,"las2geotiff-exists","%s already exists", outfile);
    }

    /* Open the TIFF file
       ------------------ */
    out = XTIFFOpen(outfile, "w");
    if (out == NULL)
    {
        c_dkfree(outfile);
        close_files(fdesc, num_infiles);
        c_lasmsg(TRUE, "las2geotiff-open", "Error opening output image file %s", 
                outfile);
    }

    gtif = GTIFNew(out);
    if (!gtif)
    {
        close_files(fdesc, num_infiles);
        XTIFFClose(out);
        c_dkfree(outfile);
        c_lasmsg(TRUE,"las2geotiff-gtif","Error creating GTIF");
    }

    /* Set the normal TIFF image fields
       -------------------------------- */
    TIFFSetField(out, TIFFTAG_SOFTWARE, software);
    TIFFSetField(out, TIFFTAG_SUBFILETYPE, 0);
    TIFFSetField(out, TIFFTAG_IMAGEWIDTH, imagewidth);
    TIFFSetField(out, TIFFTAG_IMAGELENGTH, imagelength);
    TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, bitspersample);
    TIFFSetField(out, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
    if (nbands_out == 1)
        TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
    else if (nbands_out == 3)
        TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
    else
        TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
    TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, nbands_out);
    TIFFSetField(out, TIFFTAG_ROWSPERSTRIP,1);
    TIFFSetField(out, TIFFTAG_XRESOLUTION,1);
    TIFFSetField(out, TIFFTAG_YRESOLUTION,1);
    TIFFSetField(out, TIFFTAG_RESOLUTIONUNIT, RESUNIT_NONE);
    TIFFSetField(out, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);

    if (ddr.valid[6]) 
    {
        /* the corner coordinates in the ddr are valid, so need to
           put them in the geoTIFF
           ------------------------------------------------------- */
        get_tiepoints(&ddr,tiepoints,&size);

        /* Some applications (e.g., ArcView) won't handle geoTIFF images with
           more than one tie point pair.  Therefore, only the upper left corner
           is being written to the geoTIFF file.  In order to write all computed
           tie points to the geoTIFF, change the 6 to size in the line below.
           -------------------------------------------------------------------*/
        TIFFSetField(out, TIFFTAG_GEOTIEPOINTS, 6, tiepoints);
    }
    else
    {
        c_lasmsg(FALSE,"las2geotiff-coord",
                "LAS image %s does not contain valid corner coordinates", 
                num_infiles > 1 ? "set" : infiles[0]);
    }
    if (ddr.valid[5])
    {
        /* the pixel scales in the ddr are valid,
           so need to put them in the geoTIFF
           -------------------------------------- */
        pixelscale[0] = ddr.pdist_x;       /* x-axis scale    */
        pixelscale[1] = ddr.pdist_y;       /* y-axis scale    */
        pixelscale[2] = 0;                 /* z-axis scale    */
        TIFFSetField(out, TIFFTAG_GEOPIXELSCALE, 3, pixelscale);
    }
    else
    {
        c_lasmsg(FALSE,"las2geotiff-dist",
                "LAS image %s does not contain valid projection distance",
                num_infiles > 1 ? "set" : infiles[0]);
    }

    /* all the tags that apply to any geoTIFF file are 
       done so time to add the projection specific tags
       ------------------------------------------------ */
    if (ddr.valid[0])
    {
        switch (ddr.proj_code)
        {
            case GEO:
                geokeyset(&ddr,gtif);
                break;
            case UTM:
                utmkeyset(&ddr,gtif);
                break;
            case SPCS:
                spcskeyset(&ddr,gtif);
                break;
            case ALBERS:
                alberskeyset(&ddr,gtif);
                break;
            case LAMCC:
                lamcckeyset(&ddr,gtif);
                break;
            case MERCAT:
                mercatkeyset(&ddr,gtif);
                break;
            case PS:
                pskeyset(&ddr,gtif);
                break;
            case POLYC:
                polyckeyset(&ddr,gtif);
                break;
            case EQUIDC:
                equidckeyset(&ddr,gtif);
                break;
            case TM:
                tmkeyset(&ddr,gtif);
                break;
            case STEREO:
                stereokeyset(&ddr,gtif);
                break;
            case LAMAZ:
                lamazkeyset(&ddr,gtif);
                break;
            case AZMEQD:
                azmeqdkeyset(&ddr,gtif);
                break;
            case GNOMON:
                gnomonkeyset(&ddr,gtif);
                break;
            case ORTHO:
                orthokeyset(&ddr,gtif);
                break;
            case GVNSP:
                gvnspkeyset(&ddr,gtif);
                break;
            case SNSOID:
                snsoidkeyset(&ddr,gtif);
                break;
            case EQRECT:
                eqrectkeyset(&ddr,gtif);
                break;
            case MILLER:
                millerkeyset(&ddr,gtif);
                break;
            case VGRINT:
                vgrintkeyset(&ddr,gtif);
                break;
            case HOM:
                homkeyset(&ddr,gtif);
                break;
            case ROBIN:
                robinkeyset(&ddr,gtif);
                break;
            case SOM:
                somkeyset(&ddr,gtif);
                break;
            case ALASKA:
                alaskakeyset(&ddr,gtif);
                break;
            case GOOD:
                goodkeyset(&ddr,gtif);
                break;
            case MOLL:
                mollkeyset(&ddr,gtif);
                break;
            case IMOLL:
                imollkeyset(&ddr,gtif);
                break;
            case HAMMER:
                hammerkeyset(&ddr,gtif);
                break;     
            case WAGIV:  
                wagivkeyset(&ddr,gtif);
                break;
            case WAGVII:
                wagviikeyset(&ddr,gtif);
                break;
            case OBEQA:
                obeqakeyset(&ddr,gtif);
                break;
        }
    }
    else
        c_lasmsg(FALSE,"las2geotiff-proj",
                "LAS image %s does not contain valid projection code", 
                num_infiles > 1 ? "set" : infiles[0]);

    if (worldfile[0] != '\0')
    {
        if(ddr.valid[5] && ddr.valid[6])
        {

            /* Create the world file.
               -------------------------*/

            if ((wf = fopen(worldfile,"w")) == NULL)
            {
                c_dkfree(outfile);
                c_lasmsg(TRUE, "las2geotiff-tfw", "Error opening world file");
            }
            fprintf(wf,"%35.14f\n%35.14f\n%35.14f\n%35.14f\n%35.14f\n%35.14f\n",
                    pixelscale[0], rowrot, colrot, -pixelscale[1],
                    ddr.upleft[1], ddr.upleft[0]);
            fclose(wf);
        }
        else
        {
            c_lasmsg(FALSE, "las2geotiff-tfw", "World file not created due to"
                    " invalid corner or pixel size information.");
        }
    }

    /* Allocate input buffer
       --------------------*/
    lasbuf = malloc(sizeof(unsigned char)*imagewidth*(bitspersample/8));
    if (lasbuf == NULL)
    {
        close_files(fdesc, num_infiles);
        GTIFFree(gtif);
        XTIFFClose(out);
        c_dkfree(outfile);
        c_lasmsg(TRUE, "las2geotiff-mem", "Error allocating memory for lasbuf");
    }

    /* Allocate output buffer
       --------------------*/
    tiffbuf = malloc(imagewidth * (bitspersample/8) * nbands_out);
    if (tiffbuf == NULL)
    {
        close_files(fdesc, num_infiles);
        GTIFFree(gtif);
        XTIFFClose(out);
        c_dkfree(outfile);
        c_lasmsg(TRUE, "las2geotiff-mem","Error allocating memory for tiffbuf");
    }

    /* Initialize the processing message
       --------------------------------*/
    curcat=1;
    curent=1;
    c_inittm("LAS2GEOTIFF","Processed","line","bands"," ",
            &imagelength,&nbands_out,&msgtim,&flag);

    c_startm();

    /* Read image in one line at a time and write out to TIFF image in
       BIP format
       ----------------------------------------------------------------------*/
    for (line = 0; line < imagelength; line++, curline++)
    {
        curent  = curline;
        curcat  = 0;            /* init band counter with each output line */
        outband = 0;
        for (file_index = 0; file_index < num_infiles; file_index++)
        {
            for(curband=1; curband <= nbands[file_index]; curband++)
            {
                status = c_eread( &fdesc[file_index],
                        &(bandnums[file_index][curband-1]),
                        &curline, lasbuf, &one);
                if (status != imagewidth)
                {
                    XTIFFClose(out);
                    c_dkfree(outfile);
                    free(lasbuf);
                    free(tiffbuf);
                    close_files(fdesc, num_infiles);
                    c_lasmsg( TRUE, "las2geotiff-read", "Error reading %s",
                            infiles[file_index]);
                }
                for (samp=0; samp < imagewidth; ++samp)
                    tiffbuf[outband + samp*nbands_out] = lasbuf[samp];
                outband++;
                curcat = outband;
            }
        }
        if (TIFFWriteScanline(out, tiffbuf, line, 0) < 0)
        {
            XTIFFClose(out);
            c_dkfree(outfile);
            free(lasbuf);
            free(tiffbuf);
            close_files(fdesc, num_infiles);
            c_lasmsg(TRUE, "las2geotiff-write", "Error writing to %s", outfile);
        }
    }

    c_stoptm();

    /* write out the geoTIFF keys
       -------------------------- */
    GTIFWriteKeys(gtif);

    /* cleanup
       ------- */
    GTIFFree(gtif);
    XTIFFClose(out);
    c_dkfree(outfile);  
    free(lasbuf);
    free(tiffbuf);
    close_files(fdesc, num_infiles);

    /* Output completion message
       -----------------------*/
    c_complt("LAS2GEOTIFF", (char (*)[256])outfile, &one);

}
