/*****************************************************************************
NAME: LAS2TIFF

PURPOSE:
     To convert LAS image(s) to a TIFF file.

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
  1.0    09/97 B. Olson           C    Initial Development
  1.1    05/98 B. Olson           C    Added handling for 16 & 32 bit images
  2.0    03/00 D. Hames		  C    Modify getpar. The values of imagewidth 
				       and imagelength were "switched" when
				       subwindowing was applied to the image.
                                       Also remove startx and starty. These
				       variables were not being used. Add
				       prototypes. CR#3422
  3.0	 03/00 D. Hames           C    Modify to support multiple bands. Modify
				       fatal error message "invalid number of 
			               bands" in getpar to a warning message. 
                                       CR#2474
  4.0    04/00 D. Hames		  C    Modify to support PLANARCONFIG_CONTIG
                                       instead of PLANARCONFIG_SEPARATE. Remove
				       call to translas. Modify to support only
 				       8 bitspersample since this is all TIFF 
                                       files will support.  CR#3581 
				       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE

PROJECT: LAS

****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "tiffio.h"
#include "util.h"
#include "las2tiff.h"
#include "imageio.h"
#include "diskio.h"

long curent;             /* current processing line                   */
long curcat;             /* current processing band                   */

static void close_files  /* Function closes files with file descriptors    */
(
    struct FDESC *fdesc[],  /* List of file descriptors                    */
    int          file_count /* The number of files in list                 */
)
{
    int ii;

    for (ii=0; ii<file_count; ii++)
        c_eclose(&fdesc[ii]);
} /* close_files */


void main
(void)
{
  struct PARBLK vblk;           /* TAE parameter block                     */
  struct FDESC *fdesc[MAXIMG];  /* File descriptors                        */

  char infiles[MAXIMG][CMLEN];  /* Input  LAS filename(s)                  */
  char outfile[CMLEN];          /* Output TIFF file                        */

  TIFF *out;                    /* Output file                             */

  int  file_index      = 0;     /* file counter                            */

  long acc=IREAD;               /* Image file open access                  */
  long bandnums[MAXIMG][MAXBND+1];/* Array to hold the bands in
                                   each input image                        */
  long curband;                 /* current band relative to input image    */
  long curline         = 1;     /* current line of output image            */
  long dtype=0;                 /* Data type                               */
  long flag = ALLCAT;           /* Display message flag                    */
  long imagelength     = 0;     /* Length of image line                    */
  long imagewidth      = 0;     /* width of image                          */
  long line=0;                  /* Line counter                            */
  long msgtim;                  /* # of seconds between processing messages*/
  long nbands[MAXIMG]  = {0};   /* # of actual bands in each output image  */
  long nbands_out      = 0;     /* sum of nbands for all input images      */ 
  long num_infiles     = 0;     /* number of input image files             */
  long num_requested;           /* The number of bands requested; 0 = all  */
  long one             = 1;     /* Value of one                            */
  int  outband;                 /* current band relative to output image   */
  long samp;                    /* Current sample read, used for looping   */
  long sl[MAXIMG];              /* Starting line of each LAS image         */
  long ss[MAXIMG];              /* Starting sample of each LAS image       */
  long status;                  /* Process status                          */
  long stdlun=0;                /* Logical unit number for std output      */

  static  long  opt[3] ={0,0,0};/* Image file open option                  */

  unsigned char *lasbuf;        /* input line buffer from LAS image        */
  unsigned char *tiffbuf;       /* line buffer for tiff image              */

  uint16 bitspersample=8;       /* Number of bits per sample               */

  /*  Initialize TAE interface
      ------------------------ */
  c_inlas (&vblk,&stdlun);

  /* Retrieve the input parameters
     ----------------------------- */
  if (getpar( &vblk, infiles, &num_infiles, outfile, &imagelength, &imagewidth,
              nbands, &dtype, &msgtim, sl, ss, bandnums) != E_SUCC)
      c_lasmsg(TRUE, "las2tiff-open","Error opening output file");

  /* Open each input LAS file for read access
     ---------------------------------------- */
  for (file_index = 0; file_index < num_infiles; file_index++)
  { 
      num_requested = 0;
      status = c_eopenr( &fdesc[file_index], infiles[file_index], 
                         &num_requested, &sl[file_index], &ss[file_index], 
                         &imagelength, &imagewidth, &acc, &dtype, opt, &one); 
      if ( status != E_SUCC)
          c_lasmsg(TRUE,"las2tiff-las","Error opening LAS image %s", 
                   infiles[file_index]);
      
      /* Keep track of total number of bands for tiff output
         --------------------------------------------------- */
      nbands_out += nbands[file_index];
  }

  /* Set the number of bits per sample from the dtype in the ddr
     ----------------------------------------------------------- */
  switch (dtype)
  {
    case 1: bitspersample = 8;       /* unsigned char            */
            break;
    default: close_files(fdesc, num_infiles);
             c_lasmsg( FALSE, "las2tiff-bits",
                       "Each LAS image in list must be of 8-bit sample size ");
             c_lasmsg(TRUE, "las2tiff-bits",
                      "because the output TIFF file can only be 8-bit.");
  }

  /* Check to see if the file already exists, and also to see if 
     the DATAPATH variable is set. Assume 16384 bytes of overhead 
     for the tiff format plus another 16 bytes per line.  Note 
     that this is just an estimate, but it should be large enough
     to estimate the size with a safety margin.
     ----------------------------------------------------------*/
  if(c_dkcre(NULL, outfile, ((long long)imagewidth * imagelength * 
             (bitspersample/8) * nbands_out + 16384 + 16 * imagelength), 0) 
             == DISK_EXISTS)
  {
      c_lasmsg(TRUE,"las2tiff-exists","%s already exists",outfile);
  }

  /* Open the TIFF file
     ------------------ */
  out = TIFFOpen(outfile, "w");
  if (out == NULL)
  {
    c_dkfree(outfile);
    close_files(fdesc, num_infiles);
    c_lasmsg(TRUE, "las2tiff-open", "Error opening output image %s", outfile);
  }

  /* Set the TIFF image fields 
     ------------------------- */
  TIFFSetField(out, TIFFTAG_SUBFILETYPE, 0);
  TIFFSetField(out, TIFFTAG_IMAGEWIDTH, imagewidth);
  TIFFSetField(out, TIFFTAG_IMAGELENGTH, imagelength);
  TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, bitspersample);
  TIFFSetField(out, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
  if (nbands_out == 1) 
    TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
  else if (nbands_out == 3)
    TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
  else
    TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
  TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, nbands_out);
  TIFFSetField(out, TIFFTAG_ROWSPERSTRIP,1);
  TIFFSetField(out, TIFFTAG_XRESOLUTION,1);
  TIFFSetField(out, TIFFTAG_YRESOLUTION,1);
  TIFFSetField(out, TIFFTAG_RESOLUTIONUNIT, RESUNIT_NONE);
  TIFFSetField(out, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);

  /* Allocate input buffer
     --------------------*/
  lasbuf = malloc(imagewidth*(bitspersample/8)); 
  if (lasbuf == NULL)
  {    
    TIFFClose(out);
    c_dkfree(outfile);
    close_files(fdesc, num_infiles);
    c_lasmsg(TRUE, "las2tiff-mem", "Error allocating memory for lasbuf");
  } 

  /* Allocate output buffer
     --------------------*/
  tiffbuf = malloc(imagewidth*(bitspersample/8)*nbands_out);
  if (tiffbuf == NULL)
  {
    TIFFClose(out);
    c_dkfree(outfile);
    free(lasbuf);
    close_files(fdesc, num_infiles);
    c_lasmsg(TRUE, "las2tiff-mem", "Error allocating memory for tiffbuf");
  }

  /* Initialize the processing message
     --------------------------------*/
  curcat=1;
  curent=1;
  c_inittm("LAS2TIFF","Processed","line","bands"," ",
           (long*)&imagelength, &nbands_out, &msgtim, &flag);
  c_startm();

      /* Read image in one line at a time; write to TIFF image in BIP format 
         ------------------------------------------------------------------- */
  for (line = 0; line < imagelength; line++, curline++)
  {
      curent  = curline;
      curcat  = 0;            /* init band counter with each output line */
      outband = 0;
      for (file_index = 0; file_index < num_infiles; file_index++)
      {
          for(curband=1; curband <= nbands[file_index]; curband++) 
          {
              status = c_eread( &fdesc[file_index], 
                                &(bandnums[file_index][curband-1]), 
                                &curline, lasbuf, &one);
              if (status != imagewidth)
              {
                  TIFFClose(out);
                  c_dkfree(outfile);
                  free(lasbuf);
                  free(tiffbuf);
                  close_files(fdesc, num_infiles);
                  c_lasmsg( TRUE, "las2tiff-read", "Error reading %s", 
                            infiles[file_index]);
              }
              for (samp=0; samp < imagewidth; ++samp)
                  tiffbuf[outband + samp*nbands_out] = lasbuf[samp];
              outband++;
              curcat = outband;
          }
      }
      if (TIFFWriteScanline(out, tiffbuf, line, 0) < 0)
      {
          TIFFClose(out);
          c_dkfree(outfile);
          free(lasbuf);
          free(tiffbuf);
          close_files(fdesc, num_infiles);
          c_lasmsg(TRUE, "las2tiff-write", "Error writing to %s", outfile); 
      }
  }

  /* Close files and free buffers
     ---------------------------*/
  TIFFClose(out);
  c_dkfree(outfile);  
  free(lasbuf);
  free(tiffbuf);
  close_files(fdesc, num_infiles);

  c_stoptm();

  /* Output completion message
     -----------------------*/
  c_complt("LAS2TIFF", (char (*)[256])outfile, &one);    

}
