#include "neighbor.h"
#include "pixman.h"
#include "util.h"
#include "protos.h"

/*
-----------------------------------------------------------------------

NAME: NEISUB
 
FUNCTION: Performs the minimum distance spread                    

PDL:
Begin neisub
        Initialize search starting and ending address arrays.
        Get number of pixels and number of lines.
        Initialize work array.
        Read initial rad lines and store in circular work array.
        Initialize Processing Line Message.
        Start PLM.
        Write first line.
        Loop through lines.
           Read a new line into the circular work array
           if the 'new line' is below the border of the input image
                   Read -1's into the work array.
           Perform a spiral search from each sample to the specified category.
           Branch to code for user-specified option.
        Flush out the last output line.
        Stop PLM.
        Close file groups.
        Free input and output buffers.
End neisub

-----------------------------------------------------------------------*/

void neisub
(
    long adim,                  /* I: size of spread                    */
    long wrksiz,                /* I: size of work area                 */
    long *work[],               /* I: work buffer                       */
    long wind[][4],             /* I: processing window                 */
    long xoff[],                /* I: spiral search offsets             */
    long yoff[],                /* I: spiral search offsets             */
    long rad,                   /* I: search radius                     */
    long option,                /* I: search option                     */
    long val,                   /* I: pixel value                       */
    long msgtim,                /* I: message interval                  */
    double *minp,               /* O: min pixels                        */
    double *maxp                /* O: max pixels                        */
)
{

    float   xocc, xneg, xtot, 
    xnmocc;                 /* weighted value of occurrences        */

    static long     ifill,  /* fill value                           */
    flag,                   /* PLM type flag                        */
    ftyp, totyp,            /* conversion types                     */
    ifind,                  /* Array index                          */
    iline,                  /* Line counter                         */
    isbscr, isamp, isrch,   /* Array indicies                       */
    j, jj,                  /* Array indicies                       */
    maxocc, minocc,         /* min and max occurrences              */
    mostoc,                 /* Most occuring value                  */
    ndiff[256],             /* no. of different values              */
    newlin, newadd, i,      /* search buffer index                  */
    nocc[256],              /* most occurring value                 */
    np, nl,                 /* no. of pixels and lines              */
    numocc,                 /* no. of occurrences                   */
    numneg,                 /* no. of pixels outside border         */
    nvals,                  /* Array index                          */
    *obuf,                  /* Temporary Output buffer work space   */
    temp, ifound, itest,    /* temp var, flag and array index       */
    type;                   /* type of work buffer                  */
    long one = 1;

    /*
       Initialize search starting and ending address arrays.
       -----------------------------------------------------*/
    static long nstart[16] = {0, 9, 21, 37, 69, 97, 137, 177, 225, 293, 349, 
        421, 489, 577, 665, 749};
    static long nfin[16] = {8, 20, 36, 68, 96, 136, 176, 224, 292, 348, 
        420, 488, 576, 664, 748, 860};

    /*
       Get number of pixels and number of lines.
       -----------------------------------------*/
    np = wind[0][NS];
    nl = wind[0][NL];


    /* Allocate memory for temporary output buffer
       -------------------------------------------*/
    if ((obuf = (long *) malloc(wind[0][NS] * sizeof(long)))== NULL)
        msg_err("!Error allocating dynamic memory for obuf","neighbor-alloc");

    /*
       Initialize work array.
       ----------------------*/
    ifill = -1;
    type = ELONG;
    for (i=0; i < adim; i++)
        c_pxfill((unsigned char *)work[i], &type, &wrksiz, 
                 (unsigned char *)&ifill);

    /*
       Read initial rad lines and store in circular work array.
       --------------------------------------------------------*/
    ftyp = EBYTE;
    totyp = ELONG;
    for (i = 0; i < rad+1; i++)
    {
        if (c_estep(&gdescin) == E_FAIL)
            msg_err("!Error in image read/write.", "neighbor-step");
        c_pxconv (&ftyp, &totyp, &inbuf[ipoint[0]], 
                (unsigned char *)&work[i+rad][rad], &np);
    }


    /*
       Initialize Processing Line Message.
       -----------------------------------*/
    flag = GEN2;
    c_inittm ("NEIGHBOR", "Processed", "line", "band", " ", &nl, &one, &msgtim,
              &flag);
    /*
       Start PLM.
       ----------*/
    curent = 1;
    c_startm();

    /*
       Write first line.
       -----------------*/
    if (c_estep(&gdescout) == E_FAIL)
        msg_err("!Error in image read/write.", "neighbor-step");

    /*
       Loop through lines.
       -------------------*/
    for (iline = 1; iline <= nl; iline++)
    {

        /*
           Read a new line into the circular work array
           if the 'new line' is below the border of the input image,
           read -1's into the work array.
           ---------------------------------------------------------*/
        if (iline != 1)
        {
            newlin = iline + rad;                                                
            newadd = ((iline - 2) % adim) + 1;                                    
            if (newlin > nl) 
            {
                c_pxfill((unsigned char *)work[newadd-1], &type, 
                        &wrksiz,(unsigned char *)&ifill);
            }
            else
            {
                if (c_estep(&gdescin) == E_FAIL)
                    msg_err("!Error in image read/write.", 
                            "neighbor-step");
                c_pxconv( &ftyp, &totyp, &inbuf[ipoint[0]], 
                        (unsigned char *)&work[newadd-1][rad], &np);
            }  
        }

        /*
           Perform a spiral search from each sample to the specified
           category.
           ---------------------------------------------------------*/
        isbscr = 0;                                                          


        /* 
           Allocate memory for obuf
           ------------------------*/


        /*
           Branch to code for user-specified option.
           -----------------------------------------*/
        switch (option)
        {
            case 1:
                /*
                   Search outward in a spiral
                   --------------------------*/
                for (isamp = 1; isamp <= np; isamp++)
                {
                    numocc = 0;                                                         
                    for (isrch = nstart[isbscr]; isrch <= nfin[rad-1];
                            isrch++)
                    {
                        i = isamp + rad + xoff[isrch];                  
                        jj = iline + rad - 1 - yoff[isrch];
                        j = (jj % adim) + 1;
                        if (work[j-1][i-1]  ==  val)
                            numocc = numocc + 1;
                    }
                    obuf[isamp-1] = numocc;
                }
                break;
            case 2:
                /*
                   Number of occurrences, weighted.
                   --------------------------------*/
                for (isamp = 1; isamp <= np; isamp++)
                {
                    numocc = 0;                                                         
                    numneg = 0;                                                         

                    /*
                       Search outward in a spiral.
                       ---------------------------*/
                    for (isrch = nstart[isbscr]; isrch <= nfin[rad-1]; 
                            isrch++) 
                    {
                        i = isamp + rad + xoff[isrch];
                        jj = iline + rad - 1 - yoff[isrch];
                        j = (jj % adim) + 1;
                        if (work[j - 1][i - 1]  ==  val)
                        {
                            numocc = numocc + 1;
                            continue;
                        }

                        /*
                           Count number of pixels encountered outside
                           of borders.
                           ------------------------------------------*/
                        if (work[j - 1][i - 1]  ==  -1) 
                            numneg = numneg + 1;                                 
                    }

                    /*
                       Compute weighted value for number of
                       occurrences.                       
                       ------------------------------------*/
                    xocc = (float)(numocc);
                    xneg = (float)(numneg);
                    xtot = (float)(nfin[rad-1]) + 1;
                    xnmocc = xocc + xocc * (xneg / (xtot -xneg));
                    if (xnmocc < 0)
                        obuf[isamp-1] = (long)(xnmocc - 0.5);
                    else
                        obuf[isamp-1] = (long)(xnmocc + 0.5);
                }
                break;
            case 3:
                /*
                   Number of different values.
                   ---------------------------*/
                for (isamp = 1; isamp <= np; isamp++)             
                {
                    nvals = 0;           
                    temp = 256;
                    c_pxfill((unsigned char *)ndiff, &type, &temp, 
                            (unsigned char *)&ifill);

                    /*
                       Search outward in a spiral.
                       ---------------------------*/
                    for (isrch = nstart[isbscr]; isrch <= nfin[rad-1];
                            isrch++)             
                    {
                        i = isamp + rad + xoff[isrch];                     
                        jj = iline + rad - 1 - yoff[isrch];
                        j = (jj % adim) + 1;
                        if (work[j - 1][i - 1]  ==  -1)
                            continue;                  

                        /*
                           Test whether this category has been
                           encountered before, and, if not, increment
                           counter.
                           ------------------------------------------*/
                        ifound = 0;                                 
                        for (itest = 0; itest < nvals; itest++) 
                        {
                            if (work[j-1][i-1]  ==  ndiff[itest])
                                ifound = 1;
                        }
                        if (ifound  !=  1)
                        {
                            ndiff[nvals] = work[j-1][i-1];  
                            ++nvals;
                        }
                    }
                    obuf[isamp - 1] = nvals;
                }
                break;
            case 4:
                /*
                   Most occurring value.
                   ---------------------*/
                for (isamp = 1; isamp <= np; isamp++)    
                {
                    temp=256;
                    c_pxfill((unsigned char *)nocc, &type, &temp,
                            (unsigned char *)&ifill);

                    /*
                       Search outward in a spiral.
                       ---------------------------*/
                    for (isrch = nstart[isbscr]; isrch <= nfin[rad-1]; 
                            isrch++)
                    {
                        i = isamp + rad + xoff[isrch];
                        jj = iline + rad - 1 - yoff[isrch];
                        j = (jj % adim) + 1;
                        if (work[j-1][i-1]  !=  -1) 
                        {
                            /*
                               Increment number of occurrences for
                               value encountered.
                               -----------------------------------*/
                            nocc[work[j-1][i-1]+1] = 
                                nocc[work[j-1][i-1]+1] + 1;
                        }
                    }

                    /*
                       Find maximum number of occurrences.
                       -----------------------------------*/
                    maxocc = -1;                                               
                    for (ifind = 0; ifind < 256; ifind++)
                    {
                        if (nocc[ifind]  >  maxocc)
                        {
                            maxocc = nocc[ifind];
                            mostoc = ifind - 1;
                        }
                    }
                    obuf[isamp-1] = mostoc;
                }
                break;
            case 5:
                /*
                   Maximum pixel value.
                   --------------------*/
                for (isamp = 1; isamp <= np; isamp++)             
                {
                    maxocc = 0;

                    /*
                       Search outward in a spiral.
                       ---------------------------*/
                    for (isrch = nstart[isbscr]; isrch <= nfin[rad-1]; 
                            isrch++)
                    {
                        i = isamp + rad + xoff[isrch];
                        jj = iline + rad - 1 - yoff[isrch];
                        j = (jj % adim) + 1;
                        if (work[j-1][i-1]  >  maxocc)
                            maxocc = work[j-1][i-1];
                    }
                    obuf[isamp-1] = maxocc;
                }
                break;
            case 6:

                /*
                   Minimum pixel value.
                   --------------------*/
                for (isamp = 1; isamp <= np; isamp++)             
                {
                    minocc = 256;

                    /*
                       Search outward in a spiral.
                       ---------------------------*/
                    for (isrch = nstart[isbscr]; isrch <= nfin[rad-1]; 
                            isrch++)       
                    {
                        i = isamp + rad + xoff[isrch];
                        jj = iline + rad - 1 - yoff[isrch];
                        j = (jj % adim) + 1;
                        if ((work[j-1][i-1]  !=  -1) && 
                                (work[j-1][i-1] < minocc))
                            minocc = work[j-1][i-1]; 
                    }
                    obuf[isamp-1] = minocc;  
                }
                break;
            default: 
                break; 
        }
        c_pxconv (&totyp, &ftyp, (unsigned char *)obuf, &outbuf[opoint[0]],&np);
        c_pxmnmx ((long *)&outbuf[opoint[0]], &np, minp, maxp, &ftyp);
        if (c_estep(&gdescout) == E_FAIL)
            msg_err("!Error in image read/write.", "neighbor-step");
        curent++;
    }

    /*
       Stop PLM.
       ---------*/
    c_stoptm();

    /*
       Close file groups.
       ------------------*/
    c_egclse(&gdescin);
    c_egclse(&gdescout);

    /*
       Free input and output buffers.
       ------------------------------*/
    free(outbuf);
    free(inbuf);
    free(obuf);
}
