#include "redist.h"
#include "pixman.h"
#include "protos.h"

/*
**********************************************************************

   name- hcum (hist, onbins, prob, transf)

   language- C

   version - 1.0       date- oct -84    programmer - y. sheu 
   version - 1.0       date- Sep -84    programmer - F. Irani:V5.0 conversion

   function- 

	     Compute discrete cumulative probabilities for the
	     input image and build up the look up table for the
	     transformation function.

	     Compute cumulative probability distribution scale up
	     to range from 0 to 255 for byte data, and 0 to lastbn
             for others. A look up table is built for the
	     transformation function.  0.5 is the offset in order
	     to find the nearest integer. 

   called by - main 

	pdl:

begin hcum
	Allocate local work space.
	Allocate probability workspace.
	Allocate transformation workspace.
	For each band
		Allocate temporary work space per band.
		Compute cumulative probability distribution scale up to range
		from 0 to 255 for byte data, and 0 to lastbn for others. 
		Build look up table for the transformation function.  
		Free this band's work area.
	Free local work space.
	Free histogram work space allocated in HISTGM.
end hcum
**********************************************************************
*/


void hcum
(
    long **hist,		/* matrix of histograms per band		*/
    long *onbins,	/* Number of bins used per band array		*/
    float **prob,		/* matrix of probailities -all bins per band	*/
    short **transf	/* matrix of transformations per band		*/
)

{

long    fatal_err = -1;
long    zero = 0;
long    wtype = EWORD;
long    ltype = ELONG;
long    rtype = EREAL;

float	rone,		/* c_pxline parameter				*/
	rdecimal,	/* c_pxline parameter				*/
	*scale,		/* Local scaler array				*/
	total,		/* Total values in histogram bins		*/
	*temp;		/* Local work space				*/

long	band,		/* Band loop counter				*/
	lastbn,		/* End bin index in histogram			*/
	maxg;		/* Maximum GLV in histogram			*/

/*
Allocate local work space.
--------------------------						*/
scale = (float *) malloc(totbands 	 * sizeof(float));

/*
Allocate probability workspace. (Free this is tranfm).
------------------------------------------------------			*/
for (band = 0; band < totbands; ++band)
{
	if ((prob[band] = 
	   (float *) malloc(onbins[band] * sizeof (float))) == NULL)
	{
		c_errmsg ("Error allocating dynamic memory","redist-alloc", 
			  &fatal_err);
	}
}

/*
Allocate transformation workspace. (Free this is tranfm).
---------------------------------------------------------		*/
for (band = 0; band < totbands; ++band)
{
	if ((transf[band] = 
	   (short *) malloc(onbins[band] * sizeof (short))) == NULL)
	{
		c_errmsg ("Error allocating dynamic memory","redist-alloc", 
			  &fatal_err);
	}
}

for (band = 0; band < totbands; ++band)
{
	/*
	Allocate temporary work space per band.
	---------------------------------------				*/
	temp  = (float *) malloc(onbins[band] * sizeof(float));

	/*
	Compute cumulative probability distribution scale up to range
	from 0 to 255 for byte data, and 0 to lastbn for others. A
	look up table is built for the transformation function.  0.5 is the
	offset in order to find the nearest integer. 
	--------------------------------------------------------------------	*/
	lastbn = onbins[band] - 1;
	
	total = (float) (hist[band][lastbn]);
	
	if (otype  ==  EBYTE) 
		maxg = 255;
	else
		maxg = lastbn;
	
	scale[band] = (float) maxg / total;

	c_pxconv(&ltype, &rtype, hist[band], prob[band], &onbins[band]);

	c_pxline (prob[band], prob[band], &onbins[band], &rtype, 
		  &scale[band], &zero);

	/*
	Convert to I*2.
	---------------					*/
	rone = 1.0;
	rdecimal = 0.5;

	c_pxline(prob[band], temp, &onbins[band], &rtype, &rone, &rdecimal);

	c_pxconv(&rtype, &wtype, temp, transf[band], &onbins[band]);

	/*
	Free this band's work area.
	---------------------------				*/
	free(temp);
}

/*
Free local work space.
----------------------						*/
free(scale);

/*
Free histgram work space allocated in HISTGM. (Free pointer in main).
---------------------------------------------------------------------	*/
for (band = 0; band < totbands; ++band)
{
	free(hist[band]);
}

return;

} 
