
/* imageio.c by David Zokaites 12-92.  This file helps to consolidate TAE and
   LAS sequential and random image processing calls.  */ 
#include <math.h>      /* for call to ceil() */ 
#include "redist2.h"   /* for globals vars and local image i/o functions */ 
#include "pixman.h"
#include "util.h"

/* global vars, declared global for convenience */ 
char procname[30];    /* name of las proc, uppercase */ 
char procfail[40];    /* name of las proc, lowercase and "-fail" */
char procwarn[40];    /* name of las proc, lowercase and "-warn" */
long fatal= -1;       /* fatal error, parm to c_errmsg() */ 
long go_on=  1;       /* warning message, informational, parm to c_errmsg()*/
char message[CMLEN];  /* error message string, parm to c_errmsg()*/




/* FUNCTION: initialize tae and las processing.  Must be called before all
   other functions in this file */ 
void begin_las
(
    char *proc             /* input:  name of las proc */
) 
{
   /* declare vars */ 
   long length;            /* length of a char string */ 
   long std_out;           /* standard output */ 
   
   /* initialize las */ 
   c_inlas (&pblock, &std_out); 

   /* set global vars for proc name, procname is uppercase, procfail and 
      procwarn are lower case */ 
   strcpy  (procname, proc); 
   length = (long)strlen (procname);   
   c_low2up (procname, &length);  
   length = (long)strlen (proc);       
   c_up2low (proc, &length); 
   sprintf (procfail, "%s-fail", proc); 
   sprintf (procwarn, "%s-warn", proc); 
}





/* FUNCTION:  return an integer parm with multiple values */ 
void integer_parm
(
    char *parmname,         /* input:  name of parameter */
    struct iparm *parmval  /* output: parm values */
) 
{
   /* declare variables */ 
   long index;            /* for loop index */ 
   struct VARIABLE *var;  /* pointer to a var in pblock */ 
   
   /* find count of parm */ 
   var = p_find (&pblock, parmname);
   parmval->count = (*var).v_count;
   
   /* set parm values */ 
   for (index=0; index < parmval->count; index++)
      parmval->value[index] = IVAL(*var,index);
}






/* FUNCTION: parse input image spec */ 
void getin2
(
    char *parmname,             /* input:  tae parameter name for image */
    struct lasinimage *ini     /* output: data structure of input las image */
)
{
   long maximg=MAXIMG;         /* max number of images allowed */ 
   long zero=0;                /* value of 0 */ 

   /* call c_getin() */ 
   if (c_getin (&pblock, parmname, &zero, ini->taename, ini->hostname, 
                &ini->nimages, ini->window, ini->bands, ini->nbands, 
                &ini->totalbands, &maximg, &ini->samewindow, ini->qflag ))
   {
      sprintf (message, "Error getting input image for parm %s.", parmname); 
      c_errmsg (message, procfail, &fatal); 
   }

   /* error check, different window sizes causes c_egroup() to fail */ 
   if (!ini->samewindow)
   {
      sprintf (message, 
      "Input images specified by parm %s do not have the same window size.",
      parmname);
      c_errmsg (message, procfail, &fatal); 
   }
/* debugging statements to dump ini components */ 
   /* 
   printf ("dumping some ini components in getin2() \n"); 
   printf ("   nimages=%d totalbands=%d samewindow=%d \n", ini->nimages,
   ini->totalbands, ini->samewindow); 
   for (index=0; index<ini->nimages; index++)
   {
      printf ("   index=%d \n", index); 
      printf ("   taename[]=%s \n", ini->taename[index]); 
      printf ("   hostname[]=%s \n", ini->hostname[index]); 
      printf ("   nbands[]=%d \n", ini->nbands[index]); 
      printf ("   window[]= %d %d %d %d \n", 
      ini->window[index][SL], ini->window[index][SS], 
      ini->window[index][NL], ini->window[index][NS]); 
   } 
   */ 
}




/* FUNCTION: get output image name */ 
void getout2
(
    char *parmname,            /* input:  tae parameter name for image */
    struct lasoutimage *outi  /* output: data structure of las image */
)
{
   if (c_getout (&pblock, parmname, outi->taename, outi->hostname) )
   {
      sprintf (message, "Error getting output image for parm %s.", parmname); 
      c_errmsg (message, procfail, &fatal); 
   }
}





/* FUNCTION: open an input image for sequential access and prepare 
   data buffer */ 
void opensin
(
    struct lasinimage *ini       /* input and output: input image to open */
) 
{
   /* declare vars */ 
   long accessm=IREAD;           /* image access mode */ 
   long bandnum;                 /* loop index, band number */ 
   long imagenum;                /* loop index, image number */ 
   long buffersz;                /* buffer size returned by ealloc */
   long zero=0;                  /* value of zero */ 
   
   /* open image file and set vars */ 
   ini->sio = TRUE;               /* T for sequential i/o */ 
   ini->fdarray[ini->nimages]=0;  /* null terminate array */ 
   ini->datatype=0; 
   for (bandnum=imagenum=0; imagenum<ini->nimages;  
        bandnum+=ini->nbands[imagenum], imagenum++)
   {
      if (c_eopens(ini->fdarray+imagenum, ini->hostname[imagenum], 
                   ini->bands[imagenum], 
                   &ini->window[imagenum][SL], &ini->window[imagenum][SS], 
                   &ini->window[imagenum][NL], &ini->window[imagenum][NS], 
                   ini->bufoff+bandnum, &accessm, &ini->datatype, &zero, &zero))
      {
         sprintf (message, "Error opening image file for image %s",
                  ini->taename[imagenum]); 
         c_errmsg (message, procfail, &fatal); 
      }
   }
   
   switch (ini->datatype)
   {
      case EBYTE:  ini->datasize=1;  break; 
      case EWORD:  ini->datasize=2;  break; 
      case ELONG:  ini->datasize=4;  break; 
      case EREAL:  ini->datasize=4;  break; 
   }
/* prepare buffers */ 
   /* allocate memory for image lines */ 
   if ((ini->ibuffer = (unsigned char *) 
        c_ealloc(ini->fdarray, &buffersz)) == NULL)
   {
         c_errmsg ("Unable to allocate memory for input image buffer.", 
                   procfail, &fatal); 
   }
   
   /* group image files */ 
   if (c_egroup (ini->fdarray, &ini->groupd, ini->ibuffer, &buffersz) )
         c_errmsg ("Error grouping input image files.", procfail, &fatal);
   /* initialize first line of image */ 
   if (c_estep (&ini->groupd) )
         c_errmsg ("Error stepping through input images.", procfail, &fatal);
}




/* FUNCTION: cleanup from an error, and delete image files */ 
void clean2
(
    char *message  /* input:  message to write */
) 
   /* additional inputs are inimage and outimage global variables */ 
{
   long one=1;     /* number of output file names in outimg.hostname */ 
   c_errmsg (message, procfail, &go_on); 
   if ((inimg.groupd != NULL) && c_egclse (&inimg.groupd) )
      c_errmsg ("Error closing input image file.", procwarn, &go_on);
   if (histimg.imagevalid && (histimg.groupd != NULL) && 
       c_egclse (&histimg.groupd) )
   {
      c_errmsg ("Error closing input image file.", procwarn, &go_on);
   }
   if ((outimg.groupd != NULL) && c_egclse (&outimg.groupd) )
      c_errmsg ("Error closing output image file.", procwarn, &go_on);
   c_clean (&pblock, procname, outimg.hostname, &one); 
}

/* FUNCTION: open an output image for sequential access and prepare 
   data buffer */ 
void opensout
(
    struct lasinimage *ini,   /* input:  input image, needed for nl,ns,nbands*/
    long minmax,              /* input:  flag, update min/max */
    char *odtype,             /* input:  data type for output image */
    struct lasoutimage *outi  /* input and output:  output image to open */
)
{
   /* declare vars */ 
   long accessm=IWRITE;   /* image access mode */ 
   long buffersz;         /* buffer size returned by ealloc */
   long one =1;           /* value of one */ 
   long zero=0;           /* value of zero */ 
   
   /* set vars in data structure */ 
   outi->nl = ini->window[0][NL]; 
   outi->ns = ini->window[0][NS]; 
   outi->sio = TRUE;      /* T for sequential i/o */ 
   outi->totalbands = ini->totalbands; 
   outi->fdarray[1] = 0;  /* null terminate array */ 
   
   /* datatype */ 
   if (!strcmp (odtype, "SAME"))
         outi->datatype = ini->datatype; 
   else if (!strcmp (odtype, "BYTE"))
         outi->datatype = EBYTE; 
   else if (!strcmp (odtype, "I*2"))
         outi->datatype = EWORD; 
   else if (!strcmp (odtype, "I*4"))
         outi->datatype = ELONG; 
   else if (!strcmp (odtype, "R*4"))
         outi->datatype = EREAL; 
   else
         c_errmsg ("Parm ODTYPE has an illegal value.", procfail, &fatal); 

   /* open image file and set datasize */ 
   if (c_eopens (outi->fdarray, outi->hostname, &outi->totalbands, 
                 &one, &one, &outi->nl, &outi->ns, outi->bufoff, &accessm, 
                 &outi->datatype, &zero, &zero) )
   {
      sprintf (message, "Error opening output image file for image %s",
      outi->taename); 
      clean2 (message); 
   }
   switch (outi->datatype)
   {
      case EBYTE:  outi->datasize=1;  break; 
      case EWORD:  outi->datasize=2;  break; 
      case ELONG:  outi->datasize=4;  break; 
      case EREAL:  outi->datasize=4;  break; 
   }
   /* prepare buffers */ 
   /* allocate memory for image lines */ 
   if ((outi->ibuffer = (unsigned char *)c_ealloc(outi->fdarray, &buffersz)) 
        == NULL)
   {
         clean2 ("Unable to allocate memory for output image buffer."); 
   }

   /* group image files */ 
   if (c_egroup (outi->fdarray, &outi->groupd, outi->ibuffer, &buffersz) )
         clean2 ("Error grouping output image file.");

   /* initialize first line of image */ 
   if (c_estep (&outi->groupd) )
         clean2 ("Error stepping through output image.");
     
   /* initialize min max arrays */ 
   outi->minmax = minmax; 
   if (outi->minmax && 
       c_inmnmx(&outi->datatype,&outi->totalbands, outi->minval, outi->maxval))
   {
      clean2 ("Error initializing min/max arrays."); 
   }
}




/* FUNCTION:  optionally update minmax for a line (or block of lines) of image 
   data */      
void minmax
(
    long band,               /* input:  image buffer holds data for this band */
    struct lasoutimage *outi /* input and output:  structure of image data to 
                                update min/max for */
) 
{
   /* return if min/max does not need updated */ 
   if (!outi->minmax)
      return; 

   /* update minmax */ 
   if (c_pxmnmx(outi->ibuffer + (outi->bufoff[band] * outi->datasize), 
                &outi->ns, outi->minval+band, outi->maxval+band, 
                &outi->datatype))
   {
      clean2 ("Error updating min max arrays."); 
   }
}




/* FUNCTION close image files and end program, last function in this file to be
   called */ 
void end_las(void)
{
   /* vars for ddr */ 
   char m_flag[8];      /* minmax flag, = YES, NO, or BOUNDED */ 
   long cflag=NOCOMB;   /* combine flag for c_upddr() */ 
   long one=1;          /* value of one */ 
   double oned=1.0;     /* one as a double */ 
   
   /* close input image files */ 
   if (c_egclse(&inimg.groupd) ) 
      c_errmsg ("Error closing input image files.", procwarn, &go_on); 
   if (histimg.imagevalid && c_egclse (&histimg.groupd) )
      c_errmsg ("Error closing input image file.", procwarn, &go_on);

   /* close output image files */ 
   if (c_egclse(&outimg.groupd)) 
      c_errmsg ("Error closing output image file.", procwarn, &go_on); 
   
   /* free buffers */ 
   free (inimg.ibuffer); 
   if (histimg.imagevalid)
      free (histimg.ibuffer); 
   free (outimg.ibuffer); 
   
   /* create ddr for outname image */ 
   if (c_upddr (&pblock, inimg.hostname, outimg.hostname, 
                &inimg.nimages, inimg.window, inimg.bands, inimg.nbands, 
                &oned, &oned, &outimg.totalbands, &cflag))
   {
      clean2 ("Error creating ddr for output image.");
   }
   
   /* update ddr with min max */ 
   if (outimg.minmax)
   {
      strcpy (m_flag, "YES"); 
      if (c_svmnmx (outimg.hostname, &outimg.totalbands, outimg.minval, 
                    outimg.maxval, m_flag))
      {
         c_errmsg ("Error updating min/max values in ddr.", procwarn, &go_on); 
      }
   }
   
   /* create history file */ 
   if (c_hstout (&pblock, outimg.hostname, inimg.hostname, 
                 &inimg.nimages, "") )
   {
      clean2 ("Error creating history file for output image."); 
   }
   if (histimg.imagevalid && 
       c_hstout (&pblock,outimg.hostname,histimg.hostname,&histimg.nimages,""))
   {
      clean2 ("Error creating history file for output image."); 
   }
   
   /* end program */ 
   c_complt(procname, outimg.taename, &one);
}


