#ifndef REDIST2_H
#define REDIST2_H

/* include file imageio.h by David Zokaites 12-92 to 2-93.  File helps to 
   consolidate LAS image processing calls.  File includes data structures for 
   LAS images, functions for evaluating TAE parms with multiple values, and 
   global vars. */
#include "las.h"    /* for las image i/o */ 



/* data structure for one las input image, used to facilitate i/o.  Because las
   image specs allow for grouping image components, (e.g. in="image1 + image2")
   one input image may correspond to multiple disk image files. */ 
   struct lasinimage
   {
      char hostname[MAXIMG][CMLEN]; /* host name for image files */ 
      char taename[MAXIMG][CMLEN];  /* tae name for image components */ 
      struct FDESC *fdarray[MAXIMG+1]; /* file descriptor array */ 
      struct GDESC *groupd;         /* group descriptor for sequential i/o */ 
      long bands[MAXIMG][MAXBND+1]; /* band numbers */ 
      long bufoff[MAXBND];          /* offsets into data buffer, for sequential
                                       i/o */ 
      long datasize;                /* number of bytes per pixel */ 
      long datatype;                /* image data type = EBYTE, EWORD, ... */ 
      long nbands[MAXIMG];          /* number of bands in each component */ 
      long nimages;                 /* number image components */ 
      long imagevalid;              /* flag, is this image valid?  this image
                                       may have no value */ 
      long qflag[MAXIMG][2];        /* flag, do image components have a band or
				       window specified */ 
      long samewindow;              /* flag, do image components have the same 
				       window size */ 
      long sio;                     /* flag, TRUE for sequential i/o, FALSE
                                       otherwise */ 
      long totalbands;              /* total number of bands */ 
      long window[MAXIMG][4];       /* window (sl ss nl ns) for each component*/
      unsigned char *ibuffer;       /* buffer for image data */ 
   }; 



/* data structure for one las output image, used to facilitate i/o */ 
   struct lasoutimage
   {
      char hostname[CMLEN];         /* host name for image file */ 
      char taename[CMLEN];          /* tae name for image */ 
      double minval[MAXBND];        /* min pixel value, used to update ddr */ 
      double maxval[MAXBND];        /* max pixel value, used to update ddr */ 
      struct FDESC *fdarray[1+1];   /* file descriptor array */ 
      struct GDESC *groupd;         /* group descriptor for sequential i/o */ 
      long bufoff[MAXBND];          /* offsets into data buffer, for sequential
				       i/o */ 
      long datasize;                /* number of bytes per pixel */ 
      long datatype;                /* image data type = EBYTE, EWORD, ... */ 
      long minmax;                  /* flag, determine min/max and update 
				       ddr? = TRUE or FALSE */ 
      long sio;                     /* flag, TRUE for sequential i/o, FALSE
                                       otherwise */ 
      long totalbands;              /* total number of bands */ 
      long nl, ns;                  /* size of output image */ 
      unsigned char *ibuffer;       /* buffer for image data */ 
   }; 





/* data structure for integer parameters with multiple values */ 
   struct iparm
   {
      long count;      /* number of values for this parm */ 
      long value[50];  /* parm values.  20 should be plenty of values as 
                          the proc only uses 5 */ 
   };




#ifdef GLOBALVARS
   /* global vars, must be global for las image i/o routines */
      struct lasinimage  inimg;    /* input image to modify */ 
      struct lasinimage  histimg;  /* input image to histogram */ 
      struct lasoutimage outimg;   /* output image */ 
      struct PARBLK pblock;        /* tae parameter block */ 
#else 
   /* global vars, must be global for las image i/o routines */
      extern struct lasinimage  inimg;   
      extern struct lasinimage  histimg;  
      extern struct lasoutimage outimg; 
      extern struct PARBLK pblock; 
#endif 


void begin_las
(
    char *proc             /* input:  name of las proc */
);

void integer_parm
(
    char *parmname,         /* input:  name of parameter */
    struct iparm *parmval  /* output: parm values */
);

void getin2
(
    char *parmname,             /* input:  tae parameter name for image */
    struct lasinimage *ini     /* output: data structure of input las image */
);

void getout2
(
    char *parmname,            /* input:  tae parameter name for image */
    struct lasoutimage *outi  /* output: data structure of las image */
);

void opensin
(
    struct lasinimage *ini       /* input and output: input image to open */
);

void clean2
(
    char *message  /* input:  message to write */
);

void opensout
(
    struct lasinimage *ini,     /* input:  input image, needed for nl,ns,nbands*/
    long minmax,                /* input:  flag, update min/max */
    char *odtype,               /* input:  data type for output image */
    struct lasoutimage *outi   /* input and output:  output image to open */
);

void minmax
(
    long band,                /* input:  image buffer holds data for this band */
    struct lasoutimage *outi /* input and output:  structure of image data to 
				update min/max for */
);

void end_las(void);

void make_hist
(
    long msgtim,                 /* input: $msgtim */
    double bweight,              /* band-specific weight for histogram */
    struct iparm *frommask,      /* BV to mask out of histogram */
    struct lasinimage *histimg,  /* input: image to histogram */
    long *chist[MAXBND]         /* output: cumulative histogram for image */
);

void make_xform
(
    long *chist[MAXBND],   /* input: cumulative histogram for image */
    long hbands,          /* input: for loop limit, bands in image */
    long maxshift,        /* input: max shift of a BV during transform */
    struct iparm *frommask,/* BV to mask out of histogram */
    struct iparm *tomask,  /* output BV for masked pixels */
    long *tlut[MAXBND]    /* output: histogram transforming lut */
);

void apply_xform
(
    long msgtim,                /* input: $msgtim */
    long *tlut[MAXBND],         /* input: histogram transforming lut */
    struct lasinimage *inimg,   /* input: image to modify */
    struct lasoutimage *outimg /* input: info on output image */
);

void debug_dump
(
    char *idstring,      /* input:  informational string to dump to disk */
    long *dlut          /* input:  data to dump */
);

#endif
