#include <string.h>
#include "scale.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
*****************************************************************************

NAME:	DOGLOBAL

PURPOSE:  Linearly rescales the input pixel intensities with optional 
          clipping of input data based on all input bands.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         03-Mar-84   J. Lavergne
    2         20-Sep-88   K. Narayanan
    3         31-Mar-91   K. Donaldson       Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   If input data type is not BYTE do the following:
      If the minimum and maximum pixel values for the whole image are equal, 
         Print an error message.
      If necessary, 
         Combine histogram bins.
   End-If
   If input data type is BYTE, 
      Set minimum and maximum values to 0 and 255 respectively.
   For all images and bands: 
      If input image data type is BYTE, 
         Call SETT2 to create a BYTE histogram.
      If input image data type is not BYTE, 
         Call SETT4 to create an I*2 histogram.
    End-For
    If input image data type is BYTE, 
      If clip flag is set, and 
         If the type of clipping for either low or high end data 
         is "PERCENT", 
            Call PERCEN2 to clip the requested number of pixels to the
            user-specified value.
         End-If
      End-If
   End-If
   If input image data type is not BYTE, 
      If clip flag is set, and 
         If the type of clipping for either low or high end data 
         is "PERCENT", 
            Call PERCEN4 to clip the requested number of pixels to the
            user-specified value.
         End-If
      End-If
   End-If
   Compute slope and offset for the output image.
   For all images and bands, 
      Call PROLAS to process the input image data by clipping and scaling.
   End-For

*****************************************************************************
*/
void doglobal
(
    char clip_flg[],		/* Clip flag */
    char cltyphi[],		/* Clipping type for high end data */
    char cltyplo[],		/* Clipping type for low end data */
    float cliphi,			/* User-requested high end clip value */
    float cliplo,			/* User-requested low end clip value */
    char scaltyp[],		/* Type of scaling to be performed */
    double amin[],			/* Minimum pixel values */
    double amax[],			/* Maximum pixel values */
    float fmaxval,		/* Maximum pixel value for the output image */
    float fminval,		/* Minimum pixel value for the output image */
    long real_flg,		/* Flag for input data of type real */
    long bands_per_img[MAXIMG], 	/* Number of bands in each image */
    long dtype,			/* Data type of input image */
    long num_img_band[MAXIMG][MAXBND+1], /* Band numbers for each input image */
    long num_inimg,		/* Number of input images */
    long tot_inband		/* Total number of input image bands */
)
{

long    three = 3;
long    seven = 7;
char    *prog_name = "SCALE";   /* program name           */


double	clip_range,		/* Range of clip values */
	pix_range[MAXBND];	/* Range of pixel values in each band */

float	clipval_hi, 		/* Final (computed) high end clip value */
	clipval_lo,		/* Final (computed) low end clip value */
	hoffset = 0.0,		/* Histogram offset */
	hscale = 0.0, 		/* Histogram scaling factor */
	imgmax, 		/* Maximum pixel value for the whole image */
	imgmin,			/* Minimum pixel value for the whole image */
	offset,			/* Image scale offset */
	slope;			/* Image scale factor */

long	band,			/* Current band being processed */
	bufsize,		/* Buffer size for dynamic memory allocation */
	fill_num,		/* Number of values to fill */
	fill_type,		/* Data type of the buffer to be filled */
	fill_val,		/* Value to be filled */
	hist[256], 		/* Histogram array for byte data */
	hist4[MAX_BINS],	/* Histogram array for I*2, I*4, and R*4 data */
	i,			/* Loop counter and array index */
	img,			/* Current image being processed */
	in_band,		/* Current band number from the image specs */
	nbins = 0,			/* Number of histogram bins */
	outband;		/* Output band number */
/*	size;			Size of the input image data type in bits */

/*
Initialize variables.
---------------------
*/
imgmax = -1.7e+38;
imgmin = 1.7e+38;
outband = 0;
fill_val = 0;
fill_num = 256;
fill_type = ELONG;
c_pxfill (hist, &fill_type, &fill_num, &fill_val);
fill_num = MAX_BINS;
c_pxfill (hist4, &fill_type, &fill_num, &fill_val);

/*
Allocate the dynamic memory for the input and output buffers.
--------------------------------------------------------------
*/
bufsize = (long) (window[0][NS] * 4 * sizeof(unsigned char));
if ((inbuf = (unsigned char *) malloc (bufsize)) == NULL)
   c_abort (&block, "Cannot allocate dynamic memory", "scale-alloc", prog_name,
            out_files, &num_outfiles);
bufsize = (long) (window[0][NS] * sizeof (long));
if ((outbuf = (long *) malloc (bufsize)) == NULL)
   c_abort (&block, "Cannot allocate dynamic memory", "scale-alloc", prog_name,
            out_files, &num_outfiles);

/*
If input data type is not BYTE do the following.
If the minimum and maximum pixel values for the whole image are equal, 
print an error message.
----------------------------------------------------------------------
*/
if (dtype != EBYTE) 
   {
   for (i = 0; i < tot_inband; i++)
      {
      if (imgmin > (float) amin[i])
         imgmin = (float) amin[i];
      if (imgmax < (float) amax[i])
         imgmax = (float) amax[i];
      }
   if (imgmax == imgmin) 
      c_abort (&block, 
               "The input image is a constant image - no scaling possible", 
               "scale-imgcon", prog_name, out_files, &num_outfiles);

/*
If necessary, combine histogram bins.
--------------------------------------
*/
   if ((imgmax - imgmin <= (float)(MAX_BINS)) && (!real_flg)) 
      {
      nbins = (long) (imgmax - imgmin) + 1;
      hscale = 1.0;
      hoffset = 0.0 - imgmin;
      }
   else
      {
      nbins = MAX_BINS - 1;
      hscale = (float) (nbins) / (float) (imgmax - imgmin);
      hoffset = (float) (nbins) - (float) (hscale * imgmax);
      }  
   }

/*
If input data type is BYTE, set minimum and maximum values to 0 and
255 respectively.
------------------------------------------------------------------------
*/
else
   {
   amin[0] = 255;
   amax[0] = 0;
   }
   
/*
For all images and bands: 
If input image data type is BYTE, call SETT2 to create a BYTE histogram.
If input image data type is not BYTE, call SETT4 to create an I*2 histogram.
-----------------------------------------------------------------------------
*/
for (img = 0; img < num_inimg; img++)
   {
   for (band = 0; band < bands_per_img[img]; band++)
      {
      in_band = num_img_band[img][band];
      if (dtype == EBYTE) 
         sett2 (hist, &amin[0], &amax[0], scaltyp, img, in_band);
      else
         sett4 (hist4, hscale, hoffset, img, in_band, dtype);
      }  
   }

/*
If input image data type is BYTE, if clip flag is set, and if the type of 
clipping for either low or high end data is "PERCENT", call PERCEN2 to clip
the requested number of pixels to the user specified value.
----------------------------------------------------------------------------
*/
if (dtype == EBYTE) 
   {
   if (strncmp (clip_flg, "YES", three) == 0)
      {
      if ((strncmp (cltyphi, "PERCENT", seven) == 0)|| 
          (strncmp (cltyplo, "PERCENT", seven) == 0)) 
         percen2 (hist, amin[0], amax[0], cltyphi, cltyplo, &clipval_hi, 
                  &clipval_lo, cliphi, cliplo);
      else
         {
         clipval_lo = cliplo;
         clipval_hi = cliphi;
         }
      }  
   }

/*
If input image data type is not BYTE, if clip flag is set, and if the type 
of clipping for either low or high end data is "PERCENT", call PERCEN4 to
clip the requested number of pixels to the user specified value.
----------------------------------------------------------------------------
*/
else
   {
   if (strncmp (clip_flg, "YES", three) == 0)
      {
      if ((strncmp (cltyphi, "PERCENT", seven) == 0)|| 
          (strncmp (cltyplo, "PERCENT", seven) == 0)) 
         percen4 (hist4, hscale, hoffset, nbins, cltyphi, cltyplo, 
                  &clipval_hi, &clipval_lo, cliphi, cliplo);
      else
         {
         clipval_lo = cliplo;
         clipval_hi = cliphi;
         }
      }  
   }  

/*
Compute slope and offset for the output image.
-----------------------------------------------
*/
if (amax[0] == amin[0]) 
   c_abort (&block, 
            "The input image is a constant image - no scaling possible", 
            "scale-imgcon", prog_name, out_files, &num_outfiles);

if (strncmp (clip_flg, "YES", three) == 0)
   {
   clip_range = clipval_hi;
   clip_range = clip_range - clipval_lo;
   slope = (fmaxval - fminval) / clip_range;
   offset = fmaxval - (slope * clipval_hi); 
   }
else
   {
   pix_range[0] = amax[0];
   pix_range[0] = pix_range[0] - amin[0];
   slope = (fmaxval - fminval) / pix_range[0];
   offset = fmaxval - (slope * amax[0]); 
   }  

/*
For all images and bands, call PROLAS to process the input image data
by clipping and scaling.
----------------------------------------------------------------------
*/
curcat = 1;
curent = 1;
c_startm (); 
for (img = 0; img < num_inimg; img++)
   {
   for (band = 0; band < bands_per_img[img]; band++)
      {
      outband++;
      in_band = num_img_band[img][band];
      prolas (clip_flg, clipval_hi, clipval_lo, fmaxval, fminval, slope,
              offset, img, in_band, outband, dtype);
      curcat++;
      }  
   }
c_stoptm (); 
}
