#include <string.h>
#include "scale.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
*****************************************************************************

NAME:	DOLOCAL

PURPOSE:  Linearly rescales the input pixel intensities with optional 
          clipping of input data for each band separately.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         03-Mar-84   J. Lavergne
    2         20-Sep-88   K. Narayanan
    3         31-Mar-91   K. Donaldson       Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   Allocate the dynamic memory for the input and output buffers.
   If for any band the maximum and mininum pixel values are the same,
      Print an error message.
   End-If
   For all input images and all input bands:
      If input image data type is not BYTE, 
         Set the histogram bins, histogram scale and histogram offset.
      End-If
      If input image data type is BYTE, 
         Call SETT2 to create a byte histogram.
         If the type of clipping for either low or high end data is "PERCENT",
            Call PERCEN2 to clip the requested number of pixels to the 
            user-specified value.
         End-If
      End-If
      If input image data type is not BYTE, 
         Call SETT4 to create an I*2 histogram.
         If the type of clipping for either low or high end data is "PERCENT",
            Call PERCEN4 to clip the requested number of pixels to the 
            user-specified value.
         End-If
      End-If
      Compute slope and offset for the image data.
      Call PROLAS to process the input image data by clipping and scaling.
      Re-initialize the histogram.
   End-For

*****************************************************************************
*/
void dolocal
(
    char clip_flg[],		/* Clip flag */
    char cltyphi[],		/* Clipping type for high end data */
    char cltyplo[],		/* Clipping type for low end data */
    float cliphi,			/* User-requested high end clip value */
    float cliplo,			/* User-requested low end clip value */
    char scaltyp[],		/* Type of scaling to be performed */
    double amin[],			/* Minimum pixel values */
    double amax[],			/* Maximum pixel values */
    float fmaxval,		/* Maximum pixel value for the output image */
    float fminval,		/* Minimum pixel value for the output image */
    long real_flg,		/* Flag for input data of type real */
    long bands_per_img[],  	/* Number of bands in each image */
    long dtype,			/* Data type of input image */
    long num_img_band[][MAXBND+1], /* Band numbers for each input image */
    long num_inimg,		/* Number of input images */
    long tot_inband		/* Total number of input image bands */
)
{

long    three = 3;
long    seven = 7;
char    *prog_name = "SCALE";   /* program name           */


char	msg[80];		/* Message text buffer */

double	clip_range,		/* Range of clip values */
	pix_range[MAXBND];	/* Range of pixel values in each band */

float	clipval_hi, 		/* Final (computed) high end clip value */
	clipval_lo,		/* Final (computed) low end clip value */
	hoffset = 0.0,		/* Histogram offset */
	hscale = 0.0, 		/* Histogram scaling factor */
	offset,			/* Image scale offset */
	slope;			/* Image scale factor */

long	band,			/* Current band being processed */
	bufsize,		/* Buffer size for dynamic memory allocation */
	fill_num,		/* Number of values to fill */
	fill_type,		/* Data type of the buffer to be filled */
	fill_val,		/* Value to be filled */
	hist[256], 		/* Histogram array for byte data */
	hist4[MAX_BINS],	/* Histogram array for I*2, I*4, and R*4 data */
	i,			/* Loop counter and array index */
	img,			/* Current image being processed */
	in_band,		/* Current band number from the image specs */
	nbins = 0,			/* Number of histogram bins */
	outband;		/* Output band number */
/*	size;			 Size of the input image data type in bits */

outband = 0;

/*
Initialize variables.
---------------------
*/
fill_val = 0;
fill_num = 256;
fill_type = ELONG;
c_pxfill (hist, &fill_type, &fill_num, &fill_val);
fill_num = MAX_BINS;
c_pxfill (hist4, &fill_type, &fill_num, &fill_val);

/*
Allocate the dynamic memory for the input and output buffers.
--------------------------------------------------------------
*/
bufsize = (long) (window[0][NS] * 4 * sizeof(unsigned char));
if ((inbuf = (unsigned char *) malloc (bufsize)) == NULL)
   c_abort (&block, "Cannot allocate dynamic memory", "scale-alloc", prog_name,
            out_files, &num_outfiles);
bufsize = (long) (window[0][NS] * sizeof (long));
if ((outbuf = (long *) malloc (bufsize)) == NULL)
   c_abort (&block, "Cannot allocate dynamic memory", "scale-alloc", prog_name,
            out_files, &num_outfiles);

/*
If for any band the maximum and mininum pixel values are the same, print
an error message.
--------------------------------------------------------------------------
*/
for (i = 0; i < tot_inband; i++)
   {
   if (amin[i] == amax[i])
      {
      sprintf (msg, 
         "Band %d of the input image is a constant image - no scaling possible",
          i+1);
      c_abort (&block, msg, "scale-bndcon", prog_name, out_files, 
               &num_outfiles);
      }  
   pix_range[i] = amax[i] - amin[i];
   }

/*
For all input images and all input bands.
-----------------------------------------
*/
curcat = 1;
curent = 1;
c_startm ();
for (img = 0; img < num_inimg; img++)
   {
   for (band = 0; band < bands_per_img[img]; band++)
      {
      outband++;
      in_band = num_img_band[img][band];
/*
If input image data type is not BYTE, set the histogram bins, histogram scale
and histogram offset.
-------------------------------------------------------------------------------
*/
      if (dtype != EBYTE) 
         {
         if ((pix_range[band] <= MAX_BINS) && (!real_flg))
            {
            nbins = (long)(pix_range[band]) + 1;
            hscale = 1.0;
            hoffset = 0.0 - amin[band];
            }
         else
            {
            nbins = MAX_BINS - 1;
            hscale = (float) (nbins) / pix_range[band];
            hoffset = (float) (nbins) - (hscale * amax[band]);
            }  
         }

/*
Call SETT2 to create a byte histogram.
If the type of clipping for either low or high end data is "PERCENT", call
PERCEN2 to clip the requested number of pixels to the user specified value.
----------------------------------------------------------------------------
*/
      if (dtype == EBYTE)  
         {
         sett2 (hist, &amin[band], &amax[band], scaltyp, img, in_band);
         pix_range[band] = amax[band] - amin[band];
         if (strncmp (clip_flg, "YES", three) == 0)
            {
            if ((strncmp (cltyphi, "PERCENT", seven) == 0)|| 
                (strncmp (cltyplo, "PERCENT", seven) == 0)) 
               percen2 (hist, amin[band], amax[band], cltyphi, cltyplo, 
                        &clipval_hi, &clipval_lo, cliphi, cliplo);
            else
               {
               clipval_lo = cliplo;
               clipval_hi = cliphi;
               }
            }  
         }
/*
If input image data type is not BYTE, call SETT4 to create an I*2 histogram.
If the type of clipping for either low or high end data is "PERCENT", call
PERCEN4 to clip the requested number of pixels to the user specified value.
----------------------------------------------------------------------------
*/
      else
         {
         sett4 (hist4, hscale, hoffset, img, in_band, dtype);
         if (strncmp (clip_flg, "YES", three) == 0)
            {
            if ((strncmp (cltyphi, "PERCENT", seven) == 0)|| 
                (strncmp (cltyplo, "PERCENT", seven) == 0)) 
               percen4 (hist4, hscale, hoffset, nbins, cltyphi, cltyplo, 
                        &clipval_hi, &clipval_lo, cliphi, cliplo);
            else
               {
               clipval_lo = cliplo;
               clipval_hi = cliphi;
               }
            }  
         }  

/*
Compute slope and offset for the image data.
---------------------------------------------
*/
      if (amax[band] == amin[band]) 
         {
         sprintf (msg, 
         "Band %d of the input image is a constant image - no scaling possible",
                  band+1);
         c_abort (&block, msg, "scale-bndcon", prog_name, out_files, 
                  &num_outfiles);
         }

      if (strncmp (clip_flg, "YES", three) == 0)
         {
         clip_range = clipval_hi - clipval_lo;
         slope = (fmaxval - fminval) / clip_range;
         offset = fmaxval - (slope * clipval_hi); 
         }
      else
         {
         slope = (fmaxval - fminval) / pix_range[band];
         offset = fmaxval - (slope * amax[band]); 
         }  

/*
Call PROLAS to process the input image data by clipping and scaling.
--------------------------------------------------------------------
*/
      prolas (clip_flg, clipval_hi, clipval_lo, fmaxval, fminval, slope,
              offset, img, in_band, outband, dtype);
      curcat++;;

/*
Re-initialize the histogram.
-----------------------------
*/
      if (dtype == EBYTE) 
         {
         fill_num = 256;
         c_pxfill (hist, &fill_type, &fill_num, &fill_val);
         }
      else
         {
         fill_num = nbins + 1;
         c_pxfill (hist4, &fill_type, &fill_num, &fill_val);
         }
      }  
   }
c_stoptm (); 
}

