#include <string.h>
#include "scale.h"
#include "util.h"
#include "protos.h"

/*
*****************************************************************************

NAME:	GETPAR

PURPOSE: Get parameters.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         05-Mar-84  J. Lavergne
    2         19-Jul-84  S. Ross             SCN#20-320
    3         31-Mar-91  K. Donaldson        Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize TAE.
   Parse the input image specification.
   If the input image windows are not equal, 
      Abort with a fatal error message.
   Get the output image file name.
   Get the type of scaling to be performed.
   Get the clipping specifications
   Validate the clipping specifications.
   Compute the output data type.
   Get the LAS gobals

*****************************************************************************
*/
void getpar
(
    char hosin[][CMLEN], 	/* Input image host names */
    char hosout[][CMLEN],		/* Output image host names */
    char taein[][CMLEN], 	/* Input image tae names */
    char taeout[][CMLEN],		/* Output image tae names */
    char scaltyp[],		/* Type of scaling to be performed */
    long *bands_per_img, 	/* Number of bands per input image */
    long num_img_band[][MAXBND+1], /* Band numbers for each input image */
    long *num_inimg,		/* number of input images */
    float *fmaxval,		/* Maximum pixel value for the output image */
    float *fminval,		/* Minimum pixel value for the output image */
    char clip_flg[],		/* Clipping flag */
    float *cliphi,		/* User-requested high end clip value */
    float *cliplo,		/* User-requested low end clip value */
    char cltyphi[],		/* Clipping type for the high end of data */
    char cltyplo[],		/* Clipping type for the low end of data */
    long *tot_inband,            /* Total number of bands to be processed */
    long *dtype,			/* Data type of the input image */
    long *outtype,		/* Data type of the output image */
    long *real_flg		/* Real data type flag */
)

{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;
long    seven = 7;

char	mnmx[4];		/* minmax option */

long	i,			/* Loop counter and array index */
	img_flag[MAXIMG][2],    /* Specified image/band options */
	img_limit,		/* maximum number of images */
	minto, 			/* User-requested maximum pixel value for the */
				/* output image */
	maxto,			/* User-requested minimum pixel value for the */
				/* output image */
	partype,		/* Parameter data type */
	same_wnd,		/* Same window size flag: 1=YES, 0=NO */
	status,			/* Return status */
	std_out;		/* logical unit for standard output */

struct VARIABLE *var;	/* pointer of a variable in TAE block */

img_limit = MAXIMG;

/* 
Initialize TAE.
-----------------
*/
c_inlas(&block, &std_out);

/*
Parse the input image specification
-----------------------------------
*/
status = c_getin (&block, "IN", &zero, taein, hosin, num_inimg, window, 
		  num_img_band, bands_per_img, tot_inband, &img_limit, 
		  &same_wnd, img_flag); 
if (status != E_SUCC)
	c_errmsg ("Cannot access input file names", "scale-getin",
                   &fatal_err);

/*
If the input image windows are not equal, abort with a fatal error message.
---------------------------------------------------------------------------
*/
if (same_wnd == 0)
	c_errmsg ("Image windows are not the same size", "scale-imgsize",
                   &fatal_err);
/* 
Get the output image file name.
---------------------------------
*/
status = c_getout (&block, "OUT", taeout, hosout);
if (status != E_SUCC)
   c_errmsg ("Cannot open output image file", "scale-getout", &fatal_err);
num_outfiles = 0;
strcpy (out_files[num_outfiles++], taeout[0]);

/*
Get the type of scaling to be performed.
-----------------------------------------
*/
var = p_find (&block, "SCALTYP");
strcpy (scaltyp, SVAL(*var, 0));

/*
Get clipping specifications
-----------------------------
*/
partype = ELONG;
c_getnum (&block, "MAXVAL", &one, &partype, &maxto); 
c_getnum (&block, "MINVAL", &one, &partype, &minto); 
*fminval = (float) minto;
*fmaxval = (float) maxto;

partype = EREAL;
c_getnum (&block, "CLIPLO", &one, &partype, cliplo); 
c_getnum (&block, "CLIPHI", &one, &partype, cliphi); 

var = p_find (&block, "CLIPFLG");
strcpy (clip_flg, SVAL(*var, 0));
var = p_find (&block, "HICLOPT");
strcpy (cltyphi, SVAL(*var, 0));
var = p_find (&block, "LOWCLOPT");
strcpy (cltyplo, SVAL(*var, 0));

/*
Validate clipping specifications.
----------------------------------
*/
if (*fminval >= *fmaxval  ||  *fmaxval <= *fminval)
   c_errmsg ("Invalid value given for MINTO and/or MAXTO", "scale-badval",
             &fatal_err); 

if ((clip_flg[0] == 'Y') || (clip_flg[0] == 'y')) 
    {
    if (((strncmp (cltyplo, "PERCENT", seven) == 0)  && 
        ((*cliplo >= 100) || (*cliplo < 0))) || 
        ((strncmp (cltyphi, "PERCENT", seven) == 0)  && 
        ((*cliphi >= 100) || (*cliphi < 0))) || 
        ((strncmp (cltyplo, "PERCENT", seven) == 0)  && 
         (strncmp (cltyphi, "PERCENT", seven) == 0)  && 
         ((*cliplo + *cliphi) >= 100))) 
      {
      c_errmsg ("Invalid values given for percentage clipping", 
                "scale-badper", &fatal_err); 
      }
   else if (((strncmp (cltyplo, "PERCENT", seven) == 0)  &&  (*cliplo == 0)) || 
            ((strncmp (cltyphi, "PERCENT", seven) == 0)  &&  (*cliphi == 0))) 
      {
      c_errmsg ("No clipping limits given for percentage clipping", 
                "scale-noclip", &fatal_err); 
      }
   }

/*
Compute the output data type.
-----------------------------
*/
*real_flg = FALSE;
c_getddr (hosin[0], &ddr);
   *dtype = ddr.dtype;
for (i = 1; i < *num_inimg; i++)
   {
   c_getddr (hosin[i], &ddr);
   if (ddr.dtype != *dtype) 
      c_errmsg ("Data types of the input images are not the same",
                "scale-dtype", &fatal_err);
   }
if (*dtype == EREAL) 
   *real_flg = TRUE;

*outtype = EBYTE;
if (*fminval < 0 || *fminval > 255 || *fmaxval < 0 || *fmaxval > 255)
   *outtype = EWORD;

/* 
Get LAS gobals
--------------
*/
partype = ELONG;
c_getnum (&block, "$MSGTIM", &one, &partype, &msgtim);
var = p_find (&block, "$MINMAX");
strcpy (mnmx, SVAL(*var, 0));
if (mnmx[0] == 'Y')
   min_max = TRUE;
else
   min_max = FALSE;

}
