
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "statio.h"
#include "dltf.h"
#include "protos.h"
#define MAX_BINS 4097                         /* number of histgram bins */

static  long    one = 1;
static  long    five = 5;
static  char    *prog_name = "SCALE";   /* program name           */

struct PARBLK block;                   /* TAE parameter block     */
struct DDR  ddr;                /* DDR structure */

struct FDESC *fdescin[MAXIMG+1];             /* input file descriptors   */
struct FDESC *fdescout[2];                   /* output file descriptor   */
long     min_max;                       /* minmax option flag      */
long     msgtim;                        /* Processing message interval */
long    num_outfiles;                   /* number of files opened */
long     window[MAXIMG][4];             /* window specification */
long     *outbuf;                       /* image output buffer     */
extern long     curcat;                 /* message utility         */
extern long     curent;                 /* parameters for processing */
char    out_files[MAXIMG][CMLEN];       /* names of files opened */
char    print_file[CMLEN];
unsigned char    *inbuf;                /* image input buffer      */

/******************************************************************************

NAME:	SCALE

PURPOSE:  Linearly rescales the input pixel intensities with optional 
          clipping of input data.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         03-Mar-84   J. Lavergne
    2         20-Sep-88   K. Narayanan
    3         31-Mar-91   K. Donaldson       Conversion to 5.0
    4	      24-Sep-92	  T. Mittan	     Modified include file to eliminate
					     warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   Call GETPAR to retrieve user-specified parameters.
   Compute the maximum and minimum pixel values for each band in the 
   input image.
   Call FILSPEC to open the input and output images.
   Initialize and enable the processing message utility.
   If scaling type is "LOCAL", 
      Call DOLOCAL to process the images.
   If scaling type is "GLOBAL", 
      call DOGLOBAL to process the images.
   Close the input and output files.
   Compute and update (if needed) the minimum and maximum values for the
   output image.
   Free the dynamic memory.
   Write the history record.
   Write the completion message.

*****************************************************************************
*/

int main
(void) /* scale */
{

char	clip_flg[4],		/* Clip flag */
	cltyphi[8],		/* Clipping type for high end data */
	cltyplo[8],		/* Clipping type for low end data */
	hosout[1][CMLEN],		/* Output image host names */
	scaltyp[7];		/* type of scaling to be performed */

	
static char hosin[MAXIMG][CMLEN];	/* Input image host names */

char        taein[MAXIMG][CMLEN],	/* Input image TAE names */
            taeout[MAXIMG][CMLEN];	/* Output image TAE names */

 double	amax[MAXBND],		/* Maximum pixel values */
	amin[MAXBND];		/* Minimum pixel values */

float	cliphi,			/* User-requested high end clip value */
	cliplo,			/* User-requested low end clip value */
	fmaxval,		/* Maximum pixel value for the output image */
	fminval;		/* Minimum pixel value for the output image */

long	bands_per_img[MAXIMG],  /* Number of bands in each image */
	dtype,			/* Data type of input image */
	img,			/* Current image being processed */
	msgflag,		/* Processing message utility flag */
	num_inimg,		/* Number of input images */
        out_bands,		/* Number of output image bands */
	outtype,		/* Data type of the output image */
	outwin[1][4],		/* Output image window specification */
	real_flg,		/* Flag for input data of type real */
	tot_inband;		/* Total number of input image bands */

static long num_img_band[MAXIMG][MAXBND+1]; /* Band numbers for each input image */
static long out_img_band[MAXIMG][MAXBND+1]; /* Band numbers for each input image */
/*
Call GETPAR to retrieve user-specified parameters.
---------------------------------------------------
*/
getpar (hosin, hosout,taein,taeout, scaltyp, bands_per_img, num_img_band, 
        &num_inimg, &fmaxval, &fminval, clip_flg, &cliphi, &cliplo, cltyphi,
        cltyplo, &tot_inband, &dtype, &outtype, &real_flg);
out_bands = tot_inband;

/*
Compute the maximum and minimum pixel values for each band in the input image.
------------------------------------------------------------------------------
*/
c_minmax (hosin, &num_inimg, window, num_img_band, bands_per_img, amin, amax);

/*
Call FILSPEC to open the input and output images.
--------------------------------------------------
*/
filspec (hosin, hosout[0], bands_per_img, num_img_band, num_inimg,
         out_bands, out_img_band, outtype);

/*
Initialize and enable the processing message utility.
-----------------------------------------------------
*/
msgflag = ONECAT;
c_inittm ("SCALE", "Processed", "line", "band", "", &window[0][NL], 
          &tot_inband, &msgtim, &msgflag);

/*
If scaling type is "LOCAL", call DOLOCAL to process the images.
----------------------------------------------------------------
*/
if (strncmp (scaltyp, "LOCAL", five) == 0)
   {
   dolocal (clip_flg, cltyphi, cltyplo, cliphi, cliplo, scaltyp, amin, amax, 
            fmaxval, fminval, real_flg, bands_per_img, dtype, num_img_band, 
            num_inimg, tot_inband);
   }
/*
If scaling type is "GLOBAL", call DOGLOBAL to process the images.
----------------------------------------------------------------
*/
else      
   {
   doglobal (clip_flg, cltyphi, cltyplo, cliphi, cliplo, scaltyp, amin, amax, 
             fmaxval, fminval, real_flg, bands_per_img, dtype, num_img_band, 
             num_inimg, tot_inband);
   }  

/*
Close the input and output files.
---------------------------------
*/
for (img = 0; img < num_inimg; img++)
   c_eclose (&fdescin[img]);
c_eclose (&fdescout[0]);

/*
Compute and update (if needed) the minimum and maximum values for the
output image.
----------------------------------------------------------------------
*/
if (min_max)
   {
   outwin[0][SL] = outwin[0][SS] = 1;
   outwin[0][NL] = window[0][NL];
   outwin[0][NS] = window[0][NS];
   c_minmax (hosout, &one, outwin, out_img_band, &out_bands, amin, amax);
   c_svmnmx (hosout[0], &out_bands, amin, amax, "YES");
   }


/*
Write the history record.
-------------------------
*/
c_hstout (&block, hosout[0], hosin, &num_inimg, " ");

/*
Write the completion message.
-----------------------------
*/
c_complt (prog_name, out_files, &num_outfiles);
/*

Free the dynamic memory.
------------------------
*/
free (inbuf);
free (outbuf);

return EXIT_SUCCESS;

}
