#include "stretch.h"
#include "util.h"
#include "protos.h"

/*
----------------------------------------------------------------------------
NAME: STATISTIC                         

PURPOSE:  Computes the minimum pixel value, maximum pixel value, mean value, 
          and standard deviation of an image.  A window and step size may 
          be applied to the input image if supplied.

PROGRAM HISTORY:
      Version          Date           Author		Request
      -------          ----           ------		-------
        1      	     20-Sep-84 	     J. Hwang
        2	     28-Jun-89	     B. Allam		Conversion
        3            22-Mar-91       K. Donaldson       Added comments

ALGORITHM:

   Get the processing message interval 
   Open input image file.
   Allocate space for I/O buffer.
   Group the input images.
   Initialize the message timing facility.
   Calculate the minminum and maximum pixel values, 
   the mean and standard deviation.
   Close the input image file.
   Free the dynamic memory.
----------------------------------------------------------------------------
*/
void statistic
(
    char hosnam[],		/* Host name of input image */
    long window[][4],		/* Processing image window */
    long inc[2],			/* Step size: inc[0] = x direction */
    float *img_min,		/* Minimum pixel value in the input image */
    float *img_max,		/* Maximum pixel value in the input image */
    float *mean, 			/* Mean value of the input image */
    float *stdv			/* Standard deviation of the input image */
)
{

long    zero = 0;

float	n;			/* Running count */

long	access,			/* File access type */
        bufsiz;			/* Image I/O buffer size */
struct FDESC *fdesc[2]; 		/* File descriptors */
struct GDESC *gdesc; 			/* Group descriptor */
long   i, j, 			/* Loop counters and array indices */
        ipnt,			/* Input pointer */
	msgtim,			/* Processing message interval */
	msg_flag, 		/* Processing message flag */
        nib[2],			/* Band offset */
	status,			/* Return status */
	type;			/* Data type */

float   *buf;			/* I/O buffer */

struct VARIABLE *var; /* Function/variable for parameter retrieval */

/*
Get the processing message interval.
------------------------------------
*/
var = p_find(&block, "$MSGTIM"); 
msgtim = IVAL(*var,0);

/*
Open input image file.
----------------------
*/
access = IREAD;
type = EREAL;
nib[0] = 1; 
nib[1] = 0;
status = c_eopens(fdesc, hosnam, nib, &window[0][SL],
                  &window[0][SS], &window[0][NL], &window[0][NS], &ipnt,
                  &access, &type, &zero, &zero);      
if (status != E_SUCC)
   msg_err("!Error opening input image", "stretch-input");
fdesc[1] = 0;

/*
Allocate space for I/O buffer.
------------------------------
*/
if ((buf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
   msg_err("!Error allocating dynamic memory", "stretch-alloc");

/*
Group the input images.
-----------------------
*/
if (c_egroup(fdesc, &gdesc, buf, &bufsiz) != E_SUCC)
   msg_err("!Error grouping image bands", "scatter-group");

n = 0.0;
*mean = 0.0;
*stdv = 0.0;
*img_min = 0.0;
*img_max = 0.0;

/*
Initialize the message timing facility.
---------------------------------------
*/
msg_flag = GEN2;
c_inittm ("STATISTIC", "Processed", "line", "", "", &window[0][NL], &zero, 
           &msgtim, &msg_flag);
curent = 1;
c_startm(); 

/*
Calculate the minminum and maximum pixel values, the mean and standard 
deviation.
-----------------------------------------------------------------------
*/
for (i = 1; i <= window[0][2]; i++)
   {
   c_estep (&gdesc);
   if (((i-1) % inc[1]) == 0) 
      {
      for (j = 1; j <= window[0][3]; j++)
         {
         if (((j-1) % inc[0]) == 0) 
            {
            n = n + 1.0;
            *stdv = *stdv + (n - 1.0) * (buf[ipnt+j-1] - *mean) *
                    ((buf[ipnt+j-1] - *mean) / n);
            *mean = *mean + (buf[ipnt+j-1] - *mean) / n;
            if (n == 1.0) 
               {
               *img_min = buf[ipnt+j-1];
               *img_max = buf[ipnt+j-1];
               }  
            if ((float) (*img_min) > buf[ipnt+j-1]) 
               *img_min = buf[ipnt+j-1];
            if ((float) (*img_max) < buf[ipnt+j-1]) 
               *img_max = buf[ipnt+j-1];
            }  /* if */
         }  /* do */
      }  /* if */
      curent++;
   }  /* do */

c_stoptm(); 

/*
Close the input image file.
---------------------------
*/
c_egclse(&gdesc);

*stdv = sqrt(*stdv/n);

/*
Free the dynamic memory.
------------------------
*/
free (buf);
}
