#include <stdlib.h>
#include "stretch.h"
#include "protos.h"

void stdist
(
    long lut_type,		/* LUT type */
    char hosnam[][CMLEN],	/* Input image host name */
    long window[][4] 		/* Processing window */
)

{
/*
----------------------------------------------------------------------------
NAME: STDIST

FUNCTION:   Produces a distribution stretch look-up table.  The user 
            specifies an array of 9 lut values to be matched with the 
            following 9 input pixel values: lutmin, mean-3*sigma, mean-2*sigma,
	    mean-sigma, mean, mean+sigma, mean+2*sigma, mean+3*sigma.

PROGRAM HISTORY:
      Version          Date        Author            Request
      -------          ----      ------     	     -------
        1      	     ??-Aug-83   J. Hagedorn       
	2     	     24-Dec-86   R. Mckinney         prf 3034         
        3	      7-Jul-89   B. Allam            conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   If the look-up table type is BYTE, 
      set the value limits to 0 and 255.
   If the look-up table type is I*2, 
      set the value limits to -32768 and 32767.
   Allocate dynamic memory for the look-up table.
   Accept and decode look-up table values.
   Call STATISTIC to read the image and compute statistcs.
   Set up input pixel value array
   Create the partial look-up table.
   Construct the look-up table by linear interpolation between entries 
   in partial look-up table.

   NOTE: The nine elements of the user specified lut value array 
         correspond (in order) to the input pixel values:   
		lut_min
		mean - 3*sigma
		mean - 2*sigma
		mean - sigma
		mean
		mean + sigma
		mean + 2*sigma
		mean + 3*sigma
		lut_max
-----------------------------------------------------------------------------
*/

char	instring[9][4];			/* Input string for look-up table */
      					/* values */

float	mean,  			        /* Mean of image */
	img_max, 			/* Maximum pixel value in the image */
	img_min, 			/* Minimum pixel value in the image */
        sigma;				/* Standard deviation */

long	curr,				/* Current position */
	ascii_val,			/* Ascii value of character */
	i, jj,				/* Loop counters and array indices */
	in[9], 				/* Input values for partial lut */
	inc[2], 			/* Step size */
	l_size, 			/* LUT size */
	lutmax, 			/* Maximum LUT range value */
	lutmin,				/* Minimum LUT range value */
	next, 				/* Continue flag */
	out[9], 			/* Output values for partial lut */
	pix,				/* Pixel index */
	plut[9][2],			/* Partial look-up table */
	temp_val;			/* Intermediate result */

struct VARIABLE *var; /* Function/variable for parameter retrieval */

/*
If the look-up table type is BYTE, set the value limits to 0 and 255.
If the look-up table type is I*2, set the value limits to -32768 and 32767.
----------------------------------------------------------------------------
*/
if (lut_type == 1) 
   {
   lutmin = 0;
   lutmax = 255;
   }
else
   {
   lutmin = -32768;
   lutmax = 32767;
   }  

/*
Allocate dynamic memory for the look-up table.
-----------------------------------------------
*/
l_size = (long) ((lutmax - lutmin + 1) * sizeof(short));
if ((templut = (short *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table", 
            "stretch-lutalloc");
if (lut_type == 1)
   lut = &templut[0];
else
   lut = &templut[(lutmax - lutmin + 1) / 2];

/*
Accept and decode look-up table values.
-----------------------------------------
*/
var = p_find (&block, "LUTVAL");
for (i = 0; i < 9; i++)
  strcpy(instring[i], SVAL(*var, i));

for (i = 0; i < 9; i++)
   {
   next = 0;
   for (jj = 0; jj < strlen(instring[i]); jj++)
      {
      ascii_val = (long)(instring[i][jj]);
      if (ascii_val < 48  ||  ascii_val > 57) 
         {
         out[i] = 32768;
         next = 1; 
         break;
         }  
      }
   if (next) continue;
   out[i] = atol(instring[i]);
   }

/*
Call STATISTIC to read the image and compute statistcs.
--------------------------------------------------------
*/
inc[0] = 1;
inc[1] = 1;
statistic (hosnam[0], window, inc, &img_min, &img_max, &mean, 
           &sigma);
printf ("\n  MEAN VALUE = %12.4f", mean);
printf ("\n  STANDARD DEVIATION = %12.4f\n", sigma);

/*
Set up input pixel value array
-------------------------------
*/
in[0] = lutmin;
in[8] = lutmax;
for (i = -3; i <= 3; i++)
   in[i+4] = (long) (mean + (float) i * sigma);

/*
Create the partial look-up table.
---------------------------------
*/
pix = 0;
for (i = 0; i < 9; i++)
   {
   if ((in[i] <= lutmax) && (in[i] >= lutmin) && (out[i] <= lutmax) && 
       (out[i] >= lutmin))
      {
      plut[pix][0] = in[i];
      plut[pix][1] = out[i];
      pix++;
      }
   else
      {
      if (i == 0) 
         {
         plut[pix][0] = lutmin;
         plut[pix][1] = lutmin;
         pix++;
         }
      else if (i == 8) 
         {
         plut[pix][0] = lutmax;
         plut[pix][1] = lutmax;
         pix++;
         }  
      } 
   }

/*
Construct the look-up table by linear interpolation between entries 
in partial look-up table.
---------------------------------------------------------------------
*/
curr = 0;
for (i = lutmin; i <= lutmax; i++)
   {
   if (plut[curr+1][0] < i ) 
      curr++;
   temp_val = (int)(((float)(i-plut[curr][0]) / (float)(plut[curr+1][0] - 
                                                        plut[curr][0])) 
             * (float) (plut[curr+1][1] - plut[curr][1]) + 
               (float) plut[curr][1] + 0.5);
   lut[i] = (short) (min (max (temp_val, lutmin), lutmax));
   }

}
