
#include "stretch.h"
#include "pixman.h"
#include "util.h"
#include "protos.h"

void stmakelut
(
    long *totbnd, 		/* Total number of bands */
    long *nimages, 		/* Number of images */
    char hosnam[][CMLEN], 	/* Input image host name */
    char hosout[][CMLEN],		/* Output image host name */
    long *img_type, 		/* Image and/or wLUT data type */
    long *otype,			/* Output image data type */
    char source[],		/* Input Display and Look-Up Table File */
    char dltfentry[],		/* Entry in input DPF */
    long window[][4] 		/* Processing window */
)
/*
----------------------------------------------------------------------------
NAME: STMAKELUT

FUNCTION:   Stmakelut produces a look up table of appropriate data type 
            (same data type as input image), according to user specifications.

PROGRAM HISTORY:
      Version          date                Author	   Request
      -------          ----                ------	   -------
        1      	     28-Jan-82   	  J. Hagedorn       
	2     	     14-Jul-87   	  B. Rieger         
        3	     24-May-90	 	  B. Allam
        4            12-Dec-90            W. Xia (stx)   modification 
        5            22-Mar-91            K. Donaldson   added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Get the input image name.
   If no input images were specified, 
      Set the image counter to 0, 
      Set the total number of bands to 1, 
      Set the look-up table type to BYTE.
      Initialize window specification to 0's.
   End-If
   Else If an input image was specified, 
      Get the image type.
      If input image type is not BYTE or I*2, 
         Print a fatal error message.
   End-If
   Get the output image name.
   If no output image was specified, 
      Set the output image count to 0 
      Set the output data type to BYTE.
   End-If
   Else, If an output image was specified, 
      Set the output image count to 1 
      Update the output file list.
   End-If
   Get the user specified-output data type.
   If the user specified data type is "SAME", 
      Get the data type of the input image.
   Else
      Set the output type to the user-specified type.
   Get the DLTF name.
   If some of the parametes are misssing, 
      Print a fatal error message.
   Get the DLTF entry name.
   Get the subcommand name.
   If subcommand is "THRESHOLD BY PERCENT", 
      Verify that a single-band, input image was specified.
      Call STPTHRESH to generate threshold lut by percent of pixels.
   End-If
   If subcommand is "THRESHOLD BY VALUE", 
      Call STVTHRESH to generate threshold lut by values.
   If subcommad is "LINEAR", 
      Call STLINEAR to generate linear stretch lut.
   If subcommand is "CONTOUR",
      Call STCONTOUR to generate contour lut.
   If subcommand is "CLIP", 
      Call STCLIP to generate bit clipping lut.
   If subcommand is "PIECE-WISE LINEAR", 
      Call STPWLIN to generate piece-wise linear lut.
   If subcommand is "CUBE ROOT", 
      Call STCUBEROOT to generate cube-root stretch lut.
   If subcommand is "POWER LAW", 
      Call STPOWER to generate power law stretch lut.
   If subcommand is "ADAPTIVE POWER LAW", 
      Call STADAPT to generate adaptive power law lut.
   If subcommand is "READ A LUT FROM DISK", 
      Call STDISKLUT to read lut from disk.
   If subcommand is "DISTRIBUTION STRETCH", 
      Verify that the input image is a single band image 
      Call STDIST to construct distribution lut.
   End-If
   Get the look-up table modification type.
   If the look-up table modification type is "FLIP", 
      Call STFLIP to reverse the order of lut entries.
   If the look-up table modification type is "COMPLEMENT", 
      Complement lut entries converting word-to-byte if necessary.

-----------------------------------------------------------------------------
*/
{

long    zero = 0;
long    one = 1;

unsigned char	blut[256];	/* Byte LUT */

char	imtype[5], 		/* Image modification type */
	od_type[5],		/* Output data type specification buffer */
	subcmd[4],		/* Subcommand name/lut generation type */	
	lasnam[MAXIMG][CMLEN], 	/* Input image las name */
	lasout[1][CMLEN];		/* Output image las name */

long	
	flag[MAXIMG][2],	/* Window or band option flag */
        ftyp, 			/* "FROM" type for conversion */
	i,			/* Loop counter and array index */
	in_cnt, 		/* Input image count */
        imglimit = MAXIMG,      /* Maximum number of images */
	itst, 
	jtst,
	out_cnt, 		/* Output image count */
	len = 5,		/* String length for odtype parameter */
	lenth, 
	out_type =0 ,
        same_window,		/* Flag for same window size */       
	size, 			/* Look-up table size */
	status, 		/* Return status */
	totyp;			/* "TO" type for conversion */

struct VARIABLE *var; /* Function/variable for parameter retrieval */

/*       
Get the input image name.
-------------------------
*/
status = c_getin (&block, "IN", &zero, lasnam, hosnam, nimages, window, 
                  bands, nbands, totbnd, &imglimit, &same_window, flag);
if (status != E_SUCC)
   msg_err("!Error accessing the input file name", "stretch-infile");

/*
If no input images were specified, set the image counter to 0, the
total number of bands to 1, and the look-up table type to BYTE.
Initialize window specification to 0's.
-------------------------------------------------------------------
*/
if (*nimages == 0) 
   {
   in_cnt = 0;
   *totbnd = 1;
   strcpy (hosnam[0], "");
   *img_type = EBYTE;

   for (i = 0; i < 4; i++) 
       window[0][i] = 0;
   }

/*
If an input image was specified, get the image type.
If input image type is not BYTE or I*2, print a fatal error message.
--------------------------------------------------------------------
*/
else
   {
   in_cnt = 1;
   c_gettyp( hosnam, nimages, img_type);
   if ((*img_type != EBYTE) && (*img_type != EWORD))
     msg_err ("!Invalid input image data type", "stretch-imgtype");
   }

/*
Get the output image name.
---------------------------
*/
status = c_getout (&block, "OUT", lasout, hosout);
if (status != E_SUCC)
   msg_err("!Error accessing the output file name", "stretch-output");

/*
If no output image was specified, set the output image count to 0 and
the output data type to BYTE.
----------------------------------------------------------------------
*/
if (strlen(hosout[0]) == 0)                         
   {                                               
   out_cnt = 0;
   *otype = EBYTE;                                                            
   strcpy(hosout[0], "");                           
   }

/*
If an output image was specified, set the output image count to 1 and 
update the output file list.
Get the user specified-output data type.
If the user specified data type is "SAME", get the data type of the 
input image.
Otherwise, set the output type to the user-specified type.
------------------------------------------------------------------------
*/
else 
   {
   out_cnt = 1;
   strcpy(out[count++], lasout[0]);
   c_getstr (&block, "ODTYPE", &one, &len, od_type);
   if (strcmp(od_type,"SAME") == 0)        
      {            
      c_getddr(hosnam[0], &ddr);
      out_type = ddr.dtype; 
      }   
   else if (strcmp(od_type,"BYTE") == 0)   
      out_type = EBYTE;
   else if (strcmp(od_type,"I*2") == 0) 
      out_type = EWORD;            
   *otype = out_type;
   }
   
/*
Get the DLTF name.
--------------------
*/
var = p_find (&block, "DLTFOUT");
if ((*var).v_count == 0)
  strcpy (source, "");
else 
  strcpy (source, SVAL(*var, 0));
                        
lenth = (*var).v_count;

/*
If some of the parametes are misssing, print a fatal error message.
--------------------------------------------------------------------
*/
itst = out_cnt + in_cnt + lenth;
jtst = out_cnt + in_cnt;
if ((itst == 0) || (jtst == 1))
  msg_err ("!Missing parameters - specify IN and OUT or DLTFOUT",
		 "stretch-parm");

/*
Get the DLTF entry name.
------------------------
*/
if (strlen (source) != 0)
   {
   var = p_find (&block, "DLTFENTR");       
   if ((*var).v_count == 0)
      msg_err ("!No DLTF entry specified", "stretch-dltfent");
   else
      strcpy (dltfentry, SVAL(*var, 0));
   }

/*
Get the subcommand name.
------------------------
*/
var = p_find (&block, "_SUBCMD");
strcpy (subcmd, SVAL(*var, 0));

/*
If subcommand is "THRESHOLD BY PERCENT", verify that a single-band, input 
image was specified.
Call STPTHRESH to generate threshold lut by percent of pixels.
--------------------------------------------------------------------------
*/
if (!(strcmp(subcmd, "PTH")))
   {
   if (!in_cnt)
      msg_err ("!Input image must be specified for this option", "stretch-in");
   if (*totbnd > 1) 
      msg_err("!Input image must be a single band image for this option", 
              "stretch-singbnd");
   stpthresh (*img_type, hosnam, hosout[0], *nimages, window, *totbnd);
   }

/*
If subcommand is "THRESHOLD BY VALUE", call STVTHRESH to generate threshold
lut by values.
----------------------------------------------------------------------------
*/
else if (!(strcmp(subcmd, "THR")))
   stvthresh (*img_type);

/*
If subcommad is "LINEAR", call STLINEAR to generate linear stretch lut.
------------------------------------------------------------------------
*/
else if (!(strcmp(subcmd, "LIN")))
   stlinear (*img_type);

/*
If subcommand is "CONTOUR", call STCONTOUR to generate contour lut.
-------------------------------------------------------------------
*/
else if (!(strcmp(subcmd, "CON")))
   stcontour (*img_type);

/*
If subcommand is "CLIP", call STCLIP to generate bit clipping lut.
------------------------------------------------------------------
*/
else if (!(strcmp(subcmd, "CLI")))
   stclip (*img_type);

/*
If subcommand is "PIECE-WISE LINEAR", call STPWLIN to generate piece-wise 
linear lut.
--------------------------------------------------------------------------
*/
else if (!(strcmp(subcmd, "PWL")))
   stpwlin (*img_type, *otype);

/*
If subcommand is "CUBE ROOT", call STCUBEROOT to generate cube-root
stretch lut.
--------------------------------------------------------------------
*/
else if (!(strcmp(subcmd, "CUB")))
   stcuberoot (*img_type);

/*
If subcommand is "POWER LAW", call STPOWER to generate power law stretch lut.
------------------------------------------------------------------------------
*/
else if (!(strcmp(subcmd, "POW")))
   stpower (*img_type);

/*
If subcommand is "ADAPTIVE POWER LAW", call STADAPT to generate adaptive
power law lut.
-------------------------------------------------------------------------
*/
else if (!(strcmp(subcmd, "ADA")))
   stadapt (*img_type);

/*
If subcommand is "READ A LUT FROM DISK", call STDISKLUT to read lut from disk.
------------------------------------------------------------------------------
*/
else if (!(strcmp(subcmd, "DIS")))
   stdisklut (*img_type);

/*
If subcommand is "DISTRIBUTION STRETCH", verify that the input image is a 
single band image and call STDIST to construct distribution lut.
--------------------------------------------------------------------------
*/
else
   {
   if (*totbnd > 1) 
      msg_err("!Input must be single band for this stretch function", 
              "stretch-singbnd");
   stdist (*img_type, hosnam, window);
   }  

/*
Get the look-up table modification type.
----------------------------------------
*/
var = p_find (&block, "LUTOPT");
strcpy (imtype, SVAL(*var, 0));

/*
If the look-up table modification type is "FLIP", call STFLIP to reverse 
the order of lut entries.
-------------------------------------------------------------------------
*/
if (!(strcmp(imtype, "FLIP")))
   stflip (*img_type);

/*
If the look-up table modification type is "COMPLEMENT", complement lut entries
converting word-to-byte if necessary.
------------------------------------------------------------------------------
*/
else if (!(strcmp(imtype, "COMP")))
   {
   if (*img_type == 1)
      {
      size = 256;
      ftyp = EWORD; 
      totyp = EBYTE;
      c_pxconv (&ftyp, &totyp, &lut[0], blut, &size);
      c_pxcomp (blut, blut, &size, &totyp); 
      c_pxconv (&totyp, &ftyp, blut, &lut[0], &size);
      }
   else
      {
      size = 131072;
      c_pxcomp (lut, lut, &size, img_type); 
      } 
   }  
}
