#include "stretch.h"
#include "util.h"
#include "protos.h"

void stpthresh
(
    long lut_type,		/* Data type of look up table */
    char hosnam[][CMLEN],	/* Input image host name */
    char hosout[],		/* Output image host name */
    long nimages,		/* Number of images */
    long wind[][4],		/* Processing window */
    long totbnd 		/* Total number of bands */
)
{
/*
-----------------------------------------------------------------------------
NAME: STPTHRESH

FUNCTION:   Makes a threshold look up table by percent of pixels.  This is a
            table where entries are either 0 or 1.  Below a certain position
            all the entries are zero, above that position all are 1.  This 
            position is chosen so that the percent of pixels of the input
            image the values of which are equal to or greater than this 
            position is equal of greater than a specified percentage.

PROGRAM HISTORY:
      Version          Date            Author		Request
      -------          ----            ------		-------
        1      	     28-Jan-82        J. Hagedorn       
	2     	     14-Jul-87        B. Rieger
        3	      7-Jul-89	      B. Allam		conversion
        4            22-Mar-91        K. Donaldson      added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:
 
   Allocate dynamic memory for the histogram.
   Get the percentage of pixels in threshold range.
   Get the processing message interval.
   Call STSETIO to open the input image.
   Initialize the histogram array.
   Initialize and enable the processing message utility.
   For all lines in the input imge, 
      Histogram each line.
   Close the input image file and free the I/O buffer.
   If the look-up table type is BYTE, 
      Set the value limits to 0 and 255.
      Set the lower limit to 256 and the upper limit to 255.
   End-If
   If the look-up table type is I*2, 
      Set the value limits to -32768 and 32767.
      Set the lower limit to 32768 and the upper limit to 32767.
   End-If
   Allocate dynamic memory for the look-up table.
   Count pixels in the histogram until the true number of pixels in threshold 
   range is equal to or greater than the specified number of pixels in 
   threshold range.
   Call STTHRESH to generate the threshold look-up table.
   Free the histogram buffer.

-----------------------------------------------------------------------------
*/

long    one = 1;

static long hmax =  32767;	/* Maximum bin value in the histogram */
static long hmin = -32768;	/* Minimum bin value in the histogram */

float	percent;		/* Percentage of pixels */

long	hist_size,		/* Histogram size */
	i, 			/* Loop counter and array index */
	ind, 			/* Index into the histogram */
	l_size, 		/* LUT size */
	limlo, 			/* Lower LUT limit */
	limup, 			/* Upper LUT limit */
	lutmax,			/* Maximum LUT entry value */
	lutmin, 		/* Minimum LUT entry value */
	mid, 			/* Middle point of the histogram */
	*thisto, 		/* Pointer to histogram array */
	*mhist, 		/* Pointer to the midle of histogram */
	msg_flag,		/* Processing message flag */
	msgtim,			/* Processing message interval */ 
	nl, 			/* Number of lines in the input image */
	np, 			/* Number of pixels in the input image line */
	npixinthr,		/* Number of pixels in threshold */
	ntruepix,		/* True number of pixels in threshold */
	out_img_band[1][MAXBND+1],	/* Band numbers of the output image  */
	pix, 			/* Current pixel */
	size, 			/* Size of the histogram/table array */
	tot_pix;		/* Total number of pixels in the input image */

struct VARIABLE *var; /* Function/variable for parameter retrieval */

/*
Allocate dynamic memory for the histogram.
-------------------------------------------
*/
size = hmax - hmin + 1;
hist_size = (long) (size * sizeof(long));
if ((thisto = (long *) malloc (hist_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for histogram", 
            "stretch-hisalloc");
mid = size/2;
mhist = &thisto[mid-1];

/*
Get the percentage of pixels in threshold range.
Get the processing message interval.
--------------------------------------------------
*/
var = p_find (&block, "THRPERC");
percent = RVAL (*var, 0);
var = p_find(&block, "$MSGTIM"); 
msgtim = IVAL(*var,0);

/*
Call STSETIO to open the input image.
----------------------------------------
*/
stsetio (hosnam, hosout, wind, &lut_type, nimages, totbnd, out_img_band);
nl = wind[0][NL];
np = wind[0][NS];
tot_pix = nl * np;
npixinthr = (long) (0.5 + (float) (tot_pix) * percent * 0.01);

/*
Initialize the histogram array.
--------------------------------
*/
printf("Creating Look up table\n");
size = hmin + 1;
for (i = size; i < hmax; i++)
   mhist[i] = 0;

/*
Initialize and enable the processing message utility.
------------------------------------------------------
*/
curcat = 1;
msg_flag = ONECAT;
c_inittm ("STRETCH", "Read", "line", "band", "", &nl, &one, &msgtim, 
           &msg_flag);
curent = 1;
c_startm(); 

/*
For all lines in the input imge, histogram each line.
------------------------------------------------------
*/
for (curent = 1; curent <= nl; curent++)
   {
   c_estep (&gdescin);
   for (pix = 0; pix < np; pix++)
      {
      ind = ibuf[pix+ipoint[0]];
      mhist[ind] = mhist[ind] + 1;
      }
   }

/*
Close the input image file and free the I/O buffer.
--------------------------------------------------
*/
c_stoptm();
c_egclse (&gdescin);
free (ibuf);

/*
If the look-up table type is BYTE, set the value limits to 0 and 255.
Set the lower limit to 256 and the upper limit to 255.
----------------------------------------------------------------------
*/
if (lut_type == 1) 
   {
   lutmin = 0;
   lutmax = 255;
   limlo = 256;
   limup = 255;
   }
/*
If the look-up table type is I*2, set the value limits to -32768 and 32767.
Set the lower limit to 32768 and the upper limit to 32767.
---------------------------------------------------------------------------
*/
else
   {
   lutmax = 32767;
   lutmin = -32768;
   limlo = 32768;
   limup = 32767;
   } 

/*
Allocate dynamic memory for the look-up table.
-----------------------------------------------
*/
l_size = (long) ((lutmax - lutmin + 1) * sizeof(short));
if ((templut = (short *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table", 
            "stretch-lutalloc");
if (lut_type == 1)
  lut = &templut[0];
else
  lut = &templut[(lutmax - lutmin + 1) / 2];
ntruepix = 0;

/*
Count pixels in the histogram until the true number of pixels in threshold 
range is equal to or greater than the specified number of pixels in 
threshold range.
---------------------------------------------------------------------
*/
while ((ntruepix < npixinthr) && (limlo > lutmin))
  {
  limlo = limlo - 1;
  ntruepix = ntruepix + mhist[limlo];
  }

/*
Call STTHRESH to generate the threshold look-up table.
-------------------------------------------------------
*/
stthresh (limup, limlo, lutmax, lutmin);

/*
Free the histogram buffer.
--------------------------
*/
free (thisto);

}
