#include "stretch.h"
#include "protos.h"

void stpwlin
(
    long lut_type, 		/* LUT data type */
    long otype			/* Output image data type */
)
{
/*
-----------------------------------------------------------------------------
NAME: STPWLIN

FUNCTION:   Produces a piece-wise linear look up table by completing 
            a partial look up table provided by the user.  The gaps 
            in the partial look up table are filled by linear interpolation.
            The partial look up table is specified by the user through
            the tae parameters FROM and TO which contain the "from" and "to"
            coordinates respectively.  

PROGRAM HISTORY:
      Version          Date           Author		Request
      -------          ----           ------		-------
        1      	     28-Jan-82	    J. Hagedorn       
	2     	     24-Dec-86      R. Mckinney	        prf 3034         
        3	     10-Jul-89	    B. Allam     	conversion
        4            22-Mar-91      K. Donaldson        added comments; 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Initialize FROM and TO arrays.
   Get the FROM and TO mapping points.
   Check mapping pairs for errors.
   Compute the number of FROM and TO pairs.
   If the number of pairs exceeds 50, 
      Copy pairs from FROM2 and TO2 arrays
   If the number of pairs is less than 1, 
      Print a fatal error message.
   Move the mapping pairs into the processing array.
   Verify that the pairs are in correct order.
   If the look-up table type is BYTE, 
      Set the value limits to 0 and 255.
   If the look-up table type is I*2, 
      Set the value limits to -32768 and 32767.
   Allocate dynamic memory for lut.
   Make sure that the partial lut extends to beginning of range but 
   not beyond it.
   Make sure that the partial lut extends to end of range but not beyond it.
   Create the look-up table by linear interpolation between entries in 
   the partial lut.

----------------------------------------------------------------------------
*/
char	msg[ERRLEN];		/* Error message buffer */

long	curr,			/* Current position LUT index */
	from[100], 		/* Array of FROM values for partial lut */
	from2[50], 		/* Additional FROM values for partial lut */
	i, j,			/* Loop counters and array indices */
	l_size, 		/* LUT size */
	lutmax,			/* Maximum LUT range value */ 
	lutmin,			/* Minimum LUT range value */
	npairs,			/* Number of FROM/TO pairs */
	partlut[100][2], 	/* Partial lut processing array */
	temp_val,		/* Intermediate interpolation value */
	to[100], 		/* Array of TO values for partial lut */
	to2[50];		/* Additional TO values for partial lut */

struct VARIABLE *f1, *f2, *t1, *t2; /* Function/variables for  */
					       /* parameter retrieval     */

/*
Initialize FROM and TO arrays.
------------------------------
*/
for (i = 0; i < 100; i++)
   {
   from[i] = 0;
   to[i] = 0;
   }
for (i = 0; i < 50; i++)
   {
   from2[i] = 0;
   to2[i] = 0;
   }

/*
Get the FROM and TO mapping points.
------------------------------------
*/
f1 = p_find (&block, "FROM1");
for (i = 0; i < 50; i++)
  from[i] = IVAL(*f1, i);
f2 = p_find (&block, "FROM2");
for (i = 0; i < 50; i++)
  from2[i] = IVAL(*f2, i);
t1 = p_find (&block, "TO1");
for (i = 0; i < 50; i++)
  to[i] = IVAL(*t1, i);
t2 = p_find (&block, "TO2");
for (i = 0; i < 50; i++)
  to2[i] = IVAL(*t2, i);

/*
Check mapping pairs for errors.
---------------------------------
*/
if (((*f1).v_count == 1) || ((*t1).v_count == 1)) 
   msg_err("!FROM1 and TO1 must have at least two values", "stretch-two");
if (((*f1).v_count < 50) && ((*f2).v_count > 0)) 
   msg_err("!FROM1 must have 50 values before using FROM2", "stretch-from");
if (((*t1).v_count < 50) && ((*t2).v_count > 0)) 
   msg_err("!TO1 must have 50 values before using TO2", "stretch-to");
if (((*f1).v_count+(*f2).v_count) != ((*t1).v_count+(*t2).v_count)) 
   msg_err("!The number of FROM values must equal the number of TO values",
           "stretch-equal");

/*
Compute the number of FROM and TO pairs.
If the number of pairs exceeds 50, copy pairs from FROM2 and TO2 arrays
------------------------------------------------------------------------
*/
npairs = (*f1).v_count + (*f2).v_count;
if (npairs > 50) 
   {
   for (i = 50; i < npairs; i++)
      {
      from[i] = from2[i-50];
      to[i] = to2[i-50];
      }
   } 

/*
If the number of pairs is less than 1, print a fatal error message.
-------------------------------------------------------------------
*/
if (npairs <= 1)  
   msg_err ("!Too few pairs were specified - at least two must be specified", 
            "stretch-pair");

/*
Move the mapping pairs into the processing array.
--------------------------------------------------
*/
j = 0;
for (i = 0; i < npairs; i++)
   {
   partlut[j][0] = from[i];
   partlut[j][1] = to[i];
   j++;
   }

/*
Verify that the pairs are in correct order.
--------------------------------------------
*/
for (i = 1; i < npairs; i++)
   {
   if (partlut[i][0] <= partlut[i-1][0])
      {
      sprintf (msg, "?Partial LUT pairs specified in wrong order - ");
      msg_err (msg, "stretch-order");
      sprintf (msg, "!they must be entered in order of ascending LUT position");
      msg_err (msg, "stretch-order");
      }  
   }

/*
If the look-up table type is BYTE, set the value limits to 0 and 255.
If the look-up table type is I*2, set the value limits to -32768 and 32767.
----------------------------------------------------------------------------
*/
if ((lut_type == 1) && (otype == 1)) 
   {
   lutmax = 255;
   lutmin = 0;
   }
else
   {
   lutmax =  32767;
   lutmin = -32768;      
   }  

/*
Allocate dynamic memory for lut.
--------------------------------
*/
l_size = (long) ((lutmax-lutmin+1) * sizeof(short));
if ((templut = (short *) malloc (l_size)) == NULL)
   msg_err ("!Cannot allocate dynamic memory for the look-up table", 
              "stretch-lutalloc");
if (lut_type == 1)
   lut = &templut[0];
else
   lut = &templut[(lutmax-lutmin+1)/2];

/*
Make sure that the partial lut extends to beginning of range but not beyond it.
------------------------------------------------------------------------------
*/
if (partlut[0][0] < lutmin) 
   msg_err ("!First LUT position specified in partial LUT is too small",
            "stretch-small");
else if (partlut[0][0] > lutmin) 
   {
   for (i = npairs-1; i >= 0; i--)
      {
      for (j = 0; j < 2; j++)
         partlut[i+1][j] = partlut[i][j];
      }
   partlut[0][0] = lutmin;
   partlut[0][1] = partlut[1][1];
   npairs = npairs + 1;
   }  

/*
Make sure that the partial lut extends to end of range but not beyond it.
--------------------------------------------------------------------------
*/
if (partlut[npairs-1][0] > lutmax) 
   msg_err ("!Final LUT position specified in partial LUT is too large",
            "stretch-big");
else if (partlut[npairs-1][0] < lutmax) 
   {
   npairs = npairs + 1;
   partlut[npairs-1][0] = lutmax;
   partlut[npairs-1][1] = partlut[npairs-2][1];
   } 

/*
Create the look-up table by linear interpolation between entries in 
the partial lut.
---------------------------------------------------------------------
*/
curr = 1;   
for (i = lutmin;i <= lutmax; i++)
   {
   if (partlut[curr][0] < i)    
      curr = curr + 1;

   temp_val = (int)(((float)(i-partlut[curr-1][0]) / 
                (float)(partlut[curr][0] - partlut[curr-1][0])) *
               (float) (partlut[curr][1] - partlut[curr-1][1]) +
               (float) partlut[curr-1][1] + 0.5);
   lut[i] = (short) min(max(temp_val, lutmin), lutmax);
   }

}
