/******************************************************************************
NAME: TIFF2BIL

PURPOSE:
	To convert a TIFF(strip/tile) file to a flat file
	suitable for ingestion through TRANSLAS-IN.  

PROGRAM HISTORY:

Version	Date	Programmer		 Code	Reason
------- ------  ---------------  -----  ---------------------------------------
1.0     07/95   K.Lowell            C   Placed under CVS control
1.1     08/95   J.Fenno             C   Updated error messages
1.2     07/97   R.Dilley            C   Added capability to handle tiled images
1.3     05/98   B.Olson             C   Added 16 and 32 bit image capability
1.4     06/98   B.Olson             C   Added capability to handle images
                                        stored in BIP format
1.5     08/99   D.Hames             C   Modified TIFFTAG_SAMPLESPERPIXEL tag
                                        because the output value of nbands was 
                                        incorrect.	
1.6     08/99   D.Hames             C   Add prototypes
1.7     05/00   A.Shimitz           C   Added capability to handle vertically
                                        inverted images
1.8     06/00   A.Shimitz           C   Added capability to create an RGB color
                                        image from a 1 band TIFF with an
                                        associated color palette

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under UNIX and TAE

PROJECT: LAS

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "tiffio.h"
#include "las.h"
#include "util.h"
#include "protos.h"

/* Macro used to quickly compute the number of an (x,y) tile
   given the number of tiles across (a).
------------------------------------------------------------*/
#define GetTileNum(x,y,a) (long)((long)(x)+(long)(y)*(long)(a))

struct PARBLK vblk;     /* TAE parameter block                              */

int main
(void)
{
   TIFF *in;        /* Input file pointer                                   */
   FILE *out;       /* Output file pointer                                  */
   char infile[CMLEN];  /* Input TIFF file                                  */
   char outfile[CMLEN]; /* Output BIL file                                  */
   char tmpstr[5];  /* Temporary string variable                            */
   char *odtype[1];     /* Array of pointers for q_string                   */
   int cmap;     /* The depth of the colormap                          */
   long fatal = -1; /* Fatal error flag                                     */
   long stdlun;     /* Logical unit number for std output                   */
   long tmp_long;   /* Temporary long                                       */
   long tmpnum;     /* Temporary number                                     */
   long line;       /* Line counter                                         */
   long result;         /* Return result from TIFFReadEncodedTile(...)      */
   long i,j;          /* Used for loops                                     */
   long num_lines;   /* The number of lines in the current tile row         */
   long zero = 0;       /* Used for the exit function                       */
   short int orientation = 0;  /* The orientation of the TIFF image         */
   short int step;  /* The increment when writing output - either 1 or -1   */
   short int type;  /*  To check the type of image                          */
   tsample_t config;    /* The planar configuration in the image (bip, bil) */ 
   tsize_t tile_rowsize; /* The number of bytes/samples per row in a tile   */
   tsize_t tile_size;   /* The number of bytes in a tile                    */
   tsize_t tile_length; /* The number of rowsin a tile                      */
   tsize_t image_width; /* The number of bytes/samples across the image     */
   tsize_t image_length; /* The number of lines down in the image           */
   tsize_t start;  /* The first row to write to the output file             */
   tsize_t tiles_across; /* The number of tiles across for the image        */
   tsize_t tiles_down;  /* The number of tiles down for the image           */
   tsize_t last_tile_width_used; /* The number of samples in the last tile  */
   tsize_t last_tile_length_used; /* The number of lines in the last tile   */
   uint16 bitspersample; /* The number of bits per sample                   */
   uint16 band;		    /* Band counter                                     */
   uint16 nbands;       /* The number of bands in the image                 */
   uint16 *rmap=NULL; /* Buffer for red palette values                      */
   uint16 *gmap=NULL; /* Buffer for green palette values                    */
   uint16 *bmap=NULL; /* Buffer for blue palette values                     */
   unsigned char **tile_data;  /* Used for allocating the tiles across      */
   unsigned char *inbuf=NULL; /* Processing buffer for strip tiff images    */
   unsigned char *output_buf=NULL; /* Buffer for palette images             */
   unsigned char *line_buf=NULL; /* Buffer for tiled palette images      */
   unsigned char *buf_ptr=NULL; /* Pointer to output_buf                    */
   unsigned char *bip2bil_buf=NULL; /* Buffer for bip2bil to use            */

/*  Initialize TAE interface
-----------------------------*/
   c_inlas(&vblk,&stdlun);


/* Retrieve the input parameters
--------------------------------*/
   getpar(&vblk,infile,outfile);

/* Open the TIFF file 
----------------------*/
   in = TIFFOpen(infile, "r");
   if(in == NULL)
      c_lasmsg(TRUE, "tiff2bil-open", "Error opening input image");

/* Check the data type
-----------------------*/
   TIFFGetField(in,TIFFTAG_BITSPERSAMPLE,&bitspersample);
   if ((bitspersample!=8) && (bitspersample!=16) && (bitspersample!=32))
      c_errmsg("Only handle 8, 16, and 32-bit images","tiff2bil-bits",&fatal);

/* Extract the buffer length from TIFF file
-------------------------------------------*/
   TIFFGetField(in, TIFFTAG_IMAGELENGTH, &image_length);
   TIFFGetField(in, TIFFTAG_PLANARCONFIG, &config);
   TIFFGetField(in, TIFFTAG_IMAGEWIDTH,&image_width);
   TIFFGetFieldDefaulted(in, TIFFTAG_SAMPLESPERPIXEL, &nbands);

/* Check the orientation and the TIFF image type
------------------------------------------------*/
   TIFFGetField(in, TIFFTAG_ORIENTATION, &orientation);
   TIFFGetField(in, TIFFTAG_PHOTOMETRIC, &type);

/* Open the output file
-----------------------*/
   out = fopen(outfile, "wb");
   if(out == NULL)
      c_lasmsg(TRUE, "tiff2bil-open", "Error opening output image");

/* Get color map info for palette color images
----------------------------------------------*/
   if(type == PHOTOMETRIC_PALETTE)
   {
      int n;
      TIFFGetField(in, TIFFTAG_COLORMAP, &rmap, &gmap, &bmap);
/* Convert 16-bit colormap to 8-bit
-----------------------------------*/
      for(n = ((1<<bitspersample)-1); n >= 0; n--)
      {
         if((rmap[n] >= 256) || (gmap[n] >= 256) || (bmap[n] >= 256))
         {
            cmap = 16;
            break;
         }
      }

      if(cmap == 16)
      {
         for (n = ((1<<bitspersample)-1); n >=0; n--)
         {
#define CVT(x) (uint16)(((x) * 255) / ((1L<<16)-1))
            rmap[n] = CVT(rmap[n]);
            gmap[n] = CVT(gmap[n]);
            bmap[n] = CVT(bmap[n]);
         }
      }
   }

/* Check to see if the tiff image is tiles or not
-------------------------------------------------*/
   if(!TIFFIsTiled(in))
   {
      int scanlinesize;  /* value returned by TIFFScanlineSize */
      scanlinesize = TIFFScanlineSize(in);

/* Copy the strip image data to the output file
-----------------------------------------------*/		

/* Allocate the memory needed for a line
----------------------------------------*/
      inbuf = malloc(scanlinesize * sizeof(char));
      if(inbuf == NULL)
         c_lasmsg(TRUE, "tiff2bil-mem", "Error allocating memory");
      if(type == PHOTOMETRIC_PALETTE)
      {
         if(nbands > 1)
         {
            c_lasmsg(TRUE,"tiff2bil-notsup2","The image is a multiband,"
                     "palette color image which is not supported");
         }
         output_buf = malloc(scanlinesize * sizeof(char) * 3);
         if(output_buf == NULL)
            c_lasmsg(TRUE, "tiff2bil-mem", "Error allocating memory");
      }

/* Process the strip image
--------------------------*/

/* TIFF image is BIL
--------------------*/
      if (config == PLANARCONFIG_SEPARATE)
      {
/* Check orientation of image
-----------------------------*/
         if((orientation == 1)||(orientation == 0)) /* 1:default 0:undefined */
         {
            start = 0;
            step = 1;
         }
         else if(orientation == 4) /* vertically inverted */
         {
            start = image_length - 1;
            step = -1;
         }
         else /* no other orientation format is supported */
         {
            c_lasmsg(TRUE, "tiff2bil-orient",
                     "Image orientation format is not supported");
         }
/* Read each line from the TIFF file
------------------------------------*/
         for(line = start; (line >= 0) && (line < image_length); line += step)
         {
            for(band = 0; band < nbands; band++)
            {
               if(TIFFReadScanline(in, inbuf, line, band) < 0)
               {
                  c_lasmsg(TRUE, "tiff2bil-read",
                           "Error reading tiff image");
               }
/* Create a full color image from a palette color image
-------------------------------------------------------*/
               if(type == PHOTOMETRIC_PALETTE)
               {
                  getcolor(inbuf, image_width, rmap, gmap, bmap, output_buf);
                  if(fwrite(output_buf,1,(scanlinesize*3),out) !=
                     (scanlinesize*3))
                  {
                     c_lasmsg(TRUE,"tiff2bil-write",
                              "Error writing output image");
                  }
               }
/* If not a pallete color image just write the data to the output file
----------------------------------------------------------------------*/
               else
               {
                  if(fwrite(inbuf,1,scanlinesize,out) != scanlinesize)
                  {
                     c_lasmsg(TRUE, "tiff2bil-read",
                              "Error writing output image");
                  }
               }
            }
         }
      }

/* TIFF image is BIP
--------------------*/
      else if (config == PLANARCONFIG_CONTIG)
      {
         bip2bil_buf = malloc(scanlinesize * sizeof(char));
         if(bip2bil_buf == NULL)
            c_lasmsg(TRUE, "tiff2bil-mem", "Error allocating memory");
           
/* Check orientation of image
-----------------------------*/
         if((orientation == 1)||(orientation == 0)) /* 1:default 0:undefined */
         {
            start = 0;
            step = 1;
         }
         else if(orientation == 4) /* vertically inverted */
         {
            start = image_length - 1;
            step = -1;
         }
         else /* no other orientation format is supported */
         {
            c_lasmsg(TRUE, "tiff2bil-orient",
                     "Image orientation format is not supported");
         }

/* Read each line from the TIFF file and convert from bip to bil
----------------------------------------------------------------*/
         for(line = start; (line >=0) && (line < image_length); line += step)
         {
            if (TIFFReadScanline(in, inbuf, line,0) < 0)
               c_lasmsg(TRUE, "tiff2bil-read", "Error reading tiff image");
            bip2bil(inbuf, bip2bil_buf, scanlinesize, nbands);
/* Create a full color image from a palette color image
-------------------------------------------------------*/
            if(type == PHOTOMETRIC_PALETTE)
            {
               getcolor(inbuf, image_width, rmap, gmap, bmap, output_buf);
               if(fwrite(output_buf,1,(scanlinesize*3),out) != (scanlinesize*3))
               {
                  c_lasmsg(TRUE,"tiff2bil-write",
                           "Error writing output image");
               }
            }
/* If not a pallete color image just write the data to the output file
----------------------------------------------------------------------*/
            else
            {
               if (fwrite(inbuf, 1, scanlinesize,out) != scanlinesize)
               {
                  c_lasmsg(TRUE,"tiff2bil-write",
                           "Error writing output image");
               }
            }
         }
     
/* Free the memory buffer
-------------------------*/
         free(inbuf);
         free(output_buf);
         free (bip2bil_buf);
         output_buf = bip2bil_buf = NULL;
      }
   }
   else
   {
      int image_width_all_bands; /* width of all bands together */

/* Copy the single band tiled image data to the output flat file
----------------------------------------------------------------*/
/* Do some initialization
-------------------------*/
      tile_rowsize = TIFFTileRowSize(in);
      tile_size = TIFFTileSize(in);
      tile_length = tile_size / tile_rowsize;
      tiles_down = (image_length + (tile_length - 1)) / tile_length;
      last_tile_length_used = image_length % tile_length;
      image_width_all_bands = image_width * (tsize_t)nbands;
      if (config == PLANARCONFIG_CONTIG)
      {
         /* multiple bands in contigous layout, so multiply the image_width
            of all bands to get the size of each row */
         tiles_across = (image_width_all_bands + (tile_rowsize - 1)) / 
                            tile_rowsize;
         last_tile_width_used = image_width_all_bands % tile_rowsize;
  
         bip2bil_buf = malloc(sizeof(char) * image_width_all_bands);
         if(bip2bil_buf == NULL)
            c_lasmsg(TRUE, "tiff2bil-mem", "Error allocating memory");
      }
      else
      {
         if (nbands > 1)
         {
            /* The tiff spec does not give information on how to handle
               PLANARCONFIG_SEPARATE for multiple bands, so until we encounter
               an image with it, we'll just error out */
            c_lasmsg(TRUE,"tiff2bil-notsup","The image is a tiled, multiband "
                     "image which is not supported");
               
         }
         tiles_across = (image_width + (tile_rowsize - 1)) / tile_rowsize;
         last_tile_width_used = image_width % tile_rowsize;
      }

/* Allocate memory for the number of tile buffers needed
--------------------------------------------------------*/
      tile_data = malloc(sizeof(unsigned char *) * tiles_across);
      if (tile_data == NULL)
            c_lasmsg(TRUE, "tiff2bil-mem", "Error allocating memory");
      for(i = 0; i < tiles_across; i++)
      {
         tile_data[i] = (unsigned char *) malloc(sizeof(char)*tile_size);
         if(tile_data[i] == NULL)
            c_lasmsg(TRUE, "tiff2bil-mem", "Error allocating memory");
      }
      inbuf = malloc(sizeof(char) * tile_rowsize * tiles_across);
      if (inbuf == NULL)
         c_lasmsg(TRUE, "tiff2bil-mem", "Error allocating memory");

/* Allocate memory for palette image output buffers
---------------------------------------------------*/
      if(type == PHOTOMETRIC_PALETTE)
      {
         if(nbands > 1)
         {
            c_lasmsg(TRUE,"tiff2bil-notsup2","The image is a multiband,"
                     "palette color image which is not supported");
         }
         line_buf = malloc(sizeof(char) * tile_rowsize * tiles_across); 
         output_buf = malloc(sizeof(char) *tile_rowsize * tiles_across * 3);
         if(output_buf == NULL || line_buf == NULL)
            c_lasmsg(TRUE, "tiff2bil-mem", "Error allocating memory");
      }

/* Image orientation is not supported
-------------------------------------*/
      /* 1 is default and 0 means it is not defined - these are OK */
      if ((orientation != 1) && (orientation != 0))
      {
         c_lasmsg(TRUE,"tiff2bil-orient",
                  "Image orientation format is not supported");
      }
/*For each row of tiles
-----------------------*/
      for(j = 0; j < tiles_down; j++)
      {
/* Determine how many lines are in the current row of tiles
-----------------------------------------------------------*/
         if(j < (tiles_down-1))
            num_lines = tile_length;
         else 
            num_lines = last_tile_length_used;
/* Read in the tiles needed to fill a row
-----------------------------------------*/
         for(i = 0; i < tiles_across; i++)
         {
            result = TIFFReadEncodedTile(in,GetTileNum(i,j,tiles_across),
                     tile_data[i],tile_size);
            if(result == -1)
               c_lasmsg(TRUE, "tiff2bil-read", "Error reading tiff image");
         }
/* Process the lines in the current row of tiles
------------------------------------------------*/
         if (config == PLANARCONFIG_CONTIG)
         {
            for (line = 0; line < num_lines; line++)
            {
               for (i = 0; i < tiles_across; i++)
               {
                  memcpy(&inbuf[i * tile_rowsize],
                         &tile_data[i][line * tile_rowsize],tile_rowsize);
               }
               bip2bil(inbuf,bip2bil_buf,image_width_all_bands,nbands);

/* Create a full color image from a palette color image
-------------------------------------------------------*/
               if(type == PHOTOMETRIC_PALETTE)
               {
                  getcolor(inbuf, image_width_all_bands, rmap, gmap, bmap,
                           output_buf);
                  if (fwrite(output_buf,1,(image_width_all_bands*3),out) != 
                      (image_width_all_bands*3))
                  {
                     c_lasmsg(TRUE,"tiff2bil-write",
                              "Error writing output image");
                  }
               }
/* If not a pallete color image just write the data to the output file
----------------------------------------------------------------------*/
               else
               {
                  if (fwrite(inbuf,1,image_width_all_bands,out) !=
                      image_width_all_bands)
                  {
                     c_lasmsg(TRUE,"tiff2bil-write",
                              "Error writing output image");
                  }
               }
            }
         }
         else
         {
            for(line = 0; line < num_lines; line++)
            {
/* Create a full color image from a palette color image
-------------------------------------------------------*/
               if(type == PHOTOMETRIC_PALETTE)
               {
                  buf_ptr = line_buf;
/* Collect all data for the current line, color it, and write it
----------------------------------------------------------------*/
                  for(i = 0; i < tiles_across; i++)
                  {
                     memcpy(buf_ptr, &tile_data[i][line*tile_rowsize],
                            tile_rowsize);
                     buf_ptr += tile_rowsize;
                  }
                  getcolor(line_buf, image_width, rmap, gmap, bmap,output_buf);
                  if (fwrite(output_buf,1,(image_width * 3), out) !=
                      (image_width * 3))
                  {
                     c_lasmsg(TRUE,"tiff2bil-write", 
                              "Error writing output image");
                  }
               }
/* If not a palette color image just write the data to the output file
----------------------------------------------------------------------*/
               else
               {
                  for(i = 0; i < tiles_across-1; i++)
                  {
                     if(fwrite(&tile_data[i][line*tile_rowsize],1,
                        tile_rowsize,out) != tile_rowsize)
                     {
                        c_lasmsg(TRUE,"tiff2bil-write",
                                 "Error writing output image");
                     }
                  }
                  if(fwrite(&tile_data[i][line*tile_rowsize],1,
                     last_tile_width_used,out) != last_tile_width_used)
                  {
                     c_lasmsg(TRUE,"tiff2bil-write",
                              "Error writing output image");
                  }
               }
            }
         }
      }
   
/* Free the tile memory buffers
-------------------------------*/
      for(i = 0; i < tiles_across; i++)
         free(tile_data[i]);

      free(tile_data);
      free(inbuf);
      free(output_buf);
      free(line_buf);
      free(bip2bil_buf);
      tile_data = NULL;
      inbuf = output_buf = line_buf = bip2bil_buf = NULL;
   }

/* Change the number of bands to 3 if we have a palette color image
-------------------------------------------------------------------*/
   if(type == PHOTOMETRIC_PALETTE)
      nbands = 3; /* so TAE knows we now have a 3 band image */

/* Close the input and output files
-----------------------------------*/
   TIFFClose(in);
   fclose(out);

/* Output value of lines and samples to TAE
-------------------------------------------*/
   if(E_SUCC != lng2tae(&image_width,1,1,"NS",&tmp_long))
      c_lasmsg(TRUE, "tiff2bil-qout", "Error passing variables to TAE");
   if(E_SUCC != lng2tae(&image_length,1,1,"NL",&tmp_long))
      c_lasmsg(TRUE, "tiff2bil-qout", "Error passing variables to TAE");
   tmpnum = nbands;
   if(E_SUCC != lng2tae(&tmpnum,1,1,"NB",&tmp_long))
      c_lasmsg(TRUE, "tiff2bil-qout", "Error passing variables to TAE");
   odtype[0] = tmpstr;
   switch (bitspersample)
   {
     case 8  : strcpy(tmpstr,"BYTE");
               break;
     case 16 : strcpy(tmpstr,"I*2");
               break;
     case 32 : strcpy(tmpstr,"I*4");
               break;
   }
   if(q_string(&vblk,"ODTYPE",1,odtype,P_UPDATE) != SUCCESS)
      c_lasmsg(TRUE, "tiff2bil-qout", "Error passing variables to TAE");
   if(q_out(&vblk) != SUCCESS)
      c_lasmsg(TRUE, "tiff2bil-qout", "Error passing variables to TAE");


/* Output a completion message
------------------------------*/
   c_complt("TIFF2BIL", (char **)NULL, &zero);
}
