#include "uhist.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void gradient
(
    long nbands,			/* total number of bands                      */
    long num_lines,		/* number of lines in the input image         */
    long num_pixels,		/* number of pixels in the input image        */
    float scale,			/* scale factor for gradient calculations     */
    float *minpix, 		/* minimum pixel value                        */
    float *maxpix, 		/* maximum pixel value                        */
    long dtype			/* Input image data type		      */
)

/*
*****************************************************************************

NAME:	 GRADIENT	

PURPOSE:    Computes and outputs the multi-spectral gradient  image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         11-Apr-83    R.J. Tighe
   2         13-Sep-85    S. Kondal          non ap version
   3         12-Dec-89    K. Kunska          conversion to 5.0   
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   Call FMAT3D to allocate dynamic memory for the work buffer.
   Initialize and enable the processing message utility.
   Set up line pointers. 
   Store first vector line in circular buffer rbuf.
   For each input line.
      Store another line in circular buffer
      For each pixel,
         Compute the horizontal and vertical distances for each pixel and
         its neighbours. 
         Take square root of sumbuf.
         Store sumbuf in a temporary line buffer.
         Take care of the rightmost (edge) pixel.
      End-for.
      Compute output line and minmax.
      Take care of the last line.
      Take care of the corner pixel of the last line.
   End-for.
   Interchange line pointers so that l1 points to current processing line.
   Display the grand maximum pixel value (for the whole output image).
   Free the dynamic memory.

*****************************************************************************
*/
{

double	sumbuf[4],		/* sum buffer for pixel distances             */
	sumlast;		/* sum of distances for the rightmost pixel   */

float	***rbuf, 		/* circular work buffer                       */
	*tbuf,			/* temporary line buffer                      */
	outval;			/* output value                               */

float	dist,			/* horizontal and vertical pixel distance     */
	fillvalue,		/* filling value                              */
	grdmax,			/* maximum pixel value of the whole image     */
	grdmin,			/* minimum pixel value of the whole image     */
	denom,			/* denominator constant                       */
	offset;			/* offset value for gradient calculations     */

long	band,			/* current band number                        */
	buf_siz,		/* size of temporary buffer                   */
 	filltype,		/* data type of the fillvalue                 */
	i, j,			/* loop counters and array indexes            */
	intmax,			/* interger version of grand max pixel value  */
	l1, l2, 		/* line pointers for circular buffer          */
	line,			/* current line                               */
	linesave,		/* temporary storage for line index           */
	msg_flag,		/* flag for the processing message utility    */
	pix,			/* current pixel number                       */
	size = 0,		/* number of bytes per input pixel	      */
	x, y, z,		/* loop counters and indexes                  */
	xdim,			/* x dimension in a 3D work buffer            */
	ydim,			/* y dimension in a 3D work buffer            */
	zdim;			/* z dimension in a 3D work buffer            */

long    zero = 0;

rinbuf = (float *) inbuf;

/*
Initialize variables.
------------------------
*/
grdmax = 0.0;
grdmin = 255.0;
offset = 0.5;
denom = 2.0;
curent = 1;
msg_flag = 0;

switch(dtype)
   {
   case EBYTE:  size = 1; break;
   case EWORD:  size = 2; break;
   case ELONG:	size = 4; break;
   case EREAL:  size = 4; break;
   }

/*
Call FMAT3D to allocate dynamic memory for the work buffer.
-----------------------------------------------------------
*/
buf_siz = (long) (num_pixels * sizeof(float));
if ((tbuf = (float *) malloc(buf_siz)) == NULL )
   c_abort (&block, "Error allocating dynamic memory for work buffer",
            "uhist-alloc", "uhist", out_files, &num_outfiles);
xdim = 2;
ydim = num_pixels;
zdim = nbands;
if ((rbuf = fmat3d (xdim, ydim, zdim)) == NULL)
   c_abort (&block, "Error allocating dynamic memory for work buffer",
            "uhist-alloc", "uhist", out_files, &num_outfiles);
for (x = 0; x < xdim; x++)
   for (y = 0; y < ydim; y++)
      for (z = 0; z < zdim; z++)
         rbuf[x][y][z] = 0.0;

/*
Initialize and enable the processing message utility.
-----------------------------------------------------
*/
c_inittm ("UHIST", "Processed", "line", "bands", " ", &num_lines, &nbands, 
          &msgtim, &msg_flag);

/*
Set up line pointers. 
----------------------
*/
l1 = 0;
l2 = 1;

/*
Store first vector line in circular buffer rbuf.
---------------------------------------------------
*/
c_estep (&gdescin);
if (dtype == EREAL)
   {
   for (pix = 0; pix < num_pixels; pix++)
      {
      for (band = 0; band < nbands; band++)
         rbuf[l1][pix][band] = rinbuf[(ipoint[band] + pix)];
      }
   }
else
   {
   for (pix = 0; pix < num_pixels; pix++)
      {
      for (band = 0; band < nbands; band++)
        rbuf[l1][pix][band] = (float) inbuf[(ipoint[band] + pix) * size];
      }
   }

curent++;
c_startm ();

/*
Process each output line.
--------------------------
*/
for (line = 1; line <= num_lines-1; line++)
   {
   c_estep (&gdescin);
   c_estep (&gdescout);
   curent++;

/*
Store another line in circular buffer
----------------------------------------
*/
   if (dtype == EREAL)
      {
      for (pix = 0; pix < num_pixels; pix++) 
         {
         for (band = 0; band < nbands; band++)
            rbuf[l2][pix][band] = rinbuf[(ipoint[band] + pix)];
         }
      }
   else
      {
      for (pix = 0; pix < num_pixels; pix++) 
         {
         for (band = 0; band < nbands; band++)
            rbuf[l2][pix][band] = (float) inbuf[(ipoint[band] + pix) * size];
         }
      }

/*
Compute the horizontal and vertical distances for each pixel and
its neighbours. 
-----------------------------------------------------------------
*/
   fillvalue = 0.0;
   filltype = EREAL;
   c_pxfill (tbuf, &filltype, &num_pixels, &fillvalue); 
   for (pix = 0; pix < num_pixels-1; pix++)
      {
      for (i = 0; i < 4; i++)
         sumbuf[i] = 0.0;
      for (band = 0; band < nbands; band++)
         {
         dist = rbuf[l1][pix][band] - rbuf[l1][pix+1][band];
         sumbuf[0] = sumbuf[0] + dist * dist;
         dist = rbuf[l1][pix][band] - rbuf[l2][pix][band];
         sumbuf[1] = sumbuf[1] + dist * dist;
         dist = rbuf[l1][pix][band] - rbuf[l2][pix+1][band]; 
         sumbuf[2] = sumbuf[2] + (dist * dist) / denom;
         dist = rbuf[l2][pix][band] - rbuf[l1][pix+1][band]; 
         sumbuf[3] = sumbuf[3] + (dist * dist) / denom; 
         }

/*
Take square root of sumbuf.
----------------------------
*/
      for (j = 0; j < 4; j++)
         sumbuf[j] = sqrt(sumbuf[j]);

/*
Store sumbuf in a temporary line buffer.
-----------------------------------------
*/
      for (j = 0; j < 4; j++)
         {
         if (sumbuf[j] > tbuf[pix]) 
            tbuf[pix] = sumbuf[j];
         }

/*
Take care of the rightmost (edge) pixel.
------------------------------------------
*/
      if (pix == num_pixels-2) 
         {
         sumlast = 0.0;
         for (band = 0; band < nbands; band++)
            {
            dist = rbuf[l1][pix+1][band] - rbuf[l2][pix+1][band];
            sumlast = sumlast + dist * dist;
            }
         sumlast = sqrt(sumlast);
         tbuf[num_pixels-1] = sumlast;
         }  
      }

/*
Compute output line and minmax.
--------------------------------
*/
   for (pix = 0; pix < num_pixels; pix++)
      {
      outval = tbuf[pix] * scale + offset;
      outbuf[(opoint[zero] + pix)] = (unsigned char) outval;
      if (outval > grdmax) 
         grdmax = outval;
      if (outval < grdmin) 
         grdmin = outval;
      }

/*
Take care of the last line.
----------------------------
*/
   if (line == num_lines-1) 
      {
      c_estep (&gdescout); 
      for (pix = 0; pix < num_pixels-1; pix++)  
         {
         sumlast = 0.0;
         for (band = 0; band < nbands; band++)
            {
            dist = rbuf[l2][pix][band] - rbuf[l2][pix+1][band];
            sumlast = sumlast + dist * dist;
            }
         sumlast = sqrt(sumlast);
         outval = sumlast * scale + offset;
         outbuf[(opoint[zero] + pix)] = (unsigned char) outval;
         if (outval > grdmax) 
            grdmax = outval;
         if (outval < grdmin) 
            grdmin = outval;
	
/*
Take care of the corner pixel of the last line.
------------------------------------------------
*/
         if (pix == num_pixels-2) 
            {
            outval = offset;
            outbuf[(opoint[zero] + pix + 1)] = (unsigned char) outval;
            if (outval > grdmax) 
               grdmax = outval;
            if (outval < grdmin) 
               grdmin = outval;
            }  
         }
      }  

/*
Interchange line pointers so that l1 points to current processing line.
------------------------------------------------------------------------
*/
   linesave = l1;
   l1 = l2;
   l2 = linesave;
   }

c_estep (&gdescout);
c_stoptm (); 
*minpix = grdmin;
*maxpix = grdmax;

/*
Display the grand maximum pixel value (for the whole output image).
--------------------------------------------------------------------	*/
intmax = (long) *maxpix;
printf ("\nGrand maximum output pixel value = %4d\n", intmax);

/* 
Free the dynamic memory.
---------------------------			*/
freefmat3d (rbuf, xdim, ydim);
free (tbuf);

}
