/* include files, global parameters, and definitions for UHIST      */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "statio.h"
#include "protos.h"

struct	PARBLK	block;                 /* TAE parameter block     */
struct  DDR	ddr;                   /* ddr                     */

unsigned char 	*inbuf;                /* image input buffer      */
unsigned char	*outbuf;  	       /* image output buffer     */
float		*rinbuf;               /* float input pointer	*/

char    out_files[2][CMLEN];           /* output file names        */
struct FDESC *fdescin[MAXIMG+1];             /* input file descriptors   */
struct FDESC *fdescout[2];                   /* output file descriptor   */
struct GDESC *gdescin;		       /* input group descriptor   */
struct GDESC *gdescout;		       /* output group descriptor  */
long	*ipoint;
long	opoint[MAXBND];

long	min_max;                       /* minmax option flag       */
long 	msgtim;                        /* message interval         */
long	num_outfiles;                  /* number of output files   */
 					       /* statistics file          */
extern	long	curent;                        /* current entity           */
extern	long	curcat;                        /* current category         */

long	window[MAXIMG][4];    /* input image dimensions (SL SS NL NS) */

/*
*****************************************************************************

NAME:	 UHIST	

PURPOSE:    Computes the gradient of a multi-spectral image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         11-Apr-83    R.J. Tighe
   2         13-Sep-85    S. Kondal          Non ap version
   3         12-Dec-89    K. Kunska          Conversion to 5.0   
   4         04-jun-90    T. Sturman         Added statics for pn9050
   5         13-Jun-90    F. Irani           Modified to handle large samples
   6         03-Oct-90    K. Kunska          Fixed min-max
   7	     10-AUG-1988  P. Pease,  	     UHIST.PDF, & UHISTN.PDF modified so
    					     the program corrosponds to the 
					     User's Guide.  
   8	     25-AUG-1988  P. Pease           UGRADN.PDF modified to add REFGBL 
      					     $MSGTIM.
   9	      7-SEP-1988  K. Narayanan       UHIST.PDF changed to add VERS as a
					     valid value for the HISTOUT 
					     parameter.  The U.G. is also 
					     changed to add VERS option for 
					     HISTOUT.  
   10	     23-Sep-1992  D. Etrheim         Modified global include file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   Call GETPAR to initialize tae and get the user supplied parameters.
   Call FILSPEC to open the input and output files.
   Call DEFAULTS to compute the default scaling factor, if needed.
   Call GRADIENT to compute multispectral gradient image.
   Close the image files.
   If the min_max flag is set to TRUE, 
      Update the minimum and maximum pixel values in the output ddr file. 
   Update the history records.
   Print the completion message.
   
*****************************************************************************
*/

int main
(void) /* uhist */
{

static	char	hosin[MAXIMG][CMLEN],	/* input image host name            */ 
		hosout[1][CMLEN],		/* output image host name           */ 
		taein[MAXIMG][CMLEN],	/* input image tae name             */ 
		taeout[1][CMLEN];		/* output image tae name            */ 

static	double	amax[MAXBND];		/* maximum pixel value for band     */
static	double 	amin[MAXBND];		/* minimum pixel value for band     */

float	scale,				/* scaling factor for gradient      */
					/* computation                      */ 
	minpix,				/* minimum pixel value              */ 
	maxpix; 			/* maximum pixel value              */

static	long	bands_per_img[MAXIMG],	/* number of bands for each image    */
		dtype,			/* data type of the input image      */
		img_flag[MAXIMG][2],	/* flags for window/band options     */
		img_limit,		/* maximum number of images          */
		num_img_bands[MAXIMG][MAXBND+1],/* band numbers for input    */
						/* image                     */
		num_inimg,		/* number of input images            */
	 	num_lines,		/* number of lines in the image      */
		num_pixels,		/* number of pixels per line         */
		out_bands,		/* number of output image bands      */
		out_img_bands[MAXBND+1],/* band numbers for output image     */
		same_wnd,		/* same window flag                  */
		tot_inband;		/* total number of bands             */

long    one = 1;

/*
Initialize variables.
----------------------
*/
scale = 0.0;
img_limit = MAXIMG;

/*
Call GETPAR to initialize tae and get the user supplied parameters.
--------------------------------------------------------------------
*/
getpar (taein, hosin, taeout, hosout, &scale, bands_per_img, &dtype, 
        &num_inimg, num_img_bands, img_limit, img_flag, &same_wnd,
        &tot_inband);

/* allocate space */
ipoint = (long *) malloc((MAXIMG*MAXBND+1) * sizeof(long));

/*
Call FILSPEC to open the input and output files.
------------------------------------------------
*/
out_bands = 1;
filspec (hosin, hosout[0],  num_inimg, num_img_bands, 
         out_img_bands, out_bands, bands_per_img, &dtype);


/*
Call DEFAULTS to compute the default scaling factor, if needed.
---------------------------------------------------------------
*/
num_lines = window[0][NL];
num_pixels = window[0][NS];

if (scale <= 0.0) 
   defaults (hosin, tot_inband, num_inimg, num_img_bands, bands_per_img, 
            amin, amax, &scale);

/*
Call GRADIENT to compute multispectral gradient image.
-------------------------------------------------------
*/
gradient (tot_inband, num_lines, num_pixels, scale, &minpix, &maxpix, dtype);

/*
Close the image files.
------------------------
*/
c_egclse (&gdescin);
c_egclse (&gdescout);

/*
If the min_max flag is set to TRUE, update the minimum and maximum pixel
values in the output ddr file. 
------------------------------------------------------------------------
*/
if (min_max)
   {
   amax[0] = maxpix;
   amin[0] = minpix;
   c_svmnmx (hosout[0], &one, amin, amax, "YES");
   }


/*
Update the history records.
---------------------------
*/
c_hstout (&block, hosout[0], hosin, &num_inimg, "");

/*
Print the completion message.
-----------------------------
*/
c_complt ("UHIST", out_files, &num_outfiles); 

return EXIT_SUCCESS;

}
