/*****************************************************************************
NAME:                               MSS_MODEL

PURPOSE: Given the geodetic latitude, longitude and elevation of a point,
         calculate the line/sample of the pixel in the MSS image

VERSION           DATE             AUTHOR             COMMENT
-------           ----             ------             -------
  1.0             2/92            C. Wivell         Original Development
  1.1             4/92            D. Steinwand      Misc file reads/updates
  1.2		  3/93		  D. Etrheim	    Comment out print statements

NOTE: ALL ARRAYS HAVE FIRST VALUE IN INDEX 1
******************************************************************************/
#include <math.h>
#include <stdio.h>
#include "siat.h"
#include "landsat.h"
#include "constants.h"
#include "ellipsoid.h"

/* Variables common to the init, update and inverse routines */


   double strtlatd,           /* starting geodetic lat. of sat. */
          strtlong,           /* starting long of satellite */
          strtalt,            /* starting altitude above ellipsoid */
          strttime,           /* starting time from center of image */
          centlatd,           /* satellite geodetic lat at scene center */
          centlong,           /* satellite longitude at scene center */
          centalt,            /* satellite altitude at scene  center */
          centtime,           /* time of scene center t=0 */
          endlatd,            /* sat. geodetic lat. at end time */
          endlong,            /* sat. longitude at end time */
          endalt,             /* sat. alt above ellipsoid  */
          endtime,            /* time of end t = 25 */
          rcoefs[4],          /* change of radii for sat. vs time */
          rollcoefs[6],       /* change of roll for sat. vs time */
          pitchcoefs[6],      /* change of pitch for sat. vs time */
          yawcoefs[6],        /* change of yaw for sat. vs time */
	  timeupdate,         /* time update due to change in pitch */
          strtlatc,           /* geocentric lat. of sat at start */
          strtrc,             /* geocentric radii of sat at start */
          centlatc,           /* geocentric lat. of sat at center */
          centrc,             /* geocentric radii of sat at center */
          endlatc,            /* geocentric lat. of sat at end */
          endrc,              /* geocentric radii of sat at end */
          rdot1,              /* radii change vs time before center */
          rdot2,              /* radii change vs time after center */
          strtlong2,          /* sat. long in inertial space ref. center */
          centlong2,          /* sat long inertial space ref. center */
          centlong3,          /* sat long inertial space with update */
          endlong2,           /* sat long inertial space ref. center */
          endlong3,           /* sat long inertial space with update */
          strtvec[4],         /* inertial geocentric pos. vec. at start */
          centvec[4],         /* inertial geocentric pos. vec. at center */
          endvec[4],          /* inertail geocentric pos. vec. at end */
          angle1,             /* earth central angle start to center vec.*/
          angle2,             /* earth central angle center to end */
          mdot1,              /* mean motion before center */
          mdot2,              /* mean motion after center */
          vmag,               /* velocity magnitude */
          velvec[4],          /* velocity vec. of sat. */
          Time,               /* time from reference */
          dtime,              /* change in time from last guess */
          roll,               /* roll angle */
          pitch,              /* pitch angle */
          yaw,                /* yaw angle */
          tar[10],            /* target point state vector info */
          sat[10],            /* satellite state vector info */
          satint[10],         /* state vec. info. of sat. at scene center */
          tarint[10],         /* target point state vec. info at s.c. */
          nadvec[4],          /* vector from satellite to nadir point */
          attvec[10],         /* attitude coord.s vectors */
          closvec[4],         /* line of sight vec. to target point */
          dmm,                /* diff of mean motion */
          offline,            /* line off set */
          offsample;          /* sample offset */

   long i,
        j;

   char updatefi[300];		/* Host name of parameter update file */

void init_mss_model (siatfi,offline,offsample)

char *siatfi;				/* Host name of SIAT file */
double offline, offsample;
{
FILE *fptr, *fopen();
SIAT siat;				/* SIAT record 7 */

/* Read record 7 from the siat file
  --------------------------------*/
   siatread7(siatfi, &siat);

/* input required data for the satellite, read record 7 from siat file */

   strtlatd = siat.latitude[0];     /* 0.529895; */
   strtlong = siat.longitude[0];    /* -1.414564; */
   strtalt = siat.latlon_alt[0];    /* 917790.0; */
   strttime = -25.0;

   centlatd = siat.latitude[5];     /* 0.504885; */
   centlong = siat.longitude[5];    /* -1.421664; */
   centalt = siat.latlon_alt[5];    /* 917425.0; */
   centtime = 0.0;

   endlatd = siat.latitude[10];     /* 0.479857; */
   endlong = siat.longitude[10];    /* -1.428620; */
   endalt = siat.latlon_alt[10];    /* 917072.0; */
   endtime = 25.0;

/* calc. coefs for roll, pitch, and yaw */

   strcpy(updatefi,siatfi);
   strcat(updatefi,"_update");
   fptr=fopen(updatefi,"r");
   if(!fptr)
      {
      /*printf("Reading Roll, Pitch, and Yaw information from SIAT file\n");*/
      rollcoefs[1] = siat.vehicle_roll_ic;   /* -0.010986; *//* + .001539892; */
      rollcoefs[2] = siat.mean_roll_rate;    /* 0.000011; */
      rollcoefs[3] = 0.0;
      rollcoefs[4] = 0.0;
      pitchcoefs[1] = siat.vehicle_pitch_ic; /* 0.001503;*/ /* - .004024879; */
      pitchcoefs[2] = siat.mean_pitch_rate;  /* 0.0000018; */
      pitchcoefs[3] = 0.0;
      pitchcoefs[4] = 0.0;
      yawcoefs[1] = MOYAW + siat.vehicle_yaw_ic;  /*0.012627;*//* -.013669609;*/
      yawcoefs[2] = siat.mean_yaw_rate;           /* -0.000013; */
      yawcoefs[3] = 0.0;
      yawcoefs[4] = 0.0;
      timeupdate = 0.0;
      }
   else
      {
      /*printf("Reading Roll, Pitch, and Yaw information from SIAT-Update file\n");*/
      fscanf(fptr,"%le %le %le %le",
                &rollcoefs[1],&rollcoefs[2],&rollcoefs[3],&rollcoefs[4]);
      fscanf(fptr,"%le %le %le %le",
                &pitchcoefs[1],&pitchcoefs[2],&pitchcoefs[3],&pitchcoefs[4]);
      fscanf(fptr,"%le %le %le %le",
                &yawcoefs[1],&yawcoefs[2],&yawcoefs[3],&yawcoefs[4]);
      fscanf(fptr,"%le", &timeupdate);
      fclose(fptr);
      }

   /*printf("timeupdate = %lf\n", timeupdate);*/

/* change from geodetic to geocentric */

   det2centh(strtlatd,strtalt,&strtlatc,&strtrc);
   det2centh(centlatd,centalt,&centlatc,&centrc);
   det2centh(endlatd,endalt,&endlatc,&endrc);

/* find the change in the mag. of the radius vector vs time */

   rdot1 = (centrc - strtrc) / (centtime - strttime);
   rdot2 = (endrc - centrc) / (endtime - centtime);

   calcrcoefs(strtrc,centrc,endrc,strttime,centtime,endtime,rcoefs);

/* back off earth's rotation from the satellite's position */   

   strtlong2 = strtlong + WE * (strttime - timeupdate);
   centlong2 = centlong + WE * (centtime - timeupdate);
   endlong2 = endlong + WE * (endtime - timeupdate);

/* calc. inertial earth center vectors referenced at center of image time 
   for the satellite and find the angle between vectors */
 
   sph2cart(strtlatc,strtlong2,strtrc,strtvec);
   sph2cart(centlatc,centlong2,centrc,centvec);
   sph2cart(endlatc,endlong2,endrc,endvec);

   anglebetwn (strtvec,centvec,&angle1);
   anglebetwn (centvec,endvec,&angle2);

/* calc. mean motion */

   mdot1 = angle1 / (centtime - strttime);
   mdot2 = angle2 / (endtime - centtime);

/* calc. velocity vector of satellite at center of image */

   vmag = sqrt(MU / centrc);

   newvec(centvec,endvec,VELANGLE,vmag,velvec);

/* store vector values for the satellite at the center of image
   as the reference position of the satellite */

   for (i = 1 ; i <= 3 ; i++)
      {
      j = i + 4;
      satint[i] = centvec[i];
      satint[j] = velvec[i];
      }
   satint[4] = centrc;
   satint[8] = vmag;

   return;

   }


void save_updates()
{
FILE *fptr, *fopen();
/*printf("Storing updated Roll, Pitch, and Yaw information\n");*/
fptr=fopen(updatefi,"w"); 
fprintf(fptr,"%le %le %le %le\n",
       rollcoefs[1],rollcoefs[2],rollcoefs[3],rollcoefs[4]);
fprintf(fptr,"%le %le %le %le\n",
                pitchcoefs[1],pitchcoefs[2],pitchcoefs[3],pitchcoefs[4]);
fprintf(fptr,"%le %le %le %le\n",
                yawcoefs[1],yawcoefs[2],yawcoefs[3],yawcoefs[4]);
fprintf(fptr,"%le\n",timeupdate);
fclose(fptr);
}
 



void newattcoefs ( acoef,bcoef,ccoef)

   double acoef[],
          bcoef[],
          ccoef[];


   {

    double rs,
           re,
           dlinef,
           dlinec,
           dlinel,
           dpitchf,
           dpitchc,
           dpitchl,
           dpitchdot,
           dyawf,
           dyawc,
           dyawl,
           dyawdot,
           drollf,
           drollc,
           drolll,
           drolldot,
           sif,
           sic,
           sil,
           si2,
           losf,
           losc,
           losl,
           dlineff,
           dsampleff,
           dlinefn,
           dsamplefn,
           dlinefl,
           dsamplefl,
           dlinecf,
           dsamplecf,
           dlinecn,
           dsamplecn,
           dlinecl,
           dsamplecl,
           dlinelf,
           dsamplelf,
           dlineln,
           dsampleln,
           dlinell,
           dsamplell;


    re = centrc - centalt;
    rs = centrc;

    dlineff = acoef[1] * 1.0 + bcoef[1] * 1.0 + ccoef[1];
    dsampleff = acoef[2] * 1.0 + bcoef[2] * 1.0 + ccoef[2];


    dlinefn = acoef[1] * 1.0 + bcoef[1] * MIDSAMPLE  + ccoef[1];
    dsamplefn = acoef[2] * 1.0 + bcoef[2] * MIDSAMPLE + ccoef[2];


    dlinefl = acoef[1] * 1.0 + bcoef[1] * TOTSAMPS + ccoef[1];
    dsamplefl = acoef[2] * 1.0 + bcoef[2] * TOTSAMPS + ccoef[2];




    dlinecf = acoef[1] * MIDLINE + bcoef[1] * 1.0 + ccoef[1];
    dsamplecf = acoef[2] * MIDLINE + bcoef[2] * 1.0 + ccoef[2];


    dlinecn = acoef[1] * MIDLINE + bcoef[1] * MIDSAMPLE  + ccoef[1];
    dsamplecn = acoef[2] * MIDLINE + bcoef[2] * MIDSAMPLE + ccoef[2];


    dlinecl = acoef[1] * MIDLINE + bcoef[1] * TOTSAMPS + ccoef[1];
    dsamplecl = acoef[2] * MIDLINE + bcoef[2] * TOTSAMPS + ccoef[2];




    dlinelf = acoef[1] * TOTLINE + bcoef[1] * 1.0 + ccoef[1];
    dsamplelf = acoef[2] * TOTLINE + bcoef[2] * 1.0 + ccoef[2];


    dlineln = acoef[1] * TOTLINE + bcoef[1] * MIDSAMPLE + ccoef[1];
    dsampleln = acoef[2] * TOTLINE + bcoef[2] * MIDSAMPLE + ccoef[2];


    dlinell = acoef[1] * TOTLINE + bcoef[1] * TOTSAMPS + ccoef[1];
    dsamplell = acoef[2] * TOTLINE + bcoef[2] * TOTSAMPS + ccoef[2];

/* calc dpitch and dpitchdot and correct satellites position */


    sif = -dlinefn * 79.0 / re;
    sic = -dlinecn * 79.0 / re;
    sil = -dlineln * 79.0 / re;


    timeupdate += -dlinecn / LINERATE;


    losf = sqrt(re * re + rs * rs - 2 * re * rs * cos(sif));
    losc = sqrt(re * re + rs * rs - 2 * re * rs * cos(sic));
    losl = sqrt(re * re + rs * rs - 2 * re * rs * cos(sil));
 

    dpitchf = asin(sin(sif) / losf * re);
    dpitchc = asin(sin(sic) / losc * re);
    dpitchl = asin(sin(sil) / losl * re);
 
    dpitchdot = (dpitchc - dpitchl) / (SCENETIME / 2.0);
    
/* rotate the earth */

   centlong3 = centlong2 - WE * timeupdate;
   endlong3 = endlong2 - WE * timeupdate;

/* calc. inertial earth center vectors referenced at center of image time 
   for the satellite and find the angle between vectors */
 
   sph2cart(centlatc,centlong3,centrc,centvec);
   sph2cart(endlatc,endlong3,endrc,endvec);

   newvec(centvec,endvec,VELANGLE,vmag,velvec);

/* store vector values for the satellite at the center of image
   as the reference position of the satellite */

   for (i = 1 ; i <= 3 ; i++)
      {
      j = i + 4;
      satint[i] = centvec[i];
      satint[j] = velvec[i];
      }
   satint[4] = centrc;
 

    pitchcoefs[1] = pitchcoefs[1] + dpitchc;
    pitchcoefs[2] -= dpitchdot;
    pitchcoefs[3] = 0.0;
    pitchcoefs[4] = 0.0;

/* calc the dyaw and the dyawdot */


   dlinef = dlinefl - dlinefn;
   dlinec = dlinecl - dlinecn;
   dlinel = dlinell - dlineln;


   dyawf = 2.0 * asin(dlinef / 2.0 * 79.2 / 92650.0);
   dyawc = 2.0 * asin(dlinec / 2.0 * 79.2 / 92650.0);
   dyawl = 2.0 * asin(dlinel / 2.0 * 79.2 / 92650.0); 


/*
printf("dyawf = %lf\n",dyawf);
printf("dyawc = %lf\n",dyawc);
printf("dyawl = %lf\n",dyawl);

printf("dlinef = %lf\n",dlinef);
printf("dlinec = %lf\n",dlinec);
printf("dlinel = %lf\n",dlinel);
*/


   dyawdot = (dyawc - dyawf) / (SCENETIME / 2.0);


   yawcoefs[1] = yawcoefs[1] + dyawc;
   yawcoefs[2] -= dyawdot;
   yawcoefs[3] = 0.0;
   yawcoefs[4] = 0.0;

/* calc the droll and the drolldot */


    sif = -dsamplefn * 57.2 / re;
    sic = -dsamplecn * 57.2 / re;
    sil = -dsampleln * 57.2 / re;


    losf = sqrt(re * re + rs * rs - 2.0 * re * rs * cos(sif));
    losc = sqrt(re * re + rs * rs - 2.0 * re * rs * cos(sic));
    losl = sqrt(re * re + rs * rs - 2.0 * re * rs * cos(sil));
 

    drollf = asin(sin(sif) / losf * re);
    drollc = asin(sin(sic) / losc * re);
    drolll = asin(sin(sil) / losl * re);
 
    drolldot = (drollc - drolll) / (SCENETIME / 2.0);

   rollcoefs[1] = rollcoefs[1] + drollc;
   rollcoefs[2] -= drolldot;
   rollcoefs[3] = 0.0;
   rollcoefs[4] = 0.0;


/*
printf("roll, rolldot %1f\t%1f\n",rollcoefs[1],rollcoefs[2]);
printf("pitch, pitchdot %1f\t%1f\n",pitchcoefs[1],pitchcoefs[2]);
printf("yaw, yawdot %1f\t%1f\n",yawcoefs[1],yawcoefs[2]);
*/

   return;

   } 


void update (refline,refsample,dline,dsample,npoints)

double refline[],
       refsample[],
       dline[],
       dsample[];

long npoints;
   {
   double acoef[4],
          bcoef[4],
          ccoef[4];


   calcplane(refline,refsample,dline,dsample,npoints,acoef,bcoef,ccoef);

   newattcoefs( acoef,bcoef,ccoef); 

   return;
   }








void inv_mss_model( tarlatd,tarlong,tarelev,line,sample)
double tarlatd,
       tarlong,
       tarelev,
       *line,
       *sample;

   {

   long  iter;


/* ok; start the line/sample calculation */

   Time = 0.0;
   dtime = 1000.0;
   inttar(tarlatd,tarlong,tarelev,tarint);

   for (iter = 1 ; iter <= 10 ; iter++)
      {
      calcroll(Time,rollcoefs,&roll);
      calcpitch(Time,pitchcoefs,&pitch);
      calcyaw(Time,yawcoefs,&yaw);
      movetar (tarint,Time,tar);
      movesat (satint,rcoefs,mdot1,mdot2,Time,sat);
      nadirvec(sat,nadvec);
      attitude(sat,nadvec,roll,pitch,yaw,attvec);
      losvec(sat,tar,closvec);
      calcdmeanm(sat,closvec,attvec,&dmm);
      calcdtime(dmm,mdot1,mdot2,&dtime);
      if (fabs(dtime) < 0.001) break;
      Time = Time + dtime;
      }

   calcline(Time,offline,line);
   calcsample(closvec,attvec,offsample,sample);


   return;

   }

















 
