/****************************************************************************
NAME:                           get_tp

PURPOSE:  Gets a tie point coordinate from the user

ALGORITHM DESCRIPTION:
        Increment the point (prompt) counter
        Do unit statisfactory input has been given:
           Print prompt
           Get the input back from the user
           If done, return EOF
           For the appropriate units:
                Unpack the user input into Point_id, Y coord, X coord
                Check the valid range of Y coord & X coord
                If valid, break out of loop
                Prompt user to re-enter invalid input
        Return SUCCESS
*****************************************************************************/
#include "trancoord.h"
#include "protos.h"

int get_tp
(
    struct PRO *pro,     /* Input and output projection parameters */
    struct TPSDATA *data /* One Tie Point Selection data record (user input) */
)
{
    TEXT mess[80];          /* Buffer for user messages and input */
    CODE term;              /* Terminal flag for t_io */
    long x_ok, y_ok, id_ok; /* Boolean flags */
    long deg1, min1;        /* Degrees & minutes part of first DMS coordinate */
    long deg2, min2;        /* Degrees & minutes part of second DMS coordinate*/
    float sec1, sec2;       /* Seconds parts of DMS coordatates */
    static long pt_num = 0; /* Prompt number (counter) */

    /* Increment coordinate counter
       ----------------------------*/
    pt_num++;

    /* Do until satisfactory user input is given
       -----------------------------------------*/
    for (;;)
    {

        /* Prompt user
           -----------*/
    #ifdef vms
        t_write(" ",T_STDCC);
    #endif
        sprintf(mess," %d.  ",pt_num);
        t_write(mess,T_NULL);

        /* Read user input
           ---------------*/
        t_read(mess, &term);

        /* Check for end of entry
           ----------------------*/
        if (strlen(mess) == 0) return(E_EOF);

        /* For the given units, unpack the fields and check validity.  
           Break out of loop if satisfactory input is given
           --------------------------------------------------------*/
        x_ok = y_ok = id_ok = FALSE;
        if (pro->in_units == 0)  /* Radians */
        {
            sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                    &(data->coord[1])); 
            if ((data->coord[0] >= NEG_HALF_PI) && (data->coord[0] <= HALF_PI))
                y_ok = TRUE;
            if ((data->coord[1] >= NEG_PI) && (data->coord[1] <= PI)) x_ok=TRUE;
            if (x_ok && y_ok)
            {
                if (c_ckptid(data->pt_id)) id_ok = TRUE;
            }
            else id_ok = TRUE;
        }
        if (pro->in_units == 1)  /* Feet */
        {
            sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                    &(data->coord[1])); 
            if (c_ckptid(data->pt_id)) id_ok = TRUE;
            x_ok = y_ok = TRUE;
        }
        if (pro->in_units == 2)  /* Meters */
        {
            sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                    &(data->coord[1])); 
            if (c_ckptid(data->pt_id)) id_ok = TRUE;
            x_ok = y_ok = TRUE;
        }
        if (pro->in_units == 3)  /* Seconds */
        {
            sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                    &(data->coord[1])); 
            if ((data->coord[0] >= -324000.0) && (data->coord[0] <= 324000.0))
                y_ok = TRUE;
            if ((data->coord[1] >= -648000.0) && (data->coord[1] <= 648000.0))
                x_ok = TRUE;
            if (x_ok && y_ok)
            {
                if (c_ckptid(data->pt_id)) id_ok = TRUE;
            }
            else id_ok = TRUE;
        }
        if (pro->in_units == 4)  /* Degrees */
        {
            sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                    &(data->coord[1])); 
            if ((data->coord[0] >= -90.0)&&(data->coord[0] <= 90.0)) y_ok=TRUE;
            if((data->coord[1] >= -180.0)&&(data->coord[1] <= 180.0)) x_ok=TRUE;
            if (x_ok && y_ok)
            {
                if (c_ckptid(data->pt_id)) id_ok = TRUE;
            }
            else id_ok = TRUE;
        }
        if (pro->in_units == 5)  /* Packed DMS */
        {
            sscanf(mess, "%s %d %d %f %d %d %f", data->pt_id, &deg1, &min1, 
                    &sec1, &deg2, &min2, &sec2);
            x_ok = y_ok = TRUE;
            if ((deg1 > 90) || (deg1 < -90)) y_ok = FALSE;
            if ((deg2 > 180) || (deg2 < -180)) x_ok = FALSE;
            if ((min1 > 60) || (min2 > 60)) y_ok = FALSE;
            if ((sec1 > 60.0) || (sec2 > 60.0)) x_ok = FALSE;

            if (deg1 < 0.0) 
                data->coord[0] = ( (float) (deg1 * -1000000) + 
                        (float) (min1 * 1000) + sec1) * -1.0;
            else
                data->coord[0] = (float) (deg1 * 1000000) + 
                (float) (min1 * 1000) + sec1;

            if (deg2 < 0.0)
                data->coord[1] = ( (float) (deg2 * -1000000) + 
                        (float) (min2 * 1000) + sec2) * -1.0;
            else
                data->coord[1] = (float) (deg2 * 1000000) + 
                        (float) (min2 * 1000) + sec2;

            if (data->coord[0] > 90000000.00) y_ok = FALSE;
            if (data->coord[0] < -90000000.00) y_ok = FALSE;
            if (data->coord[1] > 180000000.00) x_ok = FALSE;
            if (data->coord[1] < -180000000.00) x_ok = FALSE;
            if (x_ok && y_ok)
            {
                if (c_ckptid(data->pt_id)) id_ok = TRUE;
            }
            else id_ok = TRUE;
        }
        if (x_ok && y_ok && id_ok) break;
        if (!x_ok) 
            t_write(" Illegal X/Longitude value entered; re-enter!", T_STDCC);
        if (!y_ok) 
            t_write(" Illegal Y/Latitude value entered; re-enter!", T_STDCC);
        if (!id_ok) 
            t_write(" Point id is not unique; re-enter!", T_STDCC);
    }
    data->elevation = 0.0;
    data->zoom_factor = 0.0;
    data->chip_size[0] = data->chip_size[1] = 0;
    data->offsets[0] = data->offsets[1] = 0;
    strcpy(data->chip_name,"No chip");

    return(E_SUCC);
}
