/*****************************************************************************
NAME:				GRIDSUB

PURPOSE:  Computes the differences between two geometric mapping grid files 
	  and creates a plot file of the differences.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0		 8/85   J. THORMODSGARD  TD&AB      Implement on UNIX/TAE
  5.0		12/89	D. Steinwand	   CSB      Conversion to LAS 5.0
							and C conversion
  5.1		09/90	D. Etrheim	   CSB      Release to all systems

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  GEOREG/LAS

ALGORITHM DESCRIPTION:
	Initialize LAS processing
	Get the name of the first geometric mapping grid file
	Read the first geometric mapping grid file
	Get the name of the second geometric mapping grid file
	Read the second geometric mapping grid file
	Get the name of the plot file
	Get the number of lines and samples in the input image space
	Get the print/report option
	Subtract the two mapping grids
	Call complete & exit
*****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "protos.h"

#define GABORT c_errmsg("Fatal error encountered","gridsub-fatal",&fatal)
#define GCONT c_errmsg("Nonfatal error encountered","gridsub-warning",&one)

int main
(void)
{
struct GEOGRID grid1;		/* First Geometric mapping grid */
struct GEOGRID grid2;		/* Second Geometric mapping grid */
struct VARIABLE *var;		/* pointer to a TAE variable in vblk */
struct PARBLK vblk;		/* TAE variable block */
long num_out = 1;		/* Number of output files */
long term, file, ln_pr;         /* Print option flags */
long access = IUPDATE;          /* Access code for output host file */
long nl, ns;			/* The number of lines and samples in input */
long std_out;			/* Standard output for c_inlas */
long acc = IREAD;		/* Access flag for c_getnam */
long zero = 0;			/* Constant of zero */
long one = 1;			/* Constant of one (1) */
long fatal = -1;		/* Failure condition for GABORT define */
char temp_file[CMLEN];          /* Temporary file name for output */
char filename[CMLEN];           /* Partial file name (wo/ext) for complt */
char out_file[CMLEN];           /* Name of output file for complt */
char fname[CMLEN];		/* Grid file name */
char tae_name[2][CMLEN];	/* TAE name of plot/print files */

/* Initialize tae parameter blocks, etc
  ------------------------------------*/
c_inlas(&vblk, &std_out);

/* Get the first input geometric mapping grid file name
  ----------------------------------------------------*/
if (c_getnam(&vblk,"INGRID", &zero, fname,"grid",&acc) != E_SUCC) GABORT;
if (c_getgrd(fname, &grid1) != E_SUCC) GABORT;

/* Get the input geometric mapping grid file name
  ----------------------------------------------*/
if (c_getnam(&vblk,"INGRID", &one, fname,"grid",&acc) != E_SUCC) GABORT;
if (c_getgrd(fname, &grid2) != E_SUCC) GABORT;

/* Get the output plot file name
  -----------------------------*/
acc = IWRITE;
if(c_getnam(&vblk,"OUTFILE", &zero, fname,"plot",&acc) != E_SUCC) GABORT;

/* Get the number of lines and samples in the input image space
  ------------------------------------------------------------*/
var = p_find(&vblk,"NL");
nl = IVAL(*var,0);

var = p_find(&vblk,"NS");
ns = IVAL(*var,0);

/* Get the print option.  Make a file name (if necessary) and open the
   temporary file
  --------------*/
strcpy(temp_file, "gridsub");
if(c_setprt(&vblk, &term, &file, &ln_pr, temp_file, filename) != E_SUCC)GABORT;
if (file == 1)
   {
   if (c_mknam(filename, out_file, "prt", &access) != E_SUCC) GCONT;
   strcpy(tae_name[1], out_file);
   num_out++;
   }

/* Subtract the two geometric mapping grid files
  ---------------------------------------------*/
if (sub_grid(temp_file,fname,nl,ns,&grid1,&grid2) != E_SUCC) GABORT;

/* Print completion message and exit
  ---------------------------------*/
strcpy(tae_name[0], fname);
c_complt("GRIDSUB", tae_name, &num_out);

return EXIT_SUCCESS;

}
