/****************************************************************************
NAME:				sub_grid

PURPOSE:  Computes the differences between two geometric mapping grid files
          and creates a plot file of the differences.  Note that the number
	  of rows and cols does not have to be identical in both grids as
	  it had to be in the version 1.0 of this code.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  1.0            8/85   J. THORMODSGARD  TD&AB      Implement on UNIX/TAE
  5.0           12/89   D. Steinwand       CSB      Conversion to LAS 5.0
                                                        and C conversion
  5.1		09/90	D. Etrheim	   CSB      Release to all systems

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  GEOREG/LAS

ALGORITHM DESCRIPTION:
	Check the output space of both grids--they must match; fail if not
	Open the plot file
	Open the print/report file
	Print grid size info to print/report file
	Initialize statistics variables
	Step along the denser of the two grids and compute differences
	    Compute residuals and write to the plot file
	Compute statistics
	Write statistics to print/report file
	Return
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

int sub_grid
(
    char *pfile,			/* Print file host name */
    char *fname,			/* Plot file host name */
    long nl,
    long ns,			/* Number of lines and samples in input */
    struct GEOGRID *grid1,		/* First grid file */
    struct GEOGRID *grid2		/* Second grid file */
)
{
double dline, dsamp;		/* Grid residual in line and sample */
double gline, gsamp;		/* Less dense grid coordinate (calculated) */
double rmse;			/* Root Mean Square error */
double smaxd, lmaxd;		/* Maximum sample and line residual */
double lmean, smean;		/* Mean line and sample residual */
long ndiff;			/* Number of grid points used in stats */
long i,j,k,xpt;			/* Misc counters, etc */
long pass = 1;			/* Pass-flag of c_errmsg */

FILE *plot, *filep;

/* Check to see that the size of the output space is the same for both
   geometric mapping grid files
  ----------------------------*/
if ((grid1->lines != grid2->lines) || (grid1->samples != grid2->samples))
   {
   c_errmsg("Output space does not match","gridsub-output",&pass);
   return(E_FAIL);
   }

/* Open the plot file
  ------------------*/
plot = fopen(fname, "w");
if (plot == NULL) 
   {
   c_errmsg("Error opening the plot file","gridsub-plotfile",&pass);
   return(E_FAIL);
   }

/* Open the printer file
  ---------------------*/
filep = fopen(pfile, "w");
if (filep == NULL)
   {
   c_errmsg("Error opening the print file","gridsub-printfile",&pass);
   return(E_FAIL);
   }

/* Print misc grid info to print file
  ----------------------------------*/
fprintf(filep,"Number of lines in output space:              %6d\n",
	grid1->lines);
fprintf(filep,"Number of samples in output space:            %6d\n",
	grid1->samples);
fprintf(filep,"Number of grid rows, first mapping grid:      %6d\n", 
	grid1->nrows);
fprintf(filep,"Number of grid columns, frist mapping grid:   %6d\n", 
	grid1->ncols);
fprintf(filep,"Number of grid rows, second mapping grid:     %6d\n", 
	grid2->nrows);
fprintf(filep,"Number of grid columns, second mapping grid:  %6d\n\n", 
	grid2->ncols);

/* Initialize statistics variables
  -------------------------------*/
rmse = 0.0;
smaxd = 0.0;
lmaxd = 0.0;
lmean = 0.0;
smean = 0.0;
ndiff = 0;

/* Step along the denser of the two mapping grids and compute differences
   and statistics
  --------------*/
if ((grid1->nrows * grid1->ncols) > (grid2->nrows * grid2->ncols))
   {
   for (xpt = 1, k = 0, i = 0; i < grid1->nrows; i++)
     for (j = 0; j < grid1->ncols; xpt++, j++, k++)
       {
       dline = dsamp = 0.0;
       if ((grid1->in_lines[k] > 0.0) && (grid1->in_lines[k] <= nl) &&
	   (grid1->in_samps[k] > 0.0) && (grid1->in_samps[k] <= ns))
	{
	grid_eval(grid2, (double)grid1->out_samps[j], 
                 (double)grid1->out_lines[i], &gsamp, &gline);
	dline = grid1->in_lines[k] - gline;
	dsamp = grid1->in_samps[k] - gsamp;
	rmse += (dline * dline) + (dsamp * dsamp);
	lmean += dline;
	smean += dsamp;
	ndiff++;
	if (fabs(dline) > fabs(lmaxd)) lmaxd = dline;
	if (fabs(dsamp) > fabs(smaxd)) smaxd = dsamp;
	}
       fprintf(plot,"%d %d %d %f %f\n",xpt, grid1->out_samps[j],
		grid1->out_lines[i], dsamp, dline);
       }
   }
else
   {
   for (xpt = 1, k = 0, i = 0; i < grid2->nrows; i++)
     for (j = 0; j < grid2->ncols; xpt++, j++, k++)
       {
       dline = dsamp = 0.0;
       if ((grid2->in_lines[k] > 0.0) && (grid2->in_lines[k] <= nl) &&
           (grid2->in_samps[k] > 0.0) && (grid2->in_samps[k] <= ns))
        {
        grid_eval(grid1, (double)grid2->out_samps[j],
                 (double)grid2->out_lines[i], &gsamp, &gline);
        dline = gline - grid2->in_lines[k];
        dsamp = gsamp - grid2->in_samps[k];
        rmse += (dline * dline) + (dsamp * dsamp);
        lmean += dline;
        smean += dsamp;
        ndiff++;
        if (fabs(dline) > fabs(lmaxd)) lmaxd = dline;
        if (fabs(dsamp) > fabs(smaxd)) smaxd = dsamp;
        }
       fprintf(plot,"%d %d %d %f %f\n",xpt, grid2->out_samps[j],
                grid2->out_lines[i], dsamp, dline);
       }
   }

/* Compute mean and RMSe of grid difference and print out
---------------------------------------------------------*/
if (ndiff == 0)
    {
    fprintf(filep, "None of the input space line and sample values are less ");
    fprintf(filep, "than NL and NS.\n");
    }
else
    {
    rmse = sqrt(rmse/ndiff);
    lmean = lmean / ndiff;
    smean = smean / ndiff;

    fprintf(filep,"Maximum grid difference in lines:     %8.2f\n", lmaxd);
    fprintf(filep,"Maximum grid difference in samples:   %8.2f\n\n", smaxd);

    fprintf(filep,"Mean of grid differences in lines:    %8.2f\n", lmean);
    fprintf(filep,"Mean of grid differences in samples:  %8.2f\n", smean);
    fprintf(filep,"RMS error of grid differences:        %8.2f\n", rmse);
    }

fclose(filep);
fclose(plot);

return(E_SUCC);
}
