/*******************************************************************************
NAME:                           INPEMETA

PURPOSE:      Read a fast format header file (version A), create a metadata
	      file, and create a tape label description and pass it back to
	      TAE. 

PROGRAM HISTORY:
PROGRAMMER              DATE            REASON
----------              ----            ------
D. Etrheim	        10/93		Initial development for ff123
T. Baltzer		02/95		Modified for INPE tapes
T. Baltzer		08/95		Modified to return record length

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS

Note:	The meta data stream for RMLC (ff123) is not documented except
	for in-line comments, making it a poor point of reference.  None
	the less, the following modifications to the MRLC meta data were
	made for INPE tapes:
	  1) The "geodetic process" field goes from 10 characters to 13
	     characters maximum.
	  2) The scene center latitude/longitude is not available for
	     INPE tapes and consequently not in the meta data.
	  3) Since a tape set consists of two tapes (INPE) instead of three
	     (MRLC), only two tape identifiers are given in the meta data.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "header.h"
#include "fhdr.h"

int main
(void)
{
   struct PARBLK parblk;         /* TAE parameter block */
   struct VARIABLE *val;         /* pointer to a TAE variable within
				      v-block */

   char date[6];		/* julian date */
   char hname[CMLEN];      	/* Host name of input file */
   char header[HEADER_SIZE];    /* buffer containing the header file */
   char momx[5];		/* mode/mux value */
   char newpr[10];         	/* path/row without hrz offset */
   char outstr[1000];		/* output string for metadata file */
   char outfile[CMLEN];    	/* Host name of output metadata file */
   char record_length[6];	/* Record length */
   char ppprrr[10];        	/* path/row value */
   char proj[9];                /* project for which tapes are being copied */
   char satno[3];		/* statillte number */
   char sceneid[18];		/* Scene identification */
   char tapedesc[50];		/* description for tape label */
   char tapeloc[2][7];		/* new tape locations */
   char tstr[80];          	/* temp buffer for return values */
   char tstr2[80];          	/* temp buffer for return values */
   char month[5],day[5],year[5];/* temp work variables */


   char *ptr;			/* string pointer */
   char *str_ptr[1];       	/* Pointer to a string for call to q_string */

   long fatal = -1;	/* fatal error code */
   long i;		/* loop counter */
   long ltemp;		/* temp work value */
   long len;		/* string length */
   long mode = IREAD;	/* file access */
   long mn,dy,yr,jdate; /* temp work values */
   long nonfatal = 1;	/* nonfatal error code */
   long proj_num;       /* projection number */
   long seq;		/* sequence number (1 or 2)*/
   long std_out;	/* standard out */
   long zero = 0;	/* value of zero */


   double projp[15];	/* projection parameters */

   FILE *fp;		/* metadata file pointer */

/* Initialization */
   c_inlas(&parblk,&std_out);     

/* Get the input file name */
   if (c_getnam(&parblk,"INFILE",&zero,hname,"fhdr",&mode) != E_SUCC)
     c_errmsg("Fatal error encountered","inepmeta-fatal",&fatal);

/* Open and read header file */
   mode = IREAD;
   if (c_open_header(header,hname,&mode) != E_SUCC)
     c_errmsg("Fatal error encountered","inepmeta-fatal",&fatal);

/* If no version letter is present it must be version A,
   which is what is expected for INPE tapes.  Version B tapes
   have a "B", version C tapes have a "C".
*/
c_fhdrrd(header,117,tstr);
if (strncmp(tstr,"",1) != 0)
   c_errmsg("Unexpected format","inepmeta-format",&fatal);


/* Gather information need to build the description for the */
/* tape label.						    */

/* Construct the scene id			*/
/* Format = SPPPRRRFFYYDDDMN			*/
/* Where: S   = Satellite			*/
/*        PPP = Path				*/
/*        RRR = Row				*/
/*        FF  = WRS offset (usually is 00)	*/
/*        YY  = Year of exposure		*/
/*        DDD = Julian date of acquisition	*/
/*        M   = Instrument mode (1 for TM)	*/
/*        N   = Instrument multiplexor (0)	*/

/* Satellite number */
c_fhdrrd(header,8,satno);
strncpy(sceneid,&satno[1],1);

/* path/row Note: Fractional part is not usually used in Version A, */
/* copy what is there just in case.				    */
c_fhdrrd(header,4,ppprrr);
strncat(sceneid,&ppprrr[0],3);
strncat(sceneid,&ppprrr[4],3);
strncat(sceneid,&ppprrr[7],2);

/* acquistion date */
c_fhdrrd(header,6,tstr);
strncat(sceneid,&tstr[2],2);
strncpy(year,tstr,4);
strncpy(month,&tstr[4],2);
strncpy(day,&tstr[6],2);
yr = atol(year);
mn = atol(month);
dy = atol(day);
c_grejul(&yr,&mn,&dy,&jdate);
if (jdate < 10)
   sprintf(tstr,"00%d",jdate);
else if (jdate < 100)
   sprintf(tstr,"0%d",jdate);
else
   sprintf(tstr,"%d",jdate);
strncat(sceneid,tstr,3);
strncpy(date,&sceneid[9],5);
      
/* Note: the mode/mux is always blank for Version A,  */
/* so make it "10" always (TM=1).		      */
strncat(sceneid,"10",2);

/* Replace any blanks in scene id with 0's	      */
sceneid[16] = '\0';
for (i = 0; i < 16; i++)
   if (sceneid[i] == ' ')  sceneid[i] = '0';


/* build the description for the tape label */
strncpy(newpr,ppprrr,7);
for (i = 0; i < 7; i++)
   if (newpr[i] == ' ')  newpr[i] = '0';
newpr[3] = ' ';
newpr[7] = '\0';
sprintf(tapedesc,"%s %s %02d/%02d/%02d FAST",sceneid,newpr,mn, dy, (yr % 100));


/* Send the description back to the calling pdf */
str_ptr[0] = tapedesc;
if (q_string(&parblk,"DESC",1, str_ptr, P_UPDATE) != SUCCESS)
   {
   c_errmsg("Error passing variables to TAE","inepmeta-pass",&nonfatal);
   c_errmsg("Fatal error encountered","inepmeta-fatal",&fatal);
   }


/* Send the project back to the calling pdf */

/* First get the projection name removing all white spaces */
c_fhdrrd(header,43,tstr2);
len = (long) strlen(tstr2);
ptr = squeeze(tstr2,len);

if (strcmp(ptr,"UTM") == 0)
   strcpy(proj,"INPEU");
else if (strcmp(ptr,"SOM") == 0)
   strcpy(proj,"INPES");
else
   strcpy(proj,"INPE?");

str_ptr[0] = proj;
if (q_string(&parblk,"PROJ",1, str_ptr, P_UPDATE) != SUCCESS)
   {
   c_errmsg("Error passing variables to TAE","inpemeta-pass",&nonfatal);
   c_errmsg("Fatal error encountered","inpemeta-fatal",&fatal);
   }


/* Send the record length back to the calling pdf */

strcpy(tstr2,"");
strncat(tstr2,&header[1405],5);
len = (long) strlen(tstr2);
ptr = squeeze(tstr2,len);
len = (long) strlen(ptr);
strncat(record_length,ptr,len);
record_length[len] = '\0';

if (len == 0)
   {
   c_errmsg("Error determining record length","inpemeta-recl",&nonfatal);
   c_errmsg("Fatal error encountered","inpemeta-fatal",&fatal);
   }
else
   {
   for (i = 0; i < len; i++)
      if (record_length[i] == ' ')  record_length[i] = '0';
   }

str_ptr[0] = record_length;
if (q_string(&parblk,"RECLEN",1, str_ptr, P_UPDATE) != SUCCESS)
   {
   c_errmsg("Error passing variables to TAE","inpemeta-pass",&nonfatal);
   c_errmsg("Fatal error encountered","inpemeta-fatal",&fatal);
   }

if (q_out(&parblk) != SUCCESS)
   {
   c_errmsg("Error passing variables to TAE","inepmeta-pass",&nonfatal);
   c_errmsg("Fatal error encountered","inepmeta-fatal",&fatal);
   }


/* Write the meta data to a file if this is the last tape in	*/
/* this sequence (tape 2 of 2).					*/

/* Get the sequence number */
val = p_find(&parblk,"seq");
seq = IVAL(*val,0);

if (seq == 2)
   {
   val = p_find(&parblk,"TAPELOC");
   if (val->v_count != 2)
      {
      c_errmsg("Insufficient number of tape locations","inpemeta-tl",&nonfatal);
      c_errmsg("Fatal error encountered","inpemeta-fatal",&fatal);
      }
   for (i=0; i < 2; i++)
      strcpy(tapeloc[i],SVAL(*val,i));

   /* build the string to write to the file*/
   ppprrr[3] = ',';
   strncpy(outstr,ppprrr,7);
   sprintf(tstr,"%s",",");
   strncat(outstr,tstr,1);

   /* hrz offset */
   strncat(outstr,&ppprrr[7],2);
   strncat(outstr,tstr,1);

   /* date */
   strncat(outstr,date,5);
   strncat(outstr,tstr,1);

   /* satellite */
   strncat(outstr,&satno[1],1);
   strncat(outstr,tstr,1);

   /* mode/mux */
   c_fhdrrd(header,10,momx);
   strncat(outstr,&momx[2],2);
   strncat(outstr,tstr,1);

   /* geodetic process */
   c_fhdrrd(header,17,tstr2);
   strncat(outstr,tstr2,13);
   strncat(outstr,tstr,1);

   /* resampling */
   c_fhdrrd(header,19,tstr2);
   strncat(outstr,tstr2,2);
   strncat(outstr,tstr,1);

   /* projection name */
   /* Remove all white spaces */
   c_fhdrrd(header,43,tstr2);
   len = (long) strlen(tstr2);
   ptr = squeeze(tstr2,len);
   strncat(outstr,ptr,len);
   strncat(outstr,tstr,1);

   /* projection number */
   strcpy(tstr2,"");
   strncat(tstr2,&header[537],6);
   sscanf(tstr2,"%d",&proj_num);
   sprintf(tstr2,"%d",proj_num);
   len = (long) strlen(tstr2);
   strncat(outstr,tstr2,len);
   strncat(outstr,tstr,1);

   /* map zone */
   c_fhdrrd(header,47,&ltemp);
   sprintf(tstr2,"%d",ltemp);
   strncat(outstr,tstr2,4);
   strncat(outstr,tstr,1);

   /* projection parameters */
   c_fhdrrd(header,49,projp);
   for (i = 0; i < 15; i++)
      {
      sprintf(tstr2,"%1.14E",projp[i]);
      if (strcmp(tstr2,"INF") == 0)
         strcpy(tstr2,"0.0E+00");
      strncat(outstr,tstr2,24);
      strncat(outstr,tstr,1);
      }
	 
   /* ellipsoid */
   c_fhdrrd(header,51,tstr2);
   /* Remove all white spaces */
   len = (long) strlen(tstr2);
   ptr = squeeze(tstr2,len);
   strncat(outstr,ptr,len);
   strncat(outstr,tstr,1);

   /* pixel size */
   strncat(outstr,&header[1063],5);
   strncat(outstr,tstr,1);

   /* pixel/line */
   strcpy(tstr2,"");
   strncat(tstr2,&header[1086],4);
   len = (long) strlen(tstr2);
   ptr = squeeze(tstr2,len);
   strncat(outstr,ptr,len);
   strncat(outstr,tstr,1);

   /* lines/image */
   strcpy(tstr2,"");
   strncat(tstr2,&header[1108],4);
   len = (long) strlen(tstr2);
   ptr = squeeze(tstr2,len);
   strncat(outstr,ptr,len);
   strncat(outstr,tstr,1);

   /* UL long */
   strncat(outstr,&header[1116],13);
   strncat(outstr,tstr,1);

   /* UL lat */
   strncat(outstr,&header[1130],12);
   strncat(outstr,tstr,1);

   /* UR long */
   strncat(outstr,&header[1174],13);
   strncat(outstr,tstr,1);

   /* UR lat */
   strncat(outstr,&header[1188],12);
   strncat(outstr,tstr,1);

   /* LR long */
   strncat(outstr,&header[1232],13);
   strncat(outstr,tstr,1);

   /* LR lat */
   strncat(outstr,&header[1246],12);
   strncat(outstr,tstr,1);

   /* LL long */
   strncat(outstr,&header[1290],13);
   strncat(outstr,tstr,1);

   /* LL lat */
   strncat(outstr,&header[1304],12);
   strncat(outstr,tstr,1);

   /* Sun elevation */
   c_fhdrrd(header,101,&ltemp);
   sprintf(tstr2,"%d",ltemp);
   strncat(outstr,tstr2,2);
   strncat(outstr,tstr,1);

   /* Sun azimuth */
   c_fhdrrd(header,103,&ltemp);
   sprintf(tstr2,"%d",ltemp);
   strncat(outstr,tstr2,3);
   strncat(outstr,tstr,1);

   /* Tape locations */
   strncat(outstr,tapeloc[0],6);
   strncat(outstr,tstr,1);
   strncat(outstr,tapeloc[1],6);
	 
   /* Construct the meta file name		*/
   /* (scene id with a "inpe" suffix		*/
   strcpy(outfile,sceneid);
   strcat(outfile,".inpe");


   /* Write the metadata file */
   fp = fopen(outfile,"a");
   fprintf(fp,"%s\n",outstr);
   fclose(fp);
   }

/* Done */
z_exit(1," ");

return EXIT_SUCCESS;

}
