/******************************************************************************
FUNCTION:       READHEADER

PURPOSE:        Read the header file copied by DITTO and extract the needed
                information to write to the metadata file.

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
1.0      6/96   C. Engebretson   Original development

ALGORITHM REFERENCES:

	"Format Specifications for Canadian Landsat MSS System
	 Computer Compatible Tape", Jennifer Murphy, Research
	 Report 79-2, August 1979

	"Subject: Processing Landsat 4,5 MSS Data From Thailand
	Receiving Station", D. R. Steinwand, December 1992

*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "jsc.h"
#include "las.h"
#include "header.h"
#include "util.h"
#include "metadata.h"

int main
(void)
{
struct PARBLK parblk;           /* TAE parameter block                  */
struct VARIABLE *val;           /* Pointer to TAE variable within block */

char header[CMLEN];		/* Name of Landsat Header file		*/
char outfile[CMLEN];		/* Name of output (metadata) file	*/
char *tapeloc[2];		/* Array for call to meta_writearchv()  */
char tempstr[255]; 		/* Array to hold extracted data		*/
char tempstr2[255];		/* Array to hold extracted data		*/
char *ptr;			/* Temporary character pointer		*/

int row;			/* WRS row number			*/
float latlong;			/* Value to hold lat/long		*/

long mode = IREAD;		/* File access mode			*/
long std_out;			/* Standard output			*/
long zero = 0;			/* Constant of zero			*/

struct JSCLH lhdr;		/* JSC Landsat Header structure		*/
struct JSCUH uhdr;		/* JSC Universal Header structure	*/


/* Initialize LAS and retrieve input parameters from TAE
--------------------------------------------------------*/
c_inlas(&parblk, &std_out);
if (c_getnam(&parblk, "HEADER", &zero, header, "lhdr", &mode) != E_SUCC)
        c_lasmsg(TRUE, "jsccopy-fatal", "Fatal error encountered");
val = p_find(&parblk, "OUTFILE");
strcpy(outfile, SVAL(*val, 0));
val = p_find(&parblk, "TAPELOC");
tapeloc[0] = malloc(strlen(SVAL(*val, 0)) + 1);
strcpy(tapeloc[0], SVAL(*val, 0));
tapeloc[1] = NULL;

/* Call the header library routines and initialize metadata module
------------------------------------------------------------------*/
c_jsc_lhdr(header, &lhdr);
c_jsc_uhdr(header, &uhdr);
meta_init(outfile);

/* Insert required metadata
---------------------------*/
meta_insert(PATH_NBR, lhdr.path_number);
meta_insert(ROW_NBR, lhdr.row_number);
row = atoi(lhdr.row_number);

/* Extract the center lat/long information from the JSCLH structure, which is
   in the format "     DDDMM", and convert to decimal. */

strncpy(tempstr, lhdr.frame_centre_lat, 8);
tempstr[8] = 0;
strncpy(tempstr2, &lhdr.frame_centre_lat[8], 2);
tempstr2[2] = 0;
latlong = (float)atoi(tempstr) + (float)((float)atoi(tempstr2) / 60.);
sprintf(tempstr, "%3.5f", latlong);
meta_insert(CENTER_LAT, tempstr);
strncpy(tempstr, lhdr.frame_centre_long, 8);
tempstr[8] = 0;
strncpy(tempstr2, &lhdr.frame_centre_long[8], 2);
tempstr2[2] = 0;
latlong = (float)atoi(tempstr) + (float)((float)atoi(tempstr2) / 60.);
sprintf(tempstr, "%3.5f", latlong);
meta_insert(CENTER_LON, tempstr);

meta_insert(SATELLITE, lhdr.mission_id);
meta_insert(MAP_ZONE, lhdr.utm_zone_number);
meta_insert(ACQU_DATE, lhdr.data_acquisition_ddmmyy);
if ((ptr = strchr(uhdr.sun_azimuth_angle_centre_deg, '.')) != NULL) *ptr = 0;
if ((ptr = strchr(uhdr.sun_elevation_angle_centre_deg, '.')) != NULL) *ptr = 0;
meta_insert(SUN_AZIMUTH, uhdr.sun_azimuth_angle_centre_deg);
meta_insert(SUN_ELEVATION, uhdr.sun_elevation_angle_centre_deg);
meta_insert(INTERLVNG, "BIL");

/* Determine the recording technique.  This reflects the sensor (in this case
   it will always be MSS) and the orientation (descending or ascending).  We
   have the WRS row number available to determine this; a row between 120 and
   246 indicates ascending, otherwise we're dealing with descending.  The
   two values used here correspond to:

   38 == MSS descending
   39 == MSS ascending */

if ((row >= 120) && (row <= 246)) meta_insert(RECD_TECH, "39");
else meta_insert(RECD_TECH, "38");

/* Clean up
-----------*/
meta_writearchv(tapeloc, "JSC", 1);
meta_write();
meta_cleanup();

return EXIT_SUCCESS;

}
