/*****************************************************************************

NAME:  JSC_WRITELHDR

PURPOSE: Open and write to tape the JSC Landsat Header File.  This is done
         using LAS tapeio routines instead of DITTO functions because the
	 file consists of records that are of varying lengths; DITTO does
	 not provide any feasible means of writing a file like this to
 	 tape when copying from disk. 

PROGRAM HISTORY:
  Version       Date        Author          Reason
  --------      ----        -----------     ----------
     1.0        6/96        C. Engebretson  Initial development 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Runs under TAE/UNIX

PROJECT:  LAS

ALGORITHM DESCRIPTION: See inline comments

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"

int main
(void)
{
struct PARBLK parblk;           /* TAE parameter block                  */
struct VARIABLE *val;		/* Pointer to TAE variable		*/
FILE *inf;			/* File structure for input .lhdr	*/

char header[CMLEN];             /* Name of Landsat Header File to write */
char drive[CMLEN];              /* Name of drive for c_tpreop()         */
char density[CMLEN];		/* Density of tape drive		*/
unsigned char buffer[2700];		/* Data read from input file		*/

/* The size of the buffer is 2700 bytes (chars), as this is the largest
   record size in the file (which itself consists of records of varying
   sizes.) */

int i;				/* Loop counter				*/

long std_out;			/* Standard output			*/
long access = 1;		/* Tape access flag			*/
long mode = HANDLE;		/* Tape IO error mode flag	 	*/
long unit;			/* Tape drive unit number		*/
long one = 1;			/* Constant of one			*/
long opt1 = 0;			/* Misc. c_tpreop() options		*/
long two = 2;			/* Constant of two			*/
long rewopt = NOREW;		/* We do NOT want to rewind the tape!	*/
long skip = 0;			/* Number of files to skip on tape 	*/
long size = 0;			/* Number of samples			*/


/* Initialize TAE parameter block
---------------------------------*/
c_inlas(&parblk,&std_out);


/* Retrieve parameters from TAE
-------------------------------*/
val = p_find(&parblk, "HEADER");
strcpy(header, SVAL(*val, 0));
val = p_find(&parblk, "DRIVE");
strcpy(drive, SVAL(*val, 0));
val = p_find(&parblk, "DENS");
strcpy(density, SVAL(*val, 0));

/* Open the tape drive
----------------------*/
if (c_tpreop(&unit, &skip, &mode, &opt1, density, &access, drive) != TAPE_SUCC)
	c_lasmsg(TRUE, "jsccopy-drive", "Error opening tape drive");


/* Back space one EOF
----------------------*/
if (c_tpbsf(&unit,&one,&mode) !=TAPE_SUCC)
	c_lasmsg(TRUE, "jsccopy-backspace", "Error backspacing tape");


/* Open Landsat Header File for reading
---------------------------------------*/
if (!(inf = fopen(header, "r")))
	c_lasmsg(TRUE, "jsccopy-file",
		 "Unable to open .lhdr file for reading");


/* Write out the Landsat Header File to tape.  This file consists of seven
   records, whose sizes are hardwired to:

   Record #1: 1440 bytes (Landsat Header Record)
   Record #2: 2700 bytes (Geometric Transformation Record)
   Records #3-7: 1620 bytes (Radiometric Transformation Records)

   For more information on this format, see the Algorithm References.
--------------------------------------------------------------------------*/
if ((fread(buffer, 1, 1440, inf)) != 1440)
	c_lasmsg(TRUE, "jsccopy-read",
		 "Error reading data from Landsat Header File");
size = 1440;
if (c_tpwrit(&unit, buffer, &size, &mode) != size)
	c_lasmsg(TRUE, "jsccopy-write",
		 "Error writing record to tape");

if ((fread(buffer, 1, 2700, inf)) != 2700)
        c_lasmsg(TRUE, "jsccopy-read",
                 "Error reading data from Landsat Header File");
size = 2700;
if (c_tpwrit(&unit, buffer, &size, &mode) != size)
        c_lasmsg(TRUE, "jsccopy-write",
                 "Error writing record to tape");

for (i = 0, size = 1620; i < 5; i++)
	{
	if ((fread(buffer, 1, 1620, inf)) != 1620)
		c_lasmsg(TRUE, "jsccopy-read",
			 "Error reading data from Landsat Header File");
	if (c_tpwrit(&unit, buffer, &size, &mode) != size)
		c_lasmsg(TRUE, "jsccopy-write",
			 "Error writing record to tape");
	}


/* Writing of the file is complete.  Close tape, clean up, and exit.
--------------------------------------------------------------------*/
c_tpclos(&unit,&mode,&rewopt,&two);
fclose(inf);

return EXIT_SUCCESS;
}

