C     *************************************************************************
C     NAME			AGGRG2        	    		
C    
C     PURPOSE 	   Collapses columns of numbers from an input IBIS tabular 
C	           file into smaller columns in the output IBIS tabular 
C		   file using a designated column as the control.  Within
C	           the control column each number which is repeated defines
C	  	   one row.
C	
C		   There are three methods of collapsing columns:
C		   
C		   1.  The first option, which is the default one,
C		       determines the output value for a column by
C		       gouldecting the maximum value from the column
C		       based on the limits defined by the control
C		       values.  Column numbers not entered in the
C		       AREACOL or SUMCOL parameters are collapsed
C		       by this method.
C
C		   2.  The second option determines the output value
C		       for a column by aggregating all the values in a
C		       column based on the limits defined by the control
C		       values.  Column numbers specified in the SUMCOL
C		       parameter are collapsed by this method.
C	
C		   3.  The third option determines the output value for
C		       a column by gouldecting a value from a column based
C		       on the row position of the maximum value from the
C	    	       area key column.  One value is gouldected for each
C		       group of control values.  Column numbers specified
C		       in the AREACOL parameter are collapsed by this
C		       method.
C
C     Note:  The "fcol1", "cs1", and "cs2" arrays were dimensioned at
C 	     32,767  because that is currently the maximum number of
C    	     rows allowed in an IBIS tabular file.  Older versions of 
C	     AGGRG2 had these arrays dimensionsed at 50,000 and 100,00.
C
C     PROGRAM HISTORY
C	PROGRAMMER	  DATE		REASON
C	----------	  ----		------
C  	JPL (A. Zobrist) 		Original
C	K. Zanter        Dec. 1986      LAMS conversion		 
C	B. Ailts	 Oct. 1988	LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C       AGGRG2 must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of the user-specified parameters.  The input IBIS
C     tabular file is opened.  The control key and area key columns
C     are retrieved, and the control ranges and maximum areas are
C     determined.  The output IBIS file is opened.  Each column is
C     then collapsed as specified and the results written to the
C     output file.
C
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
      program aggrg2

	include 'las.fin'

	include 'lams.fin'

      	real*4		ABS
      	real*4	 	fcol1(MMXROW)
      	real*4		r

      	integer*4	access, action(MMXCOL), agco, area
      	integer*4	byarcl(MMXCOL)
      	integer*4	clen, count
      	integer*4	getnam
      	integer*4	i, ix
      	integer		fdesc1
      	integer		fdesc2
      	integer*4	j, jl, jmax, ju
      	integer*4	nagco, narea, nbyar, ncol
      	integer*4 	nrcol, nsumc
      	integer*4	nwclen

      	integer*4	parblk(XPRDIM)

      	integer*4	sumcl(MMXCOL)
      	integer*4 	status
      	integer*4	stdout
	integer*4	tbopen
	integer*4	tbrcol
	integer*4	tbwcol
	integer*4	tbclos
      	integer*4	type
	
      	integer*2	cs1(MMXROW)
      	integer*2 	cs2(MMXROW)
	
      	logical		agcodf
      	logical		areadf
      	logical		byardf
      	logical	   	sumcdf

      	character*(CMLEN)	infile
      	character*(CMLEN)	ofile
	
      	data agco,area/1,1/,action/MMXCOL*0/
	
C	Place the large variables in a common block to work around a
C	recent change in the sgi compiler that causes core dumps when
C	large variables are declared on the stack
	common parblk


C     initialization
C     --------------
      call inlas(parblk,stdout)

C     get the input IBIS file name
C     ----------------------------
      status = getnam(parblk,'INFILE',1,infile,'ibis_tab',0)
      if (status .ne. ESUCC) then
         call errmsg('Fatal error encountered','aggrg2-fatal',-1)
      endif

C     open the input IBIS file
C     ------------------------
      access = 0
      status = tbopen(fdesc1,infile,clen,ncol,access)
      if (status .ne. ESUCC) then
         call errmsg('Error opening input file','aggrg2-open',1)
	 call errmsg('Fatal error encountered','aggrg2-fatal',-1)
      endif

C     get user-specified parameters
C     -----------------------------
      call xrattr(parblk,'CONKEY',type,nagco,agcodf,access,status)
      call xrattr(parblk,'AREAKEY',type,narea,areadf,access,status)
      call xrattr(parblk,'AREACOL',type,nbyar,byardf,access,status)
      call xrattr(parblk,'SUMCOL',type,nsumc,sumcdf,access,status)

      call xrintg(parblk,'CONKEY',1,agco,count,status)
      call xrintg(parblk,'AREAKEY',1,area,count,status)
      call xrintg(parblk,'AREACOL',35,byarcl,count,status)
      call xrintg(parblk,'SUMCOL',35,sumcl,count,status)

      if (byardf) go to 72
      do 71 i = 1,nbyar
 71   action(byarcl(i)) = 1
 72   if (sumcdf) go to 2
      do 73 i = 1,nsumc
 73   action(sumcl(i)) = 2
 2    continue
      nrcol = clen

C  place control ranges in cs1 vector
C  ----------------------------------
      status = tbrcol(fdesc1,agco,fcol1)
      if (status .ne. ESUCC) then
	 call errmsg('Error reading input file','aggrg2-read',1)
	 call errmsg('Fatal error encountered','aggrg2-fatal',-1)
      endif
      nwclen = 0
      i = 0
 3    i = i+1
      if (i .gt. clen) go to 5
      r = fcol1(i)
      do 6 j = i,clen
      if (fcol1(j) .eq. r) go to 6
      nwclen = nwclen + 1
      cs1(nwclen) = j - i
      i = j - 1
      go to 3
 6    continue
      nwclen = nwclen + 1
      cs1(nwclen) = j-i+1

C  place maximum areas in cs2 vector
C  ---------------------------------
 5    continue
      status = tbrcol(fdesc1,area,fcol1)
      if (status .ne. ESUCC) then
	 call errmsg('Error reading input file','aggrg2-read',1)
	 call errmsg('Fatal error encountered','aggrg2-fatal',-1)
      endif
      ju = 0
      do 7 i = 1,nwclen
         jl = ju + 1
         ju = ju + cs1(i)
         r = -1.
         do 8 j = jl,ju
            cs2(j) = 0
            if (ABS(fcol1(j)) .le. r) go to 8
            jmax = j
            r = ABS(fcol1(j))
 8       continue
         cs2(jmax) = 1
 7    continue

C     get the output IBIS file name
C     -----------------------------
      status = getnam(parblk,'OUTFILE',1,ofile,'ibis_tab',1)
      if (status .ne. ESUCC) then
	 call clean(parblk,'aggrg2',ofile,0)
      endif

C     open the output IBIS file
C     -------------------------
      access = 1
      status = tbopen(fdesc2,ofile,nwclen,ncol,access)
      if (status .ne. ESUCC) then
         status = tbclos(fdesc2,ofile)
	 call errmsg('Error opening output file','aggrg2-open',1)
	 call clean(parblk,'aggrg2',ofile,1)
      endif
	
C     for each column perform the appropriate function
C     ------------------------------------------------
      do 20 ix = 1,ncol
         status = tbrcol(fdesc1,ix,fcol1)
	 if (status .ne. ESUCC) then
            status = tbclos(fdesc2,ofile)
	    call errmsg('Error reading input file','aggrg2-read',1)
	    call clean(parblk,'aggrg2',ofile,1)
	 endif
         if (action(ix) - 1) 10,11,12
 10      ju = 0
         do 17 i = 1,nwclen
            jl = ju + 1
            ju = ju + cs1(i)
            r = -1.
            do 18 j = jl,ju
               if (ABS(fcol1(j)) .le. r) go to 18
               jmax = j
               r = ABS(fcol1(j))
 18         continue
            fcol1(i) = fcol1(jmax)
 17      continue
         go to 19
 11      do 14 i = 1,clen
            if (cs2(i) .eq. 0) fcol1(i) = 0.
 14      continue
         go to 10
 12      ju = 0
         do 27 i = 1,nwclen
            jl = ju + 1
            ju = ju + cs1(i)
            r = 0.
            do 28 j = jl,ju
               r = r + fcol1(j)
 28         continue
            fcol1(i) = r
 27      continue
 19      continue
         status = tbwcol(fdesc2,ix,fcol1)
	 if (status .ne. ESUCC) then
            status = tbclos(fdesc2,ofile)
	    call errmsg('Error writing output file','aggrg2-write',1)
	    call clean(parblk,'aggrg2',ofile,1)
	 endif
 20   continue

C     close the input and output IBIS files
C     -------------------------------------
      status = tbclos(fdesc1,infile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','aggrg2-close',1)
	endif
      status = tbclos(fdesc2,ofile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','aggrg2-close',1)
	endif

C     output completion message
C     -------------------------
      call complt('AGGRG2',ofile,1)

      end
