/*****************************************************************************
NAME				 SHOWGRID

PURPOSE    Writes a radiometric mapping grid to a user defined output device.
	   User's terminal, an ASCII file, or the line printer are valid
	   output devices.


PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   S. Reichardt   11/82        CSC      Original Development
  4.0      Y. Sheu         5/85        SAR
  5.0      D. Steinwand   10/88        EDC      LAS 5.0 development,
						C conversion
  5.1      B. Ailts   	   1/89        EDC      Converion to a radiometric grid

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   DSPRGRD must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION
	Print the name of the radiometric Mapping Grid file
	Print the number of grid rows & columns, and the number of output
		space lines and samples
	Print the mapping grid in tabular form
	Print the polynomial coefficients
	Print the modeling error statistics, if available
	Print the gridding error statistics, if available

ALGORITHM REFERENCES		none
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "radpak.h"

#define NROWS_GROUP	5

FUNCTION show_grid(fname, filep, grid)

struct RADGRID *grid;		/* radiometric mapping grid */
FILE  *filep;			/* File pointer for output */
char  *fname;			/* TAE name of print file */
{
double *z_value;		/* Pointer to input grid line coordinates */
long i, lcnt, cnt, count;	/* Loop counters */
long nc_remain;			/* Number of columns remaining; this pass */
long zero = 0;			/* Constant of zero for pterm */
char proj_str[80];		/* Character version of projection code */
char term[15][20];		/* Character constants:  terms of polynomial */

/* Initialize the "terms of polynomial" contsant
  ---------------------------------------------*/
strcpy(&(term[0][0]), "Constant");
strcpy(&(term[1][0]), "X");
strcpy(&(term[2][0]), "Y");
strcpy(&(term[3][0]), "X**2");
strcpy(&(term[4][0]), "XY");
strcpy(&(term[5][0]), "Y**2");
strcpy(&(term[6][0]), "X**3");
strcpy(&(term[7][0]), "(X**2)Y");
strcpy(&(term[8][0]), "X(Y**2)");
strcpy(&(term[9][0]), "Y**3");
strcpy(&(term[10][0]), "X**4");
strcpy(&(term[11][0]), "(X**3)Y");
strcpy(&(term[12][0]), "(X**2)(Y**2)");
strcpy(&(term[13][0]), "X(Y**3)");
strcpy(&(term[14][0]), "Y**4");

/* Output radiometric mapping grid file name
  ---------------------------------------*/
fprintf(filep,"Radiometric Mapping Grid File Name:\n");
fprintf(filep,"  %s\n\n",fname);

/* Print first record--nrows, ncols, nlines, nsamples
  --------------------------------------------------*/
fprintf(filep, "Grid dimensions:    %d rows by %d columns\n",
                              grid->nrows,grid->ncols);

fprintf(filep, "Output image size:  %d lines by %d samples\n\n",
                              grid->lines, grid->samples);

fprintf(filep, "Headings indicate output space line and sample.\n");
fprintf(filep, "Matrix values indicate corresponding radiometric delta values.\n");

for (lcnt = 0; lcnt < grid->nrows; lcnt += NROWS_GROUP)
   for (nc_remain = grid->ncols, cnt = 0; nc_remain;)
     {
     if (nc_remain >= 3)
	{
	fprintf(filep,"\n\n                                OUTPUT SAMPLES\n");
	fprintf(filep, 
	"OUTPUT           %6d                 %6d                 %6d\n",
		grid->out_samps[cnt], grid->out_samps[cnt+1], 
		grid->out_samps[cnt+2]);
	fprintf(filep,
    " LINES           ------                 ------                 ------\n");
	for (count = 0; count < NROWS_GROUP; count ++)
 	   {
	   if ((lcnt + count) >= grid->nrows) break;
	   z_value = grid->z_values + (((lcnt + count) * grid->ncols) + cnt);
	   fprintf(filep,
	          "%6d |      %9.2lf              %9.2lf              %9.2lf\n",
		grid->out_lines[lcnt + count], z_value[0], z_value[1], 
		z_value[2]);
	   }
	nc_remain -= 3;
	cnt += 3;
	}
     else if (nc_remain == 2)
	{
	fprintf(filep,"\n\n                         OUTPUT SAMPLES\n");
	fprintf(filep, "OUTPUT           %6d                 %6d\n",
	   grid->out_samps[cnt],grid->out_samps[cnt+1]);
	fprintf(filep," LINES           ------                 ------\n");
	for (count = 0; count < NROWS_GROUP; count ++)
 	   {
	   if ((lcnt + count) >= grid->nrows) break;
	   z_value = &(grid->z_values[((lcnt + count) * grid->ncols)+cnt]);
	   fprintf(filep, "%6d |      %9.2lf              %9.2lf\n",
		grid->out_lines[lcnt + count], z_value[0], z_value[1]);
	   }
	nc_remain -= 2;
	cnt += 2;
	}
     else if (nc_remain == 1)
	{
	fprintf(filep,"\n\n              OUTPUT SAMPLES\n");
	fprintf(filep, "OUTPUT           %6d\n", grid->out_samps[cnt]);
	fprintf(filep," LINES           ------\n");
	for (count = 0; count < NROWS_GROUP; count ++)
 	   {
	   if ((lcnt + count) >= grid->nrows) break;
	   z_value = &(grid->z_values[((lcnt + count) * grid->ncols)+cnt]);
	   fprintf(filep, "%6d |      %9.2lf\n",
		grid->out_lines[lcnt + count], z_value[0]);
	   }
	nc_remain--;
	cnt++;
	}
   }

/* Print model coefficients
  ------------------------*/
fprintf(filep, "\n\nModel method:  %s\n",grid->model);
if (strcmp(grid->model, "POLYNOMIAL") == 0)
   {
   fprintf(filep, "Number of coefficients:  %d\n\n", grid->ncoeff);
   fprintf(filep, 
     "First polynomial:\n");
   for (i = 0; i < grid->ncoeff; i++)
      fprintf(filep, "      a%d:  %20.12e  (%s)\n",i, grid->coeffs[i],
			&(term[i][0]));
   fprintf(filep, 
    "\nSecond polynomial:\n");
   for (i = 0; i < grid->ncoeff; i++)
      fprintf(filep, "      b%d:  %20.12e  (%s)\n",i,
		grid->coeffs[i + grid->ncoeff], &(term[i][0]));
   fprintf(filep,"\n");
   }

/* Print modelling error statistics, if available
  ----------------------------------------------*/
if (grid->fiterr_valid == GEO_VALID)
   {
   fprintf(filep,"\nModeling error summary:  (units given in pixels)\n\n");
   fprintf(filep,"  Maximum line residual error:    %lf\n",
		grid->max_fit_err[0]);
   fprintf(filep,"  Maximum sample residual error:  %lf\n",
		grid->max_fit_err[1]);
   fprintf(filep,"  Average line residual error:    %lf\n",
		grid->ave_fit_err[0]);
   fprintf(filep,"  Average sample residual error:  %lf\n",
		grid->ave_fit_err[1]);
   fprintf(filep,"  Residual RMS error:             %lf\n", grid->rms_fit_err);
   }
else fprintf(filep,
   "\nModelling error summary not contained in Geometric Mapping Grid\n");

/* Print gridding error statistics, if available
  ---------------------------------------------*/
if (grid->griderr_valid == GEO_VALID)
   {
   fprintf(filep,
	"\nGrid generation error summary:  (units given in pixels)\n\n");
   fprintf(filep,"  Maximum line residual error:    %lf\n",
		grid->max_grid_err[0]);
   fprintf(filep,"  Maximum sample residual error:  %lf\n",
		grid->max_grid_err[1]);
   fprintf(filep,"  Average line residual error:    %lf\n",
		grid->ave_grid_err[0]);
   fprintf(filep,"  Average sample residual error:  %lf\n",
		grid->ave_grid_err[1]);
   fprintf(filep,"  Residual RMS error:             %lf\n", grid->rms_grid_err);
   }
else fprintf(filep,
   "\nGridding error summary not contained in Geometric Mapping Grid\n");

return(E_SUCC);
}
