/*    ..................................................................

        FUNCTION DGEFA

        PURPOSE
           TO FACTOR A GENERAL SYSTEM OF SIMULTANEOUS LINEAR EQUATIONS.

        USAGE
           status = dgefa(a, m, pivi, ipiv, eps)

        DESCRIPTION OF PARAMETERS
           A      - DOUBLE PRECISION M BY M COEFFICIENT MATRIX
                    (DESTROYED).
           M      - THE NUMBER OF EQUATIONS IN THE SYSTEM.
           PIVI   - DOUBLE PRECISION VECTOR CONSISTING OF THE INVERSES
                    OF THE PIVOT VALUES.
           IPIV   - THE PIVOT ROW NUMBERS.
           EPS    - DOUBLE PRECISION INPUT CONSTANT WHICH IS USED AS
                    RELATIVE TOLERANCE FOR TEST ON LOSS OF
                    SIGNIFICANCE.

        RETURN VALUE IS THE RESULTING ERROR PARAMETER CODED AS FOLLOWS:
                    0  - NO ERROR,
                   -1  - NO RESULT BECAUSE OF M LESS THAN 1 OR
                         PIVOT ELEMENT AT ANY ELIMINATION STEP
                         EQUAL TO 0,
                    K  - WARNING DUE TO POSSIBLE LOSS OF SIGNIFI-
                         CANCE INDICATED AT ELIMINATION STEP K+1,
                         WHERE PIVOT ELEMENT WAS LESS THAN OR
                         EQUAL TO THE INTERNAL TOLERANCE EPS TIMES
                         ABSOLUTELY GREATEST ELEMENT OF MATRIX A.

        REMARKS
           INPUT MATRICES R AND A ARE ASSUMED TO BE STORED COLUMNWISE
           IN M*N RESP. M*M SUCCESSIVE STORAGE LOCATIONS. ON RETURN
           SOLUTION MATRIX R IS STORED COLUMNWISE TOO.
           THE PROCEDURE GIVES RESULTS IF THE NUMBER OF EQUATIONS M IS
           GREATER THAN 0 AND PIVOT ELEMENTS AT ALL ELIMINATION STEPS
           ARE DIFFERENT FROM 0. HOWEVER WARNING IER=K - IF GIVEN -
           INDICATES POSSIBLE LOSS OF SIGNIFICANCE. IN CASE OF A WELL
           SCALED MATRIX A AND APPROPRIATE TOLERANCE EPS, IER=K MAY BE
           INTERPRETED THAT MATRIX A HAS THE RANK K. NO WARNING IS
           GIVEN IN CASE M=1.

        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
           NONE

        METHOD
           SOLUTION IS DONE BY MEANS OF GAUSS-ELIMINATION WITH
           COMPLETE PIVOTING.

     ..................................................................*/
#include <math.h>


long dgefa(double *a, long m, double *pivi, long *ipiv, double eps)
{
  double newpiv, piv, tb, tol;
  long mm, l, i, k, lst, ll, lend, ii, ist, j, ier;


  if (m <= 0)
    return(-1);

  /* Search for greatest element in matrix a.
  -------------------------------------------*/
  ier = 0;
  newpiv = 0.;
  mm = m*m;
  for (l=0; l<mm; l++)
    {
    tb = fabs(a[l]);
    if(tb-newpiv <= 0.)
      continue;
    newpiv = tb;
    i = l;
    }
  tol = eps*newpiv;

  /* a[i] is pivot element. newpiv contains the absolute value of a[i].

     Start elimination loop.
  ------------------------------------------------------------------*/
  lst = 0;
  for (k=0; k<m; k++)
    {

    /* Test on singularity.
    -----------------------*/
    if(newpiv <= 0.)
      return(-1);
    if(ier == 0 && newpiv-tol <= 0.)
      ier = k;
    pivi[k] = 1./a[i];
    j = i/m;
    i -= j*m + k;
    j -= k;
    ipiv[k] = i;

    /* i + k is row-index, j + k column-index of pivot element.
    -----------------------------------------------------------*/

    /* Is elimination terminated?
    -----------------------------*/
    if(k+1-m >= 0)
      break;

    /* Column interchange in matrix a.
    ----------------------------------*/
    lend = lst + m - k;
    if(j > 0)
      {
      ii = j*m;
      for (l=lst; l<lend; l++)
        {
        tb = a[l];
        ll = l + ii;
        a[l] = a[ll];
        a[ll] = tb;
        }
      }
    /* Row interchange and pivot row reduction in matrix a.
    -------------------------------------------------------*/
    for (l=lst; l<mm; l+=m)
      {
      ll = l + i;
      tb = pivi[k]*a[ll];
      a[ll] = a[l];
      a[l] = tb;
      }

    /* Save column interchange information.
    ---------------------------------------*/
    a[lst] = j;

    /* Element reduction and next pivot search.
    -------------------------------------------*/
    newpiv = 0.;
    lst++;
    j = 0;
    for (ii=lst; ii<lend;ii++)
      {
      piv = -a[ii];
      ist = ii + m;
      j++;
      for (l=ist; l<mm; l+=m)
        {
        ll = l - j;
        a[l] += piv*a[ll];
        tb = fabs(a[l]);
        if(tb-newpiv > 0.)
          {
          newpiv = tb;
          i = l;
          }
        }
      }
    lst += m;
    }

  return(ier);
}
