/*    ..................................................................

        FUNCTION DGESO

        PURPOSE
           TO SOLVE A GENERAL SYSTEM OF SIMULTANEOUS LINEAR EQUATIONS.

        USAGE
           status = dgeso(r, a, m, n, pivi, ipiv)

        DESCRIPTION OF PARAMETERS
           R      - DOUBLE PRECISION M BY N RIGHT HAND SIDE MATRIX
                    (DESTROYED). ON RETURN R CONTAINS THE SOLUTIONS
                    OF THE EQUATIONS.
           A      - DOUBLE PRECISION M BY M COEFFICIENT MATRIX
                    (DESTROYED).
           M      - THE NUMBER OF EQUATIONS IN THE SYSTEM.
           N      - THE NUMBER OF RIGHT HAND SIDE VECTORS.
           PIVI   - DOUBLE PRECISION VECTOR CONSISTING OF THE INVERSES
                    OF THE PIVOT VALUES.
           IPIV   - THE PIVOT ROW NUMBERS.

        RETURN VALUE IS THE RESULTING ERROR PARAMETER CODED AS FOLLOWS:
                    0  - NO ERROR,
                   -1  - NO RESULT BECAUSE OF M LESS THAN 1 OR
                         PIVOT ELEMENT AT ANY ELIMINATION STEP
                         EQUAL TO 0,

        REMARKS
           INPUT MATRICES R AND A ARE ASSUMED TO BE STORED COLUMNWISE
           IN M*N RESP. M*M SUCCESSIVE STORAGE LOCATIONS. ON RETURN
           SOLUTION MATRIX R IS STORED COLUMNWISE TOO.
           THE PROCEDURE GIVES RESULTS IF THE NUMBER OF EQUATIONS M IS
           GREATER THAN 0 AND PIVOT ELEMENTS AT ALL ELIMINATION STEPS
           ARE DIFFERENT FROM 0.

        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
           NONE

        METHOD
           SOLUTION IS DONE BY MEANS OF GAUSS-ELIMINATION WITH
           COMPLETE PIVOTING.

     ..................................................................*/
#include <math.h>
#include "las.h"
#include "finite.h"

long dgeso(double *r, double *a, long m, long n, double *pivi, long *ipiv)
{
  double piv, tb;
  long mm, nm, l, i, k, lst, ll, lend, ii, ist, j;
  long ier = 0;


  if (m <= 0)
    return(-1);

  mm = m*m;
  nm = n*m;

  /* Apply pivots to RHS.
  -----------------------*/

  lst = 0;
  for (k=0; k<m; k++)
    {
    for (l=k; l<nm; l+=m)
      {
      ll = l + ipiv[k];
      tb = pivi[k]*r[ll];
      r[ll] = r[l];
      r[l] = tb;
      }
    lend = lst + m - k;
    lst++;
    j = 0;
    for (ii=lst; ii<lend;ii++)
      {
      piv = -a[ii];
      j++;
      for (l=k; l<nm; l+=m)
        {
        ll = l + j;
        r[ll] += piv*r[l];
        }
      }
    lst += m;
    }

  /* Back substitution and back interchange.
  ------------------------------------------*/
  if(m-1 < 0)
    return(-1);
  else if (m-1 == 0)
    return(ier);
  ist = mm + m - 1;
  lst = m;
  for (i=1; i<m; i++)
    {
    ii = lst - i - 1;
    ist -= lst + 1;
    l = ist - m;
    l = (int)(a[l] + 0.5);
    for (j=ii; j<nm; j+=m)
      {
      tb = r[j];
      ll = j;
      for (k=ist; k<mm; k+=m)
        {
        ll++;
        tb -= a[k]*r[ll];
        }
      k = j + l;
      r[j] = r[k];
      r[k] = tb;
      }
    }
  return(ier);
}
