/*****************************************************************************
FUNCTION:	FINITE

PURPOSE:     TO CREATE A TRANSFORAMTION GRID FILE --
             RADIOMETRIC OR GEOMETRIC USING THE FINITE
             ELEMENT METHOD.

PROGRAM HISTORY:
  Version  Date    Author        Code/Conr   Change
  -------  ----    ------	 ---------   -------
   1.0	   9/79	 A.L.ZOBRIST	    JPL	    Initial devel.
   2.0	   8/86	 J.THORMODSGARD\   IMRD
	         B.AILTS	    CSB	    Conversion of LAMS
   3.0	   1/89	 B.Ailts	  EDC/CSB   LAS 5.0 conver.
   3.1     4/92    D.Etrheim      EDC/CSB   PRF #5911  
				            Increase maximum grid 
			                    size for SG
   7.0    10/98    L. Johnson       CSB     Converted from FORTRAN to C.
                                            Replaced "Greedy" triangulation
                                            with Delaunay triangulation.
   7.1    04/99    L. Johnson       CSB     SR2613  Increased the number of
                                            tiepoints allowed.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT:	LAS(LAMS)

ALGORITHM:

  if RADIOMETRIC then
      Read point locations and brightness shift values
  else (GEOMETRIC)
      Read point pair (input, output) locations.
  end if
  Compute first order polynomial fit of points.
  if RADIOMETRIC then
      Calculate the brightness shift values of the four triangulation
           corner points projected outside the image border.
  end if
  Calculate the locations of the four triangulation corner locations:
      left point = minline - delta, (minsamp + maxsamp)/2
      top point = (minline + maxline)/2, maxsamp + delta
      right point = (minline + maxline)/2, minsamp - delta
      bottom point = maxline + delta, (minsamp + maxsamp)/2
  where
      minline, maxline = the minimum and maximum line locations of
           the input points
      minsamp, maxsamp = the minimum and maximum sample locations of
           the input points
      delta = 5 * (maxline - minline + maxsamp - minsamp)
  Triangulate the control points using the Delaunay triangulation
       method.
  if RADIOMETRIC then
      At each grid intersection,locate the containing triangle and
           compute the radiometric difference value based on the first order
           polynomial fit of the three vertices of the triangle.
  else (GEOMETRIC)
      At each grid intersection, locate the containing triangle and
           compute the input location of the grid based on the first
           order polynomial fit of the three vertices of the triangle.
  end if
  Write the grid.

	ALGORITHM REFERENCES:
	FINITE ELEMENT SURFACE INTERPOLATION by June Thormosdgard

*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "lams.h"
#include "finite.h"


long fatal = -1;            /* Fatal flag for errmsg                        */
long nonfatal = 1;          /* Nonfatal flag for errmsg                     */


int main(void)
{
  struct GEOGRID *grid;     /* LAS grid structure                           */
  struct PARBLK parblk;     /* TAE parameter block                          */

  double avx;               /* Average radiometric difference value applied */
			    /* to four corner points far outside output     */
			    /* image area if option trend="average"         */
  double coeff[MMXPT][6];   /* Coefficients of polynomial.                  */
  double csol[6];           /* Coefficients of polynomial computed by llsq  */
  double dtol;              /* Grid tolerance value                         */
  double dx;                /* Temporary storage                            */
  double elsqxy[2*MMXPT];   /* Residuals calculated after llsq is run       */
  double maxg;              /* Maximum radiometric shift of output grid     */
  double maxgx;             /* Line location of maxg                        */
  double maxgy;             /* Sample location of maxg                      */
  double maxz;              /* Maximum radiometric shift of input points    */
  double maxzx;             /* Line location of maxz                        */
  double maxzy;             /* Sample location of maxz                      */
  double ming;              /* Minimum radiometric shift of output grid     */
  double mingx;             /* Line location of ming                        */
  double mingy;             /* Sample location of ming                      */
  double minz;              /* Minimum radiometric shift of input points    */
  double minzx;             /* Line location of minz                        */
  double minzy;             /* Sample location of minz                      */
  double rejpt[MMXPT];      /* Output location of rejected points           */
  double rpar[4*MMXPT];     /* Array containing control point locations.    */
                            /* If geometric transformation requested, the   */
                            /* order is input line,input sample, output     */
                            /* line, output sample.  If radiometric         */
                            /* transformation output line, output sample,   */
                            /* radiometric difference.                      */
  double tmaxx;             /* Maximum line location in output space        */
  double tmaxy;             /* maximum sample location in output space      */
  double tminx;             /* minimum line location in output space        */
  double tminy;             /* minimum sample location in output space      */
  double *tpts;             /* address of temporary workspace for control   */

  double corner[4];         /* If trend="USER", the user supplies the four  */
                            /* corner radiometric difference values in the  */
                            /* order: left, lower, upper, right             */
  double maxx;              /* Maximum line location in final output space  */
  double maxy;              /* Maximum sample location in final output space*/
  double minx;              /* Minimum line location in final output space  */
  double miny;              /* Minimum sample location in final output space*/
  double reject;            /* Tolerance defining maximum separation of     */
                            /* duplicate control points                     */
  double rparr[4*MMXPT];    /* rpar values before precision truncation      */

  long cnt1;                /* Input plot flag: 0 = don't plot input points */
                            /*                  1 = plot                    */
  long cnt2;                /* Output plot flag:0 = don't plot output points*/
                            /*                  1 = plot                    */
  long count;               /* Number of elements in input file             */
  long ftype;               /* Type of input file: 1 = IBIS graphics        */
                            /*                     2 = tie points location  */
  int  gridnl;              /* Number of lines within a single grid cell    */
  int  gridns;              /* Number of samples within a single grid cell  */
  long ii;                  /* Loop counter                                 */
  long nah, nav;            /* Number of rows and columns for grid.         */
                            /* Sent thru a grid reduction routine.          */
  long ndum;                /* Temporary storage                            */
  long nklsq;               /* Number of control points used in least       */
                            /* squares fit                                  */
  long nlret;               /* Number of edges in triangulation             */
  int  nn;                  /* Number of control points                     */
  long npoint;              /* Defaulted to 4 for pointing into rpar for a  */
                            /* geometric transformation.  Reset to 3 for    */
                            /* radiometric transformation.                  */
  long ntri;                /* Number of triangles in triangulation         */
  long numfile;             /* Number of files opened                       */
  long numrej;              /* Number of duplicate points rejected          */
  long offset;              /* Pointer offset for generating plot files     */
  long ptr;                 /* Array pointer                                */
  long status;              /* Function return status flag                  */
  long stdlun;              /* Logical unit for standard ouput              */
  int  window[4];           /* Image window: SL, SS, NL, NS                 */
  long zero = 0;            /* The value of zero                            */

  short con1[MMXPT],        /* Pointers to nodes of triangle                */
        con2[MMXPT],
        con3[MMXPT];
  short zgeom;              /* If radiometric transformation, then true(1), */
                            /* if geometric transformation, then false(0)   */

  char debug;               /* If debug desired, then true(1)               */
  char ha4[CMLEN];          /* Input control point filename (IBIS)          */
  char ha5[CMLEN];          /* Input control point filename (TPL)           */
  char outfile[4][CMLEN];   /* Host filenames: 0 = output grid              */
                            /*                 1 = finite.pr                */
                            /*                 2 = input space plot file    */
                            /*                 3 = output space plot file   */
  char tname[4][CMLEN];     /* LAS filenames:                               */
			    /*        0 = output grid                       */
                            /*      1-3 = finite.pr (optional)              */
                            /*            input space plot file (optional)  */
                            /*            output space plot file (optional) */
  char tname4[CMLEN];       /* LAS Input control points filename            */
  char trend[SZSUBCMD];     /* The method applied to calculate the          */
                            /* radiometric difference value for the four    */
			    /*  corner points far outside the output image  */
			    /* area.                                        */
                            /* = FIRST, the first order polynomial fit of   */
			    /*   all the radiometric difference values      */
                            /* = AVERAGE, the average of all the radiometric*/
                            /*   difference values                          */
                            /* = USER, the user supplies the four corner    */
			    /*   point values                               */

  FILE *outfp[4];           /* Output files:   0 = output grid              */
                            /*                 1 = finite.pr                */
                            /*                 2 = input space plot file    */
                            /*                 3 = output space plot file   */


  /* tpts and coeff will share the same memory.
  ---------------------------------------------*/
  tpts = &coeff[0][0];
         
  npoint = 4;

  /* Initialize TAE parameter block.
  ----------------------------------*/
  c_inlas(&parblk, &stdlun);

  /* Read parameters from TAE.
  ----------------------------*/
  getpar(&parblk, &zgeom, outfile, outfp, &numfile, ha4, ha5, &ftype,
         window, &reject, &dtol, &cnt1, &cnt2, trend, corner, &debug, &gridnl,
         &gridns, &miny, &minx, &maxy, &maxx);

  /* Convert host filenames to LAS filenames.
  -------------------------------------------*/
  strcpy(tname[0], outfile[0]);
  ii = 0;
  if (numfile-cnt1-cnt2-1 == 1)      /* Is there a print file? */
    {
    ii++;
    strcpy(tname[ii], outfile[1]);
    }
  if (cnt1)                          /* Is there an input space plot file? */
    {
    ii++;
    strcpy(tname[ii], outfile[2]);
    }
  if (cnt2)                          /* Is there an output space plot file? */
    {
    ii++;
    strcpy(tname[ii], outfile[3]);
    }

  /* Get the control points.  
     GEOMETRIC:  Count is the number of points*4 or the size of rpar.
     rpar is the input and output coordinates of the control stored
     as input line, input sample, output line, output sample and
     repeated for number of points.
     RADIOMETRIC:  Count is the number of points*3 or the size of rpar.
     rpar is the output coordinates of the control stored
     as line, sample, and radiometric offset for all points.
  ---------------------------------------------------------------------*/
  status = c_getcfp(ha4, ha5, &ftype, rparr, &count, &zgeom, &grid, window);
  if (status != E_SUCC)
    cleanup(&parblk, outfile, outfp);

  /* If using a TPL file, copy TPL filename (ha5) to IBIS filename (ha4)
     since ha4 is the string used later to denote the control point
     file used.
  --------------------------------------------------------------------*/
  if (ftype == 2)
    strcpy(ha4, ha5);

  /* 12-15-86 -- B.AILTS -- PREC6 limits the precision of the real inputs to 
     lose the digits that are not significant.
  --------------------------------------------------------------------------*/
  prec6_(rparr, &count, rpar);

  /* If radiometric correction grid is desired, the number of variables
     per point is 3: output line, output sample, radiometric difference.
  ----------------------------------------------------------------------*/
  if (zgeom)
    npoint = 3;

  /* nn is set to the number of control points.  A minimum of four points
     is required.  Abort if less than four points.
  -----------------------------------------------------------------------*/
  nn = count/npoint;
  if (nn < 4)
    {
    c_errmsg ("A minimum of four sets of tiepoints are needed",
              "finite-tiepoint", &nonfatal);
    cleanup(&parblk, outfile, outfp);
    }
  ptr = 0;
  ndum = count + ptr - 1;
  if (!zgeom)
    {
    /* Exchange the input and output line, sample locations.
    --------------------------------------------------------*/
    for (ii=ptr; ii<ndum; ii+=4)
      {
      dx = rpar[ii];
      rpar[ii] = rpar[ii+2];
      rpar[ii+2] = dx;
      dx = rpar[ii+1];
      rpar[ii+1] = rpar[ii+3];
      rpar[ii+3] = dx;
      }
    }

  /* Perform first order fit of control points.
  ---------------------------------------------*/
  status = fof(rpar, csol, elsqxy, &nklsq, &avx, &maxz, &maxzx,
               &maxzy, &minz, &minzx, &minzy, &tmaxx, &tmaxy, &tminx,
               &tminy, tpts, nn, npoint, zgeom);

  if (status != E_SUCC)
    cleanup(&parblk, outfile, outfp);

  /* Convert host name for input control point file to TAE form.
  --------------------------------------------------------------*/
  strcpy(tname4, ha4);

  /* Write header information to output file.
  -------------------------------------------*/
  outheader(outfp[1], avx, csol, maxz, maxzx, maxzy, minz, minzx,
            minzy, tname4, zgeom, debug);

  /* Set pointers into rpar depending on if radiometric or geometric 
     transformation requested. 
  ------------------------------------------------------------------*/
  ptr = 0;
  if (!zgeom)
    ptr += 2;

  /* Compute residuals from the first order fit at only the points 
     used in the fit (no more than first MMXPT points).  The polynomial 
     fit maps output to input in a geometric transformation.  The 
     polynomial maps output to radiometric difference in a radiometric 
     transformation.
  -----------------------------------------------------------------------*/
  for (ii=0; ii<nklsq; ii++)
    {
    elsqxy[ii] += -rpar[ptr]*csol[0] - rpar[ptr+1]*csol[1] - csol[2];
    if (!zgeom)
      elsqxy[ii+nklsq] +=  -rpar[ptr]*csol[3] - rpar[ptr+1]*csol[4] - csol[5];
    if (debug)
      {
      if (zgeom)
        fprintf(outfp[1], "   %11.2f %11.2f    %8.2f\n",
               rpar[ptr], rpar[ptr+1], elsqxy[ii]);
      else
        fprintf(outfp[1], "   %11.2f %11.2f    %8.2f %8.2f\n",
               rpar[ptr], rpar[ptr+1], elsqxy[ii], elsqxy[ii+nklsq]);
      }
    ptr += npoint;
    }

  fprintf(outfp[1],
      "\n    MINIMUM AND MAXIMUM OUTPUT LINES   =  %5.0f %5.0f\n", miny,maxy);
  fprintf(outfp[1],
        "    MINIMUM AND MAXIMUM OUTPUT SAMPLES =  %5.0f %5.0f\n", minx,maxx);

  /* Generate four corner points to add to control point mesh.  These four
     points are well outside the image area to ensure that the entire image
     can be mapped using the control points.
  -------------------------------------------------------------------------*/
  fourcorners(tmaxx, tmaxy, tminx, tminy, &nn, npoint, zgeom, rpar, outfp[1],
              tpts, csol, trend, avx, corner);

  /*  tri:  Triangulizes the control points using Delaunay triangulation.
            The node numbers for all triangle vertices are stored in
            con1, con2, and con3.
  -----------------------------------------------------------------------*/
  c_errmsg("Triangulizing input points-Please wait...", "informational",
           &nonfatal);
  status = tri_(&nn, &nlret, &ntri, tpts, &tpts[MMXPT], con1, con2, con3,
                &reject, &numrej, rejpt, &zero);
  if (status != E_SUCC)
    cleanup(&parblk, outfile, outfp);

  fprintf(outfp[1], "\n    NUMBER OF INPUT CONTROL POINTS       = %5d\n", nn);
  fprintf(outfp[1], "    NUMBER OF DUPLICATE POINTS REJECTED  = %5d\n",
                                                                      numrej);
  if (numrej > 0)
    {
    fprintf(outfp[1], "    OUTPUT LOCATION (line,sample) OF REJECTED ");
    fprintf(outfp[1], "POINTS:\n");
    }
  for (ii=0; ii<numrej*2; ii+=2)
    fprintf(outfp[1], "             %10.2f              %10.2f\n",
           (float)rejpt[ii], (float)rejpt[ii+1]);
  fprintf(outfp[1], "\n    NUMBER OF EDGES AND TRIANGLES    = %4d %4d\n",
                                         nlret, ntri);

  /* Solve triangles.
  -------------------*/
  solve(rpar, coeff, con1, con2, con3, ntri, outfp[1], debug, zgeom, npoint);

  /* For geometric mapping:
     mkggrd calculates the geometric grid by
     calculating the line, sample input coordinates of grid intersection
     from first order polynomial fit of three vertices of the
     triangle containing the grid intersection.

     For radiometric mapping:
     mkrgrd calculates the geometric grid by calculating the 
     radiometric differences of the grid intersection from the first
     order polynomial fit of the triangle vertices containing the
     grid intersection.
  ----------------------------------------------------------------------*/
  c_errmsg("Creating the OUTGRID file-Please wait...", "informational",
           &nonfatal);
  if (!zgeom)
    status = c_mkggrd(grid, outfile[0], window, &gridns, &gridnl,
                      coeff, con1, con2, con3, rpar, dtol, ntri,&nav,&nah);
  else
    {
    maxg = -1.e20;
    ming =  1.e20;
    status = c_mkrgrd(outfile[0], window, &nav, &nah, &gridns, &gridnl,
                      coeff, con1, con2, con3, rpar, dtol, ntri,
                      &maxg, &maxgy, &maxgx, &ming, &mingy, &mingx);
    }
  if (status != E_SUCC)
    cleanup(&parblk, outfile, outfp);

  fprintf(outfp[1], "\n    NUMBER OF ROWS AND COLUMNS IN GRID = %5d %5d\n\n",
                                  nav, nah);
  fprintf(outfp[1], "    NUMBER OF OUTPUT LINES AND SAMPLES WITHIN A GRID ");
  fprintf(outfp[1], "CELL =  %4d %4d\n\n", gridnl, gridns);

  if (zgeom)
    {
    fprintf(outfp[1], "    MINIMUM RADIOMETRIC SHIFT VALUE OF OUTPUT GRID ");
    fprintf(outfp[1], "AND ITS LOCATION (L,S):\n");
    fprintf(outfp[1], "             %11.7f    %8.2f     %8.2f\n",
                         ming, mingy, mingx);
    fprintf(outfp[1], "    MAXIMUM RADIOMETRIC SHIFT VALUE OF OUTPUT GRID ");
    fprintf(outfp[1], "AND ITS LOCATION (L,S):\n");
    fprintf(outfp[1], "             %11.7f    %8.2f     %8.2f\n",
                         maxg, maxgy, maxgx);
    }

  fprintf(outfp[1], "\n    OUTPUT GRID FILE NAME = %s\n", tname[0]);

  /* Write out triangle plot in input space.
  ------------------------------------------*/
  if (cnt1 == 1)
    {
    c_errmsg("Creating the INPLOT file-Please wait...", "informational",
             &nonfatal);
    offset = 2;
    outplot(rpar, con1, con2, con3, ntri, outfp[2], offset);
    fclose (outfp[2]);
    }

  /* Write out triangle plot in output space.
  -------------------------------------------*/
  if (cnt2 == 1)
    {
    c_errmsg("Creating the OUTPLOT file-Please wait...", "informational",
             &nonfatal);
    offset = 0;
    outplot(rpar, con1, con2, con3, ntri, outfp[3], offset);
    fclose(outfp[3]);
    }

  fclose(outfp[1]);

  c_complt("FINITE",tname,&numfile);
  return (0);
}


void cleanup(struct PARBLK *parblk, char hosout[][CMLEN], FILE *outfp[])
{
  long four = 4;

  fclose(outfp[1]);
  fclose(outfp[2]);
  fclose(outfp[3]);
  c_clean(parblk, "finite", hosout, &four);
}
