/*****************************************************************************
FUNCTION:	FOF

PURPOSE:     Generate a first order fit of a set of control points.

PROGRAM HISTORY:
  Version  Date	   Author      Code/Conr     Request
  -------  ----	   ------      ---------     -------
   1.0    01/99  L. Johnson      CSB      Extracted from finite.c.
   1.1    04/99  L. Johnson      CSB      SR2613  Increased the number of
                                          tiepoints allowed.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT:	LAS(LAMS)

ALGORITHM:

*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "lams.h"
#include "finite.h"


int fof
(
    double *rpar,           /* Array containing control point locations.    */
    double *csol,           /* Coefficients of polynomial computed by llsq  */
    double *elsqxy,         /* Residuals calculated after llsq is run       */
    long *nklsq,            /* Number of control points used in least       */
    double *avx,            /* Average radiometric difference value applied */
    double *maxz,           /* Maximum radiometric shift of input points    */
    double *maxzx,          /* Line location of maxz                        */
    double *maxzy,          /* Sample location of maxz                      */
    double *minz,           /* Minimum radiometric shift of input points    */
    double *minzx,          /* Line location of minz                        */
    double *minzy,          /* Sample location of minz                      */
    double *tmaxx,          /* Maximum line location in output space        */
    double *tmaxy,          /* maximum sample location in output space      */
    double *tminx,          /* minimum line location in output space        */
    double *tminy,          /* minimum sample location in output space      */
    double *tpts,           /* address of temporary workspace for control   */
    long nn,                /* Number of control points                     */
    long npoint,            /* Defaulted to 4 for pointing into rpar for a  */
    int zgeom               /* If radiometric transformation, then true(1), */
                            /* if geometric transformation, then false(0)   */
)
{
  long ipiv[3];             /* Array of pivot row numbers.                  */
  double clsq[3*MMXPT];     /* Output line,sample locations of control      */
  double clsqxy[2*MMXPT];   /* Input line,sample locations of control       */
  double aux[6];            /* Auxiliary array for matrix computations      */
  double eps;               /* Very small number                            */
  long ndum;                /* Temporary storage                            */
  long ii;                  /* Loop counter                                 */
  long ier;                 /* Error flag returned from matrix routines     */
  long nklsq2;              /* 2*nklsq                                      */
  long ptr;                 /* Array pointer                                */
  long three = 3;           /* The value of three.                          */


  /* Set minimum and maximum valid line and sample values.
     Provides initialization for later search of control points.
  --------------------------------------------------------------*/
  *tmaxx = 0.;
  *tmaxy = 0.;
  *tminx = 1.e20;
  *tminy = 1.e20;
  *minz =  1.e20;
  *maxz = -1.e20;
  *avx = 0.;

  /* Set nklsq to the minimum of MMXPT or total number of points.
     This is the number of control points used in first order polynomial
     fit used to detect outliers and create four outside control points.
  ----------------------------------------------------------------------*/
  *nklsq = min(nn, MMXPT);
  nklsq2 = 2*(*nklsq);
  ptr = 0;
  if (!zgeom)
    ptr += 2;

  /* Fill tpts as a temporary array for all locations of the 
     control points.  Search for minimum and maximum line
     and sample locations of the control points in output space.
  --------------------------------------------------------------*/
  for (ii=0; ii<nn; ii++)
    {
    tpts[ii] = rpar[ptr];
    tpts[ii+MMXPT] = rpar[ptr+1];
    if (rpar[ptr] > *tmaxx)
      *tmaxx = rpar[ptr];
    if (rpar[ptr] < *tminx)
      *tminx = rpar[ptr];
    if (rpar[ptr+1] > *tmaxy)
      *tmaxy = rpar[ptr+1];
    if (rpar[ptr+1] < *tminy)
      *tminy = rpar[ptr+1];

    /* If the number of control points is greater than MMXPT,
       then ignore the remaining control points in the
       first order polynomial fit of the control.
    -------------------------------------------------------*/
    if (ii < MMXPT)
      {

      /* Fill the clsq array with output line, sample locations
         of the control.  clsqxy is filled with the input line,
         sample locations if a geometric transformation
         is desired (!zgeom) or with the delta radiometric
         value if a radiometric transformation is requested.
      ---------------------------------------------------------*/
      clsq[ii] = rpar[ptr];
      clsq[ii+(*nklsq)] = rpar[ptr+1];
      clsq[ii+nklsq2] = 1.;
      if (!zgeom)
        {
        clsqxy[ii] = rpar[ptr-2];
        clsqxy[ii+(*nklsq)] = rpar[ptr-1];
        }
      else
        clsqxy[ii] = rpar[ptr+2];
      elsqxy[ii] = clsqxy[ii];
      elsqxy[ii+(*nklsq)] = clsqxy[ii+(*nklsq)];
      }
    *avx += rpar[ptr+2];
    if (rpar[ptr+2] < *minz)
      {
      *minz = rpar[ptr+2];
      *minzx = rpar[ptr];
      *minzy = rpar[ptr+1];
      }
    if (rpar[ptr+2] > *maxz) 
      {
      *maxz = rpar[ptr+2];
      *maxzx = rpar[ptr];
      *maxzy = rpar[ptr+1];
      }
    ptr += npoint;
    }
  *avx /= (double)nn;

  /* llsq performs a first order polynomial fit either between the
     input and output control point locations if a geometric
     transformation is desired or between the output control
     locations and the delta radiometric values at the control.
  ----------------------------------------------------------------*/
  eps = 1.e-7;
  ndum = npoint - 2;
  ier = llsqd(clsq, clsqxy, *nklsq, three, ndum, csol, ipiv, eps, aux);
  if (ier != 0)
    return (E_FAIL);
  else
    return (E_SUCC);
}
