/*****************************************************************************
FUNCTION:	fourcorners

PURPOSE:     TO CREATE FOUR CORNER POINTS WELL OUTSIDE THE IMAGE REGION
             SO THAT ALL IMAGE PIXELS HAVE A DEFINED POLYNOMIAL CONVERSION.

PROGRAM HISTORY:
VERSION  DATE      AUTHOR       CODE/CONT   REASON
-------  ----      ------       ---------   ------
 1.0    07/98   L.D. Johnson       CSB      Moved source out of finite.c
 1.1    04/99     L. Johnson       CSB      SR2613  Increased the number of
                                            tiepoints allowed.

ALGORITHM:

  The location of the four triangulation corner points that lie far
  outside the image border are calculated as follows.
    left point = minline - delta, (minsamp + maxsamp)/2
    top point = (minline + maxline)/2, maxsamp + delta
    right point = (minline + maxline)/2, minsamp - delta
    bottom point = maxline + delta, (minsamp + maxsamp)/2
  where
    minline, maxline = the minimum and maximum line locations of the
        input points
    minsamp, maxsamp = the minimum and maximum sample locations of the
        input points
    delta = 5 * (maxline - minline + maxsamp - minsamp)

    ALGORITHM REFERENCES:
    FINITE ELEMENT SURFACE INTERPOLATION by June Thormosdgard

*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "lams.h"
#include "finite.h"

void fourcorners
(
    double tmaxx,    /* Maximum line location in output space               */
    double tmaxy,    /* maximum sample location in output space             */
    double tminx,    /* minimum line location in output space               */
    double tminy,    /* minimum sample location in output space             */
    int  *nn,        /* The number of control points                        */
    long npoint,     /* Defaulted to 4 for pointing into rpar for a         */
                     /* geometric transformation.  Reset to 3 for           */
                     /* radiometric transformation.                         */
    short zgeom,     /* If radiometric transformation, then true(1)         */
    double *rpar,    /* Array containing control point locations.           */
                     /* If geometric transformation requested, the          */
                     /* order is input line,input sample, output            */
                     /* line, output sample.  If radiometric                */
                     /* transformation output line, output sample,          */
                     /* radiometric difference.                             */
    FILE *outfp,     /* Output file                                         */
    double *tpts,    /* address of temporary workspace for control          */
    double *csol,    /* Coefficients of polynomial computed by llsq         */
    char *trend,     /* The method applied to calculate the                 */
                     /* radiometric difference value for the four corner    */
                     /* points far outside the output image area.           */
    double avx,      /* Average radiometric difference value applied to     */
                     /* four corner points far outside output image area    */
                     /* if option trend="average"                           */
    double corner[4] /* If trend="USER", the user supplies the four         */
                     /* corner radiometric difference values in the         */
                     /* order: left, lower, upper, right                    */
)
{
  double dx;         /* Number >> range of the output lines and samples     */
  double xsq, ysq;   /* Temporary line, sample locations                    */
  long ptr;          /* Array pointer                                       */
  long ii;           /* Loop counter                                        */


  /* dx = 5 x (range of output lines + range of output samples)
     The output locations of the four corner locations are
     calculated far outside the output area.  The corners are:
     1) line = middle line of output space
        sample = minimum sample - dx
     2) line = maximum line + dx
        sample = middle sample of output space
     3) line = minimum line - dx
        sample = middle sample of output space
     4) line = middle line of output space
        sample = maximum sample + dx
  -------------------------------------------------------------*/
  dx = (tmaxx - tminx + tmaxy - tminy)*5.;
  ptr = *nn*npoint;
  if (!zgeom)
    ptr += 2;
  rpar[ptr] = tminx - dx;
  rpar[ptr+1] = (tminy + tmaxy)/2.;
  rpar[ptr+npoint]   = (tminx + tmaxx)/2.;
  rpar[ptr+npoint+1] = tmaxy + dx;
  rpar[ptr+2*npoint]   = (tminx + tmaxx)/2.;
  rpar[ptr+2*npoint+1] = tminy - dx;
  rpar[ptr+3*npoint]   = tmaxx + dx;
  rpar[ptr+3*npoint+1] = (tminy + tmaxy)/2.;

  /*  If geometric transformation, then compute input locations
      of four corner points using first order polynomial fit of
      first MMXPT control points.
  -------------------------------------------------------------*/
  fprintf(outfp, "  FOUR TRIANGULATION CORNER POINTS FAR OUTSIDE ");
  fprintf(outfp, "OUTPUT IMAGE:\n");
  if (!zgeom)
    {
    fprintf(outfp, "         OLINE            OSAMP");
    fprintf(outfp, "            ILINE          ISAMP\n");
    for (ii=0; ii<4; ii++)
      {
      xsq = rpar[ptr];
      ysq = rpar[ptr+1];
      tpts[*nn+ii] = xsq;
      tpts[*nn+MMXPT+ii] = ysq;
      rpar[ptr-2] = csol[0]*xsq + csol[1]*ysq + csol[2];
      rpar[ptr-1] = csol[3]*xsq + csol[4]*ysq + csol[5];
      fprintf(outfp, "     %15.7E  %15.7E  %15.7E  %15.7E\n",
              rpar[ptr], rpar[ptr+1], rpar[ptr-2], rpar[ptr-1]);
      ptr += 4;
      }
    }
  else
    {
    fprintf(outfp, "         OLINE           OSAMP");
    fprintf(outfp, "      RADIOMETRIC DIFFERENCE\n");
    for (ii=0; ii<4; ii++)
      {
      tpts[*nn+ii] = rpar[ptr];
      tpts[*nn+MMXPT+ii] = rpar[ptr+1];
      if (!strcmp(trend, "FIRST"))
        rpar[ptr+2] = csol[0]*rpar[ptr] + csol[1]*rpar[ptr+1] + csol[2];
      if (!strcmp(trend, "AVERAGE"))
        rpar[ptr+2] = avx;
      if (!strcmp(trend, "USER"))
        rpar[ptr+2] = (double)corner[ii];
      fprintf(outfp, "     %15.7E  %15.7E  %15.7E\n", rpar[ptr], rpar[ptr+1],
                                                         rpar[ptr+2]);
      ptr += 3;
      }
    fprintf(outfp, "\n    OPTION FOR FOUR CORNER POINTS = %s\n", trend);
    }
  *nn += 4;
}


