/****************************************************************************
NAME:				getcfp

PURPOSE:  Retrieve control points from input file (IBIS graphics or TPL).

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  1.0	 01/99	L. Johnson    CSB      Header added.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  Run under TAE

PROJECT:  LAS

ALGORITHM:

  if GEOMETRIC then
      Fill in the coordinate part of the grid structure.
  endif
  if IBIS file then
      Read control points from IBIS-graph file.
  else (TPL file)
      Read control points from TPL file.
  endif

*****************************************************************************/

#include "las.h"
#include "lams.h"
#include "geompak.h"
#include "proj.h"
#include "finite.h"

int c_getcfp
(
    char *ha1,
    char *ha2,
    long *ftype,
    double *buf,
    long *tcount,
    short *zgflg,
    struct GEOGRID **geogrid,
    int  *window
)
{ /*  getcfp  */

struct GEOGRID *grid;
struct TPLDATA *tpldata;
struct TPLHEAD *pthdr;

long access = 0;
long i;
long j;
long nl;
long nw;
long one = 1;
long pp = 0;
long pttype;
long status;

int fdesc;
struct PTDESC *ptdesc;
 
float *rbuf;            /* float version of buf; for use with c_grread      */


/* If the grid being created is not a radiometric grid fill in the coordinate
   part of the geometric grid.  This information is pulled from the tie
   point location file if specified, or it will be nulled out.
   --------------------------------------------------------------------------*/
if (!*zgflg)
   {
   grid = *geogrid = (struct GEOGRID *)malloc(sizeof(struct GEOGRID));
   if ( grid == NULL)
      {
      c_errmsg("Error allocating dynamic memory","finite-alloc",&one);
      return(E_FAIL);
      }

   /*  If a tie point location file was specified, use the data from it to
       fill in the projection coordinate section of the geometric grid.
       ------------------------------------------------------------------*/
   if (strncmp(ha2,"\0",1) != 0)
      {
      pthdr = (struct TPLHEAD *)malloc(sizeof(struct TPLHEAD));
      if (pthdr == NULL)
         {
         c_errmsg("Error allocating dynamic memory","finite-alloc",&one);
         return(E_FAIL);
         }
      access = IREAD;
      pttype = TPL;
      status = c_ptopen(&ptdesc,ha2,&pttype,&access,pthdr);
      if (status != E_SUCC)
         return(E_FAIL);
      grid->proj_valid = pthdr->proj_valid;
      grid->code = pthdr->code;
      grid->zone = pthdr->zone;
      grid->datum = pthdr->datum;
      grid->pdist[0] = pthdr->pdist[0];
      grid->pdist[1] = pthdr->pdist[1];
      for (j = 0; j < COEFCT; j++)
         grid->projprms[j] = pthdr->projprms[j];
      strcpy(grid->units,pthdr->units);
      if (cal_corners(window,pthdr->corners,grid) != E_SUCC)
         return(E_FAIL);
      free(pthdr);
      }
   else
      { /*  null out the projection coordinate section of the grid  */
      grid->proj_valid = GEO_NOTVALID;
      grid->code = 0;
      grid->zone = 0;
      grid->datum = 0;
      grid->pdist[0] = 0;
      grid->pdist[1] = 0;
      for (j = 0; j < COEFCT; j++)
         grid->projprms[j] = 0.0;
      strcpy(grid->units," ");
      }
   grid->fiterr_valid = GEO_NOTVALID;
   grid->griderr_valid = GEO_NOTVALID;
   grid->ncoeff = 0;
   grid->max_grid_err[0] = 0.0;
   grid->max_grid_err[1] = 0.0;
   grid->ave_grid_err[0] = 0.0;
   grid->ave_grid_err[1] = 0.0;
   grid->rms_grid_err = 0.0;
   for (j = 0; j < 32; j++)
      grid->act_grid_err[j] = 0.0;
   grid->max_fit_err[0] = 0.0;
   grid->max_fit_err[1] = 0.0;
   grid->ave_fit_err[0] = 0.0;
   grid->ave_fit_err[1] = 0.0;
   grid->rms_fit_err = 0.0;
   }

/*  The points may be read from an IBIS graphics file or a tie point location
    file.
    -------------------------------------------------------------------------*/
if (*ftype == 1)
   {
   status = c_gropen(&fdesc,ha1,&nw,&nl,&access);
   if (status != E_SUCC)
      {
      c_errmsg("Cannot open input file","finite-open",&one);
      return (E_FAIL);
      }

   /* Allocate storage for a single tiepoint location.
      ------------------------------------------------*/
   if ((rbuf = (float *)malloc((nw+1)*sizeof(float))) == NULL)
   {
       c_lasmsg(FALSE, "finite-alloc", "Error allocating dynamic memory");
       return E_FAIL;
   }
   
   for (i = 1; i <= nl; i++)
      {
      status = c_grread(&fdesc,&i,rbuf);
      if (status != E_SUCC)
        {
        c_errmsg("Error reading input file","finite-read",&one);
        return(E_FAIL);
        }
   
      if ( i == 1)
         {
         *tcount = (long) rbuf[0];
         if (*tcount > MMXPT * 2)
            {
            c_errmsg("Maximum number of points has been reached",
	             "finite-max",&one);
            return(E_FAIL);
            }
         for ( j = 1; j <= nw; j++)
            buf[pp + j - 1] = (double)rbuf[j];
         } /*  if i == 1 */
      else
         {
         for ( j = 0; j < nw; j++)
             buf[pp + j] = (double)rbuf[j];
         }
   
      pp += nw;
      } /*  for i <= nl  */

   /* Free temporary storage. */
   free(rbuf);
   
   status = c_grclos(&fdesc,ha1);

   }  /* if ftype == 1  */
else
   {
   tpldata = (struct TPLDATA *)malloc(sizeof(struct TPLDATA));
   if (tpldata == NULL)
      {
      c_errmsg("Error allocating dynamic memory","finite-alloc",&one);
      return(E_FAIL);
      }
   for (status = E_SUCC, *tcount = 0;status == E_SUCC;)
      {
      status = c_ptio(&ptdesc,tpldata);
      if (status == E_EOF)
         break;
      if (status != E_SUCC)
         return(E_FAIL);
      if ((tpldata->active <= 1) || (tpldata->active > 9))
 	 {
         *buf = tpldata->ref_coord[0];
         *(buf + 1) = tpldata->ref_coord[1];
         *(buf + 2) = tpldata->sea_coord[0];
         *(buf + 3) = tpldata->sea_coord[1];
         buf += 4;
         *tcount += 4;
         }
      }

   free(tpldata);
   }

/*  Close tiepoint location file if it is open
    ------------------------------------------*/
if (strncmp(ha2,"\0",1) != 0)
   {
   status = c_ptclse(&ptdesc);
   if (status != E_SUCC)
      c_errmsg("Error closing input file","finite-close",&one);
   }

   
return(E_SUCC);
} /*  getcfp  */
