/*****************************************************************************
NAME:	getpar

PURPOSE:     Inputs all parameters from TAE (finite)

PROGRAM HISTORY:
VERSION     DATE    AUTHOR      CODE/CONT   REASON
-------     ----    ------      ---------   ------
 1.0       07/98   L.D. Johnson       CSB   Moved calls out of finite.c

ALGORITHM:

*****************************************************************************/

#include "las.h"
#include "util.h"
#include "finite.h"

#ifdef sgi 
#define MXGRSZ 2048
#define GRSZ1 2047.0
#else
#define MXGRSZ 127
#define GRSZ1 126.0
#endif
#define MXGRRED 127
#define GRRED 126.0


extern long fatal;          /* Fatal flag for errmsg                        */
extern long nonfatal;       /* Nonfatal flag for errmsg                     */

void getpar
(
    struct PARBLK *parblk,  /* TAE variable block                           */
    short *zgeom,           /* If radiometric transformation, then true(1)  */
    char outfile[4][CMLEN], /* Host filenames: 0 = output grid              */
                            /*                 1 = finite.pr                */
                            /*                 2 = input space plot file    */
                            /*                 3 = output space plot file   */
    FILE *outfp[4],         /* Output files:   0 = output grid              */
                            /*                 1 = finite.pr                */
                            /*                 2 = input space plot file    */
                            /*                 3 = output space plot file   */
    long *numfile,          /* Number of files opened                       */
    char ha4[CMLEN],        /* Input control point filename (IBIS)          */
    char ha5[CMLEN],        /* Input control point filename (TPL)           */
    long *ftype,            /* Type of input file: 1 = IBIS graphics        */
                            /*                     2 = tie points location  */
    int  window[4],         /* Image window: SL, SS, NL, NS                 */
    double *reject,         /* Tolerance defining maximum separation of     */
                            /* duplicate control points                     */
    double *tol,            /* Grid tolerance value                         */
    long *cnt1,             /* Input plot flag: 0 = don't plot input points */
                            /*                  1 = plot                    */
    long *cnt2,             /* Output plot flag:0 = don't plot output points*/
                            /*                  1 = plot                    */
    char trend[SZSUBCMD],   /* The method applied to calculate the          */
                            /* radiometric difference value for the four    */
                            /* corner points far outside the output image   */
                            /* area.                                        */
                            /* = FIRST, the first order polynomial fit of   */
                            /* all the radiometric difference values        */
                            /* = AVERAGE, the average of all the radiometric*/
                            /*   difference values                          */
                            /* = USER, the user supplies the four corner    */
                            /*   point values                               */
    double corner[4],       /* If trend="USER", the user supplies the four  */
                            /* corner radiometric difference values in the  */
                            /* order: left, lower, upper, right             */
    char *debug,            /* If debug desired, then true(1)               */
    int  *gridnl,           /* Number of lines within a single grid cell    */
    int  *gridns,           /* Number of samples within a single grid cell  */
    double *miny,           /* Minimum sample location in final output space*/
    double *minx,           /* Minimum line location in final output space  */
    double *maxy,           /* Maximum sample location in final output space*/
    double *maxx            /* Maximum line location in final output space  */
)
{
  struct VARIABLE *val;     /* Pointer to parameter in vblock       */
  long prnt[3];             /* c_setprt return flags corresponding to term, */
			    /* file, lp                                     */
  long read_access = IREAD;
  long status;              /* Function return status                       */
  long write_access = IWRITE;
  long zero = 0;            /* Value of zero                                */
  char format[SZSUBCMD];    /* Format for print output: SHORT or LONG       */
  char subcmd[SZSUBCMD];    /* Buffer for subcommand parameter              */
  char tname[CMLEN];        /* LAS print filename returned from c_setprt    */


  val = p_find(parblk, "_SUBCMD");
  strcpy(subcmd, SVAL(*val,0));
  if (!strcmp(subcmd, "GEOMETRIC"))
    *zgeom = 0;
  else
    *zgeom = 1;

  /* Open the input file.
  ----------------------------*/
  status = c_getnam(parblk, "INFILE", &zero, ha4, "ibis_graph", &read_access);
  if (status != E_SUCC)
    cleanup(parblk, outfile, outfp);
  if (!(*zgeom))
    {
    status = c_getnam(parblk,"INTL", &zero, ha5, "tpl", &read_access);
    if (status != E_SUCC)
      cleanup(parblk, outfile, outfp);
    }
  else
    strncpy(ha5, "\0", 1);

  if (!strncmp(ha4, "\0", 1))
    {                            /* no IBIS file entered */
    if (!strncmp(ha5, "\0", 1))
      {                          /* no TPL file entered  */
      c_errmsg("INFILE or INTL must be specified", "finite-file", &nonfatal);
      cleanup(parblk, outfile, outfp);
      }
    *ftype = 2;
    }
  else
    *ftype = 1;

  /* Open the output grid file.
  -----------------------------*/
  status = c_getnam(parblk, "OUTGRID", &zero, outfile[0], "grid",
                    &write_access);
  if (status != E_SUCC)
    cleanup(parblk, outfile, outfp);

  /* Obtain the minumum and maximum line and sample locations
     in output space from parameter block.
  -----------------------------------------------------------*/
  val = p_find(parblk, "GRIDSPEC");
  window[0] = IVAL(*val, 0);
  window[1] = IVAL(*val, 1);
  window[2] = IVAL(*val, 2);
  window[3] = IVAL(*val, 3);
  *miny = (double)window[0];
  *minx = (double)window[1];
  *maxy = (double)(window[0] + window[2] - 1);
  *maxx = (double)(window[1] + window[3] - 1);

  val = p_find(parblk, "DUPRAD");
  *reject = RVAL(*val, 0);
  *reject *= *reject;

  val = p_find(parblk, "TOLVAL");
  *tol = RVAL(*val, 0);

  *numfile = 1;

  strcpy(outfile[1], "finite");
  if (c_setprt(parblk, &prnt[0], &prnt[1], &prnt[2], outfile[1],
      tname) != E_SUCC)
    cleanup(parblk, outfile, outfp);
  if (prnt[1] != 0)
    (*numfile)++;

  if ((outfp[1] = fopen(outfile[1], "w")) == NULL)
    cleanup(parblk, outfile, outfp);

  /* outfile[2] is a plot file of the triangles in input space.
     outfile[3] is a plot file of the triangles in output space.
     outfile[2] is not created if a radiometric grid was requested.
  -----------------------------------------------------------------*/
  *cnt2 = 0;
  *cnt1 = 0;

  if (!(*zgeom))
    {
    status = c_getnam(parblk, "INPLOT", &zero, outfile[2], "plot",
                      &write_access);
    if (status != E_SUCC)
      cleanup(parblk, outfile, outfp);
    if (strncmp(outfile[2], "\0", 1) != 0)
      {
      if ((outfp[2] = fopen(outfile[2], "w")) == NULL)
        {
        c_errmsg ("Cannot open PLOTIN file", "finite-open", &nonfatal);
        cleanup(parblk, outfile, outfp);
        }
      *cnt1 = 1;
      (*numfile)++;
      }
    }

  status = c_getnam(parblk, "OUTPLOT", &zero, outfile[3], "plot",
                    &write_access);
  if (status != E_SUCC)
    cleanup(parblk, outfile, outfp);

  if (strncmp(outfile[3], "\0", 1) != 0)
    {
    if ((outfp[3] = fopen(outfile[3], "w")) == NULL)
      {
      c_errmsg ("Cannot open PLOTOUT file", "finite-open", &nonfatal);
      cleanup(parblk, outfile, outfp);
      }
    *cnt2 = 1;
    (*numfile)++;
    }

  /* Get radiometric trend option for four outside corner points.
  ---------------------------------------------------------------*/
  if (*zgeom) 
    {
    val = p_find(parblk, "CORNOPT");
    strcpy(trend, SVAL(*val,0));
    if (!strcmp(trend, "USER"))
      {
      val = p_find(parblk, "CORNVALS");
      corner[0] = RVAL(*val, 0);
      corner[1] = RVAL(*val, 1);
      corner[2] = RVAL(*val, 2);
      corner[3] = RVAL(*val, 3);
      }
    }

  /* ptest is used to test the values in the input parameter block parblk
     and set several logical parameters based on the input.
  -----------------------------------------------------------------------*/
  val = p_find(parblk, "OUTFORM");
  strcpy(format, SVAL(*val,0));
  if (!strcmp(format, "SHORT"))
    *debug = 0;
  else
    *debug = 1;

  /* Obtain the number of rows and columns of grid from parameter block.
  ----------------------------------------------------------------------*/
  *gridnl = 0;
  val = p_find(parblk, "NLGRID");
  if (val->v_count > 0)
  {
      *gridnl = IVAL(*val, 0);
      if (*gridnl < 1)
      {
          c_lasmsg(FALSE,"finite-invalid","NLGRID must be greater than zero");
          cleanup(parblk, outfile, outfp);
      }
  }
  
  *gridns = 0;
  val = p_find(parblk, "NSGRID");
  if (val->v_count > 0)
  {
      *gridns = IVAL(*val, 0);
      if (*gridns < 1)
      {
          c_lasmsg(FALSE,"finite-invalid","NSGRID must be greater than zero");
          cleanup(parblk, outfile, outfp);
      }
  }
}
