/*    ..................................................................

        FUNCTION LLSQD

        PURPOSE
           TO SOLVE LINEAR LEAST SQUARES PROBLEMS, I.E. TO MINIMIZE
           THE EUCLIDEAN NORM OF B-A*X, WHERE A IS A M BY N MATRIX
           WITH M NOT LESS THAN N. IN THE SPECIAL CASE M=N SYSTEMS OF
           LINEAR EQUATIONS MAY BE SOLVED.

        USAGE
           IER = LLSQD (A,B,M,N,L,X,IPIV,EPS,AUX)

        DESCRIPTION OF PARAMETERS
           A      - M BY N COEFFICIENT MATRIX (DESTROYED).
           B      - M BY L RIGHT HAND SIDE MATRIX (DESTROYED).
           M      - ROW NUMBER OF MATRICES A AND B.
           N      - COLUMN NUMBER OF MATRIX A, ROW NUMBER OF MATRIX X.
           L      - COLUMN NUMBER OF MATRICES B AND X.
           X      - N BY L SOLUTION MATRIX.
           IPIV   - INTEGER OUTPUT VECTOR OF DIMENSION N WHICH
                    CONTAINS INFORMATIONS ON COLUMN INTERCHANGES
                    IN MATRIX A. (SEE REMARK NO.3).
           EPS    - INPUT PARAMETER WHICH SPECIFIES A RELATIVE
                    TOLERANCE FOR DETERMINATION OF RANK OF MATRIX A.
           IER    - A RESULTING ERROR PARAMETER.
           AUX    - AUXILIARY STORAGE ARRAY OF DIMENSION MAX(2*N,L).
                    ON RETURN FIRST L LOCATIONS OF AUX CONTAIN THE
                    RESULTING LEAST SQUARES.

        REMARKS
           (1) NO ACTION BESIDES ERROR MESSAGE IER=-2 IN CASE
               M LESS THAN N.
           (2) NO ACTION BESIDES ERROR MESSAGE IER=-1 IN CASE
               OF A ZERO-MATRIX A.
           (3) IF RANK K OF MATRIX A IS FOUND TO BE LESS THAN N BUT
               GREATER THAN 0, THE PROCEDURE RETURNS WITH ERROR CODE
               IER=K INTO CALLING PROGRAM. THE LAST N-K ELEMENTS OF
               VECTOR IPIV DENOTE THE USELESS COLUMNS IN MATRIX A.
               THE REMAINING USEFUL COLUMNS FORM A BASE OF MATRIX A.
           (4) IF THE PROCEDURE WAS SUCCESSFUL, ERROR PARAMETER IER
               IS SET TO 0.

        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
           NONE

        METHOD
           HOUSEHOLDER TRANSFORMATIONS ARE USED TO TRANSFORM MATRIX A
           TO UPPER TRIANGULAR FORM. AFTER HAVING APPLIED THE SAME
           TRANSFORMATION TO THE RIGHT HAND SIDE MATRIX B, AN
           APPROXIMATE SOLUTION OF THE PROBLEM IS COMPUTED BY
           BACK SUBSTITUTION. FOR REFERENCE, SEE
           G. GOLUB, NUMERICAL METHODS FOR SOLVING LINEAR LEAST
           SQUARES PROBLEMS, NUMERISCHE MATHEMATIK, VOL.7,
           ISS.3 (1965), PP.206-216.

     ..................................................................*/

#include <math.h>

long llsqd(double *a, double *b, long m, long n, long l, double *x,
           long *ipiv, double eps, double *aux)
{
  double h, piv, sig, tol, beta;
  long iend, k, ist, kpiv, id, ii, jst, lm, kst, j, ln, i;

  /* Error test
  -------------*/
  if (m-n < 0)
    return(-2);       /* m < n */

  /* Generation of initial vector s[k] (k=0,1,...,n-1) in storage
     locations aux[k] (k=0,1,...,n-1).
  ---------------------------------------------------------------*/
  piv = 0.;
  iend = 0;
  for (k=0; k<n; k++)
    {
    ipiv[k] = k;
    h = 0.;
    ist = iend;
    iend += m;
    for (i=ist; i<iend; i++)
      h += a[i]*a[i];
    aux[k] = h;
    if(h-piv > 0)
      {
      piv = h;
      kpiv = k;
      }
    }

  /* Error test
  -------------*/
  if(piv <= 0.)
    return(-1);           /* zero-matrix a */

  /* Define tolerance for checking rank of a.
  -------------------------------------------*/
  sig = sqrt(piv);
  tol = sig*fabs(eps);

  /* Decomposition loop
  ---------------------*/
  lm = l*m;
  ist = -m - 1;
  for (k=0; k<n; k++)
    {
    ist += m + 1;
    iend = ist + m - k;
    i = kpiv - k;
    if (i > 0)
      {
      /* Interchange k-th column of a with kpiv-th in case kpiv > k.
      --------------------------------------------------------------*/
      h = aux[k];
      aux[k] = aux[kpiv];
      aux[kpiv] = h;
      id = i*m;
      for (i=ist; i<iend; i++)
        {
        j = i + id;
        h = a[i];
        a[i] = a[j];
        a[j] = h;
        }
      }

    /* Computation of parameter sig.
    --------------------------------*/
    if(k > 0)
      {
      sig = 0.;
      for (i=ist; i<iend; i++)
        sig += a[i]*a[i];
      sig = sqrt(sig);

      /* Test on singularity.
      -----------------------*/
      if(sig-tol <= 0)
        return(k);      /* Rank of matrix a less than n. */
      }

    /* Generate correct sign of parameter sig.
    ------------------------------------------*/
    h = a[ist];
    if(h < 0)
      sig = -sig;


    ipiv[kpiv] = ipiv[k];
    ipiv[k] = kpiv;

    /* Generation of vector uk in k-th column of matrix a and of
       parameter beta.
    ------------------------------------------------------------*/
    beta = h + sig;
    a[ist] = beta;
    beta = 1./(sig*beta);
    j = n + k;
    aux[j] = -sig;
    if(k-n+1 < 0)
      {
      /* Transformation of matrix a.
      ------------------------------*/
      piv = 0.;
      id = 0;
      jst = k + 1;
      kpiv = jst;
      for (j=jst; j<n; j++)
        {
        id += m;
        h = 0.;
        for (i=ist; i<iend; i++)
          {
          ii = i + id;
          h += a[i]*a[ii];
          }
        h *= beta;
        for (i=ist; i<iend; i++)
          {
          ii = i + id;
          a[ii] -= a[i]*h;
          }

        /* Updating of element s[j] stored in location aux[j].
        ------------------------------------------------------*/
        ii = ist + id;
        h = aux[j] - a[ii]*a[ii];
        aux[j] = h;
        if(h-piv > 0)
          {
          piv = h;
          kpiv = j;
          }
        }
      }

    /* Transformation of right hand side matrix b.
    ----------------------------------------------*/
    for (j=k; j<lm; j+=m)
      {
      h = 0.;
      iend = j + m - k;
      ii = ist;
      for (i=j; i<iend; i++)
        {
        h += a[ii]*b[i];
        ii++;
        }
      h *= beta;
      ii = ist;
      for (i=j; i<iend; i++)
        {
        b[i] -= a[ii]*h;
        ii++;
        }
      }
    }

  /* End of decomposition loop.

     Back substitution and back interchange.
  ------------------------------------------*/
  i = n - 1;
  ln = l*n;
  piv = 1./aux[2*n-1];
  for (k=n-1; k<ln; k+=n)
    {
    x[k] = piv*b[i];
    i += m;
    }
  if(n-1 > 0)
    {
    jst = (n - 1)*m + n - 1;
    for (j=1; j<n; j++)
      {
      jst -= m + 1;
      k = n + n - 1 - j;
      piv = 1./aux[k];
      kst = k - n;
      id = ipiv[kst] - kst;
      ist = -j;
      for (k=0; k<l; k++)
        {
        h = b[kst];
        ist += n;
        iend = ist + j;
        ii = jst;
        for (i=ist; i<iend; i++)
          {
          ii += m;
          h -= a[ii]*x[i];
          }
        i = ist - 1;
        ii = i + id;
        x[i] = x[ii];
        x[ii] = piv*h;
        kst += m;
        }
      }
    }

  /* Computation of least squares.
  --------------------------------*/
  ist = n;
  iend = 0;
  for (j=0; j<l; j++)
    {
    iend += m;
    h = 0.;
    if(m-n > 0)
      {
      for (i=ist; i<iend; i++)
        h += b[i]*b[i];
      ist += m;
      }
    aux[j] = h;
    }

  return(0);
}
