/****************************************************************************
NAME:				mkggrd

PURPOSE:  Create a geometric grid.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  1.0	 01/99	L. Johnson    CSB      Header added.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  Run under TAE

PROJECT:  LAS

ALGORITHM:

  Build the grid using the c_makegrid library function, providing the 
    proper output to input mapping function
  Place the needed information into the correct fields of the GEOGRID struct.
  Write the grid to a file.

*****************************************************************************/

#include "las.h"
#include "geompak.h"
#include "lams.h"
#include "finite.h"

extern long nonfatal;

typedef struct
{
    double (*coeff)[6]; /* coefficients for all the triangles */
    int num_triangles;  /* total number of triangles */
    int last_triangle;  /* last triangle found - this is an attempt at
                           optimization by guessing that the last triangle
                           found will be close to the next one requested */
    short *con1;        /* triangle parameters */
    short *con2;
    short *con3;
    double *rpar;
} OLS2ILS_PARAMS;

static int ols2ils
(
    void *data,     /* I: pointer to a OLS2ILS_PARAMS structure for the 
                          parameters specific to this gridding application */
    int oline,      /* I: output space line */
    int osamp,      /* I: output space sample */
    double *iline,  /* O: input space line */
    double *isamp   /* O: input space sample */
)
{
    double x = (double)osamp;
    double y = (double)oline;
    OLS2ILS_PARAMS *ols2ils_data = (OLS2ILS_PARAMS *)data;
    double (*coeff)[6] = ols2ils_data->coeff;
    int num_triangles = ols2ils_data->num_triangles;
    int inside;
    int isign;
    int center_triangle;  /* center triangle in search */
    int search;
    int triangle;
    short *con1 = ols2ils_data->con1;
    short *con2 = ols2ils_data->con2;
    short *con3 = ols2ils_data->con3;
    double *rpar = ols2ils_data->rpar;

    /* find the triangle containing this point in output space */
    isign = -1;
    center_triangle = ols2ils_data->last_triangle - 1 + num_triangles;
    inside = FALSE;
    for (search = 1; search <= num_triangles; search++)
    {
        /* perform the search around the center triangle going under the 
           assumption that the last triangle found will be close to the 
           next one */
        triangle = ((center_triangle + (search / 2) * isign) % num_triangles) 
                   + 1;
        isign = -1 * isign;

        if (coeff[triangle - 1][0] != 1.E35)
        {
            /* valid coefficients, so see if the point is in the triangle */
            if (insided(y,x,triangle - 1,con1,con2,con3,rpar)) 
            {
               inside = TRUE;
               break;
            }
        }
    }
    if (!inside)
    {
         c_errmsg("Grid intersection is not in a triangle",
        		  "finite-grid",&nonfatal);
         return (E_FAIL);
    }
    
    /* calculate the input space point using the coefficients for the 
       triangle found to contain this point in output space */
    *iline = coeff[triangle - 1][0] * y + coeff[triangle - 1][1] * x + 
             coeff[triangle - 1][2];
    *isamp = coeff[triangle - 1][3] * y + coeff[triangle - 1][4] * x + 
             coeff[triangle - 1][5];
     
    /* update the last triangle in the tracking structure with the triangle 
       used */
    ols2ils_data->last_triangle = triangle;

    return (E_SUCC);
}


int c_mkggrd
(
    struct GEOGRID *in_grid,
    char *hname,
    int  window[4],
    int  *gridns,           /* I/O: samples per grid cell */
    int  *gridnl,           /* I/O: lines per grid cell */
    double coeff[][6],
    short *con1,
    short *con2,
    short *con3,
    double *rpar,
    double tol,
    long ntri,
    long *grid_rows,        /* O: rows in the created grid */
    long *grid_cols         /* O: columns in the created grid */
)
{
struct GEOGRID  *grid;

long one = 1; 			/*  the value of one			*/
long status;
OLS2ILS_PARAMS ols2ils_data;    /* parameters to ols2ils */

grid = in_grid;

/* prepare for the c_makegrid call */
ols2ils_data.coeff = coeff;
ols2ils_data.num_triangles = ntri;
ols2ils_data.last_triangle = 1;
ols2ils_data.con1 = con1;
ols2ils_data.con2 = con2;
ols2ils_data.con3 = con3;
ols2ils_data.rpar = rpar;

/* construct the grid */
if (c_makegrid(window, *gridnl, *gridns, ols2ils, &ols2ils_data, tol, "FINITE",
               0, grid) != E_SUCC)
{
    c_lasmsg(FALSE,"makegrid-fail","Error returned from c_makegrid");
    return (E_FAIL);
}

grid->grid_tol = tol;
strcpy(grid->model,"FINITE ELEMENT");

/* update the output parameters */
*gridnl = grid->out_lines[1] - grid->out_lines[0];
*gridns = grid->out_samps[1] - grid->out_samps[0];
*grid_rows = grid->nrows;
*grid_cols = grid->ncols;

/*  Write the grid to a file
----------------------------*/
status = c_putgrd(hname,grid);
if (status != E_SUCC)
   {
   c_errmsg("Error encountered writing to grid file","finite-write",&one);
   free(grid->out_lines); 
   free(grid->out_samps); 
   free(grid->in_lines);  
   free(grid->in_samps); 
   return(E_FAIL);
   }
 
free(grid->out_lines); 
free(grid->out_samps); 
free(grid->in_lines);  
free(grid->in_samps); 

return(E_SUCC);
} 
