/****************************************************************************
NAME:				mkrgrd

PURPOSE:  Create a radiometric grid.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  1.0	 01/99	L. Johnson    CSB      Header added.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  Run under TAE

PROJECT:  LAS

ALGORITHM:

  Build the grid using the c_makegrid library function, providing the 
    proper output to input mapping function
  Setup the RADGRID for writing to the file
  Place the needed information into the correct fields of the RADGRID struct.
  Calculate the output lines and samples of the grid.

*****************************************************************************/

#include "las.h"
#include "geompak.h"
#include "radpak.h"
#include "lams.h"
#include "finite.h"

extern long nonfatal;

typedef struct
{
    double (*coeff)[6]; /* coefficients for all the triangles */
    int num_triangles;  /* total number of triangles */
    int last_triangle;  /* last triangle found - this is an attempt at
                           optimization by guessing that the last triangle
                           found will be close to the next one requested */
    short *con1;        /* triangle parameters */
    short *con2;
    short *con3;
    double *rpar;
} OLS2ILS_PARAMS;

static int ols2ils
(
    void *data,     /* I: pointer to a OLS2ILS_PARAMS structure for the 
                          parameters specific to this gridding application */
    int oline,      /* I: output space line */
    int osamp,      /* I: output space sample */
    double *iline,  /* O: input space line */
    double *isamp   /* O: input space sample */
)
{
    double x = (double)osamp;
    double y = (double)oline;
    OLS2ILS_PARAMS *ols2ils_data = (OLS2ILS_PARAMS *)data;
    double (*coeff)[6] = ols2ils_data->coeff;
    int num_triangles = ols2ils_data->num_triangles;
    int inside;
    int isign;
    int center_triangle;  /* center triangle in search */
    int search;
    int triangle;
    short *con1 = ols2ils_data->con1;
    short *con2 = ols2ils_data->con2;
    short *con3 = ols2ils_data->con3;
    double *rpar = ols2ils_data->rpar;

    /* find the triangle containing this point in output space */    
    isign = -1;
    center_triangle = ols2ils_data->last_triangle - 1 + num_triangles;
    inside = FALSE;
    for (search = 1; search <= num_triangles; search++)
    {
        /* perform the search around the center triangle going under the 
           assumption that the last triangle found will be close to the 
           next one */
        triangle = ((center_triangle + (search / 2) * isign) % num_triangles) 
                   + 1;
        isign = -1 * isign;

        if (coeff[triangle - 1][0] != 1.E35)
        {
            /* valid coefficients, so see if the point is in the triangle */
            if (insided(y,x,triangle - 1,con1,con2,con3,rpar)) 
            {
               inside = TRUE;
               break;
            }
        }
    }
    if (!inside)
    {
         c_errmsg("Grid intersection is not in a triangle",
        		  "finite-grid",&nonfatal);
         return (E_FAIL);
    }
      
    /* treat the line as the z value */
    *iline = coeff[triangle - 1][0] * y + coeff[triangle - 1][1] * x + 
             coeff[triangle - 1][2];
    *isamp = 0.0;

    /* update the last triangle in the tracking structure with the triangle 
       used */
    ols2ils_data->last_triangle = triangle;

    return (E_SUCC);
}


int c_mkrgrd
(
    char *hname,
    int  window[4],
    long *grid_rows,
    long *grid_cols,
    int  *gridns,
    int  *gridnl,
    double coeff[][6],
    short *con1,
    short *con2,
    short *con3,
    double *rpar,
    double tol,
    long   ntri,
    double *maxg,
    double *maxgy,
    double *maxgx,
    double *ming,
    double *mingy,
    double *mingx
)

{  /*  mkrgrd  */

struct RADGRID  radgrid;
struct GEOGRID  grid;
double t_value;

long status;			/*  status of function calls		*/
int row, col;
OLS2ILS_PARAMS ols2ils_data;    /* parameters to ols2ils */

/* zero out the grid */
memset (&grid,0,sizeof(grid));

/* prepare for the c_makegrid call */
ols2ils_data.coeff = coeff;
ols2ils_data.num_triangles = ntri;
ols2ils_data.last_triangle = 1;
ols2ils_data.con1 = con1;
ols2ils_data.con2 = con2;
ols2ils_data.con3 = con3;
ols2ils_data.rpar = rpar;

if (c_makegrid(window, *gridnl, *gridns, ols2ils, &ols2ils_data, tol, 
               "FINITE",0,&grid) != E_SUCC)
{
    c_lasmsg(FALSE,"makegrid-fail","Error returned from c_makegrid");
    return (E_FAIL);
}

/* Place the needed information into the correct fields of the GEOGRID struct
-----------------------------------------------------------------------------*/
grid.ncoeff = 0;
grid.grid_tol = tol;
strcpy(grid.model,"FINITE ELEMENT");

/* find the minimum and maximum radiometric shift in the grid */
for (row = 0; row < grid.nrows; row++)
{
    for (col = 0; col < grid.ncols; col++)
    {
        t_value = grid.in_lines[row * grid.ncols + col];
        if (t_value < *ming)
        {
            *ming = t_value;
            *mingx = grid.out_samps[col];
            *mingy = grid.out_lines[row];
        }
        if (t_value > *maxg)
        {
            *maxg = t_value;
            *maxgx = grid.out_samps[col];
            *maxgy = grid.out_lines[row];
        }
    }
}

/* update the output parameters */
*gridnl = grid.out_lines[1] - grid.out_lines[0];
*gridns = grid.out_samps[1] - grid.out_samps[0];
*grid_rows = grid.nrows;
*grid_cols = grid.ncols;

/* convert geogrid to radgrid */
radgrid.fiterr_valid = grid.fiterr_valid;
radgrid.griderr_valid = grid.griderr_valid;
radgrid.ncoeff = grid.ncoeff;
radgrid.nrows = grid.nrows;
radgrid.ncols = grid.ncols;
radgrid.lines = grid.lines;
radgrid.samples = grid.samples;
radgrid.out_lines = grid.out_lines;
radgrid.out_samps = grid.out_samps;
radgrid.coeffs = grid.coeffs;
radgrid.max_grid_err[0] = grid.max_grid_err[0];
radgrid.max_grid_err[1] = grid.max_grid_err[1];
radgrid.ave_grid_err[0] = grid.ave_grid_err[0];
radgrid.ave_grid_err[1] = grid.ave_grid_err[1];
radgrid.rms_grid_err = grid.rms_grid_err;
radgrid.act_grid_err[0] = grid.act_grid_err[0];
radgrid.act_grid_err[1] = grid.act_grid_err[1];
radgrid.grid_tol = grid.grid_tol;
radgrid.max_fit_err[0] = grid.max_fit_err[0];
radgrid.max_fit_err[1] = grid.max_fit_err[1];
radgrid.ave_fit_err[0] = grid.ave_fit_err[0];
radgrid.ave_fit_err[1] = grid.ave_fit_err[1];
radgrid.rms_fit_err = grid.rms_fit_err;
radgrid.z_values = grid.in_lines;
strcpy(radgrid.model,grid.model);

/* write out grid */
status = c_putrgd(hname,&radgrid);
if (status != E_SUCC)
   {
   c_errmsg("Error encountered writing to grid file","finite-write",&nonfatal);
   return(E_FAIL);
   }

/* Test code to write out a normal geometric grid containing the same 
   information as a radiometric grid so it can be examined with dspgrid 
{
char testgrid[CMLEN];
strcpy(testgrid,hname);
strcat(testgrid,"geo");
status = c_putgrd(testgrid,&grid);
}
*/

free(grid.out_lines); 
free(grid.out_samps); 
free(grid.in_lines);  
free(grid.in_samps); 
 
return(E_SUCC);
}
