/*****************************************************************************
FUNCTION:	SOLVE

PURPOSE:     Solve for the coefficients associated with the input triangles.

PROGRAM HISTORY:
  Version   Date     Author      Code/Conr  Request
  -------   ----     ------      ---------  -------
   1.0	   01/99    L.Johnson	   CSB	    Extracted from finite.c
   1.1     04/99    L.Johson       CSB      SR2613 Added missing braces.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:	LAS(LAMS)

ALGORITHM:

  For all triangles do
      Construct a working space of triangle vertices (work).
      Factor the first order polynomial fit matrix of the triangle.
      Solve for the first order polynomial fit of the triangle.
      If debug then
          Print the coefficients for the first order fit of the triangle
      end if
  end do

*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "finite.h"

void solve
(
    double *rpar,           /* Array containing control point locations.    */
                            /* If geometric transformation requested, the   */
                            /* order is input line,input sample, output     */
                            /* line, output sample.  If radiometric         */
                            /* transformation output line, output sample,   */
                            /* radiometric difference.                      */
    double coeff[][6],      /* Coefficients of polynomial.                  */
    short  *con1,           /* Pointers to nodes of triangle                */ 
    short  *con2,           /* Pointers to nodes of triangle                */ 
    short  *con3,           /* Pointers to nodes of triangle                */
    long   ntri,            /* Number of triangles in triangulation         */
    FILE   *outfp,          /* Output file finite.pr                        */
    char   debug,           /* If debug desired, then true(1)               */
    short  zgeom,           /* If radiometric transformation, then true(1)  */
    long   npoint           /* Defaulted to 4 for pointing into rpar for a  */
                            /* geometric transformation.  Reset to 3 for    */
                            /* radiometric transformation.                  */
)
{
  double eps = 1.e-14;      /* Very small number                            */
  double tab[6];            /* If geometric transformation, the input       */
			    /* locations of the triangle vertices.  If      */
			    /* radiometric transformation, the radiometric  */
			    /* differences at the triangle vertices.        */
  double work[9];           /* Work space for the computation of the first  */
                            /* order fit of the vertices of the triangles.  */
  double pivi[3];           /* Array of inverse pivot values.               */
  long ier;                 /* Error flag returned from matrix routines     */
  long ipiv[3];             /* Array of pivot row numbers.                  */
  long ii, jj;              /* Loop counter                                 */
  long nrank = 3;           /* Number of coefficients in second order       */
                            /* polynomial                                   */
  long one = 1;             /* The value of one                             */
  long ptr;                 /* Array pointer                                */


  ptr = 0;
  if (debug)
    {
    fprintf(outfp, "\n                   LINE,SAMPLE VERTICES OF THE ");
    fprintf(outfp, "TRIANGLES\n");
    }
  if (!zgeom)
    ptr += 2;
  for (ii=0; ii<ntri; ii++)
    {
    con1[ii] = (short)(ptr + (con1[ii] - 1)*npoint);
    con2[ii] = (short)(ptr + (con2[ii] - 1)*npoint);
    con3[ii] = (short)(ptr + (con3[ii] - 1)*npoint);
    work[0]  = rpar[con1[ii]];
    work[1]  = rpar[con2[ii]];
    work[2]  = rpar[con3[ii]];
    work[3]  = rpar[con1[ii]+1];
    work[4]  = rpar[con2[ii]+1];
    work[5]  = rpar[con3[ii]+1];
    work[6]  = 1.;
    work[7]  = 1.;
    work[8]  = 1.;

    /* dgefa factors the first order polynomial fit matrix of the triangle.
    -----------------------------------------------------------------------*/
    ier = dgefa(work, nrank, pivi, ipiv, eps);
    if (ier != 0)
      {
      coeff[ii][0] = 1.e35;
      continue;
      }

    if (!zgeom)
      {
      /* Geometric transformation:
         work[0-2] contains the output line locations of a triangle.
         work[3-5] contains the output sample locations of a triangle.
         tab[0-5] contains the input line, sample locations of a triangle.
      --------------------------------------------------------------------*/
      tab[0] = rpar[con1[ii]-2];
      tab[1] = rpar[con2[ii]-2];
      tab[2] = rpar[con3[ii]-2];
      tab[3] = rpar[con1[ii]-1];
      tab[4] = rpar[con2[ii]-1];
      tab[5] = rpar[con3[ii]-1];

      if (debug)
        {
        fprintf(outfp, "%5d   %8.1f %8.1f      %8.1f %8.1f     ",
               ii+1, tab[0], tab[3], tab[1], tab[4]);
        fprintf(outfp, " %8.1f %8.1f\n", tab[2], tab[5]);
        }

      ier = dgeso(&tab[3], work, nrank, one, pivi, ipiv);
      }
    else
      {
      /* Radiometric transformation:
         tab[0-2] contains the radiometric difference values
              at the vertices.
         work[0-5] contains the output line, sample locations
              of the vertices.
      --------------------------------------------------------*/
      tab[0] = rpar[con1[ii]+2];
      tab[1] = rpar[con2[ii]+2];
      tab[2] = rpar[con3[ii]+2];
      if (debug)
        {
        fprintf(outfp, "%5d   %7.1f %7.1f      %7.1f %7.1f      ",
               ii+1, work[0], work[3], work[1], work[4]);
        fprintf(outfp, "%7.1f %7.1f\n", work[2], work[5]);
        }
      }

    /* dgeso solves for the first order polynomial fit of the triangle.
       On return, tab contains the coefficients.
    -------------------------------------------------------------------*/
    ier = dgeso(tab, work, nrank, one, pivi, ipiv);
    for (jj=0; jj<3; jj++)
      {
      coeff[ii][jj] = tab[jj];
      if (!zgeom)
        coeff[ii][jj+3] = tab[jj+3];
      }
    }
  if (debug)
    {
    if (!zgeom)
      {
      fprintf(outfp, "\n  TRIANGLE      LINE COEFFICIENTS");
      fprintf(outfp, "                 SAMPLE COEFFICIENTS\n");
      for (ii=0; ii<ntri; ii++)
        fprintf(outfp, "%5d   %9.3f %9.3f %9.3f      %9.3f %9.3f %9.3f\n",
                ii+1, coeff[ii][0], coeff[ii][1], coeff[ii][2],
                      coeff[ii][3], coeff[ii][4], coeff[ii][5]);
      }
    else
      {
      fprintf(outfp, "\n  TRIANGLE      DELTA Z COEFFICIENTS\n");
      for (ii=0; ii<ntri; ii++)
        fprintf(outfp, "%5d   %9.3f %9.3f %9.3f\n",
                ii+1, coeff[ii][0], coeff[ii][1], coeff[ii][2]);
      }
    }
}
