/****************************************************************************
NAME:				tri

PURPOSE:  Triangulate a set of input points using Voronoi triangulation.

PROGRAM HISTORY:
VERSION	  DATE     AUTHOR     CODE/CONT  REASON
-------	  ----     ------     ---------  -----------------------------
  1.0    01/99 	L. Johnson       CSB     Modified from main routine to
                                         subroutine for use with FINITE.
  1.1    04/99  L. Johnson       CSB     SR2613  Fixed error in indices for
                                         vertices listed after duplicates.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  None

PROJECT:  LAS


  The author of this software is Steven Fortune.  Copyright (c) 1994 by AT&T
  Bell Laboratories.
  Permission to use, copy, modify, and distribute this software for any
  purpose without fee is hereby granted, provided that this entire notice
  is included in all copies of any software which is or includes a copy
  or modification of this software and in all copies of the supporting
  documentation for such software.
  THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTY.  IN PARTICULAR, NEITHER THE AUTHORS NOR AT&T MAKE ANY
  REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
  OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
 
*****************************************************************************/

#include <stdio.h>
#include "defs.h"
#include "las.h"
#include "finite.h"


extern long nonfatal;
static int		siteidx;
static struct	Site	*sites;
static int     nsites;

/* Triangulate a given set of xy points using Delaunay triangulation.  Ignore
 * any points that are duplicates (to within the given value, reject).
-----------------------------------------------------------------------------*/
int tri_(int *npts, long *nedge, long *ntri, double *xpt, double *ypt,
              short *n1, short *n2, short *n3, double *reject, long *numrej,
              double *rejpt, long *abendl) 
{	
  int ii, jj, *edges;
  int ne1, ne2, ne3; /* New edge flags: 1 = new edge, */
                     /*                 0 = previously counted edge. */
  long status;
  struct Site *(*next)();
  double xmin, xmax, ymin, ymax;

  /* Create the storage structure for the vertices.
  -------------------------------------------------*/
  status = setsites(npts, xpt, ypt, reject, numrej, rejpt, abendl,
                    &xmin, &xmax, &ymin, &ymax);
  if (status == E_FAIL)
    return(E_FAIL);

  /* If there are duplicate points, abort if so directed by the user.
  --------------------------------------------------------------------*/
  if (*abendl && (*numrej > 0))
    return(E_FAIL);

  next = nextone;

  siteidx = 0;

  /* Create the triangles.
  ------------------------*/
  voronoi(next, ntri, n1, n2, n3, xmin, xmax, ymin, ymax, nsites); 

  /* Compute the number of edges.
  -------------------------------*/
  if ((edges = (int *)malloc(6*(*ntri)*sizeof(int))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","finite-alloc", &nonfatal);
    return(E_FAIL);
    }
  *nedge = 0;
  for (ii=0; ii<*ntri; ii++)
    {
    ne1 = 1;
    ne2 = 1;
    ne3 = 1;
    for (jj=0; jj<*nedge; jj++)
      {
      /* If a triangle edge matches any previously recorded edge, move on.
         Otherwise, record it.
      --------------------------------------------------------------------*/
      if ((ne1 == 1) &&
          ((edges[2*jj] == n1[ii]) || (edges[2*jj] == n2[ii])) &&
          ((edges[2*jj+1] == n1[ii]) || (edges[2*jj+1] == n2[ii])))
          ne1 = 0;
      else if ((ne2 == 1) &&
               ((edges[2*jj] == n2[ii]) || (edges[2*jj] == n3[ii])) &&
               ((edges[2*jj+1] == n2[ii]) || (edges[2*jj+1] == n3[ii])))
          ne2 = 0;
      else if ((ne3 == 1) &&
               ((edges[2*jj] == n1[ii]) || (edges[2*jj] == n3[ii])) &&
               ((edges[2*jj+1] == n1[ii]) || (edges[2*jj+1] == n3[ii])))
          ne3 = 0;

      /* All three edges have been found to match existing edges, so no new
         edges.
      ---------------------------------------------------------------------*/
      if ((ne1 == 0) && (ne2 == 0) && (ne3 == 0))
        break;
      }
    if (ne1 == 1)
      {
      edges[2*(*nedge)] = n1[ii];
      edges[2*(*nedge)+1] = n2[ii];
      (*nedge)++;
      }
    if (ne2 == 1)
      {
      edges[2*(*nedge)] = n2[ii];
      edges[2*(*nedge)+1] = n3[ii];
      (*nedge)++;
      }
    if (ne3 == 1)
      {
      edges[2*(*nedge)] = n3[ii];
      edges[2*(*nedge)+1] = n1[ii];
      (*nedge)++;
      }
    }
  free(edges);

  return(E_SUCC);
}


/* Sort points on y, then x, coord.
---------------------------------- */
int scomp(const void *s1_in, const void *s2_in)
{
  const struct Point *s1 = (const struct Point *)s1_in;
  const struct Point *s2 = (const struct Point *)s2_in;

  if(s1->y < s2->y)
    return(-1);
  if(s1->y > s2->y)
    return(1);
  if(s1->x < s2->x)
    return(-1);
  if(s1->x > s2->x)
    return(1);
  return(0);
}


/* Return a single in-storage point.
-----------------------------------*/
struct Site *nextone
(void)
{
  struct Site *s;
  if(siteidx < nsites)
    {
    s = &sites[siteidx];
    siteidx++;
    return(s);
    }
  else
    return((struct Site *)NULL);
}


/* Read all points, sort, and compute xmin, xmax, ymin, ymax.
-------------------------------------------------------------*/
int setsites(int *npts, double *xpt, double *ypt, double *reject,
             long *numrej, double *rejpt, long *abendl, double *xmin_out, 
             double *xmax_out, double *ymin_out, double *ymax_out) 
{
  int i, j, rej, rejflag;

  double dx, dy, dist;
  double xmin, xmax;


  if ((sites = (struct Site *)malloc(*npts*sizeof(*sites))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","finite-alloc", &nonfatal);
    return(E_FAIL);
    }
  nsites = 0;
  rej = 0;
  *numrej = 0;
  for (i=0; i<*npts; i++)
    {
    rejflag = 0;
    for (j=0; j<i; j++)
      {
      dx = xpt[i] - xpt[j];
      dy = ypt[i] - ypt[j];
      dist = dx*dx + dy*dy;
      if (dist <= *reject)
        {
        rejpt[rej] = xpt[i];
        rejpt[rej+1] = ypt[i];
        rej += 2;
        (*numrej)++;
        rejflag = 1;
        if (*abendl)
          return(E_FAIL);
        break;
        }
      }
    if (rejflag == 0)
      {
      sites[nsites].coord.x = xpt[i];
      sites[nsites].coord.y = ypt[i];
      sites[nsites].sitenbr = i + 1;
      sites[nsites].refcnt = 0;
      nsites++;
      }
    }
  qsort(sites, nsites, sizeof(*sites), scomp);
  xmin=sites[0].coord.x; 
  xmax=sites[0].coord.x;
  for(i=1; i<nsites; i++)
    {
    if(sites[i].coord.x < xmin)
      xmin = sites[i].coord.x;
    if(sites[i].coord.x > xmax)
      xmax = sites[i].coord.x;
    }
  *xmin_out = xmin;
  *xmax_out = xmax;
  *ymin_out = sites[0].coord.y;
  *ymax_out = sites[nsites-1].coord.y;

  return(E_SUCC);
}
