/****************************************************************************
NAME:				voronoi

PURPOSE:  Apply Voronoi algorithm for facetization.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  1.0	 01/99	L. Johnson    CSB      Minor modifications for LAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  None

PROJECT:  LAS


  The author of this software is Steven Fortune.  Copyright (c) 1994 by AT&T
  Bell Laboratories.
  Permission to use, copy, modify, and distribute this software for any
  purpose without fee is hereby granted, provided that this entire notice
  is included in all copies of any software which is or includes a copy
  or modification of this software and in all copies of the supporting
  documentation for such software.
  THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTY.  IN PARTICULAR, NEITHER THE AUTHORS NOR AT&T MAKE ANY
  REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
  OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
 
*****************************************************************************/

#include <math.h>
#include "defs.h"
#include "las.h"
#include "finite.h"


/* parameters: nsites, sqrt_nsites, xmin, xmax, ymin, ymax,
   deltax, deltay (can all be estimates).
   Performance suffers if they are wrong; better to make nsites,
   deltax, and deltay too big than too small.  (?)
--------------------------------------------------------------------*/

void voronoi
(
    struct Site *(*nextsite)(void), 
    long *ntri, 
    short *n1,
    short *n2, 
    short *n3,
    double xmin,
    double xmax,
    double ymin,
    double ymax,
    int   nsites
)
{
  struct Site *newsite, *bot, *top, *temp, *p;
  struct Site *v;
  struct Point newintstar;
  int pm;
  struct Halfedge *lbnd, *rbnd, *llbnd, *rrbnd, *bisector;
  struct Edge *e;
  struct Site *bottomsite;
  int sqrt_nsites;

  sqrt_nsites = sqrt(nsites + 4);
  
  geominit(sqrt_nsites);
  PQinitialize(ymin, ymax, sqrt_nsites);
  bottomsite = (*nextsite)();
  ELinitialize(bottomsite,xmin,xmax, sqrt_nsites);

  *ntri = 0;

  newsite = (*nextsite)();
  while(1)
    {
    if(!PQempty())
      newintstar = PQ_min();

    if((newsite != (struct Site *)NULL) && (PQempty() 
                  || (newsite->coord.y < newintstar.y)
                  || ((newsite->coord.y == newintstar.y)
                   && (newsite->coord.x < newintstar.x))))
      {         /* new site is smallest */
      lbnd = ELleftbnd(&(newsite->coord));
      rbnd = ELright(lbnd);
      bot = rightreg(lbnd);
      e = bisect(bot, newsite);
      bisector = HEcreate(e, LE);
      ELinsert(lbnd, bisector);
      if((p = intersect(lbnd, bisector)) != (struct Site *)NULL) 
        {
        PQdelete(lbnd);
        PQinsert(lbnd, p, dist(p,newsite));
        }
      lbnd = bisector;
      bisector = HEcreate(e, RE);
      ELinsert(lbnd, bisector);
      if((p = intersect(bisector, rbnd)) != (struct Site *)NULL)
        {
        PQinsert(bisector, p, dist(p,newsite));	
        }
      newsite = (*nextsite)();	
      }
    else if (!PQempty()) 
      {  /* intersection is smallest */
      lbnd = PQextractmin();
      llbnd = ELleft(lbnd);
      rbnd = ELright(lbnd);
      rrbnd = ELright(rbnd);
      bot = leftreg(lbnd);
      top = rightreg(rbnd);
      n1[*ntri] = (short)bot->sitenbr;
      n2[*ntri] = (short)rightreg(lbnd)->sitenbr;
      n3[*ntri] = (short)top->sitenbr;
      (*ntri)++;
      v = lbnd->vertex;
      makevertex(v);
      endpoint(lbnd->ELedge, lbnd->ELpm, v);
      endpoint(rbnd->ELedge, rbnd->ELpm, v);
      ELdelete(lbnd); 
      PQdelete(rbnd);
      ELdelete(rbnd); 
      pm = LE;
      if(bot->coord.y > top->coord.y)
        {
        temp = bot;
        bot = top;
        top = temp;
        pm = RE;
        }
      e = bisect(bot, top);
      bisector = HEcreate(e, pm);
      ELinsert(llbnd, bisector);
      endpoint(e, RE-pm, v);
      deref(v);
      if((p = intersect(llbnd, bisector)) != (struct Site *)NULL)
        {
        PQdelete(llbnd);
        PQinsert(llbnd, p, dist(p,bot));
        }
      if((p = intersect(bisector, rrbnd)) != (struct Site *)NULL)
        {
        PQinsert(bisector, p, dist(p,bot));
        }
      }
    else
      break;
    }
}
