/******************************************************************************
  NAME:	GEOMZ

  FUNCTION: 
     To perform rubber-sheet vertical correction of images by a set of
     points specified as parameters or in a parameter data set.

  EXTERNAL ASSOCIATIONS:
        GEOMZ is a LAMS module running under las.

  PROGRAM HISTORY:
  Version	Date	Author		Code/Conr	Request
  -------	----	------		---------	-------
    1.0			A.L.ZOBRIST	   JPL
    2.0		7/86	B.AILTS		   CSB		LAMS conversion
							to unix(newLAS)
    5.0		2/89	B.Ailts		 EDC/CSB	LAS 5.0 conver.
    6.0       	10/90	B.Ailts		 EDC/CSB	Corrected problem when 
							the grid rows and 
							columns where not equal.
    6.1       	12/92   D.Etrheim        EDC/CSB        Error out if the grid 
							is larger than 127x127
    6.2        	1/99    L.Johnson        EDC/CSB        Changed pblock to
                                                        real*8 for ALL
                                                        platforms due to
                                                        apparent alignment
                                                        problems between
                                                        Fortran and C.
    7.2         11/99   G.Schmidt        EDC/CSB        Converted from FORTRAN
                                                        to C.
    7.2         12/99   G.Schmidt        EDC/CSB        Changed the buffer to
                                                        be 3 arrays of line,
                                                        sample, z_value.

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

  PROJECT:	las(LAMS)

  ALGORITHM/DESCRIPTION:

  After reading the input parameters; IN and OUT are opened for read and 
  write respectively.  INGRID is opened and the line/sample locations and 
  brightness shift values are read.  If SCALFACT is greater than one the 
  window specifications are divided by SCALFACT and the input image is 
  reduced by a factor of SCALFACT.

  The pixel values of the image are adjusted according to the tiepoints 
  and brightness shift values given in INGRID.  The tiepoints form a 
  rectangular grid and the pixel is adjusted according to the brightness 
  shift value associated with the four tiepoints that enclose that pixel.  
  If a point lies outside the grid, then the nearest rectangle of the grid 
  is used to adjust the pixel value.  If the adjusted pixel value is greater 
  than maximum or less than the minimum allowed by the data type the pixel 
  will have a value of the maximum or the minimum allowed by the data type.

  The vertical adjustment is bilinear interpolation of the brightness 
  shift values(DN) of the four tiepoints that enclose a pixel.  The formula is

	DN(output) = DN(input) + ax + by + cxy + d

  where x and y are pixel coordinates and a,b,c,d are constants chosen so that

	^DN(k) = ax(k) + by(k) + cx(k)y(k) + d

	for k = 1,2,3,4

  where x(k), y(k) are coordinate values and ^DN(k) is the brightness shift 
  values of the four tiepoints the enclose the point x,y.

  For a single line, the DN increment per sample is given by b+cx.  The 
  starting values for subsequent lines are incremented by a+cy.  Thus, the full 
  solution is needed only at the upper left corner of each rectangle and all 
  other values are obtained by incrementing.

  After every line is processed; all the files are closed, the history 
  file is created and the completion message is displayed.

  ALGORITHM REFERENCES:  none

******************************************************************************/

#include <stdlib.h>
#include <math.h>

#include "las.h"
#include "lams.h"
#include "util.h"
#include "typlim.h"
#include "imageio.h"
#include "ddr.h"
#include "pixman.h"
#include "geomz.h"

extern void dgelg_ (double r[], double a[], long *m, long *n, float *eps,
                    long *ier);

extern long curent;       /* current entity being processed */

void main ()
{
  double **coeff = NULL;  /* coefficient matrix for the DN solutions    */
  double dxx;             /* distance between the grid intersections for
                             the samples                                */
  double dx;              /* current grid column intersection           */
  double dyy;             /* distance between the grid intersections for
                             the lines                                  */
  double dy;              /* current grid row intersection              */
  double *incl = NULL;    /* increment in the line direction (a+cy)     */
  double *incs = NULL;    /* increment in the sample direction (b+cx)   */
  double *line = NULL;    /* buffer for lines in the grid, top to bottom
                             (space is allocated for this in rdfinr)    */
  double lineinc;         /* increment in the line direction (upddr)    */
  double *lval= NULL;     /* array to store the last grid row's z-values*/
  double maxv;            /* minimum intensity value                    */
  double minv;            /* maximum intensity values array             */
  double sampinc;         /* increment in the sample direction (upddr)  */
  double *sample = NULL;  /* buffer for samps in the grid, left to right
                             (space is allocated for this in rdfinr)    */
  double tab[4];          /* 4x1 right hand side matrix, also contains
                             the solutions of the linear equations      */
  double tab1[16];        /* 4x4 coefficient matrix                     */
  double *val = NULL;     /* array of z_values for the current grid row */
  double *z_value = NULL; /* buffer for z_values in the grid, left to
                             right, top to bottom
                             (space is allocated for this in rdfinr)    */

  float *bufin = NULL;    /* input buffer for one line                  */
  float *bufout = NULL;   /* output buffer for one line                 */
  float eps;              /* tolerance value for dgelg call             */
  float epsx;             /* epsilon in the x direction                 */
  float epsy;             /* epsilon in the y direction                 */
  float imax;             /* MAXVAL value                               */
  float imin;             /* MINVAL value                               */
  float imval;            /* intermediate value                         */
  float temp;             /* temporary value                            */
  float *work = NULL;     /* temporary work buffer for one line (contains
                             the scaled image adjusted with the OFFSET) */

  long access = IREAD;    /* file access type                           */
  long bands[2];          /* image bands of the input file              */
  long buf_size;          /* size of the user buffer pool               */
  long dspflag;           /* format display flag for c_inittm           */
  long dtype;             /* temporary data type                        */
  long fatal = -1;        /* fatal error flag                           */
  long four = 4;          /* constant four                              */
  long i, j, k, l;        /* looping/counting variables                 */
  long incoef;            /* current coefficient                        */
  long ismp;              /* looping variable for the samples           */
  long mincnt;            /* number of values entered for MINVAL        */
  long maxcnt;            /* number of values entered for MAXVAL        */
  long msgtim;            /* time between display messages              */
  long nah;               /* (nah1 - 1)                                 */
  long nah1;              /* number of columns in the grid              */
  long nav;               /* (nav1 - 1)                                 */
  long nav1;              /* number of rows in the grid                 */
  long nchx;              /* current grid column being processed        */
  long nchy;              /* current row column being processed         */
  long nl;                /* number of lines in the image (from grid)   */
  long nl2;               /* number of lines after scaling applied      */
  long ns;                /* number of samples in the image (from grid) */
  long ns2;               /* number of samples after scaling applied    */
  long odtype;            /* data type of the image                     */
  long offset[2];         /* band offset array                          */
  long offs;              /* OFFSET value                               */
  long one = 1;           /* constant one                               */
  long opt[2];            /* option array for c_eopen* call             */
  long outpix;            /* current pixel in the work buffer           */
  long pixval;            /* PIXVAL value                               */
  long rec2;              /* current line being processed (takes scaling
                             into account)                              */
  long scale;             /* input scaling factor                       */
  long status;            /* return status                              */
  long tmpflag;           /* temporary flag for updating the DDR        */
  long ul_cell;           /* upper left grid cell z_value position      */
  long ur_cell;           /* upper right grid cell z_value position     */
  long ll_cell;           /* lower left grid cell z_value position      */
  long lr_cell;           /* lower right grid cell z_value position     */
  long window[4];         /* window of the input file                   */
  long zero = 0;          /* constant zero                              */

  char ha1[CMLEN];        /* name of the INGRID file                    */
  char taein[CMLEN];      /* TAE input filename for IN                  */
  char taeout[CMLEN];     /* TAE input filename for OUT                 */
  char hosin[CMLEN];      /* host input filename for IN                 */
  char hosout[CMLEN];     /* host input filename for OUT                */
  char minmax[16];        /* minmax flag                                */

  struct FDESC *fdesc1 = NULL;
                          /* file descriptor of IN                      */
  struct FDESC *fdesc2[2] = {NULL, NULL};
                          /* file descriptor of OUT                     */
  struct GDESC *gdesc = NULL;
                          /* group descriptor for OUT                   */
  struct PARBLK vblock;   /* TAE parameter block                        */

  /* Get TAE parameters
     ------------------ */
  getpar (&vblock, taein, hosin, taeout, hosout, window, bands, &pixval,
          &scale, &offs, &mincnt, &maxcnt, &imin, &imax, ha1, &msgtim,
          minmax);

  /* Initialize the min/max values for the EREAL data type
     -----------------------------------------------------*/
  dtype = EREAL;
  c_inmnmx (&dtype, &one, &minv, &maxv);

  /* Open the grid file and read it into the par array
     -------------------------------------------------*/
  rdfinr (ha1, &line, &sample, &z_value, &nav1, &nah1, &nl, &ns);

  /* Open the input image file
     -------------------------*/
  access = IREAD;
  dtype = EREAL;
  opt[0] = zero;
  opt[1] = 0;  /* use the COMPRES1 environment variable */
  status = c_eopenr (&fdesc1, hosin, &one, &window[0], &window[1], &window[2],
                     &window[3], &access, &dtype, opt, &one);
  if (status != E_SUCC)
  {
    c_errmsg ("Cannot open input image file", "geomz-open", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Verify that the number of lines and samples in the image is not
     greater than the number of lines and samples of the image used to
     create the grid
     -----------------------------------------------------------------*/
  if (window[2] > nl)
  {
    c_errmsg ("Number of lines of image is greater than grid",
              "geomz-greater", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  if (window[3] > ns)
  {
    c_errmsg ("Number of samples of image is greater than grid",
              "geomz-greater", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Dynamically allocate space for bufin and work
     ------------------------------------------------------*/
  if ((bufin = (float *) malloc (EREALSIZE * window[3])) == NULL)
  {
    c_errmsg ("Error allocating memory", "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }
  if ((work = (float *) malloc (EREALSIZE * window[3])) == NULL)
  {
    c_errmsg ("Error allocating memory", "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Dynamically allocate space for incl, incs, lval, and val
     --------------------------------------------------------*/
  if ((incl = (double *) malloc (EDOUBLESIZE * nah1)) == NULL)
  {
    c_errmsg ("Error allocating memory", "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }
  if ((incs = (double *) malloc (EDOUBLESIZE * nah1)) == NULL)
  {
    c_errmsg ("Error allocating memory", "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }
  if ((lval = (double *) malloc (EDOUBLESIZE * nah1)) == NULL)
  {
    c_errmsg ("Error allocating memory", "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }
  if ((val = (double *) malloc (EDOUBLESIZE * nah1)) == NULL)
  {
    c_errmsg ("Error allocating memory", "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Dynamically allocate space for the coeff matrix[ns][4]
     ------------------------------------------------------*/
  if ((coeff = (double **) malloc (sizeof (double *) * nah1)) == NULL)
  {
    c_errmsg ("Error allocating memory", "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }
  for (i = 0; i < nah1; i++)
  {
    if ((coeff[i] = (double *) malloc (EDOUBLESIZE * 4)) == NULL)
    {
      c_errmsg ("Error allocating memory", "geomz-alloc", &one);
      c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
    }
  }

  /* Get the type of the input image
     -------------------------------*/
  status = c_gettyp (hosin, &one, &odtype);
  if (status != E_SUCC)
  {
    c_errmsg ("Error retrieving data type of image", "geomz-dtype", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Set up maximum pixel value allowed by data type
     -----------------------------------------------*/
  if (mincnt == 0)
  {
    if (odtype == EBYTE)
      imin = MINUTINY;
    else if (odtype == EWORD)
      imin = MINWORD;
    else
      imin = MINLONG;
  }

  if (maxcnt == 0)
  {
    if (odtype == EBYTE)
      imax = MAXUTINY;
    else if (odtype == EWORD)
      imax = MAXWORD;
    else 
      imax = MAXLONG;
  }

  /* Scale window parameters for the output image
     --------------------------------------------*/
  if (scale > 1)
  {
    if (window[2] > scale)
    {
      temp = (float) window[2] / (float) scale;
      nl2 = (long) temp;
    }
    else
    {
      nl2 = 1;
    }

    if (window[3] > scale)
    {
      temp = (float) window[3] / (float) scale;
      ns2 = (long) temp;
    }
    else
    {
      ns2 = 1;
    }
  }
  else
  {
    nl2 = window[2];
    ns2 = window[3];
  }

  /* Open the output file
     --------------------*/
  access = IWRITE;
  dtype = EREAL;
  opt[0] = odtype;
  opt[1] = 0;  /* use the COMPRES1 environment variable */
  status = c_eopens (&fdesc2[0], hosout, &one, &one, &one, &nl2, &ns2,
                   offset, &access, &dtype, &zero, opt);
  if (status != E_SUCC)
  {
    c_errmsg ("Cannot open output image file", "geomz-open", &one);
    c_eclose (&fdesc2[0]);
    c_clean (&vblock, "geomz", hosout, &one);
  }
  fdesc2[1] = NULL;

  /* allocate memory for the output buffer */
  if ((bufout = (float *) c_ealloc(fdesc2,&buf_size)) == NULL)
  {
    c_errmsg ("Error allocating memory", "geomz-alloc", &one);
    c_eclose (&fdesc2[0]);
    c_clean (&vblock, "geomz", hosout, &one);
  }

  status = c_egroup (fdesc2, &gdesc, (unsigned char *)bufout, &buf_size);
  if (status != E_SUCC)
  {
    c_errmsg ("Cannot group image files", "geomz-group", &one);
    c_eclose (&fdesc2[0]);
    c_clean (&vblock, "geomz", hosout, &one);
  }

  /* Scale down the grid intersections
     ---------------------------------*/
  nah = nah1 - 1;
  nav = nav1 - 1;

  if (scale != 1)
  {
    /* If the line intersection is not 1.0 then scale it
       -------------------------------------------------*/
    for (i = 0; i < nav1; i++)
    {
      if (line[i] != 1.0)
        line[i] = line[i] / (float) scale + 1.0;
    }

    /* If the sample intersection is not 1.0 then scale it
       ---------------------------------------------------*/
    for (i = 0; i < nah1; i++)
    {
      if (sample[i] != 1.0)
        sample[i] = sample[i] / (float) scale + 1.0;
    }
  }

  /* Calculate the distance between two grid intersections in both the
     line (y) and sample (x) directions
     -----------------------------------------------------------------*/
  dyy = line[1] - line[0];
  dxx = sample[1] - sample[0];

  /* 10-24-86  B. Ailts  The variables EPSX and EPSY were DXX and DYY divided
     by 1000 but are now a constant 1 to account for the changing of input 
     line/sample locations of the grid which was created in FINITE from real
     to integer -- the rounding done to create integers creates a grid which
     is not as uniform as with reals.
     ------------------------------------------------------------------------*/
  epsx = 1.0;
  epsy = 1.0;

  /* Verify that the grid is a uniform rectangular grid
     --------------------------------------------------*/
  for (i = 0; i < nav1; i++)  /* number of rows */
  {
    /* Compare this row with the next row
       ----------------------------------*/
    if (i != nav1-1)
    {
      /* Compare line[x+1] - line[x] with dyy
         ------------------------------------*/
      if (fabs (line[i+1] - line[i] - dyy) > epsy)
      {
        c_errmsg ("Row tiepoints are not evenly spaced", "geomz-tiepoint",
                  &one);
        closelas (fdesc1, gdesc);
        c_clean (&vblock, "geomz", hosout, &one);
      }
    }
  }

  for (i = 0; i < nah1; i++)  /* number of columns */
  {
    /* Compare this column with the next column
       ----------------------------------------*/
    if (i != nah1-1)
    {
      /* Compare samp[y+1] - samp[y] with dxx
         ------------------------------------*/
      if (fabs (sample[i+1] - sample[i] - dxx) > epsx)
      {
        c_errmsg ("Column tiepoints are not evenly spaced", "geomz-tiepoint",
                  &one);
        closelas (fdesc1, gdesc);
        c_clean (&vblock, "geomz", hosout, &one);
      }
    }
  }

  /* Initialize estep
     ----------------*/
  dy = -line[0];   /* allow the first line to be processed as a grid
                      row intersection */
  status = c_estep (&gdesc);
  if (status != E_SUCC)
  {
    c_errmsg ("Error encountered writing to output image", "geomz-write",
              &one);
    closelas (fdesc1, gdesc);
    c_clean (&vblock, "geomz", hosout, &one);
  }

  /* Initialize to start from the first line
     ---------------------------------------*/
  rec2 = window[0];  /* starting line of input image */
  curent = 1;
  dspflag = GEN1;  /* general display message one */
  c_inittm ("GEOMZ", "Processed", "line", " ", "of output image", &nl2,
    &one, &msgtim, &dspflag);
  c_startm ();
  curent++;

  /* Start stepping through the image by line (take scaling into account)
     --------------------------------------------------------------------*/
  nchx = 0;
  nchy = 0;
  for (i = 0; i < nl2; i++, curent++)
  {
    incoef = 0;
    dx = sample[1];

    /* Read the next line
       ------------------*/
    status = c_eread (&fdesc1, &bands[0], &rec2, (unsigned char *) bufin,
                      &one);
    if (status == E_FAIL)
    {
      c_errmsg ("Error encountered reading input image", "geomz-read", &one);
      closelas (fdesc1, gdesc);
      c_clean (&vblock, "geomz", hosout, &one);
    }

    /* Loop from the starting sample to the ending sample in the image and
       add the OFFSET value
       -------------------------------------------------------------------*/
    for (ismp = window[1], outpix = 0; ismp < window[1]+window[3];
         ismp += scale, outpix++)
    {
      if (bufin[ismp-1] == (float) pixval)
        work[outpix] = bufin[ismp-1];
      else
        work[outpix] = bufin[ismp-1] + (float) offs;
    }

    /* Step through samples of the image for this line (take scaling
       into account)
       -------------------------------------------------------------*/
    for (j = 0; j < ns2; j++)
    {
      if (i >= dy)  /* if current line is at the next grid row intersection */
      {
        nchy++;

        /* If this is the second to last grid row intersection
           ---------------------------------------------------*/
        if (nchy == nav)
          dy = 1.0e10;

        /* Find the next grid row intersection
           -----------------------------------*/
        dy = fabs(dy) + dyy;

        /* Loop through the columns  of the grid
           -------------------------------------*/
        for (k = 0; k < nah; k++)
        {
          /* Calculate the four corners of the current grid cell
             for the z_value positions
             ---------------------------------------------------*/
          ul_cell = (nchy-1) * nah1 + k;
          ur_cell = (nchy-1) * nah1 + (k+1);
          ll_cell = nchy * nah1 + k;
          lr_cell = nchy * nah1 + (k+1);

          /* Get the z_value for the upper left grid cell
             --------------------------------------------*/
          val[k] = z_value[ul_cell];

          /* Create the 4x1 right hand side matrix of z_values and the
             4x4 coefficient matrix for the solution to the linear
             equations for this grid cell
             ---------------------------------------------------------*/
             tab[0] = z_value[ul_cell];
             tab1[0] = line[nchy-1];
             tab1[4] = sample[k];
             tab1[8] = line[nchy-1] * sample[k];
             tab1[12] = 1.0;

             tab[1] = z_value[ur_cell];
             tab1[1] = line[nchy-1];
             tab1[5] = sample[k+1];
             tab1[9] = line[nchy-1] * sample[k+1];
             tab1[13] = 1.0;

             tab[2] = z_value[ll_cell];
             tab1[2] = line[nchy];
             tab1[6] = sample[k];
             tab1[10] = line[nchy] * sample[k];
             tab1[14] = 1.0;

             tab[3] = z_value[lr_cell];
             tab1[3] = line[nchy];
             tab1[7] = sample[k+1];
             tab1[11] = line[nchy] * sample[k+1];
             tab1[15] = 1.0;

          /* Calculate the input matrix grid.
             dgelg solves a general system of simultaneous linear
             equations.  tab is a double precision 4x1 right hand
             side matrix.  On return tab contains the solution of
             equations.  tab1 is a 4x4 coefficient matrix.  four
             is the number of equations in the system.  one is the
             number of right hand side vectors.  eps is used as a
             relative tolerance for test on loss of significance.
             input matrices tab and tab1 are assumed to be stored
             columnwise for the dgelg call.
             (NOTE: the call to dgelg is a FORTRAN call)
             -----------------------------------------------------*/
          eps = 1.0e-14;
          dgelg_ (tab, tab1, &four, &one, &eps, &status);
          if (status == E_FAIL)
          {
            c_errmsg ("Error in calculating the input matrix grid",
                      "geomz-matrix", &one);
            closelas (fdesc1, gdesc);
            c_clean (&vblock, "geomz", hosout, &one);
          }

          /* Calculate the increments in the line and sample direction
             Increment in the line direction is a + cy
             Increment in the sample direction is b + cx
             ---------------------------------------------------------*/
          incl[k] = tab[0] + tab[2] * sample[k];
          incs[k] = tab[1] + tab[2] * (i+1);
          lval[k] = val[k];
          val[k] -= incs[k];

          /* Store the coefficients for this line
             ------------------------------------*/
          for (l = 0; l < 4; l++)
            coeff[k][l] = tab[l];
        }  /* for k */
      }  /* if i >= dy */

      if (j >= dx)
      {
        nchx++;
        if (nchx == nah)
          dx = 1.0e10;
        dx += dxx;
        incoef++;
      }

      val[incoef] += incs[incoef];
      if (work[j] != (float) pixval)
      {
        imval = work[j] + val[incoef];
        bufout[offset[0] + j] = imval;
      }
      else
      {
        bufout[offset[0] + j] = work[j];
      }

      /* Make sure the buffer values are not less than the minimum value
         or greater than the maximum value
         ---------------------------------------------------------------*/
      if (bufout[offset[0] + j] < imin)
        bufout[offset[0] + j] = imin;
      if (bufout[offset[0] + j] > imax)
        bufout[offset[0] + j] = imax;
    }  /* for j */

    for (j = 0; j < nah; j++)
    {
      incs[j] += coeff[j][2];
      val[j] = lval[j] + incl[j];
      lval[j] = val[j];
      val[j] -= incs[j];
    }

    /* Increment rec2 to process the next line and reset nchx
       ------------------------------------------------------*/
    rec2 += scale;
    nchx = 1;

    if (!strncmp (minmax, "YES", 3))
    {
      dtype = EREAL;
      c_pxmnmx ((long *) &bufout[offset[0]], &ns2, &minv, &maxv, &dtype);
    }

    status = c_estep (&gdesc);
    if (status != E_SUCC)
    {
      c_errmsg ("Error encountered writing to output image", "geomz-write",
        &one);
      closelas (fdesc1, gdesc);
      c_clean (&vblock, "geomz", hosout, &one);
    }
  }  /* for i */

  /* Finish the processing
     ---------------------*/
  c_stoptm ();
  closelas (fdesc1, gdesc);

  /* Update the DDR file
     -------------------*/
  lineinc = 1.0;
  sampinc = 1.0;
  tmpflag = NOCOMB;
  status = c_upddr (&vblock, hosin, hosout, &one, window, bands, &one,
                    &lineinc, &sampinc, &one, &tmpflag);
  if (status != E_SUCC)
  {
    c_errmsg ("DDR file not updated", "geomz-warn", &one);
  }

  /* Save the min/max
     ----------------*/
  status = c_svmnmx (hosout, &one, &minv, &maxv, minmax);
  if (status == E_FAIL)
  {
    c_errmsg ("Unsuccessful save min/max", "geomz-warn", &one);
  }

  /* Update the history file
     -----------------------*/
  status = c_hstout (&vblock, hosout, hosin, &one, "GEOMZ");
  if (status != E_SUCC)
  {
    c_errmsg ("History file not created", "geomz-warn", &one);
  }

  /* Call complt to end the routine
     ------------------------------*/
  c_complt ("GEOMZ", taeout, &one);
}
