/****************************************************************************
   NAME:                          GETPAR

   FUNCTION:
      GETPAR retrieves the parameters.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2     11/99    G. Schmidt        CSB       Original Development
****************************************************************************/

#include <string.h>
#include "las.h"
#include "util.h"     /* c_inlas, c_getin, c_getout proto */
#include "tae.h"      /* p_find proto */
#include "geomz.h"

void getpar (
  struct PARBLK *vblock,  /* O: TAE parameter block                        */
  char taein[CMLEN],      /* O: TAE input filenames of images specified    */
  char hosin[CMLEN],      /* O: host input filenames of images specified   */
  char taeout[CMLEN],     /* O: TAE output filename                        */
  char hosout[CMLEN],     /* O: host output filename                       */
  long window[4],         /* O: window of the input file                   */
  long bands[2],          /* O: image bands of the input file              */
  long *fixval,           /* O: PIXVAL value                               */
  long *scale,            /* O: input scaling factor                       */
  long *offs,             /* O: OFFSET value                               */
  long *mincnt,           /* O: number of values entered for MINVAL        */
  long *maxcnt,           /* O: number of values entered for MAXVAL        */
  float *imin,            /* O: MINVAL value                               */
  float *imax,            /* O: MAXVAL value                               */
  char ha1[CMLEN],        /* O: INGRID value                               */
  long *msgtim,           /* O: time between display messages              */
  char minmax[])          /* O: minmax flag                                */

{
  long access = IREAD;       /* reading the grid                           */
  long fatal = -1;           /* fatal error flag                           */
  long images;               /* number of images input from TAE            */
  long numbnd;               /* number of bands                            */
  long one = 1;              /* constant one                               */
  long qflag[2];             /* window and/or band flag                    */
  long same;                 /* 1 if all windows are the same, 0 if not    */
  long status;               /* return status                              */
  long std_out = 0;          /* standard output                            */
  long zero = 0;             /* constant zero                              */

  struct VARIABLE *var;      /* pointer to TAE par block var               */

  c_inlas (vblock, &std_out);

  /* Get the input image and its attributes
     ---------------------------------------*/
  status = c_getin(vblock, "IN", &zero, taein, hosin, &images, window,
                   bands, &one, &numbnd, &one, &same, qflag);
  if (status == E_FAIL)
  {
    c_errmsg("Error getting the IN parameter", "geomz-getin", &one);
    c_errmsg("Fatal error encountered", "geomz-fatal", &fatal);
  }
  if (numbnd > 1)
  {
    c_errmsg("Too many bands specified in the input image", "geomz-getin",
             &one);
    c_errmsg("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Get the output image name
     -------------------------*/
  status = c_getout(vblock, "OUT", taeout, hosout);
  if (status == E_FAIL)
  {
    c_errmsg("Error getting the OUT parameter", "geomz-getout", &one);
    c_errmsg("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Get the PIXVAL
     --------------*/
  var = p_find(vblock, "PIXVAL");
  *fixval = IVAL(*var, 0);

  /* Get the SCALFACT
     ----------------*/
  var = p_find(vblock, "SCALFACT");
  *scale = IVAL(*var, 0);

  /* Get the OFFSET
     ----------------*/
  var = p_find(vblock, "OFFSET");
  *offs = IVAL(*var, 0);

  /* Get the MINVAL
     --------------*/
  var = p_find(vblock, "MINVAL");
  *mincnt = var->v_count;   /* need to know if this is 0 later on */
  *imin = RVAL(*var,0);

  /* Get the MAXVAL
     --------------*/
  var = p_find(vblock, "MAXVAL");
  *maxcnt = var->v_count;   /* need to know if this is 0 later on */
  *imax = RVAL(*var,0);

  /* Get the INGRID
     --------------*/
  if (c_getnam (vblock, "INGRID", &zero, ha1, "grid", &access)
      != E_SUCC)
  {
    c_errmsg("Error retrieving the INFILE name", "geomz-infile", &one);
    c_errmsg("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Get the $MSGTIM
     ---------------*/
  var = p_find(vblock, "$MSGTIM");
  *msgtim = IVAL(*var, 0);

  /* Get the $MINMAX
     ---------------*/
  var = p_find(vblock, "$MINMAX");
  strcpy (minmax, SVAL(*var, 0));
}
