/*****************************************************************************
NAME: RDFINR

FUNCTION:
RDFINR is a utility to read the radiometric grid file created by FINITE.

PROGRAM HISTORY:
Version		Date	Author		Code/conr	Request
-------		----	------		---------	-------
   1.0		7/86	B.AILTS		   CSB		Original devel.
   2.0		9/88	B.Ailts		   CSB		Update to LAS 5
   7.2         11/99    G.Schmidt          CSB          Removed limitations
                                                        on the size of the
                                                        grid and "cleaned-up"
                                                        the code.
   7.2         12/99    G.Schmidt          CSB          Changed the packed
                                                        buffer to be 3 arrays
                                                        for line/samp/z_value

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS: Run under TAE 

PROJECT:	LAMS

ALGORITHM DESCRIPTION:
The grid file is read and the number of lines and samples from the image
and the number of rows and columns in the grid are returned.  The line/sample
locations and their corresponding z_values are read and returned to the calling 
routine.

NOTES: Space is allocated for the line/sample/z_value buffers in this routine.

ALGORITHM REFERENCES:	none 

******************************************************************************/

#include "las.h"
#include "lams.h"
#include "radpak.h"
#include "geomz.h"

void rdfinr (
  char fname[],      /* I: host filename of radiometric mapping grid file  */
  double **line,     /* O: address of line pointer to store the grid line
                           values at the grid intersections (space is
                           allocated for this) stored top to bottom        */
  double **sample,   /* O: address of sample pointer to store the grid
                           sample values at the grid intersections (space
                           is allocated for this) stored left to right     */
  double **z_value,  /* O: address of z_value pointer to store the grid
                           z_values at all the grid intersections (space
                           is allocated for this) stored left to right,
                           top to bottom                                   */
  long *nr,          /* O: number of rows in the grid                      */
  long *nc,          /* O: number of columns in the grid                   */
  long *nl,          /* O: number of lines in the output image             */
  long *ns)          /* O: number of samples in the output image           */

{
  struct RADGRID *grid = NULL;  /* radiometric mapping grid       */
  long fatal = -1;       /* fatal value                           */
  long i;                /* looping variable for rows in the grid */
  long j;                /* looping variable for cols in the grid */
  long l;                /* index of current z_value              */
  long one = 1;          /* value of 1                            */
  long status;           /* return status                         */

  /* Allocate space for the radiometric grid structure.
   */
  grid = (struct RADGRID *) malloc (sizeof (struct RADGRID));
  if (grid == NULL)
  {
    c_errmsg ("Error allocating dynamic memory for radiometric grid",
              "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Read the radiometric grid.
   */
  status = c_getrgd (fname, grid);
  if (status != E_SUCC)
  {
    c_errmsg ("Error reading grid file", "geomz-rdgrid", &one);
    c_errmsg ("Fatal error eoncountered", "geomz-fatal", &fatal);
  }

  /* Save the number of rows/cols and the number of lines/samples from
     the grid.
   */
  *nr = grid->nrows;
  *nc = grid->ncols;
  *nl = grid->lines;
  *ns = grid->samples;

  /* Allocate space for the line buffer.
   */
  *line = (double *) malloc (grid->nrows * sizeof (double));
  if (grid == NULL)
  {
    c_errmsg ("Error allocating dynamic memory for grid line buffer",
              "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Allocate space for the sample buffer.
   */
  *sample = (double *) malloc (grid->ncols * sizeof (double));
  if (grid == NULL)
  {
    c_errmsg ("Error allocating dynamic memory for grid sample buffer",
              "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Allocate space for the z_value buffer to hold one value for each grid
     (row,col) intersection.
   */
  *z_value = (double *) malloc (grid->nrows * grid->ncols * sizeof (double));
  if (grid == NULL)
  {
    c_errmsg ("Error allocating dynamic memory for grid z_value buffer",
              "geomz-alloc", &one);
    c_errmsg ("Fatal error encountered", "geomz-fatal", &fatal);
  }

  /* Store the lines from the grid.
   */
  for (i = 0; i < *nr; i++)
    (*line)[i] = (double) grid->out_lines[i];

  /* Store the samples from the grid.
   */
  for (i = 0; i < *nc; i++)
    (*sample)[i] = (double) grid->out_samps[i];

  /* Loop through the rows and columns of the grid and store the z_value
     for each (row, col) intersection of the grid.
   */
  l = 0;  /* index of current z_value */
  for (i = 0; i < *nr; i++)
    for (j = 0; j < *nc; j++, l++)
      (*z_value)[l] = grid->z_values[l];

  free (grid);

  return;
}
