C*****************************************************************************
C 	NAME: INTCUT
C
C	FUNCTION:
C	INTCUT reads two polygon cut files and calculates the intersection
C	of the two cutline polygons.  It then creates cutline files of 
C	the intersection for each of the input files. 
C
C	PROGRAM HISTORY:
C	Version		Date	Author		Code/conr	Request
C	-------		----	------		---------	-------
C	   1.0		1/87	B.AILTS		   CSB		Orig. develop.
C	   1.1		12/88   B.Ailts		EDC/CSB		LAS 5.0 conv.
C	   1.2		 9/92	T.Mittan	EDC/CSB		Modified temy1,
C								temy2, temx1,
C								anc temx2 to 	C								eliminate 
C								warning 
C	   1.4		01/00	D.Hames		EDC/CSB	Remove valid=(1:50000)
C                                                       from SL and SS in
C                                                       intcut.pdf. Add error   
C                                                       checks to ensure they 
C                                                       are positive. Modified
C                                                       to support images > 2gb
C                                                       SRF 893
C
C	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
C			Run under TAE 
C
C	PROJECT:	LAMS
C
C	ALGORITHM DESCRIPTION:
C	The tae parmaters are read in.
C
C	INFILE(1) and INFILE(2) are opened and the data is read.  The
C	starting line and sample for each file is added to the cut
C	file coordinates as they are split into the x and y coordinates
C	for the subroutine PSIUD.  PSIUD calculates the intersection
C	of the two polygons.  The starting line and sample are subtracted 
C	from this intersection polygon in order for the coordinates to be 
C	coorelated with the original input files.  These coordinates are
C	then written to a cutline file.	PSBCX is called to calculate the 
C	minimum and maximum of the intersection polygon and these values 
C	are returned as variables to the pdf.
C
C	After all the data is written, the completion message is 
C	displayed.
C
C	ALGORITHM REFERENCES:	none
C
C******************************************************************************

      program intcut

C  INCLUDE FILES
C	Name		Description
C	----		-----------
C	las.mfi		general las constants
C	lams.mfi	general lams include file

	include 'las.fin'

	include 'lams.fin'

C  INTERNAL VARIALBES
C	Name		Type 	I/O	Description
C	----		----	---	-----------
C	oxbuf(MMXPPR)	r*4	 O	output sample corrdinates
C	oybuf(MMXPPR)	r*4	 O	output line corrdinates
C	xbuf1(MMXPPR)	r*4	 I	input sample corrdinates of INFILE1
C	xbuf2(MMXPPR)	r*4	 I	input sample corrdinates of INFILE2
C	xmax		r*4	 O	maximum output sample corrdinate
C	xmin		r*4	 O	minimum output sample corrdinate
C	ybuf1(MMXPPR)	r*4	 I	input line corrdinates of INFILE1
C	ybuf2(MMXPPR)	r*4	 I	input line corrdinates of INFILE2
C	ymax		r*4	 O	maximum output line corrdinate
C	ymin		r*4	 O	minimum output line corrdinate
C	id1		i*4	 I	input id needed by polygon 
C					intersection routine
C	id2		i*4	 I	input id needed by polygon 
C					intersection routine
C	oid		i*4	 O	output id given back by polygon 
C					intersection routine

	real*4 oxbuf(MMXPPR)
	real*4 oybuf(MMXPPR)
	real*4 xbuf1(MMXPPR)
	real*4 xbuf2(MMXPPR)
	real*4 xmax
	real*4 xmin
	real*4 ybuf1(MMXPPR)
	real*4 ybuf2(MMXPPR)
	real*4 ymax
	real*4 ymin

	integer*4 id1(3)
	integer*4 id2(3)
	integer*4 insline(2)
	integer*4 inssamp(2)
	integer*4 oid(100)

      	integer*4	parb(XPRDIM)

	integer*4 prt(3)
	integer*4 status
	integer*4 stdout

	character*(CMLEN) hosin1
	character*(CMLEN) hosin2
	character*(CMLEN) hosout1
	character*(CMLEN) hosout2
	character*(CMLEN) taeout(3)
	character*(CMLEN) prtfile

       	call inlas(parb,stdout)

	call init(hosin1,hosout1,hosin2,hosout2,insline,inssamp,parb,
     .		  prt,prtfile,taeout)

	call input(hosin1,hosin2,ybuf1,ybuf2,xbuf1,xbuf2
     .			,insline,inssamp,id1,id2)
	
	call psiud (2,id1,xbuf1,ybuf1,id2,xbuf2,ybuf2,oid,oxbuf,oybuf)
	if (oid(2) .gt. 1) then
	   call hole(oid,oybuf,oxbuf,insline,inssamp)
	else if (oid(2) .lt. 1) then
	   call errmsg('Two input polygons do not intersect'
     .			,'intcut-intersect',1)
	call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if

	call psbox(oid,oxbuf,oybuf,xmin,ymin,xmax,ymax)

	call output(hosout1,hosout2,oid,oybuf,oxbuf,insline,inssamp
     .		    ,xmin,ymin,xmax,ymax,parb,prt,prtfile)

        taeout(1) = hosout1
        taeout(2) = hosout2

	if (prt(2) .ne. 0) then
	   call complt('INTCUT',taeout,3)
	else
	   call complt('INTCUT',taeout,2)
	endif

	end

C**************************************************************************
C
C	Subroutine INIT retreives the TAE parameters
C
C**************************************************************************
	subroutine init(hosin1,hosout1,hosin2,hosout2,insline,
     .			inssamp,parb,prt,prtfile,taeout)

C  INCLUDE FILES
C	Name		Description
C	----		-----------
C	las.mfi	general las constants

	include 'las.fin'

	include 'lams.fin'

C  ARGUMENTS
C	Name		Type 	I/O	Description
C	----		----	---	-----------

C  INTERNAL VARIALBES
C	Name		Type 	I/O	Description
C	----		----	---	-----------

	integer*4 count
	integer*4 getnam
	integer*4 insline(2)
	integer*4 inssamp(2)

      	integer*4	parb(XPRDIM)

	integer*4 prt(3)
	integer*4 setprt
	integer*4 status

	character*(CMLEN) hosin1
	character*(CMLEN) hosin2
	character*(CMLEN) hosout1
	character*(CMLEN) hosout2
	character*(CMLEN) prtfile
	character*(CMLEN) taeout(3)

       	status = getnam(parb,'INFILE',1,hosin1,'ibis_graph',IREAD)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if
       	status = getnam(parb,'OUTFILE',1,hosout1,'ibis_graph',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if
       	status = getnam(parb,'INFILE',2,hosin2,'ibis_graph',IREAD)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if
       	status = getnam(parb,'OUTFILE',2,hosout2,'ibis_graph',IWRITE)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if
	call xrintg(parb,'SL',2,insline,count,status)
	call xrintg(parb,'SS',2,inssamp,count,status)
	prtfile = 'intcut'
	status = setprt(parb,prt(1),prt(2),prt(3),prtfile,taeout(3))
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if
		
	return
	end

C******************************************************************************
C
C	Subroutine INPUT reads in the coordinates of the two input files
C
C******************************************************************************
	subroutine input(hosin1,hosin2,ybuf1,ybuf2,xbuf1,xbuf2
     .			,insline,inssamp,id1,id2)

C  INCLUDE FILES
C	Name		Description
C	----		-----------
C	las.mfi	general las constants

	include 'las.fin'

	include 'lams.fin'

C  ARGUMENTS
C	Name		Type 	I/O	Description
C	----		----	---	-----------
C	xbuf1(1)	r*4	 I	input sample corrdinates of INFILE1
C	xbuf2(1)	r*4	 I	input sample corrdinates of INFILE2
C	ybuf1(1)	r*4	 I	input line corrdinates of INFILE1
C	ybuf2(1)	r*4	 I	input line corrdinates of INFILE2
C	id1		i*4	 I	input id needed by polygon 
C					intersection routine
C	id2		i*4	 I	input id needed by polygon 
C					intersection routine
C	insline(2)	i*4	 I	line offsets for input coordinates
C	inssamp(2)	i*4	 I	sample offsets for input coordinates
C
C  INTERNAL VARIALBES
C	Name		Type 	I/O	Description
C	----		----	---	-----------
C	ibuf1		r*4	 I	input coordinates from INFILE1
C	ibuf2		r*4	 I	input coordinates from INFILE2
C	nb1		i*4	 I	no. of bytes in a line of INFILE1
C	nb2		i*4	 I	no. of bytes in a line of INFILE2
C	nw1		i*4	 I	no. of coordinates in a line of INFILE1
C	nw2		i*4	 I	no. of coordinates in a line of INFILE2
C	iu1		i*4	 I	no. of lines in INFILE1
C	iu2		i*4	 I	no. of lines in INFILE2

	real*4 ibuf1(MMXPT)
	real*4 ibuf2(MMXPT)
	real*4 xbuf1(1)
	real*4 xbuf2(1)
	real*4 ybuf1(1)
	real*4 ybuf2(1)

	integer*4 cnt
	integer*4 i
	integer*4 id1(3)
	integer*4 id2(3)
	integer   fdesc1
	integer   fdesc2
	integer*4 grclos
	integer*4 gropen
	integer*4 grread
	integer*4 insline(2)
	integer*4 inssamp(2)
	integer*4 iu1
	integer*4 iu2
	integer*4 j
	integer*4 nb1
	integer*4 nb2
	integer*4 rec
	integer*4 status

	character*(CMLEN) hosin1
	character*(CMLEN) hosin2

C  Open the two input files
C--------------------------
	status = gropen(fdesc1,hosin1,nb1,iu1,0)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','intcut-open',1)
	   call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if

	status = gropen(fdesc2,hosin2,nb2,iu2,0)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening input file','intcut-open',1)
	   call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if

C  Read  data from the INFILE1 
C-------------------------------------
	REC = 1
	cnt = 0
	DO 200 I=1,IU1
	   status = grread(fdesc1,rec,IBUF1)
	   if (status .ne. ESUCC) then
	      call errmsg('Error reading input file','intcut-read',1)
	      call errmsg('Fatal error encountered','intcut-fatal',-1)
	   end if
	   REC = REC+1

C  Place input coordinates in a format readable by the intersection routine
C--------------------------------------------------------------------------
	   do 210 j = 1 , nb1 ,2
	      if (ibuf1(j) .ne. 0) then
	         cnt = cnt + 1
	         ybuf1(cnt) = ibuf1(j) + insline(1) - 1
	         xbuf1(cnt) = ibuf1(j + 1) + inssamp(1) - 1
	      end if
210	   continue
200	CONTINUE

	id1(1) = cnt
	id1(2) = 1
	id1(3) = id1(1)

	if ((ybuf1(1) .ne. ybuf1(cnt)) .or. 
     .	   (xbuf1(1) .ne. xbuf1(cnt))) then
	   call errmsg ('Input polygon 1 is not closed'
     .			,'intcut-close',1)
	   call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if

	status = grclos(fdesc1,hosin1)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','intcut-close',1)
	end if

C  Read  data from the INFILE2 
C-------------------------------------
	REC = 1
	cnt = 0
	DO 250 I=1,IU2
	   status = grread(fdesc2,rec,IBUF2)
	   if (status .ne. ESUCC) then
	      call errmsg('Error reading input file','intcut-read',1)
	      call errmsg('Fatal error encountered','intcut-fatal',-1)
	   end if
	   REC = REC+1

C  Place input coordinates in a format readable by the intersection routine
C--------------------------------------------------------------------------
	   do 260 j = 1 , nb2 ,2
	      if (ibuf2(j) .ne. 0) then
	         cnt = cnt + 1
	         ybuf2(cnt) = ibuf2(j) + insline(2) - 1
	         xbuf2(cnt) = ibuf2(j + 1) + inssamp(2) - 1
	      end if
260	   continue
250	CONTINUE

	id2(1) = cnt
	id2(2) = 1
	id2(3) = id2(1)

	if ((ybuf2(1) .ne. ybuf2(cnt)) .or. 
     .	   (xbuf2(1) .ne. xbuf2(cnt))) then
	   call errmsg ('Input polygon 2 is not closed'
     .			,'intcut-close',1)
	   call errmsg('Fatal error encountered','intcut-fatal',-1)
	end if

	status = grclos(fdesc2,hosin2)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing input file','intcut-close',1)
	end if

	return
	end

C***************************************************************************
C
C	Subroutine OUTPUT outputs the coordinates to the two output files
C
C***************************************************************************
	subroutine output(hosout1,hosout2,oid,oybuf,oxbuf,insline,
     .			  inssamp,xmin,ymin,xmax,ymax,parb,prt,prtfile)

C  INCLUDE FILES
C	Name		Description
C	----		-----------
C	las.mfi	general las constants

	include 'las.fin'

	include 'lams.fin'

C  ARGUMENTS
C	Name		Type 	I/O	Description
C	----		----	---	-----------
C	oxbuf(1)	r*4	 O	output sample corrdinates
C	oybuf(1)	r*4	 O	output line corrdinates
C	xmax		r*4	 O	maximum output sample corrdinate
C	xmin		r*4	 O	minimum output sample corrdinate
C	ymax		r*4	 O	maximum output line corrdinate
C	ymin		r*4	 O	minimum output line corrdinate
C	oid		i*4	 O	output id given back by polygon 
C					intersection routine
C	insline		i*4	 I	offsets for the line coordinates of 
C					input file
C	inssamp		i*4	 I	offsets for the sample coordinates of 
C					input file
C
C  INTERNAL VARIALBES
C	Name		Type 	I/O	Description
C	----		----	---	-----------
C	iu		i*4	 O	no. of lines in the output files
C	nb		i*4	 O	no. of bytes per line in output files
C	nw		i*4	 O	no. of coordinates in the output files
C	window1(4)	i*4	 O	coordinates to form a window around the
C					output polygon for OUTFILE1
C	window2(4)	i*4	 O	coordinates to form a window around the
C					output polygon for OUTFILE2

	real*4 obuf1(MMXPT)
	real*4 obuf2(MMXPT)
	real*4 oxbuf(1)
	real*4 oybuf(1)
	real*4 xmax
	real*4 xmin
	real*4 ymax
	real*4 ymin

	integer*4 i
	integer   fdesc1
	integer   fdesc2
	integer*4 grclos
	integer*4 gropen
	integer*4 grwrit
	integer*4 insline(2)
	integer*4 inssamp(2)
	integer*4 iu
	integer*4 j
	integer*4 k
	integer*4 l
	integer*4 nw
	integer*4 oid(1)

      	integer*4	parb(XPRDIM)

	integer*4 prt(3)
	integer*4 rec
	integer*4 status
	integer*4 unit
	integer*4 window1(4)
	integer*4 window2(4)

	character*(CMLEN) hosout1
	character*(CMLEN) hosout2
	character*(CMLEN) out(2)
	character*(CMLEN) prtfile
	character*80 prtout1
	character*80 prtout2
	character*80 prtout3
	character*80 prtout4
	character*2  prtout5

	out(1) = hosout1
	out(2) = hosout2
	unit = 10

C  Calculate the size of the output files
C----------------------------------------
	if (((oid(1) + 1)  * 2) .ge. MMXPPR) then
	   iu = (((oid(1) + 1) * 2) + (MMXPPR - 1)) / MMXPPR
	   nw = MMXPPR
	else
	   iu = 1
	   nw = (oid(1) + 1) * 2
	end if

C  Open the output files
C-----------------------
	status = gropen(fdesc1,hosout1,nw,iu,1)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening the output file',
     .     'intcut-write',1)
	   status = grclos(fdesc1,hosout1) 
	   call clean(parb,'intcut',out,2)
	end if

	status = gropen(fdesc2,hosout2,nw,iu,1)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening the output file',
     .     'intcut-write',1)
	   status = grclos(fdesc1,hosout1) 
	   status = grclos(fdesc2,hosout2)
	   call clean(parb,'intcut',out,2)
	end if

C  Write the intersection polygon to output cutline files
C--------------------------------------------------------
	REC = 1
	k = 0
	DO 200 I=1,IU

C  Zero out the output buffer
C----------------------------
	   do 230 l = 1,nw
	      obuf1(l) = 0
	      obuf2(l) = 0
230	   continue

C  Subtract the offset from the intersection polygon for each output file
C------------------------------------------------------------------------
	   if ( i .eq. iu ) then
	      nw = mod(((oid(1) + 1) * 2),MMXPPR)
	   end if

	   j = 0
	   do 250 l = 1 , (nw - 2) / 2
	      k = k + 1
	      j = j + 2
	      obuf1(j - 1) = oybuf(k) - insline(1) + 1
	      obuf1(j) = oxbuf(k) - inssamp(1) + 1
	      obuf2(j - 1) = oybuf(k) - insline(2) + 1
	      obuf2(j) = oxbuf(k) - inssamp(2) + 1
250	   continue

C  Close the polygon by making the last point equal the first point
C------------------------------------------------------------------
	   obuf1(j + 1) = obuf1(1)
	   obuf1(j + 2) = obuf1(2)
	   obuf2(j + 1) = obuf2(1)
	   obuf2(j + 2) = obuf2(2)

C  Write the output coordinates to the output files
C--------------------------------------------------
	   status = grwrit(fdesc1,rec,obuf1)
	   if (status .ne. ESUCC) then
	      call errmsg('Error writing to the output file',
     .        'intcut-write',1)
	      status = grclos(fdesc1,hosout1) 
	      status = grclos(fdesc2,hosout2)
	      call clean(parb,'intcut',out,2)
	   end if

	   status = grwrit(fdesc2,rec,obuf2)
	   if (status .ne. ESUCC) then
	      call errmsg('Error writing to the output file',
     .        'intcut-write',1)
	      status = grclos(fdesc1,hosout1) 
	      status = grclos(fdesc2,hosout2)
	      call clean(parb,'intcut',out,2)
	   end if

	   REC = REC+1
 200	CONTINUE

	status = grclos(fdesc1,hosout1) 
	if (status .ne. ESUCC) then
	   call errmsg('Error closing output file','intcut-close',1)
	endif
	status = grclos(fdesc2,hosout2)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing output file','intcut-close',1)
	endif

C Calculate the values for global variables $WINDOW1 and $WINDOW2
C---------------------------------------------------------------- 
	window1(1) = nint(ymin) - insline(1) + 1
	window1(2) = nint(xmin) - inssamp(1) + 1
	window1(3) = nint(ymax) - insline(1) + 1
	window1(4) = nint(xmax) - inssamp(1) + 1

	window2(1) = nint(ymin) - insline(2) + 1
	window2(2) = nint(xmin) - inssamp(2) + 1
	window2(3) = nint(ymax) - insline(2) + 1
	window2(4) = nint(xmax) - inssamp(2) + 1

	do 3000 l = 1,4
	   if (window1(l) .lt. 1) then
	       window1(l) = 1
	   endif

	   if (window2(l) .lt. 1) then
	       window2(l) = 1
	   endif

3000	continue

C  Output the value of window1 and window2
C  ---------------------------------------
	prtout5 = ' '
	prtout1 = ' The window specifications for OUTFILE(1) are:'
	prtout3 = ' The window specifications for OUTFILE(2) are:'
	write(prtout2,4000) window1(1),window1(2),window1(3),
     .			    window1(4)
	write(prtout4,4000) window2(1),window2(2),window2(3),
     .			    window2(4)
4000	format (8X,2X,I5,2X,I5,2X,I5,2X,I5)
	if (prt(1) .ne. 0) then
	call pterm(prtout1,0)
	call pterm(prtout2,0)
	call pterm(prtout5,0)
	call pterm(prtout3,0)
	call pterm(prtout4,0)
	call pterm(prtout5,0)
	endif

	if ((prt(2) .ne. 0) .or. (prt(3) .ne. 0)) then
	open (unit,status = 'new',file=prtfile)
	write (unit,'(A)') prtout1
	write (unit,'(A)') prtout2
	write (unit,'(A)') prtout5
	write (unit,'(A)') prtout3
	write (unit,'(A)') prtout4
	write (unit,'(A)') prtout5
	close (unit)
	endif
	
C  Set global varialbes $WINDOW1 and $WINDOW2
C--------------------------------------------
	call xqini(parb,XPRDIM,XABORT)
	call xqintg(parb,'$WINDOW1',4,window1,XADD,status)
	call xqintg(parb,'$WINDOW2',4,window2,XADD,status)
	call xqout(parb,status)

	return
	end

C****************************************************************************
C
C	Subroutine HOLE outputs the coordinates of a hole in the 
C	intersection of the two polygons
C
C****************************************************************************
	subroutine hole(oid,oybuf,oxbuf,insline,inssamp)

C  INCLUDE FILES
C	Name		Description
C	----		-----------
C	las.mfi	general las constants

	include 'las.fin'

	include 'lams.fin'

	include 'typlimsg.fin'

C  ARGUMENTS
C	Name		Type 	I/O	Description
C	----		----	---	-----------
C	oxbuf(1)	r*4	 O	output sample corrdinates
C	oybuf(1)	r*4	 O	output line corrdinates
C	insline		i*4	 I	offsets for the line coordinates of 
C					input file
C	inssamp		i*4	 I	offsets for the sample coordinates of 
C					input file

C  INTERNAL VARIALBES
C	Name		Type 	I/O	Description
C	----		----	---	-----------
C	temx1		r*4	 O	top sample location of hole
C	temx2		r*4 	 O	bottom location of hole
C	temy1		r*4	 O	top line location of hole
C	temy2		r*4	 O	bottom location of hole
C	hnum		C*5	 O	hole number
C	maxx		C*6	 O 	char. representation of top sample
C					location of hole
C	minx		C*6	 O 	char. representation of bottom sample
C					location of hole
C	maxy		C*6	 O 	char. representation of top line
C					location of hole
C	miny		C*6	 O 	char. representation of bottom line
C					location of hole

	real*4 oybuf(1)
	real*4 oxbuf(1)
	integer*4 temx1
	integer*4 temx2
	integer*4 temy1
	integer*4 temy2
	integer*4 tem

	integer*4 cnt
	integer*4 i
	integer*4 insline(2)
	integer*4 inssamp(2)
	integer*4 j
	integer*4 oid(100)
	integer*4 temp1
	integer*4 temp2

	character*5 hnum
	character*6 maxx
	character*6 maxy
	character*6 minx
	character*6 miny
	character*(ERRLEN) msg

	call errmsg('Two input polygons do not overlap correctly'
     .			,'intcut-overlap',1)
	temp2=1
	temp1= oid(3)
	cnt = 1

C  Find the minimum and maximum line, sample locations for each hole
C-------------------------------------------------------------------
	do 100 i = 2 , oid(2) , 2
	   temy1 = MINLONG
	   temx1 = MINLONG
	   do 10 j = temp2,temp1
              tem = oybuf(j)
	      temy1 = max(temy1,tem)
              tem = oxbuf(j)
	      temx1 = max(temx1,tem)
10	   continue
	
	   temp2 = temp2 + temp1
	   temp1 = temp1 + oid(i + 2)

	   temy2 = MAXLONG
	   temx2 = MAXLONG
	   do 20 j = temp2,temp1
              tem = oybuf(j)
	      temy2 = min(temy2,tem)
              tem = oxbuf(j)
	      temx2 = min(temx2,tem)
20	   continue

C  Subtract offsets and convert to text to output in the errmsg
C--------------------------------------------------------------
	   write (hnum,'(I5)') cnt
	   write (miny,'(F6.0)') temy1 - insline(1) + 1
	   write (minx,'(F6.0)') temx1 - inssamp(1) + 1
	   write (maxy,'(F6.0)') temy2 - insline(1) + 1
	   write (maxx,'(F6.0)') temx2 - inssamp(1) + 1

	   msg = 'Hole '//hnum//' is between '//miny//','//minx/
     .	   /' and '//maxy//','//maxx//' of INFILE1.'
	   call errmsg (msg,'intcut-overlap',1)
	   
C  Subtract offsets and convert to text to output in the errmsg
C--------------------------------------------------------------
	   write (miny,'(F6.0)') temy1 - insline(2) + 1
	   write (minx,'(F6.0)') temx1 - inssamp(2) + 1
	   write (maxy,'(F6.0)') temy2 - insline(2) + 1
	   write (maxx,'(F6.0)') temx2 - inssamp(2) + 1

	   msg = 'Hole '//hnum//' is between '//miny//','//minx/
     .	   /' and '//maxy//','//maxx//' of INFILE2.'
	   call errmsg (msg,'intcut-overlap',1)

	   cnt = cnt + 1
100	continue
	call errmsg ('Fatal error encountered','intcut-fatal',-1)

	return
	end
