C     *************************************************************************
C     NAME 			MERGE       	    		
C
C     PURPOSE	   Transfers data into an existing IBIS tabular file from 
C		   a second IBIS tabular file based on the equality of 
C		   specified values.
C	
C		   NOTE:  The input files must be sorted into ascending
C			  order on the columns used as the key columns.
C
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	JPL(A. Zobrist)  		        Original
C	K. Zanter	  March l987		LAMS conversion  
C	B. Ailts	  Nov 1988		Las 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	MERGE must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  The input files are
C     opened.  The values from the key columns in the first row
C     are read from the first input file.  This is compared to
C     the first key read from the second input file.  Keys are
C     read sequentially from rows of the second file until either
C     a match is made with the first key or the second key becomes
C     greater than the first indicating that there is no match for
C     the first key.  Either the row number where the match was
C     found or a zero value, meaning no match, is stored in an array.
C     The next key is then read from the first file and compared to
C     the current key from the second file.  Processing continues
C     as described above for the remainder of the files.  The input
C     file is then updated as indicated by the values in the array.
C     
C     ALGORITHM REFERENCES 		none
C     *****************************************************************
	program merge

	include 'las.fin'

	include 'lams.fin'

	real*4		abuf(MMXCOL)
	real*4		bbuf(MMXCOL)
	real*4		defval(MMXCOL)

	integer*4	clen1
	integer*4	clen2
	integer*4	col(MMXROW)
	integer*4	getnam
	integer*4	i
	integer*4	icount
	integer*4	fdesc1
	integer*4	fdesc2
	integer*4	incol(MMXCOL)
	integer*4	irow
	integer*4	jrow
	integer*4	key1(MMXCOL)
	integer*4	key2(MMXCOL)
	integer*4	ncol1
	integer*4	ncol2
	integer*4	nkey1
	integer*4	nkey2
	integer*4	null
	integer*4	ocount
	integer*4	ojrow
	integer*4	outcol(MMXCOL)

      	integer*4	parblk(XPRDIM)

	integer*4	status
	integer*4	stdout
	integer*4	tbclos
	integer*4	tbopen
	integer*4	tbrrow
	integer*4	tbwrow

	character*(CMLEN)	fname1
	character*(CMLEN)	fname2
	character*(CMLEN)	msg

C	Place the large variables in a common block to work around a
C	recent change in the sgi compiler that causes core dumps when
C	large variables are declared on the stack
	common parblk


C	initialization
C	--------------
	call inlas(parblk,stdout)

C	get the first input file name (one to be modified)
C	--------------------------------------------------
	status = getnam(parblk,'INFILE1',1,fname1,'ibis_tab',0)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','merge-fatal',-1)
	endif

C	open first IBIS tabular file (to be modified)
C	---------------------------------------------
	status = tbopen(fdesc1,fname1,clen1,ncol1,3)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening first input file','merge-open',1)
	   call errmsg('Fatal error encountered','merge-fatal',-1)
	endif

C	get the second input file name
C	------------------------------
	status = getnam(parblk,'INFILE2',1,fname2,'ibis_tab',0)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','merge-fatal',-1)
	endif

C	open second IBIS tabular file
C	-----------------------------
	status = tbopen(fdesc2,fname2,clen2,ncol2,0)
	if (status .ne. ESUCC) then
	   call errmsg('Error opening second input file','merge-open',1)
	   call errmsg('Fatal error encountered','merge-fatal',-1)
	endif

C	get remaining parameters
C	------------------------
	call xrintg(parblk,'KEYCOL1',MMXCOL,key1,nkey1,status)
	call xrintg(parblk,'KEYCOL2',MMXCOL,key2,nkey2,status)
	call xrintg(parblk,'INCOL',MMXCOL,incol,icount,status)
	call xrintg(parblk,'OUTCOL',MMXCOL,outcol,ocount,status)
	call xrreal(parblk,'DEFVAL',MMXCOL,defval,null,status)

C	validation
C	----------
1       FORMAT('The number of values specified for the INCOL, ',
     +	       'OUTCOL, and DEFVAL parameters are not equal')

2	FORMAT('The number of values specified for KEYCOL1 ',
     +	        'and KEYCOL2 parameters are not equal')

	if (icount .ne. ocount) then
	   write (msg,1)
	   call errmsg(msg,'merge-unequal',1)
	   call errmsg('Fatal error encountered','merge-fatal',-1)
	endif

	if (null .ne. 0) then
	   if (null .ne. ocount) then
	      write (msg,1)
	      call errmsg(msg,'merge-unequal',1)
	      call errmsg('Fatal error encountered','merge-fatal',-1)
	   endif
	endif

	if (nkey1 .ne. nkey2) then
	   call errmsg(msg,'merge-key',1)
	   call errmsg('Fatal error encountered','merge-fatal',-1)
	endif

C 	read and compare keys
C	---------------------
        ojrow = 0
        jrow = 1
        do 50 irow = 1,clen1
           col(irow) = 0
           status = tbrrow(fdesc1,key1,nkey1,irow,abuf)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','merge-fatal',-1)
	   endif

    5      if (jrow .ne. ojrow) then
             status = tbrrow(fdesc2,key2,nkey2,jrow,bbuf)
	     if (status .ne. ESUCC) then
	        call errmsg('Fatal error encountered','merge-fatal',-1)
	     endif
	   endif
	
           ojrow = jrow

           do 10 i = 1,nkey1
              if (abuf(i) - bbuf(i) .lt. 0.0) goto 50
              if (abuf(i) - bbuf(i) .gt. 0.0) goto 20
   10      continue
           col(irow) = jrow
           go to 50

   20      jrow = jrow + 1
           if (jrow .gt. clen2) go to 51
           go to 5
   50   continue
   51   continue

C       update the appropriate columns (OUTCOL) 
C	---------------------------------------
        ojrow = 0
        jrow = 1
        do 110 irow = 1,clen1
           status = tbrrow(fdesc1,outcol,ocount,irow,abuf)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','merge-fatal',-1)
	   endif

           if (col(irow) .eq. 0) go to 90

   70      status = tbrrow(fdesc2,incol,icount,jrow,abuf)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','merge-fatal',-1)
	   endif

           if (jrow .eq. col(irow)) go to 80
           jrow = jrow + 1
           go to 70

   80      continue
           status = tbwrow(fdesc1,outcol,ocount,irow,abuf)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','merge-fatal',-1)
	   endif
           go to 110

   90      if (null .ne. 0) then
             status = tbwrow(fdesc1,outcol,ocount,irow,defval)
	   else
             status = tbwrow(fdesc1,outcol,ocount,irow,abuf)
	   endif
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','merge-fatal',-1)
	   endif
  110   continue

C	close the IBIS files
C	--------------------
	status = tbclos(fdesc1,fname1)
	   if (status .ne. ESUCC) then
	      call errmsg('Error closing input file','merge-close',1)
	   endif

	status = tbclos(fdesc2,fname2)
	   if (status .ne. ESUCC) then
	      call errmsg('Error closing input file','merge-close',1)
	   endif

C	output completion message
C	-------------------------
	call complt('MERGE',fname1,1)

	end
