/*****************************************************************************
   NAME:                          BLACK

   FUNCTION:
     Blackens the image line Y = YLINE.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      9/99    G.Schmidt          CSB      FORTRAN to C conversion
*****************************************************************************/
#include <math.h>
#include "pixman.h"
#include "las.h"
#include "pmask.h"

void black
(
  long yline,           /* I: line to be blackened Y = yline         */
  long intarc[],        /* I: arc intersections (needs to be zero based) */
  long nedges,          /* I: number of edges in xvert and yvert     */
  int clokws,           /* I: flag to specify a clockwise direction  */
  float inset,          /* I: fill value for the pxfill call         */
  float xvert[],        /* I: x vertices                             */
  float yvert[],        /* I: y vertices                             */
  float slope[],        /* I: slope of the lines                     */
  long nspic,           /* I: number of samples in the image         */
  long window[][4],     /* I: window of the input images             */
  long ib,              /* I: index for band offset (0-based)        */
  long offset[],        /* I: band offset array                      */
  float bufout[]        /* O: output buffer                          */
)
{

    long dtype;           /* temporary data type                         */
    long i, j, k;         /* looping variables                           */
    long ipix[50][2];     /* storage for the pixels                      */
    long iseg1 = -1;      /* image segments                              */
    long iseg2 = -1;
    long nblack;          /* number of black pixels                      */
    long nseg;            /* number of segments                          */
    long xinter[100];     /* points of intersection of the line and arcs */
    long temp;            /* temporary window value                      */

    if (nedges == 0)
    {
        if (clokws)
        {
            dtype = EREAL;
            c_pxfill ((unsigned char *) &bufout[offset[ib]], &dtype,
                      &window[0][3], (unsigned char *) &inset);
        }
    }
    else
    {
        /* Find the points of intersection between the line and arcs.
           ----------------------------------------------------------*/
        for (i = 0; i < nedges; i++)
        {
            k = intarc[i];
            xinter[i] = (long)(xvert[k] + slope[k] *
                               ((float) yline - yvert[k]));
            if (fabs (slope[k]) > 100.0)
                xinter[i] = (long) xvert[k];
            if (window[0][1] != 1)
                (xinter[i])++;
        }

        /* Determine the ends of black strips depending upon the direction
           of the leftmost arc.
           ---------------------------------------------------------------*/
        if (clokws)
        {
            /* Leftmost arc looks up (nose < tail).
               ------------------------------------*/
            nseg = nedges/2 + 1;
            for (i = 0; i < nseg; i++)
            {
                if (i != 0)
                    ipix[i][0] = xinter[2*i-1];
                ipix[i][1] = xinter[2*i];
            }
            ipix[0][0] = -MAXNS;
            ipix[nseg-1][1] = MAXNS;
        }
        else
        { /* not clokws */
            /* Leftmost arc looks down (nose > tail).
               --------------------------------------*/
            nseg = nedges/2;
            for (i = 0; i < nseg; i++)
            {
                ipix[i][0] = xinter[2*i];
                ipix[i][1] = xinter[2*i+1];
            }
        }

        /* Clean the list of segment ends.
           -------------------------------*/
        for (i = 0; i < nseg; i++)
        {
            iseg1 = i;
            if (ipix[i][0] > window[0][1])
                break;
            if (ipix[i][1] < window[0][1])
                continue;
            ipix[i][0] = window[0][1];
            break;
        }

        for (i = 0; i < nseg; i++)
        {
            j = nseg - i - 1;
            iseg2 = j;
            if (ipix[j][1] < nspic)
                break;
            if (ipix[j][0] > nspic)
                continue;

            if (window[0][1] != 1)
                ipix[j][1] = nspic + 2;
            else
            {
                ipix[j][1] = nspic - 1;
                if (ipix[j][1] - ipix[j][0] >= 0)
                    ipix[j][1] = nspic + 1;
            }
            break;
        }
		
        /* Paint the line of the image.
           ----------------------------*/
        for (i = iseg1; i <= iseg2; i++)
        {
            nblack = ipix[i][1] - ipix[i][0];
            if (nblack <= 1)
                continue;
            if (window[0][1] == 1)
                temp = -1;
            else
                temp = window[0][1];
            dtype = EREAL;

            c_pxfill ((unsigned char *) &bufout[offset[ib] + ipix[i][0]- temp - 2], 
				&dtype, &nblack, (unsigned char *) &inset);
        }
    }
}
