/****************************************************************************
   NAME:                          GETPAR

   FUNCTION:
      GETPAR retrieves the parameters.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      9/99    G. Schmidt        CSB       Original Development
****************************************************************************/
#include <string.h>
#include "las.h"
#include "pmask.h"
#include "util.h"     /* c_inlas, c_getin, c_getout proto */
#include "tae.h"      /* p_find proto */

void getpar
(
  struct PARBLK *vblock,  /* O: TAE parameter block                        */
  char tname[][CMLEN],    /* O: TAE input filenames of images specified    */
  char hosin[][CMLEN],    /* O: host input filenames of images specified   */
  char taeout[][CMLEN],   /* O: TAE output filename                        */
  char hosout[][CMLEN],   /* O: host output filename                       */
  char infile[CMLEN],     /* O: TAE INFILE filename                        */
  long *images,           /* O: number of images input from TAE            */
  long window[][4],       /* O: window of the input files                  */
  long bands[][MAXBND+1], /* O: image bands of the input files             */
  long numbnd[],          /* O: number of bands per image                  */
  long *totbnds,          /* O: total number of bands specified            */
  long *ndata,            /* O: number of elements in datarr               */
  float datarr[],         /* O: LSCOORS values                             */
  long *mask,             /* O: input mask value                           */
  long *scale,            /* O: input scaling factor                       */
  char cutarea[],         /* O: input cutarea value (IN, OUT, TBD)         */
  long *msgtim,           /* O: time between display messages              */
  char minmax[]           /* O: minmax flag                                */
)
{
    long access = IREAD;       /* reading the infile                         */
    long fatal = -1;           /* fatal error flag                           */
    long i;                    /* looping variable                           */
    long imglimit;             /* maximum number of images allowed           */
    long one = 1;              /* constant one                               */
    long qflag[MAXIMG][2];     /* window and/or band flag                    */
    long same;                 /* 1 if all windows are the same, 0 if not    */
    long status;               /* return status                              */
    long std_out = 0;          /* standard output                            */
    long zero = 0;             /* constant zero                              */

    struct VARIABLE *var;      /* pointer to TAE par block var               */

    c_inlas (vblock, &std_out);

    /* Get the input image and its attributes
       --------------------------------------*/
    imglimit = MAXIMG;
    status = c_getin(vblock, "IN", &zero, tname, hosin, images, window, bands,
            numbnd, totbnds, &imglimit, &same, qflag);
    if (status == E_FAIL)
    {
        c_errmsg("Error getting the IN parameter", "pmask-getin", &one);
        c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
    }

    if (same == 0)
    {
        c_errmsg("Input images do not have the same window size",
                 "pmask-window", &one);
        c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
    }

    /* Get the LSCOORS value
       ---------------------*/
    var = p_find(vblock, "LSCOORS");
    *ndata = var->v_count;
    for (i = 0; i < *ndata; i++)
        datarr[i] = RVAL(*var,i);

    /* Get the INFILE value only if LSCOORS was not specified
       ------------------------------------------------------*/
    if (*ndata <= 0)
    {
        if (c_getnam (vblock, "INFILE", &zero, infile, "ibis_graph", &access)
                != E_SUCC)
        {
            c_errmsg("Error retrieving the INFILE name", "pmask-infile", &one);
            c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
        }
    }

    /* Get the output image name
       -------------------------*/
    status = c_getout(vblock, "OUT", taeout, hosout);
    if (status == E_FAIL)
    {
        c_errmsg("Error getting the OUT parameter", "pmask-getout", &one);
        c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
    }

    /* Get the MASKVAL
       ---------------*/
    var = p_find(vblock, "MASKVAL");
    *mask = IVAL(*var, 0);

    /* Get the SCALFACT
       ----------------*/
    var = p_find(vblock, "SCALFACT");
    *scale = IVAL(*var, 0);

    /* Get the CUTAREA
       ---------------*/
    var = p_find(vblock, "CUTAREA");
    strcpy (cutarea, SVAL(*var, 0));

    /* Get the $MSGTIM
       ---------------*/
    var = p_find(vblock, "$MSGTIM");
    *msgtim = IVAL(*var, 0);

    /* Get the $MINMAX
       ---------------*/
    var = p_find(vblock, "$MINMAX");
    strcpy (minmax, SVAL(*var, 0));
}
