/******************************************************************************
  NAME:	PMASK

  FUNCTION: 
     To cut out the portion of an image delineated by the set of polygons,
     contained in the IBIS graphics file.  Depending on the direction of
     the polygons, the area outside (clockwise polygon) or inside (counter-
     clockwise polygon) the polygons will be painted into the specified
     color (set to the specified DN value).  The rest of the image will
     remain intact.

  EXTERNAL ASSOCIATIONS:
	PMASK is a LAMS module running under las.

  PROGRAM HISTORY:
	Version		Date	Author		Code/Conr	Request
	-------		----	------		---------	-------
	  1.0			B.GOKHMAN	JPL/VICAR
	  2.0		7/86	B.AILTS		CSB		Conversion to
								unix(newLAS)
	  5.0		1/89	B.Ailts		EDC/CSB		Conversion to 
								LAS 5.0
	  5.1		8/90	D.Etrheim	EDC/CSB		ADP #5484 
								Error out 
							if maximum number of
							points in polygon is 
							exceeded. PRF #5511 Add
						       multiple band capability
          5.2		9/90	D.Etrheim	EDC/CSB 	ADP #A5791
							Added CUTAREA parameter
          5.3          11/91    T.Mittan        CSB     Corrected for 
							horizontal lines
          5.4           3/93	D.Etrheim	CSB	Corrected a wrapping
							problem when 
							CUTAREA=OUT
	  5.5		4/95	S.Nelson	CSB	Corrected error where
							the first sample of
							original data remained
							on certain lines.
							Modified subroutine
							BLACK.
          5.6          12/96	D.Etrheim	CSB	Raise limit to MAXNS
          5.7           1/99    L.Johnson       CSB     Changed parb to real*8
                                                       for ALL platforms due
                                                       to apparent alignment
                                                       problems between
                                                       Fortran and C.
          7.2           9/99    G.Schmidt       CSB    FORTRAN to C conversion

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

  PROJECT:	las(LAMS)

  ALGORITHM DESCRIPTION:

  After reading the input parameters which consist of the following:

  IN and OUT are opened for read and write respectively.  If specified, 
  INFILE is opened and the line/sample locations are read, otherwise, the 
  line/sample locations are read from the LSCOORS parameter. If SCALFACT 
  is greater than one the window specification and the line/sample locations 
  are divided by SCALFACT and the input image header is read 
  to see if the image has been scaled by GEOMZ or not.  If the image has not 
  been scaled by GEOMZ , it will be reduced by a factor of SCALFACT.

  The slope and direction of the polygon lines are calculated.  The image
  is processed in the following manner if the polygon is clockwise in 
  direction:

  The lines before the polygon begins are written to the output image 
  with a pixel value of VALUE.
  The left and right edges of the polygon are calculated.
  The samples before the polygon's left edge are written to the output 
  image with a pixel value of VALUE.
  The samples within the polygon are copied from the input image to the 
  output image.
  The samples after the right edge of the polygon are written to the 
  output image with a pixel value of VALUE.
  The lines after the polygon ends are written to the output image with
  a pixel value of VALUE.

  The image is processe in the following manner if the polygon is 
  clockwise in direction:

  The lines before the polygon begins are copied from the input image to 
  the output image.
  The left and right edges of the polygon are calculated.
  The samples before the polygon's left edge are copied from the input 
  image to the output image.
  The samples within the polygon are written to the output image with a 
  pixel value of VALUE.
  The samples after the right edge of the polygon are copied from the
  input image to the output image.
  The lines after the polygon ends are copied from the input image to 
  the output image.

  After every line is processed and written to OUT; all the files are 
  closed, the history and ddr files are updated and the completion message 
  is displayed.

******************************************************************************/
#include <math.h>
#include <string.h>
#include "las.h"
#include "lams.h"
#include "util.h"
#include "imageio.h"
#include "ddr.h"
#include "pixman.h"
#include "pmask.h"

extern long curent;       /* current entity being processed */

void main ()
{
    struct FDESC *fdesc1[MAXIMG]; /* file descriptor of IN                   */
    struct FDESC *fdesc2[2];      /* file descriptor of OUT                  */
    struct MFDESC *fdesc3;        /* file descriptor of INFILE               */
    struct GDESC *gdesc;          /* group descriptor for OUT                */

    double minv[MAXBND];       /* minimum intensity values array             */
    double maxv[MAXBND];       /* maximum intensity values array             */
    double lineinc;            /* line increment                             */
    double sampinc;            /* sample increment                           */

    float *bufin = NULL;       /* input buffer                               */
    float *bufout = NULL;      /* output buffer                              */
    float datarr[100];         /* LSCOORS values                             */
    float grbuf[MMXPT];        /* group buffer                               */
    float inset;               /* fill value (pmask value)                   */
    float slope[MMXPPR];       /* slope of the lines                         */
    float temp;                /* temporary floating point variable          */
    float xvert[MMXPPR];       /* x vertices                                 */
    float ys1 = -1;
    float ys2 = -1;
    float yswat1 = -1;
    float yswat2;
    float yvert[MMXPPR];       /* y vertices                                 */
    float yvsort[MMXPPR];      /* sorted y vertices                          */

    long access = IREAD;       /* file access type                           */
    long band;                 /* looping variable to loop through numbnd    */
    long bands[MAXIMG][MAXBND+1]; /* image bands of the input files          */
    long buf_size;             /* size of the user buffer pool               */
    long dspflag;              /* format display flag for c_inittm           */
    long dtype;                /* temporary data type                        */
    long fatal = -1;           /* fatal error flag                           */
    long i, j, k;              /* looping and index variables                */
    long ib;                   /* index for band offset                      */
    long images;               /* number of images input from TAE            */
    long imgcnt;               /* loop variable to loop through the images   */
    long ivert;                /* variable for the count of vertices
                                  (needs to be zero-based)                   */
    long iptr[MMXPPR];         /* array of indices for listin and listout 
                                  (needs to be zero-based)                   */
    long intarc[MMXPPR/2];     /* arc intersections (needs to be zero based) */
    long listin[10];         /* list of input vertices (needs to be 0-based) */
    long listout[10];        /* list of output vertices (needs to be 0-based) */
    long mask;                 /* input mask value                           */
    long max_nspic = -1;       /* max number of samples in the image         */
    long msgtim;               /* time between display messages              */
    long ndata;                /* number of elements in datarr               */
    long nedges;               /* number of edges in xvert and yvert         */
    long nlgrap;               /* number of lines in the group               */
    long nlpic;                /* number of lines in the image               */
    long npairs;               /* number of points available for the polygon */
    long npoly;                /* number of values in the respective array   */
    long nsgrap;               /* number of samples in the group             */
    long nspic;                /* number of samples in the image             */
    long numbnd[MAXIMG+1];     /* number of bands per image                  */
    long nvert;                /* number of vertices                         */
    long odtype;               /* data type of the images                    */
    long offset[MAXBND+1];     /* band offset array                          */
    long one = 1;              /* constant one                               */
    long opt[2];               /* option array for c_eopen* call             */
    long outpix;
    long sca;                  /* scale factor read from the image           */
    long scale;                /* input scaling factor                       */
    long sflag = -1;           /* scale flag                                 */
    long sl2;                  /* starting line                              */
    long ss2;                  /* starting sample                            */
    long status;               /* return status                              */
    long temp_size;            /* temporary window size                      */
    long tmpflag;              /* temporary flag for updating the DDR        */
    long totbnds;              /* total number of bands specified            */
    long window[MAXIMG][4];    /* window of the input files                  */
    long yline;                /* line value for Y = YLINE                   */
    long zero = 0;             /* constant zero                              */

    char infile[CMLEN];        /* TAE INFILE filename                        */
    char hosin[MAXIMG][CMLEN]; /* host input filenames of images specified   */
    char hosout[1][CMLEN];     /* host output filename                       */
    char taeout[1][CMLEN];     /* TAE output filename                        */
    char tname[MAXIMG][CMLEN]; /* TAE input filenames of images specified    */
    char minmax[16];           /* minmax flag                                */
    char cutarea[4];           /* input cutarea value (IN, OUT, TBD)         */

    int newpol;
    int clokws;                /* flag to specify a clockwise direction      */
    int flag = FALSE;          /* boolean flag used in direct.c              */

    struct PARBLK vblock;      /* TAE parameter block                        */


    /* Initialize certain variables
       ----------------------------*/
    for (i = 0; i < MMXPPR; i++)
        slope[i] = 99999.0;
    sl2 = one;
    ss2 = one;

    /* Get TAE parameters
       ------------------*/
    getpar (&vblock, tname, hosin, taeout, hosout, infile, &images, window,
            bands, numbnd, &totbnds, &ndata, datarr, &mask, &scale, cutarea,
            &msgtim, minmax);

    /* Calculate the number of bands requested
       ---------------------------------------*/
    inset = (float) mask;
    dtype = EREAL;
    c_inmnmx (&dtype, &totbnds, minv, maxv);

    /* Open input images
       -----------------*/
    access = IREAD;
    for (i = 0; i < images; i++)
    {
        nlpic = window[i][2] + window[i][0] - 1;
        nspic = window[i][3];
        opt[0] = zero;
        opt[1] = 0;  /* use the COMPRES1 environment variable */
        status = c_eopenr (&fdesc1[i], hosin[i], &numbnd[i], &one,
                           &window[i][1], &nlpic, &nspic, &access, &dtype, opt,
                           &one);
        if (status != E_SUCC)
        {
            c_errmsg("Cannot open input image file","pmask-open", &one);
            c_errmsg("Fatal error encountered","pmask-fatal",&fatal);
        }

        /* Keep track of the maximum number of samples so we can allocate
           the buffer to the maximum size needed.
           --------------------------------------------------------------*/
        if (nspic > max_nspic)
            max_nspic = nspic;
    }

    /* Dynamically allocate space for bufin 
       ------------------------------------*/
    if ((bufin = (float *) malloc (EREALSIZE * max_nspic * totbnds)) == NULL)
    {
        c_errmsg("Error allocating memory", "pmask-alloc", &one);
        c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
    }

    /* Get the type of the input images
       --------------------------------*/
    status = c_gettyp (hosin, &images, &odtype);
    if (status != E_SUCC)
    {
        c_errmsg("Error getting the type of hosin", "pmask-gettyp", &one);
        c_errmsg("Fatal error encountered","pmask-fatal",&fatal);
    }

    /* Read scale factor from image header and check for errors
       --------------------------------------------------------*/
    sca = 1;
    status = c_rdsca (hosin[0], &sca);
    if ((status != E_SUCC) && (scale != 1))
    {
        c_errmsg("Error encountered retrieving scale factor", "pmask-scale",
                &one);
        c_errmsg("Fatal error encountered","pmask-fatal",&fatal);
    }

    if ((sca != scale) && (scale != 1) && ((sca != 1) && (sca != 0)))
    {
        c_errmsg("Scale factor of input image and SCALFACT are not equal",
                "pmask-sf", &one);
        c_errmsg("Fatal error encountered","pmask-fatal",&fatal);
    }
    else if ((sca == 1) && (scale == 1))
        sflag = 1;
    else if (sca == scale)
        sflag = 1;
    else
        sflag = scale;

    /* Apply scaling factor to window
       ------------------------------*/
    if (window[0][2] <= sflag)
        window[0][2] = 1;
    else
    {
        temp = (float) window[0][2];
        temp /= (float) sflag;
        window[0][2] = (long) temp;
    }

    if (window[0][3] <= sflag)
        window[0][3] = 1;
    else
    {
        temp = (float) window[0][3];
        temp /= (float) sflag;
        window[0][3] = (long) temp;
    }

    /* Recalculate NLPIC and NSPIC in case scaling was done
       ----------------------------------------------------*/
    nlpic = window[0][2] + window[0][0] - 1;
    nspic = window[0][3] + window[0][1] - 1;

    /* Read in and process the polygon vertices
       ----------------------------------------*/
    listout[0] = 0;
    npoly = 0;
    ivert = 0;
    newpol = TRUE;

    /* Read the INFILE file
       --------------------*/
    if (ndata <= 0)
    {
        access = IREAD;
        status = c_gropen (&fdesc3, infile, &nsgrap, &nlgrap, &access);
        if (status != E_SUCC)
        {
            c_errmsg("Cannot open INFILE file", "pmask-open", &one);
            c_errmsg("Fatal error encountered","pmask-fatal",&fatal);
        }

        npairs = nsgrap * nlgrap / 2;
        for (i = 1; i <= nlgrap; i++)
        {
            status = c_grread (&fdesc3, &i, grbuf);
            if (status != E_SUCC)
            {
                c_errmsg("Error encountered reading INFILE file", "pmask-read",
                         &one);
                status = c_grclos (&fdesc3, infile);
                c_errmsg("Fatal error encountered","pmask-fatal",&fatal);
            }

            polyin (npairs, &newpol, &ivert, scale, grbuf, xvert, yvert,
                    listin, listout, &npoly);
        }

        /* Close INFILE
           ------------*/
        status = c_grclos (&fdesc3, infile);
    }
    else
    {
        /* Read the vertices from the param statement
           ------------------------------------------*/
        polyin (ndata/2, &newpol, &ivert, scale, datarr, xvert, yvert, listin,
                listout, &npoly);
    }

    nvert = ivert;

    /* Open output image
       -----------------*/
    access = IWRITE;
    dtype = EREAL;
    opt[0] = odtype;
    opt[1] = 0;  /* use the COMPRES1 environment variable */
    status = c_eopens (&fdesc2[0], hosout[0], &totbnds, &sl2, &ss2,
                       &window[0][2], &window[0][3], offset, &access, &dtype,
                       &zero, opt);
    if (status != E_SUCC)
    {
        c_errmsg("Cannot open output image file", "pmask-open", &one);
        closelas (fdesc1, images, NULL);
        c_clean (&vblock, "pmask", hosout, &one);
    }
    fdesc2[1] = NULL;

    if ((bufout = (float *) c_ealloc(fdesc2, &buf_size)) == NULL)
    {
        c_errmsg("Error allocating memory", "pmask-alloc", &one);
        c_errmsg("Fatal error encountered", "pmask-fatal", &fatal);
    }
    status = c_egroup (fdesc2, &gdesc, (unsigned char *)bufout, &buf_size);
    if (status != E_SUCC)
    {
        c_errmsg("Cannot group image files", "pmask-group", &one);
        closelas (fdesc1, images, NULL);
        c_clean (&vblock, "pmask", hosout, &one);
    }

	/* Calculate slopes and sort the vertices
       --------------------------------------*/
    for (i = 0; i < nvert; i++)
    {
        k = nose (i, listin, listout, npoly);
        if (fabs (yvert[k] - yvert[i]) >= 1.0E-6)
            slope[i] = (xvert[k] - xvert[i]) / (yvert[k] - yvert[i]);
        iptr[i] = i;
        yvsort[i] = yvert[i];
    }

    /* Sort the data array and the corresponding pointer array.
       --------------------------------------------------------*/
    r4sort (yvsort, iptr, nvert);

    status = c_estep (&gdesc);
    if (status != E_SUCC)
    {
        c_errmsg ("Error encountered writing to output image", "pmask-write",
                &one);
        closelas (fdesc1, images, gdesc);
        c_clean (&vblock, "pmask", hosout, &one);
    }

    /* Blacken the top of the image
       ----------------------------*/
    curent = 1;
    dspflag = ALLCAT;  /* processing all bands at once */
    c_inittm ("PMASK", "Processed", "line", "bands", " ", &window[0][2],
            &totbnds, &msgtim, &dspflag);
    c_startm();

    if (yvsort[0] > (float) window[0][0])
    {
        yswat2 = yvsort[0] - 1;
        edges (intarc, iptr, 0, &nedges, listin, listout, npoly);
        yline = (long) (yvsort[0] + 1.0);
        direct (yline, intarc, nedges, &clokws, xvert, yvert, slope, listin,
                listout, npoly, &flag);

        /* Set the direction if necessary
           ------------------------------*/
        if (!strncmp (cutarea, "IN", 2))
            clokws = TRUE;
        else if (!strncmp (cutarea, "OUT", 3))
            clokws = FALSE;

        for (i = window[0][0]; i <= (long) yswat2 * sflag; i += sflag)
        {
            curent++;
            ib = 0;

            for (imgcnt = 0; imgcnt < images; imgcnt++)
            {
                for (band = 0; band < numbnd[imgcnt]; band++, ib++)
                {
                    status = c_eread (&fdesc1[imgcnt], &bands[imgcnt][band],
                                      &i, (unsigned char *) bufin, &one);
                    if (status == E_FAIL)
                    {
                        c_errmsg ("Error encountered reading input image",
                                  "pmask-read", &one);
                        closelas (fdesc1, images, gdesc);
                        c_clean (&vblock, "pmask", hosout, &one);
                    }

                    if (clokws)
                    {
                        dtype = EREAL;
                        c_pxfill ((unsigned char *) &bufout[offset[ib]], &dtype,
                                &window[0][3], (unsigned char *) &inset);
                    }
                    else
                    {
                        if (sflag > 1)
                        {
                            outpix = 0;
                            for (j = 1; j <= window[0][3] * sflag; j += sflag,
                                    outpix++)
                                bufout[offset[ib] + outpix] = bufin[j-1];
                        }
                        else
                        {
                            dtype = EREAL;
                            c_pxcopy ((unsigned char *) bufin,
                                      (unsigned char *) &bufout[offset[ib]],
                                      &dtype, &window[0][3]);
                        }

                        if (!strncmp (minmax, "YES", 3))
                        {
                            dtype = EREAL;
                            c_pxmnmx ((long *)&bufout[offset[ib]],
                                      &window[0][3], &minv[ib], &maxv[ib],
                                      &dtype);
                        }
                    }
                }
            }

            status = c_estep (&gdesc);
            if (status != E_SUCC)
            {
                c_errmsg ("Error encountered writing to output image",
                          "pmask-write", &one);
                closelas (fdesc1, images, gdesc);
                c_clean (&vblock, "pmask", hosout, &one);
            }
        }
    }

    /* Process the lines intersecting the polygon
       ------------------------------------------*/
    for (i = 0; i < nvert - 1; i++)
    {
        edges (intarc, iptr, i, &nedges, listin, listout, npoly);
        if (yvsort[i] == yvsort[i+1])
            continue;

        yswat1 = yvsort[i];
        if (yvsort[i+1] == (float) nlpic)
            yswat2 = yvsort[i+1];
        else
            yswat2 = yvsort[i+1] - 1;

        if (yswat2 < yswat1)
            continue;
        if ((ys1 == yswat1) && (ys2 == yswat2))
            continue;
        if (yswat2 < (float) window[0][0])
            continue;

        direct ((long) ((yswat1+yswat2)/2.0), intarc, nedges, &clokws, xvert,
                yvert, slope, listin, listout, npoly, &flag);

        /* Set the direction if necessary
           ------------------------------*/
        if (!strncmp (cutarea, "IN", 2))
            clokws = TRUE;
        else if (!strncmp (cutarea, "OUT", 3))
            clokws = FALSE;

        for (yline = (long) yswat1 * sflag; yline <= (long) yswat2 * sflag;
                yline += sflag)
        {
            if (yline < window[0][0])
                continue;
            if (yline > nlpic * sflag)
                break;
            curent++;
            ib = 0;

            for (imgcnt = 0; imgcnt < images; imgcnt++)
            {
                for (band = 0; band < numbnd[imgcnt]; band++, ib++)
                {
                    status = c_eread (&fdesc1[imgcnt], &bands[imgcnt][band],
                                      &yline, (unsigned char *) bufin, &one);
                    if (status == E_FAIL)
                    {
                        c_errmsg ("Error encountered reading input image",
                                  "pmask-read", &one);
                        closelas (fdesc1, images, gdesc);
                        c_clean (&vblock, "pmask", hosout, &one);
                    }

                    if (sflag > 1)
                    {
                        outpix = 0;
                        for (j = 0; j < window[0][3] * sflag; j += sflag,
                                outpix++)
                            bufout[offset[ib] + outpix] = bufin[j];
                    }
                    else
                    {
                        dtype = EREAL;
						//copy one line
                        c_pxcopy ((unsigned char *)bufin,
                                  (unsigned char *) &bufout[offset[ib]],
                                  &dtype, &window[0][3]);
                    }
                    black (yline, intarc, nedges, clokws, inset, xvert, yvert,
                            slope, nspic, window, ib, offset, bufout);

                   if (!strncmp (minmax, "YES", 3))
                    {
                        dtype = EREAL;
                        c_pxmnmx ((long *)&bufout[offset[ib]],
                                  &window[0][3], &minv[ib], &maxv[ib], &dtype);
                    }
                }
            }

            status = c_estep (&gdesc);
            if (status != E_SUCC)
            {
                c_errmsg ("Error encountered writing to output image",
                          "pmask-write", &one);
                closelas (fdesc1, images, gdesc);
                c_clean (&vblock, "pmask", hosout, &one);
            }
        }

        if (yline > nlpic * sflag)
            break;

        ys1 = yswat1;
        ys2 = yswat2;
    }

    /* Blacken the bottom of the image
       -------------------------------*/
    if (yline <= nlpic * sflag)
        yswat1 = yvsort[nvert-1];

    if (((int) yswat1 < nlpic) && (yline <= nlpic * sflag))
    {
        for (i = (long) yswat1 * sflag; i <= nlpic * sflag; i += sflag)
        {
            curent++;
            ib = 0;

            for (imgcnt = 0; imgcnt < images; imgcnt++)
            {
                for (band = 0; band < numbnd[imgcnt]; band++, ib++)
                {
                    status = c_eread (&fdesc1[imgcnt], &bands[imgcnt][band],
                                      &i, (unsigned char *) bufin, &one);
                    if (status == E_FAIL)
                    {
                        c_errmsg ("Error encountered reading input image",
                                  "pmask-read", &one);
                        closelas (fdesc1, images, gdesc);
                        c_clean (&vblock, "pmask", hosout, &one);
                    }

                    if (clokws)
                    {
                        dtype = EREAL;
                        temp_size = window[0][3] + window[0][1];
                        c_pxfill ((unsigned char *) &bufout[offset[ib]], &dtype,
                                &temp_size, (unsigned char *) &inset);
                    }
                    else
                    {
                        if (sflag > 1)
                        {
                            outpix = 0;
                            for (j = 1; j <= window[0][3]; j += sflag, outpix++)
                                bufout[offset[ib] + outpix] = bufin[j-1];
                        }
                        else
                        {
                            dtype = EREAL;
                            c_pxcopy ((unsigned char *)bufin,
                                      (unsigned char *) &bufout[offset[ib]],
                                      &dtype, &window[0][3]);
                        }
                    }

                    if (!strncmp (minmax, "YES", 3))
                    {
                        dtype = EREAL;
                        c_pxmnmx ((long *)&bufout[offset[ib]], &window[0][3],
                                  &minv[ib], &maxv[ib], &dtype);
                    }
                }
            }

            status = c_estep (&gdesc);
            if (status != E_SUCC)
            {
                c_errmsg ("Error encountered writing to output image",
                          "pmask-write", &one);
                closelas (fdesc1, images, gdesc);
                c_clean (&vblock, "pmask", hosout, &one);
            }
        }
    }

    /* Finish the processing
       ---------------------*/
    curent = window[0][2];
    c_stoptm ();
    closelas (fdesc1, images, gdesc);

    /* Update the DDR file
       -------------------*/
    lineinc = 1.0;
    sampinc = 1.0;
    tmpflag = NOCOMB;
    status = c_upddr (&vblock, hosin, hosout[0], &images, window, bands,
                      numbnd, &lineinc, &sampinc, &totbnds, &tmpflag);
    if (status != E_SUCC)
    {
        c_errmsg("DDR file not updated", "pmask-warn", &one);
    }

    /* Save the min/max
       ----------------*/
    status = c_svmnmx (hosout[0], &totbnds, minv, maxv, minmax);
    if (status == E_FAIL)
    {
        c_errmsg("Unsuccessful save min/max", "pmask-warn", &one);
    }

    /* Update the history file
       -----------------------*/
    status = c_hstout(&vblock, hosout[0], hosin, &one, "PMASK");
    if (status != E_SUCC)
    {
        c_errmsg("History not created", "pmask-warn", &one);
    }

    /* Call complt to end the routine
       ------------------------------*/
    c_complt("PMASK", taeout, &one);
}
