/*****************************************************************************
NAME:				MAP_POLY

PURPOSE:  Maps polygon verticies from one reference frame to another, using
	  a Geometric Mapping Grid.  Options exist to perform a forward
	  mapping or an inverse mapping.  Another flag controls whether or not
	  points outside the mapping grid are transformed.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT 
-------		----	------		---------
  1.0		5/91	D. Steinwand	EROS-AB/ISS	    

ALGORITHM Reference:

  1.  "Matrix Computations (2nd ed.)", Gene H. Golub & Charles F. Van Loan,
      John Hopkins University Press, Baltimore 1989 
      ISBN 0-8018-3739-1

*****************************************************************************/
#include <math.h>
#include "las.h"
#include "worgen.h"
#include "geompak.h"
#include "typlim.h"
#include "protos.h"

double m1, b1;
double m2, b2;
double m3, b3;
double m4, b4;
double m5, b5;
double m6, b6;

int map_poly
(
    double *yx,		/* y,x pairs (y1,x1,y2,x2,...,yN,xN) */
    long N,			/* Number of y,x pairs */
    long inv,		/* Boolean--FALSE in->out; TRUE out->in */
    long outside,		/* Boolean--FALSE don't map; TRUE closest cell */
    struct GEOGRID *grid	/* Geometric Mapping Grid */
)
{
long index,i,j,k,m,n,x,y; /* Loop counters & indices */
double cross_term;	/* Cross term (x * y)  */
double *samps, *lines;	/* Pointers to input space grid (short-hand) */
long upleft, uprite, 
     loleft, lorite;	/* Indices to input grid cell corners */
long *mapped;		/* Boolean--one for each point--is it mapped? */
long *closest;		/* Which grid cell is closest to outling point? */
long first;		/* Boolean--saves recalculation of coefficients */
double A[16];		/* Matrix for QR factorization */
double v[4];		/* Work vector */
double a[4];		/* Coefficients mapping x */
double b[4];		/* Coefficients mapping y */
double center_x = 0.0;	/* Center coordinate of grid cell */
double center_y = 0.0;	/* Center coordinate of grid cell */
double temp;		/* Temporary variable */
double *min_dist;	/* Minimum distance from cell center array */
double dist;		/* Distatance from current cell center */
double factor;		/* Factor for Gaussian elimination */
double grid_x, grid_y;	/* Grid x,y for inverse mapping */

if (inv)

/* Perform inverse mapping (map output space to input space).  This mode of
   processing ignores the outside boolean...and always maps points outside.
  ------------------------------------------------------------------------*/
   for (y=0, x=1, j=0; j < N; j++, y+=2, x+=2)
	{
	grid_eval(grid, yx[x], yx[y], &grid_x, &grid_y);
	yx[x] = grid_x;
	yx[y] = grid_y;
	}
else
 {

/* Perform forward mapping (map input space to output space).
  ----------------------------------------------------------*/
 samps = grid->in_samps;
 lines = grid->in_lines;
 upleft = 0;
 uprite = 1;
 loleft = grid->ncols;
 lorite = loleft + 1;

/* Allocate memory
  ---------------*/
 mapped = (long *)malloc(N * sizeof(long));
 if (mapped == NULL) return(NO_SPACE);
 for (i=0; i<N; i++) mapped[i] = FALSE;

 closest = (long *)malloc(N * sizeof(long));
 if (closest == NULL) return(NO_SPACE);
 for (i=0; i<N; i++) closest[i] = 0;

 min_dist = (double *)malloc(N * sizeof(double));
 if (min_dist == NULL) return(NO_SPACE);
 for (i=0; i<N; i++) min_dist[i] = MAXREAL;

/* Loop through the Geometric Mapping Grid.  Find equations for each side of
   the current grid cell.  Then, loop through all points and test to see if
   the given point is contained in the grid cell.  If it is, calculate 
   coefficients and map the point...
  ---------------------------------*/
 for (i=0, m=0; m<(grid->nrows-1); m++, loleft++, lorite++, upleft++, uprite++)
  for (n=0; n<(grid->ncols-1); n++, i++, loleft++, lorite++, upleft++, uprite++)
   {
   find_bot(samps[loleft], samps[lorite], lines[loleft], lines[lorite]);
   find_rit(samps[uprite], samps[lorite], lines[uprite], lines[lorite]);
   find_top(samps[upleft], samps[uprite], lines[upleft], lines[uprite]);
   find_lft(samps[upleft], samps[loleft], lines[upleft], lines[loleft]);

   if (outside)  /* Find center points of grid cells for distance calc later */
	{
	find_cr1(samps[loleft], samps[uprite], lines[loleft], lines[uprite]);
	find_cr2(samps[upleft], samps[lorite], lines[upleft], lines[lorite]);

	factor = -(m6/m5);  /* Next 3 lines, Gaussian Elimination w/backsolve */
        center_y = (b5 * factor + b6) / (factor + 1);
	center_x = (b5 - center_y) / -(m5);
	}

   for (first = TRUE, y=0, x=1, j=0; j < N; j++, y+=2, x+=2)
	if (!mapped[j] && yx[y] < bot(yx[x]) && yx[y] > rit(yx[x]) 
	  && yx[y] > top(yx[x]) && yx[y] < lft(yx[x])) 
	   {
	   mapped[j] = TRUE;
	   if (first)
		{
	   	first = FALSE;
		for (k=0; k<4; k++) A[k] = 1.0;
		A[4] = samps[upleft];
		A[5] = samps[uprite];
		A[6] = samps[loleft];
		A[7] = samps[lorite];
	 	A[8] = lines[upleft];
	 	A[9] = lines[uprite];
	 	A[10] = lines[loleft];
	 	A[11] = lines[lorite];
		A[12] = samps[upleft]*lines[upleft];
		A[13] = samps[uprite]*lines[uprite];
		A[14] = samps[loleft]*lines[loleft];
		A[15] = samps[lorite]*lines[lorite];
		QR(A, 4, 4, v);  /* QR factorization w/Householder transforms */
		index = IMOD(i, (grid->ncols-1));
		a[0] = grid->out_samps[index];
		a[1] = grid->out_samps[index+1];
		a[2] = a[0];
		a[3] = a[1];
		index = (long)(i/(grid->ncols-1));
		b[0] = grid->out_lines[index];
		b[1] = b[0];
		b[2] = grid->out_lines[index+1];
		b[3] = b[2];
		QR_solve(A, 4, 4, v, a, 0);
		QR_solve(A, 4, 4, v, b, 0);
		}
	   cross_term = yx[x] * yx[y];
	   temp =  a[0] + a[1]*yx[x] + a[2]*yx[y] + a[3]*cross_term;
	   yx[y] = b[0] + b[1]*yx[x] + b[2]*yx[y] + b[3]*cross_term;
	   yx[x] = temp;
	   }
	else if (outside && !mapped[j])  /* Find closest cell */
	   {
	   dist = center_x - yx[x];
	   dist *= dist;
	   temp = center_y - yx[y];
	   temp *= temp;
	   dist += temp;
	   dist = sqrt(dist);
	   if (dist < min_dist[j]) { min_dist[j] = dist; closest[j] = i;}
           }
   }

/* Map points which fall outside of the geometric mapping grid
  -----------------------------------------------------------*/
 if (outside)
  {
  upleft = 0;
  uprite = 1;
  loleft = grid->ncols;
  lorite = loleft + 1;
  for (i=0, m=0; m<(grid->nrows-1); m++, loleft++, lorite++, upleft++, uprite++)
   for (n=0; n<(grid->ncols-1); n++,i++, loleft++, lorite++, upleft++, uprite++)
    {
    for (first = TRUE, y=0, x=1, j=0; j < N; j++, y+=2, x+=2)
        if (!mapped[j] && closest[j] == i)
           {
           mapped[j] = TRUE;
           if (first)
                {
                first = FALSE;
                for (k=0; k<4; k++) A[k] = 1.0;
                A[4] = samps[upleft];
                A[5] = samps[uprite];
                A[6] = samps[loleft];
                A[7] = samps[lorite];
                A[8] = lines[upleft];
                A[9] = lines[uprite];
                A[10] = lines[loleft];
                A[11] = lines[lorite];
                A[12] = samps[upleft]*lines[upleft];
                A[13] = samps[uprite]*lines[uprite];
                A[14] = samps[loleft]*lines[loleft];
                A[15] = samps[lorite]*lines[lorite];
                QR(A, 4, 4, v);
                index = IMOD(i, (grid->ncols-1));
                a[0] = grid->out_samps[index];
                a[1] = grid->out_samps[index+1];
                a[2] = a[0];
                a[3] = a[1];
                index = (long)(i/(grid->ncols-1));
                b[0] = grid->out_lines[index];
                b[1] = b[0];
                b[2] = grid->out_lines[index+1];
                b[3] = b[2];
                QR_solve(A, 4, 4, v, a, 0);
                QR_solve(A, 4, 4, v, b, 0);
                }
	   cross_term = yx[x] * yx[y];
	   temp =  a[0] + a[1]*yx[x] + a[2]*yx[y] + a[3]*cross_term;
	   yx[y] = b[0] + b[1]*yx[x] + b[2]*yx[y] + b[3]*cross_term;
	   yx[x] = temp;
	   }
   }
  }
 }
return(E_SUCC);
}

/* Find the equation of the line which passes through (x1,y1) & (x2,y2)
  --------------------------------------------------------------------*/
void find_top
(
    double x1,
    double x2,
    double y1,
    double y2
){m1=(y2-y1)/(x2-x1);b1=y1-m1*x1;return;}
void find_rit
(
    double x1,
    double x2,
    double y1,
    double y2
){m2=(y2-y1)/(x2-x1);b2=y1-m2*x1;return;}
void find_bot
(
    double x1,
    double x2,
    double y1,
    double y2
){m3=(y2-y1)/(x2-x1);b3=y1-m3*x1;return;}
void find_lft
(
    double x1,
    double x2,
    double y1,
    double y2
){m4=(y2-y1)/(x2-x1);b4=y1-m4*x1;return;}
void find_cr1
(
    double x1,
    double x2,
    double y1,
    double y2
){m5=(y2-y1)/(x2-x1);b5=y1-m5*x1;return;}
void find_cr2
(
    double x1,
    double x2,
    double y1,
    double y2
){m6=(y2-y1)/(x2-x1);b6=y1-m6*x1;return;}

/* Evalulate y = f(x) at the appropriate grid cell side
  ----------------------------------------------------*/
double top
(
    double x
) { return(m1*x+b1);}
double rit
(
    double x
) { return(m2*x+b2);}
double bot
(
    double x
) { return(m3*x+b3);}
double lft
(
    double x
) { return(m4*x+b4);}


/* Calculate the QR factorization of the m by n matrix A.  On return,
   matrix A will contain R in its upper triangle.  The strict lower triangle
   will contain information about Q.  Vector v is a work vector, but on
   return will contain additional information about Q.
  --------------------------------------------------*/
void QR
(
    double *A,	/* Matrix A (stored by columns) */
    long m,         /* Number of rows in matrix A */
    long n,         /* Number of columns in matrix A */
    double *v       /* Vector v */
)
{
long N;
long k,j;		/* Loop counters */
double *cur_col_A;	/* Pointer to current column of matrix A */
double vtv;

N = ((m-1) > n) ? n : (m-1);
for (k = 0; k < N; k++)
   {
   cur_col_A = A + (k * m);
   house ((m-k), &(cur_col_A[k]), &(v[k]), &vtv, &(cur_col_A[k]));
	
   for (j = k + 1; j < m; j++)
      cur_col_A[j] = v[j];

   row_house(&(A[(k+1)*m+k]), (m-k), (n-k-1), m, &(v[k]), vtv);
   v[k] = vtv;
   }
return;
}

/* Find the least squares solution of Ax = b where A is a full rank 
   m by n matrix with m >= n.  On entry, matrix A should contain the results
   of the QR routine.  If iflag == 0, then v is a vector of length m
   containing the results of the QR routine.  If iflag != 0, the entries
   in v are reconstructed--the vector v is not changed.  The solution will
   be stored in the first n entries of vector b.
  ---------------------------------------------*/
void QR_solve
(
    double *A,       /* Matrix A (stored by columns) */
    long m,         /* Number of rows in matrix A */
    long n,         /* Number of columns in matrix A */
    double *v,       /* Vector v */
    double *b,	/* Vector b */
    long iflag
)
{
long N;
long k,j;		/* Loop counters */
double *cur_col_A;       /* Pointer to current column of matrix A */
double sum;		/* Accumulator */
double vtv;
double alpha;

N = ((m-1) > n) ? n : (m-1);
for (k = 0; k < N; k++)
   {
   cur_col_A = A + (k * m);
   sum = b[k];
   for (j = k + 1; j < m; j++)
      sum += (b[j] * cur_col_A[j]);
   if (iflag == 0) vtv = v[k];
   else
      {
      vtv = 1.0;
      for (j = k + 1; j < m; j++)
         vtv += (cur_col_A[j] * cur_col_A[j]);
      }
   alpha = -2.0 / vtv * sum;
   b[k] += alpha;
   for (j= k + 1; j < m; j++)
      b[j] += (alpha * cur_col_A[j]);
   }

/* Backsolve
  ---------*/
b[n-1] /= A[(n-1)*m+n-1];
for (k = n-1; k--;)
   {
   cur_col_A = A + ((k+1) * m);
   for (j = 0; j < k+1; j++)
      b[j] -= (b[k+1] * cur_col_A[j]);
   b[k] /= A[k*m+k];
   }
return;
}

/* Given vector x of length n, find a vector v of length n such that 
   v[0] = 1, and for the Householder matrix P defined by v,
   Px = (Px, 0, ..., 0) transpose.
  -------------------------------*/
void house
(
    long n,		/* Input--dimension of vectors x and v */
    double *x,	/* Input--vector x */
    double *v,	/* Output--vector v */
    double *vtv,	/* Output--Product of vector v transpose and v */
    double *Px	/* Output--Product of Householder matrix P and vector x */
)
{
long i;		/* Loop counter */
double u;
long sign_x0 = 0;
double beta;

for (i = 0; i < n; i++)
   v[i] = x[i];
u = norm_2(x, n);
if (u != 0.0)
   {
   sign_x0 = sign(x[0]);
   beta = x[0] + sign_x0 * u;
   for (i = 1; i < n; i++)
      v[i] /= beta;
   *vtv = 2.0 * ((u * u) + (u * fabs(x[0]))) / (beta * beta);
   }
else *vtv = 1.0;
v[0] = 1.0;
*Px = -sign_x0 * u;
return;
}

/* Replace the m by n matrix A by the product PA, where P is the Householder
   Transformation defined by vector v.  vtv should contain the product of
   v transpose v when this routine is called.
  ------------------------------------------*/
void row_house
(
    double *A,	/* Matrix A (stored by columns) */
    long m,		/* Number of rows in matrix A */
    long n,		/* Number of columns in matrix A */
    long mdim,	/* Row dimension */
    double *v,	/* Vector v */
    double vtv	/* Product of vector v transpose and vector v */
)
{
double *cur_col_A;	/* Pointer to current column of matrix A */
double alpha;
long i,k;		/* Loop counters */

for (k = 0; k < n; k++)
   {
   cur_col_A = A + (k * mdim);
   for (alpha = 0.0, i = 0; i < m; i++)
      alpha += v[i] * cur_col_A[i];
   alpha *= -(2.0 / vtv);
   for (i = 0; i < m; i++)
      cur_col_A[i] += (alpha * v[i]);
   }
return;
}
   
/* Function to calculate the 2-norm of a vector
  --------------------------------------------*/
double norm_2
(
    double *x, 
    long n
)
{
long i; double sum = 0.0;
for (i = 0; i < n; i++) sum += x[i] * x[i];
return(sqrt(sum)); }

/* Function to return the sign of a number
  ---------------------------------------*/
int sign
(
    double val
) { if (val < 0.0) return(-1); return(1); }
