/******************************************************************************
  NAME:	POLYGEO

  FUNCTION: 
  POLYGEO PERFORMS GEOMETRIC TRANSFORMATIONS OF A POINT POLYGON DATA SET.

  PROGRAM HISTORY:
  Version		Date	Author		Code/Conr	Request
  -------		----	------		---------	-------
  1.0			A.L.SOBRIST	   JPL
  2.0		9/86	B.AILTS		   CSB		LAMS conversion
							to unix(las)
  3.0		1/87	B.AILTS		   CSB		ADD SKIP +
							INTERVAL
  5.0		1/89	B.Ailts		 EDC/CSB	Conversion to
							LAS 5.0
  6.0		5/91	B.Ailts/D.Steinwand EDC/CSB	Implemented a new 
							algorithm so that the
							transformed points
							are more accurate
  7.0		9/92	T. Mittan	EDC/CSB		Added extern to curent
							to eliminate warnings 
							on link

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Run under TAE

  PROJECT:	las(LAMS)

  ALGORITHM/DESCRIPTION: 
  After reading the input parameters
  INGRID and OUTFILE are opened for read and write respectively.  
  INFILE (if specified) is opened and the line/sample pairs are read in.  

  The line/sample locations of the input points are transformed 
  according to the line/sample pairs in INGRID.  The pairs form a 
  rectangular grid and the input point is transformed according to the 
  line/sample pairs associated with the four tiepoints that enclose that 
  point.  For points that lie outside of the grid, then the nearest 
  rectangle of the grid is used to transform the input points if PNTFLG 
  is set to YES, otherwise these points are left as is.  

  After all the points are processed and the points is written to the 
  output file; all the files are closed, the completion message is 
  displayed.

  ALGORITHM REFERENCES:

******************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "pixman.h"
#include "util.h"
#include "lams.h"
#include "geompak.h"
#include "protos.h"

extern long curent;

int main
(void)

{

struct PARBLK parb;
struct VARIABLE *val;           /* pointer to a TAE variable within v-block  */
struct GEOGRID *grid;
struct MFDESC *fdesc1;
struct MFDESC *fdesc2;

double *work;			/*  work buffer				*/

float *lines;			/*  I/O buffer				*/

long access = IREAD;
long cnt;
long dtype = EREAL;
long fatal = -1;
long i;
long iline;
long interval;
long invers;
long insect;
long jsk;
long jx;
long maxent;
long msgtim;
long msgtyp;
long nline;
long nonfatal = 1;
long npoints = MEMXPT;
long nsl;
long nsamp;
long one = 1;
long skip;
long rec1 = 0;
long rec2 = 0;
long std_out;
long status;
long zero = 0;

char ha1[CMLEN];
char ha2[CMLEN];
char ingrid[CMLEN];

short pdef;

/* INITIALIZE, GET PARAMETERS, OPEN FILES.
------------------------------------------*/
c_inlas(&parb,&std_out);

/* Alloate buffer space needed
------------------------------*/
grid = (struct GEOGRID *) malloc(sizeof(struct GEOGRID));
lines = (float *) malloc(MEMXPT * sizeof(float));
if ((grid == NULL) || (lines == NULL))
   {
   c_errmsg("Error allocating dynamic memory","polygeo-alloc",&one);
   c_errmsg("Fatal error eoncountered","polygeo-fatal",&fatal);
   }

getpar(&parb,&invers,&insect,&skip,&interval,&msgtim);

/*  fill input coordinate buffer with zeros
-------------------------------------------*/
c_pxfill(lines,&dtype,&npoints,&zero);

val = p_find(&parb,"LSCOORS");
cnt = val->v_count;

for ( i = 0; i < cnt;i ++)
  lines[i] = IVAL(*val,i);

/* PDEF is used to know which parameter conatins the input coordinates-LSCOORS
   or INFILE
------------------------------------------------------------------------------*/
if (cnt == 0)
   pdef = TRUE;
else
   pdef = FALSE;

if (pdef)
   {
   val = p_find(&parb,"INFILE");
   if (val->v_count <= 0)
      {
      c_errmsg("No input coordinates were specified","polygeo-coord",
                  &nonfatal);
      c_errmsg("Fatal error encountered","polygeo-fatal",&fatal);
      }
   strcpy(ha2,SVAL(*val,0));

   status = c_getnam(&parb,"INFILE",&zero,ha2,"ibis_graph",&access);
   if (status != E_SUCC)
     c_errmsg("Fatal error encountered","polygeo-fatal",&fatal);
   status = c_gropen(&fdesc2,ha2,&nsamp,&nline,&zero);
   if (status != E_SUCC)
      {
      c_errmsg("Cannot open input file","polygeo-open",&nonfatal);
      c_errmsg("Fatal error encountered","polygeo-fatal",&fatal);
      }
   }
else 
   {
   nline = 1;
   nsamp = cnt;
   }

work = (double *) malloc(nsamp * sizeof(double));
if (work == NULL)
   {
   c_errmsg("Error allocating dynamic memory","polygeo-alloc",&one);
   c_errmsg("Fatal error eoncountered","polygeo-fatal",&fatal);
   }

/*  Retreive name and open input grid file
------------------------------------------*/
status = c_getnam(&parb,"INGRID",&zero,ingrid,"grid",&access);
if (status != E_SUCC)
  c_errmsg("Fatal error encountered","polygeo-fatal",&fatal);

status = c_getgrd(ingrid,grid);
if (status != E_SUCC)
   {
   c_errmsg("Error reading grid file","polygeo-rdgrid",&one);
   c_errmsg("Fatal error eoncountered","polygeo-fatal",&fatal);
   }

/*  Retreive name and open output IBIS graphics file
----------------------------------------------------*/
access = IWRITE;
status = c_getnam(&parb,"OUTFILE",&zero,ha1,"ibis_graph",&access);
if (status != E_SUCC)
  c_errmsg("Fatal error encountered","polygeo-fatal",&fatal);

status = c_gropen(&fdesc1,ha1,&nsamp,&nline,&one);
if (status != E_SUCC)
   {
   c_errmsg("Cannot open output file","polygeo-open",&nonfatal);
   status = c_grclos(&fdesc1,ha1);
   c_clean(&parb,"polygeo",ha1,&one);
   }

curent = 1;
maxent = ((nline * (nsamp) / 2) / (interval + 2)) * 2;
msgtyp = GEN2;
c_inittm("POLYGEO","Processed","point"," ","of input file",&maxent,&one,&
         msgtim,&msgtyp);
c_startm();

/* start line by line processing
--------------------------------*/
for (iline = 0; iline < nline; iline++)
   {
   if (pdef)
      {
      rec2 ++;
      status = c_grread(&fdesc2,&rec2,lines);
      if (status != E_SUCC)
         {
         c_errmsg("Error reading input file","polygeo-read",&nonfatal);
	 status = c_grclos(&fdesc1,ha1);
	 c_clean(&parb,"polygeo",ha1,&one);
         }
      }
      nsl = 1 + skip;
      jsk = 2 + interval;

      /* Place needed values into a work buffer -- Skipping the values
         specified
      ----------------------------------------------------------------*/
      for (jx = nsl,i = 0; jx < nsamp; jx += jsk,i += 2)
         {
         if ((lines[jx - 1] == 0) && (lines[jx] == 0))
            break;
	 curent++;
      	 work[i] = lines[jx-1];
      	 work[i + 1] = lines[jx];
         }
      npoints = i / 2;

      /*  Transform the points based on the grid
      ------------------------------------------*/
      map_poly(work,npoints,invers,insect,grid);

      /*  Place transformed points back into the I/O buffer
      -----------------------------------------------------*/
      for (jx = nsl,i = 0; i < (npoints * 2); jx += jsk,i += 2)
         {
         lines[jx-1] = work[i];
         lines[jx] = work[i + 1];
         }

   rec1 ++;
   status = c_grwrit(&fdesc1,&rec1,lines);
   if (status != E_SUCC)
      {
      c_errmsg("Error writing to output file","polygeo-write",&nonfatal);
      status = c_grclos(&fdesc1,ha1);
      c_clean(&parb,"polygeo",ha1,&one);
      }
   } /*  for iline < nline  */

/*
curent = curent + npoints;
*/
c_stoptm();

status = c_grclos(&fdesc1,ha1);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","polygeo-warn",&nonfatal);

if (pdef)
   {
   status = c_grclos(&fdesc2,ha2);
   if (status != E_SUCC)
      c_errmsg("Nonfatal error encountered","polygeo-warn",&nonfatal);
   }

c_complt("POLYGEO",ha1,&one);

return EXIT_SUCCESS;

}  /* polygeo  */

void getpar
(
    struct PARBLK *parb,
    long *invers,
    long *insect,
    long *skip,
    long *interval,
    long *msgtim
)
{
struct VARIABLE *val;           /* pointer to a TAE variable within v-block  */

char invopt[4];
char invpnt[4];

val = p_find(parb,"MAPOPT");
strcpy(invopt,SVAL(*val,0));
if (strcmp(invopt,"FWD") == 0)
   *invers = FALSE;
else
   *invers = TRUE;

val = p_find(parb,"PNTFLG");
strcpy(invpnt,SVAL(*val,0));
if (strcmp(invpnt,"NO") == 0)
   *insect = FALSE;
else
   *insect = TRUE;

val = p_find(parb,"SKIPVAL");
*skip = IVAL(*val,0);

val = p_find(parb,"PNTINT");
*interval = IVAL(*val,0);

val = p_find(parb,"$MSGTIM");
*msgtim = IVAL(*val,0);

return;
}
