C     ******************************************************************
C     NAME 			RANDADD       	    		
C
C     PURPOSE	   Adds extra points along a densified cutline.  The 
C		   coordinates for the cutline are stored in an IBIS
C		   graphics file.
C	
C     PROGRAM HISTORY
C	PROGRAMMER	     DATE		REASON
C	----------	     ----		------
C	D.VanderZee	     1/91		Improve radiometric matching
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C	RANDADD must be run under TAE.
C
C     PROJECT		        LAMS		
C
C     ALGORITHM DESCRIPTION
C        The program begins with the standard initialization and
C     retrieval of user-specified parameters.  The points of the
C     input polygon are read in from the IBIS graphics file.
C     Points are generated around the input cut line the density
C     of the points is dependent on the specified step size (PNTINC).
C     This same parameter is used by CUTADD to densify the cutline 
C     originally.  The resulting points are written to the output IBIS
C     graphics file.
C
C     ALGORITHM REFERENCES 		See inline documentation 
C     ******************************************************************
	program randadd

	include 'las.fin'

	include 'lams.fin'

C     	buff must be able to hold x and y values for twice as many points as 
C	the densified cut line may contain--plus four corner points 
C       --------------------------------------------------------------------
	real*4		buff(MEMXPT*2*2+8)

	real*4		grbuf(MMXPT)
	real*4		xvert(MEMXPT)
	real*4		yvert(MEMXPT)

	integer*4	access
	integer*4	count
	integer*4	getnam
	integer*4	grclos
	integer*4	gropen
	integer*4	grwrit
	integer*4	grread
	integer*4	i
	integer		fdesc1
	integer		fdesc3
	integer*4	il 
	integer*4	innl
	integer*4	inns
	integer*4	ivert
	integer*4	j
	integer*4	listou(10)
	integer*4	npairs
	integer*4	npoly
	integer*4	nrec
	integer*4	outnl
	integer*4	outns

      	integer*4	parblk(XPRDIM)

	integer*4	reclen
	integer*4	pntinc
	integer*4	status
	integer*4	stdout
	integer*4	sl,ss,nl,ns,seed,np
	
	logical		newpol

	character*(CMLEN)	infile
	character*(CMLEN)	ofile

      	common/buffer/grbuf

C	initialization
C	--------------
	call inlas(parblk,stdout)
        listou(1) = 1
        npoly = 0
        ivert = 0
        newpol = .true.

C	initialize output buffer
C	-----------------------
	do 10 i = 1,MEMXPT*2*2+8
	   buff(i) = 0.0
10	continue

C	get the source of the input coordinates
C	---------------------------------------
	   status = getnam(parblk,'INFILE',1,infile,'ibis_graph',0)
	   if (status .ne. ESUCC) then
	      call errmsg('Fatal error encountered','randadd-fatal',-1)
	   endif

C	get the output IBIS file name
C	-----------------------------
	status = getnam(parblk,'OUTFILE',1,ofile,'ibis_graph',1)
	if (status .ne. ESUCC) then
	   call errmsg('Fatal error encountered','randadd-fatal',-1)
	endif

C	get the PNTINC parameter
C	--------------------------
        call xrintg(parblk,'PNTINC',1,pntinc,count,status)

C	get the SL parameter
C	--------------------------
        call xrintg(parblk,'SL',1,sl,count,status)

C	get the SS parameter
C	--------------------------
        call xrintg(parblk,'SS',1,ss,count,status)

C	get the NL parameter
C	--------------------------
        call xrintg(parblk,'NL',1,nl,count,status)

C	get the NS parameter
C	--------------------------
        call xrintg(parblk,'NS',1,ns,count,status)

C	get the SEEDVAL parameter
C	--------------------------
        call xrintg(parblk,'SEEDVAL',1,seed,count,status)

C	read in and process the polygon points
C	(Note that the points are expected to be 
C	 listed in y,x or line,sample order )
C       ----------------------------------------

C	   open the IBIS graphics file
C	   ---------------------------
	   access = 0
           status = gropen(fdesc3,infile,reclen,nrec,access)
	   if (status .ne. ESUCC) then
	      call errmsg('Error opening input file','randadd-open',1)
	      call errmsg('Fatal error encountered','randadd-fatal',-1)
	   endif

C	   read the points from the graphics file
C	   --------------------------------------
           innl = nrec
           inns = reclen
           npairs = inns/2
           do 20 il = 1,innl
              status = grread(fdesc3,il,grbuf)
	      if (status .ne. ESUCC) then
	         call errmsg('Error reading input file','randadd-read',1)
	         call errmsg('Fatal error encountered', 'randadd-fatal',-1)
	      endif

              call polyin(npairs,newpol,npoly,ivert,xvert,yvert,
     +			  listou)
20	   continue

C	   close the graphics file
C	   -----------------------
	   status = grclos(fdesc3,infile)
	   if (status .ne. ESUCC) then
	      call errmsg('Error closing input file','randadd-close',1)
	   endif

C       generate random of points along densified cutline
C       -------------------------------------------------

	call randpts (sl,ss,nl,ns,seed,pntinc,yvert,xvert,ivert,buff,np)
 
	j = np * 2

C	set the size for the output graphics file
C	-----------------------------------------
	if (j  .le. MMXPPR) then
	   outnl = 1
	   outns = j
	else
	   outnl = (j + MMXPPR - 1)/MMXPPR
	   outns = MMXPPR
	endif

C	open the output IBIS graphics file
C	----------------------------------
	access = 1
	status = gropen(fdesc1,ofile,outns,outnl,access)
	if (status .ne. ESUCC) then
	   status = grclos(fdesc1,ofile)
	   call errmsg('Error opening output file','randadd-open',1)
	   call clean(parblk,'randadd',ofile,1)
	endif

C	write to the output graphics file
C	---------------------------------
	i = 1
	do 3000 il = 1,outnl
	   status = grwrit(fdesc1,il,buff(i))
	   if (status .ne. ESUCC) then
	      status = grclos(fdesc1,ofile)
	      call errmsg('Error writing output file','randadd-write',1)
	      call clean(parblk,'randadd',ofile,1)
	   endif

	   i = i + MMXPPR
3000	continue

C	close the output IBIS file
C	--------------------------
	status = grclos(fdesc1,ofile)
	if (status .ne. ESUCC) then
	   call errmsg('Error closing output file','randadd-close',1)
	endif

C	output completion message
C	-------------------------
	call complt('RANDADD',ofile,1)

      	end

C       ****************************************************************
C	POLYIN puts the points of the input polygon into the approrpriate
C       x and y arrays and determines the number of segments.
C       ****************************************************************
        subroutine polyin(npairs,newpol,npoly,ivert,xvert,yvert,
     +			  listou)

	include 'las.fin'

	include 'lams.fin'

	real*4		grbuf(MMXPT)
	real*4		x
	real*4		xvert(1)
	real*4		y
	real*4		yvert(1)

	integer*4	ipair
	integer*4	ivert
	integer*4	listou(10)
	integer*4	npairs
	integer*4	npoly

        logical 	newpol

        common/buffer/grbuf

        do 10 ipair = 1,npairs
           x = grbuf(2*ipair)
           y = grbuf(2*ipair-1)
           if ((x.ne.0. .or. y.ne.0.).and.(x.ne.-99. .or. y.ne.-99.)
     +        .and. (x.ne.xvert(listou(npoly+1)) .or. y.ne.
     +        yvert(listou(npoly+1))))  go to 12
           if (newpol) go to 10
           npoly = npoly + 1
           listou(npoly+1) = ivert+1
           newpol = .true.
           go to 10
   12      continue
           ivert = ivert + 1
           newpol = .false.
           xvert(ivert) = x
           yvert(ivert) = y
   10   continue

        return
        end
