/****************************************************************************
NAME:		GETNAME	

PURPOSE:	Checks the database record for SCENEID, TAPEID, SL.  Places
		scene in TMP status as processing, and places in YES status
		when done.

VERSION	 	DATE		AUTHOR
-------	 	----		------	
1.0		Sep-94		T.Mittan

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adapsutil.h"
#include "exbrows.h"
#include "protos.h"

static int numsc = 0;
static struct DBFIL db[MSCENE];	/* Database records from file		*/

void getname
(
    char *scene,			/* scene id to be checked		*/
    char *tapeid,			/* tapeid passed in			*/
    long sl,			/* starting line number			*/
    long *use_flag,			/* flag to determine to complete scene or
				   to check for validity -- returns
				   TRUE to process and FALSE to skip	*/
    long *position,			/* position in the db file to seek to   */
    char *ptype,                 	/* product type                         */
    FILE **fp1,			/* File pointer into db file		*/
    long block_add,			/* block address			*/
    long *rectec,			/* Recording type			*/
    long *satnum,			/* Satellite number			*/
    double center[2]		/* Center latitude/logitude (degrees)	*/
)
{
FILE *fp2;			/* Temporary work file pointer 		*/

long ii;			/* loop variable		*/
long status;			/* return status			*/
long fatal = -1;		/* fatal error flag			*/
long nonfatal = 1;		/* nonfatal error flag			*/
long find_flag = FALSE;		/* Found the record of interest		*/
long position_flag = TRUE;	/* Position in the file is known	*/

char filename[CMLEN];		/* filename of tables file		*/
char disk[CMLEN];		/* path of LBTABLES directory		*/
char buffer[100];		/* input buffer				*/
char logdir[CMLEN];		/* directory of logfiles		*/
char logkey[16];		/* key to be written to logfile if error*/
char string[CMLEN];		/* string to be written to logfile	*/

/* Get log file
-------------*/
strcpy(disk,getenv("LBLOG"));
sprintf(logdir,"%s/reformat.log",disk);
sprintf(logkey,"%s",tapeid);

/*
  Make a copy of the file pointer to work with out losing the fp1 location
*/
fp2 = *fp1;

/* Get temporary the Database file location
-----------------------------------------*/
strcpy(disk,getenv("LBTABLES"));
if (disk == NULL)
    {
    sprintf(string,"%s","Error getting the tables directory environment name");
    c_wrtlog(logdir, logkey, "getname-getenv", string);
    c_errmsg(string, "getname-getenv",&nonfatal);
    c_errmsg("Fatal error encountered","getname-fatal",&fatal);
    }

/*
  If the database file hasn't been opened yet; open the appropriate db file
*/
if (fp2 == NULL)
   {
   if (strcmp(ptype,"MSP")==0)
      {
      sprintf(filename,"%s/msp",disk);
      fp2 = fopen(filename,"r+");
      }
   else if (strcmp(ptype,"MSA")==0)
      {
      sprintf(filename,"%s/msa",disk);
      fp2 = fopen(filename,"r+");
      }
   else if (strcmp(ptype,"TMR")==0)
      {
      sprintf(filename,"%s/tmr",disk);
      fp2 = fopen(filename,"r+");
      }
   else if (strcmp(ptype,"TMA")==0)
      {
      sprintf(filename,"%s/tma",disk);
      fp2 = fopen(filename,"r+");
      }
   else
      {
      sprintf(filename,"%s/msx",disk);
      fp2 = fopen(filename,"r+");
      }

   if (fp2 == NULL)
       {
	   sprintf(string,"Error opening db file %s", filename);
	   c_wrtlog(logdir, logkey, "getname-error", string);
	   c_errmsg(string, "getname-dbopen",&nonfatal);
       c_errmsg("Fatal error encountered","getname-fatal",&fatal);
       }

   *fp1 = fp2;
   position_flag = FALSE;
   }

/*
  Initalize the use flag
*/
*use_flag = FALSE;

/* Skip to new position in file 
   ---------------------------- */
if ((position_flag == FALSE) && (numsc == 0))
    {
    /*
      Position the file pointer to the top of the database file
    */
    fseek(fp2,0,0);

    /*
      Gather all the database records that have the same tapeid and the
      tapeid of interest.
    */
    for(;;)
	{
	/*
	  Save the current position
	*/
	db[numsc].position = ftell(fp2);

	/*
	  Read a record from the database file
	*/
	if (fgets(buffer,DBREC,fp2) == NULL)
	    {
	    if (!feof(fp2))
		{
		sprintf(string,"Error reading db buffer");
		c_wrtlog(logdir, logkey, "getname-error", string);
		c_errmsg(string, "getname-dbread",&nonfatal);
		fclose(fp2);
		*fp1 = NULL;
		}

	    break;
	    }

	/*
	  Extract the information from the buffer
	*/
	status = sscanf(buffer,"%16s  %6s  %d  %d  %d  %lf  %lf  %d  %s",
			db[numsc].scene, db[numsc].tape, &db[numsc].sl,
			&db[numsc].block, &db[numsc].sat, &db[numsc].clat,
			&db[numsc].clon, &db[numsc].rectec, db[numsc].flag);
	if (status != 9)
	    {
	    sprintf(string,"Error extracting database fields");
	    c_wrtlog(logdir, logkey, "getname-error", string);
	    c_errmsg(string, "getname-dbfld",&nonfatal);
	    fclose(fp2);
	    *fp1 = NULL;
	    break;
	    }

	/*
	  Determine if the tape id from the database file matches the
 	  tape id of interest
	*/
	if (strcmp(tapeid,db[numsc].tape) == 0)
	    {
	    /*
	      Found a tape id match.  Set the found flag and increment
	      the number of scenes found counter.
	    */
	    find_flag = TRUE;
	    numsc++;
            if (numsc > MSCENE)
               {
               sprintf(string,"Error reading database for %s",tapeid);
               c_wrtlog(logdir, logkey, "getname-error", string);
               c_errmsg(string, "getname-mscene",&fatal);
               }
	    }
	else if (find_flag == TRUE)
	    {
	    /*
	      There are no more tape ids that will match the tape id of
	      interest, so break out of the infinite loop
	    */
	    break;
	    }
	}
    }

if (*fp1 == NULL)
    return;

/*
  Search the saved list from the database (based on tape id) for the
  scene of interest.
*/
for (ii = 0; ii < numsc; ii++)
    {
    *position = db[ii].position;

    if (strcmp(scene,db[ii].scene) == 0)
	{
	if ((strcmp(db[ii].flag,"YES") == 0))
	    {
	    /*
	      Already generated a browse for this scene
	    */
	    sprintf(string,"Scene %s has already been used",scene);
	    c_wrtlog(logdir, logkey, "getname-error", string);
	    c_errmsg(string, "getname-used",&nonfatal);
	    break;
	    }
	else if (strcmp(tapeid,db[ii].tape) != 0)    /* Redundant test */
	    {
	    /*
	      The tape ids do not match.  The scene must be on a
	      different tape.
	    */
	    sprintf(string,"Scene %s should be on %s", scene,db[ii].tape);
	    c_wrtlog(logdir, logkey, "getname-error", string);
	    c_errmsg(string, "getname-tapeid",&nonfatal);
	    fclose(fp2);
	    *fp1 = NULL;
	    break;
	    }
	else if (sl != db[ii].sl)
	    {
	    /*
	      The starting line in the tape header does not match
	      with the database.
	    */
	    sprintf(string,"Scene %s starts on line %d instead of %d",
	            scene,db[ii].sl,sl);
	    c_wrtlog(logdir, logkey, "getname-error", string);
	    c_errmsg(string, "getname-sl",&nonfatal);
	    break;
	    }
	else if (abs(block_add - db[ii].block) > 50)
	    {
	    /*
	      Block addresses do not match.  Some kind of problem.
	      After encountering some problem with this error, I 
              discussed the situation with Cheryl G. and Steve M.
              When a DCT is reprocessed, different block addresses
              are used (generated).  For one reason or another, these block 
              addresses did not get updated in the landsat database which
              is where our DB file got its info from.  A change was made
	      to check for a address within a specific range.   The number
              chosen is a conservative number. This was the largest I've found
              to this point.
	    */
	    sprintf(string,
		    "Block addresses do not match for scene %s, on tape %s",
                    scene,db[ii].tape);
	    c_wrtlog(logdir, logkey, "getname-error", string);
	    c_errmsg(string, "getname-block",&nonfatal);
	    fclose(fp2);
	    *fp1 = NULL;
	    break;
	    }
	else
	    {
	    *use_flag = TRUE;

	    /*
	      Mark the scene as being processed
	    */
	    sprintf(string,"Scene %s is being processed", scene);
	    c_wrtlog(logdir, logkey, "getname-use", string);

	    /*
	      Pass back relative information about the scene
	    */
	    *rectec = db[ii].rectec;
	    *satnum = db[ii].sat;
	    center[0] = db[ii].clat;
	    center[1] = db[ii].clon;

	    break;
	    }
	}
    }

return;
}
