/******************************************************************************
NAME:   			PROCESX 

PURPOSE:  Convert a subsampled MSS-X image band file into a jpeg image band
file.  Update database file with successful conversions.  

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
1.5        B. Gross        2/02     CR5734      Data Management wants browse
                                                created for all images.  
                                                Previously, browse was not 
                                                created for images that were
                                                completely filled with a fill
                                                value (0), or a zero value 
                                                was found in the first line at
                                                a sample position of less than
                                                375.  This was completed for
                                                MSS-X data from the WBVTCS
1.6        B. Gross        3/02      CR5734     The pointer to the buffer 
                                                where an image is stored was
                                                not being reset while 
                                                attempting to locate a valid
                                                sample count,  corrected.
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>

#include "las.h"
#include "util.h"
#include "pixman.h"
#include "typlim.h"
#include "exbrows.h"
#include "protos.h"

int procesx
(
    char *hosin,        /* host file names of input images */
    char *ptype,        /* product type                    */
    struct LINE *brw_line,/* header record for output file */
    char *tapeid        /* tapeid                          */
)
{
  struct FDESC *fdesc[2]; /* image file descriptors used for image I/O */
  unsigned char *thebuf=NULL; /* image I/O buffer          */
  long  nsamps;         /* number of samples               */
  long  nlines;         /* number of lines                 */

  /* Misc allocations */
  long  k;              /* Looping variable                */
  long  band;           /* band loop counter               */
  long  len;            /* temporary length variable       */
  long  status;         /* function return status code     */
  long  nonfatal=1;     /* Non-fatal error flag            */
  long  bd;             /* Browse band number              */
  long  use_flag;       /* scene use flag                  */
  struct  stat fstat;   /* file status                     */
  long  nbands;         /* number of bands                 */
  char  msg[CMLEN];     /* character string for messages   */
  
  /* Database file information allocations */
  
  FILE  *fp1;           /* file pointer	into the database file */
  char  infiles[3][CMLEN]; /* input file names             */
  char  outfile[CMLEN]; /* specific output name            */
  char  outname[CMLEN]; /* output name                     */
  char  scene[17];      /* sene id                         */
  long  sl;             /* starting line                   */
  long  position;       /* position in database file       */
  long  rectec;         /* Recording technique             */
  long  satnum;         /* Satellite number                */
  double  center[2];    /* Center latitude/longitude (degrees) */
  char  command[CMLEN]; /* system command 			       */
  char  badraw[CMLEN];  /* temporary directory name		   */
  
  /* Image I/O data allocations */
  
  struct  DDR ddr;      /* DDR structure                   */
  struct  BDDR bddr;    /* BDDR structure                  */

  long  access = IREAD; /* image I/O access type           */
  long  opt[3];         /* option array for image I/O      */
  long  one = 1;        /* value of one                    */
  long  imgsiz;         /* image size                      */
  long  size;           /* image size                      */
  long  count;          /* histogram count value           */
  char  wbuf[10];       /* working buffer                  */
  char  syst[12];       /* system                          */
  char  disk[CMLEN];    /* temporary directory name        */
  unsigned char *ptr;   /* Place holder for image io address */
  char  direc[CMLEN],root[CMLEN],ext[CMLEN];
  long  cc;
  long  two55 = 255, i;
  long  index = -1;     /* index used to find end of valid
                           image data and start of fill data
                           which should be a 0 */
  long  line;           /* line that is under test         */
  
  /* Histogram information */
  
  long  histo[256];     /* histogram array                 */
  long  mean;           /* average value                   */
  long  minval;         /* minimum value                   */
  long  maxval;         /* maximum value                   */
  long  pnt0;           /* Value at  0.05 percent of image data */
  long  pnt1;           /* Value at  2.5  percent of image data */
  long  pnt2;           /* Value at 97.5  percent of image data */
  long  zero = 0;       /* Pixel fill value of zero        */
  long  nfill = 256;    /* Number of pixels to be filled   */
  long  dtype = ELONG;  /* Data type of the fill data      */
  long  dummy = 0;      /* Dummy variable for pxhist call  */
  double  pc;           /* percentage value                */
  double  cum;          /* cummulative percentage          */
  double  sx;           /* cummulative sum                 */

  struct stat buf;
  
  /* Get the archive directory
  --------------------------*/
  strcpy(disk,getenv("LBINT"));

  nlines = 390;
  nsamps = 432;

  /*Create the DDR for the browse set 
  ---------------------------------*/
  c_intddr(&ddr);
  c_intbdr(&bddr);

  ddr.nl = nlines;
  ddr.ns = nsamps;
  strcpy(ddr.system,"sg1");
  ddr.nbands = 1;
  ddr.dtype = EBYTE;
  bddr.bandno = 1;
  if (c_getsys(syst) != E_SUCC)
      c_errmsg("Error returned from getsys", "eopenr-call",&nonfatal);
  else
      strcpy(ddr.system,syst);

  /* Set up file names. hosin is the name of band 1. Create the other
     file names from this. */
  len = (long) strlen(hosin);
  hosin[len-1] = '\0';
  c_splitname(hosin,direc,root,ext);

  strcpy(infiles[0],direc);
  strcat(infiles[0],root);
  strcat(infiles[0],"1");
  strcpy(infiles[1],direc);
  strcat(infiles[1],root);
  strcat(infiles[1],"2");
  strcpy(infiles[2],direc);
  strcat(infiles[2],root);
  strcat(infiles[2],"4");

  /* copy only first 16 characters of root into the scene (id) */
  strncpy(scene,root,16);
  scene[16]='\0';

  opt[0] = opt[1] = opt[2] = 0;

  nbands=3;
  fp1 = NULL;

  /* Check to be sure this is the scene to process */
  sl = 0;
  getname(scene,tapeid,sl,&use_flag,&position,ptype,
      &fp1,zero,&rectec,&satnum,center);

  /* Found a browse image to extract */
  if (use_flag == TRUE)
  {
      /* Allocate space for the entire browse image band */
      imgsiz = nsamps * nlines;
      thebuf = (unsigned char *) malloc(imgsiz);
      
      if(thebuf == NULL)
      {
          c_errmsg("Error allocating dynamic memory","exbrows-malloc",
              &nonfatal);
          
          /* Close the database file */
          if (fp1 != NULL)
              fclose(fp1);
          
          return(E_FAIL);
      }
      
      /* Save the address of the image io buffer.  This is needed later
         because the jpeg compression function manipulates the image i/o
         buffer to get the job done. */
      ptr = thebuf;

      /* Create ddr files for each band that will be processed */
      for (band = 0; band < nbands; band++)
      {     
          /* Create a DDR for this file */
          if (c_putddr(infiles[band],&ddr) != E_SUCC)
          {
              c_errmsg("Error creating DDR","exbrows-ddr",&nonfatal);
                  
              /* Close the database file */
              if (fp1 != NULL)
                  fclose(fp1);
                  
              /* Free the image I/O buffer space allocated */
              free(ptr);
                  
              return(E_FAIL);
          }

          if (c_putbdr(infiles[band],&bddr) != E_SUCC)
          {
              c_errmsg("Error creating DDR","exbrows-ddr",&nonfatal);

              /* Free the image I/O buffer space allocated */
              free(ptr);              
                                
              /* Close the database file */
              if (fp1 != NULL)
                  fclose(fp1);
                  
              return(E_FAIL);
          }
      }
     
      /* Determine the number of samples in the image.  Browse images
         can have a value of samples from 375 to 431.  Search through
         the images line by line until a valid value is found.    */

      /* Find the last pixel (sample) not equal to zero (fill) */
      
      /* start with first band  = 0 */
      band = 0;
      
      do
      {
          /* Open the input file */
          status = c_eopenr(&fdesc[0],infiles[band],&one,&one,
              &one,&nlines,&nsamps,&access,&ddr.dtype,opt,&one);
        
          if (status != E_SUCC)
          {    
              c_errmsg("error opening file","exbrows-open",&nonfatal);
              
              /* Free the image I/O buffer space allocated */
              free(ptr);       		             
              
              /* Close the database file */
              if (fp1 != NULL)
                  fclose(fp1);
              
              return(E_SUCC);
          }
                  
          /* find sample where fill data starts */
          
          /* start with first line */
          line = 1;
          
          do 
          {
              /* read in line */
              if (c_eread(&fdesc[0],&one,&line,thebuf,&one) != nsamps)
              {
                  c_errmsg("Error reading from file","exbrows-read",&nonfatal);
                  /* Close input image */
                  c_eclose(&fdesc[0]);

                  /* Close the database file */
                  if (fp1 != NULL)
                      fclose(fp1);
                  
                  /* Free the image I/O buffer space allocated */
                  free(ptr);       				          
                      
                  return(E_SUCC);
              }
             
              /* find the first value that is not zero */
              c_pxindr(thebuf,&zero,&nsamps,&index,&ddr.dtype);

              /* update index value because we are zero relative */
              index++; 
              
              /* If this happens, check to see if the fill data
                 was 255.  For some reason some (one?) browse scene 
                 had fill data of 255.  This one scene has bad data in 
                 the original (BIP2) scene. */
              if (index == 432)
              {
                  c_pxindr(thebuf,&two55,&nsamps,&index,&ddr.dtype);
                  index++;
              }
              
              thebuf += nsamps;
              line++;
          } while ((line < nlines) && ( index < 375 || index > 431));  

          /* Close input image */
          c_eclose(&fdesc[0]);
          
          
          band++;

          /* reset the buffer for next band */
          thebuf = ptr;
      } while ((index < 375) && (band < nbands));
      
         
      /* if the sample value was not valid, save scene browse file
         to the bad_scenes directory 
         
         Modified the check to search all lines in a browse file 
         to find a valid sample count, then if it can't be found, search
         the remaining band files.  What this will do is allow scenes with
         an empty band file to still be processed into a browse, and scenes
         where a value of zero (which is an allowed value) happened to be
         found in the first line of the browse image in band 0, and the 
         sixth line of the original image, to still be processed.    */ 
      if (index < 375)
      { 
          sprintf(msg,"Invalid sample count %s",infiles[band]);
          c_errmsg(msg,"exbrows-samps",&nonfatal);
 
          /* Create a directory where these scenes will be copied */
          strcpy(badraw,getenv("LBRAW"));
          if(strlen(badraw) == 0)
          {
              c_errmsg("environment variable LBRAW not set","exbrows-globals",
                  &nonfatal);
              return(E_FAIL);
          }
          strcat(badraw,"/bad_scenes");
          
          /* Check if directory exists, 
             Create it if it does not */
          if(stat(badraw,&buf)!= 0)
          {
              /* notify user that directory did not exist */
              sprintf(msg,"Directory %s did not exist, creating",badraw);
              c_errmsg(msg,"exbrows-nonexist", &nonfatal);
                                                     
              /* create directory to place scenes with errors in */
              sprintf(command,"mkdir %s",badraw);
              
              /* if create directory was not successful
                 issue error and return failure */
              if(system(command) != 0)
              {
                  sprintf(msg,"Unable to create directory %s",badraw);
                  c_errmsg(msg,"exbrows-create",&nonfatal);
                  return(E_FAIL);
              }              
          }
          
          /* Copy the scene browse files to the bad_scenes directory */
          for (i = 0; i < 3; i++)   
          {
              sprintf(command,"cp %s %s",infiles[i],badraw); 
              if(system(command) != 0)
              {
                  sprintf(msg,"Error moving scene %s to directory %s",
                      infiles[i],badraw);
                  c_errmsg(msg,"exbrows-move", &nonfatal);
              }
          }
          
          /* Dont forget band 3 */
          len = (long) strlen(infiles[0]);
          infiles[0][len-1] = '3';
          sprintf(command,"cp %s %s",infiles[0],badraw); 
          
          if(system(command) != 0)
          {        
              sprintf(msg,"Error moving scene %s to directory %s",
                  infiles[0],badraw);
              c_errmsg(msg,"exbrows-move", &nonfatal);
          }          
          
          /* Close the database file */
          if (fp1 != NULL)
              fclose(fp1);
                  
          /* Free the image I/O buffer space allocated */
          free(ptr);  
          
          /* Continue on with next scene.*/
          return (E_SUCC);
      }
    
      sprintf(msg,"Number of samples per image line: %d",index);
      c_errmsg(msg,"exbrows-samps",&nonfatal);
      
      /* Process each of the bands for this scene */
      for (band = 0; band < nbands; band++)
      {
          /* Open the input file */
          status = c_eopenr(&fdesc[0],infiles[band],&one,&one,
              &one,&nlines,&nsamps,&access,&ddr.dtype,opt,&one);
        
          if (status != E_SUCC)
          {    
              /* Free the image I/O buffer space allocated */
              free(ptr);       		             
              
              /* Close the database file */
              if (fp1 != NULL)
                  fclose(fp1);
              
              return(E_SUCC);
          }

          /* Append the browse band number to the output file name */
          len = (long) strlen(infiles[band]);
          bd = atol(&infiles[band][len-1]);
          sprintf(outfile,"%s/LM%s",disk,root);
          sprintf(wbuf,"_%d",bd);
          sprintf(outname,"LM%s",root);
	      
          /* Create the output file name */
          if (   (scene[0] == '1') 
              || (scene[0] == '2') 
              || (scene[0] == '3'))
          {
              if (wbuf[1] == '1')
                  strcat(outfile,"_4");
              else if (wbuf[1] == '2')
                  strcat(outfile,"_5");
              else if (wbuf[1] == '4')
                  strcat(outfile,"_7");
          }
          else
              strncat(outfile,wbuf,2);
       
          /* Initailize the histogram information */
          c_pxfill(histo,&dtype,&nfill,&zero);
   
          /* read entire image into memory, one line at a time.
             reset the pointer to ignore fill data */
          for (cc = 1; cc <= nlines; cc++)
          {
              if (c_eread(&fdesc[0],&one,&cc,thebuf,&one) != nsamps)
              {
                  /* Close input image */
                  c_eclose(&fdesc[0]);

                  /* Close the database file */
                  if (fp1 != NULL)
                      fclose(fp1);
                  
                  /* Free the image I/O buffer space allocated */
                  free(ptr);       				          
                      
                  return(E_SUCC);
              }
              thebuf += index;
          }

          /* Reset to the begining of the buffer */
          thebuf = ptr;

          /* histogram input image */
          size = index * nlines;
          c_pxhist(&size,thebuf,&ddr.dtype,histo,&dummy);
          count = size;

          /* zero out 255 even though it is MSS. For some reason
             some (one?) browse scene had fill data of 255.  This
             one scene has bad data in the original (BIP2) scene. 
             zero out 127 because mainly appears to be caused by
             noise? as evidenced by viewing histogram on lines 
             with errant lines (streaks) */
          count -= histo[0];
          count -= histo[127];
          count -= histo[255];
          histo[0] = histo[127] = histo[255] = 0;

          minval = MAXUTINY;
          maxval = MINUTINY;
          cum = 0.0;
          sx = 0.0;
          pnt0 = pnt1 = pnt2 = 0;

          if (count != 0)
          {
              for (k = 0; k <= 255; k++)
              {
                  if (histo[k] > 0)
                  {
                      pc = 100.0 * histo[k]/count;
                      cum += pc;

                      if (cum <= 0.05)
                          pnt0 = k;
                      else if (cum <= 2.5)
                          pnt1 = k;
                      else if (cum <= 97.5)
                          pnt2 = k;
                        
                      if (k < minval)
                          minval = k;

                      if (k > maxval)
                          maxval = k;
          
                      sx += (k*histo[k]);
                  }
                  mean = (long)sx/count;
              }
          }
          else 
              mean = 0;      
          
          /* Setup header */
          setup_head(brw_line,bd,nlines,nsamps,outname,center,
              rectec,satnum,minval,maxval,pnt0,pnt1,pnt2,mean);
   
          /* Write JPEG image */
          write_JPEG_file(thebuf,outfile,80,brw_line,nlines,index,1);
          
          /* Check file size as a safe guard.  Some files have
             been processed with the output file size being zero,
             possibly because /usr/tmp was full. */
          if (stat(outfile,&fstat) != 0)
          {
              c_errmsg("File status error","exbrows-stat",&nonfatal);
              
              /* Close input image */
              c_eclose(&fdesc[0]);
              
              /* Close the database file */
              if (fp1 != NULL)
                  fclose(fp1);
              
              /* Free the image I/O buffer space allocated */
              free(ptr);             
              
              return (E_FAIL);
          }

          if (fstat.st_size == 0)
          {
              c_errmsg("Invalid file size","exbrows-zero",&nonfatal);
 
              /* Close input image */
              c_eclose(&fdesc[0]);
              
              /* Close the database file */
              if (fp1 != NULL)
                  fclose(fp1);
              
              /* Free the image I/O buffer space allocated */
              free(ptr);
              
              return (E_FAIL);
          }

          /* Close input image */
          c_eclose(&fdesc[0]);

          /* Reset the image io buffer address.  This gets changed in
             the call to "write_JPEG_file" and it dosen't hurt if it
             wasn't called.  */
          thebuf = ptr;
      }

      /* Update the database record to indicate the record was processed */
      if (updb(fp1,position,scene,tapeid) != E_SUCC)
      {
          /* Free the image I/O buffer space allocated */
          free(ptr);

          /* Close the database file */
          if (fp1 != NULL)
              fclose(fp1);
          
          return(E_FAIL);
      }
    
      /* Free the image I/O buffer space allocated */
      free(thebuf);
  }

  /* Close the database file */
  if (fp1 != NULL)
      fclose(fp1);

  return(E_SUCC);
}
