/*******************************************************************************
FUNCTION:	create_semaphore

PURPOSE:	Create a lock or stopping semaphore file.  A lock semaphore
		will not allow other versions of a function to run at the same
		time.  A stop semaphore will signal a function to stop at the
		next convenient time.

		If type = "lock" then return
		    E_FAIL    If a semaphore already exists
		    E_SUCC    If a lock is successfully created

		If type = "stop" then return
		    E_FAIL    If a lock semaphore does not exist
		    E_SUCC    If a stop is successfully created or if one exists

		Otherwise return E_FAIL


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    07/92  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "stopreform.h"

#define LOCK_EXT "lock"
#define STOP_EXT "stop"

FUNCTION create_semaphore (directory, function, object, type)

    char *directory;   /* IN - Directory where the archive images are stored  */
    char *function;    /* IN - Function that is locking the directory         */
    char *object;      /* IN - Object being locked                            */
    char *type;        /* IN - Type of semaphore. "lock" or "stop"            */
{
char buffer[CMLEN];    /* Buffer used in writing to the file                  */
char file_name[CMLEN]; /* Full path specification of the semaphore file       */

int fd;                       /* File descriptor                              */
int flags = O_RDWR | O_CREAT; /* Access flags for opening the file            */
int length;                   /* Length of a string                           */

long access = 0;              /* File access(0=exist 1=execute 2=write 3=read)*/


/* Create a full path specification of the semaphore file
---------------------------------------------------------*/
length = strlen(function);
c_up2low(function, &length);
length = strlen(object);
c_up2low(object, &length);
sprintf(file_name, "%s%s_%s.%s", directory, function, object, LOCK_EXT);


/* Determine the type of semaphore requested
--------------------------------------------*/
if (strcmp(type, LOCK_EXT) == 0)
    {
    /* If the lock file exists then return E_FAIL
    ---------------------------------------------*/
    if (c_lsstat(file_name, &access) == E_SUCC)
	return(E_FAIL);

    /* Lock the file.  This will also open it for reading and writing.  This
       step will not allow any other occurrence of create_semaphore to attempt
       to create the lock semaphore or file at the same time.
    --------------------------------------------------------------------------*/
    if (cmlock(file_name, &fd, flags, 5) != E_SUCC)
	return(E_FAIL);

    /* Write an informational message to the file
    ---------------------------------------------*/
    sprintf(buffer, "%s has locked %s\n", function, object);
    length = strlen(buffer);
    write(fd, buffer, length);

    /* Now that the semaphore has been created and no other occurrences of
       create_semaphore will succeed, unlock it so that we do not have any
       opened files.
    ----------------------------------------------------------------------*/
    unlock(&fd, 5);
    return(E_SUCC);
    }
else if (strcmp(type, STOP_EXT) == 0)
    {
    /* If the lock file does not exist then return E_FAIL
    -----------------------------------------------------*/
/*
    if (c_lsstat(file_name, &access) != E_SUCC)
	return(E_FAIL);
*/

    /* Create a full path specification of the semaphore file
    ---------------------------------------------------------*/
    sprintf(file_name, "%s%s_%s.%s", directory, function, object, STOP_EXT);

    /* If the stop file exists then return E_SUCC
    ---------------------------------------------*/
    if (c_lsstat(file_name, &access) == E_SUCC)
	return(E_SUCC);

    /* Lock the file.  This will also open it for reading and writing.  This
       step will not allow any other occurrence of create_semaphore to attempt
       to create the stop semaphore or file at the same time.
    --------------------------------------------------------------------------*/
    if (cmlock(file_name, &fd, flags, 5) != E_SUCC)
	return(E_FAIL);

    /* Write an informational message to the file
    ---------------------------------------------*/
    sprintf(buffer, "%s signaled to stop using %s\n", function, object);
    length = strlen(buffer);
    write(fd, buffer, length);

    /* Now that the semaphore has been created and no other occurrences of
       create_semaphore will succeed, unlock it so that we do not have any
       opened files.
    ----------------------------------------------------------------------*/
    unlock(&fd, 5);
    return(E_SUCC);
    }
else
    return(E_FAIL);
}




/*******************************************************************************
FUNCTION:	check_semaphore

PURPOSE:	Check for the existence of the indicated type of semaphore.
		Return E_SUCC if the semaphore exists, E_FAIL if it does not.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    07/92  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
FUNCTION check_semaphore (directory, function, object, type)

    char *directory;   /* IN - Directory where the archive images are stored  */
    char *function;    /* IN - Function that has locked the directory         */
    char *object;      /* IN - Object that has been locked                    */
    char *type;        /* IN - Type of semaphore. "lock" or "stop"            */
{
char file_name[CMLEN]; /* Full path specification of the semaphore file       */
long access = 0;       /* File access check (0=exist 1=execute 2=write 3=read)*/
long length;           /* Length of a string                                  */


/* Ensure the type of semaphore is valid
----------------------------------------*/
if ((strcmp(type, LOCK_EXT) != 0) && (strcmp(type, STOP_EXT) != 0))
    return(E_FAIL);


/* Lowercase the function name and object and then create a semaphore file name
------------------------------------------------------------------------------*/
length = strlen(function);
c_up2low(function, &length);
length = strlen(object);
c_up2low(object, &length);
sprintf(file_name, "%s%s_%s.%s", directory, function, object, type);


/* Check for the existence of the semaphore
-------------------------------------------*/
if (c_lsstat(file_name, &access) == E_SUCC)
    return(E_SUCC);
else
    return(E_FAIL);
}




/*******************************************************************************
FUNCTION:	delete_semaphore

PURPOSE:	Delete the indicated type of semaphore file.  Return E_SUCC if
		the semaphore file is successfully deleted and E_FAIL in all
		other cases.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    07/92  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
FUNCTION delete_semaphore (directory, function, object, type)

    char *directory;   /* IN - Directory where the archive images are stored  */
    char *function;    /* IN - Function that has locked the directory         */
    char *object;      /* IN - Object that has been locked                    */
    char *type;        /* IN - Type of semaphore. "lock" or "stop"            */
{
char file_name[CMLEN]; /* Full path specification of the semaphore file       */
long access = 0;       /* File access check (0=exist 1=execute 2=write 3=read)*/
long length;           /* Length of a string                                  */


/* Ensure the type of semaphore is valid
----------------------------------------*/
if ((strcmp(type, LOCK_EXT) != 0) && (strcmp(type, STOP_EXT) != 0))
    return(E_FAIL);


/* Lowercase the function name and object and then create a semaphore file name
------------------------------------------------------------------------------*/
length = strlen(function);
c_up2low(function, &length);
length = strlen(object);
c_up2low(object, &length);
sprintf(file_name, "%s%s_%s.%s", directory, function, object, type);


/* Check for the existence of the semaphore and possibly delete it
------------------------------------------------------------------*/
if (c_lsstat(file_name, &access) == E_SUCC)
    return(c_dkdel(file_name));
else
    return(E_FAIL);
}
