/******************************************************************************

SUBROUTINE: addreq

PURPOSE:
	Subroutine add will add a request to the Custom Request File.  The 
	structure of the file is a header record of 80 bytes followed by
	records containing the requests. The header record contains the head 
	of the two linked lists.

	The record field "delflg" forms a singly linked list containing all the
	records that do not have valid requests (i.e. the requests have been
	deleted).  The record fields "bptr" and "fptr" forms a doubly linked
	list among the requests containg the same priority.

COMMON BLOCK REFERENCES:  (located in include file "struct.cin")
  Name		Type		Description
  ----		----		-----------
  rec		recfmt		record format of a request
  hdr		hdrfmt		record format of header record of request file
  hdrlen	int		length of header record in bytes
  reclen	int		length of one request in bytes

PROGRAM HISTORY:
  Version	Date	    Author	Change Request
  -------	----	    ------	--------------
   1.0		8/93        K. Gacke	initial development -- copied from QLP

PROJECT:	Unix Color FIRE

Algorithm:
	Basic algorithm of the add subroutine:  The records containing invalid
	requests are stored in a singly linked list.  If this list is not
	empty, the head record is removed, and the new request is written into
	this record.  If the list is empty, a new record is appended to the end
	of the file, and the new request is written into this record.  Before
	the request is written into the file, the doubly linked list must be 
	updated.  A doubly linked list exists for each priority (priority 
	values range from 1 - 9).  The request is added to the end of its 
	priority list.  When the file is processed by the color recorder, the
	requests with high priority will be processed first.  Note that the
	requests within the same priority will be processed as FIFO.

******************************************************************************/

#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "lj.h"
#include "ljsupport.h"
#include "local_protos.h"

static long nonfatal = 1;

int addreq
(
    long fd,
    long crflg,
    long nrec
)
{
extern struct recfmt rec;
extern struct hdrfmt hdr;
extern long suflg;
struct recfmt tmprec;
long  	i;
char    msg[256];

if (rec.ncop * rec.npanel > 1)
    {
    sprintf(msg,"Total frames = %d   (%d panels multiplied by %d copies)",
            rec.ncop * rec.npanel,rec.npanel,rec.ncop);
    c_errmsg(msg," ",&nonfatal);
    }
rec.lad = setlad();		/* set last access date */
if (crflg)			/* if custom request file was created, */
    {				/*   initialize the hdr record to 0's. */
    for (i=0;i<=9;i++)
        {
        hdr.fptr[i] = 0;
        hdr.bptr[i] = 0;
        }
    }
else				/* if custom request file existed, read */
    {				/*   the header record. */
    (void) crfrdhdr(fd,TRUE,&hdr);
    }
/* 
*  Since the request to be added, is inserted at the end of its priority list,
*  its front pointer is set to 0 (indicating it is the end of the list), and
*  its back pointer is set to the last request currently in the list (pointed
*  to by the back pointer of the header).  The delflg of the request is set
*  to -1 indicating it is a valid request.				     
*/
rec.fptr = 0;
rec.bptr = hdr.bptr[rec.prio];
rec.delflg = -1;
/* 
*  If the free list (singly linked list) is empty a new record will be appended
*  to the end of the file.  Note that a request with a sequence number of
*  i will be the ith record in the file.  Else if the free list is not empty, 
*  the record at the head of the list is removed.
*/
if (hdr.fptr[0] == 0)
   rec.seqnum = nrec + 1;
else
   {
   rec.seqnum = hdr.fptr[0];
   crfrd(fd,TRUE,hdr.fptr[0],&tmprec);
   hdr.fptr[0] = tmprec.delflg;
   }
/* 
*  If the request's priority list is empty then the front pointer of the head
*  of the list is set equal to the request's sequence number.                
*/
if (hdr.fptr[rec.prio] == 0)
   hdr.fptr[rec.prio] = rec.seqnum;
/*
*  If the request's priority list is not empty then the front pointer of the
*  last request in the list must be updated to point to the new request.     
*/
if (hdr.bptr[rec.prio] != 0)
   {
   crfrd(fd,TRUE,hdr.bptr[rec.prio],&tmprec);
   tmprec.fptr = rec.seqnum;
   crfwr(fd,TRUE,hdr.bptr[rec.prio],&tmprec);
   }
/* 
*  The back pointer of the head is set to the request's sequence number because
*  the new request will now be the last request in the list.		     
*/
hdr.bptr[rec.prio] = rec.seqnum;
/* 
*  The header record is written into the custom request file.		     
*/
crfwrhdr(fd,TRUE,&hdr);
/* 
*  The request is written into the custom request file.			     
*/
crfwr(fd,TRUE,rec.seqnum,&rec);
return(0);
}     /* END of the ADD subroutine */
