/******************************************************************************

FUNCTION:  ASSIGN_FORMAT

PURPOSE:
   Assign the format for the current frame.  The format is assigned
   based on the NL and NS of the image.  The largest spot size that
   the image fits in is used.
 
PROGRAM HISTORY:
  Version	Date	    Author	Change Request
  -------	----	    ------	--------------
   1		8/90	    K. Gacke	PRF 5701 -- ddr & multiple files on tape
   2		4/91	    K. Gacke	Check for valid color FIRE image size
   3		8/93	    K. Gacke	Copied from vax color FIRE

PROJECT:	LightJet

LANGUAGE	C

ALGORITHM (FLOW):

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "lj.h"
#include "local_protos.h"

static int assign_font
(
	long fmt,
	long nl,
	long nannot_line,
	struct recfmt *rec,
	struct IMGOPT *imgopt
);


int assign_format
(
	struct LJ_PRINTER printer[],
	struct recfmt *rec,
	struct IMGOPT *imgopt,
	long nannot_line
)
{
extern struct fmttab ftable[];
extern struct annot_tab atable[];
float expand;
float expand_nl;
float expand_ns;
long auto_fmt = NULLVAL;
long cnt;
long fmt;
long minfmt;
long maxfmt;
long nonfatal = 1;
long nl;
long ns;
long prcnt;
long printer_cnt;
char msg[256];

maxfmt = NFORMAT;
minfmt = 1;
if (rec->format != NULLVAL)
    {
    maxfmt = rec->format;
    minfmt = rec->format;
    }
printer_cnt = NULLVAL;
imgopt->maxnl = imgopt->maxns = NULLVAL;
for (fmt=maxfmt; fmt>=minfmt; fmt--)  /* check various spot sizes */
    {
    for (prcnt=0; prcnt<MAX_LJ_PARAM; prcnt++) /* check various media types */
	{
	if (strlen(printer[prcnt].mtype) == 0)
	    break;
	if (strcmp(ftable[fmt].pname,printer[prcnt].pname) != 0)
	    continue;
	if (strcmp(rec->pname,printer[prcnt].pname) != 0)
	    continue;
	if ((strlen(rec->film)>0)&&(strcmp(rec->film,printer[prcnt].mtype)!=0))
	    continue;
	if ((imgopt->prntsize) &&
	    ((rec->prntsize[0] * 25.4F > printer[prcnt].msz_line) ||
	     (rec->prntsize[1] * 25.4F > printer[prcnt].msz_sample)))
	    {
	    continue;
	    }
	printer_cnt = prcnt;
	auto_fmt = fmt;
	ftable[fmt].nl = printer[prcnt].msz_line * ftable[fmt].nl_mm;
	ftable[fmt].ns = printer[prcnt].msz_sample * ftable[fmt].nl_mm;
/*
* If ouput print size is specified, reset the output frame size.  
*/
	if (imgopt->prntsize)
	    {
	    ftable[fmt].nl = (long)
		(rec->prntsize[0] * 25.4F * ftable[fmt].nl_mm +1L); 
	    ftable[fmt].ns = (long)
		(rec->prntsize[1] * 25.4F * ftable[fmt].nl_mm +1L);
	    }
        if (imgopt->mapscale)
	    {
	    if (rec->pixsize < 0)
	        {
	        sprintf(msg,"%s %s","For map scale computation,",
		    "the pixel size is required to be specified in meters");
 	        c_errmsg(msg," ",&nonfatal);
	        return(E_FAIL);
	        }
	    expand = 
		((rec->pixsize * 1000000.0) / ftable[fmt].spot) / rec->mapscale;
	    if (expand < 1)
	        {
	        sprintf(msg,"Map scale of %5.2f %s %5.2f %s",
		    rec->mapscale,"would result in an image expansion of",
		    expand,"Image expansion less than 1 is not supported");
 	        c_errmsg(msg," ",&nonfatal);
	        return(E_FAIL);
	        }
/*
*           Check to see if the next higher printer resolution can be used to
*	    obtain the desired map scale.
*/
	    if ((fmt-1 >= minfmt) &&
	    (((rec->pixsize*1000000.0)/ftable[fmt-1].spot)/rec->mapscale >= 1))
	        {
	        continue;
	        }
	    imgopt->expand = TRUE;
	    rec->expand[0] = rec->expand[1] = expand;
	    }
        if (imgopt->reduce)
            {
            nl = rec->nl / rec->reduce[0];
            if (rec->nl % rec->reduce[0] != 0)
                ++nl;
            ns = rec->ns / rec->reduce[1];
            if (rec->ns % rec->reduce[1] != 0)
                ++ns;
            }
        else if (imgopt->expand)
            {
            nl = (long) ((float)rec->nl * rec->expand[0]);
            ns = (long) ((float)rec->ns * rec->expand[1]);
            }
        else
            {
            nl = rec->nl;
            ns = rec->ns;
            }
        if (imgopt->reduce)
	    assign_font(fmt,nl,nannot_line,rec,imgopt);
        else
            assign_font(fmt,rec->nl,nannot_line,rec,imgopt);
        imgopt->margin[0] = (long)((float)rec->margin[0] * 25.4F *
                                ftable[fmt].nl_mm);
        imgopt->margin[1] = (long)((float)rec->margin[1] * 25.4F *
                                ftable[fmt].nl_mm);
        if (imgopt->expand)
	    {
	    imgopt->margin[0] /= rec->expand[0];
	    imgopt->margin[1] /= rec->expand[1];
	    }
        imgopt->maxnl = ftable[fmt].nl-(imgopt->fontnl)-(imgopt->margin[0]*2);
        imgopt->maxns = ftable[fmt].ns - (imgopt->margin[1] * 2);
        if ((nl<=imgopt->maxnl) && (ns<=imgopt->maxns))
	    break;
	}
    if (imgopt->maxnl == NULLVAL)
	continue;
    if ((nl<=imgopt->maxnl) && (ns<=imgopt->maxns))
	break;
    }
if (printer_cnt == NULLVAL)
    {
    sprintf(msg,
	    "Invalid media and/or print size specification for printer %s.",
	    rec->pname);
    c_errmsg(msg," ",&nonfatal);
    return(E_FAIL);
    }
if (strlen(rec->film)==0)
    {
    strcpy(rec->film,printer[printer_cnt].mtype);
    ftable[fmt].nl = printer[printer_cnt].msz_line * ftable[fmt].nl_mm;
    ftable[fmt].ns = printer[printer_cnt].msz_sample * ftable[fmt].nl_mm;
    }
if (fmt < minfmt)		/* image size > max LightJet size => Panel */
    {
    fmt = auto_fmt;
    if (imgopt->reduce) 
	assign_font(fmt,nl,nannot_line,rec,imgopt);
    else
        assign_font(fmt,rec->nl,nannot_line,rec,imgopt);
    }
if (rec->format == NULLVAL)
    rec->format = fmt;
imgopt->margin[0] = (long)((float)rec->margin[0] * 25.4F *
                        ftable[rec->format].nl_mm);
imgopt->margin[1] = (long)((float)rec->margin[1] * 25.4F *
                        ftable[rec->format].nl_mm);
if (imgopt->prntsize)
    {
    imgopt->maxnl = ftable[fmt].nl = 
	(long) (rec->prntsize[0] * 25.4F * ftable[fmt].nl_mm +1L);
    imgopt->maxns = ftable[fmt].ns = 
	(long) (rec->prntsize[1] * 25.4F * ftable[fmt].nl_mm +1L); 
    expand_nl = (float)(ftable[fmt].nl) / 
	(float)(rec->nl + imgopt->fontnl + (2 * imgopt->margin[0]));
    expand_ns = (float)(ftable[fmt].ns) / 
	(float)(rec->ns + (2 * imgopt->margin[1]));
    expand = (expand_nl < expand_ns) ? expand_nl : expand_ns;
    if (expand > 1)
	{
	imgopt->expand = TRUE;
	rec->expand[0] = rec->expand[1] = expand;
/*
*  loop to zero in on correct font and margin settings reducing the
*  font size and/or margins.
*/
	for (cnt=0; cnt<5; cnt++)
	    {    
	    imgopt->margin[0] = (long)((float)rec->margin[0] * 25.4F *
                        ftable[rec->format].nl_mm);
	    imgopt->margin[1] = (long)((float)rec->margin[1] * 25.4F *
                        ftable[rec->format].nl_mm);
            imgopt->margin[0] /= rec->expand[0];
            imgopt->margin[1] /= rec->expand[1];
            assign_font(fmt,rec->nl,nannot_line,rec,imgopt);
            expand_nl = (float)(ftable[fmt].nl) / 
		(float)(rec->nl + imgopt->fontnl + (2 * imgopt->margin[0]));
            expand_ns = (float)(ftable[fmt].ns) / 
		(float)(rec->ns + (2 * imgopt->margin[1]));
            expand = (expand_nl < expand_ns) ? expand_nl : expand_ns;
	    rec->expand[0] = rec->expand[1] = expand;
	    }
	}
    else if (expand < 1)
	{
	sprintf(msg,"Print size of %5.2fx%5.2f %s %5.2f %s",
	    rec->prntsize[0],rec->prntsize[1],
	    "would result in an image expansion of",
	    expand,"Image expansion less than 1 is not supported");
 	c_errmsg(msg," ",&nonfatal);
	return(E_FAIL);
	}
    }
else if (imgopt->expand)
    {
    imgopt->maxnl = nl + (long)((float)imgopt->fontnl*rec->expand[0]) +
                    (imgopt->margin[0]*2) + 1;
    imgopt->maxns = ns + (imgopt->margin[1] * 2) + 1;
    imgopt->margin[0] /= rec->expand[0];
    imgopt->margin[1] /= rec->expand[1];
    }
else
    {
    imgopt->maxnl = nl + imgopt->fontnl + (imgopt->margin[0] * 2);
    imgopt->maxns = ns + (imgopt->margin[1] * 2);
    }
imgopt->format = rec->format;
if (imgopt->maxnl > ftable[rec->format].nl)
    imgopt->maxnl = ftable[rec->format].nl;
if (imgopt->maxns > ftable[rec->format].ns)
    imgopt->maxns = ftable[rec->format].ns;
/*
*  If printer is the LJ2080, then set the max nl and ns to the full frame.
*/
if (strcmp(ftable[rec->format].pname,"LJ2080") == 0)
    {
    imgopt->maxnl = ftable[rec->format].nl;
    imgopt->maxns = ftable[rec->format].ns;
    }
return(E_SUCC);
}

static int assign_font
(
	long fmt,
	long nl,
	long nannot_line,
	struct recfmt *rec,
	struct IMGOPT *imgopt
)
{
extern struct fmttab ftable[];
extern struct annot_tab atable[];
long target;
long actual;
long sf_var;
long sf;
long font_nmr;

/*
*  Set target font size for the LJ2080 to be 2.5mm.
*/
if (strcmp(ftable[fmt].pname,"LJ2080") == 0)
    {
    target = (long) (2.5F * ftable[fmt].nl_mm);
    if (imgopt->expand)
	target /= rec->expand[0];
    }
/*
*  Set target font size for the LJ5000 to be ~.5% of image size.  If the 
*  LightJet print is not going to be expanded, the minium font size is set 
*  to 3mm. The maximum font size is set to 8mm.  If image is being expanded,
*  ensure the expanded font size does not exceed 8mm.
*/
if (strcmp(ftable[fmt].pname,"LJ5000") == 0)
    {
    target = (long) (nl * .005);		
    if (rec->npanel > 1)
	target /= rec->npanel;
    if ((!imgopt->expand) && (target / ftable[fmt].nl_mm < 3))
        target = 3 * ftable[fmt].nl_mm;
    if (target / ftable[fmt].nl_mm > 8)
        target = 8 * ftable[fmt].nl_mm;
    if ((imgopt->expand) && 
	(rec->expand[0] * ((float)target / (float)ftable[fmt].nl_mm) > 8.0F))
	target = (long)((float) (8.0F / (float) rec->expand[0]) * ftable[fmt].nl_mm);
    }
actual = 10000;
for (font_nmr=0; font_nmr<=4; font_nmr++)
    {
    for (sf=1; sf<=4; sf++)
	{
	sf_var = abs(atable[font_nmr].nl * sf - target);
	if (sf_var < actual)
	    {
	    actual = sf_var;
	    imgopt->font_nmr = font_nmr;
	    imgopt->font_scale = sf;
	    }
	else if ((sf_var == actual) && (imgopt->font_scale > sf))
	    {
	    actual = sf_var;
	    imgopt->font_nmr = font_nmr;
	    imgopt->font_scale = sf;
	    }
	}
/*
* Font0 is very blocky font; therefore, do not allow scaling factor greater
* than one.
*/
    if ((imgopt->font_nmr == 0) && (imgopt->font_scale > 1))
	actual = 10000;
    }
imgopt->fontnl = nannot_line * 
	(atable[imgopt->font_nmr].nl * imgopt->font_scale);
return(0);
}
