/******************************************************************************

FUNCTION:  GETLUT

PURPOSE:
	GETLUT generates the three look up tables (red, green, and blue).  It 
	constructs the final look up table using a combination of the control, 
	calibration, and enhancement LUT files as specified in the input 
	parameters.  

PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0           1/98       K. Gacke    initial development -- copied from QLP

PROJECT:        LightJet 5000

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include "las.h"
#include "lj.h"
#include "lj2000.h"
#include "local_protos.h"

extern int Vflg;

int lj_initlut
(
   struct lj2000_lut_struct *lj_lut
);

int lj_fillout_header
(
    struct lj2000_lut_struct *lj_lut
);

int tothepowerof (int n, int exp);


int getlut
(
   struct recfmt *rec,
   char *calfile,
   char *confile,
   long lutmode,			/* what mode? (LUT_PROD or LUT_TEST) */
   struct lj2000_lut_struct *lj_lut
)
{
struct lut callut[3];
struct lut conlut[3];
struct lut enhlut[3];
float	enh[MAXPT],cal[MAXPT],con[MAXPT];
long	nonfatal = 1;
long 	i;
long	rstat;
long	status;
short int tlut[3][MAXPT];
char	fname[256];
char	msg[256];
char	*lutdir;

rstat = E_SUCC;
(void) lj_initlut(lj_lut);
for (i=0; i<3; i++)
    {
    enhlut[i].in[0] = enhlut[i].out[0] = 0;
    enhlut[i].in[1] = enhlut[i].out[1] = 4095;
    enhlut[i].ne = 2;
    }
if (lutmode == LUT_TEST)			/* test lut mode */
    lutdir = getenv("LJTEST");
else
    lutdir = getenv("LJLUT");
if (lutdir == NULL)
    {
    c_errmsg("getlut: Environment variable LJLUT not set"," ",&nonfatal);
    rstat = E_FAIL;
    goto xit;
    }
sprintf(fname,"%s/%s/%s",lutdir,rec->film,confile);
sprintf(msg,"getlut con lut: %s",fname);
lj_logmess(2," ",msg); 
status = rdlut("RGB",NULLVAL,fname,"CON",conlut); /* read control file */
if (status != E_SUCC)
    {
    sprintf(rec->errtext,"Error reading control LUT %s",fname);
    rstat = E_FAIL;
    goto xit;
    }
sprintf(fname,"%s/%s/%s",lutdir,rec->film,calfile);
sprintf(msg,"getlut cal lut: %s",fname);
lj_logmess(2," ",msg); 
status = rdlut("RGB",NULLVAL,fname,"CAL",callut); /* read calibration file */
if (status != E_SUCC)
    {
    sprintf(rec->errtext,"Error reading calibration LUT %s",fname);
    rstat = E_FAIL;
    goto xit;
    }
/*
*  Do for each color (red, green, blue).
*  call br_enhance to create the enhance lut
*  call br_calbrt to create the calibration lut
*  call br_contrl to create the control lut
*  call br_punt to combine the enhancement, calibration, and control luts
*/
for (i=0; i<3; i++)
   {
   status = br_enhance(enhlut[i].in,enhlut[i].out,&enhlut[i].ne,enh); 
   if (status == -1)
      {
      sprintf(rec->errtext,"Enhnacement LUT not terminated with 4095");
      rstat = E_FAIL;
      goto xit;
      }
   if (status == -2)
      {
      sprintf(rec->errtext,"Enhancement LUT does not begin with 0");
      rstat = E_FAIL;
      goto xit;
      }
   br_calbrt(callut[i].in,callut[i].out,&callut[i].ne,cal);
   br_contrl(conlut[i].in,conlut[i].out,&conlut[i].ne,con);
   br_punt(cal,con,enh,tlut[i]);
   }
for (i=0; i<MAXPT; i++)
    {
    lj_lut->lut[0][i] = lj_byteswap(tlut[0][i]);
    lj_lut->lut[1][i] = lj_byteswap(tlut[1][i]);
    lj_lut->lut[2][i] = lj_byteswap(tlut[2][i]);
    }
/* lj_align_lut(lj_lut); */
lj_fillout_header(lj_lut);
{
FILE *fp;

fp = fopen("ken.lut","w");
if (fp != NULL)
    {
    fwrite(lj_lut, sizeof(*lj_lut), 1, fp);
    fclose(fp);
    }
}
xit:
return(rstat);
}

/*
 * Copy the postlut into a temporary table
 * Clear the postlut table
 * realign the poslut into position, as defined by the prelut
 */
int lj_align_lut
(
    struct lj2000_lut_struct *lj_lut
)
{

int i, j, color;
UInt16 templut[LUT_S];

for (color = 0; color < NO_OF_LUTS; color++) 
    {
    for (i = 0; i < LUT_S; i++) 
	{
	templut[i] = lj_lut->lut[color][i];
	lj_lut->lut[color][i] = 0;
	}
    for (i = 0; i < LUT_ENTRIES; i++)
	for (j=0; j<POSTLUT_PER_ENTRY; j++)
	    lj_lut->lut[color][j+(i*POSTLUT_PER_ENTRY)] = templut[i]; 
    }
return(E_SUCC);
}

int lj_fillout_header
(
    struct lj2000_lut_struct *lj_lut
)
{

int i;
unsigned int checksum;

/* Fillout comment field */
strncpy ((char *)&lj_lut->header[LUTHEAD_COMMENT],
	    "LightJet LUT",LUTHEAD_COMMENT_S);

/* Fillout "File ID" (static) */
strncpy ((char *)&lj_lut->header[LUTHEAD_ID],LUTHEAD_ID_STR, LUTHEAD_ID_S);

/* Fillout "Number of Colors" (static) */
lj_lut->header[LUTHEAD_NUMCOL] = LUTHEAD_NUMCOL_STR;

/* Fillout "pointer to start of LUT data" (static) */
strncpy ((char *)&lj_lut->header[LUTHEAD_POINTTOSTART],
	    LUTHEAD_POINTTOSTART_STR, LUTHEAD_POINTTOSTART_S);

/* Fillout "Number of bytes to download" (static) */
strncpy ((char *)&lj_lut->header[LUTHEAD_BYTES],
	    LUTHEAD_BYTES_STR, LUTHEAD_BYTES_S);

/* Zero out checksum one byte at a time */
for (i = 0; i < LUTHEAD_CHECKSUM_S; i++) 
    {
    lj_lut->header[LUTHEAD_CHECKSUM + i] = 0x00;
    }

/* Calculate checksum (dynamic) */
checksum = lj_lut->header[0];
for (i = 1; i < sizeof(struct lj2000_lut_struct); i++)
    checksum += lj_lut->header[i];
checksum += 1;

/* Fill out checksum one byte at a time */
for (i = 0; i < LUTHEAD_CHECKSUM_S; i++) 
    {
    lj_lut->header[LUTHEAD_CHECKSUM + i] = checksum & 0xFF;
    checksum >>= 8;
    }
return(E_SUCC);
}

int lj_initlut
(
   struct lj2000_lut_struct *lj_lut
)
{
UInt16 i, j, color;

for (color = 0; color < NO_OF_LUTS; color++) 
    {
    /* Initialize 3D LUTs */
    for( i = 0; i < THREEDLUT_S; i++ ) 
	{
        j = i / (UInt16)(tothepowerof(LUT3D_CYCLE,color));
        lj_lut->threedlut[color][i] = 
		lj_byteswap ((UInt16)((j%LUT3D_CYCLE) * LUT3D_MAX) / 
		(UInt16)(LUT3D_CYCLE - 1));
        }
    /* Initialize PreLUTs */
    for( i = 0; i < PRELUT_S; i++ )
	{
        lj_lut->prelut[color][i] = lj_byteswap(
		(unsigned short)((float)i * (float)((float)PRE_LUT_MAX3_VAL / (float)(PRELUT_S-1))));
	}
    /* Initial k PreLuts */
    for( i = 0; i < PRELUT_S; i++ )
	{
        lj_lut->kprelut[color][i] = 0;
        }
    }
return(1);
}

int tothepowerof (int n, int exp)
{
int i, temp;

switch (exp) 
    {
    case 0:
        return (1);
    case 1:
        return (n);
    default:
        temp = n;
        for (i=1; i < exp; i++)
            n *= temp;
        return (n);
    }
}

