/******************************************************************************
FUNCTION:       MAKEVDFS

PURPOSE:        Create associated VDF files for each output tape.  This
		function uses the data from the original, single VDF to produce
		three VDFs that correctly reflect the changes necessary when
		spanning three tapes as opposed to one.

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
1.0      6/96   C. Engebretson   Original development

ALGORITHM REFERENCES:

        "Landsat Technical Working Group (LTWG) Format for Thematic
        Mapper CCT, Full Scene", EOSAT, January 1, 1990

	"TM CCT Format from TRSA (Thailand)"

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util.h"

int main
(void)
{
struct PARBLK parblk;           /* TAE parameter block                  */
struct VARIABLE *val;           /* Pointer to TAE variable within block */

FILE *inf;                      /* Input file                           */
FILE *outf[3];                  /* Output files                         */
unsigned char *buffer;		/* I/O buffer				*/
char session[CMLEN];		/* TAE session ID			*/
char infile[CMLEN];		/* String used to hold input filename	*/
char outfile[3][CMLEN];		/* String used to hold output filenames */
char temp[CMLEN];		/* Temporary string			*/

long std_out;			/* Standard output			*/
int lsize;			/* Size of leader file records		*/
int isize;			/* Size of image records		*/
int tsize;			/* Size of trailer file records		*/
int *ptr;			/* Temporary pointer			*/
int i, j;			/* Loop counter				*/


/* Initialize LAS
-----------------*/
c_inlas(&parblk, &std_out);


/* Retrieve the input parameters
--------------------------------*/
val = p_find(&parblk, "$SESSION");
strcpy(session, SVAL(*val, 0));


/* Construct input and output filenames and open them for reading/writing
-------------------------------------------------------------------------*/
sprintf(infile, "ltwg_%s_1.vdf", session);
for(i = 1; i <= 3; i++)
	sprintf(outfile[(i-1)], "ltwg_%s_tape%d_1.vdf", session, i);

/* The most data we will read from the VDF file is 9 records at 360 bytes
   each; therefore, we need 3240 bytes of buffer memory. */

buffer = (unsigned char *) malloc (3240 * sizeof(unsigned char));
if (!buffer)
	c_lasmsg(TRUE, "ltwg123-alloc", "Error allocating dynamic memory");

if ((inf = fopen(infile, "r")) == NULL)
	c_lasmsg(TRUE, "ltwg123-file", "Cannot open VDF file for reading");

for (i = 0; i < 3; i++)
	if ((outf[i] = fopen(outfile[i], "w")) == NULL)
		c_lasmsg(TRUE, "ltwg123-file",
			 "Cannot open VDF file for writing");


/* Read in the first record from the input VDF and write to each file.
----------------------------------------------------------------------*/
if (fread(buffer, sizeof(unsigned char), 360, inf) != 360)
	c_lasmsg(TRUE, "ltwg123-read", "Error reading data from VDF file");
for (i = 0; i < 3; i++)
	{

	buffer[97] = '3';		/* Total number of volumes	*/
	buffer[99] = ('1' + i);		/* Number of current volume	*/
	buffer[103] = '1';		/* Number of first volume	*/

	if (fwrite(buffer, sizeof(unsigned char), 360, outf[i]) != 360)
		c_lasmsg(TRUE, "ltwg123-write",
			 "Error writing data to VDF file");
	}


/* Read in the next nine records from the VDF.  These correspond to the
   pointer records for the leader, image, and trailer files of the first
   three bands of the image.  Update the information to reflect that they
   are found on the first tape.
------------------------------------------------------------------------*/
for (i = 0; i < 9; i++)
{
    if (fread(buffer, sizeof(unsigned char), 360, inf) != 360)
	    c_lasmsg(TRUE, "ltwg123-read", "Error reading data from VDF file");
    switch(i)
    {

	/* To ensure that LTWG123 is as generic as possible, we must extract
	   the sizes of the various image file records (the leader, image, 
	   and trailer files.)  These are found in the pointer records for
	   each of the aforementioned files.  The values are converted to
	   longs and then passed back through the parblk to TAE for later use.
	--------------------------------------------------------------------*/
	case 0:
	case 1:
	case 2:
		strncpy(temp, (char *) &buffer[108], 8);
		buffer[16] = 0;

		/* This is messy, but it works!
		-------------------------------*/
		ptr = (i == 0 ? &lsize : (i == 1 ? &isize : &tsize));
		*ptr = atoi(temp);
		q_intg(&parblk, (i == 0 ? "LSIZE":(i == 1 ? "ISIZE" : "TSIZE")),
		       1,
		       (i == 0 ? &lsize : (i == 1 ? &isize : &tsize)),
		       P_UPDATE);
		break;
	default:
		break;
    }


    buffer[141] = '1';	/* Volume number this file begins on		*/
    buffer[143] = '1';	/* Volume number this file ends on (same!)	*/

    for (j = 0; j < 3; j++)
    if (fwrite(buffer, sizeof(unsigned char), 360, outf[j]) != 360)
	    c_lasmsg(TRUE, "ltwg123-write", "Error writing data to VDF file");
}


/* The next nine records in the VDF are the pointers to the leader, image,
   and trailer files for bands four through seven, found on volume #2.
---------------------------------------------------------------------------*/
for (i = 0; i < 9; i++)
{
    if (fread(buffer, sizeof(unsigned char), 360, inf) != 360)
	    c_lasmsg(TRUE, "ltwg123-read", "Error reading data from VDF file");
    buffer[141] = '2'; 
    buffer[143] = '2';
    for (j = 0; j < 3; j++)
    if (fwrite(buffer, sizeof(unsigned char), 360, outf[j]) != 360)
	    c_lasmsg(TRUE, "ltwg123-write", "Error writing data to VDF file");
}


/* Finally, the next three records in the VDF file are the information for
   band seven, which are found on the third 3480 physical volume.
--------------------------------------------------------------------------*/
for (i = 0; i < 3; i++)
{
    if (fread(buffer, sizeof(unsigned char), 360, inf) != 360)
	    c_lasmsg(TRUE, "ltwg123-read", "Error reading data from VDF file");
    buffer[141] = '3';
    buffer[143] = '3';
    for (j = 0; j < 3; j++)
    if (fwrite(buffer, sizeof(unsigned char), 360, outf[j]) != 360)
	    c_lasmsg(TRUE, "ltwg123-write", "Error writing data to VDF file");
}


/* To finish up, we need only to write the imagery logical volume text
   record (the final record of the VDF) to all three output files.
----------------------------------------------------------------------*/
if (fread(buffer, sizeof(unsigned char), 360, inf) != 360)
        c_lasmsg(TRUE, "ltwg123-read", "Error reading data from VDF file");
for (i = 0; i < 3; i++)
	{

	/* We have to modify some of this information slightly.  Instead of
	   reading "TAPE 1 OF 1", the information in this field should read
	   "TAPE <x> OF 3". */

	buffer[207] = (i + '1');
	buffer[213] = '3';
        if (fwrite(buffer, sizeof(unsigned char), 360, outf[i]) != 360)
                c_lasmsg(TRUE, "ltwg123-write",
                         "Error writing data to VDF file");
	}

/* Clean up
-----------*/
free(buffer);
fclose(inf);
for (i = 0; i < 3; fclose(outf[i++]));
q_out(&parblk); 		/* Return the parameters to TAE 	*/

return EXIT_SUCCESS;

}
