/******************************************************************************
FUNCTION:     	READLEADER 

PURPOSE:        Read the leader file copied by DITTO and extract the needed
  		information to write to the metadata file. 

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
1.0      6/96   C. Engebretson   Original development

ALGORITHM REFERENCES:

	"Landsat Technical Working Group (LTWG) Format for Thematic 
	Mapper CCT, Full Scene", EOSAT, January 1, 1990

	"TM CCT Format From TRSA (Thailand)"

	See also code for LTWGIN module 'getlead.c'

NOTES:  This code was designed to extract valid metadata from the header
	files for Thailand data.  It should also extract valid metadata from
	any other data that conforms _strictly_ to the EOSAT document's
	specified format.  However, modifications to this code may be
	necessary to extract information from other foreign data formats
	that deviate from this document's specifications.

*******************************************************************************/

#include <stdio.h>   
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "header.h"
#include "metadata.h"

int main
(void)
{
struct PARBLK parblk;		/* TAE parameter block			*/
struct VARIABLE *val;		/* Pointer to TAE variable within block	*/

FILE *inf;			/* Input file				*/
FILE *vdfinf;			/* Input VDF file			*/
char *buffer;		/* Leader file buffer			*/
char *vdfbuffer;	/* VDF file buffer			*/

char leader[CMLEN];		/* Name of input leader file		*/
char vdf[CMLEN];		/* Name of input VDF file 		*/
char outfile[CMLEN];		/* Name of output (metadata) file	*/
char project[CMLEN];		/* Project name (for metadata)		*/
char tapeloc[3][7];		/* Output tape locations		*/
char *outloc[4];		/* Tape locations for call metadata	*/
char tempstr[255];		/* General-purpose temporary string 	*/
char tempstr2[255];		/* General-purpose temporary string	*/
char *str_ptr[1];		/* Temporary pointer for q_string()	*/
char *ptr;			/* General-purpose string ptr		*/
long recsize = 4320;		/* Size of leader file records		*/
long bytes_read;		/* Bytes read from leader file		*/
long std_out;			/* Standard output			*/
long mode = IREAD;		/* File access				*/
long fatal = -1;		/* Flag for fatal error			*/
long zero = 0;			/* Constant of zero 			*/
long utmflag = 0;			/* Flag for UTM data			*/
long thaiflag = 0;			/* Flag for Thailand data		*/
int i;				/* Loop counter				*/


/* Initialize LAS
-----------------*/ 
c_inlas(&parblk, &std_out);


/* Retrieve the input parameters
--------------------------------*/
if (c_getnam(&parblk, "LEADER", &zero, leader, "lead", &mode) != E_SUCC)
	c_errmsg("Fatal error encountered", "ltwg123-fatal", &fatal);
if (c_getnam(&parblk, "VDF", &zero, vdf, "vdf", &mode) != E_SUCC)
	c_errmsg("Fatal error encountered", "ltwg123-fatal", &fatal);
val = p_find(&parblk, "OUTFILE");
strcpy(outfile, SVAL(*val, 0));
val = p_find(&parblk, "PROJECT");
strcpy(project, SVAL(*val, 0));
val = p_find(&parblk, "TAPELOC");
for (i = 0; i < 3; i++)
	strcpy(tapeloc[i], SVAL(*val, i));


/* Open the input files for reading
-----------------------------------*/
inf = fopen(leader, "r");
if (!inf) c_errmsg("Unable to open leader file for reading",
		   "ltwg123-file", &fatal);
vdfinf = fopen(vdf, "r");
if (!vdfinf) c_errmsg("Unable to open VDF file for reading",
		   "ltwg123-file", &fatal);


/* Allocate buffer memory and read data in from the leader/VDF file
-------------------------------------------------------------------*/

/* The leader file is composed of 5 records of recsize bytes each, which means
   that we need to allocate 21600 bytes of space to hold it all. */

buffer = (char *) malloc (5 * recsize);
if (!buffer)
	c_errmsg("Error allocating dynamic memory", "ltwg123-alloc", &fatal);
vdfbuffer = (char *) malloc (720);
if (!vdfbuffer)
	c_errmsg("Error allocating dynamic memory", "ltwg123-alloc", &fatal);
bytes_read = (long) fread(buffer, sizeof(unsigned char), 5 * recsize, inf);
if (bytes_read != 5 * recsize)
	c_errmsg("Error reading data from leader file", "ltwg123-read", &fatal);
bytes_read = (long) fread(vdfbuffer, sizeof(unsigned char), 720, vdfinf);
if (bytes_read != 720)
	c_errmsg("Error reading data from VDF file", "ltwg123-read", &fatal);
fclose(inf);


/* Initialize the metadata module and send out project name
-----------------------------------------------------------*/
meta_init(outfile);
for (i = 0; i < 3; i++) outloc[i] = tapeloc[i];
outloc[3] = NULL;
meta_writearchv(outloc, "CEOS", 1);
meta_insert(PROJECT, project);


/* Determine the scene ID and write it to the metadata file.  This infor-
   mation also must be sent back to TAE for use on the tape label.
-------------------------------------------------------------------------*/
strncpy(tempstr, &buffer[recsize + 36], 16);       /* Scene ID              */
tempstr[16] = 0;
meta_insert(OLD_SCENE, tempstr);
str_ptr[0] = tempstr;
if (q_string(&parblk, "SCENEID", 1, str_ptr, P_UPDATE) != SUCCESS)
        c_lasmsg(TRUE, "ltwg123-pass", "Error passing variables to TAE");
if (q_out(&parblk) != SUCCESS)
        c_lasmsg(TRUE, "ltwg123-pass", "Error passing variables to TAE");


/* This image format (LGSOWG TM) has seven bands (band #6 being the thermal
   band).  Reflect this in SPECT_BAND (bands available)
---------------------------------------------------------------------------*/
meta_insert(SPECT_BAND, "1234567");


/* Recieving station
--------------------*/
strncpy(tempstr, &vdfbuffer[128], 12);
tempstr[12] = 0;
meta_insert(ORIG_SOURCE, tempstr);


/* If we are dealing with Thailand data, we will have to allow for several
   Thailand-specific deviations in the leader file format.
--------------------------------------------------------------------------*/
if (!strncmp(tempstr, "THAILAND", 8)) thaiflag = 1;


/* WRS path and row
-------------------*/
tempstr[3] = 0;
strncpy(tempstr, &buffer[recsize + 165], 3);
meta_insert(PATH_NBR, tempstr);
strncpy(tempstr, &buffer[recsize + 168], 3);
meta_insert(ROW_NBR, tempstr);


/* Scene center date
--------------------*/
strncpy(tempstr, &buffer[recsize + 116], 8);
tempstr[8] = 0;
meta_insert(ACQU_DATE, tempstr);


/* Product Identification
-------------------------*/
strncpy(tempstr, &buffer[recsize + 20], 16);
tempstr[16] = 0;
meta_insert(PROD_SIZE, tempstr);


/* Satellite Number
-------------------*/
tempstr[0] = buffer[50];
tempstr[1] = 0;
meta_insert(SATELLITE, tempstr);


/* Recording Technique
----------------------*/
strncpy(tempstr, &buffer[recsize + 324], 3); /* Sensor */

/* The significance of the Recording Technique values are as follows:

   Sensor	Orientation	Recording Technique
   ------	-----------	-------------------
   MSS		Descending	38
   MSS		Ascending	39
   TM		Descending	41
   TM 		Ascending	42 				   */

if (tempstr[0] == 'T')
{
	if (buffer[recsize + 356] == 'A')
		meta_insert(RECD_TECH, "42");
	else meta_insert(RECD_TECH, "41");
}
if (tempstr[0] == 'M')
{
	if (buffer[recsize + 356] == 'A')
		meta_insert(RECD_TECH, "39");
	else meta_insert(RECD_TECH, "38");
}


/* Thailand data resampling type is found in a different location than is
   described in the generic format document.
-------------------------------------------------------------------------*/
if (thaiflag)
{
    strncpy(tempstr, &buffer[recsize + 1552], 4);  /* Resampling type	    */
    tempstr[4] = 0;
    meta_insert(RESAMP_TECH, tempstr);
}
else
{
    strncpy(tempstr, &buffer[recsize + 1540], 2);
    tempstr[2] = 0;
    meta_insert(RESAMP_TECH, tempstr);
}


/* Thailand data has a "flag" for UTM data that is set to either 'Y' or 'N'.
   This differs from the standard data format where the map projection is
   named explicitly.
----------------------------------------------------------------------------*/
if (thaiflag)
{
    utmflag = (buffer[recsize + 1557] == 'Y');
    meta_insert(MAP_PROJEC, utmflag ? "UTM" : "SOM");
}
else
{
    strncpy(tempstr, &buffer[recsize + 1556], 3);
    tempstr[3] = 0;
    meta_insert(MAP_PROJEC, tempstr);
    if (!strcmp(tempstr, "UTM")) utmflag = 1;
}

/* UTM Zone
-----------*/
if (utmflag)
{
	strncpy(tempstr, &buffer[(recsize * 2) + 98], 10);
	tempstr[10] = 0;
	meta_insert(MAP_ZONE, tempstr);
}


/* Band Leaving Type
--------------------*/
strncpy(tempstr, &vdfbuffer[392], 3);
tempstr[3] = 0;
meta_insert(INTERLVNG, tempstr);


/* Latitude and longitude.  The data is stored in the leader file as
   positive or negative floating point values; convert these to read
   as East/West or North/South based on the sign of the value.
--------------------------------------------------------------------*/
strncpy(tempstr, &buffer[recsize + 52], 16);
tempstr[16] = 0;

/* Truncate floating-point representation to conform to the standard
   format expected in the metadata.  Directions (i.e., "north or south"
   latitude) are not used; rather, the signs of the numbers indicate
   the orientation. */

sprintf(tempstr2, "%2.5f", atof(tempstr));
meta_insert(CENTER_LAT, tempstr2);

strncpy(tempstr, &buffer[recsize + 68], 16);
tempstr[16] = 0;
sprintf(tempstr2, "%3.5f", atof(tempstr));
meta_insert(CENTER_LON, tempstr2);


/* Horizontal Datum
-------------------*/
strncpy(tempstr, &buffer[(recsize * 2) + 92], 6);
tempstr[6] = 0;
meta_insert(HORIZ_DATUM, tempstr);


/* Pixel Width and Height
-------------------------*/
tempstr[16] = 0;
strncpy(tempstr, &buffer[(recsize * 2) + 364], 16);
sprintf(tempstr2, "%3.4f", atof(tempstr));
meta_insert(PIXEL_WIDTH, tempstr2);
strncpy(tempstr, &buffer[(recsize * 2) + 380], 16);
sprintf(tempstr2, "%3.4f", atof(tempstr));
meta_insert(PIXEL_HGHT, tempstr2);


/* Sun Elevation and Azimuth
----------------------------*/
strncpy(tempstr, &buffer[(recsize * 2) + 604], 16);

/* "Chop off" values beyond the decimal point.  They are not needed for
   this metadata format. */

if ((ptr = strchr(tempstr, '.')) != NULL) *ptr = '\0';
meta_insert(SUN_ELEVATION, tempstr);
strncpy(tempstr, &buffer[(recsize * 2) + 620], 16);
if ((ptr = strchr(tempstr, '.')) != NULL) *ptr = '\0';
meta_insert(SUN_AZIMUTH, tempstr);


/* Close output file and clean up
---------------------------------*/
meta_write();
meta_cleanup();
fclose(inf);
fclose(vdfinf);
free(buffer);
free(vdfbuffer);

return EXIT_SUCCESS;

}
