/*******************************************************************************
FUNCTION: C_GETFILES

PURPOSE:  Given a file specification find all files that match this
          specification.  For example, /sg2/csb/user/mydir/*.c for UNIX or
          [user.mydir]* for VMS.


Version  Date   Programmer       Modification
-------  -----  ---------------  -----------------------------------------------
  1.0    05/92  J.Fenno          New version of dirlist that will work 
				 correctly on both UNIX and VMS systems.
  1.1    07/92  J.Fenno          Added code so that the pipe gets closed
				 properly and does not leave defunct processes.
  1.2	 07/92  T.Baltzer        Add locked file flag logic.
  1.3    12/93  J.Fenno          Added check for !first_call and file_ptr==NULL


PROJECT: LAS

ALGORITHM REFERENCES: LAS support dirlist
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"


/*****************
*  UNIX VERSION  *
*****************/
#ifdef unix

    static FILE *file_ptr = NULL;/* Temp file ptr to the list of file names   */

    FUNCTION c_getfiles (filespec, file_names, nfiles,
                         maxfiles, first_call, ck_lock)

	char *filespec;		/* Host file specification for files to find  */
	char file_names[][CMLEN];/* Array of file names to be output          */
	long *nfiles;		/* Number of files found matching filespec    */
	long maxfiles;		/* Max number of files to return at one time  */
	long first_call;	/* Flag for first call to this function (0,1) */
	long ck_lock;		/* 1 = return list of unlocked files only,    */
	                        /* 0 = return list of all files found         */
    {
    char filename[CMLEN];	/* Name of one file                           */
    char lockfile[CMLEN];	/* Name of one 'locked' file		      */
    char *strcpy();		/* Function for copying strings               */
    char sys_command[CMLEN];	/* System command which produces file names   */

    int  status;		/* Return status for the wait command         */

    long i        = 0;		/* Loop control                               */
    long nonfatal = 1;		/* Nonfatal error flag                        */
    long zero     = 0;		/* constant value			      */

#ifdef _IBMR2
    short c;			/* char returned by fget */
#else
#ifdef sgi
    short c;			/* char returned by fget */
#else
#ifdef DGUX
    short c;			/* char returned by fget */
#else
    char c;			/* char returned by fget */
#endif
#endif
#endif


    /* Ensure that maxfiles is at least one and initialize variables
    ----------------------------------------------------------------*/
    if (maxfiles < 1)
	{
	c_errmsg("Number of requested files must be greater than zero",
	    "c_getfiles-num", &nonfatal);
	return(E_FAIL);
	}
    *nfiles = 0;
    strcpy(filename, "");


    /* Create a file with the list of desired file names.  If the file
       specification uses an asterisk and no files are found, the file
       name will contain the given specification.  Therefore, if an
       asterisk is present then no files were found.
    ------------------------------------------------------------------*/
    if (first_call)
	{	
	if (file_ptr != NULL)
	    {
	    pclose(file_ptr);
	    file_ptr = NULL;
	    wait(&status);
	    }
	sprintf(sys_command, "echo %s", filespec);
	file_ptr = popen(sys_command, "r");
	if (file_ptr == NULL)
	    {
	    c_errmsg("Error opening temporary file",
		"c_getfiles-open", &nonfatal);
	    return(E_FAIL);
	    }
	}
    else if (!first_call && (file_ptr == NULL))
	{
	/* If all files have already been returned then exit successfully
	-----------------------------------------------------------------*/
	*nfiles = 0;
	return(E_SUCC);
	}


    /* Get the desired number of file names from the temporary file
    ---------------------------------------------------------------*/
    do  {
	c = fgetc(file_ptr);

	if (c == '*')
	    {
	    *nfiles = 0;
	    pclose(file_ptr);
	    file_ptr = NULL;
	    wait(&status);
	    return(E_SUCC);
	    }
	else if ((c != ' ') && (c != '\n'))
	    filename[i++] = c;
	else if ((c == ' ') || (c == '\n') || ((c == EOF) && (i > 0)))
	    {
	    filename[i] = '\0';
	    if (ck_lock)  /* Return the file only if it is not locked */
		{
		c_lsmknm(filename,"lock",lockfile);
		if (c_lsstat(lockfile,&zero) != E_SUCC)
		    strcpy(file_names[(*nfiles)++], filename);
		}
	    else  /* Return the file whether it is locked or not */
		strcpy(file_names[(*nfiles)++], filename);
	    strcpy(filename, "");
	    i = 0;
	    }
	} while ((c != EOF) && (*nfiles < maxfiles));


    /* If we only found one file then make sure this file really exists.  For
       example, if the user's file specification is /sg2/csb/user/myfile then
       the command "echo /sg2/csb/user/myfile" will do just that, echo
       "/sg2/csb/user/myfile", whether or not the file exists.  Thus we must
       check to see if the file does exist.
    -------------------------------------------------------------------------*/
    if (*nfiles == 1)
	{
	i = 0;
	if (c_lsstat(file_names[0], &i) != E_SUCC)
	    *nfiles = 0;
	}


    /* Close the temporary file if done and return successfully
    -----------------------------------------------------------*/
    if (*nfiles < maxfiles) 
	{
	pclose(file_ptr);
	file_ptr = NULL;
	wait(&status);
	}

    return (E_SUCC);
    }

#endif



/****************
*  VMS VERSION  *
****************/
#ifdef vms
#include "desc.h"

    static long num_files_reported = 0;	/* Number of files reported so far    */


    FUNCTION c_getfiles(filespec, file_names, nfiles, maxfiles, startf, ck_lock)

	char *filespec;		/* Specification of desired files to find     */
	char file_names[][CMLEN];/* Names of all the files found              */
	long  maxfiles;		/* Max number of files to report              */
	long  *nfiles;		/* Number of files found                      */
	long  startf;		/* Flag for first call or not (0,1)           */
	long ck_lock;		/* 1 = return list of unlocked files only,    */
				/* 0 = return list of all files found	      */
    {
    char filerslt[CMLEN];	/* Resultant file name from genf              */
    char lockfile[CMLEN];	/* Name of one 'locked' file		      */
    char *ptr;			/* Ptr to strings                             */
    char start;			/* Sent to genf-is this the first call? (0,1) */
    char *strcpy();		/* function for copying strings               */
    char temp[CMLEN];		/* Temp string to hold local modifications    */
    char wildstr[CMLEN];	/* Holds wildcard file specification          */

    long count;			/* Loop control                               */
    long i;			/* Loop control                               */
    long status;		/* Function return status                     */
    long dir_found;		/* Flag indicating if genf found a directory  */
    long nonfatal = 1;		/* Nonfatal error flag                        */
    long zero = 0;		/* constant value			      */

    short int name_size;	/* Size of directory name returned by genf    */

    struct DESC wild;		/* Wildcard file spec passed to genf          */
    struct DESC file;		/* Filename returned by genf                  */

    FILE *tmpfp, *tmpfile();	/* File ptrs                                  */


    /******************************************************************
    * This routine builds a list of file names from a "wildcard" file *
    * specification.  The genf routine (see worrmsio.c) is called to  *
    * actually generate the filenames (VMS).                          *
    ******************************************************************/

    /* Ensure that maxfiles is at least one
    ---------------------------------------*/
    if (maxfiles < 1)
	return(E_FAIL);


    /* Create a temporary file
    --------------------------*/
    tmpfp = tmpfile();
    if (tmpfp == NULL)
	{
	c_errmsg("Error opening temporary file","c_getfiles-open", &nonfatal);
	return(E_FAIL);
	}


    /* Build a descriptor pointing to the file specification
    --------------------------------------------------------*/
    wild.addr   = filespec;
    wild.length = strlen(wild.addr);


    /* Build a descriptor pointing to the first entry in
       the filelist array to receive the first file name.
    -----------------------------------------------------*/
    file.addr   = filerslt;
    file.length = 100;


    /* Call genf (see worrmsio.c) to get filenames,
       one at a time, from the wildcard string.
    -----------------------------------------------*/
    start     = TRUE;	
    *nfiles   = 0;
    count     = 0;
    dir_found = FALSE;
    status    = genf(&wild,&file,&name_size,&start,&dir_found);
    while ((status == E_SUCC) && dir_found)
	{
	filerslt[name_size] = '\0';

	/* lose the version number
	--------------------------*/
	strcpy(temp,filerslt);
	ptr = strchr(temp,';');
	*ptr = '\0';
	fprintf(tmpfp, "%s\n", temp);
	(*nfiles)++;
	count++;
	start = FALSE;  /* indicates this is not the first call to genf */
	status = genf(&wild, &file, &name_size, &start, &dir_found);
	}

    /* Reset to the beginning of the file for reading
    -------------------------------------------------*/
    fseek(tmpfp,0,0);


    /* Skip all of the files that have already been reported
    --------------------------------------------------------*/
    if (startf)
	num_files_reported = 0;
    else
	{
	for (count = 0; count < num_files_reported; count++)
	    {
	    status = fgets(temp, CMLEN, tmpfp);
	    if (temp[0] == '\0')
		break;
	    }
	}

    /* Find all file names up to maxfiles
    -------------------------------------*/
    for (*nfiles = 0; *nfiles < maxfiles;    )
	{
	status = fgets(temp, CMLEN, tmpfp);
	if (temp[0] == '\0')
	    break;
	i = strlen(temp);
	temp[i-1] = '\0';
	if (ck_lock)  /* Return the file only if it is not a 'locked' file */
	    {
	    c_lsmknm(temp,"lock",lockfile);
	    if (c_lsstat(lockfile,&zero) != E_SUCC)
		strcpy(file_names[(*nfiles)++], temp);
	    }
	else  /* Return the file whether it is 'locked' or not */
	    strcpy(file_names[(*nfiles)++], temp);
	}
    fclose(tmpfp);
    num_files_reported += *nfiles;
    return (E_SUCC);
    }
#endif
 
