/******************************************************************************
FUNCTION: 	PROCESS_LEAD	

PURPOSE:	Reads an image's leader file using the previously found
		record size, extracts the WRS path and row nunbers, and checks
		them against those supplied by the user.  If a match is 
		found, the current directory is assumed to be the location
		of the desired image.

ALGORITHM REFERENCES:	none
******************************************************************************/

#include <stdio.h>
#include "las.h"

FUNCTION process_lead(filename, recsize, path, row, date, foundpath, spot)

char filename[CMLEN];	/* Name of leader file to open			*/
long recsize;		/* Size of leader file records			*/
long path;		/* WRS path number				*/
long row;		/* WRS row number				*/
char date[9];		/* Acquisition date				*/
char foundpath[CMLEN];	/* Path to image on CD if it is found 		*/
long spot;		/* Flag for SPOT data				*/

{
FILE *fp;		/* Pointer to leader file			*/
char *in_buf;		/* Buffer for data read from leader file	*/
char tempbuf[4];	/* Temporary buffer				*/
char founddate[9];	/* Full extracted date				*/

long fatal = -1;	/* Flag for fatal error				*/
long nonfatal = 1;	/* Flag for nonfatal error			*/
long status;		/* Number of bytes read from leader file	*/
long fpath;		/* Path number extracted from the leader file	*/
long frow;		/* Row number extracted from the leader file	*/

/* Allocate memory for input buffer
-----------------------------------*/
in_buf = (unsigned char *) malloc((recsize * 2) * (sizeof(unsigned char)));
if (in_buf == NULL)
    c_lasmsg(TRUE, "ltwgcd-alloc", "Error allocating memory");

/* Open leader file
-------------------*/
fp = fopen(filename, "r");
if (fp == NULL)
    c_lasmsg(TRUE, "ltwgcd-diskread", "Error reading file %s", filename);

/* Read two records from the leader file 
----------------------------------------*/
status = fread(in_buf, 1, (2 * recsize), fp);
if (status != (2 * recsize))
    {
    fclose(fp);
    c_lasmsg(TRUE, "ltwgcd-diskread", "Error reading file %s", filename);
    }

/* Extract path and row numbers
-------------------------------*/
if (spot)

    /* SPOT data uses J and K numbers, which are references to WRS numbers.
       These numbers are located in a different place in the leader file
       than the WRS numbers in Landsat data. */

    {
    strncpy(tempbuf, &in_buf[20 + recsize], 3);
    tempbuf[3] = '\0';
    sscanf(tempbuf, "%d", &fpath);
    strncpy(tempbuf, &in_buf[23 + recsize], 3);
    tempbuf[3] = '\0';
    sscanf(tempbuf, "%d", &frow);
    }
else

    /* Extract Landsat path and row numbers from their "standard" place */

    {
    strncpy(tempbuf, &in_buf[165 + recsize], 3);
    tempbuf[3] = '\0';
    sscanf(tempbuf, "%d", &fpath);
    strncpy(tempbuf, &in_buf[168 + recsize], 3);
    tempbuf[3] = '\0';              
    sscanf(tempbuf, "%d", &frow);
    }

/* Extract date and check for equivalance
-----------------------------------------*/
if (spot)
    {
    strncpy(&founddate[6], &in_buf[40 + recsize], 3);
    strncpy(&founddate[0], &in_buf[42 + recsize], 3);
    strncpy(&founddate[3], &in_buf[44 + recsize], 3);
    }
else
    {
    strncpy(&founddate[6], &in_buf[118 + recsize], 3);
    strncpy(&founddate[0], &in_buf[120 + recsize], 3);
    strncpy(&founddate[3], &in_buf[122 + recsize], 3);
    }

founddate[2] = '/';
founddate[5] = '/';
founddate[8] = '\0';

/* Since DATE is nullable and performing a search using it as a qualifier
   is optional, its equivalence isn't checked until after a path/row
   match is found. */

if ((row == frow) && (path == fpath))
    {
    foundpath[0] = 'Y';	/* FLAG that proper image was found */
    if ((strlen(date) != 0) && (strcmp(date, founddate) != 0))

	/* Image has correct path/row numbers, but user gave an acqusition
	   date that does not match. */

	foundpath[0] = '\0';
    }

/* Close leader file, free buffer, and exit
-------------------------------------------*/
fclose(fp);
free(in_buf);
}
