/**********************************************************************
NAME: get_v2_fields    
 
PURPOSE:  This program is called by NDFCOPY.PDF for version 2 of NDF
          products to create the fields needed for the NDFCOPY process
          that were in version 1 of NDF, but are no longer present in 
          version 2 of the NDF format.
 
 
PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     01/00   T. Beckmann       Original
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE
 
PROJECT: LAS

Notes:
    This program creates the following fields that are not present in 
    NDF version 2 by reconstructing them from other fields:

    PRODUCT_CODE
    IMAGE_ORIENTATION
    RAD_ENHANCEMENT
    NBR_SCENES
    SCENE_ID

    The fields are obtained in the following manners:

    PRODUCT_CODE

        The product code can be reconstructed by utilizing a number of 
        different fields.  The product code format is "N###" where the 'N' 
        simply stands for NLAPS.

        #1 is the processing level (2=systematic, 3=precision, 4=terrain). The 
        processing level is contained in the NDF header PROCESSING_LEVEL where 
        8=systematic, 9=precision, 10=terrain.

        #2 is the instrument type (0=TM, 2=MSS, 4=ETM, 6=SPOT). The 
        NDF header SATELLITE_INSTRUMENT field can be used to obtain this value 

        #3 is the single/multi-scene product indication (0=single scene, 
        1=multi-scene).  The NDF header PRODUCT_SIZE field can be used to 
        obtain this value (MULTI=multi-scene, all others are single scene)
    
    IMAGE_ORIENTATION

        A field will be added to the work order file from NLAPS.  The field is
        "Product Orient.:" in the work order file.  The options are "Map North"
        for MAP and "Satellite" for SAT.

    RAD_ENHANCEMENT

        The work order file can be searched for which algorithms were run on 
        the product.  The possibilities are as follows:

            work order      database load file
            ----------      ------------------
            Despike         DSPIK
            Deband          DBAND
            Memory Effect   not used apparently
            None of above   NONE

    NBR_SCENES

        The NDF header PRODUCT_SIZE field can be used to determine single scene
        or multi-scene product (anything but MULTI is a single scene).  If the
        product is multi-scene, the "Start Row no.:" and "End Row no.:" fields
        in the work order file can be subtracted to determine the number of 
        scenes (round fractional scenes to the next higher integer).

    SCENE_ID
    
        The Landsat scene id can be constructed by combining several fields.
        The scene id format is:
            SPPPRRR00YYJJJI0
        where S = satellite #
              P = path
              R = row
              00 = offset (always 00)
              Y = year of acquisition
              J = Julian day of acquisition
              I = Instrument/sensor (1=TM, 5=ETM+, 9=MSS)
              0 = filler

        The fields obtained from the following areas:

            Satellite:  NDF Header SATELLITE field
            Path/Row: NDF Header WRS field which is formatted as PPP/RRR
            Year and Julian Day: NDF Header Acquisition Date field
    

INPUTS: (all strings)
        HEADER_FILE
        WOR_FILE

OUTPUTS: (all strings)
        PRODUCT_CODE
        IMAGE_ORIENT
        RAD_ENHANCE
        NBR_SCENES
        SCENE_ID

Read the following from the NDF header:
    PROCESSING_LEVEL            for PRODUCT_CODE
    SATELLITE_INSTRUMENT        for PRODUCT_CODE
    PRODUCT_SIZE                for PRODUCT_CODE and NBR_SCENES
    SATELLITE                   for SCENE_ID
    WRS                         for SCENE_ID
    ACQUISITION_DATE/TIME       for SCENE_ID


Read the following from the work order file:
    Product Orient.:            for IMAGE_ORIENTATION
    Start Row no.:              for NBR_SCENES
    End Row no.:                for NBR_SCENES

Search for the following in the work order file following 
"EXECUTION INFORMATION":
    Despike                     for RAD_ENHANCEMENT
    Deband                      for RAD_ENHANCEMENT


************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "las.h"
#include "util.h"
#include "read_ndf_parameters.h"
#include "get_file_size.h"


static int getpar
(
    struct PARBLK *vblk,        /* I: pointer to the TAE parameter block */
    char *header_filename,      /* O: pointer to header file name */
    char *work_order_filename,  /* O: pointer to work order file name */
    char *order_number          /* O: order number */
)
{
    struct VARIABLE *var;  /* ptr to a variable in the vblk */

    /* get the header file name */
    var = p_find(vblk, "HEADER_FILE");
    strcpy(header_filename,SVAL(*var,0));

    /* get the work order file name */
    var = p_find(vblk, "WOR_FILE");
    strcpy(work_order_filename,SVAL(*var,0));

    /* get the work order file name */
    var = p_find(vblk, "ORDID");
    strcpy(order_number,SVAL(*var,0));

    return (E_SUCC);
}

static char *read_file
(
    const char *filename   /* I: file name to read into memory */
)
{
    long filesize;          /* size of file to read */
    char *buffer = NULL;    /* buffer to read file into */
    FILE *file;             /* file pointer */
    
    /* determine the number of bytes in the file */
    if (get_file_size(filename, &filesize) != 0)
    {
        c_lasmsg(FALSE,"getfield-stat","Unable to determine size of %s",
                 filename);
        return NULL;
    }

    /* allocate memory to hold the entire file */
    buffer = (char *)malloc(filesize + 1);
    if (buffer == NULL)
    {
        c_lasmsg(FALSE,"getfield-alloc","Error allocating memory");
        return NULL;
    }

    /* read the file into memory */
    file = fopen(filename,"rt");
    if (file == NULL)
    {
        c_lasmsg(FALSE,"getfield-file","Error opening file %s", filename);
        free (buffer);
        return NULL;
    }

    if (fread(buffer, sizeof(char), filesize, file) != 
                filesize)
    {
        c_lasmsg(FALSE,"getfield-file","Error reading file %s", filename);
        free (buffer);
        return NULL;
    }
    buffer[filesize] = '\0';

    fclose(file);

    /* return the pointer to the file */
    return (buffer);
}

static int get_header_parameters
(
    char *filename,     /* I: header file name */
    char *product_code, /* O: NLAPS product code */
    int  *multi_scene_flag, /* O: 1=multiple scenes, 0=single scene product */
    char *scene_id      /* O: Landsat database scene id */
)
{
    static int proc_level;
    static char instrument[CMLEN];
    static char product_size[CMLEN];
    static char satellite[CMLEN];
    static char wrs[CMLEN];
    char *ptr;
    static char acq_date[CMLEN];
    long year;
    long month;
    long day;
    long julian_day;
    int param_count;
    int satellite_number;
    int path;
    int row;
    int instrument_number;
    NDF_PARAMETER_DEF_TYPE params[] = 
    {
        {"PROCESSING_LEVEL", INT_PARAM, &proc_level, sizeof(int),1,NULL},
        {"SATELLITE_INSTRUMENT", STRING_PARAM, instrument, sizeof(instrument),
                1, NULL},
        {"PRODUCT_SIZE", STRING_PARAM, product_size, sizeof(product_size),
                1, NULL},
        {"SATELLITE", STRING_PARAM, satellite, sizeof(satellite), 1, NULL},
        {"WRS", STRING_PARAM, wrs, sizeof(wrs), 1, NULL},
        {"ACQUISITION_DATE/TIME", STRING_PARAM,acq_date, sizeof(acq_date), 1, 
                NULL}
    };

    /* read the parameters from the NDF header */
    param_count = sizeof(params)/sizeof(NDF_PARAMETER_DEF_TYPE);
    if (read_ndf_parameters(filename, params, param_count) != E_SUCC)
    {
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
    
    /* process the NDF parameters read into the forms needed 
    --------------------------------------------------------*/
   
    /* build the product code */
    memset (product_code,0, 5);
    product_code[0]='N';
    switch (proc_level)
    {
        case 9:  /* precision product */
            product_code[1] = '3';
            break;
        case 10: /* terrain corrected product */
            product_code[1] = '4';
            break;
        case 8:  default: /* systematic and all others consider systematic */
            product_code[1] = '2';
            break;
    }
    
    if (strncmp(instrument,"MSS",3) == 0)
    {
        product_code[2] = '2';
        instrument_number = 9;
    }
    else if (strncmp(instrument,"TM",2) == 0)
    {
        product_code[2] = '0';
        instrument_number = 1;
    }
    else if (strncmp(instrument,"ETM",3) == 0)
    {
        product_code[2] = '4';
        instrument_number = 5;
    }
    else 
    {
        /* SPOT not supported */
        c_lasmsg(FALSE,"getfield-spot","SPOT products not supported");
        return (E_FAIL);
    }
    if (strncmp(product_size,"MULTI",5) == 0)
    {
        product_code[3] = '1';
        *multi_scene_flag = 1;
    }
    else
    {
        product_code[3] = '0';
        *multi_scene_flag = 0;
    }

    /* get the satellite number from the last digit of the satellite string */
    satellite_number = atoi(&satellite[strlen(satellite) - 1]);
    
    /* get the path and row */
    path = atoi(wrs);
    ptr = strchr(wrs,'/');
    if (ptr)
    {
        ptr++;
        row = atoi(ptr);
    }
    else
    {
        /* bad row, so default it to zero */
        row = 0;
    }

    /* get the year and julian day from the acquisition date */
    if (strlen(acq_date) > 10)
    {
        /* extract the date (format YYYY-MM-DD) */
        month=0;
        day=0;
        year = atoi(acq_date);
        ptr = strchr(acq_date,'-');
        if (ptr)
        {   
            ptr++;
            month = atoi(ptr);
            ptr = strchr(ptr,'-');
            if (ptr)
            {
                ptr++;
                day = atoi(ptr);
            }
        }

        /* convert to a julian day */
        c_grejul(&year,&month,&day,&julian_day);
    }
    else
    {
        /* bad acq_date, so default it to zeros */
        year = 0;
        julian_day = 0;
    }
    
    /* build the Landsat database scene id */
    sprintf(scene_id,"%1d%03d%03d00%02ld%03ld%1d0",satellite_number,path,row,
                     year % 100, julian_day, instrument_number);
    
    return (E_SUCC);
}

static char *find_tag
(
    const char *buffer,         /* I: buffer to search */
    const char *tag             /* I: tag to find */
)
{
    char *ptr;
    
    ptr = strstr(buffer,tag);
    if (ptr)
    {
        /* skip the tag to get to the data */
        ptr = strchr(ptr,':');
        if (ptr)
        {
            ptr++;
            
            /* skip empty spaces */
            while (*ptr == ' ')
                ptr++;
        }
    }

    return (ptr);
}

static int get_work_order_parameters
(
    const char *filename,       /* I: work order file name */
    int  multi_scene_flag,      /* I: 1=multiple scenes, 0=single scene */
    char *order_number,         /* I: order number */
    char *image_orientation,    /* O: image orientation */
    char *rad_enhancement,      /* O: radiometric enhancement */
    int  *number_scenes         /* O: number of scenes in product */
)
{
    char *file_data;
    char *ptr;
    
    /* read the parameters from the work order file
    -----------------------------------------------*/
    file_data = read_file(filename);
    if (file_data == NULL)
    {
        return (E_FAIL);
    }

    /* get the image orientation by looking for the "Product Orient.:" tag 
       in the file */
    strcpy(image_orientation, "");
    ptr = find_tag(file_data,"Product Orient.:");
    if (ptr)
    {
        /* get the orientation (leaving it blank if no recognized value 
           is found) */
        if (strncmp(ptr,"Map",3) == 0)
        {
            strcpy(image_orientation,"MAP");
        }
        else if (strncmp(ptr,"Satellite",9) == 0)
        {
            strcpy(image_orientation,"SAT");
        }
    }
    
    /* issue a warning if the image orientation wasn't found */
    if (image_orientation[0] == '\0')
    {
        c_lasmsg(FALSE,"getfield-warn","Could not determine image orientation");
    }
   
    /* if the product is a multiscene product, find the number of scenes */
    *number_scenes = 1;
    if (multi_scene_flag)
    {
        float start_row;
        float end_row;
        char ppffile[1024];
        char row_first[1][CMLEN];
        char row_last[1][CMLEN];
        double dbl_val;
        long lng_val;
        
        /* get the number of scenes by finding the difference between the
           starting and ending row and forcing to the next higher integer */
        ptr = find_tag(file_data,"Start Row no.:");
        if (ptr)
        {
            start_row = atof(ptr);

            ptr = find_tag(file_data,"End Row no.:");
            if (ptr)
            {
                end_row = atof(ptr);
                    
                *number_scenes = (int)ceil(fabs(end_row - start_row));

                /* limit number of scenes to something reasonable in case
                   something went wrong in parsing */
                if (*number_scenes > 9)
                    *number_scenes = 1;
    
                /* put the ROW_FIRST and ROW_LAST out to the PPF file just like
                   the version 1 path in nlapckcopy.  It would seem to make 
                   more sense to just pass them through TAE, but maybe they're 
                   needed in the PPF file for some unknown reason? */
                sprintf(row_first[0],"%f",start_row);
                sprintf(row_last[0],"%f",end_row);
                sprintf(ppffile,"%s/%s/%s",getenv("PDSASSM"),order_number,
                        order_number);
                c_putparm(ppffile,"ROW_FIRST",row_first,"S",&dbl_val,1,
                          &lng_val);
                c_putparm(ppffile,"ROW_LAST",row_last,"S",&dbl_val,1,&lng_val);
            }
        }
    }
    
    /* look for Despike and Deband in the work order file after the string
       "EXECUTION INFORMATION" to determine the rad_enhancement */
    strcpy(rad_enhancement,"NONE");
    ptr = strstr(file_data,"EXECUTION INFORMATION");
    if (ptr)
    {
        if (strstr(ptr,"Despike"))
        {
            strcpy(rad_enhancement,"DSPIK");
        }
        else if (strstr(ptr,"Deband"))
        {
            strcpy(rad_enhancement,"DBAND");
        }
    }
    
    /* free the buffer */
    free(file_data);
                  
    return (E_SUCC);
}

int main(void)
{
    long std_out = 0;   /* standard out */
    struct PARBLK vblk; /* TAE parameter block */
    char header_filename[CMLEN];     /* header file name */
    char work_order_filename[CMLEN]; /* work order file name */
    char order_number[CMLEN]; /* order number */
    char product_code[5];   /* NLAPS product code */
    int  multi_scene_flag;  /* 1=multiple scenes, 0=single scene product */
    char scene_id[17];      /* Landsat database scene id */
    char image_orientation[4]; /* image orientation (MAP or SAT) */
    char rad_enhancement[6];   /* radiometric enhancement (NONE,DSPIK,DBAND) */
    int  number_scenes;        /* number of scenes in product */
    char scenes_string[4];     /* string for returning the number of scenes */
    char *string_ptr[1];       /* pointer for returning strings to tae */

    /* Initialize tae block
    -------------------------*/
    c_inlas (&vblk, &std_out);

    /* read the input parameters 
    ----------------------------*/
    if (getpar(&vblk, header_filename, work_order_filename, order_number)
            != E_SUCC)
    {
        c_lasmsg(FALSE,"getfield-parm", "Error reading input parameters");
        c_lasmsg(TRUE,"getfield-fatal", "Fatal error encountered");
    }
    
    /* read the parameters from the NDF header file 
    -----------------------------------------------*/
    if (get_header_parameters(header_filename, product_code, &multi_scene_flag,
                              scene_id) != E_SUCC)
    {
        c_lasmsg(TRUE,"getfield-fatal", "Fatal error encountered");
    }

    /* read the parameters from the work order file
    -----------------------------------------------*/
    if (get_work_order_parameters(work_order_filename,multi_scene_flag,
                                  order_number, image_orientation,
                                  rad_enhancement, &number_scenes) != E_SUCC)
    {
        c_lasmsg(TRUE,"getfield-fatal", "Fatal error encountered");
    }

    /* return the parameters to the calling routine */
    string_ptr[0] = product_code;
    if (q_string(&vblk,"PRODUCT_CODE",1,string_ptr,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FALSE,"getfield-tae","Error passing PRODUCT_CODE to TAE");
        c_lasmsg(TRUE,"getfield-fatal","Fatal error encountered");
    }  
  
    string_ptr[0] = image_orientation;
    if (q_string(&vblk,"IMAGE_ORIENT",1,string_ptr,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FALSE,"getfield-tae","Error passing IMAGE_ORIENT to TAE");
        c_lasmsg(TRUE,"getfield-fatal","Fatal error encountered");
    }  

    string_ptr[0] = rad_enhancement;
    if (q_string(&vblk,"RAD_ENHANCE",1,string_ptr,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FALSE,"getfield-tae","Error passing RAD_ENHANCE to TAE");
        c_lasmsg(TRUE,"getfield-fatal","Fatal error encountered");
    }  

    sprintf(scenes_string,"%02d",number_scenes);
    string_ptr[0] = scenes_string;
    if (q_string(&vblk,"NBR_SCENES",1,string_ptr,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FALSE,"getfield-tae","Error passing NBR_SCENES to TAE");
        c_lasmsg(TRUE,"getfield-fatal","Fatal error encountered");
    }

    string_ptr[0] = scene_id;
    if (q_string(&vblk,"SCENE_ID",1,string_ptr,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FALSE,"getfield-tae","Error passing SCENE_ID to TAE");
        c_lasmsg(TRUE,"getfield-fatal","Fatal error encountered");
    }  

    /* return the output block */
    if (q_out(&vblk) != SUCCESS)
    {
        c_lasmsg(FALSE,"getfield-tae","Error passing variables to TAE");
        c_lasmsg(TRUE,"getfield-fatal","Fatal error encountered");
    }
    
    return (E_SUCC);
}
