/***********************************************************************

NAME: NLDCCEOS_READ 
 
PURPOSE:  This program is called from NLDCCEOS_READ.                     
          1)  Copies header files and image file to output file array.
          2)  Copies files to output file names.
          3)  Calculates record size, bands, and total bands.
          4)  Returns the output file array (writefile), record size
              array, scene id, and tape description.

RECEIVING: This program receives four input parameters.

           1) DIRECTORY   : The name of the directory where all the files 
				are located.
           2) TAPE_COUNT  : The count of input tapes.
           3) INFILE      : The base for the input filenames.
           4) TAPE_FILE   : The files per input tape.

OUTPUT: This program returns four output parameter.

           1) WRITE_FILE  : Array of sorted file names to be returned to
				the calling program.
           2) REC_SIZE    : Record Length of image.
           3) SCENE_ID    : Scene Identifier.
           4) TAPE_DESC   : Additional tape description.

ALGORITHM REFERENCES:
            "Landsat Technical Working Group (LTWG) format for
            Thematic Mapper (TM) Computer Compatible Tapes (CCTs)"
            Effective: January 1, 1990


PROGRAM HISTORY
Version   Date    Programmer        Code  Reason
-------   ----    -------------     ----- ------
  1.0     05/98   M.Lundblad          C   Initial Development
 
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
Must be run under UNIX and TAE
 
PROJECT: LAS
 
************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
#include <sys/types.h>
#include <sys/stat.h> 
#include "string.h" 
#include "las.h" 
#include "util.h"

int getpar
(
    struct PARBLK *vblk,
    char directory[120],
    long *tape_count,
    long tape_file[100],
    char infile[120]
);


int main
(void)

{
  
  struct PARBLK vblk;        /* Variable Block                            */

  /* Input Variables */
  char directory[CMLEN];     /* Working Directory for files are located.  */
  char infile[CMLEN];        /* The input filenames base.                 */
  long tape_count = 0;       /* Number of Input Tapes read                */
  long tape_file[295];       /* Files per input tape.                     */

  /* Output Variables */
  char write_file[295][CMLEN]; /* Hold file names for writing to tapes.     */
  char scene_id[17];         /* Scene Identification                      */
  char tape_desc[100];       /* Archive Tape Description                  */
  long rec_size[295];        /* Record Length of the image file           */

  char path[4];              /* Scene Path                                */
  char row[4];               /* Scene Row                         */
  char satno[2];             /* Satellite Number                          */
  char mon[5],day[5],year[5];/* temp work variables                       */
  char filename[CMLEN];      /* filename buffer                           */
  char srcfile[CMLEN];       /* Source filename string                    */
  char tgtfile[CMLEN];       /* Target filename string                    */
  char tmpstr[64];           /* Temporary string buffer                   */
  char *tempfile[500];       /* Temporary file buffer for output          */
  char *tempptr[1];          /* Temporary file pointer for output         */
  char cmdline[CMLEN];       /* Hold the name of commands                 */
  char buffer[21600]; /* Buffer for CEOS header                  */

  long mn,dy,yr,jdate = 0;   /* temp work values                          */
  long lead_size = 0;        /* Leader and Traildeer file sizes           */
  long img_size = 0;         /* Image file size                           */
  long head_size = 0;        /* Header file size                          */
  long std_out = 0;          /* Standard output                           */
  long status = -1;          /* Holds completion statuses                 */
  long ii, jj, kk = 0;       /* For loop indices                          */
  long imgcnt = 0;           /* Image counter                             */
 
  FILE *fptr;                /* CEOS header file pointer                  */

  /* Retrieve parameter block
  ----------------------------*/
  c_inlas(&vblk,&std_out);
  if ((getpar(&vblk,directory,&tape_count,tape_file,infile)) != E_SUCC)
     c_lasmsg(TRUE,"nldc321-getpar","Error retrieving input paramters");

  /* Open Volume Header file to read
  -----------------------------------*/
  sprintf(filename,"%s%s_1.tape1",directory,infile);
  fptr = fopen(filename,"r");
  if (!fptr)
     c_lasmsg(TRUE,"nldc321-openhdr","Error opening header for ceos format");

  /* Read all bytes of Volume Header file
  ----------------------------------------*/
  status = (long) fread(buffer,1,8280,fptr);
  if (!status)
     c_lasmsg(TRUE,"nldc321-readhdr","Error reading from header for ceos format");
  fclose(fptr);

  /* Determine record size for header files 
  ------------------------------------------*/
  head_size = 360;

  /* Determine record size for leader and 
     trailer files 
  ----------------------------------------*/
  strncpy(tmpstr,&buffer[480],4);
  tmpstr[4] = '\0';
  lead_size = atol(tmpstr);

  /* Determine record size for image files 
  -----------------------------------------*/
  strncpy(tmpstr,&buffer[840],4);
  tmpstr[4] = '\0';
  img_size = atol(tmpstr);

  /* Rename Volume Header files
  ------------------------------*/
  sprintf(tgtfile,"%s%s.HD",directory,infile);
  strcpy(write_file[kk],tgtfile);
  rec_size[kk++] = head_size;
  for (ii=1; ii <= tape_count; ii++)
     {
     sprintf(tgtfile,"%s%shdr%d.nhdr",directory,infile,ii);
     sprintf(srcfile,"%s%s_1.tape%d",directory,infile,ii);
     sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
     system(cmdline);
     }

  /* Rename All image files and associated files
  -----------------------------------------------*/
  for (ii=1; ii <= tape_count; ii++)
     for (jj=2; jj <= (tape_file[ii-1] - 2); jj++)
        {
        imgcnt++;
        /* Image Leader file */
        sprintf(tgtfile,"%s%s.h%d",directory,infile,imgcnt);
        sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,jj,ii);
        sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
        system(cmdline);
        strcpy(write_file[kk],tgtfile);
        rec_size[kk++] = lead_size;
        jj++;

        /* Image file */
        sprintf(tgtfile,"%s%s.i%d",directory,infile,imgcnt);
        sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,jj,ii);
        sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
        system(cmdline);
        strcpy(write_file[kk],tgtfile);
        rec_size[kk++] = img_size;
        jj++;

        /* Image Trailer file */
        sprintf(tgtfile,"%s%s.t%d",directory,infile,imgcnt);
        sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,jj,ii);
        sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
        system(cmdline);
        strcpy(write_file[kk],tgtfile);
        rec_size[kk++] = lead_size;
        }

  /* Rename Volume Trailer files
  --------------------------------*/
  sprintf(tgtfile,"%s%s.SP",directory,infile);
  strcpy(write_file[kk],tgtfile);
  rec_size[kk++] = head_size;
  sprintf(tgtfile,"%s%s.NVD",directory,infile);
  strcpy(write_file[kk],tgtfile);
  rec_size[kk++] = head_size;
  for (ii=1; ii <= tape_count; ii++)
     {
     /* Volume Supplement File */
     sprintf(tgtfile,"%s%s.sp%d",directory,infile,ii);
     sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,(tape_file[ii-1] - 1),ii);
     sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
     system(cmdline);
     /* Null Volume Directory File */
     sprintf(tgtfile,"%s%s.nvd%d",directory,infile,ii);
     sprintf(srcfile,"%s%s_%d.tape%d",directory,infile,tape_file[ii-1],ii);
     sprintf(cmdline,"mv %s %s",srcfile,tgtfile);
     system(cmdline);
     }

  /* Read leader file to extract information
  -------------------------------------------*/
  sprintf(filename,"%s%s.h%d",directory,infile,1);
  fptr = fopen(filename,"r");
  if (!fptr)
     c_lasmsg(TRUE,"nldc321-openlead","Error opening image leader for ceos format");
  status = (long) fread(buffer,1,21600,fptr);
  if (!status)
     c_lasmsg(TRUE,"nldc321-readlead","Error reading from image leader for ceos format");
  fclose(fptr);

  /* Extract Satellite Number
  ---------------------------*/
  strncpy(satno,&buffer[4356],1);

  /* Extract Path and Row
  ------------------------*/
  strncpy(path,&buffer[4485],3);
  strncpy(row,&buffer[4488],3);

  /* Extract Acquistition Date
  -----------------------------*/
  strncpy(year,&buffer[4438],2);
  strncpy(mon,&buffer[4440],2);
  strncpy(day,&buffer[4442],2);
  yr = atol(year);
  mn = atol(mon);
  dy = atol(day);
  c_grejul(&yr,&mn,&dy,&jdate);
  if (jdate < 10)
     sprintf(tmpstr,"00%d",jdate);
  else if (jdate < 100)
     sprintf(tmpstr,"0%d",jdate);
  else
     sprintf(tmpstr,"%d",jdate);

  /* Build Scene Id
  ------------------*/
  sprintf(scene_id,"%s%s%s00%02d%s10",satno,path,row,yr,tmpstr);

  /* Build the Tape Description
  ------------------------------*/
  sprintf(tape_desc,"%s %s %s %02d/%02d/%02d",scene_id,path,row,mn,dy,(yr % 100));


/* Return the output Parameters
---------------------------------*/

  /* Output Filename array
  -------------------------*/
  for(ii=0; ii < kk; ++ii)
     tempfile[ii] = &write_file[ii][0];
  status = q_string(&vblk,"WRITE_FILE",kk,tempfile,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing write_file to variable block (#%d)",status);

  /* Output Scene Id
  -------------------*/
  tempptr[0] = &scene_id[0];
  status = q_string(&vblk,"SCENE_ID",1,tempptr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing scene_id to variable block (#%d)",status);

  /* Output tape description for label
  -------------------------------------*/
  tempptr[0] = &tape_desc[0];
  status = q_string(&vblk,"TAPE_DESC",1,tempptr,P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing tape_desc to variable block (#%d)",status);

  /* Output the record size array
  --------------------------------*/
  status = q_intg(&vblk,"REC_SIZE",kk,&rec_size[0],P_UPDATE);
  if (status != SUCCESS)
     c_lasmsg(TRUE,"nldc321-q_out","Error writing rec_size to variable block (#%d)",status);

  q_out(&vblk);

  exit(E_SUCC);

  return EXIT_SUCCESS;
}
