/******************************************************************************

SUBROUTINE:	mod

PURPOSE:
	Subroutine mod will modify a request in the Custom Request File.
	The structure of the file is a header record of 80 bytes followed by
	records containing the requests. The header record contains the head 
	of the two linked lists.

	The record field "delflg" forms a singly linked list containing all the
	records that do not have valid requests (i.e. the requests have been
	deleted).  The record fields "bptr" and "fptr" forms a doubly linked
	list among the requests with the same priority.

ARGUMENTS:
  Name		Type	  I/O	Description
  ----		----	  ---	-----------
  *blkptr	PARBLK	   I    pointer to the V-block


COMMON BLOCK REFERENCE:  (defined in include file "struct.cin")
  Name		Type 		Description
  ----		----		-----------
  HDRLEN	int		integer defining length of header record
  RECLEN	int		integer defining length of request record
  rec		recfmt		record format of one request record
  hdr		hdrfmt		record format of the header record
  
PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   2.0           3/97       KGacke      qlp/qlpmgr unix command line option
   1.0          10/91       K. Gacke    initial development -- copied from
                                        color FIRE

PROJECT:        Quick Look Printer

Algorithm:
	Basic algorithm of the mod subroutine:  The request to be modified is
	identified by the parameter SEQNUM.  The record containing this request
	is read, and a dynamic parameter block is built.  The default values of
	the dynamic parameters will be set to the existing values of the 
	request being modified.  The user will modify the parameters to their 
	desired values and type RUN to continue processing.  The new parameters
	are read to form the updated request.  The updated request is written
	back into the same record of the Custom Request File.  If the priority
	of the request is modified, the request must be deleted from its old
	priority list and added to the new priority list.

******************************************************************************/

				
#include <stdio.h>
#include "las.h"
#include "ntqlp.h"
#include "qlp.h"
#include "qlpsupport.h"
#include "local_protos.h"

static long fatal = -1;

int modreq
(
    int argc,                   /* argument count */
    char *argv[],               /* argument vector */
    long tae,                   /* TAE flag, TRUE=>valid param block */
    struct PARBLK *blkptr	/* TAE param block */
)
{
extern struct recfmt rec;
extern struct hdrfmt hdr;
struct VARIABLE *seqnum;
struct QLP_PRINTER printer[MAX_QLP_PRINTER];
long 		pcnt;
int		fd,seq,prio,nrec;
long		crflg;
long 		suflg;
char		msg[80];
char 		user[64];

getuser(user);
/*
* retrieve sequence number from V-block or Unix command line argument 
*/
if (tae)
    {
    seqnum = p_find(blkptr,"RECNUM");
    seq = IVAL(*seqnum,0);
    }
else
    {
    if (argc <= 1)
	{
	qlp_usage(PMOD);
	return(E_FAIL);
	}
    if (sscanf(argv[1],"%d",&seq) != 1)
	{
	sprintf(msg,"Error retrieving sequence number to modify, arg=%s",
		argv[1]);
	c_errmsg(msg," ",&fatal);
	}
    argc--;
    argv++;
    }
(void) crfinit("QLP-MOD");
(void) crfpr(printer,&pcnt,TRUE);
(void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec); /* open the CRF */
if (seq > nrec) 		   /* if sequence number > number of requests*/
   {				   /*   write error message & terminate proc */
   crfdone();
   crfcl(fd,TRUE); 
   sprintf(msg,"Sequence Number %d does not exist",seq);
   c_errmsg(msg," ",&fatal);
   }
(void) crfrd(fd,TRUE,seq,&rec);	    /* read the request to be modified */
(void) crfcl(fd,TRUE);		    /* close the CRF */
(void) crfdone();
suflg = qlp_chkuser(printer,pcnt,&rec);
if ((suflg != SU_OWN) && (strcmp(rec.stflg,QLP_RUN) == 0))
   {				   /*   write error message & terminate proc */
   sprintf(msg,"Sequence number %d is currently being processed by printer",
	seq);
   c_errmsg(msg," ",&fatal);
   }
if (rec.delflg != -1)	   	   /* if request is not valid then	     */
   {				   /*   write error message & terminate proc */
   sprintf(msg,"Sequence Number %d does not exist",seq);
   c_errmsg(msg," ",&fatal);
   }
if (suflg < SU_PRIV)
   {
   if (strcmp(rec.usercode,user) != 0)
      {
      sprintf(msg,"ERROR: User <%s> is not owner of custom request %d",
	   user,seq);
      c_errmsg(msg," ",&fatal);
      }
   }
prio = rec.prio;		  /* set prio equal to priority of request   */
/*
* Build the dynamic tutor session. dynam will return with the modified 
* request
*/
if (dynam(argc,argv,tae,printer,pcnt) != E_SUCC)
    return(E_FAIL);
rec.lad = setlad();		  /* update last access date */
(void) crfinit("QLP-MOD");
(void) crfop(1,TRUE,ABORT_IGNORE,&fd,&crflg,&nrec);
if (prio == rec.prio) 		  /* if priority not modified, write the rec */
   {
   (void) crfwr(fd,TRUE,rec.seqnum,&rec);
   }
else				        /* if priority was modified */
   {
   delreq(printer,pcnt,fd,nrec,&seq,1);	/*   delete request from the CRF */
   addreq(fd,crflg,nrec);	        /*   add request back to the CRF */
   }
(void) crfcl(fd,TRUE);
(void) crfdone();
return(E_SUCC);
}   /* END of MOD subroutine */
