/*******************************************************************************
FUNCTION:	READMSS

PURPOSE:	Read MSS archive images from a 3480 converted tape.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    10/92  J.Fenno          Development
  2.1    03/93  T.Baltzer        Include orbit stitched images and lower case
                                 scene id's
  2.2    07/93  J.Fenno          Stolen from READARCH
  2.3	 07/93	S.Nelson	 Stolen from READARCH_NEW and adapted for
				 converted MSS-X tapes.  

PROJECT:  LAS
*******************************************************************************/
#define FULL 1
#define BROWSE 2

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include "las.h"
#include "util.h"
#include "protos.h"


/* System return status
=======================*/
#define SYS_SUCC      0    /* UNIX successful exit status                     */


int main
(void)
{
struct PARBLK   vblock;    /* TAE parameter block                           */
struct VARIABLE *parm;     /* Pointer to TAE variable within vblock         */

long difflg;		 /* flag for checking scenes not found on tape	    */
long diff;		 /* number of scenes different than expected	    */
long fatal    = -1;      /* Fatal error code                                */
long i,j;		 /* Counter					    */
long len;		 /* Length of the scene id			    */
long nimages  =  0;      /* Number of archive images read to disk           */
long nscenes = 0;	 /* Number of scenes specified in scene_ids	    */
long pass = 1;		 /* Non fatal error code			    */
long sflag;		 /* Flag indicating specific scenes		    */
long size;		 /* Size of file				    */
long stdlun;             /* Standard output                                 */
long status;		 /* Return status				    */
int typflg[2];		 /* Type of image flag: standard=1, browse=2	    */

char drive[CMLEN];       /* Tape drive the tape is mounted on               */
char *images[22];	 /* Scenes actually copied to disk by ansiin        */
char outdir[CMLEN];      /* Disk location where the images should be placed */
char scene_ids[10][22];	 /* Scene IDs requested by the user                 */
char str[CMLEN];         /* Temporary string                                */
char string[CMLEN];	 /* String location for error log		    */
char sys_command[CMLEN]; /* Command sent to the operating system            */
char tapeid[CMLEN];      /* Tape ID                                         */

struct stat statp;	 /* file status structure			    */

/* Initialize LAS
-----------------*/
c_inlas(&vblock, &stdlun);

/* initialize output directory if none is entered
   ---------------------------------------------*/
strcpy(outdir,"");

/* Get the tape drive that the tape is mounted on
-------------------------------------------------*/
parm = p_find(&vblock, "DRIVE");
strcpy(drive, SVAL(*parm, 0));


/* Get the tape ID
------------------*/
parm = p_find(&vblock, "TAPEID");
strcpy(tapeid, SVAL(*parm, 0));


/* Get the scene ID of the images to be read.  Lower case the scene ID 
---------------------------------------------------------------------*/

parm = p_find(&vblock, "SCENEID");
nscenes = parm->v_count;

for (i = 0; i < nscenes; i++)
     {
     strcpy(scene_ids[i], SVAL(*parm, i));
     len = (long) strlen(scene_ids[i]);
     c_up2low(scene_ids[i], &len);
     }

if (strcmp(scene_ids[0], "*") == 0)
    sflag = FALSE;

/*  Check for valid scene ids (16 characters)--delete invalid scene ids
   -------------------------------------------------------------------*/
else
    {
    sflag = TRUE;
    i = 0;
    while (i < nscenes)
       {
       len = (long) strlen(scene_ids[i]);
       if (len != 16)
	   {
	   sprintf(string,"%s%s%s","Scene ID ",scene_ids[i],
						" must be 16 characters");
	   c_errmsg(string,"readmss-sidlen",&pass);
	   for (j = i; j < nscenes; j++)
		strcpy(scene_ids[j],scene_ids[j+1]);
	   nscenes--;
	   }
       else
	   i++;
       }
    }


/* Get the directory where the image should be placed and then chdir to that
   directory.  After calling c_cmhost the last character for UNIX directories
   should be a '/'.
-----------------------------------------------------------------------------*/
parm = p_find(&vblock, "OUTDIR");
if (parm->v_count != 0)
    {
    strcpy(str, SVAL(*parm, 0));
    c_cmhost(str, outdir);
    if (outdir[strlen(outdir) - 1] != HOST_DIR_END)
	    strcat(outdir, "/\0");
    if (chdir(outdir))
	    c_errmsg("Error moving to specified directory","readmss-chdir",&fatal);
    }

/* Set up the ansiin command that will read the scene(s) from tape.  UNIX
   systems must have single quotes around the scene IDs to protect any
   asterisks from shell expansion.
-------------------------------------------------------------------------*/
sprintf(sys_command, "ansiin -Tvl %s %s", drive, tapeid);
if (sflag)
   for (i = 0; i < nscenes; i++)
	 {
	 /* Add a specification for the header file(s)
	 ---------------------------------------------*/
	 strcat(sys_command, " '");
	 scene_ids[i][16] = '*';
	 scene_ids[i][17] = '\0';
	 strcat(sys_command, scene_ids[i]);
	 strcat(sys_command, "'");
	 }


/* Make the system call to ansiin
---------------------------------*/
if (system(sys_command) != SYS_SUCC)
    c_errmsg("Error reading image(s) from tape", "readmss-read", &fatal);

/* Rename all of the files read from tape and enter into TAE variable
--------------------------------------------------------------------*/
rename_files(images, &nimages, typflg);

/* Report scenes not found in ansiin
   -------------------------------*/
difflg = FALSE;
i = 0;
diff = nscenes - nimages;
if ((diff > 0) && (sflag == TRUE))
   {
   while((diff !=0) && (i < nscenes))
      {
      j = 0;
      while(j < nimages)
	 {
	 scene_ids[i][16] = '\0';
	 if (strcmp(images[j],scene_ids[i]))
	    j++;
	 else
	    {
	    j = nimages;
	    difflg = TRUE;
	    }
	 }
      if (difflg != 1)
	 {
	 sprintf(string,"%s%s%s","Scene ID ",scene_ids[i]," not found on tape");
	 c_errmsg(string,"readmss-sidfnd",&pass);
	 diff--;
	 }
      i++;
      difflg = FALSE;
      }
   }

/* TEST the size of the first file--this test could be extended to all files
   if desired.  At this time, only one file is checked for size.
   ------------------------------------------------------------------------*/
if (nimages != 0)
   {
   for (i = 0; i < 16; i++)
      {
      scene_ids[0][i] = images[0][i];
      }
   scene_ids[0][16] = '\0';
   strcat(scene_ids[0],".img1");
   stat(scene_ids[0], &statp);
   size = (long) statp.st_size;
   if ((size != 8087040) && (typflg[0] == FULL))
      {
      for (i = 0; i < nimages; i++)
              free(images[i]);
      c_errmsg("Incorrect file size","readmss-size",&fatal);
      }
   else if ((size != 168480)&&(typflg[0] == BROWSE))
      {
      for (i = 0; i < nimages; i++)
              free(images[i]);
      c_errmsg("Incorrect file size","readmss-size",&fatal);
      }
   }
/* Return TAE variable
  -------------------*/
status = q_intg(&vblock,"TYPFLG",1, typflg,P_UPDATE);
if (status != SUCCESS)
    {
    for (i = 0; i < nimages; i++)
        free(images[i]);
    strcpy(str, "Error sending parameters to TAE");
    c_errmsg(str, "readmss-parm", &fatal);
    }

/* Return the LAS names of each archive image read to disk
----------------------------------------------------------*/
q_string(&vblock, "SCENES", nimages, images, P_UPDATE);
if (q_out(&vblock) != SUCCESS)
    {
    strcpy(str, "Error sending parameters to TAE");
    for (i = 0; i < nimages; i++)
        free(images[i]);
    c_errmsg(str, "readmss-parm", &fatal);
    }
for (i = 0; i < nimages; i++)
    free(images[i]);

z_exit(1, "READMSS_NEW completed successfully");

return EXIT_SUCCESS;

}
