/******************************************************************************
FUNCTION:	PROC_DISK


PURPOSE:	Allocates a DITTO device to a disk file.

ALGORITHM 
  Device allocated to disk file
  Retrieve Device type parameter
  Retrieve the input file name parameter
  Retrieve the block size parameter
  Retrieve the record size parameter
  Retrieve the carrage control format parameter
  Allocate the input DITTO device
     Check to see if the input device has already been allocated
	 if so output an error message.
     Retrieve file attributes
     Determine the type of records
     Determine type of carriage control
     The TAE global DITTO variables are updated to represent the allocation
     of the disk file.
  Else allocate the output DITTO device
     Check to see if the output device has already been allocated
	 if so output an error message.
     Retrieve file attributes
     Determine the type of records
     Determine type of carriage control
     The TAE global DITTO variables are updated to represent the allocation
     of the disk file.
  Write new TAE parameter block

ALGORITHM REFERENCES
  DITTO device is allocated by updating several TAE global variables within 
  the TAE parameter block.  The variables include:
	device name:  specifies disk or tape
	host name  :  host file name or physical tape drive
	record size:  record size of fixed length records, or max record size
		      of variable length records
	block size :  block size of how records are stored
        cur_rec    :  current record being processed
	cur_file   :  current file being processed
	record type:  fixed or variable length records
	carriage   :  type of carriage control
	density    :  tape density in bytes per inch
	skip	   :  number of records to back space on open (tapes only).
		      This is used because if a tape is opened for write access
		      it writes an EOF mark when it is closed.
  If the input DITTO device is to be allocated, the disk file must exist.
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "util.h"           /* c_clean prototype                 */
#include "ditto.h"			/* ditto include file                */

void proc_disk(void)		
{
extern struct PARBLK vblock;		/* TAE parameter block               */
struct VARIABLE *val;			/* pointer to TAE variable           */
struct DISK_FD disk_file;		/* disk file structure               */

long  status;				/* function return status code       */
long  zero = 0;				/* value of zero                     */
int   izero = 0;            /* value of zero as an int */
int   temp;                 /* temp int variable */
char *cc[1];				/* type of carriage control          */
char *device[1];			/* disk device is to be allocated    */
char *fname[1];				/* host file name of the disk file   */
char *rectyp[1];			/* fixed or variable record type     */
char which[10];				/* input or output DITTO device      */
char out[1][CMLEN];			/* Output files created		     */

/*
  Device allocated to disk file
*/
device[0] = "DISK";

/*
  Retrieve Device type parameter
*/
val = p_find(&vblock,"devtype");
strcpy(which,SVAL(*val,0));

/*
  Retrieve the input file name parameter
*/
val = p_find(&vblock,"infile");
strcpy(disk_file.fname,SVAL(*val,0));

/*
  Retrieve the block size parameter
*/
val = p_find(&vblock,"blksiz");
if (val->v_count > 0)
   disk_file.blksiz = IVAL(*val,0);
else 
   disk_file.blksiz = NOVAL;

/*
  Retrieve the record size parameter
*/
val = p_find(&vblock,"recsiz");
if (val->v_count > 0)
   disk_file.recsiz = IVAL(*val,0);
else
   disk_file.recsiz = NOVAL;

/*
  Retrieve the record type parameter
*/
val = p_find(&vblock,"rectype");
if (val->v_count == 0)
   disk_file.rectyp = NOVAL;
else if (strcmp(SVAL(*val,0),"FIX") == 0)
   disk_file.rectyp = FIX;
else
   disk_file.rectyp = VAR;

/*
  Retrieve the carrage control format parameter
*/
val = p_find(&vblock,"ccform");
if (val->v_count == 0)
   disk_file.cc = NOVAL;
else if (strcmp(SVAL(*val,0),"CARRIAGE") == 0)
   disk_file.cc = CARRIAGE;
else if (strcmp(SVAL(*val,0),"PRINT") == 0)
   disk_file.cc = PRINT;
else if (strcmp(SVAL(*val,0),"FORTRAN") == 0)
   disk_file.cc = FORTRAN;
else
   disk_file.cc = NONE;

/*
  Allocate the input DITTO device or the output DITTO device
*/
if (strcmp(which,"INPUT") == 0)
   {
   /*
     Check to see if the input device has already been allocated (if so output
     an error message).
   */
   disk_file.cur_file = 1;
   val = p_find(&vblock,"idevice");
   if (val->v_count > 0)
      {
      ditto_error(which,DITTO_DOUBLE,ERROR_WARN,NO_UPDATE_FD);
      c_clean(&vblock,"ditto alloc",out,&zero);
      }

   /*
     Retrieve file attributes
   */
   status = dstat(&disk_file,0);
   if (status == E_FAIL)
      {
      ditto_error(which,DITTO_DSTAT,ERROR_WARN,NO_UPDATE_FD);
      c_clean(&vblock,"ditto alloc",out,&zero);
      }

   /*
     Determine the type of records
   */
   switch (disk_file.rectyp)
      {
      case FIX:		rectyp[0] = "FIX";	break;

      case VAR:		rectyp[0] = "VAR";	break;
      }

   /*
     Determine type of carriage control
   */
   switch (disk_file.cc)
      {
      case NONE:	cc[0] = "NONE";		break;

      case CARRIAGE:	cc[0] = "CARRIAGE";	break;

      case PRINT:	cc[0] = "PRINT";	break;

      case FORTRAN:	cc[0] = "FORTRAN";	break;
      }

   /*
     The TAE global DITTO variables are updated to represent the allocation
     of the disk file.
   */
   fname[0] = disk_file.fname;
   status = q_string(&vblock,"idevice",1,device,P_UPDATE);
   status = q_string(&vblock,"ihost",1,fname,P_UPDATE);

   if (disk_file.rectyp != NOVAL)
      status = q_string(&vblock,"irectyp",1,rectyp,P_UPDATE);

   if (disk_file.cc != NOVAL)
      status = q_string(&vblock,"icc",1,cc,P_UPDATE);

   temp = (int)disk_file.recsiz;
   status = q_intg(&vblock,"irecsiz",1,&temp,P_UPDATE);
   temp = (int)disk_file.blksiz;
   status = q_intg(&vblock,"iblksiz",1,&temp,P_UPDATE);
   status = q_intg(&vblock,"irec",1,&izero,P_UPDATE);
   temp = (int)disk_file.cur_file;
   status = q_intg(&vblock,"ifile",1,&temp,P_UPDATE);
   }

else			/* Allocate the OUTPUT DITTO device		    */
   {
   /*
     Check to see if the output device has already been allocated (if so output
     an error message).
   */
   disk_file.cur_file = NOVAL;
   val = p_find(&vblock,"odevice");
   if (val->v_count > 0)
      {
      ditto_error(which,DITTO_DOUBLE,ERROR_WARN,NO_UPDATE_FD);
      c_clean(&vblock,"ditto alloc",out,&zero);
      }

   /*
     Retrieve file attributes
   */
   status = dstat(&disk_file,0);
   if (status != E_SUCC)
	disk_file.cur_file = NOVAL;

   /*
     Determine the type of records
   */
   switch (disk_file.rectyp)
      {
      case FIX:		rectyp[0] = "FIX";	break;

      case VAR:		rectyp[0] = "VAR";	break;
      }

   /*
     Determine the type of carriage control
   */
   switch (disk_file.cc)
      {
      case NONE:	cc[0] = "NONE";		break;

      case CARRIAGE:	cc[0] = "CARRIAGE";	break;

      case PRINT:	cc[0] = "PRINT";	break;

      case FORTRAN:	cc[0] = "FORTRAN";	break;
      }

   /*
     The TAE global DITTO variables are updated to represent the allocation of 
     the disk file.
   */
   fname[0] = disk_file.fname;
   status = q_string(&vblock,"odevice",1,device,P_UPDATE);
   status = q_string(&vblock,"ohost",1,fname,P_UPDATE);

   if (disk_file.rectyp != NOVAL)
      status = q_string(&vblock,"orectyp",1,rectyp,P_UPDATE);

   if (disk_file.cc != NOVAL)
      status = q_string(&vblock,"occ",1,cc,P_UPDATE);

   if (disk_file.recsiz != NOVAL)
   {
      temp = (int)disk_file.recsiz;
      status = q_intg(&vblock,"orecsiz",1,&temp,P_UPDATE);
   }

   if (disk_file.blksiz != NOVAL)
   {
      temp = (int)disk_file.blksiz;
      status = q_intg(&vblock,"oblksiz",1,&temp,P_UPDATE);
   }

   status = q_intg(&vblock,"orec",1,&izero,P_UPDATE);
   temp = (int)disk_file.cur_file;
   status = q_intg(&vblock,"ofile",1,&temp,P_UPDATE);
   }

/*
  Write new TAE parameter block
*/
status = q_out(&vblock);

}
