/******************************************************************************

FUNCTION:	PROC_TAPE

PURPOSE:	Allocates a DITTO device to a magnetic tape.

ALGORITHM 
  DITTO device allocated to tape
  Retrieve the device type parameter
  Retrieve the tape comment parameter
  Retrieve the tape density parameter
  Retrieve the block size parameter
  Retrieve the record type parameter
  Retrieve the carriage control parameter
  Allocate the input DITTO device
     Check to see if the input device has already been allocated
	if so output an error message
     The tape is mounted onto a drive
     The tape is then closed, but not dismounted so that the other DITTO
     commands can reopen the tape.
     The TAE global DITTO variables are updated to represent the allocation
     of the magnetic tape.
  Else allocate the output DITTO device
     Check to see if the output device has already been allocated
	if so output an error message
     The tape is mounted onto a drive
     The tape is then closed, but not dismounted so that the other DITTO
     commands can reopen the tape.
     The TAE global DITTO variables are updated to represent the allocation
     of the magnetic tape.
  Write new TAE parameter block

ALGORITHM REFERENCES	none
  The tape is mounted onto a drive when it is allocated.  A DITTO device is 
  allocated by updating several TAE global variables within the TAE parameter 
  block.  The variables include:
	device name:  specifies disk or tape
	host name  :  host file name or physical tape drive
	record size:  record size of fixed length records, or max record size
		         of variable length records
	block size :  block size of how records are stored
	cur_rec    :  current record being processed
	cur_file   :  current file being processed
	record type:  fixed or variable length records
	carriage   :  type of carriage control
	density    :  tape density in bytes per inch
	skip	   :  number of records to back space on open (tapes only).
		      This is used because if a tape is opened for write access
		      it writes an EOF mark when it is closed.
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "util.h"           /* c_clean prototype */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"			/* ditto include file                */

extern long Tapelen;

void proc_tape(void)
{
extern struct PARBLK vblock;		/* TAE parameter block               */
struct VARIABLE *val;			/* pointer to a TAE variable         */
struct TAPEDESC *tp;            /* tape drive description */

long  access = TREAD;			/* type of I/O access to the tape    */
long  mode = HANDLE;			/* tape routines output error msg.   */
long  rewopt = REWIND;			/* tape rewind option                */
long  eofopt = 1;			/* end of file option for tape close */
int   one = 1;				/* value of one                      */
long  skip = 0;				/* number of tape files to skip      */
long  zero = 0;				/* value of zero                     */
int   izero = 0;            /* integer value of zero */
int   temp;                 /* temp int variable */

int   blksiz;				/* block size in bytes               */
long  carriage;				/* type of carriage control          */
long  maxbyte;				/* maximum number of bytes           */
int   recsiz;				/* record size in bytes              */
long  status;				/* function return status code       */
long  type;				/* record type                       */
long  unit;				/* unit number to the tape           */
long  length;				/* tape length			     */

char out[1][CMLEN];			/* Output files created		     */
char comment[81];			/* Buffer space for mount message    */
char mount_msg[CMLEN];			/* Save the mount message	     */
char *comsav[1];			/* Pointer to comment space	     */
char *drive[1];				/* Tape drive assigned		     */
char *cc[1];				/* type of carriage control          */
char *device[1];			/* DITTO device allocated to tape    */
char *rectyp[1];			/* Record type                       */
char *which;				/* input/output DITTO device         */
char density[5];			/* tape density in BPI               */
char tapeid[CMLEN];			/* tape id			     */
char *tpdens[1];
char *tpid[1];
char drive_user[20];			/* drive to be allocated	     */
long drive_user_flag = FALSE;		/* flag to indicate if user selected 
					   drive			     */

/*
  DITTO device allocated to tape
*/
device[0] = "TAPE";

/*
  Retrieve the device type parameter
*/
val = p_find(&vblock,"devtype");
which = SVAL(*val,0);
/*
if (strcmp(which,"OUTPUT") == 0) 
   access = TWRITE;
*/

/* Get the drive if user selected
-------------------------------*/
val = p_find(&vblock,"DRIVE");
if (val->v_count > 0)
   {
   strcpy(drive_user,SVAL(*val,0));
   drive_user_flag = TRUE;
   }
else	
  drive_user_flag = FALSE;

/*
  Retrieve the tape comment parameter
*/
val = p_find(&vblock,"comment");
strcpy(comment,SVAL(*val,0));

/*
  Retrieve the tape density parameter
*/
val = p_find(&vblock,"tapedens");
strcpy(density,SVAL(*val,0));

/*
  Retrieve the block size parameter
*/
val = p_find(&vblock,"blksiz");
if (val->v_count > 0)
   blksiz = IVAL(*val,0);
else 
   blksiz = NOVAL;

/*
  Retrieve the record size parameter
*/
val = p_find(&vblock,"recsiz");
if (val->v_count > 0)
   recsiz = IVAL(*val,0);
else
   recsiz = NOVAL;

/*
  Retrieve the record type parameter
*/
val = p_find(&vblock,"rectype");
if (val->v_count == 0)
   type = NOVAL;
else if (strcmp(SVAL(*val,0),"FIX") == 0)
   {
   type = FIX;
   rectyp[0] = "FIX";
   }
else
   {
   type = VAR;
   rectyp[0] = "VAR";
   }

/*
  Retrieve the carriage control parameter
*/
val = p_find(&vblock,"ccform");
if (val->v_count == 0)
   carriage = NOVAL;
else if (strcmp(SVAL(*val,0),"CARRIAGE") == 0)
   {
   carriage = CARRIAGE;
   cc[0] = "CARRIAGE";
   }
else if (strcmp(SVAL(*val,0),"PRINT") == 0)
   {
   carriage = PRINT;
   cc[0] = "PRINT";
   }
else if (strcmp(SVAL(*val,0),"FORTRAN") == 0)
   {
   carriage = FORTRAN;
   cc[0] = "FORTRAN";
   }
else
   {
   carriage = NONE;
   cc[0] = "NONE";
   }

/*
  Allocate the input DITTO device or the output DITTO device
*/
if (strcmp(which,"INPUT") == 0)
   {
   /*
     Check to see if the input device has already been allocated (if so output
     an error message)
   */
   val = p_find(&vblock,"idevice");
   if (val->v_count > 0)
      {
      ditto_error(which,DITTO_DOUBLE,ERROR_WARN,NO_UPDATE_FD);
      c_clean(&vblock,"ditto alloc",out,&zero);
      }

   /*
     Save the input comment
   */
   strcpy(mount_msg,comment);

   /*
     The tape is mounted onto a drive
   */

  if (drive_user_flag == TRUE)
    {
    status = c_tpreop (&unit, &skip, &mode, &rewopt, density, &access, 
                       drive_user);
    if (status != TAPE_SUCC)
      {
      ditto_error(which,DITTO_TPREOP,ERROR_WARN,NO_UPDATE_FD);
      c_clean(&vblock,"ditto alloc",out,&zero);
      }
    }
  else
    {
    status = c_tpopen(&unit,&skip,&mode,&rewopt,density,&access,mount_msg,
		      tapeid);
    if (status != TAPE_SUCC)
      {
      ditto_error(which,DITTO_TPOPEN,ERROR_WARN,NO_UPDATE_FD);
      c_clean(&vblock,"ditto alloc",out,&zero);
      }
    }

   /* If no record or block size was specified, set it to be the
      maximum of the specific drive. */
   tp = (struct TAPEDESC *) unit;
   maxbyte = tp->maxbyte;
   if (recsiz == NOVAL)
      recsiz = maxbyte;
   if (blksiz == NOVAL)
      blksiz = maxbyte;

   /* Save the device assigned
   -------------------------*/
   if (drive_user_flag == TRUE)
      drive[0] = drive_user;
   else
      drive[0] = mount_msg;
   comsav[0] = comment;
   tpid[0] = tapeid;

   /*
     The tape is then closed, but not dismounted so that the other DITTO
     commands can reopen the tape.
   */
   status = c_tpclos(&unit,&mode,&rewopt,&eofopt);
   if (status != TAPE_SUCC)
      ditto_error(which,DITTO_TPCLOS,ERROR_WARN,NO_UPDATE_FD);

   /*
     The TAE global DITTO variables are updated to represent the allocation
     of the magnetic tape.
   */
   status = q_string(&vblock,"idevice",1,device,P_UPDATE);
   status = q_string(&vblock,"ihost",1,drive,P_UPDATE);
   status = q_string(&vblock,"itapeid",1,tpid,P_UPDATE);
   tpdens[0] = density;
   status = q_string(&vblock,"idensity",1,tpdens,P_UPDATE);
   temp = (int)skip;
   status = q_intg(&vblock,"iskip",1,&temp,P_UPDATE);
   status = q_intg(&vblock,"irec",1,&izero,P_UPDATE);
   status = q_intg(&vblock,"ifile",1,&one,P_UPDATE);

   if (type != NOVAL)
      status = q_string(&vblock,"irectyp",1,rectyp,P_UPDATE);

   if (carriage != NOVAL)
      status = q_string(&vblock,"icc",1,cc,P_UPDATE);

   if (recsiz != NOVAL)
      status = q_intg(&vblock,"irecsiz",1,&recsiz,P_UPDATE);

   if (blksiz != NOVAL)
      status = q_intg(&vblock,"iblksiz",1,&blksiz,P_UPDATE);

   status = q_string(&vblock,"icomment",1,comsav,P_UPDATE);
   }

else			/* Allocate the OUTPUT DITTO device		*/
   {
   /*
     Check to see if the output device has already been allocated (if so
     output an error message).
   */
   val = p_find(&vblock,"odevice");
   if (val->v_count > 0)
      {
      ditto_error(which,DITTO_DOUBLE,ERROR_WARN,NO_UPDATE_FD);
      c_clean(&vblock,"ditto alloc",out,&zero);
      }

   /*
     Save the output comment
   */
   strcpy(mount_msg,comment);

   /*
    The tape is mounted onto a drive.
   */
  if (drive_user_flag == TRUE)
    {
    printf("%s\n",drive_user);
    status = c_tpreop (&unit, &skip, &mode, &rewopt, density, &access, 
                       drive_user);
    if (status != TAPE_SUCC)
      {
      ditto_error(which,DITTO_TPREOP,ERROR_WARN,NO_UPDATE_FD);
      c_clean(&vblock,"ditto alloc",out,&zero);
      }
    }
  else
    {
    status = c_tpopen(&unit,&skip,&mode,&rewopt,density,&access,mount_msg,
                      tapeid);
    if (status != TAPE_SUCC)
       {
       ditto_error(which,DITTO_TPOPEN,ERROR_WARN,NO_UPDATE_FD);
       c_clean(&vblock,"ditto alloc",out,&zero);
       }
    }

   /*
     The tape is then closed, but not dismounted so that the other DITTO
     commands can reopen the tape.
   */
   status = c_tpclos(&unit,&mode,&rewopt,&eofopt);
   if (status != TAPE_SUCC)
      ditto_error(which,DITTO_TPCLOS,ERROR_WARN,NO_UPDATE_FD);

   /*
     Save the device assigned
   */
   if (drive_user_flag == TRUE)
      drive[0] = drive_user;
   else
      drive[0] = mount_msg;
   comsav[0] = comment;
   tpid[0] = tapeid;

   /*
     The TAE global DITTO variables are updated to represent the allocation
     of the magnetic tape.
   */
   status = q_string(&vblock,"odevice",1,device,P_UPDATE);
   status = q_string(&vblock,"ohost",1,drive,P_UPDATE);
   status = q_string(&vblock,"otapeid",1,tpid,P_UPDATE);
   tpdens[0] = density;
   status = q_string(&vblock,"odensity",1,tpdens,P_UPDATE);
   temp = (int)skip;
   status = q_intg(&vblock,"oskip",1,&temp,P_UPDATE);
   status = q_intg(&vblock,"orec",1,&izero,P_UPDATE);
   status = q_intg(&vblock,"ofile",1,&one,P_UPDATE);

   if (type != NOVAL)
      status = q_string(&vblock,"orectyp",1,rectyp,P_UPDATE);

   if (carriage != NOVAL)
      status = q_string(&vblock,"occ",1,cc,P_UPDATE);

   if (recsiz != NOVAL)
      status = q_intg(&vblock,"orecsiz",1,&recsiz,P_UPDATE);

   if (blksiz != NOVAL)
      status = q_intg(&vblock,"oblksiz",1,&blksiz,P_UPDATE);

   status = q_string(&vblock,"ocomment",1,comsav,P_UPDATE);
   }

/*
  Write new TAE parameter block
*/
status = q_out(&vblock);

/* Set tape length if specified
-----------------------------*/
val = p_find(&vblock,"LENGTH");
if (val->v_count > 0)
   {
   length = IVAL(*val,0);
   Tapelen = length; 
   }

}
