/****************************************************************************
NAME				c_getrgd

PURPOSE		Read a geometric mapping grid file from disk

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
D. Steinwand	     	10/88	Original development
B. Ailts		01/89	Create a new version for radiometric grids

PROJECT		LAS

ALGORITHM
   Check for existance of radiometric mapping grid file
   Open the radiometric mapping grid file for read access using label services.
   Read the first radiometric mapping grid record
   Read the second radiometric mapping grid record
   Close the radiometric mapping grid file
******************************************************************************/
#include <string.h>
#include "worgen.h"
#include "geompak.h"
#include "labserv.h"
#include "radpak.h"
#include "lams.h"


int c_getrgd
(
    char *host_name,		/* Host file name of the mapping grid file */
    struct RADGRID *grid		/* Geometric mapping grid */
)
{
long i;			/* Loop counter  */
long j = 0;		/* Loop counter  */
long one = 1;		/* constant of 1, used for non-fatal error status */
long mode = 0;		/* access mode for lsstat */
long access = 0;	/* access mode for lsopen, set to read access */
long fd;		/* file descriptor */
long rowcol;		/* grid->nrows * grid->ncols to reduce computations */
long clen;		/* number of bytes of character data per record */
long dlen;		/* number of bytes of data per record */
long action = 0;	/* action parameter for lsclos */
long *tmpbuf;		/* Temporary buffer for read */
long num_bytes;		/* Number of bytes in double portion of grid */
long num_recs;		/* Number of records of double */
long rec_inc;		/* Record length / sizeof(double) */
long rec_len;		/* Record length of double precision records */
long last_rec_len;	/* Record length of last double grid record */
float num_temp;		/* Temporary floating point value of num_recs */
double *tmpdbl;		/* Temporary buffer for read -- doubles */
double *buf_ptr;	/* Pointer to temporary buffer used during read */
char dummy[15];		/* Dummy argument for second ls read */
char key[16];
char dtype[4];

/* Check to see if the file aready exists.  If it does exist, an error message
   is printed.
 -----------------*/
if (c_lsstat(host_name, &mode) != E_SUCC)
   {
   c_errmsg("Radiometric mapping grid file does not exist on disk",
            "gridio-exist", &one);
   return(E_FAIL);
   }

/* Open the output geometric mapping grid file
 --------------------------------------------*/
if (c_lsopen(&fd, host_name, &access) != E_SUCC)
   {
   c_errmsg("Error opening radiometric mapping grid file for read access",
            "gridio-open", &one);
   return(E_FAIL);
   }

/* Read the first geometric mapping grid file record.  This read will fail,
   but will return buffer lengths
 --------------------------------*/
clen = 1;
dlen = 1;
strcpy(key,"RAD_GRID_INT");
strcpy(dtype,"I4");
if (c_lsread(&fd, key, &clen, &dlen, dummy, tmpbuf, dtype) != E_SMAL)
   {
   c_errmsg("Error reading from the radiometric mapping grid file", "gridio-read",
             &one);
   return(E_FAIL);
   }

/* Allocate buffer space for the integer portion of the geometric mapping grid
 ----------------------------------------------------------------------------*/
tmpbuf = (long *) malloc(dlen);
if (tmpbuf == NULL)
   {
   c_errmsg("Error allocating buffer space for radiometric mapping grid",
            "gridio-alloc",&one);
   return(E_FAIL);
   }

/* Reread the first geometric mapping grid file record
 ---------------------------------------------------*/
strcpy(key,"RAD_GRID_INT");
strcpy(dtype,"I4");
if (c_lsread(&fd, key, &clen, &dlen, dummy, tmpbuf, dtype) != E_SUCC)
   {
   c_errmsg("Error reading from the radiometric mapping grid file", "gridio-read",
             &one);
   return(E_FAIL);
   }

/* Unpack the first record into the grid stucture
  ------------------------------------------------*/
strncpy(grid->model, dummy, 15);

grid->fiterr_valid = tmpbuf[0];
grid->griderr_valid = tmpbuf[1];
grid->ncoeff = tmpbuf[2];
grid->nrows = tmpbuf[3];
grid->ncols = tmpbuf[4];
grid->lines = tmpbuf[5];
grid->samples = tmpbuf[6];
grid->out_lines = &(tmpbuf[7]);
grid->out_samps = &(tmpbuf[7 + grid->nrows]);

/* Calculate the sizes of the second grid record, allocate space, and read
 --------------------------------------------------------------------------*/
clen = 1;
rowcol = grid->nrows * grid->ncols;
rec_len = last_rec_len = DBL_RECSIZE;
rec_inc = DBL_RECSIZE / sizeof(double);
num_bytes = (11 + (grid->ncoeff * 2) + (2 * rowcol)) * sizeof(double);
num_temp = ((float) num_bytes) / DBL_RECSIZE;
num_recs = (long)num_temp;
if (num_temp != num_recs)
   {
   last_rec_len = num_bytes - (num_recs * DBL_RECSIZE);
   num_recs++;
   }

tmpdbl = (double *) malloc(num_bytes);
if (tmpdbl == NULL)
   {
   c_errmsg("Error allocating buffer space for radiometric mapping grid",
            "gridio-alloc",&one);
   return(E_FAIL);
   }

/* Read records from disk
  --------------------*/
for (buf_ptr = tmpdbl, i = 0; i < num_recs; i++, buf_ptr += rec_inc)
   {
   if ((i + 1) == num_recs) rec_len = last_rec_len;
   strcpy(key," ");
   strcpy(dtype,"R8");
   if (c_lsread(&fd, key, &clen, &rec_len, dummy, buf_ptr, dtype) != E_SUCC)
      {
      c_errmsg("Error reading from the radiometric mapping grid file", 
			"gridio-read", &one);
      return(E_FAIL);
      }
   }

/* Unpack read buffer into structure
  ---------------------------------*/ 
grid->coeffs = &(tmpdbl[j]);
for(i = 0, j += (grid->ncoeff * 2); i < 2; j++, i++)
   grid->max_grid_err[i] = tmpdbl[j];
for(i = 0; i < 2; j++, i++)
   grid->ave_grid_err[i] = tmpdbl[j];
grid->rms_grid_err = tmpdbl[j++];
grid->grid_tol = tmpdbl[j++];
for(i = 0; i < 2; j++, i++)
   grid->max_fit_err[i] = tmpdbl[j];
for(i = 0; i < 2; j++, i++)
   grid->ave_fit_err[i] = tmpdbl[j];
grid->rms_fit_err = tmpdbl[j];
j++;
grid->z_values = &(tmpdbl[j]);

/* Close the geometric mapping grid file
 --------------------------------------*/
if (c_lsclos(&fd, host_name, &action) != E_SUCC)
   {
   c_errmsg("Error closing the geometric mapping grid file", "gridio-close",
             &one);
   return(E_FAIL);
   }

return(E_SUCC);
}
