/****************************************************************************
NAME				c_putrgd

PURPOSE		Write a radiometric  mapping grid file to disk

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
D. Steinwand	     	10/88		Original development
B. Ailts		01/89		Create a new version to write a 
					radiometric grid

PROJECT		LAS

ALGORITHM
   Check for existance of the output file.  Abort if already there.
   Open (create) the radiometric  mapping grid file for write access using
    label services.
   Write the first radiometric  mapping grid record
   Write the second radiometric  mapping grid record
   Close the radiometric  mapping grid file
******************************************************************************/
#include <string.h>
#include "worgen.h"
#include "geompak.h"
#include "labserv.h"
#include "radpak.h"
#include "lams.h"


int c_putrgd
(
    char *host_name,		/* Host file name of the mapping grid file */
    struct RADGRID *grid	/* radiometric mapping grid                */
)
{
long one = 1;		/* constant of 1, used for non-fatal error status */
long mode = 0;		/* access mode for lsstat */
long access = 1;	/* access mode for lsopen, set to write access */
long fd;		/* file descriptor */
long rowcol;		/* grid->nrows * grid->ncols to reduce computations */
long clen;		/* number of bytes of character data per record */
long dlen;		/* number of bytes of data per record */
long action = 0;	/* action parameter for lsclos */
long *tmpbuf;		/* Temporary buffer for write */
long i,j;		/* Loop counters */
long num_bytes;		/* Number of bytes in double portion of grid */
long num_recs;		/* Number of label services records required */
long rec_inc;		/* Record length / sizeof (double) */
long rec_len;		/* Record length of double portion of grid */
long last_rec_len;	/* Length of last record, double part of grid */
float num_temp;		/* Temp var to hold floating point num_recs */
double *tmpdbl;		/* Temporary buffer for write--double */
double *buf_ptr;	/* Pointer to temporary buffer used during write */
char dummy[15];		/* Dummy argument for second ls write */


/* Check to see if the file aready exists.  If it does exist, an error message
   is printed.
 -----------------*/
if (c_lsstat(host_name, &mode) == E_SUCC)
   {
   c_errmsg("Radiometric mapping grid file already exists",
	    "gridio-exist", &one);
   return(E_FAIL);
   }

/* Open the output radiometric mapping grid file
 --------------------------------------------*/
if (c_lsopen(&fd, host_name, &access) != E_SUCC)
   {
   c_errmsg("Error opening radiometric mapping grid file for write access",
            "gridio-open", &one);
   return(E_FAIL);
   }


/* Calculate the size of the integer portion of the first radiometric mapping
   grid record 
 -------------*/
clen = 15;
dlen = (7 + grid->nrows + grid->ncols) * sizeof(long);

/* Allocate temporary buffer and transfer data to it
  --------------------------------------------------*/
tmpbuf = (long *) malloc(dlen);
if (tmpbuf == NULL)
   {
   c_errmsg("Error alocating temporary buffer space for write",
            "gridio-alloc", &one);
   return(E_FAIL);
   }

tmpbuf[0] = grid->fiterr_valid;
tmpbuf[1] = grid->griderr_valid;
tmpbuf[2] = grid->ncoeff;
tmpbuf[3] = grid->nrows;
tmpbuf[4] = grid->ncols;
tmpbuf[5] = grid->lines;
tmpbuf[6] = grid->samples;
for(i = 0, j = 7; i < grid->nrows; i++, j++)
   tmpbuf[j] = grid->out_lines[i];
for(i = 0; i < grid->ncols; i++, j++)
   tmpbuf[j] = grid->out_samps[i];

strncpy(dummy, grid->model, 15);

/* Write the first radiometric mapping grid file record
 ---------------------------------------------------*/
if (c_lswrit(&fd, "RAD_GRID_INT", &clen, &dlen, dummy, tmpbuf, "I4") != E_SUCC)
   {
   c_errmsg("Error writing to the radiometric mapping grid file", "gridio-write",
             &one);
   if (c_lsclos(&fd, host_name, &action) != E_SUCC)
      c_errmsg("Error closing the radiometric mapping grid file", 
               "gridio-close",&one);
   return(E_FAIL);
   }

/* Calculate the sizes of the second radiometric mapping grid record
 -----------------------------------------------------------------*/
clen = 0;
rowcol = grid->nrows * grid->ncols;
rec_len = last_rec_len = DBL_RECSIZE;
rec_inc = DBL_RECSIZE / sizeof(double);
num_bytes = (11 + (grid->ncoeff * 2) + (2 * rowcol)) * sizeof(double);
num_temp = ((float) num_bytes) / DBL_RECSIZE;
num_recs = (long)num_temp;
if (num_temp != num_recs)
   {
   last_rec_len = num_bytes - (num_recs * DBL_RECSIZE);
   num_recs++;
   }

/* Allocate temporary buffer and transfer data to it
  --------------------------------------------------*/
tmpdbl = (double *) malloc(num_bytes);
if (tmpdbl == NULL)
   {
   c_errmsg("Error allocating temporary buffer space for write",
            "gridio-alloc", &one);
   return(E_FAIL);
   }

j = 0;
for(i = 0; i < (grid->ncoeff * 2); i++, j++)
   tmpdbl[j] = grid->coeffs[i];
for(i = 0; i < 2; i++, j++)
   tmpdbl[j] = grid->max_grid_err[i];
for(i = 0; i < 2; i++, j++)
   tmpdbl[j] = grid->ave_grid_err[i];
tmpdbl[j++] = grid->rms_grid_err;
tmpdbl[j++] = grid->grid_tol;
for(i = 0; i < 2; i++, j++)
   tmpdbl[j] = grid->max_fit_err[i];
for(i = 0; i < 2; i++, j++)
   tmpdbl[j] = grid->ave_fit_err[i];
tmpdbl[j] = grid->rms_fit_err;
for(i = 0, j++; i < rowcol; i++, j++)
   tmpdbl[j] = grid->z_values[i];

/* Write records to disk
  --------------------*/
for (buf_ptr = tmpdbl, i = 0; i < num_recs; i++, buf_ptr += rec_inc)
   {
   if ((i + 1) == num_recs) rec_len = last_rec_len;
   if (c_lswrit(&fd, " ", &clen, &rec_len, dummy, buf_ptr, "R8") != E_SUCC)
      {
      c_errmsg("Error writing to the radiometric mapping grid file", 
                      "gridio-write", &one);
      if (c_lsclos(&fd, host_name, &action) != E_SUCC)
         c_errmsg("Error closing the radiometric mapping grid file", 
		  "gridio-close",&one);
      return(E_FAIL);
      }
   }

/* Close the radiometric mapping grid file and free temp buffer space
 -----------------------------------------------------------------*/
if (c_lsclos(&fd, host_name, &action) != E_SUCC)
   {
   c_errmsg("Error closing the radiometric mapping grid file", "gridio-close", &one);
   /*return(E_FAIL);*/
   return(E_FAIL);
   }

free(tmpbuf);
free(tmpdbl);

return(E_SUCC);
}
