/****************************************************************************
NAME:				c_grdred	

PURPOSE:  Discard rows and columns of the radiometric mapping grid that are
	  within the tolerance of delta z approximation across neighboring
	  rows and columns.  Note:  Output grid cells are rectangular and
	  are all identical in size.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0      	 5/84  	M. NISHIHAMA       SAR        N/A
  5.0		 1/89	D. Steinwand	   CSB      LAS 5.0 C conversion
  5.1		 1/89	B. Ailts	   CSB      Create a new version to 							    handle radiometric grids
  5.2		 5/90	D. Etrheim         CSB      Add type to function calls 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

*****************************************************************************/
#include <math.h>
#include "worgen.h"
#include "geompak.h"
#include "radpak.h"
#include "lams.h"


/****************************************************************************
NAME:				check_rcol

PURPOSE:    Check tolerance along group of columns being tested for elimination

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0      	 5/84  	M. NISHIHAMA       SAR        N/A
  5.0		 1/89	D. Steinwand	   CSB      LAS 5.0 C conversion

ALGORITHM DESCRIPTION:
	Set max deviation to zero
	Do for each row
	   Calculate starting and ending indicies into the Geometric Mapping
			Grid
	   Calculate linear coefficients
	   Find the maximum deviation along a group of columns
	   If the max deviation > tol, return (fail)
	Return (success)
*****************************************************************************/
long check_rcol
(
    long start,           /* Starting column, zero relative                 */
    long end,             /* Ending column, zero relative                   */
    double tol,           /* Grid reduction tolerance value                 */
    long size,            /* Size of "new" output space per "new" grid cell */
    struct RADGRID *grid  /* Geometric mapping grid                         */
)
{
double maxdev;		/* Maximum deviation */
long row;		/* Current row number */
long start_index;	/* Index into input grid; starting point */
long end_index;		/* Index into input grid; ending point */
double a0,a1;		/* Linear approximation coefficients for "new" cell */
long cnt;		/* Loop counter */
long k;			/* Offset into mapping grid for each group element */
long z;			/* Value to evaluate "new" grid cell at */
double fz;		/* Results of evaluating "new" grid cell at x */
double devz;		/* Deviation from actual point & fx, fy */

maxdev = 0.0;
for (row = 0; row < grid->nrows; row++)
   {
   start_index = (row * grid->ncols) + start;
   end_index = (row * grid->ncols) + end;
   a0 = grid->z_values[start_index];
   a1 = (grid->z_values[end_index] - a0) / size;

/* Find maximum deviation in group from linear approximation
  ---------------------------------------------------------*/
   for (k = 1, cnt = (start + 1); cnt < end; cnt++, k++)
	{
	z = grid->out_samps[start + k] - grid->out_samps[start];
	fz = a0 + (a1 * z);	
	devz = fabs(fz - grid->z_values[start_index + k]);
	maxdev = GMAX(maxdev, devz);
	if (maxdev > tol) return(E_FAIL);
	}
   }
return(E_SUCC);
}


/****************************************************************************
NAME:				check_rrow

PURPOSE:    Check tolerance along group of rows being tested for elimination

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0      	 5/84  	M. NISHIHAMA       SAR        N/A
  5.0		 1/89	D. Steinwand	   CSB      LAS 5.0 C conversion

ALGORITHM DESCRIPTION:
	Set max deviation to zero
	Do for each row
	   Calculate starting and ending indicies into the Geometric Mapping
			Grid
	   Calculate linear coefficients
	   Find the maximum deviation along a group of rows
	   If the max deviation > tol, return (fail)
	Return (success)
*****************************************************************************/
long check_rrow
(
    long start,           /* Starting column, zero relative                 */
    long end,             /* Ending column, zero relative                   */
    double tol,           /* Grid reduction tolerance value                 */
    long size,            /* Size of "new" output space per "new" grid cell */
    struct RADGRID *grid  /* Geometric mapping grid                         */
)
{
double maxdev;		/* Maximum deviation */
long row;		/* Current row number */
long start_index;	/* Index into input grid; starting point */
long end_index;		/* Index into input grid; ending point */
double a0,a2;		/* Linear approximation coefficients for "new" cell */
long cnt;		/* Loop counter */
long k;			/* Offset into mapping grid for each group element */
long z;			/* Value to evaluate "new" grid cell at */
double fz;		/* Results of evaluating "new" grid cell at y */
double devz;		/* Deviation from actual point & fx, fy */

maxdev = 0.0;
for (row = 0; (row + end) < grid->nrows; row++)
   {
   start_index = (row + start) * grid->ncols;
   end_index = (row + end) * grid->ncols;
   a0 = grid->z_values[start_index];
   a2 = (grid->z_values[end_index] - a0) / size;

/* Find maximum deviation in group from linear approximation
  ---------------------------------------------------------*/
   for (k = 1, cnt = (start + 1); cnt < end; cnt++, k++)
	{
	z = grid->out_lines[start + k] - grid->out_lines[start];
	fz = a0 + (a2 * z);	
	devz = fabs(fz - grid->z_values[start_index + (k * grid->ncols)]);
	maxdev = GMAX(maxdev, devz);
	if (maxdev > tol) return(E_FAIL);
	}
   }
return(E_SUCC);
}


/****************************************************************************
NAME:				elim_rcol

PURPOSE:  Elimates redundant grid columns within tolerance

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		1/89	D. Steinwand	   CSB      New development

ALGORITHM DESCRIPTION:
	Print informational message and initialize variables
	Loop
	   Check for column elimination
	   If fail, decrement inc and try again until no columns are skiped
	   If pass, increment inc to the next grid spacing to check, until done
	Return

ALGORITHM REFERENCES:
	This routine is modeled after the grid reduction routines found in
	  the LAS 4.0 GRIDGEN by M. Nishihama of SAR, 5/84.  It is, however,
	  a different approach to the problem, optimized for techniques used
	  in GEOM, the geometric rectification function.
*****************************************************************************/
int elim_rcol
(
    struct RADGRID *grid,	/* Geometric mapping grid */
    double tol,			/* Grid reduction tolerance */ 
    long *next,			/* List of increments for reduction check */ 
    long *inc			/* Increment (index) into NEXT -- returned */ 
)
{ 
long max_inc = 11;		/* Maximum value of inc */
long size;			/* Size of output space grid cell */ 
long start;			/* Starting grid location (zero relative) */ 
long end;			/* Ending grid location (zero relative) */ 
long pass = 1;			/* Non-fatal (informational) message flag */

/* Initialize test spacing to skip one column
  ------------------------------------------*/
c_errmsg("Attempting grid reduction in the column direction"," ",&pass);
*inc = 1;
start = 0;
end = next[*inc];
size = grid->out_samps[end] - grid->out_samps[start];
for (;;)
   {
   if (check_rcol(start, end, tol, size, grid) != E_SUCC)

/* If test failed, try the previous spacing, starting at the column which
   failed the test.  This is now the maximum allowed spacing and the rest of
   the grid will be checked with it.  If the previous spacing is zero, the
   grid cannot be reduced; return fail.
  -----------------------------------*/
	{
	(*inc)--;
	if (*inc == 0) return (E_FAIL);
	max_inc = *inc;
	start = next[*inc];
	end = start + next[*inc];
	size = grid->out_samps[end] - grid->out_samps[start];
	}
   else

/* If test passed, increment the skip interval to the next value of "next".
   However, is a failure occured previously, the grid spacing is not adjusted;
   starting and ending points are adjusted to the next group of columns to test
  ---------------------------------------------------------------------------*/
	{
	if(max_inc == 11)
	   {
	   if (*inc == 11) return (E_SUCC);
	   (*inc)++;
	   end = start + next[*inc];
	   size = grid->out_samps[end] - grid->out_samps[start];
	   }
	else
	   {
	   if (end == next[11]) return(E_SUCC);
	   start = end;
	   end += next[*inc];
	   }
	}
   }
}


/****************************************************************************
NAME:				elim_rrow

PURPOSE:  Elimates redundant grid rows within tolerance

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		1/89	D. Steinwand	   CSB      New development

ALGORITHM DESCRIPTION:
	Print informational message and initialize variables
	Loop
	   Check for row elimination
	   If fail, decrement inc and try again until no rows are skiped
	   If pass, increment inc to the next grid spacing to check, until done
	Return

ALGORITHM REFERENCES:
	This routine is modeled after the grid reduction routines found in
	  the LAS 4.0 GRIDGEN by M. Nishihama of SAR, 5/84.  It is, however,
	  a different approach to the problem, optimized for techniques used
	  in GEOM, the geometric rectification function.
*****************************************************************************/
int elim_rrow
(
    struct RADGRID *grid,	/* Geometric mapping grid */
    double tol,			/* Grid reduction tolerance */
    long *next,			/* List of increments for reduction check */
    long *inc			/* Increment (index) into NEXT -- returned */
)
{
long max_inc = 11;		/* Maximum value of inc */
long size;			/* Size of output space grid cell */
long start;			/* Starting grid location (zero relative) */
long end;			/* Ending grid location (zero relative) */
long pass = 1;			/* Non-fatal (informational) message flag */

/* Initialize test spacing to skip one row 
  ---------------------------------------*/
c_errmsg("Attempting grid reduction in the row direction"," ",&pass);
*inc = 1;
start = 0;
end = next[*inc];
size = grid->out_lines[end] - grid->out_lines[start];
for (;;)
   {
   if (check_rrow(start, end, tol, size, grid) != E_SUCC)

/* If test failed, try the previous spacing, starting at the row which failed 
   the test.  This is now the maximum allowed spacing and the rest of the grid 
   will be checked with it.  If the previous spacing is zero, the grid cannot 
   be reduced; return fail.
  ------------------------*/
	{
	(*inc)--;
	if (*inc == 0) return (E_FAIL);
	max_inc = *inc;
	start = next[*inc];
	end = start + next[*inc];
	size = grid->out_lines[end] - grid->out_lines[start];
	}
   else

/* If test passed, increment the skip interval to the next value of "next".
   However, is a failure occured previously, the grid spacing is not adjusted;
   starting and ending points are adjusted to the next group of rows to test.
  -------------------------------------------------------------------------*/
	{
	if(max_inc == 11)
	   {
	   if (*inc == 11) return (E_SUCC);
	   (*inc)++;
	   end = start + next[*inc];
	   size = grid->out_lines[end] - grid->out_lines[start];
	   }
	else
	   {
	   if (end == next[11]) return(E_SUCC);
	   start = end;
	   end += next[*inc];
	   }
	}
   }
}


/****************************************************************************
NAME:				c_grdred	

PURPOSE:  Discard rows and columns of the radiometric mapping grid that are
	  within the tolerance of delta z approximation across neighboring
	  rows and columns.  Note:  Output grid cells are rectangular and
	  are all identical in size.


ALGORITHM DESCRIPTION:
	Eliminate columns within tolerance
	Eliminate rows within tolerance
	If grid cannot be reduced, return
	Remove columns not needed in output space
	Remove rows not needed in output space
	Loop through new output space, reducing the input space in both the
		row and column direction
	Return
*****************************************************************************/
int c_rgdred
(
    struct RADGRID *grid,	/* Geometric mapping grid             */
    double *tolerance		/* Tolerance for linear approximation */
)
{
long count, cnt;		/* loop counters */
long loc;			/* Location in non-reduced grid */
long loc_reduced;		/* Location in reduced grid */
long col_status;		/* Reduction of grid columns flag */
long row_status;		/* Reduction of grid rows flag */
long next[12];			/* Column/row increments allowable */
long row_inc;			/* Increment into NEXT array, row direction */
long col_inc;			/* Increment into NEXT array, col direction */
long k;				/* Local counter */

/* The "next" array contains increments to test which will keep the output 
   space grid evenly spaced.  This sceme also saves time
  -----------------------------------------------------*/
next[0] = 1;
next[1] = 2;	
next[2] = 3;
next[3] = 6;
next[4] = 7;
next[5] = 9;
next[6] = 14;
next[7] = 18;
next[8] = 21;
next[9] = 42;
next[10] = 63;
next[11] = 126;

/* Test grid for column and row elimination
  ----------------------------------------*/
col_status = elim_rcol(grid, *tolerance, next, &col_inc);
row_status = elim_rrow(grid, *tolerance, next, &row_inc);

/* If grid cannot be reduced, return (fail).  NOTE:  The routine did not get
   a fatal error--the grid failed to reduce.
  -----------------------------------------*/
if ((col_status == E_FAIL) && (row_status == E_FAIL)) return(E_FAIL);

/* Eliminate output space columns
  ------------------------------*/
if (col_status == E_SUCC)
   {
   for(count = 0, k = 0; count < grid->ncols; count += next[col_inc], k++)
	grid->out_samps[k] = grid->out_samps[count];
   grid->ncols = k;
   }

/* Eliminate output space rows
  ---------------------------*/
if (row_status == E_SUCC)
   {
   for(count = 0, k = 0; count < grid->nrows; count += next[row_inc], k++)
	grid->out_lines[k] = grid->out_lines[count];
   grid->nrows = k;
   }

/* Reduce input space grid in the row and column directions
  --------------------------------------------------------*/
for (loc_reduced = 0, cnt = 0; cnt < grid->nrows; cnt++)
   for (count = 0; count < grid->ncols; count++, loc_reduced++)
	{
       	loc = (cnt * MAX_GRID_SIZE * next[row_inc]) + (count * next[col_inc]);
       	grid->z_values[loc_reduced] = grid->z_values[loc];
       	}
return(E_SUCC);
}
