/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/*
 *      Micro-editor.   This allows the user to edit a line before
 *      sending it to TAE with Carriage Return.
 *
 *      TBD: handling of tabs correctly.
 *      TBD: a line longer than 80 from t_read not handled correctly.
 *
 *      CHANGE LOG:
 *      11-apr-84       new calling sequence for UNIX portability...palm
 *
 *                      The problem attacked here is that termcap/termlib
 *                      does not have facilities for reading cursor
 *                      position.  So in UNIX, we assume that the
 *                      line number can be passed to the editor;  this
 *                      works ok in all cases except in command mode
 *                      where the caller does not know the line number.
 *
 *                      In UNIX command mode, cmd_editor cannot obtain
 *                      the current line number, so we repeat the
 *                      prompt and the response on the last line for
 *                      editing.
 *
 *                      Note that in UNIX, t_getpos does nothing--the
 *                      line number passed by caller is un-changed.
 *      30-apr-84       Mode flag in editor calling sequence for UNIX
 *                      portability...dm
 *      04-may-84       VALUE_x to xVAL ... ces
 *      07-may-84       Clean-up the IMPORT declarations...lim
 *      16-jun-84       New PF2 to cycle valid strings...palm
 *      09-jul-84       Update to be in sync with UNIX changes...dm
 *      14-sep-84       Add check for index beyond max count of parm...lia
 *      01-dec-84       TCL 67: Use TUTCTX instead of EDTCTX...peb
 *      04-apr-85       VMS V4 editor compatibility...palm
 *      30-apr-85       Fix cursor position after CR in command mode
 *                      on a 'retrieved' command...palm
 *      03-may-85       Fix pf2 crash referencing outside of v_count...palm
 *      08-may-85       Avoid screen scroll with t_edit up/down arrows...palm
 *      25-jul-85       Fix UNIX lint errors...dm
 *      19-sep-85       PR 1012: Fixed bld_response so it doesn't crash TM when
 *                      tutoring on NAME parm w/o value via up/down arrow...dab
 *      12-nov-86       Added TAE Facelift...krw
 *      27-mar-87       Safety check: if current value not valid, don't crash
 *                      in valid_cycle....palm
 *      28-mar-87       PF3 for cycle valids backwards...palm
 *      18-may-87       Ported fix in cmd_editor (shows up under UNIX only): 
 *                      to invoke t_input() for screen mode...ljn
 *      07-aug-87       Add new menu_editor...palm
 *      21-sep-87       Move high_off out of nonfacelift restriction...tpl
 *      19-nov-87       Call t_read_event() to receive x events while 
 *                      waiting for input...dm
 *      03-feb-88       Changed DisplayId to XFACELIFT
 *      21-mar-88       Uknown terminal line number passed to t_output()
 *                      as negative number...ljn
 *      05-apr-88       PR1203: When looking for delete key, allow ^H...ljn
 *      27-jul-88       Removed X Window stuff...ljn
 *      13-jun-89       Removed TAE_FACELIFT...ljn
 *      23-mar-90       ifdef xfacelift in call to t_edit...tpl/dm
 *      27-jun-90       Remove Facelift code...ljn
 *      11-sep-92 PR875 Save real value and displayed value separately...tpl 
 *
 */

#include <string.h>
#include "taeconf.inp"   /* TAE configuration (REQUIRED)         */
#include "inttaelib.h"
#include "symtab.inc"    /* TM symbol table                      */
#include "terminc.inc"   /* terminal package                     */
#include "tminc.inc"     /* TM structures                        */
#include "tmhost.inp"    /* Host specific definitions            */
#include "menuinc.inc"   /* menu structures                      */
#include "vmcoproto.h" 
#include "taelib.h"
#include "local_protos.h"

FUNCTION static CODE fetch_up 
(
    BOOL   gold,            /* in: TRUE if GOLD/UP  */
    FUNINT maxl,            /* in: max line length  */
    TEXT   outline[],       /* out: response line   */
    TEXT   line[]           /* out: response line   */
);
FUNCTION static CODE fetch_down 
(
    BOOL   gold,            /* in: TRUE if GOLD/UP  */
    FUNINT maxl,            /* in: max line length  */
    TEXT   outline[],       /* out: response line   */
    TEXT   line[]           /* out: response line   */
);
FUNCTION static COUNT bld_response 
(
    struct VARIABLE *v,     /* in: TCL parameter to display         */
    FUNINT      index,      /* in: value index                      */
    FUNINT      maxl,       /* in: max size of response line        */
    FUNINT      vindex,     /* in: if positive, the S_VALID index   */
                            /* to use for obtaining a value         */
    TEXT        line[],     /* out: restricted response line        */
    TEXT        rline[]     /* out: unrestricted response line      */
);
FUNCTION static struct VARIABLE *previous 
(
    struct SYMTAB *symtab,  /* in: symbol table             */
    struct VARIABLE *vcur   /* in: current VARIABLE         */
);
FUNCTION static VOID high_on (void);
FUNCTION static VOID move_down
(
    BOOL gold               /* in: TRUE if moving way down  */
);
FUNCTION static BOOL visible 
(
    struct VARIABLE *v,     /* in: variable of interest     */
    FUNINT          index,  /* in: variable's index         */
    FUNINT          page    /* in: current page number      */
);
FUNCTION static COUNT valid_cycle 
(
    FUNINT maxl,            /* in: max line size            */
    TEXT   outline[],       /* out: reponse line            */
    TEXT   line[],          /* out: reponse line            */
    BOOL   forward          /* in: TRUE if cycle forward.   */
                            /*     FALSE for backwards      */
);
FUNCTION static VOID menu_fetch_up 
(
    BOOL   gold,            /* in: TRUE if GOLD/UP  */
    FUNINT maxl,            /* in: max line length  */
    TEXT   line[]           /* out: response line   */
);
FUNCTION static VOID menu_fetch_down 
(
    BOOL   gold,            /* in: TRUE if GOLD/UP  */
    FUNINT maxl,            /* in: max line length  */
    TEXT   line[]           /* out: response line   */
);
FUNCTION static VOID menu_high_on (void);
FUNCTION static VOID menu_high_off (void);
FUNCTION static VOID high_off (void);

/*      cmd_editor.   Command mode micro-editor.
 *
 *      Return code:
 *              SUCCESS -- line obtained successfully
 *              FAIL -- user killed command with double escape
 *      
 *      The special control sequences honored are:
 *                                              
 *              UP arrow: get most recent command
 *              DOWN arrow: down one from UP
 *              RIGHT arrow: move right one character
 *              LEFT arrow: move left one character
 *              GOLD/any-other: do extreme of the key
 *              CONTROL/U: clear the line
 *              CONTROL/X: clear the line
 *              CR: enter the line
 */

FUNCTION CODE cmd_editor 
(
    CODE   archive,         /* in: A_NONE or A_LASTCMD              */
    TEXT   line[],          /* out: user-typed line                 */
    TEXT   prompt[],        /* in: prompt string                    */
    COUNT  line_num,        /* in: line number on screen            */
                            /*     (-1 if line number not known)    */
    FUNINT screen           /* TRUE if in screen mode               */
)
{
    IMPORT struct VARIABLE *last_gbl;
    IMPORT COUNT        termcols;
    IMPORT CODE         termtype;

    CODE                terminator;
    COUNT               index, lm, c, maxl, i, j, l;
    COUNT               count, column;
    TEXT                tmpline[STRINGSIZ+1];
    BOOL                gold_active, gold;
    CODE                keytoke;        

#ifdef VAX_VMS
    CODE        code;
    IMPORT BOOL tt_editing;
    /* is "SET TERM/LINE_EDITING" active    */
    if (tt_editing)
        goto edit_with_vms;
#endif
    /* copy prompt, set left margin */
    strcpy(tmpline,prompt);
    /* copy prompt, set left margin */
    lm = strlen(tmpline) + 1;
    /* highlight prompt             */
    t_highlight (tmpline);

    if (screen)
    {
        /* prompt for screen    */
        t_output (line_num, 1, tmpline);
        t_input (line_num, lm, line, &terminator);
    }
    else
    {
        /* scrolling prompt     */
        t_write (tmpline, T_PROMPT);
        /* read line            */
        t_read (line, &terminator);
    }
    if (terminator == T_CR)
    {
        /* line read efficiently with no editing*/
        return (SUCCESS);
    }
    if (termtype != T_CRT)
    {
        /* not a CRT, so kill the command       */
        return (T_2ESC);
    }
    gold_active = FALSE;
    count = (*last_gbl).v_count;
    maxl = termcols - lm ;      /* maximum chars that can be read       */
    line [maxl] = EOS;          /* clip line if too long                */
    index = -1;                 /* current $LASTCMD index               */
    l = strlen(line);           /* current number of chars in line      */
    keytoke = t_1gettok();      /* get first token (escape already came)*/
    t_getpos (&line_num, &column);  /* get line nr (unless UNIX)    */
    if (line_num < 0)               /* if line nr still not known   */
    {                               /* stay on the most recent line */
        strcpy (tmpline,prompt);
        i = strlen(tmpline);
        /* add highlighting */
        i += t_highlight (tmpline);
        /* and the line typed so far */
        strcpy(&tmpline[i],line);
        t_output (line_num, 1, tmpline);
    }
    /* current index in line buffer */
    i = l;
    for (; keytoke != TERM_RETCHAR; keytoke=t_gettok())
    {
        /* copy for this iteration */
        gold = gold_active;
        /* so we can clear this once */
        gold_active = FALSE;
        if (keytoke == '\177' || keytoke == '\010')
        {
            /* DELETE key or Ctrl-H         */
            /* already at left?     */
            if (i == 0)
            {
                /* t_bell(); */
                continue;
            }
            if (gold)
            {
                /* number of chars to delet */
                c = i;
            }
            else
                c = 1;          

            /* delete c chars to left */
            for (j=i-c; j+c <= l; j++)
                line[j] = line[j+c];            

            /* temp copy of line */
            strcpy(tmpline,line);
            /* add spaces for each char deleted    */
            for (j=l-c; j < l; j++)
                tmpline[j] = ' ';
            tmpline[l] = EOS;
            /* new line length      */
            l = l - c;
            /* new current positon  */
            i = i - c;
            /* re-paint line*/
            t_output (line_num, lm+i, &tmpline[i]);
            /* position to current  */
            t_pos (line_num, lm + i);
        }
        else if (keytoke == '\025' || keytoke == '\030')
        {
            /* CONTROL/U or CONTROL/X */
            i = 0;
            l = 0;
            line[0] = EOS;
            t_lclear (line_num, lm);    
        }
        /* normal key?          */
        else if (keytoke < T_SPECIAL)
        {
            /* ignore control chars */
            if (keytoke < ' ' && keytoke != '\t')
                continue;
            /* tab: crude approach  */
            if (keytoke == '\t')
                keytoke = ' ';
            if (l >= maxl)                      
            {
                /* too many chars */
                t_bell();
                continue;
            }
            /* move chars over      */
            for (j=l; j > i; j--)
                line[j] = line[j-1];
            /* insert new char      */
            line[i] = keytoke;
            /* new length and EOS   */
            line[++l] = EOS;
            /* re-paint right side  */
            t_write (&line[i], T_NULL);
            /* move to next char    */
            i++;
            /* position to next     */
            t_pos (line_num, lm + i);
        }   
        else if (keytoke == T_LEFT)
        {
            if (i == 0)
                ; /* t_bell(); */               /* too far left         */
            else if (gold)
            {
                /* GOLD/LEFT            */
                /* position to first    */
                t_pos (line_num, lm);
                i = 0;
            }
            else
            {
                /* normal LEFT          */
                /* move left one char   */
                i--;
                t_pos(line_num, lm + i);
            }
        }
        else if (keytoke == T_RIGHT)
        {
            if (i >= l)
                ; /* t_bell(); */               /* too far right        */
            else if (gold)
            {
                /* GOLD/RIGHT           */
                i = l;
                t_pos (line_num, lm + i);
            }
            else
            {
                /* RIGHT                */
                i++;
                t_pos (line_num, lm + i);
            }
        }
        else if (keytoke == T_UP)
        {
            /* $LASTCMD ok?    */
            if (archive != A_LASTCMD || count <= 0)  
            {
                /* t_bell(); */                 /* if not, error        */
                continue;
            }
            if (gold)
            {
                /* GOLD/UP              */
                /* goto oldest cmd      */
                index = count-1;
            }
            else
            {
                /* if more, then next oldest cmd */
                if (index+2 < count)
                    index++;
            }
            s_bcopy (SVAL(*last_gbl, index), line, maxl);
            t_output (line_num, lm, line);
            l = strlen(line);
            i = l;
            /* clear rest of line   */
            t_lclear (line_num, lm + l);
        }
        else if (keytoke == T_DOWN)
        {
            /* $LASTCMD ok?    */
            if (archive != A_LASTCMD || count <= 0)
            {
                /* t_bell(); */
                continue;
            }
            if (gold)   
            {
                /* get most recent cmd  */
                index = 0;
            }
            else if (index < 0)                 /* starting up?         */
                continue;                       /* already down         */
            else
            {
                /* if room to move, move down one (-1 is new line) */
                if (index > -1)
                    index--;
            }
            /* retrieve command if the index is legal, otherwise clear the
               command line */
            if (index > -1)
                s_bcopy(SVAL(*last_gbl, index), line, maxl);
            else
                line[0] = EOS;
            t_output (line_num, lm, line);
            l = strlen(line);
            i = l;
            /* clear rest of line   */
            t_lclear (line_num, lm + l);
        }   
        else if (keytoke == T_GOLD)             /* GOLD                 */
            gold_active = TRUE;
        /* double escape?       */
        else if (keytoke == T_2ESC)
        {
            /* formality            */
            /* indicate 'kill cmd'  */
            line[0]=0;
            return (FAIL); 
        }
        else
            ; /* t_bell(); */                   /* unknown token        */
    }
    tmpline[0] = TERM_RETCHAR;
    tmpline[1] = '\0';
    /* echo CR */
    t_write (tmpline, T_NULL);
    return (SUCCESS);

#ifdef VAX_VMS
/*      This code make use of the VMS line editor and is executed
 *      when "SET TERM/LINE_EDIT" is active.
 */
edit_with_vms:                                  /* cmd_editor:          */
    line[0] = EOS;                              /* start edit with null */
    lm = strlen (prompt) + 1;                   /* left margin          */
    code = t_edit(prompt, line, screen);
    if (code  == T_CR)
        return (SUCCESS);
    t_getpos (&line_num, &column);              /* get line nr          */
    maxl = termcols - lm ;                      /* max chars to read    */
    gold_active = FALSE;
    count = (*last_gbl).v_count;                /* nr of $LASTCMDs      */
    index = -1;                                 /* $LASTCMD index       */
    do
        {
        t_lclear (line_num, 1);         /* clear line if CRT            */
        line[0] = EOS;                  /* start new line               */
        gold = gold_active;             /* copy for this iteration      */
        gold_active = FALSE;            /* so we can clear this once    */
        if (code == T_2ESC)
            return (FAIL);
        else if (code == T_GOLD)                /* GOLD                 */
            gold_active = TRUE;
        else if (code == T_UP)
            {
            if (archive != A_LASTCMD || count <= 0)  /* $LASTCMD ok?    */
                continue;
            if (gold)                           /* GOLD/UP              */
                index = count-1;                /* goto oldest cmd      */
            else
                {
                if (index+1 < count)            /* if more, then...     */
                    index++;                    /* next oldest cmd      */
                }
            s_bcopy (SVAL(*last_gbl, index), line, maxl);
            }
        else if (code == T_DOWN)
            {
            if (archive != A_LASTCMD || count <= 0)  /* $LASTCMD ok?    */
                continue;
            if (gold)   
                index = 0;                      /* get most recent cmd  */
            else if (index < 0)                 /* starting up?         */
                continue;                       /* already down         */
            else
                {
                if (index > 0)                  /* if room to move...   */
                    index--;                    /* move down one        */
                }
            s_bcopy(SVAL(*last_gbl, index), line, maxl);
            }   
        }
        while ((code = vms_edit1 (prompt, line, screen)) != T_CR);
    return(SUCCESS);

#endif
}

#ifdef VAX_VMS
FUNCTION CODE vms_edit1 ( prompt, line, screen )

    TEXT        prompt[];       /* in: prompt string                    */
    TEXT        line[];         /* out: user-typed line                 */
    FUNINT      screen;         /* TRUE if in screen mode               */

{
    CODE code;

    code = t_edit1 (prompt, line, screen);
    return(code);
}
#endif

/************************************************************************/
/* SEE TUTOR.CNP for the TAE_FACELIFT version of the following routines */
/************************************************************************/
/*      tut_editor.   Tutor mode micro-editor.
 *
 *      Return code:
 *              SUCCESS -- line obtained successfully
 *              FAIL -- user killed command with double escape
 *      
 *      The special control sequences honored are:
 *
 *              UP arrow: get most recent variable
 *              DOWN arrow: down one variable
 *              RIGHT arrow: move right one character
 *              LEFT arrow: move left one character
 *              GOLD/any-other: do extreme of the key
 *              PF2: cycle current VALID display
 *              CONTROL/U: clear the line
 *              CONTROL/X: clear the line
 *              CR: enter the line
 */

FUNCTION CODE tut_editor 
(
    TEXT  line[],           /* out: user-typed line                 */
    TEXT  prompt[],         /* in: characters in prompt string      */
    COUNT line_num          /* in: line number                      */
)
{
    IMPORT COUNT         termcols;
    IMPORT struct TUTCTX tutctx;

    CODE        code, terminator;
    COUNT       lm, c, maxl, i, j, l;
    TEXT        tmpline[STRINGSIZ+1];
    TEXT        outline[STRINGSIZ+1];
    BOOL        gold_active, gold;
    CODE        keytoke;        

#ifdef VAX_VMS
    IMPORT BOOL tt_editing;
    if (tt_editing)             /* is "SET TERM/LINE_EDITING" active    */
        goto edit_with_vms;
#endif
    /* copy prompt, set left margin */
    strcpy(tmpline,prompt);
    /* copy prompt, set left margin */
    lm = strlen(tmpline) + 1;
    /* highlight prompt             */
    t_highlight (tmpline);
    /* prompt */
    t_output (line_num, 1, tmpline);
    /* max chars that can be read */
    maxl = termcols - lm ;
    /* "call back" after generating a SHOW command?   */
    if (tutctx.edtcmd)
    {
        tutctx.edtcmd = FALSE;  
        i = l = bld_response (tutctx.vcur, tutctx.index, maxl, -1,outline,line);
        /* highlight current value      */
        high_on ();
        t_output (line_num, lm, outline);
        t_lclear (line_num, lm + l);
        /* get first keyboard token     */
        keytoke = t_gettok();
    }
    else 
    {
        /* first call for this line     */
        t_input (line_num, lm, line, &terminator);
        if (terminator == T_CR)
        {
            /* line read without editing */
            return (SUCCESS);
        }
        /* clip line if too long */
        line [maxl] = EOS;
        /* get remainder of esc sequence */
        keytoke = t_1gettok ();
        i = l = strlen (line);
    }
    gold_active = FALSE;
    for (; keytoke != TERM_RETCHAR; keytoke=t_gettok())
    {
        /* copy for this iteration */
        gold = gold_active;
        /* so we can clear this once */
        gold_active = FALSE;
        if (keytoke == '\177' || keytoke == '\010')
        {
            /* DELETE key or Ctrl-H         */
            if (i == 0)
            {
                /* already at left      */
                /* t_bell(); */
                continue;
            }
            if (gold)
            {
                /* GOLD/DELETE:         */
                /* find first '='       */
                j = s_index (line, '=');
                /* if found             */
                if (j >= 0)
                {
                    /* delete to '='      */
                    c = (i > j+1) ? (i-j-1) : i;
                }
                else
                {
                    /* delete all: no '='   */
                    c = i;
                }
            }
            else
                c = 1;          
            /* delete c chars to left */
            for (j=i-c; j+c <= l; j++)
                line[j] = line[j+c];            
            /* temp copy of line    */
            strcpy(tmpline,line);
            /* add spaces for each char deleted */
            for (j=l-c; j < l; j++)
                tmpline[j] = ' ';
            tmpline[l] = EOS;
            /* new line length      */
            l = l - c;
            /* new current positon  */
            i = i - c;
            /* if now at left side turn highlight off   */
            if (i == 0)
                high_off();
            /* re-paint     */
            t_output (line_num, lm+i, &tmpline[i]);
            /* position to current  */
            t_pos (line_num, lm + i);
        }
        else if (keytoke == '\025'  || keytoke == '\030')
        {
            /* CONTROL/U or CONTROL/X */
            i = 0;
            l = 0;
            line[0] = EOS;
            /* turn off highlight   */
            high_off();
            t_lclear (line_num, lm);    
        }
        else if (keytoke < T_SPECIAL)
        {
            /* normal key?          */
            /* ignore control chars */
            if (keytoke < ' ' && keytoke != '\t')
                continue;
            /* tab: crude approach  */
            if (keytoke == '\t')
                keytoke = ' ';
            if (l >= maxl)                      
            {
                /* too many chars       */
                t_bell();
                continue;
            }
            /* move chars over      */
            for (j=l; j > i; j--)
                line[j] = line[j-1];
            /* insert new char      */
            line[i] = keytoke;
            /* new length and EOS   */
            line[++l] = EOS;
            /* re-paint right side  */
            t_write (&line[i], T_NULL);
            /* move to next char    */
            i++;
            /* position to next     */
            t_pos (line_num, lm + i);
        }   
        else if (keytoke == T_LEFT)
        {
            if (i == 0)
            {
                /* already at far left  */
                continue;
            }
            else if (gold)
            {
                /* GOLD/LEFT            */
                /* equals present?      */
                j = s_index (line, '=');
                if (j >= 0)
                {
                    /* move to = if to right*/
                    i = (i > j+1) ? j+1 : 0;
                }
                else
                {
                    /* move to begin        */
                    i = 0;
                }
            }
            else
            {
                /* normal LEFT, move left one char */
                i--;
            }
            /* new position */
            t_pos (line_num, lm+i);             
        }
        else if (keytoke == T_RIGHT)
        {
            if (i >= l)
                ; /* t_bell(); */               /* too far right        */
            else if (gold)
            {
                /* GOLD/RIGHT           */
                i = l;
                t_pos (line_num, lm + i);
            }
            else
            {
                /* RIGHT                */
                i++;
                t_pos (line_num, lm + i);
            }
        }
        else if (keytoke == T_UP)
        {
            /* ignore if subcmd     */
            if (tutctx.subtut)
                continue;
            /* get previous         */
            code = fetch_up (gold, maxl, outline, line);
            if (code != SUCCESS)
            {
                /* must SHOW first      */
                return (SUCCESS);
            }
            t_output (line_num, lm, outline);
            l = strlen (line);
            /* position to end */
            i = l;
            /* clear rest of line   */
            t_lclear (line_num, lm + i);
        }
        else if (keytoke == T_DOWN)
        {
            /* ignore if subcmd     */
            if (tutctx.subtut)
                continue;
            /* get next */
            code = fetch_down (gold, maxl, outline, line);
            if (code != SUCCESS)
            {
                /* must SHOW first      */
                return (SUCCESS);
            }
            t_output (line_num, lm, outline);
            l = strlen (line);
            i = l;
            /* clear rest of line */
            t_lclear (line_num, lm + l);
        }   
        else if (keytoke == T_F2  ||  keytoke == T_F3)  
        {
            /* valid cycle  */
            if (tutctx.highact && (*tutctx.vcur).v_type == V_STRING &&
               (*tutctx.vcur).v_valid != NULL)
            {
                i = l = valid_cycle (maxl, outline, line, keytoke==T_F2);
                t_output (line_num, lm, outline);
                t_lclear (line_num, lm+ l);
            }
        }
        else if (keytoke == T_GOLD)             /* GOLD                 */
            gold_active = TRUE;
        else if (keytoke == T_2ESC)
        {
            /* double escape? */
            /* formality */
            line[0]=0;
            /* turn highlight off */
            high_off();
            /* indicate 'kill cmd' */
            return (FAIL);
        }
        else
            ; /* t_bell(); */                   /* unknown token        */
    }
/*  tmpline[0] = TERM_RETCHAR;
 *  tmpline[1] = 0;
 *  t_write (tmpline, T_NULL);                  **      echo CR         **
 */
    /* turn highlight off   */
    high_off();
    return (SUCCESS);

#ifdef VAX_VMS
/*      This code make use of the VMS line editor and is executed
 *      when "SET TERM/LINE_EDIT" is active.
 */
edit_with_vms:                          /* tutor mode editor:           */
    lm = strlen(prompt) + 1;            /* start column of response     */
    maxl = termcols - lm ;              /* max chars that can be read   */
    if (tutctx.edtcmd)                  /* "call back" after ...        */
        {                               /* generating a SHOW command?   */
        tutctx.edtcmd = FALSE;  
        i = l = bld_response (tutctx.vcur, tutctx.index,maxl,-1,outline,line);
        high_on ();                     /* highlight current value      */
        }
    else                                /* first call for this line     */
        line[0] = EOS;                  /* start with no line           */
    t_lclear (line_num, 1);             /* position/clear prompt line   */
    gold_active = FALSE;
    while ((code = t_edit (prompt, outline, TRUE)) != T_CR)
        {
        gold = gold_active;             /* copy for this iteration      */
        gold_active = FALSE;            /* so we can clear this once    */
        if (code == T_UP)
            {
            if (!tutctx.subtut)                 /* if real tutor:       */
                {
                code = fetch_up (gold, maxl, outline, line);/* get previous     */
                if (code != SUCCESS)
                    return (SUCCESS);           /* must SHOW first*/
                }
            }
        else if (code == T_DOWN)
            {
            if (!tutctx.subtut)                 /* if real tutor        */
                {
                code = fetch_down (gold, maxl, outline, line);/* get next*/
                if (code != SUCCESS)
                    return (SUCCESS);           /* must SHOW first      */
                }
            }   
        else if (code == T_F2  || code == T_F3) /* valid cycle          */
            {
            if (tutctx.highact && (*tutctx.vcur).v_type == V_STRING &&
               (*tutctx.vcur).v_valid != NULL)
               valid_cycle (maxl, outline, line, code==T_F2);
            }
        else if (code == T_GOLD)                /* GOLD                 */
            gold_active = TRUE;
        else if (code == T_2ESC)                /* double escape?        */
            {
            line[0]=0;                          /* formality            */
            high_off();                         /* turn highlight off   */
            return (FAIL);                      /* indicate 'kill cmd'  */
            }
        t_lclear (line_num, 1);                 /* clear prompt line    */
        }
    high_off();
    return (SUCCESS);
#endif /* VAX_VMS */

}

/*      fetch_up.   Get response for UP arrow in tutor.
 *
 *      Return:
 *              SUCCESS -- if up arrow handling completed
 *              FAIL -- Tutor command was generated and editing is
 *                      temporarily finished until tut_editor re-called.
 */
FUNCTION static CODE fetch_up 
(
    BOOL   gold,            /* in: TRUE if GOLD/UP  */
    FUNINT maxl,            /* in: max line length  */
    TEXT   outline[],       /* out: response line   */
    TEXT   line[]           /* out: response line   */
)
{
    IMPORT struct TUTCTX tutctx;        /* tutor context for editor     */

    struct VARIABLE *v;
    struct CONTXT *pctx;

    /* turn current highlighting off */
    high_off ();
    /* proc context ptr */
    pctx = tutctx.ctx;
    if ((*pctx).parmst.link == NULL) 
    {
        /* if no parms, give user a RUN command */
        strcpy(line,"RUN");
        return (SUCCESS);
    }
    if (tutctx.start)
    {
        /* position already set      */
        tutctx.start = FALSE;                   
    }
    else
    {
        /* if moving to top of .vcur */
        if (tutctx.index > 0  && gold)
            tutctx.index = 0;
        else if (tutctx.index == 0)
        {
            /* if moving to previous var */
            tutctx.vcur = v = previous (&(*pctx).parmst, tutctx.vcur);
            if ((*v).v_type == V_NAME  ||  gold)
            {
                /* gold or NAME: to top */
                tutctx.index = 0;
            }
            else
            {
                /* else to last element */
                tutctx.index = (*v).v_maxc - 1;
            }
        }
        else
        {
            /* simple up in current parm */
            tutctx.index--;
        }
    }
    if (!visible (tutctx.vcur, tutctx.index, tutctx.curpag))
    {
        /* must display value */
        tutctx.edtcmd = TRUE;
        strcpy(line,"SHOW");
        return (FAIL);
    }
    bld_response (tutctx.vcur, tutctx.index, maxl, -1, outline,  line);
    /* turn highlight on    */
    high_on ();
    return (SUCCESS);
}

/*      fetch_down.   Get response for DOWN arrow in tutor.
 *
 *      Return:
 *              SUCCESS -- response generated okay
 *              FAIL -- tutor cmd generated and must wait for call back
 */
FUNCTION static CODE fetch_down 
(
    BOOL   gold,            /* in: TRUE if GOLD/UP  */
    FUNINT maxl,            /* in: max line length  */
    TEXT   outline[],       /* out: response line   */
    TEXT   line[]           /* out: response line   */
)
{
    IMPORT struct TUTCTX tutctx;        /* tutor context                */

    struct CONTXT *pctx;

    /* turn highlight off */
    high_off ();
    /* proc context ptr */
    pctx = tutctx.ctx;
    /* if no parms, give him a RUN command */
    if ((*pctx).parmst.link == NULL)
    {
        strcpy(line,"RUN");
        return (SUCCESS);
    }
    if (tutctx.start)
    {
        /* position already set         */
        tutctx.start = FALSE;
    }
    else
    {
        /* move down to next parm       */
        move_down (gold);
        if (!visible (tutctx.vcur, tutctx.index, tutctx.curpag))
        {
            /* must make it visible */
            tutctx.edtcmd = TRUE;
            strcpy(line,"SHOW");
            return (FAIL);
        }
    }
    bld_response (tutctx.vcur, tutctx.index, maxl, -1, outline, line);
    /* highlight current            */
    high_on ();
    return (SUCCESS);
}

/*      bld_response.    Build a response line for tutor.
 *      The string length is returned as function value.
 *
 *      "variable = value" or
 *      "variable(index) = value"
 *
 */

FUNCTION static COUNT bld_response 
(
    struct VARIABLE *v,     /* in: TCL parameter to display         */
    FUNINT      index,      /* in: value index                      */
    FUNINT      maxl,       /* in: max size of response line        */
    FUNINT      vindex,     /* in: if positive, the S_VALID index   */
                            /* to use for obtaining a value         */
    TEXT        line[],     /* out: restricted response line        */
    TEXT        rline[]     /* out: unrestricted response line      */
)
{
    struct VARIABLE *vref;
    struct S_VALID  *valid;
    COUNT       i;
    CODE        type;

    strcpy(rline,(*v).v_name);
    i = strlen(rline);
    /* if multi-valued */
    if ((*v).v_type != V_NAME  &&  (*v).v_maxc > 1)
    {
        rline[i++] = '(';
        s_i2s (index+1, &rline[i]);
        i = strlen(rline);
        rline[i++] = ')';
    }
    rline[i++] = '=' ;
    type = (*v).v_type;
    if (vindex >= 0)
    {
        /* use S_VALID:  */
        valid = (struct S_VALID *)(*v).v_valid;
        strcpy(&rline[i],(*valid).slist[vindex].string);
        addqu (&rline[i]);
    }       
    /* NAME parm must have ref value */
    else if ( (type != V_NAME && index >= (*v).v_count) || 
         (type == V_NAME && (*v).v_ref == NULL) ) 
    {
        /* no value     */
        rline [i] = EOS;
    }
    else if (type == V_INTEGER)
        s_i2s(IVAL(*v, index), &rline[i]);
    else if (type == V_REAL)
        s_r2s(RVAL(*v, index), &rline[i]);
    else if (type == V_STRING)
    {
        strcpy(&rline[i],SVAL(*v, index));
        /* add quotes */
        addqu (&rline[i]);
    }
    else if (type == V_NAME)
    {
        vref = (*v).v_ref;
        /* use name of reference */
        strcpy(&rline[i],(*vref).v_name);
    }
    if (strlen(rline) > maxl)
    {
        /* indicate clipped     */
        s_bcopy (rline, line, maxl);
        /* indicate clipped only for strings: */
        strcpy(&line[maxl-4],"...\"");
        line[maxl] = EOS;
        i = maxl;
    }
    else
    {
        strcpy(line,rline);
        i = strlen(line );
    }
    return (i);
}

/*      previous.   Get previous variable name.
 *
 *      The previous may be NULL, if there are no parameters.
 */

FUNCTION static struct VARIABLE *previous 
(
    struct SYMTAB *symtab,  /* in: symbol table             */
    struct VARIABLE *vcur   /* in: current VARIABLE         */
)
{
    struct VARIABLE *v, *p;     /* working VARIABLE pointer     */

    /* safety check */
    if (vcur == NULL)
        return (NULL);
    /* first variable?      */
    if (vcur == (*symtab).link) 
    {
        /* search for last      */
        v = vcur;
        while (1)
        {
            if ((*v).v_link == NULL)
            {
                /* last is previous to first */
                return (v);
            }
            v = (*v).v_link;
        }
    }

    p = NULL;   
    for (v=(*symtab).link; v != NULL; v = (*v).v_link)
    {
        if (v == vcur)
            return (p);
        /* new previous                 */
        p = v;
    }       
    /* safety catch for bad vcur */
    return (NULL);
}

/*      high_on.        Turn on highlighting for current value.
 */
FUNCTION static VOID high_on (void)
{
    IMPORT struct TUTCTX tutctx;

    /* no PARMs protection     */
    if (tutctx.vcur != NULL)
    {
        high_value (tutctx.vcur, tutctx.index, TRUE);
        tutctx.highact = TRUE;
        /* no valid cycle established yet */
        tutctx.vindex = -1;
    }
}

/*      move_down.   Move current tutor position down.
 *      This updates the tutctx.vcur and tutctx.index variables.
 */
FUNCTION static VOID move_down
(
    BOOL gold               /* in: TRUE if moving way down  */
)
{
    IMPORT struct TUTCTX tutctx;                /* editor context       */

    struct CONTXT       *pctx;                  /* current proc context */
    struct VARIABLE      *v;

    pctx = tutctx.ctx;                  
    /* current position */
    v = tutctx.vcur;
    /* move to new parm?    */
    if (gold || (*v).v_type == V_NAME  || tutctx.index >= (*v).v_maxc - 1)
    {
        /* get next parm */
        tutctx.vcur = v = (*v).v_link;
        if (v == NULL)
        {
            /* wrap to beginning */
            tutctx.vcur = v = (*pctx).parmst.link;
        }
        tutctx.index = 0;
    }
    else
    {
        /* move to next component */
        tutctx.index++;
    }
}

/*      visible.    TRUE if the variable and component is visible
 *      on the tutor screen.
 */

FUNCTION static BOOL visible 
(
    struct VARIABLE *v,     /* in: variable of interest     */
    FUNINT          index,  /* in: variable's index         */
    FUNINT          page    /* in: current page number      */
)
{
    struct TUTEXT       *tp;

    tp = (*v).v_tp;             
    /* on another page? */
    if (tp == NULL || page != (*tp).pagnum) 
        return (FALSE);                 
    /* outside panel? */
    if (index+1 > (*tp).panbot ||  index+1 < (*tp).pantop) 
        return (FALSE);
    return (TRUE);
}

/*      valid_cycle.     Cycle thru valid values for a string.
 *
 *      Since PF2 means forward and PF3 means backwards, this
 *      is usually called with forward = "key==PF2".
 */

FUNCTION static COUNT valid_cycle 
(
    FUNINT maxl,            /* in: max line size            */
    TEXT   outline[],       /* out: reponse line            */
    TEXT   line[],          /* out: reponse line            */
    BOOL   forward          /* in: TRUE if cycle forward.   */
                            /*     FALSE for backwards      */
)
{
    IMPORT struct TUTCTX tutctx;
    COUNT       vindex;
    struct S_VALID *valid;
    struct VARIABLE *v;

    /* local convenience copy */
    v = tutctx.vcur;
    valid = (struct S_VALID *) (*v).v_valid;
    if (tutctx.vindex < 0)
    {
        /* establish VALID position: */
        if (tutctx.index < (*v).v_count)
        {
            /* find current value in VALID list in case not found: start at 
               begin */
            tutctx.vindex = -1;
            for (vindex = 0; vindex < (*valid).count; vindex++)
            {
                if (s_equal ((*valid).slist[vindex].string,
                             SVAL(*v, tutctx.index)))
                {
                    /* set current valid index */
                    tutctx.vindex = vindex;
                    break;
                }
            }       
        }
        else
        {
            /* no value: start at beginning */
            tutctx.vindex = -1;
        }
    }
    if (forward)
    {
        tutctx.vindex =
            (tutctx.vindex >= (*valid).count-1) ? 0 : tutctx.vindex+1;
    }
    else
    {
        tutctx.vindex = 
            (tutctx.vindex <= 0) ? (*valid).count-1 : tutctx.vindex-1;  
    }
    return (bld_response (v, tutctx.index, maxl, tutctx.vindex,outline, line));
}

/************************************************************************/
/*      menu_editor.   Menu mode micro-editor.
 *
 *      Return code:
 *              SUCCESS -- line obtained successfully
 *              FAIL -- user killed command with double escape
 *      
 *      The special control sequences honored are:
 *
 *              UP arrow: get most recent entry
 *              DOWN arrow: down one entry 
 *              RIGHT arrow: move right one character
 *              LEFT arrow: move left one character
 *              GOLD/any-other: do extreme of the key
 *              CONTROL/U: clear the line
 *              CONTROL/X: clear the line
 *              CR: execute the currently displayed command
 */

FUNCTION CODE menu_editor 
(
    TEXT  line[],           /* out: user-typed line                 */
    TEXT  prompt[],         /* in: characters in prompt string      */
    COUNT line_num          /* in: line number                      */
)
{
    IMPORT COUNT         termcols;

    CODE        terminator;
    COUNT       lm, c, maxl, i, j, l;
    TEXT        tmpline[STRINGSIZ+1];
    BOOL        gold_active, gold;
    CODE        keytoke;        

#ifdef VAX_VMS
    CODE        code;
    IMPORT BOOL tt_editing;
    /* is "SET TERM/LINE_EDITING" active    */
    if (tt_editing)
        goto edit_with_vms;
#endif
    /* copy prompt, set left margin */
    strcpy(tmpline,prompt);
    /* copy prompt, set left margin */
    lm = strlen(tmpline) + 1;
    /* highlight prompt */
    t_highlight (tmpline);
    /* prompt */
    t_output (line_num, 1, tmpline);
    /* max chars that can be read */
    maxl = termcols - lm ;
    t_input (line_num, lm, line, &terminator);
    /* line read without editing */
    if (terminator == T_CR)
        return (SUCCESS);
    /* clip line if too long */
    line [maxl] = EOS;
    /* get remainder of esc sequence */
    keytoke = t_1gettok ();
    i = l = strlen (line);
    gold_active = FALSE;
    for (; keytoke != TERM_RETCHAR; keytoke=t_gettok())
    {
        /* copy for this iteration */
        gold = gold_active;
        /* so we can clear this once */
        gold_active = FALSE;
        if (keytoke == '\177' || keytoke == '\010')
        {
            /* DELETE key or Ctrl-H         */
            if (i == 0)
            {
                /* already at left      */
                /* t_bell(); */
                continue;
            }
            if (gold)
            {
                /* GOLD/DELETE:         */
                /* delete whole line    */
                c = i;
            }
            else
            {
                /* delete one char      */
                c = 1;
            }
            /* delete c chars to left */
            for (j=i-c; j+c <= l; j++)
                line[j] = line[j+c];            
            /* temp copy of line */
            strcpy(tmpline,line);
            /* add spaces for each char deleted */
            for (j=l-c; j < l; j++)
                tmpline[j] = ' ';
            tmpline[l] = EOS;
            /* new line length */
            l = l - c;
            /* new current positon */
            i = i - c;
            /* if now at left side turn highlight off */
            if (i == 0)
                menu_high_off();
            /* re-paint */
            t_output (line_num, lm+i, &tmpline[i]);   
            /* position to current  */
            t_pos (line_num, lm + i);
        }
        else if (keytoke == '\025'  || keytoke == '\030')
        {
            /* CONTROL/U or CONTROL/X */
            i = 0;
            l = 0;
            line[0] = EOS;
            /* turn off highlight */
            menu_high_off();
            t_lclear (line_num, lm);    
        }
        /* normal key?          */
        else if (keytoke < T_SPECIAL)
        {
            /* ignore control chars */
            if (keytoke < ' ' && keytoke != '\t')
                continue;
            /* tab: crude approach */
            if (keytoke == '\t')
                keytoke = ' ';
            if (l >= maxl)                      
            {
                /* too many chars */
                t_bell();
                continue;
            }
            /* move chars over */
            for (j=l; j > i; j--)
                line[j] = line[j-1];
            /* insert new char */
            line[i] = keytoke;
            /* new length and EOS */
            line[++l] = EOS;
            /* re-paint right side */
            t_write (&line[i], T_NULL);
            /* move to next char */
            i++;
            /* position to next */
            t_pos (line_num, lm + i);
        }   
        else if (keytoke == T_LEFT)
        {
            if (i == 0)
            {
                /* already at far left  */
                continue;
            }
            else if (gold)
            {
                /* GOLD/LEFT, move to begin        */
                i = 0;
            }
            else
            {
                /* normal LEFT, move left one char   */
                i--;
            }
            /* new position */
            t_pos (line_num, lm+i);
        }
        else if (keytoke == T_RIGHT)
        {
            if (i >= l)
            {
                /* too far right        */
                ; /* t_bell(); */
            }
            else if (gold)
            {
                /* GOLD/RIGHT           */
                i = l;
                t_pos (line_num, lm + i);
            }
            else 
            {
                /* RIGHT                */
                i++;
                t_pos (line_num, lm + i);
            }
        }
        else if (keytoke == T_UP)
        {
            /* get previous */
            menu_fetch_up (gold, maxl, line);           
            t_output (line_num, lm, line);
            l = strlen (line);
            /* position to end */
            i = l;
            /* clear rest of line */
            t_lclear (line_num, lm + i);
        }
        else if (keytoke == T_DOWN)
        {
            /* get next ent */
            menu_fetch_down (gold, maxl, line);
            t_output (line_num, lm, line);
            l = strlen (line);
            i = l;
            /* clear rest of line */
            t_lclear (line_num, lm + l);
        }   
        else if (keytoke == T_GOLD)             /* GOLD                 */
            gold_active = TRUE;
        else if (keytoke == T_2ESC)
        {
            /* double escape? */
            /* formality */
            line[0]=0;
            /* turn highlight off */
            menu_high_off();
            /* indicate 'kill cmd' */
            return (FAIL);
        }
        else
        {
            /* unknown token        */
            ; /* t_bell(); */
        }
    }
    /* turn highlight off */
    menu_high_off();
    return (SUCCESS);

#ifdef VAX_VMS
/*      This code make use of the VMS line editor and is executed
 *      when "SET TERM/LINE_EDIT" is active.
 */
edit_with_vms:                          /* tutor mode editor:           */
    lm = strlen(prompt) + 1;            /* start column of response     */
    maxl = termcols - lm ;              /* max chars that can be read   */
    line[0] = EOS;                      /* start with no line           */
    t_lclear (line_num, 1);             /* position/clear prompt line   */
    gold_active = FALSE;
    while ((code = t_edit (prompt, line, TRUE)) != T_CR)
        {
        gold = gold_active;             /* copy for this iteration      */
        gold_active = FALSE;            /* so we can clear this once    */
        if (code == T_UP)
            menu_fetch_up (gold, maxl, line);           /* get previous */
        else if (code == T_DOWN)
            menu_fetch_down (gold, maxl, line);         /* get next     */
        else if (code == T_GOLD)                /* GOLD                 */
            gold_active = TRUE;
        else if (code == T_2ESC)                /* double escape?        */
            {
            line[0]=0;                          /* formality            */
            high_off();                         /* turn highlight off   */
            return (FAIL);                      /* indicate 'kill cmd'  */
            }
        t_lclear (line_num, 1);                 /* clear prompt line    */
        }
    menu_high_off();
    return (SUCCESS);
#endif /* VAX_VMS */
}

/*      menu_fetch_up.   Get response for UP arrow in menu. 
 *
 */
FUNCTION static VOID menu_fetch_up 
(
    BOOL   gold,            /* in: TRUE if GOLD/UP  */
    FUNINT maxl,            /* in: max line length  */
    TEXT   line[]           /* out: response line   */
)
{
    IMPORT struct CURMEN  *Curmenu;             /* menu context              */

    /* turn off highlighting */
    menu_high_off ();
    line[0] = EOS;
    /* safety check */
    if (Curmenu == NULL || (*Curmenu).nent <= 0)
        return ;                
    /* move to first entry? */
    if (gold)
        (*Curmenu).index = 0;
    /* wrap to last? */
    else if ((*Curmenu).index <= 0)
        (*Curmenu).index = (*Curmenu).nent - 1;
    else 
    {
        /* move back one */
        (*Curmenu).index--;
    }
    /* build response */
    sprintf (line, "%d", (*Curmenu).index+1);
    /* turn highlighting on */
    menu_high_on ();  
}


/*      menu_fetch_down.   Get response for DOWN arrow in menu.
 *
 */
FUNCTION static VOID menu_fetch_down 
(
    BOOL   gold,            /* in: TRUE if GOLD/UP  */
    FUNINT maxl,            /* in: max line length  */
    TEXT   line[]           /* out: response line   */
)
{
    IMPORT struct CURMEN *Curmenu;              /* menu context         */


    /* turn highlight off */
    menu_high_off ();
    line[0] = EOS;
    /* safety check */
    if (Curmenu == NULL || (*Curmenu).nent <= 0)
        return ;                
    if (gold)   
    {
        /* move to bottom       */
        (*Curmenu).index = (*Curmenu).nent-1;
    }
    else if ((*Curmenu).index < 0 || (*Curmenu).index >= (*Curmenu).nent - 1)
    {
        /* wrap to top          */
        (*Curmenu).index = 0;
    }
    else
    {
        /* move down one        */
        (*Curmenu).index++;
    }
    sprintf (line, "%d", (*Curmenu).index+1);
    /* highlight current */
    menu_high_on ();
    return ;
}

/*      menu_high_on and menu_high_off.
 *
 *      Turn menu entry highlight on/off
 */

FUNCTION static VOID menu_high_on (void)
{
    IMPORT struct CURMEN *Curmenu; 
    TEXT        line[STRINGSIZ+1];

    
    /* safety check */
    if (Curmenu == NULL || (*Curmenu).nent <= 0 || (*Curmenu).index < 0 || 
        (*Curmenu).index >= (*Curmenu).nent)
        return;
    strcpy(line,(*Curmenu).entryline[(*Curmenu).index]);
    t_highlight (line);
    t_output ((*Curmenu).linenr[(*Curmenu).index], 1, line);
}


FUNCTION static VOID menu_high_off (void)
{
    IMPORT struct CURMEN *Curmenu; 

    /* safety check */
    if (Curmenu == NULL || (*Curmenu).nent <= 0 || (*Curmenu).index < 0 || 
        (*Curmenu).index >= (*Curmenu).nent)
        return;
    t_output ((*Curmenu).linenr[(*Curmenu).index], 1, 
                                (*Curmenu).entryline[(*Curmenu).index]);
}

/*      high_off.   Turn off any highlighting if active.
 */
FUNCTION static VOID high_off (void)
{
    IMPORT struct TUTCTX tutctx;

    /* highlight active? */
    if (tutctx.highact)
    { 
        /* turn off highlight */
        high_value (tutctx.vcur, tutctx.index, FALSE);
        /* no highlight active */
        tutctx.highact = FALSE;
    }
}
