/*
 *      PDF Logging
 *
 *      CHANGE LOG:
 *
 *      06-dec-95       Development
 *                      Jim Fenno - EROS Data Center
 *      13-aug-96       Added option of not logging specified directories
 *                      Also added option of disabling logging completely.
 *                      Dan Etrheim - EROS Data Center
 *      10-oct-96       Change the way we were getting the username.
 *                      cuserid would sometimes give incorrect results. 
 *                      Also enhanced the logging capabilities
 */

#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#ifndef linux
#include <sys/systeminfo.h>
#endif
#include <sys/signal.h>
#include <time.h>
#include <unistd.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "taeconf.inp"  /* TAE configuration (REQUIRED)         */
#include "inttaelib.h"
#include "symtab.inc"   /* TM symbol table                      */
#include "tmhost.inp"   /* host-dependent                       */
#include "parblk.inc"   /* Parameter block                      */
#include "terminc.inc"  /* terminal support                     */
#include "syninc.inc"   /* syntax package structs & defs        */
#include "tminc.inc"    /* TM defs                              */
#include "local_protos.h"



#define DO_NOT_LOG "do_not_log" /* File within $LASTABLES containing PDF names*/
#define GID      30     /* Group ID for the log file                          */
#define MAXPDFS 100     /* Max PDFs allowed in $LASTABLES/DO_NOT_LOG       */
#define PDFLEN   80     /* Max length of PDF names allowed in DO_NOT_LOG      */
#define UID   16567     /* User ID of the account to own the log file         */


#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 256      /* Max length of a host name                  */
#endif


#ifndef EROS

FUNCTION VOID pdflog 
(
    struct FSBLOCK *fsblock /* in: fsblock for the pdf            */
)
{
    return;
}

#else

static char hostname[MAXHOSTNAMELEN] = {""};/* Name of the current system */
static char logfile[MAXSTRSIZ]       = {""};/* Name of the log file       */
static long npdfs                    = 0;   /* Number of PDFs not to log  */
static char pdfnames[MAXPDFS][PDFLEN+1];    /* PDFs not to be logged      */
static char username[L_cuserid]      = {""};/* Name of the current user   */
FUNCTION static void get_pdfnames (void);

/*
 * pdflog
 *
 * This function will log the execution of all PDFs run in TAE.  The log
 * file will be created in $APPLOG or the current directory if $APPLOG is
 * not defined.  An example log file is shown below.
 *
 * testgen    fenno      12/06/1995 08:38 sg2        $LASBASE
 * logtest    fenno      12/06/1995 08:39 edcsgw3    /sg2/csb/fenno/run
 *
 */
FUNCTION VOID pdflog 
(
    struct FSBLOCK *fsblock /* in: fsblock for the pdf            */
)
{
    char buf[1024];             /* Buffer for writing to the file             */
    char dir[MAXSTRSIZ];        /* Temporary copy of the library directory    */
    char pdf[MAXSTRSIZ];        /* Temporary copy of the pdf name             */
    char *ptr;                  /* Ptr used to retrieve environment variables */
    char tmp[MAXSTRSIZ];        /* Temporary string                           */

    time_t curtime;             /* Current time of day                        */
    struct tm *date;            /* Current date in a C structure              */
    struct flock lock_info;     /* Structure containing the file locking info */
    off_t size;                 /* Size of log before appending the string    */

    int access;                 /* File access type                           */
    int fd;                     /* File descriptor                            */

    long i;                     /* String index                               */
    long nbytes;                /* Bytes in the buffer written to the log     */

    int uid;
    struct passwd *pwptr;

    /*
     * Get the current date and time
     */
    time(&curtime);
    date = localtime(&curtime);


    /*
     * Initialize the static variables
     */
    if (strlen(logfile) == 0)
    {
        /*
         * Create the name of the log file to be used
         */
        ptr = getenv("APPLOG");
        if (ptr == NULL)
            sprintf(logfile, "tae_%04d_%03d.log",
                date->tm_year + 1900, date->tm_yday + 1);
        else
            sprintf(logfile, "%s/tae_%04d_%03d.log", ptr,
                date->tm_year + 1900, date->tm_yday + 1);


        /*
         * Get the system name.  sysinfo() will return the number of characters
         * stored in 'hostname', including the null (\0) character.
         */
#ifndef linux
        if (sysinfo(SI_HOSTNAME, hostname, MAXHOSTNAMELEN) < 2)
#else
        ptr = getenv("HOST");
        if (strcpy(hostname, ptr) == NULL)
#endif
            strcpy(hostname, "unknown");

        /*
         * Get the user name
         */
        uid = getuid();
        pwptr = getpwuid(uid);
        strcpy(username, pwptr->pw_name);

        /*
         * Retrieve the names of the PDFs that are not to be logged
         */
        get_pdfnames();
    }

    /* No logging specified */
    if (-1 == npdfs)
        return;

    /*
     * Do not log applications coming from the following directories.  This
     * should save space in the log file.
     *
     * Reasons:
     * -------------------------------------------------------------------------
     *    $TAEPDF:      These PDFs are used to initialize TAE and we expect
     *                  them to be run.
     *    /LOCAL/:      These are internal process PDFs and we really only need
     *                  to know about the parent PDF.
     */
    if ((strcmp(fsblock->libr, "$TAEPDF") == 0) ||
        (strcmp(fsblock->libr, "/LOCAL/") == 0))
        return;
    
    /*
     * Make local copies of the library where the PDF is coming from and of the
     * PDF name itself.
     */
    strcpy(dir, fsblock->libr);
    strcpy(pdf, fsblock->name);

    /*
     * Pdf has size zero if it is the ~/.lasrc file
     */
    if (strlen(pdf) == 0)
        return;

    /*
     * Copy the library directory to a local variable and ensure the last
     * character is not a '/'.  This is just to be consistent in the way all
     * directories are reported.
     */
    i = strlen(dir);
    if (dir[i - 1] == '/')
        dir[i - 1] = '\0';


    /*
     * Force the PDF name to lowercase.  At the EROS Data Center we force all
     * PDF names to lowercase since we started writing applications on a VMS
     * system and programmers were able mix upper and lowercase.
     */
    s_lower(pdf);
    
    /*
     * Do not log those PDFs indicated in the static array pdfnames.
     */
    for (i=0; i<npdfs; i++)
    {
        if (strcmp(pdf, pdfnames[i]) == 0)
            return;
    }

    /*
     * Do not log the PDF called by slogon used to generate the TAE env.
     * Do not log PDFs that start with a digit. These are generally
     * unique PDF names generated with a session id.
     */
    if (strncmp(pdf,"lassetup",8) == 0)
        return;
    if (isdigit(pdf[0]))
       return;

    /*
     * Modify dir so it isn't relative to the user's current directory
     */
    if (strcmp(dir, ".") == 0)
    {
        if (getcwd(dir, MAXSTRSIZ) == NULL)
        {
            perror("taetm pdflog()");
            m_put("Error obtaining the current directory %s %s %s %s %s",
                "TAE-LOGFILE", "", "", "", "", "");
            strcpy(dir, ".");
        }
    }
    else if (strncmp(dir, "./", 2) == 0)
    {
        /*
         * Save the portion that is relative to the current directory so it can
         * be appended to the end of the current directory later.
         */
        strcpy(tmp, dir);

        /*
         * Get the current directory.  If there is an error, restore the
         * directory to what it was.  If there isn't an error, generate a full
         * directory specification.
         */
        if (getcwd(dir, MAXSTRSIZ) == NULL)
        {
            perror("taetm pdflog()");
            m_put("Error obtaining the current directory %s %s %s %s %s",
                "TAE-LOGFILE", "", "", "", "", "");
            strcpy(dir, tmp);
        }
        else
        {
            strcat(dir, "/");
            strcat(dir, (tmp+2));
        }
    }

    /*
     * Do this once more. This will allow us to not log those PDFs 
       from specified directories.  It's a hack but it works.
     */
    for (i=0; i<npdfs; i++)
    {
        if (strcmp(dir, pdfnames[i]) == 0)
            return;
    }

    /*
     * Open the file for write access.
     * NOTES:
     *    - File locking doesn't work with append access.  The work around is
     *      to open the file for read/write access and seek to EOF.  This also
     *      enables us to determine if the process should change the
     *      permissions, owner, and group of the file.
     *    - Opening the file with 666 access is still subject to the user's
     *      umask.  Therefore, it is necessary to explicitly change the
     *      file permissions later.
     */
    access = O_RDWR | O_CREAT;
    fd = open(logfile, access, 0666);
    if (fd == -1)
    {
        perror("taetm pdflog()");
        m_put("Error opening the log file %s %s %s %s %s",
            "TAE-LOGFILE", "", "", "", "", "");
        return;
    }


    /*
     * Obtain a lock on the file.  Note that F_SETLKW will cause the process to
     * block until a lock is obtained.  F_SETLK will not block.
     */
    lock_info.l_type   = F_WRLCK;   /* Type of lock F_RDLCK, F_WRLCK, F_UNLCK */
    lock_info.l_whence = SEEK_SET;  /* Flag for starting offset               */
    lock_info.l_start  = 0;         /* Relative offset in bytes               */
    lock_info.l_len    = 0;         /* Size; if 0 then until EOF              */
    if (fcntl(fd, F_SETLKW, &lock_info) == -1)
    {
        perror("taetm pdflog()");
        m_put("Error locking the log file %s %s %s %s %s",
            "TAE-LOGFILE", "", "", "", "", "");
        close(fd);
        return;
    }


    /*
     * Seek to the end of the file
     */
    if ((size = lseek(fd, 0, SEEK_END)) == -1)
    {
        perror("taetm pdflog()");
        m_put("Error seeking in the log file %s %s %s %s %s",
            "TAE-LOGFILE", "", "", "", "", "");
        close(fd);
        return;
    }


    /*
     * Create a string and write it to the file
     */
    sprintf(buf, "%-15.15s %-10.10s %02d/%02d/%04d %02d:%02d %-10.10s %s\n",
        pdf, username, date->tm_mon + 1, date->tm_mday, date->tm_year + 1900,
        date->tm_hour, date->tm_min, hostname, dir);
    nbytes = strlen(buf);
    if (write(fd, buf, nbytes) != nbytes)
    {
        perror("taetm pdflog()");
        m_put("Error writing to the log file %s %s %s %s %s",
            "TAE-LOGFILE", "", "", "", "", "");
        close(fd);
        return;
    }


    /*
     * Change the access permissions, owner, and group of the file if it was
     * created by this process.
     */
    if (size == 0)
    {
        fchmod(fd, 0666);
        fchown(fd, UID, GID);
    }


    /*
     * Unlock the file
     */
    lock_info.l_type   = F_UNLCK;   /* Type of lock F_RDLCK, F_WRLCK, F_UNLCK */
    lock_info.l_whence = SEEK_SET;  /* Flag for starting offset               */
    lock_info.l_start  = 0;         /* Relative offset in bytes               */
    lock_info.l_len    = 0;         /* Size; if 0 then until EOF              */
    if (fcntl(fd, F_SETLKW, &lock_info) == -1)
    {
        perror("taetm pdflog()");
        m_put("Error unlocking the log file %s %s %s %s %s",
            "TAE-LOGFILE", "", "", "", "", "");
    }


    /*
     * Close the file
     */
    if (close(fd))
    {
        perror("taetm pdflog()");
        m_put("Error closing the log file %s %s %s %s %s",
            "TAE-LOGFILE", "", "", "", "", "");
    }
}


FUNCTION static void get_pdfnames (void)
{
    char *ptr;                  /* Ptr to a string                            */
    char line[1024];            /* One line from the file                     */
    char name[MAXSTRSIZ];       /* File name for $LASTABLES/DO_NOT_LOG        */
    char eoh[81];               /* End Of Header string (80 '-' characters)   */

    long i;                     /* Loop control                               */
    long count;                 /* Loop control                               */

    short done;                 /* Loop control                               */

    FILE *fp;                   /* File ptr                                   */


    /*
     * Retrieve the environment variable used for logging
     */
    ptr = getenv("LASTABLES");
    if (ptr == NULL)
    {
        npdfs = 0;
        return;
    }
    

    /*
     * Open the file containing PDFs that are not to be logged.
     * If it cannnot be opened, do not log anything.  This will
     * also allow us to disable logging if needed.
     */
    sprintf(name, "%s/%s", ptr, DO_NOT_LOG);
    fp = fopen(name, "r");
    if (fp == NULL)
    {
        npdfs = -1;
        return;
    }

    /*
     * Initialize the string that indicates the end of the header
     */
    for (i=0; i<80; i++)
        eoh[i] = '-';
    eoh[i] = '\0';

    /*
     * Skip past the header.
     */
    done = FALSE;
    while (!done)
    {
        /*
         * If we encountered EOF, there must not be any PDFs specified in the
         * file.  Close the file and return.
         */
        if (fgets(line, 1024, fp) == NULL)
        {
            npdfs = 0;
            fclose(fp);
            return;
        }

        /*
         * Remove the trailing '\n' from the string that was read
         */
        i = strlen(line);
        if (line[i-1] == '\n')
            line[i-1] = '\0';

        /*
         * Determine if the 80 '-' characters have been read
         */
        if (strcmp(line, eoh) == 0)
            done = TRUE;
    }

    /*
     * Read the names of those PDFs that are not to be logged.
     */
    npdfs = 0;
    while (fgets(line, 1024, fp) != NULL)
    {
        /*
         * Remove the trailing '\n' from the string that was read
         */
        i = strlen(line);
        if (line[i-1] == '\n')
            line[i-1] = '\0';

        /*
         * Ensure the PDF name isn't too long
         */
        if (strlen(line) > PDFLEN)
        {
            m_put("PDF name in $LASTABLES/%s exceeds %d characters %s %s %s",
                "TAE-LOGFILE", DO_NOT_LOG, PDFLEN, "", "", "");
        }
        else
            sscanf(line, "%s", pdfnames[npdfs++]);
    }

    /*
     * Close the file and return
     */
    fclose(fp);

    /* 
     * Go through the list one time and expand all env variables and
     * add them to the list.
     */ 
    count = npdfs; 
    for (i = 0; i < count; i++)
    {
        if (strncmp(pdfnames[i],"$",1) == 0)
       {
           ptr = getenv(&pdfnames[i][1]);
           if (ptr != NULL)
              strcpy(pdfnames[npdfs++],ptr);
       }
    }
}


#endif
