/****************************************************************************
 *      Copyright (c) 1993, 1994
 *      Century Computing, Inc.
 *      ALL RIGHTS RESERVED
 *
 *      The software (programs, data bases and/or documentation) on or in
 *      any media can not be reproduced, disclosed, or used except under
 *      the terms of the TAE Plus Software License Agreement.
 *
 ***************************************************************************/

/*
 * This file contains functions to manipulate tutor displays,
 * both in screen and noscreen mode.
 *
 * This file is extracted from tutor.c during the tutor.c breakup of 6-dec-84.
 * See full explanation in tutor.c.
 *
 * The functions in this source file are in alphabetical order.
 *
 * CHANGE LOG:
 *
 *      10-dec-84       TCL 67: parm screen qual notice no descr check...peb
 *      19-jul-85       Fix bug in left_pbld calling sequence...dm
 *      19-sep-85       PR 1012: Change high_value to display blanks when
 *                      NAME parm does'nt have current value...dab
 *      31-jul-86       Added code for the TAE_FACELIFT...krw
 *      11-feb-87       Merged TAE FACELIFT with V1.4 TAE...krw
 *
 ********************* Post TAE 2.1 changes *******************************
 *
 *      19-may-87       Updated for TAE-Plus philosophy. Checked global 
 *                      variable DisplayId to determine VT100 or Window 
 *                      mode screen operations...dm
 *      12-aug-87       Fix "show x.q" and "page x.q" and "x.q = value":
 *                      they crash tutor in getpgnum...tpl
 *      02-dec-87       Declare DisplayId as GENPTR...dm
 *      03-feb-88       Changed DisplayId to XFACELIFT...tpl
 *      24-may-88       Fix incomplete screen update with up or douwm arrow
 *                      upon tutor entry of paging beyond last page...hsk
 *      08-dec-88       Honor COMPRESS_TUTOR...palm
 *      29-jan-89       Truncate parm name display to 8 chars (even tho
 *                      NAMESIZ will soon be larger than 8). This so
 *                      that the tutor screen design is the same...palm
 *      06-feb-89       Fix bug in above when highlighting...palm 
 *      12-jun-89       Removed TAE_FACELIFT...ljn
 *      27-jun-90       Removed Facelift code...ljn
 *      20-may-91       fix longstanding crash with "tutor save-",
 *                      where you enter subcommand mode first; this
 *                      bug was apparently masked by architectures other
 *                      than Sun4 (but what about VAX?)...palm
 *      22-oct-92       Prototyping tae_alloc is unnecessary and Ultrix 4.3
 *                      does not like it...rt
 */

#include <string.h>
#include "taeconf.inp"           /* TAE configuration definitions        */
#include "inttaelib.h"
#include "tmhost.inp"            /* host-dependent defs                  */
#include "symtab.inc"            /* symbol table                         */
#include "tminc.inc"             /* TM-only host-independent definitions */
#include "dirinc.inc"            /* includes for d_ package              */
#include "terminc.inc"
#include "vmcoproto.h"
#include "taelib.h"
#include "local_protos.h"

IMPORT  struct TUTCTX tutctx;       /* tutor & micro-editor context */

static TEXT msg_mmov[] = "Terminal Monitor internal memory overflow.";
static TEXT key_mmov[] = "TAE-MEMOVR";      /* dyn mem overflow in tutor*/
static TEXT msg_ambi[] = "Ambiguous parameter abbreviation, '%s'.";
static TEXT key_ambi[] = "TAE-AMBIGPAR";
static TEXT msg_unre[] = "Unrecognized parameter name, '%s'.";
static TEXT key_unre[] = "TAE-TUNRECPAR";
static TEXT msg_ambs[] = "Ambiguous subcommand abbreviation, '%s'.";
static TEXT key_ambs[] = "TAE-AMBIGSUB";
static TEXT msg_unrs[] = "Unrecognized subcommand, '%s'.";
static TEXT key_unrs[] = "TAE-UNRECSUB";
static TEXT nodescr_notice[] = "Description not available";

FUNCTION static COUNT getbot 
(
    struct VARIABLE *v      /* in:  var containing the panel        */
);
FUNCTION static VOID listpqnam 
(
    struct SYMTAB *symtab   /* in:  symbol table containing parms   */
);
FUNCTION static VOID disppag 
(
    struct CONTXT  *pctx,   /* in:  proc context                    */
    FUNINT         pagnum,  /* in:  page # to display               */
    struct TXTSTOR *title   /* in:  title text                      */
);

/* adjtop - adjust the value panel top & bottom component numbers.
 * Uses the supplied component number as the new bottom component
 * number and adjusts the panel top to fit the display.
 * Assumes the value panel position array is up to date.
 * This function should not be called except for string parameters, and
 * then only if a component is being updated that will overflow the
 * currently displayed value panel.
 */

FUNCTION static VOID adjtop 
(
    struct VARIABLE *v,     /* in/out: var containing panel to be adjusted*/
    FUNINT          compnum /* in:  component # of new panel bottom */
)
{
    struct TUTEXT       *t;

    t = (*v).v_tp;
    if (t == NULL)    /* to protect against unallocated tutor extension*/
        return;
    for ((*t).pantop = 1, (*t).panbot = getbot(v);
            (*t).panbot < (*v).v_maxc;
            (*t).pantop++, (*t).panbot = getbot(v))
    {
        if ((*t).panbot >= compnum)     /* if we've found the right top */
            break;
    }
}

/* adjval - adjust the value panel top & bottom component numbers.
 * Assumes the value panel position array is up to date.
 * Puts the requested component at the top of the panel with one
 * exception.  If the requested component is close enough to the
 * highest possible component number (.v_maxc) that this highest
 * component can fit on the panel and still allow components above the
 * requested component, the lower numbered components are displayed
 * above the requested component to the extent possible.
 */

FUNCTION VOID adjval 
(
    struct VARIABLE *v,     /* in/out: var containing panel to be adjusted*/
    FUNINT          compnum /* in:  component # of new panel top    */
)
{
    struct TUTEXT       *t;

    t = (*v).v_tp;
    if (t == NULL) /* to protect against unallocated tutor extension*/
        return;
    if ((*v).v_type == V_NAME)
        (*t).pantop = (*t).panbot = 1;
    else
    {
        for ((*t).pantop = 1, (*t).panbot = getbot(v);
                (*t).panbot < (*v).v_maxc;
                (*t).pantop++, (*t).panbot = getbot(v))
        {
            if ((*t).pantop == compnum)
                break;
        }
    }
}

/* dispbld - display a specific tutor page.  Do help reads & data storage as
 * necessary.
 *
 * NOTE:  For screen tutor, the page number to parameter correspondence
 *        is assigned here, and is permanent for this tutor session.
 *        Each parameter is allowed enough space on its page to allow for
 *        all level one help text, and for the parameter value to grow
 *        to its maximum possible length.  For string type parameters,
 *        this maximum possible length must allow for the maximum possible
 *        string length for each component of the parameter value.
 *        If, after the maximum possible display length for the parameter
 *        length for the parameter is calculated, there is room remaining,
 *        tutor attempts to fit the next parameter on the same page, and
 *        so on.  No parameter is allowed more than one full tutor page.
 *        The value panel concept, and value panel scrolling, are used
 *        to make the entire set of value components available to the user.
 *
 *        One result of the above algorithm is that after the tutor data
 *        structures have been built for the particular proc being tutored
 *        on, tutor code can assume that any parameter whose value display
 *        might overflow the available space is alone on the tutor page.
 *
 *  NOTE:  Under TAE-Plus, we check if TM is initialized to talk
 *        to the X Window server (implying that it is using a graphics
 *        display device as its screen). In that case we call the routine
 *        DispTutWin() to display tutor page in a window. Otherwise, we
 *        display the page in regular VT100 simulated mode as described
 *        above.
 *
 */

FUNCTION CODE dispbld 
(
    struct CONTXT  *pctx,   /* in/out: proc context                 */
    struct TXTSTOR *title,  /* in:  .TITLE text                     */
    struct SFILE   *hf      /* in/out: help file control block      */
)
{
    struct VARIABLE *v;             /* variable structure                   */
    struct SUBCMD   *s;
    struct TUTEXT   *t=NULL;        /* tutor extension                      */
    COUNT           lvarpg;         /* page # of last variable or subc      */
    CODE            code;

    /* If TAE is initialized to use X Windows, assume the entire Tutor
     * display is a big page, containing all the parameters. We position to
     * the appropriate place through scroll.
     */

    code   = SUCCESS;
    lvarpg = 0;
    if (!tutctx.srch1cplt)  /* if search not yet complete for lev 1 data*/
    {
        if (!tutctx.subtut) /* if tutoring on parms                 */
        {
            for (v = (*pctx).parmst.link; v != NULL; v = (*v).v_link)
            {
                t = (*v).v_tp;         /* point to tutor extension     */
                /* if parm not yet integrated in tutor disp*/
                if (t == NULL  ||  (*t).pagnum == 0)    
                {
                    /* read in lev 1 help for parm & remain parms on pg*/
                    code = bldvhlp(pctx, v, lvarpg, title, hf); 
                    t = (*v).v_tp;
                }
                /* if this parm is on right page        */
                if ((*t).pagnum == tutctx.curpag)
                    break;
                lvarpg = (*t).pagnum;
            }
            /* end of symbol table.  Last page defined*/
            if (v == NULL)
            {
                /* always at least 1 pg  */
                tutctx.lastpag = (lvarpg == 0)  ?  1 : lvarpg; 
                tutctx.srch1cplt = TRUE;
                tutctx.curpag = tutctx.lastpag;
            }
            else if ((*v).v_link == NULL)
            {
                tutctx.lastpag   = (*t).pagnum;
                tutctx.srch1cplt = TRUE;
            }
        }
        else    /* tutoring on subcommands      */
        {
            for (s = (*pctx).subptr; s != NULL; s = (*s).link)
            {
                /* point to tutor extension     */
                t = &(*s).ext;
                /* if parm not yet integrated in tutor disp*/
                if (t == NULL  ||  (*t).pagnum == 0)
                {
                    /* read in lev 1 help for parm & remain parms on pg*/
                    code = bldshlp(pctx, s, lvarpg, title, hf); 
                    t = &(*s).ext;
                }
                /* if this parm is on right page        */
                if ((*t).pagnum == tutctx.curpag)
                    break;
                lvarpg = (*t).pagnum;
            }
            if (s == NULL)     /* end of symbol table.  Last page defined*/
            {
                /* always at least 1 pg  */
                tutctx.lastpag = (lvarpg == 0)  ?  1 : lvarpg; 
                tutctx.srch1cplt = TRUE;
                tutctx.curpag    = tutctx.lastpag;
            }
            else if ((*s).link == NULL)
            {
                tutctx.lastpag = (*t).pagnum;
                tutctx.srch1cplt = TRUE;
            }
        }   
    }
    else
        if (tutctx.curpag > tutctx.lastpag)
            tutctx.curpag = tutctx.lastpag;
    if (tutctx.dispreq)
    {
        disppag(pctx, tutctx.curpag, title);   /* display the page      */
        tutctx.dispreq = FALSE;
    }
    return(code);
}

/* dspl1hlp - display level 1 help for NOSCREEN tutor given a
 * valid tutor extension.
 */

FUNCTION static VOID dspl1hlp 
(
    struct TUTEXT *t        /* in:  tutor extension                 */
)
{
    struct TXTSTOR      *block;
    TEXT                buf[STRINGSIZ+1];
    TEXT                **str;
    COUNT               i;

    if ((*t).l1hexist)
    {
        block = &(*t).l1help;
        str   = (*block).tp;
        for (i = 0; i < (*block).numline; i++)
        {
            s_blank(buf, 4);
            strcat(buf,str[i]);
            t_write(buf, T_STDCC);
        }
    }
    else
    {
        strcpy(buf,"    ");
        strcat(buf,nodescr_notice);
        t_write (buf, T_STDCC);
    }
}

/*      get_parm1.   Get the first parameter of a given page.
 */
FUNCTION struct VARIABLE *get_parm1 
(
    struct CONTXT *pctx,    /* in: proc context             */
    FUNINT        page,     /* in: page number              */
    struct SFILE  *hf       /* in: help file                */
)
{
    struct VARIABLE *v, *savev;
    COUNT vpage, savepage;

    vpage = -1;                 /* in case getpgnum returns FAIL */
    savev = (*pctx).parmst.link;
    for (v=(*pctx).parmst.link; v != NULL; v=(*v).v_link)
    {                                       
        savepage = vpage;
        getpgnum (pctx, (*v).v_name, &vpage, hf);       
        if (vpage == page)
            return (v);
        if (savepage != vpage)
            savev = v;
    }
     
    /* End of symbol table. Reaches this point when user provided page # */
    /* (page n) is greater than the lastpage of parms for the first time */
/*    if (v == NULL)*/
    {
        tutctx.lastpag = (vpage == 0) ? 1 : vpage; /* always at least 1 pg */
        tutctx.srch1cplt = TRUE;  /* since pagnum is assigned to last parm */ 
        tutctx.curpag = tutctx.lastpag;
        return (savev);
    }
}  

/* getbot - return the bottom component number of a tutor value panel.
 * Assumes the panel top specifier and the panel posit array are up to date.
 * For string type, uses only the space required for the current value of
 * each string (i.e., does not allow for growth to the max possible length
 * of the last component).
 */

FUNCTION static COUNT getbot 
(
    struct VARIABLE *v      /* in:  var containing the panel        */
)
{
    IMPORT COUNT        termlines;      /* number of lines on CRT */

    COUNT               i;
    TINY                *nlin;
    COUNT               begline, endline;
    COUNT               maxline;
    COUNT               sofar;
    struct TUTEXT       *t;

    if ((*v).v_type == V_NAME)
        return(1);
    t      = (*v).v_tp;
    nlin   = (*t).compnlin;
    begline = tutctx.pageStart + (*t).startlin;
    if (tutctx.spaceLines)                      /* maintain bug: field compat */
        endline = termlines  - (ERRLIN + 2);    /* keyin+prompt+error */
    else                                        /* compress mode: do right */ 
        endline = termlines - (ERRLIN + 1);     /* keyin+prompt+error */
    maxline = endline - begline + 1;
    sofar   = 0;
    /* for each component   */
    for (i = (*t).pantop, nlin += i-1; i <= (*v).v_maxc; i++, nlin++)  
    {
        if (sofar + *nlin > maxline)
            return(i-1);
        sofar += *nlin;      /* add number lines for this component  */
    }
    return(i-1);
}

/* getoffs - get the offset in display lines of this component
 *           from the top of the display panel.
 * Assumes the panel position array and the panel top specifier are
 * up to date.
 */

FUNCTION static COUNT getoffs 
(
    struct VARIABLE *v,     /* in:  variable containing the panel   */
    FUNINT          compnum /* in:  component number to find offset of*/
)
{
    TINY                *nlin;
    struct TUTEXT       *t;
    COUNT               offset;
    COUNT               i;

    t = (*v).v_tp;
    nlin = (*t).compnlin;
    offset = 0;
    for (i = (*t).pantop, nlin += i-1; i < compnum; i++, nlin++)
        offset += *nlin;
    return(offset);
}

/* getpgnum - get level 1 display page number of a proc parameter.
 * Performs help reads & page numbering of unencountered parms as necessary.
 * Returns FAIL if parmname is not in the parameter symbol table,
 * or we ran out of dynamic memory.
 */

FUNCTION CODE getpgnum 
(
    struct CONTXT *pctx,          /* in/out: proc context to find parm in */
    TEXT          parmname[],     /* in:  proc parameter name             */
    COUNT         *pagnum,        /* out: display page number             */
    struct SFILE  *hf             /* in/out: help file control block      */
)
{
    struct VARIABLE     *v;
    struct TUTEXT       *t=NULL;
    COUNT               lvarpg;
    CODE                code;

    if ((code = lookab(&(*pctx).parmst, parmname, &v)) != SUCCESS) 
        goto parm_err;
    if (s_index (parmname, '.')  >= 0)          /* qualified ?        */
        goto parm_err;
    parmname = (*v).v_name;                     /* use full spelling  */
    lvarpg = 0;
    for (v = (*pctx).parmst.link; v != NULL; v = (*v).v_link)
    {
        t = (*v).v_tp;
        /* if parm not yet integrated in tutor display*/
        if (t == NULL  ||  (*t).pagnum == 0)
        {
            if (bldvhlp(pctx, v, lvarpg, &tutctx.title, hf) != SUCCESS)
                                        /*integrat parm & remain parms on pg*/
                return(FAIL);
            t = (*v).v_tp;
        }
        if (s_equal (parmname, (*v).v_name))
            break;
        lvarpg = (*t).pagnum;
    }
    *pagnum = (*t).pagnum;
    return(SUCCESS);

parm_err:
    if (code == AMBIG)
    {
        /* ambiguous parameter abbreviation     */
        tutmsg(msg_ambi, key_ambi, parmname);
    }
    else
    {
        /* unrecognized parameter               */
        tutmsg(msg_unre, key_unre, parmname);
    }
    return(FAIL);
}

/* getspnum - get level 1 display page number of a proc subcommand.
 * Performs help reads & page numbering of unencountered subcs as necessary.
 * Returns FAIL if subcommand name is not in the subcommand table,
 * or we ran out of dynamic memory.
 */

FUNCTION CODE getspnum 
(
    struct CONTXT *pctx,    /* in/out: proc context to find subc in */
    TEXT          subname[],/* in:  subcmd name                     */
    COUNT         *pagnum,  /* out: display page number             */
    struct SFILE  *hf       /* in/out: help file control block      */
)
{
    struct SUBCMD       *s;
    struct TUTEXT       *t;
    COUNT               lsubpg;
    CODE                code;

    if ((code = subab((*pctx).subptr, subname, &s)) != SUCCESS) 
        goto subc_err;
    lsubpg = 0;
    for (s = (*pctx).subptr; ; s = (*s).link)
    {
        t = &(*s).ext;
        /* if subc not yet integrated in tutor display*/
        if (t == NULL  ||  (*t).pagnum == 0)
        {
            if (bldshlp(pctx, s, lsubpg, &tutctx.title, hf) != SUCCESS)
                                        /*integrat subc & remain subcs on pg*/
                return(FAIL);
            t = &(*s).ext;
        }
        if (s_lseq(subname, (*s).name))
            break;
        lsubpg = (*t).pagnum;
    }
    *pagnum = (*t).pagnum;
    return(SUCCESS);

subc_err:
    if (code == AMBIG)
    {
        /* ambiguous subcommand abbreviation    */
        tutmsg(msg_ambs, key_ambs, subname);    
    }
    else
    {
        /* unrecognized subcommand              */
        tutmsg(msg_unrs, key_unrs, subname);
    }
    return(FAIL);
}

/*      high_value.    Highlight (or un-highlight)
 *      a parameter name and one component's value.
 *      Parameter must currently be displayed.
 */

FUNCTION VOID high_value 
(
    struct VARIABLE *v,           /* in: parameter to highlight   */
    FUNINT          index,        /* in: index to highlight       */
    BOOL            highlight     /* in: TRUE to highlight;       */
                                  /*     FALSE to turn off        */
)
{
    struct TXTSTOR      txt;                    /* text storage block   */
    COUNT               comp_line, start_line;
    COUNT               i;
    struct TUTEXT       *t;
    CODE                code;
    TEXT                string [STRINGSIZ+1];
    BOOL                nonblank;       /* blanks or value to be displayed ? */
    TEXT                nameString[STRINGSIZ+1];

    t = (*v).v_tp;              /* tutor extension ptr          */
    initxt (&txt);              /* init text storage block      */
    start_line = tutctx.pageStart + (*t).startlin;
    if (highlight)
    {
        s_bcopy ((*v).v_name, nameString, 8);   /* tutor displays 8 max */
        high_lfield (nameString, 8, string);
    }
    else
    {
        i = s_bcopy ((*v).v_name, string, 8);   /* tutor displays 8 max */
        s_blank (&string[i], 8-i);
    }       
    t_output (start_line, TUTNAMECOL, string);
    comp_line = getoffs (v, index+1)  +  start_line;
    code = addcomp (&txt, v, index+1);
    if (code == SUCCESS)                /* (if no memory, do nothing)   */
    {
        if ((*v).v_type == V_NAME)      /* Any current reference value? */
            nonblank = (*v).v_ref != NULL ? TRUE : FALSE;
        else                            /* Any current explicit value?  */
            nonblank = (index < (*v).v_count)  ||
                       ((*v).v_count == 0 && index == 0) ? TRUE : FALSE;
        if (nonblank)
        {                                           /* non-blank field: */
            if ((*v).v_type == V_STRING)
                hdisptxt (comp_line, TUTVALCOL, &txt, TUTVALSIZ, highlight);
            else
                hdisprtxt (comp_line, TUTVALCOL+TUTNUMRTCOL, 
                           &txt, TUTNUMRTCOL-1, highlight);
        }
        else
        {                                                   
            s_blank (string, TUTVALSIZ);                /* no value...  */
            if (highlight)
                t_highlight (string);
            t_output (comp_line, TUTVALCOL, string);    /* so blank it  */
        }
    }
    fretxt (&txt);
}

/* h1dsp_parm - display level 1 help for a parm.
 */

FUNCTION VOID h1dsp_parm 
(
    struct CONTXT   *ctx,   /* in:  proc context                    */
    struct VARIABLE *v,     /* variable to display help on          */
    struct SFILE    *hf     /* in:  help file                       */
)
{
    COUNT          pagnum;         /* dummy to receive page #              */
    CODE           code;
    struct TUTEXT  *t;

    /* make sure help reads done for this parm*/
    code = getpgnum(ctx, (*v).v_name, &pagnum, hf);
    if (code != SUCCESS) 
        return;
    t = (*v).v_tp;
    /* display lev 1 help   */
    dspl1hlp(t);
}

/*      h1dsp_subc - display level 1 help for a subcommand.
 */

FUNCTION COUNT h1dsp_subc 
(
    struct CONTXT *ctx,     /* in:  proc context             */
    struct SUBCMD *s,       /* subcommand to display help on */
    struct SFILE  *hf       /* in:  help file                */
)
{
    COUNT         pagnum;         /* dummy to receive page #              */
    CODE          code;
    struct TUTEXT *t;

    /* make sure help reads done for this subc*/
    code = getspnum(ctx, (*s).name, &pagnum, hf); 
    if (code != SUCCESS) 
        return(FAIL);
    t = &(*s).ext;
    /* display lev 1 help   */
    dspl1hlp(t);
    return(SUCCESS);
}

/* inipan - initialize tutor value panel position array.
 * Allocates the panel position array if necessary.
 * If the tutor extension for this variable hasn't yet been allocated
 * (i.e., if this variable has never been displayed and isn't now being
 * set up for display) then this function just returns.
 */

FUNCTION CODE inipan 
(
    struct VARIABLE *v      /* in/out: variable containing the panel*/
)
{
    struct TUTEXT       *t;
    TINY                *nlin;
    COUNT               i;
    struct TXTSTOR      txt;
    COUNT               maxcount;
    CODE                type;

    if ((t = (*v).v_tp) == NULL)
        return(SUCCESS);
    type = (*v).v_type;
    maxcount = (type == V_NAME)  ?  1 : (*v).v_maxc;
    /* if posit array not yet allocated     */
    if ((nlin = (*t).compnlin) == NULL)
    {
        /* alloc the posti array*/
        nlin = (*t).compnlin = (TINY *)tae_alloc(maxcount, sizeof(TINY));
        if (nlin == NULL)
            return(FAIL);
    }
    if (type == V_NAME)
        *nlin = 1;
    else
    {
        for (i = 1; i <= maxcount; i++, nlin++)
        {
            if (i <= (*v).v_count)
            {
                /* init the text struct         */
                initxt(&txt);
                if (addcomp(&txt, v, i) != SUCCESS)
                    return(FAIL);
                *nlin = (TINY)txt.numline;
                fretxt(&txt);
            }
            else
                *nlin = 1;
        }
    }
    return(SUCCESS);
}

/* list_parm - list a parm and it's value.
 */

FUNCTION VOID list_parm 
(
    struct VARIABLE *v      /* in:  variable to list                */
)
{
    IMPORT COUNT        termcols;

    TEXT                buf[STRINGSIZ+1];
    COUNT               i;
    CODE                code;
    COUNT               length;
    CODE                type;
    struct VARIABLE     *vref;

    IMPORT TEXT         qual_notice[];

    buf[0] = EOS;                               /* initialize   */
    type   = (*v).v_type;
    m_fpname((*v).v_name, buf, STRINGSIZ);      /* get name     */
    if (type == V_NAME)
    {
        vref = (*v).v_ref;
        if (vref == NULL)
            strcat(buf," (no reference)");
        else
        {
            if ((*v).v_default)                 /* ref local to PDF     */
                strcat(buf,"+");
            strcat(buf,(*vref).v_name);
        }
    }
    else
    {
        for (i = 0; i < (*v).v_count; i++)              /* for each component */
        {
            /* put in buffer */
            code = m_fpval(v, i, buf, termcols, STRINGSIZ);
            if (code == FAIL)                           /* no more room */
            {
                t_write(buf, T_STDCC);                  /* write the line */
                buf[0] = EOS;                   
                /* ignore last increment        */
                --i;
            }
        }
        if ((*v).v_count > 0)
        {
            length = strlen(buf);
            buf[length-2] = EOS;  /* delete last ','      */
        }
        else if ((*v).v_count < 0)
            strcat(buf," (no value specified)");
        else if ((*v).v_count == 0)
            strcat(buf," --  (null value)");
    }
    /* final flush of output buffer */
    t_write(buf, T_STDCC);
    if (type != V_NAME)
    {
        /* if parm has qualifs */
        if ((*v).v_pv12  &&  (*v).v_qualst.link != NULL)
            t_write(qual_notice, T_STDCC);/* tell the user        */
    }
}

/*      list_preface.   Display caller-supplied preface.
 */

FUNCTION VOID list_preface 
(
    struct VARIABLE *p      /* in: strings to display       */
)
{
    COUNT       i;

    for (i=0; i < (*p).v_count; i++)
    {
        s_strip (SVAL(*p, i));  /* remove trailing blanks */
        t_write (SVAL(*p, i), T_STDCC);
    }
}

/*      list_subc - list a subcommand and whether or not it's the default.
 */

FUNCTION VOID list_subc 
(
    struct SUBCMD *s        /* in:  subcommand to list              */
)
{
    TEXT                buf[STRINGSIZ+1];

    buf[0] = EOS;                               /* initialize   */
    strcpy(buf,(*s).name);
    /* if this is the default subc  */
    if ((*s).deflt)
        strcat(buf,"  ** DEFAULT **");
    else
        strcat(buf,"  (not default)");
    /* final flush of output buffer */
    t_write(buf, T_STDCC);
}

/*      listpnam - list parameter names for NOSCREEN tutor.
 *      Also lists those parms which have defined qualifiers.
 */

FUNCTION VOID listpnam 
(
    struct SYMTAB *symtab   /* in:  symbol table containing parms   */
)
{
    struct VARIABLE     *v;
    TEXT                buf[STRINGSIZ+1];
    COUNT               cushion_len;
    COUNT               num_parm;
    COUNT               len;

    v = (*symtab).link;
    if (v == NULL)
        strcpy(buf,"Parameters requested: none.");
    else
    {
        cushion_len = STRINGSIZ - NAMESIZ - 5;
        strcpy(buf,"Parameters requested:  ");
        num_parm = 0;
        for ( ; v != NULL; v = (*v).v_link)
        {
            num_parm++;
            /* if last parm                 */
            if ((*v).v_link == NULL)
            {
                if (num_parm == 2)
                {
                    len = strlen(buf);
                    buf[len-2] = EOS;           /* strip trailing ", " */
                    strcat(buf," ");
                }
                if (num_parm > 1)
                    strcat(buf,"and ");
                strcat(buf,(*v).v_name);
                strcat(buf,".");
            }
            else
            {
                strcat(buf,(*v).v_name);
                strcat(buf,", ");
                if (strlen(buf) > cushion_len)
                {
                    strcat(buf,"...");
                    break;
                }
            }
        }
    }
    t_write(buf, T_STDCC);
    /* list those parms with def'd quals    */
    listpqnam (symtab);
}

/*      listpqnam - lists those parms which have defined qualifiers,
 *      for NOSCREEN tutor.
 */

FUNCTION static VOID listpqnam 
(
    struct SYMTAB *symtab   /* in:  symbol table containing parms   */
)
{
    struct VARIABLE     *v;
    TEXT                qual_buf[STRINGSIZ+1];
    TEXT                tmp_buf[STRINGSIZ+1];
    COUNT               cushion_len;
    COUNT               num_q_parm;     /* no. parms with def'd quals   */
    COUNT               len;
    COUNT               i;

    v = (*symtab).link;
    if (v == NULL)
        return;
    qual_buf[0] = EOS;
    cushion_len = STRINGSIZ - NAMESIZ - 5;
    num_q_parm = 0;     
    for ( ; v != NULL; v = (*v).v_link)
    {
        if (!(*v).v_pv12  ||  (*v).v_qualst.link == NULL)
            continue;
        num_q_parm++;
        if (num_q_parm == 1)
            strcpy(qual_buf,"Parameters with defined qualifiers:  ");
        strcat(qual_buf,(*v).v_name);
        strcat(qual_buf,", ");
        if (strlen(qual_buf) > cushion_len)
        {
            strcat(qual_buf,"...");
            break;
        }
    }
    if (NULLSTR(qual_buf))
        return;
    len = strlen (qual_buf);
    if (!s_equal (&qual_buf[len-3], "..."))
    {
        if (num_q_parm == 1)
        {
            qual_buf[len-2] = '.';
            qual_buf[len-1] = EOS;
        }
        else   /* there was more than 1 parm with quals*/
        {
            /* search backwards for last "," betw words*/
            for (i = len-3; i > 0; i--) 
            {
                if (qual_buf[i] == ',')
                {
                    /* last parm name to scratch buf*/
                    strcpy(tmp_buf,&qual_buf[i+2]);
                    len = strlen (tmp_buf);
                    /* close with "."       */
                    tmp_buf[len-2] = '.';
                    tmp_buf[len-1] = EOS;
                    if (num_q_parm == 2)
                        strcpy(&qual_buf[i]," and ");
                    else   /* more than 2 parms with quals*/
                        strcpy(&qual_buf[i],", and ");
                    strcat(qual_buf,tmp_buf);   /* put back last parm name*/
                    break;
                }
            }
        }
    }
    t_write (qual_buf, T_STDCC);
}

/*      listqnam - list names of qualifiers desired for a particular
 *      parameter in qualifier space of NOSCREEN tutor.
 */

FUNCTION VOID listqnam 
(
    struct SYMTAB *symtab,        /* in:  symbol table containing quals   */
    TEXT          parm_name[]     /* in:  name of parm being qualified    */
)
{
    struct VARIABLE     *v;     /* points to each qual (quals look like parms)*/
    TEXT                buf[STRINGSIZ+1];
    COUNT               cushion_len;
    COUNT               num_parm;
    COUNT               len;


    v = (*symtab).link;
    cushion_len = STRINGSIZ - NAMESIZ - 5;
    strcpy(buf,"'");
    strcat(buf,parm_name);
    strcat(buf,"' parameter qualifiers requested:  ");
    num_parm = 0;
    for ( ; v != NULL; v = (*v).v_link)
    {
        num_parm++;
        if ((*v).v_link == NULL)  /* if last parm                 */
        {
            if (num_parm == 2)
            {
                len = strlen(buf);
                buf[len-2] = EOS;  /* strip trailing ", "          */
                strcat(buf," ");
            }
            if (num_parm > 1)
                strcat(buf,"and ");
            strcat(buf,(*v).v_name);
            strcat(buf,".");
        }
        else
        {
            strcat(buf,(*v).v_name);
            strcat(buf,", ");
            if (strlen(buf) > cushion_len)
            {
                strcat(buf,"...");
                break;
            }
        }
    }
    t_write(buf, T_STDCC);
}

/* listsnam - list subcommand names for NOSCREEN tutor.
 */

FUNCTION VOID listsnam 
(
    struct SUBCMD *subptr   /* in:  subcommand chain                */
)
{
    struct SUBCMD       *s;
    TEXT                buf[STRINGSIZ+1];
    COUNT               cushion_len;
    COUNT               num_subc;
    COUNT               len;

    s = (*subptr).link;
    if (s == NULL)
        strcpy(buf,"No subcommands.");
    else
    {
        cushion_len = STRINGSIZ - SUBCMDSIZ - 5;
        strcpy(buf,"Subcommands:  ");
        num_subc = 0;
        for ( ; s != NULL; s = (*s).link)
        {
            num_subc++;
            if ((*s).link == NULL)              /* if last subc     */
            {
                if (num_subc == 2)
                {
                    len = strlen(buf);
                    buf[len-2] = EOS;           /* strip trailing ", " */
                    strcat(buf," ");
                }
                if (num_subc > 1)
                    strcat(buf,"and ");
                strcat(buf,(*s).name);
                strcat(buf,".");
            }
            else
            {
                strcat(buf,(*s).name);
                strcat(buf,", ");
                if (strlen(buf) > cushion_len)
                {
                    strcat(buf,"...");
                    break;
                }
            }
        }
    }
    t_write(buf, T_STDCC);
}

/* nlinval - return the max number of lines required for the components of
 * one parm value in a tutor display.
 */

FUNCTION COUNT nlinval 
(
    struct VARIABLE *v      /* in:  variable to fin # lines for     */
)
{
    COUNT               num;
    COUNT               mult;

    if ((*v).v_type == V_NAME)
        return(1);
    num = (*v).v_maxc;            /* should be 1 line per possible component */
    if ((*v).v_type == V_STRING)  /* unless oversize strings              */
    {
        mult = ( (COUNT) (*v).v_size - 1 + 2)   /* "+2" is for */
           / TUTVALSIZ + 1;                     /* surrounding quotes*/ 
        num *= mult;
    }
    return(num);
}

/* dispcnum - display component numbers for a value panel.
 * Assumes the panel position array is up to date.
 * Assumes the panels top component specifier is up to date.
 * Assumes the variable is not of type V_NAME.
 */

FUNCTION static VOID dispcnum 
(
    struct VARIABLE *v,     /* in:  variable containing value       */
    FUNINT          top,    /* in:  top component to be numbered    */
    FUNINT          bottom  /* in:  bottom component to be numbered */
)
{
    struct TUTEXT       *t;             /* tutor ext with panel posit array     */
    COUNT               i;
    TINY                *nlin;
    COUNT               line;
    TEXT                buf[STRINGSIZ+1];
    TEXT                tmpbuf[4];

    t    = (*v).v_tp;      /* point to the tutor extension         */
    line = tutctx.pageStart + (*t).startlin;
    nlin = (*t).compnlin;
    for (i = (*t).pantop, nlin += i-1; i <= bottom; i++, nlin++)
    {
        if (i >= top)      /* if we're supposed to number this component*/
        {
            strcpy(buf,"(");
            s_i2s(i, tmpbuf);
            strcat(buf,tmpbuf);
            strcat(buf,")");
            if (i == (*t).panbot)  /* if this is last component in panel   */
            {
                /* and if it's the last possible component*/
                if (i == (*v).v_maxc)
                    strcat(buf,".");
                else
                    strcat(buf,"+");
            }
            t_output(line, TUTCOMPNUMCOL, buf);
        }
        line += *nlin;   /* posit for next component             */
    }
}

/* dispcomp - display a value component.
 * Assumes this value is already on display.
 * This function updates .panbot, .compnlin.
 * If the component's new value uses the same number of display lines, just
 * redisplay the component.  If the number of lines changes, all the lower
 * components in the panel must be scrolled, including eliminating or adding
 * component(s) at the bottom of the panel.  If the component is for a string
 * parameter near the bottom of the displayed value panel, and the new value
 * causes the component to overflow the panel, the following algorithm is used:
 *
 *      - the updated component becomes the new bottom component of the
 *        value panel.
 *      - the panel top component is reset to a higher numbered component
 *        as necessary to allow room for the new bottom component value.
 *      - the entire value panel is redisplayed.
 *
 * If the component update involved fill, the filled components
 * whose component numbers are already on the screen get their new
 * values displayed.
 */

FUNCTION VOID dispcomp 
(
    struct VARIABLE *v,     /* in: the variable to display         */
    FUNINT          compnum,/* in: subscr (starting with 1) of the component*/
    FUNINT          fillcomp/*in: lowest #d comp that was filled (0 = no fill)*/
)
{
    IMPORT COUNT   termlines;      /* number of lines on CRT               */

    struct TXTSTOR comptxt;
    TINY           *nlin;          /* number of display lines              */
    COUNT          sline, soffset; /* starting line, offset for this component*/
    COUNT          valstlin, numclrln;
    CODE           type;
    BOOL           disp_done;
    struct TUTEXT  *t;
    COUNT          i;

    type = (*v).v_type;
    t = (*v).v_tp;
    valstlin = tutctx.pageStart + (*t).startlin;
    /* get starting displ offset for this component*/
    soffset  = getoffs(v, compnum);
    sline    = valstlin + soffset;
    numclrln = min( (int) (*t).linrange,
                 (int) (termlines-ERRLIN-tutctx.pageStart) ) - soffset;
    disp_done = FALSE;
    /* init component text structure        */
    initxt(&comptxt);
    /* point to number of lines array       */
    nlin = (*t).compnlin;
    /* point to element for this component  */
    nlin += compnum-1;
    if (addcomp(&comptxt, v, compnum) != SUCCESS) 
        goto over_err;        /* add one component text*/
    /* if number display lines hasn't changed*/
    if (comptxt.numline == *nlin)
    {
        /* clear the old component value*/
        clrmlin(sline, TUTVALCOL, *nlin);
        if (type != V_NAME  &&  (*v).v_maxc > 1)
            dispcnum(v, compnum, compnum);
    }
    else
    {
        *nlin = (TINY)comptxt.numline;
        (*t).panbot = getbot(v);     /* get new bottom component of panel*/
        if ((*t).panbot < compnum)   /* if new string overflowed panel*/
        {
            adjtop(v, compnum);      /* adj panel top to conform...  */
                                     /* to bigger bottom component   */
            dispval(v);              /* display whole value panel    */
            disp_done = TRUE;
        }
        else
        {
            /* for each lower component in panel*/
            for (i = compnum+1; i <= (*t).panbot; i++)
                if (addcomp(&comptxt, v, i) != SUCCESS) goto over_err;
            clrmlin(sline, TUTVALCOL, numclrln);
            if (type != V_NAME  &&  (*v).v_maxc > 1)
            {
                /* display component numbers    */
                dispcnum(v, compnum, (*t).panbot);
            }
        }
    }
    if (!disp_done)    /* if val not yet disp'd*/
    {
        if (type == V_STRING)
            disptxt(sline, TUTVALCOL, &comptxt);
        else
            disprtxt(sline, TUTVALCOL + TUTNUMRTCOL, &comptxt);
    }
    fretxt(&comptxt);
    if (fillcomp > 0) /* if there were filled components*/
    {
        /* for each filled comp on screen*/
        for (i = max(fillcomp, (*t).pantop); i < compnum; i++)  
        {
            soffset = getoffs(v, i);
            sline   = valstlin + soffset;
            initxt(&comptxt);
            if (addcomp(&comptxt, v, i) != SUCCESS) goto over_err;
            if (type == V_STRING)
                disptxt(sline, TUTVALCOL, &comptxt);
            else
                disprtxt(sline, TUTVALCOL + TUTNUMRTCOL, &comptxt);
            fretxt(&comptxt);
        }
    }
    return;

over_err:
    tutmsg(msg_mmov, key_mmov);  /* dynam mem overflow           */
}

/* disppag - display a page of tutor.
 * Assumes that all necessary level 1 help reads have been done,
 * and that the parms or subcommands for this page have been
 * properly page numbered.
 *
 */

FUNCTION static VOID disppag 
(
    struct CONTXT  *pctx,   /* in:  proc context                    */
    FUNINT         pagnum,  /* in:  page # to display               */
    struct TXTSTOR *title   /* in:  title text                      */
)
{
    IMPORT COUNT        termlines;  /* number lines on terminal screen      */
    IMPORT COUNT        termcols;

    struct VARIABLE     *v = NULL;
    struct SUBCMD       *s = NULL;
    struct TUTEXT       *t = NULL;
    COUNT               line;
    BOOL                lastflag;   /* TRUE if this is the last page        */
    COUNT               stlin;      /* starting screen line for vars or subcs*/
    TEXT                lefthead[STRINGSIZ+1];
    TEXT                string[STRINGSIZ+1];
    COUNT               col;
    TEXT                name[STRINGSIZ+1];
    TEXT                hdrtype[STRINGSIZ+1];
    static TEXT run_msg[] = "Type RUN to execute.";
    COUNT               l;
    COUNT               q_notice_line;

    IMPORT TEXT         qual_notice[];
    IMPORT      struct  VARIABLE *tutoropt_gbl;  /* tutor options global      */
    BOOL        no_tag, no_name, no_library;

    no_tag = search_vector ((*tutoropt_gbl).v_cvp, 
                                    (*tutoropt_gbl).v_count, "NO_TAG");
    no_name = search_vector ((*tutoropt_gbl).v_cvp, 
                                    (*tutoropt_gbl).v_count, "NO_NAME");
    no_library = search_vector ((*tutoropt_gbl).v_cvp, 
                                    (*tutoropt_gbl).v_count, "NO_LIBRARY");
    if (pagnum > tutctx.lastpag  &&  tutctx.srch1cplt)
        pagnum = tutctx.lastpag;
    lastflag = (pagnum == tutctx.lastpag) ? TRUE : FALSE;
    t_clear();   /* clear the screen                     */
    disptxt(tutctx.titleLine, 1, title);   /* display title text */
    line = tutctx.titleLine + (*title).numline + tutctx.spaceLines;
    if ((*pctx).intrinsic)
        strcpy(hdrtype,"command");
    else
        strcpy(hdrtype,"proc");
    if (tutctx.subtut)
    {
        left_fbld  (                            /* build left header         */
                   no_library ? "" : (*pctx).pdf.libr,
                   no_name ? "" : (*pctx).pdf.name,
                   "",                          /* subcommand not used */
                   hdrtype,                     /* header type */
                   lefthead);
        dsphdr (no_tag ? "" : "Tutor/Subcommands",  /* display header      */
                lefthead, pagnum, lastflag);   
        /* find 1st subc on pg*/
        for (s = (*pctx).subptr; s != NULL; s = (*s).link)
        {
            t = &(*s).ext;
            if ((*t).pagnum == pagnum)
                break;
        }
        strcpy(name,(*s).name);
    }
    else
    {
        if (tutctx.qualtut)
        {
            left_pbld ((*pctx).pdf.name, (*pctx).subcmd, tutctx.parmname,
                hdrtype, lefthead); /* build header name */
            /* display page header*/
            dsphdr("Tutor/Qualifiers", lefthead, pagnum, lastflag); 
        }
        else
        {
            left_fbld  (                                /* build left header */
                   no_library ? "" : (*pctx).pdf.libr,
                   no_name ? "" : (*pctx).pdf.name,
                   (*pctx).subcmd, hdrtype, lefthead);  /* build header name */
            dsphdr (no_tag ? "" : "Tutor",       /* display header      */
                    lefthead, pagnum, lastflag); /* display page header*/
        }
        /* find 1st var on pg*/
        for (v = (*pctx).parmst.link; v != NULL; v = (*v).v_link)       
        {
            t = (*v).v_tp;
            if ((*t).pagnum == pagnum)
                break;
        }
        if (v == NULL)     /* if proc has no parameters            */
        {
            line = ((termlines - PMTLIN) - 1 - line) / 2  +  line;
            strcpy(string,"Proc '");
            strcat(string,(*pctx).pdf.name);
            if ((*pctx).subcmd[0] != EOS)
            {
                strcat(string,"-");
                strcat(string,(*pctx).subcmd);
            }
            strcat(string,"' has no parameters.");
            l = strlen(string);
            t_output(line, (termcols-l)/2, string);
            t_output(line+2, (termcols-sizeof(run_msg))/2, run_msg);    
            return;
        }
        s_bcopy((*v).v_name, name, 8);  /* tutor only displays 8 chars !*/
    }
    /* display field labels      */
    dispflab(line, tutctx.subtut, tutctx.qualtut);
    line = tutctx.pageStart;
    stlin = line;
    col = (tutctx.subtut)  ?  TUTSUBDESCCOL : TUTVARDESCCOL;
    /* for each parm on page                */
    while ((*t).pagnum == pagnum)
    {
        t_output(line, TUTNAMECOL, name);
        if ((*t).l1hexist)
            disptxt(line, col, &(*t).l1help);   /* display help lev 1 text*/
        else if (v && !(*v).v_qualst.link)
            t_output(line, col, nodescr_notice);    /* no l1 help & no quals */
        if (tutctx.subtut)
        {
            if ((*s).deflt)
                t_output(line, TUTSUBDFLTCOL, "**DEFAULT**");
            line = stlin + (*t).linrange + tutctx.spaceLines;
            if ((s = (*s).link) == NULL)
                break;
            if ((t = &(*s).ext) == NULL)
                break;
            strcpy(name,(*s).name);
        }
        else if (v)                             /* safety check */
        {
            /* if parm has quals...*/
            if ((*v).v_pv12  &&  (*v).v_qualst.link != NULL) 
            {
                /*  if no level1 help, this msg replaces the "no descr avail" */
                q_notice_line = line + (*t).l1help.numline;
                t_output(q_notice_line, col, qual_notice);
            }
            dispval(v);         /* display parm value           */
            line = stlin + (*t).linrange + tutctx.spaceLines;
            if ((v = (*v).v_link) == NULL)      /* point to next parm */
                break;
            if ((t = (*v).v_tp) == NULL)
                break;
            s_bcopy((*v).v_name, name, 8);      /* tutor only displays 8 */
        }
    }
}

/* dispval - display value components for this parameter on tutor display.
 * Assumes the value panel position array is up to date.
 * Assumes the value panel top & bottom specifiers are up to date.
 */

FUNCTION VOID dispval 
(
    struct VARIABLE *v      /* in:  variable to display value of    */
)
{
    IMPORT COUNT        termlines;      /* number lines on terminal screen      */

    CODE                type;
    struct TXTSTOR      valtxt;
    COUNT               i;
    struct TUTEXT       *t;
    COUNT               line;
    COUNT               maxvalln;   /* max display lines allowed for value  */
    COUNT               numclrln;

    type     = (*v).v_type;
    t        = (*v).v_tp;
    line     = tutctx.pageStart + (*t).startlin;
    maxvalln = max(termlines - ERRLIN - line, 0);
    /* get number of lines to clear */
    numclrln = min(nlinval(v), maxvalln);
    initxt(&valtxt);
    for (i = (*t).pantop; i <= (*t).panbot; i++)
        if (addcomp(&valtxt, v, i) != SUCCESS) 
            goto over_err;   /* add each compoment's text*/
    clrmlin(line, TUTVALCOL, numclrln);
    if (type != V_NAME  &&  (*v).v_maxc > 1)
        dispcnum(v, (*t).pantop, (*t).panbot);  /* display component numbers*/
    if (type == V_STRING)
        disptxt(line, TUTVALCOL, &valtxt);
    else
        disprtxt(line, TUTVALCOL+TUTNUMRTCOL, &valtxt);
    fretxt(&valtxt);            /* free strings from dyn memory         */
    return;

over_err:
    tutmsg(msg_mmov, key_mmov);         /* dyn mem overflow */
}

/* tutmsg - display an error message.  Depending on tutor source specific
 * flag, writes thru wrterr or tmmsg.
 * If called from processing of a cmd returned from help, hold the message.
 */

FUNCTION VOID tutmsg 
(
    TEXT msg[],
    TEXT key[],
    ...
)
{
    va_list arglist;

    /* get the arguments to this function */
    va_start(arglist,key);
    va_end(arglist);

    if (!tutctx.prev_mode || tutctx.msg_held)
    {
        if (tutctx.crt_messages)
            wrterr_valist(msg, key, arglist);
        else
            tmmsg_valist(PROCFAIL, msg, key, arglist);
        tutctx.msg_held = FALSE;
    }
    else     /* must hold message for screen display */
    {
        tutctx.msg_held = TRUE;
        vsprintf(tutctx.held_msg, msg, arglist);
        strcpy(tutctx.held_key,key);
    }
}
